/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

/*
 * Description (config.c)
 *
 *	This module contains routines used by administration CGI
 *	programs to set/get the attribute(s) of Admin server
 *
 */
#include <stdio.h>
#ifdef XP_UNIX
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <pwd.h>
#include <grp.h>
#include <dirent.h>
#include <errno.h>
#endif
#include <string.h>
#include "libadmin/libadmin.h"
#include "libadmin/install.h"
#include "libadminutil/resource.h"
#include "libadminutil/distadm.h"
#include "libadminutil/admutil.h"
#include "libadmsslutil/psetcssl.h"
#include "libadmsslutil/admsslutil.h"

#ifdef XP_WIN32
#define EADDRINUSE WSAEADDRINUSE
#endif

/*
  properties file name = "config.properties"
 */
#define RESOURCE_FILE "config"

#define resource_key(a,b)   a b

#define DBT_NO_METHOD 			resource_key(RESOURCE_FILE, "1")
#define DBT_UNKNOWN_METHOD 		resource_key(RESOURCE_FILE, "2")
#define DBT_NO_QUERY_STRING 	resource_key(RESOURCE_FILE, "3")
#define DBT_NO_USER_NAME 		resource_key(RESOURCE_FILE, "4")
#define DBT_NO_USER_DN 			resource_key(RESOURCE_FILE, "5") 
#define DBT_PSET_CREATE_ERROR 	resource_key(RESOURCE_FILE, "6") 
#define DBT_NO_OP 				resource_key(RESOURCE_FILE, "7") 
#define DBT_ILLEGAL_OP 			resource_key(RESOURCE_FILE, "8") 
#define DBT_NO_ATTRS 			resource_key(RESOURCE_FILE, "9") 
#define DBT_PSET_GET_ERROR 		resource_key(RESOURCE_FILE, "10") 
#define DBT_PSET_PARTIAL_GET 	resource_key(RESOURCE_FILE, "11") 
#define DBT_ATTR_NOT_EXIST 		resource_key(RESOURCE_FILE, "12") 
#define DBT_ATTR_NO_VALUE 		resource_key(RESOURCE_FILE, "13") 
#define DBT_PSET_SET_ERROR 		resource_key(RESOURCE_FILE, "14") 
#define DBT_PSET_ADD_ERROR 		resource_key(RESOURCE_FILE, "15") 
#define DBT_PSET_SET_NOT_EXIST 	resource_key(RESOURCE_FILE, "16") 
#define DBT_SSL_INIT_ERROR 		resource_key(RESOURCE_FILE, "17")
#define DBT_ATTR_INVALID_VALUE  	resource_key(RESOURCE_FILE, "18") 
#define DBT_PORT_INVALID        resource_key(RESOURCE_FILE, "19")
#define DBT_PORT_INUSE          resource_key(RESOURCE_FILE, "20")
#define DBT_PORT_EACCESS        resource_key(RESOURCE_FILE, "21")
#define DBT_PORT_NOT_AVAIL      resource_key(RESOURCE_FILE, "22")
#define DBT_ACCESSLOG_INVALID   resource_key(RESOURCE_FILE, "23")
#define DBT_ERRORLOG_INVALID    resource_key(RESOURCE_FILE, "24")

static int update_adm_conf(char *newport);
static int update_magnus_conf(char *name, char *val);
static int validate_addr(char* ip);
static int validate_logfile(char* name);
static int get_logfile_path(char *name, char *buf);
#ifdef XP_UNIX
static int rename_pidlog_file(PsetHndl pset, char* newname);
static int change_uid_all(char *dir, int curuid, int newuid);
static int change_uid(char *fpath, int curuid, int newuid);
static int verify_sever_uname(char *newuname);
static int change_server_uid(PsetHndl pset, char* newuname);
#endif

/* modxml.c functions */
extern int modxml_server_port(char *sroot, char *port);
extern int modxml_server_ip(char *sroot, char *ip);
extern int modxml_server_accesslog(char *sroot, char *accesslog);

int main(int argc, char *argv[])
{
  int            _ai = ADMUTIL_Init();
  PsetHndl       pset;
  char           *method;
  char           *admroot = getenv("ADMSERV_ROOT");
  int            rv, cnt, i, j, x, errorCode;
  int            err;
  int            *errp = &err;
  char           *username = 0;
  char           *localAdmin = 0;
  char           *binddn = 0;
  char           *bindpw = 0;
  char           *newValue = 0;
  char           *portVal = 0;
  char           **inputs = 0;
  char           *operation = 0;
  char           *qs = 0;
  char           *nameptr, *valptr, *val;
  char           error_info[128];
  char           *valsbuf[2];
  int            setFlag = 0, getFlag = 0, forceSetFlag = 0;
  AttributeList  resultList, updateList, addList, nvl;
  AttrNameList   nl;
  AttributePtr   nv;
  char           resPath[256], *execPath;
  char           *acceptLanguage = (char*)"en";
  Resource       *i18nResource = NULL;
  AdmldapInfo    adminfo = NULL;
#if 0
  int            waitforever = 1;

  while (waitforever);
#endif

  execPath = getcwd(resPath, 256);
  if (execPath) {
    strcpy(resPath, execPath);
    strcat(resPath, "/property");
    i18nResource = res_init_resource(resPath, RESOURCE_FILE);
  }
  valsbuf[0] = NULL;
  valsbuf[1] = NULL;

  memset((void *)errp, 0, sizeof(int));
  method = getenv("REQUEST_METHOD");

  /* Get new port number */
  if (!method || !*method) {
    /* non UI CGI */
    if (i18nResource) 
      rpt_err(SYSTEM_ERROR, 
	      res_getstring(i18nResource, DBT_NO_METHOD, acceptLanguage),
	      NULL, NULL);
    else rpt_err(SYSTEM_ERROR, "No method is specified", NULL, NULL);
  }
  else if (!strcmp(method, "GET")) {
    /* UI CGI - either upgrade or normal admin CGI */
    qs = getenv("QUERY_STRING");
	
    if (!qs || !*qs) {
      if (i18nResource) 
	rpt_err(INCORRECT_USAGE,
		res_getstring(i18nResource, DBT_NO_QUERY_STRING,
			      acceptLanguage),
		NULL, NULL);
      else rpt_err(INCORRECT_USAGE, "NO QUERY_STRING DATA", NULL, NULL);
    }
    else {
      get_begin(qs);
    }
  }
  else if (!strcmp(method, "POST")) {
    post_begin(stdin);
  }
  else {
    if (i18nResource) 
      sprintf(error_info, 
	      res_getstring(i18nResource, DBT_UNKNOWN_METHOD, acceptLanguage),
	      method);
    else sprintf(error_info, "Unknown Method (%s)", method);
    rpt_err(INCORRECT_USAGE, error_info, NULL, NULL);
  }

  /* Get UserDN and User Password  */

  rv = ADM_GetUserDNString(&err, &binddn);
  if (rv < 0 || !binddn || !*binddn) {
    rv = ADM_GetCurrentUsername(&err, &username);
    if (rv < 0 || !username || !*username) {
      if (i18nResource) 
	rpt_err(ELEM_MISSING,
		res_getstring(i18nResource, DBT_NO_USER_NAME,
			      acceptLanguage),
		NULL, NULL);
      else rpt_err(ELEM_MISSING, "NO USER NAME", NULL, NULL);
    }
    else {
      /* No DN, maybe it is local super */
      localAdmin = admGetLocalAdmin(NULL, &rv);
      if (localAdmin) {
	if (strcmp(username, localAdmin)) {
	  if (i18nResource) 
	    rpt_err(ELEM_MISSING,
		    res_getstring(i18nResource, DBT_NO_USER_DN,
				  acceptLanguage),
		    NULL, NULL);
	  else rpt_err(ELEM_MISSING, "NO USER DN!", NULL, NULL);
	}
	else {
	  binddn = NULL;
	  bindpw = NULL;
	}
      }
      else {
	if (i18nResource) 
	  rpt_err(ELEM_MISSING,
		  res_getstring(i18nResource, DBT_NO_USER_DN,
				acceptLanguage),
		  NULL, NULL);
	else rpt_err(ELEM_MISSING, "NO USER DN!", NULL, NULL);
      }
    }
  }

  if (binddn) rv = ADM_GetCurrentPassword(&err, &bindpw);

  if (admroot) {
    adminfo = admldapBuildInfo(admroot, &rv);
    if (adminfo) {
      binddn = admldapGetSIEDN(adminfo);
      bindpw = admldapGetSIEPWD(adminfo);
    }
  }

  errorCode = ADMSSL_InitSimple();
  if (errorCode) {
    if (i18nResource) 
      rpt_err(APP_ERROR,
	      res_getstring(i18nResource, 
			    DBT_SSL_INIT_ERROR,
			    acceptLanguage),
	      NULL, NULL);
    else rpt_err(APP_ERROR, "SSL related initialization failed", NULL, NULL);
  }

  /* Initialize the pset  */
  pset = psetCreateSSL("admin-serv", 
		       /* configRoot */ NULL,
		       /* userDN */ binddn, 
		       /* passwd */ bindpw,
		       /* errorcode */ &rv);

  if (!pset) {
    sprintf(error_info,
	    "PSET_ERROR_NUMBER: %d\nPSET_ERROR_INFO: %s", 
	    rv,
	    psetErrorString(rv, NULL));
    if (i18nResource) 
      rpt_err(APP_ERROR,
	      res_getstring(i18nResource, DBT_PSET_CREATE_ERROR, 
			    acceptLanguage),
	      NULL, error_info);
    else rpt_err(APP_ERROR, "PSET Creation Failed", NULL, error_info);
  }

  /* Process the input  */

  inputs = get_input_ptr();

  operation = get_cgi_var("op", NULL, NULL);

  if (!operation) {
    psetDelete(pset);
    if (i18nResource) 
      rpt_err(INCORRECT_USAGE,
	      res_getstring(i18nResource, DBT_NO_OP, acceptLanguage),
	      NULL, NULL);
    else rpt_err(INCORRECT_USAGE, "No operation defined", NULL, NULL);
  }

  if (!strcmp(operation, "get")) getFlag = 1;
  else if (!strcmp(operation, "set")) setFlag = 1;
  else if (!strcmp(operation, "force_set")) { 
    setFlag = 1; 
    forceSetFlag = 1; 
  }
  else {
    if (i18nResource) 
      sprintf(error_info,
	      res_getstring(i18nResource, DBT_ILLEGAL_OP, acceptLanguage),
	      operation);
    else sprintf(error_info, "Illegal operation defined (%s)", operation);
    rpt_err(INCORRECT_USAGE, error_info, NULL, NULL);
  }

  cnt = 0;
  x = 0;
  while (inputs[x] && *(inputs[x++]) != '\0') cnt++;

  if (cnt <= 1) {
    psetDelete(pset);
    if (i18nResource) 
      rpt_err(INCORRECT_USAGE, 
	      res_getstring(i18nResource, DBT_NO_ATTRS, acceptLanguage),
	      NULL, NULL);
    else rpt_err(INCORRECT_USAGE, "No attribute specified ", NULL, NULL);
  }

  if (getFlag) { /* get operation */
    nl = createAttrNameList(cnt);
    x = 0; i = 0;
    while (inputs[x] && *(inputs[x]) != '\0') {
      char namebuf[128], *cptr;
      int j;
      memset(namebuf, '\0', 128);
      cptr = inputs[x++];
      j = 0;
      while (*cptr != '=') namebuf[j++] = *cptr++;
      /* Ignore "op" */
      if (strncasecmp(namebuf, "op", 2)) addName(nl, i++, namebuf);
    }

    resultList = psetGetAttrList(pset, nl, &errorCode);

    if (errorCode && 
	errorCode != PSET_PARTIAL_OP && 
	errorCode != PSET_PARTIAL_GET ) {
      psetDelete(pset);
      sprintf(error_info,
	      "PSET_ERROR_NUMBER: %d\nPSET_ERROR_INFO: %s", 
	      errorCode,
	      psetErrorString(errorCode, NULL));
      if (i18nResource) 
	rpt_err(APP_ERROR,
		res_getstring(i18nResource, DBT_PSET_GET_ERROR, 
			      acceptLanguage),
		NULL, error_info);
      else rpt_err(APP_ERROR, "PSET Get Failed", NULL, error_info);
    }

    deleteAttrNameList(nl);
    if (errorCode) {
      if (resultList) {
	if (i18nResource) 
	  rpt_warning(WARNING, 
		      res_getstring(i18nResource, DBT_PSET_PARTIAL_GET,
				    acceptLanguage),
		      NULL, NULL);
	else rpt_warning(WARNING, "Partial Get\n", NULL, NULL);
      }
      else {
	psetDelete(pset);
	if (i18nResource) 
	  rpt_warning(APP_ERROR,
		      res_getstring(i18nResource, DBT_ATTR_NOT_EXIST,
				    acceptLanguage),
		      NULL, NULL);
	else rpt_err(APP_ERROR, "Attribute(s) does not exist", NULL, NULL);
      }
    }
    else rpt_success(NULL);
    
    nvl = resultList;
    while (nv = *nvl++) {
      fprintf(stdout, "%s: %s\n", 
	      nv->attrName, 
	      nv->attrVal ? (nv->attrVal[0] ? nv->attrVal[0] : "") : "");
    }
    deleteAttributeList(resultList);
#if 0
    if (execPath) fprintf(stdout, "execPath: %s\n", execPath);
    else  fprintf(stdout, "execPath: EMPTY\n");
    if (resPath[0] != '\0') fprintf(stdout, "resPath: %s\n", resPath);
    else  fprintf(stdout, "resPath: EMPTY\n");
    if (i18nResource) fprintf(stdout, "info1: Property file OK!");
    else fprintf(stdout, "info1: Can not find Property file!");
#endif
    psetDelete(pset);
    exit(0);
  }
  else if (setFlag) {  /* Set/Force_Set operation */
    updateList = createAttributeList(cnt);
    if (forceSetFlag) addList = createAttributeList(cnt);
    x = 0; i = 0; j = 0;
    while (inputs[x]) {
      valptr = strchr(inputs[x], '=');
      if (valptr) {
	*valptr = '\0';
	valptr++;
	while(*valptr != '\0' && *valptr == ' ') valptr++;
	if (*valptr == '\0') {

	  /* some values are allowed to be empty string*/	  
	  if ((strcasecmp(inputs[x], "configuration.nsAdminAccessHosts") != 0) &&
	      (strcasecmp(inputs[x], "configuration.nsAdminAccessAddresses") != 0) &&
	      (strcasecmp(inputs[x], "configuration.nsServerAddress") != 0) &&
	      (strcasecmp(inputs[x], "configuration.nsAdminEnableDSGW") != 0) &&
	      (strcasecmp(inputs[x], "configuration.nsAdminEnableEnduser") != 0)) {	
	    psetDelete(pset);
	    if (i18nResource) 
	      sprintf(error_info, 
		      res_getstring(i18nResource, DBT_ATTR_NO_VALUE,
				    acceptLanguage),
		      inputs[x]);
	    else sprintf(error_info, "Attribute[%s] does not have value", 
			 inputs[x]);
	    rpt_err(INCORRECT_USAGE, error_info, NULL, NULL);
	  }
	}
      }
      else {
	psetDelete(pset);
	if (i18nResource) 
	  sprintf(error_info, 
		  res_getstring(i18nResource, DBT_ATTR_NO_VALUE,
				acceptLanguage),
		  inputs[x]);
	else sprintf(error_info, "Attribute[%s] does not have value", 
		     inputs[x]);
	rpt_err(INCORRECT_USAGE, error_info, NULL, NULL);
      }
      nameptr = inputs[x];
      /* Ignore the OP  */
      if (strncasecmp(nameptr, "op", 2)) {

	/* Check if port is being changed */
	if (strcasecmp(nameptr,"configuration.nsServerPort") == 0) {
	  int port;
	  
	  port = atoi(valptr);
	  
	  err = 0;
	  if (port < 1 || port > 65535)
	    {
	      err = -1;
	    }
	   
	  if (try_bind(NULL, port) == -1)
	    {
	      if(errno == EADDRINUSE)
		err = -2;
	      else if(errno == EACCES) 
		err = -3;
	      else 
		err = -4;
	    }
	   
	  if(err == 0) {
	    /* modify server.xml */
	    err = modxml_server_port(getenv("NETSITE_ROOT"), valptr); 
	  }

	  if(err == 0) {
	    err = update_adm_conf(valptr);
	  }

	  if (err < 0)
	    {
              psetDelete(pset);
              if (i18nResource)
		{
		  if (err == -1)
                    sprintf(error_info, res_getstring(i18nResource, DBT_PORT_INVALID, acceptLanguage));
		  else if (err == -2)
                    sprintf(error_info, res_getstring(i18nResource, DBT_PORT_INUSE, acceptLanguage));
		  else if (err == -3)
                    sprintf(error_info, res_getstring(i18nResource, DBT_PORT_EACCESS, acceptLanguage));
		  else
                    sprintf(error_info, res_getstring(i18nResource, DBT_PORT_NOT_AVAIL, acceptLanguage));
		}
              else
		{
		  sprintf(error_info, "Port %s is either in use or invalid", valptr);
		}
              rpt_err(APP_ERROR, error_info, NULL, NULL);
	    }
	}

	/* Check if the IP address is valid */
	if (strcasecmp(nameptr,"configuration.nsserveraddress") == 0) {
	  if (!validate_addr(valptr)) {
	    /* report error */
	    psetDelete(pset);
	    if (i18nResource) 
	      sprintf(error_info, 
		      res_getstring(i18nResource, DBT_ATTR_INVALID_VALUE,
				    acceptLanguage),
		      nameptr);
	    else sprintf(error_info, "Attribute[%s] has invalid value", 
			 nameptr);
	    rpt_err(APP_ERROR, error_info, NULL, NULL);
	  }
	  else {
	    /*
	      If ip address is not defined, it means that server should
	      listen on all interfaces. IWS is expecting  0.0.0.0 value
	      for such case.
	    */
	    if (valptr && valptr[0] ) {
	      err = modxml_server_ip(getenv("NETSITE_ROOT"), valptr);
	    } else {
	      valptr = "";
	      err = modxml_server_ip(getenv("NETSITE_ROOT"), "0.0.0.0");
	    }
	  }
	}
	
	/* Check if access log file is being changed */
	if (strcasecmp(nameptr,"configuration.nsaccesslog") == 0) {
	  if (!validate_logfile(valptr)) {
	    /* report error */
	    psetDelete(pset);
	    if (i18nResource) 
	      sprintf(error_info, 
		      res_getstring(i18nResource, DBT_ACCESSLOG_INVALID,
				    acceptLanguage),
		      nameptr);
	    else sprintf(error_info, "Invalid access log file specified");
	    rpt_err(APP_ERROR, error_info, NULL, NULL);
	  }
	  else {
	    char newpath[BIG_LINE];
	    get_logfile_path(valptr, newpath);
	    /* modify server.xml */
	    err = modxml_server_accesslog(getenv("NETSITE_ROOT"), newpath);
	  }
	}

	/* Check if error log file is being changed */
	if (strcasecmp(nameptr,"configuration.nserrorlog") == 0) {
	  if (!validate_logfile(valptr)) {
	    /* report error */
	    psetDelete(pset);
	    if (i18nResource) 
	      sprintf(error_info, 
		      res_getstring(i18nResource, DBT_ERRORLOG_INVALID,
				    acceptLanguage),
		      nameptr);
	    else sprintf(error_info, "Invalid error log file specified"); 
	    rpt_err(APP_ERROR, error_info, NULL, NULL);
	  }
	  else {
	    char newpath[BIG_LINE];
	    get_logfile_path(valptr, newpath);
	    err = update_magnus_conf("ErrorLog", newpath);
	  }
	}

#ifdef XP_UNIX
	/* Check if pidlog  is being changed */
	if (strcasecmp(nameptr,"configuration.nsPidLog") == 0) {

	  if (!rename_pidlog_file(pset, valptr)) {
	    rpt_err(APP_ERROR, "Failed to rename log file", NULL, NULL);
	  }
	}

	/* Check if server uid is being changed */
	if (strcasecmp(nameptr,"configuration.nsSuiteSpotUser") == 0) {

	  if (change_server_uid(pset, valptr) < 0) {
	    rpt_err(APP_ERROR, "Failed to change server uid", NULL, NULL);
	  }
	}
#endif

	errorCode = PSET_OP_OK;
	val = psetGetAttrSingleValue(pset, nameptr, &errorCode);
	if (errorCode) {
	  if (forceSetFlag) 
	    addSingleValueAttribute(addList, j++, nameptr, valptr);
	  else j++;
	}
	else {
	  addSingleValueAttribute(updateList, i++, nameptr, valptr);
	}
      }
      x++;
    }
    errorCode = PSET_OP_OK;
    if (i) errorCode = psetSetAttrList(pset, updateList);
    if (errorCode) {
      psetDelete(pset);
      sprintf(error_info,
	      "PSET_ERROR_NUMBER: %d\nPSET_ERROR_INFO: %s\n", 
	      errorCode,
	      psetErrorString(errorCode, NULL));
      if (i18nResource) 
	rpt_err(APP_ERROR,
		res_getstring(i18nResource, DBT_PSET_SET_ERROR, 
			      acceptLanguage),
		NULL, error_info);
      else rpt_err(APP_ERROR, "PSET Set Failed", NULL, error_info);
    }

    if (forceSetFlag) {
      if (j) {
	for (i=0; i < j; i++) {
	  errorCode = psetAddAttribute(pset, 
				       addList[i]->attrName, 
				       addList[i]->attrVal);
	  if (errorCode) {
	    psetDelete(pset);
	    sprintf(error_info,
		    "PSET_ERROR_NUMBER: %d\nPSET_ERROR_INFO: %s\n", 
		    errorCode,
		    psetErrorString(errorCode, NULL));
	    if (i18nResource) 
	      rpt_err(APP_ERROR,
		      res_getstring(i18nResource, DBT_PSET_ADD_ERROR, 
				    acceptLanguage),
		      NULL, error_info);
	    else rpt_err(APP_ERROR, "PSET Add Failed", NULL, error_info);
	  }
	}
      }
    }
    else {
      if (j) {
	psetDelete(pset);
	if (i18nResource) 
	  rpt_err(APP_ERROR,
		  res_getstring(i18nResource, DBT_PSET_SET_NOT_EXIST, 
				acceptLanguage),
		  NULL, NULL);
	else rpt_err(APP_ERROR,
		     "PSET SET OP Failed for setting non-existing attribute",
		     NULL, NULL);
      }
    }
    
    rpt_success(NULL);
    psetDelete(pset);
    if (i18nResource != NULL) {
      res_destroy_resource(i18nResource);
    }
    exit (0);
  }
  if (updateList) deleteAttributeList(updateList);
  if (addList) deleteAttributeList(addList);  
  psetDelete(pset);

}


/*
 * Modify Port in adm.conf
 */
static int update_adm_conf(char *newport) {

  FILE *f;
  int i, modified=0;
  static char filename[BIG_LINE];
  static char inbuf[BIG_LINE];
  static char buf[BIG_LINE];
  int linecnt=0;	
  char *lines[50];

  sprintf(filename, "%s/adm.conf", getenv("ADMSERV_ROOT"));

  f = fopen(filename, "r");
  if (f==NULL) {
    rpt_err(SYSTEM_ERROR, "Can not open adm.conf for reading", NULL, NULL);
  }

  while(fgets(inbuf, sizeof(inbuf), f) != NULL) {
    if (strncasecmp(inbuf,"port:",5) == 0) { /* Line starts with "Port:" */
      sprintf(buf, "port: %s\n", newport);
      lines[linecnt++] = strdup(buf);
      modified=1;
    }
    else {
      lines[linecnt++] = strdup(inbuf);
    }
  }	
  fclose(f);

  if (!modified) { /* Add the Port attribute */
    sprintf(buf, "Port: %s\n", newport);
    lines[linecnt++] = strdup(buf);
  }

  f = fopen(filename, "w");
  if (f==NULL) {
    fclose(f);
    rpt_err(SYSTEM_ERROR, "Can not open adm.conf for writing", NULL, NULL);
  }

  for (i=0; i < linecnt; i++) {
    fprintf(f, "%s", lines[i]);
  }

  fclose(f);

  return 0;

}


/*
 * Modify any attribute in magnus.conf with a name/value pair
 * If the attribute value is NULL, remove it from magnus.conf completely.
 */
static int update_magnus_conf(char *name, char *val) {

  FILE *f;
  int i, modified=0;
  static char filename[BIG_LINE];
  static char inbuf[BIG_LINE];
  static char buf[BIG_LINE];
  int linecnt=0;	
  char *lines[50];

  sprintf(filename, "%s/magnus.conf", getenv("ADMSERV_ROOT"));

  f = fopen(filename, "r");
  if (f==NULL) {
    rpt_err(SYSTEM_ERROR, "Can not open magnus.conf for reading", NULL, NULL);
  }

  while(fgets(inbuf, sizeof(inbuf), f) != NULL) {
    if (strncasecmp(inbuf,name,strlen(name)) == 0) { /* Line starts with the attribute name */
      if(val && *val != '\0') {
	sprintf(buf, "%s %s\n", name, val);
	lines[linecnt++] = strdup(buf);
	modified=1;
      }
      else {
	modified=1;
      }
    }
    else {
      lines[linecnt++] = strdup(inbuf);
    }
  }	
  fclose(f);

  if (!modified && (val && *val != '\0')) { /* Add the attribute name/val pair*/
    sprintf(buf, "%s %s\n", name, val);
    lines[linecnt++] = strdup(buf);
  }

  f = fopen(filename, "w");
  if (f==NULL) {
    fclose(f);
    rpt_err(SYSTEM_ERROR, "Can not open magnus.conf for writing", NULL, NULL);
  }

  for (i=0; i < linecnt; i++) {
    fprintf(f, "%s", lines[i]);
  }

  fclose(f);

  return 0;

}



/*
 * Rename logs/pid file when configuration.nsPidLog is changed
*/
#ifdef XP_UNIX
static int  rename_pidlog_file(PsetHndl pset, char* newname) {
  char oldpath[BIG_LINE];
  char newpath[BIG_LINE];
  char *sroot = getenv("NETSITE_ROOT");
  int errorCode;

  char *oldname = psetGetAttrSingleValue(pset, 
					 "configuration.nsPidLog", 
					 &errorCode);
  if (oldname != NULL && strcmp(oldname, newname) != 0) {
    sprintf(oldpath, "%s/%s", sroot, oldname);
    sprintf(newpath, "%s/%s", sroot, newname);
    if(rename (oldpath, newpath) == 0)
      return 1;
    return update_magnus_conf("PidLog", newname);
  }
  else return 1;
}
#endif


/*
 * Change admin server uiuserd
*/
#ifdef XP_UNIX
static int  change_server_uid(PsetHndl pset, char* newuname) {
  char *sroot = getenv("NETSITE_ROOT");
  int errorCode;
  static char buf[BIG_LINE];
  int newuid;
  char *olduname = psetGetAttrSingleValue(pset, 
					  "configuration.nsSuiteSpotUser", 
					  &errorCode);

  if (olduname != NULL && strcmp(olduname, newuname) != 0) {

    /* Can change uid only id running as a root */
    if (getuid() != 0) {
      rpt_err(SYSTEM_ERROR, "Can not change Server UID, "
	      "Admin Server instance in not running under ROOT UID",
	      NULL,NULL);
    }

    newuid = verify_sever_uname(newuname);

    if (newuid >= 0) {
      if(change_uid_all(sroot, 0, newuid) == 0)
	return update_magnus_conf("User", newuname);
      else
	return -1;
    }
    else {
      return -1;
    }
  }

  else return 0;

}
#endif

/*
 * Verify if newuname can be used as the admin server uid. The new uid must
 * belong to the SuiteSpot administrator group (recorded in ssusers.conf) 
 *
 * Returns error : -1 
 *         ok    :  uid for the newuname
 */
#ifdef XP_UNIX
static int verify_sever_uname(char *newuname) {

  FILE *f;
  int i=0, ssgrp_found=0, grmem_found=0;
  static char filename[BIG_LINE];
  static char linebuf[BIG_LINE];
  static char buf[BIG_LINE];
  char *grpname;
  struct group  *grp;
  struct passwd *pwd;

  /*
   * Read 'SuiteSpotGroup' attribute from
   *  <server-root>/shared/config/ssusers.conf
   */
  sprintf(filename, "%s/shared/config/ssusers.conf", getenv("NETSITE_ROOT"));

  f = fopen(filename, "r");
  if (f==NULL) {
    rpt_err(SYSTEM_ERROR, "Can not open ssusers.conf for reading", NULL, NULL);
    return -1;
  }

  while(fgets(linebuf, sizeof(linebuf), f) != NULL) {

    /* Remove newline ch */
    if (linebuf[strlen(linebuf)-1] == '\n') {
      linebuf[strlen(linebuf)-1]=0;
    }

    if ( (grpname = strchr(linebuf, '\t')) == NULL) {
      fclose(f); 
      rpt_err(SYSTEM_ERROR, "Bad format for ssusers.conf entrties", NULL, NULL);
      return -1;
    }

    *grpname++=0;

    if (strcasecmp(linebuf,"SuiteSpotGroup") == 0) {
      ssgrp_found=1;
      break;
    }
  }	
  fclose(f);

  if (!ssgrp_found) {
    rpt_err(SYSTEM_ERROR, "SuiteSpotGroup not found in ssusers.conf", NULL, NULL);
    return -1;
  }

  grp = getgrnam(grpname);
  if (grp == NULL) {
    sprintf(buf, "SuiteSpotGroup '%s' does not exist", grpname);
    rpt_err(SYSTEM_ERROR, buf, NULL, NULL);
    return -1;
  }

  /*
   * newuname must belong to the SuiteSpot group
   */
  pwd = getpwnam(newuname);
  if (pwd == NULL) {
    sprintf(buf, "Can not change Server UID, "
	    "User '%s' does not exist", newuname);
    rpt_err(INCORRECT_USAGE, buf, NULL, NULL);
    return -1;
  }

  /* Check all grp members */
  while (grp->gr_mem[i] && !grmem_found) {
    if (strcmp(grp->gr_mem[i], newuname) == 0) {
      grmem_found = 1;
    }
    else i++;	
  }

  if (!grmem_found && pwd->pw_gid != grp->gr_gid) {
    sprintf(buf, "Can not change Server UID, "
	    "User '%s' does not belong to the SuiteSpot Group '%s'", newuname, grpname);
    rpt_err(INCORRECT_USAGE, buf, NULL, NULL);
    return -1;
  }

  return pwd->pw_uid;

}
#endif

/*
 * Change recursivly file ownership to newuid for all files and subdirecrories
 * for wich the current owner is olduid
 */
#ifdef XP_UNIX
static int change_uid_all(char *dir, int olduid, int newuid) {
  DIR *dirp; 
  struct dirent *dp;
  char fpath[BIG_LINE];
  char buf[BIG_LINE];
  struct stat filestat;
	

	
  dirp = opendir(dir);

  if (dirp == NULL) {
    sprintf(buf, "Change Server UID: opendir(%s) failed, errno=%d\n", dir, errno);
    rpt_err(SYSTEM_ERROR, buf, NULL, NULL);
    return -1;
  }

  if (change_uid(dir, olduid, newuid) != 0) {
    closedir(dirp);
    return -1;
  }

  for (dp = readdir(dirp); dp != NULL; dp = readdir(dirp)) {
    if (strcmp(dp->d_name,".") == 0 ||
	strcmp(dp->d_name, "..") == 0) {
      continue;
    }

    sprintf(fpath, "%s/%s", dir, dp->d_name);

    if (stat(fpath, &filestat) != 0) {
      sprintf(buf, "Change Server UID: stat(%s) failed, errno=%d\n", fpath, errno);
      rpt_err(SYSTEM_ERROR, buf, NULL, NULL);
      return -1;
    }


    if (S_ISDIR(filestat.st_mode)) {
      if (change_uid_all(fpath, olduid, newuid) != 0) {
	closedir(dirp);
	return -1;
      }
    }
    else if (change_uid(fpath, olduid, newuid) != 0) {
      closedir(dirp);
      return -1;
    }
  }

  closedir(dirp);
  return 0;
}
#endif

#ifdef XP_UNIX
/*
 * Change file ownership for a files
 */
static int change_uid(char *fpath, int olduid, int newuid) {
  struct stat filestat;
  char buf[BIG_LINE];
	
  if (stat(fpath, &filestat) != 0) {
    sprintf(buf, "Change Server UID: stat(%s) failed, errno=%d\n", fpath, errno);
    rpt_err(SYSTEM_ERROR, buf, NULL, NULL);
    return -1;
  }

  if (filestat.st_uid == olduid) {
    int rc = chown(fpath, newuid, -1);
    if (rc != 0) {
      sprintf(buf, "Change Server UID: Can not change file owner for %s, errno=%d\n", fpath, errno);
      rpt_err(SYSTEM_ERROR, buf, NULL, NULL);
      return -1;
    }
  }
  return 0;
}
#endif

static int validate_addr(char* ip) {

  char systemInfo[SYS_INFO_BUFFER_LENGTH];
  char buf[PR_NETDB_BUF_SIZE];
  PRIntn index;
  PRNetAddr  netaddr, netaddr1;
  PRStatus   pr_st;
  PRHostEnt  hostentry;

  /* If ip address is not define, it means that server should listen on all interfaces */
  if (ip==NULL || *ip=='\0') return 1;

  /* If ip address is equal to localhost (127.0.0.1), no need to validate the address */
  if (!strcmp(ip, "127.0.0.1")) return 1;
  if (!strcmp(ip, "0.0.0.0")) return 1;

  PR_StringToNetAddr(ip, &netaddr);

  pr_st = PR_GetSystemInfo(PR_SI_HOSTNAME, systemInfo, SYS_INFO_BUFFER_LENGTH);

  pr_st = PR_GetHostByName(systemInfo, buf, PR_NETDB_BUF_SIZE, &hostentry);

  index = 0;
  while (index = PR_EnumerateHostEnt(index, &hostentry, 8000, &netaddr1)) {
    if (netaddr1.inet.ip == netaddr.inet.ip)  return 1;
  }
  
  return 0;
}

/*
 * Try to open a log file in append mode. If the file is not present, it will be created
 * Returns :  1 ok
 *            0 error
 *           
 */
static int validate_logfile(char *name)  {
  FILE *f;
  char fullname[BIG_LINE];

  get_logfile_path(name, fullname);

  f = fopen(fullname, "a+");
  if (f != NULL) {
    fclose(f);
    return 1; /* ok*/
  }
  return 0; /* error */
}

static int get_logfile_path(char *name, char *buf)  {

  if (name[0] == '/') {
    sprintf(buf,"%s",name);
  }
  else {
    sprintf(buf,"%s/%s", getenv("NETSITE_ROOT"), name);
  }

  return 0;
}
