/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*///////////////////////////////////////////////////////*/
/*/                                                     /*/
/*/ David Tompkins (mailto:dt@netscape.com)             /*/
/*/ Netscape Communications Corp.                       /*/
/*/ 3/4/97                                              /*/
/*/                                                     /*/
/*/ download: returns the available nmcc downloads      /*/
/*/                                                     /*/
/*/ 4/13/98: Changed to return the entire page...       /*/
/*/ 4/27/98: Now reads the page from start-console.html /*/
/*/          and inserts the menu options.              /*/
/*/                                                     /*/
/*///////////////////////////////////////////////////////*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef XP_UNIX
#   include <dirent.h>
#   define SYS_DIR DIR
#   define SYS_DIRENT struct dirent
#   define dir_open opendir
#   define dir_read readdir
#   define dir_close closedir
#   define dir_name(de) (de->d_name)
#   define FILE_SEP '/'
#   define STRNCASECMP strncasecmp
#   define STRCASECMP  strcasecmp
#   define STRNCMP     strncmp
#   define STRDUP      strdup
#else /* XP_WIN32 */
#   include <prio.h>
#   define SYS_DIR PRDir
#   define SYS_DIRENT PRDirEntry
#   define dir_open PR_OpenDir
#   define dir_read(d) PR_ReadDir(d, PR_SKIP_BOTH)
#   define dir_close PR_CloseDir
#   define dir_name(de) (de->name)
#   define FILE_SEP '/'
#   define STRNCASECMP _strnicmp
#   define STRCASECMP  _stricmp
#   define STRNCMP     strncmp
#   define STRDUP      strdup
#endif

#define PATH_LENGTH		1024
#define MAX_NUM_DOWNLOADS	32

#define MENU_BLOCK		"<download_menu_block>"
#define MENU_BLOCK_CLOSE	"</download_menu_block>"
#define MENU_OPTIONS_BLOCK	"<download_menu_options>"

#define DSGW_BLOCK		"<dsgw_menu_block>"
#define DSGW_BLOCK_CLOSE	"</dsgw_menu_block>"

int
error_exit(char *msg)
{
   printf("Content-type: text/plain\n\n");
   printf("Error: %s\n", msg);
   exit(0);
}

static int
dsgw_exists ()
{
   char          *serverroot = NULL;
   char          path[PATH_LENGTH];
   SYS_DIR       *dp1;

   if (!(serverroot = getenv("NETSITE_ROOT")))
      error_exit("NETSITE_ROOT not found");

   /* Check whether dsgw is installed */
   sprintf(path, "%s%cdsgw", serverroot, FILE_SEP);

   if (!(dp1 = dir_open(path)))
   {
      return 0;
   }

   dir_close(dp1);
   return 1;
}

static int
populate_download_menu(char *platforms[], char *paths[])
{
   int            baselen;
   char           path[PATH_LENGTH];
   char           line[PATH_LENGTH];
   char          *serverroot = NULL;
   SYS_DIR       *dp1, *dp2;
   SYS_DIRENT    *ep1, *ep2;
   int            count = 0;

   if (!(serverroot = getenv("NETSITE_ROOT")))
      error_exit("NETSITE_ROOT not found");

   /* build list of supported downloads of the form <sr>/dist/<OSname>/n*.zip */

   sprintf(path, "%s%cdist", serverroot, FILE_SEP);

   baselen = strlen(path);
   
   if (!(dp1 = dir_open(path)))
      error_exit("Unable to open NETSITE_ROOT/dist");

   while ((ep1 = dir_read(dp1)))
   {
      /* don't check anything starting with . */
      if (dir_name(ep1)[0] == '.')
	 continue;
     
      /* check if this is a dir and has an executable */
      sprintf(&(path[baselen]), "%c%s", FILE_SEP, dir_name(ep1));
      if (!(dp2 = dir_open(path)))
	 continue;
      
      while ((ep2 = dir_read(dp2)))
      {
         if (dir_name(ep2)[0] == '.')
	    continue;
         
         if (!STRNCASECMP(dir_name(ep2), "uxmcc", 5) ||
             !STRNCASECMP(dir_name(ep2), "ntmcc", 5) ||
             !STRNCASECMP(dir_name(ep2), "winmcc", 6))
	 {
	    platforms[count] = STRDUP(dir_name(ep1));
	    sprintf(line, "%s:%s", dir_name(ep1), dir_name(ep2));
	    paths[count++] = STRDUP(line);
	 }
      }

      dir_close(dp2);
   }

   dir_close(dp1);

   return (count);
}

int
main(int argc, char *argv[])
{
   char *platforms[MAX_NUM_DOWNLOADS];
   char *paths[MAX_NUM_DOWNLOADS];
   char  line[PATH_LENGTH];
   FILE *html;
   int   count, i, menublock, dsgwblock, has_dsgw;
   char          *acceptLanguage = NULL;
   char          *loc;
   
   count = populate_download_menu(platforms, paths);
   has_dsgw = dsgw_exists();

   acceptLanguage = getenv("HTTP_ACCEPT_LANGUAGE");
   if (acceptLanguage == NULL)
   {
      acceptLanguage = "en";
   }

   /*
    * Search for list of accept language for a match
    */
   loc = strtok(strdup(acceptLanguage), ",");
   while (1)
   {
      sprintf(line, "..%cjava%chtml%c%c%c%cstart-console.html", FILE_SEP, FILE_SEP, FILE_SEP, loc[0],loc[1],FILE_SEP);
      if ((html = fopen(line, "r")) != NULL)
      {
         break;
      }
      else
      {
         loc = strtok(NULL, ", ");
         if (loc == NULL)
         {
            sprintf(line, "..%cjava%chtml%cstart-console.html", FILE_SEP, FILE_SEP, FILE_SEP);
            if ((html = fopen(line, "r")) == NULL)
            {
              error_exit("start-console.html not found");
            }
            break;
         }
      }
   } 

   printf("Content-type: text/html\n\n");

   menublock = 0;
   dsgwblock = 0;

   while (fgets(line, PATH_LENGTH, html))
   {
      if (!STRNCASECMP(line, MENU_BLOCK, strlen(MENU_BLOCK)))
      {
	 menublock = 1;
         continue;
      }
      else if (!STRNCASECMP(line, DSGW_BLOCK, strlen(DSGW_BLOCK)))
      {
         dsgwblock = 1;
         continue;
      }

      if (!STRNCASECMP(line, MENU_BLOCK_CLOSE, strlen(MENU_BLOCK_CLOSE)))
      {
	 menublock = 0;
         continue;
      }
      else if (!STRNCASECMP(line, DSGW_BLOCK_CLOSE, strlen(DSGW_BLOCK_CLOSE)))
      {
         dsgwblock =0;
         continue;
      }

      if (!STRNCASECMP(line, MENU_OPTIONS_BLOCK, strlen(MENU_OPTIONS_BLOCK)))
      {
         for (i = 0 ; i < count ; i++)
            printf("<option value='%s'>%s\n", paths[i], platforms[i]);
         continue;
      }

      if (!menublock && !dsgwblock)
      {
         printf("%s", line);
	 continue;
      }

      if ((menublock && count > 0) || (dsgwblock && has_dsgw))
            printf("%s", line);
   }
   
   fflush(stdout);
   fclose(html);
}
