/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * Description (listOldSrvs.c)
 *
 *	This module contains routines used by administration CGI
 *	programs to retrieve the server instances from an old server root
 *
 */
#ifdef XP_UNIX
#include <unistd.h>
#endif
#include <stdio.h>
#include <string.h>
#include "prio.h"
#include "plstr.h"
#include "prmem.h"
#include "libadminutil/resource.h"
#include "libadminutil/distadm.h"
#include "libadminutil/admutil.h"
#include "libadminutil/resource.h"


#define RESOURCE_FILE "listOldSrvs"

#define resource_key(a,b)   a b

#define DBT_NO_METHOD 			    resource_key(RESOURCE_FILE, "1")
#define DBT_UNKNOWN_METHOD 		    resource_key(RESOURCE_FILE, "2")
#define DBT_NO_QUERY_STRING 	    resource_key(RESOURCE_FILE, "3")
#define DBT_ERROR_OPEN_FILE 		resource_key(RESOURCE_FILE, "4")
#define DBT_ERROR_READ_FILE 		resource_key(RESOURCE_FILE, "5") 
#define DBT_ERROR_OPEN_DIR 	        resource_key(RESOURCE_FILE, "6") 
#define DBT_DBT_NO_OLD_SERVER_ROOT 	resource_key(RESOURCE_FILE, "7") 
#define DBT_NO_SERVER_FOUND 		resource_key(RESOURCE_FILE, "8") 


char           **installed = NULL;
int            num_installed = 0;
Resource       *i18nResource = NULL;
char           *acceptLanguage = "en";


int
readServersList(char* serverRoot, FILE *fstream, char *fileName, char* errorInfo)
{
  char		line[1024];
  int		ln;
  int		stop;
  int		t;
  const char    *errorMsg = NULL;

 
  stop=0; ln=0;
  while(!stop)  {
    char *d;
    char *v;

    ++ln;
    switch( (t = admutil_getline(fstream, 1024, ln, line))) {
    case -1:
      fclose(fstream);
      if (i18nResource && 
	  (errorMsg = res_getstring(i18nResource, 
				    DBT_ERROR_READ_FILE,
				    acceptLanguage)))
	sprintf(errorInfo, errorMsg, fileName);
      else sprintf(errorInfo, "Error reading file: %s", fileName);
      return 0;
    case 1:
      fclose(fstream);
      stop=1;
      break;
 
    default:
      d = line;
      v = PL_strchr(line, ':');
      if (v == NULL) break;
      *v++ = '\0';
      if(!installed)  {
	num_installed++;
	installed = (char **) PR_Malloc((num_installed+1)*sizeof(char *));
      }  
      else  {
	int	i;

	for ( i = 0 ; i < num_installed ; ++i ) {
	  if ( PL_strcmp( d, installed[i] ) == 0 ) {
	    break;	/* already read this one */
	  }
	}
	if ( i < num_installed ) {
	  break;	/* already read it */
	}
	num_installed++;
	installed = (char **) PR_Realloc(installed,(num_installed+1)*sizeof(char *));
      }
      installed[num_installed-1] = PL_strdup(d);
      installed[num_installed] = NULL;
      break;
    }
  }
  return 1;
}

char** 
get_server_list(char* oldServerRoot, char* errorInfo) {

  PRDir 	*sr;
  PRDirEntry 	*dirname;
  FILE          *fstream;
  int 		result, num = 0, i, match;
  char  	**servers = NULL;
  char          fileName[256];
  const char    *errMsg = NULL;

  if (!oldServerRoot) return 0;

  sprintf(fileName, "%s/admin-serv/config/%s", oldServerRoot, "servers.lst" );

  if(!(fstream = fopen(fileName, "r"))) {
    /* Not a 3x server Root - try for 2x  */
    sprintf(fileName, "%s/admserv/%s", oldServerRoot, "servers.lst" );
    if(!(fstream = fopen(fileName, "r"))) {
      
      if (i18nResource &&
	  (errMsg = res_getstring(i18nResource, 
				  DBT_ERROR_OPEN_FILE,
				  acceptLanguage)))
	sprintf(errorInfo, errMsg, fileName);
      else 
        sprintf(errorInfo, "Error open file: %s", fileName);
      return 0;
    }
  }
  
  if (!(result = readServersList(oldServerRoot, fstream, fileName, errorInfo))) {
    return NULL;
  }

  sr = PR_OpenDir(oldServerRoot);
  if (!sr) {
    if (i18nResource &&
	(errMsg = res_getstring(i18nResource, 
				DBT_ERROR_OPEN_DIR,
				acceptLanguage)))
      sprintf(errorInfo, errMsg, oldServerRoot);
    else sprintf(errorInfo, "Error open directory: %s", oldServerRoot);
    return NULL;
  }


  while (dirname = PR_ReadDir(sr, PR_SKIP_BOTH)) {
    match = 0;
    if (!PL_strcmp(dirname->name, "admin-serv")) match = 1;
    if (!match && num_installed) {
      for (i=0; i < num_installed; i++) {
	if (!PL_strncmp(dirname->name, installed[i], PL_strlen(installed[i])))  {
	  if (dirname->name[PL_strlen(installed[i])] == '-') match = 1;
	}
      }
    }
    if (match) {
      if (!servers) servers = (char **) PR_Malloc((num+2)*sizeof(char*));
      else servers = (char **) PR_Realloc(servers, (num+2)*sizeof(char*));
      servers[num++] = PL_strdup(dirname->name);
      servers[num] = NULL;
    }
  }
  PR_CloseDir(sr);
  return servers;
}



int main(int argc, char *argv[])
{
  int            _ai = ADMUTIL_Init();
  char           *method;
  int            rv, cnt, i, j, x, errorCode;
  char           *url;
  int            err;
  int            *errp = &err;
  char           *oldSR = 0;
  char           **inputs = 0, **server_list = 0;
  char           *operation = 0;
  char           *qs = 0;
  char           *nameptr, *valptr, *val;
  char           error_info[128];
  int            setFlag = 0, getFlag = 0, forceSetFlag = 0;
  char           resPath[256], *execPath, *tmpptr;
  char           *lang = getenv("HTTP_ACCEPT_LANGUAGE");
  const char     *errMsg = NULL;
#if 0
  int            waitforever = 1;

  while (waitforever);
#endif

  i18nResource = res_init_resource("bin/admin/admin/bin/property",
				   RESOURCE_FILE);
  if (lang) acceptLanguage = strdup(lang);


  memset((void *)errp, 0, sizeof(int));
  method = getenv("REQUEST_METHOD");

  /* Get new port number */
  if (!method || !*method) {
    /* non UI CGI */
    if (i18nResource &&
	(errMsg = res_getstring(i18nResource, 
				DBT_NO_METHOD,
				acceptLanguage)))
      rpt_err(SYSTEM_ERROR, errMsg, NULL, NULL);
    else rpt_err(SYSTEM_ERROR, "No method is specified", NULL, NULL);
  }
  else if (!PL_strcmp(method, "GET")) {
    /* UI CGI - either upgrade or normal admin CGI */
    qs = getenv("QUERY_STRING");
	
    if (!qs || !*qs) {
      if (i18nResource &&
	  (errMsg = res_getstring(i18nResource, 
				  DBT_NO_QUERY_STRING,
				  acceptLanguage)))
	rpt_err(INCORRECT_USAGE, errMsg, NULL, NULL);
      else rpt_err(INCORRECT_USAGE, "NO QUERY_STRING DATA", NULL, NULL);
    }
    else {
      get_begin(qs);
    }
  }
  else if (!PL_strcmp(method, "POST")) {
    post_begin(stdin);
  }
  else {
    if (i18nResource &&
	(errMsg = res_getstring(i18nResource, 
				DBT_UNKNOWN_METHOD, 
				acceptLanguage)))
      sprintf(error_info, errMsg, method);
    else sprintf(error_info, "Unknown Method (%s)", method);
    rpt_err(INCORRECT_USAGE, error_info, NULL, NULL);
  }

  inputs = get_input_ptr();

  oldSR = get_cgi_var("oldServerRoot", NULL, NULL);

  if (!oldSR) {
    if (i18nResource &&
	(errMsg = res_getstring(i18nResource, 
				DBT_DBT_NO_OLD_SERVER_ROOT,
				acceptLanguage)))
       rpt_err(INCORRECT_USAGE, errMsg, NULL, NULL);
    else rpt_err(INCORRECT_USAGE, "No old server root specified", NULL, NULL);
  }

  server_list = get_server_list(oldSR, error_info);

  if (server_list) {
    rpt_success(NULL);
    x = 0;
    fprintf(stdout, "SR:");
    while (server_list[x]) fprintf(stdout, " %s", server_list[x++]);
    fprintf(stdout, "\n");
  }
  else {
    if (i18nResource &&
	(errMsg = res_getstring(i18nResource, 
				DBT_NO_SERVER_FOUND,
				acceptLanguage)))
      rpt_warning(WARNING, errMsg, NULL, NULL);
    else
      rpt_warning(WARNING, "No server found", NULL, NULL);
  }
  if (i18nResource) res_destroy_resource(i18nResource);
}
