/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

/*
 * Description (config.c)
 *
 *	This module contains routines used by administration CGI
 *	programs to set/get the attribute(s) of Admin server
 *
 */
#ifdef XP_UNIX
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#endif

#ifdef XP_UNIX
#   define STRNCASECMP strncasecmp
#   define STRCASECMP  strcasecmp
#   define STRNCMP     strncmp
#   define STRDUP      strdup
#else /* XP_WIN32 */
#   define STRNCASECMP _strnicmp
#   define STRCASECMP  _stricmp
#   define STRNCMP     strncmp
#   define STRDUP      strdup
#endif




#include <prio.h>
#include <stdio.h>
#include <string.h>
#include "plstr.h"
#include "libadminutil/resource.h"
#include "libadminutil/distadm.h"
#include "libadminutil/admutil.h"
#include "libadmsslutil/psetcssl.h"
#include "libadmsslutil/admsslutil.h"

#include "libadminutil/resource.h"

static int update_adm_conf(char *newport);
static int update_magnus_conf(char *newport);
/*
  properties file name = "config.properties"
 */
#define RESOURCE_FILE "migrateconfig"

#define resource_key(a,b)   a b

#define DBT_NO_METHOD 			    resource_key(RESOURCE_FILE, "1")
#define DBT_UNKNOWN_METHOD 		    resource_key(RESOURCE_FILE, "2")
#define DBT_NO_QUERY_STRING 	    resource_key(RESOURCE_FILE, "3")
#define DBT_ERROR_OPEN_FILE 		resource_key(RESOURCE_FILE, "4")
#define DBT_ERROR_READ_FILE 		resource_key(RESOURCE_FILE, "5") 
#define DBT_ERROR_OPEN_DIR 	        resource_key(RESOURCE_FILE, "6") 
#define DBT_NO_OLD_SERVER_ROOT 	    resource_key(RESOURCE_FILE, "7") 
#define DBT_NO_SERVER_FOUND 		resource_key(RESOURCE_FILE, "8") 
#define DBT_NO_USER_NAME 			resource_key(RESOURCE_FILE, "9") 
#define DBT_NO_USER_DN 		        resource_key(RESOURCE_FILE, "10") 
#define DBT_PSET_CREATE_ERROR 	    resource_key(RESOURCE_FILE, "11") 
#define DBT_PSET_SET_ERROR 		    resource_key(RESOURCE_FILE, "12") 
#define DBT_PSET_ADD_ERROR 		    resource_key(RESOURCE_FILE, "13") 
#define DBT_SSL_INIT_ERROR 		    resource_key(RESOURCE_FILE, "14") 

#define CONVERT_NUMBER 8

char*
getAttributeName(char* oldName) {

  if (!PL_strcasecmp(oldName, "port")) return "configuration.nsServerPort";
  else if (!PL_strcasecmp(oldName, "user")) return "configuration.nsSuiteSpotUser";
  else if (!PL_strcasecmp(oldName, "servername")) return "serverhostname";
  else if (!PL_strcasecmp(oldName, "hosts")) return "configuration.nsAdminAccesshosts";
  else if (!PL_strcasecmp(oldName, "addresses")) return "configuration.nsAdminAccessaddresses";
  else if (!PL_strcasecmp(oldName, "CGIWaitPid")) return "configuration.nsAdminCGIWaitPid";
  else if (!PL_strcasecmp(oldName, "DefaultLanguage")) return "configuration.nsdefaultacceptlanguage.";
  else return NULL;
}

int main(int argc, char *argv[])
{
  int            _ai = ADMUTIL_Init();
  FILE           *fstream;
  char           fileName[256];
  int		 ln;
  int		 stop;
  int		 t;
  char           *oldSR = 0;
  char           *newSR = getenv("NETSITE_ROOT");
  char           error_info[128];
  PsetHndl       pset;
  char           *method;
  int            rv, cnt, i, j, x, errorCode;
  char           *url;
  int            err;
  int            *errp = &err;
  char           *username = 0;
  char           *localAdmin = 0;
  char           *binddn = 0;
  char           *bindpw = 0;
  char           **inputs = 0;
  char           *qs = 0;
  char           *nameptr, *valptr, *val;
  int            setFlag = 0, getFlag = 0, forceSetFlag = 0;
  Resource       *i18nResource = NULL;
  char           *acceptLanguage = "en", *lang=getenv("HTTP_ACCEPT_LANGUAGE");
  AttributeList  resultList, updateList=NULL, addList=NULL;
  AttrNameList   nl;
  AttributePtr   nv;
  char           resPath[256], buf[1024], cpcmd[1024], *execPath, *tmpptr;
  const char     *errMsg = NULL;
  PRFileInfo     fileInfo;
#if 0
  int            waitforever = 1;

  while (waitforever);
#endif

  execPath = getcwd(resPath, 256);
  if (execPath) {
    strcpy(resPath, execPath);
    strcat(resPath, "/property");
    i18nResource = res_init_resource(resPath, RESOURCE_FILE);
  }

  if (lang) acceptLanguage = strdup(lang);

  memset((void *)errp, 0, sizeof(int));
  method = getenv("REQUEST_METHOD");

  /* Get new port number */
  if (!method || !*method) {
    /* non UI CGI */
    if (i18nResource &&
	(errMsg = res_getstring(i18nResource, DBT_NO_METHOD, acceptLanguage))) 
      rpt_err(SYSTEM_ERROR, errMsg, NULL, NULL);
    else rpt_err(SYSTEM_ERROR, "No method is specified", NULL, NULL);
  }
  else if (!strcmp(method, "GET")) {
    /* UI CGI - either upgrade or normal admin CGI */
    qs = getenv("QUERY_STRING");
	
    if (!qs || !*qs) {
      if (i18nResource &&
	  (errMsg = res_getstring(i18nResource, 
				  DBT_NO_QUERY_STRING, 
				  acceptLanguage)))
	rpt_err(INCORRECT_USAGE, errMsg, NULL, NULL);
      else rpt_err(INCORRECT_USAGE, "NO QUERY_STRING DATA", NULL, NULL);
    }
    else {
      get_begin(qs);
    }
  }
  else if (!strcmp(method, "POST")) {
    post_begin(stdin);
  }
  else {
    if (i18nResource &&
	(errMsg = res_getstring(i18nResource,
				DBT_UNKNOWN_METHOD, 
				acceptLanguage)))
      sprintf(error_info, errMsg, method);
    else sprintf(error_info, "Unknown Method (%s)", method);
    rpt_err(INCORRECT_USAGE, error_info, NULL, NULL);
  }

  /* Get UserDN and User Password  */

  rv = ADM_GetUserDNString(&err, &binddn);
  if (rv < 0 || !binddn || !*binddn) {
    rv = ADM_GetCurrentUsername(&err, &username);
    if (rv < 0 || !username || !*username) {
      if (i18nResource &&
	  (errMsg = res_getstring(i18nResource,
				  DBT_NO_USER_NAME,
				  acceptLanguage)))
	 rpt_err(ELEM_MISSING, errMsg, NULL, NULL);
      else rpt_err(ELEM_MISSING, "NO USER NAME", NULL, NULL);
    }
    else {
      /* No DN, maybe it is local super */
      localAdmin = admGetLocalAdmin(NULL, &rv);
      if (localAdmin) {
	if (strcmp(username, localAdmin)) {
	  if (i18nResource &&
	      (errMsg = res_getstring(i18nResource, 
				      DBT_NO_USER_DN,
				      acceptLanguage)))
	    rpt_err(ELEM_MISSING,errMsg, NULL, NULL);
	  else rpt_err(ELEM_MISSING, "NO USER DN!", NULL, NULL);
	}
	else {
	  binddn = NULL;
	  bindpw = NULL;
	}
      }
      else {
	if (i18nResource &&
	    (errMsg = res_getstring(i18nResource,
				    DBT_NO_USER_DN,
				    acceptLanguage)))
	  rpt_err(ELEM_MISSING, errMsg, NULL, NULL);
	else rpt_err(ELEM_MISSING, "NO USER DN!", NULL, NULL);
      }
    }
  }

  if (binddn) rv = ADM_GetCurrentPassword(&err, &bindpw);
  rv = ADMSSL_InitSimple();
  if (rv) {
    if (i18nResource &&
	(errMsg = res_getstring(i18nResource,
				DBT_SSL_INIT_ERROR,
				acceptLanguage)))
      rpt_err(APP_ERROR, errMsg, NULL, NULL);
    else rpt_err(APP_ERROR, "SSL related initialization failed", NULL, NULL);
  }

  /* Initialize the pset  */

  pset = psetCreateSSL("admin-serv", 
		       /* configRoot */ NULL,
		       /* userDN */ binddn, 
		       /* passwd */ bindpw,
		       /* errorcode */ &rv);

  if (!pset) {
    sprintf(error_info,
	    "PSET_ERROR_NUMBER: %d\nPSET_ERROR_INFO: %s", 
	    rv,
	    psetErrorString(rv, NULL));
    if (i18nResource &&
	(errMsg = res_getstring(i18nResource,
				DBT_PSET_CREATE_ERROR,
				acceptLanguage))) 
      rpt_err(APP_ERROR, errMsg, NULL, error_info);
    else rpt_err(APP_ERROR, "PSET Creation Failed", NULL, error_info);
  }

  /* Process the input  */

  inputs = get_input_ptr();

  oldSR = get_cgi_var("oldServerRoot", NULL, NULL);

  if (!oldSR) {
    if (i18nResource &&
	(errMsg = res_getstring(i18nResource,
				DBT_NO_OLD_SERVER_ROOT,
				acceptLanguage)))
      rpt_err(INCORRECT_USAGE, errMsg, NULL, NULL);
    else rpt_err(INCORRECT_USAGE, "No old server root specified", NULL, NULL);
  }

  sprintf(fileName, "%s/admin-serv/config/ns-admin.conf", oldSR);

  if(!(fstream = fopen(fileName, "r"))) {
    psetDelete(pset);
    if (i18nResource &&
	(errMsg = res_getstring(i18nResource,
				DBT_ERROR_OPEN_FILE,
				acceptLanguage)))
      sprintf(error_info, errMsg, fileName);
    else sprintf(error_info, "Error open file: %s", fileName);
    rpt_err(SYSTEM_ERROR, error_info, NULL, NULL);
  }

  updateList = createAttributeList(CONVERT_NUMBER);
  if (forceSetFlag) addList = createAttributeList(CONVERT_NUMBER);
  i = 0; j = 0;

  stop=0; ln=0;
  while(!stop)  {
    ++ln;
    switch( (t = admutil_getline(fstream, 1024, ln, buf))) {
    case -1:
      fclose(fstream);
      psetDelete(pset);
      if (i18nResource &&
	  (errMsg = res_getstring(i18nResource,
				  DBT_ERROR_OPEN_FILE,
				  acceptLanguage)))
	sprintf(error_info, errMsg, fileName);
      else sprintf(error_info, "Error reading file: %s", fileName);
      rpt_err(SYSTEM_ERROR, error_info, NULL, NULL);
      return 0;
    case 1:
      fclose(fstream);
      stop=1;
      break;
 
    default:
      valptr = strchr(buf, ' ');
      if (valptr) {
	*valptr = '\0';
	valptr++;
	nameptr = getAttributeName(buf);
      
	if (nameptr) {
	  errorCode = PSET_OP_OK;
	  val = psetGetAttrSingleValue(pset, nameptr, &errorCode);
	  if (errorCode) {
	    if (forceSetFlag) {
	      addSingleValueAttribute(addList, j++, nameptr, valptr);
	    }
	    else j++;
	  }
	  else {
	    addSingleValueAttribute(updateList, i++, nameptr, valptr);
	  }
	  if (!PL_strcmp(nameptr, "configuration.nsServerPort")) {
	    err = update_adm_conf(valptr);
	    if(err >= 0)
	      err = update_magnus_conf(valptr);
	    if (err < 0) {
              psetDelete(pset);
	      if (i18nResource &&
		  (errMsg = res_getstring(i18nResource,
					  DBT_ERROR_OPEN_FILE,
					  acceptLanguage)))
		sprintf(error_info, errMsg, "adm.conf");
	      else sprintf(error_info, "Error open file: adm.conf");
	      rpt_err(SYSTEM_ERROR, error_info, NULL, NULL);
	    }
	    val = psetGetAttrSingleValue(pset, "adminurl", &errorCode);
	    if (val) {
	      char  urlBuf[256];
	      char  *tmpptr;
	      PL_strcpy(urlBuf, val);
	      tmpptr = PL_strrchr(urlBuf, ':');
	      if (tmpptr) {
		tmpptr++;
		*tmpptr = '\0';
		PL_strcat(urlBuf, valptr);
		addSingleValueAttribute(updateList, i++, "adminurl", urlBuf);
	      }
	    }
	  }
	}
      }
    }
  }
  
  errorCode = PSET_OP_OK;
  if (i) errorCode = psetSetAttrList(pset, updateList);
  if (errorCode) {
    psetDelete(pset);
    sprintf(error_info,
	    "PSET_ERROR_NUMBER: %d\nPSET_ERROR_INFO: %s\n", 
	    errorCode,
	    psetErrorString(errorCode, NULL));
    if (i18nResource &&
	(errMsg = res_getstring(i18nResource, 
				DBT_PSET_SET_ERROR, 
				acceptLanguage))) 
      rpt_err(APP_ERROR, errMsg, NULL, error_info);
    else rpt_err(APP_ERROR, "PSET Set Failed", NULL, error_info);
  }

  if (j) {
    for (i=0; i < j; i++) {
      errorCode = psetAddAttribute(pset, 
				   addList[i]->attrName, 
				   addList[i]->attrVal);
      if (errorCode) {
	psetDelete(pset);
	sprintf(error_info,
		"PSET_ERROR_NUMBER: %d\nPSET_ERROR_INFO: %s\n", 
		errorCode,
		psetErrorString(errorCode, NULL));
	if (i18nResource &&
	    (errMsg = res_getstring(i18nResource, 
				    DBT_PSET_ADD_ERROR, 
				    acceptLanguage)))
	  rpt_err(APP_ERROR, errMsg, NULL, error_info);
	else rpt_err(APP_ERROR, "PSET Add Failed", NULL, error_info);
      }
    }
  }
  rpt_success(NULL);

  sprintf(fileName, "%s/userdb/certmap.conf", oldSR);
  if (PR_GetFileInfo(fileName, &fileInfo) == PR_SUCCESS) {
    if ((fileInfo.type == PR_FILE_FILE) && newSR) {
      sprintf(cpcmd, "cp %s %s/shared/config/certmap.conf", fileName, newSR);
      system(cpcmd);
    }
  }
  if (updateList) deleteAttributeList(updateList);
  if (addList) deleteAttributeList(addList);
  psetDelete(pset);
  if (i18nResource) res_destroy_resource(i18nResource);
}


/*
 * Modify Port in adm.conf
 */
static int update_adm_conf(char *newport) {

  FILE *f;
  int i, modified=0;
  static char filename[BIG_LINE];
  static char inbuf[BIG_LINE];
  static char buf[BIG_LINE];
  int linecnt=0;	
  char *lines[50];
  int port;
  const char     *errMsg = NULL;
  char           error_info[128];

  sprintf(filename, "%s/adm.conf", getenv("ADMSERV_ROOT"));

  f = fopen(filename, "r");
  if (f==NULL) return -1;
  
  while(fgets(inbuf, sizeof(inbuf), f) != NULL) {
    if (STRNCASECMP(inbuf,"port:",5) == 0) { /* Line starts with "Port:" */
      sprintf(buf, "port: %s\n", newport);
      lines[linecnt++] = strdup(buf);
      modified=1;
    }
    else {
      lines[linecnt++] = strdup(inbuf);
    }
  }	
  fclose(f);

  if (!modified) { /* Add the Port attribute */
    sprintf(buf, "Port: %s\n", newport);
    lines[linecnt++] = strdup(buf);
  }

  f = fopen(filename, "w");
  if (f==NULL) return -1;
  
  for (i=0; i < linecnt; i++) {
    fprintf(f, "%s", lines[i]);
  }

  fclose(f);
  
  return 0;
}


/*
 * Modify Port in magnus.conf
 */
static int update_magnus_conf(char *newport) {

  FILE *f;
  int i, modified=0;
  static char filename[BIG_LINE];
  static char inbuf[BIG_LINE];
  static char buf[BIG_LINE];
  int linecnt=0;	
  char *lines[50];
  int port;
  const char     *errMsg = NULL;
  char           error_info[128];

  sprintf(filename, "%s/magnus.conf", getenv("ADMSERV_ROOT"));

  f = fopen(filename, "r");
  if (f==NULL) return -1;
  
  while(fgets(inbuf, sizeof(inbuf), f) != NULL) {
    if (STRNCASECMP(inbuf,"Port",4) == 0) { /* Line starts with "Port:" */
      sprintf(buf, "Port %s\n", newport);
      lines[linecnt++] = strdup(buf);
      modified=1;
    }
    else {
      lines[linecnt++] = strdup(inbuf);
    }
  }	
  fclose(f);

  if (!modified) { /* Add the Port attribute */
    sprintf(buf, "Port %s\n", newport);
    lines[linecnt++] = strdup(buf);
  }

  f = fopen(filename, "w");
  if (f==NULL) return -1;
  
  for (i=0; i < linecnt; i++) {
    fprintf(f, "%s", lines[i]);
  }

  fclose(f);
  
  return 0;
}
