/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * monreplication.c:  Generate form for replication monitoring
 */

#include "libadminutil/resource.h"
#include "libadminutil/distadm.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <signal.h>
#include <sys/types.h>
#include "libadmin/libadmin.h"

#define MY_PAGE "monreplication.html"
#define DEF_SIZE "25"

/*
 * i18n conversions defines and function;
 * properties file  = "monreplication.properties"
 */
#define RESOURCE_FILE "monreplication"
#define resource_key(a,b)   a b

#define DBT_REFRESH_URL		resource_key(RESOURCE_FILE, "1")
#define DBT_REFRESH_INTERVAL	resource_key(RESOURCE_FILE, "2")
#define DBT_CONFIG_FILE		resource_key(RESOURCE_FILE, "3")

static  char *acceptLanguage = (char*)"en";
static  Resource *i18nResource = NULL;

char*
getResourceString(char *key) {
	return (char *)(res_getstring(i18nResource, key, acceptLanguage));
}

static void i18nInit() {
	char resPath[256], *execPath;
	execPath = getcwd(resPath, 256);
	if (execPath) {
		strcpy(resPath, execPath);
		strcat(resPath, "/property");
		i18nResource = res_init_resource(resPath, RESOURCE_FILE);
	}
	if (getenv("HTTP_ACCEPT_LANGUAGE")) {
		acceptLanguage = getenv("HTTP_ACCEPT_LANGUAGE");
	}
}


int main(int argc, char *argv[])
{
	int _ai = ADMUTIL_Init();
	char *qs = getenv("QUERY_STRING");
	char line[BIG_LINE];
	FILE *html = open_html_file(MY_PAGE);
	char *host;
	char *port;
	char *admurl;
	char *siteroot;
	char refreshurl[256] = {'\0'};
	char refreshinterval[16];
	char configfile[256] = {'\0'};
	char *resstr;

	i18nInit();

	fprintf(stdout, "Content-type: text/html;charset=utf-8\n\n");

	if(qs)  {
		get_begin(qs);
		host = get_cgi_var("servhost", NULL, NULL);
		port = get_cgi_var("servport", NULL, NULL);
		admurl = get_cgi_var("admurl", NULL, NULL);
		siteroot = get_cgi_var("siteroot", NULL, NULL);
	}

	
	if ((resstr = getResourceString(DBT_REFRESH_URL))) {
		sprintf(refreshurl, resstr, admurl ? admurl : "");
	}
	if ((resstr = getResourceString(DBT_REFRESH_INTERVAL))) {
		sprintf(refreshinterval, resstr);
	}
	else {
		sprintf(refreshinterval, "300");
	}
	if ((resstr = getResourceString(DBT_CONFIG_FILE))) {
		sprintf(configfile, resstr);
	}

	while(next_html_line(html, line))  {
		if(parse_line(line, NULL))  {
			if(directive_is(line, "SERVHOST"))  {
				output_input("hidden", "servhost", host, NULL);
			}
			else if(directive_is(line, "SERVPORT"))  {
				output_input("hidden", "servport", port, NULL);
			}
			else if(directive_is(line, "ADMURL"))  {
				output_input("hidden", "admurl", refreshurl, NULL);
			}
			else if(directive_is(line, "SITEROOT"))  {
				output_input("hidden", "siteroot", siteroot, NULL);
			}
			else if(directive_is(line, "CONFIGFILE"))  {
				output_input("text", "configfile", configfile, "size=80");
			}
			else if(directive_is(line, "REFRESHINTERVAL"))  {
				output_input("text", "refreshinterval", refreshinterval, NULL);
			}
			else
		    		fputs(line, stdout);
		}  
	}

	return 0;	
}
