/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

/***********************************************************************
** Includes
***********************************************************************/

#include	<stdio.h>
#include	<stdlib.h>
#include	<limits.h>
#include	<string.h>

#ifdef XP_UNIX
#include	<unistd.h>
#define PERL "/install/perl"
#define PATH_SEP '/'
#ifndef PATH_MAX
#define PATH_MAX 512
#endif
#else
#include	<direct.h>
#include	<process.h>
#define PERL "\\install\\perl.exe"
#define PATH_SEP '\\'
#define PATH_MAX 512
#endif

static void
fixSlashes( char *path )
{
#ifdef XP_WIN32
  while ( *path ) {
    if ( *path == '/' ) {
      *path = PATH_SEP;
    }
    ++path;
  }
#endif
}

static void
bong(char *p)
{
   printf("content-type: text/html\n\n");
   printf("unable to find %s\n", p);
   exit(0);
}

/*
 * Use environment to figure out what admin perl script to execute
 */

main( int argc, char **argv )
{
  char perl[PATH_MAX];
  char script[PATH_MAX];
  char dir[PATH_MAX];
  char *p, *tmpptr, *qs, *reqMethod;

  /* Get path to perl executable */
  if (!(p = getenv("NETSITE_ROOT"))) bong("NETSITE_ROOT");
  strcpy(perl, p);
  strcat(perl, PERL);

  /* Test to see if our local perl exists, if it doesn't then use the system perl */
  /* This is for Linux, where we want to use the system's Perl in /usr/bin/perl */
  /* We want to do that, in turn, because the historical version of perl that */
  /* we package on the other platforms no longer works on recent Linux versions */
  /* In the future we should try to use the system perl on other platforms, but */
  /* they might not be located at /usr/bin/perl, so it may be necessary to introduce */
  /* a generic mechanism for finding the local system perl binary */
#if defined(WIN32)
#define ACCESS_FN _access
#define ACCESS_CHECK 04
#else
#define ACCESS_FN access
#define ACCESS_CHECK X_OK
#endif
  if (0 != ACCESS_FN(perl,ACCESS_CHECK)) {
	  /* No local perl, use the system perl */
	  strcpy(perl,"/usr/bin/perl");
  }

  getcwd(dir, PATH_MAX); 

  if (!(reqMethod = getenv("REQUEST_METHOD"))) bong("REQUEST_METHOD");
  if (!strcmp(reqMethod, "GET")) {
    if (!(qs = getenv("QUERY_STRING"))) bong("QUERY_STRING");
    sprintf(script, "%s%c%s", dir, PATH_SEP, qs);
  }
  else if (!strcmp(reqMethod, "POST")) {
     sprintf(script, "%s%c%s", dir, PATH_SEP, argv[1]);
  }
  else bong("WRONG_REQUEST_METHOD");

  if (tmpptr = strchr(script, '&')) *tmpptr = '\0';

  fixSlashes(perl);
  fixSlashes(script);
  fixSlashes(dir);

  execl(perl, script, script, 0);
}
