/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * statpingserv.c - determine the on/off status of any server.
 *
 * Takes a host and port, and the result can either be an
 * on or off icon, or text describing status.
 *
 * All blame to Adam Prishtina (adam@netscape.com)
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <base/util.h>
#include "libadminutil/distadm.h"
#include "libadminutil/resource.h"

#ifdef __cplusplus
extern "C" {
#endif

#include "libadmsslutil/certmgt.h"
#include "libadmsslutil/psetcssl.h"
#include "libadmsslutil/admsslutil.h"
#include "key.h"
#include "cert.h"
#include "secport.h"
#include "secmod.h"
#include "prnetdb.h"

#ifdef __cplusplus
}
#endif


int main(int argc, char *argv[])
{
  int _ai = ADMUTIL_Init();
  char *m;
  char msg[BIG_LINE];

  char *qs = getenv("QUERY_STRING");
  PRHostEnt   hent;
  char        buf[PR_NETDB_BUF_SIZE];
  PRStatus    err;

  PRFileDesc *req_socket= NULL;
  int retcode;
  PRNetAddr netAddr;

  char *result;
  char *host;
  int port;

  FILE *gif;
  int character;
  int count=0;
  char *file;

  m = getenv("REQUEST_METHOD");
  /*
    {
    int d=1;
    while(d>0)
    {;}
    }
  */

  if(!strcmp(m, "GET")) {
    if(qs)
      get_begin(qs);

    result = get_cgi_var("result", NULL, NULL);
    host = get_cgi_var("host", NULL, NULL);
    port = atoi(get_cgi_var("port", NULL, NULL));

    if(result && !strcmp(result, "text"))
      fprintf(stdout, "Content-type: text/html\n");
    else
      fprintf(stdout, "Content-type: image/gif\n");

    if(!isdigit(host[0]))  {
      err = PR_GetHostByName(host, buf, sizeof(buf), &hent);
      if(err == PR_FAILURE)
	rpt_err(APP_ERROR, "Cannot resolve host name!", NULL, NULL);
    }

    PR_InitializeNetAddr(PR_IpAddrNull, (PRUint16)port, &netAddr);
    memcpy((void *) &netAddr.inet.ip, (void const *) hent.h_addr_list[0], sizeof(netAddr.inet.ip));	

    req_socket = PR_NewTCPSocket();
      
    retcode = PR_Connect(req_socket, &netAddr, 10000);

    if (retcode != 0) {
      /* SERVER IS DOWN - output the OFF button */
      if(result && !strcmp(result, "text")) {
	rpt_success("The server is DOWN.");
	return 1;
      }
      file = strdup("../icons/togoff.gif");
    }
    else {
      /* SERVER IS RUNNING - output of ON button */
      if(result && !strcmp(result, "text")) {
	rpt_success("The server is UP.");
	return 1;
      }
      file = strdup("../icons/togon.gif");
    }

    PR_Close(req_socket);

    gif = fopen(file, "rb");
    if(!gif)
      rpt_err(APP_ERROR, "Cannot open gif file!", NULL, NULL);
    while((character = fgetc(gif)) != EOF)
      count++;
    fclose(gif);

    fprintf(stdout, "Content-length: %d\n\n", count);

    gif = fopen(file, "rb");
    if(!gif)
      rpt_err(APP_ERROR, "Cannot open gif file!", NULL, NULL);
    while((character = fgetc(gif)) != EOF)
      fputc(character, stdout);
    fclose(gif);
  }

  return 0;
}

