/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * viewlog.c:  View various log files in HTML Admin interface.
 * 
 * Initial version: 3/11/99 by Adam Prishtina
 */

#include "libadminutil/resource.h"
#include "libadminutil/distadm.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <signal.h>
#include <sys/types.h>
#include "libadmin/libadmin.h"

#define MY_PAGE "viewlog.html"
#define DEF_SIZE "25"

/*
 * i18n conversions defines and function;
 * properties file  = "viewlog.properties"
 */
#define RESOURCE_FILE "viewlog"
#define resource_key(a,b)   a b

#define DBT_INV_NUMBER 			resource_key(RESOURCE_FILE, "1")
#define DBT_INV_NUMBER_DESC 		resource_key(RESOURCE_FILE, "2")
#define DBT_SUBTITLE 		        resource_key(RESOURCE_FILE, "3")
#define DBT_WITH   		        resource_key(RESOURCE_FILE, "4")
#define DBT_NO_DIR       		resource_key(RESOURCE_FILE, "5")
#define DBT_NO_DIR_DESC   		resource_key(RESOURCE_FILE, "6")
#define DBT_NO_FILE   		        resource_key(RESOURCE_FILE, "7")
#define DBT_NO_FILE_DESC   		resource_key(RESOURCE_FILE, "8")
#define DBT_NO_DIR_FOUND                resource_key(RESOURCE_FILE, "9")
#define DBT_NO_ID        		resource_key(RESOURCE_FILE, "10")
#define DBT_NO_ID_DESC                  resource_key(RESOURCE_FILE, "11")


/* html resource */
#define DBT_DISPLAY_LOGFILE_SELECT_BEGIN    resource_key(RESOURCE_FILE, "20")
#define DBT_DISPLAY_LOGFILE_VALUE_SELECTED  resource_key(RESOURCE_FILE, "21")
#define DBT_DISPLAY_LOGFILE_SELECT_VALUE    resource_key(RESOURCE_FILE, "22")
#define DBT_DISPLAY_LOGFILE_NO_LOG          resource_key(RESOURCE_FILE, "23")
#define DBT_DISPLAY_LOGFILE_SELECT_END      resource_key(RESOURCE_FILE, "24")
#define DBT_MAIN_HIDDEN_TRACK               resource_key(RESOURCE_FILE, "25")
#define DBT_MAIN_ID_TITLE                   resource_key(RESOURCE_FILE, "26")
#define DBT_MAIN_WIDTH                      resource_key(RESOURCE_FILE, "27")
#define DBT_MAIN_TABLE                      resource_key(RESOURCE_FILE, "28")

static  char *acceptLanguage = (char*)"en";
static  Resource *i18nResource = NULL;

char*
getResourceString(char *key) {
  return (char *)(res_getstring(i18nResource, key, acceptLanguage));
}

void search_file(FILE *cmd, int num, char *str) {

  int replace, count, temp;
  char line[BIG_LINE];
  char **buffer = NULL; /* buffer to store log matches */

  if(num == 0) return;

  replace=0;
  count=0;
  while(fgets(line, BIG_LINE, cmd)) {
    if(strncmp(line, "format=", 7)) 
      if((!str) || (strstr(line, str))) {
	count++;
	if(count<=num) {
	  if(!buffer)
	    buffer = (char **)MALLOC(sizeof(char *));
	  else
	    buffer = (char **)REALLOC(buffer, count*(sizeof(char *)));
	}
	else
	  FREE(buffer[replace]);
	temp = strlen(line);
	line[temp-1] = ' ';
	line[temp] = '\n';
	line[temp+1] = '\0';
	buffer[replace] = STRDUP(line);
	if(replace == (num-1))
	  replace=0;
	else replace++;
      }
  }
  if(count < num) { /* could not find num entries */
    replace=0;
  }
  else count=num;
  
  /* replace is at the first entry to output */
  for(temp=0; temp<count; temp++) {
    fputs(buffer[replace], stdout);		  
    FREE(buffer[replace]);
    if(replace == (num-1))
      replace=0;
    else replace++;
  }
  if(buffer)
    FREE(buffer);
}

static void i18nInit() {
	char resPath[256], *execPath;
	execPath = getcwd(resPath, 256);
	if (execPath) {
		strcpy(resPath, execPath);
		strcat(resPath, "/property");
		i18nResource = res_init_resource(resPath, RESOURCE_FILE);
	}
	if (getenv("HTTP_ACCEPT_LANGUAGE")) {
		acceptLanguage = getenv("HTTP_ACCEPT_LANGUAGE");
	}
}


void display_logfiles(char *dirs, char *default_logfile)
{
    int  x; 
    int  cnt=0;
    int  remcnt=0;
    char *seps = "/";
    char *token; 
    char *curlog = NULL;
    char *keyword = NULL;
    char **logfiles;
    char full_directory[PATH_MAX];
    char *current_dir;
    char temp_dirs[BIG_LINE];
    int at_least_one=0;

    if (default_logfile != NULL) {
      token = strtok(default_logfile, seps);
      while(token != NULL) {
        curlog = token;
        token = strtok(NULL, seps);
      }
    }
    fprintf(stdout, getResourceString(DBT_DISPLAY_LOGFILE_SELECT_BEGIN));

    strcpy(temp_dirs, dirs);
    current_dir = strtok(temp_dirs, "|");

    while(current_dir) {
      sprintf(full_directory, "%s%c%s",
	      getenv("NETSITE_ROOT"),
	      FILE_PATHSEP,
	      current_dir);

      logfiles = list_directory(full_directory, 0);
      
      for(x=0; logfiles && logfiles[x]; x++) {
	if (curlog && !strcmp(curlog, logfiles[x])) {
	  fprintf(stdout, getResourceString(DBT_DISPLAY_LOGFILE_VALUE_SELECTED),
		  logfiles[x], logfiles[x]);
	}
	else {
	  fprintf(stdout, getResourceString(DBT_DISPLAY_LOGFILE_SELECT_VALUE),
		  logfiles[x], logfiles[x]);
	}
	at_least_one = 1;
      }
      
      current_dir = strtok(NULL, "|");
    }

    if(!at_least_one)
      fprintf(stdout, getResourceString(DBT_DISPLAY_LOGFILE_NO_LOG));

    fprintf(stdout, getResourceString(DBT_DISPLAY_LOGFILE_SELECT_END));

}


int already_exists(char *list, char *element) {
  char *temp = strdup(list);
  char *ptr;

  ptr = strtok(temp, "|");
  while(ptr) {
    if(!strcmp(ptr, element))
      return 1;
    ptr = strtok(NULL, "|");
  }
  return 0;
}


/*
 * Given a list of log directories, find the one containing file.
 */

char *get_current_dir(char *directories, char *file) {

  char temp[BIG_LINE];
  char path[PATH_MAX];
  char *ptr;
  char **list;
  int i;

  if(!file)
    return NULL;

  strcpy(temp, directories);

  ptr = strtok(temp, "|");
  while(ptr != NULL) {
    sprintf(path, "%s%c%s",
	    getenv("NETSITE_ROOT"),
	    FILE_PATHSEP,
	    ptr);
    list = list_directory(path, 0);
    i=0;
    while(list && list[i] != NULL) {
      if(!strcmp(list[i], file))
	return strdup(ptr);
      i++;
    }
    ptr = strtok(NULL, "|");
  }

  return NULL;
}


/*
 * Given the server's ID, find the directories containing all log files for that instance.
 * This is hardcoded for each 4.0/4.1 server product type.
 */

char *guess_directories(char *id) {

  char config_file[PATH_MAX];
  FILE *file;
  char inbuf[BIG_LINE];
  char *return_dirs = NULL;
  char *ptr, *ptr2;

  char *errorlog_attr = NULL;
  char *accesslog_attr = NULL;

  if(!strncmp(id, "slapd-", 6)) {
    /* DIRECTORY SERVER */
    sprintf(config_file, "%s%c%s%cconfig%cslapd.conf",
	    getenv("NETSITE_ROOT"),
	    FILE_PATHSEP,
	    id,
	    FILE_PATHSEP,
	    FILE_PATHSEP);
    file = fopen(config_file, "r");
    if(!file) {
      /* This might be a 5.0 version, get it from dse.ldif */
      sprintf(config_file, "%s%c%s%cconfig%cdse.ldif",
	      getenv("NETSITE_ROOT"),
	      FILE_PATHSEP,
	      id,
	      FILE_PATHSEP,
	      FILE_PATHSEP);
      errorlog_attr = strdup("nsslapd-errorlog:");
      accesslog_attr = strdup("nsslapd-accesslog:");      
    }
    else {
      errorlog_attr = strdup("errorlog\t");
      accesslog_attr = strdup("accesslog\t");
    }
  }
  else if(!strncmp(id, "https-", 6)) {
    /* ENTERPRISE SERVER */
    sprintf(config_file, "%s%c%s%cconfig%cmagnus.conf",
	    getenv("NETSITE_ROOT"),
	    FILE_PATHSEP,
	    id,
	    FILE_PATHSEP,
	    FILE_PATHSEP);
    errorlog_attr = strdup("ErrorLog\t");
  }
  else if(!strncmp(id, "cert-", 5)) {
    /* CERT SERVER */
    sprintf(config_file, "%s%c%s%cconfig%cCMS.cfg",
	    getenv("NETSITE_ROOT"),
	    FILE_PATHSEP,
	    id,
	    FILE_PATHSEP,
	    FILE_PATHSEP);
    errorlog_attr = strdup("logError.fileName=");
    accesslog_attr = strdup("logAudit.fileName=");
  }

  file = fopen(config_file, "r");
  if(!file)
    return NULL;
  while(fgets(inbuf, sizeof(inbuf), file)) {
    if((accesslog_attr && strstr(inbuf, accesslog_attr)) || 
       (errorlog_attr && strstr(inbuf, errorlog_attr))) {
      int x;
      char *net_root = getenv("NETSITE_ROOT");
      if(strchr(net_root, '/')) {
         if(FILE_PATHSEP == '\\') {
            /* replace */
            x=0;
            while(net_root[x] != '\0') {
               if(net_root[x] == '/')
                  net_root[x] = FILE_PATHSEP;
               x++;
            }
         }
      }
      else {
         if(FILE_PATHSEP == '/') {
            /* replace */
            x=0;
            while(net_root[x] != '\0') {
               if(net_root[x] == '\\')
                  net_root[x] = FILE_PATHSEP;
               x++;
            }
         }
      }      

      ptr = strstr(inbuf, net_root);
      ptr = &(ptr[strlen(net_root)+1]);
      
      ptr2 = strrchr(ptr, FILE_PATHSEP);
      ptr2[0] = '\0';
      
      if(!return_dirs)
	return_dirs = strdup(ptr);
      else {
	if(!already_exists(return_dirs, ptr)) {
	  return_dirs = (char *)realloc(return_dirs, (strlen(return_dirs) + strlen(ptr) + 1));
	  sprintf(return_dirs, "%s|%s", return_dirs, ptr);
	}
      }
    }
  }
  return return_dirs;
}


int main(int argc, char *argv[])
{

    int _ai = ADMUTIL_Init();
    char *qs = getenv("QUERY_STRING");
    char line[BIG_LINE];
    FILE *html = open_html_file(MY_PAGE);
    char *num=NULL;
    char *str=NULL;
    char *file;
    char *directories;
    char *id;
    char log_dir[PATH_MAX];
    char **logfiles = NULL;
    char tmp[BIG_LINE];
    int x;

    i18nInit();

    fprintf(stdout, "Content-type: text/html;charset=utf-8\n\n");

    if(qs)  {
       get_begin(qs);

       num=get_cgi_var("num", NULL, NULL);
       str=get_cgi_var("str", NULL, NULL);
       file=get_cgi_var("file", NULL, NULL);
       directories=get_cgi_var("directories", NULL, NULL);
       id=get_cgi_var("id", NULL, NULL);

       if(!id)
	 rpt_err(INCORRECT_USAGE,
		 getResourceString(DBT_NO_ID),
		 getResourceString(DBT_NO_ID_DESC), NULL);

       if((!directories) && (!id))
	 rpt_err(INCORRECT_USAGE,
		 getResourceString(DBT_NO_DIR),
		 getResourceString(DBT_NO_DIR_DESC), NULL);
       
       if(num)  {
	 for(x=0; num[x]; x++)  {
	   if(!isdigit(num[x]))
	     rpt_err(INCORRECT_USAGE, getResourceString(DBT_INV_NUMBER), 
		     getResourceString(DBT_INV_NUMBER_DESC), NULL);
	 }
       }
    }
    else
      /* need at least the directory to look in */
      rpt_err(INCORRECT_USAGE,
	      getResourceString(DBT_NO_DIR),
	      getResourceString(DBT_NO_DIR_DESC), NULL);      

    if(!num) num=DEF_SIZE;

    if(!directories) {
      directories = guess_directories(id);
      if(!directories)
	rpt_err(INCORRECT_USAGE,
		getResourceString(DBT_NO_DIR),
		getResourceString(DBT_NO_DIR_FOUND), NULL);		  
    }

    while(next_html_line(html, line))  {
        if(parse_line(line, NULL))  {
            if(directive_is(line, "LOG_TO_VIEW"))  {
	      display_logfiles(directories, file);
            }
            else if(directive_is(line, "NUM_TO_VIEW"))  {
                output_input("text", "num", num, NULL);
            }
            else if(directive_is(line, "STRING_TO_VIEW"))  {
                output_input("text", "str", str, NULL);
            }
            else if(directive_is(line, "HIDDEN_DIR"))  {
	        /* keep hidden track of what the current log directories and id are */
	        fprintf(stdout, (const char*)getResourceString(DBT_MAIN_HIDDEN_TRACK), directories, id);
            }
            else if(directive_is(line, "ID_TITLE"))  {
		fprintf(stdout, (const char*)getResourceString(DBT_MAIN_ID_TITLE), id);
            }
            else if(directive_is(line, "ACCESS_LOG"))  {
                FILE *cmd;
		char full_path[PATH_MAX];

		char *current_dir = get_current_dir(directories, file);
		if(!current_dir)
		  continue;
#ifdef AIX
		fflush(stdout);
#endif

		sprintf(full_path, "%s%c%s%c%s", getenv("NETSITE_ROOT"), FILE_PATHSEP, current_dir, FILE_PATHSEP, file);
		form_unescape(full_path);  /* bug 27409, 31807 */
		cmd = fopen(full_path, "r");
                fprintf(stdout, getResourceString(DBT_MAIN_WIDTH));
		sprintf(tmp, getResourceString(DBT_SUBTITLE), num, file, (str)? getResourceString(DBT_WITH) : "",
			(str)? str : "");

                fprintf(stdout, (const char*)getResourceString(DBT_MAIN_TABLE), 
			        tmp);

		/* begin search */
		if(cmd) {
		    search_file(cmd, atoi(num), str);
		    fclose(cmd);
		}
            }  else
                fputs(line, stdout);
        }  
    }

    return 0;	
}
