/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
//--------------------------------------------------------------------------//
//                                                                          //
//  Name: admconfig.c                                                       //
//  Admin Server command line utility.                                      //
//  Platforms: WIN32                                                        //
//                                                                          //
//  Note: This code is based on the NT startconsole program located in      //
//        ns/netsite/admserv/startconsole/nt/startconsole.c.                //
//                                                                          //
//--------------------------------------------------------------------------//
#include <windows.h>
#include <stdio.h>
#include "nt/regparms.h"

char szJavaHome[MAX_PATH];
char szUserJavaHome[MAX_PATH];
char szVersion[MAX_PATH];
char szMicroVersion[MAX_PATH];
char szUserJREexe[MAX_PATH];
char szJREexe[MAX_PATH];

// ADM_SUF is defined in the nsconfig.mk
#define JAR_VERSION ADM_SUF

static BOOL fileExists(LPCSTR pszFileName)
{
    OFSTRUCT ofs;

    if(OpenFile(pszFileName, &ofs, OF_EXIST) != HFILE_ERROR)
    {
        return TRUE;
    }
    return FALSE;
}

static char *getServerRoot()
{

    static char szRoot[MAX_PATH];
    char szKey[MAX_PATH];
    HKEY hKey;

    // HKLM\Software\Netscape\Administration\Version : RootPath
    wsprintf(szKey, KEY_SOFTWARE_NETSCAPE "\\" ADM_KEY_ROOT);
    if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, szKey, 0, KEY_QUERY_VALUE, &hKey) == ERROR_SUCCESS)
    {
        DWORD dwSize = sizeof(szRoot);
        if (RegQueryValueEx(hKey, VALUE_ROOT_PATH, 0, NULL, (unsigned char *)szRoot, &dwSize) != ERROR_SUCCESS)
        {
            GetCurrentDirectory(sizeof(szRoot), szRoot);
        }
        RegCloseKey(hKey);
    }
    else
    {
        GetCurrentDirectory(sizeof(szRoot), szRoot);
    }
    return szRoot;
}
   

static int getProductInfoString(const char* pszKey, char* pszValue, int cbValue, const char* pszFileName)
{
    FILE *fp;
    char szText[MAX_PATH];
    int nReturn = -1;

    pszValue[0] = '\0';

    if ((fp = fopen(pszFileName, "r")) != NULL)
    {
        while (fgets(szText, sizeof(szText), fp) != NULL)
        {
            char *pszToken;
            pszToken = strtok(szText, " : \t\n");
            if (pszToken && strcmp(pszToken, pszKey) == 0)
            {
                if ((pszToken = strtok(NULL, "\n")) != NULL)
                {
                    while(*pszToken == 0x20) pszToken++; // get rid of leading spaces
                    strncpy(pszValue, pszToken, cbValue);
                    pszValue[cbValue-1] = '\0'; //
                    nReturn = strlen(pszValue);
                }
                else
                {
                    nReturn = 0;
                }
                break;
            }
        }
        fclose(fp);
    }
    return nReturn;
}


static BOOL checkUserJRE(char *szVersion, char *szMicroVersion)
{
    BOOL bReturn = FALSE;
    HKEY hKey;
    char szKey[MAX_PATH];

    wsprintf(szKey, "SOFTWARE\\JavaSoft\\Java Runtime Environment\\%s", szVersion);
    if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, szKey, 0, KEY_ALL_ACCESS, &hKey) == ERROR_SUCCESS)
    {
        DWORD dwSize = sizeof(szUserJavaHome);

        if (RegQueryValueEx(hKey, "JavaHome", 0, NULL, (unsigned char *) szUserJavaHome, &dwSize) == ERROR_SUCCESS)
        {

            wsprintf(szUserJREexe, "%s\\bin\\java.exe", szUserJavaHome);
            bReturn = fileExists(szUserJREexe);
        }
        RegCloseKey(hKey);
    }
    return (bReturn);
}


static void checkJRE(char *szServerRoot)
{
    BOOL bInit = TRUE;
    char szPath[MAX_PATH];


    wsprintf(szPath, "%s\\java\\jre.inf", szServerRoot);
    if(fileExists(szPath))
    {
        char szValue[MAX_PATH];
        if (getProductInfoString("Version", szValue, MAX_PATH, szPath) > 0)
        {
            wsprintf(szVersion, szValue);
        }
        else 
            bInit = FALSE;

        if (getProductInfoString("MicroVersion", szValue, MAX_PATH, szPath) > 0)
        {
            wsprintf(szMicroVersion, szValue);
        }
        else 
            bInit = FALSE;
    }
    else
    {
        bInit = FALSE;
    }

    if(bInit == FALSE)
    {
        fprintf(stderr, "The file '%s' either does not exist or is corrupted.\n", szPath);
        fprintf(stderr, "Please re-install Fedora Management Console.\n", szPath);
        exit(1);
    }

    // Check if JRE is installed in the server root
    wsprintf(szPath, "%s\\bin\\base\\jre\\bin\\java.exe", szServerRoot);
    if (fileExists(szPath))
    {
          strcpy(szJREexe, szPath); //set global jre.exe path
          wsprintf(szJavaHome, "JavaHome=%s\\bin\\base\\jre", szServerRoot);
          _putenv(szJavaHome);
    }
    // Check in the windows registry if the user has the right JRE version installed
    else if (checkUserJRE(szVersion, szMicroVersion))
    {
          strcpy(szJREexe, szUserJREexe); //set global jre.exe path
          wsprintf(szJavaHome, "JavaHome=%s", szUserJavaHome);
          _putenv(szJavaHome);
    }
    // Then the user must have the JRE version in the path
    else
    {
        strcpy(szJREexe, "java.exe");
    }
}

char* getClassPath()
{
    WIN32_FIND_DATA FindFileData;
    LPWIN32_FIND_DATA lpFindFileData = &FindFileData;
    HANDLE hFile;
    static char szClassPath[MAX_PATH * 6];
    char szJava[MAX_PATH];
    char szFileSpec[MAX_PATH];

    strcpy(szJava, ".\\java");
    strcpy(szClassPath, ".\\java\\jars\\admserv" JAR_VERSION ".jar;");

    wsprintf(szFileSpec, "%s\\*.jar", szJava);
    hFile = FindFirstFile(szFileSpec, lpFindFileData);
    if(hFile != INVALID_HANDLE_VALUE)
    {
        do
        {
            char *szFileName = lpFindFileData->cFileName;
            strlwr(szFileName);
            strcat(szClassPath, szJava);
            strcat(szClassPath, "\\");
            strcat(szClassPath, szFileName);
            strcat(szClassPath, ";");
        } 
        while(FindNextFile(hFile, lpFindFileData));
        FindClose(hFile);
    }

    wsprintf(szFileSpec, "%s\\*.zip", szJava);
    hFile = FindFirstFile(szFileSpec, lpFindFileData);
    if(hFile != INVALID_HANDLE_VALUE)
    {
        do
        {
            char *szFileName = lpFindFileData->cFileName;
            strlwr(szFileName);
            strcat(szClassPath, szJava);
            strcat(szClassPath, "\\");
            strcat(szClassPath, szFileName);
            strcat(szClassPath, ";");
        } 
        while(FindNextFile(hFile, lpFindFileData));
        FindClose(hFile);
    }
    return szClassPath;
}

static BOOL runAdmconfig(int debug, char *szUserCommandLine)
{
    char szJRECommandLine[MAX_PATH * 7];
    char szJit[MAX_PATH];
    char szJavaLibPath[MAX_PATH * 5];

    checkJRE(getServerRoot());
    wsprintf(szJit, "-classic -Djava.compiler=NONE");
    wsprintf(szJavaLibPath, "-Djava.library.path=.\\lib\\jss");
    wsprintf(szJRECommandLine, "%s %s -ms8m -mx64m %s -cp \"%s\" com.netscape.management.admserv.cmdln.CommandLine %s ",
             szJREexe, szJit, szJavaLibPath, getClassPath(), szUserCommandLine);
    
    if (debug) {
       fprintf(stderr,"%s\n",szJRECommandLine);
    }

    return system(szJRECommandLine);
}

int main(int argc, char **argv)
{
   char szCommandLine[MAX_PATH];
   int i, debug=0;

   szCommandLine[0]=0;
   for (i=1; i < argc; i++) {
       
       if (i == 1) {
           // if the first parameter is -D print the command line        
           if (strcmp(argv[i], "-D") == 0) {
               debug = 1;
               continue;
           }
       }
       sprintf(szCommandLine, "%s %s", szCommandLine, argv[i]);
   }       

   SetCurrentDirectory(getServerRoot());
   return runAdmconfig(debug, szCommandLine);
}
 
