/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
package com.netscape.management.admserv.config;

/**
 * A generic interface for config dialog data model
 */
public interface IConfigDataModel {

    /**
     * Returns true id data model is loaded with server data
    */
    public boolean isLoaded();

    /**
     * Load data model from the server
     * @exception RemoteRequestException
    */
    public void load() throws RemoteRequestException;

    /**
     * Returns true is data model is modified
    */
    public boolean isModified();

    /**
     * Save data model to the server
     * @exception RemoteRequestException
    */
    public void save() throws RemoteRequestException;

    /**
     * Returns enumeration for attributes
     * @return Enumeration for attributes
    */
    public java.util.Enumeration getAttributes();

    /**
     * Get Value for the attribute with the specified name
     * @param attr the name of an attribute
     * @return value for the attribute
    */
    public String getAttribute(String attr);

    /**
     * Set the value for an attribute
     * @param attr the name for an attribute
     * @param val the value for the attribute
    */
    public void setAttribute(String attr,
            String val) throws ValidationException;

    /**
     * Return the model name. This can be the server name.
     * @return the identifier for the data model
    */
    public String getModelName();
}
