/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
//--------------------------------------------------------------------------//
//                                                                          //
//  Name: Administration Server Starter                                     //
//  Platforms: WIN32                                                        //
//  ......................................................................  //
//  Connects browser to Admin Server, starting Admin Server service if      //
//  necessary.  This program replaces admin.exe (ns/netsite/admsrv/ntadmin) //
//  ......................................................................  //
//  Revision History:                                                       //
//  02-09-97  Initial Version, Andy Hakim (ahakim@netscape.com)             //
//--------------------------------------------------------------------------//
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <ddeml.h>
#include <stdlib.h>
#include <stdio.h>
#include "resource.h"
#include "nt/regparms.h"

#define DDE_TIMEOUT_MS   2000

//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
BOOL _ControlServer(char *szServiceId, BOOL bStartStop)
{
    BOOL bReturn = FALSE;
    DWORD dwError = 0;
    SC_HANDLE schService = NULL;
    SC_HANDLE schSCManager = NULL;
    SERVICE_STATUS lpss;

	if((schSCManager = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS)))
    {
        if((schService = OpenService(schSCManager,
                                    szServiceId,
                                    SERVICE_ALL_ACCESS)))
        {
            if(bStartStop)
                bReturn = StartService(schService, 0, NULL);
            else
                bReturn = ControlService(schService, SERVICE_CONTROL_STOP, &lpss);
            CloseServiceHandle(schService);
		    bReturn = TRUE;
        }
        dwError = GetLastError();
        CloseServiceHandle(schSCManager);
    }
    return(bReturn);
}
//--------------------------------------------------------------------------//

//                                                                          //
//--------------------------------------------------------------------------//
BOOL _GetServerRoot(char *szPath, int cbPath)
{
    BOOL bReturn = FALSE;
    HKEY hKey;
    char szKey[MAX_PATH];

    // HKLM\Software\Netscape\Administration\Version : RootPath
    wsprintf(szKey, KEY_SOFTWARE_NETSCAPE "\\" SVR_KEY_ROOT);
    if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, szKey, 0, KEY_QUERY_VALUE, &hKey) == ERROR_SUCCESS)
    {
        DWORD dwSize = cbPath;
        RegQueryValueEx(hKey, VALUE_ROOT_PATH, 0, NULL, szPath, &dwSize);
        bReturn = (RegCloseKey(hKey) == ERROR_SUCCESS);
    }
    return(bReturn);
}





//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
BOOL _StartAdminService(void)
{
    char szServiceId[MAX_PATH];
    char szFileName[MAX_PATH];
    BOOL bReturn = FALSE;
    FILE *f;


    wsprintf(szServiceId, "%s-serv", ADM_ID_SERVICE);
    bReturn = _ControlServer(szServiceId, TRUE);

    //Give time for config local.conf to get created
    if(_GetServerRoot(szFileName, sizeof(szFileName)))
    {
     short i = 0; //safety
     strcat(szFileName, "\\admin-serv\\config\\local.conf");
     while ( (!(f = fopen(szFileName, "r")))  && (i < 150) )
     {
		_sleep(100);
        i++;
     }
     if (i < 150) fclose(f);
    }

    return bReturn;
}




//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
BOOL _GetBrowserPath(char *szPath, int cbPath)
{
    BOOL bReturn = FALSE;
    HKEY hKey;
    char szKey[MAX_PATH];

    // HKLM\SOFTWARE\Microsoft\Windows\CurrentVersion\App Paths\netscape.exe (2.x, 3.x)
    wsprintf(szKey, KEY_APP_PATH "\\netscape.exe");
    if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, szKey, 0, KEY_QUERY_VALUE, &hKey) == ERROR_SUCCESS)
    {
        DWORD dwSize = cbPath;
        bReturn = (BOOL)(RegQueryValueEx(hKey, "", 0, NULL, szPath, &dwSize) == ERROR_SUCCESS);
        RegCloseKey(hKey);
    }

    // HKLM\SOFTWARE\Netscape\Netscape Navigator\xxx\Main:Install Directory (4.x)
    wsprintf(szKey, "SOFTWARE\\Netscape\\Netscape Navigator");
    if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, szKey, 0, KEY_READ, &hKey) == ERROR_SUCCESS)
    {
        char szSubKey[MAX_PATH];
        DWORD dwIndex = 0;
        DWORD dwSize = sizeof(szSubKey);
        FILETIME pf;

        while(RegEnumKeyEx(hKey, dwIndex++, szSubKey, &dwSize, NULL, NULL, NULL, &pf) == ERROR_SUCCESS)
        {
            char szMainKey[MAX_PATH];
            HKEY hMainKey;

            wsprintf(szMainKey, "%s\\%s\\Main", szKey, szSubKey);
            if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, szMainKey, 0, KEY_READ, &hMainKey) == ERROR_SUCCESS)
            {
				dwSize = cbPath;
                bReturn = (BOOL)(RegQueryValueEx(hMainKey, "Install Directory", NULL, NULL, szPath, &dwSize) == ERROR_SUCCESS);
                if(bReturn)
				{
                    strcat(szPath, "\\Program\\netscape.exe");
					break;
				}

                RegCloseKey(hMainKey);
            }
			dwSize = sizeof(szSubKey);
        }
        RegCloseKey(hKey);
    }

    return(bReturn);
}


BOOL FindToken(char* szFileName, char* szToken, char *szValue)
{
  FILE *f;
  char szLine[MAX_PATH];
  char* szTok;
  BOOL bFound = FALSE;

	szValue[0] = '\0';

 if (f = fopen(szFileName, "r"))
 {
    while (!feof(f))
    {
      if (fgets(szLine, sizeof(szLine), f) > 0)
      {
        if (szTok = strtok(szLine, " \n"))
        {
          if (stricmp(szTok, szToken) == 0)
          {
            if (szTok = strtok(NULL, " \n"))
            {
              wsprintf(szValue, szTok);
              bFound = TRUE;
              break; //got the value
            }
          }

        }
      }
    }
    fclose(f);
  }
  return bFound;
}

//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
BOOL _GetAdminConfig(char *szUrl)
{
    BOOL bReturn = TRUE;
    BOOL bErrorBuildingAdminUrl = FALSE;
    char szFileName[MAX_PATH];

    if (_GetServerRoot(szFileName, sizeof(szFileName)))
    {
      char szValue[MAX_PATH];
      strcat(szFileName, "\\admin-serv\\config\\adm.conf");
      wsprintf(szUrl, "http://");
      FindToken(szFileName, "security:", szValue);
      if (strlen(szValue))
      {
        wsprintf(szUrl, "http(s)://");
      }
      FindToken(szFileName, "host:", szValue);
      if (!strlen(szValue)) bErrorBuildingAdminUrl = TRUE;
      strcat(szUrl, szValue);
      FindToken(szFileName, "port:", szValue);
      if (!strlen(szValue)) bErrorBuildingAdminUrl = TRUE;
      strcat(szUrl, ":");
      strcat(szUrl, szValue);
    }
    else bErrorBuildingAdminUrl = TRUE;

    if (bErrorBuildingAdminUrl)
    {
        bReturn = FALSE;
    }
    return (bReturn);
}



//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
HDDEDATA CALLBACK DdeCallback(UINT uType, UINT uFmt, HCONV hconv, HSZ hsz1, HSZ hsz2, HDDEDATA hdata, DWORD dwData1, DWORD dwData2)
{
    return (HDDEDATA)NULL;
}



//--------------------------------------------------------------------------//
// start new window in Navigator                                            //
//--------------------------------------------------------------------------//
BOOL _StartNavigatorDDE(char *szUrl, BOOL bNewWindow)
{
    BOOL bReturn = FALSE;
    DWORD dwResult = 0;
    DWORD idInst = 0;
    char szItem[MAX_PATH];
    HDDEDATA hData = NULL;
    HCONV hConv = NULL;
    HSZ hszTopic;
    HSZ hszService;
    HSZ hszItem;

    if(DdeInitialize(&idInst, (PFNCALLBACK)DdeCallback, APPCMD_CLIENTONLY | CBF_SKIP_ALLNOTIFICATIONS, 0L) == DMLERR_NO_ERROR)
    {
        wsprintf(szItem, "\"%s\",,%s,0,,,ADMSTART", szUrl, (bNewWindow ? "0" : "0xFFFFFFFF") );
        hszTopic = DdeCreateStringHandle(idInst, "Netscape", CP_WINANSI);
        hszService = DdeCreateStringHandle(idInst, "WWW_OpenURL", CP_WINANSI);
        hszItem = DdeCreateStringHandle(idInst, szItem, CP_WINANSI);

        if(hszTopic && hszService && hszItem)
        {
            if(hConv = DdeConnect(idInst, hszTopic, hszService, NULL))
            {
                DdeClientTransaction(NULL, 0L, hConv, hszItem, CF_TEXT, XTYP_REQUEST, DDE_TIMEOUT_MS, &dwResult);
                bReturn = DdeDisconnect(hConv);
            }
            DdeFreeStringHandle(idInst, hszItem);
            DdeFreeStringHandle(idInst, hszService);
            DdeFreeStringHandle(idInst, hszTopic);
        }
        DdeUninitialize(idInst);
    }
    return bReturn;
}


//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
int _StartNavigatorEXE(char *szUrl)
{
    int nReturn = 0;
    char szBrowserPath[MAX_PATH];
    char szCommandLine[MAX_PATH];

    if(_GetBrowserPath(szBrowserPath, sizeof(szBrowserPath)))
    {
        wsprintf(szCommandLine, "%s %s", szBrowserPath, szUrl);
        nReturn = WinExec(szCommandLine, SW_SHOW);
    }
    return(nReturn);
}



//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
BOOL _StartBrowser(char *szUrl, BOOL bNewWindow)
{
    BOOL bReturn = FALSE;

    bReturn = _StartNavigatorDDE(szUrl, bNewWindow);
    if(!bReturn)
        bReturn = _StartNavigatorEXE(szUrl);

    return(bReturn);
}



//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR szCmdLine, int nCmdShow)
{
    int nReturn = 0;
    char szUrl[MAX_PATH];
    char szText[MAX_PATH];
    BOOL bSecurity = FALSE;
	BOOL bNewWindow = FALSE;

    if(szCmdLine && strlen(szCmdLine))
    {
		char *token;
		char *szUrl;
		token = strtok(szCmdLine, " ");
		if(token != NULL)
		{
			szUrl = token;
			if(strcmp(token, "-newwindow") == 0)
			{
				bNewWindow = TRUE;
				szUrl = strtok(NULL, " ");
			}
		}
		nReturn = _StartBrowser(szUrl, bNewWindow);
    }
    else
    if( _GetAdminConfig(szUrl) )
    {
		_StartAdminService();
        nReturn = _StartBrowser(szUrl, bNewWindow);
    }
    else
    {
        LoadString(hInstance, IDS_NOSERVER, szText, sizeof(szText));
        MessageBox(NULL, szText, SVR_NAME_FULL, MB_ICONERROR);
        nReturn = 1;
    }

    if(!nReturn)
    {
        LoadString(hInstance, IDS_NOBROWSER, szText, sizeof(szText));
        MessageBox(NULL, szText, SVR_NAME_FULL, MB_ICONERROR);
    }

    return(nReturn);
}
