/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
//--------------------------------------------------------------------------//
//                                                                          //
//  Name: Administration Server Configuration                               //
//  Platforms: WIN32                                                        //
//  ......................................................................  //
//  Most of this code comes from /ns/netsite/httpd/newinst/src/ntconfig.c   //
//  which is similar to /ns/netsite/admserv/newinst/src/ux-config.c.  This  //
//  file should be consolidated with ux-config.c at some point.             //
//  ......................................................................  //
//  Revision History:                                                       //
//     use cvs log                                                          //
//                                                                          //
//--------------------------------------------------------------------------//
#include <windows.h>
#include <regstr.h>
#include <crtdbg.h>

#include <stdio.h>              /* printf, file I/O */
#include <direct.h>             /* access, getcwd */
#include <io.h>             /* close */
#include <string.h>             /* strlen */
#include <stdlib.h>             /* memset, rand stuff */
#include <time.h>               /* time */
#include <fcntl.h>              /* creat */
#include <sys/stat.h>           /* stat */
#include <process.h>                /* execl */
#include <errno.h>
#include <sys/types.h>

#include <nssetup.h>
#include <ldap.h>
#include <ldapu.h>
#include <setupldap.h>
#include <global.h>
#include <code.h>
#include "insadm.h"
#include "resource.h"
#include "nt/regparms.h"

#include "../common/include/servlet-conf.h"
#include "../common/include/acidef.h"

#define DEFAULT_SYSUSER "root"
#define DEFAULT_OLDROOT	"/usr/ns-home"
#define DEFAULT_TASKCONF "admin-serv/ldif/tasks.ldif"
#define DEFAULT_GLOBALCONF "admin-serv/ldif/global.ldif"

#define DEFAULT_SIECONF    "admin-serv/config/adm.conf"
#define DEFAULT_MAGNUSCONF "admin-serv/config/magnus.conf"
#define DEFAULT_NSFCCONF   "admin-serv/config/nsfc.conf"	
#define DEFAULT_OBJCONF "admin-serv/config/obj.conf"
#define DEFAULT_ACLCONF "adminacl/generated.https-admserv.acl"
#define DEFAULT_ESDBSWITCH "userdb/dbswitch.conf"
#define	DEFAULT_LCACHECONF "userdb/ldap/config/lcache.conf"
#define	DEFAULT_LCACHEDIR  "userdb/ldap/db"

extern MODULEINFO mi;
extern CONTROLDATA cd;

// Previous installed registry key to remove
#define ADM40_KEY_ROOT           "Administration\\4.0"
#define ADM40_ID_SERVICE         "admin40-serv"
#define ADM41_KEY_ROOT           "Administration\\4.1"
#define ADM41_ID_SERVICE         "admin41-serv"
#define ADM42_KEY_ROOT           "Administration\\4.2"
#define ADM42_ID_SERVICE         "admin42-serv"
#define ADM60_KEY_ROOT           "Administration\\6.0"
#define ADM60_ID_SERVICE         "admin60-serv"
#define ADM61_KEY_ROOT           "Administration\\6.1"
#define ADM61_ID_SERVICE         "admin61-serv"
#define ADM70_KEY_ROOT           "Administration\\7.0" /* current */
#define ADM70_ID_SERVICE         "admin70-serv"        /* current */


/*
 * The following escape LDAP URL is from ns/netsite/ldap/clients/dsgw/htmlout.c
 */
#define HREF_CHAR_ACCEPTABLE( c )       (( c >= '-' && c <= '9' ) ||    \
                                         ( c >= '@' && c <= 'Z' ) ||    \
                                         ( c == '_' ) ||                \
                                         ( c >= 'a' && c <= 'z' ))

typedef struct {
char *section;
char *attr;
char *val;
} xml_file_mod_t;

static BOOL modify_xml_file(char *filename, xml_file_mod_t **mods);

void
escape_ldap_uri(char *s1, const char *s2)
{
  const unsigned char       *q;
  char                *p, *hexdig = "0123456789ABCDEF";

  p = s1 + strlen( s1 );
  for (q = (const unsigned char *)s2; *q != '\0'; ++q)
  {
    if (HREF_CHAR_ACCEPTABLE( *q ))
    {
      *p++ = *q;
    }
    else
    {
      *p++ = '%';
      *p++ = hexdig[ *q >> 4 ];
      *p++ = hexdig[ *q & 0x0F ];
    }
  }

  *p = '\0';
}

//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
BOOL CreateEventLog(char *szServerRoot, char *szServiceId)
{
  BOOL bReturn = FALSE;
  char szMessageFile[MAX_PATH];
  char szKey[MAX_PATH];
  HKEY hKey;
  DWORD dwData;

  sprintf(szMessageFile, "%s\\bin\\https\\bin\\messages.dll", szServerRoot);
  sprintf(szKey, REGSTR_PATH_SERVICES "\\EventLog\\Application\\%s", szServiceId);

  if (RegCreateKey(HKEY_LOCAL_MACHINE, szKey, &hKey) == ERROR_SUCCESS)
  {
    if (RegSetValueEx(hKey, "EventMessageFile", 0, REG_SZ, (LPBYTE)szMessageFile, strlen(szMessageFile) + 1) == ERROR_SUCCESS)
    {
      dwData = EVENTLOG_ERROR_TYPE | EVENTLOG_WARNING_TYPE | EVENTLOG_INFORMATION_TYPE;
      RegSetValueEx(hKey, "TypesSupported", 0, REG_DWORD, (LPBYTE) &dwData, sizeof(DWORD));
      bReturn = TRUE;
    }
    RegCloseKey(hKey);
  }
  return (bReturn);
}



//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
//LONG ConvertNtSlashesToUnix( LPSTR lpszNtPath, LPSTR lpszUnixPath )
//{
//  if (lpszNtPath == NULL)
//    return 0;
//
//  /* create reverse slashes and escape them */
//  while (*lpszNtPath)
//  {
//    if (*lpszNtPath == '\\')
//      *lpszUnixPath = '/';
//    else
//      *lpszUnixPath = *lpszNtPath;
//    lpszNtPath++;
//    lpszUnixPath++;
//  }
//  return 0;
//}



//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
///* XXXrobm this should not be copied and pasted from libadmin! */
//char *make_shexp(char *source)
//{
//  char *answer;
//  int hascomma, x, y;
//
//  if (!source) return source;
//
//  hascomma = (strchr(source, ',')) ? 1 : 0;
//
//  /* + 2 = 1 for trailing \0 and one put in by Aruna for NT */
//  answer = malloc(strlen(source) + 2);
//
//  for (x=0, y=0; source[x]; x++)
//  {
//    if (source[x] == ',')
//      answer[y++] = '|';
//    else if (source[x] != ' ')
//      answer[y++] = source[x];
//    answer[y] = '\0';
//  }
//  if (!hascomma)
//    return (answer);
//  else
//  {
//    char *newanswer = malloc(strlen(answer)+2);
//    sprintf(newanswer, "(%s)", answer);
//    return (newanswer);
//  }
//}


//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
LONG CreateStartStopCmdFiles(char *szRoot, char* szAdminServID )
{
  FILE *f;
  char tstr[MAX_PATH];

  /*
     Write new admpw.   -user -password
   */
  sprintf(tstr, "%s/stop-admin.cmd", szRoot);
  if (!(f = fopen(tstr, "w")))
  {
    return FALSE;
  }
  fprintf(f, "net stop %s", szAdminServID);
  fclose(f);

  sprintf(tstr, "%s/start-admin.cmd", szRoot);
  if (!(f = fopen(tstr, "w")))
  {
    return FALSE;
  }
  fprintf(f, "net start %s", szAdminServID);
  fclose(f);

  sprintf(tstr, "%s/restart-admin.cmd", szRoot);
  if (!(f = fopen(tstr, "w")))
  {
    return FALSE;
  }
  fprintf(f, "net stop %s\n", szAdminServID);
  fprintf(f, "net start %s", szAdminServID);
  fclose(f);

  return TRUE;
}

//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
LONG CreateAdminAdmpwFile(char *sroot, char *httpUser, char *httpPassword )
{
  FILE *f;
  char tstr[MAX_PATH];

  /*
     Write new admpw.   -user -password
   */
  sprintf(tstr, "%s/admin-serv/config/admpw", sroot);
  if (!(f = fopen(tstr, "w")))
  {
    return FALSE;
  }
  fprintf(f, "%s:{SHA}%s", httpUser, sha1_pw_enc(httpPassword));
  fclose(f);
  return TRUE;
}




BOOL CreateSieConf(char *sroot)
{
  HSIE hSie;
  FILE* fp;
  char szPath[MAX_PATH];
  char szDomain[MAX_PATH];
  char szTemp[MAX_PATH];
  char szTemp1[MAX_PATH];
  char szSIE[4096];
  char szAPP[4096];
  char szUpgrade[4096];
  char szInstanceName[MAX_PATH];
  char* psz;
  BOOL bSuccess = TRUE;
  char *userDirRef;
  char szLdapStart[MAX_PATH];

  lstrcpy(szTemp, cd.szHostname);
  if ((psz = strchr(szTemp, '.')) != NULL)
    *psz = '\0';
   sprintf(szInstanceName, "admin-serv-%s", szTemp);
  GetNetworkDomain(szDomain, sizeof(szDomain));

  sprintf(szSIE,
           "objectclass: top\n"
           "objectclass: netscapeServer\n"
           "objectclass: nsAdminServer\n"
           "objectclass: nsResourceRef\n"
           "cn: %s\n"
           "nsServerID: admin-serv\n"
           "userPassword: %s\n"
           "serverHostName: %s\n"
	   "serverProductName: %s\n"
           "serverRoot: %s\n",
           szInstanceName,
           cd.szPassword,
           cd.szHostname,
           "Administration Server", /* Used to be cd.szName, */
           TARGETDIR
          );

//load APP data
  sprintf(szAPP,
           "objectclass: top\n"
           "objectclass: nsApplication\n"
		   "cn: Fedora Administration Server\n" // 387667 force old name
           "nsVendor: %s\n"
           "description: %s\n"
           "nsProductName: %s\n"
           "nsNickName: %s\n"
           "nsProductVersion: %s\n"
           "nsBuildNumber: %s\n"
           "nsRevisionNumber: %s\n"
           "nsInstalledLocation: %s\n"
           "nsExpirationDate: %s\n"
           "nsBuildSecurity: %s\n"
           "nsServerMigrationClassname: com.netscape.management.admserv.AdminServerProduct@admserv70.jar\n",
           cd.szVendor,
           cd.szDescription,
           cd.szName,
           cd.szNickname,
           cd.szVersion,
           cd.szBuildNumber,
           cd.szRevision,
           sroot,
           cd.szExpireDate,
           cd.szSecurity
          );

  hSie = SIECreate(szSIE, szAPP, cd.szHostname);
  if (hSie == INVALID_SIE_HANDLE)
  {
    SetupErrorBox("Unable to create Server Instance Entry.");
    return FALSE;
  }


  SIEGetBaseDN(hSie, szTemp, sizeof(szTemp));
  SetLdapAdminSuffix(szTemp);

//load Configuration
//addresses added for admim server to get applet working for now

  ISIEGetBaseDN(hSie, szTemp1, sizeof(szTemp1));
  userDirRef = strchr(szTemp1, ',') + 1;
  while (*userDirRef == ' ') ++userDirRef;
  sprintf(szAPP,
           "objectclass: top\n"
           "objectclass: nsConfig\n"
           "objectclass: nsAdminConfig\n"
           "objectclass: nsAdminObject\n"
	        "objectclass: nsDirectoryInfo\n"
           "cn: Configuration\n"
           "nsErrorlog: admin-serv/logs/error\n"
           "nsPidLog: admin-serv/logs/pid\n"
           "nsAccessLog: admin-serv/logs/access\n"
           "nsAdminAccessHosts: *.%s\n"
           "nsAdminAccessAddresses: *\n"
           "nsAdminOneACLDir: adminacl\n"
           "nsDefaultAcceptLanguage: %s\n"
           "nsClassName: com.netscape.management.admserv.AdminServer@admserv70.jar@%s\n"
           "nsServerPort: %d\n"
           "nsSuiteSpotUser: nobody\n"
           "nsAdminUsers: admin-serv/config/admpw\n"
           "nsAdminCacheLifetime: 600\n"
           "nsServerAddress: %s\n"
           "nsDirectoryInfoRef: %s\n"
           "aci: (targetattr=*)(version 3.0; acl \"Enable delegated admin to access configuration\"; allow (read,search) groupdn=\"ldap:///%s\";)\n",
           szDomain,
           cd.szLanguage,
           szTemp,
           cd.nPort,
           cd.szServerAddress,
           userDirRef,
	   userDirRef
          );

  sprintf(szUpgrade,
         "nsClassName: com.netscape.management.admserv.AdminServer@admserv70.jar@%s\n"
         "nsServerPort: %d\n"
         "nsSuiteSpotUser: nobody\n",
          szTemp,
          cd.nPort
        );
  

  if (!SIEAddAdminConfig(hSie, szAPP, szUpgrade))
  {
    SetupErrorBox("Error creating ldap Config configuration.");
    return FALSE;
  }



//load Tasks
  sprintf(szPath, "%s/tasks.ldif", SRCDIR);
  if (!SIELoadTasks(hSie, szPath))
  {
    SetupErrorBox("Error loading tasks in Server Instance Entry.");
    return FALSE;
  }

// load Global Parameters
  sprintf(szPath, "%s/global.ldif", SRCDIR);
  if (!SIELoadGlobalPar(hSie, szPath))
  {
    SetupErrorBox("Error loading Global Parameters in Installed Server Instance Entry.");
    return FALSE;
  }

//write adm.conf
  SIEGetBaseDN(hSie, szTemp, sizeof(szTemp));
  ISIEGetBaseDN(hSie, szTemp1, sizeof(szTemp1));

  sprintf(szPath, "%s/%s", sroot, DEFAULT_SIECONF);
  szLdapStart[0] = '\0';
  if (setupFileExists(szPath))
  {
    char *sz = NULL;

    sz = setupGetFileString("ldapStart", ":", szPath);
    if (sz)
    {
      strcpy(szLdapStart, sz);
    }
  }
  if ((fp = fopen(szPath, "w")) == NULL)
  {
    bSuccess = FALSE;
  }
  else
  {
    if (FileExists(cd.szCacheFile))
    {
      char szValue[MAX_PATH];
      GetPrivateProfileString(NS_SLAPD_CFG_SECTION, NS_SLAPD_CFG_MCC, "", szValue, MAX_PATH, cd.szCacheFile);
      if (strlen(szValue))
      {
        GetPrivateProfileString(NS_SLAPD_CFG_SECTION, NS_SLAPD_INSTANCE_ID, "", szValue, MAX_PATH, cd.szCacheFile);
        if (strlen(szValue))
        {
          sprintf(szLdapStart, "slapd-%s/start-slapd", szValue);
        }
      }
    }
    if (strlen(szLdapStart) > 0)
    {
       fprintf(fp,
               "ldapStart: %s\n",
                szLdapStart
               );
    }

    fprintf(fp,
            "ldapHost: %s\n"
            "ldapPort: %d\n"
            "SIE: %s\n"
            "siepid: %s\n"
            "ISIE: %s\n"
            "host: %s\n"
            "port: %d\n",
            GetLdapHost(),
            GetLdapPort(),
            szTemp,
            cd.szPassword,
            szTemp1,
            cd.szHostname,
            cd.nPort
           );
    fclose(fp);
  }

  SIEClose(hSie);
  return bSuccess;
}

//--------------------------------------------------------------------------//
// Create magnus.conf file (needed by ES core engine)                       //
//--------------------------------------------------------------------------//

BOOL CreateAdminMagnusConf(char *sroot)
{
  FILE* fp;
  char szPath[MAX_PATH];

  sprintf(szPath, "%s/%s", sroot, DEFAULT_MAGNUSCONF);
  if ((fp = fopen(szPath, "w")) == NULL)
  {
    return FALSE;
  }
  else
  {
    fprintf(fp,
            "NetsiteRoot %s\n"
            "ServerID admin-serv\n"
            "ServerName %s\n"
            "ExtraPath ${NSES_JRE_RUNTIME_LIBPATH}\n"
            "ErrorLog %s/admin-serv/logs/error\n"
            "PidLog %s/admin-serv/logs/pid\n"
            "User %s\n"
            "AdminUsers %s/admin-serv/config/admpw\n"
            "MtaHost localhost\n"
            "DNS on\n"
            "Security off\n"
            "ClientLanguage %s\n"
            "AdminLanguage %s\n"
            "DefaultLanguage %s\n"
            "RqThrottle 64\n"
            "ConnQueueSize 256\n"
            "StackSize 131072\n"
            "TempDir %s/admin-serv/tmp\n",
            sroot,
            cd.szHostname,
            sroot,
            sroot,
            cd.szUsername,
            sroot,
            cd.szLanguage,
            cd.szLanguage,
            cd.szLanguage,
            sroot
           );
    if((cd.szServerAddress != NULL) && (cd.szServerAddress[0] != '\0'))
      fprintf(fp, "Address %s\n", cd.szServerAddress);

    fprintf(fp, "\n");
    fprintf(fp, "Init fn=flex-init access=%s\n\n", "\"$accesslog\" format.access=\"%Ses->client.ip% - %Req->vars.auth-user% [%SYSDATE%] \\\"%Req->reqpb.clf-request%\\\" %Req->srvhdrs.clf-status% %Req->srvhdrs.content-length%\"");
    fprintf(fp, "Init fn=\"load-modules\" shlib=\"%s/bin/https/bin/AdmservPlugin.dll\" funcs=\"admin40_task_eval_init,admin40_host_ip_init,admin40_host_ip_check,admin40_uri2path,admin40_authenticate,admin40_runtime_commands,admin40_ftype,admin40_isdiropen,admin40_flex_log,admin40_check_ds_availability_init\"\n", sroot);
    fprintf(fp, "Init fn=\"admin40_check_ds_availability_init\" nsRoot=\"%s\"\n", sroot);
    fprintf(fp, "Init fn=\"admin40_task_eval_init\" nsRoot=\"%s\" cacheLifeTime=\"600\"\n", sroot);
    fprintf(fp, "Init fn=\"admin40_host_ip_init\"\n");

    fprintf(fp, "\n#Servlet engine startup directives\n");
    fprintf(fp, "Init fn=\"load-modules\" shlib=\"%s/bin/https/bin/NSServletPlugin.dll\" funcs=\"NSServletEarlyInit,NSServletLateInit,NSServletNameTrans,NSServletService\"\n", sroot);
    fprintf(fp, "Init fn=\"NSServletEarlyInit\" EarlyInit=yes\n");
    fprintf(fp, "Init fn=\"NSServletLateInit\"  LateInit=yes\n");
    fclose(fp);
  }

  return TRUE;
}

//--------------------------------------------------------------------------// 
// Create nsfc.conf file (New to AS 6.2, tunes server-side cache for        // 
// static content)                                                          // 
//--------------------------------------------------------------------------// 

BOOL CreateAdminNSFCConf(char *sroot)
{
  FILE* fp;
  char szPath[MAX_PATH];

  sprintf(szPath, "%s/%s", sroot, DEFAULT_NSFCCONF);
  if ((fp = fopen(szPath, "w")) == NULL)
  {
    return FALSE;
  }
  else
  {
    fprintf(fp,
            "MaxFiles=128\n");
            
    fclose(fp);
  }
  return TRUE;
}

//--------------------------------------------------------------------------//
// Create server.xml (new for ES6.0)                                        //
//--------------------------------------------------------------------------//

BOOL CreateServerXML(char *sroot)
{
   char szPath[MAX_PATH];
   char accesslog[BIG_BUF], aclfile[BIG_BUF], portnum[32];

   /* Set port */
   xml_file_mod_t port_mod =
       {"LS id=\"ls1\"", "port", portnum};

   /* Set server name */
   xml_file_mod_t srvname_mod =
       {"CONNECTIONGROUP", "servername", cd.szHostname}; 
   xml_file_mod_t urlhosts_mod =
       {"VS id=\"admin-serv\"", "urlhosts", cd.szHostname};

   xml_file_mod_t accesslog_mod =
   {"VARS", "accesslog", accesslog };

   xml_file_mod_t aclfile_mod =
   {"ACLFILE id=\"acl1\"", "file", aclfile };

   xml_file_mod_t *mods[6] = { 
       &port_mod, &srvname_mod, &urlhosts_mod,
       &accesslog_mod, &aclfile_mod, NULL};

   sprintf(portnum, "%d", cd.nPort);
   sprintf(accesslog, "%s/admin-serv/logs/access", sroot);
   sprintf(aclfile,   "%s/adminacl/generated.https-admserv.acl", sroot);

   sprintf(szPath, "%s/admin-serv/config/server.xml", sroot);
   return modify_xml_file(szPath, mods);
}


//--------------------------------------------------------------------------//
// Create obj.conf file (needed by ES core engine)                          //
//--------------------------------------------------------------------------//

BOOL CreateAdminObjConf(char *sroot)
{
  FILE* fp;
  char szPath[MAX_PATH];

  sprintf(szPath, "%s/%s", sroot, DEFAULT_OBJCONF);
  if ((fp = fopen(szPath, "w")) == NULL)
  {
    return FALSE;
  }
  else
  {
    fprintf(fp, "\n");
    fprintf(fp, "<Object name=\"default\">\n");
    fprintf(fp, "AuthTrans fn=\"admin40_host_ip_check\"\n");
    fprintf(fp, "AuthTrans fn=\"admin40_isdiropen\"\n");
    fprintf(fp, "AuthTrans fn=\"admin40_authenticate\"\n");
    fprintf(fp, "AuthTrans fn=\"admin40_uri2path\"\n");
/*  fprintf(fp, "NameTrans fn=\"pfx2dir\" from=\"/clients\" dir=\"%s/clients\"\n", sroot);*/
    fprintf(fp, "PathCheck fn=\"ntcgicheck\" extension=\".exe\"\n");
    fprintf(fp, "PathCheck fn=\"nt-uri-clean\"\n");
    fprintf(fp, "ObjectType fn=\"admin40_ftype\"\n");
    fprintf(fp, "Service fn=\"admin40_runtime_commands\" type=\"admin-internal/command\"\n");
    fprintf(fp, "Service fn=\"send-cgi\" type=\"magnus-internal/cgi\"\n");
    fprintf(fp, "Service fn=\"NSServletService\" type=\"magnus-internal/servlet\"\n");
    fprintf(fp, "Service fn=\"imagemap\" method=\"GET\" type=\"magnus-internal/imagemap\"\n");
    fprintf(fp, "Service method=(GET|HEAD|POST) type=*~magnus-internal/* fn=send-file\n");
    fprintf(fp, "Error fn=\"admin-error\" reason=\"server error\"\n");
    fprintf(fp, "AddLog fn=\"admin40_flex_log\" name=\"access\"\n");
    fprintf(fp, "NameTrans fn=\"NSServletNameTrans\" name=\"servlet\"\n");
    fprintf(fp, "PathCheck fn=find-index index-names=\"index.html,home.html\"\n");
    fprintf(fp, "Service type=\"magnus-internal/jsp\" fn=\"NSServletService\"\n");
    fprintf(fp, "</Object>\n");
    fprintf(fp, "\n");
    fprintf(fp, "<Object name=\"servlet\">\n");
    fprintf(fp, "ObjectType fn=force-type type=text/html\n");
    fprintf(fp, "Service fn=\"NSServletService\"\n");
    fprintf(fp, "</Object>\n");
    fclose(fp);
  }

  return TRUE;
}


//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
BOOL CreateLdapDefaultSwitch(char *sroot)
{
  char Path[MAX_PATH];
  FILE *fp;
  char *escaped_BaseDN;
  sprintf(Path, "%s/%s", sroot, NS_DEFAULT_LDAPSWITCH);
  if (!(fp = fopen(Path, "w")))
  {
    return FALSE;
  }
  escaped_BaseDN = malloc(3 * strlen(GetLdapSuffix()) + 1);
  *escaped_BaseDN = '\0';
  (void) escape_ldap_uri(escaped_BaseDN, GetLdapSuffix());
  fprintf(fp, "directory default ldap://%s:%d/%s\n",
          GetLdapHost(), GetLdapPort(), escaped_BaseDN);
  (void) fclose(fp);

  nsLogCreatedFile("admin", "admin", Path);

  //create ds.conf
  sprintf(Path, "%s/%s", sroot, NS_DS_CONF);
  if (!(fp = fopen(Path, "w")))
  {
    return FALSE;
  }
  fprintf(fp,
          "AdminDomain: %s\n",
          GetLdapAdminDomain()
          );
  (void) fclose(fp);

  nsLogCreatedFile("admin", "admin", Path);

  // New code for 4.5 ES engine - create dbswitch.conf in <sr>/userdb
  sprintf(Path, "%s/%s", sroot, DEFAULT_ESDBSWITCH);
  if (!(fp = fopen(Path, "w")))
  {
    return FALSE;
  }
  fprintf(fp, "directory default ldap://%s:%d/%s\n",
          GetLdapHost(), GetLdapPort(), escaped_BaseDN);
  (void) fclose(fp);

  nsLogCreatedFile("admin", "admin", Path);

  return TRUE;
}


//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
BOOL CreateAdminACL(char *sroot, char *httpuser)
{
  char szTemp[MAX_PATH];
  char szPath[MAX_PATH];
  FILE* fp;

  // Pre-4.5 code - simply create adminacl directory
  sprintf(szTemp, "%s\\adminacl", sroot);
  CreateDirectory(szTemp, NULL);
  
  // New code for 4.5 - create dummy ACL file for ES core engine
  sprintf(szPath, "%s/%s", sroot, DEFAULT_ACLCONF);
  if ((fp = fopen(szPath, "w")) == NULL)
  {
    return FALSE;
  }
  else
  {
    fprintf(fp, "version 3.0;\n");
    fprintf(fp, "acl \"admin-defaults\";\n");
    fprintf(fp, "authenticate (user, group) {\n");
    fprintf(fp, "  prompt = \"WebServer Server\";\n");
    fprintf(fp, "};\n");
    fprintf(fp, "allow (read, list, execute,info) user = \"anyone\";\n");
    fprintf(fp, "allow (write, delete) user = \"all\";\n");
    fprintf(fp, "\n");
    fclose(fp);
  }
  return (TRUE);
}


//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
BOOL _GetAdminConfig(const char *szServerRoot, char *szHostName, int *nPort)
{
  char szFileName[MAX_PATH];
  char szValue[MAX_PATH];
  BOOL bReturn = FALSE;

  if (szServerRoot)
  {
    sprintf(szFileName, "%s\\admin-serv\\config\\adm.conf", szServerRoot);
    szValue[0] = '\0';
    GetProductInfoString("host", szValue, MAX_PATH, szFileName);
    if (strlen(szValue))
    {
			strcpy(szHostName, szValue);
			bReturn = TRUE;
    }
    GetProductInfoString("port", szValue, MAX_PATH, szFileName);
    if (strlen(szValue))
    {
			*nPort = atoi(szValue);
			bReturn = TRUE;
    }
  }
//  nsLogCreatedFile("admin", "admin", szFileName);
  return bReturn;
}


//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
int _UpdateConfig(char *szServerRoot)
{
   char szPath[MAX_PATH];
   char szName[MAX_PATH];

   if (cd.nReplaceVersion == 70) /* The current version;
                                    target of brandAdminServer */
   {
     LoadString(mi.m_hModule, IDS_START_ADMINSERVER, szName, sizeof(szName));
     strcat(szName, " 7.0"); /* current */
     strcpy(szPath, "\"");
     strcpy(szPath, TARGETDIR);
     strcat(szPath, "\\start-admin.cmd\"");
     RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
     DeleteServer("admin70-serv"); /* current */
     DeleteServerRegistryKeys(ADM70_KEY_ROOT, ADM70_ID_SERVICE); /* current */
   }
   else if (cd.nReplaceVersion == 1)
   {
     LoadString(mi.m_hModule, IDS_START_ADMINSERVER, szName, sizeof(szName));
     strcat(szName, " 4.0");
     strcpy(szPath, "\"");
     strcpy(szPath, TARGETDIR);
     strcat(szPath, "\\start-admin.cmd\"");
     RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
     DeleteServer("admin40-serv");

     DeleteServerRegistryKeys(ADM40_KEY_ROOT, ADM40_ID_SERVICE);
   }
   else if (cd.nReplaceVersion == 2)
   {
     LoadString(mi.m_hModule, IDS_START_ADMINSERVER, szName, sizeof(szName));
     strcat(szName, " 4.1");
     strcpy(szPath, "\"");
     strcpy(szPath, TARGETDIR);
     strcat(szPath, "\\start-admin.cmd\"");
     RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
     DeleteServer("admin41-serv");
     DeleteServerRegistryKeys(ADM41_KEY_ROOT, ADM41_ID_SERVICE);
   }
   else if (cd.nReplaceVersion == 3)
   {
     LoadString(mi.m_hModule, IDS_START_ADMINSERVER, szName, sizeof(szName));
     strcat(szName, " 4.2");
     strcpy(szPath, "\"");
     strcpy(szPath, TARGETDIR);
     strcat(szPath, "\\start-admin.cmd\"");
     RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
     DeleteServer("admin42-serv");
     DeleteServerRegistryKeys(ADM42_KEY_ROOT, ADM42_ID_SERVICE);
   }
   else if (cd.nReplaceVersion == 4 || cd.nReplaceVersion == 5)
   {
     LoadString(mi.m_hModule, IDS_START_ADMINSERVER, szName, sizeof(szName));
     strcat(szName, " 6.0");
     strcpy(szPath, "\"");
     strcpy(szPath, TARGETDIR);
     strcat(szPath, "\\start-admin.cmd\"");
     RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
     DeleteServer("admin60-serv");
     DeleteServerRegistryKeys(ADM60_KEY_ROOT, ADM60_ID_SERVICE);
   }
   else if (cd.nReplaceVersion == 6)
   {
     LoadString(mi.m_hModule, IDS_START_ADMINSERVER, szName, sizeof(szName));
     strcat(szName, " 6.1");
     strcpy(szPath, "\"");
     strcpy(szPath, TARGETDIR);
     strcat(szPath, "\\start-admin.cmd\"");
     RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
     DeleteServer("admin61-serv");
     DeleteServerRegistryKeys(ADM61_KEY_ROOT, ADM61_ID_SERVICE);
   }
   else
   {
     DeleteServer("admin62-serv");
   }
   return _CreateConfig(szServerRoot);
}



//--------------------------------------------------------------------------//
// Setup up parameters for the servlet engine                               //
//   szServerRoot must have forward slashes Unix style                      //
//                                                                          //
// This code is based on the ES4.x installer. See function installJC()      //
// in ES4.x source file ns/netsite/installer/src/src/httpconf.cpp           //
//                                                                          //
//--------------------------------------------------------------------------//
static int SetServletRegistryValues(char *szServerRoot)
{
  char path[MAX_PATH];
  DWORD regRez;
  HKEY hKey;
  
  sprintf(path, KEY_SOFTWARE_NETSCAPE "\\" ADM_KEY_ROOT);

  if(RegCreateKeyEx(HKEY_LOCAL_MACHINE, path, 0, "",
                     REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, 
                     NULL, &hKey, &regRez) != ERROR_SUCCESS)
  {

     return FALSE;
  }

  RegSetValueEx(hKey, "NSES_SERVER_HOME", 0, REG_SZ, 
                (const unsigned char*)szServerRoot, strlen(szServerRoot) + 1);

  RegSetValueEx(hKey, "NSES_JRE", 0, REG_SZ, 
                    (const unsigned char*)NSES_JRE, strlen(NSES_JRE) + 1);

  RegSetValueEx(hKey, "NSES_JRE_RUNTIME_LIBPATH", 0, REG_SZ, 
                (const unsigned char*)NSES_JRE_RUNTIME_LIBPATH,
                strlen(NSES_JRE_RUNTIME_LIBPATH) + 1);

  RegSetValueEx(hKey, "NSES_CLASSPATH"  , 0, REG_SZ,
                  (const unsigned char *)NSES_CLASSPATH, strlen(NSES_CLASSPATH)+1);

  RegCloseKey(hKey);
  return TRUE;
}


//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
int _CreateConfig(char *szServerRoot)
{
  int nReturn = TRUE;
  FILE *f;
  char szServiceId[MAX_PATH];
  char szConfigPath[MAX_PATH];
  char szTemp[MAX_PATH];
  char szSRoot[MAX_PATH];
  char *tmp;

//  sprintf(szTemp, "%s\\admserv" , szServerRoot);
//  if (f = fopen(szTemp, "w"))
//  {
//    fclose(f);
//    SetFileAttributes(szTemp, FILE_ATTRIBUTE_HIDDEN | FILE_ATTRIBUTE_READONLY);
//  }

  sprintf(szTemp, "%s\\admin-serv" , szServerRoot);
  CreateDirectory(szTemp, NULL);

  strcpy(szSRoot, szServerRoot);
  while ((tmp = strchr(szSRoot, '\\')))   /* change all back slashes introduced by szServerRoot */
    tmp[0] = '/';                           /* into forward slashes */


  sprintf(szConfigPath, "%s/admin-serv/config", szSRoot);
  
  CreateDirectory(szConfigPath, NULL);

  sprintf(szTemp, "%s\\admin-serv\\logs", szServerRoot);
  CreateDirectory(szTemp, NULL);

  sprintf(szTemp, "%s\\alias", szServerRoot);
  CreateDirectory(szTemp, NULL);
  strcat(szTemp, "\\cert.log");
  if (f = fopen(szTemp, "w"))
    fclose(f);

  sprintf(szServiceId, "%s-serv", ADM_ID_SERVICE);

  sprintf(szTemp, "%s\\bin\\admin", szServerRoot);



  if (!CreateServerRegistryKeys(SVR_KEY_ROOT, szServiceId, szServerRoot, szTemp, szConfigPath))
  {
    NsSetupMessageBox(NULL, "Could not create admin registry keys.", NULL, MB_OK|MB_ICONEXCLAMATION);
    nReturn = FALSE;
  }

  if (!SetServletRegistryValues(szSRoot))
  {
    NsSetupMessageBox(NULL, "Could not set servlet engine registry values.", NULL, MB_OK|MB_ICONEXCLAMATION);
    nReturn = FALSE;
  }

  sprintf(szTemp, "%s\\bin\\https\\bin\\ns-httpd.exe", szServerRoot);
  if (!CreateServer(szServiceId, ADM_NAME_FULL_VERSION, szTemp))
  {
//        NsSetupMessageBox(NULL, "Could not setup admin service.", NULL, MB_OK|MB_ICONEXCLAMATION);
// do NOT abort just because of this, may exist due to a re-install
  }

  CreateStartStopCmdFiles(szServerRoot, szServiceId );

  if (!CreateEventLog(szServerRoot, szServiceId))
  {
    NsSetupMessageBox(NULL, "Could not setup admin event logging.", NULL, MB_OK|MB_ICONEXCLAMATION);
    nReturn = FALSE;
  }

  if (!CreateAdminAdmpwFile(szServerRoot, cd.szUsername, cd.szPassword))
  {
    NsSetupMessageBox(NULL, "Could not setup admin password file.", NULL, MB_OK|MB_ICONEXCLAMATION);
    return FALSE;
  }

  if (!CreateLdapDefaultSwitch(szServerRoot))
  {
    NsSetupMessageBox(NULL, "Could not setup default LDAP switch.", NULL, MB_OK|MB_ICONEXCLAMATION);
    return FALSE;
  }

  if (!CreateSieConf(szServerRoot))
  {
    CHAR szErrorMsg[MAX_PATH];
    LoadString(mi.m_hModule, IDS_ERR_SIECREATE, szErrorMsg, sizeof(szErrorMsg));
    NsSetupMessageBox(NULL, szErrorMsg, NULL, MB_OK|MB_ICONEXCLAMATION);
    return FALSE;
  }

  if (!CreateAdminMagnusConf(szSRoot))
  {
    NsSetupMessageBox(NULL, "Could not setup magnus.conf file.", NULL, MB_OK|MB_ICONEXCLAMATION);
    return FALSE;
  }

  if (!CreateAdminNSFCConf(szSRoot))
  {
    NsSetupMessageBox(NULL, "Could not setup nsfc.conf file.", NULL, MB_OK|MB_ICONEXCLAMATION);
    return FALSE;
  } 

  if (!CreateServerXML(szSRoot))
  {
    NsSetupMessageBox(NULL, "Could not setup server.xml file.", NULL, MB_OK|MB_ICONEXCLAMATION);
    return FALSE;
  }

  if (!CreateAdminObjConf(szSRoot))
  {
    NsSetupMessageBox(NULL, "Could not setup obj.conf file.", NULL, MB_OK|MB_ICONEXCLAMATION);
    return FALSE;
  }

  if (!CreateAdminACL(szSRoot, cd.szUsername))
  {
    NsSetupMessageBox(NULL, "Could not setup Admin ACLs.", NULL, MB_OK|MB_ICONEXCLAMATION);
    nReturn = FALSE;
  }

  sprintf(szConfigPath, "%s\\admin-serv\\logs\\access", szServerRoot);
  if(setupFileExists(szConfigPath)) {
    sprintf(szTemp, "copy %s %s.bak", szConfigPath, szConfigPath);
    system(szTemp);
    
    sprintf(szTemp, "del %s", szConfigPath);
    system(szTemp);
  }
  return (nReturn);
}


//--------------------------------------------------------------------------//
// Update schema to latest version for older versions of Directory Server   //
//--------------------------------------------------------------------------//
int _UpdateSchema(char *szServerRoot)
{
   char szSchemaPath[MAX_PATH];
   char szLdapUrl[MAX_PATH];
   
   sprintf(szSchemaPath, "%s\\setup\\admin\\ns-common-schema.conf", szServerRoot);
   
   sprintf(szLdapUrl, "ldap://%s:%d/%s",
           GetLdapHost(),
           GetLdapPort(),
           GetLdapSuffix());
   
   return setupInsertPluginSchemaEntries(szLdapUrl, 
                                         cd.szUsername,
                                         cd.szPassword,
                                         szSchemaPath);
}


/*
 * 359259: Add ACIs under o=netscapeRoot to allow regular, non-admin users to
 * locate admin servers. This is required for downloads of jars referenced from
 *  Users and Groups configuration under ou=Global Preferences.
 */
BOOL  SetLocateAdmservACI(Ldap *ldap) {
    LDAPMod *mods[2];
    LdapErrorCode ldapError;
    LDAPMod mod;
    char *aciVal[2];

    // Allow read access to host, port, address and security attributes for SIEs
    aciVal[0] = (char*)LOCATE_ADMSERV_ACI;
    aciVal[1] = NULL;

    mod.mod_op = LDAP_MOD_ADD;
    mod.mod_type = "aci";
    mod.mod_values = aciVal;

    mods[0] = &mod;
    mods[1] = NULL;

    ldapError = setupLdapModify(ldap, "o=netscapeRoot", mods);

    if (ldapError != LDAP_SUCCESS &&
        ldapError != LDAP_TYPE_OR_VALUE_EXISTS) {
        CHAR szErrorMsg[MAX_PATH];
        wsprintf(szErrorMsg, 
           "Could not modify ACI for o=netscapeRoot, \"%s\"", ldap_err2string(ldapError));
        NsSetupMessageBox(NULL, szErrorMsg, NULL, MB_OK|MB_ICONEXCLAMATION);
        return FALSE;
    }

    return TRUE;
}

/* 
 * 337487: Fix "Enable Group Expansion" ACI under o=netscapeRoot. The ACI
 * is defined as <groupdnattr="ldap:///o=NetscapeRoot?uniquemember?sub">.
 * This is instructing that all groups will be located under the
 * o=netscapeRoot suffix, which is true only for administrators. Set
 * <groupdnattr="uniquemember"> which allows a group to be under any suffix.
 */
BOOL RepairGroupExpansionACI(Ldap *ldap) {
   LdapErrorCode ldapError;
   LdapEntry *root = createLdapEntry(ldap);
   char ** aciVals, *aci;

    ldapError = retrieveLdapEntry(root, "o=netscapeRoot");
    if (ldapError != OKAY) {
        CHAR szErrorMsg[MAX_PATH];
        wsprintf(szErrorMsg, 
           "Could not read o=netscapeRoot, \"%s\"", ldap_err2string(ldapError));
        NsSetupMessageBox(NULL, szErrorMsg, NULL, MB_OK|MB_ICONEXCLAMATION);
        return FALSE;
    }
    
    aciVals = getLdapAttributes(root, "aci");
    while (aci=*aciVals++) {

        if (strstr(aci, "Enable Group Expansion") &&
            strstr(aci, "groupdnattr=\"ldap:///o=NetscapeRoot?uniquemember?sub\"")) {
        
            LDAPMod *mods[3];
            LDAPMod mod1, mod2;
            char *prevVal[2], *newVal[2];

            prevVal[0] = aci;
            prevVal[1] = NULL;

            newVal[0]  = (char*)GROUP_EXPANSION_ACI;
            newVal[1]  = NULL;

            mod1.mod_op = LDAP_MOD_DELETE;
            mod1.mod_type = "aci";
            mod1.mod_values = prevVal;

            mod2.mod_op = LDAP_MOD_ADD;
            mod2.mod_type = "aci";
            mod2.mod_values = newVal;

            mods[0] = &mod1;
            mods[1] = &mod2;
            mods[2] = NULL;

            ldapError = setupLdapModify(ldap, "o=netscapeRoot", mods);

            if (ldapError != LDAP_SUCCESS) {
                CHAR szErrorMsg[MAX_PATH];
                wsprintf(szErrorMsg, 
                   "Could not modify ACI for o=netscapeRoot, \"%s\"", ldap_err2string(ldapError));
                NsSetupMessageBox(NULL, szErrorMsg, NULL, MB_OK|MB_ICONEXCLAMATION);
                return FALSE;
            }
            else {
                return TRUE;
            }
        }
    }

    return TRUE;
}

//--------------------------------------------------------------------------//
// Fix ACIs under o=netscapeRoot                                            //
//--------------------------------------------------------------------------//
BOOL  _FixRootACIs() {
    char szLdapUrl[MAX_PATH];
    Ldap *ldap;
    LdapErrorCode ldapError;
    BOOL nReturn = TRUE;

    sprintf(szLdapUrl, "ldap://%s:%d/%s",
           GetLdapHost(),
           GetLdapPort(),
           GetLdapSuffix());

    ldapError = setupCreateLdap(&ldap, szLdapUrl,
       cd.szUsername, cd.szPassword, NULL, NULL);
    if (ldapError !=  LDAP_SUCCESS) {
        CHAR szErrorMsg[MAX_PATH];
        wsprintf(szErrorMsg, 
           "Could not authenticate ldap connection, \"%s\"", ldap_err2string(ldapError));
        NsSetupMessageBox(NULL, szErrorMsg, NULL, MB_OK|MB_ICONEXCLAMATION);
        return FALSE;
    }

    nReturn = RepairGroupExpansionACI(ldap) && SetLocateAdmservACI(ldap);

    destroyLdap(ldap);

    return nReturn;
}

//--------------------------------------------------------------------------//
// Set xml attribute value on a line that starts with section id. See       //
// modify_xml_file                                                          // 
// Returns: -1 error, 0 no changes,  n>0 number of changes on the line      //
//--------------------------------------------------------------------------//
int set_xml_attr(char *line, char *section,  char *attr, char *val) {
    /*
       Check if the line starts with the section id string. The id string may
       be prefixed with white spaces and '<'.
     */
    char *section_ref = strstr(line, section);
    char *c, *attr_ref;
    char out[BIG_BUF];
    if (section_ref == NULL) {
        return 0;
    }
    if (section_ref != line) {
        for (c=line; c != section_ref; c++) {
			if (*c != ' ' && *c != '\t' && *c!= '<') {
                return 0;
            }
        }
    }

    /* Find the attribute */
    attr_ref = strstr (line, attr);
    if (attr_ref == NULL) {
       return 0;
    }

    /* Replace the attribute value */
    memset(out, 0, BIG_BUF); 
    memcpy(out, line, attr_ref-line);
    for (c=attr_ref; c && *c != '"'; c++) {
       ; /* do nothing */ 
    }
    if (c == NULL) {
        return -1; /* malformed line */
    }
    else {
        c++;
    }

    /* Skip the current value */
    while ( c && *c != '"') {
        c++;
    }
    if (c == NULL) {
        return -1; /* malformed line */
    }
    sprintf(out,"%s%s=\"%s%s", out, attr, val, c);
    strcpy(line, out);
    return 1;
}

//--------------------------------------------------------------------------//
// Modifies attributes in a xml file according to the mod descroptors of    //
// type xml_file_mod_t                                                      // 
//--------------------------------------------------------------------------//    
BOOL modify_xml_file(char *filename, xml_file_mod_t **mods) {
    char filename0[BIG_BUF];
    char line[BIG_BUF];
    FILE *f0, *f;
    int i;

    /* Create temporary file with the current filename value */
    sprintf(filename0, "%s.0", filename);
    if (rename(filename, filename0) != 0) {
        CHAR szErrorMsg[MAX_PATH];
        wsprintf(szErrorMsg, "Can't rename %s", filename);
        NsSetupMessageBox(NULL, szErrorMsg, NULL, MB_OK|MB_ICONEXCLAMATION);
        return FALSE;
    }

   f0 = fopen (filename0, "r");
   if (f0 == NULL) {
        CHAR szErrorMsg[MAX_PATH];
        wsprintf(szErrorMsg, "Can not open %s", filename0);
        NsSetupMessageBox(NULL, szErrorMsg, NULL, MB_OK|MB_ICONEXCLAMATION);
        return FALSE;
   }
    
   /* The output file */
   f = fopen (filename, "w");
   if (f == NULL) {
        CHAR szErrorMsg[MAX_PATH];
        wsprintf(szErrorMsg, "Can not open %s for writing", filename);
        NsSetupMessageBox(NULL, szErrorMsg, NULL, MB_OK|MB_ICONEXCLAMATION);
        return FALSE;
   }

   while (fgets(line, BIG_BUF, f0)) {
       for (i=0; mods[i]; i++) {
           if (set_xml_attr(line, mods[i]->section, mods[i]->attr, mods[i]->val) < 0) {
               CHAR szErrorMsg[MAX_PATH];
               wsprintf(szErrorMsg, "%s has invalid format", filename);
               NsSetupMessageBox(NULL, szErrorMsg, NULL, MB_OK|MB_ICONEXCLAMATION);
               return FALSE;
           }
       }
       fprintf(f, "%s", line);
   }

   fclose(f);
  
   /* Remove temporary file */
   fclose(f0);
   remove(filename0);

   return TRUE;
}
