/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
//--------------------------------------------------------------------------//
//                                                                          //
//  Name: Administration Server Configuration                               //
//  Platforms: WIN32                                                        //
//  Revision History:                                                       //
//     use cvs log
//
//--------------------------------------------------------------------------//

#include <windows.h>
#include <commctrl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <crtdbg.h>
#include <wingdi.h>

#include <nssetup.h>
#include <ldapu.h>
#include "resource.h"
#include "insadm.h"
#include "nt/regparms.h"

MODULEINFO mi = { NULL, NULL, NS_WIZERROR};
CONTROLDATA cd;

static BOOL _RemoveSIE(char *szServerRoot)
{
   char szFileName[MAX_PATH];
   char szValue[MAX_PATH];

   sprintf(szFileName, "%s\\admin-serv\\config\\adm.conf", szServerRoot);
   szValue[0] = '\0';
   GetProductInfoString("SIE", szValue, MAX_PATH, szFileName);
   if (strlen(szValue))
   {
      SIEDelete(szValue);
   }
   else return FALSE;
   szValue[0] = '\0';
   GetProductInfoString("ISIE", szValue, MAX_PATH, szFileName);
   if (strlen(szValue))
   {
      SIEDeleteAll(szValue);
   }
   else return FALSE;
   return TRUE;
}


//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
static void _InitializeControlData(void)
{
   ZeroMemory(&cd, sizeof(CONTROLDATA));

   cd.nReplaceVersion = 0;
   cd.bShowAdmConfig = TRUE;
   cd.bDoAdmInstall = TRUE;
   sprintf(cd.szUsername, ADM_DEFAULT_USER);
   cd.szServerAddress = NULL;
   GetHostName(cd.szHostname, sizeof(cd.szHostname));
   cd.nPort = GetRandomNetworkPort();
   sprintf(cd.szUrl, "http://%s:%d", cd.szHostname, cd.nPort);

   cd.szServerAddress = _strdup("\0");

#if 0
   wVersionRequested = MAKEWORD( 2, 0 );

   err = WSAStartup( wVersionRequested, &wsaData );
   if (err == 0)
   {
    /* Confirm that the WinSock DLL supports 2.0.*/
    /* Note that if the DLL supports versions greater    */
    /* than 2.0 in addition to 2.0, it will still return */
    /* 2.0 in wVersion since that is the version we      */
    /* requested.                                        */
      if (LOBYTE( wsaData.wVersion ) != 2 ||
          HIBYTE( wsaData.wVersion ) != 0)
      {
      /* Tell the user that we couldn't find a usable */
      /* WinSock DLL.                                  */
         WSACleanup( );
      }
      else /* The WinSock DLL is acceptable. Proceed. */
      {
         HOSTENT *hHost;

         hHost = gethostbyname(cd.szHostname);
      //		err =  WSAGetLastError ();
         if (hHost != NULL)
         {
            IN_ADDR in;
            ZeroMemory(&in, sizeof(in));
            memcpy(&in.s_addr, hHost->h_addr_list[0], sizeof(in.s_addr));
            cd.szServerAddress = _strdup(inet_ntoa(in));
         }
      }
   }
   if (cd.szServerAddress == NULL)
   {
      cd.szServerAddress = _strdup(" ");
   }
#endif
}

//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
static void _SaveControlData(void)
{

   if (MODE == CUSTOMMODE)
   {
      if (cd.hwndPage[NDX_PG_FIRST])
      {
         char szIPaddr[MAX_PATH];
//         _ASSERT(IsWindow(cd.hwndPage[NDX_PG_FIRST]));
         if (cd.szServerAddress) free(cd.szServerAddress);
         GetDlgItemText(cd.hwndPage[NDX_PG_FIRST], IDC_IPADDR, szIPaddr, MAX_PATH);
         cd.szServerAddress = _strdup(szIPaddr);
      }
#if 0
      // Take this page out! admin-server's user/password is confusion. User should set it
      // through console.
      if (cd.hwndPage[NDX_PG_SECOND])
      {
//         _ASSERT(IsWindow(cd.hwndPage[NDX_PG_SECOND]));
         GetDlgItemText(cd.hwndPage[NDX_PG_SECOND], IDC_USERNAME, cd.szUsername, sizeof(cd.szUsername));
         GetDlgItemText(cd.hwndPage[NDX_PG_SECOND], IDC_PASSWORD, cd.szPassword, sizeof(cd.szPassword));
         GetDlgItemText(cd.hwndPage[NDX_PG_SECOND], IDC_PASSWORD_CONFIRM, cd.szPasswordConfirm, sizeof(cd.szPasswordConfirm));
      }
#endif
      if (cd.hwndPage[NDX_PG_THIRD])
      {
         BOOL bSuccess;
         int nPort;

//         _ASSERT(IsWindow(cd.hwndPage[NDX_PG_THIRD]));

         nPort = GetDlgItemInt(cd.hwndPage[NDX_PG_THIRD], IDC_ADMIN_PORT, &bSuccess, FALSE);

         if (bSuccess)
         {
            cd.nPort = nPort;
            sprintf(cd.szUrl, "http://%s:%d", cd.szHostname, cd.nPort);
         }
      }
   }
   else
   {
      if (cd.hwndPage[NDX_PG_FIRST])
      {
         BOOL bSuccess;
         int nPort;

//         _ASSERT(IsWindow(cd.hwndPage[NDX_PG_FIRST]));

         nPort = GetDlgItemInt(cd.hwndPage[NDX_PG_FIRST], IDC_ADMIN_PORT, &bSuccess, FALSE);

         if (bSuccess)
         {
            cd.nPort = nPort;
            sprintf(cd.szUrl, "http://%s:%d", cd.szHostname, cd.nPort);
         }
      }
   }
}


//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
static void _RefreshPW(HWND hwndDlg)
{
//   _ASSERT(IsWindow(hwndDlg));

   SetDlgItemText(hwndDlg, IDC_USERNAME, cd.szUsername);
   SetDlgItemText(hwndDlg, IDC_PASSWORD, cd.szPassword);
   SetDlgItemText(hwndDlg, IDC_PASSWORD_CONFIRM, cd.szPasswordConfirm);
}

static void _RefreshIPAddr(HWND hwndDlg)
{
//   _ASSERT(IsWindow(hwndDlg));
   SetDlgItemText(hwndDlg, IDC_IPADDR, cd.szServerAddress);
}

//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
static void _RefreshPORT(HWND hwndDlg, BOOL bUpdatePort)
{
   char szText[MAX_PATH];

//   _ASSERT(IsWindow(hwndDlg));

   if (bUpdatePort)
      SetDlgItemInt(hwndDlg, IDC_ADMIN_PORT, cd.nPort, FALSE);

   sprintf(szText, "http://%s:%d/", cd.szHostname, cd.nPort);
   SetDlgItemText(hwndDlg, IDS_ADMIN_URL, szText);
}

//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
static void _GetPortAdmin(HWND hwndDlg)
{
   BOOL bSuccess;
   int nPort;

//   _ASSERT(IsWindow(hwndDlg));

   nPort = GetDlgItemInt(hwndDlg, IDC_ADMIN_PORT, &bSuccess, FALSE);

   if (bSuccess)
      cd.nPort = nPort;
}

//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
static BOOL _IsNextPORT(HWND hwndDlg)
{
   BOOL bReturn = FALSE;
   char szUserName[MAX_PATH];
   char szPassword1[MAX_PATH];
   char szPassword2[MAX_PATH];

//   _ASSERT(IsWindow(hwndDlg));

   GetDlgItemText(hwndDlg, IDC_USERNAME, szUserName, sizeof(szUserName));
   GetDlgItemText(hwndDlg, IDC_PASSWORD, szPassword1, sizeof(szPassword1));
   GetDlgItemText(hwndDlg, IDC_PASSWORD_CONFIRM, szPassword2, sizeof(szPassword2));

   if (strlen(szUserName) && strlen(szPassword1) && strlen(szPassword2))
      bReturn = TRUE;

   return (bReturn);
}

//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
static BOOL _IsNextIPaddr(HWND hwndDlg)
{
   return (TRUE);
}

//--------------------------------------------------------------------------//
// Administration Server Authentication                                     //
//--------------------------------------------------------------------------//
static BOOL CALLBACK _DialogProcPW(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
   BOOL bReturn = FALSE;

//   _ASSERT(IsWindow(hwndDlg));

   switch (uMsg)
   {
   case WM_INITDIALOG:
      {
         HGDIOBJ hGdiObj;
         hGdiObj =  GetStockObject(DEFAULT_GUI_FONT);
         SendMessage(hwndDlg, WM_SETFONT, (WPARAM) ((struct HFONT__ *)hGdiObj) , MAKELPARAM(TRUE, 0));
         cd.hwndPage[NDX_PG_SECOND] = hwndDlg;
         _RefreshPW(hwndDlg);
         break;
      }
   case WM_NOTIFY:
      switch (((NMHDR FAR *)lParam)->code)
      {
      case PSN_SETACTIVE:
#if 1
         SetWindowLong(hwndDlg, DWL_MSGRESULT, -1);
         bReturn = TRUE;
#else
         CenterWindow(GetParent(hwndDlg));
         if (MODE == CUSTOMMODE)
         {
            PropSheet_SetWizButtons(GetParent(hwndDlg), PSWIZB_BACK | (_IsNextPORT(hwndDlg) ? PSWIZB_NEXT : 0));
         }
         else PropSheet_SetWizButtons(GetParent(hwndDlg), PSWIZB_BACK );
#endif

         break;

      case PSN_KILLACTIVE:
         break;

      case PSN_RESET:
         break;

      case PSN_QUERYCANCEL:
         if (mi.m_nResult == NS_WIZERROR)
         {
            if (QueryExit(hwndDlg))
               mi.m_nResult = NS_WIZCANCEL;
            else
            {
               SetWindowLong(hwndDlg, DWL_MSGRESULT, -1);
               bReturn = TRUE;
            }
         }
         break;

      case PSN_WIZBACK:
         _SaveControlData();
         if (MODE!=CUSTOMMODE)
         {
            mi.m_nResult = NS_WIZBACK;
            SendMessage(GetParent(hwndDlg), WM_CLOSE, 0, 0); // brute force method
         }
         break;

      case PSN_WIZNEXT:
         _SaveControlData();
         if (MODE == CUSTOMMODE)
         {
            if (strcmp(cd.szPassword, cd.szPasswordConfirm) != 0)
            {
               NsSetupMessageBox(GetParent(hwndDlg), ADM_ERR_PASSWORD_DIFF, ADM_ERR_DLG_TITLE, MB_ICONWARNING | MB_OK);
               cd.szPassword[0] = '\0';
               cd.szPasswordConfirm[0] = '\0';
               _RefreshPW(hwndDlg);
               SetFocus(GetDlgItem(hwndDlg, IDC_PASSWORD));
               SetWindowLong(hwndDlg, DWL_MSGRESULT, -1);
               bReturn = TRUE;
            }
         }
         else
         {
            mi.m_nResult = NS_WIZNEXT;
            SendMessage(GetParent(hwndDlg), WM_CLOSE, 0, 0); // brute force method
         }

      default:
         break;
      }
      break;

   case WM_COMMAND:
      switch (LOWORD(wParam))
      {
      case IDC_USERNAME:
      case IDC_PASSWORD:
      case IDC_PASSWORD_CONFIRM:
         if (HIWORD(wParam) == EN_CHANGE)
         {
            PropSheet_SetWizButtons(GetParent(hwndDlg), PSWIZB_BACK | (_IsNextPORT(hwndDlg) ? PSWIZB_NEXT : 0));
         }
         break;

      default:
         break;
      }

   default:
      bReturn = FALSE;
      break;
   }

   return (bReturn);
}
////////////////////////////////////////////////////////////////////////////
// ip address dialog
//////////////////////////////////////////////////////////
static BOOL CALLBACK _DialogProcIPAddr(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
   BOOL bReturn = FALSE;

//   _ASSERT(IsWindow(hwndDlg));
   switch (uMsg)
   {
   case WM_INITDIALOG:
      {
         HGDIOBJ hGdiObj;
         hGdiObj =  GetStockObject(DEFAULT_GUI_FONT);
         SendMessage(hwndDlg, WM_SETFONT, (WPARAM) ((struct HFONT__ *)hGdiObj) , MAKELPARAM(TRUE, 0));
         cd.hwndPage[NDX_PG_FIRST] = hwndDlg;
         _RefreshIPAddr(hwndDlg);
         break;
      }

   case WM_NOTIFY:
      switch (((NMHDR FAR *)lParam)->code)
      {
      case PSN_SETACTIVE:
         CenterWindow(GetParent(hwndDlg));
         if (MODE == CUSTOMMODE)
         {
            PropSheet_SetWizButtons(GetParent(hwndDlg), PSWIZB_BACK | (_IsNextIPaddr(hwndDlg) ? PSWIZB_NEXT : 0));
         }
         else PropSheet_SetWizButtons(GetParent(hwndDlg), PSWIZB_BACK );

         break;

      case PSN_KILLACTIVE:
         break;

      case PSN_RESET:
         break;

      case PSN_QUERYCANCEL:
         if (mi.m_nResult == NS_WIZERROR)
         {
            if (QueryExit(hwndDlg))
               mi.m_nResult = NS_WIZCANCEL;
            else
            {
               SetWindowLong(hwndDlg, DWL_MSGRESULT, -1);
               bReturn = TRUE;
            }
         }
         break;

      case PSN_WIZBACK:
         mi.m_nResult = NS_WIZBACK;
         SendMessage(GetParent(hwndDlg), WM_CLOSE, 0, 0); // brute force method
         break;

      case PSN_WIZNEXT:
         _SaveControlData();
         if (MODE == CUSTOMMODE)
         {
         }
         else
         {
            mi.m_nResult = NS_WIZNEXT;
            SendMessage(GetParent(hwndDlg), WM_CLOSE, 0, 0); // brute force method
         }

      default:
         break;
      }
      break;

   case WM_COMMAND:
      switch (LOWORD(wParam))
      {
      case IDC_IPADDR:
         if (HIWORD(wParam) == EN_CHANGE)
         {
            PropSheet_SetWizButtons(GetParent(hwndDlg), PSWIZB_BACK | (_IsNextIPaddr(hwndDlg) ? PSWIZB_NEXT : 0));
         }
         break;

      default:
         break;
      }

   default:
      bReturn = FALSE;
      break;
   }
   return (bReturn);
}

//--------------------------------------------------------------------------//
// Administration Server Port Selection                                     //
//--------------------------------------------------------------------------//
static BOOL CALLBACK _DialogProcPort(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
   BOOL bReturn = FALSE;

//   _ASSERT(IsWindow(hwndDlg));

   switch (uMsg)
   {
   case WM_INITDIALOG:
      {
         HGDIOBJ hGdiObj;
         hGdiObj =  GetStockObject(DEFAULT_GUI_FONT);
         SendMessage(hwndDlg, WM_SETFONT, (WPARAM) ((struct HFONT__ *)hGdiObj) , MAKELPARAM(TRUE, 0));

         if (MODE!=CUSTOMMODE)
            cd.hwndPage[NDX_PG_FIRST] = hwndDlg;
         else
            cd.hwndPage[NDX_PG_THIRD] = hwndDlg;

         SendDlgItemMessage(hwndDlg, IDC_ADMIN_PORT_SPIN, UDM_SETBUDDY, (WPARAM)GetDlgItem(hwndDlg, IDC_ADMIN_PORT), 0);
         SendDlgItemMessage(hwndDlg, IDC_ADMIN_PORT_SPIN, UDM_SETRANGE, 0, MAKELONG((short)UD_MAXVAL, (short)1));
         _RefreshPORT(hwndDlg, TRUE);
         break;
      }
   case WM_NOTIFY:
      switch (((NMHDR FAR *)lParam)->code)
      {
      case PSN_SETACTIVE:
         CenterWindow(GetParent(hwndDlg));
         PropSheet_SetWizButtons(GetParent(hwndDlg), PSWIZB_BACK | PSWIZB_NEXT);
         break;

      case PSN_KILLACTIVE:
         break;

      case PSN_RESET:
         break;

      case PSN_QUERYCANCEL:
         if (mi.m_nResult == NS_WIZERROR)
         {
            if (QueryExit(hwndDlg))
               mi.m_nResult = NS_WIZCANCEL;
            else
            {
               SetWindowLong(hwndDlg, DWL_MSGRESULT, -1);
               bReturn = TRUE;
            }
         }
         break;

      case PSN_WIZBACK:
         if (MODE!=CUSTOMMODE)
         {
            _SaveControlData();
            mi.m_nResult = NS_WIZBACK;
            SendMessage(GetParent(hwndDlg), WM_CLOSE, 0, 0); // brute force method
         }
         break;


      case PSN_WIZNEXT:
         _SaveControlData();
         if (!IsValidNetworkPort(cd.nPort))
         {
            if (NsSetupMessageBox(GetParent(hwndDlg), ADM_ERR_PORT_BUSY, ADM_ERR_DLG_TITLE, MB_ICONWARNING | MB_OK) == IDOK)
            {
               SetFocus(GetDlgItem(hwndDlg, IDC_ADMIN_PORT));
               SetWindowLong(hwndDlg, DWL_MSGRESULT, -1);
               break;
            }
         }
         mi.m_nResult = NS_WIZNEXT;
         SendMessage(GetParent(hwndDlg), WM_CLOSE, 0, 0); // brute force method
         break;

      default:
         break;
      }
      break;

   case WM_COMMAND:
      switch (LOWORD(wParam))
      {
      case IDC_ADMIN_PORT:
         if (HIWORD(wParam) == EN_UPDATE)
         {
            _GetPortAdmin(hwndDlg);
            _RefreshPORT(hwndDlg, FALSE);
         }
         else if (HIWORD(wParam) == EN_KILLFOCUS)
         {
            _GetPortAdmin(hwndDlg);
            _RefreshPORT(hwndDlg, TRUE);
         }
         break;

      default:
         break;
      }


   default:
      bReturn = FALSE;
      break;
   }

   return (bReturn);
}

//////////////////////////////////////////////////////////////////////////////
// Exported Functions
//
// The following functions are exported by this DLL for use by the
// installation shell.  These functions conform to the guidelines set forth
// for SuiteSpot Windows NT installation programs.
//
//

BOOL __declspec(dllexport) ADM_PreInstall(VOID)
{
   char szFilePath[MAX_PATH];
   char szFormat[MAX_PATH];
   char szErrMsg[MAX_PATH * 2];
   char szService[MAX_PATH];

   cd.nReplaceVersion = 0;
   wsprintf(szFilePath, "%s\\setup\\admin\\admin.inf", TARGETDIR);
   if (setupFileExists(szFilePath))
   {
      char *szVer;
      szVer = setupGetInfString("admin", SETUP_INF_VERSION, NULL, szFilePath);
      if (szVer)
      {
         if (strcmp(szVer, "7.0") == 0) /* The current version;
                                           target of brandAdminServer */
         {
            cd.nReplaceVersion = 70; /* current */
            sprintf(szService, "admin70-serv"); /* current */
         }
         else if (strcmp(szVer, "4.0") == 0)
         {
            cd.nReplaceVersion = 1;
            sprintf(szService, "admin40-serv");
         }
         else if (strcmp(szVer, "4.1") == 0)
         {
            cd.nReplaceVersion = 2;
            sprintf(szService, "admin41-serv");
         }
         else if (strcmp(szVer, "4.2") == 0)
         {
            cd.nReplaceVersion = 3;
            sprintf(szService, "admin42-serv");
         }
         else if (strcmp(szVer, "6.0") == 0)
         {
            cd.nReplaceVersion = 4;
            sprintf(szService, "admin60-serv");
         }
         else if (strcmp(szVer, "6.01") == 0)
         {
            cd.nReplaceVersion = 5;
            sprintf(szService, "admin60-serv");
         }
         else if (strcmp(szVer, "6.1") == 0)
         {
            cd.nReplaceVersion = 6;
            sprintf(szService, "admin61-serv");
         }
         else
         {
            sprintf(szService, "%s-serv", ADM_ID_SERVICE);
         }
      }
   }

   LoadString(mi.m_hModule, IDS_ERR_MISSINGFILE, szFormat, sizeof(szFormat));

   sprintf(szFilePath, "%s\\tasks.ldif", SRCDIR);
   if (!FileExists(szFilePath))
   {
      sprintf(szErrMsg, szFormat, szFilePath);
      NsSetupMessageBox(NULL, szErrMsg, NULL, MB_OK|MB_ICONSTOP);
      return FALSE;
   }

   sprintf(szFilePath, "%s\\global.ldif", SRCDIR);
   if (!FileExists(szFilePath))
   {
      sprintf(szErrMsg, szFormat, szFilePath);
      NsSetupMessageBox(NULL, szErrMsg, NULL, MB_OK|MB_ICONSTOP);
      return FALSE;
   }

   if (_GetAdminConfig(TARGETDIR, cd.szHostname, &cd.nPort))
   {

      int iTries = 1;
      BOOL bServerRunning = TRUE;
      char szUrl[MAX_PATH];

      LogData(NULL, "Admin Reinstall ");
      cd.bShowAdmConfig = FALSE;
      sprintf(szUrl, "http://%s:%d", cd.szHostname, cd.nPort);
      strcpy(cd.szUrl, szUrl);

      // Stop the AS if running
      ControlServer(szService, FALSE);

      while (bServerRunning)
      {
         bServerRunning = isServiceRunning(szService);
         if (bServerRunning)
         {
            Sleep(10);
            if (iTries == 500)
            {
               ControlServer(szService, FALSE);
            }
         }
         iTries++;
         if (iTries == 1000)
         {
            char szErrMsg[MAX_PATH];
        //unable to stop server
            bServerRunning = FALSE;
            sprintf(szErrMsg, "Error unable to stop service %s installation may not be successful.", szService); //need to put string in RC
            NsSetupMessageBox(NULL, szErrMsg, NULL, MB_OK|MB_ICONSTOP);
         }
      }
   }

   return TRUE;
}

int __declspec(dllexport) ADM_AskOptions(HWND hWnd, int nDirection)
{
   PROPSHEETPAGE psp[MAX_PROP_PAGES];
   int nPages = 1;
   int nStartPage = 0;

   if (!cd.bShowAdmConfig)
   {
      mi.m_nResult = nDirection;  // keep moving in same direction...
   }
   else
   {
      ZeroMemory(cd.hwndPage, sizeof(cd.hwndPage));

      mi.m_nResult = NS_WIZERROR;

      if (MODE != EXPRESSMODE)
      {
         if (MODE == CUSTOMMODE)
         {
            nStartPage = ((nDirection == NS_WIZNEXT) ? NDX_PG_FIRST : NDX_PG_THIRD);
            AddWizardPage(mi.m_hModule, &psp[NDX_PG_FIRST],   IDD_PG_IPADDRESS,  _DialogProcIPAddr);
            nPages++;
            AddWizardPage(mi.m_hModule, &psp[NDX_PG_SECOND],   IDD_PG_PW,   _DialogProcPW);
            nPages++;
            AddWizardPage(mi.m_hModule, &psp[NDX_PG_THIRD], IDD_PG_PORT, _DialogProcPort);
         }
         else
         {
            nStartPage = NDX_PG_FIRST;
            AddWizardPage(mi.m_hModule, &psp[NDX_PG_FIRST], IDD_PG_PORT, _DialogProcPort);
         }
         if (WizardDialog(mi.m_hModule, hWnd, psp, nPages, nStartPage) < 0)
         {
            mi.m_nResult = NS_WIZERROR;
         }
      }
      else  mi.m_nResult = nDirection;
   }

   return mi.m_nResult;
}

void __declspec(dllexport) ADM_GetSummary(char* pszSummaryText)
{
   char* psz = pszSummaryText;

   if (cd.bShowAdmConfig)
   {
      psz += sprintf(psz, ADM_INFO_ADMUSER, cd.szUsername);
      psz += sprintf(psz, ADM_INFO_ADMPORT, cd.nPort);
      psz += sprintf(psz, ADM_INFO_ADMURL,  cd.szUrl);
      if (MODE == CUSTOMMODE)
      {
         psz += sprintf(psz, ADM_INFO_SERVERIP,  cd.szServerAddress);
      }

   }
   else if (cd.bDoAdmInstall)
   {
      psz += sprintf(psz, ADM_INFO_REFRESH);
   }
   else
   {
      psz += sprintf(psz, ADM_INFO_NOTHING);
   }
}

BOOL __declspec(dllexport) ADM_WriteGlobalCache(LPCSTR pszCacheFile, LPCSTR pszSection)
{

   WritePrivateProfileString(pszSection, GLOBAL_INF_FULLMACHNAME, cd.szHostname, pszCacheFile);
   WritePrivateProfileString(pszSection, GLOBAL_INF_SUITESPOT_USERID, "nobody", pszCacheFile);
   WritePrivateProfileString(pszSection, GLOBAL_INF_SUITESPOT_USERGROUP,  "nobody", pszCacheFile);
   return TRUE;
}

BOOL __declspec(dllexport) ADM_WriteLocalCache(LPCSTR pszCacheFile, LPCSTR pszSection)
{
   char szPort[16];

   sprintf(szPort, "%d", cd.nPort);
   WritePrivateProfileString(pszSection, ADM_CFG_PORT,     szPort,        pszCacheFile);
   WritePrivateProfileString(pszSection, ADM_CFG_SYSUSER,  "root",        pszCacheFile);

   {
      char szValue[MAX_PATH];
      char *szParse;

      szParse = strpbrk(cd.szUsername, "=");
      if (szParse != NULL)
      {
         char*szRemove;
         szRemove = strpbrk(szParse, ",");
         szRemove = '\0';
         strcpy(szParse, szValue);
         WritePrivateProfileString(pszSection, ADM_CFG_USERID,   szValue, pszCacheFile);
      }
      else WritePrivateProfileString(pszSection, ADM_CFG_USERID,   cd.szUsername, pszCacheFile);
   }
   WritePrivateProfileString(pszSection, ADM_CFG_PASSWORD, cd.szPassword, pszCacheFile);
   if (MODE == CUSTOMMODE)
   {
      WritePrivateProfileString(pszSection, ADM_CFG_IPADDRESS,  cd.szServerAddress, pszCacheFile);
   }

   return TRUE;
}

BOOL __declspec(dllexport) ADM_ReadGlobalCache(LPCSTR pszCacheFile, LPCSTR pszSection)
{
   char szValue[MAX_PATH];
   char *szParse = NULL;

   cd.szCacheFile = _strdup(pszCacheFile);
   GetPrivateProfileString(pszSection, GLOBAL_INF_FULLMACHNAME, "", szValue,    sizeof(szValue), pszCacheFile);
   if (strlen(szValue))
   {
      strcpy(cd.szHostname, szValue);
   }

   GetPrivateProfileString(pszSection, GLOBAL_INF_LDAP_USER, "", szValue, sizeof(szValue), pszCacheFile);
   if (strlen(szValue))
   {
      strcpy(cd.szUsername, szValue);
   }
   else
   {
      sprintf(szValue, "Error install cache missing %s", GLOBAL_INF_LDAP_USER);
      LogData(NULL, szValue);
      return FALSE;
   }

   GetPrivateProfileString(pszSection, GLOBAL_INF_LDAP_PASSWD, "", szValue, sizeof(szValue), pszCacheFile);
   if (strlen(szValue))
   {
      strcpy(cd.szPassword, szValue);
   }
   else
   {
      sprintf(szValue, "Error install cache missing %s", GLOBAL_INF_LDAP_PASSWD);
      LogData(NULL, szValue);
      return FALSE;
   }

   if (MODE == CUSTOMMODE)
   {
    //get the ldap user then ldap password

      GetPrivateProfileString(ADM_CFG_SECTION, ADM_CFG_IPADDRESS, "", szValue, sizeof(szValue), pszCacheFile);
      if (strlen(szValue))
      {
         cd.szServerAddress = _strdup(szValue);
      }
    // else leave it as was initialized.
   }
   else
   {
      WritePrivateProfileString(ADM_CFG_SECTION, ADM_CFG_USERID,   cd.szUsername, pszCacheFile);
      WritePrivateProfileString(ADM_CFG_SECTION, ADM_CFG_PASSWORD, cd.szPassword, pszCacheFile);
   }

   return TRUE;
}

BOOL __declspec(dllexport) ADM_ReadLocalCache(LPCSTR pszCacheFile, LPCSTR pszSection)
{
   char szPort[16];
   char szValue[MAX_PATH];

   GetPrivateProfileString(pszSection, ADM_CFG_PORT,     "", szPort,        sizeof(szPort),        pszCacheFile);
   cd.nPort = atoi(szPort);

#if 0
  //if cd.username and cd.szPassword will allread have values because adm_readglobalcache is call first
  //values will only be change if this is a custom install
   if (MODE == CUSTOMMODE)
   {
      GetPrivateProfileString(pszSection, ADM_CFG_USERID,   cd.szUsername, cd.szUsername, sizeof(cd.szUsername), pszCacheFile);
      GetPrivateProfileString(pszSection, ADM_CFG_PASSWORD, cd.szPassword, cd.szPassword, sizeof(cd.szPassword), pszCacheFile);
   }
#endif
   sprintf(cd.szUrl, "http://%s:%s", cd.szHostname, szPort);

  //only get setver ip address if it is there otherwise use default
   GetPrivateProfileString(pszSection, ADM_CFG_IPADDRESS, "", szValue, sizeof(szValue), pszCacheFile);
   if (strlen(szValue))
   {
      if (cd.szServerAddress)
      {
         free(cd.szServerAddress);
      }
      cd.szServerAddress = _strdup(szValue);  //overwrite existing value set in _InitializeControlData
   }
  // TODO: add code to verify the data just read...
   return TRUE;
}

BOOL __declspec(dllexport) ADM_ReadComponentInf(LPCSTR pszCacheFile, LPCSTR pszSection)
{

   char szValue[MAX_PATH];

   GetPrivateProfileString(pszSection, NS_INF_DEFAULT_LANGUAGE, "en", cd.szLanguage, sizeof(cd.szLanguage), pszCacheFile);
   GetPrivateProfileString(pszSection, NS_INF_VENDOR, "", szValue, sizeof(szValue), pszCacheFile);
   cd.szVendor = _strdup(szValue);
   GetPrivateProfileString(pszSection, NS_INF_DESC, "", szValue, sizeof(szValue), pszCacheFile);
   cd.szDescription = _strdup(szValue);
   GetPrivateProfileString(pszSection, NS_INF_NAME, "", szValue, sizeof(szValue), pszCacheFile);
   cd.szName = _strdup(szValue);
   GetPrivateProfileString(pszSection, NS_INF_NICKNAME, "", szValue, sizeof(szValue), pszCacheFile);
   cd.szNickname = _strdup(szValue);
   GetPrivateProfileString(pszSection, NS_INF_VERSION, "", szValue, sizeof(szValue), pszCacheFile);
   cd.szVersion = _strdup(szValue);
   GetPrivateProfileString(pszSection, NS_INF_BUILDNUMBER, "", szValue, sizeof(szValue), pszCacheFile);
   cd.szBuildNumber = _strdup(szValue);
   GetPrivateProfileString(pszSection, NS_INF_REVISION, "", szValue, sizeof(szValue), pszCacheFile);
   cd.szRevision = _strdup(szValue);
   GetPrivateProfileString(pszSection, NS_INF_EXPIRY, "", szValue, sizeof(szValue), pszCacheFile);
   cd.szExpireDate = _strdup(szValue);
   GetPrivateProfileString(pszSection, NS_INF_SECURITY, "", szValue, sizeof(szValue), pszCacheFile);
   cd.szSecurity = _strdup(szValue);
   get_gmtime(szValue, sizeof(szValue));
   cd.szTimeStamp = _strdup(szValue);
  // TODO: add code to verify the data just read...
   return TRUE;

}

//
// Check if a file is a Console jar :
// base.jar ldapjdk.jar jss*.jar mcc* nmclf*
//
static int isNewInstFile(char *f) {
    if (strcmp(f, "base.jar") == 0) {
        return 1;
    }
    else if (strcmp(f, "ldapjdk.jar") == 0) { 
         return 1;
    }
    else if (strcmp(f, "jss3.jar") == 0) { 
         return 1;
    }
    else if (strstr(f, "mcc70") == f) { 
         return 1;
    }
    else if (strstr(f, "nmclf70") == f) { 
         return 1;
    }
    return 0;
}

// In AS 6.0 and 6.01 the ES core components DLLs are stored along with 
// the ES executable thus overriding the system32 directory. Force new
// installation to use system32 DLLs i.e. components packaged by AS.
static void removeESShadowDLLs() {
   char szSystemRoot[MAX_PATH];
   char szSystem32Path[MAX_PATH];
   char szHttpsBinDir[MAX_PATH];
   char szFileSpec[MAX_PATH];
   WIN32_FIND_DATA FindFileData;
   LPWIN32_FIND_DATA lpFindFileData = &FindFileData;
   HANDLE hFile;


   if (GetEnvironmentVariable("SystemRoot", szSystemRoot, sizeof(szSystemRoot)) == 0) {
       setupLogMessage("Error", "admin", "removeESShadowDLLs(): SystemRoot environment variable undefined");
       return;
   }

   sprintf(szHttpsBinDir, "%s\\bin\\https\\bin", TARGETDIR);

   // Remove jar files from previous installation
   wsprintf(szFileSpec, "%s\\*.dll", szHttpsBinDir);
   hFile = FindFirstFile(szFileSpec, lpFindFileData);
   if (hFile != INVALID_HANDLE_VALUE)
   {
      char szSrcFile[MAX_PATH];
      char *szFileName;
      do
      {
         szFileName = lpFindFileData->cFileName;

         if (strcmp(szFileName, "PSAPI.DLL") == 0) {
             continue;
         }

         sprintf(szSystem32Path, "%s\\system32\\%s", szSystemRoot, szFileName);
         if (FileExists(szSystem32Path))
         {
            setupLogMessage("Info", "admin", "removing bin\\https\\bin\\%s (use system32)", szFileName);
            sprintf(szSrcFile, "%s\\bin\\https\\bin\\%s", TARGETDIR, szFileName);
            DeleteFile(szSrcFile);
         }
      } 
      while (FindNextFile(hFile, lpFindFileData));
      FindClose(hFile);
   }
}

BOOL __declspec(dllexport) ADM_PostInstall(void)
{
   BOOL bSuccess = TRUE;
   char szPath[MAX_PATH];
   char szTemp[MAX_PATH];
   char szName[MAX_PATH];
  //set the server address
   char szFileSpec[MAX_PATH];
   WIN32_FIND_DATA FindFileData;
   LPWIN32_FIND_DATA lpFindFileData = &FindFileData;
   HANDLE hFile;


   strcpy(szPath, TARGETDIR);
   strcat(szPath, "\\java\\");

   // Remove jar files from previous installation
   wsprintf(szFileSpec, "%s*.jar", szPath);
   hFile = FindFirstFile(szFileSpec, lpFindFileData);
   if (hFile != INVALID_HANDLE_VALUE)
   {
      char szSrcFile[MAX_PATH];
      char *szFileName;
      do
      {
         szFileName = lpFindFileData->cFileName;
         if (!isNewInstFile(szFileName))
         {
            setupLogMessage("Info", "admin", "removing file %s", szFileName);
            strcpy(szSrcFile, szPath);
            strcat(szSrcFile, szFileName);
            DeleteFile(szSrcFile);
         }
      } 
      while (FindNextFile(hFile, lpFindFileData));
      FindClose(hFile);
   }

   // Remove zip files from previous installation
   wsprintf(szFileSpec, "%s*.zip", szPath);
   hFile = FindFirstFile(szFileSpec, lpFindFileData);
   if (hFile != INVALID_HANDLE_VALUE)
   {
      char szSrcFile[MAX_PATH];
      char *szFileName;
      do
      {
         szFileName = lpFindFileData->cFileName;
         if (!isNewInstFile(szFileName))
         {
            setupLogMessage("Info", "admin", "removing file %s", szFileName);
            strcpy(szSrcFile, szPath);
            strcat(szSrcFile, szFileName);
            DeleteFile(szSrcFile);
         }
      } 
      while (FindNextFile(hFile, lpFindFileData));
      FindClose(hFile);
   }
   
   if (cd.nReplaceVersion == 70) /* The current version;
                                    target of brandAdminServer */
   {
      LoadString(mi.m_hModule, IDS_README, szTemp, sizeof(szTemp));
      strcpy(szName, "Fedora Server Products 7.0"); /* current */
      strcat(szName, " ");
      strcat(szName, szTemp);
      RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
      LoadString(mi.m_hModule, IDS_START_ADMINSERVER, szName, sizeof(szName));
      strcat(szName, " ");
      strcat(szName, "7.0"); /* current */
      strcpy(szPath, "\"");
      strcpy(szPath, TARGETDIR);
      strcat(szPath, "\\start-admin.cmd\"");
      RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
      if (IsProgramFolderEmpty(PROGRAM_GROUP_NAME))
      {
         DeleteProgramFolder(PROGRAM_GROUP_NAME);
      }
   }
   else if (cd.nReplaceVersion == 1)
   {
      LoadString(mi.m_hModule, IDS_README, szTemp, sizeof(szTemp));
      strcpy(szName, "Netscape Server Family 4.0");
      strcat(szName, " ");
      strcat(szName, szTemp);
      RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
      LoadString(mi.m_hModule, IDS_START_ADMINSERVER, szName, sizeof(szName));
      strcat(szName, " ");
      strcat(szName, "4.0");
      strcpy(szPath, "\"");
      strcpy(szPath, TARGETDIR);
      strcat(szPath, "\\start-admin.cmd\"");
      RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
      if (IsProgramFolderEmpty(PROGRAM_GROUP_NAME))
      {
         DeleteProgramFolder(PROGRAM_GROUP_NAME);
      }
   }
   else if (cd.nReplaceVersion == 2)
   {
      LoadString(mi.m_hModule, IDS_README, szTemp, sizeof(szTemp));
      strcpy(szName, "Netscape Server Products 4.1");
      strcat(szName, " ");
      strcat(szName, szTemp);
      RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
      LoadString(mi.m_hModule, IDS_START_ADMINSERVER, szName, sizeof(szName));
      strcat(szName, " ");
      strcat(szName, "4.1");
      strcpy(szPath, "\"");
      strcpy(szPath, TARGETDIR);
      strcat(szPath, "\\start-admin.cmd\"");
      RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
      if (IsProgramFolderEmpty(PROGRAM_GROUP_NAME))
      {
         DeleteProgramFolder(PROGRAM_GROUP_NAME);
      }
   }
   else if (cd.nReplaceVersion == 3)
   {
      LoadString(mi.m_hModule, IDS_README, szTemp, sizeof(szTemp));
      strcpy(szName, "Netscape Server Products 4.2");
      strcat(szName, " ");
      strcat(szName, szTemp);
      RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
      LoadString(mi.m_hModule, IDS_START_ADMINSERVER, szName, sizeof(szName));
      strcat(szName, " ");
      strcat(szName, "4.2");
      strcpy(szPath, "\"");
      strcpy(szPath, TARGETDIR);
      strcat(szPath, "\\start-admin.cmd\"");
      RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
      if (IsProgramFolderEmpty(PROGRAM_GROUP_NAME))
      {
         DeleteProgramFolder(PROGRAM_GROUP_NAME);
      }
   }
   else if (cd.nReplaceVersion == 4)
   {
      LoadString(mi.m_hModule, IDS_README, szTemp, sizeof(szTemp));
      strcpy(szName, "Netscape Server Products 6.0");
      strcat(szName, " ");
      strcat(szName, szTemp);
      RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
      LoadString(mi.m_hModule, IDS_START_ADMINSERVER, szName, sizeof(szName));
      strcat(szName, " ");
      strcat(szName, "6.0");
      strcpy(szPath, "\"");
      strcpy(szPath, TARGETDIR);
      strcat(szPath, "\\start-admin.cmd\"");
      RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
      if (IsProgramFolderEmpty(PROGRAM_GROUP_NAME))
      {
         DeleteProgramFolder(PROGRAM_GROUP_NAME);
      }
   }
   else if (cd.nReplaceVersion == 5)
   {
      LoadString(mi.m_hModule, IDS_README, szTemp, sizeof(szTemp));
      strcpy(szName, "Netscape Server Products 6.01");
      strcat(szName, " ");
      strcat(szName, szTemp);
      RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
      LoadString(mi.m_hModule, IDS_START_ADMINSERVER, szName, sizeof(szName));
      strcat(szName, " ");
      strcat(szName, "6.01");
      strcpy(szPath, "\"");
      strcpy(szPath, TARGETDIR);
      strcat(szPath, "\\start-admin.cmd\"");
      RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
      if (IsProgramFolderEmpty(PROGRAM_GROUP_NAME))
      {
         DeleteProgramFolder(PROGRAM_GROUP_NAME);
      }
   }
   else if (cd.nReplaceVersion == 6)
   {
      LoadString(mi.m_hModule, IDS_README, szTemp, sizeof(szTemp));
      strcpy(szName, "Netscape Server Products 6.1");
      strcat(szName, " ");
      strcat(szName, szTemp);
      RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
      LoadString(mi.m_hModule, IDS_START_ADMINSERVER, szName, sizeof(szName));
      strcat(szName, " ");
      strcat(szName, "6.1");
      strcpy(szPath, "\"");
      strcpy(szPath, TARGETDIR);
      strcat(szPath, "\\start-admin.cmd\"");
      RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
      if (IsProgramFolderEmpty(PROGRAM_GROUP_NAME))
      {
         DeleteProgramFolder(PROGRAM_GROUP_NAME);
      }
   }

   // Cleanup specific to upgrade from AS6.0 or AS6.01
   if (cd.nReplaceVersion == 4 || cd.nReplaceVersion == 5)
   {
       // Bug 622995: Undid the removing of core component dlls from bin\https\bin;
       // we no longer put dependencies in system32
       // removeESShadowDLLs();

       // Remove java\patch dir. Not used any more
       sprintf(szName, "%s\\java\\patch", TARGETDIR);
       setupLogMessage("Info", "admin", "removing %s", szName);
       DeleteRecursively(szName);

       // Remove admin-serv\config\res dir. Not used
       sprintf(szName, "%s\\admin-serv\\config\\res", TARGETDIR);
       setupLogMessage("Info", "admin", "removing %s", szName);
       DeleteRecursively(szName);

       // Starting with AS6.1 secmod.db is in the alias dir.
       sprintf(szName, "%s\\admin-serv\\config\\nssckbi.dll", TARGETDIR);
       setupLogMessage("Info", "admin", "removing %s", szName);
       DeleteFile(szName);
       sprintf(szName, "%s\\admin-serv\\config\\secmod.db", TARGETDIR);
       sprintf(szTemp, "%s\\alias\\secmod.db", TARGETDIR);
       setupLogMessage("Info", "admin", "move %s to alias dir", szName);
       MoveFile(szName, szTemp);
   }

   CreateProgramFolder(PROGRAMFOLDER);
   
   strcpy(szPath, "notepad.exe \"");
   strcat(szPath, TARGETDIR);
   strcat(szPath, "\\README.txt\"");
   LoadString(mi.m_hModule, IDS_README, szTemp, sizeof(szTemp));
   strcpy(szName, PROGRAMNAME);
   strcat(szName, " ");
   strcat(szName, PROGRAMVERSION);
   strcat(szName, " ");
   strcat(szName, szTemp);
   AddProgramFolderItem(PROGRAMFOLDER, szName, szPath);

   if (cd.bShowAdmConfig)
   {
      LogData(NULL, "create admin config");
      if (!_CreateConfig(TARGETDIR))
      {
         NsSetupMessageBox(NULL, "Unable to create Administration Server configuration", NULL, MB_OK); 
         bSuccess = FALSE;
      }
   }
   else
   {
      LogData(NULL, "admin re-installing over admin");
      if (!_UpdateConfig(TARGETDIR))
      {
         NsSetupMessageBox(NULL, "Error occured while updating Administration Server configuration", NULL, MB_OK); 
         bSuccess = FALSE;
      }
   }
   
   // Fix ACIs under o=netscapeRoot
   /*if (!_FixRootACIs()) {
      NsSetupMessageBox(NULL, "Unable to set ACI in Confguration Directory Server", NULL, MB_OK);
      bSuccess = FALSE;
   }*/

   if (bSuccess)
   {
      char szServiceId[MAX_PATH];
      BOOL bIsRunning = FALSE;
      INT iLimit = 0;
      INT iAttemps = 0;

      sprintf(szServiceId, "%s-serv", ADM_ID_SERVICE);

      do
      {
         LogData(NULL, "Trying to start Admin Server");
         ControlServer(szServiceId, TRUE);
         iLimit = 0;
         do
         {
            bIsRunning = isServiceRunning(szServiceId);
            if (!bIsRunning)
            {
               iLimit++;
               Sleep(100);
               if (iLimit == 100)
               {
                  LogData(NULL, "Admin Server did not start");
                  break;
               }
            }
         }while (bIsRunning == FALSE);

         if (bIsRunning == FALSE)
         {
            iAttemps++;
         }

      } while ((bIsRunning == FALSE) && (iAttemps < 2));


      if (bIsRunning)
         LogData(NULL, "Admin Server started");
      else
         LogData(NULL, "Admin Server did not start");
   }
   return bSuccess;
}


BOOL WINAPI DllMain(HANDLE hModule, ULONG ulReasonForCall, LPVOID lpReserved)
{
   switch (ulReasonForCall)
   {
   case DLL_PROCESS_ATTACH:
      ZeroMemory(&mi, sizeof(MODULEINFO));
      mi.m_hModule = hModule;
      _InitializeControlData();
      break;
   case DLL_PROCESS_DETACH:
      break;
   case DLL_THREAD_ATTACH:
      break;
   case DLL_THREAD_DETACH:
      break;
   }
   return TRUE;
}


BOOL __declspec(dllexport) ADM_PreUnInstall(LPCSTR pszServerRoot)
{
   char szServerRoot[MAX_PATH];
   LogData(NULL, "stop admin Service");
   lstrcpy(szServerRoot, pszServerRoot);
   ControlServer(ADM_ID_SERVICE "-serv", FALSE);
   LogData(NULL, "remove admin sie");
   _RemoveSIE(szServerRoot);
   return TRUE;
}

static BOOL _RemoveMiscFiles(char *szServerRoot)
{
   BOOL bReturn = FALSE;
   char szFileName[MAX_PATH];

   sprintf(szFileName, "%s\\adminacl", szServerRoot);
   DeleteRecursively(szFileName);

   sprintf(szFileName, "%s\\admin-serv", szServerRoot);
   DeleteRecursively(szFileName);

   sprintf(szFileName, "%s\\shared\\config\\dbswitch.conf", szServerRoot);
   DeleteFile(szFileName);

   sprintf(szFileName, "%s\\admin.txt", szServerRoot);
   DeleteFile(szFileName);

   sprintf(szFileName, "%s\\start-admin.cmd", szServerRoot);
   DeleteFile(szFileName);

   sprintf(szFileName, "%s\\stop-admin.cmd", szServerRoot);
   DeleteFile(szFileName);

   sprintf(szFileName, "%s\\restart-admin.cmd", szServerRoot);
   DeleteFile(szFileName);

   sprintf(szFileName, "%s\\bin\\admin", szServerRoot);
   DeleteRecursively(szFileName);

   sprintf(szFileName, "%s\\shared\\config\\ds.conf", szServerRoot);
   DeleteFile(szFileName);


   sprintf(szFileName, "%s\\bin", szServerRoot);
   if (IsDirEmpty(szFileName))
      DeleteRecursively(szFileName);

   sprintf(szFileName, "%s\\manual", szServerRoot);
   if (IsDirEmpty(szFileName))
      DeleteRecursively(szFileName);

   sprintf(szFileName, "%s\\shared", szServerRoot);
   if (IsDirEmpty(szFileName))
      DeleteRecursively(szFileName);

   sprintf(szFileName, "%s\\httpacl", szServerRoot);
   if (IsDirEmpty(szFileName))
      DeleteRecursively(szFileName);

   return (bReturn);
}


BOOL __declspec(dllexport) ADM_PostUnInstall(LPCSTR pszServerRoot)
{
   char szServerRoot[MAX_PATH];
   char szFileSpec[MAX_PATH];
   char szPath[MAX_PATH];
   WIN32_FIND_DATA FindFileData;
   LPWIN32_FIND_DATA lpFindFileData = &FindFileData;
   HANDLE hFile;

   strcpy(szPath, pszServerRoot);
   strcat(szPath, "\\java\\jars\\");
   wsprintf(szFileSpec, "%s*.jar", szPath);
   hFile = FindFirstFile(szFileSpec, lpFindFileData);
   if (hFile != INVALID_HANDLE_VALUE)
   {
      char szSrcFile[MAX_PATH];
      char *szFileName;
      do
      {
         szFileName = lpFindFileData->cFileName;
         if (strstr(szFileName, "40") || strstr(szFileName, "41"))
         {

            strcpy(szSrcFile, szPath);
            strcat(szSrcFile, szFileName);
            if (setupFileExists(szSrcFile))
            {
               DeleteFile(szSrcFile);
            }
         }
      } 
      while (FindNextFile(hFile, lpFindFileData));
      FindClose(hFile);
   }

   lstrcpy(szServerRoot, pszServerRoot);
   _RemoveMiscFiles(szServerRoot);
   return TRUE;
}

BOOL __declspec(dllexport) CON_PreInstall(void)
{
  char szFile[MAX_PATH];

  cd.nReplaceVersion = 0;
  wsprintf(szFile, "%s\\setup\\admin\\admin.inf", TARGETDIR);
  if (setupFileExists(szFile))
  {
    char *szVer;
    szVer = setupGetInfString("admin-client", SETUP_INF_VERSION, NULL, szFile);
    if (szVer)
    {
       if (strcmp(szVer, "7.0") == 0) /* The current version;
                                         target of brandAdminServer */
       {
          cd.nReplaceVersion = 70; /* current */
       }
       else if (strcmp(szVer, "4.0") == 0)
       {
          cd.nReplaceVersion = 1;
       }
       else if (strcmp(szVer, "4.1") == 0)
       {
          cd.nReplaceVersion = 2;
       }
       else if (strcmp(szVer, "4.2") == 0)
       {
          cd.nReplaceVersion = 3;
       }
       else if (strcmp(szVer, "6.0") == 0)
       {
          cd.nReplaceVersion = 4;
       }
       else if (strcmp(szVer, "6.01") == 0)
       {
          cd.nReplaceVersion = 5;
       }
       else if (strcmp(szVer, "6.1") == 0)
       {
          cd.nReplaceVersion = 6;
       }
    }
  }

  return TRUE;
}

BOOL __declspec(dllexport) CON_PostInstall(void)
{
   BOOL bSuccess = TRUE;
   char szPath[MAX_PATH];
   char szName[MAX_PATH];
   char szTemp[MAX_PATH];

  //
  // Create the program manager group icons
  //

   if (cd.nReplaceVersion == 70) /* The current version;
                                    target of brandAdminServer */
   {
      //delete 7.0 readme (current)
      LoadString(mi.m_hModule, IDS_README, szTemp, sizeof(szTemp));
      strcpy(szName, "Fedora Server Products 7.0"); /* current */
      strcat(szName, " ");
      strcat(szName, szTemp);
      RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
      //delete 7.0 console icon (current)
      strcpy(szPath, "\"");
      strcat(szPath, TARGETDIR);
      strcat(szPath, "\\startconsole.exe\"");
      LoadString(mi.m_hModule, IDS_NETSCAPE_CONSOLE, szName, sizeof(szName));
      strcat(szName, "7.0"); /* current */
      RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
      if (IsProgramFolderEmpty(PROGRAM_GROUP_NAME))
      {
         DeleteProgramFolder(PROGRAM_GROUP_NAME);
      }
   }
   else if (cd.nReplaceVersion == 1)
   {
      
      //delete 4.0 readme
      LoadString(mi.m_hModule, IDS_README, szTemp, sizeof(szTemp));
      strcpy(szName, "Netscape Server Family 4.0");
      strcat(szName, " ");
      strcat(szName, szTemp);
      RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
      //delete 4.0 console icon
      strcpy(szPath, "\"");
      strcat(szPath, TARGETDIR);
      strcat(szPath, "\\startconsole.exe\"");
      LoadString(mi.m_hModule, IDS_NETSCAPE_CONSOLE, szName, sizeof(szName));
      strcat(szName, " 4.0");
      RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
      if (IsProgramFolderEmpty(PROGRAM_GROUP_NAME))
      {
         DeleteProgramFolder(PROGRAM_GROUP_NAME);
      }
   }
   else if (cd.nReplaceVersion == 2)
   {
      
      //delete 4.1 readme
      LoadString(mi.m_hModule, IDS_README, szTemp, sizeof(szTemp));
      strcpy(szName, "Netscape Server Products 4.1");
      strcat(szName, " ");
      strcat(szName, szTemp);
      RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
      //delete 4.1 console icon
      strcpy(szPath, "\"");
      strcat(szPath, TARGETDIR);
      strcat(szPath, "\\startconsole.exe\"");
      LoadString(mi.m_hModule, IDS_NETSCAPE_CONSOLE, szName, sizeof(szName));
      strcat(szName, " 4.1");
      RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
      if (IsProgramFolderEmpty(PROGRAM_GROUP_NAME))
      {
         DeleteProgramFolder(PROGRAM_GROUP_NAME);
      }
   }
   else if (cd.nReplaceVersion == 3)
   {
      
      //delete 4.2 readme
      LoadString(mi.m_hModule, IDS_README, szTemp, sizeof(szTemp));
      strcpy(szName, "Netscape Server Products 4.2");
      strcat(szName, " ");
      strcat(szName, szTemp);
      RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
      //delete 4.2 console icon
      strcpy(szPath, "\"");
      strcat(szPath, TARGETDIR);
      strcat(szPath, "\\startconsole.exe\"");
      LoadString(mi.m_hModule, IDS_NETSCAPE_CONSOLE, szName, sizeof(szName));
      strcat(szName, " 4.2");
      RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
      if (IsProgramFolderEmpty(PROGRAM_GROUP_NAME))
      {
         DeleteProgramFolder(PROGRAM_GROUP_NAME);
      }
   }
   else if (cd.nReplaceVersion == 4)
   {
      
      //delete 6.0 readme
      LoadString(mi.m_hModule, IDS_README, szTemp, sizeof(szTemp));
      strcpy(szName, "Netscape Server Products 6.0");
      strcat(szName, " ");
      strcat(szName, szTemp);
      RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
      //delete 6.0 console icon
      strcpy(szPath, "\"");
      strcat(szPath, TARGETDIR);
      strcat(szPath, "\\startconsole.exe\"");
      LoadString(mi.m_hModule, IDS_NETSCAPE_CONSOLE, szName, sizeof(szName));
      strcat(szName, " 6.0");
      RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
      if (IsProgramFolderEmpty(PROGRAM_GROUP_NAME))
      {
         DeleteProgramFolder(PROGRAM_GROUP_NAME);
      }
   }
   else if (cd.nReplaceVersion == 5)
   {
      
      //delete 6.01 readme
      LoadString(mi.m_hModule, IDS_README, szTemp, sizeof(szTemp));
      strcpy(szName, "Netscape Server Products 6.01");
      strcat(szName, " ");
      strcat(szName, szTemp);
      RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
      //delete 6.01 console icon
      strcpy(szPath, "\"");
      strcat(szPath, TARGETDIR);
      strcat(szPath, "\\startconsole.exe\"");
      LoadString(mi.m_hModule, IDS_NETSCAPE_CONSOLE, szName, sizeof(szName));
      strcat(szName, " 6.01");
      RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
      if (IsProgramFolderEmpty(PROGRAM_GROUP_NAME))
      {
         DeleteProgramFolder(PROGRAM_GROUP_NAME);
      }
   }
   else if (cd.nReplaceVersion == 6)
   {
      
      //delete 6.1 readme
      LoadString(mi.m_hModule, IDS_README, szTemp, sizeof(szTemp));
      strcpy(szName, "Netscape Server Products 6.1");
      strcat(szName, " ");
      strcat(szName, szTemp);
      RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
      //delete 6.1 console icon
      strcpy(szPath, "\"");
      strcat(szPath, TARGETDIR);
      strcat(szPath, "\\startconsole.exe\"");
      LoadString(mi.m_hModule, IDS_NETSCAPE_CONSOLE, szName, sizeof(szName));
      strcat(szName, " 6.1");
      RemoveProgramFolderItem(PROGRAM_GROUP_NAME, szName);
      if (IsProgramFolderEmpty(PROGRAM_GROUP_NAME))
      {
         DeleteProgramFolder(PROGRAM_GROUP_NAME);
      }
   }

   CreateProgramFolder(PROGRAMFOLDER);
   //create readme
   strcpy(szPath, "notepad.exe \"");
   strcat(szPath, TARGETDIR);
   strcat(szPath, "\\README.txt\"");
   LoadString(mi.m_hModule, IDS_README, szTemp, sizeof(szTemp));
   strcpy(szName, PROGRAMNAME);
   strcat(szName, " ");
   strcat(szName, PROGRAMVERSION);
   strcat(szName, " ");
   strcat(szName, szTemp);
   AddProgramFolderItem(PROGRAMFOLDER, szName, szPath);
   //create console icon
   LoadString(mi.m_hModule, IDS_STARTCONSOLEEXE, szTemp, sizeof(szTemp));
   strcpy(szPath, "\"");
   wsprintf(szName, szTemp, TARGETDIR);
   strcat(szPath, szName);
   strcat(szPath, "\"");
   LoadString(mi.m_hModule, IDS_NETSCAPE_CONSOLE, szName, sizeof(szName));
   strcat(szName, " ");
   strcat(szName, PROGRAMVERSION);
   AddProgramFolderItem(PROGRAMFOLDER, szName, szPath);

   return TRUE;
}

BOOL __declspec(dllexport) CON_PreUnInstall(LPCSTR szServerRoot)
{
   char szPath[MAX_PATH];
  //
  // Create the program manager group icons
  //
   strcpy(szPath, szServerRoot);
   strcat(szPath, "\\startconsole.exe");
   if (FileExists(szPath))
   {
      if (!DeleteFile(szPath))
      {
         char szErrMsg[MAX_PATH * 2];

         LoadString(mi.m_hModule, IDS_ERR_STARTCONSOLE, szErrMsg, sizeof(szErrMsg));
         NsSetupMessageBox(NULL, szErrMsg, NULL, MB_OK|MB_ICONSTOP);
      }
   }
   return TRUE;
}

BOOL __declspec(dllexport) CON_PostUnInstall(LPCSTR szServerRoot)
{
   char szFileName[MAX_PATH];
   char *szHomeDir;

   szHomeDir = getenv("USERPROFILE");
   if(szHomeDir != NULL)
   {
      sprintf(szFileName, "%s\\.mcc", szHomeDir);
      DeleteRecursively(szFileName);
   }

   sprintf(szFileName, "%s\\java\\jars", szServerRoot);
   if (IsDirEmpty(szFileName))
      DeleteRecursively(szFileName);

   sprintf(szFileName, "%s\\java", szServerRoot);
   if (IsDirEmpty(szFileName))
      DeleteRecursively(szFileName);

   return TRUE;
}

