/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#include <stdio.h>
#include <string.h>
#if defined(AIX)
   #include <strings.h>
#endif
#include <unistd.h>
#include "nsdefs.h"
#include "uninstall.h"


// Global variables
InstallInfo *installInfo = NULL;
Bool reConfig = False;
int  installMode;
char *logFile = NULL;
char *infoFile = NULL;
char docroot[256];


// Check if a file is a jar from this installation
// base.jar ldapjdk.jar jss*.jar mcc* nmclf*
//
static int isNewInstFile(char *f) {
    if (strcmp(f, "base.jar") == 0) {
        return 1;
    }
    else if (strcmp(f, "ldapjdk.jar") == 0) { 
         return 1;
    }
    else if (strcmp(f, "jss3.jar") == 0) { 
         return 1;
    }
    else if (strstr(f, "mcc70") == f) { 
         return 1;
    }
    else if (strstr(f, "nmclf70") == f) { 
         return 1;
    }
    return 0;
}


void configJava()
{
   DIR *dirp;
   struct dirent *dp;
   struct stat fi;
   NSString serverRoot;
   NSString javaDir;
   NSString jarsDir;
   NSString patchDir;
   NSString sourceFile;
   NSString destFile;

   serverRoot = installInfo->get("ServerRoot");

   javaDir = serverRoot + "/java";

   dirp = opendir(javaDir);

   // Remove all .jar and .zip files which are not 50
   while ((dp = readdir(dirp)) != NULL)
   {
      if (strstr(dp->d_name, ".zip") || strstr(dp->d_name, ".jar"))
      {
         if (!isNewInstFile(dp->d_name)) {
            sourceFile = javaDir + "/";
            sourceFile = sourceFile + dp->d_name;
            unlink(sourceFile.data());
            //printf("unlink %s\n",sourceFile.data());
         }
      }
   }

   closedir(dirp);

   // Remove java/patch dir. Not packaged starting with Console 6.1
   patchDir = javaDir + "/patch";
   if (stat(patchDir.data(), &fi) == 0) {
       char cmd[BIG_BUF];
       sprintf(cmd, "rm -fr %s", patchDir.data());
       system(cmd);
   }
}

int
main(int argc, char **argv)
{
   const char *sroot;
   int opt;

   while ((opt = getopt(argc, argv, "m:sl:f:r")) != -1)
   {
      switch (opt)
      {
      case 'r':
         reConfig = True;
         break;
      case 's':
         installMode = Silent;
         break;
      case 'l':
         logFile = strdup(optarg); /* Log file to use */
         break;
      case 'f':
         infoFile = strdup(optarg);   /* Install script */
         installInfo = new InstallInfo(infoFile);
         break;
      default:
         break;
      }
   }

   if (installInfo)
      configJava();

   return 0;
}


