/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*********************************************************************
**
** NAME:
**   startconsole.cc
**
** DESCRIPTION:
**   Administration Server Configuration Program
**
** NOTES:
** This program is intended to be UNIX-only, and not thread safe.
**
*********************************************************************/

extern "C" {
#include <stdio.h>		/* printf, file I/O */
#include <string.h>		/* strlen */
#include <strings.h>
#include <unistd.h>
#include <stdlib.h>		/* memset, rand stuff */
#include <sys/types.h>
#include <errno.h>
#include <dirent.h>
}

#include "nsdefs.h"
#include "utf8.h"
#include "ux-util.h"

const char *buildClassPath(const NSString javaDir);
void       displayHelp();

const char *JAVA_DIR = "java";
const char *CONSOLE_CMD = "com.netscape.management.client.console.Console";

const char *initStack = " -ms8m -mx64m ";

int main(int argc, char **argv)
{
   NVPair admConf;
   NSString serverRoot;
   NSString adminHost;
   NSString ld_library_path;
/*for AIX it need both LD_LIBRARY_PATH and LIBPATH set...if not JSS won't work*/
#ifdef AIX
   NSString libpath;
#endif
   NSString java_library_path;
   const char *adminPort;
   const char *adminSecurity;
   char  *s;
   char  tmp[BIG_BUF];
   char  *adminURL = NULL;
   char  *localAdminURL = NULL;
   char  *language = NULL;
   char  *sieDN = NULL;
   char  *ldapHost = NULL;
   char  *ldapPort = NULL;
   char  *ldapPwd = NULL;
   char  *ldapUser = NULL;
   char  *ldapBaseDN = NULL;
   char  *debugOption = NULL;
   char  *outputFile = NULL;
   char  *usergroupDN = NULL;



   /* 335197 workaround for the start browser problem */
   char  *hpuxOptions = (char*)"-lazyswap";

   NSString  extras;
   int   opt;
   NSString cmd;
   char jitOpt[256] = "";

#if defined(AIX)
   const char *relJavaCmd = "sh/java";
#else
   const char *relJavaCmd = "bin/java";
#endif

const char *nojitOpt   = "-Djava.compiler=NONE";

   char *javaHome=NULL;
   NSString absJavaCmd;

   if (readlink(argv[0], tmp, BIG_BUF) == -1)
   {
      // Not a symbolic link
      s = argv[0];
   }
   else
   {
      s = tmp;
   }

   serverRoot = InstUtil::dirName(s);
   chdir(serverRoot.data());
   serverRoot = InstUtil::getCurrentDir();

   /*set up ld library path so jss can correctly pick up nspr lib*/
   {
#ifdef HPUX
     ld_library_path = "SHLIB_PATH=";
     ld_library_path = ld_library_path + getenv("SHLIB_PATH");
#else
     ld_library_path = "LD_LIBRARY_PATH=";
     ld_library_path = ld_library_path + getenv("LD_LIBRARY_PATH");
#endif

     ld_library_path = ld_library_path + ":";
     ld_library_path = ld_library_path + serverRoot.data();
     ld_library_path = ld_library_path + "/lib";
/* 622185: jss' deps now live in NSROOT/bin/admin/lib */
     ld_library_path = ld_library_path + ":";
     ld_library_path = ld_library_path + serverRoot.data();
     ld_library_path = ld_library_path + "/bin/admin/lib";

/*
   JSS native code has dependency on JVM shared libs; On Linix with 
   JRE1.4 we need to have them in the LD_LIBRARY_PATH as well.
*/
#ifdef LINUX
     ld_library_path = ld_library_path + ":";
     ld_library_path = ld_library_path + serverRoot.data();
     ld_library_path = ld_library_path + "/bin/base/jre/lib/i386/client";
     ld_library_path = ld_library_path + ":";
     ld_library_path = ld_library_path + serverRoot.data();
     ld_library_path = ld_library_path + "/bin/base/jre/lib/i386/native_threads";
#endif

     putenv((char*)ld_library_path.data());
   }

#ifdef AIX
   {
     libpath = "LIBPATH=";
     libpath = libpath + getenv("LIBPATH");
     libpath = libpath + ":";
     libpath = libpath + serverRoot.data();
     libpath = libpath + "/lib";
     putenv((char*)libpath.data());

   }
#endif


   while ((opt = getopt(argc, argv, "ha:n:l:f:Ds:o:u:w:d:p:b:gx:Z")) != -1)
   {
      switch(opt)
      {
         case 'h':
            displayHelp();
            break;
         case 'a':
            adminURL = strdup(optarg);
            break;
         case 'n':
            if (!strcmp(optarg, "ojit")) {
                sprintf(jitOpt, " %s ", nojitOpt);
            }
            else {
                printf("startconsole: illegal option -- n%s\n", optarg);
            }
            break;
         case 'l':
            language = strdup(optarg);
            break;
         case 'D':
            debugOption = (char*)"";
            if (optind < argc && *argv[optind] != '-')
            {
               debugOption = strdup(argv[optind]);
               optind ++;
            }

            break;
         case 'f':
            outputFile = strdup(optarg);
            break;
         case 'x':
            extras = extras + " -x " + optarg;
            break;
         /*
          * Back door stuffs 
          */
         case 's':
            sieDN = localToUTF8(optarg);
            break;
         case 'u':
            ldapUser = strdup(optarg);
            break;
         case 'w':
            ldapPwd = strdup(optarg);
            break;
         case 'd':
            ldapHost = strdup(optarg);
            break;
         case 'o':
            usergroupDN = localToUTF8(optarg);
            break;
         case 'p':
            ldapPort = strdup(optarg);
            break;
         case 'b':
            ldapBaseDN = localToUTF8(optarg);
            break;
      } 
   }

   if (adminURL == NULL && ldapHost == NULL)
   {
      sprintf(tmp, "%s/admin-serv/config/adm.conf", serverRoot.data());

      admConf.setFormat(2);
      admConf.read(tmp);

      if (admConf.isEmpty() == False)
      {
         adminPort = admConf.get("Port");
         adminSecurity = admConf.get("Security");
         adminHost = InstUtil::guessHostname();

         if (adminPort != NULL && adminHost != (char *)NULL)
         {
            char *httpPrefix;

            if (adminSecurity == NULL || !strcasecmp(adminSecurity, "Off"))
            {
               httpPrefix = (char*)"http";
            }
            else
            {
               httpPrefix = (char*)"https";
            }
            sprintf(tmp, "%s://%s:%s", httpPrefix, adminHost.data(), adminPort);
            localAdminURL = strdup(tmp);
         }

      }
   }

   /* 
    * Check whether the JRE is installed
    */
   absJavaCmd = serverRoot +  "/bin/base/jre/" + relJavaCmd;

   if (!setupFileExists( (const char *)absJavaCmd ))
   {
      /*
       * Check whether the user has installed the JRE
       */
      javaHome = getenv("JAVA_HOME");
      if (javaHome == NULL)
      {
         /*
          * The user has not set the JavaHome environment
          */
         printf("ERROR: JAVA_HOME environment is not set. You need to\n");
         printf("       set JAVA_HOME to the directory where the Java\n");
         printf("       Run Time (JRE) is installed\n");
         exit(1);
     }
     else
     {
         absJavaCmd = NSString(javaHome) + "/" + relJavaCmd;
         if (!setupFileExists( (const char *)absJavaCmd )) {
             absJavaCmd = NSString(javaHome) + "/jre/" + relJavaCmd;
             if (!setupFileExists( (const char *)absJavaCmd )) {
                 printf("ERROR: JAVA_HOME environment is not set correcly. You need to\n");
                 printf("       set JAVA_HOME to the directory where the Java\n");
                 printf("       Run Time (JRE) is installed\n");
                 exit(1);
              }
          }

         printf("INFO: using user's Java Run Time Environment: %s\n", absJavaCmd.data());
         printf("      Make sure you have installed the required JRE version\n");
         printf("      otherwise the Console might not function correctly.\n");
     }
   }

   const char *classPath = buildClassPath(serverRoot + "/" + JAVA_DIR);

   cmd = absJavaCmd + jitOpt + initStack  + " -cp " + classPath;

   /*setup java.library.path for JSS*/
   {
#ifdef OSF1
/*
  For osf we need to add -taso option, jss is compile with taso option.
  taso option is a flag that is used to compiled lib/bin into a 32bit format.
  A taso build lib is not compatible (can't mix them) with a none taso build lib.
  see bug#396612
*/

     cmd = cmd + " -taso";
#endif
     cmd = cmd + " -Djava.library.path=";
     cmd = cmd + serverRoot.data();
     cmd = cmd + "/lib/jss ";
   }

   /*
    * 604379: Specify directory for JDK1.4 preference package. Otherwise,
    * /etc/.java and $HOME/.java are used for the system and user preferences,
    * and there is a JDK info message when the dirctories are created.
    */

   cmd = cmd + " -Djava.util.prefs.systemRoot=" + serverRoot + "/java/.java" +
         " -Djava.util.prefs.userRoot=" + serverRoot + "/java"; 

    

   cmd = cmd + " " + CONSOLE_CMD;

   if (debugOption)
   {
      cmd = cmd + " -D " + debugOption;
   }

   if (outputFile)
   {
      cmd = cmd + " -f " + outputFile;
   }

   if (extras)
   {
      cmd = cmd + extras;
   }

   if (localAdminURL)
   {
      cmd = cmd + " -A " + localAdminURL;
   }

   if (adminURL)
   {
      cmd = cmd + " -a " + adminURL;
   }
   else if (ldapHost != NULL)
   {
      if (ldapPort == NULL)
         ldapPort = (char*)"389";

      cmd = cmd + " -d " + ldapHost + " -p " + ldapPort;
      if (ldapBaseDN)
      {
         cmd = cmd + " -b " + ldapBaseDN;
      }

   }
   if (ldapUser)
   {
      cmd = cmd + " -u " + "'" +  ldapUser +  "'";
   }
   if (ldapPwd)
   {
      cmd = cmd + " -w " +  "'" + ldapPwd + "'";
   }

   if (language)
      cmd = cmd + " -l " + language;

   if (sieDN)
   {
      cmd = cmd + " -s " + "'" +  sieDN + "'";
   }

   if (usergroupDN)
   {
      cmd = cmd + " -o " + usergroupDN;
   }
   
   chdir(JAVA_DIR);
   if (debugOption)
   {
      cerr << cmd.data() << "\n";
   }
   return system(cmd.data());
}

const char *
buildClassPath(const NSString javaDir)
{
   static char classPath[BIG_BUF];
   struct dirent *entry = NULL;
   DIR *dir = opendir(javaDir.data());

   sprintf(classPath, "%s", ".");

   while ((entry = readdir(dir)) != NULL)
   {
      const char *s = strrchr(entry->d_name, '.');

      if (s != NULL && (!strcasecmp(s, ".jar") || !strcasecmp(s, ".zip")))
      {
         strcat(strcat(classPath, ":./"), entry->d_name);
      }
   }
   return classPath;
}

void
displayHelp()
{
   printf("Fedora Management Console Start Program Version 7.0.\n");
   printf("Usage: startconsole [-options]\n");
   printf("Options:\n");
   printf("   -h            print out this message\n");
#if (!defined(AIX))
#endif
   printf("   -a            Adminstration Server URL. If you are running\n");
   printf("                 the Fedora Management Console as a standalone application,\n");
   printf("                 enter the URL of the Administration server in the\n");
   printf("                 form:\n");
   printf("                   http(s)://<host>:<port>\n");
   printf("   -l            Language. e.g. en, ja, fr ...\n");
   printf("   -f <file>     Capture stderr and stdout to <file> (like Unix tee command)\n");
   printf("   -x <extras>   Extra options where <extras>: \n");
   printf("                   -x nologo : do not display logo\n");
   printf("                   -x nowinpos: disable window positioning\n");
   printf("                   -x nologo,nowinpos : sets both nologo and nowinpos options\n");
   exit(0);
}




