/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*********************************************************************
**
** NAME:
**   ux-remove.cc
**
** DESCRIPTION:
**   Administration Server Uninstall program
**
** NOTES:
** This program is intended to be UNIX-only, and not thread safe.
** Derived from the original ux-config.c with UI moved to ux-config.cc
**
**
*********************************************************************/

extern "C"
{
#include <stdio.h>		/* printf, file I/O */
#include <sys/stat.h>
#include <string.h>
#include <strings.h>
#include <signal.h>		/* kill() */
}

#include "global.h"
#include "ux-util.h"
#include "ldapu.h"

int main(int argc, char *argv[])
{
  NSString serverRoot;
  NSString instanceDir;
  NSString admConfFile;
  NVPair *admConf;
  InstallInfo *uninstallInfo;
  char *infoFile;
  const char *ldapURL;
  const char *sieDN;
  const char *isieDN;
  const char *siePwd;
  /*const char *adminPwd;*/
  int opt;
  int err;
  char temp[MED_BUF];
  struct stat fi;

  while ((opt = getopt(argc, argv, "f:")) != EOF)
  {
    switch (opt)
	  {
	    case 'f':
	      infoFile = strdup(optarg);
	      uninstallInfo = new InstallInfo(infoFile);
        uninstallInfo->toUTF8();
        break;
	    default:
	      break;
	  }
  }
  if (infoFile == NULL || uninstallInfo == NULL ||
      uninstallInfo->isEmpty() == True)
  {
     printf("[Admin] Answer cache not found\n");
     err = -1;
  }
  else
  {
     serverRoot = uninstallInfo->get(SERVER_ROOT);

     instanceDir = serverRoot + "/" + "admin-serv";
     sprintf(temp, "%s/admin-serv/config/adm.conf", serverRoot.data());
     admConf = new NVPair(temp);

     if (admConf->isEmpty() == False)
     {
	     /*
	      * PVO - should we get from adm.conf instead ? How about if
	      * there's a mismatch
	      */
	     ldapURL = uninstallInfo->get(CONFIG_LDAP_URL);
	     sieDN = admConf->get("sie");
       siePwd = admConf->get("siepid");
	     isieDN = admConf->get("isie");
      
	     if (ldapURL == NULL || sieDN == NULL || isieDN == NULL)
	     {
	       printf("[Admin] LDAP information not found\n");
	       err = -1;
	     }
	     else
	     {
	       LdapError ldapError;
	       Ldap ldap(ldapError, ldapURL, uninstallInfo->get(MC_ADMIN_ID),
		               uninstallInfo->get(MC_ADMIN_PWD),
		               sieDN, siePwd);

	       if (ldapError != OKAY)
	       {
           /*  remove the message which causes confusion when DS is down
	          sprintf(temp, "%ld", ldapError.errorCode());
	          printf("[Admin] Ldap authentication failed (%s:%s)\n", temp, ldapError.msg()),
           */
		       err = -1;
	       }
	       else
	       {
	         ldapError = removeSIE(&ldap, sieDN, True);

	         if (ldapError != OKAY)
	         {
           /*  remove the message which causes confusion when DS is down
		         sprintf(temp, "%ld", ldapError.errorCode());
		         printf("[Admin] Cannot remove Admin server instance (%s)\n", temp);
           */
	         }
	       }
	     }
     }
   }

   char pidFile[BIG_BUF];
   char stopProgram[BIG_BUF];
   int  pid;
   FILE *fhdl = NULL;

   /* Stop the Admin Server */
   sprintf(pidFile, "%s/admin-serv/logs/pid", serverRoot.data());
   if (stat(pidFile, &fi) == 0)
   {
      sprintf(stopProgram, "%s/stop-admin", serverRoot.data());
      if (stat (stopProgram, &fi) != 0)
      {
         sprintf(stopProgram, "kill `cat %s`", pid);
      }
      system(stopProgram);
   }

   /* Stop SNMP Master Agent if running */
   sprintf(pidFile, "%s/admin-serv/logs/pid_masteragt", serverRoot.data());
   if ((fhdl = fopen(pidFile, "r")) != NULL)
   {
      fscanf(fhdl, "%d\n", &pid);
      fclose(fhdl);
      kill(pid, SIGKILL);
      unlink(pidFile);
   }
   
   char szCmdBuf[BIG_BUF];
   char *szHomeDir;

   szHomeDir = getenv("home");
   if(szHomeDir != NULL)
   {
      sprintf(szCmdBuf, "rm -fr %s/.mcc", szHomeDir);
      system(szCmdBuf);
   }

   /* Remove dynamic and temporary files */
   sprintf(szCmdBuf, "rm -fr %s/admin-serv/tmp", serverRoot.data());
   system(szCmdBuf);
    
   sprintf(szCmdBuf, "rm -fr %s/admin-serv/ClassCache", serverRoot.data());
   system(szCmdBuf);
    
   sprintf(szCmdBuf, "rm -fr %s/admin-serv/SessionData", serverRoot.data());
   system(szCmdBuf);
    
   sprintf(szCmdBuf, "rm -fr %s/admin-serv/config/*.properties", serverRoot.data());
   system(szCmdBuf);

   sprintf(szCmdBuf, "rm -fr %s/java/jars/*.icon", serverRoot.data());
   system(szCmdBuf);

   sprintf(szCmdBuf, "rm -fr %s/java/.java", serverRoot.data());
   system(szCmdBuf);

   return 0;
}
