/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * adminit.c: Initialization routines for the admin server
 * 
 * Rob McCool
 *
 * $Log: adminit.c,v $
 * Revision 1.2  2005/03/02 23:31:33  nhosoi
 * [147535] Brand the Administration Server as "Fedora"
 * Brand name: BrandX -> Fedora; Updated copyrights.
 *
 * Revision 1.1.1.1  2005/02/01 23:14:50  cvsadm
 * Move Admin Server from branch to TRUNK
 *
 * Revision 1.1.1.5  2001/11/05 20:05:06  ahakim
 * XXX AOL/SBS/NSCP Copyright XXX
 *
 * Revision 1.1.1.4  1999/07/29 00:23:06  svbld
 * Merged the admin42_branch back onto the admin_branch..
 *
 * Revision 1.1.1.2.2.4  1999/07/21 00:44:47  shihcm
 * bug#358492, change tmp='/' to *tmp='/' to fix ns-admin crash
 *
 * Revision 1.1.1.2.2.3  1999/07/13 03:24:32  shihcm
 * fix solaris build break
 *
 * Revision 1.1.1.2.2.2  1999/07/13 02:04:35  shihcm
 * bug#356724, add help button to password dialog
 *
 * Revision 1.1.1.2.2.1  1999/06/15 05:03:42  adam
 * Include admsslutil.h for ADM_GetPassword().
 *
 * Revision 1.1.1.2  1999/06/10 16:29:24  thayes
 * Allow prompting for password.
 *
 * Revision 1.1.1.1  1999/03/03 23:21:54  svbld
 * importing admin41 closer to the trunk
 *
 * Revision 1.24.2.13.4.3.6.1.4.63.4.2  1999/01/15 19:16:34  miodrag
 * 324423 Do not exit if a log file can not be opened, just disable logging
 *
 * Revision 1.24.2.13.4.3.6.1.4.63.4.1  1998/09/10 02:44:20  yjh
 * Linux Port for Linux 2.1.108 kernel
 *
 * Revision 1.24.2.13.4.3.6.1.4.63  1998/08/04 02:08:48  yjh
 * Bug 311494,fix possible null dereference problem in sync_task_sie_data
 *
 * Revision 1.24.2.13.4.3.6.1.4.62  1998/07/30 21:31:40  adam
 * Bug fix #311011: global vars get STRDUP value, not strdup. Avoids warning when using FREE().
 *
 * Revision 1.24.2.13.4.3.6.1.4.61  1998/07/23 17:37:05  atulb
 * Bug#251344: Fixed enabling/disabling of DSGW implementation in the server
 *
 * Revision 1.24.2.13.4.3.6.1.4.60  1998/07/18 01:29:38  yjh
 * Bug301028 related Fix:
 *
 * This actually is a jumbo fix which fix the following problems which are found
 * while fixing Bug 301028:
 * (1) In case of Admin server start up when Configuration LDAP is down, the user/group won't be
 * setup even the Configuration LDAP was up later.
 * (2) In the same situation, the tasks under admin-serv was not repopulated even when Configuration
 * LDAP was up later.
 * (3) Move cache lifetime code to be with cache initialization.
 * (4) Fix the problem in LDAP rebind, when password is NULL which will crash the Admin server.
 *
 * Revision 1.24.2.13.4.3.6.1.4.59  1998/07/15 17:37:52  yjh
 * Bug 301028: Make sure certain cache are created even when task_evel_init failed
 * and send oot warning message instead of exit from admin server when
 * task_eval_init failed.
 *
 * Revision 1.24.2.13.4.3.6.1.4.58  1998/06/27 03:14:32  yjh
 * Bug 117376: Send password expiring information back to client
 * Adn up libnls30 to 19980604
 *
 * Revision 1.24.2.13.4.3.6.1.4.57  1998/06/19 01:57:23  yjh
 * Bug 119953: Use _PR_IRIX_CHILD_PROCESS to allow child daemon process continue
 * running NSPR functions without problem, also pick up NSPR patched version.
 *
 * Revision 1.24.2.13.4.3.6.1.4.56  1998/06/16 02:20:14  miodrag
 * Created change-sie-password runtime command
 *
 * Revision 1.24.2.13.4.3.6.1.4.55  1998/06/03 01:09:17  yjh
 * Bug 116841, redirect the error message from stderr to error log and close
 * stdin, stdout, stderr after daemon detached from calling process
 *
 * Revision 1.24.2.13.4.3.6.1.4.54  1998/05/13 17:18:53  yjh
 * Bug: 117360 (Add the path check which can handle NT's drive name in the path)
 *
 * Revision 1.24.2.13.4.3.6.1.4.53  1998/04/06 20:35:20  dt
 * Bug 112533. The AS will now attempt to resync with the DS when it sees a
 * server-id it does not know about.
 *
 * Revision 1.24.2.13.4.3.6.1.4.52  1998/03/25 00:09:19  dt
 * Changes to support configurable server bind address.
 *
 * Revision 1.24.2.13.4.3.6.1.4.51  1998/03/20 03:18:10  adam
 * Bug fix #108407: LDAP/SSL and core AS SSL initialized.
 *
 * Revision 1.24.2.13.4.3.6.1.4.50  1998/03/16 00:52:03  dt
 * Bug #109999. Support for admin server runtime command namespace, and
 * server resync command for task sie data.
 *
 * Revision 1.24.2.13.4.3.6.1.4.49  1998/03/10 21:35:46  yjh
 * Change PSET to support multi-valued attribute and convert the core server and
 * cgis to support this changes. (BUG# 108904)
 *
 * Revision 1.24.2.13.4.3.6.1.4.48  1998/03/10 17:38:03  dt
 * Changes to support cache tuning.
 *
 * Revision 1.24.2.13.4.3.6.1.4.47  1998/03/08 02:56:51  adam
 * Part of bug fix #108726.
 *
 * Revision 1.24.2.13.4.3.6.1.4.46  1998/03/07 04:40:48  pvo
 * Fixed typos made during schema changes.
 *
 * Revision 1.24.2.13.4.3.6.1.4.45  1998/03/06 23:55:51  yjh
 * Really refelect schema changes
 *
 * Revision 1.24.2.13.4.3.6.1.4.42  1998/03/06 21:42:07  shihcm
 * *** empty log message ***
 *
 * Revision 1.24.2.13.4.3.6.1.4.39  1998/02/24 03:20:58  yjh
 * Fixed Admin server task evaluation/initialization problem (not able to
 * communicate to LDAP in SSL mode)
 *
 * Revision 1.24.2.13.4.3.6.1.4.38  1998/02/13 02:21:49  yjh
 * Integarte PSET with LDAPSDK31/HCL support which allow PSET to communicate with
 * LDAP in SSL mode, plus the necessary changes in server and CGIs.
 *
 * Revision 1.24.2.13.4.3.6.1.4.37  1998/02/03 03:48:10  dt
 * Major changes to task dispatch code:
 *
 * 1) Task invocation now supported for task entries in ISIE subtree.
 * 2) Task lookup is now dynamic. No more static init at startup.
 *
 * Revision 1.24.2.13.4.3.6.1.4.36  1998/02/02 21:48:42  dt
 * Turned off object debugging output. Bug #105319.
 *
 * Revision 1.24.2.13.4.3.6.1.4.35  1998/01/27 02:50:28  dt
 * Removed last bits of 3.x acldir usage.
 *
 * Revision 1.24.2.13.4.3.6.1.4.34  1998/01/27 02:31:23  dt
 * Changes to support the new authentication order of evaluation, and a bug
 * fix for the host and ip checking code.
 *
 * Revision 1.24.2.13.4.3.6.1.4.33  1998/01/17 01:17:58  adam
 * Enable the user to turn on individual cipher families without having to turn on the internal token. This is done by checking sslactivation of each cipher family under configuration.encryption.
 *
 * Revision 1.24.2.13.4.3.6.1.4.32  1998/01/09 20:49:42  dt
 * Bug fix for task resolution problem.
 *
 * Revision 1.24.2.13.4.3.6.1.4.31  1998/01/08 23:04:04  adam
 * Change userpw to user_pw.
 *
 * Revision 1.24.2.13.4.3.6.1.4.30  1997/12/15 20:41:16  yjh
 * Add default to ssl2, ssl3 and sslckientAuth and fix certdb file name problem.
 *
 * Revision 1.24.2.13.4.3.6.1.4.29  1997/12/13 05:26:42  yjh
 * Add CGI error reporting and I18n support utility.
 *
 * Revision 1.24.2.13.4.3.6.1.4.28  1997/12/13 03:22:31  dt
 * Fix for authentication problems. Keep your fingers crossed...
 *
 * Revision 1.24.2.13.4.3.6.1.4.27  1997/12/05 01:19:16  dt
 * Changed disableDSGW name to dsgw, fixed build breakage for S_IXUSR,
 * S_IXGRP, and S_IXOTH posix macros on NT.
 *
 * Revision 1.24.2.13.4.3.6.1.4.26  1997/12/05 00:48:35  dt
 * Fixed dsgw content-type bug, and implemented dsgw disable feature via
 * a search for a configuration.disableDSGW attribute.
 *
 * Revision 1.24.2.13.4.3.6.1.4.25  1997/11/26 01:40:21  dt
 * Fix for server-id task resolution problem.
 *
 * Revision 1.24.2.13.4.3.6.1.4.24  1997/11/25 22:59:20  dt
 * Changes to support static arguments in the execRef attributes of tasks.
 *
 * Revision 1.24.2.13.4.3.6.1.4.23  1997/11/20 20:06:29  yjh
 * First HCL_1_01 Integration check in.
 *
 * Revision 1.24.2.13.4.3.6.1.4.13  1997/10/02 03:18:28  yjh
 * Fix literal string problem.
 *
 * Revision 1.24.2.13.4.3.6.1.4.12  1997/10/02 02:25:32  yjh
 * Another try to clear up admin config problem. :-(
 *
 * Revision 1.24.2.13.4.3.6.1.4.11  1997/10/02 02:09:06  yjh
 * Fix the core dump caused by when server root is not set.
 *
 * Revision 1.24.2.13.4.3.6.1.4.10  1997/10/02 00:35:50  yjh
 * Fix some configuration parameter missing probelm.
 *
 * Revision 1.24.2.13.4.3.6.1.4.9  1997/09/30 07:51:53  dt
 * Changes to support the 4.0 evaluation scheme. -DT 9/30/97
 *
 * Revision 1.24.2.13.4.3.6.1.4.8  1997/09/30 02:39:33  yjh
 * Add NSAPI_PUBLIC to PSET library
 *
 * Revision 1.24.2.13.4.3.6.1.4.7  1997/09/27 03:04:12  pvo
 * Changed NetsiteRoot to ServerRoot.
 *
 * Revision 1.24.2.13.4.3.6.1.4.6  1997/09/27 02:45:28  pvo
 * Changed serverName to serverHostName and added ...log.
 *
 *
 */

#include "netsite.h"
#include "base/systems.h"
#include "base/daemon.h"   /* daemon_s */
#include "base/util.h"     /* getline */
#include "base/shexp.h"    /* shexp_casecmp */
#include "frame/conf.h"    /* globals */
#include "frame/req.h"     /* REQ_ABORTED */
#include "frame/http.h"

#include <ldap.h>
#include "frame/servact.h" /* directive_name2num */
#include <frame/aclinit.h>

#include "libaccess/nserror.h"          /* error stack */
#include <libaccess/acl.h>
#include <libaccess/las.h>
#include <libaccess/ldapacl.h>
#include <libaccess/aclproto.h>

#include <libadmsslutil/psetcssl.h>
#include <libadmsslutil/srvutilssl.h>
#include <libadmsslutil/admsslutil.h>

#include <prlock.h>

#include "safs/init.h"

#include "httpdaemon/multihome.h"

#include "i18n.h"  /* SetClient,Admin,Default languages */

#include <ctype.h>       /* isspace */
#include <string.h>      /* strdup, strcmp */
#include <sys/types.h>
#include <sys/stat.h>

#include <sys/types.h>
#ifdef XP_UNIX
#include <unistd.h>
#include <pwd.h>
#else /* XP_WIN32 */
#include <nt/regparms.h>
#include <base/eventlog.h>
#include <nt/messages.h>
#endif /* XP_WIN32 */

#ifdef XP_UNIX
#define ERROR_EXIT(str)     \
    fprintf(stderr, str);   \
    exit(1) 
#define ERROR_MSG(str) fprintf(stderr, str)
#else /* XP_WIN32 */
#define ERROR_EXIT(str) \
    LogErrorEvent(NULL,EVENTLOG_ERROR_TYPE,0,MSG_BAD_STARTUP,str,NULL); \
    exit(1)
#define ERROR_MSG(str) LogErrorEvent(NULL,EVENTLOG_ERROR_TYPE,0,MSG_BAD_STARTUP,str,NULL)
#endif /* XP_WIN32 */

/* MLM - dump obj.conf to stdout if defined */
#undef MLM_DEBUG 

/* This should be exported from conf.cpp */
extern void  _conf_setdefaults(void);
extern char *_conf_setciphers (char *ciphers);
extern char *addr; /* DT -- addr is the wonderful global var to set the bind address */

/* And this one from addlog.c in safs */
extern int clf_init(pblock *pb, Session *sn, Request *rq);

#ifdef MALLOC_POOLS
extern int pool_internal_init(void);
#endif
#ifdef ACCELERATOR_CACHE
extern int accel_file_cache_internal_init(void);
extern int time_cache_internal_init(void);
#endif


/* XXXrobm win32 daemon module thinks maxprocs == maxthreads */
#if defined(THREAD_ANY) && !defined(XP_WIN32)
#define ADMIN_MAXPROCS 1
#define ADMIN_MINTHREADS 4
#define ADMIN_MAXTHREADS 32
#else
#define ADMIN_MAXPROCS 4
#endif

#define ADMIN_HASH_SIZE 2

#include <errno.h>
#ifndef LINUX
extern char *sys_errlist[];
#endif
#undef	system_errmsg
#define system_errmsg() (sys_errlist[errno]) /* from base/file.h */

/* DT -- 4.0 task acl stuff */
#include "taskacl.h"

/* A place to store the server short IDs from servers.adm lst. */
char **installed = NULL;
int num_installed = 0;
time_t slist_changed = 0;
PRLock *serverListLock;

char *nsroot = NULL;
char *admroot = NULL;
char *admusers = NULL;

int  adm_secureLDAP = 0;


static int 
admin_is_full_path(char *filepath) {
#ifdef XP_UNIX

  if (filepath[0] == '/') return 1;
  else return 0;
#else /* XP_WIN32 */
  if (filepath[0] == '/') return 1;
  else if (filepath[0] == '\\')  return 1;
  else if (filepath[1] == ':' && 
	   (filepath[2] == '\\' || filepath[2] == '/')) return 1;
  else return 0;
#endif /* XP_WIN32 */
}


/* ------------------------------- _getport ------------------------------- */


/*
 * Keeps trying to find a randomly selected open port on the machine
 */

int _getport(void)  
{
    SYS_NETFD sd;
    int one=1;
    struct sockaddr_in sa_server;
    int port, x;

#ifdef XP_WIN32
	CHAR ErrMsg[512];

	// Lets initialize winsock and then clean it up,just to get a random
	// port

	WSADATA wsd;

	if(WSAStartup(MAKEWORD(1, 1), &wsd) != 0) {
	    sprintf( ErrMsg, "Administration Server Startup Failed: "
	    	"Windows Sockets Initialisation failed: Error %d\n", 
		     WSAGetLastError());
	    LogErrorEvent(NULL, EVENTLOG_ERROR_TYPE, 0, 
			  MSG_BAD_WINSOCK_INIT, ErrMsg, NULL);
		exit(1);
	}
#endif /* XP_WIN32 */

    if ((sd = net_socket(AF_INET,SOCK_STREAM,IPPROTO_TCP)) == SYS_NET_ERRORFD) {
        return 29876;
    }

    if((net_setsockopt(sd, SOL_SOCKET, SO_REUSEADDR,
                       (void *)&one, sizeof(one))) == -1)
    {
        return 29876;
    }

    ZERO((char *) &sa_server, sizeof(sa_server));
    sa_server.sin_family=AF_INET;
    sa_server.sin_addr.s_addr=htonl(INADDR_ANY);

    srand(time(NULL));
    for(x=0; x<15000; x++)  {
         port = ( ((rand())%(65535-1024)) + 1024); 
         sa_server.sin_port=htons(port);
         if(net_bind(sd, (struct sockaddr *) &sa_server,sizeof(sa_server)) != -1){ 
                net_close(sd);
#ifdef XP_WIN32
                WSACleanup();
#endif /* XP_WIN32 */
                return port;
         }
    }
#ifdef XP_WIN32
	WSACleanup();
#endif /* XP_WIN32 */

    return 29876;
}

static int numProgramGroups = 0;
extern struct program_groups *ADM_programGroups;

static void
getProgramGroups( char *product )
{
  char		buf[CONF_MAXLEN];
  SYS_FILE	fd;
  filebuf_t	*buffer;
  int		ln = 0;
  int		groupIndex = -1;
  char		**groupNames = NULL;
  char		**groupMembers = NULL;

  sprintf( buf, "%s/bin/%s/admin/html/index.lst", nsroot, product );
  fd = system_fopenRO( buf );
  if ( fd == SYS_ERROR_FD ) {
    return;
  }
  buffer = filebuf_open(fd, FILE_BUFFERSIZE);
  while ( util_getline( buffer, ++ln, CONF_MAXLEN, buf ) == 0 ) {
    if ( buf[0] == '-' && buf[1] == '-' ) {
      if ( strncmp( "Category:", buf + 2, 9 ) == 0 ) {
	if ( ++groupIndex ) {
	  groupNames = (char **) PERM_REALLOC( groupNames,
					       sizeof( char * ) *
					       ( groupIndex + 2 ) );
	  groupMembers = (char **) PERM_REALLOC( groupMembers,
						 sizeof( char * ) *
						 ( groupIndex + 2 ) );
	} else {
	  groupNames = (char **) PERM_MALLOC( sizeof( char * ) * 2 );
	  groupMembers = (char **) PERM_MALLOC( sizeof( char * ) * 2 );
	}
	groupNames[groupIndex + 1] = NULL;
	groupMembers[groupIndex + 1] = NULL;
	groupNames[groupIndex] = PERM_STRDUP( strchr( buf + 11, ',' ) + 1 );
	groupMembers[groupIndex] = PERM_STRDUP( "" );
      } else if ( strncmp( "Option:", buf + 2, 7 ) == 0 && buf[9] ) {
	char	*t = strchr( buf + 9, ',' );
	int	len = strlen( groupMembers[groupIndex] );

	*t = '\0';
	groupMembers[groupIndex] =
	  PERM_REALLOC( groupMembers[groupIndex],
			len + ( t - ( buf + 9 ) ) + 2 );
	if ( len ) {
	  groupMembers[groupIndex][len++] = ',';
	  groupMembers[groupIndex][len] = '\0';
	}
	if ( buf[9] == '.' ) {	/* most likely a ../admin */
	  char	*start = buf + 9;
	  int	skipDirs = 0;

	  while ( strncmp( start, "../", 3 ) == 0 ) {
	    ++skipDirs;
	    start += 3;
	  }
	  while ( skipDirs-- ) {
	    start = strchr( start, '/' ) + 1;
	  }
	  strcat( groupMembers[groupIndex] + len, start );
	} else {
	  strcat( groupMembers[groupIndex] + len, buf + 9 );
	}
	if ( ( t = strchr( groupMembers[groupIndex] + len, '?' ) ) ||
	     ( t = strchr( groupMembers[groupIndex] + len, '/' ) ) ) {
	  *t = '\0';
	}
      } else if ( strncmp( "Programs:", buf + 2, 9 ) == 0 && buf[11] ) {
	int	pLen = strlen( buf + 11 );
	int	len = strlen( groupMembers[groupIndex] );

	groupMembers[groupIndex] =
	  PERM_REALLOC( groupMembers[groupIndex], len + pLen + 2 );
	if ( len ) {
	  groupMembers[groupIndex][len++] = ',';
	  groupMembers[groupIndex][len] = '\0';
	}
	strcat( groupMembers[groupIndex], buf + 11 );
      }
    }
  }
  filebuf_close( buffer );
  if ( groupNames ) {
    if ( numProgramGroups ) {
      ADM_programGroups = (struct program_groups *)
	PERM_REALLOC( ADM_programGroups, sizeof( struct program_groups ) *
		      ( numProgramGroups + 2 ) );
    } else {
      ADM_programGroups =
	(struct program_groups *) PERM_MALLOC( sizeof( struct program_groups )
					       * 2 );
    }
    ADM_programGroups[numProgramGroups].type = strdup( product );
    ADM_programGroups[numProgramGroups].groups = groupNames;
    ADM_programGroups[numProgramGroups].programs = groupMembers;
    ADM_programGroups[++numProgramGroups].type = NULL;
  }
}

/* ------------------------------ admin_getadmroot ------------------------ */

NSAPI_PUBLIC VOID admin_getadmroot(char **adm_root)
{
   *adm_root = admroot;
}

NSAPI_PUBLIC VOID admin_setadmroot(char *adm_root)
{
   admroot = adm_root;
}

/* return 0 if no change, else 1 */
NSAPI_PUBLIC int
admin_readServersList( int firstTime )
{
  char		line[BUFSIZ];
  char		*admRoot;
  struct stat	statBuf;
  SYS_FILE	fd;
  int		ln;
  int		stop;
  int		t;
  int		result = 0;

  admin_getadmroot( &admRoot );
  sprintf( line, "%s/%s", admRoot, "servers.lst" );
  if ( stat( line, &statBuf ) == 0 && statBuf.st_mtime > slist_changed ) {
    filebuf_t	*buf;
    PR_Lock( serverListLock );

    if((fd = system_fopenRO("servers.lst")) == SYS_ERROR_FD) {
      sprintf(line, "error reading servers.lst: %s\nExit\n", system_errmsg());
      ERROR_EXIT(line);
    }
    buf = filebuf_open(fd, FILE_BUFFERSIZE);
 
    stop=0; ln=0;
    while(!stop)  {
	char *d;
	char *v;

        ++ln;
        switch( (t = util_getline(buf, ln, CONF_MAXLEN, line))) {
            case -1:
                filebuf_close(buf);
                sprintf(line, "error reading servers.lst: %s\nExit\n",
                        system_errmsg());
                ERROR_EXIT(line);
 
            case 1:
                filebuf_close(buf);
                if(!installed)  {
                    sprintf(line, "error reading servers.lst: "
                                    "no servers found\nExit\n");
                    ERROR_EXIT(line);
                }
                stop=1;
                break;
 
            default:
                d = line;
                v = strchr(line, ':');
                *v++ = '\0';
                if(!installed)  {
                    num_installed++;
                    installed = (char **)
                                PERM_MALLOC((num_installed+1)*sizeof(char *));
                }  else  {
		    int	i;

		    for ( i = 0 ; i < num_installed ; ++i ) {
		      if ( strcmp( d, installed[i] ) == 0 ) {
			break;	/* already read this one */
		      }
		    }
		    if ( i < num_installed ) {
		      break;	/* already read it */
		    }
                    num_installed++;
                    installed = (char **) PERM_REALLOC(installed,
						       (num_installed+1)*
						       sizeof(char *));
                }
                installed[num_installed-1] = PERM_STRDUP(d);
                installed[num_installed] = NULL;
		if ( firstTime ) {	/* do this on restart */
		  getProgramGroups( d );
		}
                break;
        }
    }
    slist_changed = statBuf.st_mtime;	/* remember, for later */
    result = 1;
    PR_Unlock( serverListLock );
  }
  return result;
}

/* ------------------------------ admin_init ------------------------------ */

/*
 * Scans a simple configuration file and initializes static objsets
 */

#define STRPARM(str, src)  { if(str) FREE(str); str = STRDUP(src); }

#ifdef FORTEZZA
static uint32 xtoi(char *in) {
    unsigned char *str= (unsigned char *)in;
    uint32 value = 0;

    if (str == NULL) return 0;

    /* skip passed spaces and 0x's */
    while (*str && (isspace(*str) || (*str == '0'))) str++;
    if ((*str == 'x') || (*str == 'X')) str++;

    while (*str) {
	if ((*str <= '9') && (*str >= '0')) {
	    value = value * 0x10 + (*str) - '0';
	} else if ((*str <= 'f') && (*str >= 'a')) {
	    value = value * 0x10 + (*str) - 'a' + 0xa;
	} else if ((*str <= 'F') && (*str >= 'A')) {
	    value = value * 0x10 + (*str) - 'A' + 0xa;
	} else break;
	str++;
    }
    return value;
}
#endif /* FORTEZZA */

int
sync_task_sie_data(char *name, char *query, void *arg, Request *rq)
{
    AttrNameList serverlist = NULL, nl=NULL;
    AttributeList installlist = NULL;
    AdmldapInfo ldapInfo = NULL;
    int errorCode;
    PsetHndl tmp;
    char *admroot = (char*)arg;
    char *serverid, errmsg[1024];
    int servercnt, i;
    
    ldapInfo = admldapBuildInfo(admroot, &errorCode);

    if (!ldapInfo) return FALSE;

    task_register_server(ADMIN_SERVER_ID, admldapGetSIEDN(ldapInfo));

    serverlist = getServerDNListSSL(ldapInfo);
    servercnt=0;
    if (serverlist) {
      while (serverlist[servercnt]) servercnt++;
    }
    if (servercnt) {
      for (i=0; i < servercnt; i++) {
	/* Create Pset for each individual server */
	tmp = psetRealCreateSSL(admldapGetHost(ldapInfo),
				admldapGetPort(ldapInfo),
				admldapGetSecurity(ldapInfo),
				serverlist[i],
				admldapGetSIEDN(ldapInfo),
				admldapGetSIEPWD(ldapInfo),
				NULL,
				&errorCode);
	if (tmp) {
#          define SERVER_ID_ATTRIBUTE "nsServerID"

	   int errorcode;
	   char*  serverid = psetGetAttrSingleValue(tmp, SERVER_ID_ATTRIBUTE, &errorcode);

	   if (!serverid) {
              sprintf(errmsg, "AdmInit: Unable to find serverid for dn=\"%s\" (error code = %d)\n", serverlist[i], errorCode);
	      ERROR_MSG(errmsg);
           }

	   task_register_server(serverid, serverlist[i]);
        }
        else {
           sprintf(errmsg, "AdmInit: Failed to create psetHandle for %s (error code = %d)\n", serverlist[i], errorCode);
	   ERROR_MSG(errmsg);
        }
      }
    }

    /* DT 02/02/98
     * Register installed product tasks
     */
    if ((installlist = getInstalledServerDNListSSL(ldapInfo)))
    {
       int ii = 0;
       while (installlist[ii])
       {
	  char *productID = installlist[ii]->attrName;
	  char *productDN = installlist[ii]->attrVal[0];
	  task_register_server(productID, productDN);
          ii++;
       }
    }

    destroyAdmldap(ldapInfo);

    return TRUE;
}

static char * admSIEpwdHelpToken;
    
static char *
pwdCbk()
{
  char *pwd;

#ifdef XP_WIN32
  pwd = ADM_GetPassword_wHelp("Administration Server Administrator password: ",  admSIEpwdHelpToken);
  free(admSIEpwdHelpToken);
#else
  pwd = ADM_GetPassword("Administration Server Administrator password: ");
#endif

  if (!pwd) return NULL;

  admSetCachedSIEPWD(pwd);

  free(pwd);

  return admGetCachedSIEPWD();
}


void admin_init(char *cwd)
{
    int admport = -1, stop, errorCode;
    long cache_lifetime;
    char *admaddr = NULL;
    char *adm_server_hostname = NULL;
    char *user = NULL, *admerrs = NULL, *admpid = NULL;
    char *admip = NULL, *admdns = NULL, *admacc = NULL;
    char *ciphers = NULL;
    char *dbstr = NULL;
    char *val = NULL;
    AttrNameList serverlist = NULL, nl=NULL;
    AttributeList installlist=NULL;
    AdmldapInfo ldapInfo = NULL;
    int  servercnt;
    PsetHndl  pset;
    int       i;
    int       taskStatus;

    char filenamebuf[512];
    SYS_FILE fd;
    int ln = 0, t;
    char line[CONF_MAXLEN], errmsg[1024], *msg, *v, *d;

    httpd_object *obj;
    pblock *pb;

#ifdef FORTEZZA
    char *fort_cardMask = NULL;
    char *fort_krl = NULL;
    char *fort_personality = NULL;
#endif /* FORTEZZA */

    AttrNameList family_list;

#ifdef MALLOC_POOLS
    (void)pool_internal_init();
#endif

#ifdef ACCELERATOR_CACHE
    (void)accel_file_cache_internal_init();
#endif
    dns_cache_admin_init();
    http_set_allowed_methods();

    /* Zap config settings, get system's host name */
    _conf_setdefaults();
#ifdef NET_SSL
    /* We want security to be off by default. */
    security_active = 0;
#endif

    admroot = cwd;

#ifdef XP_WIN32
    {
	char *serverRoot = (char*)MALLOC(strlen(admroot));
	char *tmp = serverRoot;
	chdir("..\\..");
	getcwd(serverRoot, strlen(admroot));
	admSIEpwdHelpToken = (char*) MALLOC(strlen("file:///")+strlen(cwd)+strlen("/manual/en/admin/help/6_acl.htm#Administration Server Administrator Password"));
	while ((tmp = strstr(tmp, "\\")) != NULL) {
	    *tmp = '/';
	}
	sprintf(admSIEpwdHelpToken, "file:///%s/manual/en/admin/help/6_acl.htm#Administration Server Administrator Password", serverRoot);
	chdir(admroot);
    }
#endif


    ldapInfo = admldapBuildInfoCbk(admroot, pwdCbk, &errorCode);
    if (!ldapInfo) {
      sprintf(errmsg, "AdmInit: Failed to read data from adm.conf\nExit\n");
      ERROR_EXIT(errmsg);
    }

    errorCode = ADMSSL_Init(ldapInfo, admroot);
    if (errorCode) {
      sprintf(errmsg, "AdmInit: Failed to perform security initialization for communicate with LDAP in SSL mode (error code = %d)\nExit\n", errorCode);
      ERROR_EXIT(errmsg);
    }

    adm_secureLDAP = admldapGetSecurity(ldapInfo);

    /* Step 1: Scan config info */

    pset = psetCreateSSL("admin-serv", 
			 /* configRoot */ cwd,
			 /* userDN */ NULL,
			 /* passwd */ NULL,
			 &errorCode);
    if (!pset) {
       sprintf(errmsg, "AdmInit: Failed to get configuration parameters (pset error  = %s)\nExit\n", psetErrorString(errorCode, NULL));
       ERROR_EXIT(errmsg);
    }

    nsroot = psetGetAttrSingleValue(pset, "serverRoot", &errorCode);
    if (nsroot) STRPARM(server_root, nsroot);

    val = psetGetAttrSingleValue(pset, "nsAdminCGIWaitPid", &errorCode);
    if (val) conf_getglobals()->wait_for_cgi=(strcmp(val, "on") ? 0 : 1);

    val = psetGetAttrSingleValue(pset, 
				 "configuration.nsServerPort", 
				 &errorCode);
    if (val) {
      if(!strcasecmp(val, "random"))
         admport = _getport();
      else
	 admport = atoi(val);
      conf_getglobals()->Vport = port = admport; /* DT -- port is a global from frame */
    }

    /* DT 3/24/98 SUpport for nsServerAddress attribute */
    admaddr = psetGetAttrSingleValue(pset, "configuration.nsServerAddress", &errorCode);
    if (admaddr) {
      conf_getglobals()->Vaddr = STRDUP(admaddr);
      addr = STRDUP(admaddr); /* DT -- addr is a global from frame */
      free(admaddr);
    }
    user = psetGetAttrSingleValue(pset, 
				  "configuration.nsSuiteSpotUser", 
				  &errorCode);

    admusers = psetGetAttrSingleValue(pset, "configuration.nsAdminUsers",
				&errorCode);
    if (admusers) {
      if (!admin_is_full_path(admusers) && server_root) {
	sprintf(filenamebuf, "%s/%s", server_root, admusers);
	free(admusers);
	admusers = STRDUP(filenamebuf);
      }
    }

    admerrs = psetGetAttrSingleValue(pset, "configuration.nsErrorLog", &errorCode);
    if (admerrs) {
      if (!admin_is_full_path(admerrs) && server_root) {
	sprintf(filenamebuf, "%s/%s", server_root, admerrs);
	free(admerrs);
	admerrs = STRDUP(filenamebuf);
      }
    }

    admpid = psetGetAttrSingleValue(pset, 
				    "configuration.nsPidLog", 
				    &errorCode);
    if (admpid) {
      if (!admin_is_full_path(admpid) && server_root) {
	sprintf(filenamebuf, "%s/%s", server_root, admpid);
	free(admpid);
	admpid = STRDUP(filenamebuf);
      }
    }
    conf_getglobals()->Vpidfn = admpid;

    admacc = psetGetAttrSingleValue(pset, 
				    "configuration.nsAccessLog", 
				    &errorCode);
    if (admacc) {
      if (!admin_is_full_path(admacc) && server_root) {
	sprintf(filenamebuf, "%s/%s", server_root, admacc);
	free(admacc);
	admacc = STRDUP(filenamebuf);
      }
    }

    adm_server_hostname = psetGetAttrSingleValue(pset, 
						"serverHostName", 
						&errorCode);
    if(adm_server_hostname) {
      server_hostname = STRDUP(adm_server_hostname);
      free(adm_server_hostname);
    }

    admdns = psetGetAttrSingleValue(pset, 
				    "configuration.nsAdminAccessHosts", 
				    &errorCode);
    admip = psetGetAttrSingleValue(pset, 
				   "configuration.nsAdminAccessAddresses", 
				   &errorCode);

#ifdef NET_SSL
    /* check for SSL active or not */

    security_active = 0;
    val =  psetGetAttrSingleValue(pset, 
				  "configuration.nsServerSecurity",
				  &errorCode);
    if(val && (!strcmp(val, "on")))
      security_active=1;

    val =  psetGetAttrSingleValue(pset, 
				  "configuration.Encryption.nsKeyfile",
				  &errorCode);
    if (val) {
      if (!admin_is_full_path(val) && server_root) {
	sprintf(filenamebuf, "%s/%s", server_root, val);
	secure_keyfn = STRDUP(filenamebuf);
	conf_getglobals()->Vsecure_keyfn = STRDUP(secure_keyfn);
      }
      else {
	secure_keyfn = STRDUP(val);
	conf_getglobals()->Vsecure_keyfn = STRDUP(secure_keyfn);
      }
    }

    val = psetGetAttrSingleValue(pset, 
				 "configuration.Encryption.nsCertfile",
				 &errorCode);
    if (val) {
      /* removed by yjh
      if ((dbstr = strstr(val, ".db")) != NULL)
	*(val+(strlen(val)-strlen(dbstr))) = '\0';
	*/
      if (!admin_is_full_path(val) && server_root) {
	sprintf(filenamebuf, "%s/%s", server_root, val);
	secure_certfn =  STRDUP(filenamebuf);
	conf_getglobals()->Vsecure_certfn = STRDUP(secure_certfn);
      }
      else {
	secure_certfn =  STRDUP(val);
	conf_getglobals()->Vsecure_certfn = STRDUP(secure_certfn);
      }
    }

    val = psetGetAttrSingleValue(pset, 
				 "configuration.Encryption.nsSSL3",
				 &errorCode);
    if (val) {
      if(!strcasecmp(val, "on")) ssl3_active = 1;
      else if(!strcasecmp(val, "off")) ssl3_active = 0;
      else {
	util_sprintf(errmsg, "SSL3 must be on or off.\nExit\n");
	ERROR_EXIT(errmsg);
      }
    }
    else ssl3_active = 1; /* Default to on */

    val = psetGetAttrSingleValue(pset, 
				 "configuration.Encryption.nsSSL2", 
				 &errorCode);
    if (val) {
      if(!strcasecmp(val, "on")) ssl2_active = 1;
      else if(!strcasecmp(val, "off")) ssl2_active = 0;
      else {
	util_sprintf(errmsg, "SSL2 must be on or off.\nExit\n");
	ERROR_EXIT(errmsg);
      }
    }
    else ssl2_active = 0; /* default to off */

    val = psetGetAttrSingleValue(pset, 
				 "configuration.Encryption.nsSSLClientAuth", 
				 &errorCode);
    if (val) {
      if (!strcasecmp(val, "on")) {
	secure_auth = 1;
      }
      else if(!strcasecmp(val, "off")) secure_auth = 0;
      else {
	util_sprintf(errmsg, "SSLClientAuth must be on or off.\nExit\n");
	ERROR_EXIT(errmsg);
      }
      if (security_active) {
	if (secure_auth && admusers) { free(admusers); admusers = NULL; }
	conf_getglobals()->Vsecure_auth = secure_auth;
      }
      else conf_getglobals()->Vsecure_auth = 0;
    }
    else conf_getglobals()->Vsecure_auth = 0;  /* Default to OFF */

    val = psetGetAttrSingleValue(pset, 
				 "configuration.Encryption.nsSSLSessionTimeout", 
				 &errorCode);
    if (val) security_session_timeout = atoi(val);

    val = psetGetAttrSingleValue(pset, 
				 "configuration.Encryption.nsSSL3SessionTimeout", 
				 &errorCode);
    if (val) ssl3_session_timeout = atoi(val);

    /* MLM */
#endif /*  NET_SSL  */

    msg = NULL;
    if(admport == -1) msg = "bind port"; 
    /*else if(!admaddr) msg = "bind address";*/ /* DT - tolerate missing admaddr for now */
    else if(!nsroot) msg = "netsite root";
    else if(!admerrs) msg = "error log";
#ifdef XP_UNIX
    else if(!admpid) msg = "pid log";
#endif /* XP_UNIX */
    else if(!server_hostname) msg = "server hostname";
#ifdef NET_SSL
    else if((security_active) && (!secure_keyfn))  msg = "security keyfile";
    else if((security_active) && (!secure_certfn)) msg = "security certificate";
    if (msg) {
      sprintf(errmsg, "AdmInit: no %s specified\nExit\n", msg);
      ERROR_EXIT(errmsg);
    }

#endif

    /* Step 2: Scan in the server types file. */
 
    serverListLock = PR_NewLock();
    admin_readServersList( 1 );

#ifndef DEBUGGING_OFF
    /*printf("Num installed products = %d\n", num_installed);*/
    for (i = 0 ; i < num_installed ; i++)
       printf("installed[%d] = %s\n", i, installed[i]);
#endif

    /* Step 3: Create the frame lib's static configuration          */

    PR_Lock( serverListLock );
    getProgramGroups( "admin" );
    PR_Unlock( serverListLock );
    /* magnus.conf parameters */
    master_error_log = admerrs;
    pidfn = admpid;
    root_object = STRDUP("default");

#if defined THREAD_ANY && !defined(XP_WIN32)
    pool_maxthreads = ADMIN_MAXTHREADS;
#endif

#ifdef XP_UNIX

    /* Resolve user into a struct passwd */
    if((user) && (!geteuid())) {
        struct passwd *pw;

        setpwent();
        if(!(pw = getpwnam(user))) {
            sprintf(errmsg, "error: can't find user %s in passwd file\nExit\n", 
                    user);
            ERROR_EXIT(errmsg);
        }
        else {
            user_pw = (struct passwd *) MALLOC(sizeof(struct passwd));
            ZERO(user_pw, sizeof(struct passwd));
            /* This is silly. */
            user_pw->pw_uid = pw->pw_uid;
            user_pw->pw_gid = pw->pw_gid;
            user_pw->pw_name = STRDUP(pw->pw_name);
        }
    }
#endif /* XP_UNIX */
    
    /* Create an access log. */
    if(admacc)  {
        pb = pblock_create(ADMIN_HASH_SIZE);

        pblock_nvinsert("global", admacc, pb);
        if((clf_init(pb, NULL, NULL)) == REQ_ABORTED)  {
            /* Need more message? */
            fprintf(stderr, "error opening access log: can't open %s\n", admacc);
            fprintf(stderr, "Access Log disabled!\n");
            /*exit(1);*/
        }
        pblock_free(pb);
    }

    /* We now create a skeleton object set */
    std_os = objset_create();

    /* Create only one object at first, more may come if ACL objects exist. */
    pb = pblock_create(ADMIN_HASH_SIZE);
    pblock_nvinsert("name", "default", pb);
    obj = objset_new_object(pb, std_os);

    /* DT 1/22/98
     * First, check if the hostname or ip address is disallowed
     */
    pb = pblock_create(ADMIN_HASH_SIZE);
    pblock_nvinsert("fn", "admin40-host-ip-check", pb);
    if(admdns)
       pblock_nvinsert("admdns", admdns, pb);
    if(admip)
       pblock_nvinsert("admip", admip, pb);
    object_add_directive(directive_name2num("AuthTrans"), pb, NULL, obj);

    /* DT 1/22/98 
     * Next, we want to check for the directories which have no security.
     */
    pb = pblock_create(ADMIN_HASH_SIZE);
    pblock_nvinsert("fn", "admin40-isdiropen", pb);
    object_add_directive(directive_name2num("AuthTrans"), pb, NULL, obj);
    
    /* DT 1/22/98 
     * Next, we want to attempt to authenticate using the directory server.
     */
    pb = pblock_create(ADMIN_HASH_SIZE);
    pblock_nvinsert("fn", "admin40-authenticate", pb);
    object_add_directive(directive_name2num("AuthTrans"), pb, NULL, obj);

    /* DT 1/22/98 
     * Lastly, we want to attempt to authenticate using the local admpw.
     */
    pb = pblock_create(ADMIN_HASH_SIZE);
    pblock_nvinsert("fn", "admin-check-admpw", pb);
    if(admusers) 
       pblock_nvinsert("admpwfile", admusers, pb);
    object_add_directive(directive_name2num("AuthTrans"), pb, NULL, obj);

    /* DT 3/10/98
     * Support for nsAdminCacheLifetime attribute
     */
    if ((val = psetGetAttrSingleValue(pset, 
				      "configuration.nsAdminCacheLifetime",
				      &errorCode))) {
      if ((cache_lifetime = atol(val)) < 0) {
          util_sprintf(errmsg, "nsAdminCacheLifetime must be >= 0 (in seconds)\nExit\n");
	  ERROR_EXIT(errmsg);
       }
    }
    else cache_lifetime = 0; /* immediate expiration */
    
    /* DT 2/2/98
     * Support for new task dispatch scheme. No more static setup...everything
     * should be dynamically determined at invocation time, for any server
     * we've called task_register_server() for...
     */
    taskStatus = task_eval_init(nsroot, cache_lifetime);

    /* The error info will be output inside task_eval_init
    if (taskStatus == FALSE) {
      util_sprintf(errmsg, "Warning: task_eval_init failed!\n");
      ERROR_MSG(errmsg);
    }
    */

    /* DT 3/15/98
     * Support for runtime commands.
     */
    admin40_runtime_command_init();
    admin40_register_runtime_command(RUNTIME_RESYNC_COMMAND, sync_task_sie_data, (void*)admroot);
    admin40_register_runtime_command(CHANGE_SIEPWD_COMMAND, change_sie_password, (void*)admroot);

    /* DT 3/15/98
     * Moved SIE/task stuff to resync_task_sie_data.
     */
    admin40_runtime_command_exec(RUNTIME_RESYNC_COMMAND, NULL, NULL);

    /* NameTrans fn=admin-uri2path root=admroot */
    pb = pblock_create(ADMIN_HASH_SIZE);
    pblock_nvinsert("fn", "admin-uri2path", pb);
    pblock_nvinsert("root", admroot, pb);
    /* DT 12/4/97 check if dsgw has been set, and pass to uri2path */
#   define DSGW_ATTRIBUTE "configuration.nsAdminEnableDSGW"
#   define ENDUSER_ATTRIBUTE "configuration.nsAdminEnableEnduser"

    /* Bug 251344: atulb 7/22/98 
     * psetGetAttrSingleValue should return either "off" or "on" only for this attribute.      * Check for NULL in any case. I now add the attributes nsadminenableenduser and
     * nsadminenabledsgw at install time into the SIE's configuration entry 
     * with initial values "on" for both.  */
    val = psetGetAttrSingleValue(pset, DSGW_ATTRIBUTE, &errorCode);
    if (val) 
       if (!strcmp(val, "off"))
          pblock_nvinsert("dsgw", "", pb);
    val = psetGetAttrSingleValue(pset, ENDUSER_ATTRIBUTE, &errorCode);
    if (val) 
       if (!strcmp(val, "off"))
       pblock_nvinsert("enduser", "", pb);
    object_add_directive(directive_name2num("NameTrans"), pb, NULL, obj);

    /* PathCheck fn=uri-clean */
    pb = pblock_create(ADMIN_HASH_SIZE);
#ifdef XP_UNIX
    pblock_nvinsert("fn", "uri-clean", pb);
    object_add_directive(directive_name2num("PathCheck"), pb, NULL, obj);    
#else /* XP_WIN32 */
    pblock_nvinsert("fn", "nt-uri-clean", pb);
    object_add_directive(directive_name2num("PathCheck"), pb, NULL, obj);    

    /* PathCheck fn=ntcgicheck extension=.exe */
    pb = pblock_create(ADMIN_HASH_SIZE);
    pblock_nvinsert("fn", "ntcgicheck", pb);
    pblock_nvinsert("extension", ".exe", pb);
    object_add_directive(directive_name2num("PathCheck"), pb, NULL, obj);    
#endif /* XP_WIN32 */

    /* PathCheck fn=find-index index-names="index.html" */
    pb = pblock_create(ADMIN_HASH_SIZE);
    pblock_nvinsert("fn", "find-index", pb);
    pblock_nvinsert("index-names", "index.html", pb);
    object_add_directive(directive_name2num("PathCheck"), pb, NULL, obj);    

    /* ObjectType fn=admin-ftype */
    pb = pblock_create(ADMIN_HASH_SIZE);
    pblock_nvinsert("fn", "admin-ftype", pb);
    object_add_directive(directive_name2num("ObjectType"), pb, NULL, obj);

    /*
     * DT 03/13/98
     * Support for runtime command execution. We need a way to perform certain operations
     * without an admin server restart. The /command/* namespace is to be used for this
     * purpose, providing a general runtime command execution namespace.
     */
    /* Service fn="admin-runtime-commands" type=admin-internal/command */
    pb = pblock_create(ADMIN_HASH_SIZE);
    pblock_nvinsert("fn", "admin40-runtime-commands", pb);
    pblock_nvinsert("type", RUNTIME_COMMAND_CONTENT_TYPE, pb);
    object_add_directive(directive_name2num("Service"), pb, NULL, obj);

    /* Service fn="send-cgi" type=magnus-internal/cgi */
    pb = pblock_create(ADMIN_HASH_SIZE);
    pblock_nvinsert("fn", "send-cgi", pb);
    pblock_nvinsert("type", "magnus-internal/cgi", pb);
    object_add_directive(directive_name2num("Service"), pb, NULL, obj);

    /* Service fn="imagemap" type="magnus-internal/imagemap" */
    pb = pblock_create(ADMIN_HASH_SIZE);
    pblock_nvinsert("fn", "imagemap", pb);
    pblock_nvinsert("method", "GET", pb);
    pblock_nvinsert("type", "magnus-internal/imagemap", pb);
    object_add_directive(directive_name2num("Service"), pb, NULL, obj);

    /* Service method=GET fn="send-file" */
    pb = pblock_create(ADMIN_HASH_SIZE);
    pblock_nvinsert("fn", "send-file", pb);
    pblock_nvinsert("method", "GET", pb);
    object_add_directive(directive_name2num("Service"), pb, NULL, obj);

    /* AddLog fn="common-log" */
    pb = pblock_create(ADMIN_HASH_SIZE);
    pblock_nvinsert("fn", "common-log", pb);
    object_add_directive(directive_name2num("AddLog"), pb, NULL, obj);

    /* Error fn=admin-error reason="server error" */
    pb = pblock_create(ADMIN_HASH_SIZE);
    pblock_nvinsert("fn", "admin-error", pb);
    pblock_nvinsert("reason", "server error", pb);
    object_add_directive(directive_name2num("Error"), pb, NULL, obj);

#ifdef MLM_DEBUG
/* MLM - dump the created obj.conf for debugging */
{
    register int x, y, z;
    int i;
    dtable *dt;
    const char *u;
    char *t;
    FILE *f = stdout;
    char objpath[1024];
    httpd_objset *os = std_os;

    for(x = 0; x < os->pos; x++) {
        t = pblock_pblock2str(os->obj[x]->name, NULL);
        fprintf(f, "<Object %s>\n", t);
        FREE(t);
        for(y = 0; y < os->obj[x]->nd; y++) {
            dt = &os->obj[x]->dt[y];
            u = directive_num2name(y);
            for(z = 0; z < dt->ni; z++) {
                t = pblock_pblock2str(dt->inst[z].param, NULL);
                fprintf(f, "%s %s\n", u, t);
                FREE(t);
            }
        }
        fputs("</Object>\n\n", f);
    }
}
#endif

    MultihomeInit();

    return;
}
