/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * adminit.c: Initialization routines for the admin server
 * 
 * Rob McCool
 *
 */

#include "netsite.h"
#include "base/daemon.h"
#include "base/util.h"
#include "frame/conf.h"


#ifdef XP_UNIX
#include <sys/types.h>
#include <unistd.h>
#include <pwd.h>
#else /* XP_WIN32 */
#include <windows.h>
#include <nt/messages.h>
#include <nt/magnus.h>
#include <nt/regparms.h>
#include <base/eventlog.h>
#include "base/ereport.h"
#include <frame/func.h>
#include <frame/http.h>
#include <process.h>

extern int admin_init(char *cwd);
void magnus_restart(int);

extern struct FuncStruct func_standard[];

SC_HANDLE schSCManager;
SC_HANDLE schService;
static struct rsf {
    void (*fn)(void *);
    void *data;
    struct rsf *next;
} *rlist;

#endif /* XP_WIN32 */


#include "../lib/libsi18n/gsadmserv.h"
extern char *nsroot;
extern void XP_InitStringDatabase(char* pathCWD, char* databaseName);


extern 

int magnus_init()
{
	CHAR ErrMsg[1024];
    char *admdir, *err;
    char *resroot;

    func_init(func_standard);
	admdir = NULL;

 	InitializeAdminLogging (EVENTLOG_APPNAME, eventlog_file);

	if (!SetCurrentDirectory(admroot)) {
        sprintf(ErrMsg, "%s Startup Error : Could not open error log: %d", 
        	SERVICE_NAME, GetLastError());
		LogErrorEvent(NULL, EVENTLOG_ERROR_TYPE, 0, 
			MSG_BAD_STARTUP, ErrMsg, NULL);
		exit(1);
	}

    (VOID)admin_init(admroot);

    /* Initialize resource database or properties */
    resroot = (char *) malloc (strlen(nsroot) + 40);
    strcpy(resroot, nsroot);
    strcat(resroot, "/bin/admin/property");
    XP_InitStringDatabase(resroot, DATABASE_NAME);

	if(err = ereport_init(master_error_log, NULL, NULL, BUILD_NUM)) {
        sprintf(ErrMsg, "%s Startup Error : Could not open error log: %s", 
        	SERVICE_NAME, err);
		LogErrorEvent(NULL, EVENTLOG_ERROR_TYPE, 0, 
			MSG_BAD_STARTUP, ErrMsg, NULL);
		exit(1);
    }
	return 0;
}


int threads_init()
{
	CHAR ErrMsg[512];
    int clport;
	clport = -1;

	d = (daemon_s *)MALLOC(sizeof(daemon_s));

    d->port = (clport == -1 ? port : clport);
    d->rcback = magnus_restart;
    d->maxprocs = pool_max;
    d->minprocs = pool_min;
	d->proclife = pool_life;
	d->child_callback = protocol_handle_session;
	d->rotate_callback = NULL;
	d->service_name = strdup(SERVICE_NAME);
	d->domains = domain_array;
    if(MainThread = _beginthreadex(NULL, 0, (void *)_ntdaemon_run, 
    	(void *)d, 0, &MainThreadId) == 0) {
		sprintf(ErrMsg, "%s Startup Error:"
			" Could not Start Service Main Thread: Error %d", service_name, errno);
		LogErrorEvent(NULL, EVENTLOG_ERROR_TYPE, 0, 
			MSG_BAD_STARTUP, ErrMsg, NULL);
		ereport(LOG_CATASTROPHE, "%s Startup Error:"
			" Could not Start Service Main Thread: Error %d", service_name, errno);
		exit(1);
	}
    return 0;
}

#ifdef XP_WIN32
void magnus_atrestart(void (*fn)(void *), void *data)
{
    struct rsf *r;

    r = (struct rsf *) MALLOC(sizeof(struct rsf));
    r->fn = fn;
    r->data = data;
    r->next = rlist;
    rlist = r;
}

void magnus_restart(int fs)
{
    struct rsf *r = rlist, *s;
    SYS_FILE fd = ereport_getfd();

    while(r) {
        (*r->fn)(r->data);
        s = r;
        r = r->next;
        FREE(s);
    }
    rlist = NULL;


    conf_terminate();
    objset_free(std_os);

	if (MainThread != NULL) {
		TerminateThread(MainThread, 0);
		MainThread = NULL;
	}

    if(magnus_init())
        exit(1);
}

BOOLEAN
RemoveService(LPCTSTR serviceName)
{
    BOOL    ret;
	int retries = 0;

    schService = OpenService(schSCManager, serviceName, SERVICE_ALL_ACCESS);

RETRY:

    if (schService == NULL) {
    	if (GetLastError() != ERROR_SERVICE_DOES_NOT_EXIST &&
    		GetLastError() != ERROR_SERVICE_MARKED_FOR_DELETE ) {
			ereport(LOG_INFORM,"Remove service failed %d",GetLastError());
        	return FALSE;
		}  else if (retries < 3) {
			Sleep(5000);
			retries++;
			goto RETRY;
		}  else {
			ereport(LOG_FAILURE, "Could not remove service %s Error %d",
				serviceName, GetLastError());
		}
    }

    ret = DeleteService(schService);

    if (ret) {
		CloseServiceHandle(schService);
		return TRUE;
	}
    else if ( GetLastError() != ERROR_SERVICE_DOES_NOT_EXIST) {
		ereport(LOG_INFORM,"Deleteservice %s failed %d", 
			serviceName, GetLastError());
		CloseServiceHandle(schService);
		return FALSE;
	}
	return TRUE;
}


BOOLEAN
StopService(LPCTSTR serviceName)
{
    BOOL    ret;
	SERVICE_STATUS ServiceStatus;
	DWORD Error;

    schService = OpenService(schSCManager, serviceName, SERVICE_ALL_ACCESS);

    if (schService == NULL && (GetLastError() != ERROR_SERVICE_DOES_NOT_EXIST)) {
        ereport(LOG_FAILURE, "Tried to stop %s service"
        	": Could not open existing service: Error %d",
        	serviceName, GetLastError());
    }

    ret = ControlService(schService, SERVICE_CONTROL_STOP, &ServiceStatus);

    if (ret) {
		CloseServiceHandle(schService);
		return TRUE;
	}
    else if (( Error = GetLastError()) != ERROR_SERVICE_NOT_ACTIVE) {
		ereport(LOG_FAILURE, "Tried to stop %s service"
			":could not send Control to service: Error %d" ,
			serviceName, GetLastError());
	}
}

VOID
DeinstallService(VOID *Argument)
{
	PCHAR serviceName = (PCHAR)Argument;
	Sleep(5000);
	if (!(schSCManager = OpenSCManager(
                        NULL,                   // machine (NULL == local)
                        NULL,                   // database (NULL == default)
                        SC_MANAGER_ALL_ACCESS   // access required
                        ))) {
		ereport(LOG_FAILURE,"Could not open Service Control Manager"
			"Error %d", GetLastError());
		return;
	}

	StopService(serviceName);
	if(!RemoveService(serviceName)) {
		ereport(LOG_FAILURE,"Could not remove the %s service"
			"Error %d", serviceName, GetLastError());
	}
	free(Argument);
	Argument = NULL;
}

#endif /* XP_WIN32 */



