/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

/*
 * Change Admin Superuser password ( A runtime command)
 *
 * in:<new password>
 * Update the following :
 *   1) "userPassword" attribute in the SIE on the DS
 *   2) "siepid" parameter in config/adm.conf
 *   3) config/admpw file. The password is stored in encrypted form
 *   4) Update cached registryServer info (a private structure in taslacl.c)
 */

#include "libadmin/libadmin.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "libadminutil/admutil.h"
#include "libadminutil/distadm.h"
#include "libadmsslutil/psetcssl.h"
#include "libadmsslutil/admsslutil.h"

#include "taskacl.h"

static int update_admpwd(char* admroot, char *newuid, char *newpw);
static int update_adm_conf(char* admroot, char *newpw);
static int update_ds(char *admroot, char *newpw, Request *rq);

/*
 * Modify SIE password 
 */
int change_sie_password(char *name, char *query, void* arg, Request* rq) {

	FILE *f;
	char *uid=NULL ,*pw=NULL, *col=NULL;
	char *newpw=query;
	static char filename[BIG_LINE];
	static char inbuf[BIG_LINE];
	char *admroot = (char *) arg;
	char *par;

	sprintf(filename, "%s/admpw", admroot);

	if (newpw==NULL) {
		fprintf(stderr, "Wrong usage, password param missing\n");
		return 0;
	}

	if ((f=fopen(filename, "r"))==NULL) {
		fprintf(stderr,"Can not open admpw file\n");
		return 0;
	}
	if (fgets(inbuf, sizeof(inbuf), f) == NULL) {
		fclose(f);
		fprintf(stderr, "Can not read admpw file\n");
		return 0;
	}
	fclose(f);

	col = strchr(inbuf, ':');
	if (col == NULL) {
		fprintf(stderr, "admpw file corrupted\n");
		return 0;
	}
	
	uid = inbuf; *col=0; pw=col+1;
	
	if (!update_ds(admroot, newpw, rq))  {
		return 0;
	}
	if (!update_adm_conf(admroot, newpw)) {
		return 0;
	}
	if (!update_admpwd(admroot, uid, pw_enc(newpw))) {
		return 0;
	}

        task_update_registry_server_bindpw(strdup(newpw));
	return 1;
}

/*
 * Modify admpw.
 */
static int update_admpwd(char *admroot, char *newuid, char *newpw) {

	FILE *f;
	int cnt;
	static char filename[BIG_LINE];
	static char inbuf[BIG_LINE];
	static char outbuf[BIG_LINE];

	sprintf(filename, "%s/admpw", admroot);

	f = fopen(filename, "w");
	if (f==NULL) {
		fprintf(stderr, "Can not open admpw file for writing\n");
		return 0;
	}

	sprintf(outbuf,"%s:%s", newuid, newpw);

	cnt = fprintf(f,"%s", outbuf);

	if (cnt != (int)strlen(outbuf)) {
		fprintf(stderr, "Failed to write to admpw\n");
		fclose(f);
		return 0;
	}
	
	fclose(f);
	return 1;
}

/*
 * Modify adm.conf
 */
static int update_adm_conf(char *admroot, char *newpw) {

	FILE *f;
	int i, modified=0;
	static char filename[BIG_LINE];
	char buf[80];
	int linecnt=0;
	char *lines[50];
	static char inbuf[BIG_LINE];


	sprintf(filename, "%s/adm.conf", admroot);

	f = fopen(filename, "r");
	if (f==NULL) {
		return 0;
	}

	while(fgets(inbuf, sizeof(inbuf), f) != NULL) {
		if (strstr(inbuf,"siepid:") == inbuf) { /* Line starts with "Password:" */
			sprintf(buf, "siepid: %s\n", newpw);
			lines[linecnt++] = strdup(buf);
			modified=1;
		}
		else {
			lines[linecnt++] = strdup(inbuf);
		}
	}	
	fclose(f);

	if (!modified) {
        admSetCachedSIEPWD(newpw);

		return 1;
	}

	f = fopen(filename, "w");
	if (f==NULL) {
		return 0;
	}

	for (i=0; i < linecnt; i++) {
		fprintf(f, "%s", lines[i]);
	}

	fclose(f);
	return 1;
}

/*
 * Modify userpassword in the DS
 */
static int update_ds(char *admroot, char *pwd, Request *rq) {
  
    int   errorCode;
    PsetHndl pset;
    char *binddn = 0;
    char *bindpw = 0;
    AdmldapInfo ldapInfo = NULL;

    /* Initialize the pset  */

    ldapInfo = admldapBuildInfo(admroot, &errorCode);

    if (!ldapInfo) {
       fprintf(stderr, "AdmInit: Failed to read data from adm.conf\nExit\n");
       return 0;
    }

    errorCode = ADMSSL_Init(ldapInfo, admroot);
    if (errorCode) {
        fprintf(stderr, "Security Initialization Failed\n");
        return 0;
    }

    binddn = pblock_findval("userdn", rq->vars);
    bindpw = pblock_findval("auth-password", rq->vars);

    pset = psetCreateSSL("admin-serv", 
  		         admroot,
		         binddn,
		         bindpw,
		         &errorCode);

    if (!pset) {
        fprintf(stderr, "PSET Creation Failed, err=%d\n", errorCode);
        return 0;
    }

    errorCode = psetSetSingleValueAttr(pset, "userpassword", pwd);

    if (errorCode) {
        psetDelete(pset);
        fprintf(stderr, "PSET SET Failed, err=%d\n", errorCode);
        return 0;
    }
    psetDelete(pset);
    return 1;
}
