/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * main.c: Startup for admin server, reads config files, etc.
 * 
 * Rob McCool
 *
 */


extern "C" {
#include "secrng.h"
extern void SEC_Init(void);
}

#include "netsite.h"
#include "base/daemon.h"
#include "base/ereport.h"
#include "base/util.h"
#include "frame/conf.h"
#include "frame/protocol.h"
#include "frame/func.h"
#include "base/systhr.h"
#include "dbtadmserv.h"
#include "../lib/libsi18n/gsadmserv.h"

#include <signal.h>


void oops(int sig)
{
    switch(sig) {
      case SIGTERM:
        ereport(LOG_WARN, "terminated, shutting down");
        exit(1);
      case SIGSEGV:
      case SIGBUS:
        ereport(LOG_CATASTROPHE,
          "caught SIGSEGV or SIGBUS, trying to dump core in admin/config");
        abort();
    }
}


#ifdef USE_NSPR
void _initialize_crits(void)
{
    extern void auth_init_crits(void);
    auth_init_crits();
}
#endif

extern char *optarg;
extern int optind;
extern struct FuncStruct func_standard[];
extern void cgi_grabstds(void);
void admin_init(char *admroot);
#ifdef	INPROC_CRON
extern int cron_init(void);
extern void cron_terminate(void);
extern void cron_kill(void);
#endif	/* INPROC_CRON */

void magnus_atrestart(void (*fn)(void *), void *data)
{
    daemon_atrestart(fn, data);
}
 
/* MLM - BONEHEAD - Error reporting?! */
extern char *admroot;
void magnus_restart(int sig)
{
    daemon_dorestart();
 
    conf_terminate();
    objset_free(std_os);

    if(ACL_InitHttp())
	exit(1);
    /* Hmm, admin_init is void, and prints stuff to stdout.  Doh. */
    admin_init(admroot);

    if(ACL_InitHttpPostMagnus())
	exit(1);
}

void admserv_exit(void)
{
#ifdef	INPROC_CRON
    cron_kill();
#endif	/* INPROC_CRON */
}


int main(int argc, char *argv[])
{
    int c, clport;
    daemon_s d;
    char *err, *admdir = ".";
    int nodetach = 0;
	char *resroot;
    char* pathCWD;

    /* get startup current working directorty for XP_InitStringDatabase */
    pathCWD=getcwd(NULL,1024);
  
    cgi_grabstds();
    clport = -1;
    umask(022);

    while((c = getopt(argc, argv, "d:nvp:")) != -1) {
        switch(c) {
          case 'd':
            admdir = STRDUP(optarg);
            chdir(optarg);
            break;
          case 'n':
            /* option "nofork" */
            nodetach = 1;
            break;
          case 'v':
            printf("%s\n%s\n", MAGNUS_VENDER_NAME, MAGNUS_VERSION_STRING);
            return 1;
          case 'p':
            clport = atoi(optarg);
            break;
          case '?':
            fprintf(stderr, "Usage: %s [-v] [-d conf_dir] [-p port]\n",
                    argv[0]);
            return 1;
        }
    }
    signal(SIGSEGV, oops);
    signal(SIGBUS, oops);
    signal(SIGTERM, oops);

#ifdef USE_NSPR
    systhread_init("netsite");
    _initialize_crits();
#endif
#ifdef NET_SSL
    /* Initialize security rng */
    SEC_Init();
    RNG_RNGInit();
    RNG_SystemInfoForRNG();

#ifdef CLIENT_AUTH
    CA_Initialize(NULL, NULL);
#endif /* CLIENT_AUTH */
#endif /* NET_SSL */

    InitializeExitVariable();

    /* Initialize the ACL stuff */
    if(ACL_InitHttp())
	exit(1);

    func_init(func_standard);
    admin_init(admdir);
    if( (err = ereport_init(master_error_log, NULL, 
                            (geteuid() ? NULL : user_pw), BUILD_NUM)))
    {
        fprintf(stderr, "error opening error log: %s\n", err);
        fprintf(stderr, "Error Log disabled!\n");
        /*exit(1);*/
    }

    /* initialize the string database using path of startup directory */
	resroot = (char *) malloc(strlen(nsroot) + 40);
	strcpy(resroot, nsroot);
	strcat(resroot, "/bin/admin/property");
    XP_InitStringDatabase(resroot,DATABASE_NAME);
    free(pathCWD);
	free(resroot);

#ifdef	INPROC_CRON
    cron_init();
#endif	/* INPROC_CRON */

    /* ACL stuff has things to initialize after the first init phase */
    if(ACL_InitHttpPostMagnus())
	exit(1);

    conf_vars2daemon(&d);
    d.rcback = magnus_restart;
    if(clport != -1)
        d.port = clport;

#if defined(AIX) && defined(THREAD_ANY)
    /* XXXrobm This is causing mysterious hangs in recv() */
    http_set_keepalive_timeout(0);
#endif

    /* XXXMB - 
     * We are about to ship the 2.0 product with the 2.0 browser.
     * Unforunately the 2.0 browser can't handle keepalives with
     * SSL- keepalive connections just timeout and it is really slow.
     * So, if we are in secure mode, there is a high likelihood that a 
     * 2.0 browser will be trying to connect (since that is what we ship with)
     *
     * For now, make sure that the keepalive timeout is set to 0 with
     * security on.
     */
    if (d.security)
       http_set_keepalive_timeout(0);
 
    d.version = BUILD_NUM;
    d.detach = !nodetach;
    d.child_callback = http_handle_session;
    d.admroot = STRDUP(admroot);
    daemon_run(&d);
    return 0;
}
