/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
//--------------------------------------------------------------------------//
//                                                                          //
//  Name: Administration Server Binary                                      //
//	Platforms: WIN32                                                        //
//  ......................................................................  //
//  HOW TO DEBUG                                                            //
//  AdminServer can run as an application (for debugging) or a service.     //
//  As a service, it gets the service name from the SCM in _ServiceMain.    //
//  When run as an application, it gets the service name from the command   //
//  line as the first argument.  Command line example:                      //
//  ns-admin.exe admin30-serv                                               //
//  ......................................................................  //
//  Revision History:                                                       //
//  01-12-94  Initial Version, Aruna Victor (aruna@netscape.com)            //
//  11-01-96  NSPR 2.0 changes, Mike Belshe (mbelshe@netscape.com)          //
//  12-02-96  3.0 registry changes, Andy Hakim (ahakim@netscape.com)        //
//--------------------------------------------------------------------------//
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <process.h>
#ifdef NSPR20
#include <prthread.h>
#endif

#include "netsite.h"
#include "base/eventlog.h"
#include "base/daemon.h"
#include "base/ereport.h"
#include "base/util.h"
#include "base/cinfo.h"
#include "base/systhr.h"

#include "frame/req.h"
#include "frame/func.h"
#include "frame/protocol.h"
#include "frame/conf.h"

#include "nt/regparms.h"
#define magnus_init HORRIBLE_KLUDGE
#include "nt/magnus.h"
#undef magnus_init
#include "nt/messages.h"

#include "ntcron.h"

//--------------------------------------------------------------------------//
// global variables                                                         //
//--------------------------------------------------------------------------//
HINSTANCE hInstance;
HANDLE hServDoneEvent = NULL;  // signalled to shut down server
SERVICE_STATUS_HANDLE gsshServiceStatus = 0L;

//--------------------------------------------------------------------------//
// external functions                                                       //
//--------------------------------------------------------------------------//
extern int magnus_init();
extern int threads_init();

extern VOID admin_getadmroot();	   // from adminit.c
extern VOID admin_setadmroot();

extern VOID ResumeHttpdService();  // from ntadmstart.c
extern VOID CleanupHttpdService();
extern VOID SuspendHttpdService();

extern BOOL kill_child_threads();     // from ntdaemon.c
extern VOID InitializeExitVariable();
extern VOID WaitForExitVariable();
 

//--------------------------------------------------------------------------//
// TODO: not used                                                           //
//--------------------------------------------------------------------------//
#ifdef NSPR20
DWORD WaitForStopEventThread(void *notused)
{
    DWORD dwWait;

    dwWait = WaitForSingleObject(hServDoneEvent, INFINITE);
 
    return dwWait;
}
#endif



//--------------------------------------------------------------------------//
// used by any thread to report an error, or stop the service
//--------------------------------------------------------------------------//
void StopHttpdService(LPTSTR lpszMsg, HANDLE Event)
{
	char szError[MAX_PATH];
	sprintf(szError, "Service Startup Error: %d", GetLastError());

	LogErrorEvent(NULL, EVENTLOG_ERROR_TYPE, 0, MSG_BAD_STARTUP, szError, lpszMsg);
	CleanupHttpdService();

    // Set a termination event to stop SERVICE MAIN FUNCTION.
    SetEvent(Event);
}


//--------------------------------------------------------------------------//
// wait for server to exit                                                  //
//--------------------------------------------------------------------------//
void _WaitSignalThread(void *unused)
{
    DWORD dwWait;
    dwWait = WaitForSingleObject (hServDoneEvent, INFINITE );
}


//--------------------------------------------------------------------------//
// get server config path (c:\netscape\server\httpd-nickname) from registry //
//--------------------------------------------------------------------------//
BOOL _GetServerConfig(char *szServiceName, char *szServerConfig, LPDWORD lpdwServerConfig)
{
	BOOL bReturn = FALSE;
	HANDLE hHttpdKey = 0;
	char szHttpdKey[MAX_PATH];
	DWORD dwValueType;
	DWORD dwResult = 0;

	// query registry key to figure out config directory
	wsprintf(szHttpdKey, "%s\\%s\\%s", KEY_SOFTWARE_NETSCAPE, ADM_KEY_ROOT, szServiceName);

	dwResult = RegOpenKey(HKEY_LOCAL_MACHINE, szHttpdKey, &hHttpdKey);
	if(dwResult == ERROR_SUCCESS) 
	{
	 	dwResult = RegQueryValueEx(hHttpdKey, VALUE_CONFIG_PATH, (LPDWORD)NULL, &dwValueType, (LPBYTE)szServerConfig, lpdwServerConfig);
		if(dwResult == ERROR_SUCCESS)
			bReturn = TRUE;
		RegCloseKey(hHttpdKey);
	}
	return(bReturn);
}



//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
BOOL _SetServiceStatus(DWORD dwCurrentState, DWORD dwError)
{
	BOOL bReturn = FALSE;
	SERVICE_STATUS ssStatus;

	if(gsshServiceStatus)
	{
		ssStatus.dwServiceType = SERVICE_WIN32_OWN_PROCESS;
		ssStatus.dwCurrentState = dwCurrentState;
	   //if(dwCurrentState == SERVICE_START_PENDING)
	   //   ssStatus.dwControlsAccepted = 0;
	   //else
			ssStatus.dwControlsAccepted = SERVICE_ACCEPT_STOP | SERVICE_ACCEPT_PAUSE_CONTINUE;
		ssStatus.dwWin32ExitCode = dwError;
		ssStatus.dwServiceSpecificExitCode = (NO_ERROR ? 0 : 1);
		ssStatus.dwCheckPoint = 0;
		ssStatus.dwWaitHint = (1000 * 5);
		bReturn = SetServiceStatus(gsshServiceStatus, &ssStatus);
	}
	return(FALSE);
}



//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
void WINAPI _ServiceHandler(DWORD fdwControl)
{
	switch(fdwControl)
	{
		case SERVICE_CONTROL_STOP:
		case SERVICE_CONTROL_SHUTDOWN:
			_SetServiceStatus(SERVICE_STOP_PENDING, 0);
            SetEvent(hServDoneEvent);
			return;

		case SERVICE_CONTROL_PAUSE:
        {
		    _SetServiceStatus(SERVICE_PAUSE_PENDING, 0);
		    SuspendHttpdService();
		    _SetServiceStatus(SERVICE_PAUSED, 0);
		    return;
        }

		case SERVICE_CONTROL_CONTINUE:
		{
			_SetServiceStatus(SERVICE_CONTINUE_PENDING, 0);
			ResumeHttpdService();
		    break;
		}

		case SERVICE_CONTROL_INTERROGATE:
			break;
	}
	_SetServiceStatus(SERVICE_RUNNING, 0);
}



//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
BOOL _CreateCronThread(HANDLE hevWatchDogExit)
{
	BOOL bReturn = FALSE;
	DWORD dwThreadID = 0;
	HANDLE hWindowThread = NULL;

	if(hWindowThread = CreateThread((LPSECURITY_ATTRIBUTES)NULL, 0, 
							 (LPTHREAD_START_ROUTINE)CRON_ThreadProc, (LPVOID)hevWatchDogExit, 0, &dwThreadID))
	{
		CLOSEHANDLE(hWindowThread);
		bReturn = TRUE;
	}
	return(bReturn);
}


//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
void _ServiceMain(DWORD dwArgc, LPTSTR *lpszArgv)
{
    HANDLE hEventLog = NULL;
    BOOL bReturn = FALSE;
    BOOL bIsService = (lpszArgv != NULL);  // if SCM calls us, lpszArgv != NULL
    DWORD dwTemp = 0;
    char szValue[MAX_PATH];
    PRThread *thread;

	if(bIsService)
	{
		gsshServiceStatus = RegisterServiceCtrlHandler(lpszArgv[0], 
									(LPHANDLER_FUNCTION)_ServiceHandler);
		if((gsshServiceStatus != (SERVICE_STATUS_HANDLE)NULL))
        {
            server_id = STRDUP(lpszArgv[0]);  // SCM provides ServiceID as 1st argument
        }
	}

    hEventLog = InitializeLogging(server_id);

	if(!(hServDoneEvent = OpenEvent(EVENT_ALL_ACCESS, FALSE, server_id)))  // is another copy running?
	{
        if(hServDoneEvent = CreateEvent(NULL, TRUE, FALSE, server_id))
        {
			_CreateCronThread(hServDoneEvent);

            // don't confuse admroot with the Server Root (c:\netscape\server\)
            // admroot actually points to Config directory (c:\netscape\server\admin-serv\config)
            dwTemp = sizeof(szValue);
            if(_GetServerConfig(server_id, szValue, &dwTemp))
            {
                char *admroot = STRDUP(szValue);
	        admin_setadmroot(admroot);
                systhread_init("netsite");
	            _SetServiceStatus(SERVICE_START_PENDING, 0);
	            if(!magnus_init())
                {
                    _SetServiceStatus(SERVICE_START_PENDING, 0);
                    if(!threads_init()) 
                    {
                        _SetServiceStatus(SERVICE_RUNNING, 0);
                        /* Putting the main thread to sleep by directly dropping into 
                         * WaitForSingleEvent() kills our user level threading system.  So 
                         * instead we spin off a thread to do the WaitForEvent() while we go to
                         * sleep on a condition variable (via PR_JoinThread).
                         */
                        thread = PR_CreateThread(PR_USER_THREAD,
                                                 _WaitSignalThread,
                                                 NULL,
                                                 PR_PRIORITY_NORMAL,
                                                 PR_GLOBAL_THREAD,
                                                 PR_JOINABLE_THREAD,
                                                 0);

                        PR_JoinThread(thread);
                        bReturn = TRUE;
                    }
                }
            }
        }
    }

    if(!bReturn)
        LogErrorEvent(NULL, EVENTLOG_ERROR_TYPE, 0, MSG_BAD_STARTUP, "Initialization Failed", "");

    TerminateLogging(hEventLog);
    _SetServiceStatus(SERVICE_STOPPED, 0);
    CLOSEHANDLE(hServDoneEvent);
    return;
}



//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
int PASCAL WinMain(HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR lpszCmdLine, int nShow)
{
    BOOL bReturn = FALSE;
    SERVICE_TABLE_ENTRY dispatchTable[] = {
        { TEXT(ADM_ID_SERVICE), // ignored
          (LPSERVICE_MAIN_FUNCTION)_ServiceMain },
        { NULL, NULL }
    };

	hInstance = hInst; // don't think we really need this -ahakim

    if(lpszCmdLine && *lpszCmdLine) // run as application
    {  
        // expect service name on command line, ex: ns-admin.exe admin30-serv
        server_id = STRDUP(lpszCmdLine);
        _ServiceMain(0, (LPTSTR *)NULL);
        bReturn = TRUE;
    }
    else  // run as service
    {
        bReturn = StartServiceCtrlDispatcher(dispatchTable);
    }
    return(bReturn);
}
