/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * adminit.c: Initialization routines for the admin server
 *
 * Rob McCool
 *
 */

#include "prtypes.h"
#include "prthread.h"

#include "netsite.h"
/* ADAM-BUILD */
/* #include "sec.h" */
#include "base/daemon.h"
#include "base/util.h"
#include "frame/conf.h"

#ifdef XP_UNIX
#include <sys/types.h>
#include <unistd.h>
#include <pwd.h>
#else /* XP_WIN32 */
#include <windows.h>
#include <nt/messages.h>
#include <nt/magnus.h>
#include <nt/regparms.h>
#include <base/daemon.h>
#include <base/eventlog.h>
#include <base/ereport.h>
#include <base/systhr.h>
#include <frame/func.h>
#include <frame/http.h>
#include <safs/acl.h>
#include <safs/clauth.h>
#include <process.h>

#define ADMIN_MAXTHREADS 8

extern struct FuncStruct func_standard[];

extern int admin_init(char *cwd);

extern VOID admin_getadmroot();	   // from adminit.c
extern VOID admin_setadmroot();

void magnus_restart(int);

/* Variables defined in ntadmmain.c */

SYS_THREAD MainThread;

SC_HANDLE schSCManager;
SC_HANDLE schService;
static struct rsf {
    void (*fn)(void *);
    void *data;
    struct rsf *next;
} *rlist;

#endif /* XP_WIN32 */

#include "../lib/libsi18n/gsadmserv.h"
extern char *nsroot;
extern void XP_InitStringDatabase(char* pathCWD, char* databaseName);

NSAPI_PUBLIC int magnus_init()
{
    CHAR ErrMsg[1024];
    char *admdir, *err;
    char *admroot;
    char *resroot;

    SYS_THREAD Thread;

    func_init(func_standard);
    admdir = NULL;

    admin_getadmroot(&admroot);
    if (!SetCurrentDirectory(admroot)) {
        sprintf(ErrMsg, "%s Startup Error : Could not open error log: %d",
        	ADM_NAME_SERVICE, GetLastError());
		LogErrorEvent(NULL, EVENTLOG_ERROR_TYPE, 0,
			MSG_BAD_STARTUP, ErrMsg, NULL);
		exit(1);
    }

    SEC_Init();
/* ADAM-BUILD */
/*     SEC_RNGInit(); */
/*     SEC_SystemInfoForRNG(); */

    RNG_RNGInit();
    RNG_SystemInfoForRNG();

#ifdef CLIENT_AUTH
    CA_Initialize(NULL, NULL);
#endif

    InitializeExitVariable();
    if(ACL_InitHttp())
	exit(1);

    (VOID)admin_init(admroot);

    /* Initialize resource database or properties */
    resroot = (char *) malloc (strlen(nsroot) + 40);
    strcpy(resroot, nsroot);
    strcat(resroot, "/bin/admin/property");
    XP_InitStringDatabase(resroot, DATABASE_NAME);

    /* Initialize the logbuf thread */

    if (logbufthread_init() == PR_FALSE)
        ereport(LOG_CATASTROPHE, "Logbuf Thread Init failed.\n");

    if(ACL_InitHttpPostMagnus())
	exit(1);


	if(err = ereport_init(master_error_log, NULL, NULL, BUILD_NUM)) {
        sprintf(ErrMsg, "%s Startup Error : Could not open error log: %s",
        	ADM_NAME_SERVICE, err);
		LogErrorEvent(NULL, EVENTLOG_ERROR_TYPE, 0,
			MSG_BAD_STARTUP, ErrMsg, NULL);
		exit(1);
    }
	return 0;
}


NSAPI_PUBLIC int threads_init()
{
	CHAR ErrMsg[512];
    int clport;
    daemon_s *d;
   char *admroot;
	clport = -1;

	d = (daemon_s *)MALLOC(sizeof(daemon_s));

    conf_vars2daemon(d);
    d->rcback = magnus_restart;
	d->child_callback = protocol_handle_session;
	d->rotate_callback = NULL;
	d->servid = strdup(ADM_ID_SERVICE);
    d->maxthreads = ADMIN_MAXTHREADS;

    admin_getadmroot(&admroot);
    d->admroot = STRDUP(admroot);
    MainThread = PR_CreateThread(PR_USER_THREAD,
                                 daemon_run,
                                 (void *)d,
                                 PR_PRIORITY_NORMAL,
                                 PR_GLOBAL_THREAD,
                                 PR_UNJOINABLE_THREAD,
                                 65536);
    return (MainThread == NULL)?-1:0;
}

NSAPI_PUBLIC void magnus_atrestart(void (*fn)(void *), void *data)
{
    struct rsf *r;

    r = (struct rsf *) MALLOC(sizeof(struct rsf));
    r->fn = fn;
    r->data = data;
    r->next = rlist;
    rlist = r;
}

NSAPI_PUBLIC void magnus_restart(int fs)
{
    struct rsf *r = rlist, *s;
    SYS_FILE fd = ereport_getfd();

    while(r) {
        (*r->fn)(r->data);
        s = r;
        r = r->next;
        FREE(s);
    }
    rlist = NULL;


    conf_terminate();
    objset_free(std_os);

	if (MainThread != NULL) {
		TerminateThread(MainThread, 0);
		MainThread = NULL;
	}

    if(magnus_init())
        exit(1);
}


NSAPI_PUBLIC VOID ResumeHttpdService()
{
	magnus_restart(0);
	return;
}

NSAPI_PUBLIC VOID CleanupHttpdService()
{
	MainThread = NULL;
	WSACleanup();
	ereport_terminate();
	return;
}

NSAPI_PUBLIC VOID SuspendHttpdService()
{
    DWORD Result;

    daemon_terminate();

    /* Set the MainThread's Monitor so we will know when it
     * goes away
     */
    WaitForExitVariable();

    CleanupHttpdService();
    return;
}

#include <nt/nsapi.h>
#include <libadmin/libadmin.h>
#include <libadmin/backup.h>
#include <libadmin/cluster.h>
/* prodinfo is removed
#include <libadminutil/prodinfo.h>
*/
#include <libadminutil/admutil.h>
#include <libadminutil/psetc.h>
#include <libadminutil/distadm.h>
#include <libadmin/install.h>
#include <libadmin/dstats.h>
#include <libaccess/nsadb.h>

VOID NsapiDummy()
{	
	int i = 0;
	SafTable = (SafFunction **)MALLOC(400 * sizeof(VOID*));

    /* Force references to libadmin */

	/* Functions from libadmin:objconf.c */
	SafTable[i++] = (SafFunction *)get_mag_init;
	/* Functions from libadmin:form_post.c */
	SafTable[i++] = (SafFunction *)post_begin;
	/* Functions from libadmin:form_get.c */
	SafTable[i++] = (SafFunction *)open_html_file;
	/* Functions from libadmin:userdb.c */
	/* SafTable[i++] = (SafFunction *)detect_db_type; */
	/* Functions from libadmin:authdb.c */
	SafTable[i++] = (SafFunction *)list_auth_dbs;
	/* Functions from libadmin:error.c */
	SafTable[i++] = (SafFunction *)report_error;
	/* Functions from libadmin:template.c */
	SafTable[i++] = (SafFunction *)directive_is;
	/* Functions from libaccess:pcontrol.c */
	SafTable[i++] = (SafFunction *)restart_http;
	/* Functions from libadmin:admserv.c */
	SafTable[i++] = (SafFunction *)read_server_lst;
	/* Functions from libadmin:multconf.c */
	SafTable[i++] = (SafFunction *)make_conflist;
	/* Functions from libadmin:install.c */
	SafTable[i++] = (SafFunction *)myprintf;
	/* Functions from libadmin:password */
	SafTable[i++] = (SafFunction *)pw_enc;
	/* Functions from libadmin:cron_conf.c */
	SafTable[i++] = (SafFunction *)cron_conf_create_obj;
	/* Functions from libadmin:dstats.c */
	SafTable[i++] = (SafFunction *)dstats_open;
	/* Functions from libadmin:backup.c */
	SafTable[i++] = (SafFunction *)ADM_bk_initTree;
	/* Functions from libadmin:httpcon.c */
	SafTable[i++] = (SafFunction *)make_http_request;
	/* Functions from libadmin:cluster.c */
	SafTable[i++] = (SafFunction *)CL_rexec;
	/* Functions from libadmin:prodinfo.c, prodinfo.c is removed */
	/* SafTable[i++] = (SafFunction *)ADM_pi_init; */
	/* Functions from libadmin:admutil.c */
	SafTable[i++] = (SafFunction *)createAttrNameList;
	/* Functions from libadmin:psetc.c */
	SafTable[i++] = (SafFunction *)psetCreate;
	/* Functions from libadmin:distadm.c */
	SafTable[i++] = (SafFunction *)ADMUTIL_Init;
}


