/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/************************************************************/
/*                                                          */
/* AdmservPlugin.h : defines for the admin server plugin    */
/*                                                          */
/* Adam Prishtina 1/13/00                                   */
/*                                                          */
/************************************************************/

#ifndef _ADMSERV_PLUGIN_H
#define _ADMSERV_PLUGIN_H

#include <base/session.h>
#include <frame/req.h>

#include <ldap.h>
#include <ldap_ssl.h>

#include "hashtable.h"

typedef unsigned long       DWORD;
typedef DWORD               *PDWORD;
typedef DWORD               *LPDWORD;

#define LDAPU_SUCCESS 0

#ifdef XP_UNIX
#   define STRNCASECMP strncasecmp
#   define STRCASECMP  strcasecmp
#   define STRCMP      strcmp
#else /* XP_WIN32 */
#   define STRNCASECMP _strnicmp
#   define STRCASECMP  _stricmp
#   define STRCMP      _stricmp
#endif

#ifndef FILE_PATHSEP
#define FILE_PATHSEP '/'
#endif

#ifndef PATH_MAX
#define PATH_MAX 256
#endif

#ifndef LINE_LENGTH
#define LINE_LENGTH 1024
#endif

#ifndef ACCESS_DENIED
#define ACCESS_DENIED 0
#endif

#ifndef ACCESS_GRANTED
#define ACCESS_GRANTED 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE 1
#endif

#define ADMIN_HASH_SIZE 		2
#define LDAP_BASE_PREFIX 		(char*)"ldap://"
#define LDAPS_BASE_PREFIX 		(char*)"ldaps://"
#define LDAP_BASE_PREFIX_LENGTH		(strlen(LDAP_BASE_PREFIX))
#define LDAPS_BASE_PREFIX_LENGTH	(strlen(LDAPS_BASE_PREFIX))
#define LDAP_PREFIX        		(char*)"ldap:///"
#define LDAP_PREFIX_LENGTH 		(strlen(LDAP_PREFIX))
#define ADMIN_SERVER_ID 		(char*)"admin-serv"
#define TASK_IDENTIFIER 		(char*)"tasks/"
#define LOCAL_SUPER_NAME 		(char*)"LocalSuper"
#define AUTHENTICATION_LDAP_URL 	(char*)"AuthenticationLdapURL"
#define RUNTIME_RESYNC_COMMAND 		(char*)"sync-task-sie-data"
#define CHANGE_SIEPWD_COMMAND		(char*)"change-sie-password"

#define RUNTIME_COMMAND_CONTENT_TYPE 	(char*)"admin-internal/command"
#define DOWNLOAD_CONTENT_TYPE		(char*)"application/octet-stream"
#define SERVLET_CONTENT_TYPE		(char*)"magnus-internal/servlet"

typedef struct RebindData {
   char *user;
   char *pw;
} RebindData;

typedef struct UserCacheEntry {
   char *userDN;
   char *userPW;
   char *ldapURL;
   long  createTime;
} UserCacheEntry;

typedef struct TaskCacheEntry {
   char      *execRef;
   char      *execRefArgs;
   int        logSuppress;
   HashTable *auth_userDNs;
} TaskCacheEntry;

typedef struct PopulateTasksData {
   LDAP *server;
   char *userDN;
   long  now;
} PopulateTasksData;

typedef struct LdapServerData {
   DWORD dPad;
   char *host;
   int   port;
   int 	 secure;  /* track whether the server is running in secure mode */
   char *baseDN;
   char *bindDN;
   char *bindPW;
} LdapServerData;

typedef struct ServletLookupData {
   char *nsExecRef;
   char *taskcn;
   char *serverDN;
   char *serverID;
} ServletLookupData;

typedef int (*RuntimeCommandFn)(char *, char *, void *, Request*);

typedef struct RuntimeCommandRecord {
   RuntimeCommandFn fn;
   void            *arg;
} RuntimeCommandRecord;

extern int task_eval_init(char *configRoot, long cache_lifetime);

extern void task_update_registry_server_bindpw(char *password);
extern int task_register_server(char *serverid, char *sieDN);

extern int admin40_runtime_command_init();
extern int admin40_register_runtime_command(char *name, RuntimeCommandFn fn, void *arg);
extern int admin40_runtime_command_exec(char *name, char *query, Request *rq);

extern int admin40_error_auth(Session *sn, Request *rq, char *reason);
extern int admin40_error_std(Session *sn, Request *rq, char *reason);
extern int admin40_error(Session *sn, Request *rq, int code, char *reason);

extern int change_sie_password(char *name, char *query, void* arg, Request* rq);

/*
 * Exported NSAPI functions
 */
#ifdef __cplusplus
extern "C" {
#endif
NSAPI_PUBLIC int admin40_task_eval_init(pblock *pb, Session *sn, Request *rq);
NSAPI_PUBLIC int admin40_task_access_eval(pblock *pb, Session *sn, Request *rq);
NSAPI_PUBLIC int admin40_host_ip_check(pblock *pb, Session *sn, Request *rq);
NSAPI_PUBLIC int admin40_host_ip_init(pblock *pb, Session *sn, Request *rq);
NSAPI_PUBLIC int admin40_authenticate(pblock *pb, Session *sn, Request *rq);
NSAPI_PUBLIC int admin40_admin40_isdiropen(pblock *pb, Session *sn, Request *rq);
NSAPI_PUBLIC int admin40_uri2path(pblock *pb, Session *sn, Request *rq);
NSAPI_PUBLIC int admin40_runtime_commands(pblock *pb, Session *sn, Request *rq);
NSAPI_PUBLIC int admin40_flex_log(pblock *pb, Session *sn, Request *rq);
#ifdef __cplusplus
}
#endif

#endif
