/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
//--------------------------------------------------------------------------//
//                                                                          //
//  Name: startconsole.c                                                    //
//  Starts java vm and console.                                             //
//  Platforms: WIN32                                                        //
//  Revision History:                                                       //
//--------------------------------------------------------------------------//
#include <windows.h>
#include <stdio.h>
#include "resource.h"
#include "nt/regparms.h"

#define VERSION                "7.0"
#define PRODUCT_NAME           "Fedora Management Console " VERSION

char szJavaHome[MAX_PATH];
char szUserJavaHome[MAX_PATH];
char szVersion[MAX_PATH];
char szMicroVersion[MAX_PATH];
char szUserJREexe[MAX_PATH];
char szJREexe[MAX_PATH];


static BOOL fileExists(LPCSTR pszFileName)
{
    OFSTRUCT ofs;

    if(OpenFile(pszFileName, &ofs, OF_EXIST) != HFILE_ERROR)
    {
        return TRUE;
    }
    return FALSE;
}

static char *getServerRoot()
{

    static char szRoot[MAX_PATH];
    char szKey[MAX_PATH];
    HKEY hKey;

    // HKLM\Software\Netscape\Administration\Version : RootPath
    wsprintf(szKey, KEY_SOFTWARE_NETSCAPE "\\" ADM_KEY_ROOT);
    if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, szKey, 0, KEY_QUERY_VALUE, &hKey) == ERROR_SUCCESS)
    {
        DWORD dwSize = sizeof(szRoot);
        if (RegQueryValueEx(hKey, VALUE_ROOT_PATH, 0, NULL, (unsigned char *)szRoot, &dwSize) != ERROR_SUCCESS)
        {
            GetCurrentDirectory(sizeof(szRoot), szRoot);
        }
        RegCloseKey(hKey);
    }
    else
    {
        GetCurrentDirectory(sizeof(szRoot), szRoot);
    }
    return szRoot;
}
   

static int getProductInfoString(const char* pszKey, char* pszValue, int cbValue, const char* pszFileName)
{
    FILE *fp;
    char szText[MAX_PATH];
    int nReturn = -1;

    pszValue[0] = '\0';

    if ((fp = fopen(pszFileName, "r")) != NULL)
    {
        while (fgets(szText, sizeof(szText), fp) != NULL)
        {
            char *pszToken;
            pszToken = strtok(szText, " : \t\n");
            if (pszToken && strcmp(pszToken, pszKey) == 0)
            {
                if ((pszToken = strtok(NULL, "\n")) != NULL)
                {
                    while(*pszToken == 0x20) pszToken++; // get rid of leading spaces
                    strncpy(pszValue, pszToken, cbValue);
                    pszValue[cbValue-1] = '\0'; //
                    nReturn = strlen(pszValue);
                }
                else
                {
                    nReturn = 0;
                }
                break;
            }
        }
        fclose(fp);
    }
    return nReturn;
}



char *getURL()
{
    char szFileName[MAX_PATH];
    char szValue[MAX_PATH];
    static char szURL[MAX_PATH];

    wsprintf(szFileName, "%s\\admin-serv\\config\\adm.conf", getServerRoot());
    if(fileExists(szFileName))
    {
      wsprintf(szURL, "-A http://");  // 353403
        if (getProductInfoString("security", szValue, MAX_PATH, szFileName) > 0)
        {
            if(stricmp(szValue, "on") == 0)
            {
                wsprintf(szURL, "-A https://");  // 353403
            }
        }

        if (getProductInfoString("host", szValue, MAX_PATH, szFileName) > 0)
        {
            strcat(szURL, szValue);
            if (getProductInfoString("port", szValue, MAX_PATH, szFileName) > 0)
            {
                strcat(szURL, ":");
                strcat(szURL, szValue);
                return _strdup(szURL);
            }
        }
    }
    return NULL;
}



char* getClassPath(const char *szServerRoot)
{
    WIN32_FIND_DATA FindFileData;
    LPWIN32_FIND_DATA lpFindFileData = &FindFileData;
    HANDLE hFile;
    static char szClassPath[MAX_PATH * 6];
    char szJava[MAX_PATH];
    char szFileSpec[MAX_PATH];

    strcpy(szJava, szServerRoot);
    strcat(szJava, "\\java");

    wsprintf(szFileSpec, "%s\\*.jar", szJava);
    hFile = FindFirstFile(szFileSpec, lpFindFileData);
    if(hFile != INVALID_HANDLE_VALUE)
    {
        do
        {
            char *szFileName = lpFindFileData->cFileName;
            strlwr(szFileName);
            strcat(szClassPath, szJava);
            strcat(szClassPath, "\\");
            strcat(szClassPath, szFileName);
            strcat(szClassPath, ";");
        } 
        while(FindNextFile(hFile, lpFindFileData));
        FindClose(hFile);
    }

    wsprintf(szFileSpec, "%s\\*.zip", szJava);
    hFile = FindFirstFile(szFileSpec, lpFindFileData);
    if(hFile != INVALID_HANDLE_VALUE)
    {
        do
        {
            char *szFileName = lpFindFileData->cFileName;
            strlwr(szFileName);
            strcat(szClassPath, szJava);
            strcat(szClassPath, "\\");
            strcat(szClassPath, szFileName);
            strcat(szClassPath, ";");
        } 
        while(FindNextFile(hFile, lpFindFileData));
        FindClose(hFile);
    }
    return szClassPath;
}

static BOOL checkUserJRE(char *szVersion, char *szMicroVersion)
{
    BOOL bReturn = FALSE;
    HKEY hKey;
    char szKey[MAX_PATH];

    wsprintf(szKey, "SOFTWARE\\JavaSoft\\Java Runtime Environment\\%s", szVersion);
    if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, szKey, 0, KEY_ALL_ACCESS, &hKey) == ERROR_SUCCESS)
    {
        DWORD dwSize = sizeof(szUserJavaHome);

        if (RegQueryValueEx(hKey, "JavaHome", 0, NULL, (unsigned char *) szUserJavaHome, &dwSize) == ERROR_SUCCESS)
        {

            wsprintf(szUserJREexe, "%s\\bin\\java.exe", szUserJavaHome);
            bReturn = fileExists(szUserJREexe);
        }
        RegCloseKey(hKey);
    }
    return (bReturn);
}


static void checkJRE(char *szServerRoot)
{
    BOOL bInit = TRUE;
    char szPath[MAX_PATH];


    wsprintf(szPath, "%s\\java\\jre.inf", szServerRoot);
    if(fileExists(szPath))
    {
        char szValue[MAX_PATH];
        if (getProductInfoString("Version", szValue, MAX_PATH, szPath) > 0)
        {
            wsprintf(szVersion, szValue);
        }
        else 
            bInit = FALSE;

        if (getProductInfoString("MicroVersion", szValue, MAX_PATH, szPath) > 0)
        {
            wsprintf(szMicroVersion, szValue);
        }
        else 
            bInit = FALSE;
    }
    else
    {
        bInit = FALSE;
    }

    if(bInit == FALSE)
    {
      char szFormat[MAX_PATH];
        char szMessage[MAX_PATH];
        LoadString(NULL, IDS_JRE_INF_BAD, szFormat, sizeof(szFormat));
      sprintf(szMessage, szFormat, szPath);
        MessageBox(NULL, szMessage, NULL, MB_OK|MB_ICONEXCLAMATION);
        exit(1);
    }

    // Check if JRE is installed in the server root
    wsprintf(szPath, "%s\\bin\\base\\jre\\bin\\java.exe", szServerRoot);
    if (fileExists(szPath))
    {
          strcpy(szJREexe, szPath); //set global jre.exe path
          wsprintf(szJavaHome, "JavaHome=%s\\bin\\base\\jre", szServerRoot);
          _putenv(szJavaHome);
    }
    // Check in the windows registry if the user has the right JRE version installed
    else if (checkUserJRE(szVersion, szMicroVersion))
    {
          strcpy(szJREexe, szUserJREexe); //set global jre.exe path
          wsprintf(szJavaHome, "JavaHome=%s", szUserJavaHome);
          _putenv(szJavaHome);
    }
    // Then the user must have the JRE version in the path
    else
    {
        char szText[MAX_PATH];
        char szMessage[MAX_PATH];
        char szTitle[MAX_PATH];

        LoadString(NULL, IDS_INSTALL_JRE_TITLE, szTitle, sizeof(szTitle));
        LoadString(NULL, IDS_INSTALL_JRE, szText, sizeof(szText));
        wsprintf(szMessage, szText, szVersion, szMicroVersion, szVersion, szMicroVersion);
        if(MessageBox(NULL, szMessage, szTitle, MB_YESNO | MB_DEFBUTTON1 | MB_ICONEXCLAMATION) != IDYES)
        {
            exit(1);
        }
    }
}

void showDebugInfo(char *cmd) {
    char buf[2048];
    sprintf(buf, "JavaHome=%s\n\n%s", getenv("JavaHome"), cmd);
    MessageBox(NULL, buf, "startconsole Debug Information", MB_OK);
}


static BOOL startConsole(char *szUserCommandLine)
{
    char szJRECommandLine[MAX_PATH * 7];
    char szUseJit[MAX_PATH];
    char *szServerRoot;
    char *szJavaHome;
    char *szClassPath;
    char *szToken;
    char *szTemp;
    char szJavaLibPath[MAX_PATH * 5];

    BOOL bDebugWindow = FALSE;
    BOOL bShowHelp = FALSE;
    BOOL iDebugLevel = -1;
    STARTUPINFO si;
    PROCESS_INFORMATION pi;
    BOOL bSuccess = FALSE;
    DWORD dwCreationFlags;

    strcpy(szJREexe, "java.exe");
    szServerRoot = getServerRoot();
    wsprintf(szUseJit, "-Djava.compiler=NONE");
    szJavaHome = getenv("JavaHome");

    checkJRE(szServerRoot);
    szClassPath = getClassPath(szServerRoot);
    if(getenv("CLASSPATH"))
    {
        char szNewClassPath[MAX_PATH * 5];
        wsprintf(szNewClassPath, "CLASSPATH=%s%s", szClassPath, getenv("CLASSPATH"));
        _putenv(szNewClassPath);
    }
    else
    {
        char szNewClassPath[MAX_PATH * 5];
        wsprintf(szNewClassPath, "CLASSPATH=%s", szClassPath);
        _putenv(szNewClassPath);
    }

    szTemp = strdup(szUserCommandLine);
    szToken = strtok(szTemp, " ");
    while(szToken)
    {
        if(strcmp(szToken, "-h") == 0)
        {
			bShowHelp = TRUE;
        }
        else
        if(strcmp(szToken, "-D") == 0)
        {
            bDebugWindow = TRUE;
            szToken = strtok(NULL, " ");
            if (szToken &&  isdigit(szToken[0])) {
                iDebugLevel = atoi(szToken);
            }
        }
        else if(strcmp(szToken, "-env") == 0)
        {
            char szTemp[MAX_PATH];
            bDebugWindow = TRUE;
            strcpy(szTemp, szJREexe);
            wsprintf(szJREexe, "%s /k %s", getenv("ComSpec"), szTemp);
        }
        else if(strcmp(szToken, "-jit") == 0)
        {
            wsprintf(szUseJit,""); // if -nojit does not exist, then JIT is on
        }
        else if(strcmp(szToken, "-nojit") == 0)
        {
            wsprintf(szUseJit, "-Djava.compiler=NONE"); // disable jit
        }
        szToken = strtok(NULL, " ");
    }

    wsprintf(szJavaLibPath, "-Djava.library.path=%s\\lib\\jss", szServerRoot);
	SetEnvironmentVariable("NSPR_NATIVE_THREADS_ONLY", "1");  // workaround for NSPR412/NSS33/JSS311 freeze -ahakim 11/18/01

    wsprintf(szJRECommandLine, "%s %s -ms8m -mx64m %s -classpath \"%s\" com.netscape.management.client.console.Console %s ", szJREexe, szUseJit, szJavaLibPath ,szClassPath, szUserCommandLine);
    strcat(szServerRoot, "\\java");
    memset(&si,0,sizeof(si));
    si.cb = sizeof(si);
    GetStartupInfo(&si);
    si.wShowWindow = SW_SHOWDEFAULT;
    if(bDebugWindow)
    {
        si.lpTitle = PRODUCT_NAME " Debug Information";
    }
    else
    {
        si.lpTitle = PRODUCT_NAME;
    }

    if (bDebugWindow || bShowHelp)
    {
        dwCreationFlags = NORMAL_PRIORITY_CLASS;
        if (iDebugLevel == 9)
        {
            showDebugInfo(szJRECommandLine);
        }
    }
    else
    {
        dwCreationFlags = NORMAL_PRIORITY_CLASS | DETACHED_PROCESS;

    }

    bSuccess = CreateProcess(NULL, szJRECommandLine, NULL, NULL, FALSE, dwCreationFlags, NULL, szServerRoot, &si, &pi);

    return bSuccess;
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine,  int nCmdShow)
{
    char szCommandLine[MAX_PATH];
   char *szUrl = NULL;

   SetCurrentDirectory(getServerRoot());

   szUrl = getURL();
   if (szUrl)
   {
      wsprintf(szCommandLine, "%s %s", lpCmdLine, szUrl);
      free(szUrl);
   }
   else
   {
      wsprintf(szCommandLine, "%s", lpCmdLine);
   }
    
    return startConsole(szCommandLine);
}
 
