/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/* 
 * SCRIPT_NAME: Handle the user environment version of the frames interface
 * 
 * All blame to Mike McCool (mlm@netscape.com)
 */

 
/**
 *	issues:
 * 1. need to add NLS support in dbtuserforms.h
 */ 

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "../lib/libsi18n/gsadmserv.h"
#include <ldap.h>
#include "ldap_ssl.h"
#include "base/shexp.h"
#include "libadmin/libadmin.h"
#ifdef XP_cplusplus
extern "C" {
#endif
#include "libadminutil/admutil.h"
#include "libadminutil/distadm.h"
#include "libadmsslutil/admsslutil.h"
#include "libadmsslutil/psetcssl.h"

#ifdef XP_cplusplus
}
#endif
#include "dbtuserforms.h"

#define SCRIPT_NAME "enduser"
#define TOP_NAME "tabs"
#define BOTTOM_NAME "category"
#define OPTIONS_NAME "options"
#define CONTENT_NAME "content"
#define DS_GLOBAL_PREF_RDN "ou=Global Preferences"
#define MEDIUM_STR 512
#define SMALL_STR 32

#define CATEGORY "--Category:"
#define OPTION "--Option:"
#define ICON    "--Icon:"
#define INCLUDE "--include"

#define CATLIST_FILE "../html/$$LANGDIR/index.lst"
#define HTML_FILE    "../html/$$LANGDIR/index.html"

/* Note: WinNT does not require this special treatment */
#ifdef XP_UNIX
#define INCLUDE_PATH "%s/bin/user/admin/html/$$LANGDIR/%s"
#else
#define INCLUDE_PATH "%s\\bin\\user\\admin\\html\\$$LANGDIR\\%s"
#endif

#define LIST_SIZE 5

#define GFX_DIR "../icons/"

typedef struct catstruct {
    char *category_id;
    char *category_desc;
    char **optionlist;
} catstruct, *catptr, **catlist;

typedef struct BindInfoStruct {
    char *binddn;
    char *bindpwd;
} BindInfoStruct;

catptr new_catstruct(void)
{
    catptr ans;
    ans = (catptr) malloc(sizeof(catstruct)+4);
    ans->category_id = NULL;
    ans->category_desc = NULL;
    ans->optionlist = NULL;
    return ans;
}

static int LDAP_CALL LDAP_CALLBACK
rebind_proc (LDAP *ld, char **dnp, char **passwdp,
                   int *authmethodp, int freeit, void *arg)
{
  if (freeit == 0) {
    BindInfoStruct *bindInfo = (BindInfoStruct *)arg;
    *dnp = bindInfo->binddn;
    *passwdp = bindInfo->bindpwd;
    *authmethodp = LDAP_AUTH_SIMPLE;
  }
  return LDAP_SUCCESS;
}

void report_err(char *msg) {
	fprintf(stdout, "<hr><FONT size=+1><center>%s</center></FONT><hr>", msg);
	exit(1);
}

/* 	atulb : 6/24/98
	Don't worry about preferred language support.  It's out of C LDAPSDK3.0. 
	Logic -> Create a comma separated list out of the env var 
	HTTP_ACCEPT_LANGUAGE.  Run through the list of attributes returned in the 
	search.  For each, look for the lang- tag if found, see if it lies in the 
	env var list.  If it does, then we can return corresponding value.  If 
	none matches, then return the 'en' lang tag value if found; else return 
	value without language tag.  If that is absent too return NULL		*/
char **getAttributeValue(LDAP *ld, LDAPMessage *e) {
   char langList[MEDIUM_STR];
   char *a, *ptr, *ptr2;
	int noLangAttr = 0;
	int enLangAttr = 0;
	char defaultNoLang[MEDIUM_STR];
	char defaultEnLang[MEDIUM_STR];
   char langTag[SMALL_STR];
	BerElement *ber;

   if (getenv("HTTP_ACCEPT_LANGUAGE") != NULL)	{
   	strcpy(langList, getenv("HTTP_ACCEPT_LANGUAGE"));
   }
   else {
   	strcpy(langList, "en");
   }

/*  Replace spaces with commas */
   for(ptr=langList; *ptr=='\0'; ptr++)
	   if ((*ptr == ' ') || (*ptr == '\t'))
			*ptr = ',';

	for(a = ldap_first_attribute(ld, e, &ber); a!=NULL;
				a = ldap_next_attribute(ld, e, ber)) {
		if (((ptr = strstr(a, ";lang-en")) != NULL) && (!enLangAttr)) {
			enLangAttr = 1;
			strcpy(defaultEnLang, a);
		}
		
		if ((ptr = strstr(a, ";lang-")) == NULL) {
			noLangAttr = 1;
			strcpy(defaultNoLang, a);
			continue;
		}
		else {
			ptr = ptr + strlen(";lang-");
			if ((ptr2 = strchr(ptr, ';')) != NULL)
				*ptr2 = '\0';
			sprintf(langTag, "%s", ptr);
			if (strstr(langList, langTag)) {
				ldap_memfree(a);
				ldap_ber_free(ber, 0);
				return ldap_get_values(ld, e, a);
			}
		}
	}
	ldap_memfree(a);
	ldap_ber_free(ber, 0);
	if (enLangAttr)
		return ldap_get_values(ld, e, defaultEnLang);
	if (noLangAttr)
		return ldap_get_values(ld, e, defaultNoLang);
	return NULL;
} 

catlist parseIndex(char **vals, int *count)
{
	int listsize=LIST_SIZE, current=0;
	char *tmp=NULL;
	char *data=NULL;
	char *div=NULL;
	int i=0;
	catlist ans=NULL;
	int item_found=1;
	catptr entry=NULL;
	int numoptions=LIST_SIZE, curopt=0;

	for (i=0;vals[i]!=NULL;i++)
	{
		 if((vals[i][0] == ';') || (vals[i][0] == '#'))
			  continue;
		 else if((vals[i][0] == '\n') || (vals[i][0] == '\r'))
			  continue;
		 else if(!strncmp(vals[i], CATEGORY, strlen(CATEGORY)))  {
			  tmp=strchr(vals[i], '\n');
			  if(tmp) *tmp='\0';
			  tmp=strchr(vals[i], '\r');
			  if(tmp) *tmp='\0';

			  data=strchr(vals[i], ':');
			  if(!data) continue; /* huh?! */
			  data++;
			  div=strchr(data, ',');
			  if(!div) continue; /* huh?! */
			  *div++ = '\0';

			  if(!ans)  {
					ans = (catlist) MALLOC(listsize*sizeof(catptr)+4);
			  }  else  {
					/* If the last item didn't have any valid options in it, and
					 * this is not the first entry, blast the last category. */
					if(!item_found)  {
						 ans[--current] = NULL;
					}
			  }
			  if(listsize == current)  {
					listsize+=LIST_SIZE;
					ans = (catlist) REALLOC(ans, listsize*sizeof(catptr)+4);
			  }
			  entry=new_catstruct();
			  entry->category_id = STRDUP(data);
			  entry->category_desc = STRDUP(div);

			  ans[current++]=entry;
			  ans[current]=NULL;
			  item_found = 0;
		 }
#ifdef NEVER
		 else if(!strncmp(vals[i], TABICON, strlen(TABICON)))  {
			  tmp=strchr(vals[i], '\n');
			  if(tmp) *tmp='\0';
			  tmp=strchr(vals[i], '\r');
			  if(tmp) *tmp='\0';

			  data=strchr(vals[i], ':');
			  if(!data) continue;
			  data++;
			  entry->tabicon = STRDUP(data);
		 }
		 else if(!strncmp(vals[i], ICON, strlen(ICON)))  {
			  tmp=strchr(vals[i], '\n');
			  if(tmp) *tmp='\0';
			  tmp=strchr(vals[i], '\r');
			  if(tmp) *tmp='\0';

			  data=strchr(vals[i], ':');
			  if(!data) continue;
			  data++;
			  entry->icon = STRDUP(data);
		 }
#endif
		 else if(!strncmp(vals[i], OPTION, strlen(OPTION)))  {
			  tmp=strchr(vals[i], '\n');
			  if(tmp) *tmp='\0';
			  tmp=strchr(vals[i], '\r');
			  if(tmp) *tmp='\0';

			  data=strchr(vals[i], ':');
			  if(!data) continue; /* huh?! */
			  data++;

			  /* Check to make sure the user has permission to see this item. */
#ifdef NEVER
			  if(perms)  {
					char *pgm = STRDUP(data);
					char *cm  = strchr(pgm, ',');
					char *qm  = strchr(pgm, '?');
					int matched;

					if(cm)  {
						 *cm++ = '\0';

						 /* The delegation stuff ignores the query string -
						  * it just uses the program name.  Strip off the
						  * query string before making the check. */
						 if(qm)  {
							  *qm++ = '\0';
						 }
						 matched = WILDPAT_CMP(pgm, perms);
						 if(matched)  {
							  /* Either the comparison failed or the program didn't 
								* match.  Ignore this option. */
							  continue;
						 }

						 /* There's an actual program item here, and it matches
						  * the permissions.  Add this item. */
						 item_found = 1;
					}  else  {
						 /* If we're dropping some items out of the list, remove
						  * the dividers; they'll just get in the way. */
						 continue;
					}
			  }  else  {
#endif
					/* No permissions - mark as found. */
					item_found = 1;
#ifdef NEVER
			  }
#endif
				

			  if(!entry->optionlist)  {
					numoptions=LIST_SIZE;
					curopt = 0;
					entry->optionlist=(char **) MALLOC(numoptions*sizeof(char *)+4);
			  }
			  if(curopt == numoptions)  {
					numoptions+=LIST_SIZE;
					entry->optionlist=(char **)REALLOC(entry->optionlist,
																  numoptions*sizeof(char *)+4);
			  }
			  entry->optionlist[curopt++]=STRDUP(data);
			  entry->optionlist[curopt]=NULL;
		 }
	}
	if(!item_found)  {
		 /* Don't forget the last one - the check is normally being done
		  * at the next Category directive, so the last one must do one
		  * last check. */
		 ans[--current] = NULL;
	}
	*count = current;
	return ans;
}

char ** parseValues(char **vals)
{
	char ** arReturn;
	char *buffer;

	if (vals[0]!=NULL)
	{
		char *sValue = vals[0];
		char *tmp = sValue;
		int iIndex;
		int iString = 0;

		buffer=(char *)malloc(sizeof(char*)*(strlen(sValue)+1));
		strcpy(buffer, sValue);

		/* count string*/
		while (*tmp!=(char)NULL)
		{
			if (*tmp=='[')
				iString++;
			tmp++;
		}

		arReturn = (char**)malloc(iString*sizeof(char*)+4);
		iIndex = 0;
		tmp=buffer;
		while (*tmp!=(char)NULL)
		{
			if (*tmp=='[')
			{
				arReturn[iIndex]=tmp+1;
			} else if (*tmp==']')
			{
				*tmp='\0';
				iIndex++;
				arReturn[iIndex]=NULL;
			}
			tmp++;
		}
	}
	return arReturn;
}

/* Returns an LDAP handle. Return NULL if failed to initialize SSL */
LDAP *getLdapConnection(char *admroot, AdmldapInfo ldapInfo, 
				char *ldapHost, int ldapPort) {

	int rv;
	
	if ((!ldapHost) || (ldapPort < 1)) 
		return NULL;

	if (admldapGetSecurity(ldapInfo)) {
		rv = ADMSSL_Init(ldapInfo, admroot);
		if (rv) {
			report_err(DBT_errorFailedSecurityInit_);
/*			return (LDAP *)0;  */
		}
		return ldapssl_init(ldapHost, ldapPort, 1);
	}
	else
		return ldap_init(ldapHost, ldapPort);
}

/* returns the searchDN underwhich to look for the endUserHtmlIndex programs */
char *getSearchDN(AdmldapInfo ldapInfo) {
	char buf[1000];
	int i, count = 0;

	strcpy(buf, admldapGetSIEDN(ldapInfo));
	if (!buf) {
            report_err( DBT_errorBuildingLdapInfo_);
            exit(0);
	}
	for(i = strlen(buf)-1; i >= 0; i--) {
		if (buf[i] == ',') {
			count++;
			if (count == 2)
				break;
		}
	}
	/* If 2 commas were not found, then exit */
	if (i <= 0) {
        report_err( DBT_errorInvalidSieDN_);
        exit(0);
   }
   /*Eliminate leading blanks */
	while (buf[++i] == ' ');
	sprintf(buf, "%s%c%s", DS_GLOBAL_PREF_RDN, ',', &buf[i]);
	return buf;
}
	
catlist read_catlist_DS(void)
{
	/* int numoptions=LIST_SIZE, curopt=0; */
	catlist	ans=NULL;
	char		*nsroot = getenv("NETSITE_ROOT");
	char     admconf[256];
	int 		rv;
	char		binddn[1000];
	char		bindpw[1000];
	AdmldapInfo	ldapInfo;
	char 		*ldapHost;
	int		ldapPort;
	LDAP		*ld;
	char 		*attr[2], **vals;
	char		EntryDN[1000];
	LDAPMessage	*result, *e;
	int current=0;
	int err;
	int i=1;
        BindInfoStruct bindInfo;

	if (nsroot)
	{
/*  Now need to bind as the admin SIE instead of the user   
		rv = ADM_GetUserDNString(&err, &binddn);
		if (rv<0 || !binddn || !*binddn)
		{
			exit(0);
		}

      rv = ADM_GetCurrentPassword(&err, &bindpw);
*/
		/* binddn=(char*)malloc(sizeof(char)*100); */
		/* strcpy(binddn,"uid=jwalker,ou=people,o=airius.com"); */

		sprintf(admconf, "%s%c%s%c%s", nsroot, FILE_PATHSEP, "admin-serv",
			FILE_PATHSEP, "config");

		ldapInfo = admldapBuildInfo(admconf, &rv);
  
		if (!ldapInfo) {
		  report_err( DBT_errorBuildingLdapInfo_);
		  exit(0);
		}
		else {
			ldapHost = admldapGetHost(ldapInfo);
			ldapPort = admldapGetPort(ldapInfo);
                        strcpy(binddn, admldapGetSIEDN(ldapInfo));
                        strcpy(bindpw, admldapGetSIEPWD(ldapInfo));
                        bindInfo.binddn = binddn;
                        bindInfo.bindpwd = bindpw;

/* atulb : Now checks whether security is turned on/off for ldap before initiating a connection */
			if ((ld = getLdapConnection(admconf, ldapInfo,  ldapHost, ldapPort )) != NULL ) 
			{
				ldap_set_rebind_proc (ld, rebind_proc, &bindInfo);
				if ((rv = ldap_simple_bind_s(ld, binddn, bindpw)) == LDAP_SUCCESS) {
			
					attr[0]="nsAdminEndUserHTMLIndex";
					attr[1]=NULL;
					strcpy(EntryDN, getSearchDN(ldapInfo));
	
					if ( i = ldap_search_s(ld, EntryDN, LDAP_SCOPE_SUBTREE, "(nsadminenduserhtmlindex=*)", attr, 0, &result)==LDAP_SUCCESS)
					{
						if ((e=ldap_first_entry(ld,result))!=NULL)
						{
							if ((vals = getAttributeValue(ld, e)) != NULL)
/*  							if ((vals=ldap_get_values(ld,e,"nsAdminEndUserHTMLIndex"))!=NULL) */
							{
								vals=parseValues(vals);
								ans = parseIndex(vals, &current);
							}
						}
					}
				}
				else 
					report_err( DBT_errorBindingToLdapServer_);
			}
			else 
				report_err( DBT_errorConnectingToLdapServer_);
		}
	}
	ldap_unbind(ld);
	if(current==0)  {
		 /* Nothing found!  For now make error. */
		 report_err( DBT_errorNoProgramsFound_);
		 report_err( DBT_errorNoProgramsMatchedPermissions_);
		 exit(0);
	}
	return ans;
}

catlist read_catlist(void)
{
    catlist ans=NULL;
    catptr entry=NULL;
    int listsize=LIST_SIZE, current=0;
    int numoptions=LIST_SIZE, curopt=0;
    char path[256];
    char existingpath[256];
    FILE *f;
    FILE *flst;
    char line[1024];
    char opt[128];
    char *tmp=NULL, *data=NULL, *div=NULL;
    char *root = getenv("NETSITE_ROOT");
    
    GetFileForLanguage(CATLIST_FILE,GetClientLanguage(),existingpath);
    f = fopen(existingpath, "r");
    if(!f)  {
        report_err( DBT_errorCouldNotOpenServersListFile_);
        exit(1);
    }
    while(fgets(line, sizeof(line), f))  {
        if(line[0] == ';')  
            continue;
        else if((line[0] == '\n') || (line[0] == '\r'))
            continue;
        else if(!strncmp(line, CATEGORY, strlen(CATEGORY)))  {
            tmp=strchr(line, '\n');
            if(tmp) *tmp='\0';
            tmp=strchr(line, '\r');
            if(tmp) *tmp='\0';

            data=strchr(line, ':');
            if(!data) continue; /* huh?! */
            data++;
            div=strchr(data, ',');
            if(!div) continue; /* huh?! */
            *div++ = '\0';

            if(!ans)  {
                ans = (catlist) malloc(listsize*sizeof(catptr)+4);
            }
            if(listsize == current)  {
                listsize+=LIST_SIZE;
                ans = (catlist) realloc(ans, listsize*sizeof(catptr)+4);
            }
            entry=new_catstruct();
            entry->category_id = strdup(data);
            entry->category_desc = strdup(div);
            ans[current++]=entry;
            ans[current]=NULL;
        }
        else if(!strncmp(line, OPTION, strlen(OPTION)))  {
            tmp=strchr(line, '\n');
            if(tmp) *tmp='\0';
            tmp=strchr(line, '\r');
            if(tmp) *tmp='\0';
 
            data=strchr(line, ':');
            if(!data) continue; /* huh?! */
            data++;
 
            if(!entry->optionlist)  {
                numoptions=LIST_SIZE;
                curopt = 0;
                entry->optionlist=(char **) malloc(numoptions*sizeof(char *)+4);
            }
            if(curopt == numoptions)  {
                numoptions+=LIST_SIZE;
                entry->optionlist=(char **)realloc(entry->optionlist,
                                                   numoptions*sizeof(char *)+4);
            }
            entry->optionlist[curopt++]=strdup(data);
            entry->optionlist[curopt]=NULL;
        }
        else if(!strncmp(line, INCLUDE, strlen(INCLUDE))) {
            tmp=strchr(line, '\n');
            if(tmp) *tmp='\0';
            tmp=strchr(line, '\r');
            if(tmp) *tmp='\0';

            data=strchr(line, ' ');
            if(!data) continue; /* huh?! */
            data++;
           
            sprintf(path, INCLUDE_PATH, root, data);
            GetFileForLanguage(path,GetClientLanguage(),existingpath);
            if (!(flst = fopen(existingpath, "r"))) {
              fprintf(stdout,
                      /*XP_GetClientStr(*/"%s",DBT_errorCouldNotOpenSServerListFile_/*)*/,
                      path);
              exit(1);
            }
            else 
            {
              while(fgets(opt, 128, flst) != (char *) NULL)
              { 
                if(!strncmp(opt, CATEGORY, strlen(CATEGORY)))  {
                  tmp=strchr(opt, '\n');
                  if(tmp) *tmp='\0';
                  tmp=strchr(opt, '\r');
                  if(tmp) *tmp='\0';

                  data=strchr(opt, ':');
                  if(!data) continue; /* huh?! */
                  data++;
                  div=strchr(data, ',');
                  if(!div) continue; /* huh?! */
                  *div++ = '\0';

                  if(!ans)  {
                    ans = (catlist) malloc(listsize*sizeof(catptr)+4);
                  }
                  if(listsize == current)  {
                    listsize+=LIST_SIZE;
                  ans = (catlist) realloc(ans, listsize*sizeof(catptr)+4);
                  }
                  entry=new_catstruct();
                  entry->category_id = strdup(data);
                  entry->category_desc = strdup(div);
                  ans[current++]=entry;
                  ans[current]=NULL;
                }
                else if(!strncmp(opt, OPTION, strlen(OPTION)))  {
                       tmp=strchr(opt, '\n');
                       if(tmp) *tmp='\0';
                       tmp=strchr(opt, '\r');
                       if(tmp) *tmp='\0';
 
                       data=strchr(opt, ':');
                       if(!data) continue; /* huh?! */
                       data++;
 
                       if(!entry->optionlist)  {
                         numoptions=LIST_SIZE;
                         curopt = 0;
                         entry->optionlist=(char **) malloc(numoptions*sizeof(char *)+4);
                       }
                       if(curopt == numoptions)  {
                         numoptions+=LIST_SIZE;
                         entry->optionlist=(char **)realloc(entry->optionlist,
                                                     numoptions*sizeof(char *)+4);
                       }
                       entry->optionlist[curopt++]=strdup(data);
                       entry->optionlist[curopt]=NULL;
                }
              }
              fclose(flst);
            }
        }
    }
    return ans;
}

char *get_partial_url(catlist cl, char *id)
{
    register int x;
    char *ans = NULL;

    for(x=0; cl[x]; x++)  {
        if(strcmp(cl[x]->category_id, id))
            continue;
        if(cl[x]->optionlist)  {
            char *comma=strchr((cl[x]->optionlist)[0], ',');
            if(comma)  {
                ans=strdup((cl[x]->optionlist)[0]);
                comma=strchr(ans, ',');
                *comma++ = '\0';
            }
        }
    }
    return ans;
}

char *get_id_bypgm(catlist cl, char *pgm)
{
    register int x, y;
    char *ans = NULL;
 
    for(x=0; cl[x]; x++)  {
        if(cl[x]->optionlist)  {
            for(y=0; (cl[x]->optionlist)[y]; y++)  {
                char *comma=strchr((cl[x]->optionlist)[y], ',');
                if(comma)  {
                    *comma = '\0';
                    if(!strcmp((cl[x]->optionlist)[y], pgm))  {
                        ans=strdup(cl[x]->category_id);
                    }
                    *comma = ',';
                }
            }
        }
    }
    return ans;
}

void return_main_index()
{
    char *user = getenv("REMOTE_USER");
    char existingpath[256];
	char title[1000];

    fprintf(stdout, "<head><title>%s</title>"
                    "<meta http-equiv=Content-Type content=text/html; charset=UTF-8></head>\n",
                    /*XP_GetClientStr(*/DBT_netscapeServerAccount_)/*)*/;
    fprintf(stdout, "<SCRIPT language=JavaScript>\n");
    fprintf(stdout, "function output_top_frame()  {\n");
	fprintf(stdout, "   top.title.document.writeln(\"<BODY TEXT=#000000 BGCOLOR=#9999CC>\");\n");

    fprintf(stdout, "   top.title.document.writeln(\"<TABLE BORDER=0 "
                            "CELLSPACING=0 CELLPADDING=0 ><TR><TD>\");\n");

    fprintf(stdout, "   top.title.document.writeln(\"<TABLE BORDER=0 "
                            "CELLSPACING=12 CELLPADDING=0 ><TR><TD>"
                            "<FONT FACE=\\\"PrimaSans BT, Verdana, sans-serif\\\">"
                            "<FONT COLOR=#FFFFFF><FONT SIZE=-1>Fedora"
                            "</FONT></FONT></FONT>\");\n");

    fprintf(stdout, "   top.title.document.writeln(\"<BR><B>"
                            "<FONT FACE=\\\"PrimaSans BT, Verdana, sans-serif\\\">"
                            "<FONT COLOR=#FFFFFF><FONT SIZE=-1>Fedora"
                            "</FONT></FONT></FONT></B></TD>\");\n");
    fprintf(stdout, "   top.title.document.writeln(\"</tr></table></td><td>"
			    "<TABLE BORDER=0 CELLSPACING=12 CELLPADDING=0><tr>\");\n");

    fprintf(stdout, "   top.title.document.writeln(\"<TD>"
                            "<FONT FACE=\\\"PrimaSans BT, Verdana, sans-serif\\\">"
                            "<FONT COLOR=#FFFFFF><FONT SIZE=+1>");

    if (user)
    {
            sprintf(title,DBT_serverAccountManagementFor_,user);
    } else
    {
            sprintf(title,DBT_serverAccountManagement_);
    }
    fprintf(stdout,title);

    fprintf(stdout, "</B></FONT></FONT></FONT>&nbsp;</td></tr></table></TD></TR><TR> <TD>"
                            "<IMG SRC=../gifs/spacer.gif HEIGHT=1 WIDTH=140></TD>"
                            "</TR></TABLE>\");\n");
    fprintf(stdout, "}\n");

    fprintf(stdout, "function output_bottom_frames()  {\n");
    fprintf(stdout, "    top.bottom.document.writeln(\"<frameset cols='140,1*' "
                                "border=0>\");\n");
    fprintf(stdout, "    top.bottom.document.writeln(\"<frame src='%s?list' "
                                "scrolling=auto marginwidth=0 marginheight=0 "
                                "noresize border=0 "
                                "name='%s'>\");\n", SCRIPT_NAME, TOP_NAME);
	 GetFileForLanguage(HTML_FILE,GetClientLanguage(),existingpath);

    fprintf(stdout, "    top.bottom.document.writeln(\"<frame "
                                "src='%s' "
                                "scrolling=auto noresize marginwidth=16 "
                                "marginheight=16 name='%s'>\");\n",
                                existingpath,
                                CONTENT_NAME);
    fprintf(stdout, "    top.bottom.document.writeln(\"</frameset>\");\n");
    fprintf(stdout, "}\n");
    fprintf(stdout, "</script>\n\n");

    fprintf(stdout, "<frameset rows=\"60,1*\" border=0 "
		    "framespacing=0 frameborder=0 framepadding=0 "
                    "marginwidth=0 marginheight=0>\n");
    fprintf(stdout, "<frame src=\"javascript:top.output_top_frame();\" "
		    "marginheight=0 marginwidth=0 scrolling=no noresize "
                    "frameborder=no name=title>\n");
    fprintf(stdout, "<frame src=\"javascript:top.output_bottom_frames();\" "
		    "marginheight=0 marginwidth=0 scrolling=\"no\" "
                    "name=bottom>\n");
    fprintf(stdout, "</frameset>\n");
    exit(0);
}

void return_list()
{
    catlist cl = read_catlist_DS();
    char  **ol = NULL;
    char *desc = NULL;
    int x = 0, y = 0;


    fprintf(stdout, "<html><head><title>%s</title>"
                       "<meta http-equiv=Content-Type content=text/html; charset=UTF-8></head>", DBT_FormsList_);

    fprintf(stdout, "<BODY BGCOLOR=#CCCCCC LINK=#FFFFFF "
		    " VLINK=#FFFFFF ALINK=#FFFFFF>\n" );

    fprintf(stdout, "<table cellspacing=0 cellpadding=4 "
                    "width=132 border=0>\n");
    fprintf(stdout, "<tr><td valign=top>");

    fprintf(stdout, "<table cellspacing=4 cellpadding=4 "
                    "width=100%% border=0>\n");

    for(x=0; cl[x]; x++)  {

        fprintf(stdout, "<tr>\n");
        fprintf(stdout, "<td> <font face=\"PrimaSans BT, Verdana, sans-serif\">%s</font></td>\n", 
                        cl[x]->category_desc);
        fprintf(stdout, "</tr>\n");

        ol = cl[x]->optionlist;
        if(!ol) continue;

        for(y=0; ol[y]; y++)  {
            desc = strchr(ol[y], ',');
            if(!desc) continue;
            *desc++ = '\0';
            fprintf(stdout, "<tr>\n");
            fprintf(stdout, "<td bgcolor=#999999><b>"
                            "<font face=\"PrimaSans BT, Verdana, sans-serif\" size=-1>"
                            "<a href=%s target=%s>%s</a>"
                            "</font>"
                            "</b></td>\n", 
                            ol[y], CONTENT_NAME, desc);
            fprintf(stdout, "</tr>\n");
        }
        fprintf(stdout, "<tr>\n");
        fprintf(stdout, "<td></td>\n");
        fprintf(stdout, "</tr>\n");
    }
    fprintf(stdout, "</table>\n");

    fprintf(stdout, "</td></tr></table>\n");

    fprintf(stdout, "</BODY>\n");
    exit(0);
}

int main(int argc, char *argv[])
{
    char *qs = getenv("QUERY_STRING");

    /* Get *Language from Environment */
    char *AcceptLang  = getenv("HTTP_ACCEPT_LANGUAGE");
    char *ClientLang  = getenv("ClientLanguage");
    char *AdminLang   = getenv("AdminLanguage");
    char *DefaultLang = getenv("DefaultLanguage");
    ADM_Init();

    /* initialize the string database */
    /* XP_InitStringDatabase("../../../admin",DATABASE_NAME); */

    /* Set *Language from environment if found */
    if (AcceptLang) {
      SetLanguage(CLIENT_LANGUAGE,AcceptLang);
    }
    else if (ClientLang) {
      SetLanguage(CLIENT_LANGUAGE,ClientLang);
    }
    if (AdminLang) {
      SetLanguage(ADMIN_LANGUAGE,AdminLang);
    }
    if (DefaultLang) {
      SetLanguage(DEFAULT_LANGUAGE,DefaultLang);
    }

    fprintf(stdout, "Content-type: text/html\n\n");

    if((!qs) || (qs[0]=='\0'))  {
        return_main_index();
    }  
    else if(!strcmp(qs, "list"))  {
        return_list();
    }
    return 0;
}
