/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
//--------------------------------------------------------------------------//
//                                                                          //
//  Name: URL Viewer                                                        //
//  Syntax: urlview <url>                                                   //
//  Platforms: WIN32                                                        //
//  ......................................................................  //
//  Displays specified url in browser window.                               //
//  Detects default registered browser for .html files.                     //
//  Starts browser process if not already running.                          //
//  Uses existing, most recent browser window if already running.           //
//                                                                          //
//  References:                                                             //
//  http://www.spyglass.com/products/smosaic/sdi/sdi%5Fspec.html            //
//  http://developer1.netscape.com/docs/manuals/communicator/DDE/abtdde.htm //
//  http://msdn.microsoft.com/developer (search for www_openurl)            //
//                                                                          //
//  Helpful Development Tools                                               //
//  REGEDIT                                                                 //
//  DDESPY   (comes with VC++)                                              //
//  NTREGMON (www.ntsys.com) also on \\ntsbuild\\install\devtools           //
//                                                                          //
//  ......................................................................  //
//  Revision History:                                                       //
//  11-23-98  Initial Version, Andy Hakim (ahakim@netscape.com)             //
//            Based on navstart, which this replaces.                       //
//            Major differences between this and navstart:                  //
//            1) does not contain code to start admin service               //
//               (use <sr>\start-admin.bat instead)                         //
//            2) is browser independant                                     //
//            3) improved dde code to fix window focus problem              //
//            4) significantly simplified code                              //
//--------------------------------------------------------------------------//
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <ddeml.h>
#include <stdlib.h>
#include <stdio.h>
#include "resource.h"

#define DDE_TIMEOUT   2000


//--------------------------------------------------------------------------//
// return HKCR\.html\(default)                                              //
//        eg: IE=htmlfile  NS=NetscapeMarkup                                //
//--------------------------------------------------------------------------//
char* _GetHtmlDefaultKeyValue()
{
    HKEY hKey;
    static char szValue[MAX_PATH];

	ZeroMemory(szValue, sizeof(szValue));
    if(RegOpenKeyEx(HKEY_CLASSES_ROOT, ".html", 0, KEY_QUERY_VALUE, &hKey) == ERROR_SUCCESS)
    {
        DWORD dwSize = sizeof(szValue);
        RegQueryValueEx(hKey, "", 0, NULL, szValue, &dwSize);
        RegCloseKey(hKey);
    }
	return szValue;
}


//--------------------------------------------------------------------------//
// return HKCR\<x>\<y>\<z>                                                  //
//        x = _GetHtmlDefaultValue()                                        //
//        y = subkey                                                        //
//        z = value name, use "" for default                                //
//--------------------------------------------------------------------------//
char* _GetGenericHtmlKeyValue(char *szSubKey, char *szName)
{
    HKEY hKey;
	char szKey[MAX_PATH];
    static char szValue[MAX_PATH];

	ZeroMemory(szValue, sizeof(szValue));
	wsprintf(szKey, "%s\\%s", _GetHtmlDefaultKeyValue(), szSubKey);
	if(RegOpenKeyEx(HKEY_CLASSES_ROOT, szKey, 0, KEY_QUERY_VALUE, &hKey) == ERROR_SUCCESS)
	{
		DWORD dwSize = sizeof(szValue);
		RegQueryValueEx(hKey, szName, 0, NULL, szValue, &dwSize);
		RegCloseKey(hKey);
	}
	return szValue;
}


//--------------------------------------------------------------------------//
// return HKCR\<htmlfile>\shell\open\command\(default)                      //
//        eg: IE=C:\PROGRA~1\Plus!\MICROS~1\iexplore.exe -nohome            //
//        eg: NS=C:\PROGRA~1\NETSCAPE\COMMUN~1\PROGRAM\NETSCAPE.EXE "%1"    //
//--------------------------------------------------------------------------//
char* _GetBrowserPathValue()
{
	static char *szValue;
	szValue = _GetGenericHtmlKeyValue("shell\\open\\command", "");

	if(szValue)
	{
		char *szRemove;
		szRemove = strstr(szValue, "%1");
		if(szRemove)
			*(szRemove+1) = 's'; // %1 now becomes %s
	}

	return szValue;
}


//--------------------------------------------------------------------------//
// return HKCR\<htmlfile>\shell\open\ddeexec\application\(default)          //
//--------------------------------------------------------------------------//
char* _GetDDEApplicationValue()
{
	return _GetGenericHtmlKeyValue("shell\\open\\ddeexec\\application", "");
}


//--------------------------------------------------------------------------//
// return HKCR\<htmlfile>\shell\open\ddeexec\application\(default)          //
//--------------------------------------------------------------------------//
char* _GetDDETopicValue()
{
	return _GetGenericHtmlKeyValue("shell\\open\\ddeexec\\topic", "");
}


//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
HDDEDATA CALLBACK DdeCallback(UINT uType, UINT uFmt, HCONV hconv, HSZ hsz1, HSZ hsz2, HDDEDATA hdata, DWORD dwData1, DWORD dwData2)
{
    return (HDDEDATA) NULL;     
}



//--------------------------------------------------------------------------//
// view url in browser window                                               //
//--------------------------------------------------------------------------//
BOOL _ViewURL(char *szUrl)
{
    BOOL bReturn = FALSE;
    DWORD dwResult = 0;
    DWORD idInst = 0;
    HDDEDATA hData = NULL;
    HCONV hConv = NULL;
    HSZ hszTopic;
    HSZ hszService;
	char *ddeUrl;
    char pData[MAX_PATH];
    int cbData = 0;
	int i;

    if(DdeInitialize(&idInst, (PFNCALLBACK)DdeCallback, APPCMD_CLIENTONLY | CBF_SKIP_ALLNOTIFICATIONS, 0L) == DMLERR_NO_ERROR)
    {
        
        hszService = DdeCreateStringHandle(idInst, _GetDDEApplicationValue(), CP_WINANSI);
        hszTopic = DdeCreateStringHandle(idInst, _GetDDETopicValue(), CP_WINANSI);

		// double-quote (") to single-quote (') replacement for DDE transaction
		ddeUrl= (char *)strdup(szUrl);
		for(i = 0; *(ddeUrl+i) != '\0'; i++)
		{
			if(*(ddeUrl+i) == '"')
				*(ddeUrl+i) = '\'';
		}

		// TODO: following syntax assumes topic is "WWW_OpenURL"
        wsprintf(pData, "\"%s\",,0x0,0x0,,,", ddeUrl);
        cbData = strlen(pData)+1;

        if(hszTopic && hszService)
        {
            if(hConv = DdeConnect(idInst, hszService, hszTopic, NULL))
            {
                DdeClientTransaction(pData, cbData, hConv, NULL, 0, XTYP_EXECUTE, DDE_TIMEOUT, &dwResult);
                bReturn = DdeDisconnect(hConv);
            }
            DdeFreeStringHandle(idInst, hszService);
            DdeFreeStringHandle(idInst, hszTopic);
        }
        DdeUninitialize(idInst);
    }
    return bReturn;
}


//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
BOOL _StartBrowser(char *szURL)
{
	char szCommandLine[MAX_PATH];
	char *szBrowserPath;
	szBrowserPath = _GetBrowserPathValue();
	if(strstr(szBrowserPath, "%s") == NULL)
	{
		strcat(szBrowserPath, " %s");
	}

	sprintf(szCommandLine, szBrowserPath, szURL);
	
    if(WinExec(szCommandLine, SW_SHOW) > 31)
		return TRUE;

    return(FALSE);
}



//--------------------------------------------------------------------------//
//                                                                          //
//--------------------------------------------------------------------------//
WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR szCmdLine, int nCmdShow)
{
	char szText[MAX_PATH];
    int nReturn = 0;

    if(szCmdLine && strlen(szCmdLine))
    {
		if(szCmdLine != NULL)
		{
			nReturn = _ViewURL(szCmdLine);
			if(nReturn == 0)
			{
				nReturn = _StartBrowser(szCmdLine);
			}
		}
		if(nReturn == 0)
		{
			char szTitle[MAX_PATH];
			LoadString(hInstance, IDS_ERROR_TITLE, szTitle, sizeof(szTitle));
			LoadString(hInstance, IDS_ERROR_BROWSER, szText, sizeof(szText));
			MessageBox(NULL, szText, szTitle, MB_ICONERROR | MB_OK);
		}
    }
	else
	{
		char szTitle[MAX_PATH];
		LoadString(hInstance, IDS_ERROR_TITLE, szTitle, sizeof(szTitle));
		LoadString(hInstance, IDS_ERROR_SYNTAX, szText, sizeof(szText));
		MessageBox(NULL, szText, szTitle, MB_ICONERROR | MB_OK);
	}

    return(nReturn);
}
