/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
//////////////////////////////////////////////////////////////////////////////
// insbase.c - SuiteSpot Installation Plug-In Template
//
// Copyright (c) 1997 Netscape Communications Corporation.
// All Rights Reserved.
//

#include <windows.h>
#include <nssetup.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <crtdbg.h>
#include "resource.h"
#include "insbase.h"
#include "nt/regparms.h"

static MODULEINFO mi = {
  NULL,           // m_hModule
  NULL,           // m_hwndParent
  NS_WIZERROR     // m_nResult
};


//////////////////////////////////////////////////////////////////////////////
// DllMain
//
// The Windows DLL main entry point.  Called upon loading the DLL into memory.
// Perform all initialization in the DLL_PROCESS_ATTACH reason handler, and
// release any resources that you have allocated in the DLL_PROCESS_DETACH
// message handler.  See the Windows SDK documentation for more information
// on this function.
//

BOOL WINAPI
DllMain(HANDLE hModule, ULONG ulReasonForCall, LPVOID lpReserved)
{
  switch (ulReasonForCall)
  {
    case DLL_PROCESS_ATTACH:
      ZeroMemory(&mi, sizeof(MODULEINFO));
      mi.m_hModule = hModule;
      break;
    case DLL_PROCESS_DETACH:
      break;
    case DLL_THREAD_ATTACH:
      break;
    case DLL_THREAD_DETACH:
      break;
  }
  return TRUE;
}

//////////////////////////////////////////////////////////////////////////////
// BASE_PreInstall
//
// This function is called by the installation framework before asking the
// user any questions.  Here you should determine if all of the requisites
// for installing this component are being met.  If this operation succeeds
// return TRUE, otherwise display an error message and return FALSE to abort
// installation.
//

BOOL __declspec(dllexport)
BASE_PreInstall(LPCSTR lpszInstallPath)
{
  char szFilePath[MAX_PATH];
  char szFormat[MAX_PATH];
  char szErrMsg[MAX_PATH * 2];

  LoadString(mi.m_hModule, IDS_ERR_MISSINGFILE, szFormat, sizeof(szFormat));


  wsprintf(szFilePath, "%s\\%s", SRCDIR, "base.inf");
  if (!FileExists(szFilePath))
  {
    wsprintf(szErrMsg, szFormat, szFilePath);
    NsSetupMessageBox(NULL, szErrMsg, NULL, MB_OK|MB_ICONSTOP);
    return FALSE;
  }

  wsprintf(szFilePath, "%s\\%s", SRCDIR, "insbase.dll");
  if (!FileExists(szFilePath))
  {
    wsprintf(szErrMsg, szFormat, szFilePath);
    NsSetupMessageBox(NULL, szErrMsg, NULL, MB_OK|MB_ICONSTOP);
    return FALSE;
  }

  wsprintf(szFilePath, "%s\\%s", SRCDIR, "basesys.z");
  if (!FileExists(szFilePath))
  {
    wsprintf(szErrMsg, szFormat, szFilePath);
    NsSetupMessageBox(NULL, szErrMsg, NULL, MB_OK|MB_ICONSTOP);
    return FALSE;
  }

  wsprintf(szFilePath, "%s\\%s", SRCDIR, "base.z");
  if (!FileExists(szFilePath))
  {
    wsprintf(szErrMsg, szFormat, szFilePath);
    NsSetupMessageBox(NULL, szErrMsg, NULL, MB_OK|MB_ICONSTOP);
    return FALSE;
  }

  return TRUE;
}

//////////////////////////////////////////////////////////////////////////////
// BASE_Install
//
// The framework calls this function to perform installation.  Here you should
// define the file sets that are required to place your files on the
// destination disk.  You should not however perform the extraction, as the
// calling level will do that once the file sets for all products have been
// defined.  This ensures that the progress indicator shows the results for
// the entire range of user selections as a whole.  If the function
// successfully defines the file set return TRUE, otherwise return FALSE to
// indicate an error.
//

BOOL __declspec(dllexport)
BASE_Install(VOID)
{
  char szPath[MAX_PATH];
  HFILESET hfs;

  hfs = FileSetCreate(STR_SUITESPOT_BASE);
  if (hfs != INVALID_FILESET_HANDLE)
  {
    lstrcpy(szPath, TARGETDIR);
    if (!FileSetAddArchive(hfs, BASE_Z, "*.*", STR_COPY_BASE, FS_UPDATE_VERSION | FS_LOCKEDFILE))
      return FALSE;

    lstrcpy(TARGETDIR, WINSYSDIR);
    if (!FileSetAddArchive(hfs, BASESYS_Z, "*.*", STR_COPY_BASE, FS_UPDATE_VERSION | FS_LOCKEDFILE))
      return FALSE;

    lstrcpy(TARGETDIR, szPath);
    FileSetClose(hfs);
  }

  return TRUE;
}

BOOL __declspec(dllexport)
CLIENT_Install(VOID)
{
  HFILESET hfs;

  hfs = FileSetCreate(STR_SUITESPOT_CLIENT);
  if (hfs != INVALID_FILESET_HANDLE)
  {
    if (!FileSetAddArchive(hfs, CLIENT_Z, "*.*", STR_COPY_CLIENT, FS_UPDATE_VERSION | FS_LOCKEDFILE))
      return FALSE;

    FileSetClose(hfs);
  }
  return TRUE;
}

