/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

extern  "C" {
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
}


#include "setupapi.h"

static int prepareJREUpgrade(char *serverRoot);

/*********************************************************************
**
** NAME:
**   ux-config.cc
**
** DESCRIPTION:
**   Base Package Pre-install Program
**
** NOTES:
**
*********************************************************************/

int main(int argc, char **argv) {
   char *serverRoot=0;
   char *infoFile = 0;
   char *logFile = 0;
   int   instType;
   int   silentInstall = 0;
   int   reconfig = 0;
   int   c;

   /* We care only about the -f <infoFile> option */
   while ((c = getopt(argc, argv, "sl:m:f:r")) != -1) {
       if (c == 'f') {
           infoFile=optarg;
       }
   }
   while ((c = getopt(argc, argv, "sl:m:f:r")) != -1) {
       switch (c) {
         case 'f':
           infoFile = strdup(optarg);
           break;
         case 'l':
           logFile = strdup(optarg);
           break;
         case 'm':
           instType = atoi(optarg);
           break;
         case 's':
           silentInstall = 1;
           break;
         case 'r':
           reconfig = 1;
           break;
         default:
           break;
       }
   }

   if (!infoFile) {
       fprintf(stderr, "ERROR: %s No -f <info-file> option.\n", argv[0]);
       return -1;
   }

   serverRoot = setupGetInfString("General", "ServerRoot", "", infoFile);

   if (!serverRoot) {
       fprintf(stderr, "ERROR: %s No ServerRoot directive in %s.\n", argv[0], infoFile);
       return -1;
   }

   prepareJREUpgrade(serverRoot);

   return 0;
}

//
// Cleanup previous JRE installation if installing in an existing server root
//
// Cleanup bin/base/jre :
// When upgrading from JRE1.1.x (Console 4.x) to JRE1.2 (Console 5.x) the java
// command might not work on certain platforms if JRE1.2 is simply installed
// over JRE1.1.x.
//
static int prepareJREUpgrade(char *serverRoot) {
   struct stat fi;
   char buf[BIG_BUF];

   sprintf(buf, "%s/bin/base/jre", serverRoot);

   if (stat(buf, &fi) == 0)
   {
       sprintf(buf, "rm -fr %s/bin/base/jre 2>/dev/null",  serverRoot);
       return system(buf);
        
   }
   return 0;
}
