# BEGIN COPYRIGHT BLOCK
# Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
# Copyright (C) 2005 Red Hat, Inc.
# All rights reserved.
# END COPYRIGHT BLOCK
#!/usr/local/bin/perl

#usage AdjustInf.pl -s srcDir [ -v -h -r -f fromLang -t toLang ]

#set some default parameters
$VERBOSE    = "noverbose"; 
$fromLang   = "";
$toLang     = "lang";
$platform   = `uname`;
$servername = "not defined";
$|          = 1;

$numComp = 0;
$platform =~ s/\n$//;
if($platform eq "Windows_NT") { $platform == "WINNT";}
$perl = "";

if($platform =~ /WINNT/i){
    $sept = "\\";
} else {
    $sept = "/";
}

#parse command line argments

while ($ARGV[0] =~ /^-/) {
    $_ = shift;
    if (/^-s/) {
        $_ = shift;
        $_ =~ /^(\S+)/;
        $rootDir = $1;
    }
    elsif (/^-v/) {
        $VERBOSE = "verbose"; 
    }
    elsif (/^-h/) {
        &Help();
        exit(0);
    }
    elsif (/^-r/) {
        &report();
        exit(0);
    }
    elsif (/^-f/) {
        $_ = shift;
        $_ =~ /^(\S+)/;
        $fromLang = $1;
    }
    elsif (/^-t/) {
        $_ = shift;
        $_ =~ /^(\S+)/;
        $toLang = $1;
    }
    elsif (/^-Perl/) {
        $_ = shift;
        $_ =~ /^(\S+)/;
        $perl = $1;
    }
    else {
        print "Unrecoginized switch: $_\n";
        &Help();
        exit(0);
    }
}

if (($perl =~ /^$/) && ($platform eq "WINNT")) { $perl = "perl";}

# must specify the source
if($rootDir =~ /^$/){
    &Help();
    exit(0);
}

# print out operation parameters
&report();

chdir("$rootDir") || die "cannot change directory to $rootDir\n"; ;

if($VERBOSE eq "verbose"){
    print `pwd`,"\n";
    print `ls -l`,"\n";
}

if ($fromLang ne "") {
    $from = "_$fromLang";
    $fromFor = " for $fromLang";
}
else {
    $from = "";
    $fromFor = "";
}
$to = "_$toLang";
$toFor = " for $toLang";

####################
# adjust setup.inf #
####################

$infIn ="setup.inf";
$infOut = "setup.new";

open(INF, "$infIn")   || die "cannot open $infIn\n";  
open(NEWINF, ">$infOut")   || die "cannot open $infOut\n";  
while($line = <INF>) {
    $line =~ s/\r*\n//;
    $_ = $line;
    if (/^\s*Components\s*=(.*)$/i) {
        # Components = complist
        @Components = split(/,/,$1);
        print NEWINF "Components = ";
        foreach $i (0 .. $#Components){
            $Comp = trim($Components[$i]);
            $Components[$i] = langChanger($Comp,$from,$to);
            if ($i>0) {
                print NEWINF ", ";
            }
            print NEWINF "$Components[$i]";
            if (! -d $Components[$i]) {
                if (-d $Comp) {
                    rename ($Comp, $Components[$i]);
                }
            }
        }
        print NEWINF "\n";
    }
    elsif (/^\s*\[(.*)\]\s*$/) {
        # [compname]
        $Component = trim($1);
        if (($Component =~ /^General$/i)==0) {
            $Component = langChanger($Component,$from,$to);
        }
        print NEWINF "["."$Component"."]\n";
    }
    elsif (/^\s*ComponentInfoFile\s*=(.*)$/i) {
        # ComponentInfoFile = infofile
        $InfoFile = trim($1);
        @InfoFileParts = split(/[\/\\]/,$InfoFile);
        $InfDir[$numComp] = langChanger($InfoFileParts[0],$from,$to);
        @NameSplinter = split(/(\.)/,$InfoFileParts[1]);
        $InfName[$numComp]="";
        $i = $#NameSplinter-2;
        $NameSplinter[$i]
            = langChanger($NameSplinter[$i],$from,$to);
        foreach $i (0 .. $#NameSplinter) {
            $InfName[$numComp] = $InfName[$numComp] . $NameSplinter[$i];
        }
        $OldInfoFile = $InfDir[$numComp] . $sept . $InfoFileParts[1];
        $NewInfoFile = $InfDir[$numComp] . $sept . $InfName[$numComp];
        print NEWINF "ComponentInfoFile = " . $NewInfoFile . "\n";
        if (! -f $NewInfoFile) {
            if (-f $OldInfoFile) {
                rename ($OldInfoFile, $NewInfoFile);
            }
        }
        if (! -f $NewInfoFile) {
            print "Warning: inf file $NewInfoFile not found.\n";
            if (! -d $InfDir[$numComp]) {
                print "Warning: Component directory $InfDir[$numComp] "
                    . "not found.\n";
            }
        }
        $numComp++;
    }
    elsif (/^\s*Name\s*=(.*)$/i) {
        $newline = langChanger($line,$fromFor,$toFor);
        print NEWINF "$newline\n";
    }
    else {
        # straight copy
        print NEWINF "$line\n";
    }
}
close(NEWINF);
close(INF);

unlink($infIn) || die "cannot delete $infIn";
rename($infOut,$infIn) || die "cannot rename $infOut to $infIn";

###############################
# fix each component inf file #
###############################

foreach $i (0 .. $numComp-1){
    if (-d $InfDir[$i]) {
        if($VERBOSE eq "verbose"){
            print `cd $InfDir[$i] ; pwd ; ls -l`;
        }
        $infIn =  "$InfDir[$i]/$InfName[$i]";
        $infOut =  $infIn;
        $infOut =~ s/\.inf$/.new/;
        open(INF, "$infIn")   || die "cannot open $infIn\n";  
        open(NEWINF, ">$infOut")   || die "cannot open $infOut\n";  
        while($line = <INF>) {
            $line =~ s/\r*\n//;
            $_ = $line;
            if (/^\s*\[(.*)\]\s*$/) {
                # [compname]
                $Component = trim($1);
                if (($Component =~ /^General$/i)==0) {
                    $Component = langChanger($Component,$from,$to);
                    NewDependencies();
                    print NEWINF "\n";
                }
                print NEWINF "["."$Component"."]\n";
                $BaseNickName = "";
                $BaseVersion = "";
                $Compatible = "";
            }
            elsif (/^\s*Archive\s*=(.*)$/i) {
                # Archive = zipfilename
                @LineSplinter = split(/(\.)/,$line);
                $newline="";
                $i = $#LineSplinter-2;
                $LineSplinter[$i]
                    = langChanger($LineSplinter[$i],$from,$to);
                foreach $i (0 .. $#LineSplinter) {
                    $newline = "$newline" . "$LineSplinter[$i]";
                }
                print NEWINF "$newline\n";
            }
            elsif (/^\s*Compatible\s*=(.*)$/i) {
                # Compatible
                $Compatible = trim($1);
                print NEWINF "$line\n";
            }
            elsif (/^\s*Components\s*=(.*)$/i) {
                # Components = complist
                @Components = split(/,/,$1);
                print NEWINF "Components = ";
                foreach $i (0 .. $#Components){
                    $Comp = trim($Components[$i]);
                    $Components[$i] = langChanger($Comp,$from,$to);
                    if ($i>0) {
                        print NEWINF ", ";
                    }
                    print NEWINF "$Components[$i]";
                }
                print NEWINF "\n";
            }
            elsif (/^\s*Name\s*=(.*)$/i) {
                # Name
                $newline = langChanger($line,$fromFor,$toFor);
                print NEWINF "$newline\n";
            }
            elsif (/^\s*NickName\s*=(.*)$/i) {
                # NickName
                $BaseNickName = trim($1);
                $BaseNickName =~ s/$from$//;
                $newline = langChanger($line,$from,$to);
                print NEWINF "$newline\n";
            }
            elsif (/^\s*Size\s*=(.*)$/i) {
                # Size
                print NEWINF "Size = 1024\n";
            }
            elsif (/^\s*SourcePath\s*=(.*)$/i) {
                # SourcePath
                $newline = langChanger($line,$from,$to);
                print NEWINF "$newline\n";
            }
            elsif (/^\s*Version\s*=(.*)$/i) {
                # Version
                $BaseVersion = trim($1);
                print NEWINF "$line\n";
            }
            elsif (/^\s*BuildNumber\s*=(.*)$/i) {
                # BuildNumber
                print NEWINF "$line\n";
            }
            elsif (/^\s*Checked\s*=(.*)$/i) {
                # Checked
                print NEWINF "$line\n";
            }
            elsif (/^\s*Description\s*=(.*)$/i) {
                # Description
                print NEWINF "$line\n";
            }
            elsif (/^\s*IsMCC\s*=(.*)$/i) {
                # IsMCC
                print NEWINF "$line\n";
            }
            elsif (/^\s*Mandatory\s*=(.*)$/i) {
                # Mandatory
                print NEWINF "$line\n";
            }
            elsif (/^\s*Vendor\s*=(.*)$/i) {
                # Vendor
                print NEWINF "$line\n";
            }
            elsif (/^\s*$/i) {
                # blank line
            }
            else {
                # all other lines: ignore line by comment
                if ($line =~ /^#/) {
                    print NEWINF "$line\n";
                } else {
                    print NEWINF "# $line\n";
                }
            }
        }

        NewDependencies();

        close(NEWINF);
        close(INF);
        
        unlink($infIn) || die "cannot delete $infIn";
        rename($infOut,$infIn) || die "cannot rename $infOut to $infIn";
    }
}

print "AdjustInf Done\n";
exit(0);

############## subroutines #######################
sub Help {
    print "Usage: AdjustInf.pl -s <srcDir>\n";
    print "       -h          : printout help\n";
    print "       -r          : printout operation report\n";
    print "       -v          : operation under verbose mode\n";
    print "       -s dir      : specify source directory name\n";
    print "       -f fromLang : specify old language [default \"\"]\n";
    print "       -t toLang   : specify new language [default \"lang\"]\n";
    print "       -Perl file  : special version of perl\n";
}

sub report {
    print "======== AdjustInf parameters ===========\n";
    print "Source directory      = ", $rootDir,    "\n";
    print "operation mode        = ", $VERBOSE,    "\n";
    print "fromLang              = ", $fromLang,   "\n";
    print "toLang                = ", $toLang,     "\n";
    print "platform              = ", $platform,   "\n";
    print "perl                  = ", $perl,       "\n";
    print "=========================================\n";
}

sub trim {
    local($String) = @_;
    $String =~ s/^\s*//;
    $String =~ s/\s*$//;
    return $String;
}

sub langChanger {
    local($String, $from, $to) = @_;
    if ( $from ne "" ) {
        if ( $String =~ /$from$/ ) {
            $String =~ s/$from$/$to/;
        } else {
            print "Error: In $String, $from can not be changed to $to.\n";
            exit(1);
        }
    } else {
        $String =~ s/$/$to/;
    }
    return $String;
}

sub NewDependencies {
    if ($BaseNickName ne "") {
        if ($BaseVersion ne "") {
            print NEWINF "## Dependencies = $BaseNickName/$BaseVersion\n";
        } else {
            print NEWINF "## Dependencies = $BaseNickName\n";
        }
    }
}
