# BEGIN COPYRIGHT BLOCK
# Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
# Copyright (C) 2005 Red Hat, Inc.
# All rights reserved.
# END COPYRIGHT BLOCK
#!/usr/local/bin/perl

#usage CheckInfZip.pl -s srcDir [ -v -h -r -t toLang ]

#set some default parameters
$VERBOSE    = "noverbose"; 
$toLang     = "lang";
$platform   = `uname`;
$servername = "not defined";
$|          = 1;

$numComp = 0;
$platform =~ s/\n$//;
if($platform eq "Windows_NT") { $platform == "WINNT";}
$perl = "";

#parse command line argments

while ($ARGV[0] =~ /^-/) {
  $_ = shift;
  if (/^-s/) {
    $_ = shift;
    $_ =~ /^(\S+)/;
    $rootDir = $1;
  }
  elsif (/^-v/) {
    $VERBOSE = "verbose"; 
  }
  elsif (/^-h/) {
    &Help();
    exit(0);
  }
  elsif (/^-r/) {
    &report();
    exit(0);
  }
  elsif (/^-t/) {
    $_ = shift;
    $_ =~ /^(\S+)/;
    $toLang = $1;
  }
  elsif (/^-Perl/) {
    $_ = shift;
    $_ =~ /^(\S+)/;
    $perl = $1;
  }
  else {
    print "Unrecoginized switch: $_\n";
    &Help();
    exit(0);
  }
}

if (($perl =~ /^$/) && ($platform eq "WINNT")) { $perl = "perl";}

# must specify the source
if($rootDir =~ /^$/){
  &Help();
  exit(0);
}

# print out operation parameters
&report();

chdir("$rootDir") || die "cannot change directory to $rootDir\n"; ;

if($VERBOSE eq "verbose"){
  print `pwd`,"\n";
  print `ls -l`,"\n";
}

$infIn ="setup.inf";

$to = "_$toLang";
$toFor = " for $toLang";

open(INF, "$infIn")   || die "cannot open $infIn\n";  
while($line = <INF>) {
  $_ = $line;
  if (/^\s*Components\s*=(.*)$/i) {
    # Components = complist
    @Components = split(/,/,$1);
    foreach $i (0 .. $#Components){
      $Components[$i] = trim($Components[$i]);
    }
  }
  elsif (/^\s*ComponentInfoFile\s*=(.*)$/i) {
    # ComponentInfoFile = infofile
    $InfoFile = trim($1);
    @InfoFileParts = split(/\//,$InfoFile);
    $InfDir[$numComp] = $InfoFileParts[0];
    $InfName[$numComp] = $InfoFileParts[1];
    $numComp++;
  }
  else {
    # nothing to do
    ;
  }
}
close(INF);

# fix each component inf file
foreach $i (0 .. $numComp-1){
  if (-d $InfDir[$i]) {
    if($VERBOSE eq "verbose"){
      print `cd $InfDir[$i] ; pwd ; ls -l`;
    }
    $infIn =  "$InfDir[$i]/$InfName[$i]";
    $infOut =  $infIn;
    $infOut =~ s/\.inf$/.new/;
    open(INF, "$infIn")   || die "cannot open $infIn\n";  
    open(NEWINF, ">$infOut")   || die "cannot open $infOut\n";  
    while($line = <INF>) {
      $_ = $line;
      if (/^\s*Archive\s*=(.*)$/i) {
        chop($line);
        $Archive = trim($1);
        if (-f "$SourcePath/$Archive") {
          print NEWINF "$line\n";
        } else {
          print NEWINF "# $line\n";
          print "# Archive not found: $SourcePath/$Archive\n";
        }
      }
      elsif (/^\s*SourcePath\s*=(.*)$/i) {
        print NEWINF "$line";
        $SourcePath = $1;
      }
      else {
        # straight copy
        print NEWINF "$line";
      }
    }
    close(NEWINF);
    close(INF);
    
    unlink($infIn) || die "cannot delete $infIn";
    rename($infOut,$infIn) || die "cannot rename $infOut to $infIn";
  }
}

print "Done\n";
exit(0);

############## subroutines #######################
sub Help {
  print "Usage: CheckInfZip.pl -s <srcDir>\n";
  print "       -h          : printout help\n";
  print "       -r          : printout operation report\n";
  print "       -v          : operation under verbose mode\n";
  print "       -s dir      : specify source directory name\n";
  print "       -t toLang   : specify new language [default \"lang\"]\n";
  print "       -Perl file  : special version of perl\n";
}

sub report {
  print "======= CheckInfZip parameters ==========\n";
  print "Source directory      = ", $rootDir,    "\n";
  print "operation mode        = ", $VERBOSE,    "\n";
  print "toLang                = ", $toLang,     "\n";
  print "platform              = ", $platform,   "\n";
  print "perl                  = ", $perl,       "\n";
print "=========================================\n";
}

sub trim {
  local($String) = @_;
  $String =~ s/^\s*//;
  $String =~ s/\s*$//;
  return $String;
}
