# BEGIN COPYRIGHT BLOCK
# Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
# Copyright (C) 2005 Red Hat, Inc.
# All rights reserved.
# END COPYRIGHT BLOCK
#!/usr/local/bin/perl

#usage GetL10nFiles.pl -s srcDir -o outputDir -v -f filter
# perl GetL10nFiles.pl -s ds4.0/04.27 -o tmp9 -f ds40.flt -v
# perl GetL10nFiles.pl -s ds40_src -o tmp9 -f ds40.flt -l zh 
# filter file contains directories can be ignored
#                      file types need to be localized

#set some default parameters
$VERBOSE    = "noverbose"; 
$lang       = "lang";
$platform   = `uname`;
$servername = "not defined";
$|          = 1;

$platform =~ s/\n$//;
if($platform eq "Windows_NT") { $platform == "WINNT";}
$perl = "";

#parse command line argments

while ($ARGV[0] =~ /^-/) {
    $_ = shift;
    if (/^-s/) {
        $_ = shift;
	$_ =~ /^(\S+)/;
        $sourceDir = $1;
    }
    elsif (/^-o/) {
        $_ = shift;
	$_ =~ /^(\S+)/;
	$outputDir = $1;
    }
    elsif (/^-v/) {
	$VERBOSE = "verbose"; 
    }
    elsif (/^-h/) {
	&Help();
	exit(0);
    }
    elsif (/^-r/) {
	&report();
	exit(0);
    }
    elsif (/^-f/) {
        $_ = shift;
	$_ =~ /^(\S+)/;
	$filter = $1;
	require($filter);
    }
    elsif (/^-l/) {
        $_ = shift;
	$_ =~ /^(\S+)/;
	$lang = $1;
    }
    elsif (/^-Perl/) {
        $_ = shift;
	$_ =~ /^(\S+)/;
	$perl = $1;
    }
    else {
	print "Unrecoginized switch: $_\n";
	&Help();
	exit(0);
    }
}

if (($perl =~ /^$/) && ($platform eq "WINNT")) { $perl = "perl";}

# must specify the source and target directories
if($sourceDir =~ /^$/  || $outputDir =~ /^$/){
    &Help();
    exit(0);
}

# print out operation parameters
&report();

#if the destination directory doesn't exist, then create it
if(opendir(DIR, $outputDir)){
    closedir(DIR);
    if($VERBOSE eq "verbose"){
	print "dir ",  $outputDir, " exists\n";
    }
}else{
    if(MakeDirP($outputDir,0777) != 0){
	print "cannot create destination directory: ", $outputDir, "\n";
	exit(1);
    }else{
	if($VERBOSE eq "verbose"){
	    print "MakeDirP ", $outputDir, "\n";
	}
    }
}

if(!defined(@ignore_dirs) || !defined(@fileTypes)){
    print "array \@ignore_dirs and \@fileTypes are not defined, cannot continue!!\n";
    &Help();
    exit(0);
}

# walk through the entire hierarchy
open(FIND, "find " . $sourceDir . " -follow -print |")   || die " cannot run find: $!\n";  
print "Main loop to examine all files in the hierarchy\n"; 

# copy files of interest to the destination directory
while($filename = <FIND>) {
    chop $filename;
    if(-d $filename){  # Is it a directory?
	next;          # It is a directory, skip it
    }
    # should this file in the ignore list ?
    if(&is_in_ignore_list($filename) == 1){
	next;
    }
    foreach $i (0 .. $#fileTypes){
	if($filename =~ /$fileTypes[$i]$/i){
	    if($filename =~ /\.jar$/i){ #process jar file
		$loc = &createNonExistDirs($filename, $sourceDir, $outputDir, "no_lang_dir");
                `cp -p \"$filename\" \"$loc\"`;
                @where = split('/',$filename);
                $jarname = $where[$#where];
                &process_jar_file($jarname,$loc);
	    }elsif($filename =~ /\.inf$/i || $filename =~ /\.dll$/i){
		$loc = &createNonExistDirs($filename, $sourceDir, $outputDir, "no_lang_dir");
		`cp -p \"$filename\" \"$loc\"`;
	    }elsif($filename =~ /\.zip$/i || $filename =~ /\.z$/i){
		$loc = &createNonExistDirs($filename, $sourceDir, $outputDir, "no_lang_dir");
		&process_zip_file($filename,$loc);
	    }elsif($filename =~ /setup.exe$/i || 
                   $filename =~ /setup$/i     ||
                   $filename =~ /ns-config$/i ||
                   $filename =~ /license.txt$/i){
		$loc = &createNonExistDirs($filename, $sourceDir, $outputDir, "no_lang_dir");
		`cp -p \"$filename\" \"$loc\"`;
	    }elsif($filename =~ /(\.txt)$/i  || 
		   $filename =~ /(\.ldif)$/i || 
		   $filename =~ /(\.bmp)$/i  || 
		   $filename =~ /(\.conf)$/i  || 
		   $filename =~ /(\.tmpl)$/i  || 
		   $filename =~ /(\.htm)$/i  || 
		   $filename =~ /(\.html)$/i){
		$loc = &createNonExistDirs($filename, $sourceDir, $outputDir, "no_lang_dir");
		`cp -p \"$filename\" \"$loc\"`;
		&add_lang_tag($filename,$loc,$1);
	    }else{
		print "============================\n";
		print "cannot handle ", $filename, "\n";
		print "============================\n";
	    }
	}
    }
}

print "Rename all _en.properties to _lang.properties\n";
open(FINDJAR, "find " . $outputDir . " -print |")   || die " cannot run find: $!\n";  
while($propfile = <FINDJAR>) {
    chop $propfile;
    if(-d $propfile){  # Is it a directory?
        next;          # It is a directory, skip it
    }
    if($propfile =~ /_en\.properties$/i){  # Is it an en file?
        $new_name = $propfile;
        $new_trailing_name = "_" . $lang . ".properties";
        $new_name =~ s/_en\.properties$/$new_trailing_name/i;
        rename($propfile, $new_name);
    }
}
close(FINDJAR);

print "Rename all simple .properties to _lang.properties\n";
open(FINDJAR, "find " . $outputDir . " -print |")   || die " cannot run find: $!\n";  
while($propfile = <FINDJAR>) {
    chop $propfile;
    if(-d $propfile){  # Is it a directory?
        next;          # It is a directory, skip it
    }
    if ($propfile =~ /\.properties$/i) {
        $new_trailing_name = "_" . $lang . ".properties";
        if($propfile =~ /$new_trailing_name$/i){  # Is it an lang file?
            next;
        }
        $new_name = $propfile;
        $new_name =~ s/\.properties$/$new_trailing_name/i;
        rename($propfile, $new_name);
    }
}
close(FINDJAR);

print "Move html files to /lang/ sub directory\n";
&make_lang_sub_dir($outputDir);

exit(0);

############## subroutines #######################

sub Help {
    print "Usage: serverL10nKit.pl -s <srcDir> -o <outputDir> -f <filter>\n";
    print "       -h      : printout help\n";
    print "       -r      : printout operation report\n";
    print "       -v      : operation under verbose mode\n";
    print "       -f file : specify filter file name\n";
    print "       -s dir  : specify source directory name\n";
    print "       -o dir  : specify output directory name\n";
    print "       -Perl file : special version of perl\n";
}

sub report {
    print "======= GetL10nFiles parameters =========\n";
    print "Source directory      = ", $sourceDir,  "\n";
    print "output directory      = ", $outputDir,  "\n";
    print "operation mode        = ", $VERBOSE,    "\n";
    print "lang                  = ", $lang,       "\n";
    print "filter                = ", $filter,     "\n";
    print "platform              = ", $platform,   "\n";
    print "server name           = ", $servername, "\n";
    print "perl                  = ", $perl,       "\n";
    print "=========================================\n";

}

sub createNonExistDirs{
    local($path, $src1, $dest1, $flag) = @_;
    local($newdir) = $dest1;
    $path =~ s/$src1\///;
    @dirnames = split('/',$path);
    $newdir = $dest1;
    foreach $i (0 .. $#dirnames-1){
        $newdir = $newdir . "/" . $dirnames[$i];
        if(opendir(DIR, $newdir)){
            closedir(DIR);
        }else{
            mkdir($newdir,0777);
            if($VERBOSE eq "verbose"){
                print "mkdir ", $newdir, "\n";
            }
        }
    }
    if($flag eq "create_lang_dir"){
	# examine the existence of the $lang sub directory, 
	# if necessary create it
	$newdir = $newdir . "/" . $lang;
	if(opendir(DIR, $newdir)){
	    closedir(DIR);
	}else{
	    mkdir($newdir,0777);
	    if($VERBOSE eq "verbose"){
		print "mkdir ", $newdir, "\n";
	    }
	}
    }
    return $newdir;
}

sub MakeDirP{
    local($path) = @_;
    local($rc) = 0;
    local($newdir);
    local(@dirnames);
    @dirnames = split('/',$path);
    $newdir = $dirnames[0];
    if(opendir(DIR, $newdir)){
        closedir(DIR);
    }else{
        mkdir($newdir,0777);
        if($VERBOSE eq "verbose"){
            print "mkdir ", $newdir, "\n";
        }
    }
    foreach $i (1 .. $#dirnames){
        $newdir = $newdir . "/" . $dirnames[$i];
        if(opendir(DIR, $newdir)){
            closedir(DIR);
        }else{
            mkdir($newdir,0777);
            if($VERBOSE eq "verbose"){
                print "mkdir ", $newdir, "\n";
            }
        }
    }
    return($rc);
}

sub process_jar_file{
    local($jarfilename, $newdirname) = @_;
    local($fulljarfile,$newjardir,$propfiles);
    local($wantpropfiles);
    local(@list);
    local($i);
    print "Process jar: $jarfilename\n";
    $fulljarfile = $newdirname . "/" . $jarfilename;
    $fulljarfile =~ s/\\/\//g;
    if($VERBOSE eq "verbose"){
        print "fulljarfile = ", $fulljarfile ,"\n";
    }
    if (&is_in_ignore_list($fulljarfile) == 1) {
        `rm \"$fulljarfile\"`;
        return;
    }
    $newjardir = $fulljarfile;
    $newjardir =~ s/\.jar$//i;
    @ary = split('/',$newjardir);
    $jname = $ary[$#ary];
    if($VERBOSE eq "verbose"){
        print "jname =", $jname, "\n";
        print "newjardir = ", $newjardir, "\n";
    }
    $newjardir =~ s/$jname$/unjar_l10n\/$jname/;
    $l10ndir = $newjardir;
    $l10ndir =~ s/$jname$//;
    if($VERBOSE eq "verbose"){
        print "l10ndir = ", $l10ndir , "\n";
        print "newjardir = ", $newjardir, "\n";
    }
    if($platform =~ /WINNT/i){
	$fulljarfile  =~ s/\//\\/g;
	$l10ndir  =~ s/\//\\/g;
	$newjardir  =~ s/\//\\/g;
	$propfiles = `unzip -Z -1 $fulljarfile *.properties`;
    }else{
	$propfiles = `unzip -Z -1 $fulljarfile \\*.properties`;
    }
    if($propfiles =~ /^$/){
	if($VERBOSE eq "verbose"){
	    print $pathname, " contains no interested files, skip it.\n";
	}
	`rm \"$fulljarfile\"`;
	return;
    }

    # This section ignores directories in @ignore_dirs
    $propfiles =~ s/\n$//;
    @list = split(' ', $propfiles);
    foreach $i (0 .. $#list){
        # is this file in the ignore list ?
        if(&is_in_ignore_list($list[$i]) == 1){
            next;
        }
        $wantpropfiles = $wantpropfiles . " " . $list[$i];
    }
    $propfiles = $wantpropfiles;
    if($propfiles =~ /^$/){
        if($VERBOSE eq "verbose"){
            print $pathname, " contains no interesting files, skip it.\n";
        }
        `rm \"$fulljarfile\"`;
        return;
    }

    mkdir($l10ndir,0777);
    mkdir($newjardir,0777);
    $propfiles =~ s/\n/ /g;
    #print "unzip -d $newjardir $fulljarfile $propfiles\n";
    `unzip $fulljarfile $propfiles -d $newjardir `;
    `rm \"$fulljarfile\"`;
}

sub add_lang_tag {
    local($pathname,$loc,$filetype) = @_;
    local($oldname, $newname, $new_trailing_name,$newtag);
    $new_trailing_name = "_" . $lang . $filetype;
    @dirnames = split('/',$pathname);
    $filename = $dirnames[$#dirnames];
    $oldname = $loc . "/" . $filename;
    $newname = $oldname;
    $newtag = $filetype;
    $newname =~ s/$newtag$/$new_trailing_name/i;
    #print "rename($oldname, $newname) \n";
    rename($oldname, $newname);
}

sub process_zip_file{
    local($pathname, $outputDir) = @_;
    local($newdirname) = "";
    local(@list);
    local($i);
    print "Process zip: $pathname\n";
    @dirnames = split('/',$pathname);
    $zipname = $dirnames[$#dirnames];
    $zipnamenotag = $zipname;
    $zipnamenotag =~ s/\.zi?p?$//i;
    $zipnamenotag = $zipnamenotag . "_" . $lang;
    if($zipname =~ /\.z$/){
	$l10ndir = $outputDir . "/unz_l10n";
    }else{
	$l10ndir = $outputDir . "/unzip_l10n";
    }
    $newdirname = $l10ndir . "/" .$zipnamenotag;
    if($VERBOSE eq "verbose"){
        print "l10ndir = ", $l10ndir ,"\n";
        print "newdirname = ",$newdirname ,"\n";
    }
    if($platform =~ /WINNT/i){
	$l10ndir  =~ s/\//\\/g;
	$newdirname  =~ s/\//\\/g;
	$files_interested = `unzip -Z -1 $pathname *.conf *.tmpl *.html *.htm *.gif *.jpg *.jpeg *.bmp *.jar *.properties `;
# Hack to get around the space in Full Outline.htm
# Replace space with _ to get over the split on space below
	$files_interested =~ s/synch\/Full Outline.htm/synch\/Full_Outline.htm/;
    }else{
	$files_interested = `unzip -Z -1 $pathname \\*.conf \\*.tmpl \\*.html \\*.htm \\*.gif \\*.jpg \\*.jpeg \\*.bmp \\*.jar \\*.properties`;
    }
    if($VERBOSE eq "verbose"){
	print $files_interested, "\n";
    }
    if($files_interested =~ /^$/){
        if($VERBOSE eq "verbose"){
            print $pathname, " contains no interesting files, skip it.\n";
        }
        return;
    }    
    mkdir($l10ndir,0777);
    mkdir($newdirname,0777);
    if($VERBOSE eq "verbose"){
	print "process zip file: ", $zipname, " at ", $newdirname, "\n";
    }
    @list = split(' ', $files_interested);
    foreach $i (0 .. $#list){
        # is this file in the ignore list ?
        if(&is_in_ignore_list($list[$i]) == 1){
            next;
        }
        # The NT platform requires all '/' to be changed to '\'
        if($platform =~ /WINNT/i){
            $list[$i]  =~ s/\//\\/g;
        }
# Part two of the Hack for the Space in Full Outline.htm
# Extract the errant file on its own ...
	if($list[$i] =~ /Full_Outline.htm/) {
	    `unzip $pathname "synch/Full Outline.htm" -d $newdirname`;
	} else {
	    `unzip  $pathname $list[$i] -d $newdirname`;
	}
        if($list[$i] =~ /\.jar$/i){
            process_jar_file($list[$i],$newdirname);
        }
    }
}

sub is_in_ignore_list {
    local($fname) = @_;
    foreach $i (0 .. $#ignore_dirs){
	if($fname =~ /$ignore_dirs[$i]$/i){
	    if($VERBOSE eq "verbose"){
		print "ignore ", $fname, "\n";
	    }
	    return 1;
	}
    }
    return 0;
}

sub make_lang_sub_dir {
    local($wdir) = @_;
    local($underscore);
    local($type);
    open(WDIR, "find " . $wdir . " -print |")   || die " cannot run find: $!\n"; 
    while($filename = <WDIR>) {
      chop $filename;
      if(-d $filename ){  # Is it a directory?
          if($filename =~ /\/en$/){
              $newname = $filename;
              $newname =~ s/\/en$/\/$lang/i;
              #print "rename($filename,$newname)\n";
              rename($filename,$newname);
          }
          next;          # It is a directory, skip it
      }
      # change all /en/ to /$lang/ since actual directories have been renamed
      $filename =~ s/\/en\//\/$lang\//i;

      # if /$lang/ is in the path do nothing
      if($filename =~ /\/$lang\//){ 
	  next;
      }

      foreach $i (0 .. $#fileTypes_2){
          $type = $fileTypes_2[$i];
          $type =~ s/^\\\.//;
          $underscore = "_" . $lang . "." . $type;
          if($filename =~ /$underscore$/i){
              next;         # skip if it's already _lang
          }
          if($filename =~ /$fileTypes_2[$i]$/i){
              $loc = &createNonExistDirs($filename, $wdir, $wdir, "create_lang_dir");
              if($VERBOSE eq "verbose"){
                  print "mv $filename $loc\n";
              }
              `mv \"$filename\" $loc`;
          }
      }
  }
}
