# BEGIN COPYRIGHT BLOCK
# Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
# Copyright (C) 2005 Red Hat, Inc.
# All rights reserved.
# END COPYRIGHT BLOCK
#!/usr/local/bin/perl

#usage LocaleToLocale.pl -s srcDir -f fromLang -t toLang

#set some default parameters
$VERBOSE    = "noverbose";
$fromLang   = "";
$toLang     = "lang";
$platform   = `uname`;
$servername = "not defined";
$|          = 1;

$platform =~ s/\n$//;
if($platform eq "Windows_NT") { $platform == "WINNT";}
$perl = "";

#remember current working directory
$CWD = `pwd`;
chop($CWD);

#parse command line argments

while ($ARGV[0] =~ /^-/) {
  $_ = shift;
  if (/^-s/) {
    $_ = shift;
    $_ =~ /^(\S+)/;
    $rootDir = $1;
  }
  elsif (/^-v/) {
    $VERBOSE = "verbose";
  }
  elsif (/^-h/) {
    &Help();
    exit(0);
  }
  elsif (/^-r/) {
    &report();
    exit(0);
  }
  elsif (/^-f/) {
    $_ = shift;
    $_ =~ /^(\S+)/;
    $fromLang = $1;
  }
  elsif (/^-t/) {
    $_ = shift;
    $_ =~ /^(\S+)/;
    $toLang = $1;
  }
  elsif (/^-Perl/) {
    $_ = shift;
    $_ =~ /^(\S+)/;
    $perl = $1;
  }
  else {
    print "Unrecoginized switch: $_\n";
    &Help();
    exit(0);
  }
}

if (($perl =~ /^$/) && ($platform eq "WINNT")) { $perl = "perl";}

# must specify the source
if($rootDir =~ /^$/){
  &Help();
  exit(0);
}

# print out operation parameters
&report();

chdir("$rootDir") || die "cannot change directory to $rootDir\n"; ;

if($VERBOSE eq "verbose"){
  print `pwd`,"\n";
  print `ls -l`,"\n";
}

# look for all yyy_$fromLang.type files, change to yyy_$toLang.type
print "Rename all yyy_$fromLang.type to yyy_$toLang.type \n";
open(FIND, "find . |") || die " cannot run find $fromLang files: $!\n";
while($fromFile = <FIND>) {
    chop $fromFile;
    if (-d $fromFile){
	next;
    }
    if($fromFile =~ /_$fromLang\.[a-z0-9]*$/i ) {
	$toFile = $fromFile;
	$toFile =~ s/_$fromLang\.([a-z]*)$/_$toLang\.$1/i;
        rename ($fromFile,$toFile);
    }
}
close(FIND);

# walk through the all $fromLang directories
print "Rename all $fromLang directories to $toLang\n";
open(FIND, "find . -type d -name $fromLang -print | sort -r |") || die " cannot run find directories: $!\n";
while($fromDir = <FIND>) {
  chop $fromDir;
  $toDir = langChanger($fromDir, $fromLang, $toLang);
  rename ($fromDir,$toDir);
}
close(FIND);

# walk through the all *_$fromLang directories
print "Rename all *_$fromLang directories to *_$toLang\n";
$findString = "\'*_" . $fromLang . "\'";
open(FIND, "find . -type d -name $findString -print |sort -r |") || die " cannot run find directories: $!\n";
while($fromDir = <FIND>) {
  chop $fromDir;
  $toDir = langChanger($fromDir, $fromLang, $toLang);
  rename ($fromDir,$toDir);
}
close(FIND);

chdir("$CWD") || die "cannot change directory to $CWD\n"; ;

# adjust the inf files
print "Adjust the inf files\n";
print `$perl AdjustInf.pl -s $rootDir -f $fromLang -t $toLang -Perl $perl`;

print "LocaleToLocale Done\n";
exit(0);

############## subroutines #######################
sub Help {
  print "Usage: LocaleToLocale.pl -s <srcDir>\n";
  print "       -h          : printout help\n";
  print "       -r          : printout operation report\n";
  print "       -v          : operation under verbose mode\n";
  print "       -s dir      : specify source directory name\n";
  print "       -f fromLang : specify old language [default \"\"]\n";
  print "       -f toLang   : specify new language [default \"lang\"]\n";
  print "       -Perl file  : special version of perl\n";
}

sub report {
  print "====== LocaleToLocale parameters ========\n";
  print "Source directory      = ", $rootDir,    "\n";
  print "operation mode        = ", $VERBOSE,    "\n";
  print "fromLang              = ", $fromLang,   "\n";
  print "toLang                = ", $toLang,     "\n";
  print "platform              = ", $platform,   "\n";
  print "perl                  = ", $perl,       "\n";
  print "=========================================\n";
}

sub trim {
  local($String) = @_;
  $String =~ s/^\s*//;
  $String =~ s/\s*$//;
  return $String;
}

sub langChanger {
  local($String, $from, $to) = @_;
  if ( $from ne "" ) {
    if ( $String =~ /$from$/ ) {
      $String =~ s/$from$/$to/;
    } else {
      print "Error: In $String, $from can not be changed to $to.\n";
      exit(1);
    }
  } else {
    $String =~ s/$/$to/;
  }
  return $String;
}

sub langFileChanger {
  local($String, $from, $to) = @_;
  local($fromFile) = "_" . $from . "\.";
  local($toFile) = "_" . $to . ".";
  if ( $from ne "" ) {
    if ( $String =~ /$fromFile/ ) {
      $String =~ s/$fromFile/$toFile/;
    } else {
      print "Error: In $String, $from can not be changed to $to.\n";
      exit(1);
    }
  }
  return $String;
}

sub langPropChanger {
  local($String, $from, $to) = @_;
  local($fromProp) = "_" . $from . "\.properties\$";
  local($toProp) = "_" . $to . ".properties";
  local($noProp) = "\.properties\$";
  if ( $from ne "" ) {
    if ( $String =~ /$fromProp/ ) {
      $String =~ s/$fromProp/$toProp/;
    } else {
      print "Error: In $String, $from can not be changed to $to.\n";
      exit(1);
    }
  } else {
    $String =~ s/$noProp/$toProp"/;
  }
  return $String;
}
