# BEGIN COPYRIGHT BLOCK
# Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
# Copyright (C) 2005 Red Hat, Inc.
# All rights reserved.
# END COPYRIGHT BLOCK
#!/usr/local/bin/perl

# Need for Solaris:

# PATH 
#   /share/builds/components/jdk/1.1.6/SunOS/bin
#   /share/builds/components/libnls31/unoffical/19980513/SunOS5.5.1_sparc_OPT.OBJ/tools

# CLASSPATH
#   /u/cxue/pseudoall

#set some default parameters
$L10nPackageLang   = "lang";
$PseudoPackageLang = "xx";
$VERBOSE           = "noverbose"; 
$platform          = `uname`;
$extrapath         = "";
$|                 = 1;

$platform =~ s/\n$//;
if($platform eq "Windows_NT") { $platform == "WINNT";}
$perl = "";

#parse command line argments

while ($ARGV[0] =~ /^-/) {
    $_ = shift;
    if (/^-u/) {
        $_ = shift;
	$_ =~ /^(\S+)/;
        $USProductDir = $1;
    }
    elsif (/^-o/) {
        $_ = shift;
	$_ =~ /^(\S+)/;
	$L10nPackageDir = $1;
    }
    elsif (/^-p/) {
        $_ = shift;
	$_ =~ /^(\S+)/;
	$PseudoPackageDir = $1;
    }
    elsif (/^-v/) {
	$VERBOSE = "verbose"; 
    }
    elsif (/^-h/) {
	&Help();
	exit(0);
    }
    elsif (/^-r/) {
	&report();
	exit(0);
    }
    elsif (/^-f/) {
        $_ = shift;
	$_ =~ /^(\S+)/;
	$Filter = $1;
    }
    elsif (/^-R/) {
        $_ = shift;
	$_ =~ /^(\S+)/;
	$Rules = $1;
    }
    elsif (/^-Perl/) {
        $_ = shift;
	$_ =~ /^(\S+)/;
	$perl = $1;
    }
    elsif (/^-jre/) {
        $_ = shift;
	$_ =~ /^(\S+)/;
	$jre = $1;
    }
    elsif (/^-classpath/) {
        $_ = shift;
	$_ =~ /^(\S+)/;
	$classpath = $1;
    }
    elsif (/^-extrapath/) {
        $_ = shift;
	$_ =~ /^(\S+)/;
	$extrapath = $1;
    }
    else {
	print "Unrecoginized switch: $_\n";
	&Help();
	exit(0);
    }
}

if (($perl =~ /^$/) && ($platform eq "WINNT")) { $perl = "perl";}

# must specify the source and target directories
if ($USProductDir =~ /^$/) {
    print "Error: missing source directory for US Server Product\n";
    &Help();
    exit(0);
}
if ($L10nPackageDir =~ /^$/) {
    print "Error: missing target directory for L10N Package\n";
    &Help();
    exit(0);
}
if ($PseudoPackageDir =~ /^$/) {
    print "Error: missing target directory for Pseudo Package\n";
    &Help();
    exit(0);
}

# must specify the filter and rules files
if ($Filter =~ /^$/) {
    print "Error: missing filter filename\n";
    &Help();
    exit(0);
}
if ($Rules =~ /^$/) {
    print "Error: missing rules filename\n";
    &Help();
    exit(0);
}

# print out operation parameters
&report();

# Get the L10N Open Package
print "Get the L10N Open Package\n";
print `rm -rf \"$L10nPackageDir\"`;
if(-d $L10nPackageDir){
    print "***************** error ****** error *********** \n";
    print "Cannot delete $L10nPackageDir, operation terminated..\n";
    print "Please make sure $L10nPackageDir doesn't exist \n";
    exit(1);
}
print `$perl GetL10nFiles.pl -s $USProductDir -o $L10nPackageDir -f $Filter -l $L10nPackageLang -Perl $perl`;
print `$perl AdjustInf.pl -s $L10nPackageDir -t $L10nPackageLang -Perl $perl`;

# Copy and make Pseudo Package
print "Copy and make Pseudo Package\n";
print `rm -rf \"$PseudoPackageDir\"`;
if(-d $PseudoPackageDir){
    print "***************** error ****** error *********** \n";
    print "Cannot delete $PseudoPackageDir, operation terminated..\n";
    print "Please make sure $PseudoPackageDir doesn't exist \n";
    exit(1);
}
print `cp -R \"$L10nPackageDir\" \"$PseudoPackageDir\"`;
print `$perl LocaleToLocale.pl -s $PseudoPackageDir -f $L10nPackageLang -t $PseudoPackageLang -Perl $perl`;

# add extra path to the environment, 
# so PseudoAll can find htmlmerge and htmlsplit
# this is required on NT, since htmlmerge and htmlsplit are ftped over
if(! $extrapath =~/^$/){
    $ppp = $ENV{PATH};
    $ppp =  $extrapath . ";" . $ppp;
    print "new path = ", $ppp, "\n";
    $ENV{PATH} = $ppp;
}

print "$jre -cp $classpath PseudoAll $PseudoPackageDir $PseudoPackageDir -R $Rules -base $PseudoPackageLang -lang $PseudoPackageLang -inline \n";
print `$jre -cp $classpath PseudoAll $PseudoPackageDir $PseudoPackageDir -R $Rules -base $PseudoPackageLang -lang $PseudoPackageLang -inline`;

# Make the Installable Packages
print "Make the Installable Packages\n";
print `$perl PackageL10n.pl -s $L10nPackageDir -l $L10nPackageLang -Perl $perl`;
print `$perl CheckInfZip.pl -s $L10nPackageDir -t $L10nPackageLang -Perl $perl`;
print `$perl PackageL10n.pl -s $PseudoPackageDir -l $PseudoPackageLang -Perl $perl`;
print `$perl CheckInfZip.pl -s $PseudoPackageDir -t $PseudoPackageLang -Perl $perl`;

# Done
print "MakeL10nPkg complete.\n";

exit(0);

############## subroutines #######################

sub Help {
    print "Usage: MakeL10nPkg.pl -u usDir -o langDir -p xxDir -f filter -R rules\n";
    print "       -h         : printout help\n";
    print "       -r         : printout operation report\n";
    print "       -v         : operation under verbose mode\n";
    print "       -u usDir   : directory for US Server Product (input)\n";
    print "       -o langDir : directory for L10N Package (output)\n";
    print "       -p xxDir   : directory for Pseudo Package (output)\n";
    print "       -f filter  : filter file for file selection\n";
    print "       -R rules   : rules file for htmlsplit\n";
    print "       -Perl file : special version of perl\n";
}

sub report {
    print "======== MakeL10nPkg parameters =========\n";
    print "US Server Product Dir = ", $USProductDir,    "\n";
    print "L10N Package Dir      = ", $L10nPackageDir,  "\n";
    print "Pseudo Package Dir    = ", $PseudoPackageDir,"\n";
    print "operation mode        = ", $VERBOSE,         "\n";
    print "filter file           = ", $Filter,          "\n";
    print "rules file            = ", $Rules,           "\n";
    print "platform              = ", $platform,        "\n";
    print "perl                  = ", $perl,            "\n";
    print "=========================================\n";

}
