# BEGIN COPYRIGHT BLOCK
# Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
# Copyright (C) 2005 Red Hat, Inc.
# All rights reserved.
# END COPYRIGHT BLOCK
#!/usr/local/bin/perl

#perl PackageL10n.pl -s src_dir -l lang
# first path: zip up zzz_jar to zzz_lang.jar
# second path: 
#    if xxx_lang is a file   -> xxx
#    if xxx_lang is a dir    -> xxx.z (on NT) xxx.zip (on UNIX)

#set some default parameters
$VERBOSE    = "noverbose"; 
$lang       = "lang";
$platform   = `uname`;
$servername = "not defined";
$|          = 1;

$platform =~ s/\n$//;
if($platform eq "Windows_NT") { $platform == "WINNT";}
$perl = "";

$pwddir = `pwd`;
$pwddir =~ s/\n//;

#parse command line argments

while ($ARGV[0] =~ /^-/) {
    $_ = shift;
    if (/^-s/) {
        $_ = shift;
	$_ =~ /^(\S+)/;
        if ($1 =~ /^\//) {
            # absolute path (starts with /)
            $sourceDir = "$1";
        }
        else {
            # relative path
            $sourceDir = contractPath("$pwddir/$1");
        }
    }
    elsif (/^-v/) {
	$VERBOSE = "verbose"; 
    }
    elsif (/^-h/) {
	&Help();
	exit(0);
    }
    elsif (/^-r/) {
	&report();
	exit(0);
    }
    elsif (/^-l/) {
        $_ = shift;
	$_ =~ /^(\S+)/;
	$lang = $1;
    }
    elsif (/^-Perl/) {
        $_ = shift;
	$_ =~ /^(\S+)/;
	$perl = $1;
    }
    else {
	print "Unrecoginized switch: $_\n";
	&Help();
	exit(0);
    }
}

if (($perl =~ /^$/) && ($platform eq "WINNT")) { $perl = "perl";}

# must specify the source and target directories
if($sourceDir =~ /^$/){
    &Help();
    exit(0);
}

# print out operation parameters
&report();

#==========jar up *.jar ====================================\n";
&jar_up_directory();

#==========zip up *.z ====================================\n";
&zip_up_directory();

chdir($pwddir);
#delete all unjar_l10n, unz_l10n, unzip_l10n directories
open(FIND_l10n, "find " . $sourceDir . " -print |")   || die " cannot run find: $!\n";  
while($filename = <FIND_l10n>) {
    chop $filename;
    if($filename =~ /\/unjar_l10n$/i ||
       $filename =~ /\/unz_l10n$/i ||
       $filename =~ /\/unzip_l10n$/i){
	`rm -rf \"$filename\"`;
    }
}

print "PackageL10n complete\n";

exit(0);

############## subroutines #######################

sub jar_up_directory {
    local(@to_be_delete);
    open(FIND1, "find " . $sourceDir . " -print |")   || die " cannot run find: $!\n";  
    $newtag = "_" . $lang . ".jar";
    while($filename = <FIND1>) {
	chop $filename;
	$newdir = $filename;
	if($newdir =~ /unjar_l10n\/\w+$/i){
	    if($VERBOSE eq "verbose"){
	       print "process $filename \n";
	       print "newdir = ", $newdir, "\n";
	   }
	    chdir($newdir);
	    $filename =~ s/$/$newtag/i;
	    @where = split('/',$filename);
	    $jarname = "../../" . $where[$#where];  
	    if($VERBOSE eq "verbose"){
	       print "jar -cMf $jarname *", "\n";
	   }
	    `jar -cMf $jarname *`;
	    @to_be_delete = (@to_be_delete, $newdir);
	}
    }
    close(FIND1);
    chdir($pwddir);
    &delete_dirs(@to_be_delete);
}

# zip up all  xxx_lang -> xxx.z (on NT) xxx.zip (on UNIX)
sub zip_up_directory{
    local(@to_be_delete);
    chdir($pwddir);
    chdir($fileCount);
    chdir($shortname);
    open(FIND3, "find " . $sourceDir . " -print |")   || die " cannot run find: $!\n";
    $langtag = "_" . $lang;
    while($filename = <FIND3>) {
	chop $filename;
	$newdir = $filename;
	if($newdir =~ /unzi?p?_l10n\/\w+$/i){
	    if(-d $newdir){
		if($VERBOSE eq "verbose"){
		   print "process directry $filename \n";
	       }
		if($newdir =~ /unz_l10n/i){
		    $zipfilename = $newdir . ".z";
		}else{
		    $zipfilename = $newdir . ".zip";
		}
		chdir($newdir);
		$zipfilename =~ s/unzi?p?_l10n\///;
		if($VERBOSE eq "verbose"){
		  print "newdir = ", $newdir, "\n";
		  print "zip -r -T $zipfilename * \n";
	      }
		&delete_unjar_dirs(".");
                $fileCount = count_files_for_zip(".");
                $shortname = $zipfilename;
                # get last 2 levels (sub directory and file name)
                $shortname =~ s/^.*(\/[^\/\s]+\/[^\/\s]+)$/$1/;
                if ($fileCount>0) {
                    print "$fileCount files in $shortname\n";
                    `zip -r -T $zipfilename *`;
                }
                else {
                    print "No files for $shortname; no file created\n";
                }
		@to_be_delete = (@to_be_delete, $newdir);
	    }
	}
    }
    close(FIND3);
    chdir($pwddir);
    &delete_dirs(@to_be_delete);
    #delete all unjar_l10n directories
    &delete_unjar_dirs(".");
}

sub Help {
    print "Usage: PackageL10n.pl -s <srcDir> \n";
    print "       -h      : printout help\n";
    print "       -r      : printout operation report\n";
    print "       -v      : operation under verbose mode\n";
    print "       -l      : language tag\n";
    print "       -s dir  : specify source directory name\n";
    print "       -Perl file : special version of perl\n";
}

sub report {
    print "======= PackageL10n parameters ==========\n";
    print "Source directory      = ", $sourceDir,  "\n";
    print "operation mode        = ", $VERBOSE,    "\n";
    print "lang                  = ", $lang,       "\n";
    print "platform              = ", $platform,   "\n";
    print "perl                  = ", $perl,       "\n";
    print "=========================================\n";

}

sub delete_dirs {
    local(@dirs) = @_;
    foreach $i (0 .. $#dirs){
	if($VERBOSE eq "verbose"){
	    print "rm -rf \"$dirs[$i]\" \n";
	}
	`rm -rf \"$dirs[$i]\"`;
    }
}

sub delete_unjar_dirs {
    local($dir) = @_;
    open(FIND_jar, "find " . $dir . " -print |")   || die " cannot run find: $!\n";  
    while($filename = <FIND_jar>) {
       chop $filename;
       if($filename =~ /\/unjar_l10n$/i){
	   if($VERBOSE eq "verbose"){
	       print "rm -rf \"$filename\" \n";
	   }
	   `rm -rf \"$filename\"`;
    }
   }
    close(FIND_jar);
}

sub count_files_for_zip {
    local($dir) = @_;
    local($count) = 0;
    open(FIND_jar, "find " . $dir . " -type f -print |")   || die " cannot run find: $!\n";  
    while($filename = <FIND_jar>) {
        $count = $count+1;
    }
    close(FIND_jar);
    return $count;
}

sub contractPath {
    local($path) = @_;
    while ($path =~ s/[^\/\s]+\/\.\.\///) {};     # remove abcxyz/../
    return $path;
}
