# BEGIN COPYRIGHT BLOCK
# Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
# Copyright (C) 2005 Red Hat, Inc.
# All rights reserved.
# END COPYRIGHT BLOCK
#!/usr/local/bin/perl

#perl UnpackageL10n.pl -s src_dir -l lang
# first path: zip up zzz_jar to zzz_lang.jar
# second path: 
#    if xxx_lang is a file   -> xxx
#    if xxx_lang is a dir    -> xxx.z (on NT) xxx.zip (on UNIX)

#set some default parameters
$VERBOSE    = "noverbose"; 
$lang       = "lang";
$platform   = `uname`;
$servername = "not defined";
$|          = 1;

$platform =~ s/\n$//;
if($platform eq "Windows_NT") { $platform == "WINNT";}
$perl = "";

$pwddir = `pwd`;
$pwddir =~ s/\n//;

#parse command line argments

while ($ARGV[0] =~ /^-/) {
    $_ = shift;
    if (/^-s/) {
        $_ = shift;
	$_ =~ /^(\S+)/;
        if ($1 =~ /^\//) {
            # absolute path (starts with /)
            $sourceDir = "$1";
        }
        else {
            # relative path
            $sourceDir = contractPath("$pwddir/$1");
        }
    }
    elsif (/^-v/) {
	$VERBOSE = "verbose"; 
    }
    elsif (/^-h/) {
	&Help();
	exit(0);
    }
    elsif (/^-r/) {
	&report();
	exit(0);
    }
    elsif (/^-l/) {
        $_ = shift;
	$_ =~ /^(\S+)/;
	$lang = $1;
    }
    elsif (/^-Perl/) {
        $_ = shift;
	$_ =~ /^(\S+)/;
	$perl = $1;
    }
    else {
	print "Unrecoginized switch: $_\n";
	&Help();
	exit(0);
    }
}

if (($perl =~ /^$/) && ($platform eq "WINNT")) { $perl = "perl";}

# must specify the source and target directories
if($sourceDir =~ /^$/){
    &Help();
    exit(0);
}

# print out operation parameters
&report();

#==========zip down *.z ====================================\n";
&zip_down_directory();

#==========jar down *.jar ==================================\n";
&jar_down_directory();

print "UnpackageL10n complete.\n";

exit(0);

############## subroutines #######################

sub jar_down_directory {
    local(@to_be_delete);
    open(FIND1, "find " . $sourceDir . " -print |")   || die " cannot run find: $!\n";  
    $newtag = "_" . $lang . ".jar";
    while($filename = <FIND1>) {
	chop $filename;
	if(-d $filename){
	    next;
	}
	$newdir = $filename;
	if($newdir =~ /$newtag$/i){
	    if($VERBOSE eq "verbose"){
	       print "process $newdir \n";
	   }
	    @ary = split('/',$newdir);
	    $jname = $ary[$#ary];
	    $fulljname = $jname;
	    $jname =~ s/$newtag$//i;
	    $newdirname = $newdir;
	    $newdirname =~ s/$fulljname$/unjar_l10n\/$jname/;
	    $l10ndir = $newdirname;
	    $l10ndir =~ s/$jname$//;
	    mkdir($l10ndir,0777);
	    mkdir($newdirname,0777);
            chdir($newdirname);
	    if($VERBOSE eq "verbose"){
		print "jname = ", $jname, "\n";
		print "newdirname = ", $newdirname, "\n";
		print "jar -xMf  $newdir \n";
	    }
	    `jar -xMf  $newdir`;
            `rm -f \"$newdir\"`;
	}
    }
    close(FIND1);
    chdir($pwddir);
}

# zip up all  xxx_lang -> xxx.z (on NT) xxx.zip (on UNIX)
sub zip_down_directory{
    local(@to_be_delete);
    chdir($pwddir);
    open(FIND3, "find " . $sourceDir . " -print |")   || die " cannot run find: $!\n";
    while($filename = <FIND3>) {
	chop $filename;
	if(-d $filename){
	    next;
	}
	$newdir = $filename;
	if($newdir =~ /\.zi?p?$/i){
	    if($VERBOSE eq "verbose"){
	      print "process file $newdir\n";
	  }
	    @dirnames = split('/',$newdir);
	    $zipname = $dirnames[$#dirnames];
	    $zipnamenotag = $zipname;
	    $zipnamenotag =~ s/\.zi?p?$//i;
	    $l10ndir = $newdir;
	    if($zipname =~ /\.z$/){
		$l10ndir =~ s/$zipname$/unz_l10n/;
	    }else{
		$l10ndir =~ s/$zipname$/unzip_l10n/;
	    }
	    mkdir($l10ndir,0777);
	    $newdirname = $l10ndir . "/" . $zipnamenotag;
	    mkdir($newdirname,0777);
           if($VERBOSE eq "verbose"){
	      print "unzip  $newdir  -d $newdirname \n";
	  }
	   `unzip  $newdir  -d $newdirname`;
           `rm -f \"$newdir\"`;
	    }
	}
    close(FIND3);
    chdir($pwddir);
}

sub Help {
    print "Usage: UnpackageL10n.pl -s <srcDir> \n";
    print "       -h      : printout help\n";
    print "       -r      : printout operation report\n";
    print "       -v      : operation under verbose mode\n";
    print "       -l      : language tag\n";
    print "       -s dir  : specify source directory name\n";
    print "       -Perl file : special version of perl\n";
}

sub report {
    print "====== UnpackageL10N parameters =========\n";
    print "Source directory      = ", $sourceDir,  "\n";
    print "operation mode        = ", $VERBOSE,    "\n";
    print "lang                  = ", $lang,       "\n";
    print "platform              = ", $platform,   "\n";
    print "perl                  = ", $perl,       "\n";
    print "=========================================\n";

}

sub delete_dirs {
    local(@dirs) = @_;
    foreach $i (0 .. $#dirs){
	if($VERBOSE eq "verbose"){
	    print "rm -rf \"$dirs[$i]\" \n";
	}
	`rm -rf \"$dirs[$i]\"`;
    }
}

sub contractPath {
    local($path) = @_;
    while ($path =~ s/[^\/\s]+\/\.\.\///) {};     # remove abcxyz/../
    return $path;
}
