/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
class PropertiesParsing {
    public static final int INVALIDLINE = 0;
    public static final int COMMENTLINE = 1;
    public static final int FIRSTLINE = 2;
    public static final int FIRSTLINEFINISH = 3;
    public static final int NEXTLINE = 4;
    public static final int NEXTLINEFINISH = 5;
    
    public String key = null;
    public String value = null;
    public int status = 0;
    
    static int id = 1;
    static String stringId = null;
    /*********************************
      return 0:  success
    **********************************/
    public static int tokenNumbering(String line, StringBuffer newline) {
        int i;
        char ch;
        String remain;
        String token;
        // check comment line
        for (i = 0; i < line.length(); i ++) {
            ch = line.charAt(i);
            if (ch == ' ' || ch == '\t')
                continue;
            if (ch == '#') {
                newline.append(line);
                return 0;
            }
            else
                break;
        }
        // empty line
        if (i == line.length()) {
            newline.append(line);
            return 0;
        }        
        
        token = getToken(line);
        if (token != null) {
            remain = getRemaining(line);
            if ((stringId == null) ||
                (token.compareTo(stringId) != 0)) {
                id = 1;
                stringId = token;
            }
            newline.append(token + '-' + id + " = " + remain);
            id = id + 1;
            return 0;
        }
        newline.append(line);
        return 0;            
    }
    
    public static String getToken (String line) {
        String src, s;
        int n;
        
        // trimming spaces on both end
        src = line.trim();

        // searching for = sign
        if ((n = src.indexOf("=")) == -1)
            return null;
            
//        remaining = src.substring(n + 1, src.length());
            
        // if no space from beginning to '=' sign, take it.
        s = (src.substring(0, n)).trim();
        if (s.indexOf (" ") != -1)
            return null;     
        return s;
    }

    /******************************
        return ""
    *******************************/
    public static String getRemaining (String line) {
        int n;
        
        // searching for = sign
        if ((n = line.indexOf("=")) == -1)
            return "";
            
        return line.substring(n + 1, line.length());
    }
    
    /*
        return:  value of property 
        
        set class value
            status 0:  invalid property line
                   1:  single line 
                   2:  multi-line
            key: key value
     */
    public String parseLine(String line) {        
        int len = line.length();
        
        if (len <= 0) {
        	value = "";
        	status = INVALIDLINE;
        	return "";
        }
        if ((status != NEXTLINE) && (status != FIRSTLINE)) {
            String src, s;
            int n;
        
            // trimming spaces on both end
            src = line.trim();

			if (src.charAt(0) == '#') {
                status = COMMENTLINE;
                return line;								
			}
			
            // searching for '=' sign
            if ((n = src.indexOf("=")) == -1) {
                status = INVALIDLINE;
                return line;
            }
            
            // if no space from beginning to '=' sign, take it.
            key = (src.substring(0, n)).trim();
            if (key.indexOf (" ") != -1) {
                status = INVALIDLINE;
                return line;
            }
            
            // Looking for line continue sign
            if (line.charAt(len-1) == '\\') {
                value = src.substring(n + 1, src.length()-1);
            	status = FIRSTLINE;
            }
            else {
                value = src.substring(n + 1, src.length());
            	status = FIRSTLINEFINISH;
            }
          
            return value;                                               
        }
        else {
            // Looking for line continue sign
            if (line.charAt(len-1) == '\\') {
                value = line.substring(0, len - 1);
                status = NEXTLINE;
            }
            else {
                value = line;
                status = NEXTLINEFINISH;
            }
          
            return value;
        }                
    }    
}
