/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
import java.io.*;
import java.lang.*;
import java.util.*;

import PseudoProperties;

class PseudoAll {
	static final int L10N_SAMEDIRECTORY = 1;
	static final int L10N_SUBDIRECTORY = 2;

	static final int PLATFORM_WINDOWS = 1;
	static final int PLATFORM_UNIX = 2;

	static final int PSEUDO_DIRECTORYTREE = 1;
	static final int PSEUDO_FILE = 2;


    String g_srcDir;
    String g_destDir;
    static String g_language = "xx";
    static String g_fileSeparator = System.getProperty("file.separator");
	static String g_os = System.getProperty("os.name");
	String g_home = System.getProperty("user.dir");
	static String g_rulesFile = "htmlsplit.rules";
	static boolean g_nlsTrans = false;
	static boolean g_inline = false;
	static int g_platform = PLATFORM_WINDOWS;
	static boolean g_debug = false;
	static boolean g_htmltrans = false;
	static FileOutputStream g_nlsLogFile = null;
	static String g_nlsLogFileName = null;
	static FileOutputStream g_logFile = null;
	static String g_glossary = null;
	static String g_baseLang = null;
	static boolean g_baseDefault = true;
	static String g_baseLangs[] = { "en", "lang" } ;
	static String g_ignoreLangs[] = { "en-US", "fr", "ja", "de", "es" } ;

    PseudoAll(String srcDir, String destDir, int type)
    {
        g_destDir = destDir;
        if (type == PSEUDO_DIRECTORYTREE) {
	        DebugPrintln("tree: srcDir = " + srcDir, "info");
	        DebugPrintln("tree: g_destDir = " + g_destDir, "info");

            g_srcDir = srcDir;
            pseudo();
        }
        else {
	        DebugPrintln("file: srcDir = " + srcDir, "info");
	        DebugPrintln("file: g_destDir = " + g_destDir, "info");

            File file = new File(srcDir);
            g_srcDir = file.getParent() + g_fileSeparator;
            pseudo(srcDir);
        }
    }

    public static void main(String[] arg) {
    	String output;
    	PseudoAll pseudo;
    	String srcDir = null;
    	String destDir = null;
    	String sep = System.getProperty("file.separator");
    	boolean clean = true;
		String logFileName = null; // global log file

		if (g_os.indexOf("Windows") == -1)
		    g_platform = PLATFORM_UNIX;
		else
		    g_platform = PLATFORM_WINDOWS;

		for (int i = 0; i < arg.length; i ++) {
			if (arg[i].compareTo("-R") == 0) {
				if ((i + 1) == arg.length)
					break;
				g_rulesFile = arg[i+1];
				i ++;
			}
			else if (arg[i].compareTo("-log") == 0) {
			    if ((i + 1) == arg.length)
				    break;
				g_nlsLogFileName = arg[i+1];
				i ++;

			}
			else if (arg[i].compareTo("-lang") == 0) {
			    if ((i + 1) == arg.length)
				    break;
				g_language = arg[i+1];
				i ++;

			}
			else if (arg[i].compareTo("-base") == 0) {
			    if ((i + 1) == arg.length)
				    break;
				g_baseLang = arg[i+1];
				i ++;

			}
			else if (arg[i].compareTo("-g") == 0) {
			    if ((i + 1) == arg.length)
				    break;
				g_glossary = arg[i+1];
				i ++;

			}
			else if (arg[i].compareTo("-nlstrans") == 0) {
				g_nlsTrans = true;

			}
			else if (arg[i].compareTo("-inline") == 0) {
				g_inline = true;

			}
			else if (arg[i].compareTo("-D") == 0) {
				g_debug = true;
			}
			else if (arg[i].compareTo("-D1") == 0) {
				clean = false;
			}
			else if (arg[i].compareTo("-htmltrans") == 0) {
				g_htmltrans = true;
			}
			else if (srcDir == null)
				srcDir = arg[i];
			else if (destDir == null)
				destDir = arg[i];
		}

        if (g_nlsTrans == false && g_nlsLogFileName != null) {
            try {
                g_nlsLogFile = new FileOutputStream(g_nlsLogFileName);
			}
			catch (Exception e) {
			    g_nlsLogFile = null;
			}
		}

    	if ((srcDir == null) || (destDir == null)) {
    		System.out.println("Syntax: PseudoAll options source destinationDirectory ");
    		System.out.println("options: ");
    		System.out.println("    -R rulesFile");
    		System.out.println("    -D :  debug mode");
    		System.out.println("    -D1 : leave working files on disk");
    		System.out.println("    -log logfile: libnls logfile name");
    		System.out.println("    -lang language");
    		System.out.println("    -g glossaryFile");
    		System.out.println("    -nlstrans :  call pseudotrans");
    		System.out.println("    -base language:  Pseudo based on language file or directory");
    		System.out.println("    -inline:  allow Pseudo on samefile");
    		System.out.println("    -htmltrans: assume properties files are generated from htmlsplit");
    		System.exit(1);
    	}

        if (g_platform == PLATFORM_WINDOWS) {
			srcDir = srcDir.replace('/', g_fileSeparator.charAt(0));
			destDir = destDir.replace('/', g_fileSeparator.charAt(0));
		}

        // append "./" if necessary
        if (g_platform == PLATFORM_UNIX &&
           (srcDir.charAt(0) != '/' && srcDir.charAt(0) != '.'))
            srcDir = "./" + srcDir;
        if (g_platform == PLATFORM_UNIX &&
           (destDir.charAt(0) != '/' && destDir.charAt(0) != '.'))
            destDir = "./" + destDir;

        if (g_platform == PLATFORM_WINDOWS && srcDir.charAt(0) != '.') {
            if ((srcDir.length() == 1) ||
                ((srcDir.length() > 1) && (srcDir.charAt(1) != ':')))
                srcDir = "." + g_fileSeparator + srcDir;
        }

        if (g_platform == PLATFORM_WINDOWS && destDir.charAt(0) != '.') {
            if ((destDir.length() == 1) ||
                ((destDir.length() > 1) && (destDir.charAt(1) != ':')))
                destDir = "." + g_fileSeparator + destDir;
        }

		// Global log file
		try {
			logFileName = "log" + System.currentTimeMillis();
			g_logFile = new FileOutputStream(logFileName);
		}
		catch (Exception e) {
			g_logFile = null;
		}


    	// create pzip file on unix
    	if (g_platform == PLATFORM_UNIX) {
    	    String src;

    	    try {
    	        FileOutputStream pzipFile = new FileOutputStream("pzip");
    	        src = "cd $1\nzip -r $2 *\n";
    	        pzipFile.write(src.getBytes());

    	        pzipFile.close();
    	    }
    	    catch (Exception e) {
    		    DebugPrintln("Can't create pzip on unix, we are dead!", "exception");
    	    }
    	}

    	// remove working directory
    	File workingDir = new File("working");
    	if (workingDir.exists() == true) {
    	    String cmd;
    	    Process process;
    	    int status;
    	    
    	    cmd = "rm -rf working";    	    
    	    status = runCommand(cmd);    	    
    	}

    	// check if src directory exists
    	File fPath = new File(srcDir);
   		if (fPath.exists() == false) {
    		DebugPrintln("SourceDirectory is not there!!!", "info");
    		System.exit(1);
   		}

        // append "/" if necessary
    	if (fPath.isDirectory() == true && srcDir.endsWith(sep) == false)
    		srcDir += sep;
    	if (destDir.endsWith(sep) == false)
    		destDir += sep;

    	// check if destination directory exists
    	File fDest = new File(destDir);
   		if (fDest.exists() == false) {
   		    fDest.mkdirs();
   		}


	    if (fPath.isFile() == true)
   	        pseudo = new PseudoAll (srcDir, destDir, PSEUDO_FILE);
   	    else
   	        pseudo = new PseudoAll (srcDir, destDir, PSEUDO_DIRECTORYTREE);
   	        
   	        
   	    // clean up working directory and remove pzip on unix
   	    if (clean == true) {
    	    workingDir = new File("working");
    	    if (workingDir.exists() == true) {
    	        String cmd;
    	        int status;
				if (g_platform == PLATFORM_UNIX)
    	        	cmd = "rm -rf working";
				else
    	        	cmd = "cmd /C rmdir /S /Q working";
    	        status = runCommand(cmd);
		    }
	        if (g_platform == PLATFORM_UNIX) {
	            fPath = new File("pzip");
	            if (fPath.exists() == true)
	                fPath.delete();	            	            
	        }

			if (g_logFile != null) {
				try {
					g_logFile.flush();
					g_logFile.close();

					if (logFileName != null)
						(new File(logFileName)).delete();
				}
				catch (Exception e) {
                	DebugPrintln("Can't remove global log", "exception");
				}
			}

    	}   	    


    }

    // Note: no charset conversion happens here!
    static void writeToLog(String key, String value) {
        if(g_nlsLogFile != null) {
            try {
                g_nlsLogFile.write(key.getBytes());
                g_nlsLogFile.write('=');
                g_nlsLogFile.write(value.getBytes());
                g_nlsLogFile.write('\n');
            }
            catch (Exception e) {
                DebugPrintln("Can't write to log file", "exception");
            }
        }
    }

    static void DebugPrintln(String msg, String type) {
		if (type.compareTo("exception") == 0) {
			System.out.println("Exception: " + msg);
		}
		else if (type.compareTo("global") == 0) {
			if (g_logFile != null) {
				try {
					g_logFile.write(msg.getBytes());
					g_logFile.write('\n');
				}
				catch (Exception e) {
					System.out.println("Exception: can't write to global log");
				}
			}
		}
		else if(g_debug == true) {
			if (type.compareTo("info") == 0) {
				System.out.println("   " + msg);
			}
		}
    }

    public static int runCommand(String cmd) {
        Process process;
        InputStream in;
		OutputStream out;
		InputStream err;
		byte buf[] = new byte[3000];
		int status = -1;
        
		DebugPrintln("cmd = " + cmd, "info");
        try {
           	process = Runtime.getRuntime().exec(cmd);
           	
			out = process.getOutputStream();
			in = process.getInputStream();
			err = process.getErrorStream();
        
			if (g_os.indexOf("Windows") != -1) {

            	while (true) {
                	try {
                    	status = process.exitValue();
                    	break;
                	}
                	catch (Exception e) {
                    	int n;
		            	int len;
		            	String s;

			        	while ((n = in.available()) > 0) {
				        	len = in.read(buf);
							in.skip(len);
           		        	DebugPrintln("out = " + new String(buf,0,len), "info");
				    	}

			        	while ((n = err.available()) > 0) {
				        	len = err.read(buf);
							err.skip(len);
           		        	DebugPrintln("err = " + new String(buf,0,len), "info");
				    	}
		        	}
				}
			}
			else {
	    		status = process.waitFor();
			}

//          process.waitFor();

			if (in != null)
			    in.close();
			else if (out != null)
			    out.close();
			else if (err == null)
			    err.close();
        }
        catch (Exception e) {
            DebugPrintln("exception2 = " + cmd, "exception");
            return status;
        }        

       	DebugPrintln("done: " + status, "info");

        return status;
    }

	int getL10nFileType(String fileExt) {
		if (fileExt.compareTo(".jar") == 0 || fileExt.compareTo(".properties") == 0)
			return L10N_SAMEDIRECTORY;
		else if (fileExt.compareTo(".html") == 0 || fileExt.compareTo(".htm") == 0) {
			return L10N_SUBDIRECTORY;
		}
		return 0;
	}

    String getDestFile(String filePath)
    {
    	// create destination directory
	    String relativePath;
	    String newFileName;
		String ext = null;
		boolean l10n_expansion = true;
		int n;

    	// Get relative path
    	if ((g_srcDir != null) && (filePath.indexOf(g_srcDir) == 0)) {
	    	relativePath = filePath.substring(g_srcDir.length(), filePath.length());
    	}
    	else
    		relativePath = filePath;


		if (g_baseLang != null) {
	        if ((n = relativePath.indexOf(g_fileSeparator + g_baseLang + g_fileSeparator)) != -1) {
    	        relativePath = relativePath.substring(0, n + 1) + g_language +
    	    			relativePath.substring(n + g_baseLang.length() + 1, relativePath.length());
    	        l10n_expansion = false;
    	    }
    	    else if ((n = relativePath.indexOf(g_baseLang + g_fileSeparator)) == 0) {
    	        relativePath = g_language + relativePath.substring(g_baseLang.length(), relativePath.length());
    	        l10n_expansion = false;
    	    }    	    	        	        	    
		}
		else {
		    int i;
		    String lang;
		    // ignore known languages
		    for (i = 0; i < g_ignoreLangs.length; i++) {
		        lang = g_ignoreLangs[i];
	            if ((n = relativePath.indexOf(lang + g_fileSeparator)) != -1) {
	                if (n == 0) {
	                    return null;
    	            }	        
	                else if ((n = relativePath.indexOf(g_fileSeparator + lang + g_fileSeparator)) != -1) {
	                    return null;
    	            }
    	        }
    	    }
		    
		    // check base languages
		    for (i = 0; i < g_baseLangs.length; i++) {
		        lang = g_baseLangs[i];
	            if ((n = relativePath.indexOf(lang + g_fileSeparator)) != -1) {
	                if (n == 0) {
    	                relativePath = g_language +	relativePath.substring(lang.length(), relativePath.length());
    	                l10n_expansion = false;
    	            }	        
	                else if ((n = relativePath.indexOf(g_fileSeparator + lang + g_fileSeparator)) != -1) {
    	                relativePath = relativePath.substring(0, n + 1) + g_language +
    	    			relativePath.substring(n + lang.length() + 1, relativePath.length());
    	                l10n_expansion = false;
    	            }
    	        }
    	    }
		}

  		File file = new File(g_destDir + relativePath);
   		String outputDir = file.getParent();
   		String filename = file.getName();
   		int l10nType;

		if (l10n_expansion == false)
   			return outputDir + g_fileSeparator + filename;

 		if (filePath.endsWith(".jar") == true)
 			ext = ".jar";
 		else if (filePath.endsWith(".properties") == true)
 			ext = ".properties";
 		else if (filePath.endsWith(".html") == true)
 			ext = ".html";
 		else if (filePath.endsWith(".htm") == true)
 			ext = ".htm";

 		if (ext != null) {
			l10nType = getL10nFileType(ext);
    		if (l10nType == L10N_SAMEDIRECTORY) {
				newFileName = filename.substring(0, filename.length() - ext.length());

                if (g_baseLang != null) {
				    if (newFileName.endsWith("_" + g_baseLang) == true) {
    			        newFileName = newFileName.substring(0, newFileName.length() - 1 - g_baseLang.length()) +
    			            "_" + g_language + ext;
    			        return outputDir + g_fileSeparator + newFileName;
				    }
				    return null;
				}                    
                else {
                    int i;
                    String lang;
                    boolean nameChange = false;
                    // ignore languages
                    for (i = 0; i < g_ignoreLangs.length; i++) {
				        if (newFileName.endsWith("_" + g_ignoreLangs[i]) == true)
				            return null;                                                                       
                    }

                    for (i = 0; i < g_baseLangs.length; i++) {
				        if (newFileName.endsWith("_" + g_baseLangs[i]) == true) {
				            nameChange = true;
    			            newFileName = newFileName.substring(0, newFileName.length() - g_baseLangs[i].length() - 1) +
    			                "_" + g_language + ext;
				        }
                    }
                    
                    if (nameChange == false)
   			            newFileName = newFileName + "_" + g_language + ext;
   			            
    			    return outputDir + g_fileSeparator + newFileName;
    	        }
    		}
    		else if (l10nType == L10N_SUBDIRECTORY) {
    		    if (g_baseLang == null)
    			    return outputDir + g_fileSeparator + g_language + g_fileSeparator + filename;
    		}
    	}
    	return null;
    }

    //	unzip file.jar *.properties -d working/jar/file.jar/
    //  PseudoAll working/jar/file.jar/  working/jar/file.jar.xx/
    //  command
    //    cd working/jar/file.jar.xx
    //    zip dest/file_xx.jar .

	void pseudoJar(String inputFile, String outputFile) {
	    File fPath = new File(inputFile);
	    String filename;
	    String outDir;
	    String cmd;
	    int status;

		DebugPrintln("  jar: " + inputFile, "global");

	    filename = fPath.getName();

	    outDir = "." + g_fileSeparator + "working" + g_fileSeparator + "jar" +
	            g_fileSeparator + filename;

	    fPath = new File(outDir);
	    if (fPath.exists() == false) {
	        fPath.mkdirs();
	    }

	    cmd = "unzip -o " + inputFile + " \"*.properties\"" + " -d " + outDir;
        status = runCommand(cmd);

	    String srcDir, destDir;
	    srcDir = "working" + g_fileSeparator + "jar" + g_fileSeparator +
	            filename + g_fileSeparator;
	    destDir = "working" + g_fileSeparator + "jar" + g_fileSeparator +
	            filename + "." + g_language + g_fileSeparator;

	    fPath = new File(destDir);
	    if (fPath.exists() == false)
	        fPath.mkdirs();

        DebugPrintln("PseudoAll = " + srcDir + " " + destDir, "info");
        PseudoAll newPseudo = new PseudoAll(srcDir, destDir, PSEUDO_DIRECTORYTREE);

	    fPath = new File(outputFile);
	    if (fPath.exists() == false) {
	        File fDir = new File(fPath.getParent());
	        if (fDir.exists() == false)
	            fDir.mkdirs();
	    }

	    filename = fPath.getName();

	    // zip files together
		if (g_platform == PLATFORM_WINDOWS) {
	        cmd = "cmd /C cd " + destDir;
	        cmd += " && ";
	        cmd += "zip -r " +  filename + " *";
	    }
	    else {
	        cmd = "sh pzip " + destDir + " " + filename;
	    }

        status = runCommand(cmd);

	    // copy from destDir to outputFile
        cmd = "cp " + destDir + filename + " " + outputFile;
        status = runCommand(cmd);

	    DebugPrintln("outputFile = " + outputFile, "info");
	}

    // htmlsplit -s iso-8859-1 -R tools/htmlsplit.rules src/file.html working/file.html.export working/file.html.properties
    // PseudoProperties  working/file.html.properties working/file_xx.html.properties
    // tools/htmlmerge -t UTF-8 dest/src/file.html working/file.html.export working/file_xx.html.properties

	void pseudoHTML(String inputFile, String outputFile) {
	    File file;
	    File fDir;
	    String filename;

		DebugPrintln("  html: " + inputFile, "global");

	    file = new File(inputFile);
	    filename = file.getName();

	    file = new File(outputFile);
	    fDir = new File (file.getParent());
	    if (fDir.exists() == false) {
	        fDir.mkdirs();
	    }

	    String exportFile, propertiesFile, newPropertiesFile;
	    String cmd1, cmd2;
		int status;
		String os;

	    exportFile = "." + g_fileSeparator + "working" + g_fileSeparator + filename + ".export";
	    propertiesFile = "." + g_fileSeparator + "working" + g_fileSeparator + filename + ".properties";
	    newPropertiesFile = "." + g_fileSeparator + "working" + g_fileSeparator + "xx_" + filename + ".properties";


	    file = new File ("." + g_fileSeparator + "working" + g_fileSeparator);
	    if (file.exists() == false)
	        file.mkdirs();

        cmd1 = "htmlsplit -s iso-8859-1 -R " + g_rulesFile + " ";
        // htmlsplit sucks! change ./file to file
        if (inputFile.indexOf("." + g_fileSeparator) == 0)
            cmd1 += inputFile.substring(2, inputFile.length());
        else
	        cmd1 += inputFile;
	    cmd1 += " " + exportFile + " " + propertiesFile;

        status = runCommand(cmd1);

	    DebugPrintln("pseudoProperties = " + propertiesFile + " " + newPropertiesFile,"info");
	    pseudoProperties(propertiesFile, newPropertiesFile, "html");

	    cmd2 = "htmlmerge -t UTF-8 ";
	    cmd2 += outputFile;
	    cmd2 += " " + exportFile + " " + newPropertiesFile;

        status = runCommand(cmd2);

	    DebugPrintln("outputFile = " + outputFile, "info");
	}

	void pseudoProperties(String inputFile, String outputFile, String type) {
	    File file;
	    File fDir;

		DebugPrintln("  properties: " + inputFile, "global");

	    file = new File(outputFile);
	    fDir = new File (file.getParent());
	    if (fDir.exists() == false) {
	        fDir.mkdirs();
	    }

        if (g_nlsTrans == false) {
	        PseudoProperties pp;
			if (g_htmltrans == true)
				type = "html";

	        pp = new PseudoProperties(inputFile, outputFile, type);

	        DebugPrintln("outputFile = " + outputFile, "info");
   		}
   		else {
   		    String cmd;
   		    int status;

   		    if (g_glossary != null) {
			    cmd = "PseudoTran -c -t -r -e UTF-8 " + inputFile + " -o " + outputFile;
			    if (g_glossary != null)
			        cmd += " -g " + g_glossary;
			    if (g_nlsLogFileName != null)
			        cmd += " -l " + g_nlsLogFileName;
			}
			else {
			    cmd = "PseudoTran " + " -p " + inputFile + " -o " + outputFile;
			}
			
			status = runCommand(cmd);
   		}
	}

    void pseudoUnknown(String input, String output) {
        String cmd;
        int status;
	    File file;
	    File fDir;

	    file = new File(output);
	    fDir = new File (file.getParent());
	    if (fDir.exists() == false) {
	        fDir.mkdirs();
	    }
        
        cmd = "cp " + input + " " + output;
        status = runCommand(cmd);
        
    }
    
    void pseudoTrans(String input, String output) {
    	if (input.endsWith(".jar") == true) {
    	    if (g_inline == true ||
    	            input.endsWith("_" + g_language + ".jar") == false)
    		    pseudoJar(input, output);
    	}
    	else if (input.endsWith(".html") == true || input.endsWith(".htm") == true) {
    		pseudoHTML(input, output);
    	}
    	else if (input.endsWith(".properties") == true  &&
    	            input.indexOf("font.properties") == -1) {
    	    if (g_inline == true ||
    	            input.endsWith("_" + g_language + ".properties") == false) {
    	                
    	        if (input.compareTo(output) == 0) {
    	            String newInput = input + ".newname";    	            
    	            File fInput = new File (input);    	            
    	            File fNew = new File (newInput);
    	            
    	            if (fNew.exists() == true)
    	                fNew.delete();
    	            
    	            if (fInput.renameTo(fNew) == true)    	            
    		            pseudoProperties(newInput, output, "properties");
    		            
    		        // clean up 
    	            if (fNew.exists() == true)
    	                fNew.delete();    		        
    	        }    	        
    	        else {
    		        pseudoProperties(input, output, "properties");
    		    }
    		 }
    	}
    	else {
    	    if (g_inline != true)			
    	    	pseudoUnknown(input, output);
    	}
    }

    void pseudo(String filePath) {
		File file = new File(filePath);
		String list[];

		if (file.isDirectory()) {
			int i;
			list = file.list();
			for (i = 0; i < list.length; i++) {
			    if (list[i].compareTo("working") == 0 || 
			        (g_inline == false && list[i].compareTo(g_language) == 0))
			        return;
				if (filePath.endsWith(g_fileSeparator) == true)
					pseudo(filePath + list[i]);
				else
					pseudo(filePath + g_fileSeparator + list[i]);
			}
		}
		else {
	        DebugPrintln("filepath = " + filePath, "info");
	        DebugPrintln("g_srcPth = " + g_srcDir, "info");

		    String destPath = getDestFile(filePath);

            DebugPrintln("destPath = " + destPath, "info");
			// call pseudotran to do file translation
			if (destPath != null)
				pseudoTrans(filePath, destPath);
		}
    }

    void pseudo() {
    	pseudo(g_srcDir);
    }
}
