/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
import java.lang.*;
import java.io.*;

import PropertiesParsing;
import PseudoAll;

public class PseudoProperties {
	static final int NUMBERING = 1;
	static final int PSEUDO_TRANSLATION = 2;

	static final int BACKSLASH = 1;

	private BufferedReader dataInput;
	private BufferedWriter dataOutput;
	private String line;
	private PropertiesParsing propertyParsing;
	private String g_type;

    PseudoProperties (String in, String out, String type) {
        g_type = type;
   	    if (openFile(in, out) == 0) {
   	    	propertyParsing = new PropertiesParsing();

		    while (true) {
			    try {
				    line = dataInput.readLine();
				    if (line == null)
					    break;

					processLine(line);
				}
				catch (Exception e) {
            		System.out.println(e);
					break;
				}
        	}
        }

        try {
        	dataInput.close();
			dataOutput.flush();
			dataOutput.close();
        }
        catch (Exception e) {
       		System.out.println(e);
        }
    }

    /*
       text2String

       change text format string to Java string
       eg.  2 characters  "\n"  -->  1 character '\n'
     */
    String text2String(String cstr) {
        String jstr = new String();
        int len;
        int i;
        int state = 0;

        len = cstr.length();
        i = 0;
        while (i < len) {
            if (state == BACKSLASH) {
                switch (cstr.charAt(i)) {
                  case 'u': {
                    int n = 0;
                    int j;
                    char ch;
                    for (j = 0; j < 4; j ++) {
                        ch = cstr.charAt(i + j + 1);
                        ch = Character.toUpperCase(ch);
                        if (ch >= '0' && ch <= '9') {
                            n = n * 16 +  (ch - '0');
                        }
                        else if (ch >= 'A' && ch <= 'F') {
                            n = n * 16 +  (short) (ch - 'A') + 10;
                        }
                    }
                    if (j > 0) {
                        i += j;
                        jstr += (char) n;
                    }
                    break;
                  }

                  case 'n':
                    jstr += '\n';
                    break;
                  case 'r':
                    jstr += '\t';
                    break;
                  default:
                    jstr += cstr.charAt(i);
                    break;
                }
                state = 0;
            }
            else if (cstr.charAt(i) == '\\') {
                state = BACKSLASH;
            }
            else {
                jstr += cstr.charAt(i);
            }

            i ++ ;
        }
        return jstr;

    }

    /*
       string2Text

       change java string to text format string
       eg.  1 character '\n' --> 2 characters  "\n"
     */
    String string2Text(String str) {
        String cstr = new String();
        int len;
        int i;
        char ch;

        len = str.length();

        i = 0;
        while (i < len) {
            ch = str.charAt(i);
            if (ch == '\n') {
                cstr += "\\u000a";
            }
            else if (ch >= 0x80) {
            	String hexstr;
            	int digits;
                cstr += "\\u";
                hexstr = Integer.toString((short)ch,16);
                digits = hexstr.length();
                if (digits == 2)
                	cstr += "00";
                else if (digits == 3)
                	cstr += "0";
				cstr += hexstr;
            }
            else {
                cstr += ch;
            }
            i ++;
        }
        return cstr;

    }

    /* getPseudoTrans

			Ignore patterns:
				Comment line
				empty line
				*.gif
				key contains string which is defined in global
			Pseudo line
				Key=value

       retrive previous translation from pseudo algorithm
     */
    String getPseudoTrans(String value) {
        String result;
        char ch;

        // Get unicode value for the string
        result = "\u4eba";
        for (int i = 0; i < value.length(); i++) {
        	ch = value.charAt(i);
			switch (ch) {
			case 'i' :
				ch = '\u00EE';   // ''
				break;
			case 'I' :
				ch = '\u00CE';   // ''
				break;
			case 'o' :
				ch = '\u00F4';  	// ''
				break;
			case 'O' :
				ch = '\u00D4';  	// ''
				break;
			case 'u' :
				ch = '\u00FB';   // ''
				break;
			case 'U' :
				ch = '\u00DB';   // ''
				break;
			case 'e' :
				ch = '\u00EA';	// ''
				break;
			case 'E' :
				ch = '\u00CA';	// ''
				break;
			case 'a' :
				ch = '\u00E2'; 	// ''
				break;
			case 'A' :
				ch = '\u00C2'; 	// ''
				break;
			}
			result += ch;
		}

		result += "\u56de";
		return string2Text(result);
    }

    void processLine(String line) {
    	int status;
    	String value, newvalue = null;
    	String key;
        PseudoValue pseudoValue;

       	try {
       		value = propertyParsing.parseLine(line);
       		status = propertyParsing.status;
       		if (status == PropertiesParsing.INVALIDLINE ||
       			status == PropertiesParsing.COMMENTLINE) {
	        	dataOutput.write(line, 0, line.length());
		    	dataOutput.newLine();
       		}
       		else {
  			    key = propertyParsing.key;
       		    if (status == PropertiesParsing.FIRSTLINE ||
       		        status == PropertiesParsing.FIRSTLINEFINISH) {
	        	    dataOutput.write(key, 0, key.length());
        	    	dataOutput.write("=", 0, 1);
       		    };

       			value = propertyParsing.value;

                pseudoValue = new PseudoValue(key, value, g_type);
                newvalue = pseudoValue.getPTValue();
       			
//     			newvalue = getPseudoTrans(value);
	        	if (newvalue == null)
	        		newvalue = value;
	        	else {
	        	    PseudoAll.writeToLog(key, value);  	        	    
	        	}

        	    dataOutput.write(newvalue, 0, newvalue.length());

                if (status == PropertiesParsing.FIRSTLINEFINISH ||
                        status == PropertiesParsing.NEXTLINEFINISH) {
                    dataOutput.newLine();
                }
      		}
       	}
       	catch (Exception e) {
   			System.out.println(e);
       	}
	}

    // return 0:  success
    //        1:  fail
	int openFile(String in, String out) {
		/*********************************
			Input File Stream
		*********************************/
        FileInputStream inFile;
        InputStreamReader finput;

        try {
        	inFile = new FileInputStream(in);
            finput = new InputStreamReader(inFile);
        }
        catch (Exception e) {
            System.out.println(e);
            return 1;   // 1: fails in FileInputStream
        }

        try {
        	dataInput = new BufferedReader(finput);
        }
        catch (Exception e) {
            System.out.println(e);
            return 2;	// 2: fails in DataInputStream
        }

        /*******************************
        	Output File Stream
        ********************************/
        FileOutputStream outFile;
        OutputStreamWriter foutput;

        try {
        	outFile = new FileOutputStream(out);
            foutput = new OutputStreamWriter(outFile);
        }
        catch (Exception e) {
            System.out.println(e);
            return 3;   // 3: fails in FileOutputStream
        }

        try {
        	dataOutput = new BufferedWriter(foutput);
        }
        catch (Exception e) {
            System.out.println(e);
            return 4;	// 4: fails in DataOutputStream
        }

        return 0;
	}

}
