/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
import java.util.*;

/**
 * PseudoValue (key, value, "html"|"properties")
 *	"html": for file_xx.html.properties 
 *	"properties": for the rest (java, libnls)
 */

class PseudoValue {
    public static int PseudoStat;
    final static int PT_NONE = 0;
    final static int PT_TEXT = 0x1;
    final static int PT_HASTAG = 0x2;
    String sRValue;
    char cHead = '\u4eba';
    char cTail = '\u56de';
    char[] cFrom = {'A', 'E', 'I', 'O', 'U', 'a', 'e', 'i', 'o', 'u'};
    char[] cTo = {'\u00c2', '\u00ca', '\u00ce', '\u00d4',
	          '\u00db', '\u00e2', '\u00ea', '\u00ee',
	          '\u00f4', '\u00fb'};
    int istag;

    public PseudoValue(String key, String value, String type) {
        istag = -1;
        PseudoStat = 0;
	if (PseudoAll.g_debug == true)
	    System.out.println("PseudoValue: key <" + key + ">, value <"
				+ value + ">, type <" + type + ">");

	if (value.equals("") || value.compareTo("") == 0 || value == null)
	    sRValue = value;		// null value
	else {
	    int pound = key.indexOf("#");
	    if (pound == 0)				// comment
	        sRValue = value;
	    else if (pound > 0) {			// potential comment
	        char tmp[];
	        int i;
	        tmp = new char[1024];
	        key.getChars(0, pound, tmp, 0);
	        for (i = 0; i < pound; i++)
		    if (tmp[i] != ' ' || tmp[i] != '\t')
		        break;
	        if (i == pound)				// comment
		    sRValue = value;
		else {					// not comment
	            if (type.equalsIgnoreCase("html")) {
	                PseudoHtmlValue(key, value);
	            } else {
	                PseudoPropValue(key, value);
	            }
		}
	    } else {
	        if (type.equalsIgnoreCase("html")) {
	            PseudoHtmlValue(key, value);
	        } else {
	            PseudoPropValue(key, value);
	        }
	    }
	}
    }

    String string2Text(String str) {
        String cstr = new String();
	int len;
	int i;
	char ch;
	len = str.length();
	i = 0;
	while (i < len) {
	    ch = str.charAt(i);
	    if (ch == '\n') {
	        cstr += "\\u000a";
	    }
	    else if (ch >= 0x80) {
	        String hexstr;
		int digits;
		cstr += "\\u";
		hexstr = Integer.toString((short)ch,16);
		digits = hexstr.length();
		if (digits == 2)
		    cstr += "00";
		else if (digits == 3)
		    cstr += "0";
		cstr += hexstr;
	    }
	    else {
		cstr += ch;
	    }
	    i ++;
	}
	return cstr;
    }

    public String getPTValue() {
	return string2Text(sRValue);
    }

    public boolean isNbSpace(String value) {
	int i;
	int idx, pidx = 0;
	int aidx = 0;
	String lvalue = value.toLowerCase();
	while ((idx = lvalue.indexOf("&nbsp;")) >= 0) {
	    for (i = 0; i < idx; i++) {
	        if (lvalue.charAt(i) != ' ' && lvalue.charAt(i) != '\t') {
		    return false;
		}
	    }
	    pidx = idx + "&nbsp;".length();
	    lvalue = lvalue.substring(pidx);
	    aidx += pidx;
	}
        for (i = aidx; i < value.length(); i++) {
	    if (value.charAt(i) != ' ' && value.charAt(i) != '\t') {
		return false;
	    }
	}
	return true;
    }

    private void PseudoHtmlValue(String key, String value) {
	int idx;
	
	idx = key.indexOf("meta");
	if (idx >= 0) {
	    idx = key.indexOf("content");
	    if (idx >= 0) {
		String lvalue = value.toLowerCase();
		idx = lvalue.indexOf("charset");
		if (idx >= 0) {
		    sRValue = value.substring(0, idx + 8) + "UTF-8";
		    return;
		}
	    }
	}

	idx = key.indexOf("text");
	if (idx < 0) {					// not text
	    sRValue = value;
	    return;
	}

	int sfx = idx + 4;				// idx + length("text")
	int len = key.length();
	if (sfx == len) {				// .text=<value>
	    if (isNbSpace(value))
	        sRValue = value;
	    else
	        sRValue = PseudoTran(value, true);
	    return;
	}

	int delta = len - sfx;
	int i;
	for (i = 0; i < delta; i++, sfx++) {
	    char ch = key.charAt(sfx);
	    if (ch < '0' || ch > '9')
		break;
	}
	if (i == delta) {				// .text##=<value>
	    if (isNbSpace(value))
	        sRValue = value;
	    else
	        sRValue = PseudoTran(value, true);
	} else						//text??.??.??=<value>
	    sRValue = value;
	return;
    }

    private boolean isPathChar(char c) {
	if (c >= 'A' && c <= 'Z')
	    return true;
	if (c >= 'a' && c <= 'z')
	    return true;
	if (c >= '0' && c <= '9')
	    return true;
	if (c == '_' || c == '.' || c == '-')
	    return true;
	return false;
    }

    private boolean isPath(String value) {
	String wvalue = value;
	int sidx = wvalue.indexOf("/");
	int bsidx = wvalue.indexOf("\\");
	int didx = wvalue.indexOf(".");

	if (sidx >= 0 && bsidx >= 0)
	    return false;

	if (sidx >= 0) {
	    String[] exceptions = {
		"User/Group",
		"View/Edit",
	    };
	    int i;
	    for (i = 0; i < exceptions.length; i++) {
		if (wvalue.equals(exceptions[i]))
		    return false;
	    }

	    while (sidx >= 0) {
	        for (i = 0; i < sidx; i++) {
	            char ch = wvalue.charAt(i);
		    if (isPathChar(ch) == false)
		        return false;
	        }
                wvalue = wvalue.substring(sidx + 1);
	        sidx = wvalue.indexOf("/");
	    }
	    return true;
	}

	if (bsidx >= 0) {
	    boolean isctlch = false;
            wvalue = wvalue.substring(bsidx + 1);
	    bsidx = wvalue.indexOf("\\");
	    do {
	        int i;
	        char ch;
		ch = wvalue.charAt(0);
		if (ch == 'n' || ch == 't' || ch == 'b' ||
		    ch == '0' || ch == '\\' || ch == 'u') {
		    isctlch = true;
		}
	        for (i = 1; i < bsidx; i++) {
	            ch = wvalue.charAt(i);
		    if (isPathChar(ch) == false)
		        return false;
	        }
                wvalue = wvalue.substring(bsidx + 1);
	        bsidx = wvalue.indexOf("\\");
		if (bsidx > 0)
		    isctlch = false;
		else if (bsidx < 0)
		    break;
	    } while (true);
	    if (isctlch)
		return false;
	    else
		return true;
	}

	if (didx >= 0) {
	    int i;
	    if (wvalue.length() >= 4) {
	        String[] exceptions = {"e.g.", "c.f.", "i.e.", "P.O."};
	        String tvalue = wvalue.substring(0, 4);
	        for (i = 0; i < exceptions.length; i++) {
		    if (tvalue.equalsIgnoreCase(exceptions[i]))
		        return false;
		}
	    }

	    if (wvalue.length() >= 3) {
	        String[] exceptions = {"...", ".\\u"};
	        for (i = 0; i < exceptions.length; i++) {
		    if (wvalue.indexOf(exceptions[i]) >= 0) {
		        return false;
		    }
		}
	    }

	    // X.##
	    if (wvalue.length() >= 3) {
	        String tvalue = wvalue.substring(0, 2);
		if (tvalue.equalsIgnoreCase("X.")) {
		    char ch = wvalue.charAt(2);
		    if (ch >= '0' && ch <= '9')
			return false;
		}
	    }

	    while (didx >= 0) {
	        for (i = 0; i < didx; i++) {
	            char ch = wvalue.charAt(i);
		    if (isPathChar(ch) == false)
		        return false;
	        }
                wvalue = wvalue.substring(didx + 1);
	        didx = wvalue.indexOf(".");
	    }
	    return true;
	}

	return false;
    }

    private boolean inStopList(String value) {
        int i;
	String[] recovercasesensitivelist = {
	    "No",
	    "NO"
	};
	for (i = 0; i < recovercasesensitivelist.length; i++) {
	    if (value.equals(recovercasesensitivelist[i])) {
		return false;
	    }
	}

	String[] stoplist = {
            "admin",
            "sansserif",
            "timesroman",
            "cn",
            "sn",
            "givenname",
            "objectclass",
            "ou",
            "l",
            "uid",
            "mail",
            "telephonenumber",
            "roomnumber",
            "userpassword",
	    "en fr da nl fi el ga it no pt ru sv de es ja ko zh af sq eu bg be ca hr cs fo gl hu is id pl ro sr sk sl tr uk",
            "zh",
            "da",
            "nl",
            "en",
            "fi",
            "fr",
            "de",
            "el",
            "ga",
            "it",
            "ja",
            "ko",
            "no",
            "pt",
            "ru",
            "es",
            "sv",
            "af",
            "sq",
            "eu",
            "bg",
            "be",
            "ca",
            "hr",
            "cs",
            "fo",
            "gl",
            "hu",
            "is",
            "id",
            "pl",
            "ro",
            "sr",
            "sk",
            "sl",
            "tr",
            "uk",
            "true",
            "false",
            "binaryphonetic",
            "slapd",
            "httpd",
            "(objectclass=organizationalunit)",
            "icon,"
	};

	for (i = 0; i < stoplist.length; i++) {
	    if (value.equalsIgnoreCase(stoplist[i])) {
		return true;
	    }
	}
        return false;
    }

    private boolean inSpecialList(String value) {
	String[] speciallist = {
	        "cn,Name",
	        "uid,User ID",
	        "mail,E-Mail",
	        "telephoneNumber,Phone",
		"cn,Member Name",
		"uid,Member User ID",
		"Users,Users",
		"Groups,Groups",
		"Users and Groups,Users and Groups",
		"contains,contains",
		"equals,equals",
		"not equals,not equals"
	};
	int i;

	for (i = 0; i < speciallist.length; i++) {
	    if (value.equals(speciallist[i]))
		return true;
	}
	return false;
    }

    private String getSpecialValue(String value) {
	int idx = value.indexOf(",");
	if (idx < 0)
	    return value;
	String lvalue = value.substring(0, idx + 1);
	String rvalue = value.substring(idx + 1);
	return lvalue + PseudoTran(rvalue, true);
    }

    private boolean isNum(String value) {
	int i;
	int idx = value.indexOf("0");
	boolean hex = false;
	if (idx >= 0 && (idx < value.length() - 1 &&
	    (value.charAt(idx+1) == 'x' || value.charAt(idx+1) == 'X')))
	    hex = true;
	    
	for (i = 0; i < value.length(); i++) {
	    char ch = value.charAt(i);
	    if (hex) {
	        if ((ch < '0' || ch > '9') && !(ch == 'x' || ch == 'X') &&
		    (ch < 'a' || ch > 'f') && (ch < 'A' || ch > 'F'))
		    return false;
	    } else {
	        if (ch < '0' || ch > '9')
		    return false;
	    }
	}
	return true;
    }

    private void PseudoPropValue(String key, String value) {
	String skey = key.toLowerCase();
	int idx = skey.indexOf("font");
	if (idx >= 0) {					// font
	    sRValue = value;
	    return;
	}

	int len = value.length();
	if (len <= 1) {					// RHS le one char
	    sRValue = value;
	    return;
	}

	String svalue = value.toLowerCase();
	idx = svalue.indexOf(".gif");			// gif
	if (idx >= 0 && len - idx == 4) {
	    sRValue = value;
	    return;
	}

	idx = svalue.indexOf(".jpg");			// jpg
	if (idx >= 0 && len - idx == 4) {
	    sRValue = value;
	    return;
	}

	if (isNum(value)) {
	    sRValue = value;
	    return;
	}

	if (inStopList(value)) {			// stop list
	    sRValue = value;
	    return;
	}

	if (inSpecialList(value)) {			// special treatment
	    sRValue = getSpecialValue(value);
	    return;
	}

	String tvalue = value;
	String wvalue;
	sRValue = "";
	boolean haspath = false;
	boolean hasnonpath = false;
        while (true) {
	    if ((idx = tvalue.indexOf(" ")) < 0)
	        idx = tvalue.indexOf("\t");
	    if (idx < 0)
		wvalue = tvalue;
	    else
                wvalue = tvalue.substring(0, idx + 1);
	    if (isPath(wvalue)) {
		haspath = true;
		sRValue = sRValue + wvalue;
	    } else {
		hasnonpath = true;
		sRValue = sRValue + PseudoTran(wvalue, false);
	    }
	    if (idx < 0)
		break;
	    else
                tvalue = tvalue.substring(idx + 1);
	} 

        if ((haspath && hasnonpath) || !haspath)
	    sRValue = addHeadTail(sRValue);
	return;
    }

    private char getTranChar(char ch) {
	int i;

	for (i = 0; i < cFrom.length; i++) {
	    if (ch == cFrom[i])
		return cTo[i];
	}
	return ch;
    }

    private String addHeadTail(String value) {
	return String.valueOf(cHead) + value + String.valueOf(cTail);
    }

    private String PseudoTran(String value, boolean full) {
	int i, j;
	int len = value.length();
	boolean isnum = false;
	char[] buf;
	String s;

	buf = new char[len];
	for (i = 0; i < len; i++) {			
	    char ch = value.charAt(i);
	    if (ch == ' ' || ch == '\t')
		buf[i] = ch;
	    else
		break;
	}

	for (j = 0; i < len; i++, j++) {
	    char ch = value.charAt(i);
	    if (ch == '&') {				// skip named entity
		String tmps = value.substring(i+1);
		int n = tmps.indexOf(";");
		if (n >= 0) {
		    n += i+1;
		    for ( ; i <= n; i++) {
			buf[i] = value.charAt(i);
		    }
		}
	    } else if (ch == '\\') {			// skip escaped unicode \\uxxxx
		if (len - i >= 6) {
		    char nch = value.charAt(i+1);
		    if (nch == 'u') {
			for ( ; i < 6; i++) {
			    buf[i] = value.charAt(i);
			}
		    }
		}
	    } else if (ch == '<') {			// tag starts
		if (istag == -1) {
                    PseudoStat |= PT_HASTAG;
		    istag = i;
		} else if (istag + 1 == i) {		// avoid << :)
                    PseudoStat &= ~PT_HASTAG;
		    istag = -1;
		}
		buf[i] = ch;
	    } else if (ch == '>') {			// tag ends
		istag = -1;
		buf[i] = ch;
	    } else if (istag == -1) {
		if (ch >= '0' && ch <= '9') {		// number
		    if (j == 0)
			isnum = true;
		    buf[i] = ch;
		} else {
		    if (isnum &&
			((ch >= 'a' && ch <= 'f') ||
			 (ch >= 'A' && ch <= 'F') ||
			 (j == 1 && (ch == 'x' || ch == 'X')))) {
                        buf[i] = ch;
		    } else {
			isnum = false;
	                buf[i] = getTranChar(ch);
		    }
		}
	    } else {
		buf[i] = ch;
	    }
	}

        s = new String(buf);
	if (isnum) {
	    return value;
	} else {
            PseudoStat |= PT_TEXT;
	    if (full == true)
	        return addHeadTail(s);
	    else
	        return s;
	}
    }

    public static boolean pseudoTraned() {
	if (PseudoStat > 0)
	    return true;
	else
	    return false;
    }
			    
    public static boolean hasText() {
	if ((PseudoStat & PT_TEXT) == PT_TEXT)
	    return true;
	else
	    return false;
    }
			    
    public static boolean hasTag() {
	if ((PseudoStat & PT_HASTAG) == PT_HASTAG)
	    return true;
	else
	    return false;
    }
}
