/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
//
// LibraryManager.hpp
//
// Manages loadable libraries
//
// Copyright (c) 1996 Netscape Communications Corporation
// Robin J. Maxwell 08-08-96
//

#ifndef _LIBMGR_HPP
#define _LIBMGR_HPP

#include <sys/types.h>
#include <rw/cstring.h>
#include <rw/tphset.h>
#include "dll.h"
#include "CritSec.hpp"

typedef void (*AnyFunction)(void);

class SymbolEntryBase: public RWCString
{

public:
    SymbolEntryBase(const RWCString& name, AnyFunction func=0);
    SymbolEntryBase(const char *name, AnyFunction func=0);

    ~SymbolEntryBase(){}
    AnyFunction virtual GetFunction()const{return _func;}
    AnyFunction virtual SetFunction(AnyFunction func){return _func=func;}
    time_t GetAccessTime(){return _lastAccessTime;}
    void SetAccessTime();
    static unsigned hashFun(const SymbolEntryBase& rhs);

private:
    AnyFunction _func;
    time_t _lastAccessTime;
};

class LibraryEntry
{
    friend class LibraryManager;

private:
    LibraryEntry(LibraryManager& lm, const RWCString& name);
    ~LibraryEntry();
    BOOLEAN Open();
    BOOLEAN Close();
    SymbolEntryBase *LoadSymbol(const RWCString& name);
    // Adds an entry for symbol name 
    SymbolEntryBase *LoadSymbol(const char *name, AnyFunction func);
    SymbolEntryBase *LoadSymbol(SymbolEntryBase& symentry);
    unsigned AddRef(){return _refcnt++;}
    unsigned RemoveRef();
    static unsigned hashFun(const LibraryEntry& rhs);

private:
    LibraryManager& _libmgr;
    unsigned _refcnt;
    DLHANDLE _handle;
    RWCString _name;
    RWTPtrHashSet<SymbolEntryBase> _symbols;
};

class LibraryManager 
{
public:
    LibraryManager();
    ~LibraryManager();
    BOOLEAN OpenLibrary(const char *libname);
    BOOLEAN CloseLibrary(const char *libname);
    // Loads symname from library libname
    SymbolEntryBase *LoadSymbol(const char *libname, const char *symname);

    // Allocates symbol entries for the LibraryManager
    // Override to allocate specialized symbol entry classes
    virtual SymbolEntryBase *CreateSymbolEntry(const RWCString& name, AnyFunction func=0);

private:
    LibraryEntry *_Open(const RWCString& name);
private:
    RWTPtrHashSet<LibraryEntry> _libs;
    CriticalSection _crtsec;
};

#endif // _LIBMGR_HPP
