/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * dll.h: Handle dynamically linked libraries
 * 
 * Rob McCool
 */

#ifndef _DLL_H
#define _DLL_H

#include "systems.h"

#if defined(DLL_CAPABLE)

/* --------------------------- Data structures ---------------------------- */


typedef void *DLHANDLE;
#define DLHANDLE_ERROR NULL


/* ------------------------------ Prototypes ------------------------------ */


/*
 * dll_open loads the library at the given path into memory, and returns
 * a handle to be used in later calls to dll_findsym and dll_close.
 */
NSAPI_PUBLIC DLHANDLE dll_open(const char *fn);


/*
 * dll_findsym looks for a symbol with the given name in the library 
 * pointed to by the given handle. Returns a pointer to the named function.
 */
NSAPI_PUBLIC void *dll_findsym(DLHANDLE dlp, const char *name);


/*
 * dll_error returns a string describing the last error on the given handle
 */
NSAPI_PUBLIC char *dll_error(DLHANDLE dlp);


/*
 * dll_close closes the previously opened library given by handle
 */
NSAPI_PUBLIC void dll_close(DLHANDLE dlp);

#endif /* DLL_CAPABLE */
#endif
