/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
#ifndef BASE_NET_H
#define BASE_NET_H

#ifndef NOINTNSAPI
#define INTNSAPI
#endif /* !NOINTNSAPI */

/* GLOBAL_FUNCTIONS:
 * DESCRIPTION:
 * system specific networking definitions
 * 
 * Rob McCool
 */

#ifndef NETSITE_H
#include "netsite.h"
#endif /* !NETSITE_H */

#ifndef BASE_FILE_H
#include "base/file.h"       /* for client file descriptors */
#endif /* !BASE_FILE_H */

#ifndef BASE_PBLOCK_H
#include "base/pblock.h"     /* for client data block */
#endif /* !BASE_PBLOCK_H */

#ifndef PUBLIC_BASE_NET_H
#include "public/base/net.h"
#endif /* !PUBLIC_BASE_NET_H */

#include "base/systems.h"

#define NET_DEFAULT_BUFFERSIZE		8192

/* --- Timeouts ---------------------------------------------------------- */
#define NET_DEFAULT_READ_TIMEOUT	120
#define NET_DEFAULT_WRITE_TIMEOUT	300
#define SSL_DEFAULT_HANDSHAKE_TIMEOUT	300

extern unsigned int NET_BUFFERSIZE;
extern unsigned int NET_READ_TIMEOUT;
extern unsigned int NET_WRITE_TIMEOUT;
extern unsigned int SSL_HANDSHAKE_TIMEOUT;
extern int net_enabledns;
extern int net_enableAsyncDNS;
extern int net_listenqsize;

/* --- Begin function prototypes --- */

#ifdef INTNSAPI

NSPR_BEGIN_EXTERN_C

/* --- Internal use only ----------------------------------------------- */
 /* XXXMB - these should be moved to netpriv.h */
NSAPI_PUBLIC int INTnet_init(int security);
NSAPI_PUBLIC int INTnet_shutdown(SYS_NETFD s, int how);
int net_accept_init(int port);
void net_accept_terminate(void);
void net_accept_enter(void);
int net_accept_tenter(void);
void net_accept_exit(void);
void net_accept_texit(void);

NSAPI_PUBLIC SYS_NETFD INTnet_socket(int domain, int type, int protocol);

NSAPI_PUBLIC int INTnet_listen(SYS_NETFD s, int backlog);

NSAPI_PUBLIC SYS_NETFD INTnet_create_listener(char *ipaddr, int port);

NSAPI_PUBLIC
int INTnet_connect(SYS_NETFD s, const void *sockaddr, int namelen);

NSAPI_PUBLIC
int INTnet_getpeername(SYS_NETFD s, struct sockaddr *name, int *namelen);

NSAPI_PUBLIC int INTnet_close(SYS_NETFD s);

NSAPI_PUBLIC
int INTnet_bind(SYS_NETFD s, const struct sockaddr *name, int namelen);

NSAPI_PUBLIC
SYS_NETFD INTnet_accept(SYS_NETFD s, struct sockaddr *addr, int *addrlen);

NSAPI_PUBLIC int INTnet_read(SYS_NETFD sd, char *buf, int sz, int timeout);

NSAPI_PUBLIC int INTnet_write(SYS_NETFD sd, char *buf, int sz);

NSAPI_PUBLIC int INTnet_writev(SYS_NETFD sd, struct iovec *iov, int iovlen);

NSAPI_PUBLIC int INTnet_isalive(SYS_NETFD sd);

NSAPI_PUBLIC char *INTnet_ip2host(char *ip, int verify);

/* --- OBSOLETE ----------------------------------------------------------
 * The following macros/functions are obsolete and are only maintained for
 * compatibility.  Do not use them.
 * -----------------------------------------------------------------------
 */

NSAPI_PUBLIC int INTnet_getsockopt(SYS_NETFD s, int level, int optname,
                                   void *optval, int *optlen);

NSAPI_PUBLIC int INTnet_setsockopt(SYS_NETFD s, int level, int optname,
                                   const void *optval, int optlen);

NSAPI_PUBLIC int INTnet_select(int nfds, fd_set *r, fd_set *w, fd_set *e, 
                               struct timeval *timeout);

NSAPI_PUBLIC int INTnet_ioctl(SYS_NETFD s, int tag, void *result);

NSAPI_PUBLIC int INTnet_socketpair(SYS_NETFD *pair);

#ifdef XP_UNIX

NSAPI_PUBLIC SYS_NETFD INTnet_dup2(SYS_NETFD prfd, int osfd);

NSAPI_PUBLIC int INTnet_is_STDOUT(SYS_NETFD prfd);

NSAPI_PUBLIC int INTnet_is_STDIN(SYS_NETFD prfd);

#endif

NSPR_END_EXTERN_C

/* --- End function prototypes --- */

#define net_init INTnet_init
#define net_shutdown INTnet_shutdown

#define net_socket INTnet_socket
#define net_listen INTnet_listen
#define net_create_listener INTnet_create_listener
#define net_connect INTnet_connect
#define net_getpeername INTnet_getpeername
#define net_close INTnet_close
#define net_bind INTnet_bind
#define net_accept INTnet_accept
#define net_read INTnet_read
#define net_write INTnet_write
#define net_writev INTnet_writev
#define net_isalive INTnet_isalive
#define net_ip2host INTnet_ip2host

/* Obsolete */

#define net_getsockopt INTnet_getsockopt
#define net_setsockopt INTnet_setsockopt
#define net_select INTnet_select
#define net_ioctl INTnet_ioctl
#define net_socketpair INTnet_socketpair
#define net_dup2 INTnet_dup2
#define net_is_STDOUT INTnet_is_STDOUT
#define net_is_STDIN INTnet_is_STDIN

#endif /* INTNSAPI */

#endif /* !BASE_NET_H */
