/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
#ifndef BASE_PBLOCK_H
#define BASE_PBLOCK_H

#ifndef NOINTNSAPI
#define INTNSAPI
#endif /* !NOINTNSAPI */

/*
 * pblock.h: Header for Parameter Block handling functions
 * 
 *
 * A parameter block is a set of name=value pairs which are generally used 
 * as parameters, but can be anything. They are kept in a hash table for 
 * reasonable speed, but if you are doing any intensive modification or
 * access of them you should probably make a local copy of each parameter
 * while working.
 *
 * When creating a pblock, you specify the hash table size for that pblock.
 * You should set this size larger if you know that many items will be in 
 * that pblock, and smaller if only a few will be used or if speed is not
 * a concern.
 *
 * The hashing function is very simple right now, and only looks at the 
 * first character of name.
 *
 * Rob McCool
 * 
 */

/*
 * Requires that the macros MALLOC and STRDUP be set to "safe" versions that 
 * will exit if no memory is available. If not under MCC httpd, define 
 * them to be the real functions and play with fire, or make your own 
 * function.
 */

#ifndef NETSITE_H
#include "../netsite.h"
#endif /* !NETSITE_H */

#ifndef PUBLIC_BASE_PBLOCK_H
#include "public/base/pblock.h"
#endif /* !PUBLIC_BASE_PBLOCK_H */

#ifdef INTNSAPI

/* --- Begin function prototypes --- */

NSPR_BEGIN_EXTERN_C

NSAPI_PUBLIC pb_param *INTparam_create(char *name, char *value);

NSAPI_PUBLIC int INTparam_free(pb_param *pp);

NSAPI_PUBLIC pblock *INTpblock_create(int n);

NSAPI_PUBLIC void INTpblock_free(pblock *pb);

NSAPI_PUBLIC char *INTpblock_findval(const char *name, pblock *pb);

NSAPI_PUBLIC
pb_param *INTpblock_nvinsert(const char *name, const char *value, pblock *pb);

NSAPI_PUBLIC
pb_param *INTpblock_nninsert(const char *name, int value, pblock *pb);

NSAPI_PUBLIC void INTpblock_pinsert(pb_param *pp, pblock *pb);

NSAPI_PUBLIC int INTpblock_str2pblock(const char *str, pblock *pb);

NSAPI_PUBLIC char *INTpblock_pblock2str(pblock *pb, char *str);

NSAPI_PUBLIC void INTpblock_copy(pblock *src, pblock *dst);

NSAPI_PUBLIC pblock *INTpblock_dup(pblock *src);

NSAPI_PUBLIC char **INTpblock_pb2env(pblock *pb, char **env);


/* --------------------------- Internal things ---------------------------- */


NSAPI_PUBLIC pb_param *INTpblock_fr(const char *name, pblock *pb, int remove);

NSAPI_PUBLIC
char * INTpblock_replace(const char *name,char * new_value,pblock *pb);

NSPR_END_EXTERN_C

#define param_create INTparam_create
#define param_free INTparam_free
#define pblock_create INTpblock_create
#define pblock_free INTpblock_free
#define pblock_findval INTpblock_findval
#define pblock_nvinsert INTpblock_nvinsert
#define pblock_nninsert INTpblock_nninsert
#define pblock_pinsert INTpblock_pinsert
#define pblock_str2pblock INTpblock_str2pblock
#define pblock_pblock2str INTpblock_pblock2str
#define pblock_copy INTpblock_copy
#define pblock_dup INTpblock_dup
#define pblock_pb2env INTpblock_pb2env
#define pblock_fr INTpblock_fr
#define pblock_replace INTpblock_replace

#endif /* INTNSAPI */

#endif /* !BASE_PBLOCK_H */
