/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * autil.h: Admin utilities, mainly cache manipulation routines
 *
 *
 *
 * Ari Luotonen
 */

#ifndef MCOM_ADMIN_UTIL_H
#define MCOM_ADMIN_UTIL_H

#include "cgiutils/ns-util.h"
#include "cgiutils/cgi-util.h"

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <sys/types.h>
#include <time.h>

#define FORM_MAXLINE 10000
#define MY_MAGCONF "../config/magnus.conf"

/*
 * Find out cache-root, cache-size and top-dirs.
 * If cache-root isn't found, program exits.  Others return zero.
 */
char *au_croot(void);
int au_csize(void);
int au_tdirs(void);


/*
 * Expire a single URL, a site, a site against regexp,
 * or entire cache against regexp.
 */
int expire_one (char *croot, char *cfile);
int expire_site(char *croot, char *ufile, char *shexp);
int expire_all (char *croot, char *hfile, char *shexp);

/*
 * Remove a single URL, a site, a site against regexp,
 * or entire cache against regexp.
 */
int remove_one (char *croot, char *cfile);
int remove_site(char *croot, char *ufile, char *shexp);
int remove_all (char *croot, char *hfile, char *shexp);


/*
 * Print a human-readable SELECT <OPTION> for a time t given in seconds.
 * Check if the value is the same as selected value selval, and if so,
 * add the SELECTED parameter.
 */
void human_time_option(long t, long selval);

/*
 * Parse a human readable time, such as:
 *
 *	2 hours 30 mins
 *
 * Knows words "day", "hour" and "min", with "sec" the default.
 */
long parse_human_time(char *str);

#endif

