/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
#ifndef FRAME_CONF_H
#define FRAME_CONF_H

#ifndef NOINTNSAPI
#define INTNSAPI
#endif /* !NOINTNSAPI */


/* ------------------------------------------------------------------------ */


/*
 * conf.h: Deals with the server configuration file.
 * 
 * Object configuration is an entirely different matter. This deals with
 * things like what port the server runs on, how many children it spawns,
 * and other server-related issues. Information related configuration is
 * handled by the object conf.
 * 
 * Rob McCool
 */

/* This needs to be defined before it's referenced in public/nsapi.h */
#define conf_getglobals INTconf_getglobals

#ifndef NETSITE_H
#include "../netsite.h"
#endif /* !NETSITE_H */

#ifndef FRAME_OBJSET_H
#include "frame/objset.h"
#endif /* !FRAME_OBJSET_H */

#ifndef BASE_DAEMON_H
#include "base/daemon.h"		/* daemon struct */
#endif /* !BASE_DAEMON_H */

#ifndef PUBLIC_FRAME_CONF_H
#include "public/frame/conf.h"
#endif /* !PUBLIC_FRAME_CONF_H */

/* ------------------------------ Constants ------------------------------- */

#if defined(DAEMON_ANY)
#ifdef MCC_PROXY
#define DEFAULT_PORT 8080
#else
#define DEFAULT_PORT 80
#endif
#endif

NSPR_BEGIN_EXTERN_C

/* ------------------------------- Globals -------------------------------- */

/*
 * These server parameters are made global because they really don't
 * belong anywhere else. 
 */

/* These can't be macros because they're used in daemon.h. */
extern char *pidfn;
extern int port;
extern char *chr;

extern char *secure_keyfn;
extern char *secure_certfn;
extern int secure_auth;

#ifdef INTNSAPI

/* ------------------------------ Prototypes ------------------------------ */
/* For NT, this function allows the EXE to pass the server root to the DLL */
NSAPI_PUBLIC void conf_setroot(char *root);


NSAPI_PUBLIC char *INTconf_run_init_functions(void);
NSAPI_PUBLIC char *INTconf_run_late_init_functions(void);

/*
 * SPconf_init_lightweight()
 * Internal use only
 */
NSAPI_PUBLIC char *INTconf_init_lightweight(char *);

/*
 * INTconf_init reads the given configuration file and sets any non-default
 * parameters to their given setting.
 */
NSAPI_PUBLIC char *INTconf_init(char *cfn);
NSAPI_PUBLIC char *INTconf_run_init_functions(void);

/*
 * INTconf_terminate frees any data the conf routines may be holding.
 */

NSAPI_PUBLIC void INTconf_terminate(void);


/*
 * Get a structure with the global variables for this server.
 */
NSAPI_PUBLIC conf_global_vars_s *INTconf_getglobals(void);

NSAPI_PUBLIC char *INTconf_getServerString(void);


/*
 * INTconf_vars2daemon transfers these globals to a daemon structure 
 */
NSAPI_PUBLIC void INTconf_vars2daemon(daemon_s *d);

#define conf_run_init_functions INTconf_run_init_functions
#define conf_run_late_init_functions INTconf_run_late_init_functions
#define conf_init_lightweight INTconf_init_lightweight

#define conf_init INTconf_init
#define conf_terminate INTconf_terminate
#define conf_vars2daemon INTconf_vars2daemon

#define conf_getServerString INTconf_getServerString

#endif /* INTNSAPI */

NSPR_END_EXTERN_C

#endif /* !FRAME_CONF_H */
