/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
#ifndef FRAME_FUNC_H
#define FRAME_FUNC_H

#ifndef NOINTNSAPI
#define INTNSAPI
#endif /* !NOINTNSAPI */

/*
 * func.h: Handles the function hash table
 * 
 * httpd uses a table of internal functions hashed by a name string such that
 * users can reference specific functions from the configuration files.
 * 
 * Any function referenced by configuration files will be passed a 
 * parameter, a Request structure. The functions do not return anything.
 *
 * Rob McCool
 */

#ifndef NETSITE_H
#include "../netsite.h"
#endif /* !NETSITE_H */

#ifndef BASE_PBLOCK_H
#include "../base/pblock.h"
#endif /* !BASE_PBLOCK_H */

#ifndef BASE_SESSION_H
#include "base/session.h"       /* Session structure */
#endif /* !BASE_SESSION_H */

#ifndef FRAME_REQ_H
#include "req.h"                /* Request structure */
#endif /* !FRAME_REQ_H */

#ifndef PUBLIC_FRAME_FUNC_H
#include "public/frame/func.h"
#endif /* !PUBLIC_FRAME_FUNC_H */

/* --------------------------- Hash definitions --------------------------- */


/* 
 * This is a primitive hash function. Once more is known about the names of
 * the functions, this will be optimized.
 */

#define NUM_HASH 20
#define FUNC_HASH(s) (s[0] % NUM_HASH)

NSAPI_PUBLIC struct FuncStruct *func_insert2(char *, FuncPtr, int);
NSAPI_PUBLIC struct FuncStruct *func_find_str(char *);

/* flags for the flags field of the FuncStruct */
#define FUNC_USE_NATIVE_THREAD 0x1


/* -------------------------Native Pool Functions-------------------------- */

#include "prthread.h"
#include "prlock.h"
#include "prcvar.h"

typedef struct func_work_item_t {
    FuncPtr func;
    Session *sn;
    Request *rq;
    pblock *pb;
    PRLock *waiter_lock;
    PRCondVar *waiter_cvar;
    PRBool work_complete;
    PRInt32 work_status;     /* return code from worker function */
    struct func_work_item_t *next;
} func_work_item_t;

NSAPI_PUBLIC PRInt32 func_native_pool_init();
NSAPI_PUBLIC PRInt32 func_native_pool_wait_work(FuncPtr fn, pblock *pb,
                                                Session *sn, Request *rq);
NSAPI_PUBLIC func_work_item_t * func_native_pool_get_work();
NSAPI_PUBLIC void func_native_pool_thread_main(void *unused);
NSAPI_PUBLIC void func_native_pool_create_thread();

#ifdef INTNSAPI

NSPR_BEGIN_EXTERN_C

/*
 * INTfunc_init reads the static FuncStruct arrays and creates the global 
 * function table from them.
 *
 * INTfunc_init will only read from the static arrays defined in func.c.
 */

NSAPI_PUBLIC void INTfunc_init(struct FuncStruct *func_standard);

/*
 * INTfunc_find returns a pointer to the function named name, or NULL if none
 * exists.
 */

NSAPI_PUBLIC FuncPtr INTfunc_find(char *name);

NSAPI_PUBLIC FuncPtr INTfunc_replace(char *name, FuncPtr newfunc);

/*
 * INTfunc_exec will try to execute the function whose name is the "fn" entry
 * in the given pblock. If name is not found, it will log a misconfig of
 * missing fn parameter. If it can't find it, it will log that. In these
 * cases it will return REQ_ABORTED. Otherwise, it will return what the 
 * function being executed returns.
 */

NSAPI_PUBLIC int INTfunc_exec(pblock *pb, Session *sn, Request *rq);

/*
 * INTfunc_insert dynamically inserts a named function into the server's
 * table of functions. Returns the FuncStruct it keeps in internal 
 * databases, because on server restart you are responsible for freeing 
 * (or not) its contents.
 */

NSAPI_PUBLIC struct FuncStruct *INTfunc_insert(char *name, FuncPtr fn);

NSAPI_PUBLIC int INTfunc_set_native_thread_flag(char *name, int flags);

NSAPI_PUBLIC int INTobject_execute(directive *inst, Session *sn, void *rq);

NSPR_END_EXTERN_C

#define func_init INTfunc_init
#define func_find INTfunc_find
#define func_replace INTfunc_replace
#define func_exec INTfunc_exec
#define func_insert INTfunc_insert
#define object_execute INTobject_execute
#define func_set_native_thread_flag INTfunc_set_native_thread_flag
#endif /* INTNSAPI */

#endif /* !FRAME_FUNC_H */

