/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * httpact.h: Defines the API characteristics for HTTP servers
 * 
 * Rob McCool
 */


#ifndef HTTPACT_H
#define HTTPACT_H

#ifndef NOINTNSAPI
#define INTNSAPI
#endif /* !NOINTNSAPI */

#include <libaccess/acl.h>
#include <libaccess/aclproto.h>

#ifndef FRAME_REQ_H
#include "frame/req.h"
#endif /* !FRAME_REQ_H */

#ifndef FRAME_OBJECT_H
#include "frame/object.h"
#endif /* !FRAME_OBJECT_H */

#ifndef PUBLIC_NSAPI_H
#include "public/nsapi.h"
#endif /* !PUBLIC_NSAPI_H */


/* ------------------------------ Constants ------------------------------- */

#ifdef MCC_BATMAN
#define NUM_DIRECTIVES 6
#else
#ifdef MCC_PROXY
#define NUM_DIRECTIVES 11
#else
#define NUM_DIRECTIVES 7
#endif
#endif


/* -------------------------- Generic Prototypes -------------------------- */

#ifdef INTNSAPI

NSPR_BEGIN_EXTERN_C

/*
 * directive_name2num will return the position of the abbreviated directive
 * dir in the directive table.
 * 
 * If dir does not exist in the table, it will return -1.
 */

NSAPI_PUBLIC int directive_name2num(char *dir);


/*
 * directive_num2name returns a string describing directive number num.
 */
NSAPI_PUBLIC const char *directive_num2name(int num);


/*
 * servact_finderror looks through the request's object set to find a
 * suitable error function to execute. Returns REQ_PROCEED if a function
 * was found and executed successfully, REQ_NOACTION otherwise.
 */
NSAPI_PUBLIC int INTservact_finderror(Session *sn, Request *rq);


/*
 * Read in and handle one request from the given session
 */
NSAPI_PUBLIC void servact_handle(Session *sn);


/* ----------------------- HTTP specific prototypes ----------------------- */


NSAPI_PUBLIC int INTservact_handle_processed(Session *sn, Request *rq);

/*
 * Returns the translated path (filename) for the given uri, NULL otherwise.
 * If authentication is required for the given uri, nothing is returned even
 * if the current user has authenticated to that area.
 */
NSAPI_PUBLIC char *INTservact_translate_uri(char *uri, Session *sn);

/*
 * Obtains the ACL List given a uri.
 */
NSAPI_PUBLIC int ACL_UriToAclList(char *uri, Session *sn, ACLListHandle_t
**acllistp);


#ifdef MCC_PROXY
NSAPI_PUBLIC int sockd_service(Session *sn, Request *rq);
NSAPI_PUBLIC int sockd_read_request(Session *sn, Request *rq);
/* ----------------------- proxy specific prototypes ---------------------- */

/*
 * Resolves the given hostname, first trying to find a resolver
 * function from obj.conf, and if that fails, just calls gethostbyname().
 *
 */
NSAPI_PUBLIC struct hostent *servact_gethostbyname(char *host, Session *sn, Request *rq);


/*
 * Establishes a connection to the specified host and port using
 * a Connect class function from obj.conf.  Returns the sockect
 * descriptor that is connected (and which should be SSL_Import()'ed
 * by the caller).
 *
 * Returns -2 (REQ_NOACTION), if no such Connect class function exists.
 * The caller should use the native connect mechanism in that case.
 *
 * Returns -1 (REQ_ABORT) on failure to connect.  The caller should not
 * attempt to use the native connect.
 *
 */
NSAPI_PUBLIC int servact_connect(char *host, int port, Session *sn, Request *rq);


#endif	/* ! MCC_PROXY */

NSAPI_PUBLIC int INTservact_uri2path(Session *sn, Request *rq);
NSAPI_PUBLIC int INTservact_pathchecks(Session *sn, Request *rq);
NSAPI_PUBLIC int INTservact_fileinfo(Session *sn, Request *rq);
NSAPI_PUBLIC int INTservact_service(Session *sn, Request *rq);

NSPR_END_EXTERN_C

#define servact_finderror INTservact_finderror
#define servact_handle_processed INTservact_handle_processed
#define servact_translate_uri INTservact_translate_uri
#define servact_uri2path INTservact_uri2path
#define servact_pathchecks INTservact_pathchecks
#define servact_fileinfo INTservact_fileinfo
#define servact_service INTservact_service

#endif /* INTNSAPI */

#endif
