/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
#ifndef FRAME_LOG_H
#define FRAME_LOG_H

#ifndef NOINTNSAPI
#define INTNSAPI
#endif /* !NOINTNSAPI */

/*
 * log.h: Records transactions, reports errors to administrators, etc.
 * 
 * Rob McCool
 */

#ifndef NETSITE_H
#include "../netsite.h"
#endif /* !NETSITE_H */

#ifndef BASE_SESSION_H
#include "../base/session.h"   /* Session structure */
#endif /* !BASE_SESSION_H */

#ifndef FRAME_REQ_H
#include "req.h"               /* Request struct */
#endif /* !FRAME_REQ_H */

#ifndef BASE_EREPORT_H
#include "../base/ereport.h"   /* Error reporting, degrees */
#endif /* !BASE_EREPORT_H */

#ifndef PUBLIC_FRAME_LOG_H
#include "public/frame/log.h"
#endif /* !PUBLIC_FRAME_LOG_H */

#define ERROR_CUTOFF 128


/* ------------------------------ Prototypes ------------------------------ */

#ifdef INTNSAPI

NSPR_BEGIN_EXTERN_C

/*
 * INTlog_error logs an error of the given degree from the function func
 * and formats the arguments with the printf() style fmt. Returns whether the
 * log was successful. Records the current date.
 *
 * sn and rq are optional parameters. If given, information about the client
 * will be reported.
 */

NSAPI_PUBLIC int INTlog_error_v(int degree, char *func, Session *sn,
                                Request *rq, char *fmt, va_list args);

NSAPI_PUBLIC int INTlog_error(int degree, char *func, Session *sn,
                              Request *rq, char *fmt, ...);

/*
 *  Internal use only 
 */
NSAPI_PUBLIC int INTlog_ereport_v(int degree, char *fmt, va_list args);
NSAPI_PUBLIC int INTlog_ereport(int degree, char *fmt, ...);

NSPR_END_EXTERN_C

/* --- End function prototypes --- */

#define log_error_v INTlog_error_v
#define log_error INTlog_error
#define log_ereport_v INTlog_ereport_v
#define log_ereport INTlog_ereport

#endif /* INTNSAPI */

#endif /* !FRAME_LOG_H */

