/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
#ifndef FRAME_OBJECT_H
#define FRAME_OBJECT_H

#ifndef NOINTNSAPI
#define INTNSAPI
#endif /* !NOINTNSAPI */

/*
 * object.h: Handle httpd objects
 * 
 * Manages information about a document from config. files. Called mainly
 * by objset.c.
 * 
 * This module does not assume anything about the directives being parsed.
 * That is handled by objset.c.
 * 
 * This module requires the pblock module from the base library.
 * 
 * Rob McCool
 * 
 */

#ifndef NETSITE_H
#include "../netsite.h"
#endif /* !NETSITE_H */

#ifndef BASE_PBLOCK_H
#include "base/pblock.h"
#endif /* !BASE_PBLOCK_H */

#ifndef BASE_SESSION_H
#include "base/session.h"
#endif /* !BASE_SESSION_H */

#ifndef PUBLIC_FRAME_OBJECT_H
#include "public/frame/object.h"
#endif /* !PUBLIC_FRAME_OBJECT_H */

#ifdef INTNSAPI

/* --- Begin function prototypes --- */

NSPR_BEGIN_EXTERN_C

/*
 * INTobject_create will create a new object and return a pointer to it.
 * It will allocate space for nd directive types and set name accordingly.
 */

NSAPI_PUBLIC httpd_object *INTobject_create(int nd, pblock *name);

/*
 * INTobject_free will free an object and any data associated with it.
 */

NSAPI_PUBLIC void INTobject_free(httpd_object *obj);

/*
 * INTobject_add_directive will add a new directive to the dtable for 
 * the directive class at position dc.
 */

NSAPI_PUBLIC void INTobject_add_directive(int dc, pblock *p, pblock *c, 
                                          httpd_object *obj);


/*
 * Executes the directive specified by inst within the context of the
 * given session and request structures. Returns what the executed function
 * returned (one of the REQ_* codes defined in req.h).
 *
 * This prototype uses void * for Request * in order to avoid including
 * all of req.h. 
 *
 * The definition of object_execute() was moved to frame/func.h.
 */

NSPR_END_EXTERN_C

/* --- End function prototypes --- */

#define object_create INTobject_create
#define object_free INTobject_free
#define object_add_directive INTobject_add_directive

#endif /* INTNSAPI */

#endif /* !FRAME_OBJECT_H */

