/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
#ifndef FRAME_REQ_H
#define FRAME_REQ_H

#ifndef NOINTNSAPI
#define INTNSAPI
#endif /* !NOINTNSAPI */

/*
 * req.h: Request-specific data structures and functions
 * 
 * Rob McCool
 */

#ifndef NETSITE_H
#include "../netsite.h"
#endif /* !NETSITE_H */

#ifndef BASE_PBLOCK_H
#include "../base/pblock.h"
#endif /* !BASE_PBLOCK_H */

#ifndef BASE_SESSION_H
#include "base/session.h"       /* Session structure */
#endif /* !BASE_SESSION_H */

#ifndef FRAME_OBJSET_H
#include "objset.h"
#endif /* !FRAME_OBJSET_H */

#ifndef BASE_FILE_H
#include "../base/file.h"   /* struct stat */
#endif /* !BASE_FILE_H */

#ifndef PUBLIC_FRAME_REQ_H
#include "public/frame/req.h"
#endif /* !PUBLIC_FRAME_REQ_H */

/* ------------------------------ Constants ------------------------------- */

#define REQ_HASHSIZE 10

#define MAX_ETAG		80

#ifdef INTNSAPI

/* --- Begin function prototypes --- */

NSPR_BEGIN_EXTERN_C

/*
 * INTrequest_create creates a new request structure.
 */

NSAPI_PUBLIC Request *INTrequest_create(void);

/*
 * INTrequest_free destroys a request structure.
 */

NSAPI_PUBLIC void INTrequest_free(Request *req);


/*
 * Restarts a request for a given URI internally. If rq is non-NULL, the
 * function will keep the old request's headers and protocol, but with a new 
 * URI and method of GET. If the previous method was HEAD, this is preserved.
 * Any other method becomes GET. You may assume that if you give it a request
 * structure that it will use the same structure.
 *
 * Once you have this new Request, you must then do what you want with
 * it (e.g. send the object back, perform uri2path translation, etc.)
 */

NSAPI_PUBLIC Request *INTrequest_restart_internal(char *uri, Request *rq);


/*
 * INTrequest_header finds the named header depending on the requesting 
 * protocol. If possible, it will not load headers until the first is 
 * requested. You have to watch out because this can return REQ_ABORTED.
 */

NSAPI_PUBLIC int INTrequest_header(char *name, char **value, 
                                   Session *sn, Request *rq);

/*
 * INTrequest_loadheaders just makes sure the headers have been loaded.
 */

#define INTrequest_loadheaders(sn,rq) (int)(REQ_PROCEED)


/*
 * INTrequest_stat_path tries to stat path. If path is NULL, it will look in
 * the vars pblock for "path". If the stat is successful, it returns the stat 
 * structure. If not, returns NULL and leaves a message in rq->staterr. If a 
 * previous call to this function was successful, and path is the same, the 
 * function will simply return the previously found value.
 *
 * User functions should not free this structure.
 */

NSAPI_PUBLIC struct stat *INTrequest_stat_path(char *path, Request *rq);

NSPR_END_EXTERN_C

/* --- End function prototypes --- */

#define request_create INTrequest_create
#define request_free INTrequest_free
#define request_restart_internal INTrequest_restart_internal
#define request_header INTrequest_header
#define request_loadheaders INTrequest_loadheaders
#define request_stat_path INTrequest_stat_path

#endif /* INTNSAPI */

#endif /* !FRAME_REQ_H */
