/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
** dstat.h: Things related to the daemon statistics
** 
** Version 1.0:  UNKNOWN
** Version 1.1:  Original version supported by this implementation
** Version 1.2:  Removes maxprocs; data field still in structure, but 
**               will always be 1.
** 
** 
**/


#ifndef _DSTAT_H_
#define _DSTAT_H_

#include "prtypes.h"
#include "netsite.h"
#include "base/shmem.h"

NSPR_BEGIN_EXTERN_C

#define DSTATS_INVALID_PID	0xffffffff
#define DSTATS_INVALID_KEY	0xffffffff
#define DSTATS_INVALID_SLOT	0xffffffff

/* Codes for child_status */
#define CHILD_EMPTY_SLOT                0xfe
#define CHILD_AWAIT_CONNECT             0xff
#define CHILD_PROCESSING                0x00
#define CHILD_READING                   0x01
#define CHILD_WRITING                   0x02
#define CHILD_RESOLVING                 0x03
#define CHILD_KEEPALIVE_LISTEN          0x04
#define CHILD_KEEPALIVE_ACCEPT          0x05
#define CHILD_DAEMONPROCESSOR           0x06

#define DSTATS_MAJOR_VERSION		(char)0x1
#define DSTATS_MINOR_VERSION		(char)0x1

#define DSTATS_SOFTWARE_ENTERPRICE	(char)0x1
#define DSTATS_SOFTWARE_PERSONAL	(char)0x2
#define DSTATS_SOFTWARE_PROXY		(char)0x3
#define DSTATS_SOFTWARE_NEWS		(char)0x4

#define DSTATS_FILE			"daemonstat."

typedef struct _StatHeader {
    PRInt32  header_size;           /* sizeof StatHeader */
    PRInt32  slot_size;             /* sizeof StatSlot */
    char     software;
    char     version_major;
    char     version_minor;
    char     active;
    char     pad1[2 * sizeof(PRUint32) - 4 * sizeof(char)];
    PRInt32  restart_cookie;              /* changing restart counter */
    PRInt32  obsolete1;                   /* was maxprocs, now always 1 */
    PRInt32  maxthreads;                  /* current MaxThreads */
    char pad2[2 * sizeof(PRUint32) - sizeof(PRInt32)];
    PRUint32 start_time;
} StatHeader;

typedef struct _StatSlot {
    char     mode;
    char     pad[sizeof(PRUint32) - sizeof(char)];

    PRUint32 tot_bytes;
    PRUint32 tot_reqs;
    PRUint32 tot_errs;

    PRUint32 tot_2xx;
    PRUint32 tot_3xx;
    PRUint32 tot_4xx;
    PRUint32 tot_5xx;
    PRUint32 tot_xxx;

    PRUint32 tot_200;
    PRUint32 tot_302;
    PRUint32 tot_304;
    PRUint32 tot_401;
    PRUint32 tot_403;
} StatSlot;

extern PRUint32 dstat_tls_key;
extern PRBool   dstat_initialized;

#define DSTATS_IS_INITIALIZED()	(dstat_initialized == PR_TRUE)

/*
** dstat_init()
**
** Initialize daemon statistics.
**
*/
NSAPI_PUBLIC PRInt32  dstat_init(PRInt32 port, char *addr, PRUint32 threads,
                                 struct passwd *pw);

/*
** dstat_get_thread_slot()
**
** Gets a new unused thread slot for this thread and commits it to this 
** thread's thread local storage
*/
NSAPI_PUBLIC PRUint32  dstat_get_thread_slot(void);

/*
** dstat_destroy()
**
** Cleanup all daemonstat resources and close dstats file.
*/
NSAPI_PUBLIC void dstat_destroy(void);

/*
** child_status()
**
** Set this thread's current state.
*/
NSAPI_PUBLIC void child_status(PRInt32 code);

/*
** child_status2()
**
** Record a status/length completion for this thread having completed
** a request.
*/
NSAPI_PUBLIC void child_status2(PRInt32 http_status, PRUint32 bytes);

NSPR_END_EXTERN_C

#endif /* _DSTAT_H_ */
