/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
** httpdaemon.h
** 
**/


#ifndef _HTTPDAEMON_H_
#define _HTTPDAEMON_H_


#include "base/net.h"
#include "base/session.h"
#include "httpdaemon/dstats.h"

NSPR_BEGIN_EXTERN_C

typedef struct {
    char             *ipstr;    /* primary IP address to bind to */
    int               port;     /* primary port */

    struct passwd    *pw;
    char             *chr;
    char             *pidfn;

    void            (*child_callback)(Session *);
    void            (*rotate_callback)();
    void            (*rcback)(int);

    char             *servid;

    /* Security related */
    int               security;
    char             *secure_keyfn;
    char             *secure_certfn;
    int               secure_auth;
    int               secure_session_timeout;
    long              ssl3_secure_session_timeout;
    int               ssl3;
    int               ssl2;
#ifdef FORTEZZA
    char             *fortezza_personality;
    char             *krlname;
    uint32            fortezza_card_mask;
#endif

    /* Misc */
    int               maxthreads;
    int               single_accept;
    int               num_keep_alives;
    int               mp_optimization;
    int               concurrency;
    char             *version;
    int               accept_language;
#ifdef XP_UNIX
    int               detach;   /* detach child process */
#endif
    char             *admroot;
} daemon_s;

#define SINGLE_ACCEPT_NONE		-1		/* uninitialized state */
#define SINGLE_ACCEPT_OFF		0
#define SINGLE_ACCEPT_ON		1
#define SINGLE_ACCEPT_PARTIAL	2		/* do mutex but not file lock */

NSAPI_PUBLIC void daemon_run(void *d);

/*
 * terminate the main daemon
 */
NSAPI_PUBLIC void daemon_terminate();

#if 0
/* The following are defined in base/daemon.h */

/*
 * daemon_atrestart registers a function to be called fn, with the given
 * void pointer as an argument, when the server is restarted.
 */
NSAPI_PUBLIC void daemon_atrestart(void (*fn)(void *), void *data);

NSAPI_PUBLIC void daemon_dorestart(void);

#endif /* 0 */

#if defined(MCC_ADMSERV) && defined(HAVE_SNMP)
/*
 * The following ifdef is to get around a core problem.
 * should not require snmp stuff.  Need to remove this for
 * Enterprise and 3.0.
 */
extern int http_subagt_run_in_thread(char *nsroot, char **installed);
#endif /* MCC_ADMSERV && HAVE_SNMP*/


/* XXXMB - can we obliterate these? */
NSAPI_PUBLIC void InitializeExitVariable(void);
NSAPI_PUBLIC void WaitForExitVariable(void);
NSAPI_PUBLIC void NotifyExitVariable(void);

NSPR_END_EXTERN_C

#endif /* DAEMON_H */
