/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * internal statistics.
 *
 * A common header file for various pieces of internal statistics.
 *
 *
 */

#ifndef _INTERNAL_STATS_
#define _INTERNAL_STATS_

#include <prtypes.h>

#ifdef __cplusplus
extern "C" {
#endif


typedef struct listenSocketInfo {
    PRUint32      port;
    char          *ipAddr;
    PRBool        securityEnabled;

    PRUint32      numActiveThreads;
    PRUint32      numWaitingThreads;
    PRUint32      numBusyThreads;
    PRUint32      numIdleThreads;

    PRUint32      numMinThreads;
    PRUint32      numMaxThreads;
} listenSocketInfo;

PRBool GetListenSocketInfo(PRUint32 *numSockets, listenSocketInfo **infoArray);


typedef struct keepAliveInfo {
    PRUint32      numKeepAlives;
    PRUint32      maxKeepAlives;

    PRUint32      numKeepAliveHits;
    PRUint32      numKeepAliveFlushes;
} keepAliveInfo;

PRBool GetKeepAliveInfo(keepAliveInfo *info);


typedef struct cacheInfo {
    PRBool        enabled;
    PRUint32      numCacheEntries;
    PRUint32      maxCacheEntries;
    PRUint32      numOpenCacheEntries;
    PRUint32      maxOpenCacheEntries;

    PRUint32      numCacheHits;
    PRUint32      numCacheMisses;

    PRUint32      numCacheInsertsOk;
    PRUint32      numCacheInsertsFail;
    PRUint32      numCacheDeletes;

    PRUint32      pollInterval;
    PRUint32      maxTotalCacheSize;
    PRUint32      maxCurrentTotalCacheSize;
    PRUint32      maxFileSize;
} cacheInfo;

PRBool GetCacheInfo(cacheInfo *info);


typedef struct DNSCacheInfo {
    PRBool        enabled;
    PRUint32      numCacheEntries;
    PRUint32      maxCacheEntries;

    PRUint32      numCacheHits;
    PRUint32      numCacheMisses;

    PRUint32      numCacheInsertsOk;
    PRUint32      numCacheInsertsFail;
    PRUint32      numCacheDeletes;
} DNSCacheInfo;

PRBool GetDNSCacheInfo(DNSCacheInfo *info);


#ifdef __cplusplus
}
#endif

#endif
