/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
** multihome.h
**
** Multihoming module
**
** This module maintains the list of Hardware Virtual Servers that the
** server is configured to use.
**
** This module is indended for internal use only.  
**  
** This module is not thread safe; it expects to be read-only after the
** initialization.
**/

#ifndef _MULTIHOME_H_
#define _MULTIHOME_H_

#include "netsite.h"
#include "prio.h"
#include "prlock.h"
#ifdef XP_UNIX
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#endif

NSPR_BEGIN_EXTERN_C

typedef struct _multiple_domains {
    char               *address;    /* The IP address for this server */
    char               *docroot;    /* The docroot for this server */
    PRFileDesc         *sd;         /* The socket open for this server */
    PRInt32             threads;    /* Max num threads for this server 
                                     * (not currently used) 
                                     */
    PRUint32            port;       /* The port for this server */
    PRBool              security;   /* Security flag for this server */
    PRBool              secure_auth;/* Secure auth flag for this server */
    PRBool              ssl3;       /* SSL V3 flag for this server */
    PRBool              ssl2;       /* SSL V2 flag for this server */
    struct sockaddr_in  addr;
    PRLock              *sockLock;  /* if access to socket is single threaded */
} multiple_domains;

/* FUNCTION: MultihomeInit
** DESCRIPTION:
**     internal init routine for multihoming. Scans config
**     file objects for domains to add.
** RETURNS
**     PR_TRUE if initializes successfully, PR_FALSE otherwise
*/
PRBool MultihomeInit();

/* FUNCTION: GetMultihomeDomains
** DESCRIPTION:
**     Get a pointer to NULL-terminated array of multihome domains
**     currently configured for the server.
** RETURNS
**     multiple_domains pointer
**/
multiple_domains *GetMultihomeDomains();

/* FUNCTION: GetCountMultihomeDomains
** DESCRIPTION:
**     Returns the number of multihome domains currently used configured
**     for the server.
** RETURNS:
**     The count of multihome domains
*/
PRInt32 GetCountMultihomeDomains();

/* FUNCTION: AddMultihomeDomain
** DESCRIPTION:
**     Adds a new server to the list of multihome domains for this server.
** RESTRICTIONS:
**     Calling this routine does not make the server automatically start
**     accepting calls for this host.  This function is expected to be called
**     only at initialization time before the server is started.
** RETURNS:
**     PR_TRUE if successful, PR_FALSE otherwise
*/
PRBool AddMultihomeDomain(char *address, char *docroot, PRUint32 port,
                          PRBool secure, PRBool secure_auth, PRBool ssl3,
                          PRBool ssl2);

/* FUNCTION: MultihomeCleanup
** DESCRIPTION:
**   Cleans up the multihoming data structures; closes all sockets.
** 
**
*/
void MultihomeCleanup();

NSPR_END_EXTERN_C

#endif /* _MULTIHOME_H_ */
