/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * install.h: Macros I use in the install programs
 * 
 * Rob McCool
 */


#ifndef NS_INSTALL_H
#define NS_INSTALL_H


#include <stdio.h>  /* FILE struct */

/* Provided by the admin server */
#define CONFIG_DIRECTORY getenv("ADMSERV_ROOT")
#define INSTALL_DIRECTORY getenv("NETSITE_ROOT")
#define DEFAULT_ADMCONF "admserv.conf"

/* For error page */
#define INSTERR_PAGE "insterr.html"

/* For myprintf() function, used in installation finish programs */
#define MAX_PRINTF 1024

/* Buffer for pathnames. This is way overallocated. */
#define PATH_SIZE 1024


/* Mode for new files */
#define NEWDIR_MODE 0755
#define NEWFILE_MODE 0644
#define NEWSCRIPT_MODE 0755


/* ------------ Output macros for installation finish programs ------------ */


/* Global to say whether interactive refresh should be used or not */
extern int interactive;
/* Used to tell libadmin that we're upgrading */
extern int upgrading;

#define OUT(str) (write(fileno(stdout), buf, strlen(buf)))

#ifdef XP_UNIX
#define END_DOCUMENT() \
{\
  if(interactive) { \
    myprintf("--THIS_STRING_NEVER_HAPPENS--\n"); \
    sleep(1); \
  } \
}
#else /* XP_WIN32 */
#define END_DOCUMENT() \
{\
  if(interactive) { \
    myprintf("--THIS_STRING_NEVER_HAPPENS--\n"); \
	Sleep(1000); \
  } \
}
#endif /* XP_WIN32 */

#define START_DOCUMENT() \
{\
  if(interactive) \
    myprintf("Content-type: text/html\n\n"); \
}

#define NEW_DOCUMENT() {END_DOCUMENT(); START_DOCUMENT(); }



/* ------------------------------- Globals -------------------------------- */


/* ------------------------------ Structures ------------------------------ */


struct admconf {
    char *adminuser;
    char *password;
    char *hosts;
    char *addresses;

    char *admport;
    char *admuser;

    char **lines;
};


/* -------------------------------- Macros -------------------------------- */


/* You know, with a simple change, "cpp" can spell "crap". */
#define STR2BOOL(str) (((str) && (!strcasecmp(str, "on"))) ? 1 : 0)
#define BOOL2STR(x)   ((x) ? "on" : "off")

#define TEXTINPUT(html, str, var) \
  {if(directive_is(line, html)) { \
     output_input("text", str, (var), "size=40"); \
     continue; \
  }}
#define PASSINPUT(html, str, var) \
  {if(directive_is(line, html)) { \
     output_input("password", str, (var), "size=40"); \
     continue; \
  }}
#define RADIOINPUT(htmloff, htmlon, str, var) \
  {if(directive_is(line, htmloff)) { \
     output_input("radio", str, "0", (var) ? NULL : "checked"); \
     continue; \
   } \
   else if(directive_is(line, htmlon)) { \
     fprintf(stdout, "<hr width=80%%>"); \
     output_input("radio", str, "1", (var) ? "checked" : NULL); \
     continue; \
   }}
#define CHECKBOXINPUT(html, str, var) \
  {if(directive_is(line, html)) { \
     output_input("checkbox", str, "1", (var) ? "checked" : NULL); \
     continue; \
  }}
#define OUTPUT(html, str, var) \
  {if(directive_is(line, html)) { \
     fputs("The server ", stdout); \
     printf(str, var); \
     fputs(".<br>\n", stdout); \
     continue; \
  }}
#define BINOUT(html, str, var, offstr, onstr) \
  {if(directive_is(line, html)) { \
     printf(str, ((!(var) || strcmp((var), "on")) ? offstr : onstr)); \
     fputs(".<br>\n", stdout); \
     continue; \
  }}


/* ------------------------------ Prototypes ------------------------------ */

NSPR_BEGIN_EXTERN_C

/* Functions used by installation finish programs */
NSAPI_PUBLIC void install_finish_error(char *errpage, char *msg, char *buf);
NSAPI_PUBLIC void myprintf(char *fmt, ...);
NSAPI_PUBLIC void die(char *page, char *msg, char *info, ...);
NSAPI_PUBLIC void create_subdirs(char *dir, char *page);
NSAPI_PUBLIC void copy_file(char *sfile, char *dfile, int mode);
NSAPI_PUBLIC void copy_dir(char *sdir, char *ddir, int sl, int dl);
NSAPI_PUBLIC void run_server(char *cmd, FILE *html);

NSAPI_PUBLIC void install_killadm(void);
NSAPI_PUBLIC void install_checkport(char *addr, char *port);
NSAPI_PUBLIC void install_checkuser(char *user);
NSAPI_PUBLIC int try_bind(char *addr, int port);

NSAPI_PUBLIC struct admconf *admconf_create(void);
NSAPI_PUBLIC void admconf_write(struct admconf *sc, char *fn);
NSAPI_PUBLIC struct admconf *admconf_scan(char *filename);

NSPR_END_EXTERN_C

#endif
