/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#ifndef __NSCGIARGS_H
#define __NSCGIARGS_H 1

#include "libnscore/nsavpair.h"
#include "libnscore/nsbtree.h"
#include "libnscore/nsmisc.h"
#include "libir/nsirutil.h"

#ifdef XP_WIN32
#define boolean int
#endif

typedef void (*ParseCB)(Str val);
typedef boolean (*NScgiargsCB)(CStr attr, CStr val, NSavpair& opts);

class NScgiargs : public NSbase {
public:
   enum NScgiargsLoad {IMMEDIATE_LOAD, DELAYED_LOAD};

   NScgiargs(NScgiargsLoad=IMMEDIATE_LOAD, NSavpair* = 0);
   NScgiargs(NSavpair*);
  ~NScgiargs();

   NScgiargs& Get(CStr attribute, Str& value, boolean clean=TRUE);
   NScgiargs& Next(CStr attribute, Str& value, boolean clean=TRUE);

   NScgiargs& Install(NScgiargsCB);
   NScgiargs& Install(CStr attribute, NScgiargsCB);
   NScgiargs& Load();

   Str      Args() const;
   Str      Cookie() const;
   Str      CookieGet(CStr attribute, boolean clean=TRUE);
   boolean  IsMozilla() const;
   u16      Length() const;
   Str      QueryString() const;
   Str      RemoteUser() const;
   Str      ServerURL() const;
   Str      UserAgent() const;
   Str      Host() const;
   Str      PathInfo() const;

protected:
   void  Dump(ostream&) const;         // from NSbase
   CStr  Name() const;                 // from NSbase

private:
   static s16  DirCompare(void*, void*);
   static void DirPrint(void*, ostream&);
   static void DirDelete(void*, ostream&);

   struct DirNode {
      Str         attr;
      NScgiargsCB cb;
      };
   
   Str         _argstring;
   NSavpair*   _avpairs;
   boolean     _avpairsAllocated;
   NScgiargsCB _cb;
   u16         _contentLength;
   Str         _cookie;
   void*       _cookieForNSavpair;
   NSavpair    _cookiepairs;
   Str         _pathInfo;
   NSbtree*    _pDir;
   Str         _querystring;
   Str         _serverURL;
   Str         _serverHost;
   Str         _userAgent;
   Str         _remoteUser;

   boolean Parse(Str args, NSavpair& opts, char delim='&');
 
   NScgiargs(const NScgiargs&);             // make inaccessible
   int operator=(const NScgiargs&);       // make inaccessible
   };


inline Str
NScgiargs::Args() const {
   return(_argstring);
   }


inline Str
NScgiargs::Cookie() const {
   return(_cookie);
   }


inline Str
NScgiargs::CookieGet(CStr attribute, boolean clean) {
   Clear();
   Str value = _cookiepairs.Get(attribute);
   if (_cookiepairs.Bad())
      Errnum(_cookiepairs.Errnum());
   else if (clean == TRUE && value != 0)
      UnescapeArg(value);
   return(value);
   }


inline NScgiargs&
NScgiargs::Get(CStr attribute, Str& value, boolean clean) {
   Clear();
   _cookieForNSavpair = 0;
   if (_avpairs == 0)
      Errnum(ctrGetErr)  ;
   else if (_avpairs->Get(attribute, value, _cookieForNSavpair).Bad())
      Errnum(_avpairs->Errnum());
   else if (clean == TRUE)
      UnescapeArg(value);
   return(*this);
   }


inline boolean
NScgiargs::IsMozilla() const {
   static Str MOZILLA = "MOZILLA";
   static size_t mozlen = strlen(MOZILLA);
   return(strncmp(_userAgent, MOZILLA, mozlen) == 0);
   }


inline Str
NScgiargs::Host() const {
   return(_serverHost);
   }


inline u16
NScgiargs::Length() const {
   return(_contentLength);
   }


inline Str
NScgiargs::PathInfo() const {
   return(_pathInfo);
   }


inline Str
NScgiargs::QueryString() const {
   return(_querystring);
   }


inline Str
NScgiargs::RemoteUser() const {
   return(_remoteUser);
   }


inline Str
NScgiargs::ServerURL() const {
   return(_serverURL);
   }


inline Str
NScgiargs::UserAgent() const {
   return(_userAgent);
   }

#endif
