/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#ifndef __NSCGIMSG_H
#define __NSCGIMSG_H 1

#include <iostream.h>
#include "nsirinc.h"
#include "libnscore/nserrmsg.h"

class NScgimsg : public NSbase {
public:
   NScgimsg(ostream&);
  ~NScgimsg();

   NScgimsg&  Cookie(CStr);
   NScgimsg&  Error(CStr key);
   NScgimsg&  Error(CStr key, CStr parm);
   NScgimsg&  Error(CStr key, int parm);
   ostream&   UserStream() const;
   NScgimsg&  Message(CStr key);
   NScgimsg&  Open(CStr filename);
   NScgimsg&  Send(CStr msg);
   NScgimsg&  SendHdr();
   NScgimsg&  SendNoHdr(CStr msg);
   NScgimsg&  SendNoHdr(s32 msg);
   NScgimsg&  Title(CStr);
   NSbool     ErrorLogged() const;
   void       ClearErrorLogged();


protected:
   void  Dump(ostream&) const;   
   CStr  Name() const; 

private:
   Str       _cookie;
   NSbool    _errLogged;
   NSerrmsg  _errormsg;
   boolean   _hdrSent;
   ostream&  _out;
   Str       _title;
   boolean   _titleSent;
   Str       _header;
   boolean   _headerSent;
   Str       _footer;
   boolean   _footerSent;

   NScgimsg&  SendTitle();
   NScgimsg&  SendHeader(void);
   NScgimsg&  SendFooter(void);

   NScgimsg(const NScgimsg&);          // make inaccessible
   int operator=(const NScgimsg&);     // make inaccessible
   };


// public member functions

inline void      
NScgimsg::ClearErrorLogged() {
   _errLogged = FALSE;
   }


inline NScgimsg&  
NScgimsg::Error(CStr key) {
   SendHdr();
   SendTitle();
   SendHeader();
   _out << "<P><B>ERROR:</B> " << _errormsg.Get(key) << endl;
   _errLogged = TRUE;
   return(*this);
   }


inline NScgimsg&  
NScgimsg::Error(CStr key, CStr parm) {
   SendHdr();
   SendTitle();
   SendHeader();
   _out << "<P><B>ERROR:</B> " << _errormsg.Get(key) << " " << parm << endl;
   _errLogged = TRUE;
   return(*this);
   }


inline NScgimsg&  
NScgimsg::Error(CStr key, int parm) {
   SendHdr();
   SendTitle();
   SendHeader();
   _out << "<P><B>ERROR:</B> " << _errormsg.Get(key) << " " << parm << endl;
   _errLogged = TRUE;
   return(*this);
   }


inline NSbool    
NScgimsg::ErrorLogged() const {
   return(_errLogged);
   }


inline NScgimsg&  
NScgimsg::Message(CStr key) {
   SendHdr();
   SendTitle();
   SendHeader();
   _out << _errormsg.Get(key) << endl;
   return(*this);
   }


inline NScgimsg&  
NScgimsg::Send(CStr msg) {
   SendHdr();
   SendTitle();
   SendHeader();
   _out << "<P><B>" << msg << "</B>" << endl;
   return(*this);
   }


inline NScgimsg&  
NScgimsg::SendNoHdr(CStr msg) {
   _out << msg << endl;
   return(*this);
   }


inline NScgimsg&  
NScgimsg::SendNoHdr(s32 msg) {
   _out << msg << endl;
   return(*this);
   }


inline ostream&  
NScgimsg::UserStream() const {
   return(_out);
   }


#endif
