/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
//
// Name:    nsgetmeta.h
// Contact: mjb<barbarin@netscape.com>
// Notes:
//    Extract HTML meta tags from a buffer.
//   
//    Code uses a FSM to "parse" out the name and content attributes of the HTML
//    meta tag.  Note that "valid" HTML is presumed (i.e., matching quote (") 
//    delimiters, matching brackets ("<" and ">"), and the first character after
//    the opening "<" is the first character of the meta tag ("m" or "M").
//
//    The "name" and "content" strings are passed to the callback function 
//    specified to the NSgetMeta function.  The pointers passed to the callback
//    point to storage allocated in the function that will be reused in 
//    subsequent callbacks -- this means that you want to keep the values that
//    are passed to the callback that you should copy them to your own store.
//
//    Additionally, the function returns a "0" on success and a -1 if the
//    function fails (on memory allocation).  The callback function should
//    return 0 (if successful); returning nonzero will cause the NSgetMeta
//    function to stop processing and return the callback's return value.
//    I'd suggest returning a value greater than 0 if you want to communicate
//    an error back to the program using NSgetMeta.
//
// Returns:
//    0       on success (note this).
//    -1      on function failure
//    >0      on callback failure.      
//    
// Example:   
//    #include "nsmfile.h"
//    #include "nsgetmeta.h"
//    
//    int
//    MyCallback(const char* name, const char* content, void* parm) {
//       cout << "name=" << name << ", content=" << content << endl;
//       return(0);
//       }
//    
//    int 
//    main(int argc, char** argv) {
//       NSmapfile infile("nsgetmeta.html");
//    
//       NSgetMeta(infile.Buffer(), infile.Size(), MyCallback, 0);
//    
//       return(EXIT_SUCCESS);
//       }
// -------------------------------------------------------------------------------
#include <ctype.h>
#include <string.h>

typedef int (*CBgetMeta)(const char* name, const char* value, void* parm);

class NSgetMeta {
public:
	NSgetMeta(char* start, int length, CBgetMeta cb, void* parm);
	static int GetMeta(char* buffer, int length, CBgetMeta, void* parm);
};
