/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
//
// Name:    nsirmle.h
// Contact: mjb<barbarin@netscape.com>
// Notes:
//    Encapsulation of a nsirmle record
// -------------------------------------------------------------------------------


#ifndef __NSIRMLE_H
#define __NSIRMLE_H 1

#ifndef NOSYS_NETFD
#include <base/net.h>
#endif

#include "libnscore/nsmfile.h"
#include "libnscore/nsbtree.h"
#include "libnscore/nsavpair.h"

typedef NSbool (*NSmleCB)(ostream&, Str& locInPatFile, PGen parm);

class NSirmle : public NSbase {
public:
   NSirmle(CStr marker = "$$");
  ~NSirmle();

   NSirmle& Cvt(NSmapfile& pat, NSavpair& opts, ostream&);
   NSirmle& Cvt(Str pat, int patlen, NSavpair& opts, ostream&);
   NSirmle& Install(Str tag, NSmleCB, PGen parm);
   NSirmle& Remove(Str tag) {return(Install(tag, 0, 0));}

protected:
   static char VALIDCHARS[];

   void  Dump(ostream&) const;         // from IAbase
   CStr  Name() const;                 // from IAbase

private:
   static s16  DirCompare(PGen, PGen);
   static void DirPrint(PGen, ostream&);
   static void DirDelete(PGen, ostream&);

   struct DirNode {
      Str      tag;
      NSmleCB  cb;
      PGen     parm;
      };
   
   Str       _marker;
   size_t    _markersz;
   NSbtree*  _pDir;

   NSirmle(const NSirmle&);            // make inaccessible
   NSirmle& operator=(const NSirmle&); // make inaccessible
   };

inline NSirmle& 
NSirmle::Cvt(NSmapfile& pat, NSavpair& opts, ostream& o) {
   Cvt(pat.Buffer(), pat.Size(), opts, o);
   return(*this);
   }

#endif
