/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
//
// Name:    nsirutil.h
// Contact: mjb<barbarin@netscape.com>
// -------------------------------------------------------------------------------

#ifndef __NSIRSAFUTIL_H
#define __NSIRSAFUTIL_H 1

#include <iostream.h>
#include "nsirinc.h"
#include "libnscore/nsavpair.h"
#include "libnscore/nsmfile.h"
#include "nscnout.h"


// Name: AddOpt
// Notes:
//    Used frequently enough in the search SAFs to move here.  Just add a name
//    and value pair to a container and output an error message.  Note that the
//    default for NSavpair::Put is to overwrite existing values for matching
//    names.


inline NSbool 
AddOpt(ClientStream* msg, NSavpair* opts, CStr attr, CStr val) {
   NSbool retVal = opts->Put(attr, val).Bad();
   if (retVal) {
      msg->Log("OPTSPUTERR", attr);
      return(FALSE);
      }
   return(TRUE);
}


// Name: FilterDBlistIni
// Notes:
//    Used frequently in the search SAFs.  Used to add dblist.ini name-value pairs
//    to a container object.  Invokes the 'inline' AddOpts, so it must follow
//    the declaration(definition) of AddOpt in a dot-h file.

struct FilterParmsStruct {
   ClientStream*  pMsg;
   NSavpair*      pOpts;
   };

inline NSbool 
FilterDBlistIni(CStr attr, CStr val, void* p) {
   NSbool retVal;
   FilterParmsStruct* parms = (FilterParmsStruct*)p;
   if (val == 0 || *val == 0)
      retVal = TRUE;
   else
      retVal = AddOpt(parms->pMsg, parms->pOpts, attr, val);
   return(retVal);
   }


// Name: BuildValidArgsList
// Notes:
//    Take the _validArgs list (in the .cpp file) and put it into a searchable 
//    container.  These strings are used to determine whether or not an 
//    'argument' passed into the search SAFs is valid.  The valid argument list 
//    is alphabetized only for ease of reading when making changes to this code 
//    (and of course, I could put comments next to each query -- but these are 
//    attributes, and they seem to be self-explanatory....)

NSbool  
BuildValidArgsList(NSavpair& validationList);


// Name: BuildValidSynonymList
// Notes:
//   Same as BuildValidArgsList, but builds a map of synonyms from other arguments
//   to the valid ones.  Used to support RDM format requests.

NSbool  
BuildValidSynonymList(NSavpair& validationList);


// Name: DumpPblocks
// Notes:
//    Okay, this code should be in the server somewhere, but I couldn't find it
//    the day I was looking.  Basically, dump the pblocks so I can see what's in
//    them.  Note that the Session and Request pointers are checked first to see
//    if they are NULL before proceeding -- after all, I'm not sure from where
//    in the server directives this thing will be invoked.

void
DumpPblocks(ostream& o, pblock* pb, Session* sn, Request* rq);


// Name: OpenPatternFile
// Notes:
//    Most of the search stuff deals with HTML pattern files for deciding how to
//    format its HTML output.  These filenames are passed as arguments and 
//    need to be checked for ".." and "//".  Also, these files are only allowed
//    in a particular directory (the 'root' argument to the function).

NSbool  
OpenPatternFile(ClientStream* msg, CStr rootdir, CStr file, NSmapfile& pat);


#endif
