/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
//
// Name:    nssafarg.h
// Contact: mjb<barbarin@netscape.com>
// -------------------------------------------------------------------------------

#ifndef __NSSAFARGS_H
#define __NSSAFARGS_H 1

#include <iostream.h>
#include "nsirinc.h"
#include "libnscore/nsavpair.h"
#include "libnscore/nsbtree.h"

typedef NSbool  (*NSsafargsCB)(CStr attr, CStr value, NSavpair&, PGen parm);

class NSsafargs : public NSbase {
public:
   NSsafargs(NSavpair*&, Session*, Request*, NSsafargsCB=0, PGen parm=0);
   NSsafargs(NSavpair*&, Str, int, NSsafargsCB=0, PGen parm=0);
  ~NSsafargs();

   Str      Get(CStr attribute);
   Str      Get(CStr attribute, PGen& cookie);
   int      Next(CStr attribute, Str& value, PGen& cookie);

   Str      Args() const;
   Str      Host() const;
   NSbool   IsMozilla() const;
   int      Length() const;
   Str      Method() const;
   Str      NameTransBase() const;
   Str      Path() const;
   Str      PathInfo() const;
   Str      QueryString() const;
   Str      RemoteAddr() const;
   Str      RemoteUser() const;
   Str      ServerURL() const;
   Str      UserAgent() const;
   Str      URI() const;

   NSbool   Parse(Str args, int arglen, NSavpair& opts);

protected:
   void Dump(ostream&) const;	// from NSbase
   CStr Name() const; 		// from NSbase

private:
   Str           _argstring;
   NSavpair*     _avpairs;
   NSbool        _avpairsAllocated;
   NSsafargsCB   _cb;
   int           _contentLength;
   PGen          _cbParm;
   Request*      _rq;
   Session*      _sn;

   int     Load();
   int     Load(Str,int);
 
   NSsafargs(const NSsafargs&);           	// make inaccessible
   NSsafargs& operator=(const NSsafargs&);	// make inaccessible
   };

inline Str
NSsafargs::Args() const {
   return(_argstring);
   }


inline Str
NSsafargs::Host() const {
   return(server_hostname);
   }


inline int
NSsafargs::Length() const {
   return(_contentLength);
   }


inline Str
NSsafargs::Method() const {
   return(_rq ? pblock_findval("method", _rq->reqpb) : 0);
   }

inline Str
NSsafargs::NameTransBase() const {
   return(_rq ? pblock_findval("ntrans-base", _rq->vars) : 0);
   }


inline Str
NSsafargs::Path() const {
   return(_rq ? pblock_findval("path", _rq->vars) : 0);
   }


inline Str
NSsafargs::PathInfo() const {
   return(_rq ? pblock_findval("path-info", _rq->vars) : 0);
   }


inline Str
NSsafargs::QueryString() const {
   return(_rq ? pblock_findval("query", _rq->reqpb) : 0);
   }


inline Str
NSsafargs::RemoteAddr() const {
   return(_rq ? pblock_findval("ip", _sn->client) : 0);
   }


inline Str
NSsafargs::RemoteUser() const {
   return(_rq ? pblock_findval("auth-user", _rq->vars) : 0);
   }


inline Str
NSsafargs::ServerURL() const {
   return(http_uri2url("",""));
   }


inline Str
NSsafargs::URI() const {
   return(_rq ? pblock_findval("uri", _rq->reqpb) : 0);
   }


inline Str
NSsafargs::UserAgent() const {
   return(_rq ? pblock_findval("user-agent", _rq->headers) : 0);
   }

#endif

