/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
//
// Name:    nssrchargs.h
// Contact: mjb<barbarin@netscape.com>
// Notes:
//   Class to encapsulate the arguments for the search engine.
//
//    Instantiating this class sets the defaults for all the values that can be
//    set/get
// -------------------------------------------------------------------------------
#ifndef __NSSRCHARGS_H
#define __NSSRCHARGS_H 1

#include "libnscore/nsbase.h"

class NSsearchArgs : public NSbase {
   friend class NSsearch;
public:
   enum {QP_BOOLEAN = 1, QP_OLDFREETEXT, QP_FREETEXT, QP_PASSTHRU};

   NSsearchArgs();
  ~NSsearchArgs();
  
   // "set" accessor functions

   NSsearchArgs&  DefineResultFields(int fieldCnt, CStr fieldNames);
   NSsearchArgs&  HLbegin(CStr defaultIsNone);
   NSsearchArgs&  HLend(CStr defaultIsNone);
   NSsearchArgs&  LargestSet(int defaultIs200);
   NSsearchArgs&  MaxRecords(int defaultIs20);
   NSsearchArgs&  MaxAttributeSize(int defaultIs255);
   NSsearchArgs&  Query(CStr query);
   NSsearchArgs&  QueryType(int queryParser);
   NSsearchArgs&  SearchBase(int defaultIs0);
   NSsearchArgs&  Select(CStr collectionName, CStr collectionPath);
   NSsearchArgs&  SortBy(CStr sortby);

   // "get" accessor functions

   int	CollectionCount();
   CStr	CollectionName(CStr collectionPath);
   CStr	CollectionPath(int collectionOffsetStartsAtZero);
   int  FieldCount() const;
   CStr Fields() const;
   CStr	HLbegin() const;
   CStr	HLend() const;
   int	LargestSet() const;
   int	MaxAttributeSize() const;
   int	MaxRecords() const;
   CStr	Query() const;
   int	QueryType() const;
   CStr	QueryTypeString() const;
   int	SearchBase() const;
   CStr	SortBy() const;

protected:
   void  Dump(ostream&) const;	// from NSbase
   CStr  Name() const;		// from NSbase

private:
   void* _argsData;

   NSsearchArgs(const NSsearchArgs&);	           // make inaccessible
   NSsearchArgs& operator=(const NSsearchArgs&);   // make inaccessible
   };
#endif
