/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
//
// Name:    nssrchres.h
// Contact: mjb<barbarin@netscape.com>
// Notes:
//   Class to encapsulate the search results from the search engine.
//
//   DocsReturned is the number of documents returned from the search  (up to the
//   maximum set by NSsearchArgs::MaxRecords.  DocsFound is the total number of 
//   documents that "matched" the query -- this number may be greater than the
//   number returned by DocsReturned.
//
//   Search result offsets begin at 0 and range up to DocsReturned() - 1;
//
//   The Field() accessor will return the value for a field for a specified name
//   at a specified offset in the result set.  The FieldName() accessor is used to 
//   return the name of a field for a specified offset in the list of fields; the
//   list of fields is defined in NSsearchArgs::DefinedResultsFields.
//
//   CollectionPath is used to return the absolute path to the collection in the
//   result set.  This call is not needed if you are searching a single collection
//   and know the collection path already (this was passed in as part of the 
//   NSsearch::OpenCollection call). 
//
//   DocPath returns the absolute path to a matched document for a specified 
//   offset in the result set.
//
//   The underlying engine is assumed to store two primary keys: the DocPath and
//   the AltKey.  Other name-value pairs are available through the Field()
//   accessor.
//
//   DocScore returns a value between 0 and 100 (inclusive).
// -------------------------------------------------------------------------------
#ifndef __NSSRCHRES_H
#define __NSSRCHRES_H 1

#include "libnscore/nsbase.h"

class NSsearchResults : public NSbase {
   friend class NSsearch;
public:
   NSsearchResults();
  ~NSsearchResults();
   
   // basic get functions

   Str  AltKey(int resultSetOffset) const;
   Str  CollectionPath(int resultSetOffset);
   Str  DocPath(int resultSetOffset) const;
   int  DocsFound() const;
   int  DocsSearched() const;
   int  DocsReturned() const;
   int  DocScore(int resultSetOffset) const;
   Str  ErrorMsg() const;
   int  ErrorNum() const;
   Str  Field(int resultSetOffset, CStr fieldname) const;
   Str  FieldName(int fieldMapOffset) const;

protected:
   void  Dump(ostream&) const;   
   CStr  Name() const; 

private:
   void* _resultsData;

   NSsearchResults(const NSsearchResults&);             // make inaccessible
   NSsearchResults& operator=(const NSsearchResults&);  // make inaccessible
   };
#endif
