/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
//
// Name:    nssrchstructs.h
// Contact: mjb<barbarin@netscape.com>
// Notes:
//    These are the basic structures for the NSsearch interface.  The motivation 
//    for this was to build a set of include files for NSsearch (and NSsearchArgs
//    and NSsearchResults) that did not include any private data members or expose
//    any internal structures.  The single private data member of each of the 
//    three classes just mentioned is a void* pointer.  In the implementation of
//    those classes, the following structures are allocated and assigned to their
//    respective classes.  
//
//    This was done so that the include files could be distributed to other 
//    search vendors as a pure interface that could be coded against.
//
//    Unfortunately, some last minute changes allowed some things to creep into
//    the interface (but they can be easily moved out again, and the interfaces
//    restored).
// -------------------------------------------------------------------------------
#ifndef __NSSRCHSTRUCTS_H
#define __NSSRCHSTRUCTS_H 1

#include "libnscore/nsavpair.h"
#include "vdk.h"

// this constant is used in building the list of attributes to return
// from a search.  by default, DOC_FN, SCORE, and the VdkVgwKey are always 
// returned and so there is no reason to request those in an RDM request.

const int NSSEARCH_DEFAULT_ATTR_CNT = 3;   // default attrs: 
                                           // DOC_FN, SCORE, VdkVgwKey


// The NSsearchArgsData structure contains data necessary for the NSsearchArgs
// class that is used to initialize an NSsearch object.  Note that NSsearchArgs
// must be passed to NSsearch.  Also, NSsearchArgs initializes all of these to
// reasonable defaults.


struct NSsearchArgsData {
   int		maxrecords;
   int		largestSet;
   int		queryParser;
   int		tableFieldsCount;
   int		maxAttributeSize;
   int		searchBase;
   Str		highlightBegin;
   Str		highlightEnd;
   Str		query;
   Str		sortby;
   Str		tableFields;
   NSbool 	highlightOn;
   NSavpair	idxPaths;
   };


// NSsearchResultsData contains information necessary to access the results of
// a search done with the NSsearch class.

struct NSsearchResultsData {
   int		docsFound;
   int		docsReturned;
   int		docsSearched;
   int		errorNum;
   int          score;
   int		searchBase;
   int		tableFieldsCount;
   VdkSession	sessHdl;
   VdkSearch	srchHdl;
   NSbool 	success;
   char		errorMsg[256];
   Str		collectionPath;
   Str*		table;
   Str		tableFields;
   NSavpair*	tableFieldsMap;

   VdkDocReadBatchOut	docReadOut;
   };


// NSsearchSessionData is the "cookie" for preserving the session data between
// search calls.  Each session consists of a set of open collections, the 
// session's language and initialization parameters and the session callback
// function (for getting search error messages).

struct NSsearchSessionData {
   int		collectionCount;
   int		errorNum;
   char		errorMsg[256];
   NSavpair	collectionOffsets;
   Str          initParms;
   Str          langParms;
   void*	callbackFunction;
   void*	callbackParm;

   VdkSession	  sessHdl;
   VdkCollection  collHdlArray[128];
   };



#endif
