/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
#ifndef  _H_IOBJSTORE
#define  _H_IOBJSTORE

#if USE_HASH_STORE
#include <prhash.h>
#endif

#include "netsite.h"
#include "ibase.h"
#include "mdbtree.h"
#include <base/crit.h>


NSPR_BEGIN_EXTERN_C


typedef int (*NSQueryProc)( char *path, char *uri, void *cData, void *XA );

/* Base Interface */
typedef struct IObjectStoreVtbl IObjectStoreVtbl ; 


typedef struct _t_IObjectStore {
	IObjectStoreVtbl  *method; 
	IBase             *owner;
	char              *serverName;
	int               refCnt;         /* Reference count */
	void              *store;
	int               swapNum;
#if USE_HASH_STORE
	PRHashTable       *objStore;
#else
	BTree             *objTree;        /* Object tree */
#endif
	CRITICAL          storeLock;      /* mutual exclusion for store access */
} IObjectStore; 


struct IObjectStoreVtbl { 
	long  (*QueryInterface)( IObjectStore * ics, long GUID, void **iPtr );
	void  (*AddRef)( IObjectStore * ics );
	long  (*Release)( IObjectStore * ics );
	long (*GetType)( IBase *idoc );
    char *(*GetName)( IBase *idoc );
	long (*GetID)( IBase *idoc );

	void  (*OverWriteMethod)( IObjectStore *iobjs,char *method, CMMethod fnc );
	void  (*SwapObjects)( IObjectStore *iobjs, int numObj );
	IBase *(*GetObjInstance)( IObjectStore * ics, long guid, char *objName, void *transaction );
	IBase *(*StoreGetObject)( IObjectStore * ics, long guid, uint32 objID );
	long  (*GetObjInstanceID)( IObjectStore * ics, long guid, char *objName );
    int   (*CStoreAddObject)( IObjectStore *iobjs, IBase *object );
    int   (*CStoreDeleteObject)( IObjectStore *iobjs, IBase *object );
    int   (*QueryStore)( IObjectStore *iobjs, char *queryStr, 
						                NSQueryProc QueryProc, void *cData );
};  

#define IOBJS(a) a->method

NSAPI_PUBLIC IObjectStore *IObjectStoreNew( char *serverName );
NSAPI_PUBLIC void IObjectStoreDestroy(  IObjectStore *iobjs );


NSPR_END_EXTERN_C

#endif  /*   _H_IOBJSTORE */

