/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/* 
 * A non-recursive binary implementation
 *
 * Daryoush Paknad
*/

#ifndef BTREE_H_
#define BTREE_H_

#include "netsite.h"

NSPR_BEGIN_EXTERN_C

#include "mdtypes.h"
#include "prtypes.h"

typedef  int (*ClientFreeProc)( void *cData );
typedef  int (*TravelProc)( void *nodeData, void *cData );


/*
 * BTree Node
 */
typedef struct _t_BNode {
    struct _t_BNode *left;
    struct _t_BNode *right;
    struct _t_BNode *next;
    uint32          key;
	void            *data;
} BNode;

//typedef  int (*KeyCollisionProc)( BNode *node, BNode *newNode );
typedef  int (*KeyCollisionProc)( BNode *node, uint32 key, void *cData );


typedef struct _t_BTree {
  BNode             *root;     /* The binary tree. */
  int               treeHeight;    /* Maximum hight the tree has had. */
  int               nodeCnt;      /* Number of nodes in the tree */
  int               btreeState;   /* BTREE_NORMAL vs. BTREE_FULL */
  int               numColl;   /* BTREE_NORMAL vs. BTREE_FULL */
  KeyCollisionProc  CollisionProc;
} BTree;




#define BTREE_NORMAL 0
#define BTREE_FULL   1


#define  BTREE_SUCCESS       1
#define  BTREE_FAILED        0
#define  BTREE_NOT_FOUND     -1
#define  BTREE_DUPLICATE_KEY -2
#define  BTREE_FMEM_FAILED   -3
#define  BTREE_FULL_TREE     -4
#define  BTREE_BAD_NODE_SIZE -5    
#define  BTREE_BAD_HANDLE    -6

    
NSAPI_PUBLIC BTree * CMNewBTree(KeyCollisionProc  CollisionProc);
NSAPI_PUBLIC BNode *CMBTreeAddNode( BTree *tree, uint32 key, void *nodeData, int *status );
NSAPI_PUBLIC BNode *CMBTreeFindNode( BTree *tree, uint32 key, BNode **parent, BNode **rel, void *cData );
NSAPI_PUBLIC void * CMBTreeDeleteNode( BTree *tree, uint32 key, void *cData );
NSAPI_PUBLIC int CMBTreeDestroy( BTree *tree, ClientFreeProc CFree );
NSAPI_PUBLIC int CMBTreeGetNumNode( BTree *tree );
NSAPI_PUBLIC void CMBTreeInorderTravel( BTree *tree, TravelProc Traversal, void *cData );


NSPR_END_EXTERN_C

#endif /* BTREE_H_ */

  
