/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
#ifndef TIMER_SERVICE_H
#define TIMER_SERVICE_H

#include <base/msgchnel.h>



NSPR_BEGIN_EXTERN_C

typedef void* TIMER_SERVICE; 
enum ServiceType {eINSERT, eDELETE};

/***********************************************************************
  Function : init_timer_service
  Input Parameters : minGranularityinSecs
  Returns : A pointer to TIMER_SERVICE if all goes well;
	    else returns NULL
  The i/p parameter minGranularityinSecs determines the granularity of the
  timer service. It is in seconds. It is set to a default of 300 seconds (5 min)
  and the minium permissible value is 180 seconds (3 minutes)
************************************************************************/
TIMER_SERVICE init_timer_service(int minGranularityinSecs);




/***********************************************************************
  Function : perform_timer_service
  Requires : serviceType - from enum ServiceType indicating wheter to add
			   a new timer alarm or delete an existing one
             A valid TIMER_SERVICE* obtained by call to init_timer_service
             The timeForAlarm
 	     The callback function to be called when the alarm goes off
             This function returns void and takes a void* which is the
             next parameter.
	     A void* clntData (or user data) which is passed onto the
	     callback function when it is invoked.
  Return  :  Should indicate the result of the operation
	     Does not do so currently since the service requester and provider
             are on separate threads. Need to find a solution to this
             Till then services requested are not gauranteed.

   This  function performs the specified service as indiacted by the
   serviceType variable. The service will be refused if the difference
   between the current time and the time for which the alarm is scheduled
   (as indiacated by timeForAlarm) if less than the min granularity specified
   when the TIMER_SERVICE was initially created.
   The parameters fnPtr and clntData have no meaning for a DELETE service and
   can be NULL
   
   
***********************************************************************/
int perform_timer_service(ServiceType serviceType, TIMER_SERVICE ptr,
                          unsigned long timeForAlarm,
                          long (*fnPtr)(short, long, void*, void*), 
			  void* clntData);




/***********************************************************************
  Function : terminate_timer_service
  Requires : TIMER_SERVICE* ptr 
  Returns :  void
  This function removes a previously allocated TIMER_SERVICE variable
************************************************************************/

void terminate_timer_service(TIMER_SERVICE ptr);

NSPR_END_EXTERN_C

#endif TIMER_SERVICE_H
