/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#ifndef nsavpair_h
#define nsavpair_h

/***********************************************************************
** Includes
***********************************************************************/
#include "nsskplst.h"


/***********************************************************************
** Functional Typedef
***********************************************************************/
typedef NSbool  (*NSavpairCB)(CStr attr, CStr val, PGen parm);


//**********************************************************************
// CLASS:	NSavpair
// DESCRIPTION:	Class to affect the storage and retrieval of attribute
// 	value pairs.
//
// EXAMPLE:
//
// RESTRICTIONS:
//
// RETURN CODE TABLE:
//**********************************************************************
class NSavpair : public NSbase {
public:
   enum {NO_DUPES=0, DUPES_OK=1};

   NSavpair();
  ~NSavpair();

   NSavpair&  Del(CStr attr);
   NSavpair&  First(Str& attr, Str& val, PGen& cookie);
   NSavpair&  Get(CStr attr, Str& val);
   NSavpair&  Get(CStr attr, Str& val, PGen& cookie);
   NSavpair&  Load(NSavpairCB, PGen parm = 0);
   NSavpair&  Next(Str& attr, Str& val, PGen& cookie);
   NSavpair&  Put(CStr attr, CStr val, NSbool =NO_DUPES);

   NSbool Get(CStr attr, NSavpairCB, PGen parm = 0);

   Str      Get(CStr attr);
   u16      Count() const;
   void     Dump(ostream&) const;   
   
protected:
   CStr  Name() const; 

private:
   NSskiplist  _dir;

   NSavpair(const NSavpair&);			// make inaccessible
   NSavpair& operator=(const NSavpair&);	// make inaccessible
   };


/***********************************************************************
** Inline Functions
***********************************************************************/
inline u16
NSavpair::Count() const {
   return(_dir.Count());
   }

inline NSavpair&
NSavpair::Del(CStr attr) {
   if (_dir.Delete(attr).Bad())
      Errnum(_dir.Errnum());
   return(*this);
   }

inline NSavpair&  
NSavpair::First(Str& attr, Str& val, PGen& cookie) {
   Clear();
   _dir.First(attr, val, cookie);
   return(*this);
   }

inline Str
NSavpair::Get(CStr attr) {
   Str val = 0;
   PGen cookie = 0;
   _dir.Get(attr, val, cookie);
   return(val);
   }

inline NSavpair&  
NSavpair::Get(CStr attr, Str& val) {
   Clear();
   PGen cookie = 0;
   _dir.Get(attr, val, cookie);
   return(*this);
   }

inline NSavpair&  
NSavpair::Get(CStr attr, Str& val, PGen& cookie) {
   Clear();
   _dir.Get(attr, val, cookie);
   return(*this);
   }

inline NSavpair&  
NSavpair::Next(Str& attr, Str& val, PGen& cookie) {
   Clear();
   _dir.Next(attr, val, cookie);
   return(*this);
   }

inline NSavpair&  
NSavpair::Put(CStr attr, CStr val, NSbool  allowDuplicates) {
   Clear();
   if (_dir.Put(attr, val, allowDuplicates).Bad())
      Errnum(_dir.Errnum());
    return(*this);
   }

#endif /* nsavpair_h */
