/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#ifndef nsbtree_h
#define nsbtree_h

/***********************************************************************
** Includes
***********************************************************************/
#include "nsbase.h"


//**********************************************************************
// CLASS:	NSbnode
// DESCRIPTION:	Node class used to support NSbtree class.
//
// EXAMPLE:
//
// RESTRICTIONS:
//
// RETURN CODE TABLE:
//**********************************************************************
class NSbnode {
public:
   NSbnode(PGen d);

   NSbnode* left;
   NSbnode* right;
   PGen     data;
   };


//**********************************************************************
// CLASS:	NSbtree
// DESCRIPTION:	Simple btree implementation
//
// EXAMPLE:
//
// RESTRICTIONS:
//
// RETURN CODE TABLE:
//**********************************************************************
class NSbtree : public NSbase {
public:
   typedef void (*TraverseFnx)(PGen data, ostream&);
   typedef s16 (*CompareFnx)(PGen data1, PGen data2);

   NSbtree(CompareFnx, TraverseFnx);
  ~NSbtree();

   PGen      Find(PGen data);
   NSbtree&  Insert(PGen data);
   void      Traverse(ostream& o) const;
   void      Traverse(TraverseFnx, ostream& o);

protected:
   void  Dump(ostream&) const;   
   CStr  Name() const; 

private:
   CompareFnx  _comp;
   NSbnode*    _root;
   TraverseFnx _traverse;

   PGen  Find(NSbnode* r, PGen data);
   void  Traverse(NSbnode* root, ostream& o) const;
   void  Traverse2(NSbnode* root, ostream& o) const;
   };


/***********************************************************************
** Global Inline Functions
***********************************************************************/

/***********************************************************************
** FUNCTION:	DeleteNode
** DESCRIPTION:
**   Bound to the tranverse function during destruction.
***********************************************************************/
inline void
DeleteNode(PGen node, ostream&) {
   delete ((NSbnode*)node);
   }


/***********************************************************************
** NSbnode Inline Functions
***********************************************************************/
inline
NSbnode::NSbnode(PGen d)
:  left(0), right(0), data(d) {
   }


/***********************************************************************
** NSbtree Inline Functions
***********************************************************************/
inline
NSbtree::NSbtree(CompareFnx f, TraverseFnx t) 
:  _comp(f), _root(0), _traverse(t) {
   }

inline PGen
NSbtree::Find(PGen data)  {
   return(Find(_root, data));
   }

inline void
NSbtree::Traverse(ostream& o) const {
   Traverse(_root, o);
   }

inline void
NSbtree::Traverse(TraverseFnx fnx, ostream& o) {
   TraverseFnx original = _traverse;
   _traverse = fnx;
   Traverse(_root, o);
   _traverse = original;
   }

#endif /* nsbtree_h */
