/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#ifndef nsdefs_h
#define nsdefs_h

/***********************************************************************
** Includes
***********************************************************************/
#include <stddef.h>
#include <sys/types.h>


/***********************************************************************
** Syntactic sugar courtesy of Barbarino et al.
***********************************************************************/
#define DISCARD        (void)
#define forever        for(;;)
#define RESET          0

#ifndef TRUE
#define TRUE  1
#endif

#ifndef FALSE
#define FALSE 0
#endif

#ifndef YES
# define YES            1
# define NO             0
#endif /* YES */

/* Copy structures */
#define nsSTRUCT_CPY(a,b)    (a) = (b)

/* Clear/zero-out a structure */
#define nsSTRUCT_CLR(a) nsMemset((char *)(&a),0,sizeof(a))

/* Clear/zero-out an array */
#define nsSTRUCTV_CLR(a)      nsMemset((char *)(a),0,sizeof(a))

/* min/max macro */
#define nsMIN(x, y)  ((x) < (y) ? (x) : (y))
#define nsMAX(x, y)  ((x) > (y) ? (x) : (y))

/* Bit test, set, clear */
#define nsBitTest(x,y) ((x) & (y))
#define nsBitSet(x,y) ((x) |= (y))
#define nsBitClear(x,y) ((x) &= (~(y)))

/*
 * nsgbits(x,p,n)   - get n right bits starting at p
 * nssbits(x,p,n,y) - set n right bits starting at p to n rightmost
 *                    bits of y
 */
#define nsgbits(x,p,n)   (((x) >> ((p)+1-(n))) & ~(~0 << (n)))
#define nssbits(x,p,n,y) ((x) | (((y) & ~(~0 << (n))) << (p)))

/* Round word up to boundary b */
#define nsrwu(w,b) (((size_t)(w)+((b)-1)) & ~((b)-1))

/* Round word down or truncate to boundary b */
#define nsrwd(w,b) ((size_t)(w) & ~((b)-1))

#ifndef offsetof
# define offsetof(type,identifier) ((size_t)&(((type*)0)->identifier))
#endif /* offsetof */


#if defined(NS_USE_OLD_ALL) || defined(NS_USE_OLD_PROTO_MACROS)
/***********************************************************************
** If there is not a K&R C++ compiler in the set of native compilers we use
** then this stuff should all be scrapped.
***********************************************************************/
/*
** If __STDC__ is defined we have an ANSI compiler.
** Also some platforms don't define __STDC__ correctly.
*/
#if defined(__STDC__) || defined(_AIX) \
    || (defined(__mips) && defined(_SYSTYPE_SVR4))
# define NS_ANSI
#endif

/*
** Usage:
**
** Prototypes
**   int
**   printf PROTO((CONST char *format, VA_TYPE));
**
** Function declaration (void args)
**   int
**   DEFUN_VOID(
**   foobar,())
**   {
**     /o code goes here o/
**   }
**
** Function declaration (fixed args)
**   FILE *
**   DEFUN(
**   fopen,(filename, mode),
**     CONST char *filename SEP
**     CONST char *mode)
**   {
**     /o code goes here o/
**   }
**
** Function declaration (variable args)
**   int
**   DEFUN_VAR(
**   printf,(format, va_alist),
**     CONST char *format SEP
**     VA_TYPE)
**   {
**      va_list ap;
**      VA_START(ap,format);
**
**      /o code goes here, use va_arg(ap, type) to skip through parameters o/
**
**      va_end(ap);
**   }
*/
#ifdef NS_ANSI
# include <stdarg.h>
# define VOIDP                            void *
# define CONST                            const
# define SIGNED                           signed
# define SEP                              ,
# define VA_TYPE                          ...
# define VA_START(argp,parm)              va_start(argp, parm)
# define DEFUN(func,arglist,argtype)      func (argtype)
# define DEFUN_VAR(func,arglist,argtype)  func (argtype)
# define DEFUN_VOID(func,arglist)         func (void)
# ifndef PROTO
# define PROTO(arglist)                   arglist
# endif
#else
# include <varargs.h>
# define VOIDP                            char *
# define CONST
# define SIGNED
# define SEP                              ;
# define VA_TYPE                          va_dcl
# define VA_START(argp,parm)              va_start(argp)
# define DEFUN(func,arglist,argtype)      func arglist argtype ;
# define DEFUN_VAR(func,arglist,argtype)  func arglist argtype
# define DEFUN_VOID(func,arglist)         func ()
# ifndef PROTO
# define PROTO(arglist)                   ()
# endif
#endif /* NS_ANSI */
#endif /* NS_USE_PROTO_MACROS */

#endif /* nsdefs_h */
