/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#ifndef nserrno_h
#define nserrno_h

/***********************************************************************
** Includes
***********************************************************************/
#include <errno.h>

#include "nscore.h"


/***********************************************************************
** Application specific error codes cover the range 50000 to 59999
***********************************************************************/
const u16 appSpecificStart	= 50000;
const u16 appSpecificEnd	= 59999;

/***********************************************************************
** General Error Codes
***********************************************************************/
const u16 noError		=     0;
const u16 unknownError		=     1;
const u16 logicError		=     2;
const u16 notImplemented	=     3;
const u16 invalidArgument	=     4;
const u16 outOfMem		=     5;
const u16 argumentOutOfRange	=     6;
const u16 userAbort		=     7;
const u16 badCRC		=     8;
const u16 invalidParmFile	=     9;
const u16 spaceNotAvailable	=    10;
const u16 endOfStream		=    11;
const u16 uninitializedData	=    12;
const u16 invalidValue		=    13;
const u16 copyFailure		=    14;
const u16 incompatibleSizes	=    15;
const u16 invalidConversion	=    16;
const u16 invalidData		=    17;
const u16 tooComplicated	=    18;
const u16 badVersion		=    20;
const u16 notFound		=    21;
const u16 invalidRequest	=    22;
const u16 duplicateField        =    23;
const u16 nothingToIndex        =    24;
const u16 endConversation       =    25;

/***********************************************************************
** File Error Codes
***********************************************************************/
const u16 fileEOF		=    41;
const u16 fileOpenErr		=    42;
const u16 fileCreateErr		=    43;
const u16 fileCloseErr		=    44;
const u16 fileFlushErr		=    45;
const u16 fileReadErr		=    46;
const u16 fileWriteErr		=    47;
const u16 fileTellErr		=    48;
const u16 fileSeekErr		=    49;
const u16 fileChsizeErr		=    50;
const u16 fileReadOnlyErr	=    51;
const u16 fileNotOpen		=    52;
const u16 fileExistsErr		=    53;

/***********************************************************************
** Container Error Codes
***********************************************************************/
const u16 ctrAllocErr		=    60;
const u16 ctrCreateErr		=    61;
const u16 ctrDelErr		=    62;
const u16 ctrFreeErr		=    63;
const u16 ctrGetErr		=    64;
const u16 ctrPutErr		=    65;
const u16 ctrReadErr		=    66;
const u16 ctrWriteErr		=    67;
const u16 ctrConcatErr		=    68;
const u16 ctrInitErr		=    69;

/***********************************************************************
** Heap Manager Error Codes
***********************************************************************/
const u16 heapAllocErr		=    80;
const u16 heapCreateErr		=    81;
const u16 heapFreeErr		=    82;
const u16 heapGetErr		=    83;

/***********************************************************************
** Locking Error Codes
***********************************************************************/
const u16 commitFailed		=   100;
const u16 lockFileOpenErr	=   101;
const u16 lockFileUnavailable	=   102;
const u16 writeLockBroken	=   103;

/***********************************************************************
** Socket Error Codes
***********************************************************************/
const u16 sockCreateErr		=   120;     // socket stuff
const u16 sockBindErr		=   121;
const u16 sockReadErr		=   122;
const u16 sockWriteErr		=   123;
const u16 sockEOF		=   124;
const u16 sockAcceptErr		=   125;
const u16 sockSetOptErr		=   126;
const u16 sockGetHostErr	=   127;
const u16 sockConnectErr	=   128;
const u16 sockListenFailed      =   129;
const u16 sockRemoteEndClosed   =   130;
const u16 sockSelectErr         =   131;
const u16 sockBadDesc           =   132;
const u16 sockSelectIntr        =   133;
const u16 sockTimedOut          =   134;
const u16 sockBadAddress        =   135;
// Some of error values renamed during migration from iasrc2.
const u16 sockCreateFailed      =   sockCreateErr;
const u16 sockSetOptFailed      =   sockSetOptErr;
const u16 sockBindFailed        =   sockBindErr;
const u16 sockConnectFailed     =   sockConnectErr;
const u16 sockIntr              =   sockSelectIntr;

/***********************************************************************
** Signal Error Codes
***********************************************************************/
const u16 sigErr		=   600;

/***********************************************************************
** System Error Codes
***********************************************************************/
const u16 sysErr		=  1000;

/***********************************************************************
** DynaLoader Error Codes
***********************************************************************/
const u16 dlLoadError		=  1300;
const u16 dlUnloadError		=  1301;

/***********************************************************************
** Security Error Codes
***********************************************************************/
const u16 secBadRNG		=  2000;
const u16 secBadRC4		=  2001;
const u16 secCheckFailed	=  2002;
const u16 secBadCookie		=  2003;

/***********************************************************************
** VDK Error Codes
***********************************************************************/
const u16 vdkCollectionSubmit	=  2100;
const u16 vdkCollectionOpen	=  2101;    
const u16 vdkSessionNew		=  2102;
const u16 vdkSearchGetInfo      =  2103;
const u16 vdkDocReadBatch       =  2104;
const u16 vdkSearchNew          =  2105;
const u16 vdkOptimize           =  2106;

/***********************************************************************
** Search Error Codes
***********************************************************************/
const u16 emptyQuery              = 4000;
const u16 complicatedQuery        = 4001;
const u16 sessionCreateFailed     = 4002;
const u16 collectionOpenFailed    = 4003;
const u16 searchError             = 4004;
const u16 resultSetReadFailed     = 4005;
const u16 searchStatusReadFailed  = 4006;
const u16 docStreamCreateFailed   = 4007;
const u16 docStreamReadFailed     = 4008;
const u16 expandOpenFailed        = 4009;
const u16 expandReadFailed        = 4010;
const u16 expandFailedNoWordAssist = 4011;

/***********************************************************************
** Loader Error Codes
***********************************************************************/
const u16 sessOpenFailed          = 6000;
const u16 collectionFound         = 6001;
const u16 collectionNotFound      = 6002;
const u16 collectionCreateFailed  = 6003;
const u16 collectionBackupFailed  = 6004;
const u16 collectionNotOpen		  = 6005;

const u16 attrFound				  = 6010;
const u16 attrNotFound            = 6011;
const u16 renameSysAttr           = 6012;
const u16 deleteSysAttr           = 6013;
const u16 userDefKeyFound         = 6014;
const u16 noAccessRight           = 6015;
const u16 docNotFound             = 6016;
const u16 invalidAttrNm			  = 6017;
const u16 exceedMaxTotAttrLimit   = 6018;
const u16 exceedMaxTxtAttrLimit   = 6019;
const u16 exceedMaxNumAttrLimit   = 6020;
const u16 exceedMaxDatAttrLimit   = 6021;
const u16 infoSavedIdxLater		  = 6022;

const u16 stylePathNotFound       = 6030;
const u16 vdkHomePathNotFound     = 6031;
const u16 dblistPathNotFound      = 6032;
const u16 fileRenameFailed		  = 6033;
const u16 idxTmpPathNotFound	  = 6034;
const u16 idxMSFltPathNotFound	  = 6035;
const u16 dirCreateFailed		  = 6036;
const u16 pathNotFound			  = 6037;

const u16 invalidDocType		=	6301;
const u16 dlObjNotFound			=	6302;
const u16 unknownDocType		=	6303;
const u16 docConvertFailed		=	6304;

#endif /* nserrno_h */
