/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#ifndef nsfname_h
#define nsfname_h

/***********************************************************************
** Includes
***********************************************************************/
#if defined(OS_osf1) // for access prototype
#include <sys/access.h>
#endif // OS_osf1

#include "nsbase.h"


//**********************************************************************
// CLASS:	NSfname
// DESCRIPTION:	File name generator.
//
// EXAMPLE:
//
// RESTRICTIONS:
//
// RETURN CODE TABLE:
//**********************************************************************
class NSfname : public NSbase
{
  public:
    enum AccessType
    {
	READ = 0x01,
	WRITE = 0x02,
	EXECUTE = 0x04,
	EXISTS = 0x08
    };

    // a 0 directory implies 'the current directory'
    NSfname(CStr fname, CStr dir = 0);

    // paths is a null terminated list of directories in which to look
    // for the file. Gets the first match. If name is an absolute path,
    // no path lookup is done.
    NSfname(CStr fname, CStr* paths);

   ~NSfname();

    NSfname& Init(CStr fname, CStr dir = 0);

    inline CStr fullname(void) const;
    operator CStr (void) const;

    // test for file accessibility
    NSbool  access(AccessType how);

    // returns TRUE if the file exists
    inline NSbool  exists(void);

    // returns TRUE if we can read, write, execute the file
    inline NSbool  canRead(void);
    inline NSbool  canWrite(void);
    inline NSbool  canExecute(void);

  protected:
    void Dump(ostream&) const;
    CStr Name() const;
    Str _fullname;

  private:

    /* turn off object copying */
    NSfname(const NSfname& old);
    NSfname& operator=(const NSfname& old);
};


/***********************************************************************
** NSfname Inline Function
***********************************************************************/
inline CStr NSfname::fullname(void) const { return _fullname; }
inline NSfname::operator CStr(void) const { return _fullname; }
inline NSbool  NSfname::exists(void) { return access(EXISTS); }
inline NSbool  NSfname::canRead(void) { return access(READ); }
inline NSbool  NSfname::canWrite(void) { return access(WRITE); }
inline NSbool  NSfname::canExecute(void) { return access(EXECUTE); }

#endif /* nsfname_h */
