/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
//
// Name:    nslink.h
// Contact: mjb<barbarin@netscape.com>
// Notes:
//   C++ to encapsulate NSPR prlink calls (this is the 3rd generation on NSDL, the
//   2nd generation used ACE, pre-NSPR2.0)
// -------------------------------------------------------------------------------


#ifndef __NSLINK_H
#define __NSLINK_H 1

#include "prlink.h"

typedef void* NSlinkHdl;

class NSlink {
public:
   NSlink(const char* libName = 0);
  ~NSlink();
   NSlinkHdl Load(const char* libName);
   int       Unload(NSlinkHdl = 0);
   void*     GetSymbol(const char* symName, NSlinkHdl = 0);

   int  Good(NSlinkHdl = 0) const;
   int  Bad(NSlinkHdl = 0) const;

   
private:
   PRLibrary* _currHdl;

   NSlink(const NSlink&);         	// make inaccessible
   NSlink& operator=(const NSlink&);	// make inaccessible
   };


inline
NSlink::NSlink(const char* libName) : _currHdl(0) {
   if (libName) {
      _currHdl = PR_LoadLibrary(libName);
      }
   }

inline
NSlink::~NSlink() {
   // note that the destructor must NOT unload the object.
   }


inline NSlinkHdl
NSlink::Load(const char* libName) {
   return((NSlinkHdl)PR_LoadLibrary(libName));
   }


inline int
NSlink::Unload(NSlinkHdl h) {
   PRLibrary* hdl = (PRLibrary*)h;
   return(hdl ? (int)PR_UnloadLibrary(hdl) : (int)PR_UnloadLibrary(_currHdl));
   }


inline void*
NSlink::GetSymbol(const char* symName, NSlinkHdl h) {
   PRLibrary* hdl = (PRLibrary*)h;
   return(hdl ? PR_FindSymbol(hdl, symName) : PR_FindSymbol(_currHdl, symName));
   }


inline int  
NSlink::Good(NSlinkHdl h) const {
   PRLibrary* hdl = (PRLibrary*)h;
   return(hdl ? (hdl != 0) : (_currHdl != 0));
   }


inline int  
NSlink::Bad(NSlinkHdl hdl) const {
   return(!Good(hdl));
   }

#endif
