/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#ifndef nsmfile_h
#define nsmfile_h

/***********************************************************************
** Includes
***********************************************************************/
#ifndef NT
#ifdef HPUX
extern "C" {
	#include <sys/mman.h>
}
#else
#include <sys/mman.h>
#endif
#endif

#include "nsbase.h"


/***********************************************************************
** Globals
***********************************************************************/
enum NSmapfileMode {READONLY, READWRITE};


//**********************************************************************
// CLASS:	NSmapfile
// DESCRIPTION:	Encapsulation of mapped file.
//
// EXAMPLE:
//
// RESTRICTIONS:
//
// RETURN CODE TABLE:
//**********************************************************************
class NSmapfile : public NSbase {
public:
   NSmapfile();   
   NSmapfile(CStr name, NSmapfileMode = READONLY);
  ~NSmapfile();

   NSmapfile& Open(CStr name, NSmapfileMode = READONLY);
   NSmapfile& Close();
   NSmapfile& Reset();

   Str     strchr(CStr source, char target);
   int     strncmp(CStr source, CStr target, size_t targetlen);
   Str     strstr(CStr source, CStr target);

   Str     Buffer() const;
   NSbool  IsOpen() const;
   Str     LastByte() const;
   int     Size() const;

protected:
   void  Dump(ostream&) const;         // from NSbase
   CStr  Name() const;                 // from NSbase

private:
   enum {DEFAULTSZ = 512};

   Str          _buffer;
   NSbool       _ismapped;
   NSbool       _isopen;
   NSmapfileMode _mode;
   int          _size;
#ifndef NT
   int          _fd;
   int          _mapmode;
   int          _sharemode;
#else
   HANDLE       _fd;
   DWORD        _mapmode;
   DWORD        _sharemode;
   HANDLE       _fdMapObj;
   NSbool       _mapObjectExists;
#endif
   };


/***********************************************************************
** NSmapfile Inline Functions
***********************************************************************/
inline Str NSmapfile::Buffer() const {
   return(_buffer);
   }


inline NSbool  NSmapfile::IsOpen() const {
   return(_isopen);
   }


inline Str NSmapfile::LastByte() const {
   return(_buffer + _size - 1);
   }


inline int NSmapfile::Size() const {
   return(_size);
   }

#endif /* nsmfile_h */
