/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#ifndef nsosd_h
#define nsosd_h

/***********************************************************************
** Includes
***********************************************************************/
#ifndef STDIO_SYSH
# include <stdio.h>
# define STDIO_SYSH
#endif

#ifdef __cplusplus
# ifndef IOSTREAM_SYSH
#  include <iostream.h>
#  define IOSTREAM_SYSH
# endif
# ifndef STRSTREAM_SYSH
#  ifdef _WINDOWS
#  include <strstrea.h>
#  else
#  include <strstream.h>
#  endif
#  define STRSTREAM_SYSH
# endif
#endif

#ifndef SYSTYPES_SYSH
# include <sys/types.h>
# define SYSTYPES_SYSH
#endif

#ifndef SYSSTAT_SYSH
# include <sys/stat.h>
# define SYSSTAT_SYSH
#endif

#ifndef SYSPARAM_SYSH
# ifndef _WINDOWS
# include <sys/param.h>
# endif /* _WINDOWS */
# ifdef OS_irix
#  undef MIN
#  undef MAX
# endif /* OS_irix */
# define SYSPARAM_SYSH
#endif

#ifndef STDARG_SYSH
# include <stdarg.h>
# define STDARG_SYSH
#endif

#ifndef FCNTL_SYSH
# include <fcntl.h>
# define FCNTL_SYSH
#endif

#ifndef STDLIB_SYSH
# include <stdlib.h>
# define STDLIB_SYSH
#endif

#ifndef STRING_SYSH
# include <string.h>
#endif

#ifdef unix
# ifndef UNISTD_SYSH
#  include <unistd.h>
#  ifdef OS_irix
#   undef EX_OK
#  endif /* OS_irix */
# endif
#endif

#if defined(OS_osf1) || defined(OS_aix)
# include <unistd.h>
#endif

#ifdef _WINDOWS
# include <windef.h>
# include <winbase.h>
# include <io.h>
# include <direct.h>
# include <time.h>
# define _POSIX_PATH_MAX 255
#endif

#ifndef ERRNO_SYSH
# include <errno.h>
# define ERRNO_SYSH
#endif

#ifndef SIGNAL_SYSH
# include <signal.h>
#endif

#ifndef TIME_SYSH
# include <time.h>
# define TIME_SYSH
#endif

#if defined(OS_aix)
#ifndef SYSTIME_SYSH
# include <sys/time.h>
# define SYSTIME_SYSH
#endif
#endif /* OS_aix */

#ifndef LIMITS_SYSH
# include <limits.h>
# define LIMITS_SYSH
#endif

#ifndef CTYPE_SYSH
#include <ctype.h>
# define CTYPE_SYSH
#endif
 

/***********************************************************************
** Standard Data Types and other architecture/os dependent stuff
***********************************************************************/
typedef const void*    CPGen;
typedef const char*    CStr;
typedef void*          PGen;
typedef char*          Str;

#if defined(OS_solaris)

#ifndef XP_UNIX
# define XP_UNIX      1
#endif
# define nsChsize(f, s)   ftruncate(f, s)
# define DIRSLASH     '/'
# define nsRandom(s)    (rand() % s)
# define nsRemove(s)    unlink(s)
# define nsSeedRandom() srand((unsigned int)time(0))

# if defined(CPU_sparc)

#  define BIGENDIAN TRUE

typedef unsigned char   u8;
typedef unsigned short  u16;
typedef unsigned long   u32;
typedef unsigned char   uchar;
#ifndef SX_ORACLE
typedef unsigned int    uword;
#endif

typedef char            s8;
typedef short           s16;
typedef long            s32;
typedef int             word;
#ifndef SX_ORACLE
typedef signed int      sword;
#endif
typedef unsigned char   schar;

typedef float		Float;
typedef double		Double;

# endif		/* CPU_sparc */

# define DllImport
# define DllExport

#elif defined(OS_irix)

#ifndef XP_UNIX
# define XP_UNIX      1
#endif
# define nsChsize(f, s)    ftruncate(f, s)
# define DIRSLASH     '/'
# define nsRandom(s)    (rand() % s)
# define nsRemove(s)    unlink(s)
# define nsSeedRandom() srand((unsigned int)time(0))

# if defined(CPU_mips)

#  define BIGENDIAN TRUE

typedef unsigned char   u8;
typedef unsigned short  u16;
typedef unsigned long   u32;
typedef unsigned char   uchar;
typedef unsigned int    uword;

typedef char            s8;
typedef short           s16;
typedef long            s32;
typedef int             word;
typedef signed int      sword;
typedef unsigned char   schar;

typedef float		Float;
typedef double		Double;

# endif		/* CPU_mips */

# define DllImport
# define DllExport

#elif defined(OS_aix)

#ifndef XP_UNIX
# define XP_UNIX      1
#endif
# define nsChsize(f, s)   ftruncate(f, s)
# define DIRSLASH     '/'
# define nsRandom(s)    (rand() % s)
# define nsRemove(s)    unlink(s)
# define nsSeedRandom() srand((unsigned int)time(0))

# if defined(CPU_powerpc)

#  define BIGENDIAN TRUE

typedef unsigned char   u8;
typedef unsigned short  u16;
typedef unsigned long   u32;
#ifdef NEVER
/* AIX already has uchar defined in types.h */
typedef unsigned char   uchar;
#endif
typedef unsigned int    uword;

typedef char            s8;
typedef short           s16;
typedef long            s32;
typedef int             word;
typedef signed int      sword;
typedef unsigned char   schar;

typedef float		Float;
typedef double		Double;

# endif		/* CPU_powerpc */

# define DllImport
# define DllExport

#elif defined(OS_hpux)

#ifndef XP_UNIX
# define XP_UNIX      1
#endif
# define nsChsize(f, s)   ftruncate(f, s)
# define DIRSLASH     '/'
# define nsRandom(s)    (rand() % s)
# define nsRemove(s)    unlink(s)
# define nsSeedRandom() srand((unsigned int)time(0))

# if defined(CPU_hppa)

#  define BIGENDIAN TRUE

typedef unsigned char   u8;
typedef unsigned short  u16;
typedef unsigned long   u32;
typedef unsigned char   uchar;
typedef unsigned int    uword;

typedef char            s8;
typedef short           s16;
typedef long            s32;
typedef int             word;
typedef signed int      sword;
typedef unsigned char   schar;

typedef float		Float;
typedef double		Double;

# endif		/* CPU_hppa */

# define DllImport
# define DllExport

#elif defined(OS_osf1)

#ifndef XP_UNIX
# define XP_UNIX      1
#endif
# define nsChsize(f, s)   ftruncate(f, s)
# define DIRSLASH     '/'
# define nsRandom(s)    (rand() % s)
# define nsRemove(s)    unlink(s)
# define nsSeedRandom() srand((unsigned int)time(0))

# if defined(CPU_alpha)

#  define BIGENDIAN TRUE

typedef unsigned char   u8;
typedef unsigned short  u16;
typedef unsigned int    u32;
typedef unsigned long   u64;
typedef unsigned char   uchar;
typedef unsigned int    uword;

typedef char            s8;
typedef short           s16;
typedef int             s32;
typedef long            s64;
typedef int             word;
typedef signed int      sword;
typedef unsigned char   schar;

typedef float           Float;
typedef double          Double;

# endif		/* CPU_alpha */

# define DllImport
# define DllExport

#elif defined(OS_unixware) || defined (OS_ncr) || defined (SOLARISx86) || defined(OS_sco)

#ifndef XP_UNIX
# define XP_UNIX      1
#endif
# define nsChsize(f, s)    ftruncate(f, s)
# define DIRSLASH     '/'
# define nsRandom(s)    (rand() % s)
# define nsRemove(s)    unlink(s)
# define nsSeedRandom() srand((unsigned int)time(0))

#  define BIGENDIAN FALSE

typedef unsigned char   u8;
typedef unsigned short  u16;
typedef unsigned long   u32;
typedef unsigned char   uchar;
typedef unsigned int    uword;

typedef char            s8;
typedef short           s16;
typedef long            s32;
typedef int             word;
typedef signed int      sword;
typedef unsigned char   schar;

typedef float		Float;
typedef double		Double;

# define DllImport
# define DllExport

#elif defined(OS_linux)

#ifndef XP_UNIX
# define XP_UNIX      1
#endif
# define nsChsize(f, s)    ftruncate(f, s)
# define DIRSLASH     '/'
# define nsRandom(s)    (rand() % s)
# define nsRemove(s)    unlink(s)
# define nsSeedRandom() srand((unsigned int)time(0))

# if defined(CPU_i486) || defined(CPU_i586)

#  define BIGENDIAN FALSE

typedef unsigned char   u8;
typedef unsigned short  u16;
typedef unsigned long   u32;
typedef unsigned char   uchar;
typedef unsigned int    uword;

typedef char            s8;
typedef short           s16;
typedef long            s32;
typedef int             word;
typedef signed int      sword;
typedef unsigned char   schar;

typedef float		Float;
typedef double		Double;

# endif		/* CPU_i486 || CPU_i586 */

# define DllImport
# define DllExport

#elif defined(OS_windows)

# define XP_UNIX      0
# undef XP_UNIX
#ifdef _WINDOWS
# define nsChsize(f, s)  _chsize(f, s)
#else
# define nsChsize(f, s)  chsize(f, s)
#endif
# define DIRSLASH     '\\'
# define nsRandom(s)    random(s)
# define nsRemove(s)    remove(s)
# define nsSeedRandom() randomize()

# if defined(CPU_x86)

#if !defined(BIGENDIAN)
#  define BIGENDIAN FALSE
#endif
typedef unsigned char   u8;
typedef unsigned int    u16;
typedef unsigned long   u32;
typedef unsigned char   uchar;

typedef signed char     s8;
typedef int             s16;
typedef long            s32;
typedef signed char     schar;

# define DllImport __declspec( dllimport )
# define DllExport __declspec( dllexport )

# endif		/* CPU_x86 */


#if defined (CPU_alpha)
#  define BIGENDIAN FALSE

typedef unsigned char   u8;
typedef unsigned short    u16;
typedef unsigned int   u32;
typedef unsigned char   uchar;

typedef signed char     s8;
typedef short            s16;
typedef int            s32;
typedef signed char     schar;

# define DllImport __declspec( dllimport )
# define DllExport __declspec( dllexport )

# endif               /* CPU_alpha */

#endif /* standard data types */


/* man, this stuff needs to be cleaned up (looks like the old comapps code) */
#ifdef XP_UNIX
#ifndef boolean
# define boolean	int
#endif /* boolean */
#endif

#ifndef NSbool
# define NSbool		int
#endif

/*
** Implementation limits
*/
#define S8MAXVAL        CHAR_MAX
#define U8MAXVAL        UCHAR_MAX
#define S16MAXVAL       SHRT_MAX
#define U16MAXVAL       USHRT_MAX
#define S32MAXVAL       INT_MAX
#define U32MAXVAL       UINT_MAX

#define U8INVALID       U8MAXVAL
#define U16INVALID      U16MAXVAL
#define U32INVALID      U32MAXVAL

/***********************************************************************
** Memory Management
***********************************************************************/
#define nsMalloc              malloc
#define nsZalloc(size)        calloc(1,(size))
#define nsRealloc             realloc
#define nsFree                free
#define nsMemset(_s, _c, _n)  memset((void*)(_s), (int)(_c), (size_t)(_n))
#define nsMemcpy(_d, _s, _n)  memcpy((void*)(_d), (void*)(_s), (size_t)(_n))
#define nsMemmove(_d, _s, _n) memmove((void*)(_d), (void*)(_s), (size_t)(_n))
#define nsMemcmp(_s, _t, _n)  memcmp((void*)(_s), (void*)(_t), (size_t)(_n))
#define nsBzero(_d, _n)       memset((void*)(_d), (int)0, (size_t)(_n))

/***********************************************************************
** String Functions
***********************************************************************/
#define nsStrlen              strlen
#define nsStrcpy              strcpy
#define nsStrcat              strcat
#define nsStrstr              strstr
#define nsStrchr              strchr
#define nsStrrchr             strrchr
#define nsStrncpy             strncpy
#define nsStrdup              strdup
#define nsStrcmp              strcmp
#define nsStrncmp             strncmp

#if (defined(OS_solaris) || defined(OS_unixware)) && !defined(strcasecmp)
#define nsStrcasecmp          strcasecmp
#define nsStrncasecmp         strncasecmp
/* Help Solaris, which doesn't give us declarations of these functions */
extern int strcasecmp(const char *, const char *);
extern int strncasecmp(const char *, const char *, size_t);
#endif

#if defined(OS_aix)  || defined(OS_irix) || defined(OS_hpux) || defined(OS_osf1)
#define nsStrcasecmp	      strcasecmp
#define nsStrncasecmp         strncasecmp
#endif

#define nsAtol                atol
#define nsStrtol	      strtol
#define nsStrtoul	      strtoul
#define nsStrtod	      strtod

#define nsIsdigit	      isdigit

/*
** Environment
*/
#define nsGetenv	getenv


/***********************************************************************
** Porting Functions 
***********************************************************************/
#ifdef _WINDOWS
#define nsMkdir(_p,_m)        mkdir((char*)_p)
#define nsSleep(_sec)         Sleep(((DWORD)_sec)*1000)
#define nsLstat               _stat
#else
#define nsMkdir(_p,_m)        mkdir((char*)_p,(int)_m)
#define nsSleep(_sec)         sleep((unsigned)_sec)
#define nsLstat               lstat
#endif

#endif /* nsosd_h */
