/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * George Dong - 01/15/96
 *
 */

#ifndef NS_SNMPIF
#define NS_SNMPIF

#ifndef NS_SNMP
#include "ns_snmp.h"
#endif

#include "conf.h"

NSPR_BEGIN_EXTERN_C

/*
 * Name:    subagt_conn_init()
 * Purpose: This is the first function called by an application.
 *          It initializes environment and prepares for SMUX 
 *          connection.
 *
 * INPUT:   self identifier
 *          eg. for http server, Our "identity" is 1.3.6.1.4.1.1450.1 
 *          so you can define
 *            
 *             static ubyte  me[] = {0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01};
 *             static ObjId_t	my_id = {sizeof(me),me,};
 *
 * RETURN:  context pointer which will be used be all other functions.
 *
 */ 
NSAPI_PUBLIC struct snmp_contxt * subagt_conn_init(ObjId_t *);


/*
 * Name:    subagt_conn_magt()
 * Purpose: setup connection with a master agent through SMUX. 
 *          
 *
 * INPUT:    1) context pointer
 *           2) host name on which a SMUX master agent runs
 *
 * RETURN:   
 *          ERR_CONN_FAIL -- connection fail  
 *          NO_ERR -- success
 *
 */
NSAPI_PUBLIC int subagt_conn_magt(struct snmp_contxt *, char *);

/*
 * Name:    subagt_register_mibrow()
 * Purpose: tell a master agent what we are responsible for. 
 *          
 *
 * INPUT:    1) context pointer
 *           2) pointer to a class definition structure
 *              generated by a mib compiler.
 *           3) pointer to a mib row structure
 *
 * RETURN:   
 *          ERR_MEM_FAIL -- malloc fails
 *          ERR_MIB_INIT -- register fails
 *          NO_ERR -- success
 *
 */
NSAPI_PUBLIC int subagt_register_mibrow(struct snmp_contxt *,
                           READONLY void *, void * );

/*
 * Name:    subagt_conn_term()
 * Purpose: disconnect and free all memory. 
 *          
 *
 * INPUT:    1) context pointer
 *
 * RETURN:   void
 *
 */
NSAPI_PUBLIC void subagt_conn_term(struct snmp_contxt *);

/*
 * Name:    subagt_select()
 * Purpose: return fd_set used by snmp module. 
 *          
 *
 * INPUT:    1) context pointer
 *           2) pointer to a fd_set structure
 *
 * RETURN:   
 *          void
 *          
 *
 */
NSAPI_PUBLIC void subagt_select(struct snmp_contxt *, fd_set * );

/*
 * Name:    subagt_fd_count()
 * Purpose: return size of fd_set used by snmp module. 
 *          
 *
 * INPUT:   void
 *
 * RETURN:   
 *          integer
 *          
 *
 */
NSAPI_PUBLIC int subagt_fd_count(void);


/*
 * Name:    subagt_poll()
 * Purpose: process snmp requests. 
 *          
 *
 * INPUT:    1) context pointer
 *           2) pointer to a timeval structure
 *              It specifies how long this routine should block if no
 *              snmp requests are pending. If it is NULL, block indefinitely
 *              and also block the sending of trap.
 * RETURN:   
 *          ERR_AGT_FAIL -- agent fails to response.
 *          NO_REQ_PROC  -- no requests were process
 *          NO_ERR -- OK
 *
 */
NSAPI_PUBLIC int subagt_poll(struct snmp_contxt *, struct timeval * );

/*
 * Name:    subagt_trap()
 * Purpose: send a trap. 
 *          
 *
 * INPUT:    1) context pointer
 *           2) pointer to a notification structure
 *              generated by a mib compiler
 * 
 * RETURN:   
 *          ERR_TRAP_FAIL -- trap-sending fails
 *          NO_ERR -- success
 *
 */
NSAPI_PUBLIC int subagt_trap(struct snmp_contxt *, void *);

/*
 * Free the config struct.
 *
 */
NSAPI_PUBLIC void subagt_conf_destroy(SubagtConfig *);


NSPR_END_EXTERN_C

#endif

