/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
#ifndef PUBLIC_ONEACL_ACLAPI_H
#define PUBLIC_ONEACL_ACLAPI_H

/*
 * File:        aclapi.h
 *
 * Description:
 *
 *      This file defines the functions available in the ACL API.
 */

#ifndef PUBLIC_ONEACL_NSERRDEF_H
#include "nserrdef.h"
#endif /* !PUBLIC_ONEACL_NSERRDEF_H */

#ifndef PUBLIC_BASE_POOL_H
#include "../base/pool.h"
#endif /* !PUBLIC_BASE_POOL_H */

#ifndef PUBLIC_ONEACL_PLISTDEF_H
#include "plistdef.h"
#endif /* !PUBLIC_ONEACL_PLISTDEF_H */

#ifndef PUBLIC_ONEACL_ACLDEF_H
#include "acldef.h"
#endif /* !PUBLIC_ONEACL_ACLDEF_H */

typedef struct ACLDispatchVector ACLDispatchVector_t;
struct ACLDispatchVector {

    /* Error frame stack support */

    void (*f_nserrDispose)(NSErr_t * errp);
    NSEFrame_t *(*f_nserrFAlloc)(NSErr_t * errp);
    void (*f_nserrFFree)(NSErr_t * errp, NSEFrame_t * efp);
    NSEFrame_t *(*f_nserrGenerate)(NSErr_t * errp, long retcode,
                                   long errorid, char * program,
                                   int errc, ...);

    /* Property list support */

    int (*f_PListAssignValue)(PList_t plist, const char *pname,
                              const void *pvalue, PList_t ptype);
    PList_t (*f_PListCreate)(pool_handle_t *mempool,
                             int resvprop, int maxprop, int flags);
    int (*f_PListDefProp)(PList_t plist, int pindex, 
                          const char *pname, const int flags);
    const void * (*f_PListDeleteProp)(PList_t plist, int pindex,
                                      const char *pname);
    int (*f_PListFindValue)(PList_t plist,
                            const char *pname, void **pvalue, PList_t *type);
    int (*f_PListInitProp)(PList_t plist, int pindex, const char *pname,
                           const void *pvalue, PList_t ptype);
    PList_t (*f_PListNew)(pool_handle_t *mempool);
    void (*f_PListDestroy)(PList_t plist);
    int (*f_PListGetValue)(PList_t plist,
                           int pindex, void **pvalue, PList_t *type);
    int (*f_PListNameProp)(PList_t plist, int pindex, const char *pname);
    int (*f_PListSetType)(PList_t plist, int pindex, PList_t type);
    int (*f_PListSetValue)(PList_t plist,
                           int pindex, const void *pvalue, PList_t type);
    void (*f_PListEnumerate)(PList_t plist, PListFunc_t *user_func, 
                             void *user_data);
    PList_t (*f_PListDuplicate)(PList_t plist,
                                pool_handle_t *new_mempool, int flags);
    pool_handle_t *(*f_PListGetPool)(PList_t plist);

    /* ACL attribute handling */

    int (*f_ACL_LasRegister)(NSErr_t *errp, char *attr_name,
                             LASEvalFunc_t eval_func,
                             LASFlushFunc_t flush_func);
    int (*f_ACL_LasFindEval)(NSErr_t *errp, char *attr_name,
                             LASEvalFunc_t *eval_funcp);
    int (*f_ACL_LasFindFlush)(NSErr_t *errp, char *attr_name,
                              LASFlushFunc_t *flush_funcp);

    /* method/dbtype registration routines */

    int (*f_ACL_MethodRegister)(NSErr_t *errp, const char *name,
                                ACLMethod_t *t);
    int (*f_ACL_MethodIsEqual)(NSErr_t *errp,
                               const ACLMethod_t t1, const ACLMethod_t t2);
    int (*f_ACL_MethodNameIsEqual)(NSErr_t *errp,
                                   const ACLMethod_t t, const char *name);
    int (*f_ACL_MethodFind)(NSErr_t *errp, const char *name, ACLMethod_t *t);
    ACLMethod_t (*f_ACL_MethodGetDefault)(NSErr_t *errp);
    int (*f_ACL_MethodSetDefault)(NSErr_t *errp, const ACLMethod_t t);
    int (*f_ACL_AuthInfoGetMethod)(NSErr_t *errp,
                                   PList_t auth_info, ACLMethod_t *t);

    int (*f_ACL_DbTypeRegister)(NSErr_t *errp, const char *name,
                                DbParseFn_t func, ACLDbType_t *t);
    int (*f_ACL_DbTypeIsEqual)(NSErr_t *errp,
                               const ACLDbType_t t1, const ACLDbType_t t2);
    int (*f_ACL_DbTypeNameIsEqual)(NSErr_t * errp,
                                   const ACLDbType_t t, const char *name);
    int (*f_ACL_DbTypeFind)(NSErr_t *errp, const char *name, ACLDbType_t *t);
    const ACLDbType_t (*f_ACL_DbTypeGetDefault)(NSErr_t *errp);
    int (*f_ACL_DbTypeSetDefault)(NSErr_t *errp, ACLDbType_t t);
    int (*f_ACL_AuthInfoGetDbType)(NSErr_t *errp,
                                   PList_t auth_info, ACLDbType_t *t);
    int (*f_ACL_DbTypeIsRegistered)(NSErr_t *errp, const ACLDbType_t dbtype);
    DbParseFn_t (*f_ACL_DbTypeParseFn)(NSErr_t *errp,
                                       const ACLDbType_t dbtype);

    int (*f_ACL_AttrGetterRegister)(NSErr_t *errp,
                                    const char *attr, ACLAttrGetterFn_t fn,
                                    ACLMethod_t m, ACLDbType_t d,
                                    int position);
    int (*f_ACL_AttrGetterFind)(NSErr_t *errp, const char *attr,
                                ACLAttrGetterList_t *getters);
    ACLAttrGetter_t *(*f_ACL_AttrGetterFirst)(ACLAttrGetterList_t *getters);
    ACLAttrGetter_t *(*f_ACL_AttrGetterNext)(ACLAttrGetterList_t *getters,
                                             ACLAttrGetter_t *last);

    int (*f_ACL_RegisterModule)(NSErr_t *errp, const char *moduleName,
                                AclModuleInitFunc func);
    int (*f_ACL_GetAttribute)(NSErr_t *errp, const char *attr, void **val,
                              PList_t subject, PList_t resource,
                              PList_t auth_info, PList_t global_auth);
    int (*f_ACL_RegisterDbName)(NSErr_t *errp, ACLDbType_t dbtype,
                                const char *dbname, const char *url,
                                PList_t plist);
    int (*f_ACL_RegisterDbFromACL)(NSErr_t *errp, const char *url,
                                   ACLDbType_t *dbtype);
    int (*f_ACL_DatabaseFind)(NSErr_t *errp, const char *dbname,
                              ACLDbType_t *dbtype, void **db);
    int (*f_ACL_LDAPDatabaseHandle )(NSErr_t *errp, const char *dbname,
                                     LDAP **ld);
    int (*f_ACL_AuthInfoGetDbname)(PList_t auth_info, char **dbname);
    int (*f_ACL_CacheFlushRegister)(AclCacheFlushFunc_t func);

    /*  ACL language and file interfaces */

    ACLListHandle_t * (*f_ACL_ParseFile)(NSErr_t *errp, char *filename);
    ACLListHandle_t * (*f_ACL_ParseString)(NSErr_t *errp, char *buffer);
    int (*f_ACL_Decompose)(NSErr_t *errp, char **acl,
                           ACLListHandle_t *acl_list);
    int (*f_ACL_WriteString)(NSErr_t *errp, char **acl,
                             ACLListHandle_t *acllist);
    int (*f_ACL_WriteFile)(NSErr_t *errp, char *filename,
                           ACLListHandle_t *acllist);
    int (*f_ACL_FileRenameAcl)(NSErr_t *errp, char *filename,
                               char *acl_name, char *new_acl_name, int flags);
    int (*f_ACL_FileDeleteAcl)(NSErr_t *errp, char *filename,
                               char *acl_name, int flags);
    int (*f_ACL_FileGetAcl)(NSErr_t *errp, char *filename,
                            char *acl_name, char **acl_text, int flags);
    int (*f_ACL_FileSetAcl)(NSErr_t *errp, char *filename,
                            char *acl_text, int flags);

    /*  ACL Expression construction interfaces  */

    ACLExprHandle_t *(*f_ACL_ExprNew)(const ACLExprType_t expr_type);
    void (*f_ACL_ExprDestroy)(ACLExprHandle_t *expr);
    int (*f_ACL_ExprSetPFlags)(NSErr_t *errp,
                               ACLExprHandle_t *expr, PFlags_t flags);
    int (*f_ACL_ExprClearPFlags)(NSErr_t *errp, ACLExprHandle_t *expr);
    int (*f_ACL_ExprTerm)(NSErr_t *errp, ACLExprHandle_t *acl_expr,
                          char *attr_name, CmpOp_t cmp, char *attr_pattern);
    int (*f_ACL_ExprNot)(NSErr_t *errp, ACLExprHandle_t *acl_expr);
    int (*f_ACL_ExprAnd)(NSErr_t *errp, ACLExprHandle_t *acl_expr);
    int (*f_ACL_ExprOr)(NSErr_t *errp, ACLExprHandle_t *acl_expr);
    int (*f_ACL_ExprAddAuthInfo)(ACLExprHandle_t *expr, PList_t auth_info);
    int (*f_ACL_ExprAddArg)(NSErr_t *errp, ACLExprHandle_t *expr, char *arg);
    int (*f_ACL_ExprSetDenyWith)(NSErr_t *errp, ACLExprHandle_t *expr,
                                 char *deny_type, char *deny_response);
    int (*f_ACL_ExprGetDenyWith)(NSErr_t *errp, ACLExprHandle_t *expr,
                                 char **deny_type, char **deny_response);

    /* ACL manipulation */

    ACLHandle_t * (*f_ACL_AclNew)(NSErr_t *errp, char *tag);
    void (*f_ACL_AclDestroy)(NSErr_t *errp, ACLHandle_t *acl);
    int (*f_ACL_ExprAppend)(NSErr_t *errp,
                            ACLHandle_t *acl, ACLExprHandle_t *expr);

    /* ACL list manipulation */

    ACLListHandle_t * (*f_ACL_ListNew)(NSErr_t *errp);
    int (*f_ACL_ListConcat)(NSErr_t *errp, ACLListHandle_t *acl_list1,
                            ACLListHandle_t *acl_list2, int flags);
    int (*f_ACL_ListAppend)(NSErr_t *errp, ACLListHandle_t *acllist,
                            ACLHandle_t *acl, int flags);
    void (*f_ACL_ListDestroy)(NSErr_t *errp, ACLListHandle_t *acllist);
    ACLHandle_t * (*f_ACL_ListFind)(NSErr_t *errp, ACLListHandle_t *acllist,
                                    char *aclname, int flags);
    int (*f_ACL_ListAclDelete)(NSErr_t *errp, ACLListHandle_t *acl_list,
                           char *acl_name, int flags);
    int (*f_ACL_ListGetNameList)(NSErr_t *errp, ACLListHandle_t *acl_list,
                                 char ***name_list);
    int (*f_ACL_NameListDestroy)(NSErr_t *errp, char **name_list);

    /* ACL evaluation */

    int (*f_ACL_EvalTestRights)(NSErr_t *errp, ACLEvalHandle_t *acleval,
                                char **rights, char **map_generic,
                                char **deny_type, char **deny_response,
                                char **acl_tag, int *expr_num);
    ACLEvalHandle_t * (*f_ACL_EvalNew)(NSErr_t *errp, pool_handle_t *pool);
    void (*f_ACL_EvalDestroy)(NSErr_t *errp,
                              pool_handle_t *pool, ACLEvalHandle_t *acleval);
    int (*f_ACL_EvalSetACL)(NSErr_t *errp, ACLEvalHandle_t *acleval,
                            ACLListHandle_t *acllist);
    PList_t (*f_ACL_EvalGetSubject)(NSErr_t *errp, ACLEvalHandle_t *acleval);
    int (*f_ACL_EvalSetSubject)(NSErr_t *errp,
                                ACLEvalHandle_t *acleval, PList_t subject);
    PList_t (*f_ACL_EvalGetResource)(NSErr_t *errp, ACLEvalHandle_t *acleval);
    int (*f_ACL_EvalSetResource)(NSErr_t *errp,
                                 ACLEvalHandle_t *acleval, PList_t resource);

    /* Built-in LAS functions */

    int (*f_LASTimeOfDayEval)(NSErr_t *errp, char *attribute,
                              CmpOp_t comparator, char *pattern,
                              ACLCachable_t *cachable, void **las_cookie,
                              PList_t subject, PList_t resource,
                              PList_t auth_info, PList_t global_auth);
    int (*f_LASDayOfWeekEval)(NSErr_t *errp, char *attribute,
                              CmpOp_t comparator, char *pattern,
                              ACLCachable_t *cachable, void **las_cookie,
                              PList_t subject, PList_t resource,
                              PList_t auth_info, PList_t global_auth);
    int (*f_LASIpEval)(NSErr_t *errp, char *attribute, CmpOp_t comparator,
                       char *pattern, ACLCachable_t *cachable,
                       void **las_cookie, PList_t subject, PList_t resource,
                       PList_t auth_info, PList_t global_auth);
    int (*f_LASDnsEval)(NSErr_t *errp, char *attribute, CmpOp_t comparator,
                        char *pattern, ACLCachable_t *cachable,
                        void **las_cookie, PList_t subject, PList_t resource,
                        PList_t auth_info, PList_t global_auth);
    int (*f_LASGroupEval)(NSErr_t *errp, char *attribute, CmpOp_t comparator,
                          char *pattern, ACLCachable_t *cachable,
                          void **las_cookie, PList_t subject,
                          PList_t resource, PList_t auth_info,
                          PList_t global_auth);
    int (*f_LASUserEval)(NSErr_t *errp, char *attribute, CmpOp_t comparator,
                         char *pattern, ACLCachable_t *cachable,
                         void **las_cookie, PList_t subject, PList_t resource,
                         PList_t auth_info, PList_t global_auth);
    int (*f_LASProgramEval)(NSErr_t *errp, char *attribute, CmpOp_t comparator,
                            char *pattern, ACLCachable_t *cachable,
                            void **las_cookie, PList_t subject,
                            PList_t resource, PList_t auth_info,
                            PList_t global_auth);

    void (*f_LASTimeOfDayFlush)(void **cookie);
    void (*f_LASDayOfWeekFlush)(void **cookie);
    void (*f_LASIpFlush)(void **cookie);
    void (*f_LASDnsFlush)(void **cookie);
};

#ifdef INTONEACL
NSAPI_PUBLIC extern ACLDispatchVector_t *__oneacl_table;
#else
#ifdef XP_WIN32
__declspec(dllimport)
#endif /* XP_WIN32 */
extern ACLDispatchVector_t *__oneacl_table;
#endif /* INTONEACL */

#ifndef INTONEACL

#define nserrDispose (*__oneacl_table->f_nserrDispose)
#define nserrFAlloc (*__oneacl_table->f_nserrFAlloc)
#define nserrFFree (*__oneacl_table->f_nserrFFree)
#define nserrGenerate (*__oneacl_table->f_nserrGenerate)

    /* Property list support */

#define PListAssignValue (*__oneacl_table->f_PListAssignValue)
#define PListCreate (*__oneacl_table->f_PListCreate)
#define PListDefProp (*__oneacl_table->f_PListDefProp)
#define PListDeleteProp (*__oneacl_table->f_PListDeleteProp)
#define PListFindValue (*__oneacl_table->f_PListFindValue)
#define PListInitProp (*__oneacl_table->f_PListInitProp)
#define PListNew (*__oneacl_table->f_PListNew)
#define PListDestroy (*__oneacl_table->f_PListDestroy)
#define PListGetValue (*__oneacl_table->f_PListGetValue)
#define PListNameProp (*__oneacl_table->f_PListNameProp)
#define PListSetType (*__oneacl_table->f_PListSetType)
#define PListSetValue (*__oneacl_table->f_PListSetValue)
#define PListEnumerate (*__oneacl_table->f_PListEnumerate)
#define PListDuplicate (*__oneacl_table->f_PListDuplicate)
#define PListGetPool (*__oneacl_table->f_PListGetPool)

    /* ACL attribute handling */

#define ACL_LasRegister (*__oneacl_table->f_ACL_LasRegister)
#define ACL_LasFindEval (*__oneacl_table->f_ACL_LasFindEval)
#define ACL_LasFindFlush (*__oneacl_table->f_ACL_LasFindFlush)

    /* method/dbtype registration routines */

#define ACL_MethodRegister (*__oneacl_table->f_ACL_MethodRegister)
#define ACL_MethodIsEqual (*__oneacl_table->f_ACL_MethodIsEqual)
#define ACL_MethodNameIsEqual (*__oneacl_table->f_ACL_MethodNameIsEqual)
#define ACL_MethodFind (*__oneacl_table->f_ACL_MethodFind)
#define ACL_MethodGetDefault (*__oneacl_table->f_ACL_MethodGetDefault)
#define ACL_MethodSetDefault (*__oneacl_table->f_ACL_MethodSetDefault)
#define ACL_AuthInfoGetMethod (*__oneacl_table->f_ACL_AuthInfoGetMethod)
#define ACL_DbTypeRegister (*__oneacl_table->f_ACL_DbTypeRegister)
#define ACL_DbTypeIsEqual (*__oneacl_table->f_ACL_DbTypeIsEqual)
#define ACL_DbTypeNameIsEqual (*__oneacl_table->f_ACL_DbTypeNameIsEqual)
#define ACL_DbTypeFind (*__oneacl_table->f_ACL_DbTypeFind)
#define ACL_DbTypeGetDefault (*__oneacl_table->f_ACL_DbTypeGetDefault)
#define ACL_DbTypeSetDefault (*__oneacl_table->f_ACL_DbTypeSetDefault)
#define ACL_AuthInfoGetDbType (*__oneacl_table->f_ACL_AuthInfoGetDbType)
#define ACL_DbTypeIsRegistered (*__oneacl_table->f_ACL_DbTypeIsRegistered)
#define ACL_DbTypeParseFn (*__oneacl_table->f_ACL_DbTypeParseFn)
#define ACL_AttrGetterRegister (*__oneacl_table->f_ACL_AttrGetterRegister)
#define ACL_AttrGetterFind (*__oneacl_table->f_ACL_AttrGetterFind)
#define ACL_AttrGetterFirst (*__oneacl_table->f_ACL_AttrGetterFirst)
#define ACL_AttrGetterNext (*__oneacl_table->f_ACL_AttrGetterNext)
#define ACL_RegisterModule (*__oneacl_table->f_ACL_RegisterModule)
#define ACL_GetAttribute (*__oneacl_table->f_ACL_GetAttribute)
#define ACL_RegisterDbName (*__oneacl_table->f_ACL_RegisterDbName)
#define ACL_RegisterDbFromACL (*__oneacl_table->f_ACL_RegisterDbFromACL)
#define ACL_DatabaseFind (*__oneacl_table->f_ACL_DatabaseFind)
#define ACL_LDAPDatabaseHandle  (*__oneacl_table->f_ACL_LDAPDatabaseHandle)
#define ACL_AuthInfoGetDbname (*__oneacl_table->f_ACL_AuthInfoGetDbname)
#define ACL_CacheFlushRegister (*__oneacl_table->f_ACL_CacheFlushRegister)

    /*  ACL language and file interfaces */

#define ACL_ParseFile (*__oneacl_table->f_ACL_ParseFile)
#define ACL_ParseString (*__oneacl_table->f_ACL_ParseString)
#define ACL_Decompose (*__oneacl_table->f_ACL_Decompose)
#define ACL_WriteString (*__oneacl_table->f_ACL_WriteString)
#define ACL_WriteFile (*__oneacl_table->f_ACL_WriteFile)
#define ACL_FileRenameAcl (*__oneacl_table->f_ACL_FileRenameAcl)
#define ACL_FileDeleteAcl (*__oneacl_table->f_ACL_FileDeleteAcl)
#define ACL_FileGetAcl (*__oneacl_table->f_ACL_FileGetAcl)
#define ACL_FileSetAcl (*__oneacl_table->f_ACL_FileSetAcl)

    /*  ACL Expression construction interfaces  */

#define ACL_ExprNew (*__oneacl_table->f_ACL_ExprNew)
#define ACL_ExprDestroy (*__oneacl_table->f_ACL_ExprDestroy)
#define ACL_ExprSetPFlags (*__oneacl_table->f_ACL_ExprSetPFlags)
#define ACL_ExprClearPFlags (*__oneacl_table->f_ACL_ExprClearPFlags)
#define ACL_ExprTerm (*__oneacl_table->f_ACL_ExprTerm)
#define ACL_ExprNot (*__oneacl_table->f_ACL_ExprNot)
#define ACL_ExprAnd (*__oneacl_table->f_ACL_ExprAnd)
#define ACL_ExprOr (*__oneacl_table->f_ACL_ExprOr)
#define ACL_ExprAddAuthInfo (*__oneacl_table->f_ACL_ExprAddAuthInfo)
#define ACL_ExprAddArg (*__oneacl_table->f_ACL_ExprAddArg)
#define ACL_ExprSetDenyWith (*__oneacl_table->f_ACL_ExprSetDenyWith)
#define ACL_ExprGetDenyWith (*__oneacl_table->f_ACL_ExprGetDenyWith)

    /* ACL manipulation */

#define ACL_AclNew (*__oneacl_table->f_ACL_AclNew)
#define ACL_AclDestroy (*__oneacl_table->f_ACL_AclDestroy)
#define ACL_ExprAppend (*__oneacl_table->f_ACL_ExprAppend)

    /* ACL list manipulation */

#define ACL_ListNew (*__oneacl_table->f_ACL_ListNew)
#define ACL_ListConcat (*__oneacl_table->f_ACL_ListConcat)
#define ACL_ListAppend (*__oneacl_table->f_ACL_ListAppend)
#define ACL_ListDestroy (*__oneacl_table->f_ACL_ListDestroy)
#define ACL_ListFind (*__oneacl_table->f_ACL_ListFind)
#define ACL_ListAclDelete (*__oneacl_table->f_ACL_ListAclDelete)
#define ACL_ListGetNameList (*__oneacl_table->f_ACL_ListGetNameList)
#define ACL_NameListDestroy (*__oneacl_table->f_ACL_NameListDestroy)

    /* ACL evaluation */

#define ACL_EvalTestRights (*__oneacl_table->f_ACL_EvalTestRights)
#define ACL_EvalNew (*__oneacl_table->f_ACL_EvalNew)
#define ACL_EvalDestroy (*__oneacl_table->f_ACL_EvalDestroy)
#define ACL_EvalSetACL (*__oneacl_table->f_ACL_EvalSetACL)
#define ACL_EvalGetSubject (*__oneacl_table->f_ACL_EvalGetSubject)
#define ACL_EvalSetSubject (*__oneacl_table->f_ACL_EvalSetSubject)
#define ACL_EvalGetResource (*__oneacl_table->f_ACL_EvalGetResource)
#define ACL_EvalSetResource (*__oneacl_table->f_ACL_EvalSetResource)

    /* Built-in LAS functions */

#define LASTimeOfDayEval (*__oneacl_table->f_LASTimeOfDayEval)
#define LASDayOfWeekEval (*__oneacl_table->f_LASDayOfWeekEval)
#define LASIpEval (*__oneacl_table->f_LASIpEval)
#define LASDnsEval (*__oneacl_table->f_LASDnsEval)
#define LASGroupEval (*__oneacl_table->f_LASGroupEval)
#define LASUserEval (*__oneacl_table->f_LASUserEval)
#define LASProgramEval (*__oneacl_table->f_LASProgramEval)
#define LASTimeOfDayFlush (*__oneacl_table->f_LASTimeOfDayFlush)
#define LASDayOfWeekFlush (*__oneacl_table->f_LASDayOfWeekFlush)
#define LASIpFlush (*__oneacl_table->f_LASIpFlush)
#define LASDnsFlush (*__oneacl_table->f_LASDnsFlush)

#endif /* !INTONEACL */

#endif /* !PUBLIC_ONEACL_ACLAPI_H */
