/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * auth.c -- CGI authentication form generator -- HTTP gateway
 */

#include "dsgw.h"

static void post_request();
static void get_request();

main( int argc, char **argv )
{
    int	reqmethod;

    reqmethod = dsgw_init( argc, argv,  DSGW_METHOD_POST | DSGW_METHOD_GET );

    if ( reqmethod == DSGW_METHOD_POST ) {
	post_request();
    } else {
	get_request();
    }

    exit( 0 );
}

static void
get_request()
{
    char *binddn = NULL;

    dsgw_send_header();
    if (( binddn = getenv( "PATH_INFO" )) != NULL ) {
	binddn++;	/* skip '/' */
	if ( !strcmp( binddn, MGRDNSTR )) {
	    if ( gc->gc_rootdn == NULL ) {
		dsgw_error( DSGW_ERR_NO_MGRDN,
		    "In order to use this feature, there must be a dirmgr "
		    "defined in dsgw.conf", DSGW_ERROPT_EXIT, 0 );
	    }
	    binddn = dsgw_ch_strdup( gc->gc_rootdn );
	} else if ( *binddn == '\0' ) {
	    binddn = NULL;
	} else {
	    binddn = dsgw_ch_strdup( binddn );
	    dsgw_form_unescape( binddn );
	}
    }
    dsgw_emit_auth_form( binddn );
    if ( binddn != NULL ) {
	free( binddn );
    }
}



static void
post_request()
{
    char *binddn = NULL;
    char *dorootbind = NULL;

    dsgw_send_header();
    /*
     * If the "authasrootdn" CGI variable is present and has the value
     * "true" then the user clicked on the "authenticate as directory
     * manager" button.  In that case, try to bind as the root dn given
     * in the dsgw config file.
     */
    dorootbind = dsgw_get_cgi_var( "authasrootdn", DSGW_CGIVAR_OPTIONAL );
    if ( dorootbind != NULL && !strcasecmp( dorootbind, "true" )) {
	binddn = dsgw_ch_strdup( gc->gc_rootdn );
    } else {
	binddn = dsgw_get_cgi_var( "binddn", DSGW_CGIVAR_OPTIONAL );
    }
	
    dsgw_emit_auth_form( binddn );
}
