/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * browse.c -- CGI program to generate directory browser - HTTP gateway
 */

#include "dsgw.h"

static void initialize_html_strings();
static void get_request();
static void mk_searchbase_def( char *base, char *defbase );
static char *html1, *html2a, *html2b, *html2c, *html2d;
char * prepend_scheme( char *purl );

char *savecookie = "none";

main( argc, argv, env )
    int		argc;
    char	*argv[];
{
    int		reqmethod;

    initialize_html_strings();
    reqmethod = dsgw_init( argc, argv, DSGW_METHOD_GET );
    dsgw_send_header();

    if ( reqmethod == DSGW_METHOD_GET ) {
	get_request( gc );
    }

    exit( 0 );
}


static void
get_request( dsgwconfig *gc )
{
    char	*base = NULL;
    char	*p, *end;
    char	*cookie;
    LDAPURLDesc	*ludp, *gcludp;
    char	*piurl, *ckurl, *gcurl, *url = NULL;
    char	*ck_ldapserver = NULL;
    int		ck_ldapport = 0;
    char	*tmp;
    int		rc;
    int		badcookie = 0;
    char	hps[ 255 ], defhps[ 255 ], *cginame;

    /* Try cookie for a partial URL */
    if (( cookie = getenv( "HTTP_COOKIE")) != NULL ) {
	savecookie = dsgw_ch_strdup( cookie );
	dsgw_form_unescape( cookie );
	if (( p = strstr( cookie, DSGW_BROWSESBCKNAME)) != NULL ) {
	    p += strlen( DSGW_BROWSESBCKNAME );
	    if ( *p++ == '=' && *p != '\0' ) {
		end = strchr( p, ';' );
		if ( end == NULL ) {
		    end = p + strlen( p );
		}
		ckurl = dsgw_ch_malloc( end - p + 1 );
		strncpy( ckurl, p, end - p + 1 );
		ckurl[ end - p ] = '\0';
		url = prepend_scheme( ckurl );
		if (( rc = ldap_url_parse( url, &ludp )) != 0 ) {
		    url = NULL;
		    badcookie = 1;
		} else {
		    ck_ldapserver = ludp->lud_host;
		    ck_ldapport = ( ludp->lud_port  == 0 ) ? 389 :
			    ludp->lud_port;
		    base = ( ludp->lud_dn == NULL ) ? "" :
			    ludp->lud_dn;
		}
	    }
	}
    } else {
	base = gc->gc_ldapsearchbase;
    }

    puts( html1 );
    if (( cginame = getenv( "SCRIPT_NAME" )) == NULL ) {
	dsgw_error( DSGW_ERR_INTERNAL, "Cannot determine script name",
		DSGW_ERROPT_EXIT, 0 );
    }
    printf( "var cginame = \"%s\"\n", cginame );
    mk_searchbase_def( base, gc->gc_ldapsearchbase );
    /* Generate "host:port" string */
    sprintf( hps, "%s:%d",
	( ck_ldapserver != NULL ) ? ck_ldapserver : gc->gc_ldapserver,
	( ck_ldapport != 0 ) ? ck_ldapport : gc->gc_ldapport );
    /* Generate default "host:port" string */
    sprintf( defhps, "%s:%d", gc->gc_ldapserver, gc->gc_ldapport );
    printf( html2a, hps, defhps );
    printf( html2b, DSGW_BROWSESBCKNAME );
    printf( html2c );
    printf( html2d );
    fflush( stdout );
}



static void
mk_searchbase_def(
char *base,
char *defbase
)
{
    char **xdn;
    char **xdnt;
    int  i, j;
    int  ndnc;

    if ( base != NULL && strlen( base ) > 0 ) {
	xdn = ldap_explode_dn( base, 0 );
	xdnt = ldap_explode_dn( base, 1 );
    } else {
	xdn = xdnt = NULL;
    }

    printf( "var sb = new Object;\n" );
    printf( "var sbn = new Object;\n" );
    printf( "var defsb = new Object;\n" );
    printf( "var defsbn = new Object;\n" );

    if ( xdn == NULL ) {
	printf( "sb.length = 0;\n" );
	printf( "sbn.length = 0;\n" );
    } else {

	/* Count how many dn components there are */
	for ( ndnc = 1; xdn[ ndnc ] != NULL; ndnc++ );

	for ( i = 0, j = ndnc; i < ndnc; i++, j-- ) {
	    printf( "sbn[%d]='%s'\n",  j, xdnt[ i ]);
	}
	printf( "sbn.length = %d;\n", ndnc );

	for ( i = 0, j = ndnc; i < ndnc; i++, j-- ) {
	    printf( "sb[%d]='%s';\n", j, xdn[ i ]);
	}
	printf( "sb.length = %d;\n", ndnc );

	ldap_value_free( xdn );
	ldap_value_free( xdnt );
    }

    if ( defbase != NULL && strlen( defbase ) > 0 ) {
        xdn = ldap_explode_dn( defbase, 0 );
        xdnt = ldap_explode_dn( defbase, 1 );
    } else {
        xdn = xdnt = NULL;
    }

    if ( xdn == NULL ) {
	printf( "defsb.length = 0;\n" );
	printf( "defsbn.length = 0;\n" );
	return;
    }

    /* Count how many dn components there are */
    for ( ndnc = 1; xdn[ ndnc ] != NULL; ndnc++ );

    for ( i = 0, j = ndnc; i < ndnc; i++, j-- ) {
	printf( "defsbn[%d]='%s';\n",  j, xdnt[ i ]);
    }
    printf( "defsbn.length = %d;\n", ndnc );

    for ( i = 0, j = ndnc; i < ndnc; i++, j-- ) {
	printf( "defsb[%d]='%s';\n", j, xdn[ i ]);
    }
    printf( "defsb.length = %d;\n", ndnc );

    ldap_value_free( xdn );
    ldap_value_free( xdnt );
 
    return;
}


char *
prepend_scheme( char *purl )
{
    char *tmp;
    
    if ( strncasecmp( purl, "ldap://", 7 )) {
	tmp = dsgw_ch_malloc( strlen( purl ) + 7 );
	sprintf( tmp, "%s%s", "ldap://", purl );
    } else {
	tmp = dsgw_ch_strdup( purl );
    }
    return tmp;
}




static void
initialize_html_strings()
{
    html1 =
"<HTML>\n"
"<HEAD>\n"
"<TITLE>Directory Server Gateway: Browser</TITLE>\n"
"\n"
"<SCRIPT LANGUAGE=\"JavaScript\">\n"
"<!-- Hide from non-JavaScript browsers\n"
"\n";

    html2a =
"\n"
"var emptyFrame = '<HTML><BODY></BODY></HTML>';\n"
"var sbphtml = \"\";\n"
"var sbhtml = \"\";\n"
"var hostport = \"%s\";\n"
"var defhostport = \"%s\";\n"
"\n"
"function reppath(path, newstr)\n"
"{\n"
"    var tmpstr;\n"
"    var tpath;\n"
"    var ind;\n"
"    if ( path.substring( path.length - 1 ) == \"/\" ) {\n"
"        tpath = path.substring(1, path.length - 2 );\n"
"    } else {\n"
"        tpath = path;\n"
"    }\n"
"    ind = tpath.lastIndexOf( \"/\" );\n"
"    if ( ind < 0) {\n"
"        ind = 0;\n"
"    }\n"
"    tmpstr = tpath.substring(0, ind) + \"/\" + newstr;\n"
"    return tmpstr;\n"
"}\n"
"\n"
"\n"
"function cookieDate(exp)\n"
"{\n"
"// cookie format Wdy, DD-Mon-YY HH:MM:SS GMT like GMT but dashes dd-mm-yy year is 2-digit\n"
"    var ck;\n"
"    var mn;\n"
"    var dayn;\n"
"    var days;\n"
"    dayn = exp.getDay();\n"
"    if (dayn == 0) {\n"
"        days = \"Sunday\";\n"
"    } else if (dayn == 1) {\n"
"        days = \"Monday\";\n"
"    } else if (dayn == 2) {\n"
"        days = \"Tuesday\";\n"
"    } else if (dayn == 3) {\n"
"        days = \"Wednesday\";\n"
"    } else if (dayn == 4) {\n"
"        days = \"Thursday\";\n"
"    } else if (dayn == 5) {\n"
"        days = \"Friday\";\n"
"    } else if (dayn == 6) {\n"
"        days = \"Saturday\";\n"
"    }\n"
"    ck = days + \", \" + exp.getDate() + \"-\";\n"
"    mn = exp.getMonth();\n"
"    if (mn == 1) {\n"
"        ms = \"Jan\";\n"
"    } else if (mn == 2) {\n"
"        ms = \"Feb\";\n"
"    } else if (mn == 3) {\n"
"        ms = \"Mar\";\n"
"    } else if (mn == 4) {\n"
"        ms = \"Apr\";\n"
"    } else if (mn == 5) {\n"
"        ms = \"May\";\n"
"    } else if (mn == 6) {\n"
"        ms = \"Jun\";\n"
"    } else if (mn == 7) {\n"
"        ms = \"Jul\";\n"
"    } else if (mn == 8) {\n"
"        ms = \"Aug\";\n"
"    } else if (mn == 9) {\n"
"        ms = \"Sep\";\n"
"    } else if (mn == 10) {\n"
"        ms = \"Oct\";\n"
"    } else if (mn == 11) {\n"
"        ms = \"Nov\";\n"
"    } else if (mn == 12) {\n"
"        ms = \"Dec\";\n"
"    }\n"
"    yn = exp.getYear();\n"
"    if (yn < 10) {\n"
"        ys = \"0\" + yn;\n"
"    } else {\n"
"        ys = yn;\n"
"    }\n"
"    hn = exp.getHours();\n"
"    if (hn < 10) {\n"
"        hs = \"0\" + hn;\n"
"    } else {\n"
"        hs = hn;\n"
"    }\n";

	html2b = 
"    minn = exp.getMinutes();\n"
"    if (minn < 10) {\n"
"        mins = \"0\" + minn;\n"
"    } else {\n"
"        mins = minn;\n"
"    }\n"
"    secn = exp.getSeconds();\n"
"    if (secn < 10) {\n"
"        secs = \"0\" + secn;\n"
"    } else {\n"
"        secs = secn;\n"
"    }\n"
"    ck += ms + \"-\" + ys + \" \" + hs + \":\" +\n"
"            mins + \":\" + secs + \" GMT\";\n"
"    return ck;\n"
"}\n"
"\n"
"\n"
"function mkcookie(expire)\n"
"{\n"
"    msPerDay = 24 * 60 * 60 * 1000;\n"
"    expdate = new Date();\n"
"    if ( expire == true ) {\n"
"        expms = expdate.getTime() - msPerDay;\n"
"    } else {\n"
"        expms = expdate.getTime() + 365 * msPerDay;\n"
"    }\n"
"    expdate.setTime(expms);\n"
"    document.cookie = \"%s=\" + hostport + \"/\" + escape(sbarr2sb()) +\n"
"            \"; expires=\" + parent.cookieDate(expdate);\n"
"}\n"
"\n"
"\n"
"function jumpSB(index)\n"
"{\n"
"    parent.sb.length = parent.sbn.length = index;\n"
"    parent.genSBPopupFrame();\n"
"    parent.genOutputFrame();\n"
"}\n"
"\n"
"\n"
"function addNewRDN(newrdn)\n"
"{\n"
"    var olen = parent.sb.length;\n"
"    newrdn = unescape(newrdn);\n"
"    parent.sb[olen + 1] = newrdn;\n"
"    parent.sb.length = olen + 1;\n"
"    parent.sbn[olen + 1] = newrdn.substring(newrdn.indexOf(\"=\") + 1, newrdn.length);\n"
"    parent.sbn.length = parent.sb.length;\n"
"    jumpSB(parent.sb.length);\n"
"}\n"
"\n"
"function goDefSB()\n"
"{\n"
"    for (var i = 1; i <= parent.defsb.length; i++) {\n"
"        parent.sb[i] = parent.defsb[i];\n"
"    }\n"
"    parent.sb.length = parent.defsb.length;\n"
"    for (var i = 1; i <= parent.defsbn.length; i++) {\n"
"        parent.sbn[i] = parent.defsbn[i];\n"
"    }\n"
"    parent.sbn.length = parent.defsbn.length;\n"
"    hostport = defhostport;\n"
"    parent.mkcookie(true);\n"
"    jumpSB(parent.sb.length);\n"
"}\n"
"\n"
"\n"
"\n"
"\n"
"function sb2sbn()\n"
"{\n"
"    parent.sbn.length = parent.sb.length;\n"
"    for (i = 1; i <= parent.sbn.length; i++) {\n"
"	ind = sb[i].indexof(\"=\")\n"
"	sbn[i] = sb[i].substring(ind, sb[i].length);\n"
"    }\n"
"}\n"
"\n"
"\n";

	html2c = 
"function sbarr2sb()\n"
"{\n"
"    var sbt = \"\";\n"
"\n"
"    if (parent.sb.length > 0) {\n"
"	for (var i = parent.sb.length; i > 1; i--) {\n"
"	    sbt += parent.sb[i];\n"
"	    sbt += \", \";\n"
"	}\n"
"	sbt += parent.sb[i];\n"
"    }\n"
"    return sbt;\n"
"}\n"
"\n"
"\n"
"\n"
"function genOutputFrame()\n"
"{\n"
"    var newloc;\n"
"    var newpathinfo;\n"
"    var tmp = reppath(parent.cginame, \"browsesrch\") + \"/\" +\n"
"	    hostport + \"/\" + sbarr2sb();\n"
"    parent.mkcookie(false);\n"
"    newloc = parent.location.protocol + '//' + parent.location.hostname;\n"
"    if (parent.location.port.length > 0) {\n"
"         newloc += ':' + parent.location.port\n"
"    }\n"
"    newloc += escape(tmp);\n"
"    parent.outputFrame.location.href = newloc;\n"
"}\n"
"\n"
"\n"
"function genSBPopup()\n"
"{\n"
"    var ret = \"\";\n"
"\n"
"    ret += '<option>The World\\n';\n"
"    if ( parent.sbn.length == 0) {\n"
"	return ret;\n"
"    }\n"
"    for (i = 1; i < parent.sbn.length; i++) {\n"
"	ret += '<option>' + parent.sbn[i] + '\\n';\n"
"    }\n"
"    ret += '<option selected>' + parent.sbn[i] + '\\n';\n"
"    return ret;\n"
"}\n"
"\n"
"\n"
"function genSBPopupFrame()\n"
"{\n"
"    sbphtml = \n"
"    '<html>\\n' +\n"
"    '<body>\\n' +\n"
"    '<form name=\"SBPopupForm\">\\n' +\n"
"    '<table>\\n' +\n"
"    '<tr>' +\n"
"    '<th align=left>\\n' +\n"
"    '<input type=button value=\"Move\" onClick=\"parent.jumpSB(document.SBPopupForm.SBPopup.selectedIndex)\"> to: ' +\n"
"    '</th>' +\n"
"    '<td>' +\n"
"    '<select name=SBPopup>' +\n"
"\n"
"    genSBPopup() +\n"
"\n";

	html2d = 
"    '</select>\\n' +\n"
"    '<input type=button onClick=\"parent.goDefSB();\" value=\"Jump to default searchbase\">\\n' +\n"
/* "    '<a href=\"parent.goDefSB()\">Jump to default searchbase</a>\\n' +\n" */
"    '</td>' +\n"
"    '</table>' +\n"
"    '</form>\\n' +\n"
"    '</body>\\n' +\n"
"    '</html>\\n';\n"
"    parent.SBPopupFrame.location = \"javascript:parent.sbphtml\";\n"
"}\n"
"\n"
"\n"
"function init()\n"
"{\n"
"    parent.genSBPopupFrame();\n"
"    parent.genOutputFrame();\n"
"}\n"
"\n"
"// end hiding -->\n"
"</SCRIPT>\n"
"</HEAD>\n"
"\n"
"\n"
"\n"
"\n"
"<FRAMESET ROWS=\"70,70,*\" onLoad=\"init()\">\n"
"  <FRAME SRC=\"/dshtml/buttonbar.html\" NORESIZE SCROLLING=\"no\">\n"
"  </FRAME>\n"
"  <FRAME NAME=\"SBPopupFrame\" SRC=\"javascript:parent.emptyFrame\"\n"
"	NORESIZE SCROLLING=\"no\">\n"
"  </FRAME>\n"
"  <FRAME NAME=\"outputFrame\" SRC=\"javascript:parent.emptyFrame\">\n"
"</FRAMESET>\n"
"\n"
"</HTML>\n";

}
