/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * browsesrch.c -- CGI program which reads directory, produces broswable
 * listing of directory contents.
 */

#include "dsgw.h"

static void get_request();

main( argc, argv, env )
    int		argc;
    char	*argv[];
#ifdef DSGW_DEBUG
    char	*env[];
#endif
{
    int		reqmethod;

    reqmethod = dsgw_init( argc, argv, DSGW_METHOD_GET );
    dsgw_send_header();

#ifdef DSGW_DEBUG
   dsgw_logstringarray( "env", env ); 
{
    char	buf[ 1024 ];
    getcwd( buf, sizeof(buf));
    dsgw_log( "cwd: \"%s\"\n", buf );
}
#endif

    if ( reqmethod == DSGW_METHOD_GET ) {
	get_request();
    }

    exit( 0 );
}


static void
get_request()
{
    LDAP	*ld;
    char	*ldapserver;
    char	*ldapportstr;
    int		ldapport;
    LDAPMessage	*msgp, *entry;
    int		msgid, rc, entries = 0;
    char	*sattrs[] = { "objectclass", "hassubordinates", NULL };
    char	*dn, *edn, **xdn, **xdnt;
    char	buf[ 1024 ]; /* XXX malloc this */
    char	portstr[ 10 ];
    int		ec;
    LDAPURLDesc	*ludp;
    char	*purl, *url, *pi, *rdn, *rdnt;
    char	**hsv;
    int		hassub;

    if (( pi = getenv( "PATH_INFO" )) == NULL ) {
	dsgw_error( DSGW_ERR_LDAPURL_NOTLDAP, pi, DSGW_ERROPT_EXIT, 0 );
    } else if ( pi[ 0 ] == '/' ) {
	pi++;
    }
    purl = strdup( pi );
    
    dsgw_form_unescape( purl );
    url = dsgw_ch_malloc( strlen( purl ) + 7 );
    sprintf( url, "ldap://%s", purl );
    if (( rc = ldap_url_parse( url, &ludp )) != 0 ) {
	switch ( rc ) {
        case LDAP_URL_ERR_NODN:
            ec = DSGW_ERR_LDAPURL_NODN;
            break;
        case LDAP_URL_ERR_BADSCOPE:
            ec = DSGW_ERR_LDAPURL_BADSCOPE;
            break;
        case LDAP_URL_ERR_MEM:
            ec = DSGW_ERR_NOMEMORY;
            break;
        case LDAP_URL_ERR_NOTLDAP:
        default:
            ec = DSGW_ERR_LDAPURL_NOTLDAP;
            break;
        }
        dsgw_error( ec, url, DSGW_ERROPT_EXIT, 0 );
    }

    printf( "<html><body>\n" );
    printf( "<script language=JavaScript>\n" );
    printf( "function dst()\n" );
    printf( "{\n" );
    printf( "    window.status=\'Click to change searchbase.\'; " );
    printf( "    return true;\n" );
    printf( "}\n" );
    printf( "function dot()\n" );
    printf( "{\n" );
    printf( "    window.status=\'Click to view entry.\'; " );
    printf( "    return true;\n" );
    printf( "}\n" );
    printf( "</script>\n" );
    printf( "<center><table border=1><tr><td>Listing of <b>%s</b></td></tr>",
	    ( ludp->lud_dn == NULL || strlen( ludp->lud_dn ) == 0 ) ?
	    "The World" : ludp->lud_dn );
    printf( "</table></center>\n" );

    sprintf( portstr, "%d", ludp->lud_port );

    if (( ld = ldap_init( ludp->lud_host, ludp->lud_port )) == NULL ) {
	dsgw_error( DSGW_ERR_LDAPINIT, NULL, DSGW_ERROPT_EXIT, 0 );
    }
    if ( ldap_simple_bind_s( ld, NULL, NULL ) != LDAP_SUCCESS ) {
	dsgw_ldap_error( ld, DSGW_ERROPT_EXIT );
    }

    rc = ldap_search_s( ld, ludp->lud_dn, LDAP_SCOPE_ONELEVEL,
	    "objectclass=*", sattrs, 0, &msgp );
    if ( rc != LDAP_SUCCESS ) {
	puts( dsgw_ldaperr2string( rc ));
    }

    entries = ldap_count_entries( ld, msgp );
    if ( entries < 1 ) {
	printf( "<center><b>No entries found at this level</b></center><p>\n" );
	printf( "</body></html>\n" );
	fflush( stdout );
	return;
    }

    if ( ldap_sort_entries( ld, &msgp, NULL, strcasecmp ) != 0 ) {
	printf( "<center><b>Error while sorting entries: %s</b></center><p>\n",
		ldap_err2string( ld->ld_errno ));
	printf( "</body></html>\n" );
	fflush( stdout );
	return;
    }

    printf( "<table>\n" );
    for ( entry = ldap_first_entry( ld, msgp ); entry != NULL;
	    entry = ldap_next_entry( ld, entry )) {
	dn = ldap_get_dn( ld, entry );
	xdn = ldap_explode_dn( dn, 1 );
	xdnt = ldap_explode_dn( dn, 0 );
	hsv = ldap_get_values( ld, msgp, "hassubordinates" );
	if ( ldap_count_values( hsv ) == 0 ) {
	    /* attribute was not present, so who knows? */
	    hassub = 1;
	} else {
	    if ( !strcmp( hsv[ 0 ], "TRUE" )) {
		hassub = 1;
	    } else {
		hassub = 0;
	    }
	}
	ldap_value_free( hsv );
	printf( "<tr><td>\n" );
	edn = dsgw_escape_quotes( dn );
	rdn = dsgw_escape_quotes( xdn[ 0 ]);
	rdnt = dsgw_ch_malloc( strlen( xdnt[ 0 ]) * 3 );
	rdnt[0] = '\0';
	dsgw_strcat_escaped( rdnt, xdnt[ 0 ]);

	if ( hassub ) {
	    printf( "<a href=javascript:parent.addNewRDN('%s'); ", rdnt );
	    printf( "onMouseOver=\"return dst();\">\n" );
	    printf( "<img src=\"/dshtml/triangle.gif\" width=10 height=10 " );
	    printf( "border=0 align=center></a>\n</td>\n" );
	} else {
	    printf( "&nbsp" );
	}

	printf( "<td>\n" );
	sprintf( buf, "%s%s%s/%s", ludp->lud_host,
		( ludp->lud_port == 0 ) ? "" : ":",
		( ludp->lud_port == 0 ) ? "" : portstr,
		dn );
	dsgw_html_href( DSGW_URLPREFIX_DOSEARCH, buf, xdn[ 0 ], NULL, 
		"onMouseOver=\"return dot();\"" );

	printf( "</td></tr>\n" );


	ldap_value_free( xdn );
	ldap_value_free( xdnt );
	free( dn );
	free( rdn );
	free( rdnt );
    }
    printf( "</table>\n" );

    ldap_msgfree( msgp );

    printf( "</body></html>\n" );
    fflush( stdout );
}
