/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * calcount.c - for CAL Accounting of Suitespot Servers
 */


#include <stdio.h>
#include <string.h>
#if !defined( XP_WIN32 )
#include <sys/param.h>
#endif

#include "dsgw.h"
#include "dbtdsgw.h"
#include "../../include/portable.h"
#ifdef	AIX
#undef	HAVE_TIME_R
#endif
#include "ldaputil/ldaputil.h"
#include "ldaputil/dbconf.h"

#define MAXARGS 100
#define MAX_SERVER_COUNT 50
#define MY_PAGE "calcount.html"


static void error(char *msg);
static void fp_getline_init(int *);
static char * fp_getline( FILE *, int * );
static void fp_parse_line( char	*, int *, char **);

static char *
strtok_quote( char *line, char *sep )
{
    int         inquote;
    char        *tmp;
    static char     *next;

    if ( line != NULL ) {
        next = line;
    }
    while ( *next && strchr( sep, *next ) ) {
        next++;
    }

    if ( *next == '\0' ) {
        next = NULL;
        return( NULL );
    }
    tmp = next;

    for ( inquote = 0; *next; ) {
        switch ( *next ) {
        case '"':
            if ( inquote ) {
                inquote = 0;
            } else {
                inquote = 1;
            }
            strcpy( next, next + 1 );
            break;

#ifndef _WIN32
        case '\\':
            strcpy( next, next + 1 );
            break;
#endif

        default:
            if ( ! inquote ) {
                if ( strchr( sep, *next ) != NULL ) {
                    *next++ = '\0';
                    return( tmp );
                }
            }
            next++;
            break;
        }
    }
    return( tmp );
}


/*
 * Read the gateway config file (dsgw.conf) for CAL info.
 */
static int
read_dsgwconf( char *filename, char **sName, char **sID, int *servCount )
{
    int		cargc, flag;
    char	*cargv[ MAXARGS ];
    FILE	*fp;
    char	*line;
    int		lineno;

    if (( fp = fopen( filename, "r" )) == NULL ) {
	dsgw_error( DSGW_ERR_BADCONFIG, NULL,
				DSGW_ERROPT_EXIT, 0 );
    }
    fp_getline_init( &lineno );

    while ( (line = fp_getline( fp, &lineno )) != NULL ) {
	/* skip comments and blank lines */
	if ( line[0] == '#' || line[0] == '\0' ) {
	    continue;
	}

	fp_parse_line( line, &cargc, cargv );

	if ( cargc < 1 ) {
	    continue;
	}

	if ( strcasecmp( cargv[0], "attrvset" ) == 0 ) {
	    if ( cargc != 5 ) {
		dsgw_error( DSGW_ERR_BADCONFIG, NULL,
				DSGW_ERROPT_EXIT, 0 );
	    }
/* Add the server name to your list of servers
	    add_avset( &gc->gc_avsets, &cargv[1] );
*/
	    sID[*servCount]=STRDUP(cargv[2]);
	    sName[*servCount]=STRDUP(cargv[4]);
	    if(++(*servCount) > MAX_SERVER_COUNT)
		return 0;

	/* include another config file */
	} else if ( strcasecmp( cargv[0], "include" ) == 0 ) {
	    char	*tmpfname;

	    if ( cargc < 2 ) {
		dsgw_error( DSGW_ERR_BADCONFIG, NULL,
				DSGW_ERROPT_EXIT, 0 );
	    }
	    tmpfname = dsgw_ch_strdup( cargv[1] );
    	    flag = read_dsgwconf(tmpfname, sName, sID, servCount);
	    free( tmpfname );
	    if (!flag) 
		return 0;
	} 
    }
    return 1;
}

static char	buf[BUFSIZ];
static char	*line;
static int	lmax, lcur;

#define CATLINE( buf )	{ \
	int	len; \
	len = strlen( buf ); \
	while ( lcur + len + 1 > lmax ) { \
		lmax += BUFSIZ; \
		line = (char *) dsgw_ch_realloc( line, lmax ); \
	} \
	strcpy( line + lcur, buf ); \
	lcur += len; \
}



static void
fp_parse_line(
    char	*line,
    int		*argcp,
    char	**argv
)
{
	char *	token, buf[ 20 ];

	*argcp = 0;
	for ( token = strtok_quote( line, " \t" ); token != NULL;
	    token = strtok_quote( NULL, " \t" ) ) {
		if ( *argcp == MAXARGS ) {
			sprintf( buf, XP_GetClientStr(DBT_maxD_), MAXARGS );
			dsgw_error( DSGW_ERR_CONFIGTOOMANYARGS, buf,
				DSGW_ERROPT_EXIT, 0 );
		}
		argv[(*argcp)++] = token;
	}
	argv[*argcp] = NULL;
}



static char *
fp_getline( FILE *fp, int *lineno )
{
	char		*p;

	lcur = 0;

	while ( fgets( buf, sizeof(buf), fp ) != NULL ) {
		if ( (p = strchr( buf, '\n' )) != NULL ) {
			*p = '\0';
		}
		if ( lcur > 0 && ! XP_IS_SPACE( buf[0] ) ) {
			return( line );	/* return previously saved line */
		}
		CATLINE( buf );
		(*lineno)++;
		if ( ! XP_IS_SPACE( buf[0] )) {
			return( line );	/* return this line */
		}
	}
	buf[0] = '\0';

	return( lcur > 0 ? line : NULL );
}

static void
fp_getline_init( int *lineno )
{
	*lineno = 0;
	buf[0] = '\0';
}


#ifdef XP_WIN32
/* convert forward slashes to backwards ones */
static void
dsgw_unix2dospath( char *path )
{
    if( path ) {
	while( *path ) {
	    if( *path == '/' ) {
		 *path = '\\';
	    }
	    path++;
	}
    }
}
#endif

int
getServerInfo(char **sName, char **sID, int *sCount)
{
    char line[BIG_LINE];
    char *tmp = getenv("NETSITE_ROOT");
    
    sprintf(line, "%s%cadmin-serv%cconfig%cdsgw.conf", tmp, FILE_PATHSEP, FILE_PATHSEP, FILE_PATHSEP);

    return read_dsgwconf(line, sName, sID, sCount);
}

int getCALcount(LDAP *ld, char *serv) 
{
    int 	authmode;
    char 	*attrlist[2] = {"nsLicensedFor", ""};
    char	*base, *filter;
    char 	*constFilterStr="(&(objectClass=nsLicenseUser)(nsLicensedFor=";
    LDAPMessage *msgp;
    int		scope;
   
/* Initializing with auth disabled 
*/
    authmode = 0;
    scope = 2;  /* subtree lying under base DN */
    base = gc->gc_ldapsearchbase;
    attrlist[1] = NULL;
      
    filter = (char *) malloc(strlen(constFilterStr)+strlen(serv) + 2);
    strcpy(filter, constFilterStr);
    strcat(filter, serv);
    strcat(filter, "))");

    if (ldap_search_s( ld, base, scope, filter, attrlist, 0, &msgp )
		== LDAP_SUCCESS) {
	free(filter);
	return ((msgp == NULL) ? 0 : ldap_count_entries( ld, msgp ));
    }
    else {
	free(filter);
	return -1;
    }
}



int
main(int argc, char *argv[]) 
{
    int flag, isPOST=0, servCount=0;
    int i, checkBox[MAX_SERVER_COUNT];
    int result=0; 
    int reqmethod, sizelimit;
    LDAPFiltDesc *lfdp;    
    LDAP *ld;
    char *servName[MAX_SERVER_COUNT];
    char *servID[MAX_SERVER_COUNT];
    char line[BIG_LINE];
    char **config = get_adm_config();
    FILE *html = open_html_file(MY_PAGE);

    reqmethod = dsgw_init( argc, argv,  DSGW_METHOD_POST | DSGW_METHOD_GET );
    dsgw_send_header();
 
/*    m = getenv("REQUEST_METHOD");
*/
    if (reqmethod == DSGW_METHOD_POST) 
	isPOST = 1;
    else 
	isPOST = 0;
	

        (void) dsgw_init_ldap( &ld, &lfdp, 0);
	sizelimit = 0;
	(void) ldap_set_option( ld, LDAP_OPT_SIZELIMIT, &sizelimit );

        flag = getServerInfo(servName, servID, &servCount);
	if (!flag) 
	    dsgw_error( DSGW_ERR_INTERNAL, NULL,
				DSGW_ERROPT_EXIT, 0 );
	if (!servCount) {
	    dsgw_error( DSGW_ERR_NOCALCONFIG, NULL,
				DSGW_ERROPT_EXIT, 0 );  
	}
        while(next_html_line(html, line))  {
            if(parse_line(line, config))  {
                if(directive_is(line, "GETSERVERLIST"))  {
		    fprintf(stdout,"    if (");
		    for(i=0; i < servCount; i++) {
 			fprintf(stdout,"(fm.%s.checked)",servID[i]);
			if (i < servCount-1)
			    fprintf(stdout, "||");
			else
			    fprintf(stdout, ")\n");
		     	checkBox[i]=dsgw_get_cgi_var(servID[i], DSGW_CGIVAR_OPTIONAL ) ? 1 : 0;
		    }
                }
	        else if(directive_is(line, "SERVER CHECKBOXES"))  {
		    fprintf(stdout, "<TABLE BORDER=0 WIDTH=70%%>");
	  	    fprintf(stdout, "<TR><TD WIDTH=10%% ALIGN=\"RIGHT\">"   "</TD>");
	  	    fprintf(stdout, "<TD WIDTH=70%% ALIGN=\"CENTER\"><B>Server</B></TD>");
	  	    fprintf(stdout, "<TD WIDTH=20%% ALIGN=\"CENTER\"><B>License Count</B></TD></TR>");
		    for(i=0; i < servCount; i++) {
		    	fprintf(stdout, "<TR><TD WIDTH=10%% ALIGN=\"RIGHT\">");
                        output_input("checkbox", servID[i], "1",
                                 checkBox[i] ? "checked" : NULL);
		    	fprintf(stdout, "</TD>");
		    	fprintf(stdout, "<TD WIDTH=70%% ALIGN=\"CENTER\">%s</TD>",servName[i]);
			if (checkBox[i]) {
	    	    	    result = getCALcount(ld, servID[i]);
		    	    if (result < 0) 
				dsgw_error( DSGW_ERR_LDAPGENERAL, NULL,
					DSGW_ERROPT_EXIT, 0 );
	    	    	    fprintf(stdout, "<TD WIDTH=20%% ALIGN=\"CENTER\">%d</TD></TR>", result);
			} 
			else 
	    	    	    fprintf(stdout, "<TD WIDTH=20%% ALIGN=\"CENTER\">%s</TD></TR>", "-");
		    }
	    	    fprintf(stdout, "</TABLE>");
                }
                else fputs(line, stdout);
	    }
	}
}
