/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/


#include <stdio.h>
#include <string.h>
#if !defined( XP_WIN32 )
#include <sys/param.h>
#endif

#include "dsgw.h"
#include "dbtdsgw.h"
#include "../../include/portable.h"
/* MLM - Include netsite.h to get ADMSERV_VERSION_STRING */
#ifdef	AIX
#undef	HAVE_TIME_R
#endif
#include "netsite.h"
#include "ldaputil/errors.h"
#include "ldaputil/ldaputil.h"
#include "ldaputil/dbconf.h"

extern char *get_userdb_dir(void);	/* Can't include libadmin.h, so this */
static int list_contains_str( char *list, char *target );
static void report_ldapu_error( int ldapu_err, int dsgw_err, int erropts );
static void adderr( dsgwconfig *gc, char *str, char *filename, int lineno );
static void fp_parse_line( char	*line, int *argcp, char	**argv );
static void fp_getline_init( int *lineno );
static char *fp_getline( FILE *fp, int *lineno );
static void fp_pushbackline( FILE *fp, int *lineno, char *str );
static void add_location( int *loccountp, dsgwloc **locarrayp,
	char *locsuffix, char **argv );
static int add_newtype( dsgwnewtype **newentlistp, int loccount,
	dsgwloc *locarray, int argc, char **argv );
static void add_tmplset( dsgwtmplset **tslp, int argc, char **argv );
static void add_langset( dsgwlang **lngp, int argc, char **argv, int selected );
static void add_avset( dsgwavset **avsp, char **argv );
static void add_includeset( dsgwinclset **isp, char **argv );
static void read_dsgwconfig( char *filename, int templatesonly );
static void get_dbconf_properties( char *filename );
static int write_dbswitch_info( FILE *fp, dsgwconfig *cfgp, char *dbhandle );
static int ldapdb_url_parse( char *url, LDAPDBURLDesc **ldbudpp );
#ifdef XP_WIN32
static void dsgw_unix2dospath( char *path );
#endif

#define MOZILLA_SIG     "Mozilla/" 
#define	MAXARGS	100
/*
 * Open and parse the dsgw config file.  If an error occurs, this function
 * does not return.
 */
dsgwconfig *
dsgw_read_config()
{
    char	*scriptname;
    char	*ss, *vs, *p, *fname;

    /* gc is a global */
    if (( gc = (dsgwconfig *) dsgw_ch_malloc( sizeof( dsgwconfig ))) == NULL ) {
	dsgw_error( DSGW_ERR_NOMEMORY, XP_GetClientStr(DBT_initializingConfigInfo_),
		DSGW_ERROPT_EXIT, 0 );
    }
    memset( gc, 0, sizeof( dsgwconfig ));

    /*
     * set non-zero configuration defaults
     */
    gc->gc_ldapport = LDAP_PORT;
    gc->gc_configerrstr = dsgw_ch_strdup( "" );
    gc->gc_sslrequired = DSGW_SSLREQ_NEVER;
    gc->gc_authlifetime = DSGW_DEF_AUTH_LIFETIME;
    gc->gc_configdir = DSGW_CONFIGDIR_HTTP;	/* may be overridden below */
    gc->gc_tmpldir = DSGW_TMPLDIR_HTTP;		/* may be overridden below */
    gc->gc_urlpfxmain = DSGW_URLPREFIX_MAIN_HTTP; /* may be overridden below */
    gc->gc_urlpfxcgi = DSGW_URLPREFIX_CGI_HTTP; /* may be overridden below */
    gc->gc_binddn = gc->gc_bindpw = "";
    gc->gc_charset = NULL;
    gc->gc_nbspchar = DSGW_LAT1_NBSP;	/* XXX should be a config file item */
    gc->gc_ClientLanguage = "";
    gc->gc_AdminLanguage = "";
    gc->gc_DefaultLanguage = "";
    gc->gc_locationurl = "ldap:///?o,ou?sub?(|(objectclass=organization)"
	    "(objectclass=organizationalunit))";
    /*
     * Figure out whether we are running under the admin server or not.  This
     * also determines where our config and html files are.  The hackage is:
     * if we're running under the admin server:
     *   configdir is  ../../../../admin-serv/config
     *   htmldir is ../html
     *   urlpfxmain is ""
     *   urlpfxcgi is ""
     *   dbswitchfile is NSHOME/userdb/dbswitch.conf
     * 
     * If we're running under any other HTTP server:
     *   configdir is ../config
     *   htmldir is ../config  (yes, that's right)
     *   urlpfxmain is "/dshtml/"
     *   gc_urlpfxcgi is "/ds"
     *   dbswitchfile is not used
     */

    /* Get the admin server name and chop off the version number */
    vs = dsgw_ch_strdup( ADMSERV_VERSION_STRING );
    if (( p = strchr( vs, '/')) != NULL ) {
	*p = '\0';
    }

    ss = getenv( "SERVER_SOFTWARE" );
    if ( ss != NULL ) {
	if ( !strncasecmp( vs, ss, strlen( vs ))) {
	    char *server_names;
	    /* We're running under the admin server */
	    gc->gc_admserv = 1;
	    gc->gc_configdir = DSGW_CONFIGDIR_ADMSERV;
	    gc->gc_tmpldir = DSGW_TMPLDIR_ADMSERV;
	    gc->gc_urlpfxmain = DSGW_URLPREFIX_MAIN_ADMSERV;
	    gc->gc_urlpfxcgi = DSGW_URLPREFIX_CGI_ADMSERV;
	    /* Check if running an end-user CGI under the admin server */
	    if (( server_names = getenv( "SERVER_NAMES" )) != NULL &&
		    strlen( server_names ) >= 4 &&
		    strncmp( server_names, "user", 4 ) == 0 ) {
		gc->gc_enduser = 1;
	    }
	}
    }


    /* Set mode (based on which CGI is currently running) */
    if (( scriptname = getenv( "SCRIPT_NAME" )) == NULL ) {
	gc->gc_mode = 0;
    } else {
	if ( !strncmp( scriptname, dsgw_getvp( DSGW_CGINUM_DOSEARCH ),
		strlen( scriptname ))) {
	    gc->gc_mode = DSGW_MODE_DOSEARCH;
	} else if ( !strncmp( scriptname, dsgw_getvp( DSGW_CGINUM_BROWSE ),
		strlen( scriptname ))) {
	    gc->gc_mode = DSGW_MODE_BROWSE;
	} else if ( !strncmp( scriptname, dsgw_getvp( DSGW_CGINUM_SEARCH ),
		strlen( scriptname ))) {
	    gc->gc_mode = DSGW_MODE_SEARCH;
	} else if ( !strncmp( scriptname, dsgw_getvp( DSGW_CGINUM_CSEARCH ),
		strlen( scriptname ))) {
	    gc->gc_mode = DSGW_MODE_CSEARCH;
	} else if ( !strncmp( scriptname, dsgw_getvp( DSGW_CGINUM_AUTH ),
		strlen( scriptname ))) {
	    gc->gc_mode = DSGW_MODE_AUTH;
	} else if ( !strncmp( scriptname, dsgw_getvp( DSGW_CGINUM_EDIT ),
		strlen( scriptname ))) {
	    gc->gc_mode = DSGW_MODE_EDIT;
	} else if ( !strncmp( scriptname, dsgw_getvp( DSGW_CGINUM_DOMODIFY ),
		strlen( scriptname ))) {
	    gc->gc_mode = DSGW_MODE_DOMODIFY;
	} else {
	    gc->gc_mode = DSGW_MODE_UNKNOWN;
	}
    }


    /* Set rest of config. by reading the appropriate config files */
    fname = dsgw_ch_malloc( MAXPATHLEN );
    if ( gc->gc_admserv ) {
	sprintf( fname, "%s/dbswitch.conf", get_userdb_dir());
	get_dbconf_properties( fname );
    }
    sprintf( fname, "%s%s", gc->gc_configdir, DSGW_CONFIGFILE );

    read_dsgwconfig( fname, gc->gc_admserv );
    free( fname );

    /* if necessary, try to set path to certificate database */
#ifndef DSGW_NO_SSL
    if ( gc->gc_ldapssl && gc->gc_securitypath == NULL ) {
	if ( gc->gc_admserv ) {
/* atulb: Changed "CertFile" to "CertDBFile" since we use adm.conf instead of 
	  ns-admin.conf for AS40 */
	    if (( p = get_nsadm_var( "CertDBFile" )) != NULL ) { 
		int	len;

		while (*p == ' ')
			p++;

		if (( len = strlen( p )) < 4 ||
			( strcasecmp( p + len - 3, ".db" )) != 0 ) {
		    /* append missing ".db" */
		    gc->gc_securitypath = dsgw_ch_malloc( strlen( p ) + 4 );
		    sprintf( gc->gc_securitypath, "%s.db", p );
		} else {
		    gc->gc_securitypath = dsgw_ch_strdup(p);
		}
	    }
	} else {
	    gc->gc_securitypath = DSGW_DEFSECURITYPATH;
	}
    }
#endif


    if (( p = getenv( "DefaultLanguage" )) != NULL ) {
	gc->gc_DefaultLanguage = p;
    }

    if (( p = getenv( "AdminLanguage" )) != NULL ) {
	gc->gc_AdminLanguage = p;
    }

    if (( p = getenv( "ClientLanguage" )) != NULL ) {
	gc->gc_ClientLanguage = p;
    }

    if (( p = getenv( "HTTP_ACCEPT_CHARSET" )) != NULL &&
	    ( list_contains_str( p, DSGW_CHARSET_UTF8 ) ||
	    list_contains_str( p, DSGW_CHARSET_UTF8ALT ))) {
	/*
	 * browser understands UTF-8:
	 *	recognize Accept-Language header sent by browser
	 *	use UTF-8 for all forms (unless config file said otherwise)
	 */
	if (( p = getenv( "HTTP_ACCEPT_LANGUAGE" )) != NULL ) {
	    gc->gc_ClientLanguage = p;
	}
	if ( gc->gc_charset == NULL ) {
	    gc->gc_charset = DSGW_CHARSET_UTF8;
	    gc->gc_nbspchar = DSGW_UTF8_NBSP;
	}
    } else if (( p = getenv( "HTTP_USER_AGENT" )) != NULL ) {
	char *agent = strstr( p, MOZILLA_SIG ); 
	if ( gc->gc_charset == NULL && 
		agent && atoi( agent + strlen( MOZILLA_SIG )) >= 4 ) {
	    gc->gc_charset = DSGW_CHARSET_UTF8;
	    gc->gc_nbspchar = DSGW_UTF8_NBSP;
	}
    } else {
	/*
	 * browser is ignorant of UTF-8: 
	 *	ignore Accept-Language header
	 *	do not send UTF-8 charset header by default
	 */
	gc->gc_charset = DSGW_CHARSET_USASCII;
    }

    return( gc );
}


/*
 * return non-zero if string "target" is an element of the comma-separated
 * list "list" (case insensitive match, leading whitespace ignored)
 */
static int
list_contains_str( char *list, char *target )
{
    char	*p, *item;
    int		at_end;

    if ( list == NULL ) {
	return( 0 );
    }

    list = dsgw_ch_strdup( list );	/* so we can clobber without fear */

    item = p = list;
    at_end = 0;
    while ( !at_end ) {
	if ( *p == '\0' || *p == ',' ) {
	    if ( *p == '\0' ) {
		at_end = 1;
	    } else {
		*p = '\0';
	    }
	    while ( XP_IS_SPACE( *item )) {
		++item;	/* skip leading white space */
	    }
	    if ( strcasecmp( item, target ) == 0 ) {
		return( 1 );
	    }
	    item = ++p;
	} else {
	    ++p;
	}
    }

    free( list );
    return( 0 );
}


static void
report_ldapu_error( int ldapu_err, int dsgw_err, int erropts )
{
    char *extra = "";

    switch( ldapu_err ) {
    case LDAPU_ERR_CANNOT_OPEN_FILE:
	extra = XP_GetClientStr(DBT_cannotOpenFile_); 
	break;
    case LDAPU_ERR_DBNAME_IS_MISSING:
    case LDAPU_ERR_NOT_PROPVAL:
	extra = XP_GetClientStr(DBT_malformedDbconfFile_);
	break;
    case LDAPU_ERR_PROP_IS_MISSING:
	extra = XP_GetClientStr(DBT_missingPropertyNameInDbconfFile_);
	break;
    case LDAPU_ERR_OUT_OF_MEMORY:
	extra = XP_GetClientStr(DBT_outOfMemory_1);
	break;
    case LDAPU_ERR_DIRECTIVE_IS_MISSING:
	extra = XP_GetClientStr(DBT_missingDirectiveInDbconfFile_);
	break;
    }

    dsgw_error( dsgw_err, extra, erropts, 0 );
}


/*
 * Read the gateway config file (dsgw.conf).
 */
static void
read_dsgwconfig( char *filename, int templatesonly )
{
    char	buf[ MAXPATHLEN + 100 ];
    int		cargc;
    char	*cargv[ MAXARGS ];
    FILE	*fp;
    char	*line, *locsuffix = NULL;
    int		lineno;
    int		rc;
    LDAPURLDesc	*ludp;

    if (( fp = fopen( filename, "r" )) == NULL ) {
	sprintf( buf, XP_GetClientStr(DBT_cannotOpenConfigFileSN_), filename );
	dsgw_error( DSGW_ERR_BADCONFIG, buf, DSGW_ERROPT_EXIT, 0 );
    }
    fp_getline_init( &lineno );

    while ( (line = fp_getline( fp, &lineno )) != NULL ) {
	/* skip comments and blank lines */
	if ( line[0] == '#' || line[0] == '\0' ) {
	    continue;
	}

	fp_parse_line( line, &cargc, cargv );

	if ( cargc < 1 ) {
	    continue;
	}

	if ( strcasecmp( cargv[0], "requireauth" ) == 0 ) {
	    if ( templatesonly ) continue;
	    gc->gc_authrequired = 1;
	}

	if ( strcasecmp( cargv[0], "authlifetime" ) == 0 ) {
	    if ( templatesonly ) continue;
	    if ( cargc < 2 ) {
		adderr( gc, XP_GetClientStr(DBT_missingArgumentForAuthlifetimeDi_),
			filename, lineno );
		continue;
	    }
	    gc->gc_authlifetime = (time_t) atol( cargv[ 1 ]);
	} else if ( strcasecmp( cargv[0], "dirmgr" ) == 0 ) {
	    if ( templatesonly ) continue;
	    if ( cargc < 2 ) {
		adderr( gc, XP_GetClientStr(DBT_missingArgumentForDirmgrDirectiv_),
			filename, lineno );
		continue;
	    }
	    gc->gc_rootdn = dsgw_ch_strdup( cargv[ 1 ]);
	} else if ( strcasecmp( cargv[0], "baseurl" ) == 0 ) {
	    if ( templatesonly ) continue;
	    if ( cargc < 2 ) {
		adderr( gc, XP_GetClientStr(DBT_missingArgumentForBaseurlDirecti_),
			filename, lineno );
		continue;
	    }
	    gc->gc_baseurl = dsgw_ch_strdup( cargv[ 1 ]);
	    if (( rc = ldap_url_parse( gc->gc_baseurl, &ludp )) != 0 ) {
		switch ( rc ) {
		case LDAP_URL_ERR_NODN:
		    adderr( gc, XP_GetClientStr(DBT_badUrlProvidedForBaseurlDirectiv_), filename, lineno );
		    break;
		case LDAP_URL_ERR_MEM:
		    dsgw_error( DSGW_ERR_NOMEMORY, XP_GetClientStr(DBT_parsingBaseurlDirective_),
			    DSGW_ERROPT_EXIT, 0 );
		    break;
		case LDAP_URL_ERR_NOTLDAP:
		    adderr( gc, XP_GetClientStr(DBT_badUrlProvidedForBaseurlDirectiv_1), filename, lineno );
		    break;
		}
	    } else {
		gc->gc_ldapserver = ludp->lud_host;
		gc->gc_ldapport = ludp->lud_port;
		if ( ludp->lud_dn == NULL ) {
		    gc->gc_ldapsearchbase = dsgw_ch_strdup( "" );
		} else {
		    gc->gc_ldapsearchbase = ludp->lud_dn;
		}
		if (( ludp->lud_options & LDAP_URL_OPT_SECURE ) != 0 ) {
#ifdef DSGW_NO_SSL
		    adderr( gc, XP_GetClientStr(DBT_LdapsUrlsAreNotYetSupportedN_),
			    filename, lineno );
#else
		    gc->gc_ldapssl = 1;
#endif
		}
	    }

	} else if ( strcasecmp( cargv[0], "template" ) == 0 ) {
	    if ( cargc < 3 ) {
		adderr( gc, XP_GetClientStr(DBT_missingArgumentsForTemplateDirec_),
			filename, lineno );
		continue;
	    }
	    dsgw_addtemplate( &gc->gc_templates, cargv[1], cargc - 2,
		    &cargv[2] );

#ifndef DSGW_NO_SSL
	} else if ( strcasecmp( cargv[0], "sslrequired" ) == 0 ) {
	    if ( templatesonly ) continue;
	    if ( cargc < 2 ) {
		adderr( gc, XP_GetClientStr(DBT_missingArgumentForSslrequiredDir_),
			filename, lineno );
		continue;
	    }
	    if ( strcasecmp( cargv[1], "never" ) == 0 ) {
		gc->gc_sslrequired = DSGW_SSLREQ_NEVER;
	    } else if ( strcasecmp( cargv[1], "whenauthenticated" ) == 0 ) {
		gc->gc_sslrequired = DSGW_SSLREQ_WHENAUTHENTICATED;
	    } else if ( strcasecmp( cargv[1], "always" ) == 0 ) {
		gc->gc_sslrequired = DSGW_SSLREQ_ALWAYS;
	    } else {
		adderr( gc, XP_GetClientStr(DBT_unknownArgumentToSslrequiredDire_), filename, lineno );
	    }

	} else if ( strcasecmp( cargv[0], "securitypath" ) == 0 ) {
	    if ( templatesonly ) continue;
	    if ( cargc < 2 ) {
		adderr( gc, XP_GetClientStr(DBT_missingArgumentForSecuritypathDi_),
			filename, lineno );
		continue;
	    }
	    gc->gc_securitypath = dsgw_ch_strdup( cargv[1] );
#endif /* !DSGW_NO_SSL */

	} else if ( strcasecmp( cargv[0], "location-suffix" ) == 0 ) {
	    if ( templatesonly ) continue;
	    if ( cargc < 2 ) {
		adderr( gc,
			XP_GetClientStr(DBT_missingArgumentForLocationSuffix_),
			filename, lineno );
		continue;
	    }
	    if ( locsuffix != NULL ) {
		free( locsuffix );
	    }
	    locsuffix = dsgw_ch_strdup( cargv[1] );

	} else if ( strcasecmp( cargv[0], "location" ) == 0 ) {
	    if ( cargc < 4 ) {
		adderr( gc,
    XP_GetClientStr(DBT_threeArgumentsAreRequiredForTheL_),
			filename, lineno );
		continue;
	    }
	    add_location( &gc->gc_newentryloccount, &gc->gc_newentrylocs,
		    locsuffix, &cargv[1] );

	} else if ( strcasecmp( cargv[0], "locationurl" ) == 0 ) {
	    if ( cargc < 2 ) {
		adderr( gc,
			XP_GetClientStr(DBT_missingArgumentForLocationurlDir_),
			filename, lineno );
		continue;
	    }
	    gc->gc_locationurl = dsgw_ch_strdup( cargv[1] );

	} else if ( strcasecmp( cargv[0], "newtype" ) == 0 ) {
	    if ( templatesonly ) continue;
	    if ( cargc < 3 ) {
		adderr( gc,
    XP_GetClientStr(DBT_atLeastTwoArgumentsAreRequiredFo_),
			filename, lineno );
		continue;
	    }
	    if ( add_newtype( &gc->gc_newentrytypes, gc->gc_newentryloccount,
			gc->gc_newentrylocs, cargc - 1, &cargv[1] ) < 0 ) {
		adderr( gc, XP_GetClientStr(DBT_unknownLocationInNewtypeDirectiv_),
			filename, lineno );
	    }

	} else if ( strcasecmp( cargv[0], "tmplset" ) == 0 ) {
	    if ( cargc != 4 && cargc != 5 ) {
		adderr( gc,
    XP_GetClientStr(DBT_threeOrFourArgumentsAreRequiredF_),
			filename, lineno );
		continue;
	    }
	    add_tmplset( &gc->gc_tmplsets, cargc - 1, &cargv[1] );

	} else if ( strcasecmp( cargv[0], "attrvset" ) == 0 ) {
	    if ( cargc != 5 ) {
		adderr( gc,
    XP_GetClientStr(DBT_fourArgumentsAreRequiredForTheAt_),
			filename, lineno );
		continue;
	    }
	    add_avset( &gc->gc_avsets, &cargv[1] );

	} else if ( strcasecmp( cargv[0], "includeset" ) == 0 ) {
	    if ( cargc != 3 ) {
		adderr( gc,
    XP_GetClientStr(DBT_twoArgumentsAreRequiredForTheInc_),
			filename, lineno );
		continue;
	    }
	    add_includeset( &gc->gc_includesets, &cargv[1] );

	} else if ( strcasecmp( cargv[0], "charset" ) == 0 ) {
	    if ( cargc < 2 ) {
		adderr( gc, XP_GetClientStr(DBT_missingArgumentForCharsetDirecti_),
			filename, lineno );
		continue;
	    }
	    gc->gc_charset = dsgw_ch_strdup( cargv[1] );

/* the following is not needed because AdminServer */
/* puts these into environment from ns-admin.conf */
#ifdef NEED_LANG_FROM_DSGW_CONF
	} else if ( strcasecmp( cargv[0], "ClientLanguage" ) == 0 ) {
	    if ( cargc < 2 ) {
		adderr( gc,
                        XP_GetClientStr(DBT_missingArgumentForClientlanguage_),
			filename, lineno );
		continue;
	    }
	    gc->gc_ClientLanguage = dsgw_ch_strdup( cargv[1] );

	} else if ( strcasecmp( cargv[0], "AdminLanguage" ) == 0 ) {
	    if ( cargc < 2 ) {
		adderr( gc,
                        XP_GetClientStr(DBT_missingArgumentForAdminlanguageD_),
			filename, lineno );
		continue;
	    }
	    gc->gc_AdminLanguage = dsgw_ch_strdup( cargv[1] );

	} else if ( strcasecmp( cargv[0], "DefaultLanguage" ) == 0 ) {
	    if ( cargc < 2 ) {
		adderr( gc,
                        XP_GetClientStr(DBT_missingArgumentForDefaultlanguag_),
			filename, lineno );
		continue;
	    }
	    gc->gc_DefaultLanguage = dsgw_ch_strdup( cargv[1] );
#endif

	/* include another config file */
	} else if ( strcasecmp( cargv[0], "include" ) == 0 ) {
	    char	*tmpfname;

	    if ( cargc < 2 ) {
		adderr( gc, XP_GetClientStr(DBT_missingFilenameForIncludeDirecti_),
			filename, lineno );
		continue;
	    }

	    tmpfname = dsgw_ch_strdup( cargv[1] );
	    read_dsgwconfig( tmpfname, templatesonly );
	    free( tmpfname );

	/* I18n: language list for the popup */
	} else if ( strcasecmp( cargv[0], "language" ) == 0 ) {
	    if ( cargc != 4 && cargc != 5 ) {
		adderr( gc, XP_GetClientStr(DBT_threeOrFourArgumentsAreRequiredF_),
			filename, lineno );
		continue;
	    }
	    if ( gc->gc_international )
	        add_langset( &gc->gc_languages, cargc - 1, &cargv[1], 1 );

	/* I18n: switch to turn on/off i18n features */
	} else if ( strcasecmp( cargv[0], "international" ) == 0 ) {
	    if ( cargc != 2 ) {
		adderr( gc, XP_GetClientStr(DBT_threeOrFourArgumentsAreRequiredF_),
			filename, lineno );
		continue;
	    }
	    if ( *cargv[1] == 'Y' || *cargv[1] == 'y' )
	        gc->gc_international = 1;	/* ON */
	    else {
	        gc->gc_international = 0;	/* OFF */
		if ( gc->gc_languages ) {
                    dsgw_free_language_popup();
                    dsgw_setup_default_laguage_popup();
		    gc->gc_defaultselectedlang = DSGW_DEFAULT_SELECTED_LANG;
		}
	    }

	} else {
#ifdef DSGW_DEBUG
            dsgw_log( "Undefined Directive: %s\n", cargv[0] );
#endif
	    adderr( gc, XP_GetClientStr(DBT_unknownDirectiveInConfigFileN_),
		    filename, lineno );
	}
    }

    if ( gc == NULL || gc->gc_configerr > 0 ) {
	dsgw_error( DSGW_ERR_BADCONFIG, ( gc->gc_configerrstr == NULL ) ?
		"" : gc->gc_configerrstr, DSGW_ERROPT_EXIT, 0 );
    }

    /* I18n: in case no language info in dsgw.conf */
    if ( !gc->gc_languages ) {
        dsgw_setup_default_laguage_popup();
	gc->gc_defaultselectedlang = DSGW_DEFAULT_SELECTED_LANG;
	gc->gc_international = 0;	/* OFF */
    }
}

int
erase_db() {

    FILE    *fp;
    int     rc, lineno;
    char    *line;
    char    *cargv[ MAXARGS ];
    int     cargc;
    char    cmd[ BIG_LINE ];

    if ( (fp = fopen( gc->gc_localdbconf, "r" )) == NULL ) {
        fprintf (stdout, XP_GetClientStr(DBT_EraseDbCouldNotOpenLcacheConfFil_),
            gc->gc_localdbconf);
        return( -1 );
    }
    fp_getline_init( &lineno );

    while ( (line = fp_getline( fp, &lineno )) != NULL ) {
        fp_parse_line( line, &cargc, cargv );
	if ( cargc == 0 ) {
		continue;	/* ignore empty lines */
	}
        if ( strcasecmp( cargv[0], "directory" ) == 0) {
#ifdef XP_WIN32
	    dsgw_unix2dospath( cargv[1] );
#endif
            sprintf (cmd, "%s %s%c* > %s 2>&1", DSGW_DELETE_CMD,
		    dsgw_escape_for_shell( cargv[1] ),
		    DSGW_PATHSEP_CHAR, DSGW_NULL_DEVICE);
            fflush (0);
            if (system (cmd) == 0) {
                /*
                 * success: display status message
                 */
		    fputs(
                    XP_GetClientStr(DBT_FontSize1NPTheDatabaseHasBeenDel_),
                    stdout );
		    rc = 0;
            }
            else {
                fputs(
                    XP_GetClientStr(DBT_FontSize1NPTheDatabaseCouldNotBe_),
                    stdout );
		    rc = -1;
            }

            fputs( "<HR>\n", stdout );
            fclose( fp );
	    return( rc );
        }
    }
}

void
app_suffix (char *ldif, char *suffix)
{
    FILE    *oldfp, *newfp;
    char    *s, *line, *type, *val, *newval;
    int     i, lineno, vlen, newlen;
    char    tmpldif[ MAXPATHLEN ];
    char    *dns[] = { "aliasedobjectname",
                        "aliasedobjectname",
                        "associatedname",
                        "dependentupon",
                        "ditredirect",
                        "dn",
                        "documentauthor",
                        "documentauthor",
                        "documentavailable",
                        "errorsto",
                        "errorsto",
                        "imagefiles",
                        "lastmodifiedby",
                        "manager",
                        "member",
                        "memberofgroup",
                        "naminglink",
                        "naminglink",
                        "obsoletedbydocument",
                        "obsoletesdocument",
                        "owner",
                        "proxy",
                        "reciprocalnaminglink",
                        "reciprocalnaminglink",
                        "replicaroot",
                        "replicabinddn",
                        "requeststo",
                        "roleoccupant",
                        "secretary",
                        "seealso",
                        "uniqueMember",
                        "updatedbydocument",
                        "updatesdocument",
                        NULL
                     };


    if ( (oldfp = fopen( ldif, "r" )) == NULL ) {
        fprintf (stdout, XP_GetClientStr(DBT_AppSuffixCouldNotOpenLdifFileSN_),
            ldif);
        return;
    }

    sprintf( tmpldif, "%s.tmp", ldif);
    if ( (newfp = fopen( tmpldif, "w" )) == NULL ) {
        fprintf (stdout, XP_GetClientStr(DBT_AppSuffixCouldNotOpenTmpFileSN_),
            ldif);
        return;
    }

    fp_getline_init( &lineno );
    while ( (line = fp_getline( oldfp, &lineno )) != NULL ) {
	/* skip comments and blank lines */
	if ( line[0] == '#' || line[0] == '\0' ) {
            fputs( line, newfp );
	    fputc( '\n', newfp );
	    continue;
	}

	if ( dsgw_ldif_parse_line( line, &type, &val, &vlen ) < 0 ) {
	    fprintf( stdout, XP_GetClientStr(DBT_AppSuffixCouldNotParseLDIF_),
		    line );
	    return;
	}

        for (i=0; dns[i]!=NULL; i++) {
            if ( strcasecmp( type, dns[i] ) == 0 ) {
		break;
	    }
	}

        if ( dns[i] == NULL ) {
	    s = dsgw_ldif_type_and_value( type, val, vlen );
	} else {
	    newlen = vlen + strlen( suffix ) + 1;
	    newval = dsgw_ch_malloc( newlen + 1 );
	    memcpy( newval, val, vlen );
	    *(newval + vlen) = ',';
	    strcpy( newval + vlen + 1, suffix );
	    s = dsgw_ldif_type_and_value( type, newval, newlen );
	    free( newval );
	}

	fputs( s, newfp );
	free( s );
    }
    fclose(newfp);
    fclose(oldfp);
    unlink( ldif );
    if ( rename( tmpldif, ldif ) != 0 ) {
        fprintf( stdout, XP_GetClientStr(DBT_unableToRenameSToS_), tmpldif, ldif );
        return;
    }
}

/*
 * Running under admserv - traverse the list of property/value pairs
 * returned by dbconf_read_default_dbinfo().
 */
static void
get_dbconf_properties( char *filename )
{
    DBConfDBInfo_t *db_info;
    DBPropVal_t	*dbp;
    int		cargc;
    char	*cargv[ MAXARGS ];
    FILE	*fp;
    char	*line, *locsuffix;
    int		rc;
    LDAPURLDesc	*ludp;
    LDAPDBURLDesc *ldbudp;
    char 	*userLdapURL;

    if (( rc = dbconf_read_default_dbinfo( filename, &db_info ))
	    != LDAPU_SUCCESS ) {
	report_ldapu_error( rc, DSGW_ERR_BADCONFIG, DSGW_ERROPT_EXIT );
    }

    if ( db_info == NULL ) {
	dsgw_error( DSGW_ERR_DBCONF,
		XP_GetClientStr(DBT_nullPointerReturnedByDbconfReadD_),
		DSGW_ERROPT_EXIT, 0 );
    }

    if ( strcasecmp( db_info->dbname, DBCONF_DEFAULT_DBNAME ) != 0 ) {
	dsgw_error( DSGW_ERR_DBCONF, db_info->dbname, DSGW_ERROPT_EXIT, 0 );
    }

#ifdef DSGW_DEBUG
    dsgw_log( "opened dbconf, dbname is %s, dburl is %s\n", db_info->dbname,
	    db_info->url );
#endif

    /* Parse the LDAPURL or LDAPDBURL */
    gc->gc_baseurl = dsgw_ch_strdup( db_info->url );
    rc = ldapdb_url_parse( gc->gc_baseurl, &ldbudp );

    if ( rc == 0 ) {
	gc->gc_localdbconf = dsgw_ch_strdup( ldbudp->ludb_path );
	gc->gc_ldapserver = NULL;
	gc->gc_ldapport = -1;
	gc->gc_ldapsearchbase = dsgw_ch_strdup( ldbudp->ludb_dn );
#ifndef DSGW_NO_SSL
	gc->gc_ldapssl = 0;
#endif

    /* If url isn't "ldapdb://", let the code below have a crack */
    } else if ( rc != DSGW_ERR_LDAPDBURL_NOTLDAPDB ) {
	switch ( rc ) {
	case DSGW_ERR_LDAPDBURL_NODN:
		adderr( gc, XP_GetClientStr(DBT_badLdapdbUrlTheBaseDnIsMissingN_), filename, 0 );
		break;
	case DSGW_ERR_LDAPDBURL_BAD:
		adderr( gc, XP_GetClientStr(DBT_badLdapdbUrlN_), filename, 0 );
		break;
	}
    } else {
        /* atulb:6/3/98  Use the AuthenticationLdapURL env var to determine
           which directory to talk to.  No longer use the one in dbswitch.conf*/
	if (getenv("AuthenticationLdapURL")) {
		userLdapURL = dsgw_ch_strdup(getenv("AuthenticationLdapURL"));
		strcpy(gc->gc_baseurl, userLdapURL);
	}
	if (( rc = ldap_url_parse( gc->gc_baseurl, &ludp )) != 0 ) {
	    switch ( rc ) {
	    case LDAP_URL_ERR_NODN:
		adderr( gc, XP_GetClientStr(DBT_badUrlProvidedForBaseurlDirectiv_2),
			filename, 0 );
		break;
	    case LDAP_URL_ERR_MEM:
		dsgw_error( DSGW_ERR_NOMEMORY,
			XP_GetClientStr(DBT_parsingBaseurlDirective_1),
			DSGW_ERROPT_EXIT, 0 );
		break;
	    case LDAP_URL_ERR_NOTLDAP:
		adderr( gc, XP_GetClientStr(DBT_badUrlProvidedForBaseurlDirectiv_3), filename, 0 );
		break;
	    }
	} else {
	    gc->gc_ldapserver = ludp->lud_host;
	    gc->gc_ldapport = ludp->lud_port;
	    if ( ludp->lud_dn == NULL ) {
		gc->gc_ldapsearchbase = dsgw_ch_strdup( "" );
	    } else {
		gc->gc_ldapsearchbase = ludp->lud_dn;
	    }
	    if ( ( ludp->lud_options & LDAP_URL_OPT_SECURE ) != 0 ) {
#ifdef DSGW_NO_SSL
		adderr( gc, XP_GetClientStr(DBT_LdapsUrlsAreNotYetSupportedN_1),
			filename, 0 );
#else
		gc->gc_ldapssl = 1;
#endif
	    }
	}
    }

    /* Look through the properties for binddn and bindpw */
    for ( dbp = db_info->firstprop; dbp != NULL; dbp = dbp->next ) {

#ifdef DSGW_DEBUG
	dsgw_log( "get prop: prop = %s, val = %s\n", dbp->prop, dbp->val );
#endif

	if ( strcasecmp( dbp->prop, "binddn" ) == 0 ) {
	    if ( dbp->val == NULL || strlen( dbp->val ) == 0 ) {
		dsgw_error( DSGW_ERR_DBCONF, XP_GetClientStr(DBT_noValueGivenForBinddn_), 
			DSGW_ERROPT_EXIT, 0 );
	    }
	    gc->gc_binddn = dsgw_ch_strdup( dbp->val );

	} else if ( strcasecmp( dbp->prop, "bindpw" ) == 0 ) {
	    if ( dbp->val == NULL || strlen( dbp->val ) == 0 ) {
		dsgw_error( DSGW_ERR_DBCONF, XP_GetClientStr(DBT_noValueGivenForBindpw_), 
			DSGW_ERROPT_EXIT, 0 );
	    }
	    gc->gc_bindpw = dsgw_ch_strdup( dbp->val );
	}
    }

    if ( gc == NULL || gc->gc_configerr > 0 ) {
	dsgw_error( DSGW_ERR_BADCONFIG, ( gc->gc_configerrstr == NULL ) ?
		"" : gc->gc_configerrstr, DSGW_ERROPT_EXIT, 0 );
    }
    if ( gc->gc_baseurl == NULL ) {
	dsgw_error( DSGW_ERR_BADCONFIG,
    XP_GetClientStr(DBT_thereIsNoDefaultDirectoryService_),
		DSGW_ERROPT_EXIT, 0 );
    }
    return;
}


/*
 * Update the dbswitch.conf file (used under admin. server) to reflect
 * the local/remote directory information contained in "cfgp".  Our basic
 * strategy is to read the existing dbswitch.conf file, replacing and adding
 * lines that look like this:
 *	directory <dbhandle> ...
 *	<dbhandle>:binddn ...
 *      <dbhandle>:encoded bindpw ...
 * as necessary.  We write a new, temporary config file (copying all other
 * lines over unchanged) and then replace the old file with our new one.
 *
 * If cfgp is configured for localdb mode, we only write a directory line.
 *
 * We return zero if all goes well and non-zero if not.
 *
 * Note that all reading and writing of the dbswitch.conf file is now done
 * using the dbconf...() functions that are part of the ldaputil library, so
 * any comments, blank lines, or unrecognized config file lines will be lost.
 * Also, all "bindpw" property values will be encoded when re-written.
 *
 * Only these members of the cfgp structure are used in this function:
 *	gc_localdbconf		(NULL if using remote LDAP server)
 *	gc_ldapsearchbase
 *	gc_ldapserver
 *	gc_ldapport
 *	gc_ldapssl
 *	gc_binddn
 *	gc_bindpw
 * Actually, if gc_localdbconf is not NULL, only it and gc_ldapsearchbase are
 * used.
 */	
int
dsgw_update_dbswitch( dsgwconfig *cfgp, char *dbhandle, int erropts )
{
    char		oldfname[ MAXPATHLEN ], newfname[ MAXPATHLEN ];
    char		*userdb_path, buf[ MAXPATHLEN + 100 ];
    int			rc, wrote_dbinfo;
    FILE		*newfp;
    DBConfInfo_t	*cip;
    DBConfDBInfo_t	*dbip;
    DBPropVal_t		*pvp;

    if ( dbhandle == NULL ) {
	dbhandle = "default";
    }

    if (( userdb_path = get_userdb_dir()) == NULL ) {
	dsgw_error( DSGW_ERR_USERDB_PATH, NULL, erropts, 0 );
	return( -1 );
    }

    /* read old dbswitch.conf contents */
    sprintf( oldfname, "%s/%s", userdb_path, DSGW_DBSWITCH_FILE );
    if (( rc = dbconf_read_config_file( oldfname, &cip )) != LDAPU_SUCCESS ) {
	report_ldapu_error( rc, DSGW_ERR_BADCONFIG, erropts );
	return( -1 );
    }

    /* write db info to new file, replacing information for "dbhandle" */
    sprintf( newfname, "%s/%s", userdb_path, DSGW_DBSWITCH_TMPFILE );
    if (( newfp = fopen( newfname, "w" )) == NULL ) {
	sprintf( buf, XP_GetClientStr(DBT_cannotOpenConfigFileSForWritingN_), newfname );
	dsgw_error( DSGW_ERR_UPDATE_DBSWITCH, buf, erropts, 0 );
	return( -1 );
    }

    wrote_dbinfo = 0;
    for ( dbip = cip->firstdb; dbip != NULL; dbip = dbip->next ) {
	if ( strcasecmp( dbip->dbname, dbhandle ) == 0 ) {
	    /*
	     * found db name to be replaced:  replace with updated information 
	     */
	    if (( rc = write_dbswitch_info( newfp, cfgp, dbhandle )) !=
		    LDAPU_SUCCESS ) {
		report_ldapu_error( rc, DSGW_ERR_UPDATE_DBSWITCH, erropts );
		return( -1 );
	    }

	    wrote_dbinfo = 1;

	} else {
	    /*
	     * re-write existing db conf information without changes
	     */
	    if (( rc = dbconf_output_db_directive( newfp, dbip->dbname,
		    dbip->url )) != LDAPU_SUCCESS ) {
		report_ldapu_error( rc, DSGW_ERR_UPDATE_DBSWITCH, erropts );
		return( -1 );
	    }

	    for ( pvp = dbip->firstprop; pvp != NULL; pvp = pvp->next ) {
		if (( rc = dbconf_output_propval( newfp, dbip->dbname,
			pvp->prop, pvp->val,
			strcasecmp( pvp->prop, "bindpw" ) == 0 ))
			!= LDAPU_SUCCESS ) {
		    report_ldapu_error( rc, DSGW_ERR_UPDATE_DBSWITCH, erropts );
		    return( -1 );
		}
	    }
	}
    }

    if ( !wrote_dbinfo ) {
	if (( rc = write_dbswitch_info( newfp, cfgp, dbhandle )) !=
		LDAPU_SUCCESS ) {
	    report_ldapu_error( rc, DSGW_ERR_UPDATE_DBSWITCH, erropts );
	    return( -1 );
	}
    }

    dbconf_free_confinfo( cip );
    fclose( newfp );

    /* replace old file with new one */
#ifdef _WIN32
    if ( !MoveFileEx( newfname, oldfname, MOVEFILE_REPLACE_EXISTING )) {
#else
    if ( rename( newfname, oldfname ) != 0 ) {
#endif
	sprintf( buf, XP_GetClientStr(DBT_unableToRenameSToS_1), newfname, oldfname );
	dsgw_error( DSGW_ERR_UPDATE_DBSWITCH, buf, erropts, 0 );
	return( -1 );
    }

    return( 0 );
}


static int
write_dbswitch_info( FILE *fp, dsgwconfig *cfgp, char *dbhandle )
{
    char	*escapeddn, *url;
    int		rc;

    escapeddn = dsgw_strdup_escaped( cfgp->gc_ldapsearchbase );

    if ( cfgp->gc_localdbconf == NULL ) { /* remote server: write ldap:// URL */
	url = dsgw_ch_malloc( 21 + strlen( cfgp->gc_ldapserver )
		+ strlen( escapeddn ));	/* room for "ldaps://HOST:PORT/DN" */
	sprintf( url, "ldap%s://%s:%d/%s",
#ifdef DSGW_NO_SSL
		"",
#else
		cfgp->gc_ldapssl ? "s" : "",
#endif
		cfgp->gc_ldapserver, cfgp->gc_ldapport, escapeddn );
    } else {				  /* local db: write ldapdb:// URL */
	url = dsgw_ch_malloc( 11 + strlen( cfgp->gc_localdbconf )
		+ strlen( escapeddn ));	/* room for "ldapdb://PATH/DN" */
	sprintf( url, "ldapdb://%s/%s\n", cfgp->gc_localdbconf, escapeddn );
    }

    rc = dbconf_output_db_directive( fp, dbhandle, url );

    free( url );
    free( escapeddn );

    if ( rc != LDAPU_SUCCESS ) {
	return( rc );
    }

    if ( cfgp->gc_localdbconf == NULL ) { /* using directory server */
	if ( cfgp->gc_binddn != NULL &&
		( rc = dbconf_output_propval( fp, dbhandle, "binddn",
		cfgp->gc_binddn, 0 ) != LDAPU_SUCCESS )) {
	    return( rc );
	}

	if ( cfgp->gc_bindpw != NULL &&
		( rc = dbconf_output_propval( fp, dbhandle, "bindpw",
		cfgp->gc_bindpw, 1 ) != LDAPU_SUCCESS )) {
	    return( rc );
	}
    }

    return( LDAPU_SUCCESS );
}


/* pass 0 for lineno if it is unknown or not applicable */
static void
adderr( dsgwconfig *gc, char *str, char *filename, int lineno )
{
    char *lbuf = dsgw_ch_malloc( MAXPATHLEN + 200 );

    gc->gc_configerr++;
    if ( lineno == 0 ) {
	sprintf( lbuf, XP_GetClientStr(DBT_configFileS_), filename );
    } else {
	sprintf( lbuf, XP_GetClientStr(DBT_configFileSLineD_), filename, lineno );
    }
    gc->gc_configerrstr = dsgw_ch_realloc( gc->gc_configerrstr,
	    strlen( gc->gc_configerrstr ) + strlen( str )
	    + strlen( lbuf ) + 6 );
    strcat( gc->gc_configerrstr, lbuf );
    strcat( gc->gc_configerrstr, str );
    strcat( gc->gc_configerrstr, "<BR>\n" );
    free( lbuf );
}


static void
add_location( int *loccountp, dsgwloc **locarrayp, char *locsuffix,
	char **argv )
{
    int		len;
    dsgwloc	*locp;

    *locarrayp = (dsgwloc *)dsgw_ch_realloc( *locarrayp, 
	    ( *loccountp + 1 ) * sizeof( dsgwloc ));
    locp = &((*locarrayp)[ *loccountp ]);
    locp->dsloc_handle = dsgw_ch_strdup( argv[0] );
    locp->dsloc_fullname = dsgw_ch_strdup( argv[1] );
    len = strlen( argv[2] );

    if (  argv[2][ len - 1 ] == '#' ) {
	/* '#' implies that locsuffix is not to be appended */
	locp->dsloc_dnsuffix = dsgw_ch_strdup( argv[2] );
	locp->dsloc_dnsuffix[ len - 1 ] = '\0';

    } else if ( locsuffix != NULL && *locsuffix != '\0' ) {
	/* append suffix, preceded by ", " if location arg. is not "" */
	locp->dsloc_dnsuffix = dsgw_ch_malloc( len + strlen( locsuffix ) + 3 );
	if ( argv[2][0] != '\0' ) {
	    strcpy( locp->dsloc_dnsuffix, argv[2] );
	    strcat( locp->dsloc_dnsuffix, ", " );
	    strcat( locp->dsloc_dnsuffix, locsuffix );
	} else {
	    strcpy( locp->dsloc_dnsuffix, locsuffix );
	}

    } else {
	locp->dsloc_dnsuffix = dsgw_ch_strdup( argv[2] );
    }
    ++(*loccountp);
}


static int
add_newtype( dsgwnewtype **newentlistp, int loccount, dsgwloc *locarray,
	int argc, char **argv )
{
    int		i, j;
    dsgwnewtype	*ntp, *prevntp;

    ntp = (dsgwnewtype *)dsgw_ch_malloc( sizeof( dsgwnewtype ));
    ntp->dsnt_template = dsgw_ch_strdup( argv[0] );
    ntp->dsnt_fullname = dsgw_ch_strdup( argv[1] );
    ntp->dsnt_rdnattr = dsgw_ch_strdup( argv[2] );
    ntp->dsnt_next = NULL;
    ntp->dsnt_loccount = argc - 3;
    argv = &argv[3];

    /* fill dsnt_locations array with indexes into gc->gc_newentrylocs */
    if ( ntp->dsnt_loccount <= 0 ) {
	ntp->dsnt_locations = NULL;
    } else {
	int foundit;
	ntp->dsnt_locations = (int *)dsgw_ch_malloc( ntp->dsnt_loccount *
		sizeof( int ));
	for ( i = 0; i < ntp->dsnt_loccount; ++i ) {
	    foundit = 0;
	    for ( j = 0; j < loccount && !foundit; ++j ) {
		if ( strcasecmp( argv[ i ], locarray[ j ].dsloc_handle )
			== 0 ) {
		    ntp->dsnt_locations[ i ] = j;
		    foundit = 1;
		}
	    }
	    /* if ( j >= loccount ) { */
	    if ( !foundit ) {
		return( -1 );	/* unknown location -- error */
	    }
	}
    }

    /* append to linked list of new entry structures */
    if ( *newentlistp == NULL ) {
	*newentlistp = ntp;
    } else {
	for ( prevntp = *newentlistp; prevntp->dsnt_next != NULL;
		prevntp = prevntp->dsnt_next ) {
	    ;
	}
	prevntp->dsnt_next = ntp;
    }

    return( 0 );
}


static void
add_tmplset( dsgwtmplset **tslp, int argc, char **argv )
{
    dsgwtmplset	*prevtsp, *tsp;
    dsgwview	*prevvp, *vp;

    prevtsp = NULL;
    tsp = *tslp;
    while ( tsp != NULL ) {
	if ( strcasecmp( tsp->dstset_name, argv[0] ) == 0 ) {
	    break;
	}
	prevtsp = tsp;
	tsp = tsp->dstset_next;
    }

    if ( tsp == NULL ) {	/* new template set */
	tsp = (dsgwtmplset *)dsgw_ch_malloc( sizeof( dsgwtmplset ));
	memset( tsp, 0, sizeof( dsgwtmplset ));
	tsp->dstset_name = dsgw_ch_strdup( argv[0] );
	if ( prevtsp == NULL ) {
	    *tslp = tsp;
	} else {
	    prevtsp->dstset_next = tsp;
	}
    }

    /* add a new view to the end of this template set's view list */
    vp = (dsgwview *)dsgw_ch_malloc( sizeof( dsgwview ));
    memset( vp, 0, sizeof( dsgwview ));
    vp->dsview_caption = dsgw_ch_strdup( argv[1] );
    vp->dsview_template = dsgw_ch_strdup( argv[2] );
    if ( argc > 3 ) {
	vp->dsview_jscript = dsgw_ch_strdup( argv[3] );
    }

    if ( tsp->dstset_viewlist == NULL ) {
	tsp->dstset_viewlist = vp;
    } else {
	for ( prevvp = tsp->dstset_viewlist; prevvp->dsview_next != NULL;
		prevvp = prevvp->dsview_next ) {
	    ;
	}
	prevvp->dsview_next = vp;
    }
    ++tsp->dstset_viewcount;
}


static void
add_avset( dsgwavset **avsp, char **argv )	/* 4 args. in argv[] */
{
    dsgwavset	*prevavp, *avp;

    /* is this the first element of a set? */
    prevavp = NULL;
    for ( avp = *avsp; avp != NULL; avp = avp->dsavset_next ) {
	if ( strcasecmp( avp->dsavset_handle, argv[0] ) == 0 ) {
	    break;
	}
	prevavp = avp;
    }

    if ( avp == NULL ) {	/* first element: add a new set */
	avp = (dsgwavset *)dsgw_ch_malloc( sizeof( dsgwavset ));
	memset( avp, 0, sizeof( dsgwavset ));
	avp->dsavset_handle = dsgw_ch_strdup( argv[0] );
	if ( prevavp == NULL ) {
	    *avsp = avp;
	} else {
	    prevavp->dsavset_next = avp;
	}
    }

    ++avp->dsavset_itemcount;
    avp->dsavset_values = (char **)dsgw_ch_realloc( avp->dsavset_values,
	    avp->dsavset_itemcount * sizeof( char * ));
    avp->dsavset_values[ avp->dsavset_itemcount - 1 ] =
	    dsgw_ch_strdup( argv[1] );
    avp->dsavset_prefixes = (char **)dsgw_ch_realloc( avp->dsavset_prefixes,
	    avp->dsavset_itemcount * sizeof( char * ));
    avp->dsavset_prefixes[ avp->dsavset_itemcount - 1 ] =
	    dsgw_ch_strdup( argv[2] );
    avp->dsavset_suffixes = (char **)dsgw_ch_realloc( avp->dsavset_suffixes,
	    avp->dsavset_itemcount * sizeof( char * ));
    avp->dsavset_suffixes[ avp->dsavset_itemcount - 1 ] =
	    dsgw_ch_strdup( argv[3] );
}


static void
add_includeset( dsgwinclset **isp, char **argv )	/* 2 args. in argv[] */
{
    dsgwinclset	*previsp, *tmpisp;

    /* is this the first element of a set? */
    previsp = NULL;
    for ( tmpisp = *isp; tmpisp != NULL; tmpisp = tmpisp->dsiset_next ) {
	if ( strcasecmp( tmpisp->dsiset_handle, argv[0] ) == 0 ) {
	    break;
	}
	previsp = tmpisp;
    }

    if ( tmpisp == NULL ) {	/* first element: add a new set */
	tmpisp = (dsgwinclset *)dsgw_ch_malloc( sizeof( dsgwinclset ));
	memset( tmpisp, 0, sizeof( dsgwinclset ));
	tmpisp->dsiset_handle = dsgw_ch_strdup( argv[0] );
	if ( previsp == NULL ) {
	    *isp = tmpisp;
	} else {
	    previsp->dsiset_next = tmpisp;
	}
    }

    ++tmpisp->dsiset_itemcount;
    tmpisp->dsiset_filenames =
	    (char **)dsgw_ch_realloc( tmpisp->dsiset_filenames,
	    tmpisp->dsiset_itemcount * sizeof( char * ));
    tmpisp->dsiset_filenames[ tmpisp->dsiset_itemcount - 1 ] =
	    dsgw_ch_strdup( argv[1] );
}

/* I18n: setting up language list */
static void
add_langset( dsgwlang **lngp, int argc, char **argv, int selected )
{
    dsgwlang	*prevlp, *lp;

    prevlp = NULL;
    for ( lp = *lngp; lp != NULL; lp = lp->dslang_next ) {
	if ( strcasecmp( lp->dslang_tag, argv[0] ) == 0 ) {
	    break;
	}
	prevlp = lp;
    }

    if ( lp == NULL ) {	/* new language list */
	lp = (dsgwlang *)dsgw_ch_malloc( sizeof( dsgwlang ));
	memset( lp, 0, sizeof( dsgwlang ));
	lp->dslang_next = NULL;
	if ( prevlp == NULL ) {
	    *lngp = lp;
	} else {
	    prevlp->dslang_next = lp;
	}
    }

    /* add a new language info */
    lp->dslang_tag  = dsgw_ch_strdup( argv[0] );
    lp->dslang_name = dsgw_ch_strdup( argv[1] );
    lp->dslang_type = atoi( argv[2] );
    lp->dslang_selected = selected;

    if ( argc > 3 && ( *argv[3] == 'Y' || *argv[3] == 'y' ) )
        gc->gc_defaultselectedlang = dsgw_ch_strdup( argv[0] );
}

char *
find_lang( dsgwlang *lngp, char *tag )
{
    dsgwlang *lp;
    
    for ( lp = lngp; lp != NULL; lp = lp->dslang_next ) {
	if ( strcasecmp( lp->dslang_tag, tag ) == 0 ) {
	    return lp->dslang_tag;
	}
    }
    return NULL;
}

static char *
strtok_quote( char *line, char *sep )
{
	int		inquote;
	char		*tmp;
	static char	*next;

	if ( line != NULL ) {
		next = line;
	}
	while ( *next && strchr( sep, *next ) ) {
		next++;
	}

	if ( *next == '\0' ) {
		next = NULL;
		return( NULL );
	}
	tmp = next;

	for ( inquote = 0; *next; ) {
		switch ( *next ) {
		case '"':
			if ( inquote ) {
				inquote = 0;
			} else {
				inquote = 1;
			}
			strcpy( next, next + 1 );
			break;

#ifndef _WIN32
		case '\\':
			strcpy( next, next + 1 );
			break;
#endif

		default:
			if ( ! inquote ) {
				if ( strchr( sep, *next ) != NULL ) {
					*next++ = '\0';
					return( tmp );
				}
			}
			next++;
			break;
		}
	}

	return( tmp );
}

static char	buf[BUFSIZ];
static char	*line;
static int	lmax, lcur;

#define CATLINE( buf )	{ \
	int	len; \
	len = strlen( buf ); \
	while ( lcur + len + 1 > lmax ) { \
		lmax += BUFSIZ; \
		line = (char *) dsgw_ch_realloc( line, lmax ); \
	} \
	strcpy( line + lcur, buf ); \
	lcur += len; \
}



static void
fp_parse_line(
    char	*line,
    int		*argcp,
    char	**argv
)
{
	char *	token, buf[ 20 ];

	*argcp = 0;
	for ( token = strtok_quote( line, " \t" ); token != NULL;
	    token = strtok_quote( NULL, " \t" ) ) {
		if ( *argcp == MAXARGS ) {
			sprintf( buf, XP_GetClientStr(DBT_maxD_), MAXARGS );
			dsgw_error( DSGW_ERR_CONFIGTOOMANYARGS, buf,
				DSGW_ERROPT_EXIT, 0 );
		}
		argv[(*argcp)++] = token;
	}
	argv[*argcp] = NULL;
}



static char *
fp_getline( FILE *fp, int *lineno )
{
	char		*p;

	lcur = 0;

	while ( fgets( buf, sizeof(buf), fp ) != NULL ) {
		if ( (p = strchr( buf, '\n' )) != NULL ) {
			*p = '\0';
		}
		if ( lcur > 0 && ! XP_IS_SPACE( buf[0] ) ) {
			return( line );	/* return previously saved line */
		}
		/* CATLINE( buf ); */
{
	int	len;
	len = strlen( buf );
	while ( lcur + len + 1 > lmax ) {
		lmax += BUFSIZ;
		line = (char *) dsgw_ch_realloc( line, lmax );
	}
	strcpy( line + lcur, buf );
	lcur += len;
}
		(*lineno)++;
		if ( ! XP_IS_SPACE( buf[0] )) {
			return( line );	/* return this line */
		}
	}
	buf[0] = '\0';

	return( lcur > 0 ? line : NULL );
}

static void
fp_getline_init( int *lineno )
{
	*lineno = 0;
        lmax = lcur = 0;
	buf[0] = '\0';
}

/* I18n: unread one line */
static void
fp_pushbackline( FILE *fp, int *lineno, char *str )
{
	(void)fseek( fp, -strlen(str), SEEK_CUR );
	*lineno--;
}


static int
ldapdb_url_parse( char *url, LDAPDBURLDesc **ldbudpp )
{
/*
 * Pick apart the pieces of an ldapdb:// quasi-URL
 */
    LDAPDBURLDesc	*ldbudp;
    char		*basedn;

    *ldbudpp = NULL;

    if ( strncasecmp( url, LDAPDB_URL_PREFIX, LDAPDB_URL_PREFIX_LEN )) {
	return( DSGW_ERR_LDAPDBURL_NOTLDAPDB );
    }

    /* allocate return struct */
    ldbudp = (LDAPDBURLDesc *) dsgw_ch_malloc( sizeof( LDAPDBURLDesc ));

    /* Make a copy */
    url = dsgw_ch_strdup( url );
    ldbudp->ludb_path = url + LDAPDB_URL_PREFIX_LEN;

    /* Must start with a "/" (or "x:" on NT) */
    if ( ldbudp->ludb_path[ 0 ] != '/'
#ifdef _WIN32
	    && ( !isalpha( ldbudp->ludb_path[ 0 ] )
	    || ldbudp->ludb_path[ 1 ] != ':' )
#endif
    ) {
	free( url );
	free( ldbudp );
	return( DSGW_ERR_LDAPDBURL_BAD );
    }

    /* Find base DN */
    if (( basedn = strrchr( ldbudp->ludb_path, '/' )) == NULL ) {
	free( url );
	free( ldbudp );
	return( DSGW_ERR_LDAPDBURL_BAD );
    }

    *basedn++ = '\0';
    ldbudp->ludb_dn = basedn;
    dsgw_form_unescape( ldbudp->ludb_dn );

    *ldbudpp = ldbudp;
    return( 0 );
}

#ifdef XP_WIN32
/* convert forward slashes to backwards ones */
static void
dsgw_unix2dospath( char *path )
{
    if( path ) {
	while( *path ) {
	    if( *path == '/' ) {
		 *path = '\\';
	    }
	    path++;
	}
    }
}
#endif

/* I18n: for managing the language list used in the pop up */
#define COUTLANG "#language"
#define LANGUAGE "language"
#define BSIZ 512
void
dsgw_set_lang()
{
    char	*ofname, *nfname;
    FILE        *ofp, *nfp;
    int         lineno;
    int		cargc;
    char	*cargv[ MAXARGS ];
    char        buf[ BSIZ ];

    /* Set rest of config. by reading the appropriate config files */
    ofname = dsgw_ch_malloc( MAXPATHLEN );
    sprintf( ofname, "%s%s", gc->gc_configdir, DSGW_LANGFILE );
    if (( ofp = fopen( ofname, "r" )) == NULL ) {
	sprintf( buf, XP_GetClientStr(DBT_cannotOpenConfigFileSN_), ofname );
	dsgw_error( DSGW_ERR_BADCONFIG, buf, DSGW_ERROPT_EXIT, 0 );
    }

    nfname = dsgw_ch_malloc( MAXPATHLEN );
    sprintf( nfname, "%s%s_new", gc->gc_configdir, DSGW_LANGFILE );
    if (( nfp = fopen( nfname, "w" )) == NULL ) {
	sprintf( buf, XP_GetClientStr(DBT_cannotOpenConfigFileSN_), nfname );
	dsgw_error( DSGW_ERR_BADCONFIG, buf, DSGW_ERROPT_EXIT, 0 );
    }

    fp_getline_init( &lineno );
    while ( (line = fp_getline( ofp, &lineno )) != NULL ) {
	/* skip comments and blank lines */
	if (( line[0] == '#' &&
              strncasecmp( line, COUTLANG, sizeof( COUTLANG )-1 ))
	      || line[0] == '\0' ) {
            fprintf( nfp, "%s\n", line );
	    continue;
	}

	strcpy( buf, line );
	fp_parse_line( line, &cargc, cargv );

	if ( cargc < 1 ) {
	    continue;
	}

	if ( strcasecmp( cargv[0], "international" ) == 0 ) {
	    if ( gc->gc_international == 0 )
	        fprintf( nfp, "international	N\n" );
	    else
	        fprintf( nfp, "international	Y\n" );
	} else if ( strcasecmp( cargv[0], LANGUAGE ) == 0 ) {
	    if ( !strcasecmp( gc->gc_defaultselectedlang, cargv[1] ))
		*cargv[4] = 'Y';
	    else
		*cargv[4] = 'N';

	    if ( find_lang( gc->gc_languages, cargv[1] ) == NULL ) {
	        /* currently enabled, but get disabled */
		if ( *cargv[4] == 'Y' ) {	
		    /* you cannot select this lang for default lang */
		    *cargv[4] = 'N';
		    gc->gc_defaultselectedlang = DSGW_DEFAULT_SELECTED_LANG;
		}
	        fprintf( nfp, "#%s\t%s\t%s\t%s\t%s\n",
			  cargv[0], cargv[1], cargv[2], cargv[3], cargv[4] );
	    } else {
	        fprintf( nfp, "%s\t%s\t%s\t%s\t%s\n",
			  cargv[0], cargv[1], cargv[2], cargv[3], cargv[4] );
	    }
	} else if ( strcasecmp( cargv[0], COUTLANG ) == 0 ) {
	    if ( !strcasecmp( gc->gc_defaultselectedlang, cargv[1] ))
		*cargv[4] = 'Y';
	    else
		*cargv[4] = 'N';

	    if ( find_lang( gc->gc_languages, cargv[1] ) != NULL ) {
	        /* currently disabled, but get enabled */
	        fprintf( nfp, "%s\t%s\t%s\t%s\t%s\n",
			  LANGUAGE, cargv[1], cargv[2], cargv[3], cargv[4] );
	    } else {
		if ( *cargv[4] == 'Y' ) {
		    /* you cannot select this lang for default lang */
		    *cargv[4] = 'N';
		}
	        fprintf( nfp, "%s\t%s\t%s\t%s\t%s\n",
			  cargv[0], cargv[1], cargv[2], cargv[3], cargv[4] );
	    }
	} else
            fprintf( nfp, "%s\n", buf );
    }

    fclose( ofp );
    fclose( nfp );

#ifdef XP_WIN32
    _fcloseall();	/* cannot unlink the conf file w/o this */
    if ( _unlink( ofname ) < 0 ) {
        dsgw_error( DSGW_ERR_BADCONFIG, NULL, DSGW_ERROPT_EXIT, 0 );
    }
    if ( rename( nfname, ofname ) < 0 ) {
        dsgw_error( DSGW_ERR_BADCONFIG, NULL, DSGW_ERROPT_EXIT, 0 );
    }
#else
    if ( unlink( ofname ) < 0 || rename( nfname, ofname ) < 0 ) {
        sprintf( buf, XP_GetClientStr(DBT_cannotUpdateConfigFile), ofname );
        dsgw_error( DSGW_ERR_BADCONFIG, buf, DSGW_ERROPT_EXIT, 0 );
    }
#endif
}

void
dsgw_get_lang( dsgwlang **managelangp )
{
    char	*fname;
    FILE        *fp;
    int         lineno;
    int		cargc;
    char	*cargv[ MAXARGS ];

    /* Set rest of config. by reading the appropriate config files */
    fname = dsgw_ch_malloc( MAXPATHLEN );
    sprintf( fname, "%s%s", gc->gc_configdir, DSGW_LANGFILE );

    if (( fp = fopen( fname, "r" )) == NULL ) {
	sprintf( buf, XP_GetClientStr(DBT_cannotOpenConfigFileSN_), fname );
	dsgw_error( DSGW_ERR_BADCONFIG, buf, DSGW_ERROPT_EXIT, 0 );
    }

    while ( (line = fp_getline( fp, &lineno )) != NULL ) {
	/* skip comments and blank lines */
	if (( line[0] == '#' && 
              strncasecmp( line, COUTLANG, strlen( COUTLANG )))
	      || line[0] == '\0' ) {
	    continue;
	}

	fp_parse_line( line, &cargc, cargv );

	if ( cargc < 1 ) {
	    continue;
	}

	if ( strcasecmp( cargv[0], LANGUAGE ) == 0 ) {
	    add_langset( managelangp, cargc - 1, &cargv[1], 1 );
	} else if ( strcasecmp( cargv[0], COUTLANG ) == 0 ) {
	    add_langset( managelangp, cargc - 1, &cargv[1], 0 );
	}
    }

    fclose( fp );
}
