/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#include "dsgw.h"

static void initialize_html_strings();
static void get_request();
void genspobjdef( struct ldap_searchobj *sop );
static char *remainder1, *remainder2a, *remainder2b;

main( argc, argv, env )
    int		argc;
    char	*argv[];
#ifdef DSGW_DEBUG
    char	*env[];
#endif
{
    int		reqmethod;

    initialize_html_strings();
    reqmethod = dsgw_init( argc, argv, DSGW_METHOD_GET );
    dsgw_send_header();

#ifdef DSGW_DEBUG
   dsgw_logstringarray( "env", env ); 
{
    char	buf[ 1024 ];
    getcwd( buf, sizeof(buf));
    dsgw_log( "cwd: \"%s\"\n", buf );
}
#endif

    if ( reqmethod == DSGW_METHOD_GET ) {
	get_request();
    }

    exit( 0 );
}


static void
get_request()
{
    FILE			*html;
    char			line[ BIG_LINE ];
    int				argc;
    char			**argv;
    struct ldap_searchobj	*sop;
    char			*ufn;

    dsgw_init_searchprefs( &sop );
    fputs( "<HTML>\n"
	"<HEAD>\n"
	"<TITLE>Directory Server Gateway: Advanced Search</TITLE>\n"
	"<SCRIPT LANGUAGE=\"JavaScript\">\n"
	"<!-- Hide from non-JavaScript-capable browsers\n"
	"var emptyFrame = '<HTML><BODY></BODY></HTML>';\n"
	"var selectedObj = 1;\n"
	"var selectedAttr = 1;\n"
	"var selectedMatch = 1;\n", stdout );
    printf( "var searchBase = \"%s\";\n", gc->gc_ldapsearchbase );
    ufn = ldap_dn2ufn( gc->gc_ldapsearchbase );
    printf( "var UFNsearchBase = \"%s\";\n", ufn );
    free( ufn );
    genspobjdef( sop );
    printf( remainder1, dsgw_getvp( DSGW_CGINUM_DOSEARCH ));
    dsgw_emit_helpbutton_qe( "ASEARCH" );
    printf( remainder2a, gc->gc_ldapsearchbase, gc->gc_ldapserver,
	    gc->gc_ldapport );
    printf( remainder2b );
}



char *
getmt(
struct ldap_searchmatch *sm,
int index
)
{
    struct ldap_searchmatch *p;
    int	i;

    for ( p = sm, i = 0; p != NULL; p = p->sm_next, i++ ) {
	if ( i == index ) {
	    return( p->sm_matchprompt );
	}
    }
    return NULL;
}


void
genspobjdef( 
struct ldap_searchobj *sop
)
{
    struct ldap_searchobj *so;
    struct ldap_searchattr *sa;
    struct ldap_searchmatch *sm;
    int oi, ai, mi, moi;

    printf( "var sp = new Object;\n" );
    for ( so = ldap_first_searchobj( sop ), oi = 1; so != NULLSEARCHOBJ;
	    so = ldap_next_searchobj( sop, so )) {

	/* Skip any marked "internal-only" */
	if ( LDAP_IS_SEARCHOBJ_OPTION_SET( so, LDAP_SEARCHOBJ_OPT_INTERNAL )) {
	    continue;
	}

	printf( "  sp[%d] = new Object;\n", oi );
	printf( "  sp[%d].objtype = \"%s\";\n", oi, so->so_objtypeprompt );
	printf( "  sp[%d].salist = new Object;\n", oi );
	for ( sa = so->so_salist, ai = 1; sa != NULL;
		sa = sa->sa_next, ai++ ) {
	    printf( "    sp[%d].salist[%d] = new Object;\n", oi, ai );
	    printf( "    sp[%d].salist[%d].name = \"%s\";\n", oi, ai,
		    sa->sa_attrlabel );
	    printf( "      matchtypes = new Object;\n" );
	    moi = 1;
	    for ( sm = so->so_smlist, mi = 1; sm != NULL;
		    sm = sm->sm_next, mi++ ) {
		if ( sa->sa_matchtypebitmap & ( 1L << ( mi - 1 ))) {
		    printf( "      matchtypes[%d] = \"%s\";\n", moi++,
			    getmt( so->so_smlist, mi - 1 ));
		}
	    }
	    printf( "      matchtypes.count = %d;\n", moi - 1 );
	    printf( "    sp[%d].salist[%d].matchtypes = matchtypes;\n",
		    oi, ai );
	}
	printf( "  sp[%d].salist.count = %d;\n", oi, ai - 1 );
	oi++;
    }
    printf( "sp.count = %d;\n", oi - 1 );
}




static void
initialize_html_strings()
{
    remainder1 = 
"\n"
"\n"
"function createSearchBaseFrame()\n"
"{\n"
"    searchBaseHTML = '<body>\\n<table>\\n<th>within:</th>\\n' +\n"
"        '<td>' + parent.UFNsearchBase + '</td>\\n' +\n"
"        '</table></body>\\n';\n"
"    self.searchBaseFrame.location.href = \"javascript:parent.searchBaseHTML\";\n"
"}\n"
"\n"
"\n"
"function reppath(path, newstr)\n"
"{\n"
"    var tmpstr;\n"
"    var ind = path.lastIndexOf( \"/\" );\n"
"    if ( ind < 0) {\n"
"        ind = 0;\n"
"    }\n"
"    tmpstr = path.substring(0, ind) + \"/\" + newstr;\n"
"    return tmpstr;\n"
"}\n"
"\n"
"function setHiddenFields(sform)\n"
"{\n"
/*
 * On Navigator 2.x, the form variable's value seems to get set
 * *after* the onSumbit handler executes, which is unfortunate.
 */
"    if (sform.searchstring.value == \"\") {\n"
"         alert(\"You did not supply a search string\");\n"
"         return false;\n"
"    }\n"
"    sform.type.value = sp[parent.selectedObj].objtype;\n"
"    sform.attr.value = sp[parent.selectedObj].salist[parent.selectedAttr].name;\n"
"    sform.match.value = sp[parent.selectedObj].salist[parent.selectedAttr].matchtypes[selectedMatch];\n"
"    sform.action = window.location.protocol + '//' + window.location.host + '%s';\n"
"    return true;\n"
"}\n"
"\n"
"\n"
"sshtml = '<BODY onLoad=\"document.searchStringForm.searchstring.focus();\">\\n' +\n"
"'<FORM NAME=\"searchStringForm\" METHOD=POST \\n' +\n"
"'TARGET=\"outputFrame\" onSubmit=\"return parent.setHiddenFields(this)\">\\n' +\n"
"'<TABLE>\\n' +\n"
"'<TR>\\n' +\n"
"'<TD><INPUT NAME=\"searchstring\" SIZE=20></TD>\\n' +\n"
"'<TD><NOBR><INPUT TYPE=\"SUBMIT\" VALUE=\"Search\" NAME=\"submitButton\">' +\n"
"'&nbsp&nbsp\\n' +\n"
"'";

    remainder2a = 
"\\n' +\n"
"'</TD>\\n' +\n"
"'</TR>\\n' +\n"
"'</TABLE>\\n' +\n"
"'<INPUT TYPE=HIDDEN NAME=\"mode\" VALUE=\"complex\">\\n' +\n"
"'<INPUT TYPE=HIDDEN NAME=\"type\" VALUE=\"' + sp[1].objtype + '\">\\n' +\n"
"'<INPUT TYPE=HIDDEN NAME=\"attr\" VALUE=\"' + sp[1].salist[1].name + '\">\\n' +\n"
"'<INPUT TYPE=HIDDEN NAME=\"match\" VALUE=\"' + sp[1].salist[1].matchtypes[1] + '\">\\n' +\n"
"'<INPUT TYPE=HIDDEN NAME=\"base\" VALUE=\"%s\">\\n' +\n"
"'<INPUT TYPE=HIDDEN NAME=\"ldapserver\" VALUE=\"%s\">\\n' +\n"
"'<INPUT TYPE=HIDDEN NAME=\"ldapport\" VALUE=\"%d\">\\n' +\n"
"'</FORM>\\n' +\n"
"'</BODY>';\n"
"\n"
"\n"
"\n"
"\n"
"function createSearchTypeFrame()\n"
"{\n"
"    searchTypeHTML = '<BODY>\\n<FORM NAME=SearchTypeForm>\\n<TABLE>\\n<TR>\\n' +\n"
"	    '<TH ALIGN=RIGHT>\\nFind:\\n</TH>\\n<TD>\\n<SELECT NAME=\"searchType\" ' +\n"
"	    'onChange=\"parent.createSearchAttrFrame(' +\n"
"	    'this.options.selectedIndex)\">\\n';\n"
"\n"
"    for (var i = 1; i <= sp.count; i++) {\n"
"	searchTypeHTML += '<OPTION VALUE=\"' + sp[i].objtype + '\">' +\n"
"		sp[i].objtype + '\\n';\n"
"    }\n"
"\n"
"    searchTypeHTML += '</SELECT>\\n</TD>\\n</TR>\\n</TABLE>\\n</FORM>\\n</BODY>\\n';\n"
"    self.searchTypeFrame.location.href = \"javascript:parent.searchTypeHTML\";\n"
"    parent.selectedObj = 1;\n"
"}\n"
"\n"
"\n"
"\n"
"\n"
"function createSearchAttrFrame(soindex)\n"
"{\n"
"    var ind = soindex + 1;\n"
"    parent.selectedObj = ind;\n"
"    parent.selectedAttr = 1;\n"
"    searchAttrHTML = '<BODY>\\n<FORM NAME=SearchAttrForm>\\n<TABLE>\\n<TR>\\n' +\n"
"	    '<TH ALIGN=RIGHT>\\nwhere<br>the:\\n</TH>\\n<TD>\\n' +\n"
"    	    '<SELECT NAME=searchAttr ' +\n"
"	    'onChange=\"parent.createSearchMatchFrame' +\n"
"	    '(this.options.selectedIndex)\">\\n';\n"
"    for ( var i = 1; i <= sp[ind].salist.count; i++ ) {\n"
"	searchAttrHTML += \"<OPTION VALUE='\" + sp[ind].salist[i].name\n"
"		+ \"'>\" + sp[ind].salist[i].name + \"\\n\";\n"
"    }\n"
"    searchAttrHTML += '</SELECT>\\n</TD>\\n</TR>\\n</TABLE>\\n</FORM>\\n</BODY>\\n';\n"
"    self.searchAttrFrame.location.href = \"javascript:parent.searchAttrHTML\";\n"
"    parent.createSearchMatchFrame(0);\n"
"}\n"
"\n"
"\n";

	remainder2b = 
"\n"
"\n"
"function updateSearchMatch(smindex)\n"
"{\n"
"    parent.selectedMatch = smindex + 1;\n"
"}\n"
"\n"
"\n"
"\n"
"\n"
"function createSearchMatchFrame(saindex)\n"
"{\n"
"    var ind = saindex + 1;\n"
"    parent.selectedAttr = ind;\n"
"    searchMatchHTML = '<BODY>\\n<FORM NAME=SearchMatchForm>\\n<TABLE>\\n<TR>\\n' +\n"
"	    '<TD>\\n' +\n"
"    	    '<SELECT NAME=searchMatch ' +\n"
"	    'onChange=\"parent.updateSearchMatch' +\n"
"	    '(this.options.selectedIndex)\">\\n';\n"
"    var mt = sp[parent.selectedObj].salist[ind].matchtypes;\n"
"    for (var i = 1; i <= mt.count; i++) {\n"
"	searchMatchHTML += \"<OPTION VALUE='\" + mt[i] + \"'>\" + mt[i] + \"\\n\";\n"
"    }\n"
"    searchMatchHTML += '</SELECT>\\n</TD>\\n</TR>\\n</TABLE>\\n</FORM>\\n</BODY>\\n';\n"
"    parent.selectedMatch = 1;\n"
"    self.searchMatchFrame.location.href = \"javascript:parent.searchMatchHTML\";\n"
"}\n"
"\n"
"\n"
"\n"
"function init()\n"
"{\n"
"    createSearchTypeFrame();\n"
"    createSearchAttrFrame(0);\n"
"    createSearchBaseFrame();\n"
"}\n"
"\n"
"\n"
"// End hiding -->\n"
"</SCRIPT>\n"
"</HEAD>\n"
"<FRAMESET ROWS=110,70,70,* BORDER=2 onLoad=\"init()\">\n"
"    <FRAME SRC=\"/dshtml/csearchtitle.html\" NAME=\"searchTitleFrame\" SCROLLING=\"NO\">\n"
"    <FRAMESET COLS=\"35%,35%,30%\" BORDER=2 >\n"
"	<FRAME SRC=\"javascript:parent.emptyFrame\" NAME=\"searchTypeFrame\"\n"
"	    NORESIZE SCROLLING=\"NO\">\n"
"	<FRAME SRC=\"javascript:parent.emptyFrame\" NAME=\"searchAttrFrame\"\n"
"	    NORESIZE SCROLLING=\"NO\">\n"
"	<FRAME SRC=\"javascript:parent.emptyFrame\" NAME=\"searchMatchFrame\"\n"
"	    NORESIZE SCROLLING=\"NO\">\n"
"    </FRAMESET>\n"
"    <FRAMESET COLS=\"55%,45%\" BORDER=2>\n"
"       <FRAME SRC=\"javascript:parent.sshtml\" NAME=\"searchStringFrame\"\n"
"           NORESIZE SCROLLING=\"NO\">\n"
"       <FRAME SRC=\"javascript:parent.emptyFrame\" NAME=\"searchBaseFrame\"\n"
"           NORESIZE SCROLLING=\"NO\">\n"
"    </FRAMESET>\n"
"    <FRAME SRC=\"javascript:parent.emptyFrame\" NAME=\"outputFrame\">\n"
"</FRAMESET>\n"
"</HTML>\n";

}
