/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#include "dsgw.h"

#ifdef DSGW_DEBUG
int main(int argc, char *argv[], char *env[] )
#else /* DSGW_DEBUG */
int main(int argc, char *argv[] )
#endif /* DSGW_DEBUG */
{
    char	*p, *tmplname, *attrname, *attrdesc, *qs, *dn, *edn;
    char	*attrs[ 2 ], **attrvals, **xdn, *avedn, *js0, *js1;
    LDAP	*ld;
    LDAPMessage	*msgp;
    int		i;

    (void)dsgw_init( argc, argv, DSGW_METHOD_GET );
    dsgw_send_header();

#ifdef DSGW_DEBUG
   dsgw_logstringarray( "env", env );
#endif

    /*
     * The URL used to invoke this CGI looks like:
     *  http://host/dnedit/DN?TEMPLATE=tmplname&ATTR=attrname&DESC=description
     *
     * where:
     *   "tmplname" is the name of the HTML template to render
     *   "attrname" is the name of a dn-valued attribute to display
     *   "description" is a textual description of the attribute
     */

    if (( dn = getenv( "PATH_INFO" )) == NULL ) {
	dsgw_error( DSGW_ERR_MISSINGINPUT, NULL, DSGW_ERROPT_EXIT, 0 );
    }

    if ( *dn == '/' ) {
	++dn;   /* skip past leading slash */
    }
    edn = dsgw_strdup_escaped( dn );

    if (( qs = getenv( "QUERY_STRING" )) == NULL ) {
	dsgw_error( DSGW_ERR_MISSINGINPUT, NULL, DSGW_ERROPT_EXIT, 0 );
    }
    
    /* Get attribute description */
    if (( p = strrchr( qs, '&' )) == NULL ) {
	dsgw_error( DSGW_ERR_MISSINGINPUT, "desc" , DSGW_ERROPT_EXIT, 0 );
    }
    *p++ = '\0';
    if ( strncasecmp( p, "desc=", 5 )) {
	dsgw_error( DSGW_ERR_MISSINGINPUT, "desc", DSGW_ERROPT_EXIT, 0 );
    }
    attrdesc = dsgw_ch_strdup( p + 5 );
    
    /* Get attribute name */
    if (( p = strrchr( qs, '&' )) == NULL ) {
	dsgw_error( DSGW_ERR_MISSINGINPUT, "attr" , DSGW_ERROPT_EXIT, 0 );
    }
    *p++ = '\0';
    if ( strncasecmp( p, "attr=", 5 )) {
	dsgw_error( DSGW_ERR_MISSINGINPUT, "attr", DSGW_ERROPT_EXIT, 0 );
    }
    attrname = dsgw_ch_strdup( p + 5 );

    /* Get template name */
    p = qs;
    if ( strncasecmp( p, "template=", 9 )) {
	dsgw_error( DSGW_ERR_MISSINGINPUT, "template", DSGW_ERROPT_EXIT, 0 );
    }
    tmplname = dsgw_ch_strdup( p + 9 );

    /* Get the current attribute values */
    (void) dsgw_init_ldap( &ld, NULL, 0 );
    attrs[ 0 ] = attrname;
    attrs[ 1 ] = NULL;
    if (ldap_search_s( ld, dn, LDAP_SCOPE_BASE, "(objectclass=*)", attrs, 0,
	    &msgp ) != LDAP_SUCCESS ) {
	dsgw_error( DSGW_ERR_ENTRY_NOT_FOUND, dn, DSGW_ERROPT_EXIT, 0 );
    }
    attrvals = ldap_get_values( ld, msgp, attrname );
	

    /* Send the top-level document HTML */
    fputs(  "<HTML>\n"
	"<SCRIPT LANGUAGE=\"JavaScript\">\n", stdout );
    fprintf( stdout, "var attrname = '%s';\n", attrname );
    fprintf( stdout, "var dn = '%s';\n", edn );
    fprintf( stdout, "var needToSaveChanges = false;\n", edn );
    fprintf( stdout, "var completion_url = '%s/%s';\n", 
	    dsgw_getvp( DSGW_CGINUM_EDIT ), edn );
    fprintf( stdout,
    /*
     * This fprintf needs to output \\\' so that when JavaScript writeln's
     * this string, it writes \' to the output document.
     *
     * I'm really, really sorry about this - ggood.
     * 
     * Moral of the story - next time someone asks you to write C code which
     * writes JavaScript code which writes JavaScript code... just say "no".
     */
    "var comp_js = 'var cu=\\\\\\\'%s/%s\\\\\\\'; this.document.location.href=cu;'\n",
	dsgw_getvp( DSGW_CGINUM_EDIT ), edn ); 
    fputs(
	"var dnlist = new Array;\n", stdout );
    for ( i = 0; attrvals && attrvals[ i ] != NULL; i++ ) {
	xdn = ldap_explode_dn( attrvals[ i ], 1 );
	avedn = dsgw_strdup_escaped( attrvals[ i ]);
	printf( "dnlist[%d] = new Object\n", i );
	printf( "dnlist[%d].edn = '%s';\n", i, avedn );
	js0 = dsgw_escape_quotes( xdn[ 0 ] );
	if ( xdn[1] != NULL ) {
	    js1 = dsgw_escape_quotes( xdn[ 1 ] );
	    printf( "dnlist[%d].rdn = '%s, %s';\n", i, js0, js1 );
	    free( js1 );
	} else {
	    printf( "dnlist[%d].rdn = '%s';\n", i, js0 );
	}
	free( js0 );
	printf( "dnlist[%d].selected = false;\n", i );
	free( avedn );
	ldap_value_free( xdn );
    }
    printf( "dnlist.count = %d;\n", i );
    fputs( 
	"var changesMade = 0;\n"
	"\n"

	/*
	 * JavaScript function processSearch
	 */

	"function processSearch(f)\n"
	"{\n"
	"    var sel = f.type;\n"
	"    var selvalue = sel.options[sel.selectedIndex].value;\n"
	"    var lt = f.listtemplate;\n"
	"    lt.value = 'fa-' + selvalue;\n", stdout );
    fprintf( stdout,
	"    f.action = '%s';\n",
	     dsgw_getvp( DSGW_CGINUM_DOSEARCH ));
    fputs( "\n"
	"    f.searchstring.select();\n"
	"    f.searchstring.focus();\n"
	"    return true;\n"
	"}\n"
	"\n"

	/*
	 * JavaScript function removeItem
	 */

	"function removeItem(itemno,refresh)\n"
	"{\n"
	"    var extantDNs = dnlist;\n"
	"    var extantDNsCount = dnlist.count;\n"
	" \n"
	"    // Get rid of element in slot dup\n"
	"    for (k = itemno; k < extantDNsCount - 1; k++) {\n"
	"	extantDNs[k] = extantDNs[k+1];\n"
	"    }\n"
	"    dnlist.count--;\n"
	"    if ( refresh ) genOutputFrame(outputFrame, dnlist);\n"
	"    this.changesMade = 1;\n"
	"}\n"
	"\n"

	/*
	 * JavaScript function dnarrcomp
	 */

	"function dnarrcomp(a,b)\n"
	"{\n"
	"    return(a.edn.toLowerCase() > b.edn.toLowerCase());\n"
	"}\n"
	" \n"
#ifdef NAV30_SORT_NO_LONGER_COREDUMPS
	/*
	 * JavaScript function sortEntries
	 */

	"function sortEntries()\n"
	"{\n"
	"    var extantDNs = dnlist;\n"
	"    extantDNs.sort(dnarrcomp);\n"
	"    genOutputFrame(outputFrame, dnlist);\n"
	"}\n"
	"\n"
#endif /* NAV30_SORT_NO_LONGER_COREDUMPS */

	/*
	 * JavaScript function genOutputFrame
	 */

	"function genOutputFrame(oframe, dnl)\n"
	"{\n"
	"    var d = oframe.document;\n"
	"\n"
	"    d.open('text/html');\n"
	"    d.writeln('<HTML>');\n", stdout );

    fprintf( stdout, "    d.writeln('<BODY %s>');\n", dsgw_html_body_colors );

    fputs( 
	"    d.writeln('<FORM>');\n"
	"    d.writeln('<CENTER>');\n"
	"    d.write('There ');\n"
	/* I'll burn in L10N Hell for this. */
	"    if (dnl.count == 0) {\n"
	"	d.write('are <B>no</B> names ');\n"
	"    } else if (dnl.count == 1) {\n"
	"	d.write('is <B>1</B> name ');\n"
	"    } else {\n"
	"	d.write('are <B>' + dnl.count + '</B> names ');\n"
	"    }\n"
#ifdef NAV30_SORT_NO_LONGER_COREDUMPS
	"    d.writeln('in the list.</FONT>\\n')\n"
	"    d.writeln('<INPUT TYPE=\"button\" VALUE=\"Sort\" onClick=\"parent.sortEntries();\"></CENTER>\\n');\n"
#else
	"    d.writeln('in the list.</FONT></CENTER>\\n')\n"
#endif
	"    if (dnl.count > 0) {\n"
	"	d.write('<PRE><B>');\n"
	"	d.writeln('Remove');\n"
	"	d.write('from\\nlist?        Name</B><HR>');\n"
	"	for (i = 0; i < dnl.count; i++) {\n"
	"	    d.write('<INPUT TYPE=CHECKBOX onClick=\"parent."
	"removeItem(' + i + ', true);\">');\n"
	"	    d.write('           ');\n"
	"	    d.write(dnl[i].rdn + '\\n');\n"
	"	}\n"
	"	d.writeln('</FORM><HR>');\n"
	"    }\n"
	"    d.writeln('</BODY>');\n"
	"    d.close();\n"
	"}\n"
	"\n"

	/*
	 * JavaScript function mergeLists
	 */

	"function mergeLists(mode, old, newl)\n"
	"{\n"
	"    var dup = -1;\n"
	"    var i, j, k;\n"
	" \n"
	"    for (i = 0; i < newl.count; i++) {\n"
	"	// Check for a duplicate\n"
	"	for (j = 0; j < old.count; j++) {\n"
	"	    dup = -1;\n"
	"	    if (newl[i].edn.toLowerCase() == "
	"old[j].edn.toLowerCase()) {\n"
	"		// Duplicate - skip\n"
	"		dup = j;\n"
	"		break;\n"
	"	    }\n"
	"	}\n"
	"	if ((dup == -1) && (mode == \"add\")) {\n"
	"	    // add new dn at end of array\n"
	"	    old[old.count] = new Array;\n"
	"	    old[old.count].edn = newl[i].edn;\n"
	"	    old[old.count].rdn = newl[i].rdn;\n"
	"	    old[old.count].sn = newl[i].sn;\n"
	"	    old[old.count].selected = false;\n"
	"	    old.count++;\n"
	"	} else if (dup != -1 && mode == \"remove\") {\n"
	"           removeItem(dup,false);\n"
	"	}\n"
	"    }\n"
	"}\n"
	"\n"

	/*
	 * JavaScript function updateList
	 */

	"function updateList(mode, old_list, new_list, outframe)\n"
	"{\n"
	"    mergeLists(mode, old_list, new_list);\n"
	"    genOutputFrame(outframe, old_list);\n"
	"    this.changesMade = 1;\n"
	"}\n"
	"\n"

	/*
	 * JavaScript function saveChanges
	 */

	"function saveChanges()\n"
	"{\n"
	"    var i, j;\n"
	"    needToSaveChanges = true;\n"
	"    of = self.stagingFrame.document;\n"
	"    of.open('text/html');\n", stdout );
    fprintf( stdout,
	"    of.write('<BODY onLoad=\"if ( parent.needToSaveChanges ) { parent.needToSaveChanges = false; document.stagingForm.submit() }\">');\n" );
    fprintf( stdout,
	"    of.write('<FORM NAME=\"stagingForm\" ACTION=\"%s\" ');\n",
	dsgw_getvp( DSGW_CGINUM_DOMODIFY ));
    fputs(
	"    of.write('METHOD=\"POST\" TARGET=\"_parent\">\\n ');\n"
	"    if (self.dnlist.count < 1) {\n"
	"	of.write('<INPUT TYPE=\"hidden\" NAME=\"replace_');\n"
	"	of.write(self.attrname);\n"
	"	of.write('\" VALUE=\"\">\\n');\n"
	"    } else {\n"
	"	for (j = 0; j < self.dnlist.count; j++) {\n"
	"	    of.write('<INPUT TYPE=\"hidden\" NAME=\"replace_');\n"
	"	    of.write(self.attrname);\n"
	"	    of.write('\" VALUE=\"');\n"
	"	    of.write(unescape(self.dnlist[j].edn));\n"
	"	    of.write('\">\\n');\n"
	"	}\n"
	"    }\n"
	"    of.writeln('<INPUT TYPE=\"hidden\" NAME=\"changetype\" "
	"VALUE=\"modify\">\\n');\n"
	"    of.writeln('<INPUT TYPE=\"hidden\" NAME=\"quiet\" "
	"VALUE=\"true\">\\n');\n"
	"    of.writeln('<INPUT TYPE=\"hidden\" NAME=\"completion_javascript\" "
	"VALUE=\"' + comp_js + '\">');\n"
	"    of.writeln('<INPUT TYPE=\"hidden\" NAME=\"dn\" VALUE=\"' "
	"+ self.dn + '\"\\n');\n"
	"    of.writeln('</FORM>\\n');\n"
	"    of.close();\n"
	"}\n"

	"</SCRIPT>\n"
	"\n"
	"<FRAMESET BORDER=2 FRAMEBORDER=YES ROWS=35%,65%,0,0 "
	"SCROLLING=\"NO\" NORESIZE onLoad=\"genOutputFrame"
	"(this.outputFrame, this.dnlist);\">\n",
	stdout );
    fprintf( stdout, "   <FRAME SRC=\"%s/%s?%s&DNATTR=%s&"
	     "DNDESC=%s\" NAME=\"controlFrame\" SCROLLING=\"yes\">\n",
	     dsgw_getvp( DSGW_CGINUM_EDIT ), edn, tmplname, attrname,
	     attrdesc );
    fputs( "   <FRAME SRC=\"javascript:parent.emptyFrame\" "
	   "NAME=\"outputFrame\">\n"
	    "   <FRAME SRC=\"javascript:parent.emptyFrame\" "
	    "NAME=\"stagingFrame\">\n"
	   "</FRAMESET>\n"
	   "</HTML>\n", stdout );
}
