/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#include "dsgw.h"

static void get_request();
static void post_request();
static void do_autherror( int rc, char *msg, int ommitclosebutton );


main( argc, argv, env )
    int		argc;
    char	*argv[];
{
    int		reqmethod;

    reqmethod = dsgw_init( argc, argv,  DSGW_METHOD_POST );

    post_request();

    exit( 0 );
}


static void
post_request()
{
    char	*binddn, *password, *authdesturl, *ufn;
    LDAP	*ld;
    int		rc;

    binddn = dsgw_get_cgi_var( "binddn", DSGW_CGIVAR_REQUIRED );
    authdesturl = dsgw_get_cgi_var( "authdesturl", DSGW_CGIVAR_OPTIONAL );
    password = dsgw_get_cgi_var( "password", DSGW_CGIVAR_OPTIONAL );

    (void) dsgw_init_ldap( &ld, NULL, 1 );

    if ( password == NULL || strlen( password ) == 0 ) {
	do_autherror( 0, "you did not provide a password. "
		"Whenever you authenticate, you must provide a password "
		"so that the server can verify your identity.",
		authdesturl == NULL );
	exit( 0 );
    }
    rc = ldap_simple_bind_s( ld, binddn, password );
    if ( rc != LDAP_SUCCESS ) {
	do_autherror( rc, NULL, authdesturl == NULL );
	exit( 0 );
    } else {

	/* Construct cookie */
	char *ckbuf;
	char *locbuf;

	if (( ckbuf = dsgw_mkcookie( binddn, password, gc->gc_authlifetime,
		&rc )) == NULL ) {
	    switch ( rc ) {
	    case DSGW_CKDB_CANTOPEN:
		do_autherror( 0,
		    "the server was unable to generate authentication "
		    "credentials.  The authentication database could "
		    "not be opened.", authdesturl == NULL );
		break;
	    case DSGW_CKDB_CANTAPPEND:
		do_autherror( 0,
		    "the server was unable to generate authentication "
		    "credentials. Data could not be appended to "
		    "the authentication database.", authdesturl == NULL );
		break;
	    }
	    exit( 1 );
	}
	dsgw_add_header( ckbuf );

	/* Construct a success message */
	dsgw_send_header();
	fputs( "<HTML>\n"
	    "<HEAD>\n" 
	    "<TITLE>Authentication Successful</TITLE>\n"
	    "<SCRIPT LANGUAGE=\"JavaScript\">\n"
	    "<!-- Hide from non-JavaScript browsers\n", stdout );
	if ( authdesturl != NULL && strlen( authdesturl ) > 0 ) {
	    printf( "var authdesturl=\"%s\";\n", authdesturl );
	} else {
	    printf( "var authdesturl=null;\n" );
	}
	fputs( "function finishAuth()\n"
	    "{\n"
	    "    if ( authdesturl != null ) {\n"
	    "        top.location.href = authdesturl;\n"
	    "    } else {\n"
	    "        top.close();\n"
	    "    }\n"
	    "}\n"
	    "var contButtons = '<FORM NAME=\"bForm\">"
	    "<TABLE BORDER=2 WIDTH=100%%>"
	    "<TD ALIGN=CENTER WIDTH=50%%>"
	    "<INPUT TYPE=BUTTON NAME=\"contButton\""
	    "VALUE=\"Continue\" onClick=\"finishAuth();\">"
	    "<TD ALIGN=CENTER WIDTH=50%%>", stdout );

	dsgw_emit_helpbutton_qe( "AUTHSUCCESS" );

	fputs(
	    "</TABLE>"
	    "</FORM>'\n"
	    "var noContButtons = 'To continue, select a task from the list above."
	    "<FORM NAME=\"bForm\">"
	    "<TABLE BORDER=2 WIDTH=100%%>"
	    "<TD ALIGN=CENTER WIDTH=50%%>", stdout );
	dsgw_emit_homebutton_qe();

	fputs( "<TD ALIGN=CENTER WIDTH=50%%>", stdout );
	dsgw_emit_helpbutton_qe( "AUTHSUCCESS" );

	
	fputs(
	    "</TABLE>"
	    "</FORM>'\n"
	    "// End hiding -->\n"
	    "</SCRIPT>\n"
	    "</HEAD>\n<BODY>\n"
	    "<CENTER>\n"
	    "<H3>Authentication Successful</H3>\n"
	    "</CENTER>\n"
	    "You are now authenticated to the directory as ", stdout );
	ufn = ldap_dn2ufn( binddn );
	printf( "<B>%s</B>.<P>\n", ufn );
	free( ufn );
	printf( "Your authentication credentials will expire in %d minutes. \n",
		gc->gc_authlifetime / 60 );
	fputs( "After your credentials expire, you will need to \n"
	    "re-authenticate to the directory.\n"
	    "<P>\n"
	    "<TR>\n"
	    "<SCRIPT LANGUAGE=\"JavaScript\">\n"
	    "<!-- Hide from non-JavaScript browsers\n"
	    "if ( authdesturl != null ) {\n"
	    "    document.write( contButtons );\n"
	    "} else {\n"
	    "    document.write( noContButtons );\n"
	    "}\n"
	    "// End hiding -->\n"
	    "</SCRIPT>\n"
	    "</BODY>\n</HTML>\n", stdout );

	free( ckbuf );
	exit( 0 );
    }
}




static void
do_autherror( int rc, char *msg, int omitclosebutton )
{
    dsgw_send_header();
    fputs( "<HTML><HEAD><TITLE>Authentication Error</TITLE></HEAD>\n"
	"<BODY>\n"
	"<CENTER>\n"
	"<FONT SIZE=+2>\n"
	"Authentication Failed\n"
	"</FONT>\n"
	"</CENTER>\n"
	"<P>\n"
	"Authentication failed because ", stdout );
    if ( msg != NULL ) {
	puts( msg );
    } else {
	switch ( rc ) {
	case LDAP_NO_SUCH_OBJECT:
	    fputs( "the entry you attempted to authenticate as does\n"
		"not exist in the directory.\n"
		"You may only authenticate as an existing directory\n"
		"entry.\n" , stdout );
	    break;
	case LDAP_INAPPROPRIATE_AUTH:
	    fputs( "the entry you attempted to authenticate as does\n"
		"not have a password.  Before you can authenticate\n"
		"as this entry, a password must be set by a\n"
		"directory administrator\n", stdout );
	    break;
	case LDAP_INVALID_CREDENTIALS:
	    fputs( "the password you supplied is incorrect.  Please\n"
		"click the Retry button and try again.  If you have\n"
		"forgotten the password for this entry, a directory\n"
		"administrator must reset the password for you.\n",
		stdout );
	    break;
	default:
	    printf( "of an unexpected error: %s\n", ldap_err2string( rc ));
	    break;
	}
    }
    fputs( "<P>\n"
	"<FORM>\n"
	"<TABLE BORDER=2 WIDTH=100%%>\n"
	"<TR>\n", stdout );
    if ( omitclosebutton ) {
	fputs( "<TD ALIGN=CENTER WIDTH=33%%>\n"
	    "<INPUT TYPE=BUTTON VALUE=\"Retry\" "
	    "onClick=\"history.back()\">\n"
	    "<TD ALIGN=CENTER WIDTH=33%%>\n", stdout );
	dsgw_emit_homebutton();
	fputs ( "<TD ALIGN=CENTER WIDTH=34%%>\n", stdout );
	dsgw_emit_helpbutton( "AUTHPROBLEM" );
    } else {
	fputs( "<TD ALIGN=CENTER WIDTH=33%%>\n"
	    "<INPUT TYPE=BUTTON VALUE=\"Retry\" "
	    "onClick=\"history.back()\">\n"
	    "<TD ALIGN=CENTER WIDTH=33%%>\n"
	    "<INPUT TYPE=BUTTON VALUE=\"Close Window\" "
	    "onClick=\"parent.close();\">\n"
	    "<TD ALIGN=CENTER WIDTH=34%%>\n", stdout );
	dsgw_emit_helpbutton( "AUTHPROBLEM" );
    }
    fputs( "</TABLE>\n"
	"</FORM>\n"
	"</BODY></HTML>\n", stdout );
    fflush( stdout );
    return;
}
