/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#include "dsgw.h"
#include "dbtdsgw.h"

#define DSGW_CHANGETYPE_UNKNOWN	0
#define DSGW_CHANGETYPE_MODIFY	1
#define DSGW_CHANGETYPE_ADD	2
#define DSGW_CHANGETYPE_DELETE	3
#define DSGW_CHANGETYPE_MODRDN	4

static void post_request();
static int entry_modify_or_add( LDAP *ld, char *dn, int add, int *pwdchangedp );
static int entry_delete( LDAP *ld, char *dn, char *entry_name, int *gobackp );
static int entry_modrdn( LDAP *ld, char *dn, char *newrdn, int deleteoldrdn,
			 char *entry_name, int *gobackp );
static int entry_cleanup(LDAP *ld, char *deldn, char *newdn, 
			 int *cleanup_lderr);
static int gather_passwd_changes( char *dn, LDAPMod ***pmodsp,
	int adding_entry, int *pwdchangedp );
static void modify_error( int lderr );
static void addmodifyop( LDAPMod ***pmodsp, int modop, char *attr,
	char *subtype, char *value, int vlen );
static void remove_modifyops( LDAPMod **pmods, char *attr );
static int starts_with( char *s, char *startswith );
static char **post2multilinevals( char *postedval );
static char **post2vals( char *postedval );
static int require_oldpasswd( char *modifydn );
static char *dsgw_processdomainid( LDAP *ld, char *dn, char *attr, char *val, int len);
static int attr_value_is_unique( LDAP *ld, char *dn, char *attr, char *real_attr, char *value );
static int	verbose = 0;
static int	quiet = 0;
static int	display_results_inline = 0;

typedef struct dsgwUniqueAttrs {
  char **AttrNames;
  char **AttrValues;
} dsgwUniqueAttrs;


dsgwUniqueAttrs *GetUniqueAttrs (char *vals);
static int CheckUniqueAttrs (LDAP *ld, char *dn, dsgwUniqueAttrs **UniqueAttrsList,
			     char *attr, char *val);
static void FreeUniqueAttrsList(dsgwUniqueAttrs **UniqueAttrsList);

main( argc, argv, env )
    int		argc;
    char	*argv[];
#ifdef DSGW_DEBUG
    char	*env[];
#endif
{
    (void)dsgw_init( argc, argv,  DSGW_METHOD_POST );
    dsgw_send_header();

#ifdef DSGW_DEBUG
   dsgw_logstringarray( "env", env ); 
#endif

    post_request();

    exit( 0 );
}

/* Returns an LDAP handle. Return NULL if failed to initialize SSL 
LDAP *getLdapConnection(char *admroot, AdmldapInfo ldapInfo,
                                char *ldapHost, int ldapPort) {

        int rv;

        if ((!ldapHost) || (ldapPort < 1))
                return NULL;

        if (admldapGetSecurity(ldapInfo)) {
                rv = ADMSSL_Init(ldapInfo, admroot);
                if (rv) {
                        fprintf(stdout, "ERROR: Failed to perform security initialization for communication with LDAP in SSL mode (error code = %d)", rv);
                        return (LDAP *)0;
                }
                return ldapssl_init(ldapHost, ldapPort, 1);
        }
        else
                return ldap_init(ldapHost, ldapPort);
}
*/

static void
post_request()
{
    LDAP	*ld;
    int		rc, changetype, dnlen, i, passwd_changed, discard_authcreds;
    int 	rv/*, ldapPort*/;
    int         displaygoback = 1;
    char	*s, *encodeddn, *dn, *newrdn, *changedesc, **rdns, **oldrdns,
		*jscomp, *entry_name, *new_name, *success_msg;
    char	buf[ 256 ];
    char  	*nsroot = getenv("NETSITE_ROOT");
/*    char	*ldapHost;
    AdmldapInfo ldapInfo;  */
#if 0
    FILE	*genfp;
#endif


    passwd_changed = discard_authcreds = 0;
    s = dsgw_get_cgi_var( "changetype", DSGW_CGIVAR_REQUIRED );
    changedesc = XP_GetClientStr( DBT_editing );

    if ( strcasecmp( s, "modify" ) == 0 ) {
	changetype = DSGW_CHANGETYPE_MODIFY;
    } else if ( strcasecmp( s, "add" ) == 0 ) {
	changetype = DSGW_CHANGETYPE_ADD;
	changedesc = XP_GetClientStr( DBT_adding );
    } else if ( strcasecmp( s, "delete" ) == 0 ) {
	changetype = DSGW_CHANGETYPE_DELETE;
	changedesc = XP_GetClientStr( DBT_deleting );
    } else if ( strcasecmp( s, "modrdn" ) == 0 ) {
	changetype = DSGW_CHANGETYPE_MODRDN;
	changedesc = XP_GetClientStr( DBT_renaming );
    } else {
	changetype = DSGW_CHANGETYPE_UNKNOWN;
    }

    if (( encodeddn = dsgw_get_cgi_var( "dn", DSGW_CGIVAR_OPTIONAL ))
	    != NULL ) {
	/* undo extra level of escaping on DN */
	dn = dsgw_ch_strdup( encodeddn );
	dsgw_form_unescape( dn );
    } else {
	dn = NULL;
    }

    /*
     * if a DN prefix variable is specified, form the DN as follows:
     *    $dnprefixtag '=' $$dnprefixvar ',' $dn
     * if DN is of zero length, we omit the last two pieces.
     */
    if (( s = dsgw_get_cgi_var( "dnprefixvar", DSGW_CGIVAR_OPTIONAL )) !=
	    NULL ) {
	char	*dnprefix, *dntag, *tmpdn;

	dnprefix = dsgw_get_cgi_var( s, DSGW_CGIVAR_REQUIRED );
	dntag = dsgw_get_cgi_var( "dnprefixtag", DSGW_CGIVAR_REQUIRED );
	if ( dn == NULL || *dn == '\0' ) {
	    tmpdn = dsgw_ch_malloc( strlen( dntag ) + strlen( dnprefix ) + 2 );
	    sprintf( tmpdn, "%s=%s", dntag, dnprefix );
	} else {
	    tmpdn = dsgw_ch_malloc( strlen( dntag ) + strlen( dnprefix ) +
		    strlen( dn ) + 3 );
	    sprintf( tmpdn, "%s=%s,%s", dntag, dnprefix, dn );
	}
	dn = tmpdn;
	if ( encodeddn != NULL ) {
	    free( encodeddn );
	}
	encodeddn = dsgw_ch_malloc( 3 * strlen( dn ) + 1 );	/* worst case */
	*encodeddn = '\0';
	dsgw_strcat_escaped( encodeddn, dn );
    }

    if ( dn == NULL || encodeddn == NULL ) {
        sprintf( buf, XP_GetClientStr(DBT_missingFormDataElement100s_), "dn" );
        dsgw_error( DSGW_ERR_BADFORMDATA, buf, DSGW_ERROPT_EXIT, 0 );
    }

    quiet = dsgw_get_boolean_var( "quiet", DSGW_CGIVAR_OPTIONAL, 0 );

#if 0
    /*
     * If the "genscreen" form variable is set, it is the name of a
     * genscreen-compatible HTML template to display the domodify results
     * within.  We replace the "DS_LAST_OP_INFO" directive with our own
     * "domodify" output.  Presence of "genscreen" also turns on quiet mode.
     */
    if (( s = dsgw_get_cgi_var( "genscreen", DSGW_CGIVAR_OPTIONAL )) != NULL &&
	    dsgw_genscreen_begin( s, &genfp, DRCT_DS_LAST_OP_INFO, 0 ) == 0 ) {
	quiet = display_results_inline = 1;
    }
#endif

    verbose = dsgw_get_boolean_var( "verbose", DSGW_CGIVAR_OPTIONAL, 0 );
    if ( verbose ) {
	quiet = 0;	/* verbose overrides quiet */
    }
      sprintf(buf, "%s%c%s%c%s", nsroot, FILE_PATHSEP, "admin-serv",
              FILE_PATHSEP, "config");

/*
      ldapInfo = admldapBuildInfo(buf, &rv);
      if (!ldapInfo) {
        fprintf(stdout, "Status: 1\nErrorString: NO LDAP INFO\n");
        exit(0);
      }
      else {
        ldapHost = admldapGetHost(ldapInfo);
        ldapPort = admldapGetPort(ldapInfo);
      }

    if ((ld = getLdapConnection(buf, ldapInfo,  ldapHost, ldapPort )) == NULL ) {
	fprintf( stdout, XP_GetClientStr(DBT_warningNoAuthenticationContinuin_) );
    }
    if ( (rv = ldap_simple_bind_s( ld, gc->gc_binddn, gc->gc_bindpw)) != LDAP_SUCCESS ) {
        dsgw_ldap_error( ld, DSGW_ERROPT_EXIT );
    }
*/
/*  atulb:  Need to call dsgw_init_ldap() - otherwise it cribs of invalid credentials! */
   if ( dsgw_init_ldap( &ld, NULL, 0 ) != DSGW_BOUND_ASUSER ) {
        fprintf( stdout, XP_GetClientStr(DBT_warningNoAuthenticationContinuin_) );
    }

    if ( !quiet ) {
	sprintf( buf, XP_GetClientStr(DBT_SDirectoryEntry_), changedesc );
	dsgw_html_begin( buf, 1 );
    } else {
	dsgw_html_begin( NULL, 0 );
    }

    fputs( "\n<FONT SIZE=+1>\n", stdout );

    rdns = ldap_explode_dn( dn, 1 );
    if ( rdns == NULL || rdns[ 0 ] == NULL ) {
	entry_name = dn;
    } else {
	entry_name = dsgw_ch_strdup( rdns[ 0 ] );
    }
    new_name = success_msg = "";
    fprintf( stdout, "%s <B>%s</B>...\n</FONT>\n\n", changedesc, entry_name );
    if ( rdns != NULL ) {
	ldap_value_free( rdns );
    }

    if ( verbose ) {
	fprintf( stdout, XP_GetClientStr(DBT_PreEntryDnSPrePN_), dn );
    }

    /*
     * For end-user CGIs under admin server, if we're talking to a local DB,
     * then there's no access control, and therefore we need to disallow
     * people from changing entries other than their own.  Do that check right
     * here.
     */
    if ( gc->gc_enduser ) {
	char *bdn;
	(void)dsgw_get_adm_identity( ld, NULL, &bdn, NULL, DSGW_ERROPT_EXIT );
	/* Make sure DN we're bound as matches the DN being modified */
	if ( dsgw_dn_cmp( dn, bdn ) == 0 ) {
	    /* Not the same - generate an error and bail out */
	    dsgw_error( DSGW_ERR_LOCALDB_PERMISSION_DENIED, NULL,
		    DSGW_ERROPT_EXIT, 0 );
	}
    }

    switch( changetype ) {
    case DSGW_CHANGETYPE_MODIFY:
    case DSGW_CHANGETYPE_ADD:
	rc = entry_modify_or_add( ld, dn, changetype == DSGW_CHANGETYPE_ADD,
		&passwd_changed );
	if ( changetype == DSGW_CHANGETYPE_MODIFY ) {
	    success_msg = XP_GetClientStr(DBT_changesToBSBHaveBeenSaved_);
	} else {
	    success_msg = XP_GetClientStr(DBT_BSBHasBeenAdded_);
	}
	break;
    case DSGW_CHANGETYPE_DELETE:
	rc = entry_delete( ld, dn, entry_name, &displaygoback );
	success_msg = XP_GetClientStr(DBT_BSBHasBeenDeleted_);
	break;
    case DSGW_CHANGETYPE_MODRDN:
	newrdn = dsgw_get_cgi_var( "newrdn", DSGW_CGIVAR_REQUIRED );
	dsgw_remove_leading_and_trailing_spaces( &newrdn );
	rc = entry_modrdn( ld, dn, newrdn, dsgw_get_boolean_var( "deleteoldrdn", DSGW_CGIVAR_OPTIONAL, 0), entry_name, &displaygoback );

	if ( rc == LDAP_SUCCESS ) {
	
	    /* construct the new DN so we can insert correct "edit" link */
	    if (( oldrdns = ldap_explode_dn( dn, 0 )) == NULL ) {
		dsgw_error( DSGW_ERR_NOMEMORY, NULL, DSGW_ERROPT_EXIT, 0 );
	    }

	    free( encodeddn );
	    dnlen = strlen( newrdn ) + 1;	/* room for "," */
	    for ( i = 1; oldrdns[ i ] != NULL; ++i ) {
		dnlen += ( 1 + strlen( oldrdns[ i ] ));
	    }
	    encodeddn = dsgw_ch_malloc( 3 * dnlen + 1 );	/* worst case */
	    *encodeddn = '\0';
	    dsgw_strcat_escaped( encodeddn, newrdn );
	    for ( i = 1; oldrdns[ i ] != NULL; ++i ) {
		dsgw_strcat_escaped( encodeddn, "," );
		dsgw_strcat_escaped( encodeddn, oldrdns[ i ] );
	    }
	    ldap_value_free( oldrdns );

	    success_msg = XP_GetClientStr(DBT_renamedBSBToBSB_);
	    if (( rdns = ldap_explode_rdn( newrdn, 1 )) == NULL
		    || rdns[ 0 ] == NULL ) {
		new_name = newrdn;
	    } else {
		new_name = rdns[ 0 ];
	    }
	}
	break;

    default:
	rc = LDAP_PARAM_ERROR;
    }

    /*
     * If we are not running under the admin. server AND the operation
     * succeeded and the user is bound as the entry they just changed,
     * AND one of these conditions is true:
     *   1. we changed the password
     *   2. we did a modrdn
     *   3. we deleted the entry
     * then the auth. credentials should be discarded.  If we do discard, we
     * print an informative message for the user.
     */
    if ( !gc->gc_admserv && rc == LDAP_SUCCESS &&
	    ( changetype == DSGW_CHANGETYPE_DELETE ||
	    changetype == DSGW_CHANGETYPE_MODRDN ||
	    ( changetype == DSGW_CHANGETYPE_MODIFY && passwd_changed )) &&
	    dsgw_bound_as_dn( dn, 0 )) {
	char	*authck;

	/* first, remove the cookie from the cookie database (ignore errors) */
	if (( authck = dsgw_get_auth_cookie()) != NULL ) {
	    (void)dsgw_delcookie( authck );
	}

	/* output JavaScript to clear the cookie in the user's browser */
	fputs( "<SCRIPT LANGUAGE=\"JavaScript\">\n", stdout );
	fputs( "<!-- Hide from non-JavaScript browsers\n", stdout );
	fprintf( stdout, "document.cookie = '%s=%s; path=/'\n",
		DSGW_AUTHCKNAME, DSGW_UNAUTHSTR );
	fputs( "// End Hiding -->\n</SCRIPT>\n", stdout );
	fprintf( stdout, XP_GetClientStr(DBT_PBNoteBBecauseYouSTheEntryYouWer_),
		( changetype == DSGW_CHANGETYPE_DELETE ) ? XP_GetClientStr(DBT_deleted_) :
		( changetype == DSGW_CHANGETYPE_MODRDN ) ? XP_GetClientStr(DBT_renamed_) :
		XP_GetClientStr(DBT_changedThePasswordOf_) );
    }

    if ( rc == LDAP_SUCCESS ) {
	/*
	 * admserv case - check for "completion_javascript" form var and
	 * execute it if present.
	 */ 
	jscomp = dsgw_get_cgi_var( "completion_javascript",
		DSGW_CGIVAR_OPTIONAL );
	if ( jscomp != NULL ) {
	    char	*entry_name_js;
	    char	*new_name_js;

	    entry_name_js = dsgw_escape_quotes( entry_name );
	    new_name_js = dsgw_escape_quotes( new_name );
	    fputs( "<SCRIPT LANGUAGE=\"JavaScript\">\n", stdout );
	    fputs( "dsmodify_info = '", stdout );
	    fprintf( stdout, success_msg, entry_name_js, new_name_js );
	    fputs( "';\n", stdout );
	    fprintf( stdout, "dsmodify_dn = '%s';\n",
		    ( changetype == DSGW_CHANGETYPE_DELETE ) ? "":
		    encodeddn );
	    fprintf( stdout, "eval('%s');\n", jscomp );
	    fputs( "</SCRIPT>\n", stdout );
	}
    } else {
	jscomp = NULL;
    }

    if ( jscomp == NULL && ( !gc->gc_admserv || ( rc != LDAP_SUCCESS ))) {
        if(displaygoback){
	    fputs( "<FORM>\n<CENTER><TABLE border=2 width=\"100%\"><TR>\n",
		   stdout );
	    /*
	     * Show framed button.  If the modify succeeded, it is "Close".
	     * If the modify failed, it is "Go Back."
	     */
	    fputs( "<TD WIDTH=\"100%\" ALIGN=\"center\">\n", stdout );
	    if ( rc == LDAP_SUCCESS ) {
	        fprintf( stdout, "<INPUT TYPE=\"button\" VALUE=\"%s\" "
			 "onClick=\"parent.close()\">\n",
			 XP_GetClientStr(DBT_closeWindow_) );
	    } else {
	        fprintf( stdout, "<INPUT TYPE=\"button\" VALUE=\"%s\" "
			 "onClick=\"history.back()\">\n",
			 XP_GetClientStr(DBT_goBack_) );
	    }
	    fputs( "\n</TABLE></CENTER></FORM>\n", stdout );
	}
    }

#if 0
    if ( display_results_inline && genfp != NULL ) {
	fputs( "<HR>\n", stdout );
	dsgw_genscreen_continue( &genfp, NULL, 0 );
    } else if ( !quiet ) {
	dsgw_html_end();
    }
#else
	dsgw_html_end();
#endif

    ldap_unbind( ld );
}


static int
entry_modify_or_add( LDAP *ld, char *dn, int add, int *pwdchangedp )
{
    int		lderr, i, j, opoffset, modop, mls, unique, value_is_escaped;
    int         unchanged_count, unique_attrs_count;
    char	*varname, *varvalue, *retval, *attr, **vals, **unchanged_attrs;
    char	*p;
    LDAPMod	**pmods;
    dsgwUniqueAttrs  *pUniqueAttrs = NULL;
    dsgwUniqueAttrs **UniqueAttrsList = 
      (dsgwUniqueAttrs **) dsgw_ch_malloc (1 * sizeof(dsgwUniqueAttrs *));
    /* I18n variables */
    char        *preferredlanguage;
    char        *prevlanguage;
    char        *langtag, *phontag;
    char        *subtype;

    pmods = NULL;
    unchanged_attrs = NULL;
    unchanged_count = 0;
    
    UniqueAttrsList[0] = NULL;


    /*
     * Gather up password changes (if present in CGI POST)
     */
    if (( lderr = gather_passwd_changes( dn, &pmods, add, pwdchangedp ))
	    != LDAP_SUCCESS ) {
	return( lderr );
    }

    if ( verbose ) {
	fprintf( stdout, "<PRE>\n" );
    }

    /*
     * Build an array of attribute sets we will cross-check for uniqueness.
     * We could just do this in the other dsgw_next_cgi_var() loop below that
     * gathers up attribute value changes, but we'd rather not require that
     * all of the "uniqueAttrs" variables come first, so we do a pre-pass to
     * find all of the "uniqueAttrs" variables.  A little inefficient, but....
     */
    /* I18n: Let me piggyback to get the preferredLanguage info */
    i = 0;
    preferredlanguage = DSGW_DEFAULT_SELECTED_LANG;
    prevlanguage = NULL;
    while (( varname = dsgw_next_cgi_var( &i, &varvalue )) != NULL ) {
	/* 
	 * if the uniqueAttrs variable was set, gather the set of unique
	 * attribues specified in varvalue and place their names and
	 * values in a UniqueAttrs struct.
	 *
	 * We keep an array of UniqueAttrs structs to hold different sets
	 * of UniqueAttrs
	 */
	
	if (strcasecmp (varname, "uniqueAttrs") == 0) {
	  /* 
	   *  Create a new UniqueAttrs struct, with AttrNames 
	   * containing the names of the UniqueAttrs. AttrValues will be
	   * filled in later
	   */
	  pUniqueAttrs = GetUniqueAttrs(varvalue);
	  for (unique_attrs_count = 0; 
	       UniqueAttrsList[unique_attrs_count];
	       unique_attrs_count++ ) {
	    /* skip to the end */
	    ;
	  }
	  /* Append the new UniqueAttrs struct the end od UniqueAttrsList */
	  UniqueAttrsList = (dsgwUniqueAttrs **) 
	    dsgw_ch_realloc (UniqueAttrsList, 
		     (unique_attrs_count + 2) * sizeof(dsgwUniqueAttrs *));
	  UniqueAttrsList[unique_attrs_count] = pUniqueAttrs;
	  UniqueAttrsList[unique_attrs_count + 1] = NULL;
	}
	/* I18n: get preferredLanguage and set it to the input language */
	else if ( strcasecmp( varname, "preferredLanguage" ) == 0 ) {
	    preferredlanguage = dsgw_ch_strdup( varvalue );
	} else if ( strcasecmp( varname, "prev_preferredlanguage" ) == 0 ) {
	    prevlanguage = dsgw_ch_strdup( varvalue );
	}
    }
    if ( gc->gc_international )
        gc->gc_inputlanguage = dsgw_tag2lang( gc->gc_languages,
				        preferredlanguage );
    else
        gc->gc_inputlanguage = gc->gc_languages;

    /*
     * Gather up other changes:  each attribute value is POSTed in a variable
     * named:
     *      add_[mls_][escaped_][unique_][en_][phonetic_][lang_]ATTR
     *      replace_[mls_][escaped_][unique_][en_][phonetic_][lang_]ATTR
     * or   delete_[mls_][escaped_][unique_][en_][phonetic_][lang_]ATTR
     *
     * where ATTR is the LDAP attribute name and "mls_", "escaped_" and
     * "unique_" are optional.  If "mls_" is present, the value is taken as
     * a multiline string as it would appear in a TEXTAREA HTML element.
     * If "escaped_" is present the value is assumed to have an extra layer
     * of URL-escaping.  If "unique_" is present, we check to make sure
     * the value is not in use before accepting a replace or add.
     *
     * I18n related options:
     *     en_ makes ATTR --> ATTR;lang-en
     *     lang_ makes ATTR --> ATTR;lang-<lang>
     *     phonetic_ makes ATTR --> ATTR;lang-<lang>;phonetic
     * where <lang> is extracted from preferredlanguage.
     *
     * Additionally, if a variable name changed_ATTR is POSTed and its value
     * is not "true", it is assumed that no values have changed for that
     * ATTRibute.  If no "changed_ATTR" variable is POSTed, we assume that
     * ATTR has in fact changed.
     */
    i = 0;
    opoffset = -1;
    while (( varname = dsgw_next_cgi_var( &i, &varvalue )) != NULL ) {
	if ( varvalue != NULL && *varvalue == '\0' ) {
	    varvalue = NULL;
	} else {
	    dsgw_remove_leading_and_trailing_spaces( &varvalue );
	}
	
	if ( starts_with( varname, "add_" )) {
	    modop = LDAP_MOD_ADD;
	    opoffset = 4;
	} else if ( starts_with( varname, "replace_" )) {
	    modop = LDAP_MOD_REPLACE;
	    opoffset = 8;
	} else if ( starts_with( varname, "delete_" )) {
	    modop = LDAP_MOD_DELETE;
	    opoffset = 7;
	} else if ( starts_with( varname, "changed_" )) {
	    attr = varname + 8;
	    if ( verbose && strcasecmp( varvalue, "true" ) == 0 ) {
		printf( XP_GetClientStr(DBT_attributeSWasChangedBrN_), attr );
	    }
	    if ( varvalue != NULL && strcasecmp( varvalue, "true" ) != 0 ) {
		unchanged_attrs = (char **)dsgw_ch_realloc( unchanged_attrs,
			( 2 + unchanged_count ) * sizeof( char * ));
		unchanged_attrs[ unchanged_count++ ] = dsgw_ch_strdup( attr );
		unchanged_attrs[ unchanged_count ] = NULL;

		if ( pmods != NULL ) {
		    remove_modifyops( pmods, attr );
		}
	    }
	}

	if ( opoffset >= 0 ) {
	    attr = varname + opoffset;
	    if ( starts_with( attr, "mls_" )) {
		attr += 4;
		mls = 1;
	    } else {
		mls = 0;
	    }
	    if ( starts_with( attr, "escaped_" )) {
		attr += 8;
		value_is_escaped = 1;
	    } else {
		value_is_escaped = 0;
	    }
	    if ( starts_with( attr, "unique_" )) {
		attr += 7;
		unique = 1;
	    } else {
		unique = 0;
	    }
	    /* I18n: handle attrs w/ subtypes */
            langtag = phontag = subtype = NULL;
	    if ( starts_with( attr, "en_" )) {
		attr += 3;
		langtag = "en";
	    }
	    if ( starts_with( attr, "phonetic_" )) {
		attr += 9;
		phontag = "phonetic";
		langtag = preferredlanguage;
	    }
	    if ( starts_with( attr, "lang_" )) {
		attr += 5;
		langtag = preferredlanguage;
	    }
	    if ( langtag ) {
	        if ( phontag ) {
		    subtype =
		      dsgw_ch_malloc( strlen( langtag ) + strlen( phontag ) + 7 );
		    sprintf( subtype, "lang-%s;%s", langtag, phontag );
		} else {
		    subtype = dsgw_ch_malloc( strlen( langtag ) + 6 );
		    sprintf( subtype, "lang-%s", langtag );
		}
	    }

	    if (( p = strchr( attr, '_' )) != NULL ) {
		attr = p + 1;	/* ignore any other prefixes */
	    }

	    for ( j = 0; j < unchanged_count; ++j ) {
		if ( strcasecmp( unchanged_attrs[ j ], attr ) == 0 ) {
		    break;
		}
	    }

	    if ( j >= unchanged_count ) {
		if ( varvalue == NULL || *varvalue == '\0' ) {
		    vals = NULL;
		    varvalue = NULL;
		} else {
		    varvalue = dsgw_ch_strdup( varvalue );
		    if ( value_is_escaped ) {
			dsgw_form_unescape( varvalue );
		    }
		    if ( mls ) {
			vals = post2multilinevals( varvalue );
		    } else {
			vals = post2vals( varvalue );
		    }
		}
		if ( vals == NULL ) {
		    if ( modop != LDAP_MOD_ADD ) {
			addmodifyop( &pmods, modop, attr, subtype, NULL, 0 );
		    }
		} else {
		    for ( j = 0; vals[ j ] != NULL; ++j ) {
				if( strcasecmp( DSGW_ATTRTYPE_NTUSERDOMAINID, attr) == 0)
					if(( retval = dsgw_processdomainid( ld, dn, attr, 
							vals[ j ], strlen( vals[ j ] ))) != 0) {
						vals[ j ] = retval;
					}

			if ( unique && modop != LDAP_MOD_DELETE && 
			     ( lderr = attr_value_is_unique( ld, dn, attr, attr, vals[ j ] ))                               != LDAP_SUCCESS ) {
			    return( lderr );
			}
			if (UniqueAttrsList[0] && modop != LDAP_MOD_DELETE && 
			    (lderr = CheckUniqueAttrs(ld, 
						      dn, 
						      UniqueAttrsList, 
						      attr, 
						      vals[j])) !=
			     LDAP_SUCCESS ) {
			  return( lderr );
			}
	                if ( langtag && !phontag && prevlanguage &&
			     strlen( prevlanguage ) &&
			     modop == LDAP_MOD_REPLACE &&
			     strcasecmp( prevlanguage, "en" ) &&
			     ( strcasecmp( langtag, "en" ) ||
			     ( !strcasecmp( langtag, "en" ) &&
			       !strcasecmp( preferredlanguage, "en" )))) {

			    char *delsubtype;
		            delsubtype =
			      dsgw_ch_malloc( strlen( prevlanguage ) + 6 );
		            sprintf( delsubtype, "lang-%s", prevlanguage );
			    addmodifyop( &pmods, LDAP_MOD_DELETE, attr,
						 delsubtype, NULL, 0 );
                            free( delsubtype );
			    if ( strncasecmp( langtag, "en", 2 ))
			        modop = LDAP_MOD_ADD;
	                }
			addmodifyop( &pmods, modop, attr, subtype, vals[ j ],
							strlen( vals[ j ] ));
		    }
		    free( vals );
		}
		if ( varvalue != NULL ) {
		    free( varvalue );
		}
	    }
	    opoffset = -1;
	    if ( subtype != NULL ) {
		free( subtype );
		subtype = NULL;
	    }
	}

	free( varname );
    }

    if ( verbose && pmods != NULL && pmods[0] != NULL ) {
	int		j, notascii;
	unsigned long	k;
	struct berval	*bvp;

	for ( i = 0; pmods[ i ] != NULL; ++i ) {
	    modop = pmods[ i ]->mod_op & ~LDAP_MOD_BVALUES;
	    printf( "%s %s:\n", modop == LDAP_MOD_REPLACE ?
		    "replace" : modop == LDAP_MOD_ADD ?
		    "add" : "delete", pmods[ i ]->mod_type );
	    if ( pmods[ i ]->mod_bvalues != NULL ) {
		for ( j = 0; pmods[ i ]->mod_bvalues[ j ] != NULL; ++j ) {
		    bvp = pmods[ i ]->mod_bvalues[ j ];
		    notascii = 0;
		    for ( k = 0; k < bvp->bv_len; ++k ) {
			if ( !isascii( bvp->bv_val[ k ] )) {
			    notascii = 1;
			    break;
			}
		    }
		    if ( notascii ) {
			printf( XP_GetClientStr(DBT_TnotAsciiLdBytesN_), bvp->bv_len );
		    } else {
			printf( "\t\"%s\"\n", bvp->bv_val );
		    }
		}
	    }
	}
    }

    if ( verbose ) {
	fprintf( stdout, "</PRE>\n" );
	fflush( stdout );
    }

    fprintf( stdout, "<FONT SIZE=+1>\n" );

    /*
     * apply the changes using LDAP
     */
    if ( pmods == NULL || pmods[0] == NULL ) {
	if ( add ) {
	    fputs( XP_GetClientStr(DBT_noValuesWereEnteredPleaseTryAgai_), stdout );
	    lderr = LDAP_PARAM_ERROR;
	} else {
	    fputs( XP_GetClientStr(DBT_noChangesWereMadeN_), stdout );
	    lderr = LDAP_SUCCESS;
	}
    } else { 
	if ( !quiet && gc->gc_localdbconf == NULL ) {
	    fprintf( stdout, XP_GetClientStr(DBT_PSendingSToTheDirectoryServerN_),
		    add ? XP_GetClientStr(DBT_information_) : XP_GetClientStr(DBT_changes_));
	    fflush( stdout );
	}

	if ( add ) {
	    lderr = ldap_add_s( ld, dn, pmods );
	} else {
	    lderr = ldap_modify_s( ld, dn, pmods );
	}

	if ( lderr == LDAP_SUCCESS ) {
	    if ( !quiet ) {
		if ( add ) {
		    fprintf( stdout, XP_GetClientStr(DBT_PSuccessfullyAddedEntryN_) );
		} else {
		    fprintf( stdout, XP_GetClientStr(DBT_PSuccessfullyEditedEntryYourChan_) );
		}
	    }
	} else {
	    modify_error( lderr );
	}

	ldap_mods_free( pmods, 1 );
    }
    
    FreeUniqueAttrsList( UniqueAttrsList );
    fprintf( stdout, "</FONT>\n" );
    return( lderr );
}

static int
entry_delete( LDAP *ld, char *dn, char *entry_name, int *gobackp )
{
    int	lderr;
    int	dsgw_err;

    fprintf( stdout, "<FONT SIZE=+1>\n" );

    if (( lderr = ldap_delete_s( ld, dn )) == LDAP_SUCCESS ) {
	if ( !quiet ) {
	    fprintf( stdout, XP_GetClientStr(DBT_PSuccessfullyDeletedEntryN_) ); 
	} else {
	    fprintf(stdout, XP_GetClientStr(DBT_done_) );
	    fprintf(stdout, "\n");
	}

	fprintf(stdout, XP_GetClientStr( DBT_PRemovingReferencesToBS_), 
		entry_name); 


	/* call the cleanup function here */
	if( (dsgw_err = entry_cleanup(ld, dn, NULL, &lderr)) != 0){
	    dsgw_error( dsgw_err, NULL, 
			( display_results_inline ? DSGW_ERROPT_INLINE : 0 ), 
			lderr);
	    *gobackp = 0;
	}else{
	  if( verbose ){
	      fprintf(stdout, 
		      XP_GetClientStr(DBT_PSuccesfullyRemovedAllReferences_) );
	  }
	}


    } else {
	modify_error( lderr );
    }

    fprintf( stdout, "</FONT>\n" );

    return( lderr );
}

static int
entry_cleanup(LDAP *ld, char *deldn, char *newrdn, int *cleanup_lderr)
{
    int		rc;
    char	*buf, *dn; 
    LDAPMessage	*result, *e;


    LDAPMod attribute1;
    LDAPMod attribute2;
    LDAPMod *list_of_attrs[3];
    int current_mod = 0;
    
    
    char *attr = NULL;
    char *deldn_values[2];
    char *newrdn_values[2];
    char *newdn, **oldrdns;
    int k, i, j; 
    int dnlen;

    char *varname, *varvalue;
    char *matched;

    int finished;
    int msgid;

    int retVal = 0;

    /* begin the cleanup operation */

    *cleanup_lderr = LDAP_SUCCESS;

    if ( verbose ) {
      fprintf(stdout, "<pre>");  
    }

    /* loop through the attributes we are interested in */
    k = 0;
    while (( varname = dsgw_next_cgi_var( &k, &varvalue )) != NULL && 
	     *cleanup_lderr == LDAP_SUCCESS ) {
	if ( varvalue != NULL && *varvalue == '\0' ) {
	    varvalue = NULL;
	} 

	if ( starts_with( varname, "cleanup_" )) {
	    attr = varvalue;
	} else {
	    attr = NULL;
	}
    
	if(attr != NULL){

		buf = dsgw_ch_malloc( strlen( attr ) + strlen( deldn )+ 4 );
		sprintf( buf, "(%s=%s)", attr, deldn );

		finished = 0;
		if(( msgid = ldap_search(ld, gc->gc_ldapsearchbase, 
					 LDAP_SCOPE_SUBTREE,
					 buf, NULL, 0)) == -1){
		    /* an error occured */
		    retVal = DSGW_ERR_CLEANUPENTRY;
		    *cleanup_lderr = ldap_get_lderrno(ld, &matched, NULL);
		    finished = 1; 
		}

            
		/* Loop, polling for results until finished */

		while(!finished){
		    result = NULL;
		    newdn = NULL;
		    rc = ldap_result(ld, msgid, 0, NULL, &result);

		    switch( rc ){

		    case -1:
		        /* some error occured */

		      retVal = DSGW_ERR_CLEANUPENTRY;
		      *cleanup_lderr = ldap_result2error(ld, result, 1); 

		      finished = 1;  

		      break;
		    case 0:
		        /* timeout was exceeded. No entries are ready for 
			   retrieval */

		      if(result != NULL){
			  ldap_msgfree( result );
		      }
		    
		      break;

		    case LDAP_RES_SEARCH_RESULT:
		        /* ldap result returned */
		      *cleanup_lderr = ldap_result2error(ld, result, 1); 
		    
		      finished = 1;
		    
		      break;

		    case LDAP_RES_SEARCH_ENTRY:
		        /* entry is ready for retrieval */

		      if(( e = ldap_first_entry(ld, result)) == NULL){
			  /* All done */
			finished = 1;
			if (result != NULL){
			    ldap_msgfree( result );
			}
			continue;
		      }

		      /* modify each entry deleting the dn from it */

		      if( ( dn = ldap_get_dn(ld, e)) != NULL) {

		      /* if renaming, change to point to new rdn */
			  if(newrdn != NULL){

			    if (( oldrdns = ldap_explode_dn( deldn, 0 )) == NULL ) {
			      dsgw_error( DSGW_ERR_NOMEMORY, NULL, DSGW_ERROPT_EXIT, 0 );
			    }

			    dnlen = strlen( newrdn ) + 1;	/* room for "," */
			    for ( i = 1; oldrdns[ i ] != NULL; ++i ) {
			        dnlen += ( 1 + strlen( oldrdns[ i ] ));
			    }
			    newdn = dsgw_ch_malloc( dnlen + 1 );	
			    *newdn = '\0';
			    strcat( newdn, newrdn );
			    for ( i = 1; oldrdns[ i ] != NULL; ++i ) {
			        strcat( newdn, "," );
				strcat( newdn, oldrdns[ i ] );
			    }
			    ldap_value_free( oldrdns );

			    attribute2.mod_type = attr;
			    attribute2.mod_op = LDAP_MOD_ADD;
		      
			    newrdn_values[0] = newdn;
			    newrdn_values[1] = NULL;
			    attribute2.mod_values = newrdn_values;
			    
			    list_of_attrs[current_mod] = &attribute2;
			    current_mod++;
			  }
		      
		      
			  if( verbose ){
			      fprintf(stdout, 
			      XP_GetClientStr(DBT_PRemovingSFromSN_), 
				      deldn, dn); 
			  }

			  attribute1.mod_type = attr;
			  attribute1.mod_op = LDAP_MOD_DELETE;
		      
			  deldn_values[0] = deldn;
			  deldn_values[1] = NULL;
			  attribute1.mod_values = deldn_values;
		      
			  list_of_attrs[current_mod] = &attribute1;
			  current_mod++;
			  list_of_attrs[current_mod] = NULL;

			  /* reset current mod for next time through loop */
			  current_mod = 0;


			  /* modify */
			  if((*cleanup_lderr = ldap_modify_s(ld, dn, 
				    list_of_attrs)) != LDAP_SUCCESS){

			      retVal = DSGW_ERR_CLEANUPENTRY;		      

			      /* quit on error */
			      finished = 1; 
			  }else{
			      if( verbose ){
				fprintf(stdout, XP_GetClientStr(
					 DBT_theOperationWasSuccessful_) );
			      }
			  }

			  if(dn != NULL){
			    ldap_memfree(dn);
			  }
		      }
		      if(result != NULL){
		          ldap_msgfree( result );
		      }
		      if(newdn != NULL){
		          free(newdn);
		      }
		      break;

		    default:
		        /* must be some kind of error */
		      retVal = DSGW_ERR_CLEANUPENTRY;		      
		      *cleanup_lderr = ldap_result2error(ld, result, 1); 

		      finished = 1;
		      break;
		    }

		    /* do other work here while you are waiting.... */
		}
		if(varname != NULL){
		    free( varname );
		}
	    
		 if(buf != NULL){
		   free(buf);
		 }
	}
    }

    if( verbose ){
        fprintf(stdout, "</pre>");
    }
    return retVal;
}


static int
entry_modrdn( LDAP *ld, char *dn, char *newrdn, int deleteoldrdn, char *entry_name, int *gobackp )
{
    int	lderr;
    int	dsgw_err;

    if ( verbose ) {
	fprintf( stdout, XP_GetClientStr(DBT_PreTheNewNameForTheEntryIsSNPreH_),
		newrdn );
    }

    fprintf( stdout, "<FONT SIZE=+1>\n" );

    if (( lderr = ldap_modrdn2_s( ld, dn, newrdn, deleteoldrdn )) 
	    == LDAP_SUCCESS ) {
	if ( !quiet ) {
	    fprintf( stdout, XP_GetClientStr(DBT_PSuccessfullyRenamedEntryN_) );
	}

	fprintf(stdout, XP_GetClientStr( DBT_PUpdatingReferencesToBS_), entry_name); 

	    /* call the cleanup function here */
	    if( (dsgw_err = entry_cleanup(ld, dn, newrdn, &lderr)) != 0){
	        dsgw_error( dsgw_err, NULL, 
			    ( display_results_inline ? DSGW_ERROPT_INLINE : 0 )
			    , lderr);
		*gobackp = 0;
	    }else{
	        if( verbose ){
		  fprintf(stdout, 
		    XP_GetClientStr(DBT_PSuccesfullyUpdatedAllReferences_) );
		}
	    }	    



    } else {
	modify_error( lderr );
    }

    fprintf( stdout, "</FONT>\n" );

    return( lderr );
}

static int isAsciiString( char *str)
{
	int isAscii=1;
	int i;

	for (i=0; i<strlen(str);i++)
	{
		if (((unsigned char)str[i])>=0x80) 
		{
			isAscii=0;
			return isAscii;
		}
	}
	return isAscii;

}

static int
gather_passwd_changes( char *dn, LDAPMod ***pmodsp, int adding_entry,
	int *pwdchangedp )
{
    int		lderr, lockpasswd;
    char	*bindpasswd, *newpasswd, *newpasswdconfirm, *errstring;

    lockpasswd = dsgw_get_boolean_var( "lockpasswd", 0, 0 );
    if ( lockpasswd ) {
	/*
	 * the userPassword attribute to a special value that no password
	 * submitted by a user can ever match.
	 */
	time_t		curtime;
	struct tm	*gmtp;
	char		*tstr;

	/* get string representation of current GMT time */
	curtime = time( NULL );
	gmtp = gmtime( &curtime );
	tstr = asctime( gmtp );

	/* remove trailing newline */
	tstr[ strlen( tstr ) - 1 ] = '\0';

	/* allocate room for "{crypt}LOCKED [" + tstr + " GMT]" + zero byte */
	newpasswd = dsgw_ch_malloc( 15 + strlen( tstr ) + 5 + 1 );
	sprintf( newpasswd, XP_GetClientStr(DBT_CryptLockedSGmt_), tstr );

    } else if (( newpasswd = dsgw_get_cgi_var( "newpasswd",
	    DSGW_CGIVAR_OPTIONAL )) == NULL ) {
	return( LDAP_SUCCESS );	/* not setting password -- nothing to do */
    }

    lderr = LDAP_PARAM_ERROR;	/* pessimistic */

    if ( !adding_entry && ( bindpasswd = dsgw_get_cgi_var( "passwd",
	    DSGW_CGIVAR_OPTIONAL )) == NULL && require_oldpasswd( dn )) {
	errstring = XP_GetClientStr(DBT_youMustProvideTheOldPassword_);
    } else if ( !lockpasswd &&
	    (( newpasswdconfirm = dsgw_get_cgi_var( "newpasswdconfirm",
	    DSGW_CGIVAR_OPTIONAL )) == NULL || strcmp( newpasswd,
	    newpasswdconfirm ) != 0 )) {
	errstring = XP_GetClientStr(DBT_theNewAndConfirmingPasswordsDoNo_);
    } else if (!isAsciiString(newpasswd))
	{
		errstring = XP_GetClientStr(DBT_notAsciiPassword);
	} else {
	addmodifyop( pmodsp, adding_entry ? LDAP_MOD_ADD : LDAP_MOD_REPLACE,
		DSGW_ATTRTYPE_USERPASSWORD, NULL, newpasswd, strlen( newpasswd ));
	*pwdchangedp = 1;
	lderr = LDAP_SUCCESS;
    }

    if ( lderr != LDAP_SUCCESS ) {
	fprintf( stdout, "<FONT SIZE=+1>\n%s\n</FONT>\n", errstring );
    }

    return( lderr );
}


static void
modify_error( int lderr )
{
    int	dsgw_err;

    if ( gc->gc_localdbconf == NULL ) {
	dsgw_err = DSGW_ERR_LDAPGENERAL;	/* LDAP server */
    } else {
	dsgw_err = DSGW_ERR_LOCALDBGENERAL;	/* local db */
    }

    dsgw_error( dsgw_err, ldap_err2string( lderr ),
	    ( display_results_inline ? DSGW_ERROPT_INLINE : 0 ), lderr );
}


/*
 * this "addmodifyop" routine is lifted with minor changes from
 * ldap/tools/ldapmodify.c
 * I18n: added an argument, subtype
 */
static void
addmodifyop( LDAPMod ***pmodsp, int modop, char *attr, char *subtype,
					char *value, int vlen )
{
    LDAPMod		**pmods;
    int			i, j;
    struct berval	*bvp;

    /* I18n: If not ldap v3 and subtype is given, do nothing */
    if (( !gc->gc_international || gc->gc_ldapversion < LDAP_VERSION3 ) &&
	 (subtype || !strcasecmp( attr, "preferredLanguage" )))
	return;		

    if ( attr == NULL || *attr == '\0' ) {
	return;
    }
    pmods = *pmodsp;
    modop |= LDAP_MOD_BVALUES;

    i = 0;
    if ( pmods != NULL ) {
	for ( ; pmods[ i ] != NULL; ++i ) {
	    if ( strcasecmp( pmods[ i ]->mod_type, attr ) == 0 &&
		    pmods[ i ]->mod_op == modop && !subtype ) {
		break;
	    }
	}
    }
    
    if ( pmods == NULL || pmods[ i ] == NULL ) {
	pmods = (LDAPMod **)dsgw_ch_realloc( pmods, (i + 2) *
		sizeof( LDAPMod * ));
	*pmodsp = pmods;
	pmods[ i + 1 ] = NULL;
	pmods[ i ] = (LDAPMod *)dsgw_ch_malloc( sizeof( LDAPMod ));
	memset( pmods[ i ], 0, sizeof( LDAPMod ));
	pmods[ i ]->mod_op = modop;
	/* I18n: add subtype to the attribute */
	if ( subtype ) {
	    pmods[ i ]->mod_type = dsgw_ch_malloc( strlen( attr ) + strlen( subtype ) + 2 );
	    sprintf( pmods[ i ]->mod_type, "%s;%s", attr, subtype);
	} else {
	    pmods[ i ]->mod_type = dsgw_ch_strdup( attr );
	}
    }

    if ( value != NULL ) {
	j = 0;
	if ( pmods[ i ]->mod_bvalues != NULL ) {
	    for ( ; pmods[ i ]->mod_bvalues[ j ] != NULL; ++j ) {
		;
	    }
	}
	pmods[ i ]->mod_bvalues =
		(struct berval **)dsgw_ch_realloc( pmods[ i ]->mod_bvalues,
		(j + 2) * sizeof( struct berval * ));
	pmods[ i ]->mod_bvalues[ j + 1 ] = NULL;
	bvp = (struct berval *)dsgw_ch_malloc( sizeof( struct berval ));
	pmods[ i ]->mod_bvalues[ j ] = bvp;

	bvp->bv_len = vlen;
	bvp->bv_val = (char *)dsgw_ch_malloc( vlen + 1 );
	memcpy( bvp->bv_val, value, vlen );
	bvp->bv_val[ vlen ] = '\0';
#ifdef DSGW_DEBUG
        dsgw_log( "mod_type: %s, mod_bvalues: %s\n",
	      pmods[ i ]->mod_type, pmods[ i ]->mod_bvalues[ j ]->bv_val );
#endif
    }
}


/* remove all modify ops that refer to "attr" */
static void
remove_modifyops( LDAPMod **pmods, char *attr )
{
    int		i, found_attr;

    if ( pmods == NULL ) {
	return;
    }

    do {
	found_attr = 0;
	for ( i = 0 ; pmods[ i ] != NULL; ++i ) {
	    if ( strcasecmp( pmods[ i ]->mod_type, attr ) == 0 ) {
		found_attr = 1;
		break;
	    }
	}

	if ( found_attr ) {
	    if ( pmods[ i ]->mod_bvalues != NULL ) {
		ber_bvecfree( pmods[ i ]->mod_bvalues );
	    }
	    free( pmods[ i ] );

	    for ( ; pmods[ i + 1 ] != NULL; ++i ) {
		pmods[ i ] = pmods[ i + 1 ];
	    }
	    pmods[ i ] = NULL;
	}

    } while ( found_attr );
}


static int
starts_with( char *s, char *startswith )
{
    int	len;

    len = strlen( startswith );
    return ( strlen( s ) > len && strncmp( s, startswith, len ) == 0 );
}


/* 
 * there is one value in "postedval" but newlines must be changed to "$",
 * '$' characters must be changed to \24, and '\' chars. changed to \5C
 */
static char **
post2multilinevals( char *postedval )
{
    int		specials;
    char	*p, *r, **vals;

    vals = dsgw_ch_malloc( 2 * sizeof( char * ));
    vals[ 1 ] = NULL;

    specials = 0;
    for ( p = postedval; *p != '\0'; ++p ) {
	if ( *p == '$' || *p == '\\' || *p == '\n' || *p == '\r') {
	    ++specials;
	}
    }

    /* allocate enough room to handle any necessary escaping */
    r = vals[ 0 ] = dsgw_ch_malloc( 2 * specials + strlen( postedval ) + 1 );

    /* copy and escape as appropriate */
    for ( p = postedval; *p != '\0'; ++p ) {
	if ( *p == '\n' || *p == '\r' ) {	/* change to "$" */
	    *r++ = '$';
	    if ( *(p+1) != '\0' && *(p+1) != *p &&
		    ( *(p+1) == '\n' || *(p+1) == '\r' )) {
		++p;	/* skip next char. if sequence is "\r\n" or "\n\r" */
	    }
	} else if ( *p == '$' ) {		/* change to "\24" */
	    *r++ = '\\';
	    *r++ = '2';
	    *r++ = '4';
	} else {
	    *r++ = *p;
	    if ( *p == '\\' ) {			/* change to "\5C" */
		*r++ = '5';
		*r++ = 'C';
	    }
	}
    }

    *r = '\0';

    return( vals );
}


/* values are delimited by newlines, preceded by optional carriage returns */
static char **
post2vals( char *postedval )
{
    int		count, len;
    char	*p, *q, **vals;

    vals = NULL;

    count = 0;
    for ( p = postedval; p != NULL && *p != '\0'; p = q ) {
	/* skip any leading CRs or NLs */
	while (( *p == '\n' || *p == '\r' ) && *p != '\0' ) {
	    ++p;
	}
	if ( *p == '\0' ) {
	    break;
	}

	/* find end of this line */
	if (( q = strchr( p, '\n' )) != NULL ) {
	    *q++ = '\0';
	}

	/* remove CR, if any */
	len = strlen( p ) - 1;
	if ( p[ len ] == '\r' ) {
	    p[ len ] = '\0';
	}

	/* add to values array */
	vals = dsgw_ch_realloc( vals, ( count + 2 ) * sizeof( char * ));
	vals[ count++ ] = p;
    }
    vals[ count ] = NULL;

    return( vals );
}


/*
 * Determine if we should insist that the old password for the entry
 * we are modifying (modifydn) be POSTed.  The rule we use is simply
 * this:  if the binddn and modifydn are the same, require the old
 * password.  This allows directory admins. to reset passwords while
 * preventing normal users from having their password changed if they
 * just happen to walk away from their computer for a while when they
 * are authenticated to the gateway.
 */
static int
require_oldpasswd( char *modifydn )
{
    return( dsgw_bound_as_dn( modifydn, 1 ));
}


/* 
 * Add the current value of uid in the entry to the ntdomain id before 
 * further processing of the domain id.
 */
static char *
dsgw_processdomainid( LDAP *ld, char *dn, char *attr, char *val, int len)
{
    int rc, count;
    LDAPMessage *msgp = NULL;
    LDAPMessage *entry;
    char **attrlist, *attrs[ 2 ];
    char *value, *newval;
	char *pch, **vals;

    if( strcasecmp( attr, DSGW_ATTRTYPE_NTUSERDOMAINID ) != 0 )
        return( NULL );

    attrs[ 0 ] = NULL; 
    attrs[ 1 ] = NULL;
    attrlist = attrs;

    if(( rc = ldap_search_s( ld, dn, LDAP_SCOPE_BASE, "(objectclass=*)", attrlist,
   	     0, &msgp )) != LDAP_SUCCESS && rc != LDAP_NO_SUCH_OBJECT) 
	{
	    return( NULL );
    }

    count = (msgp == NULL) ? 0 : ldap_count_entries( ld, msgp );

    if( count > 0 ) 
	{
        entry = ldap_first_entry( ld, msgp );
        if( entry ) 
		{

	        if(( vals = ldap_get_values( ld, entry, DSGW_ATTRTYPE_NTUSERDOMAINID )) != NULL)
			{
				if( vals[0] != NULL )
				{
					value = dsgw_ch_strdup( vals[0] );
					newval = dsgw_ch_malloc( len + strlen( value ) +1 );
					strcpy( newval, val );
					pch = strchr( value, DSGW_NTDOMAINID_SEP );
					if( pch )  
					{
						strcat( newval, pch );
						return( newval );
					}
				}
			}
        }
    }

    return( NULL );
}


/*
 * search directory to find out if an attribute value is unique.  If the
 * value doesn't already exist or if it exists only in the same entry as 
 * the same attribute, of the entry we are changing we return LDAP_SUCCESS.  
 * If it does exist, we return  LDAP_TYPE_OR_VALUE_EXISTS.  If some other 
 * error occurs, we return another LDAP error code.
 *
 */
static int
attr_value_is_unique( LDAP *ld, char *dn, char *attr, char *real_attr, char *value )
{

  int		rc, count;
    char	*attrs[2], *buf, *tmpdn, *attrdesc;
    LDAPMessage	*res, *e;


    /* allocate room for "(attr=value)" filter */
    buf = dsgw_ch_malloc( strlen( attr ) + strlen( value ) + 4 );
    sprintf( buf, "(%s=%s)", attr, value );

    attrs[ 0 ] = attr;
    attrs[ 1 ] = NULL;

    rc = ldap_search_s( ld, gc->gc_ldapsearchbase, LDAP_SCOPE_SUBTREE,
	    buf, attrs, 1, &res );
    free( buf );

    if ( rc != LDAP_SUCCESS || res == NULL ) {
	modify_error( rc );
	return( rc );
    }

    if (( count = ldap_count_entries( ld, res )) == 0 ) {
	rc = LDAP_SUCCESS;
    } else if ( count > 1 ) {
	rc = LDAP_TYPE_OR_VALUE_EXISTS;
    } else {	/* found one entry: see if it is the entry we are modifying */
	if (( e = ldap_first_entry( ld, res )) == NULL ||
	    ( tmpdn = ldap_get_dn( ld, e )) == NULL ) {
	    rc = ldap_get_lderrno( ld, NULL, NULL );
	} else if ( dsgw_dn_cmp( dn, tmpdn ) != 0 && 

		    /* This might work. -- atom */
		    strcasecmp (attr, real_attr) == 0) {
	    rc = LDAP_SUCCESS;	/* same entry */
	} else {
	    rc = LDAP_TYPE_OR_VALUE_EXISTS;
	}
    }

    ldap_msgfree( res );

    if ( rc == LDAP_TYPE_OR_VALUE_EXISTS ) {
	buf = dsgw_ch_malloc( strlen( attr ) + 6 );	/* room for "desc_" */
	sprintf( buf, "desc_%s", attr );
	if (( attrdesc = dsgw_get_cgi_var( buf, DSGW_CGIVAR_OPTIONAL ))
		== NULL ) {
	    attrdesc = attr;
	}
	free( buf );

	fputs( "\n<FONT SIZE=+1>\n", stdout );
	fprintf( stdout, XP_GetClientStr(DBT_BrTheSBSBIsAlreadyInUsePleaseCho_), attrdesc, value );
	fputs( "\n</FONT>\n", stdout );
    }

    return( rc );
}


dsgwUniqueAttrs *GetUniqueAttrs(char *val) {
  dsgwUniqueAttrs *retVal;
  int i;
  char *pChar;
  int numAttrs;
  char *pAttrName = (char *) dsgw_ch_malloc (strlen(val) + 1);


  if (val[0]) {
    numAttrs = 1;
  }

  /* the number of attributes is the number of blanks + 1 */
  for (pChar = strchr(val, ' '); pChar; pChar = strchr(pChar + 1, ',')) {
    numAttrs++;
  }

  
  retVal = (dsgwUniqueAttrs *) dsgw_ch_malloc (sizeof(dsgwUniqueAttrs));
  retVal->AttrNames = (char **) dsgw_ch_malloc ((numAttrs + 1) * sizeof (char *));

  if (numAttrs == 1) {
    retVal->AttrNames[0] = (char *) dsgw_ch_malloc ((strlen(val) + 1) * sizeof(char));
    strcpy (retVal->AttrNames[0], val);
    retVal->AttrNames[1] = NULL;
  }

  else {
    for (i = 0, pChar = val; i < numAttrs; i++) {
      sscanf (pChar, "%s", pAttrName);
      retVal->AttrNames[i] = (char *) dsgw_ch_malloc ((strlen(pAttrName) + 1) * sizeof(char));
      strcpy (retVal->AttrNames[i], pAttrName);      
      pChar = strchr (pChar, ' ');

    }
    retVal->AttrNames[i] = NULL;
  }
  
  retVal->AttrValues = (char **) dsgw_ch_malloc (sizeof (char *));

  retVal->AttrValues[0] = NULL;
  

  
  return retVal;
}  
  
/*
 * CheckUniqueAttrs checks to see if attr is a UniqueAttr, and if so, makes sure
 * that val is not already in UniqueAttr, either in an entry which already
 * exists in the Directory, or is a duplicate in the entry being added/modified
 */
 
static int CheckUniqueAttrs (LDAP *ld, char *dn, dsgwUniqueAttrs **UniqueAttrsList,
			     char *attr, char *val) {
  int i, j;
  int isUniqueAttr = 0;
  int retVal;
  char *attrdesc;
  dsgwUniqueAttrs *pUniqueAttrs;
  char *buf; 

  buf = dsgw_ch_malloc( strlen( attr ) + 6 );	/* room for "desc_" */
  sprintf( buf, "desc_%s", attr );
  if (( attrdesc = dsgw_get_cgi_var( buf, DSGW_CGIVAR_OPTIONAL ))
      == NULL ) {
    attrdesc = attr;
  }


  if (!UniqueAttrsList[0]) {
    /* There aren't any UniqueAttrs */
    return LDAP_SUCCESS;
  }

  /* Look for attr in the UniqueAttrsList */

  for (j = 0, isUniqueAttr = 0; UniqueAttrsList[j]; j++) {
    for (i = 0; UniqueAttrsList[j]->AttrNames[i]; i++){
      if (strcasecmp(UniqueAttrsList[j]->AttrNames[i], attr) == 0) {
	isUniqueAttr = 1;
	break;
      }
    }
    if (isUniqueAttr) {
      break;
    }
  }

  /* attr isn't in UniqueAttrsList, so ignore it */
  if (!isUniqueAttr) {
    /* it's not a UniqueAttr */
    return LDAP_SUCCESS;
  }

  pUniqueAttrs = UniqueAttrsList[j];
  for (i = 0; pUniqueAttrs->AttrValues[i]; i++) {

    /* Check to see if val was entered twice in this entry */
    if (strcasecmp(pUniqueAttrs->AttrValues[i], val) == 0) {
      /* we have a duplicate! */
      fputs( "\n<FONT SIZE=+1>\n", stdout );
      fprintf( stdout, XP_GetClientStr(DBT_BrTheSBSBIsAlreadyInUsePleaseCho_), attrdesc, val );
      fputs( "\n</FONT>\n", stdout );
      return LDAP_TYPE_OR_VALUE_EXISTS;
    }
  }
  
  for (i = 0, retVal = LDAP_SUCCESS; 
       pUniqueAttrs->AttrNames[i] && retVal == LDAP_SUCCESS; i++) {
    /* Check to see if the value exists in an entry already in the directory */
    
    retVal = attr_value_is_unique (ld, dn, pUniqueAttrs->AttrNames[i], attr,  val);
  }
  
  if (retVal != LDAP_SUCCESS) {
    return retVal;
  }


  /* Add val to AttrValues */
  
  for (i = 0; pUniqueAttrs->AttrValues[i]; i++) {
    /* skip to the end of the array... */ 
    ;
  }
  pUniqueAttrs->AttrValues = 
    (char **) dsgw_ch_realloc (pUniqueAttrs->AttrValues, (i+2) * sizeof(char *));
  

  pUniqueAttrs->AttrValues[i] = (char *) dsgw_ch_malloc( (strlen (val)+1) * sizeof(char));
  strcpy (pUniqueAttrs->AttrValues[i], val);
  pUniqueAttrs->AttrValues[i+1] = NULL;

  return LDAP_SUCCESS;
}
    
static void FreeUniqueAttrsList(dsgwUniqueAttrs **UniqueAttrsList) {
  int i, j;

  if (!UniqueAttrsList) {
    return;
  }

  for (i = 0; UniqueAttrsList[i]; i++) {
    for (j = 0; UniqueAttrsList[i]->AttrNames[j]; j++) {
      free (UniqueAttrsList[i]->AttrNames[j]);
    }
    for (j = 0; UniqueAttrsList[i]->AttrValues[j]; j++) {
      free (UniqueAttrsList[i]->AttrValues[j]);
    }
    free (UniqueAttrsList[i]);
  }
  free (UniqueAttrsList);
}
