/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#include "dsgw.h"

static void get_request();
static void post_request();


main( argc, argv, env )
    int		argc;
    char	*argv[];
#ifdef DSGW_DEBUG
    char	*env[];
#endif
{
    int		reqmethod;

    reqmethod = dsgw_init( argc, argv,  DSGW_METHOD_POST | DSGW_METHOD_GET );
    dsgw_send_header();

#ifdef DSGW_DEBUG
   dsgw_logstringarray( "env", env ); 
#endif

    if ( reqmethod == DSGW_METHOD_GET ) {
	get_request();
    } else {
	post_request();
    }

    exit( 0 );
}


static void
get_request()
{
    int			urllen, pilen, qslen;
    char		*p, *q, *ldapurl, *pathinfo, *querystring;
    unsigned long	options;

    /*
     * On a GET request, we do an LDAP URL search (which will just display
     * a single entry if all that is included is "host:port/DN").
     * PATH_INFO + QUERY_STRING should make up an LDAP URL, minus the
     * "ldap://" part.  The host:port is required, since PATH_INFO can't
     * start with a '/' (web server sees that as a different program).
     */
    if (( p = getenv( "PATH_INFO" )) == NULL ) {
	pathinfo = "";
	pilen = 0;
    } else {
	++p;	/* skip past leading '/' */
	pathinfo = dsgw_ch_malloc( 3 * strlen( p ) + 1 );

	if (( q = strchr( p, '/' )) != NULL ) {
	    pilen = q - p + 1;
	    memcpy( pathinfo, p, pilen );
	    pathinfo[ pilen ] = '\0';
	    ++q;
	} else {
	    *pathinfo = '\0';
	    q = p;
	}

	dsgw_strcat_escaped( pathinfo, q );
	pilen = strlen( pathinfo );
    }

    if (( querystring = getenv( "QUERY_STRING" )) == NULL ) {
	qslen = 0;
    } else {
	qslen = strlen( querystring );
    }

    if ( pilen + qslen == 0 ) {
	dsgw_error( DSGW_ERR_MISSINGINPUT, NULL, DSGW_ERROPT_EXIT, 0 );
    }

    urllen = LDAP_URL_PREFIX_LEN + pilen + qslen + 1;

    p = ldapurl = (char *)dsgw_ch_malloc( urllen );

    strcpy( p, LDAP_URL_PREFIX );
    p += LDAP_URL_PREFIX_LEN;

    if ( pilen > 0 ) {
	strcpy( p, pathinfo );
    }

    if ( qslen > 0 ) {
	char *tmpstring;
	int first = 1;
	p = querystring;
	q = querystring + strlen( querystring );
        tmpstring = (char *)dsgw_ch_malloc( qslen );
	while ((( p = strrchr( querystring, '&' )) != NULL ) ||
	       ( q - querystring > 1 )) {
	    if ( p )
	        *p++ = '\0';
	    else
		p = querystring;
	    q = p;

	    if ( p != NULL && strncasecmp( p, "dn=", 3 ) == 0 ) {
		sprintf( ldapurl + strlen( ldapurl ), "/%s", p + 3 );
	    } else {
		if ( first ) {
		    sprintf( ldapurl + strlen( ldapurl ), "?%s", p );
		    first = 0;
		} else
		    sprintf( ldapurl + strlen( ldapurl ), "&%s", p );
	    }
	}
	free( tmpstring );
    }

    options = 0;

    /*
     * When running as part of Admin Server (Users & Groups U/I), always
     * display editable views of entries.
     */
    if ( gc->gc_admserv ) {
	options |= DSGW_DISPLAY_OPT_EDITABLE;
    }

#ifdef DSGW_DEBUG
    dsgw_log( "get_request: processing LDAP URL \"%s\"\n", ldapurl );
#endif
    dsgw_ldapurl_search( NULL, ldapurl, options );
}


static void
post_request()
{
    char			*modestr, *searchstring, *type, *base;
    LDAP			*ld;
    LDAPFiltDesc		*lfdp;
    struct ldap_searchobj	*solistp, *sop;
    int				authmode, mode;
    unsigned long		options;
    /* I18n variable */
    char        		*preferredlanguage;

    options = 0;
    modestr = dsgw_get_cgi_var( "mode", DSGW_CGIVAR_REQUIRED );
    searchstring = dsgw_get_cgi_var( "searchstring", DSGW_CGIVAR_OPTIONAL );
    dsgw_remove_leading_and_trailing_spaces( &searchstring );

    authmode = 0;
    if ( strcasecmp( modestr, DSGW_SRCHMODE_AUTH ) == 0 ) {
	/*
	 * treat authenticate as a variant of the smart search mode
	 */
	authmode = 1;
	mode = DSGW_SRCHMODE_SMART_ID;
	options |= DSGW_DISPLAY_OPT_AUTH;
    } else if ( strcasecmp( modestr, DSGW_SRCHMODE_SMART ) == 0 ) {
	mode = DSGW_SRCHMODE_SMART_ID;
    } else if ( strcasecmp( modestr, DSGW_SRCHMODE_COMPLEX ) == 0 ) {
	mode = DSGW_SRCHMODE_COMPLEX_ID;
    } else if (  strcasecmp( modestr, DSGW_SRCHMODE_PATTERN ) == 0 ) {
	mode = DSGW_SRCHMODE_PATTERN_ID;
    } else {
	dsgw_error( DSGW_ERR_SEARCHMODE, modestr, 0, 0 );
    }

    if ( mode != DSGW_SRCHMODE_PATTERN_ID
	    && ( searchstring == NULL || *searchstring == '\0' )) {
	dsgw_error( DSGW_ERR_NOSEARCHSTRING, NULL, DSGW_ERROPT_EXIT, 0 );
    }

    if (( type = dsgw_get_cgi_var( "type", authmode ? DSGW_CGIVAR_OPTIONAL :
	    DSGW_CGIVAR_REQUIRED )) == NULL ) {
	type = DSGW_SRCHTYPE_AUTH;
    }

    if (( base = dsgw_get_cgi_var( "base", DSGW_CGIVAR_OPTIONAL )) == NULL ) {
	base = gc->gc_ldapsearchbase;
    }

    /* check for options (carried in boolean CGI variables) */
    if ( dsgw_get_boolean_var( "listifone", DSGW_CGIVAR_OPTIONAL, 0 )) {
	options |= DSGW_DISPLAY_OPT_LIST_IF_ONE;
    }

    if ( dsgw_get_boolean_var( "editable", DSGW_CGIVAR_OPTIONAL,
	    gc->gc_admserv )) {
	options |= DSGW_DISPLAY_OPT_EDITABLE;
    }

    if ( dsgw_get_boolean_var( "link2edit", DSGW_CGIVAR_OPTIONAL, 0 )) {
	options |= DSGW_DISPLAY_OPT_LINK2EDIT;
    }

    if ( dsgw_get_boolean_var( "dnlist_js", DSGW_CGIVAR_OPTIONAL, 0 )) {
	options |= DSGW_DISPLAY_OPT_DNLIST_JS;
    }

    (void) dsgw_init_ldap( &ld, &lfdp, ( authmode == 1 ) ? 1 : 0 );

    /* I18n: get preferredLanguage sent from the form */
    if (( preferredlanguage =
          dsgw_get_cgi_var( "preferredLanguage", DSGW_CGIVAR_OPTIONAL ))
	  == NULL ) {
	  preferredlanguage = DSGW_DEFAULT_SELECTED_LANG;
    }
    if ( gc->gc_international )
        gc->gc_inputlanguage = dsgw_tag2lang( gc->gc_languages,
				        preferredlanguage );
    else
        gc->gc_inputlanguage = gc->gc_languages;

#ifdef DSGW_DEBUG
    dsgw_log( "inputlang: %s\n", preferredlanguage ); 
#endif

    if ( mode != DSGW_SRCHMODE_PATTERN_ID ) {
	dsgw_init_searchprefs( &solistp );

	if (( sop = dsgw_type2searchobj( solistp, type )) == NULL ) {
	    ldap_unbind( ld );
	    dsgw_error( DSGW_ERR_UNKSRCHTYPE, type, DSGW_ERROPT_EXIT, 0 );
	}
    }

    switch( mode ) {
    case DSGW_SRCHMODE_SMART_ID:
	/*
	 * smart search mode -- try to do the right kind of search for the
	 * client based on what the user entered in the search box
	 */
	dsgw_smart_search( ld, sop, lfdp, base, searchstring, options );
	break;

    case DSGW_SRCHMODE_COMPLEX_ID: {
	/*
	 * complex search mode -- construct a specific filter based on
	 * user's form selections
	 */
	int			scope;
	char			*attrlabel, *matchprompt;
	struct ldap_searchattr	*sap;
	struct ldap_searchmatch	*smp;

	attrlabel = dsgw_get_cgi_var( "attr", DSGW_CGIVAR_REQUIRED );
	if (( sap = dsgw_label2searchattr( sop, attrlabel )) == NULL ) {
	    ldap_unbind( ld );
	    dsgw_error( DSGW_ERR_UNKATTRLABEL, attrlabel, DSGW_ERROPT_EXIT, 0 );
	}
	
	matchprompt = dsgw_get_cgi_var( "match", DSGW_CGIVAR_REQUIRED );
	if (( smp = dsgw_prompt2searchmatch( sop, matchprompt )) == NULL ) {
	    ldap_unbind( ld );
	    dsgw_error( DSGW_ERR_UNKMATCHPROMPT, matchprompt,
		    DSGW_ERROPT_EXIT, 0 );
	}

	scope = dsgw_get_int_var( "scope", DSGW_CGIVAR_OPTIONAL,
		sop->so_defaultscope );
	dsgw_complex_search( ld, sop, sap, smp, base, scope, searchstring,
		options );
	}
	break;

    case DSGW_SRCHMODE_PATTERN_ID: {
	/*
	 * pattern-based search mode (no searchprefs or filter file used)
	 */
	char	*attr, *pattern, *prefix, *suffix, *searchdesc;
	int	scope;

	attr = dsgw_get_cgi_var( "attr", DSGW_CGIVAR_REQUIRED );
	pattern = dsgw_get_cgi_var( "filterpattern", DSGW_CGIVAR_REQUIRED );
	prefix = dsgw_get_cgi_var( "filterprefix", DSGW_CGIVAR_OPTIONAL );
	suffix = dsgw_get_cgi_var( "filtersuffix", DSGW_CGIVAR_OPTIONAL );
	scope = dsgw_get_int_var( "scope", DSGW_CGIVAR_OPTIONAL,
		LDAP_SCOPE_SUBTREE );
	options |= DSGW_DISPLAY_OPT_CUSTOM_SEARCHDESC;
	if (( searchdesc = dsgw_get_cgi_var( "searchdesc",
		DSGW_CGIVAR_OPTIONAL )) == NULL ) {
	    searchdesc = "pattern";
	}

	dsgw_pattern_search( ld, type, searchdesc, pattern, prefix, suffix,
		attr, base, scope, searchstring, options );
	}
	break;
    }

    ldap_unbind( ld );
}
