/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#include "dsgw.h"
#include "libadmin/libadmin.h"
static void handle_request( int reqmethod );
static void handle_post();

static char *ldiffile, *suffix;
static char *fully_qualified_path( char *path, char *cwd );

main( argc, argv, env )
    int		argc;
    char	*argv[];
#ifdef DSGW_DEBUG
    char	*env[];
#endif
{
    int		reqmethod;

    reqmethod = dsgw_init( argc, argv,  DSGW_METHOD_POST | DSGW_METHOD_GET );
    dsgw_send_header();

#ifdef DSGW_DEBUG
   dsgw_logstringarray( "env", env ); 
#endif

    handle_request( reqmethod );

    exit( 0 );
}


#define DSGWCONFIG_EMPTY_IF_NULL( s )	( (s) == NULL ? "" : (s) )


static void
handle_request( int reqmethod )
{
    FILE	*fp;
    char	**argv, *buf, line[ BIG_LINE ];
    char	*str_valuefmt = " VALUE=\"%s\" ";
    int		did_post, argc;

    buf = dsgw_ch_malloc( strlen( progname ) + 6 );	/* room for ".html\0" */
    sprintf( buf, "%s.html", progname );
    fp = dsgw_open_html_file( buf, DSGW_ERROPT_EXIT );
    free( buf );
    did_post = 0;

    while ( dsgw_next_html_line( fp, line ))  {
	if ( dsgw_parse_line( line, &argc, &argv, 0, dsgw_simple_cond_is_true,
		NULL )) {
	    if ( dsgw_directive_is( line, DRCT_DS_INLINE_POST_RESULTS )) {
		if ( !did_post && reqmethod == DSGW_METHOD_POST ) {
		    handle_post();
		    did_post = 1;
		}
	    } else if ( dsgw_directive_is( line, DS_LDIF_FILE )) {
		fprintf( stdout, str_valuefmt,
			    DSGWCONFIG_EMPTY_IF_NULL( ldiffile ));
	    } else if ( dsgw_directive_is( line, DS_SUFFIX )) {
		fprintf( stdout, str_valuefmt,
			    DSGWCONFIG_EMPTY_IF_NULL( suffix ));
	    }
	}
    }

    fclose( fp );
}


static void
handle_post()
{
    char cmd[BIG_LINE], path[BIG_LINE];
    char    *userdb_path;

    ldiffile = dsgw_get_cgi_var( "ldif", DSGW_CGIVAR_REQUIRED );
    suffix = dsgw_get_cgi_var( "suffix", DSGW_CGIVAR_OPTIONAL );

    /* if the schema checking is off, put out a warning message */ 

    if (( userdb_path = get_userdb_dir()) == NULL ) {
  	dsgw_error( DSGW_ERR_USERDB_PATH, NULL, DSGW_ERROPT_EXIT, 0 ); 
    }

    ldiffile = fully_qualified_path( ldiffile, userdb_path );

    if (gc->gc_localdbconf == NULL) {	/* LDAP server */
	LDAP	*ld;
	char	*dn, *pwd;

	(void)dsgw_init_ldap( &ld, NULL, 0 );
	dsgw_get_bind_credentials( &dn, &pwd );
	if ( dn == NULL ) {
	    dn = "";
	}
	if ( pwd == NULL ) {
	    pwd = "";
	}
        sprintf (cmd, 
#ifdef _WIN32
	    "%s -b \"%s\" -h %s -p %d -D \"%s\" -w \"%s\" \"objectclass=*\" > %s", 
#else
	    "./%s -b %s -h %s -p %d -D %s -w %s \"objectclass=*\" > %s", 
#endif
	    DSGW_LDAPSEARCH, dsgw_escape_for_shell( gc->gc_ldapsearchbase ),
	    dsgw_escape_for_shell( gc->gc_ldapserver ), gc->gc_ldapport,
	    dsgw_escape_for_shell( dn ), dsgw_escape_for_shell( pwd ),
	    ldiffile);
    }
    else {
        /* local database */
        sprintf (cmd, 
#ifdef _WIN32
	    "%s -b \"\" -C %s \"objectclass=*\" > %s",
#else
	    "./%s -b \"\" -C %s \"objectclass=*\" > %s",
#endif
            DSGW_LDAPSEARCH, dsgw_escape_for_shell( gc->gc_localdbconf ),
	    dsgw_escape_for_shell( ldiffile ));
    }
    sprintf (path, "%s%s", userdb_path, DSGW_TOOLSDIR);
    chdir (path);

    fflush (stdout);
    if (system (cmd) == 0){

	/* if local database and suffix is not null, append suffix to 
	   appropriate attributes. */   

	if (( gc->gc_localdbconf != NULL) && (suffix != NULL )) {
	   app_suffix (ldiffile, suffix);
	}
	/*
	 * success: display status message 
	 */
	fprintf( stdout, "<FONT SIZE=\"+1\">\n<P>The ldif file <TT>%s</TT> "
		"has been created.\n</FONT>\n ", ldiffile );
    } else {
	fprintf( stdout, "<FONT SIZE=\"+1\">\n<P>The ldif file <TT>%s</TT> "
		"could not be created.\n</FONT>\n ", ldiffile );
    }

    fputs( "<HR>\n", stdout );
}


/*
 * if "path" does not look like a fully qualified path, make it so.
 * return a malloc'd string.
 */
static char *
fully_qualified_path( char *path, char *cwd )
{
    char	*p;

    if ( path == NULL ) {
	path = "";
    }

    if ( path[ 0 ] != '/'
#ifdef _WIN32
	&& path[ 0 ] != '\\' && ( !isalpha( path[ 0 ] ) || path[ 1 ] != ':' )
#endif
    ) {
	/* partial path */
	p = dsgw_ch_malloc( strlen( path ) + strlen( cwd ) + 2 );
	sprintf( p, "%s%c%s", cwd, DSGW_PATHSEP_CHAR, path );
    } else {
	/* full path */
	p = dsgw_ch_strdup( path );
    }

    return( p );
}
