/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#include "dsgw.h"
static void handle_request( int reqmethod );
static void handle_post();

static char *ldiffile;
static int erase = 0, stop = 1;

main( argc, argv, env )
    int		argc;
    char	*argv[];
#ifdef DSGW_DEBUG
    char	*env[];
#endif
{
    int		reqmethod;

    reqmethod = dsgw_init( argc, argv,  DSGW_METHOD_POST | DSGW_METHOD_GET );
    dsgw_send_header();

#ifdef DSGW_DEBUG
    dsgw_logstringarray( "env", env ); 
#endif

    handle_request( reqmethod );

    exit( 0 );
}


#define DSGWCONFIG_EMPTY_IF_NULL( s )	( (s) == NULL ? "" : (s) )


static void
handle_request( int reqmethod )
{
    FILE	*fp;
    char	**argv, *buf, line[ BIG_LINE ];
    char	*checked = " CHECKED ";
    char	*str_valuefmt = " VALUE=\"%s\" ";
    int		did_post, argc;

    buf = dsgw_ch_malloc( strlen( progname ) + 6 );	/* room for ".html\0" */
    sprintf( buf, "%s.html", progname );
    fp = dsgw_open_html_file( buf, DSGW_ERROPT_EXIT );
    free( buf );
    did_post = 0;

    while ( dsgw_next_html_line( fp, line ))  {
	if ( dsgw_parse_line( line, &argc, &argv, 0, dsgw_simple_cond_is_true,
		NULL )) {
	    if ( dsgw_directive_is( line, DRCT_DS_INLINE_POST_RESULTS )) {
		if ( !did_post && reqmethod == DSGW_METHOD_POST ) {
		    handle_post();
		    did_post = 1;
		}
	    } else if ( dsgw_directive_is( line, DS_LDIF_FILE )) {
		fprintf( stdout, str_valuefmt,
			    DSGWCONFIG_EMPTY_IF_NULL( ldiffile ));
	    } else if ( dsgw_directive_is( line, DS_CHECKED_IF_ERASE )) {
		if ( erase ) {
		    fputs( checked, stdout );
		}
	    } else if ( dsgw_directive_is( line, DS_CHECKED_IF_NOTERASE )) {
		if ( !erase ) {
		    fputs( checked, stdout );
		}
	    } else if ( dsgw_directive_is( line, DS_CHECKED_IF_STOP )) {
		if ( stop ) {
		    fputs( checked, stdout );
		}
	    } else if ( dsgw_directive_is( line, DS_CHECKED_IF_NOTSTOP )) {
		if ( !stop ) {
		    fputs( checked, stdout );
		}
	    }
	}
    }

    fclose( fp );
}


static void
handle_post()
{
    char cmd[ BIG_LINE ], path[BIG_LINE ];
    char *userdb_path;
    int	err;

    ldiffile = dsgw_get_cgi_var( "ldif", DSGW_CGIVAR_REQUIRED );
    erase = dsgw_get_boolean_var( "erase", DSGW_CGIVAR_OPTIONAL, 0 );
    stop = dsgw_get_boolean_var( "stop", DSGW_CGIVAR_OPTIONAL, 0 );

    if (erase) { 
	if ( gc->gc_localdbconf == NULL) {
	    /* don't erase the real ldap database */
	    dsgw_error (DSGW_ERR_DB_ERASE, NULL,DSGW_ERROPT_EXIT,0);
	}
	/* erase the local database */
	if ( erase_db() != 0 ) {
	    return;
	}
    }

    if (( userdb_path = get_userdb_dir()) == NULL ) {
        dsgw_error( DSGW_ERR_USERDB_PATH, NULL, DSGW_ERROPT_EXIT, 0 );
    }

    if (gc->gc_localdbconf == NULL) {	/* LDAP server */
	LDAP	*ld;
	char	*dn, *pwd;

	(void)dsgw_init_ldap( &ld, NULL, 0 );
	dsgw_get_bind_credentials( &dn, &pwd );
	if ( dn == NULL ) {
	    dn = "";
	}
	if ( pwd == NULL ) {
	    pwd = "";
	}

#ifdef _WIN32
	sprintf (cmd, "%s -a %s -h %s -p %d -D \"%s\" -w \"%s\" -f %s", 
#else
	sprintf (cmd, "./%s -a %s -h %s -p %d -D %s -w %s -f %s", 
#endif
		DSGW_LDAPMODIFY, stop?"":"-c",
		dsgw_escape_for_shell( gc->gc_ldapserver ), gc->gc_ldapport,
		dsgw_escape_for_shell( dn ), dsgw_escape_for_shell( pwd ),
		dsgw_escape_for_shell( ldiffile ));

    } else { /* local database */
#ifdef _WIN32
    	sprintf (cmd, "%s -a %s -C %s -f %s",
#else
    	sprintf (cmd, "./%s -a %s -C %s -f %s",
#endif
		DSGW_LDAPMODIFY, stop?"":"-c",
		dsgw_escape_for_shell( gc->gc_localdbconf ),
		dsgw_escape_for_shell( ldiffile ));
    }
    sprintf (path, "%s%s", userdb_path, DSGW_TOOLSDIR);
    chdir ( path );
    fputs( "<PRE>\n", stdout );
    fflush (stdout);

    err =  system (cmd);

    fputs( "</PRE><FONT SIZE=\"+1\">\n<P>", stdout );
    if (err == 0) { 
	/*
	 * success: display status message 
	 */
	fputs( "The ldif file has been added.\n</FONT>\n ", stdout );
    }
    else {
	fputs( "The ldif file could not be added.\n</FONT>\n ", stdout );
    }

    fputs( "<HR>\n", stdout );
}

