/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#include "dsgw.h"
#include "dbtdsgw.h"

static int isauth = 0, isexp = 0;

void emit_authsearchform( char *authdesturl, char *authhint );
void emit_authpwform( char *binddn, char *authdesturl );
void emit_authinfo( int isEditing, int isPwForm, char *binddn );

void
dsgw_emit_auth_form( char *binddn )
{
    char *authdesturl = NULL;
    
    /*
     * Pull out the "authdesturl" form variable, if it's there.
     */
    authdesturl = dsgw_get_cgi_var( "authdesturl", DSGW_CGIVAR_OPTIONAL );

    /*
     * If dn is NULL, then we don't know who we want to bind as yet.
     * Generate a simplified search form.  This form needs to post:
     * mode=auth
     * searchstring
     * authdesturl
     *
     * If dn was given, then prompt for the password.  Needs to post:
     * password
     * authdesturl
     * binddn
     */
    if ( binddn == NULL ) {
	emit_authsearchform( authdesturl,
		dsgw_get_cgi_var( "authhint", DSGW_CGIVAR_OPTIONAL ));
    } else {
	emit_authpwform( binddn, authdesturl );
    }
}



void
emit_authsearchform( char *authdesturl, char *authhint )
{
    fprintf( stdout, "<HTML>\n<HEAD>\n<TITLE>%s</TITLE>\n"
	"<SCRIPT NAME=\"JavaScript\">\n"
	"<!-- Hide from non-JavaScript browsers\n"
	"function doUnauth()\n"
	"{\n"
	"    if ( confirm( '%s' )) {\n"
	"        window.location.href='/ds/unauth';\n"
	"    }\n"
	"}\n",
	XP_GetClientStr(DBT_authenticate_),
	XP_GetClientStr(DBT_discardAuthenticationCredentials_) );

#ifdef notdef /* This doesn't work with Navigator 2.x */
    printf( "function checkSS(sform)\n" );
    printf( "{\n" );
    printf( "    if (sform.searchstring.value == null || sform.searchstring.value == \"\") {\n" );
    printf( "        alert(\"%s\");\n", XP_GetClientStr(DBT_youDidNotSupplyASearchString_) );
    printf( "        return false;\n" );
    printf( "    }\n" );
    printf( "}\n" );
#endif /* notdef */
    fputs( "function init()\n"
	"{\n"
	"    document.authSearchForm.searchstring.focus();\n"
	"    if (top.history.length == 1 && top.opener != null && top.opener.location.href != "
	"top.location.href) {\n"
	"        if (top.closewin == true) {\n"
	"            top.opener.document.clear();\n"
	"            top.opener.document.open();\n"
	"            top.opener.document.write('');\n"
	"            top.opener.document.close();\n"
	"        }\n"
	"    }\n"
	"    top.closewin = false;\n"
	"}\n"
	"// End hiding -->\n"
	"</SCRIPT>\n"
	"</HEAD>\n"
	"<BODY onLoad=\"setTimeout('init()', 10);\">\n", stdout  );
    emit_authinfo( authdesturl != NULL, 0, NULL );
    fputs( "<FORM NAME=\"authSearchForm\" ACTION=\"/ds/dosearch\" "
#ifdef notdef /* This doesn't work with Navigator 2.x */
	    "onSubmit=\"return checkSS(this);\" method=\"post\">\n", stdout );
#else /* notdef */
	    "method=\"post\">\n", stdout );
#endif /* notdef */
    fputs( "<INPUT TYPE=\"hidden\" NAME=\"mode\" VALUE=\"auth\">\n", stdout );
    if ( authdesturl != NULL ) {
	printf( "<INPUT TYPE=\"hidden\" NAME=\"authdesturl\" VALUE=\"%s\">\n",
		authdesturl );
    }
    fputs(
	XP_GetClientStr(DBT_theFirstStepInAuthenticatingToTh_), stdout );
    printf( "<INPUT NAME=\"searchstring\" VALUE=\"%s\" SIZE=40>\n",
	    ( authhint == NULL ) ? "" : authhint );
    fputs( "<P>\n"
	"<CENTER>\n"
	"<TABLE BORDER=1 WIDTH=100%%>\n"
	"<TR>\n", stdout );
    if ( authdesturl == NULL ) {
	fprintf( stdout, "<TD ALIGN=CENTER WIDTH=50%%>\n"
	    "<INPUT TYPE=\"submit\" VALUE=\"%s\">\n"
	    "<TD ALIGN=CENTER WIDTH=50%%>\n",
	    XP_GetClientStr(DBT_continue_) );
	dsgw_emit_helpbutton( "AUTHHELP_ID" );
    } else {
	fprintf( stdout, "<TD ALIGN=CENTER WIDTH=33%%>\n"
	    "<INPUT TYPE=\"submit\" VALUE=\"%s\">\n"
	    "<TD ALIGN=CENTER WIDTH=33%%>\n"
	    "<INPUT TYPE=\"button\" VALUE=\"%s\" "
	    "onClick=\"parent.close();\">\n"
	    "<TD ALIGN=CENTER WIDTH=34%%>\n", 
	    XP_GetClientStr(DBT_continue_1), XP_GetClientStr(DBT_cancel_) );
	dsgw_emit_helpbutton( "AUTHHELP_ID" );
    }
    fputs( "</TABLE>\n"
	"</FORM>\n", stdout );

    if ( gc->gc_rootdn != NULL ) {
	fputs( "<P><FORM NAME=\"AuthAsRootDNForm\" METHOD=\"POST\" "
		"ACTION=\"/ds/auth\">\n", stdout );
	if ( authdesturl != NULL ) {
	    printf( "<INPUT TYPE=\"hidden\" NAME=\"authdesturl\" "
		    "VALUE=\"%s\">\n", authdesturl );
	}
	fprintf( stdout,
		"<INPUT TYPE=\"hidden\" NAME=\"authasrootdn\" VALUE=\"true\">\n"
		"<INPUT TYPE=\"submit\" "
		"VALUE=\"%s"
		"</FORM>\n",
		XP_GetClientStr(DBT_authenticateAsDirectoryManagerNb_) );
    }

    fputs( "</BODY>\n"
	"</HTML>\n", stdout );
    fflush( stdout );
}

void
emit_authpwform( char *binddn, char *authdesturl )
{
    char *ufn;
    char **xdn;

    dsgw_form_unescape( binddn );
    ufn = ldap_dn2ufn( binddn );

    fprintf( stdout, "<HTML>\n<HEAD>\n<TITLE>%s</TITLE>\n"
	"<SCRIPT NAME=\"JavaScript\">\n"
	"<!-- Hide from non-JavaScript browsers\n", XP_GetClientStr(DBT_authenticate_1) );

    /* doUnauth function - invoke CGI which tosses cookies. */
    fprintf( stdout, "function doUnauth()\n"
	"{\n"
	"    if ( confirm( '%s' )) {\n"
	"        window.location.href='/ds/unauth';\n"
	"    }\n"
	"}\n"
	"// End hiding -->\n"
	"</SCRIPT>\n\n"
	"</HEAD>\n"
	"<BODY onLoad=\"document.authPwForm.password.focus();\">\n",
	XP_GetClientStr(DBT_discardAuthenticationCredentials_1) );

    emit_authinfo( authdesturl != NULL, 1, binddn );

    fputs( "<FORM NAME=\"authPwForm\" ACTION=\"/ds/doauth\" "
	    "METHOD=\"post\">\n", stdout );
    if ( authdesturl != NULL ) {
	printf( "<input type=\"hidden\" name=\"authdesturl\" value=\"%s\">\n",
		authdesturl );
    }
    printf( "<input type=\"hidden\" name=\"binddn\" value=\"%s\">\n",
	    binddn );
    fputs( "<P>\n", stdout );

    xdn = ldap_explode_dn( binddn, 1 );
    printf( XP_GetClientStr(DBT_passwordForBSB_), xdn[ 0 ]);
    ldap_value_free( xdn );

    fputs( "<INPUT NAME=\"password\" TYPE=\"password\" SIZE=16>\n"
	"<P>\n"
	"<CENTER>\n"
	"<TABLE BORDER=2 WIDTH=100%%>\n"
	"<TR>\n", stdout );
    if ( authdesturl == NULL ) {
	fprintf( stdout, "<TD ALIGN=CENTER WIDTH=50%%>\n"
	    "<INPUT TYPE=\"submit\" VALUE=\"%s\">\n"
	    "<TD ALIGN=CENTER WIDTH=50%%>\n",
	    XP_GetClientStr(DBT_continue_2) );
	dsgw_emit_helpbutton( "AUTHHELP_PW" );
    } else {
	fprintf( stdout, "<TD ALIGN=CENTER WIDTH=33%%>\n"
	    "<INPUT TYPE=\"submit\" VALUE=\"%s\">\n"
	    "<TD ALIGN=CENTER WIDTH=33%%>\n"
	    "<INPUT TYPE=\"button\" VALUE=\"%s\" "
	    "onClick=\"parent.close();\">\n"
	    "<TD ALIGN=CENTER WIDTH=34%%>\n",
	    XP_GetClientStr(DBT_continue_3), XP_GetClientStr(DBT_cancel_1) );
	dsgw_emit_helpbutton( "AUTHHELP_PW" );
    }
    fputs( "</TABLE>\n"
	"</FORM>\n"
	"</BODY>\n"
	"</HTML>\n", stdout );
    fflush( stdout );
    free( ufn );
}


static void
emit_authinfo( int isEditing, int isPwForm, char *binddn )
{
    char *cookie, *dn, *rndstr, *pw;
    int rc;

    /* try to get the DN the user is bound as, and determine if authentication
     * credentials have expired.
     */

    if (( cookie = dsgw_get_auth_cookie()) != NULL ) {
	if ( dsgw_parse_cookie( cookie, &rndstr, &dn ) == 0 ) {
	    if ( dn == NULL ) {
		isauth = 0;
	    } else {
		if (( rc = dsgw_ckdn2passwd( rndstr, dn, &pw )) == 0 ) {
		    isauth = 1;
		} else {
		    isauth = 0;
		    if ( rc == DSGW_CKDB_EXPIRED ) {
			isexp = 1;
		    }
		}
	    }
	} else {
	    isauth = 0;
	}
    } else {
	isauth = 0;
    }

    fprintf( stdout, "<CENTER>\n"
	"<FONT SIZE=+2>%s</FONT>\n"
	"</CENTER>\n"
	"<p>", XP_GetClientStr(DBT_authenticateLogInToTheDirectory_) );

    if ( isPwForm ) {
	char *ufn;

	fputs( XP_GetClientStr(DBT_youAreAboutToAuthenticateToTheDi_), stdout);
	ufn = ldap_dn2ufn( binddn );
	fputs( "<B>", stdout );
	fputs( ufn, stdout );
	free( ufn );
	fputs( "</B>. ", stdout );
	fputs( XP_GetClientStr(DBT_toCompleteTheAuthenticationProce_), stdout );
    } else if ( isEditing ) {
	fputs( XP_GetClientStr(DBT_beforeYouCanEditOrAddEntriesYouM_), stdout );
    } else {
	fputs( XP_GetClientStr(DBT_fromThisScreenYouMayAuthenticate_), stdout );
    }
    if ( isEditing ) {
	return;
    }
    fprintf( stdout, "<HR>\n"
	"<CENTER>\n"
	"<FONT SIZE=+2>%s</FONT>\n"
	"</CENTER>\n"
	"<P>\n", XP_GetClientStr(DBT_authenticationStatus_) );
    if ( isauth ) {
	char *ufn;

	fputs( XP_GetClientStr(DBT_FormNyouAreCurrentlyAuthenticate_), stdout );
	ufn = ldap_dn2ufn( dn );
	printf( "<b>%s</b>\n", ufn );
	free( ufn );
	fprintf( stdout, "%s<BR>"
	    "<CENTER>\n"
	    "<INPUT TYPE=BUTTON "
	    "VALUE=\"%s\""
	    "onClick=\"doUnauth();\">\n"
	    "</FORM>\n"
	    "</CENTER>\n"
	    "<HR>\n",
	    XP_GetClientStr(DBT_NifYouWishToDiscardYourAuthentic_),
	    XP_GetClientStr(DBT_discardAuthenticationCredentials_2) );
    } else if ( isexp ) {
	fputs( XP_GetClientStr(DBT_yourAuthenticationCredentialsFor_),
               stdout );
	printf( "<b>%s</b> ", dn );
	fputs( XP_GetClientStr(DBT_haveExpiredN_), stdout );
    } else {
	fputs( XP_GetClientStr(DBT_currentlyYouAreNotAuthenticatedT_),
               stdout );
    }
}
