/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#include "dsgw.h"
#include "dbtdsgw.h"

/*
 * Note: the value of the following DSGW_ATTRHTML_XXX #defines must match
 * their position in the attrhtmltypes[] and attrhtmlvals[] arrays.
 */
#define DSGW_ATTRHTML_HIDDEN		0
#define DSGW_ATTRHTML_TEXT		1
#define DSGW_ATTRHTML_TEXTAREA		2
#define DSGW_ATTRHTML_RADIO		3
#define DSGW_ATTRHTML_CHECKBOX		4
#define DSGW_ATTRHTML_PASSWORD		5
static char *attrhtmltypes[] = {
	"hidden",
	"text",
	"textarea",
	"radio",
	"checkbox",
	"password",
	NULL
};
static int attrhtmlvals[] = {
	DSGW_ATTRHTML_HIDDEN,
	DSGW_ATTRHTML_TEXT,
	DSGW_ATTRHTML_TEXTAREA,
	DSGW_ATTRHTML_RADIO,
	DSGW_ATTRHTML_CHECKBOX,
	DSGW_ATTRHTML_PASSWORD,
};

#define DSGW_ATTROPT_SORT	0x00000001
#define DSGW_ATTROPT_NOLINK	0x00000002
#define DSGW_ATTROPT_DNTAGS	0x00000004
#define DSGW_ATTROPT_DATEONLY	0x00000008	/* only for syntax=time */
#define DSGW_ATTROPT_READONLY	0x00000010	/* over-rides ..._EDITABLE */
#define DSGW_ATTROPT_DNPICKER	0x00000020	/* display dns for find-n-add */
#define DSGW_ATTROPT_UNIQUE	0x00000040	/* attr values must be unique */
#define DSGW_ATTROPT_LANGTAG	0x00000080	/* include ;lang-LANG option */
#define DSGW_ATTROPT_PHONETIC	0x00000100	/* include ;phonetic option */
/* I18n addition */
#define DSGW_ATTROPT_ENTAG	0x00000200	/* include ;lang-en option */
#define DSGW_ATTROPT_EN		0x00000400	/* attr w/ lang-en */
#define DSGW_ATTROPT_LANG	0x00000800	/* attr w/ lang-<LANG> */
#define DSGW_ATTROPT_PHNTC	0x00001000	/* attr w/ phonetic */
#define DSGW_ATTROPT_QUOTED	0x00002000	/* output quoted string */
#define DSGW_ATTROPT_NOFALLBACK	0x00004000	/* no fall back in get_values */
#define DSGW_ATTROPT_LANGINFO	0x00008000	/* return langtag */
/* I18n addition */
#define DSGW_ATTROPT_EDITABLE	0x10000000	/* not exposed in HTML */
#define DSGW_ATTROPT_ADDING	0x20000000	/* not exposed in HTML */
#define DSGW_ATTROPT_LINK2EDIT	0x40000000	/* not exposed in HTML */
static char *attroptions[] = {
	"sort",
	"nolink",
	"dntags",
	"dateonly",
	"readonly",
	"dnpicker",
	"unique",
	"langtag",
	"phonetic",
	/* I18n addition */
	"entag",
	/* these are ugly: needed for generate op_en/lang/phonetic_ATTR */
	"en",		
	"lang",
	"phntc",
	"quoted",
	"nofallback",
	"langinfo",
	/* I18n addition */
	NULL
};

static unsigned long attroptvals[] = {
	DSGW_ATTROPT_SORT,
	DSGW_ATTROPT_NOLINK,
	DSGW_ATTROPT_DNTAGS,
	DSGW_ATTROPT_DATEONLY,
	DSGW_ATTROPT_READONLY,
	DSGW_ATTROPT_DNPICKER,
	DSGW_ATTROPT_UNIQUE,
	DSGW_ATTROPT_LANGTAG,
	DSGW_ATTROPT_PHONETIC,
	/* I18n addtion */
        DSGW_ATTROPT_ENTAG,
        DSGW_ATTROPT_EN,
        DSGW_ATTROPT_LANG,
        DSGW_ATTROPT_PHNTC,
        DSGW_ATTROPT_QUOTED,
        DSGW_ATTROPT_NOFALLBACK,
        DSGW_ATTROPT_LANGINFO
	/* I18n addtion */
};


#define DSGW_ATTRARG_ATTR	"attr"
#define DSGW_ATTRARG_SYNTAX	"syntax"
#define DSGW_ATTRARG_HTMLTYPE	"type"
#define DSGW_ATTRARG_OPTIONS	"options"
#define DSGW_ATTRARG_DEFAULT	"defaultvalue"
#define DSGW_ATTRARG_WITHIN	"within"	/* overrides href & hrefextra */
#define DSGW_ATTRARG_HREF	"href"
#define DSGW_ATTRARG_HREFEXTRA	"hrefextra"
#define DSGW_ATTRARG_LABEL	"label"		/* only used with syntax=dn */
#define DSGW_ATTRARG_DNCOMP	"dncomponents"	/* only used with syntax=dn */
#define DSGW_ATTRARG_TRUESTR	"true"		/* only used with syntax=bool */
#define DSGW_ATTRARG_FALSESTR	"false"		/* only used with syntax=bool */
#define DSGW_ATTRARGS_SIZE	"size"
#define DSGW_ATTRARGS_ROWS	"rows"
#define DSGW_ATTRARGS_COLS	"cols"
#define DSGW_ATTRARGS_NUMFIELDS	"numfields"
#define DSGW_ATTRARGS_VALUE	"value"

#define DSGW_POSTEDVARARG_NAME		"name"
#define DSGW_ARG_BUTTON_LABEL		"label"
#define DSGW_ARG_BUTTON_PROMPT		"prompt"
#define DSGW_ARG_BUTTON_TEMPLATE	"template"
#define DSGW_ARG_BUTTON_CHECKSUBMIT	"checksubmit"
#define DSGW_ARG_BUTTON_TOPIC		"topic"
#define DSGW_ARG_DNEDIT_LABEL		"label"
#define DSGW_ARG_DNEDIT_TEMPLATE	"template"
#define DSGW_ARG_DNEDIT_ATTR		"attr"
#define DSGW_ARG_DNEDIT_DESC		"desc"

#define	DSGW_ARG_FABUTTON_LABEL		"label"
#define	DSGW_ARG_FABUTTON_ATTRNAME	"attr"
#define	DSGW_ARG_FABUTTON_ATTRDESC	"attrdesc"

#define DSGW_ARG_AVSET_SET		"set"

/*
 * structure used simply to avoid passing a lot of parameters in call to
 * the attribute syntax handlers
 */
struct dsgw_attrdispinfo {
    struct attr_handler	*adi_handlerp;
    char		*adi_attr;
    int			adi_argc;
    char		**adi_argv;
    char		**adi_vals;
    int			adi_htmltype;
    unsigned long	adi_opts;
    char		*adi_entrydn;
};

typedef void (*attrdisplay)( struct dsgw_attrdispinfo *adip );
typedef void (*attredit)( struct dsgw_attrdispinfo *adip );
typedef int (*attrcompare)( const char **a, const char **b );

struct attr_handler {
    char		*ath_syntax;	/* dn, tel, cis, etc. */
    attrdisplay		ath_display;	/* function to display values */
    attredit		ath_edit;	/* function to display for editing */
    attrcompare		ath_compare;	/* compare function */
};

/* functions local to this file */
static void output_prelude( dsgwtmplinfo *tip );
static void output_nonentry_line( dsgwtmplinfo *tip, char *line );
static struct attr_handler *syntax2attrhandler( char *syntax );
static int numfields( int argc, char **argv, int valcount );
static void element_sizes( int argc, char **argv, char **vals, int valcount,
	int *rowsp, int *colsp );
#define DSGW_TEXTOPT_FOCUSHANDLERS	0x0001
#define DSGW_TEXTOPT_CHANGEHANDLERS	0x0002
static void output_text_elements( int argc, char **argv, char *attr,
	char **vals, char *prefix, int htmltype, unsigned long opts );
static void output_textarea( int argc, char **argv, char *attr,
	char **vals, int valcount, char *prefix, unsigned long opts );
static void output_text_checkbox_or_radio( struct dsgw_attrdispinfo *adip,
	char *prefix, int htmltype );
static void do_attribute( dsgwtmplinfo *tip, char *dn, unsigned long dispopts,
	int argc, char **argv );
static void do_attrvalset( dsgwtmplinfo *tip, char *dn, unsigned long dispopts,
	int argc, char **argv );
static void do_editbutton( char *dn, char *encodeddn, int argc, char **argv );
static void do_savebutton( unsigned long dispopts, int argc, char **argv );
static void do_deletebutton( int argc, char **argv );
static void do_renamebutton( char *dn, int argc, char **argv );
static void do_editasbutton( int argc, char **argv );
static void do_dneditbutton( unsigned long dispopts, int argc, char **argv );
static void do_avaeditbutton( unsigned long dispopts, int argc, char **argv );
static void do_postedvalue( int argc, char **argv );
static void do_searchdesc( dsgwtmplinfo *tip );
static void do_passwordfield( unsigned long dispopts, int argc, char **argv,
	char *fieldname );
static void do_helpbutton( unsigned long dispopts, int argc, char **argv );
static void do_closebutton( unsigned long dispopts, int argc, char **argv );
static void do_viewswitcher( char *template, char *dn, int argc, char **argv );
static int did_output_as_special( int argc, char **argv, char *label,
	char *val );
static char *time2text( char *ldtimestr, int dateonly );
static long gtime( struct tm *tm );
static int looks_like_dn( char *s );
static int condition_is_true( int argc, char **argv, void *arg );


/* attribute syntax handler routines */
static void ntdomain_display( struct dsgw_attrdispinfo *adip );
static void ntdomain_edit( struct dsgw_attrdispinfo *adip );
static void str_display( struct dsgw_attrdispinfo *adip );
static void str_edit( struct dsgw_attrdispinfo *adip );
static void dn_display( struct dsgw_attrdispinfo *adip );
static void dn_edit( struct dsgw_attrdispinfo *adip );
static void mail_display( struct dsgw_attrdispinfo *adip );
static void mls_display( struct dsgw_attrdispinfo *adip );
static void mls_edit( struct dsgw_attrdispinfo *adip );
static char *mls_convertlines( char *val, char *sep, int *linesp );
static void url_display( struct dsgw_attrdispinfo *adip );
static void bool_display( struct dsgw_attrdispinfo *adip );
static void bool_edit( struct dsgw_attrdispinfo *adip );
static void time_display( struct dsgw_attrdispinfo *adip );
static void ava_display( struct dsgw_attrdispinfo *adip );
static int caseexact_sort( const char **a, const char **b );


/* static variables */
#define DSGW_MOD_PREFIX_NORMAL		0
#define DSGW_MOD_PREFIX_UNIQUE		1
/* I18n addtion */
#define DSGW_MOD_PREFIX_EN		2
#define DSGW_MOD_PREFIX_LANG		3
#define DSGW_MOD_PREFIX_PHNTC		4
#define DSGW_MOD_PREFIX_EN_UNIQUE	5
#define DSGW_MOD_PREFIX_LANG_UNIQUE	6
#define DSGW_MOD_PREFIX_PHNTC_UNIQUE	7

static char *replace_prefixes[] =
    { "replace_", "replace_unique_",
      "replace_en_", "replace_lang_", "replace_phonetic_",
      "replace_unique_en_", "replace_unique_lang_",
      "replace_unique_phonetic_" };
static char *replace_mls_prefixes[] =
    { "replace_mls_", "replace_mls_unique_",
      "replace_mls_en_", "replace_mls_lang_", "replace_mls_phonetic_",
      "replace_mls_unique_en_", "replace_mls_unique_lang_",
      "replace_mls_unique_phonetic_" };
static char *add_prefixes[] =
    { "add_", "add_unique_",
      "add_en_", "add_lang_", "add_phonetic_",
      "add_unique_en_", "add_unique_lang_", "add_unique_phonetic_" };
static char *add_mls_prefixes[] =
    { "add_mls_", "add_mls_unique_",
      "add_mls_en_", "add_mls_lang_", "add_mls_phonetic_",
      "add_mls_unique_en_", "add_mls_unique_lang_",
      "add_mls_unique_phonetic_" };
/* I18n addtion */

struct attr_handler attrhandlers[] = {
    { "cis",	str_display,	str_edit,	ldap_sort_strcasecmp	},
    { "dn",	dn_display,	dn_edit,	ldap_sort_strcasecmp	},
    { "mail",	mail_display,	str_edit,	ldap_sort_strcasecmp	},
    { "mls",	mls_display,	mls_edit,	ldap_sort_strcasecmp	},
    { "tel",	str_display,	str_edit,	ldap_sort_strcasecmp	},
    { "url",	url_display,	str_edit,	caseexact_sort       	},
    { "ces",	str_display,	str_edit,	caseexact_sort       	},
    { "bool",	bool_display,	bool_edit,	ldap_sort_strcasecmp	},
    { "time",	time_display,	str_edit,	ldap_sort_strcasecmp	},
    { "ntdomain", ntdomain_display, str_edit, ldap_sort_strcasecmp	},
    { "ava", 	ava_display, 	str_edit, 	ldap_sort_strcasecmp	},
};
#define DSGW_AH_COUNT ( sizeof( attrhandlers ) / sizeof( struct attr_handler ))


static char *
template_filename( int tmpltype, char *template )
{
    char	*fn, *prefix, *suffix = ".html";

    if ( tmpltype == DSGW_TMPLTYPE_LIST ) {
	prefix = DSGW_CONFIG_LISTPREFIX;
    } else if ( tmpltype == DSGW_TMPLTYPE_EDIT ) {
	prefix = DSGW_CONFIG_EDITPREFIX;
    } else if ( tmpltype == DSGW_TMPLTYPE_ADD ) {
	prefix = DSGW_CONFIG_ADDPREFIX;
    } else {
	prefix = DSGW_CONFIG_DISPLAYPREFIX;
    }

    fn = dsgw_ch_malloc( strlen( prefix ) + strlen( template )
	    + strlen( suffix ) + 1 );
    sprintf( fn, "%s%s%s", prefix, template, suffix );

    return( fn );
}


dsgwtmplinfo *
dsgw_display_init( int tmpltype, char *template, unsigned long options )
{
    dsgwtmplinfo	*tip;
    int			argc, attrcount, sortbyindex, skip_line, in_entry;
    int			i, found;
    char		**argv, *attr, *filename, line[ BIG_LINE ];

    tip = (dsgwtmplinfo *)dsgw_ch_malloc( sizeof( dsgwtmplinfo ));
    memset( tip, 0, sizeof( dsgwtmplinfo ));
    tip->dsti_type = tmpltype;
    tip->dsti_searchdesc = dsgw_ch_strdup( "" );
    tip->dsti_options = options;
    tip->dsti_template = dsgw_ch_strdup( template );

    if (( options & DSGW_DISPLAY_OPT_ADDING ) != 0 ) {
	options |= DSGW_DISPLAY_OPT_EDITABLE;	/* add implies editable */

	if ( tmpltype != DSGW_TMPLTYPE_ADD ) {
	    /*
	     * if we are going to display an "add" view of an entry and
	     * an add template has not been explicitly requested, first look
	     * for a file called "add-TEMPLATE.html" and fall back on using
	     * whatever we would use if just editing an existing entry.
	     */
	    filename = template_filename( DSGW_TMPLTYPE_ADD, template );
	    tip->dsti_fp = dsgw_open_html_file( filename, DSGW_ERROPT_IGNORE );
	    free( filename );
	}
    }

    if ( tip->dsti_fp == NULL && ( options & DSGW_DISPLAY_OPT_EDITABLE ) != 0
	    && tmpltype != DSGW_TMPLTYPE_EDIT ) {
	/*
	 * if we are going to display an editable view of an entry and
	 * an edit template has not been explicitly requested, first look
	 * for a file called "edit-TEMPLATE.html" and fall back on using
	 * "list-TEMPLATE.html" or "display-TEMPLATE.html", as indicated by
	 * the value of tmpltype.
	 */
	filename = template_filename( DSGW_TMPLTYPE_EDIT, template );
	tip->dsti_fp = dsgw_open_html_file( filename, DSGW_ERROPT_IGNORE );
	free( filename );
    }

    if ( tip->dsti_fp == NULL ) {
	filename = template_filename( tmpltype, template );
	tip->dsti_fp = dsgw_open_html_file( filename, DSGW_ERROPT_EXIT );
	free( filename );
    }

    tip->dsti_preludelines = dsgw_savelines_alloc();
    tip->dsti_entrylines = dsgw_savelines_alloc();
    in_entry = sortbyindex = 0;

    /* prime attrs array so we always retrieve objectClass values */
    attrcount = 1;
    tip->dsti_attrs = (char **)dsgw_ch_realloc( tip->dsti_attrs,
	    2 * sizeof( char * ));
    tip->dsti_attrs[ 0 ] = dsgw_ch_strdup( DSGW_ATTRTYPE_OBJECTCLASS );
    tip->dsti_attrs[ 1 ] = NULL;

    while ( dsgw_next_html_line( tip->dsti_fp, line )) {
	skip_line = 0;
	if ( dsgw_parse_line( line, &argc, &argv, 1, condition_is_true, tip )) {
	    if ( in_entry && dsgw_directive_is( line, DRCT_DS_ENTRYEND )) {
		dsgw_argv_free( argv );
		break;	/* the rest is read inside dsgw_display_done */
	    }
	    if ( dsgw_directive_is( line, DRCT_DS_ENTRYBEGIN )) {
		in_entry = skip_line = 1;
	    } else if ( dsgw_directive_is( line, DRCT_DS_ATTRIBUTE ) ||
		    dsgw_directive_is( line, DRCT_DS_ATTRVAL_SET )) {
		if (( attr = get_arg_by_name( DSGW_ATTRARG_ATTR, argc,
			argv )) != NULL ) {
		    tip->dsti_attrs = (char **)dsgw_ch_realloc( tip->dsti_attrs,
			    ( attrcount + 2 ) * sizeof( char * ));
		    tip->dsti_attrs[ attrcount++ ] = dsgw_ch_strdup( attr );
		    tip->dsti_attrs[ attrcount ] = NULL;
		}
	    } else if ( dsgw_directive_is( line, DRCT_DS_SORTENTRIES ) &&
		    sortbyindex < 2 ) {
		if (( attr = get_arg_by_name( DSGW_ATTRARG_ATTR, argc,
			argv )) == NULL ) {
		    /* no attr=, so sort by DN */
		    tip->dsti_sortbyattrs[sortbyindex] = NULL;
		} else {
		    tip->dsti_sortbyattrs[sortbyindex] = dsgw_ch_strdup( attr );
		} 
		++sortbyindex;
		skip_line = 1;	/* completely done with directive */
	    } 
	    dsgw_argv_free( argv );
	}

	if ( !skip_line ) {
	    if ( in_entry ) {	/* in entry */
		dsgw_savelines_save( tip->dsti_entrylines, line );
	    } else {		/* in prelude */
		dsgw_savelines_save( tip->dsti_preludelines, line );
	    }
	}
    }

    if ( attrcount > 0 ) {
	tip->dsti_attrflags = (unsigned long *)dsgw_ch_malloc( attrcount
		* sizeof( unsigned long ));
	memset( tip->dsti_attrflags, 0, attrcount * sizeof( unsigned long ));
    }

    /*
     * Add the sortattrs to the list of attrs retrieved, if it's not
     * already in the list.
     */
    i = found = 0;
    for ( sortbyindex = 0; tip->dsti_sortbyattrs[ sortbyindex ] != NULL;
	    ++sortbyindex ) {
	for ( i = 0; i < attrcount; i++ ) {
	    if ( !strcasecmp( tip->dsti_sortbyattrs[ sortbyindex ],
		    tip->dsti_attrs[ i ])) {
		found = 1;
		break;
	    }
	}
	if ( !found ) {
	    tip->dsti_attrs = (char **)dsgw_ch_realloc( tip->dsti_attrs,
		    ( attrcount + 2 ) * sizeof( char * ));
	    tip->dsti_attrs[ attrcount++ ] =
		    dsgw_ch_strdup( tip->dsti_sortbyattrs[ sortbyindex ] );
	    tip->dsti_attrs[ attrcount ] = NULL;
	}
    }

    return( tip );
}


void
dsgw_display_entry( dsgwtmplinfo *tip, LDAP *ld, LDAPMessage *entry, char *dn )
{
    int		argc, editable, adding;
    char        **argv, *encodeddn, *line;

    editable = (( tip->dsti_options & DSGW_DISPLAY_OPT_EDITABLE ) != 0 );
    adding = (( tip->dsti_options & DSGW_DISPLAY_OPT_ADDING ) != 0 );

    if ( entry == NULL && !adding ) {
	dsgw_error( DSGW_ERR_MISSINGINPUT, NULL, DSGW_ERROPT_EXIT, 0 );
    }

    tip->dsti_ld = ld;
    tip->dsti_entry = entry;

#ifdef DSGW_DEBUG
    {
    BerElement *p;
    char *ret;
    if (( ret = ldap_first_attribute( ld, entry, &p )) != NULL ) {
	dsgw_log( "attr: %s\n", ret );
	ldap_memfree( ret );
	while (( ret = ldap_next_attribute( ld, entry, p )) != NULL ) {
	    dsgw_log( "attr: %s\n", ret );
	    ldap_memfree( ret );
	}
    }
    }
#endif

    if ( dn == NULL ) {
	if ( entry == NULL ) {
	    dn = "dn=unknown";
	} else if (( dn = ldap_get_dn( ld, entry )) == NULL ) {
	    dsgw_ldap_error( ld, DSGW_ERROPT_EXIT );
	}
    }
    tip->dsti_entrydn = dsgw_ch_strdup( dn );
    encodeddn = dsgw_strdup_escaped( dn );

    if ( adding ) {
	tip->dsti_rdncomps = dsgw_rdn_values( dn );
    }

    if ( tip->dsti_preludelines != NULL ) {
	output_prelude( tip );
    }

    dsgw_savelines_rewind( tip->dsti_entrylines );
    while (( line = dsgw_savelines_next( tip->dsti_entrylines )) != NULL ) {
	if ( dsgw_parse_line( line, &argc, &argv, 0, condition_is_true, tip )) {
	    if ( dsgw_directive_is( line, DRCT_DS_ATTRIBUTE )) {
		do_attribute( tip, dn, tip->dsti_options, argc, argv );

	    } else if ( dsgw_directive_is( line, DRCT_DS_ATTRVAL_SET )) {
		do_attrvalset( tip, dn, tip->dsti_options, argc, argv );

	    } else if ( dsgw_directive_is( line, DRCT_DS_EMIT_BASE_HREF )) {
		char *p;
		char *sname = dsgw_ch_strdup( getenv( "SCRIPT_NAME" ));
		if (( p = strrchr( sname, '/' )) != NULL ) {
		    *p = '\0';
		}
		fprintf( stdout, "<BASE HREF=\"%s%s/\">\n",
			getenv( "SERVER_URL" ), sname );

	    } else if ( dsgw_directive_is( line, DRCT_DS_BEGIN_DNSEARCHFORM )) {
		fputs( "<FORM METHOD=POST", stdout );
		fprintf( stdout,
			" NAME=\"searchForm\" ACTION=\"%s\" ",
			dsgw_getvp( DSGW_CGINUM_DOSEARCH ));
		fputs( "TARGET=stagingFrame onSubmit=\"parent."
			"processSearch(searchForm);\">\n",
			stdout );
		fprintf( stdout, "<INPUT TYPE=\"hidden\" "
			"NAME=\"dn\" VALUE=\"%s\";>\n", encodeddn );

	    } else if ( dsgw_directive_is( line, DRCT_DS_BEGIN_ENTRYFORM )) {
		fputs( "<FORM METHOD=POST", stdout );
		if ( editable ) {
		    fprintf( stdout,
			     /*" NAME=\"modifyEntryForm\" ACTION=\"%s\">\n",*/
			     /* Hard coded right now for enduser -Ashfaq 01/22/98*/
			     " NAME=\"modifyEntryForm\" ACTION=\"/bin/user/admin/bin/domodify.cgi\">\n");
		    /*   dsgw_getvp( DSGW_CGINUM_DOMODIFY ));*/
		    fputs( "<INPUT TYPE=\"hidden\" NAME=\"changetype\">\n",
			    stdout );
		    fprintf( stdout, "<INPUT TYPE=\"hidden\" "
			    "NAME=\"dn\" VALUE=\"%s\">\n", encodeddn );
		    fputs( "<INPUT TYPE=\"hidden\" NAME=\"newrdn\">\n",
			    stdout );
		    
		} else {
		    fprintf( stdout, " NAME=\"editEntryForm\" "
			"ACTION=\"%s\" TARGET=\"_blank\" >\n",
			dsgw_getvp( DSGW_CGINUM_AUTH ));
		}

	    } else if ( dsgw_directive_is( line, DRCT_DS_END_ENTRYFORM )) {
		fprintf( stdout, "</FORM>\n" );

	    } else if ( dsgw_directive_is( line, DRCT_DS_END_DNSEARCHFORM )) {
		fprintf( stdout, "</FORM>\n" );

	    } else if ( dsgw_directive_is( line, DRCT_DS_EDITBUTTON )) {
		if ( !editable ) do_editbutton( dn, encodeddn, argc, argv );

	    } else if ( dsgw_directive_is( line, DRCT_DS_DELETEBUTTON )) {
		if ( editable && !adding ) do_deletebutton( argc, argv );

	    } else if ( dsgw_directive_is( line, DRCT_DS_RENAMEBUTTON )) {
		if ( editable && !adding ) do_renamebutton( dn, argc, argv );

	    } else if ( dsgw_directive_is( line, DRCT_DS_EDITASBUTTON )) {
		if ( editable ) do_editasbutton( argc, argv );

	    } else if ( dsgw_directive_is( line, DRCT_DS_SAVEBUTTON )) {
		if ( editable ) do_savebutton( tip->dsti_options, argc, argv );

	    } else if ( dsgw_directive_is( line, DRCT_DS_POSTEDVALUE )) {
		do_postedvalue( argc, argv );

	    } else if ( dsgw_directive_is( line, DRCT_DS_HELPBUTTON )) {
		do_helpbutton( tip->dsti_options, argc, argv );

	    } else if ( dsgw_directive_is( line, DRCT_DS_CLOSEBUTTON )) {
		do_closebutton( tip->dsti_options, argc, argv );

	    } else if ( dsgw_directive_is( line, DRCT_DS_OBJECTCLASS )) {
		/* omit objectClass lines */

	    } else if ( dsgw_directive_is( line, DRCT_DS_NEWPASSWORD )) {
		if ( editable ) do_passwordfield( tip->dsti_options, argc,
			argv, "newpasswd" );

	    } else if ( dsgw_directive_is( line, DRCT_DS_CONFIRM_NEWPASSWORD )) {
		if ( editable ) do_passwordfield( tip->dsti_options, argc,
			argv, "newpasswdconfirm" );

	    } else if ( dsgw_directive_is( line, DRCT_DS_OLDPASSWORD )) {
		if ( editable ) do_passwordfield( tip->dsti_options, argc,
			argv, "passwd" );

	    } else if ( dsgw_directive_is( line, DRCT_DS_DNATTR )) {
		if ( dsgw_dnattr != NULL ) fputs( dsgw_dnattr, stdout );

	    } else if ( dsgw_directive_is( line, DRCT_DS_DNDESC )) {
		if ( dsgw_dndesc != NULL ) fputs( dsgw_dndesc, stdout );
	    
	    } else if ( dsgw_directive_is( line, DRCT_DS_DNEDITBUTTON )) {
		if ( editable ) {
		    do_dneditbutton( tip->dsti_options, argc, argv );
		}
	    } else if ( dsgw_directive_is( line, DRCT_DS_AVAEDITBUTTON )) {
		if ( editable ) {
		    do_avaeditbutton( tip->dsti_options, argc, argv );
		}

	    } else if ( dsgw_directive_is( line, DRCT_DS_VIEW_SWITCHER ) &&
		    tip->dsti_entry != NULL ) {
		do_viewswitcher( tip->dsti_template, tip->dsti_entrydn,
			argc, argv );

	    }
	    /* I18n: display language popup */
	    else if ( dsgw_directive_is( line, DRCT_DS_LANGUAGEPOPUP )) {
		dsgw_emit_language_popup( tip, argc, argv );
	    }
	    /* I18n: display language map table */
	    else if ( dsgw_directive_is( line, DRCT_DS_LANGUAGETABLES )) {
		dsgw_emit_language_tables( argc, argv );
	    } else if ( dsgw_directive_is( line, DRCT_DS_RUNCOMMAND )) {
			char *start;
			char *end;

			/* do some really lame parsing... */
			start = strchr( line, '\"');
			start++;
			end = strchr( start, '\"');
			*end = 0;

			fflush( stdout);
			system( start);

	    } else {
		fputs( line, stdout );
	    }

	    dsgw_argv_free( argv );
	}
    }

    free( encodeddn );
}


void
dsgw_set_searchinfo( dsgwtmplinfo *tip, int entrycount, char *searchdesc,
	char *searcherror )
{
    if ( entrycount >= 0 ) {
	tip->dsti_entrycount = entrycount;
    }
    if ( searchdesc != NULL ) {
	if ( tip->dsti_searchdesc != NULL ) {
	    free( tip->dsti_searchdesc );
	}
	tip->dsti_searchdesc = searchdesc;
    }

    if ( searcherror != NULL ) {
	if ( tip->dsti_searcherror != NULL ) {
	    free( tip->dsti_searcherror );
	}
	tip->dsti_searcherror = searcherror;
    }
}


void
dsgw_display_done( dsgwtmplinfo *tip )
{
    char	line[ BIG_LINE ], *jscomp;

    if ( tip->dsti_preludelines != NULL ) {
	output_prelude( tip );
    }

    while ( dsgw_next_html_line( tip->dsti_fp, line )) {
	output_nonentry_line( tip, line );
    }

    /*
     * check for "completion_javascript" form var and
     * execute it if present.
     */ 
    jscomp = dsgw_get_cgi_var( "completion_javascript",
	    DSGW_CGIVAR_OPTIONAL );
    if ( jscomp != NULL ) {
	fputs( "<SCRIPT LANGUAGE=\"JavaScript\">\n", stdout );
	fprintf( stdout, "eval('%s');\n", jscomp );
	fputs( "</SCRIPT>\n", stdout );
    }

    fflush( stdout );
    fflush( stdout );

    dsgw_savelines_free( tip->dsti_entrylines );
    fclose( tip->dsti_fp );
    if ( tip->dsti_attrs != NULL ) {
	ldap_value_free( tip->dsti_attrs );
    }
    if ( tip->dsti_attrflags != NULL ) {
	free( tip->dsti_attrflags );
    }
    if ( tip->dsti_rdncomps != NULL ) {
	ldap_value_free( tip->dsti_rdncomps );
    }
    free( tip );
}


static void
output_prelude( dsgwtmplinfo *tip )
{
    int		editable, adding;
    char	*line, *encodeddn;

    if ( tip->dsti_preludelines != NULL ) {	/* output the prelude */
	dsgw_savelines_rewind( tip->dsti_preludelines );
	while (( line = dsgw_savelines_next( tip->dsti_preludelines ))
		!= NULL ) {
	    output_nonentry_line( tip, line );
	}
	dsgw_savelines_free( tip->dsti_preludelines );
	tip->dsti_preludelines = NULL;
    }

    /* output any JavaScript functions we want to include before the entry */
    fputs( "<SCRIPT LANGUAGE=\"JavaScript\">\n", stdout );
    fputs( "<!-- Hide from non-JavaScript-capable browsers\n", stdout );
    fputs( "var emptyFrame = '';\n", stdout );
    editable = ( tip->dsti_options & DSGW_DISPLAY_OPT_EDITABLE ) != 0;
    adding = ( tip->dsti_options & DSGW_DISPLAY_OPT_ADDING ) != 0;

    if ( !editable ) {
	/* include the functions used to support "Edit" buttons */
	/* function haveAuthCookie() */
	fputs( "function haveAuthCookie()\n{\n", stdout );
	fprintf( stdout, "    return ( document.cookie.indexOf( '%s=' ) >= 0 "
		"&& document.cookie.indexOf( '%s=%s' ) < 0 );\n}\n\n",
		DSGW_AUTHCKNAME, DSGW_AUTHCKNAME, DSGW_UNAUTHSTR );

	/* function authOrEdit() -- calls haveAuthCookie() */
	fputs( "function authOrEdit(encodeddn)\n{\n", stdout );
	fprintf( stdout, "    editURL = location.protocol + \"//\" + "
		"location.host + \"%s?&dn=\" + encodeddn;\n",
		dsgw_getvp( DSGW_CGINUM_EDIT ));
	fputs( "    if ( haveAuthCookie()) {\n", stdout );
	fputs( "\tnw = open(editURL, \"NewWindow\");\n", stdout );
	/* Navigator 2.0 bug wrkarnd */
	fputs( "\topen(editURL, \"NewWindow\");\n"
		"\tdocument.clear();\n"
		"\tdocument.open();\n"
		"\tdocument.write('');\n"
		"\tdocument.close();\n"
		"    } else {\n"
		"\tdocument.editEntryForm.authdesturl.value = editURL\n"
		"\ta = open('', 'AuthWin');\n"
		"\ta.opener = self;\n"
		"\ta.closewin = true;\n"
		"\tdocument.editEntryForm.target = 'AuthWin';\n"
		"\tdocument.editEntryForm.submit();\n"
		"    }\n}\n", stdout );

    } else {
	/* include variables and functions used to support edit mode */
	fputs( "var changesHaveBeenMade = 0;\n\n", stdout );
	fputs( "var possiblyChangedAttr = null;\n\n", stdout );

	/* function aChg() -- called from onChange and onClick handlers */
	fputs( "function aChg(attr)\n{\n", stdout );
	if ( !adding ) {
	    fputs( "    cmd = 'document.modifyEntryForm.changed_' + "
		    "attr + '.value = \"true\"';\n    eval( cmd );\n"
		    "    if ( possiblyChangedAttr != null ) {\n"
		    "\t    if ( possiblyChangedAttr != attr ) {\n"
		    "\t\taChg( possiblyChangedAttr );\n\t    }\n"
		    "\t    possiblyChangedAttr = null;\n\t}\n",
		    stdout );
	}
	fputs( "    changesHaveBeenMade = 1;\n}\n\n", stdout );


	if ( !adding ) {
	    /* function aFoc() -- called when text area gets focus. */
	    fputs( "function aFoc(attr)\n{\n"
		    "   if ( possiblyChangedAttr != null ) "
		    " aChg(possiblyChangedAttr);\n"
		    "   possiblyChangedAttr = attr;\n}\n\n", stdout );

	    /* function renameEntry() */
	    if(tip->dsti_entrydn != NULL) {
              encodeddn = dsgw_strdup_escaped( tip->dsti_entrydn );
              fputs( "function renameEntry(tag, tmplname, oldname)\n{\n",
			       stdout );
              fprintf( stdout,
	        "    var newurl = '%s?' + tmplname + '&dn=%s';\n",
                dsgw_getvp(DSGW_CGINUM_RENAME), encodeddn );
              fputs( "    window.location.href = newurl;\n}\n", stdout );
	    }
	}

	/* function submitModify() */
	fputs( "function submitModify(changetype, confirmmsg)\n{\n", stdout );
	fputs( "    if ( confirmmsg == null || confirm( confirmmsg )) {\n",
		stdout );
	if ( !adding ) {
	    fputs( "\tif ( possiblyChangedAttr != null ) "
		    "aChg(possiblyChangedAttr);\n", stdout );
	}
	fputs( "\tdocument.modifyEntryForm.changetype.value = changetype;\n",
		stdout );
	fputs( "\tdocument.modifyEntryForm.submit();\n    }\n}\n", stdout );

	/* function EditEntryAs() */
/*
	fputs( "function EditEntryAs(template)\n{\n", stdout );
	fputs( "    newurl = window.location.protocol + '//' +\n"
		"\twindow.location.host +\n"
		"\twindow.location.pathname + '?' + template;\n", stdout );
	fputs( "\twindow.location.href = newurl;\n}\n", stdout );
*/

	if ( tip->dsti_entrydn != NULL ) {
	    encodeddn = dsgw_strdup_escaped( tip->dsti_entrydn );
	    fputs( "function EditEntryAs(template)\n{\n", stdout );
	    fprintf( stdout, "    newurl = '%s?' + template + '&dn=%s';\n",
		    dsgw_getvp( DSGW_CGINUM_EDIT ), encodeddn );
	    fputs( "\twindow.location.href = newurl;\n}\n", stdout );
	}

	/* function DNEdit() */
	if ( tip->dsti_entrydn != NULL ) {
	    encodeddn = dsgw_strdup_escaped( tip->dsti_entrydn );
	    fputs( "function DNEdit(template, attr, desc)\n{\n", stdout );
	    fprintf( stdout, "    newurl = '%s?template=' + template + "
		     "'&dn=%s&ATTR=' + attr + '&DESC=' + escape(desc);\n",
		     dsgw_getvp( DSGW_CGINUM_DNEDIT ), encodeddn );
	    fputs( "\twindow.location.href = newurl;\n}\n", stdout );
	}

	/* function AVAEdit() */
	if ( tip->dsti_entrydn != NULL ) {
	    encodeddn = dsgw_strdup_escaped( tip->dsti_entrydn );
	    fputs( "function AVAEdit(template, attr, desc)\n{\n", stdout );
	    fprintf( stdout, "    newurl = '%s?template=' + template + "
		     "'&DN=%s&ATTR=' + attr + '&DESC=' + escape(desc);\n",
		     dsgw_getvp( DSGW_CGINUM_AVAEDIT ), encodeddn );
	    fputs( "\twindow.location.href = newurl;\n}\n", stdout );
	}

	/* function changesMade() */
	fputs( "function changesMade()\n{\n", stdout );
	if ( !adding ) {
	    fputs( "\tif ( possiblyChangedAttr != null ) "
		    "aChg(possiblyChangedAttr);\n", stdout );
	}
	fputs( "    return( changesHaveBeenMade );\n}\n", stdout );

	/* function closeIfOK() */
	fputs( "function closeIfOK()\n{\n"
		"    if ( !changesMade() || confirm('Discard Changes?')) {\n"
		"\ttop.close();\n"
		"    }\n}\n", stdout );

	/* set unload handler to catch unsaved changes */
	fputs( "document.onUnload = \""
		"return ( !changesMade() || prompt( 'Discard Changes?' ));\"\n",
		stdout );
    }

    fputs( "// End hiding -->\n</SCRIPT>\n", stdout );
}


static void
output_nonentry_line( dsgwtmplinfo *tip, char *line )
{
    int		argc;
    char	**argv;

    if ( dsgw_parse_line( line, &argc, &argv, 0, condition_is_true, tip )) {
	if ( dsgw_directive_is( line, DRCT_DS_SEARCHDESC )) {
	    do_searchdesc( tip );
	} else if ( dsgw_directive_is( line, DRCT_DS_POSTEDVALUE )) {
	    do_postedvalue( argc, argv );
	} else if ( dsgw_directive_is( line, DRCT_DS_HELPBUTTON )) {
	    do_helpbutton( tip->dsti_options, argc, argv );
	} else if ( dsgw_directive_is( line, DRCT_DS_CLOSEBUTTON )) {
	    do_closebutton( tip->dsti_options, argc, argv );
	} else if ( dsgw_directive_is( line, DRCT_DS_OBJECTCLASS )) {
	    /* omit objectClass lines */
	} else {
	    fputs( line, stdout );
	}
	dsgw_argv_free( argv );
    }
}

static void
do_attribute( dsgwtmplinfo *tip, char *dn, unsigned long dispopts,
	int argc, char **argv )
{
    char			*attr, *syntax, *defval, *tmpvals[ 2 ], *s;
    char			**ldvals, **vals;
    unsigned long		options;
    int				i, len, attrindex, htmltype;
    int				attrcopied, langtag, phonetic, langinfo;
    struct dsgw_attrdispinfo	adi;
    /* I18n variables */
    char			*coreattr;
    int				entag;

    attrcopied = 0;

    if (( attr = get_arg_by_name( DSGW_ATTRARG_ATTR, argc, argv )) == NULL ) {
	fprintf( stdout, XP_GetClientStr(DBT_missingS_), DSGW_ATTRARG_ATTR );
	return;
    }
    coreattr = attr;
    if (( syntax = get_arg_by_name( DSGW_ATTRARG_SYNTAX, argc, argv ))
	    == NULL ) {
	syntax = "cis";
    }

    if (( s = get_arg_by_name( DSGW_ATTRARG_HTMLTYPE, argc, argv )) == NULL ) {
	htmltype = DSGW_ATTRHTML_TEXT;
    } else {
	for ( i = 0; attrhtmltypes[ i ] != NULL; ++i ) {
	    if ( strcasecmp( s, attrhtmltypes[ i ] ) == 0 ) {
		htmltype = attrhtmlvals[ i ];
		break;
	    }
	}
	if ( attrhtmltypes[ i ] == NULL ) {
	    fprintf( stdout, XP_GetClientStr(DBT_unknownSS_), DSGW_ATTRARG_HTMLTYPE, s );
	    return;
	}
    }

    options = 0;
    if (( s = get_arg_by_name( DSGW_ATTRARG_OPTIONS, argc, argv )) != NULL ) {
	char	*p, *q;

	for ( p = dsgw_ch_strdup( s ); p != NULL; p = q ) {
	    if (( q = strchr( p, ',' )) != NULL ) {
		*q++ = '\0';
	    }
	    for ( i = 0; attroptions[ i ] != NULL; ++i ) {
		if ( strcasecmp( p, attroptions[ i ] ) == 0 ) {
		    options |= attroptvals[ i ];
		    break;
		}
	    }
	    if ( attroptions[ i ] == NULL ) {
		fprintf( stdout, XP_GetClientStr(DBT_unknownOptionS_), p );
		break;
	    }
	}
	free( p );
    }
    if (( dispopts & DSGW_DISPLAY_OPT_EDITABLE ) != 0
	    && ( options & DSGW_ATTROPT_READONLY ) == 0 ) {
	options |= DSGW_ATTROPT_EDITABLE;
	if ((  dispopts & DSGW_DISPLAY_OPT_ADDING ) != 0 ) {
	    options |= DSGW_ATTROPT_ADDING;
	}
    }

    if (( dispopts & DSGW_DISPLAY_OPT_LINK2EDIT ) != 0 ) {
	options |= DSGW_ATTROPT_LINK2EDIT;
    }

    if ( gc->gc_international ) {
        langtag = ( gc->gc_inputlanguage != NULL )
                && (( options & DSGW_ATTROPT_LANGTAG ) != 0 );
        phonetic = (( options & DSGW_ATTROPT_PHONETIC ) != 0 );
        entag = (( options & DSGW_ATTROPT_ENTAG ) != 0 );
        langinfo = (( options & DSGW_ATTROPT_LANGINFO ) != 0 );
        if ( langtag ) {
            /*
             * allocate space for attr + ";lang-TAG" + ";phonetic" and
             * create new attr string.
	     * note: phonetic cannot appear by itself.
             */
            s = dsgw_ch_malloc( strlen( attr ) + 1 + 6
                    + strlen( gc->gc_inputlanguage->dslang_tag ) + 9 );
            strcpy( s, attr );
            if ( langtag ) {
                strcat( s, ";lang-" );
                strcat( s, gc->gc_inputlanguage->dslang_tag );
            }
            if ( phonetic ) {
                strcat( s, ";phonetic" );
            }
            attrcopied = 1;
            attr = s;
        } else if ( entag ) {
            /*
             * allocate space for attr + ";lang-en"
             * create new attr string.
             */
            s = dsgw_ch_malloc( strlen( attr ) + 1 + 6 + sizeof("en") );
            sprintf( s, "%s;lang-en", attr );
            attrcopied = 1;
            attr = s;
        }
    } else {
        langtag = phonetic = entag = NULL;
    }

    ldvals = vals = NULL;

    if ( strcasecmp( attr, "dn" ) == 0 ) {	/* dn pseudo-attribute */
	tmpvals[ 0 ] = dn;
	tmpvals[ 1 ] = NULL;
	vals = tmpvals;
	options &= ~DSGW_ATTROPT_EDITABLE;	/* always read-only */
	options &= ~DSGW_ATTROPT_ADDING;	/* always read-only */
	options |= DSGW_ATTROPT_READONLY;
    }
    else if ( tip->dsti_entry != NULL ) {
	/* I18n: If LDAP V3, get the results in the specified language */
        if ( gc->gc_ldapversion >= LDAP_VERSION3 ) {
	    if ( langinfo ) {
                BerElement *tmpelm;
                char *ret;
		char *p, *q;
		int alen;
		int eflag = 0;
                if (( ret = ldap_first_attribute( tip->dsti_ld,
			tip->dsti_entry, &tmpelm )) != NULL ) {
		    vals = tmpvals;
		    *vals = *(vals + 1) = 0;
                    do {
			alen = strlen( coreattr );
			if ( !strncmp( coreattr, ret, alen )) {
			    p = ret + alen;
			    if ( q = strstr( p, "lang-" )) {
				p = q + 5;
			    } else {
	                        ldap_memfree( ret );    /* no lang */
				continue;
			    }
			    if ( q = strchr( p, ';' ))
				*q = '\0';
			    if ( !strcmp( p, "en" )) { /* en is special */
				eflag = 1;
	                        ldap_memfree( ret );
				continue;
			    }
			    *vals = dsgw_ch_strdup( p );
#ifdef DSGW_DEBUG
                            dsgw_log( "do_attribute: langinfo ( %s )\n",
					 *vals );
#endif
			}
	                ldap_memfree( ret );
			if ( !*vals && eflag ) {
			    *vals = dsgw_ch_strdup( "en" );
#ifdef DSGW_DEBUG
                            dsgw_log( "do_attribute: langinfo ( %s )\n",
					 *vals );
#endif
                        }
	            } while (( ret = ldap_next_attribute( tip->dsti_ld,
			tip->dsti_entry, tmpelm )) != NULL );
                }
            } else if ( langtag || phonetic || entag ) {
                /* fallback */
                ldvals = ldap_get_lang_values( tip->dsti_ld, tip->dsti_entry,
							     attr, NULL );
#ifdef DSGW_DEBUG
                dsgw_log( "do_attribute: get_lang_values( %s ) -> \"%s\"\n",
					 attr, ldvals?*ldvals:"" );
#endif
                if ( !( options & DSGW_ATTROPT_NOFALLBACK ) &&
		    ldvals == NULL ) {
                    ldvals = ldap_get_values( tip->dsti_ld, tip->dsti_entry,
							    coreattr );
#ifdef DSGW_DEBUG
                    dsgw_log( "do_attribute: get_values( %s ) -> \"%s\"\n",
					 coreattr, ldvals?*ldvals:"" );
#endif
                }
                if ( ldvals != NULL )
                    vals = ldvals;
            } else {
                ldvals = ldap_get_values( tip->dsti_ld, tip->dsti_entry, attr );
                if ( ldvals != NULL )
                    vals = ldvals;
            }
	} else {
	    ldvals = ldap_get_values( tip->dsti_ld, tip->dsti_entry, coreattr );
	    if ( ldvals != NULL )
	        vals = ldvals;
	}
    }

    if ( vals == NULL && tip->dsti_rdncomps != NULL
	    && ( options & DSGW_ATTROPT_ADDING ) != 0 ) {
	/*
	 * include values from the DN of new entry being added
	 */
	len = strlen( attr );
	ldvals = NULL;

	for ( i = 0; tip->dsti_rdncomps[ i ] != NULL; ++i ) {
	    if (( s = strchr( tip->dsti_rdncomps[ i ], '=' )) != NULL &&
		    s - tip->dsti_rdncomps[ i ] == len &&
		    strncasecmp( attr, tip->dsti_rdncomps[ i ], len ) == 0 ) {
		tmpvals[ 0 ] = ++s;
		tmpvals[ 1 ] = NULL;
		vals = tmpvals;
		break;
	    }
	}
    }

    if ( vals == NULL && ( defval = get_arg_by_name( DSGW_ATTRARG_DEFAULT,
	    argc, argv )) != NULL ) {
	tmpvals[ 0 ] = defval;
	tmpvals[ 1 ] = NULL;
	vals = tmpvals;
    }

    if ( vals == NULL && ( options & DSGW_ATTROPT_EDITABLE ) == 0 ) {
	fputs( gc->gc_nbspchar, stdout );
    } else {
	/* I18n: additional option "quoted" */
        if ( options & DSGW_ATTROPT_QUOTED )
            fputs( "'", stdout );
	if (( adi.adi_handlerp = syntax2attrhandler( syntax )) == NULL ) {
	    fprintf( stdout, XP_GetClientStr(DBT_unknownSyntaxSN_), syntax );
	} else {
	    if ( vals != NULL && vals[1] != NULL
			&& ( options & DSGW_ATTROPT_SORT ) != 0 ) {
		ldap_sort_values( tip->dsti_ld, vals,
			adi.adi_handlerp->ath_compare );
	    }
	    adi.adi_attr = coreattr;	/* I18n: attr -> coreattr */
	    adi.adi_argc = argc;
	    adi.adi_argv = argv;
	    adi.adi_vals = vals;
	    adi.adi_htmltype = htmltype;
	    adi.adi_opts = options;
	    adi.adi_entrydn = dn;

	    if (( options & DSGW_ATTROPT_EDITABLE ) == 0 ) {
		(*adi.adi_handlerp->ath_display)( &adi );
	    } else {
		if (( options & DSGW_ATTROPT_ADDING ) == 0 ) {
		    /* set flag to track attrs. we have seen */
		    for ( attrindex = 0; tip->dsti_attrs[ attrindex ] != NULL;
			    ++attrindex ) {
		        /* I18n: attr -> coreattr */
                        if ( strcasecmp( coreattr, tip->dsti_attrs[ attrindex ])
				       == 0 ) { 
			    break;
			}
		    }

		    if ( tip->dsti_attrs[ attrindex ] != NULL &&
			    ( tip->dsti_attrflags[ attrindex ] &
			    DSGW_DSTI_ATTR_SEEN ) == 0 ) {
	                /* I18n: additional option "quoted" */
                        if ( options & DSGW_ATTROPT_QUOTED ) {
			      fprintf( stdout, "<INPUT TYPE=\"hidden\" "
				"NAME=\"changed_%s\" VALUE=\"false\">",
				coreattr );
			} else {
			      fprintf( stdout, "<INPUT TYPE=\"hidden\" "
				"NAME=\"changed_%s\" VALUE=\"false\">\n",
				coreattr );
                        }
			tip->dsti_attrflags[ attrindex ] |= DSGW_DSTI_ATTR_SEEN;
		    }
		}

		/* display for editing */
		(*adi.adi_handlerp->ath_edit)( &adi );
	    }
	}
	/* I18n: additional option "quoted" */
        if ( options & DSGW_ATTROPT_QUOTED )
            fputs( "'", stdout );
    }

    if ( ldvals != NULL ) {
	ldap_value_free( ldvals );
    }

    if ( attrcopied ) {
	free( attr );
    }
}


static struct attr_handler *
syntax2attrhandler( char *syntax )
{
    int		i;

    for ( i = 0; i < DSGW_AH_COUNT; ++i ) {
	if ( strcasecmp( syntax, attrhandlers[ i ].ath_syntax ) == 0 ) {
	    return( &attrhandlers[ i ] );
	}
    }

    return( NULL );
}


static int
numfields( int argc, char **argv, int valcount )
{
    char	*s;
    int		fields;

    if (( s = get_arg_by_name( DSGW_ATTRARGS_NUMFIELDS, argc,
	    argv )) == NULL ) {
	fields = 1;
    } else {
	if ( *s == '+' || *s == ' ') {
	    /* "numfields=+N" means show N more than number of values */
	    fields = valcount + atoi( s + 1 );
	} else {
	    if ( *s == '>' ) ++s;
	    /* "numfields=N" or "=>N" means show at least N fields */
	    fields = atoi( s );
	}
    }

    if ( fields < 1 ) {
	fields = 1;
    } else if ( fields < valcount ) {
	fields = valcount;
    }

    return( fields );
}

/*
 * calculate size of TEXT or TEXTAREA elements based on arguments,
 * the number of values, and the length of longest value.
 */
static void
element_sizes( int argc, char **argv, char **vals, int valcount,
	int *rowsp, int *colsp )
{
    int		i, len, maxlen;
    char	*s;

    /* set *colsp (number of columns in each input item) */
    if ( colsp != NULL ) {
	/*
	 * columns are set using the "cols=N" or "size=N" argument
	 * "cols=>N" can be used to indicate at least N columns should be shown
	 * "cols=+N" can be used to size to N more than longest value
	 * in the absence of any of these, we set columns to one more than
	 * the longest value in the "vals" array
	 */
	if (( s = get_arg_by_name( DSGW_ATTRARGS_COLS, argc, argv )) == NULL ) {
	    s = get_arg_by_name( DSGW_ATTRARGS_SIZE, argc, argv );
	}

	if ( s != NULL && *s != '+' && *s != ' ' && *s != '>' ) {
	    *colsp = atoi( s );	/* extact width specified */
	} else if ( valcount == 0 ) {
	    if ( s != NULL && *s == '>' ) {
		*colsp = atoi( s + 1 );
	    } else {
		*colsp = 0;	/* use default width */
	    }
	} else {
	    /* determine ( length of longest value ) + 1  */
	    maxlen = 0;
	    for ( i = 0; i < valcount; ++i ) {
		if (( len = strlen( vals[ i ] )) > maxlen ) {
		    maxlen = len;
		}
	    }
	    ++maxlen;

	    if ( s != NULL ) {
		i = atoi( s + 1 );
		if ( *s == ' ' || *s == '+' ) {
		    maxlen += i;
		} else {	/* '>' */
		    if ( maxlen < i ) {
			maxlen = i;
		    }
		}
	    }
	    *colsp = maxlen;
	}
    }

    /* set *rowsp (number of rows in each input item) */
    if ( rowsp != NULL ) {
	/*
	 * rows are set using "rows=M" ("=>M" and "=+M" are supported also)
	 * in the absense of this, we set it to the number of values in the
         * "vals" array
	 */
	if (( s = get_arg_by_name( DSGW_ATTRARGS_ROWS, argc, argv )) == NULL ) {
	    *rowsp = valcount;
	} else if ( *s == ' ' || *s == '+' ) {
	    *rowsp = valcount + atoi( s + 1 );
	} else if ( *s == '>' ) {
	    if (( *rowsp = atoi( s + 1 )) < valcount ) {
		*rowsp = valcount;
	    }
	} else {
	    *rowsp = atoi( s );
	}
    }
}


static void
output_text_elements( int argc, char **argv, char *attr, char **vals,
	char *prefix, int htmltype, unsigned long opts )
{
    int		i, valcount, fields, cols;

    if ( vals == NULL ) {
	valcount = 0;
    } else {
	for ( valcount = 0; vals[ valcount ] != NULL; ++valcount ) {
            char *syntax = get_arg_by_name( DSGW_ATTRARG_SYNTAX, argc, argv );
            if ( syntax && 0 == strcasecmp( syntax, "ntdomain" )) {
                 char *pch = strchr( vals[ valcount ], DSGW_NTDOMAINID_SEP );
                 if( pch )
                     *pch = NULL;
	    }
	}
    }

    fields = numfields( argc, argv, valcount );
    element_sizes( argc, argv, vals, valcount, NULL, &cols );

    for ( i = 0; i < fields; ++i ) {
	fprintf( stdout, "<INPUT TYPE=\"%s\"", attrhtmltypes[ htmltype ] );

	if ( i == 0 ) {
		fprintf( stdout, " NAME=\"%s%s\"", prefix, attr );
	} else {
		fprintf( stdout, " NAME=\"%s%d_%s\"", prefix, i, attr );
	}

	if ( cols > 0 ) {
	    fprintf( stdout, " SIZE=%d", cols );
	}

	if ( i < valcount ) {
	    fprintf( stdout, " VALUE=\"%s\"", vals[ i ] );
	}

	/* I18n: if quoted, ' should be escaped */
        if ( opts & DSGW_ATTROPT_QUOTED ) {
	    if (( opts & DSGW_TEXTOPT_CHANGEHANDLERS ) != 0 ) {
	        fprintf( stdout, " onChange=\"aChg(\\'%s\\')\"", attr );
	    }
	    if (( opts & DSGW_TEXTOPT_FOCUSHANDLERS ) != 0 ) {
	        fprintf( stdout, " onFocus=\"aFoc(\\'%s\\')\"", attr );
	    }
	    fprintf( stdout, ">%s", ( i < fields - 1 &&
		htmltype != DSGW_ATTRHTML_HIDDEN ) ? "<BR>" : "" );
	} else {
	    if (( opts & DSGW_TEXTOPT_CHANGEHANDLERS ) != 0 ) {
	        fprintf( stdout, " onChange=\"aChg('%s')\"", attr );
	    }
	    if (( opts & DSGW_TEXTOPT_FOCUSHANDLERS ) != 0 ) {
	        fprintf( stdout, " onFocus=\"aFoc('%s')\"", attr );
	    }
	    fprintf( stdout, ">\n%s", ( i < fields - 1 &&
		htmltype != DSGW_ATTRHTML_HIDDEN ) ? "<BR>\n" : "" );
	}
    }
}


static void
output_textarea( int argc, char **argv, char *attr, char **vals,
	int valcount, char *prefix, unsigned long opts )
{
    int		i, rows, cols;

    element_sizes( argc, argv, vals, valcount, &rows, &cols );

    fputs( "<TEXTAREA", stdout );
    fprintf( stdout, " NAME=\"%s%s\"", prefix, attr );
    if ( rows > 0 ) {
	if ( rows == 1 ) {
	    rows = 2;	/* one line TEXTAREAs are ugly! */
	}
	fprintf( stdout, " ROWS=%d", rows );
    }

    if ( cols > 0 ) {
	fprintf( stdout, " COLS=%d", cols );
    }

    if (( opts & DSGW_TEXTOPT_CHANGEHANDLERS ) != 0 ) {
	fprintf( stdout, " onChange=\"aChg('%s')\"", attr );
    }
    if (( opts & DSGW_TEXTOPT_FOCUSHANDLERS ) != 0 ) {
	fprintf( stdout, " onFocus=\"aFoc('%s')\"", attr );
    }

    fputs( ">\n", stdout );

    for ( i = 0; i < valcount; ++i ) {
	fputs( vals[ i ], stdout );
	putc( '\n', stdout );
    }

    fputs( "</TEXTAREA>\n", stdout );
}


static void
output_text_checkbox_or_radio( struct dsgw_attrdispinfo *adip, char *prefix,
	int htmltype )
{
    int		i, checked;
    char	*value, *onclick;

    /*
     * for checkboxes or radio buttons that are associated with string values,
     * we "check the box" if the value found in the "value=XXX" parameter is
     * present.
     */
    checked = 0;
    if (( value = get_arg_by_name( DSGW_ATTRARGS_VALUE, adip->adi_argc,
	    adip->adi_argv )) == NULL ) {
	value = "TRUE";	/* assume LDAP Boolean value */
    }
    if ( adip->adi_vals == NULL ) {
	if ( *value == '\0' ) {
	    /*
	     * There are no existing values in the entry and this checkbox or
	     * radio button has a zero-length value associated with it.  We
	     * check this box/enable this radio button as a special case to
	     * support an "off" or "none of the rest" scenario. 
	     */
	    checked = 1;
	}

    } else {
	for ( i = 0; adip->adi_vals[ i ] != NULL; ++i ) {
	    if ( (*adip->adi_handlerp->ath_compare)( (const char **)&value,
		    (const char **)&(adip->adi_vals[ i ]) ) == 0 ) {
		checked = 1;
		break;
	    }
	}
    }
    fprintf( stdout, "<INPUT TYPE=\"%s\" NAME=\"%s%s\" "
	    "VALUE=\"%s\"%s onClick=\"aChg('%s')\">\n",
	    ( htmltype == DSGW_ATTRHTML_RADIO ) ? "radio" : "checkbox",
	    prefix, adip->adi_attr, value, checked ? " CHECKED" : "",
	    adip->adi_attr );
}


/*
 * display handler for NT Domain Identifier string
 */
static void
ntdomain_display( struct dsgw_attrdispinfo *adip )
{
    int		i;

    /* Write values with a break (<BR>) separating them, 
	removing all after ":" */
    for ( i = 0; adip->adi_vals[ i ] != NULL; ++i ) {
	if ( !did_output_as_special( adip->adi_argc, adip->adi_argv,
		    adip->adi_vals[ i ], adip->adi_vals[ i ] )) {
            char *pch = strchr( adip->adi_vals[ i ], DSGW_NTDOMAINID_SEP );
            if( pch )
               *pch = NULL;
	    fputs( adip->adi_vals[ i ], stdout );
	}

	if ( adip->adi_vals[ i + 1 ] != NULL ) {
	    fputs( "<BR>\n", stdout );
	}
    }
}

/*
 * display handler for simple strings
 */
static void
str_display( struct dsgw_attrdispinfo *adip )
{
    int		i, valcount;

    if ( adip->adi_htmltype == DSGW_ATTRHTML_CHECKBOX ||
	    adip->adi_htmltype == DSGW_ATTRHTML_RADIO ) {
	output_text_checkbox_or_radio( adip, "", adip->adi_htmltype );
	return;
    }

    /* honor "numfields=XXX" parameter */
    if ( adip->adi_vals == NULL ) {
	valcount = 0;
    } else {
	for ( valcount = 0; adip->adi_vals[ valcount ] != NULL; ++valcount ) {
	    ;
	}
    }
    valcount = numfields( adip->adi_argc, adip->adi_argv, valcount );

    /* just write values with a break (<BR>) separating them */
    for ( i = 0; i < valcount && adip->adi_vals[ i ] != NULL; ++i ) {
	if ( !did_output_as_special( adip->adi_argc, adip->adi_argv,
		adip->adi_vals[ i ], adip->adi_vals[ i ] )) {
	    fputs( adip->adi_vals[ i ], stdout );
	}

	if ( adip->adi_vals[ i + 1 ] != NULL ) {
	    fputs( "<BR>\n", stdout );
	}
    }
}


/*
 * edit handler for simple strings
 */
static void
str_edit( struct dsgw_attrdispinfo *adip )
{
    int			valcount, adding, pre_idx;
    char		*prefix;
    unsigned long	textopts;

    adding = (( adip->adi_opts & DSGW_ATTROPT_ADDING ) != 0 );
    /* I18n: additional language tag support */
    if ( gc->gc_ldapversion >= LDAP_VERSION3 ) {
        if (( adip->adi_opts & DSGW_ATTROPT_UNIQUE )) {
            if (( adip->adi_opts & DSGW_ATTROPT_LANG )) {
                pre_idx = DSGW_MOD_PREFIX_LANG_UNIQUE;
            } else if (( adip->adi_opts & DSGW_ATTROPT_EN )) {
                pre_idx = DSGW_MOD_PREFIX_EN_UNIQUE;
            } else if (( adip->adi_opts & DSGW_ATTROPT_PHNTC )) {
                pre_idx = DSGW_MOD_PREFIX_PHNTC_UNIQUE;
            } else {
                pre_idx = DSGW_MOD_PREFIX_UNIQUE;
            }
        } else if (( adip->adi_opts & DSGW_ATTROPT_LANG )) {
            pre_idx = DSGW_MOD_PREFIX_LANG;
        } else if (( adip->adi_opts & DSGW_ATTROPT_EN )) {
            pre_idx = DSGW_MOD_PREFIX_EN;
        } else if (( adip->adi_opts & DSGW_ATTROPT_PHNTC )) {
            pre_idx = DSGW_MOD_PREFIX_PHNTC;
        } else {
            pre_idx = DSGW_MOD_PREFIX_NORMAL;
        }
    } else {
        if (( adip->adi_opts & DSGW_ATTROPT_UNIQUE ) == 0 ) {
            pre_idx = DSGW_MOD_PREFIX_NORMAL;
        } else {
            pre_idx = DSGW_MOD_PREFIX_UNIQUE;
        }
    }
    prefix = adding ? add_prefixes[ pre_idx ] : replace_prefixes[ pre_idx ];
#ifdef DSGW_DEBUG
    dsgw_log( "str_edit: prefix is %s (odi_opts: 0x%x, pre_idx: %d)\n",
			 prefix, adip->adi_opts, pre_idx );
#endif

    textopts = DSGW_TEXTOPT_CHANGEHANDLERS |
		( adip->adi_opts & DSGW_ATTROPT_QUOTED );
    if ( !adding ) {
	textopts |= DSGW_TEXTOPT_FOCUSHANDLERS;
    }

    switch( adip->adi_htmltype ) {
    case DSGW_ATTRHTML_TEXTAREA:
	if ( adip->adi_vals == NULL ) {
	    valcount = 0;
	} else {
	    for ( valcount = 0; adip->adi_vals[ valcount ] != NULL;
		    ++valcount ) {
		;
	    }
	}
	output_textarea( adip->adi_argc, adip->adi_argv, adip->adi_attr,
		adip->adi_vals, valcount, prefix, textopts );
	break;

    case DSGW_ATTRHTML_TEXT:
    case DSGW_ATTRHTML_HIDDEN:
	output_text_elements( adip->adi_argc, adip->adi_argv, adip->adi_attr,
		adip->adi_vals, prefix, adip->adi_htmltype, textopts );
	break;

    case DSGW_ATTRHTML_CHECKBOX:
    case DSGW_ATTRHTML_RADIO:
	output_text_checkbox_or_radio( adip, prefix, adip->adi_htmltype );
	break;

    default:
	fprintf( stdout, XP_GetClientStr(DBT_HtmlTypeSNotSupportedBrN_),
		attrhtmltypes[ adip->adi_htmltype ] );
    }
}


/*
 * display handler for multi-line strings, e.g. postalAddress
 * these are funny in that over LDAP, lines are separated by " $ "
 * this only support "htmltype=text"
 */
static void
mls_display( struct dsgw_attrdispinfo *adip )
{
    int		i;
    char	*valcopy;

    for ( i = 0; adip->adi_vals[ i ] != NULL; ++i ) {
	if ( !did_output_as_special( adip->adi_argc, adip->adi_argv,
		adip->adi_vals[ i ], adip->adi_vals[ i ] )) {
	    valcopy = mls_convertlines( adip->adi_vals[ i ], "<BR>\n", NULL );
	    fputs( valcopy, stdout );
	    free( valcopy );
	}

	if ( adip->adi_vals[ i + 1 ] != NULL ) {
	    fputs( "<BR><BR>\n", stdout );
	}
    }
}


/*
 * edit handler for multi-line strings
 */
static void
mls_edit( struct dsgw_attrdispinfo *adip )
{
    char		*prefix, **valscopy, *tval[ 2 ];
    int			i, valcount, adding, pre_idx, *lines;
    unsigned long	textopts;

    adding = (( adip->adi_opts & DSGW_ATTROPT_ADDING ) != 0 );
    textopts = DSGW_TEXTOPT_CHANGEHANDLERS;
    if ( !adding ) {
	textopts |= DSGW_TEXTOPT_FOCUSHANDLERS;
    }

    /* I18n: additional language tag support */
    if ( gc->gc_ldapversion >= LDAP_VERSION3 ) {
        if (( adip->adi_opts & DSGW_ATTROPT_UNIQUE )) {
            if (( adip->adi_opts & DSGW_ATTROPT_LANG )) {
                pre_idx = DSGW_MOD_PREFIX_LANG_UNIQUE;
            } else if (( adip->adi_opts & DSGW_ATTROPT_EN )) {
                pre_idx = DSGW_MOD_PREFIX_EN_UNIQUE;
            } else if (( adip->adi_opts & DSGW_ATTROPT_PHNTC )) {
                pre_idx = DSGW_MOD_PREFIX_PHNTC_UNIQUE;
            } else {
                pre_idx = DSGW_MOD_PREFIX_UNIQUE;
            }
        } else if (( adip->adi_opts & DSGW_ATTROPT_LANG )) {
            pre_idx = DSGW_MOD_PREFIX_LANG;
        } else if (( adip->adi_opts & DSGW_ATTROPT_EN )) {
            pre_idx = DSGW_MOD_PREFIX_EN;
        } else if (( adip->adi_opts & DSGW_ATTROPT_PHNTC )) {
            pre_idx = DSGW_MOD_PREFIX_PHNTC;
        } else {
            pre_idx = DSGW_MOD_PREFIX_NORMAL;
        }
    } else {
        if (( adip->adi_opts & DSGW_ATTROPT_UNIQUE ) == 0 ) {
            pre_idx = DSGW_MOD_PREFIX_NORMAL;
        } else {
            pre_idx = DSGW_MOD_PREFIX_UNIQUE;
        }
    }
    prefix = adding ? add_mls_prefixes[ pre_idx ] :
	    replace_mls_prefixes[ pre_idx ];

    if ( adip->adi_vals == NULL ) {
	valscopy = NULL;
    } else {
	for ( valcount = 0; adip->adi_vals[ valcount ] != NULL; ++valcount ) {
	    ;
	}
	valscopy = (char **)dsgw_ch_malloc( (valcount + 1) * sizeof( char * ));
	lines = (int *)dsgw_ch_malloc( valcount * sizeof( int ));
	for ( i = 0; i < valcount; ++i ) {
	    valscopy[ i ] = mls_convertlines( adip->adi_vals[ i ], "\n",
		    &lines[ i ] );
	}
	valscopy[ valcount ] = NULL;
    }

    if ( adip->adi_htmltype == DSGW_ATTRHTML_TEXTAREA ) {
	if ( adip->adi_vals == NULL ) {
	    output_textarea( adip->adi_argc, adip->adi_argv, adip->adi_attr,
		    NULL, 0, prefix, textopts );
	} else {
	    tval[ 1 ] = NULL;
	    for ( i = 0; i < valcount; ++i ) {
		tval[ 0 ] = valscopy[ i ];
		output_textarea( adip->adi_argc, adip->adi_argv,
			adip->adi_attr, tval, 1, prefix, textopts );
		if ( i < valcount - 1 ) {
		    fputs( "<BR>\n", stdout );
		}
	    }
	}
    } else {
	output_text_elements( adip->adi_argc, adip->adi_argv, adip->adi_attr,
		valscopy, prefix, adip->adi_htmltype, textopts );
    }

    if ( valscopy != NULL ) {
	ldap_value_free( valscopy );
	free( lines );
    }
}


/*
 * convert all occurrences of "$" in val to sep
 * un-escape any \HH sequences
 * if linesp != NULL, set *linesp equal to number of lines in val
 * return a malloc'd result
 */
static char *
mls_convertlines( char *val, char *sep, int *linesp )
{
    char	*valcopy, *p, *q;
    int		i, c, lines, seplen;

    if ( sep == NULL ) {
	sep = "";
	seplen = 0;
    } else {
	seplen = strlen( sep );
    }

    lines = 0;
    for ( q = val; *q != '\0'; ++q ) {
	if ( *q == '$' ) {
	    ++lines;
	}
    }

    if ( linesp != NULL ) {
	*linesp = lines;
    }

    valcopy = dsgw_ch_malloc( strlen( val ) + lines * seplen + 1 );

    /*
     * p points to the place we are copying to
     * q points to start of the line we are working on
     */
    p = valcopy;
    for ( q = val; *q != '\0'; ++q ) {
	if ( *q == '$' ) {		/* line separator */
	    strcpy( p, sep );
	    p += seplen;
	} else if ( *q == '\\' ) {	/* undo hex escapes */
	    if ( *++q == '\0' ) {
		break;
	    }
	    c = toupper( *q );
	    i = ( c >= 'A' ? ( c - 'A' + 10 ) : c - '0' );
	    i <<= 4;
	    if ( *++q == '\0' ) {
		break;
	    }
	    c = toupper( *q );
	    i += ( c >= 'A' ? ( c - 'A' + 10 ) : c - '0' );
	    *p++ = i;
	} else {
	    *p++ = *q;
	}
    }

    *p = '\0';

    return( valcopy );
}


static void
dn_edit( struct dsgw_attrdispinfo *adip )
{
    if (( adip->adi_opts & DSGW_ATTROPT_DNPICKER ) != 0 ) {
	dn_display( adip );
    } else {
	str_edit( adip );
    }
    return;
}


static void
dn_display( struct dsgw_attrdispinfo *adip )
{
    int		i, j, len, dncomps;
    char	*p, *staticlabel, *label, *urlprefix;
    char	*tmps = NULL, **rdns = NULL;

    staticlabel = get_arg_by_name( DSGW_ATTRARG_LABEL, adip->adi_argc,
	    adip->adi_argv );

    if (( p = get_arg_by_name( DSGW_ATTRARG_DNCOMP, adip->adi_argc,
		adip->adi_argv )) == NULL ) { 
	dncomps = 1;
    } else {
	dncomps = atoi( p );	/* 0 or "all" means show all components */
    }

    if (( adip->adi_opts & DSGW_ATTROPT_LINK2EDIT ) != 0 ) {
	urlprefix = dsgw_ch_strdup( dsgw_getvp( DSGW_CGINUM_EDIT ));
    } else {
	urlprefix = dsgw_build_urlprefix( dsgw_getvp( DSGW_CGINUM_DOSEARCH ));
    }
#ifdef DSGW_DEBUG
    dsgw_log( "dn_display: urlprefix is %s\n", urlprefix );
#endif

    for ( i = 0; adip->adi_vals != NULL && adip->adi_vals[ i ] != NULL; ++i ) {
	if ( staticlabel != NULL ) {
	    label = staticlabel;
	} else if ( !looks_like_dn( adip->adi_vals[ i ]) || 
		( rdns = ldap_explode_dn( adip->adi_vals[ i ],
		( adip->adi_opts & DSGW_ATTROPT_DNTAGS ) == 0 )) == NULL ) {
	    /* explode DN failed -- show entire DN */
	    label = adip->adi_vals[ i ];
	} else {
	    len = 1;	/* room for zero-termination */
	    for ( j = 0; rdns[ j ] != NULL && ( dncomps == 0 || j < dncomps );
		    ++ j ) {
		len += ( 2 + strlen( rdns[ j ] ));	/* rdn + ", " */
	    }
	    label = p = tmps = dsgw_ch_malloc( len );
	    for ( j = 0; rdns[ j ] != NULL && ( dncomps == 0 || j < dncomps );
		    ++ j ) {
		if ( j > 0 ) {
		    strcpy( p, ", " );
		    p += 2;
		}
		strcpy( p, rdns[ j ] );
		p += strlen( p );
	    }
	}

	if ( !did_output_as_special( adip->adi_argc, adip->adi_argv, label,
		adip->adi_vals[ i ] )) {
	    if (( adip->adi_opts & DSGW_ATTROPT_NOLINK ) == 0 &&
		    looks_like_dn( adip->adi_vals[ i ] )) {
		if (( adip->adi_opts & DSGW_ATTROPT_DNPICKER ) != 0 ) {
		    fputs( "<TR><TD>", stdout );
		}
		dsgw_html_href( urlprefix, adip->adi_vals[ i ], label,
			adip->adi_vals[ i ],
			get_arg_by_name( DSGW_ATTRARG_HREFEXTRA,
			adip->adi_argc, adip->adi_argv ));
		if (( adip->adi_opts & DSGW_ATTROPT_DNPICKER ) != 0 ) {
		    fputs( "</TD>\n<TD ALIGN=CENTER><INPUT TYPE=CHECKBOX ",
			    stdout );
		    fprintf( stdout, "VALUE=\"%s\" NAME=delete_%s ",
			   adip->adi_vals[ i ], adip->adi_attr );
		    fprintf( stdout, "onClick=\"aChg('%s');\"</TD>\n</TR>\n",
			    adip->adi_attr );
		}
	    } else {
		fputs( label, stdout );
	    }
	}

	if ( !( adip->adi_opts & DSGW_ATTROPT_DNPICKER ) &&
		adip->adi_vals[ i + 1 ] != NULL ) {
	    fputs( "<BR>\n", stdout );
	}

	if ( tmps != NULL ) {
	    free( tmps );
	}

	if ( rdns != NULL ) {
	    ldap_value_free( rdns );
	}
    }

    /* Output a javascript array of values for this attribute */
    if (( adip->adi_opts & DSGW_ATTROPT_DNPICKER ) != 0 ) {
	fputs( "<SCRIPT LANGUAGE=\"JavaScript\">\n", stdout );
	fputs( "<!-- Hide from non-JavaScript-capable browsers\n", stdout );
	fprintf( stdout, "var %s_values = new Object;\n", adip->adi_attr );
	for ( i = 0; adip->adi_vals != NULL && adip->adi_vals[ i ] != NULL; ++i ) {
	    char *edn;
	    edn = dsgw_strdup_escaped( adip->adi_vals[ i ]);
	    fprintf( stdout, "%s_values[%d] = \"%s\";\n", adip->adi_attr, i,
		    edn );
	    free( edn );
	}
	fprintf( stdout, "%s_values.count = %d;\n", adip->adi_attr, i );
	fputs( "// End hiding -->\n", stdout );
	fputs( "</SCRIPT>\n", stdout );
    }

    free( urlprefix );
}
static  void
ava_display (struct dsgw_attrdispinfo *adip )
{
    int		i;
    char	*s_label, *label, *urlprefix;
    char	line[BIG_LINE];

    urlprefix = dsgw_build_urlprefix( dsgw_getvp( DSGW_CGINUM_AVAEDIT ));

    for ( i = 0; adip->adi_vals[ i ] != NULL; ++i ) {
	if (( label = strchr( adip->adi_vals[ i ], '{' )) == NULL ) {
	    label = adip->adi_vals[ i ];
	} else {
	    *label = ' ';
	}
	s_label = ldap_dn2ufn(label);
	label = s_label;
	if (strlen(label) > 45) {
		label += 41;
		*label++ = '.';
		*label++ = '.';
		*label++ = '.';
		*label = '\0';
	}
	label = s_label;

	if ( !did_output_as_special( adip->adi_argc, adip->adi_argv, label,
		adip->adi_vals[ i ] )) {
	    sprintf (line,"%s/%s", adip->adi_entrydn, adip->adi_vals[ i ]);
	    if (( adip->adi_opts & DSGW_ATTROPT_NOLINK ) == 0 ) {
		dsgw_html_href(urlprefix, line, label,line,
			get_arg_by_name( DSGW_ATTRARG_HREFEXTRA,
				adip->adi_argc, adip->adi_argv ));
	    } else {
		fputs( line, stdout );
	    }
	}

	if ( adip->adi_vals[ i + 1 ] != NULL ) {
	    fputs( "<BR>\n", stdout );
	}
    }
    free (s_label);
}


static void
mail_display( struct dsgw_attrdispinfo *adip )
{
    int		i;

    for ( i = 0; adip->adi_vals[ i ] != NULL; ++i ) {
	if ( !did_output_as_special( adip->adi_argc, adip->adi_argv,
		    adip->adi_vals[ i ], adip->adi_vals[ i ] )) {
	    if (( adip->adi_opts & DSGW_ATTROPT_NOLINK ) == 0 ) {
		dsgw_html_href( "mailto:", adip->adi_vals[ i ], adip->adi_vals[ i ], NULL,
			get_arg_by_name( DSGW_ATTRARG_HREFEXTRA,
				adip->adi_argc, adip->adi_argv ));
	    } else {
		fputs( adip->adi_vals[ i ], stdout );
	    }
	}

	if ( adip->adi_vals[ i + 1 ] != NULL ) {
	    fputs( "<BR>\n", stdout );
	}
    }
}


static void
url_display( struct dsgw_attrdispinfo *adip )
{
    int		i;
    char	*savep, *label;

    for ( i = 0; adip->adi_vals[ i ] != NULL; ++i ) {
	if (( label = strchr( adip->adi_vals[ i ], ' ' )) == NULL ) {
	    label = adip->adi_vals[ i ];
	    savep = NULL;
	} else {
	    savep = label;
	    *label++ = '\0';
	}

	if ( !did_output_as_special( adip->adi_argc, adip->adi_argv, label,
		adip->adi_vals[ i ] )) {
	    if (( adip->adi_opts & DSGW_ATTROPT_NOLINK ) == 0 ) {
		dsgw_html_href( NULL, adip->adi_vals[ i ], label, NULL,
			get_arg_by_name( DSGW_ATTRARG_HREFEXTRA,
				adip->adi_argc, adip->adi_argv ));
	    } else {
		fputs( adip->adi_vals[ i ], stdout );
	    }
	}

	if ( savep != NULL ) {
	    *savep = ' ';
	}

	if ( adip->adi_vals[ i + 1 ] != NULL ) {
	    fputs( "<BR>\n", stdout );
	}
    }
}


static void
bool_display( struct dsgw_attrdispinfo *adip )
{
    int		boolval, free_onclick, pre_idx;
    char	*usestr, *truestr, *falsestr, *checked;
    char	*nameprefix, *onclick;

    if ( adip->adi_vals == NULL || adip->adi_vals[ 0 ] == NULL ) {
	return;
    }

    checked = " CHECKED";

    if (( adip->adi_opts & DSGW_ATTROPT_EDITABLE ) == 0 ) {
	nameprefix = onclick = "";
	free_onclick = 0;
    } else {
	char *onclickfmt = " onClick=\"aChg('%s')\"";
        /* I18n: additional language tag support */
        if ( gc->gc_ldapversion >= LDAP_VERSION3 ) {
            if (( adip->adi_opts & DSGW_ATTROPT_UNIQUE )) {
                if (( adip->adi_opts & DSGW_ATTROPT_LANG )) {
                    pre_idx = DSGW_MOD_PREFIX_LANG_UNIQUE;
                } else if (( adip->adi_opts & DSGW_ATTROPT_EN )) {
                    pre_idx = DSGW_MOD_PREFIX_EN_UNIQUE;
                } else if (( adip->adi_opts & DSGW_ATTROPT_PHNTC )) {
                    pre_idx = DSGW_MOD_PREFIX_PHNTC_UNIQUE;
                } else {
                    pre_idx = DSGW_MOD_PREFIX_UNIQUE;
                }
            } else if (( adip->adi_opts & DSGW_ATTROPT_LANG )) {
                pre_idx = DSGW_MOD_PREFIX_LANG;
            } else if (( adip->adi_opts & DSGW_ATTROPT_EN )) {
                pre_idx = DSGW_MOD_PREFIX_EN;
            } else if (( adip->adi_opts & DSGW_ATTROPT_PHNTC )) {
                pre_idx = DSGW_MOD_PREFIX_PHNTC;
            } else {
                pre_idx = DSGW_MOD_PREFIX_NORMAL;
            }
	} else {
            if (( adip->adi_opts & DSGW_ATTROPT_UNIQUE ) == 0 ) {
                pre_idx = DSGW_MOD_PREFIX_NORMAL;
            } else {
                pre_idx = DSGW_MOD_PREFIX_UNIQUE;
            }
	}
	nameprefix = (( adip->adi_opts & DSGW_ATTROPT_ADDING ) == 0 ) ?
		replace_prefixes[ pre_idx ] : add_prefixes[ pre_idx ];
#ifdef DSGW_DEBUG
    dsgw_log( "bool_display: prefix is %s (odi_opts: 0x%x, pre_idx: %d)\n",
			 nameprefix, adip->adi_opts, pre_idx );
#endif
	onclick = dsgw_ch_malloc( strlen( onclickfmt ) +
		strlen( adip->adi_attr ) + 1 );
	sprintf( onclick, onclickfmt, adip->adi_attr );
	free_onclick = 1;
    }

    if (( truestr = get_arg_by_name( DSGW_ATTRARG_TRUESTR, adip->adi_argc,
	    adip->adi_argv )) == NULL ) {
	truestr = DSGW_ATTRARG_TRUESTR;
    }
    if (( falsestr = get_arg_by_name( DSGW_ATTRARG_FALSESTR, adip->adi_argc,
	    adip->adi_argv )) == NULL ) {
	falsestr = DSGW_ATTRARG_FALSESTR;
    }

    boolval = ( toupper( adip->adi_vals[ 0 ][ 0 ] ) == 'T' );

    if ( adip->adi_htmltype == DSGW_ATTRHTML_RADIO ) {
	fprintf( stdout, "<INPUT TYPE=\"radio\" NAME=\"%s%s\" "
		"VALUE=\"TRUE\"%s%s>%s<BR>\n", nameprefix, adip->adi_attr,
		boolval ? checked : "", onclick, truestr );
	fprintf( stdout, "<INPUT TYPE=\"radio\" NAME=\"%s%s\" "
		"VALUE=\"FALSE\"%s%s>%s<BR>\n", nameprefix, adip->adi_attr,
		boolval ? "" : checked, onclick, falsestr );
    } else if ( adip->adi_htmltype == DSGW_ATTRHTML_CHECKBOX ) {
	fprintf( stdout, "<INPUT TYPE=\"checkbox\" NAME=\"%s%s\" "
		"VALUE=\"TRUE\"%s%s\">%s\n", nameprefix, adip->adi_attr,
		boolval ? checked : "", onclick, truestr );
    } else {
	usestr =  boolval ? truestr : falsestr;
	if ( !did_output_as_special( adip->adi_argc, adip->adi_argv, usestr,
		adip->adi_vals[ 0 ] )) {
	    fputs( boolval ? truestr : falsestr, stdout );
	}
    }
}


static void
bool_edit( struct dsgw_attrdispinfo *adip )
{
    if ( adip->adi_htmltype == DSGW_ATTRHTML_RADIO ||
	    adip->adi_htmltype == DSGW_ATTRHTML_CHECKBOX ) {
	bool_display( adip );
    } else {
	str_edit( adip );
    }
}


static void
time_display( struct dsgw_attrdispinfo *adip )
{
    int		i;

    for ( i = 0; adip->adi_vals[ i ] != NULL; ++i ) {
	if ( !did_output_as_special( adip->adi_argc, adip->adi_argv,
		adip->adi_vals[ i ], adip->adi_vals[ i ] )) {
	    fputs( time2text( adip->adi_vals[ i ],
		( adip->adi_opts & DSGW_ATTROPT_DATEONLY ) != 0 ), stdout );
	}

	if ( adip->adi_vals[ i + 1 ] != NULL ) {
	    fputs( "<BR>\n", stdout );
	}
    }
}


/*
 * handle special "within=" and "href=" options
 * return 0 if nothing was output or 1 if something was.
 */
static int
did_output_as_special( int argc, char **argv, char *label, char *val )
{
    char	*href, *within;

    if (( href = get_arg_by_name( DSGW_ATTRARG_HREF, argc, argv )) == NULL &&
	    ( within = get_arg_by_name( DSGW_ATTRARG_WITHIN, argc,
	    argv )) == NULL ) {
	return( 0 );
    }

    if ( href == NULL ) {
	dsgw_substitute_and_output( within, "--value--", val, 1 );
    } else {
	dsgw_html_href( NULL, href, label, val,
		get_arg_by_name( DSGW_ATTRARG_HREFEXTRA, argc, argv ));
    }

    return( 1 );
}


/*
 * The GET2BYTENUM() macro, time2text(), and gtime() functions are taken
 * with slight changes (to handle 4-digit years) from libldap/tmplout.c
 */
#define GET2BYTENUM( p )	(( *p - '0' ) * 10 + ( *(p+1) - '0' ))

static char *
time2text( char *ldtimestr, int dateonly )
{
    int			len;
    struct tm		t;
    char		*p, *timestr, zone;
    time_t		gmttime;

    memset( (char *)&t, 0, sizeof( struct tm ));
    if (( len = strlen( ldtimestr )) < 13 ) {
	return( ldtimestr );
    }
    if ( len > 15 ) {	/* throw away excess from 4-digit year time string */
	len = 15;
    } else if ( len == 14 ) {
	len = 13;	/* assume we have a time w/2-digit year (len=13) */
    }

    for ( p = ldtimestr; p - ldtimestr + 1 < len; ++p ) {
	if ( !isdigit( *p )) {
	    return( ldtimestr );
	}
    }

    p = ldtimestr;
    t.tm_year = GET2BYTENUM( p ); p += 2;
    if ( len == 15 ) {
	t.tm_year = 100 * t.tm_year;
	t.tm_year += GET2BYTENUM( p ); p += 2;
    }
    t.tm_mon = GET2BYTENUM( p ) - 1; p += 2;
    t.tm_mday = GET2BYTENUM( p ); p += 2;
    t.tm_hour = GET2BYTENUM( p ); p += 2;
    t.tm_min = GET2BYTENUM( p ); p += 2;
    t.tm_sec = GET2BYTENUM( p ); p += 2;

    if (( zone = *p ) == 'Z' ) {	/* GMT */
	zone = '\0';	/* no need to indicate on screen, so we make it null */
    }

    gmttime = gtime( &t );
    timestr = ctime( &gmttime );

    timestr[ strlen( timestr ) - 1 ] = zone;	/* replace trailing newline */
    if ( dateonly ) {
	strcpy( timestr + 11, timestr + 20 );
    }

    return( timestr );
}



/* gtime.c - inverse gmtime */

#if !defined( MACOS ) && !defined( _WINDOWS ) && !defined( DOS )
#include <sys/time.h>
#endif /* !MACOS */

/* gtime(): the inverse of localtime().
	This routine was supplied by Mike Accetta at CMU many years ago.
 */

static int	dmsize[] = {
    31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31
};

#define	dysize(y)	\
	(((y) % 4) ? 365 : (((y) % 100) ? 366 : (((y) % 400) ? 365 : 366)))

#define	YEAR(y)		((y) >= 100 ? (y) : (y) + 1900)

/*  */

static long	gtime ( struct tm *tm )
{
    register int    i,
                    sec,
                    mins,
                    hour,
                    mday,
                    mon,
                    year;
    register long   result;

    if ((sec = tm -> tm_sec) < 0 || sec > 59
	    || (mins = tm -> tm_min) < 0 || mins > 59
	    || (hour = tm -> tm_hour) < 0 || hour > 24
	    || (mday = tm -> tm_mday) < 1 || mday > 31
	    || (mon = tm -> tm_mon + 1) < 1 || mon > 12)
	return ((long) -1);
    if (hour == 24) {
	hour = 0;
	mday++;
    }
    year = YEAR (tm -> tm_year);

    result = 0L;
    for (i = 1970; i < year; i++)
	result += dysize (i);
    if (dysize (year) == 366 && mon >= 3)
	result++;
    while (--mon)
	result += dmsize[mon - 1];
    result += mday - 1;
    result = 24 * result + hour;
    result = 60 * result + mins;
    result = 60 * result + sec;

    return result;
}


static int
caseexact_sort( const char **a, const char **b )
{
    return( strcmp( *a, *b ));
}


static int
looks_like_dn( char *s )
{
    return( strchr( s, '=' ) != NULL );
}


static void
do_searchdesc( dsgwtmplinfo *tip )
{
    char	countbuf[ 64 ];	/* plenty big enough */

    if ( tip->dsti_entrycount == 0 ) {
	strcpy( countbuf, XP_GetClientStr(DBT_noEntries_) );
    } else if ( tip->dsti_entrycount == 1 ) {
	strcpy( countbuf, XP_GetClientStr(DBT_1Entry_) );
    } else {
	sprintf( countbuf, XP_GetClientStr(DBT_DEntries_), tip->dsti_entrycount );
    }

    fprintf( stdout, "%s %s%s\n", countbuf,
	    ( tip->dsti_options & DSGW_DISPLAY_OPT_CUSTOM_SEARCHDESC ) == 0
	    ? XP_GetClientStr(DBT_whereThe_) : "", tip->dsti_searchdesc );

    if ( tip->dsti_searcherror != NULL && *tip->dsti_searcherror != '\0' ) {
	fprintf( stdout, "<BR>%s\n", tip->dsti_searcherror );
    }
}


static void
do_editbutton( char *dn, char *encodeddn, int argc, char **argv )
{
    char        *buttonlabel, **rdns;

    if (( buttonlabel = get_arg_by_name( DSGW_ARG_BUTTON_LABEL, argc,
	    argv )) == NULL ) {
	buttonlabel = XP_GetClientStr(DBT_edit_);
    }

    if (( rdns = ldap_explode_dn( dn, 1 )) != NULL ) {
	fprintf( stdout,
		"<INPUT TYPE=\"hidden\" NAME=\"authhint\" VALUE=\"%s\">\n",
		rdns[ 0 ] );
	ldap_value_free( rdns );
    }

    fprintf( stdout, "<INPUT TYPE=\"hidden\" NAME=\"authdesturl\">\n"
	    "<INPUT TYPE=\"button\" VALUE=\"%s\" "
	    "onClick=\"authOrEdit('%s')\">\n", buttonlabel, encodeddn );
}


static void
do_savebutton( unsigned long dispopts, int argc, char **argv )
{
    char	*buttonlabel, *checksubmit;

    if (( buttonlabel = get_arg_by_name( DSGW_ARG_BUTTON_LABEL, argc,
	    argv )) == NULL ) {
	buttonlabel = XP_GetClientStr(DBT_saveChanges_);
    }

    fprintf( stdout, "<INPUT TYPE=\"button\" VALUE=\"%s\" onClick=\"",
	    buttonlabel );
    if (( checksubmit = get_arg_by_name( DSGW_ARG_BUTTON_CHECKSUBMIT, argc,
	    argv )) != NULL ) {
	fprintf( stdout, "if (%s) ", checksubmit );
    }
    fprintf( stdout, "submitModify('%s', null )\">\n",
	    ( dispopts & DSGW_DISPLAY_OPT_ADDING ) == 0 ? "modify" : "add" );
}


static void
do_deletebutton( int argc, char **argv )
{
    char	*buttonlabel, *prompt;

    if (( buttonlabel = get_arg_by_name( DSGW_ARG_BUTTON_LABEL, argc,
	    argv )) == NULL ) {
	buttonlabel = XP_GetClientStr(DBT_delete_);
    }

    if (( prompt = get_arg_by_name( DSGW_ARG_BUTTON_PROMPT, argc,
	    argv )) == NULL ) {
	prompt = XP_GetClientStr(DBT_deleteThisEntry_);
    }

    fprintf( stdout,
	    "<INPUT TYPE=\"button\" VALUE=\"%s\" onClick=\"submitModify("
	    "'delete', '%s' )\">\n", buttonlabel, prompt );
}


static void
do_renamebutton( char *dn, int argc, char **argv )
{
    char	*buttonlabel, *prompt, *oldname, **rdns, *tag, *template;
    int		len;

    if (( buttonlabel = get_arg_by_name( DSGW_ARG_BUTTON_LABEL, argc,
	    argv )) == NULL ) {
	buttonlabel = XP_GetClientStr(DBT_rename_);
    }

    if (( prompt = get_arg_by_name( DSGW_ARG_BUTTON_PROMPT, argc,
	    argv )) == NULL ) {
	prompt = XP_GetClientStr(DBT_enterANewNameForThisEntry_);
    }

    if (( template = get_arg_by_name( DSGW_ARG_BUTTON_TEMPLATE, argc,
	    argv )) == NULL ) {
	template = "";
    }

    if (( rdns = ldap_explode_dn( dn, 0 )) != NULL &&
	    ( oldname = strchr( rdns[ 0 ], '=' )) != NULL ) {
	*oldname++ = '\0';
	tag = rdns[ 0 ];
	if ( *oldname == '"' ) {
	    ++oldname;
	    if (( len = strlen( oldname )) > 0
		    && oldname[ len - 1 ] == '"' ) {
		oldname[ len - 1 ] = '\0';
	    }
	}
    } else {
	oldname = dn;
	tag = "";
    }

    fprintf( stdout , "<INPUT TYPE=\"button\" VALUE=\"%s\""
      " onClick=\"renameEntry('%s','%s','", buttonlabel, tag, template );
    dsgw_escape_quotes_output( stdout, oldname );
    fputs( "')\">\n", stdout );

    if ( rdns != NULL ) {
	ldap_value_free( rdns );
    }
}


static void
do_editasbutton( int argc, char **argv )
{
    char	*template, *buttonlabel;

    if (( template = get_arg_by_name( DSGW_ARG_BUTTON_TEMPLATE, argc,
	    argv )) == NULL ) {
	template = "";
    }

    if (( buttonlabel = get_arg_by_name( DSGW_ARG_BUTTON_LABEL, argc,
	    argv )) == NULL ) {
	buttonlabel = XP_GetClientStr(DBT_editAs_);
    }

    fprintf( stdout, "<INPUT TYPE=\"button\" VALUE=\"%s\""
	    " onClick=\"EditEntryAs('%s')\">\n", buttonlabel, template );
}

static void
do_postedvalue( int argc, char **argv )
{
    char	*name, *postedvalue;

    fputs( "VALUE=\"", stdout );

    if (( name = get_arg_by_name( DSGW_POSTEDVARARG_NAME, argc, argv ))
	    == NULL ) {
	fprintf( stdout, XP_GetClientStr(DBT_missingS_1), DSGW_POSTEDVARARG_NAME );
    } else if (( postedvalue = dsgw_get_cgi_var( name, DSGW_CGIVAR_OPTIONAL ))
	    != NULL ) {
	fputs( postedvalue, stdout );
    }

    fputs( "\"\n", stdout );
}


static void
do_passwordfield( unsigned long dispopts, int argc, char **argv,
	char *fieldname )
{
    output_text_elements( argc, argv, fieldname, NULL, "",
	    DSGW_ATTRHTML_PASSWORD, dispopts );
}


static void
do_helpbutton( unsigned long dispopts, int argc, char **argv )
{
    char        *topic;

    if (( topic = get_arg_by_name( DSGW_ARG_BUTTON_TOPIC, argc,
	    argv )) == NULL ) {
	topic = "";
    }

    dsgw_emit_helpbutton( topic );
}


static void
do_closebutton( unsigned long dispopts, int argc, char **argv )
{
    char	*buttonlabel;

    if (( buttonlabel = get_arg_by_name( DSGW_ARG_BUTTON_LABEL, argc,
	    argv )) == NULL ) {
	buttonlabel = XP_GetClientStr(DBT_closeWindow_3);
    }

    fprintf( stdout, "<INPUT TYPE=\"button\" VALUE=\"%s\" "
	    "onClick=\"%s\">", buttonlabel,
	    ( dispopts & DSGW_DISPLAY_OPT_EDITABLE ) == 0 ? "top.close()" :
	    "closeIfOK()" );
}


static void
do_dneditbutton( unsigned long dispopts, int argc, char **argv )
{
    char *label, *template, *attr, *desc;

    if (( label = get_arg_by_name( DSGW_ARG_DNEDIT_LABEL, argc,
	    argv )) == NULL ) {
	label = XP_GetClientStr(DBT_edit_1);
    }
    if (( template = get_arg_by_name( DSGW_ARG_DNEDIT_TEMPLATE, argc,
	    argv )) == NULL ) {
	template = "dnedit";
    }
    if (( attr = get_arg_by_name( DSGW_ARG_DNEDIT_ATTR, argc,
	    argv )) == NULL ) {
	fputs( "<!-- Error: missing attr= argument in DS_DNEDITBUTTON "
		"directive -->\n", stdout );
	return;
    }
    if (( desc = get_arg_by_name( DSGW_ARG_DNEDIT_DESC, argc,
	    argv )) == NULL ) {
	desc = attr;
    }

    fprintf( stdout, "<INPUT TYPE=\"button\" VALUE=\"%s\""
	     " onClick=\"DNEdit('%s', '%s', '%s')\">\n", label, template,
	     attr, desc );
}

static void
do_avaeditbutton( unsigned long dispopts, int argc, char **argv )
{
    char *label, *template, *attr, *desc;

    if (( label = get_arg_by_name( DSGW_ARG_DNEDIT_LABEL, argc,
	    argv )) == NULL ) {
	label = XP_GetClientStr(DBT_edit_1);
    }
    if (( template = get_arg_by_name( DSGW_ARG_DNEDIT_TEMPLATE, argc,
	    argv )) == NULL ) {
	template = "avaedit";
    }
    if (( attr = get_arg_by_name( DSGW_ARG_DNEDIT_ATTR, argc,
	    argv )) == NULL ) {
	fputs( "<!-- Error: missing attr= argument in DS_DNEDITBUTTON "
		"directive -->\n", stdout );
	return;
    }
    if (( desc = get_arg_by_name( DSGW_ARG_DNEDIT_DESC, argc,
	    argv )) == NULL ) {
	desc = attr;
    }

    fprintf( stdout, "<INPUT TYPE=\"button\" VALUE=\"%s\""
	     " onClick=\"AVAEdit('%s', '%s', '%s')\">\n", label, template,
	     attr, desc );
}


static void
do_viewswitcher( char *template, char *dn, int argc, char **argv )
{
    dsgwtmplset	*tsp;
    dsgwview	*vp;
    char	*s, *altprefix, *altsuffix, *curprefix, *cursuffix;

    /* first we see if this template is part of a template set */
    for ( tsp = gc->gc_tmplsets; tsp != NULL; tsp = tsp->dstset_next ) {
	for ( vp = tsp->dstset_viewlist; vp != NULL; vp = vp->dsview_next ) {
	    if ( strcasecmp( vp->dsview_template, template ) == 0 ) {
		break;
	    }
	}
	if ( vp != NULL ) {
	    break;
	}
    }

    if ( tsp == NULL || tsp->dstset_viewcount == 1 ) {
	return;	/* not part of a set at all or only one view in the set */
    }

    /* emit view switcher prefix */
    if (( s = get_arg_by_name( "prefix", argc, argv )) == NULL ) {
	s = "<TABLE CELLPADDING=6 BORDER=0><TR VALIGN=center>\n";
    }
    fputs( s, stdout );

    /* retrieve view item prefix and suffix arguments */
    if (( altprefix = get_arg_by_name( "altprefix", argc, argv )) == NULL ) {
	altprefix = "<TD BGCOLOR=#B0B0B0>\n";
    }
    if (( altsuffix = get_arg_by_name( "altsuffix", argc, argv )) == NULL ) {
	altsuffix = "</TD>\n";
    }
    if (( curprefix = get_arg_by_name( "curprefix", argc, argv )) ==
	    NULL ) {
	curprefix = "<TD BGCOLOR=#808080><FONT COLOR=#000000><B>\n";
    }
    if (( cursuffix = get_arg_by_name( "currentsuffix", argc, argv )) ==
	    NULL ) {
	cursuffix = "</B></FONT></TD>\n";
    }

    /* emit one table cell item (or similar) for each available view */
    for ( vp = tsp->dstset_viewlist; vp != NULL; vp = vp->dsview_next ) {
	if ( strcasecmp( vp->dsview_template, template ) == 0 ) {
	    fprintf( stdout, "%s%s%s", curprefix, vp->dsview_caption,
		    cursuffix );
	} else {
	    fprintf( stdout, "%s\n<A HREF=\"", altprefix );
	    if ( vp->dsview_jscript == NULL ) {
		fprintf( stdout, "javascript:EditEntryAs('%s')",
			vp->dsview_template );
	    } else {
		dsgw_substitute_and_output( vp->dsview_jscript, "--dn--",
			dn, 1 );
	    }
	    fprintf( stdout, "\">%s</A>\n%s", vp->dsview_caption, altsuffix );
	}
    }

    /* emit view switcher suffix */
    if (( s = get_arg_by_name( "suffix", argc, argv )) == NULL ) {
	s = "</TR></TABLE>\n";
    }
    fputs( s, stdout );
}


static void
do_attrvalset( dsgwtmplinfo *tip, char *dn, unsigned long dispopts,
	int argc, char **argv )
{
    dsgwavset	*avp;
    char	*s, *valuearg, *prefix, *suffix;
    int		i, setpos, len, maxvallen;

    /*
     * locate "set" element in argv array so we can replace it later
     * with "value="
     */
    if (( setpos = dsgw_get_arg_pos_by_name( DSGW_ARG_AVSET_SET, argc,
	    argv )) < 0 ) {
        fprintf( stdout, XP_GetClientStr(DBT_missingSN_), DSGW_ARG_AVSET_SET );
        return;
    }
    s = &argv[ setpos ][ 4 ];

    for ( avp = gc->gc_avsets; avp != NULL; avp = avp->dsavset_next ) {
	if ( strcasecmp( s, avp->dsavset_handle ) == 0 ) {
	    break;
	}
    }
    if ( avp == NULL ) {
	fprintf( stdout, XP_GetClientStr(DBT_unknownSetSN_), s );
	return;
    }

    prefix = get_arg_by_name( "prefix", argc, argv );
    suffix = get_arg_by_name( "suffix", argc, argv );

    /* repeatedly call on do_attribute() to perform all the difficult work */
    maxvallen = 0;
    valuearg = NULL;
    for ( i = 0; i < avp->dsavset_itemcount; ++i ) {
	if ( prefix != NULL ) {
	    fputs( prefix, stdout );
	}
	fputs( avp->dsavset_prefixes[ i ], stdout );

	/* construct "value=XXX" arg. and place in argv array */
	if (( len = strlen( avp->dsavset_values[ i ] )) > maxvallen ||
		valuearg == NULL ) {
	    maxvallen = len;
	    valuearg = dsgw_ch_realloc( valuearg, maxvallen + 7 );
	}
	sprintf( valuearg, "value=%s", avp->dsavset_values[ i ] );
	argv[ setpos ] = valuearg;

	do_attribute( tip, dn, dispopts, argc, argv );

	fputs( avp->dsavset_suffixes[ i ], stdout );
	if ( suffix != NULL ) {
	    fputs( suffix, stdout );
	    fputc( '\n', stdout );
	}
    }
}


/*
 * function called back by dsgw_parse_line() to evaluate IF directives.
 * return non-zero for true, zero for false.
 */
static int
condition_is_true( int argc, char **argv, void *arg )
{
    dsgwtmplinfo	*tip;

    if ( argc < 1 ) {
	return( 0 );
    }

    tip = (dsgwtmplinfo *)arg; 

    if ( strcasecmp( argv[0], DSGW_COND_FOUNDENTRIES ) == 0 ) {
	return( tip->dsti_entrycount > 0 );
    }

    if ( strcasecmp( argv[0], DSGW_COND_ADDING ) == 0 ) {
	return(( tip->dsti_options & DSGW_DISPLAY_OPT_ADDING ) != 0 );
    }
 
    if ( strcasecmp( argv[0], DSGW_COND_EDITING ) == 0 ) {
	return(( tip->dsti_options & DSGW_DISPLAY_OPT_EDITABLE ) != 0 &&
		( tip->dsti_options & DSGW_DISPLAY_OPT_ADDING ) == 0 );
    }
 
    if ( strcasecmp( argv[0], DSGW_COND_DISPLAYING ) == 0 ) {
	return(( tip->dsti_options & DSGW_DISPLAY_OPT_EDITABLE ) == 0 );
    }

    if ( strcasecmp( argv[0], DSGW_COND_BOUND ) == 0 ) {
	char	*dn;

	dsgw_get_bind_credentials( &dn, NULL );
	return( dn != NULL );
    }

    if ( strcasecmp( argv[0], DSGW_COND_BOUNDASTHISENTRY ) == 0 ) {
	return( dsgw_bound_as_dn( tip->dsti_entrydn, 0 ));
    }

    if ( strcasecmp( argv[0], DSGW_COND_ATTRHASVALUES ) == 0 ) {
	/*
	 * format of IF statment is:
	 *    <-- IF "AttributeHasValues" "ATTRIBUTE" "MINIMUM_COUNT" -->
	 * MINIMUM_COUNT is an optional number.
	 */
	char	**vals;
	int	rc, minimum;

	if ( argc < 2 ||  tip->dsti_entry == NULL ||
		( vals = ldap_get_values( tip->dsti_ld, tip->dsti_entry,
		argv[1] )) == NULL ) {
	    return( 0 );
	}
	minimum = ( argc < 3 ) ? 1 : atoi( argv[ 2 ] );
	rc = ( minimum <= 1 || ldap_count_values( vals ) >= minimum );
	ldap_value_free( vals );
	return( rc );
    }

    if ( strcasecmp( argv[0], DSGW_COND_ATTRHASTHISVALUE ) == 0 ) {
	/*
	 * format of IF statment is:
	 *    <-- IF "AttributeHasThisValue" "ATTRIBUTE" "SYNTAX" "VALUE" -->
	 */
	char			**vals;
	int			i, rc;
	struct attr_handler	*ahp;

	if ( argc < 4 ||  tip->dsti_entry == NULL ||
		( vals = ldap_get_values( tip->dsti_ld, tip->dsti_entry,
		argv[1] )) == NULL ) {
	    return( 0 );
	}
	if (( ahp = syntax2attrhandler( argv[2] )) == NULL ) {
	    fprintf( stdout, XP_GetClientStr(DBT_unknownSyntaxSN_1), argv[2] );
	    return( 0 );
	}

	rc = 0;
	for ( i = 0; vals[ i ] != NULL; ++i ) {
	    if ( (*ahp->ath_compare)( (const char **)&vals[i],
		    (const char **)&argv[3] ) == 0 ) {
		rc = 1;
		break;
	    }
	}
	ldap_value_free( vals );
	return( rc );
    }

    /* pass unrecognized conditionals to simple conditional handler */
    return( dsgw_simple_cond_is_true( argc, argv, NULL ));
}
