/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#include "dsgw.h"
#include "dbtdsgw.h"


static char *dsgw_err2string( int err );
static char *dsgw_ldaperr2longstring( int err );

struct dsgwerr {
    int		dsgwerr_code;
    int		dsgwerr_msg;
};


/* all of the DSGW_ERR_... #defines are in dsgw.h */
static struct dsgwerr dsgw_errs[] = {
    { DSGW_ERR_BADMETHOD,
        DBT_unknownHttpRequestMethod_ },
    { DSGW_ERR_BADFORMDATA,
        DBT_invalidOrIncompleteHtmlFormData_ },
    { DSGW_ERR_NOMEMORY,
        DBT_outOfMemory_ },
    { DSGW_ERR_MISSINGINPUT,
        DBT_requiredQueryFormInputIsMissing_ },
    { DSGW_ERR_BADFILEPATH,
        DBT_illegalCharacterInFilePath_ },
    { DSGW_ERR_BADCONFIG,
        DBT_badOrMissingConfigurationFile_ },
    { DSGW_ERR_LDAPINIT,
        DBT_unableToInitializeLdap_ },
    { DSGW_ERR_LDAPGENERAL,
        DBT_anErrorOccurredWhileContactingTh_ },
    { DSGW_ERR_UNKSRCHTYPE,
        DBT_unknownSearchObjectType_ },
    { DSGW_ERR_UNKATTRLABEL,
        DBT_unknownAttributeLabel_ },
    { DSGW_ERR_UNKMATCHPROMPT,
        DBT_unknownMatchPrompt_ },
    { DSGW_ERR_NOFILTERS,
        DBT_noSearchFiltersForObjectType_ },
    { DSGW_ERR_OPENHTMLFILE,
        DBT_unableToOpenHtmlTemplateFile_ },
    { DSGW_ERR_SEARCHMODE,
        DBT_unknownSearchModeUseSmartComplex_ },
    { DSGW_ERR_LDAPURL_NODN,
        DBT_distinguishedNameMissingInUrl_ },
    { DSGW_ERR_LDAPURL_BADSCOPE,
        DBT_unknownScopeInUrlShouldBeBaseSub_ },
    { DSGW_ERR_LDAPURL_NOTLDAP,
        DBT_unrecognizedUrlOrUnknownError_ },
    { DSGW_ERR_LDAPURL_BAD,
        DBT_badUrlFormat_ },
    { DSGW_ERR_INTERNAL,
        DBT_internalError_ },
    { DSGW_ERR_WRITEINDEXFILE,
        DBT_unableToWriteTemplateIndexFile_ },
    { DSGW_ERR_OPENINDEXFILE,
        DBT_unableToOpenTemplateIndexFile_ },
    { DSGW_ERR_OPENDIR,
        DBT_unableToReadDirectory_ },
    { DSGW_ERR_SSLINIT,
        DBT_ldapSslInitializationFailedCheck_ },
    { DSGW_ERR_NOSECPATH,
        DBT_forTheUsersAndGroupsFormsToWorkO_ },
    { DSGW_CKDB_KEY_NOT_PRESENT,
        DBT_authenticationCredentialsNotFoun_ },
    { DSGW_CKDB_DBERROR,
        DBT_errorRetrievingDataFromTheAuthen_ },
    { DSGW_CKDB_EXPIRED,
        DBT_yourAuthenticationCredentialsHav_ },
    { DSGW_CKDB_RNDSTRFAIL,
        DBT_unableToCreateAuthenticationCred_ },
    { DSGW_CKDB_NODN,
        DBT_noDistinguishedNameWasProvidedWh_ },
    { DSGW_CKDB_CANTOPEN,
        DBT_cannotOpenAuthenticationDatabase_ },
    { DSGW_CKDB_CANTAPPEND,
        DBT_couldNotAppendDataToTheAuthentic_ },
    { DSGW_ERR_NO_MGRDN,
        DBT_noDirectoryManagerIsDefined_ },
    { DSGW_ERR_NOSEARCHSTRING,
        DBT_noSearchStringWasProvidedPleaseT_ },
    { DSGW_ERR_CONFIGTOOMANYARGS,
        DBT_tooManyArgumentsOnOneLineInTheCo_ },
    { DSGW_ERR_WSAINIT,
        DBT_failedToInitializeWindowsSockets_ },
    { DSGW_ERR_ADMSERV_CREDFAIL,
        DBT_authenticationCredentialsCouldNo_ },
    { DSGW_ERR_LDAPDBURL_NODN,
        DBT_distinguishedNameMissingInLdapdb_ },
    { DSGW_ERR_LDAPDBURL_NOTLDAPDB,
        DBT_unrecognizedUrlOrUnknownError_1 },
    { DSGW_ERR_LDAPDBURL_BAD,
        DBT_badUrlFormat_1 },
    { DSGW_ERR_LCACHEINIT,
        DBT_anErrorOccurredWhileInitializing_ },
    { DSGW_ERR_SERVICETYPE,
        DBT_unknownDirectoryServiceTypeUseLo_ },
    { DSGW_ERR_DBCONF,
        DBT_anErrorOccurredWhileReadingTheDb_ },
    { DSGW_ERR_USERDB_PATH,
        DBT_nshomeUserdbPathWasNull_ },
    { DSGW_ERR_UPDATE_DBSWITCH,
        DBT_theDirectoryServiceConfiguration_ },
    { DSGW_ERR_ENTRY_NOT_FOUND,
        DBT_theEntryCouldNotBeReadFromTheDir_ },
    { DSGW_ERR_DB_ERASE,
        DBT_theLdapDatabaseCouldNotBeErased_ },
    { DSGW_ERR_LOCALDB_PERMISSION_DENIED,
        DBT_youMayNotChangeEntriesBesidesYou_ },
    { DSGW_ERR_LOCALDBGENERAL,
	DBT_anErrorOccurredWhileTryingToAcce_ },
    { DSGW_ERR_NOCALCONFIG,
        DBT_noneOfTheServersConfiguredForCAL },
    { DSGW_ERR_CLEANUPENTRY,
	DBT_allReferencesToEntryCouldNotBeCh_ },
    { DSGW_ERR_FAILEDTOBUILD_LDAPINFO,
	DBT_errorBuildingLdapInfo_ },
    { DSGW_ERR_ENV_AUTHLDAPURL_NOTSET,
	DBT_errorAuthLdapUrlNotSet_ },
    { DSGW_ERR_MISSING_DS_ENTRY_FOR_USERNAME,
	DBT_noEntryInDSforGiverUsername_ },
};
#define DSGW_ERROR_CNT	( sizeof( dsgw_errs ) / sizeof( struct dsgwerr ))


/*
 * dsgw_error -- report error as HTML text
 */
void
dsgw_error( int err, char *extra, int options, int ldaperr )
{
    char	*msg, *prelude = XP_GetClientStr(DBT_problem_);

    if (( options & DSGW_ERROPT_IGNORE ) != 0 ) {
	return;
    }

    if (( options & DSGW_ERROPT_INLINE ) == 0 ) {
	/*
	 * changed to fix bug # 88741.
	 * to display the error message for the frame based New U/G/O.
	 */
	fputs( "</SCRIPT></HTML>", stdout );
	dsgw_send_header();

	fputs( "\n"
	       "<HTML>\n<HEAD>\n", stdout );

	if ( prelude != NULL ) {
	    fprintf( stdout, "<TITLE>%s</TITLE>\n", prelude );
	}
	fputs( "<SCRIPT LANGUAGE=\"JavaScript\">\n"
	       "function errPrint()\n"
	       "{\n"
	       "    var d = top.frames[1].frames[1].document;\n"
	       "    d.open('text/html');\n"
	       "    d.writeln('<HTML>');\n"
	       "    d.writeln('<HEAD>');\n", stdout );
	if ( gc->gc_charset )
		fprintf( stdout,
	       "    d.writeln('<META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html;charset=%s\">');\n",
		gc->gc_charset );
	fputs( "    d.writeln('</HEAD>');\n"
	       "    d.writeln('<BODY BGCOLOR=\"#C0C0C0\">');\n", stdout );
        if ( prelude != NULL ) {
	    fputs( "    d.writeln('<CENTER>');\n"
		   "    d.writeln('<TABLE BORDER=\"2\" CELLPADDING=\"10\" WIDTH=100%%>');\n"
		   "    d.writeln('<TR>');\n"
		   "    d.writeln('<TD ALIGN=\"center\" WIDTH=\"100%%\">');\n"
		   "    d.writeln('<FONT SIZE=\"+2\">');\n", stdout );
	    fprintf( stdout, "    d.writeln('<B>%s</B></FONT>');\n", prelude );
	    fputs( "    d.writeln('</TD></TR></TABLE></CENTER><P>');\n", stdout );
        }
        msg = dsgw_err2string( err );
#ifdef DSGW_DEBUG
        if ( extra == NULL ) {
            dsgw_log( "%s: %s\n", prelude, msg );
        } else {
            dsgw_log( "%s: %s (%s)\n", prelude, msg, extra );
        }
#endif
	fputs( "    d.writeln('<FONT SIZE=\"+1\">');\n", stdout );
	fprintf( stdout, "    d.writeln('%s.');\n", msg );
	fputs( "    d.writeln('</FONT>');\n", stdout );

        if ( extra != NULL ) {
            fprintf( stdout, "    d.writeln('<BR>(%s)');\n", extra );
        }

        if ( ldaperr  != 0 ) {
	    char *p;
            msg = dsgw_ldaperr2longstring( ldaperr );
	    p = msg;
	    while (( p = strchr( p, '\n' )) != NULL ) {
		*p++ = ' ';
	    }
            fprintf( stdout, "    d.writeln('<P>%s');\n", msg );
        }
        fputs( "    d.writeln('</BODY></HTML>');\n"
	       "    d.close();\n"
	       "}\n"
	       "</SCRIPT>\n", stdout );
	fprintf( stdout, "</HEAD>\n<BODY %s onLoad=\"errPrint()\">\n",
			 dsgw_html_body_colors );
        dsgw_html_end();

    } else { /* options & DSGW_ERROPT_INLINE ) != 0 */
        msg = dsgw_err2string( err );
    
        fprintf( stdout, "<FONT SIZE=\"+1\">\n%s.\n</FONT>\n", msg );
        if ( extra != NULL ) {
            printf( "<BR>(%s)", extra );
        }
    
#ifdef DSGW_DEBUG
        if ( extra == NULL ) {
            dsgw_log( "%s: %s\n", prelude, msg );
         } else {
            dsgw_log( "%s: %s (%s)\n", prelude, msg, extra );
        }
#endif
        if ( ldaperr  != 0 ) {
            msg = dsgw_ldaperr2longstring( ldaperr );
            printf("<P>%s", msg );
        }
    }

    if (( options & DSGW_ERROPT_EXIT ) != 0 ) {
	exit( 0 );
    }
}


/*
 * special handling for cookie expired or cookie database problems
 *	delete cookie on both server and client
 *	send helpful error with appropriate buttons:
 * * if searching, display an error message, and a re-auth button, along
 *   with a help button.
 * * if authenticating,  (does this ever happen?)
 * * if generating an editable view, display an error messge, and tell
 *   user to bring main window to front and requthenticate.
 * * if submitting a modify operation, include an "Authenticate" button
 *   which brings up a new auth window, which only offers you a
 *   "close" button when finished.
 */
void
dsgw_dn2passwd_error( int ckrc )
{
    char	*authck;

    /*
     * cookie is expired or bad -- delete it on both server and client sides
     * 
     */
    if (( authck = dsgw_get_auth_cookie()) != NULL ) {
	dsgw_delcookie( authck );
    }

    dsgw_send_header();
    dsgw_html_begin( XP_GetClientStr(DBT_authenticationProblem_), 1 );

    fputs( "<SCRIPT LANGUAGE=\"JavaScript\">\n<!-- hide\n\n", stdout );
    fprintf( stdout, "document.cookie = '%s=%s; path=/';\n\n", DSGW_AUTHCKNAME,
	    DSGW_UNAUTHSTR );

    fputs( "function reAuth()\n{\n", stdout );
    fprintf( stdout, "    a = open( '%s', 'AuthWin');\n",
	    dsgw_getvp( DSGW_CGINUM_AUTH ));
    fputs( "    a.opener = self;\n", stdout );
    fputs( "    a.closewin = false;\n", stdout );
    fputs( "}\n// end hiding -->\n</SCRIPT>\n", stdout );

    fputs( dsgw_err2string( ckrc ), stdout );
    switch ( gc->gc_mode ) {
    case DSGW_MODE_DOSEARCH:
	/* Just display a helpful error message */
	fputs( XP_GetClientStr(DBT_NPYouMustReAuthenticateBeforeCon_),
		stdout );
	fputs( "<P>\n<FORM>\n<CENTER><TABLE border=2 width=\"100%\"><TR>\n",
		stdout );
	fputs( "<TD WIDTH=\"50%\" ALIGN=\"center\">\n", stdout );
	printf( "<INPUT TYPE=\"button\" VALUE=\"%s\" "
		"onClick=\"top.document.location.href='%sauth.html'\">\n", 
		XP_GetClientStr(DBT_reAuthenticate_), DSGW_URLPREFIX_MAIN );
	fputs( "<TD WIDTH=\"50%\" ALIGN=\"center\">\n", stdout );
	dsgw_emit_helpbutton( "AUTHEXPIRED" );
	fputs( "\n</TABLE></CENTER></FORM>\n", stdout );
	break;
    case DSGW_MODE_EDIT:
    case DSGW_MODE_DOMODIFY:
	fputs( XP_GetClientStr(DBT_NPYouMustReAuthenticateBeforeCon_1),
		stdout );
	fputs( "<P>\n<FORM>\n<CENTER><TABLE border=2 width=\"100%\"><TR>\n",
		stdout );
	fputs( "<TD WIDTH=\"50%\" ALIGN=\"center\">\n", stdout );
	fprintf( stdout, "<INPUT TYPE=\"button\" VALUE=\"%s\" "
		"onClick=\"top.close();\">\n",
                XP_GetClientStr(DBT_closeWindow_4) );
	fputs( "<TD WIDTH=\"50%\" ALIGN=\"center\">\n", stdout );
	dsgw_emit_helpbutton( "AUTHEXPIRED" );
	fputs( "\n</TABLE></CENTER></FORM>\n", stdout );
	break;
    }
    dsgw_html_end();
}


static char *
dsgw_err2string( int err )
{
    int		i;

    for ( i = 0; i < DSGW_ERROR_CNT; ++i ) {
        if ( dsgw_errs[ i ].dsgwerr_code == err ) {
            return( XP_GetClientStr(dsgw_errs[ i ].dsgwerr_msg) );
        }
    }

    return( XP_GetClientStr(DBT_unknownError_) );
}


static char *
dsgw_ldaperr2longstring( int err )
{
    char 	*s = "";

    switch ( err ) {
    case LDAP_SUCCESS:
	s = XP_GetClientStr(DBT_theOperationWasSuccessful_);
	break;
    case LDAP_OPERATIONS_ERROR:
	s = XP_GetClientStr(DBT_anInternalErrorOccurredInTheServ_);
	break;
    case LDAP_PROTOCOL_ERROR:
	s = XP_GetClientStr(DBT_theServerCouldNotUnderstandTheRe_);
	break;
    case LDAP_TIMELIMIT_EXCEEDED:
	s = XP_GetClientStr(DBT_aTimeLimitWasExceededInRespondin_);
	break;
    case LDAP_SIZELIMIT_EXCEEDED:
	s = XP_GetClientStr(DBT_aSizeLimitWasExceededInRespondin_);
	break;
    case LDAP_COMPARE_FALSE:
	break;
    case LDAP_COMPARE_TRUE:
	break;
    case LDAP_STRONG_AUTH_NOT_SUPPORTED:
	s = XP_GetClientStr(DBT_theGatewayAttemptedToAuthenticat_);
	break;
    case LDAP_STRONG_AUTH_REQUIRED:
	s = XP_GetClientStr(DBT_theGatewayAttemptedToAuthenticat_1);
	break;
#ifdef LDAP_REFERRAL				/* new in LDAPv3 */
    case LDAP_REFERRAL:
#endif
    case LDAP_PARTIAL_RESULTS:
	s = XP_GetClientStr(DBT_yourRequestCouldNotBeFulfilledPr_);
	break;
#ifdef LDAP_ADMIN_LIMIT_EXCEEDED		/* new in LDAPv3 */
    case LDAP_ADMIN_LIMIT_EXCEEDED:
	s = XP_GetClientStr(DBT_yourRequestExceededAnAdministrat_);
	break;
#endif
#ifdef LDAP_UNAVAILABLE_CRITICAL_EXTN		/* new in LDAPv3 */
    case LDAP_UNAVAILABLE_CRITICAL_EXTN:
	s = XP_GetClientStr(DBT_aCriticalExtensionThatTheGateway_);
	break;
#endif
    case LDAP_NO_SUCH_ATTRIBUTE:
	s = XP_GetClientStr(DBT_theServerWasUnableToProcessTheRe_);
	break;
    case LDAP_UNDEFINED_TYPE:
	break;
    case LDAP_INAPPROPRIATE_MATCHING:
	break;
    case LDAP_CONSTRAINT_VIOLATION:
	s = XP_GetClientStr(DBT_theServerWasUnableToFulfillYourR_);
	break;
    case LDAP_TYPE_OR_VALUE_EXISTS:
	s = XP_GetClientStr(DBT_theServerCouldNotAddAValueToTheE_);
	break;
    case LDAP_INVALID_SYNTAX:
	break;
    case LDAP_NO_SUCH_OBJECT:
	s = XP_GetClientStr(DBT_theServerCouldNotLocateTheEntryI_);
	break;
    case LDAP_ALIAS_PROBLEM:
	break;
    case LDAP_INVALID_DN_SYNTAX:
	s = XP_GetClientStr(DBT_aDistinguishedNameWasNotInThePro_);
	break;
    case LDAP_IS_LEAF:
	break;
    case LDAP_ALIAS_DEREF_PROBLEM:
	break;
    case LDAP_INAPPROPRIATE_AUTH:
	s = XP_GetClientStr(DBT_theEntryYouAttemptedToAuthentica_);
	break;
    case LDAP_INVALID_CREDENTIALS:
	s = XP_GetClientStr(DBT_thePasswordOrOtherAuthentication_);
	break;
    case LDAP_INSUFFICIENT_ACCESS:
	s = XP_GetClientStr(DBT_youDoNotHaveSufficientPrivileges_);
	break;
    case LDAP_BUSY:
	s = XP_GetClientStr(DBT_theServerIsTooBusyToServiceYourR_);
	break;
    case LDAP_UNAVAILABLE:
	s = XP_GetClientStr(DBT_theLdapServerCouldNotBeContacted_);
	break;
    case LDAP_UNWILLING_TO_PERFORM:
	s = XP_GetClientStr(DBT_theServerWasUnwilliingToProcessY_);
	break;
    case LDAP_LOOP_DETECT:
	break;
    case LDAP_NAMING_VIOLATION:
	break;
    case LDAP_OBJECT_CLASS_VIOLATION:
	s = XP_GetClientStr(DBT_theDirectoryServerCouldNotHonorY_);
	break;
    case LDAP_NOT_ALLOWED_ON_NONLEAF:
	s = XP_GetClientStr(DBT_theDirectoryServerWillNotAllowYo_);
	break;
    case LDAP_NOT_ALLOWED_ON_RDN:
	break;
    case LDAP_ALREADY_EXISTS:
	s = XP_GetClientStr(DBT_theServerWasUnableToAddANewEntry_);
	break;
    case LDAP_NO_OBJECT_CLASS_MODS:
	break;
    case LDAP_RESULTS_TOO_LARGE:
	break;
#ifdef LDAP_AFFECTS_MULTIPLE_DSAS		/* new in LDAPv3 */
    case LDAP_AFFECTS_MULTIPLE_DSAS:
	s = XP_GetClientStr(DBT_yourRequestWouldAffectSeveralDir_);
	break;
#endif
    case LDAP_OTHER:
	break;
    case LDAP_SERVER_DOWN:
#ifndef DSGW_NO_SSL
	if ( gc->gc_ldapssl ) {
	    s = XP_GetClientStr(DBT_theDirectoryServerCouldNotBeCont_2);
	} else {
	    s = XP_GetClientStr(DBT_theDirectoryServerCouldNotBeCont_);
	}
#else
	s = XP_GetClientStr(DBT_theDirectoryServerCouldNotBeCont_);
#endif
	break;
    case LDAP_LOCAL_ERROR:
	break;
    case LDAP_ENCODING_ERROR:
	s = XP_GetClientStr(DBT_anErrorOccuredWhileSendingDataTo_);
	break;
    case LDAP_DECODING_ERROR:
	s = XP_GetClientStr(DBT_anErrorOccuredWhileReadingDataFr_);
	break;
    case LDAP_TIMEOUT:
	s = XP_GetClientStr(DBT_theServerDidNotRespondToTheReque_);
	break;
    case LDAP_AUTH_UNKNOWN:
	s = XP_GetClientStr(DBT_theServerDoesNotSupportTheAuthen_);
	break;
    case LDAP_FILTER_ERROR:
	s = XP_GetClientStr(DBT_theSearchFilterConstructedByTheG_);
	break;
    case LDAP_USER_CANCELLED:
	s = XP_GetClientStr(DBT_theOperationWasCancelledAtYourRe_);
	break;
    case LDAP_PARAM_ERROR:
	break;
    case LDAP_NO_MEMORY:
	s = XP_GetClientStr(DBT_anInternalErrorOccurredInTheLibr_);
	break;
    case LDAP_CONNECT_ERROR:
#ifndef DSGW_NO_SSL
	if ( gc->gc_ldapssl ) {
	    s = XP_GetClientStr(DBT_aConnectionToTheServerCouldNotBe_2);
	} else {
	    s = XP_GetClientStr(DBT_aConnectionToTheServerCouldNotBe_);
	}
#else
	s = XP_GetClientStr(DBT_aConnectionToTheServerCouldNotBe_);
#endif
	break;
    default:
	s = XP_GetClientStr(DBT_anUnknownErrorWasEncountered_);
    }
    return s;
}




