/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/* 
 * Generate a screen.
 */

#include "dsgw.h"

static int dsgw_genscreen_begin( char *fname, FILE **fpp,
	char *stop_at_directive, int erropts );
static int dsgw_genscreen_continue( FILE **fpp, char *stop_at_directive,
	int erropts );

static LDAP *ld = NULL;

main( argc, argv, env )
    int		argc;
    char	*argv[];
#ifdef DSGW_DEBUG
    char	*env[];
#endif
{
    char	*p, *tmplname, *buf;

    (void)dsgw_init( argc, argv, DSGW_METHOD_GET );
    dsgw_send_header();

#ifdef DSGW_DEBUG
   dsgw_logstringarray( "env", env ); 
#endif

    /*
     * If the QUERY_STRING is non-NULL, it looks like this:
     *
     *    template [ &INFO=infostring ] [ &LANG=language ]
     *
     * where:
     *   "template" is the name of the HTML template to render
     *   "infostring" is a message used to replace DS_LAST_OP_INFO directives
     *   "language" is the current language to use for display, e.g., en-us
     *
     * If the QUERY_STRING is NULL, the name of this program is used as the
     * template.
     */

    if (( tmplname = getenv( "QUERY_STRING" )) == NULL ) {
	tmplname = progname;
    } else {
	tmplname = dsgw_ch_strdup( tmplname );
	while (( p = strrchr( tmplname, '&' )) != NULL ) {
	    *p++ = '\0';
	    if ( strncasecmp( p, "info=", 5 ) == 0 ) {
		dsgw_last_op_info = dsgw_ch_strdup( p + 5 );
		dsgw_form_unescape( dsgw_last_op_info );
	    } else if ( strncasecmp( p, "lang=", 5 ) == 0 ) {
                if ( gc->gc_international )
		    gc->gc_inputlanguage =
			dsgw_tag2lang( gc->gc_languages, p + 5 );
                else
                    gc->gc_inputlanguage = gc->gc_languages;
	    }
	}
    }
	

    buf = dsgw_ch_malloc( strlen( tmplname ) + 6 );	/* room for ".html\0" */
    sprintf( buf, "%s.html", tmplname );

    dsgw_genscreen_begin( buf, NULL, NULL, DSGW_ERROPT_EXIT );

    exit( 0 );
}


static int
dsgw_genscreen_begin( char *fname, FILE **fpp, char *stop_at_directive,
	int erropts )
{
    FILE	*html;

    if ( fpp == NULL ) {
	fpp = &html;
    }

    if (( *fpp = dsgw_open_html_file( fname, erropts )) == NULL ) {
	*fpp = NULL;
	return( -1 );
    }

    return( dsgw_genscreen_continue( fpp, stop_at_directive, erropts ));
}

#define DSGW_LOCAL_IF		"<!-- IF "
#define DSGW_LOCAL_IF_LEN	sizeof( DSGW_LOCAL_IF )

static int
dsgw_genscreen_continue( FILE **fpp, char *stop_at_directive, int erropts )
{
    char	**argv, line[ BIG_LINE ];
    int		argc;
 
    while ( dsgw_next_html_line( *fpp, line ))  {
	/* I18n: to use IF in the page created by genscreen */
	if ( !ld && dsgw_directive_is( line, "IF" )) {
	    char *p;
#ifdef DSGW_DEBUG
            dsgw_log( "IF is detected: %s", line );
#endif
            p = line + DSGW_LOCAL_IF_LEN;
            if ( !strncasecmp( DSGW_COND_LDAPVERSION, p,
				sizeof( DSGW_COND_LDAPVERSION ) - 1 )) {
                (void)dsgw_init_ldap( &ld, NULL, 0 );
            }
	} 
	if ( dsgw_parse_line( line, &argc, &argv, 0, dsgw_simple_cond_is_true,
		NULL )) {
	    if ( stop_at_directive != NULL &&
		    dsgw_directive_is( line, stop_at_directive )) {
		return( 0 );
	    }
	    if ( dsgw_directive_is( line, DRCT_DS_LOCATIONPOPUP )) {
		dsgw_emit_location_popup( &ld, argc, argv, erropts );
	    }
	    /* I18n: display language popup */
	    else if ( dsgw_directive_is( line, DRCT_DS_LANGUAGEPOPUP )) {
		dsgw_emit_language_popup( NULL, argc, argv );
	    }
	    /* I18n: display language map table */
	    else if ( dsgw_directive_is( line, DRCT_DS_LANGUAGETABLES )) {
		dsgw_emit_language_tables( argc, argv );
	    }	
	}
    }

    fclose( *fpp );
    *fpp = NULL;

    return( 0 );
}
