/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#include "dsgw.h"

void
dsgw_html_begin( char *title, int titleinbody )
{
    static int	header_done = 0;

    if ( !header_done ) {
	fputs( "<HTML>\n<HEAD>\n", stdout );
	if ( title != NULL ) {
	    fprintf( stdout, "<TITLE>%s</TITLE>\n", title );
	}
	fprintf( stdout, "</HEAD>\n<BODY %s>\n", dsgw_html_body_colors );
    } else {
	fputs( "\n<HR>\n", stdout );
    }

    if (( title != NULL ) && ( header_done || titleinbody )) {
	printf( "<CENTER><TABLE BORDER=\"2\" CELLPADDING=\"10\" WIDTH=100%%>\n"
		"<TR><TD ALIGN=\"center\" WIDTH=\"100%%\">\n<FONT SIZE=\"+2\">"
		"<B>%s</B></FONT></TD></TR></TABLE></CENTER>\n<P>\n", title );
    }

    header_done = 1;
}


void
dsgw_html_end()
{
    fputs( "</BODY></HTML>\n", stdout );

    /* make sure everything has been written to the server before we exit */
    fflush( stdout );
    fflush( stderr );
}


/*
 * output a hypertext reference/URL:
 * if "urlprefix" != NULL, it is prepended to "url" and "url" is hex-escaped.
 * if "urlprefix" == NULL, "url" is assumed to be already escaped as needed.
 *
 * if "value" != NULL, any occurrence of "--value--" in "url" is replaced by
 *    a URL-escaped version of the actual value.
 * if "value" == NULL, no substitution is done.
 *
 * if "label" == NULL or is of zero length, the closing ">LABEL</A>" is omitted.
 */
void
dsgw_html_href( char *urlprefix, char *url, char *label, char *value,
	char *extra )
{
    char	*escaped_url, *tag = "--value--";

#ifdef DSGW_DEBUG
    dsgw_log( "dsgw_html_href:\n\tprefix: %s\n\turl: %s\n\tlabel: %s\n\tvalue: %s\n\textra: %s\n",
		urlprefix?urlprefix:"(null)", url?url:"(null)", 
		label?label:"(null)", value?value:"(null)",
		extra?extra:"(null)" );
#endif

    if ( urlprefix == NULL ) {
	fputs( "<A HREF=", stdout );
	escaped_url = NULL;
    } else {
	fprintf( stdout, "<A HREF=%s?&", urlprefix );
	escaped_url = dsgw_ch_malloc( 3 * strlen( url ) + 1 );
	*escaped_url = '\0';
	dsgw_strcat_escaped( escaped_url, url );
	url = escaped_url;
    }

    if ( value != NULL ) {
	char *tmpurl = dsgw_ch_malloc( strlen( url ) + 4 );
	sprintf( tmpurl, "dn=%s", url );
	dsgw_substitute_and_output( tmpurl, tag, value, 1 );
	free( tmpurl );
    } else {
	fprintf( stdout, "dn=%s", url );
    }

    if ( extra != NULL ) {
	putc( ' ', stdout );
	if ( value != NULL ) {
	    dsgw_substitute_and_output( extra, tag, value, 1 );
	} else {
	    fputs( extra, stdout );
	}
    }

    if ( label != NULL && *label != '\0' ) {
	fprintf( stdout, ">%s</A>\n", label );
    }

    if ( escaped_url != NULL ) {
	free( escaped_url );
    }
}


void
dsgw_substitute_and_output( char *s, char *tag, char *value, int escape )
{
    char	*p, *escval;

    escval = NULL;

    while ( ( p = strstr( s, tag )) != NULL ) {
	if ( p > s ) {
	    fwrite( s, p - s, 1, stdout );
	}
	if ( escape ) {
	    if ( escval == NULL ) {
		escval = dsgw_strdup_escaped( value );
	    }
	    fputs( escval, stdout );
	} else {
	    fputs( value, stdout );
	}
	
	s = p + strlen( tag );
    }

    if ( escval != NULL ) {
	free( escval );
    }

    if ( *s != '\0' ) {
	fputs( s, stdout );
    }
}


char *
dsgw_strdup_escaped( char *s )
{
    char	*p;

    p = dsgw_ch_malloc( 3 * strlen( s ) + 1 );
    *p = '\0';
    dsgw_strcat_escaped( p, s );
    return( p );
}


/* this macro was copied from libldap/tmplout.c */

#define HREF_CHAR_ACCEPTABLE( c )	(( c >= '-' && c <= '9' ) ||	\
					 ( c >= '@' && c <= 'Z' ) ||	\
					 ( c == '_' ) ||		\
					 ( c >= 'a' && c <= 'z' ))

/* this function is copied from libldap/tmplout.c:strcat_escaped */
void
dsgw_strcat_escaped( char *s1, char *s2 )
{
    unsigned char	*q;
    char		*p, *hexdig = "0123456789ABCDEF";

    p = s1 + strlen( s1 );
    for ( q = (unsigned char *)s2; *q != '\0'; ++q ) {
	if ( HREF_CHAR_ACCEPTABLE( *q )) {
	    *p++ = *q;
	} else {
	    *p++ = '%';
	    *p++ = hexdig[ *q >> 4 ];
	    *p++ = hexdig[ *q & 0x0F ];
	}
    }

    *p = '\0';
}
