/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * htmlparse.c -- routines to parse HTML templates -- HTTP gateway.
 * Stolen from libadmin/template.c and libadmin/form_get.c, originally
 *	by Mike McCool.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include "dsgw.h"
#include "dbtdsgw.h"

extern char *Versionstr;	/* from Versiongw.c */

/* global variables */
char *dsgw_last_op_info;	/* set in edit.c and genscreen.c */
char *dsgw_dnattr;		/* set in edit.c */
char *dsgw_dndesc;		/* set in edit.c */

/*
 * Save yourself a lot of grief and put a space after the name.
 */

static struct template_s templates[] = {
  {"IF ", "FUNC conditional"},
  {"ELSE ", "FUNC conditional"},
  {"ELIF ", "FUNC conditional"},
  {"ENDIF ", "FUNC conditional"},
  {"TITLE ", "FUNC title"},
  {"BODY ", "FUNC body"},
  {"COLORS ", "FUNC colors"},
  {"PAGEHEADER ", "FUNC pageheader"},
  {"BEGININFO ", "<table border=2 width=100%% cellpadding=2>\n"
                 "<tr><td align=center colspan=2>"
                 "<b><FONT size=+1>%s</FONT></b></td></tr>"
                 "<td colspan=2>\n"},
  {"ADDINFO ", "</td></tr><tr><td colspan=2>"},
  {"ENDINFO ", "</td></tr></table>\n<hr width=10%%>\n"},
  {"SUBMIT ", "FUNC submit\n"},
  {"BEGINELEM ", "<pre>"},
  {"ELEM ", "\n<b>%s</b>"},
  {"ENDELEM ", "</pre>\n"},
  {"ELEMADD ", "<b>%s</b>"},
  {"ELEMDIV ", "\n"},
  {"INDEX ", "<a href=\"index\">%s</a>\n"},
  {"HELPBUTTON", "FUNC helpbutton"},
  {"DIALOGSUBMIT", "FUNC dialogsubmit"},
  {DRCT_DS_LAST_OP_INFO, "FUNC emit_last_op_info"},
  {DRCT_DS_GATEWAY_VERSION, "FUNC emit_version_str"},
  {DRCT_DS_LANGUAGE_TAG, "FUNC emit_language_tag"},
  {DRCT_DS_LANGUAGE_TYPE, "FUNC emit_language_type"},
  {DRCT_DS_LANGUAGE_NAME, "FUNC emit_language_name"},
  {"ENDHTML", "</BODY></HTML>"},
  { NULL, NULL }
};

/* global to track output status */
#define DSGW_PARSE_STATUS_NO_IF_SEEN	-1
#define DSGW_PARSE_STATUS_NO_OUTPUT	0
#define DSGW_PARSE_STATUS_OUTPUT	1
static int parse_status = DSGW_PARSE_STATUS_NO_IF_SEEN;

static int dsgw_get_directive(char *string);
static char **dsgw_get_vars(char *string, int *argc);
static void dsgw_output(char *line);
static void dsgw_pageheader(int argc, char **argv);
static void dsgw_title(int argc, char **argv);
static void dsgw_body(int argc, char **argv);
static void dsgw_colors(int argc, char **argv);
static void dsgw_submit(int verify, char **vars);
static void dsgw_dialogsubmit(void);
static void dsgw_conditional(char *name, int argc, char **argv,
	condfunc conditionalfn, void *condarg);
static int dsgw_condition_true( int argc, char **argv,
	condfunc conditionalfn, void *condarg );
static void emit_with_prefix_and_suffix( char *text, int argc, char **argv );
static void emit_last_op_info(int argc, char **argv);
static void emit_version_str( void );
static void emit_language_tag( int argc, char **argv );
static void emit_language_type( int argc, char **argv );
static void emit_language_name( int argc, char **argv );
static void template_error( char *msg );

/* Filter a page.  Takes the page to filter as an argument.  Uses above 
 * filters to process.  If we encounter a directive we don't know about,
 * we set argc and argv, and return -1.  The caller is responsible for
 * figuring out what to do with the directive and arg vector.
 *
 * If parseonly is non-zero, this routine will just parse lines that contain
 * directives -- nothing will be written to stdout.
 */
int
dsgw_parse_line(
char *line_input,
int  *argc,
char ***argv,
int parseonly,
condfunc conditionalfn,
void *condarg
)
{
     register int index;
     char *position;
     int dirlen = strlen(DIRECTIVE_START);
     char **vars;
     char c;

     *argc = 0;
     *argv = NULL;
     if ( !strncmp( line_input, DIRECTIVE_START, dirlen )) {
         position = (char *) ( line_input + dirlen );
	 if ( parseonly ) {
	     index = -1;	/* treat all directives as "unknown" */
	 } else {
	     index = dsgw_get_directive( position );
	 }

         /* did we get one? */
         if ( index != -1 )  {
             /* if so, get the vars. */
             position += strlen( templates[index].name );
             vars = dsgw_get_vars( position, argc );
             /* Dispatch the correct function (done for readability) */
             if ( !strncmp(templates[ index ].format, "FUNC ", 5 ))  {
                 if ( !strncmp( templates[ index ].format+5, "pageheader",10 ))
                     dsgw_pageheader( *argc, vars );
		 else if ( !strncmp( templates[index].format+5,"title",5))
		     dsgw_title( *argc, vars );
		 else if ( !strncmp( templates[index].format+5,"body",4))
		     dsgw_body( *argc, vars );
		 else if ( !strncmp( templates[index].format+5,"colors",6))
		     dsgw_colors( *argc, vars );
                 else if ( !strncmp( templates[ index ].format+5, "submit",6 ))
                     dsgw_submit( 0, vars );
                 else if ( !strncmp( templates[ index ].format+5, "verify",6 ))
                     dsgw_submit( 1, vars );
                 else if ( !strncmp( templates[index].format+5,
			"dialogsubmit",12 ))
                     dsgw_dialogsubmit();
		 else if ( !strncmp( templates[index].format+5, "helpbutton",
			10 ) && ( *argc > 0 ))
		     dsgw_emit_helpbutton( vars[ 0 ] );
                 else if ( !strncmp( templates[index].format+5, "conditional", 
                         11 ))
                     dsgw_conditional( templates[index].name, *argc, vars,
				conditionalfn, condarg );
                 else if ( !strncmp( templates[index].format+5,
			"emit_last_op_info", 17 ))
		     emit_last_op_info( *argc, vars );
                 else if ( !strncmp( templates[index].format+5,
			"emit_version_str", 16 ))
		     emit_version_str();
                 else if ( !strncmp( templates[index].format+5,
			"emit_language_tag", 17 ))
		     emit_language_tag( *argc, vars );
                 else if ( !strncmp( templates[index].format+5,
			"emit_language_type", 18 ))
		     emit_language_type( *argc, vars );
                 else if ( !strncmp( templates[index].format+5,
			"emit_language_name", 18 ))
		     emit_language_name( *argc, vars );
                 /* We don't know what this template is.  Send it back. */
                 else {
		    *argv = vars;
		    return -1;
		}
             } else if ( parse_status != DSGW_PARSE_STATUS_NO_OUTPUT ) { 
                 /* I just can't believe there's no easy way to create 
                  * a va_list. */
                 char line[ BIG_LINE ];
                 sprintf( line, templates[ index ].format, 
                         ( *argc > 0 && vars[ 0 ] != NULL ) ? vars[ 0 ]: "",
                         ( *argc > 1 && vars[ 1 ] != NULL ) ? vars[ 1 ]: "",
                         ( *argc > 2 && vars[ 2 ] != NULL ) ? vars[ 2 ]: "",
                         ( *argc > 3 && vars[ 3 ] != NULL ) ? vars[ 3 ]: "");
                 dsgw_output( line );
             }
         } else if ( parse_status != DSGW_PARSE_STATUS_NO_OUTPUT ) { 
	    /* We found a directive, but we can't identify it. Return non-zero
	     * value so caller knows to deal with it.
	     */
	     vars = dsgw_get_vars( position, argc );
	     *argv = vars;
	     return -1;
         }
     } else if ( !parseonly && parse_status != DSGW_PARSE_STATUS_NO_OUTPUT ) {
         /* We found no directive.  The line is normal. */
         dsgw_output( line_input );
    }

    /* If we're here, we either handled it correctly or the line was benign.*/
    return 0;
}


FILE *
dsgw_open_html_file(char *filename, int erropts)
{
    FILE *f;
    char *tfname = NULL;

    tfname = dsgw_file2path( gc->gc_tmpldir, filename);
    if (!(f = fopen(tfname, "r"))) {
	/* punt */
	dsgw_error(DSGW_ERR_OPENHTMLFILE, tfname, erropts, 0 );
    }

    free( tfname );

    return f;
}


FILE *
dsgw_open_config_file(char *filename, int erropts)
{
    FILE *f;
    char *tfname = NULL;

    tfname = dsgw_file2path( gc->gc_configdir, filename);
    if (!(f = fopen(tfname, "r"))) {
	/* punt */
	dsgw_error(DSGW_ERR_BADCONFIG, tfname, erropts, 0 );
    }

    free( tfname );

    return f;
}


#define DSGW_INCLUDE_DRCT	"<!-- INCLUDE "
#define DSGW_INCLUDE_DRCT_LEN	13
#define DSGW_INCLSET_DRCT	"<!-- INCLUDESET "
#define DSGW_INCLSET_DRCT_LEN	16
#define DSGW_INCLCFG_DRCT	"<!-- INCLUDECONFIG "
#define DSGW_INCLCFG_DRCT_LEN	19

int
dsgw_next_html_line(FILE *f, char *line)
{
    char		*p, *incfile;
    int			linelen;
    static FILE		*incfp = NULL;
    static FILE		*parentfp = NULL;
    static int		incset_index = 0;
    static dsgwinclset	*incsetp = NULL;

    if ( incfp != NULL && parentfp == f ) {
	/* we're in the midst of an include -- read from include file */
	if ( fgets(line, BIG_LINE, incfp ) != 0 ) {
	    return 1;	/* success */
	}

	/* end of include file */
	fclose( incfp );

	/* if in middle of an include set, open and use next file in set */
	if ( incsetp != NULL && ++incset_index < incsetp->dsiset_itemcount ) {
	    incfp = dsgw_open_html_file(
		    incsetp->dsiset_filenames[ incset_index ],
		    DSGW_ERROPT_EXIT );
	    return( dsgw_next_html_line( f, line ));
	}
	incfp = NULL;
	incsetp = NULL;
    }

    if(!(fgets(line, BIG_LINE, f))) {
	return 0;	/* end of file */
    }

    if ( incfp != NULL ) {
	return 1;	/* ignore nested includes */
    }

    /* check for start of a simple or an include set based include */
    linelen = strlen( line );
    if ( linelen > DSGW_INCLUDE_DRCT_LEN && strncasecmp( line,
	    DSGW_INCLUDE_DRCT, DSGW_INCLUDE_DRCT_LEN ) == 0 ) {
	/*
	 * include a regular HTML file
	 */
	incfile = line + DSGW_INCLUDE_DRCT_LEN;
	if (( p = strchr( incfile, ' ' )) != NULL ) {
	    *p = '\0';
	}
	incfp = dsgw_open_html_file( incfile, DSGW_ERROPT_EXIT );

    } else if ( linelen > DSGW_INCLSET_DRCT_LEN && strncasecmp( line,
	    DSGW_INCLSET_DRCT, DSGW_INCLSET_DRCT_LEN ) == 0 ) {
	/*
	 * include a whole set of HTML files
	 */
	char		*sethandle;

	sethandle = line + DSGW_INCLSET_DRCT_LEN;
	if (( p = strchr( sethandle, ' ' )) != NULL ) {
	    *p = '\0';
	}

	for ( incsetp = gc->gc_includesets; incsetp != NULL;
		incsetp = incsetp->dsiset_next ) {
	    if ( strcasecmp( sethandle, incsetp->dsiset_handle ) == 0 ) {
		break;
	    }
	}
	if ( incsetp == NULL ) {	/* set not found -- ignore it */
	    if ( p != NULL ) {
		*p = ' ';
	    }
	    return( 1 );
	}
	incset_index = 0;
	incfp = dsgw_open_html_file( incsetp->dsiset_filenames[ 0 ],
		DSGW_ERROPT_EXIT );

    } else if ( linelen > DSGW_INCLCFG_DRCT_LEN && strncasecmp( line,
	    DSGW_INCLCFG_DRCT, DSGW_INCLCFG_DRCT_LEN ) == 0 ) {
	/*
	 * include an HTML file from the config directory
	 */
	incfile = line + DSGW_INCLCFG_DRCT_LEN;
	if (( p = strchr( incfile, ' ' )) != NULL ) {
	    *p = '\0';
	}
	incfp = dsgw_open_config_file( incfile, DSGW_ERROPT_EXIT );
    }

    if ( incfp != NULL ) {	/* found an include */
	parentfp = f;
	return( dsgw_next_html_line( f, line ));
    }

    return 1;
}


static void
dsgw_pageheader(int argc, char **argv)
{
    char line[BIG_LINE];

    dsgw_output("<center><table border=2 width=100%%>\n");

    util_snprintf(line, BIG_LINE, "<tr>");
    dsgw_output(line);

    util_snprintf(line, BIG_LINE, "<td align=center width=100%%>");
    dsgw_output(line);
    util_snprintf(line, BIG_LINE, "<hr size=0 width=0>");
    dsgw_output(line);
    util_snprintf(line, BIG_LINE, "<FONT size=+2><b>%s</b></FONT>"
                                  "<hr size=0 width=0>"
                                  "</th>", ( argc > 0 ) ? argv[0] : "" );
    dsgw_output(line);
    
    dsgw_output("</tr></table></center>\n");
}


static void
dsgw_title( int argc, char **argv)
{
    char line[BIG_LINE];

    util_snprintf(line, BIG_LINE, "<HEAD><TITLE>%s</TITLE></HEAD>\n<HTML>\n"
	    "<BODY %s>\n", ( argc > 0 ) ? argv[0] : "", dsgw_html_body_colors );
    dsgw_output(line);
}


static void
dsgw_body( int argc, char **argv)
{
    char line[BIG_LINE];

    if ( argc > 0 ) {
	util_snprintf(line, BIG_LINE, "<BODY %s %s>\n", dsgw_html_body_colors,
		( argc > 0 ) ? argv[0] : "" );
    } else {
	util_snprintf(line, BIG_LINE, "<BODY %s>\n", dsgw_html_body_colors );
    }

    dsgw_output(line);
}


static void
dsgw_colors( int argc, char **argv)
{
    if ( argc > 0 ) {
	dsgw_html_body_colors = dsgw_ch_strdup( argv[0] );
    } else {
	dsgw_html_body_colors = "";
    }
}


static void
dsgw_submit(int verify, char **vars)
{
    char line[BIG_LINE];

    if(verify)  {
        util_snprintf(line, BIG_LINE, "<SCRIPT language=JavaScript>\n"
                      "function verify(form)  {\n"
                      "    if(confirm('%s%s%s'))\n"
                      "        form.submit();\n"
                      "}\n"
                      "</SCRIPT>\n",
                      XP_GetClientStr(DBT_doYouReallyWantTo_),
                      vars[0],
                      XP_GetClientStr(DBT_QQ_) );
        dsgw_output(line);
    }

    dsgw_output("<center><table border=2 width=100%%><tr>");

    if(!verify)  {
        char outstr[256];
        sprintf(outstr,"<td width=50%% align=center>"
               "<input type=submit value=\"%s\">"
               "</td>\n",
               XP_GetClientStr(DBT_ok_1));
        dsgw_output(outstr);
    }  else  {
        char outstr[256];
        sprintf(outstr,"<td width=50%% align=center>"
               "<input type=button value=\"%s\" "
               "onclick=\"verify(this.form)\">"
               "</td>\n",
               XP_GetClientStr(DBT_ok_2));
        dsgw_output(outstr);
    }
    {
        char outstr[256];
        sprintf(outstr,"<td width=50%% align=center>"
               "<input type=reset value=\"%s\"></td>\n",
               XP_GetClientStr(DBT_reset_));
        dsgw_output(outstr);
    }       

    dsgw_output("</tr></table></center>\n");

    dsgw_output("</form>\n");

    dsgw_output("<SCRIPT language=JavaScript>\n");
    dsgw_output("</SCRIPT>\n");
}


static void
dsgw_dialogsubmit(void)
{
    char outstr[256];

    dsgw_output("<center><table border=2 width=100%%><tr>");

    sprintf(outstr,"<td width=50%% align=center>"
           "<input type=submit value=\"%s\">"
           "</td>\n",
           XP_GetClientStr(DBT_done_));
    dsgw_output(outstr);
    sprintf(outstr,"<td width=50%% align=center>"
           "<input type=button value=\"%s\" "
           "onClick=\"top.close()\"></td>\n",
           XP_GetClientStr(DBT_cancel_2));
    dsgw_output(outstr);

    dsgw_output("</tr></table></center>\n");

    dsgw_output("</form>\n");

    dsgw_output("<SCRIPT language=JavaScript>\n");
    dsgw_output("</SCRIPT>\n");
}


static void
dsgw_conditional( char *name, int argc, char **argv, condfunc conditionalfn,
	void *condarg )
{
#define DSGW_COND_STATUS_NO_COND_SEEN	0
#define DSGW_COND_STATUS_IN_IF		1
#define DSGW_COND_STATUS_IN_ELSE	2
#define DSGW_COND_STATUS_IN_ELIF	3

    static int	cond_status = DSGW_COND_STATUS_NO_COND_SEEN;
    static int	cond_was_true = 0;

    if ( strncmp( name, "IF", 2 ) == 0 ) {
	if ( cond_status != DSGW_COND_STATUS_NO_COND_SEEN ) {
	    template_error( XP_GetClientStr(DBT_foundAnotherIfNestedIfsAreNotSup_) );
	    return;
	}
	cond_was_true = dsgw_condition_true( argc, argv, conditionalfn,
		condarg );
	parse_status = cond_was_true ? DSGW_PARSE_STATUS_OUTPUT
		: DSGW_PARSE_STATUS_NO_OUTPUT;
	cond_status = DSGW_COND_STATUS_IN_IF;

    } else if ( strncmp( name, "ELSE", 4 ) == 0 ) {
	if ( cond_status == DSGW_COND_STATUS_NO_COND_SEEN ) {
	    template_error( XP_GetClientStr(DBT_foundElseButDidnTSeeAnIf_) );
	    return;
	}
	if ( cond_status == DSGW_COND_STATUS_IN_ELSE ) {
	    template_error( XP_GetClientStr(DBT_foundElseAfterElseExpectingEndif_) );
	    return;
	}
	parse_status = cond_was_true ? DSGW_PARSE_STATUS_NO_OUTPUT
		: DSGW_PARSE_STATUS_OUTPUT;
	cond_status = DSGW_COND_STATUS_IN_ELSE;

    } else if ( strncmp( name, "ELIF", 4 ) == 0 ) {
	if ( cond_status == DSGW_COND_STATUS_NO_COND_SEEN ) {
	    template_error( XP_GetClientStr(DBT_foundElifButDidnTSeeAnIf_) );
	    return;
	}
	if ( cond_status == DSGW_COND_STATUS_IN_ELSE ) {
	    template_error( XP_GetClientStr(DBT_foundElifAfterElseExpectingEndif_) );
	    return;
	}

	if ( cond_was_true ) {
	    parse_status = DSGW_PARSE_STATUS_NO_OUTPUT;
	} else {
	    cond_was_true = dsgw_condition_true( argc, argv, conditionalfn,
		    condarg );
	    parse_status = cond_was_true ? DSGW_PARSE_STATUS_OUTPUT
		    : DSGW_PARSE_STATUS_NO_OUTPUT;
	}
	cond_status = DSGW_COND_STATUS_IN_ELIF;

    } else if ( strncmp( name, "ENDIF", 5 ) == 0 ) {
	if ( cond_status == DSGW_COND_STATUS_NO_COND_SEEN ) {
	    template_error( XP_GetClientStr(DBT_foundEndifButDidnTSeeAnIf_) );
	}
	parse_status = DSGW_PARSE_STATUS_NO_IF_SEEN;
	cond_status = DSGW_COND_STATUS_NO_COND_SEEN;
    }
}



static void
emit_with_prefix_and_suffix( char *text, int argc, char **argv )
{
    char	*s;

    if (( s = get_arg_by_name( "prefix", argc, argv )) != NULL ) {
	dsgw_output( s );
    }

    dsgw_output( text );

    if (( s = get_arg_by_name( "suffix", argc, argv )) != NULL ) {
	dsgw_output( s );
    }
}


static void
emit_last_op_info( int argc, char **argv )
{
    if ( dsgw_last_op_info != NULL ) {
	emit_with_prefix_and_suffix( dsgw_last_op_info, argc, argv );
    }
}


static void
emit_version_str()
{
    fputs( Versionstr, stdout );
}


static void
emit_language_tag( int argc, char **argv )
{
    emit_with_prefix_and_suffix( ( gc == NULL || gc->gc_inputlanguage == NULL )
	    ? "" : gc->gc_inputlanguage->dslang_tag, argc, argv );
}


static void
emit_language_type( int argc, char **argv )
{
    char	buf[ 32 ];
    int		type;

    if ( gc == NULL || gc->gc_inputlanguage == NULL ) {
	type = 0;
    } else {
	type =  gc->gc_inputlanguage->dslang_type;
    }
    sprintf( buf, "%d", type );
    emit_with_prefix_and_suffix( buf, argc, argv );
}


static void
emit_language_name( int argc, char **argv )
{
    emit_with_prefix_and_suffix( ( gc == NULL || gc->gc_inputlanguage == NULL )
	    ? "" : gc->gc_inputlanguage->dslang_name, argc, argv );
}


static void
template_error( char *msg )
{
    printf( XP_GetClientStr(DBT_BrBTemplateErrorBSBrN_), msg );
}


static int
dsgw_condition_true( int argc, char **argv, condfunc conditionalfn,
	void *condarg )
{
    char	*save_argv0;
    int		rc;

    if ( argc < 1 || conditionalfn == NULL ) {
	return( 1 );	/* unknown, but we default to true */
    }

    if ( argv[0][0] == '!' ) {	/* NOT */
	save_argv0 = argv[0];
	argv[0] = save_argv0 + 1;
    } else {
	save_argv0 = NULL;
    }

    rc = (*conditionalfn)( argc, argv, condarg );

    if ( save_argv0 != NULL ) {
	argv[0] = save_argv0;
	rc = !rc;	/* '!' was seen -- reverse the result */
    }

    return( rc );
}

static int
dsgw_get_directive(
char *string
)  
{
    int index = -1;
    register int x;

    for ( x = 0; templates[ x ].name != NULL; x++ )  {
        if ( !strncmp( string, templates[ x ].name,
                    strlen( templates[ x ].name ))) {
            index = x;
            break;
        }
    }
    return index;
}

int
dsgw_directive_is(char *target, char *directive)
{
    char *position = (target + strlen(DIRECTIVE_START));
    return(!(strncmp(directive, position, strlen(directive))));
}

static char **
dsgw_get_vars(
char *string,
int *argc
)
{
    char **vars = (char **) NULL;
    register int x;
    int isvar;
    char scratch[BIG_LINE];
    char lastchar, *p;
    int numvars = 0;

    isvar = -1;
    x = 0;
    scratch[0] = '\0';
    lastchar = ' ';
 
    while ( *string != '\0' ) {
        if (( *string == '\"' ) && ( lastchar != '\\' )) {
            if ( isvar != -1 )  {
		numvars++;
		vars = (char **)dsgw_ch_realloc( vars,
			( numvars + 1 ) * sizeof ( char * ));
                vars[ numvars - 1 ] = (char *) dsgw_ch_strdup( scratch );
		if (( p = strchr( vars[ numvars - 1 ], '=' )) != NULL ) {
		    dsgw_form_unescape( p + 1 );
		}
		vars[ numvars ] = NULL;
                isvar = -1;
            }  else {
                isvar = 0;
	    }
        } else {
            if ( isvar != -1 )  {
                scratch[ isvar++ ] = *string; 
                scratch[ isvar ] = '\0';
            } else {
                if ( *string == DIRECTIVE_END ) {
                    break;
		}
	    }
	}
        lastchar = *string;
        string++;
    }
    *argc = numvars;
    return vars;
}




static void
dsgw_output(char *line)  
{
    fputs(line, stdout);
}


/*
 * Search the given arg vector for a "tag=value" string where "tag" is
 * the same string as "name".  If found, return a pointer to the beginning
 * of the "value" string.  If the value string is missing (e.g. "tag="
 * was given), return a zero-length string.  If no matching tag was found,
 * return NULL.
 */
char *
get_arg_by_name( char *name, int argc, char **argv )
{
    int		i;

    if (( i = dsgw_get_arg_pos_by_name( name, argc, argv )) >= 0 ) {
	return( &argv[ i ][ strlen( name ) + 1 ] );
    } else {
	return( NULL );
    }
}


int
dsgw_get_arg_pos_by_name( char *name, int argc, char **argv )
{
    int i;
    int nl = strlen( name );

    for ( i = 0; i < argc; i++ ) {
	if ( argv[ i ] != NULL ) {
	    if ( !strncasecmp( name, argv[ i ], nl )) {
		if (( argv[ i ][ nl ] == '=' )) {
		    return( i );
		}
	    }
	}
    }
    return( -1 );
}


void
dsgw_argv_free( char **argv )
{
    char	**p;

    if ( argv != NULL ) {
	for ( p = argv; *p != NULL; ++p ) {
	    free( *p );
	}
	free( argv );
    } 
}


savedlines *
dsgw_savelines_alloc()
{
    savedlines	*slp;

    slp = dsgw_ch_malloc( sizeof( savedlines ));
    memset( slp, 0, sizeof( savedlines ));
    return( slp );
}


void
dsgw_savelines_free( savedlines *svlp )
{
    int		i;

    for ( i = 0; i < svlp->svl_count; ++i ) {
	free( svlp->svl_line[ i ] );
    }
    free( svlp );
}


void
dsgw_savelines_rewind( savedlines *svlp )
{
    svlp->svl_current = 0;
}


void
dsgw_savelines_save( savedlines *svlp, char *line )
{
    svlp->svl_line = (char **)dsgw_ch_realloc( svlp->svl_line,
	    (1 + svlp->svl_count ) * sizeof( char * ));
    svlp->svl_line[ svlp->svl_count++ ] = dsgw_ch_strdup( line );
}


char *
dsgw_savelines_next( savedlines *svlp )
{
    char	*p;

    if ( svlp->svl_current >= svlp->svl_count ) {
	return( NULL );
    }

    p = svlp->svl_line[ svlp->svl_current ];
    ++svlp->svl_current;

    return( p );
}
