/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#include "dsgw.h"
#include "dbtdsgw.h"
#include "../../include/disptmpl.h"
#include "libadminutil/distadm.h"
/* removed for LDAPSDK31 integration... no more lcache
#include <lcache.h>


static int bind_using_v2_or_v3( LDAP *ld, char *dn, char *passwd );
static int highest_ldap_version_supported( LDAP *ld, int defaultversion );
static dsgwtmplinfo *init_listdisplay( char *tmplname, unsigned long options );
static int do_search( dsgwtmplinfo *tip, LDAP *ld, char *base, int scope,
	char *filter, LDAPMessage **msgpp );
/*
static void handle_search_results( dsgwtmplinfo *tip, LDAP *ld, int rc,
	LDAPMessage *msgp, unsigned long options );
static int LDAP_CALL LDAP_CALLBACK 
get_rebind_credentials( LDAP *ld, char **whop, char **credp,
	int *methodp, int freeit, void *arg ) ;
static int LDAP_CALL LDAP_CALLBACK 
get_rebind_credentials( LDAP *, char **, char **,
	int , int, void * ) ;
*/
static void strcpy_special_undo( char *d, char *s );
static int entry2htmlwrite( void *fp, char *buf, int len );
static void emit_one_loc_dn( char *dn, char *friendlyname, char *rootname,
	char **base_rdns, int base_rdn_count, int only_one );
static char *uid2dn( LDAP *ld, char *uid, char *base, int *ldaprc,
	char **errsp );
#ifdef FORTEZZA
static void enable_fortezza_ciphers( void );
#endif /* FORTEZZA */

/* binddn and bindpasswd are used in get_rebind_credentials() */
static char *binddn = NULL, *bindpasswd = NULL;

#ifndef DSGW_NO_SSL
static CERTCertDBHandle certdbh;
#endif


/*
 * this function is called back by LIBLDAP when chasing referrals
 */
static int LDAP_CALL LDAP_CALLBACK
get_rebind_credentials( LDAP *ld, char **whop, char **credp,
	int *methodp, int freeit, void *arg )
{
    if ( !freeit ) {
	*whop = binddn;
	*credp = bindpasswd;
	*methodp = LDAP_AUTH_SIMPLE;
    }

    return( LDAP_SUCCESS );
}

static void
handle_search_results( dsgwtmplinfo *tip, LDAP *ld, int rc, LDAPMessage *msgp,
	unsigned long options )
{
    int		count;
    LDAPMessage	*entry;
    char	*dn, *errortext, **ocvals;

    count = ( msgp == NULL ) ? 0 : ldap_count_entries( ld, msgp );
    errortext = ( rc == LDAP_SUCCESS ) ? "" : dsgw_ldaperr2string( rc );
    dsgw_set_searchinfo( tip, count, NULL, errortext );

    if ( count > 0 ) {
	entry = ldap_first_entry( ld, msgp );

	if ( count == 1 && ( options & DSGW_DISPLAY_OPT_LIST_IF_ONE ) == 0 ) {
	    /* found exactly one entry:  read and display it */
	    dn = ldap_get_dn( ld, entry );
	    ocvals = ldap_get_values( ld, entry, DSGW_ATTRTYPE_OBJECTCLASS );
	    ldap_msgfree( msgp );

	    /*
	     * I18n: When only one result is found, turn off the lang selection
	     *       to display the result in the user's preferred language.
	     */
	    gc->gc_inputlanguage = NULL;
	    dsgw_read_entry( ld, dn, ocvals, NULL, options );

	    if ( ocvals != NULL ) {
		ldap_value_free( ocvals );
	    }
	    return;
	}

	/* list entries */
	ldap_multisort_entries( ld, &msgp, tip->dsti_sortbyattrs[0] == NULL ?
		NULL : tip->dsti_sortbyattrs, strcasecmp );
	for ( entry = ldap_first_entry( ld, msgp ); entry != NULL;
		entry = ldap_next_entry( ld, entry )) {
	    dsgw_display_entry( tip, ld, entry, NULL );
	}
	if ( options & DSGW_DISPLAY_OPT_DNLIST_JS ) {
	    int i;
	    char *edn, *js0, *js1;
	    char **xdn;
	    char **sn;

	    printf( "<SCRIPT LANGUAGE=\"JavaScript\">\n" );
	    printf( "var dnlist = new Array;\n" );
	    for ( i = 0, entry = ldap_first_entry( ld, msgp ); entry != NULL;
		    i++, entry = ldap_next_entry( ld, entry )) {
		dn = ldap_get_dn( ld, entry );
		edn = dsgw_strdup_escaped( dn );
		xdn = ldap_explode_dn( dn, 1 );
		printf( "dnlist[%d] = new Object\n", i );
		printf( "dnlist[%d].edn = '%s';\n", i, edn );
		js0 = dsgw_escape_quotes( xdn[ 0 ] );
		if ( xdn[1] != NULL ) {
		    js1 = dsgw_escape_quotes( xdn[ 1 ] );
		    printf( "dnlist[%d].rdn = '%s, %s';\n", i, js0, js1 );
		    free( js1 );
		} else {
		    printf( "dnlist[%d].rdn = '%s';\n", i, js0 );
		}
		free( js0 );
		if (( sn = ldap_get_values( ld, entry, "sn" )) == NULL ) {
		    js0 = NULL;
		} else {
		    js0 = dsgw_escape_quotes( sn[ 0 ] );
		    ldap_value_free( sn );
		}
		printf( "dnlist[%d].sn = '%s';\n", i, ( js0 == NULL ) ?
			" " : js0 );
		if ( js0 != NULL ) {
		    free( js0 );
		}

		printf( "dnlist[%d].selected = false;\n", i );
		free( edn );
		ldap_value_free( xdn );
		ldap_memfree( dn );
	    }
	    printf( "dnlist.count = %d;\n", i );
	    printf( "</SCRIPT>\n" );
	}
	ldap_msgfree( msgp );
    } else {
	/* Count <= 0 */
	if ( options & DSGW_DISPLAY_OPT_DNLIST_JS ) {
	    printf( "<SCRIPT LANGUAGE=\"JavaScript\">\n" );
	    printf( "var dnlist = new Array;\n" );
	    printf( "dnlist.count = 0;\n" );
	    printf( "</SCRIPT>\n" );
	}
    }

    dsgw_display_done( tip );
}


/*
 * initialize various LDAP library things -- any non-NULL parameters are
 *	initialized and set.  If an error occurs, this function will not
 *	return at all.
 * If an LDAP connection was opened, this function will return either
 * DSGW_BOUND_ASUSER if a valid cookie was found in the environment
 * and we were able to bind to the directory as that user.  If no
 * cookie was found, or the cookie would not be used to bind, then
 * an anonymous bind is performed and DSGW_BOUND_ANONYMOUS is returned.
 * If skipac (skip authentication check) is non-zero, then this
 * function will always authenticate as NULL.
 *
 * If we are configured to use a local LDAP database instead of a real
 * directory server, we always do an unauthenticated bind but we return
 * DSGW_BOUND_ASUSER.  This is done to keep our CGIs that check for a
 * return code of DSGW_BOUND_ASUSER happy.
 */

int
dsgw_init_ldap( LDAP **ldp, LDAPFiltDesc **lfdpp, int skipac )
{
    char	*path;
    char	*userid, *dn, *rndstr, *passwd, *cookie, *p;
    int		limit, ret;
#ifdef XP_WIN32
    WSADATA	wsadata;
#endif
    int		ldapversion;	/* ldap version: LDAP_VERSION2 or 3 */
    int res;

    /* LDAP search filters */
    if ( lfdpp != NULL ) {
	path = dsgw_file2path( gc->gc_configdir, DSGW_FILTERFILE );
	if (( *lfdpp = ldap_init_getfilter( path )) == NULL ) {
	    dsgw_error( DSGW_ERR_BADCONFIG, path, DSGW_ERROPT_EXIT, 0 );
	}
	free( path );
	ret =  0;
    }

#ifdef XP_WIN32

    if( ret = WSAStartup(0x0101, &wsadata ) != 0 )
	dsgw_error( DSGW_ERR_WSAINIT, NULL, DSGW_ERROPT_EXIT, 0 );

#endif /* XP_WIN32 */

    /* LDAP connection */
    if ( ldp != NULL ) {
	if ( gc == NULL ) {
	    dsgw_error( DSGW_ERR_INTERNAL, 
		    XP_GetClientStr(DBT_ldapInitLcacheInitAttemptedBefor_),
		    DSGW_ERROPT_EXIT, 0 );
	}
	if ( gc->gc_localdbconf == NULL ) {
	    /* "Real LDAP server" case */
#ifdef DSGW_NO_SSL
	    *ldp = ldap_init( gc->gc_ldapserver, gc->gc_ldapport );
#else /* DSGW_NO_SSL */
	    if ( gc->gc_ldapssl ) {
		if ( gc->gc_securitypath == NULL ) {
		    dsgw_error( DSGW_ERR_NOSECPATH, NULL, DSGW_ERROPT_EXIT, 0 );
		}

		if ( (res = ldapssl_client_init( gc->gc_securitypath,
			&certdbh )) < 0 ) {
		    dsgw_error( DSGW_ERR_SSLINIT, gc->gc_securitypath,
			    DSGW_ERROPT_EXIT, 0 );
		}
#ifdef FORTEZZA
		enable_fortezza_ciphers();
#endif /* FORTEZZA */
		*ldp = ldapssl_init( gc->gc_ldapserver, gc->gc_ldapport, 1 );

	    } /* if ( gc->gc_ldapssl ) */ 
	    else {
		*ldp = ldap_init( gc->gc_ldapserver, gc->gc_ldapport );	
	    }
#endif /* !DSGW_NO_SSL */
	    if ( *ldp == NULL ) {
		dsgw_error( DSGW_ERR_LDAPINIT, NULL, DSGW_ERROPT_EXIT,
			ldap_get_lderrno( *ldp, NULL, NULL ));
	    }

	} else {
	    /* Local DB case */
          /* No more lcache support  */
#if 0
	    int		optval;
	    if (( *ldp = ldap_init( NULL, 0 )) == NULL ) {
		dsgw_error( DSGW_ERR_LDAPINIT, NULL, DSGW_ERROPT_EXIT,
			ldap_get_lderrno( *ldp, NULL, NULL ));
	    }
	    if ( lcache_init( *ldp, gc->gc_localdbconf ) != 0 ) {
		dsgw_error( DSGW_ERR_LCACHEINIT, strerror(errno), DSGW_ERROPT_EXIT,
			ldap_get_lderrno( *ldp, NULL, NULL ));
	    }
	    optval = 1;
	    (void) ldap_set_option( *ldp, LDAP_OPT_CACHE_ENABLE, &optval );
	    optval = LDAP_CACHE_LOCALDB;
	    (void) ldap_set_option( *ldp, LDAP_OPT_CACHE_STRATEGY, &optval );

            gc->gc_ldapversion = LDAP_VERSION2;
#endif
	}
	rndstr = dn = NULL;
	passwd = dsgw_get_cgi_var( "passwd", DSGW_CGIVAR_OPTIONAL );

	if (( p = dsgw_get_cgi_var( "ldapsizelimit", DSGW_CGIVAR_OPTIONAL ))
		!= NULL ) {
	    limit = atoi( p );
	    (void) ldap_set_option( *ldp, LDAP_OPT_SIZELIMIT, &limit );
	}

	if (( p = dsgw_get_cgi_var( "ldaptimelimit", DSGW_CGIVAR_OPTIONAL ))
		!= NULL ) {
	    limit = atoi( p );
	    (void) ldap_set_option( *ldp, LDAP_OPT_TIMELIMIT, &limit );
	}

	/*
	 * we don't bother with authentication if:
	 *  the "skipac" flag is non-zero OR
	 *  no "passwd" form element was passed in and we are using local db
	 */
	if ( !skipac && ( passwd != NULL || gc->gc_localdbconf == NULL )) {
	    /*
	     * There are several ways in which authentication might
	     * happen.
	     */
	    if ( gc->gc_admserv ) {
		/*
		 * We're running under the admin server, so ask libadmin
		 * for the user's credentials.  If a password comes as a form
		 * field, it overrides value we get from admin server
		 */
		(void)dsgw_get_adm_identity( *ldp, &userid, &dn,
			( passwd == NULL ) ? &passwd : NULL, DSGW_ERROPT_EXIT );

		if ( dn == NULL ) {
		    int		lderr;
		    char	*errstr;

		    if ( userid == NULL ) {
			dsgw_error( DSGW_ERR_ADMSERV_CREDFAIL,
				"Got NULL user id from Admin Server", 0, 0 );
		    } else if (( dn = uid2dn( *ldp, userid,
			    gc->gc_ldapsearchbase, &lderr, &errstr ))
			    == NULL ) {
			dsgw_error( DSGW_ERR_ADMSERV_CREDFAIL,
				errstr, DSGW_ERROPT_EXIT, lderr );
		    }
		}

#ifdef DSGW_DEBUG
		dsgw_log( "dsgw_init_ldap: run under admserv, user id = %s, "
			"dn = %s, passwd = %s, skipac = %d, dn = 0x%x\n",
			userid == NULL ? "NULL" : userid,
			dn == NULL ? "NULL" : dn,
			passwd == NULL ? "NULL" : passwd,
			skipac, dn );
#endif
	    } else {
		/*
		 * Not running under admin server.  The DN and password
		 * might come in as form fields, or the authentication
		 * might be accomplished via a client-side cookie which
		 * gets looked up in the gateway's cookie database.
		 */

		/* check for dn/binddn in request */
		if ( passwd != NULL ) {
		    if (( dn = dsgw_get_cgi_var( "binddn",
			    DSGW_CGIVAR_OPTIONAL )) == NULL &&
			    ( dn = dsgw_get_cgi_var( "dn",
			    DSGW_CGIVAR_OPTIONAL )) == NULL ) {
			free( passwd );
			passwd = NULL;
		    } else {
			/* got DN:  undo extra level of escaping */
			dsgw_form_unescape( dn );
		    }
		}

		if ( passwd == NULL ) {
		    /* Check for a valid authentication cookie */
		    cookie = dsgw_get_auth_cookie();
		    if ( cookie != NULL ) {
			if ( dsgw_parse_cookie( cookie, &rndstr, &dn ) == 0 ) {
			    int ckrc;
			    if (( ckrc = dsgw_ckdn2passwd( rndstr, dn,
				    &passwd )) != 0 ) {
				dsgw_dn2passwd_error( ckrc );
				exit( 0 );
			    }
			}
		    }

		    if ( rndstr != NULL ) {
			free( rndstr );
		    }
		    if ( cookie != NULL ) {
			free( cookie );
		    }
		}
	    }
	}

	if ( bind_using_v2_or_v3( *ldp, dn, passwd ) != LDAP_SUCCESS ) {
	    dsgw_ldap_error( *ldp, DSGW_ERROPT_EXIT );
	}

	if (( dn != NULL ) && ( passwd != NULL )) {
	    ret = DSGW_BOUND_ASUSER;
	    binddn = dn;
	    bindpasswd = passwd;
	    ldap_set_rebind_proc( *ldp, get_rebind_credentials, NULL );
	} else if ( gc->gc_localdbconf != NULL ) {
	    ret = DSGW_BOUND_ASUSER;	/* a small, harmless lie */
	} else {
	    ret = DSGW_BOUND_ANONYMOUS;
	}

#ifdef DSGW_DEBUG
        dsgw_log( "dsgw_init_ldap: ldap version: %d (err: %d)\n",
                                       gc->gc_ldapversion, ret );
#endif
        /* I18n: If older than VERSION3, won't display the lang list */
	if ( gc->gc_ldapversion < LDAP_VERSION3 || !gc->gc_international ) {
	    dsgw_free_language_popup();
	    dsgw_setup_default_laguage_popup();

	    gc->gc_inputlanguage = gc->gc_languages;
	    gc->gc_defaultselectedlang = DSGW_DEFAULT_SELECTED_LANG;
	}

	return ret;
    }
}

/*  Similar to dsgw_init_ldap but without the SSL initialization
    or the user authentication.  Used by the edit.c and domodify.c */

int
dsgw_init_ldap_woSSL( LDAP **ldp, LDAPFiltDesc **lfdpp )
{
    char	*path;
    char	*userid, *dn, *rndstr, *passwd, *cookie, *p;
    int		limit, ret;
#ifdef XP_WIN32
    WSADATA	wsadata;
#endif
    int		ldapversion;	/* ldap version: LDAP_VERSION2 or 3 */
    int res;

    /* LDAP search filters */
    if ( lfdpp != NULL ) {
	path = dsgw_file2path( gc->gc_configdir, DSGW_FILTERFILE );
	if (( *lfdpp = ldap_init_getfilter( path )) == NULL ) {
	    dsgw_error( DSGW_ERR_BADCONFIG, path, DSGW_ERROPT_EXIT, 0 );
	}
	free( path );
	ret =  0;
    }

#ifdef XP_WIN32

    if( ret = WSAStartup(0x0101, &wsadata ) != 0 )
	dsgw_error( DSGW_ERR_WSAINIT, NULL, DSGW_ERROPT_EXIT, 0 );

#endif /* XP_WIN32 */

    /* LDAP connection */
    if ( ldp != NULL ) {
	if ( gc == NULL ) {
	    dsgw_error( DSGW_ERR_INTERNAL, 
		    XP_GetClientStr(DBT_ldapInitLcacheInitAttemptedBefor_),
		    DSGW_ERROPT_EXIT, 0 );
	}
	rndstr = dn = NULL;
	passwd = dsgw_get_cgi_var( "passwd", DSGW_CGIVAR_OPTIONAL );

	if (( p = dsgw_get_cgi_var( "ldapsizelimit", DSGW_CGIVAR_OPTIONAL ))
		!= NULL ) {
	    limit = atoi( p );
	    (void) ldap_set_option( *ldp, LDAP_OPT_SIZELIMIT, &limit );
	}

	if (( p = dsgw_get_cgi_var( "ldaptimelimit", DSGW_CGIVAR_OPTIONAL ))
		!= NULL ) {
	    limit = atoi( p );
	    (void) ldap_set_option( *ldp, LDAP_OPT_TIMELIMIT, &limit );
	}

	if ( bind_using_v2_or_v3( *ldp, dn, passwd ) != LDAP_SUCCESS ) {
	    dsgw_ldap_error( *ldp, DSGW_ERROPT_EXIT );
	}

	if (( dn != NULL ) && ( passwd != NULL )) {
	    ret = DSGW_BOUND_ASUSER;
	    binddn = dn;
	    bindpasswd = passwd;
	    ldap_set_rebind_proc( *ldp, get_rebind_credentials, NULL );
	} else if ( gc->gc_localdbconf != NULL ) {
	    ret = DSGW_BOUND_ASUSER;	/* a small, harmless lie */
	} else {
	    ret = DSGW_BOUND_ANONYMOUS;
	}

#ifdef DSGW_DEBUG
        dsgw_log( "dsgw_init_ldap: ldap version: %d (err: %d)\n",
                                       gc->gc_ldapversion, ret );
#endif
        /* I18n: If older than VERSION3, won't display the lang list */
	if ( gc->gc_ldapversion < LDAP_VERSION3 || !gc->gc_international ) {
	    dsgw_free_language_popup();
	    dsgw_setup_default_laguage_popup();

	    gc->gc_inputlanguage = gc->gc_languages;
	    gc->gc_defaultselectedlang = DSGW_DEFAULT_SELECTED_LANG;
	}

	return ret;
    }
}


/*
 * do a simple bind, using LDAPv3 if desired and supported.
 *
 * return an LDAP error code (LDAP_SUCCESS if all goes well).
 * 
 * side effects: sets gc->gc_ldapversion variable first time a bind succeeds.
 */ 
static int
bind_using_v2_or_v3( LDAP *ld, char *dn, char *passwd )
{
    int		ldapversion, lderr;
    static int	already_bound = 0;

    if ( !already_bound ) {
	if ( gc->gc_international ) {
	    /* 
	     * try to use LDAP version 3 but fall back to v2 if not supported
	     */ 
	    ldapversion = highest_ldap_version_supported( ld, LDAP_VERSION2 ); 
	    if ( ldapversion > LDAP_VERSION3 ) {
		ldapversion = LDAP_VERSION3;
	    }
	    (void)ldap_set_option( ld, LDAP_OPT_PROTOCOL_VERSION,
		    &ldapversion ); 
	} else {
	    ldapversion = LDAP_VERSION2;
	}
    }

    lderr = ldap_simple_bind_s( ld, dn, passwd );

    if ( !already_bound && lderr == LDAP_SUCCESS ) {
	already_bound = 1;
	gc->gc_ldapversion = ldapversion;
    }

    return( lderr );
}


/*
 * read a server's rootDSE supportedLDAPVersion attribute and return the
 * highest number found.  If anything goes wrong, return defaultversion.
 *
 * side effect: the LDAP_OPT_REFERRALS setting is changed by the function.
 *	Before returning, we always set it to LDAP_OPT_ON.
 */
static int
highest_ldap_version_supported( LDAP *ld, int defaultversion )
{
    int		i, ldapversion, tmpversion;
    char	*attrs[2], **vals;
    LDAPMessage	*res, *e;

    ldapversion = defaultversion;	/* pessimistic */

    (void)ldap_set_option( ld, LDAP_OPT_REFERRALS, LDAP_OPT_OFF );

    res = NULL;
    attrs[0] = "supportedLDAPVersion";
    attrs[1] = NULL;
    if ( ldap_search_s( ld, "", LDAP_SCOPE_BASE, "(objectclass=*)", attrs, 0,
	    &res ) == LDAP_SUCCESS
	    && ( e = ldap_first_entry( ld, res )) != NULL
	    && ( vals = ldap_get_values( ld, e, attrs[0] )) != NULL ) {
	ldapversion = 0;
	for ( i = 0; vals[ i ] != NULL; ++i ) {
	    if (( tmpversion = atoi( vals[ i ] )) > ldapversion ) {
		ldapversion = tmpversion;
	    }
	}
    }

    if ( res != NULL ) {
	ldap_msgfree( res );
    }
    (void)ldap_set_option( ld, LDAP_OPT_REFERRALS, LDAP_OPT_ON );

    return( ldapversion );
}


/* 
 * get user identity from the admin. server (if running under it)
 *	if uidp is non-NULL, it is set to point to user's login id.
 *	if dnp is non-NULL, it is set to point to user's DN.
 *	if pwdp is non-NULL, it is set to point to user's password.
 * Returns: 0 if all goes well, -1 if an error occurs.
 *
 * Note that ld is used only if dnp != NULL, and then only if the admin server
 * returns NULL when asked for the DN.
 */
int
dsgw_get_adm_identity( LDAP *ld, char **uidp, char **dnp, char **pwdp,
	int erropts  )
{
    int		rc, need_to_get_dn;
    char	*uid;

    if ( !gc->gc_admserv ) {
	dsgw_error( DSGW_ERR_ADMSERV_CREDFAIL,
	    XP_GetClientStr(DBT_notRunningUnderTheAdministration_),
	    0, erropts );
	return( -1 );
    }

    need_to_get_dn = ( dnp != NULL );

    if ( need_to_get_dn && ADM_GetUserDNString( &rc, dnp ) < 0 ) {
	dsgw_error( DSGW_ERR_ADMSERV_CREDFAIL,
		XP_GetClientStr(DBT_couldNotMapUsernameToADnErrorFro_),
		erropts, 0 );
	return( -1 );
    }

    /*
     * get userid if:
     *    1. requested by caller (uidp != NULL)
     * or 2. DN was requested but Admin Server didn't return the DN
     */
    if (( uidp != NULL || ( need_to_get_dn && *dnp == NULL )) &&
	    ( ADM_GetCurrentUsername( &rc, &uid ) < 0 || uid == NULL )) {
	dsgw_error( DSGW_ERR_ADMSERV_CREDFAIL,
		XP_GetClientStr(DBT_couldNotGetCurrentUsername_), erropts, 0 );
	return( -1 );
    }

    if ( uidp != NULL ) {
	*uidp = uid;
    }

    if ( need_to_get_dn && *dnp == NULL ) {
	/*
	 * try to map userid to DN using LDAP search
	 */
	int	lderr;
	char	*errstr;

	if (( *dnp = uid2dn( ld, uid, gc->gc_ldapsearchbase, &lderr, &errstr ))
		== NULL ) {
	    dsgw_error( DSGW_ERR_ADMSERV_CREDFAIL, errstr, erropts, lderr );
	    return( -1 );
	}
    }

    if ( pwdp != NULL && ADM_GetCurrentPassword( &rc, pwdp ) < 0 ) {
	dsgw_error( DSGW_ERR_ADMSERV_CREDFAIL,
		XP_GetClientStr(DBT_couldNotGetCurrentUserPassword_), erropts, 0 );
	return( -1 );
    }

    return( 0 );
}


void
dsgw_ldap_error( LDAP *ld, int erropts )
{
    int	lderr, dsgw_err;

    lderr = ldap_get_lderrno( ld, NULL, NULL );

    if ( gc->gc_localdbconf == NULL ) {
	dsgw_err = DSGW_ERR_LDAPGENERAL;	/* LDAP server */
    } else {
	dsgw_err = DSGW_ERR_LOCALDBGENERAL;	/* local db */
    }

    dsgw_error( dsgw_err, ldap_err2string( lderr ), erropts, lderr );
}


struct ldap_searchobj *
dsgw_type2searchobj( struct ldap_searchobj *solistp, char *type )
{
    struct ldap_searchobj	*sop;

    for ( sop = ldap_first_searchobj( solistp ); sop != NULL;
	    sop = ldap_next_searchobj( solistp, sop )) {
	if ( strcasecmp( type, sop->so_objtypeprompt ) == 0 ) {
	    return( sop );
	}
    }

    return( NULL );
}


struct ldap_searchattr *
dsgw_label2searchattr( struct ldap_searchobj *sop, char *label )
{
    struct ldap_searchattr *sap;

    for ( sap = sop->so_salist; sap != NULL; sap = sap->sa_next ) {
	if ( strcasecmp( label, sap->sa_attrlabel ) == 0 ) {
	    return( sap );
	}
    }

    return( NULL );
}


struct ldap_searchmatch *
dsgw_prompt2searchmatch( struct ldap_searchobj *sop, char *prompt )
{
    struct ldap_searchmatch *smp;

    for ( smp = sop->so_smlist; smp != NULL; smp = smp->sm_next ) {
	if ( strcasecmp( prompt, smp->sm_matchprompt ) == 0 ) {
	    return( smp );
	}
    }

    return( NULL );
}


static dsgwtmplinfo *
init_listdisplay( char *tmplname, unsigned long options )
{
    char	*s;

    if (( s = dsgw_get_cgi_var( "listtemplate", DSGW_CGIVAR_OPTIONAL ))
	    != NULL ) {
	tmplname = s;
    }

    return( dsgw_display_init( DSGW_TMPLTYPE_LIST, tmplname, options ));
}


void
dsgw_smart_search( LDAP *ld, struct ldap_searchobj *sop, LDAPFiltDesc *lfdp,
	char *base, char *value, unsigned long options )
{
    char		*searchdesc;
    int			rc;
    LDAPFiltInfo	*lfip;
    dsgwtmplinfo	*tip;
    LDAPMessage		*msgp;

    ldap_setfilteraffixes( lfdp, sop->so_filterprefix, NULL );
    tip = init_listdisplay( sop->so_objtypeprompt, options );

    if (( lfip = ldap_getfirstfilter( lfdp, sop->so_filtertag, value ))
	    == NULL ) {
	dsgw_error( DSGW_ERR_NOFILTERS, sop->so_objtypeprompt,
		DSGW_ERROPT_EXIT, 0 );
    }

    for ( ; lfip != NULL; lfip = ldap_getnextfilter( lfdp )) {
	searchdesc = dsgw_ch_malloc( strlen( lfip->lfi_desc ) +
		strlen( value ) + 4 );
	sprintf( searchdesc, "%s '%s'", lfip->lfi_desc, value );
	dsgw_set_searchinfo( tip, 0, searchdesc, NULL );

	rc = do_search( tip, ld, base, sop->so_defaultscope, lfip->lfi_filter,
		&msgp );

	if ( rc != LDAP_SUCCESS ||
		( msgp != NULL && ldap_count_entries( ld, msgp ) > 0 )) {
	    if ( strstr( lfip->lfi_filter, "~=" ) != NULL ) {
		/* always list if approximate filter used to find entry */
		options |= DSGW_DISPLAY_OPT_LIST_IF_ONE;
	    }
	    break;	/* error or got some entries:  stop searching */
	}
    }

    handle_search_results( tip, ld, rc, msgp, options );
}


void
dsgw_complex_search( LDAP *ld, struct ldap_searchobj *sop,
	struct ldap_searchattr *sap, struct ldap_searchmatch *smp,
        char *base, int scope, char *value, unsigned long options )
{
    char		*searchdesc;

    searchdesc = dsgw_ch_malloc( strlen( sap->sa_attrlabel )
	    + strlen( smp->sm_matchprompt ) + strlen( value ) + 5 );
    sprintf( searchdesc, "%s %s '%s'", sap->sa_attrlabel, smp->sm_matchprompt,
	    value );

    dsgw_pattern_search( ld, sop->so_objtypeprompt, searchdesc,
	    smp->sm_filter, sop->so_filterprefix, NULL, sap->sa_attr,
	    base, scope, value, options );
}


void
dsgw_pattern_search( LDAP *ld, char *listtmpl, char *searchdesc,
	char *filtpattern, char *filtprefix, char *filtsuffix, char *attr,
	char *base, int scope, char *value, unsigned long options )
{
    char		buf[ 4096 ];
    int			rc;
    dsgwtmplinfo	*tip;
    LDAPMessage		*msgp;

    tip = init_listdisplay( listtmpl, options );

    ldap_build_filter( buf, sizeof( buf ), filtpattern,
	    filtprefix, filtsuffix, attr, value, NULL );

    dsgw_set_searchinfo( tip, 0, searchdesc, NULL );

    rc = do_search( tip, ld, base, scope, buf, &msgp );
    handle_search_results( tip, ld, rc, msgp, options );
}


/*
 * Perform URL-based search.
 * Note that if "ld" is NULL, this routine sets gc->gc_ldapserver and
 * gc->gc_ldapport globals itself, calls dsgw_init_ldap(), and then does
 * the URL-based search.  If "ld" is not NULL, no initialization is done
 * here.
 */
void
dsgw_ldapurl_search( LDAP *ld, char *ldapurl, unsigned long options )
{
    int			rc, ec, saveport, did_init_ldap;
    LDAPMessage		*msgp;
    LDAPURLDesc		*ludp;
    char		*saveserver;

    if (( rc = ldap_url_parse( ldapurl, &ludp )) != 0 ) {
	switch ( rc ) {
	case LDAP_URL_ERR_NODN:
	    ec = DSGW_ERR_LDAPURL_NODN;
	    break;
	case LDAP_URL_ERR_BADSCOPE:
	    ec = DSGW_ERR_LDAPURL_BADSCOPE;
	    break;
	case LDAP_URL_ERR_MEM:
	    ec = DSGW_ERR_NOMEMORY;
	    break;
	case LDAP_URL_ERR_NOTLDAP:
	default:
	    ec = DSGW_ERR_LDAPURL_NOTLDAP;
	    break;
	}
	dsgw_error( ec, ldapurl, DSGW_ERROPT_EXIT, 0 );
    }

    if ( ld == NULL ) {
	saveserver = gc->gc_ldapserver;
	gc->gc_ldapserver = ludp->lud_host;
	saveport = gc->gc_ldapport;
	gc->gc_ldapport = ludp->lud_port;

	(void)dsgw_init_ldap( &ld, NULL, 0 );
	did_init_ldap = 1;
    } else {
	did_init_ldap = 0;
    }

    /* XXX a bit of a hack:  if it looks like only a DN was included, we
     * assume that a read of the entry is desired.
     */
    if ( ludp->lud_scope == LDAP_SCOPE_BASE && strcasecmp( ludp->lud_filter,
	    "(objectClass=*)" ) == 0 ) {
	dsgw_read_entry( ld, ludp->lud_dn, NULL, NULL, options );
    } else {
	dsgwtmplinfo	*tip;

	tip = init_listdisplay( "urlsearch", options );
	dsgw_set_searchinfo( tip, 0, dsgw_ch_strdup( ldapurl ), NULL );
	rc = do_search( tip, ld, ludp->lud_dn, ludp->lud_scope,
		ludp->lud_filter, &msgp );
	handle_search_results( tip, ld, rc, msgp, options );
    }

    if ( did_init_ldap ) {
	ldap_unbind( ld );
	gc->gc_ldapserver = saveserver;
	gc->gc_ldapport = saveport;
    }
}


/*
 * do the actual search over LDAP.  Return an LDAP error code.
 */
static int
do_search( dsgwtmplinfo *tip, LDAP *ld, char *base, int scope, char *filter,
	LDAPMessage **msgpp )
{
    char		**attrlist, *attrs[ 4 ];

    *msgpp = NULL;

    if ( tip == NULL || tip->dsti_attrs == NULL ) {
	attrs[ 0 ] = DSGW_ATTRTYPE_OBJECTCLASS;
	if ( tip != NULL && tip->dsti_sortbyattrs[0] != NULL ) {
	    attrs[ 1 ] = tip->dsti_sortbyattrs[0];
	    attrs[ 2 ] = tip->dsti_sortbyattrs[1];
	    attrs[ 3 ] = NULL;
	} else {
	    attrs[ 1 ] = NULL;
	}
	attrlist = attrs;
    } else {
	attrlist = tip->dsti_attrs;
    }

    return( ldap_search_s( ld, base, scope, filter, attrlist, 0, msgpp ));
}


char *
dsgw_ldaperr2string( int lderr )
{
    char	*errstr, *s;

    switch( lderr ) {
    case LDAP_TIMELIMIT_EXCEEDED:
	errstr = DSGW_MSG_TLIMIT;
	break;
    case LDAP_SIZELIMIT_EXCEEDED:
	errstr = DSGW_MSG_SLIMIT;
	break;
    case LDAP_PARTIAL_RESULTS:
	errstr = DSGW_MSG_PARTIAL;
	break;
    default:
	s = ldap_err2string( lderr );
	/* alloc enough room for error string + "Error: " + zero termination */
	errstr = dsgw_ch_malloc( strlen( s ) + 8 );
	sprintf( errstr, XP_GetClientStr(DBT_errorS_), s );
	return( errstr );
    }

    return( dsgw_ch_strdup( errstr ));
}


/*
 * read and display a single entry.  If ocvals is non-NULL, it should
 * contain the list of objectClass values for this entry.
 */ 
void
dsgw_read_entry( LDAP *ld, char *dn, char **ocvals, char *tmplname,
	unsigned long options )
{
    int			rc, freeocvals;
    LDAPMessage		*msgp, *entry;
    dsgwtmpl		*tmpl;
    dsgwtmplinfo	*tip;

    if (( options & DSGW_DISPLAY_OPT_AUTH ) != 0 ) {
	/*
	 * XXX hack -- if we are trying to authenticate, we don't generate an
	 * entry display at all.  Instead, we generate an authenticate form.
	 */
	dsgw_emit_auth_form( dn );
	return;
    }

    if ( tmplname == NULL && ( tmplname = dsgw_get_cgi_var( "displaytemplate",
	    DSGW_CGIVAR_OPTIONAL )) == NULL ) {
	/* determine what display template to use based on objectClass values */
	freeocvals = 0;
	if ( ocvals == NULL ) {	/* read entry to get objectClasses */
	    char	*attrs[ 2 ];

	    attrs[ 0 ] = DSGW_ATTRTYPE_OBJECTCLASS;
	    attrs[ 1 ] = NULL;

	    if (( rc = ldap_search_s( ld, dn, LDAP_SCOPE_BASE, "objectClass=*",
		    attrs, 0, &msgp )) != LDAP_SUCCESS ||
		    ( entry = ldap_first_entry( ld, msgp )) == NULL ) {
		dsgw_ldap_error( ld, DSGW_ERROPT_EXIT );
	    }
	    ocvals = ldap_get_values( ld, msgp, DSGW_ATTRTYPE_OBJECTCLASS );
	    freeocvals = 1;
	    ldap_msgfree( msgp );
	}


	if ( ocvals == NULL || ( tmpl = dsgw_oc2template( ocvals )) == NULL ) {
	    tmplname = NULL;
	} else {
	    tmplname = tmpl->dstmpl_name;
	}

	if ( freeocvals ) {
	    ldap_value_free( ocvals );
	}
    }

    if ( tmplname == NULL ) {
	char	*title;

	tip = NULL;
	if (( title = ldap_dn2ufn( dn )) == NULL ) {
	    title = dn;
	}
	dsgw_html_begin( title, 1 );
	fprintf( stdout, "<FONT SIZE=\"+1\">\n%s\n</FONT>\n",
        XP_GetClientStr(DBT_noteThereIsNoDisplayTemplateForT_) );
    } else {
	tip = dsgw_display_init( DSGW_TMPLTYPE_DISPLAY, tmplname, options );
    }

    /* now read the attributes needed for the template */
    if (( rc = ldap_search_s( ld, dn, LDAP_SCOPE_BASE, "objectClass=*",
	    ( tip == NULL ) ? NULL : tip->dsti_attrs, 0, &msgp ))
	    != LDAP_SUCCESS ) {
	dsgw_ldap_error( ld, DSGW_ERROPT_EXIT );
    }

    if (( entry = ldap_first_entry( ld, msgp )) == NULL ) {
	ldap_msgfree( msgp );
	dsgw_ldap_error( ld, DSGW_ERROPT_EXIT );
    }

    /* display it (finally!) */
    if ( tip == NULL ) {
	/* no template available -- display in an ugly but complete manner */
	if (( rc = ldap_entry2html( ld, NULL, entry, NULL, NULL, NULL,
		entry2htmlwrite, stdout, "\n", 0, LDAP_DISP_OPT_HTMLBODYONLY,
		NULL, NULL )) != LDAP_SUCCESS ) {
	    dsgw_ldap_error( ld, DSGW_ERROPT_EXIT );
	}
	dsgw_html_end();
    } else {
	/* use template to create a nicely formatted display */
	dsgw_display_entry( tip, ld, entry, NULL );
	dsgw_display_done( tip );
    }

    ldap_msgfree( msgp );
}


/*
 * return 1 if the entry already exists, 0 if not, -1 if some error occurs
 */
int
dsgw_ldap_entry_exists( LDAP *ld, char *dn, char **matchedp,
	unsigned long erropts )
{
    LDAPMessage *msgp;
    int		rc;

    msgp = NULL;
    if ( matchedp != NULL ) {
	*matchedp = NULL;
    }

    if (( rc = do_search( NULL, ld, dn, LDAP_SCOPE_BASE, "(objectClass=*)",
	    &msgp )) != LDAP_SUCCESS && rc != LDAP_NO_SUCH_OBJECT ) {
	dsgw_ldap_error( ld, erropts );
    }

    if ( msgp == NULL ) {
	rc = 0;
	if ( matchedp != NULL ) {
	    (void)ldap_get_lderrno( ld, matchedp, NULL );
	}
    } else {
	rc = ( ldap_count_entries( ld, msgp ) > 0 ? 1 : 0 );
	ldap_msgfree( msgp );
    }

    return( rc );
}


static int
entry2htmlwrite( void *fp, char *buf, int len )
{
        return( fwrite( buf, len, 1, (FILE *)fp ) == 0 ? -1 : len );
}


/*
 * return 1 if the entry's parent exists, 0 if not, -1 if some error occurs.
 * If the entry is the same as gc->gc_ldapsearchbase, then we return 1,
 * so we don't prevent people from adding their organizational entry.
 */
int
dsgw_ldap_parent_exists( LDAP *ld, char *dn, unsigned long erropts )
{
    LDAPMessage *msgp;
    int		rc;

    /* Is "dn" == gc->gc_ldapsearchbase? */
    msgp = NULL;
    if (( rc = do_search( NULL, ld, dn, LDAP_SCOPE_BASE, "(objectClass=*)",
	    &msgp )) != LDAP_SUCCESS && rc != LDAP_NO_SUCH_OBJECT ) {
	dsgw_ldap_error( ld, erropts );
    }

    if ( msgp == NULL ) {
	rc = 0;
    } else {
	rc = ( ldap_count_entries( ld, msgp ) > 0 ? 1 : 0 );
	ldap_msgfree( msgp );
    }

    return( rc );
}



void
dsgw_get_bind_credentials( char **dnp, char **pwdp )
{
    if ( dnp != NULL ) {
	*dnp = binddn;
    }

    if ( pwdp != NULL ) {
	*pwdp = bindpasswd;
    }
}

/*
 * return 1 if bound using "dn"
 * return 0 if definitely bound as someone else
 * return "def_answer" is we can't tell for sure
 */
int
dsgw_bound_as_dn( char *dn, int def_answer )
{
    int		i, rc;
    char	**rdns1, **rdns2;

    if ( binddn == NULL ) {
	/*
	 * not authenticated: if not using local db or using it as an
	 * end-user, return the default
	 */
	if ( gc->gc_localdbconf == NULL || gc->gc_enduser ) {
	    return( def_answer );
	}

	/*
         * if using local db as an admin, return "bound as someone else"
	 * since there is no access control enforced anyways.
	 */
	return( 0 );
    }

    /* first try a simple case-insensitive comparison */
    if ( strcasecmp( binddn, dn ) == 0 ) {
	return( 1 );	/* DNs are the same */
    }

    /*
     * These DNs may not have the same spacing or punctuation.  Compare RDN
     * components to eliminate any differences.
     */
    if (( rdns1 = ldap_explode_dn( binddn, 0 )) == NULL ) {
	return( def_answer );	/* we don't know: return the default */
    }

    if (( rdns2 = ldap_explode_dn( dn, 0 )) == NULL ) {
	ldap_value_free( rdns1 );
	return( def_answer );	/* we don't know: return the default */
    }

    for ( i = 0; rdns1[ i ] != NULL && rdns2[ i ] != NULL; ++i ) {
	if ( strcasecmp( rdns1[ i ], rdns2[ i ] ) != 0 ) {
	    break;	/* DNs are not the same */
	}
    }

    rc = ( rdns1[ i ] == NULL && rdns2[ i ] == NULL );

    ldap_value_free( rdns1 );
    ldap_value_free( rdns2 );

    return( rc );
}



/*
 * Compare 2 DNs.  Return 1 if they are equivalent, 0 if not.
 */
int
dsgw_dn_cmp( char *dn1, char *dn2 )
{
    int		i, rc;
    char	**rdns1, **rdns2;

    /* first try a simple case-insensitive comparison */
    if ( strcasecmp( dn1, dn2 ) == 0 ) {
	return( 1 );	/* DNs are the same */
    }

    /*
     * These DNs may not have the same spacing or punctuation.  Compare RDN
     * components to eliminate any differences.
     */
    if (( rdns1 = ldap_explode_dn( dn1, 0 )) == NULL ) {
	return( 0 );	/* we don't know: return 0 */
    }

    if (( rdns2 = ldap_explode_dn( dn2, 0 )) == NULL ) {
	ldap_value_free( rdns1 );
	return( 0 );	/* we don't know: return 0 */
    }

    for ( i = 0; rdns1[ i ] != NULL && rdns2[ i ] != NULL; ++i ) {
	if ( strcasecmp( rdns1[ i ], rdns2[ i ] ) != 0 ) {
	    break;	/* DNs are not the same */
	}
    }

    rc = ( rdns1[ i ] == NULL && rdns2[ i ] == NULL );

    ldap_value_free( rdns1 );
    ldap_value_free( rdns2 );

    return( rc );
}


/*
 * Return the parent of dn.  The caller is responsible for freeing the
 * returned value.  Returns NULL on error.
 */
char *
dsgw_dn_parent( char *dn )
{
    char *dnp;
    int i;
    char **rdns;

    if ( dn == NULL ) {
	return( NULL );
    }

    dnp = dsgw_ch_malloc( strlen( dn ));
    dnp[ 0 ] = '\0';
    if (( rdns = ldap_explode_dn( dn, 0 )) == NULL ) {
	return NULL;
    }
    for ( i = 1; rdns[ i ] != NULL; i++ ) {
	strcat( dnp, rdns[ i ] );
	strcat( dnp, "," );
    }
    /* Get rid of the trailing "," we just appended */
    dnp[ strlen( dnp ) - 1 ] = '\0';
    ldap_value_free( rdns );
    return( dnp );
}
    

/*
 * Return 1 if dn1 is the immediate ancestor of dn2, 0 otherwise.
 */
int
dsgw_is_dnparent( char *dn1, char *dn2 )
{
    char *dnp;
    int rc;

    /* A null or zero-length DN cannot have a parent */
    if ( dn2 == NULL || strlen( dn2 ) == 0 ) {
	return 0;
    }

    dnp = dsgw_dn_parent( dn2 );
    rc = dsgw_dn_cmp( dn1, dnp );
    free( dnp );

    return rc;
}


/*
 * return malloc'd array of RDN attribute value pairs
 * each element of the array is a string that looks like:  TAG=VALUE
 * this is used to extract values from the RDN when a new entry is added
 */
char **
dsgw_rdn_values( char *dn )
{
    char	**rdns, **rdncomps, *val;
    int		i;

    if (( rdns = ldap_explode_dn( dn, 0 )) == NULL ) {
	return( NULL );
    }

    rdncomps = ldap_explode_rdn( rdns[0], 0 );
    ldap_value_free( rdns );
    if ( rdncomps == NULL ) {
	return( NULL );
    }

    for ( i = 0; rdncomps[ i ] != NULL; ++i ) {
	if (( val = strchr( rdncomps[ i ], '=' )) == NULL ) {
	    ldap_value_free( rdncomps );
	    return( NULL );
	}
	++val;
	strcpy_special_undo( val, val );	/* undo in place */
    }

    return( rdncomps );
}


/*
 * the following routine was lifted from servers/slapd/ava.c
 * it removes special quoting, etc. from values that appear in an LDAP DN
 */
static void
strcpy_special_undo( char *d, char *s )
{
    int     quote;
 
    quote = 0;
    if ( *s == '"' ) {
	    s++;
	    quote = 1;
    }
    for ( ; *s; s++ ) {
	switch ( *s ) {
	case '"':
	    break;
	case '\\':
	    s++;
	    /* FALL */
	default:
	    *d++ = *s;
	    break;
	}
    }
    *d-- = '\0';
    if ( quote && *d == '"' ) {
	*d = '\0';
    }
}


static char *
uid2dn( LDAP *ld, char *uid, char *base, int *ldaprc, char **errsp )
{
    char *attrs[] = { "objectclass", NULL };
    char filtbuf[ 85 ];	/* max of 80 char. uid + "uid=" + zero terminator */
    int rc, count;
    LDAPMessage *result;
    LDAPMessage *e;
    char *dn, *save_binddn, *save_bindpw;
    
    *ldaprc = LDAP_SUCCESS;	/* optimistic */
    *errsp = NULL;

    if ( ld == NULL || uid == NULL || strlen( uid ) > 80 ) {
	*errsp = XP_GetClientStr(DBT_invalidUserIdOrNullLdapHandle_);
	return NULL;
    }

    /*
     * When mapping a user id (uid) to a DN, always bind as entity in
     * dbswitch.conf file
     */
    dsgw_get_bind_credentials( &save_binddn, &save_bindpw );
    if (( rc = bind_using_v2_or_v3( ld, gc->gc_binddn, gc->gc_bindpw ))
	    != LDAP_SUCCESS ) {
	if ( ldaprc != NULL ) {
	    *ldaprc = rc;
	    return NULL;
	}
    }

    sprintf( filtbuf, "uid=%s", uid );

    if (( rc = ldap_search_s( ld, base, LDAP_SCOPE_SUBTREE, filtbuf,
	    attrs, 1, &result )) != LDAP_SUCCESS ) {
	if ( ldaprc != NULL ) {
	    *ldaprc = rc;
	    return NULL;
	}
    }

    /*
     * restore previous bind identity (if non-NULL)
     */
    if ( save_binddn != NULL && save_bindpw != NULL ) {
        (void)bind_using_v2_or_v3( ld, save_binddn, save_bindpw );
    }

    if (( count = ldap_count_entries( ld, result )) != 1 ) {
	/* Search either returned no entries, or more than one entry */
	ldap_msgfree( result );
	if ( count == 0 ) {
	    *errsp = XP_GetClientStr(DBT_noMatchForUserId_);
	} else {
	    *errsp = XP_GetClientStr(DBT_moreThanOneMatchForUserId_);
	}
	return NULL;
    }

    dn = NULL;
    if (( e = ldap_first_entry( ld, result )) == NULL ||
	    ( dn = ldap_get_dn( ld, e )) == NULL ) {
	*ldaprc = ldap_get_lderrno( ld, NULL, NULL );
    }
    ldap_msgfree( result );
    return( dn );
}


/*
 * Emit an HTML "SELECT" object that contains all the o's and ou's that
 * are underneath our default searchbase.  If there are none other than
 * the searchbase, we emit a hidden HTML TEXT object that contains the
 * searchbase and the "prefix" and "suffix" are not used.  The values for
 * the SELECT options and for the TEXT object are all escaped DNs.
 *  
 * Location popup directives look like this:
 *	<-- DS_LOCATIONPOPUP "name=VARNAME" "prefix=PREFIX" "suffix=SUFFIX" -->
 *
 * If "prefix" and/or "suffix" are omitted, they default to "".
 * If "name" is omitted it defaults to "base".
 *
 * If there are "location" directives in the dsgw.conf file, we use those
 * instead of actually searching the directory.
 */
void
dsgw_emit_location_popup( LDAP **ldp, int argc, char **argv, int erropts )
{
    char	line[BIG_LINE];
    char	*varname, *prefix, *suffix, *rootname, *dn;
    char	*basedn, **base_rdns;
    int		i, base_rdn_count, count, did_init_ldap;
    LDAPMessage	*res, *e;

    if (( varname = get_arg_by_name( "name", argc, argv )) == NULL ) {
	varname = "base";
    }
    if (( prefix = get_arg_by_name( "prefix", argc, argv )) == NULL ) {
	prefix = "";
    }
    if (( suffix = get_arg_by_name( "suffix", argc, argv )) == NULL ) {
	suffix = "";
    }
    rootname = get_arg_by_name( "rootname", argc, argv );

    did_init_ldap = 0;
    res = NULL;

    if ( gc->gc_newentryloccount > 0 ) {
	count = gc->gc_newentryloccount;
	basedn = gc->gc_ldapsearchbase;
    } else {
	int		rc;
	LDAPURLDesc	*ludp;

	if ( ldap_url_parse( gc->gc_locationurl, &ludp ) != 0 ) {
	    dsgw_error( DSGW_ERR_BADCONFIG,
		    XP_GetClientStr(DBT_invalidLocationURL_), erropts, 0 );
	    return;
	}

	if ( *ldp == NULL ) {
	    (void)dsgw_init_ldap( ldp, NULL, 0 );
	    did_init_ldap = 1;
	}

	if ( ludp->lud_dn == NULL ) {
	    ludp->lud_dn = gc->gc_ldapsearchbase;
	}
	basedn = ludp->lud_dn;

	rc = ldap_search_s( *ldp, ludp->lud_dn, ludp->lud_scope,
		ludp->lud_filter, ludp->lud_attrs, 1, &res );
	if ( rc != LDAP_SUCCESS || res == NULL ) {
	    dsgw_ldap_error( *ldp, erropts );
	    return;
	}

	count = ldap_count_entries( *ldp, res );
	if ( gc->gc_ldapsearchbase == NULL || *gc->gc_ldapsearchbase == '\0' ) {
	    ++count;	/* include base DN even if it is "" */
	} else {
	    /*
	     * check to see if search base was one of the entries returned
	     * we want to always list the base entry, so we need to check
	     */
	    for ( e = ldap_first_entry( *ldp, res ); e != NULL;
		    e = ldap_next_entry( *ldp, e )) {
		if (( dn = ldap_get_dn( *ldp, e )) == NULL ) {
		    dsgw_ldap_error( *ldp, erropts );
		    ldap_msgfree( res );
		    return;
		}

		rc = dsgw_dn_cmp( dn, gc->gc_ldapsearchbase );
		free( dn );
		if ( rc ) {	/* base DN was returned */
		    break;
		}
	    }
	    if ( e == NULL ) {
		++count;	/* include base DN even if was not returned */
	    }
	}
    }

    base_rdn_count = 0;
    if ( basedn != NULL && *basedn != '\0' ) {
	if (( base_rdns = ldap_explode_dn( basedn, 1 )) != NULL ) {
	    while ( base_rdns[ base_rdn_count ] != NULL ) {
		++base_rdn_count;
	    }
	}
    } else {
	base_rdns = NULL;
    }

    if ( count > 1 ) {
	util_snprintf( line, BIG_LINE, "%s\n<SELECT NAME=\"%s\">\n",
		prefix, varname );
    } else {
	util_snprintf( line, BIG_LINE, "<INPUT TYPE=\"hidden\" NAME=\"%s\" ",
		varname );
    }
    fputs( line, stdout );

    if ( gc->gc_newentryloccount > 0 ) {
	for ( i = 0; i < gc->gc_newentryloccount; ++i ) {
	    emit_one_loc_dn( gc->gc_newentrylocs[ i ].dsloc_dnsuffix,
		    gc->gc_newentrylocs[i].dsloc_fullname, rootname,
		    base_rdns, base_rdn_count, ( count < 2 ));
	}
    } else {
	/* always include the base dn first */
	emit_one_loc_dn( gc->gc_ldapsearchbase, NULL, rootname, base_rdns,
		base_rdn_count, ( count < 2 ));

	/* XXXmcs it would be nice to do a more intelligent sort here */
	ldap_sort_entries( *ldp, &res, NULL, strcasecmp );

	for ( e = ldap_first_entry( *ldp, res ); e != NULL;
		e = ldap_next_entry( *ldp, e )) {
	    if (( dn = ldap_get_dn( *ldp, e )) == NULL ) {
		dsgw_ldap_error( *ldp, erropts );
		ldap_msgfree( res );
		return;
	    }

	    if ( !dsgw_dn_cmp( dn, gc->gc_ldapsearchbase )) {
		emit_one_loc_dn( dn, NULL, rootname, base_rdns, base_rdn_count,
			( count < 2 ));
	    }
	    free( dn );
	}
    }

    if ( count > 1 ) {
	util_snprintf( line, BIG_LINE, "</SELECT>\n%s\n", suffix );
	fputs( line, stdout );
    }

    if ( res != NULL ) {
	ldap_msgfree( res );
    }
    if ( base_rdns != NULL ) {
	ldap_value_free( base_rdns );
    }
    if ( did_init_ldap ) {
	ldap_unbind( *ldp );
	*ldp = NULL;
    }
}


static void
emit_one_loc_dn( char *dn, char *friendlyname, char *rootname,
    char **base_rdns, int base_rdn_count, int only_one )
{
    char	*escapeddn, **rdns;

    rdns = NULL;
    escapeddn = dsgw_strdup_escaped( dn );

    if ( !only_one ) {
	fputs( "<OPTION", stdout );
    }

    fprintf( stdout, " VALUE=\"%s\">", escapeddn );
    free( escapeddn );

    if ( !only_one ) {
	if ( friendlyname != NULL ) {
	    fputs( friendlyname, stdout );
	} else { /* derive friendly name from the DN */
	    if ( *dn == '\0' ) {
		fputs( ( rootname == NULL ) ?
		    XP_GetClientStr(DBT_theEntireDirectory_)
		    : rootname, stdout );
	    } else if (( rdns = ldap_explode_dn( dn, 1 )) == NULL
		    || rdns[ 0 ] == NULL ) {
		fputs( dn, stdout );
	    } else {
		int	rdncount, i;

		for ( rdncount = 0; rdns[ rdncount ] != NULL; ++rdncount ) {
		    ;
		}

		/* skip trailing components included in basedn... */
		if ( rdncount > base_rdn_count ) {
		    while ( rdncount > 0 && base_rdn_count > 0 ) {
			if ( strcasecmp( base_rdns[ base_rdn_count - 1 ],
				rdns[ rdncount - 1 ] ) != 0 ) {
			    break;
			}
			--rdncount;
			--base_rdn_count;
		    }
		}
		for ( i = 0; i < rdncount; ++i ) {
		    if ( i > 0 ) {
			fputs( ", ", stdout );
		    }
		    fputs( rdns[ i ], stdout );
		}
	    }
	    if ( rdns != NULL ) {
		ldap_value_free( rdns );
	    }
	}
    }

    fputc( '\n', stdout );
}


#ifdef FORTEZZA
/*
 * Enable Fortezza ciphers for use by SSL if appropriate.
 *
 * Under Admin Server, we look at the ns-admin.conf configuration to see if
 * Fortezza ciphers are in use.  The code in the if ( gc->gc_admserv ) block
 * below was lifted from the CL_secinit() function found in the file
 * ns/netsite/lib/libadmin/httpcon.c revision 1.11.2.9.12.8 which was on
 * the tip of admin35_branch on 7-Oct-1997.
 */
void
enable_fortezza_ciphers( void )
{
    char	*var = NULL;

    if ( gc->gc_admserv ) {
	/*
	 * Disable Fortezza ciphers if not using Fortezza,
	 * enable them otherwise. Don't tamper with other ciphers.
	 */
	var = get_nsadm_var("SSL3Ciphers");
	if (var && strstr(var, "-fortezza,") &&
		strstr(var, "-fortezza_rc4_128_sha,") &&
		strstr(var, "-fortezza_null,")) {
	    /* No Fortezza ciphers - disable them! */
	    SSL_EnableGroup(SSL_GroupFortezza, DSFalse);
	} else {
	    /* Fortezza is on - set Fortezza ciphers and reauth card */
	    SSL_EnableGroup(SSL_GroupFortezza, DSTrue);
	    Fortezza_reauth_card();
	}
    } else {
	return;		/* XXXmcs: handle non-admin server case here */
    }
}
#endif /* FORTEZZA */
