/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
#include "dsgw.h"

static void initialize_html_strings();
static void get_request();
static void gen_entrytypes( dsgwnewtype *ntp, dsgwloc *locarray );
static void gen_locations( dsgwloc *locarray, int loccount );

static char *header, *remainder1a, *remainder1b, *remainder1c, *remainder2;


main( argc, argv, env )
    int		argc;
    char	*argv[];
#ifdef DSGW_DEBUG
    char	*env[];
#endif
{
    initialize_html_strings();
    (void)dsgw_init( argc, argv, DSGW_METHOD_GET );
    dsgw_send_header();

#ifdef DSGW_DEBUG
   dsgw_logstringarray( "env", env ); 
#endif

    get_request();

    exit( 0 );
}


static void
get_request()
{
    FILE			*html;
    char			line[ BIG_LINE ];
    int				argc;
    char			**argv;
    struct ldap_searchobj	*sop;
    char			*ufn;

    fputs( header, stdout );

    gen_entrytypes( gc->gc_newentrytypes, gc->gc_newentrylocs );
    gen_locations( gc->gc_newentrylocs, gc->gc_newentryloccount );

    fputs( remainder1a, stdout );
    fputs( remainder1b, stdout );
    fputs( remainder1c, stdout );
    dsgw_emit_helpbutton_qe( "ADDING" );
    fputs( remainder2, stdout );
}


static void
gen_entrytypes( dsgwnewtype *ntp, dsgwloc *locarray )
{
    int		i, j;

    for ( i = 0; ntp != NULL; ++i, ntp = ntp->dsnt_next ) {
	putc( '\n', stdout );
	fprintf( stdout, "// Entry type:  %s\n", ntp->dsnt_fullname );
	fprintf( stdout, "entryTypes[%d] = new Object;\n", i );
	fprintf( stdout, "entryTypes[%d].template = '%s';\n",
		i, ntp->dsnt_template );
	fprintf( stdout, "entryTypes[%d].fullname = '%s';\n",
		i, ntp->dsnt_fullname );
	fprintf( stdout, "entryTypes[%d].rdnattr = '%s';\n",
		i, ntp->dsnt_rdnattr );
	if ( ntp->dsnt_loccount == 0 ) {
	    fprintf( stdout, "entryTypes[%d].locations = null;\n", i );
	} else {
	    fprintf( stdout, "entryTypes[%d].locations = new Object;\n", i );
	    for ( j = 0; j < ntp->dsnt_loccount; ++j ) {
		fprintf( stdout, "entryTypes[%d].locations[%d] = %d;\n",
			i, j, ntp->dsnt_locations[ j ] );
	    }
	}
    }
    putc( '\n', stdout );
}


static void
gen_locations( dsgwloc *locarray, int loccount )
{
    int		i;

    for ( i = 0; i < loccount; ++i ) {
	putc( '\n', stdout );
	fprintf( stdout, "// Location:  %s\n", locarray[ i ].dsloc_fullname );
	fprintf( stdout, "entryLocs[%d] = new Object;\n", i );
	fprintf( stdout, "entryLocs[%d].handle = '%s';\n",
		i, locarray[ i ].dsloc_handle );
	fprintf( stdout, "entryLocs[%d].fullname = '%s';\n",
		i, locarray[ i ].dsloc_fullname );
	fprintf( stdout, "entryLocs[%d].dnsuffix = '%s';\n",
		i, locarray[ i ].dsloc_dnsuffix );
    }
    putc( '\n', stdout );
}


static void
initialize_html_strings()
{
    header =
"<HTML>\n"
"<HEAD>\n"
"<TITLE>Directory Server Gateway: New Entry</TITLE>\n"
"<SCRIPT LANGUAGE=\"JavaScript\">\n"
"<!-- Hide from non-JavaScript-capable browsers\n"
"var emptyFrame = '<HTML><BODY></BODY></HTML>';\n"
"var selectedType = -1;\n"
"var entryTypes = new Object;\n"
"var entryLocs = new Object;\n"
"\n"
;

    remainder1a =
"function checkEntryType()\n"
"{\n"
"    newType = window.newentTypeFrame.document.typeForm.selectType.selectedIndex;\n"
"    if ( newType != selectedType ) {\n"
"	selectedType = newType;\n"
"	generateLocationFrame();\n"
"    }\n"
"}\n"
"\n"
"function locationChanged(index)\n"
"{\n"
"    window.newentLocFrame.document.nameForm.dnsuffix.value =\n"
"	newentLocFrame.document.nameForm.selectLocation.options[index].value;\n"
"}\n"
"\n"
"function haveAuthCookie()\n"
"{\n"
"    return ( document.cookie.indexOf( 'nsdsgwauth=' ) >= 0 &&\n"
"	    document.cookie.indexOf( 'nsdsgwauth=[unauthenticated]' ) < 0 );\n"
"}\n"
"\n"
"function editNewEntry(nameForm)\n"
"{\n"
"    if ( nameForm.entryname.value == '' ) {\n"
"	alert( 'Please provide a name for the new entry.' );\n"
"    } else if ( nameForm.dnsuffix != null &&\n"
"\tnameForm.dnsuffix.value == '' ) {\n"
"	alert( 'Please select a location for the new entry.' );\n"
"    } else {\n"
"	entryName = nameForm.entryname.value;\n"
"	if ( entryName.indexOf( ',' ) >= 0 || entryName.indexOf( ';' ) >= 0 ) {\n"
"	    entryName = '\"' + entryName + '\"';\n"
"	}\n"
"	dn = nameForm.rdntag.value + '=' + entryName;\n"
"	if ( nameForm.dnsuffix != null ) {\n"
"	    dn += ', ' + nameForm.dnsuffix.value;\n"
"	}\n"
"	newurl = '/ds/edit/' + escape( dn ) + '?' +\n"
"		nameForm.entrytype.value + '&ADD';\n"
"	if ( haveAuthCookie()) {\n"
"	    open( newurl, '_blank' );\n"
"	} else {\n"
"	    a = open('', 'AuthWin');\n"
"	    a.opener = window;\n"
"	    a.closewin = false;\n"
"	    window.newentTypeFrame.document.authForm.target = 'AuthWin';\n"
"	    window.newentTypeFrame.document.authForm.authdesturl.value = newurl;\n"
"	    window.newentTypeFrame.document.authForm.submit();\n"
"	}\n"
"    }\n"
"}\n"
"\n"
"\n";

	remainder1b = 
"function generateTypeFrame()\n"
"{\n"
"    typeFrameHTML = '<BODY onLoad=\"parent.checkEntryType()\">' +\n"
"\t'<FORM NAME=\"typeForm\">\\n<FONT SIZE=\"+2\">Step 1.</FONT>  \\n' +\n"
"\t'Select the type of entry to create.\\n' +\n"
"\t'<SELECT NAME=\"selectType\" onChange=\"parent.checkEntryType()\">\\n';\n"
"    for (var i in parent.entryTypes) {\n"
"\ttypeFrameHTML += '<OPTION VALUE=\"' + parent.entryTypes[i].template + '\">';\n"
"\ttypeFrameHTML += parent.entryTypes[i].fullname + '\\n';\n"
"    }\n"
"    typeFrameHTML += '</SELECT></FORM>\\n' +\n"
"\t'<FORM NAME=\"authForm\" METHOD=\"post\" ACTION=\"/ds/auth\">' +\n"
"\t'<INPUT TYPE=\"hidden\" NAME=\"authdesturl\">' +\n"
"\t'</FORM></BODY>\\n';\n"
"    self.newentTypeFrame.location.href= \"javascript:parent.typeFrameHTML\";\n"
"}\n"
"\n"
"function generateLocationFrame()\n"
"{\n"
"    var entType = entryTypes[selectedType];\n"
"\n"
"    locFrameHTML = '<BODY onLoad=\"';\n"
"    if ( entType.locations != null ) {\n"
"\tlocFrameHTML += 'parent.locationChanged(0);'\n"
"    }\n"
"    locFrameHTML += 'document.nameForm.entryname.focus()\">\\n<P>\\n' +\n"
"\t'<FORM NAME=\"nameForm\" onSubmit=\"parent.editNewEntry(this);' +\n"
"\t'return false\">\\n\\n<FONT SIZE=\"+2\">Step 2.</FONT>\\n' +\n"
"\t'Provide a name for the new ' + entType.fullname + '.\\n' +\n"
"\t'<P><NOBR><FONT SIZE=\"+1\">\\n' +\n"
"\tentType.rdnattr + '=' + '</FONT>\\n' +\n"
"\t'<INPUT TYPE=\"text\" NAME=\"entryname\" SIZE=\"40\">\\n' +\n"
"\t'</NOBR>\\n<P>\\n\\n' +\n"
"\t'<INPUT TYPE=\"hidden\" NAME=\"rdntag\" VALUE=\"' +\n"
"\tentType.rdnattr + '\">\\n' +\n"
"\t'<INPUT TYPE=\"hidden\" NAME=\"entrytype\" VALUE=\"' +\n"
"\tentType.template + '\">\\n';\n"
"   if (entType.locations == null) {\n"
"\tlocFrameHTML += '<P>\\n<FONT SIZE=\"+2\">Step 3.</FONT>\\n';\n"
"   } else {\n"
"\tlocFrameHTML += '<FONT SIZE=\"+2\">Step 3.</FONT>\\n' +\n"
"\t'Use the popup below to select a directory location for this '\n"
"\t    + entType.fullname + '.\\n' +\n"
"\t'If you select Other, you will need to enter the complete \\n' +\n"
"\t'distinguished name where this entry should be added.\\n<P>\\n' +\n"
"\t'<SELECT NAME=\"selectLocation\"\\n' +\n"
"\t' onChange=\"parent.locationChanged(this.selectedIndex)\">\\n';\n"
"    for (var i in entType.locations) {\n"
"\tvar index = entType.locations[i];\n"
"\tlocFrameHTML += '<OPTION VALUE=\"' + entryLocs[index].dnsuffix + \n"
"\t'\">\\n' + entryLocs[index].fullname + '\\n';\n"
"    }\n";

	remainder1c = 
"    locFrameHTML += '<OPTION VALUE=\"\">Other\\n</SELECT>\\n' +\n"
"\t'<INPUT TYPE=\"text\" NAME=\"dnsuffix\" SIZE=\"70\">\\n' +\n"
"\t'<P>\\n<FONT SIZE=\"+2\">Step 4.</FONT>\\n';\n"
"    }\n"
"    locFrameHTML +=\n"
"\t'Click Continue.  You will be presented with an editable view\\n' +\n"
"\t' of the entry.  When you are done filling in information, \\n' +\n"
"\t'save the entry.\\n' +\n"
"\t'<P>\\n<CENTER><TABLE BORDER=\"2\" WIDTH=\"100%\">\\n' +\n"
"\t'<TR><TD ALIGN=\"center\" WIDTH=\"50%\">\\n' +\n"
"\t'<INPUT TYPE=\"button\" VALUE=\"Continue\"\\n' +\n"
"\t'    onClick=\"parent.editNewEntry(document.nameForm)\">\\n' +\n"
"\t'<TD ALIGN=\"center\" WIDTH=\"50%\">\\n' +\n"
"\t'"
;

remainder2 =
"\\n</TABLE></CENTER></FORM>\\n</BODY>\\n';\n"
"    self.newentLocFrame.location.href = \"javascript:parent.locFrameHTML\";\n"
"}\n"
"\n"
"function init()\n"
"{\n"
"    generateTypeFrame();\n"
"    checkEntryType();\n"
"}\n"
"\n"
"// end hiding -->\n"
"</SCRIPT>\n"
"\n"
"</HEAD>\n"
"<FRAMESET ROWS=100,70,* BORDER=\"2\" onLoad=\"init()\">\n"
"    <FRAME SRC=\"/dshtml/newentrytitle.html\"\n"
"         NAME=\"newentTitleFrame\" NORESIZE SCROLLING=\"NO\">\n"
"    <FRAME SRC=\"javascript:parent.emptyFrame\"\n"
"         NAME=\"newentTypeFrame\" SCROLLING=\"NO\"> \n"
"    <FRAME SRC=\"javascript:parent.emptyFrame\"\n"
"         NAME=\"newentLocFrame\">\n"
"</FRAMESET>\n"
"\n"
"<NOFRAMES>\n"
"<BODY>\n"
"You must use a client that supports frames to view this document.\n"
"</BODY>\n"
"</NOFRAMES>\n"
"\n"
"</HTML>\n"
;
}
