/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#include "dsgw.h"
#include "dbtdsgw.h"

static void get_request();


main( argc, argv, env )
    int         argc;
    char        *argv[];
#ifdef DSGW_DEBUG
    char        *env[];
#endif
{
    (void)dsgw_init( argc, argv,  DSGW_METHOD_GET );
    dsgw_send_header();



#ifdef DSGW_DEBUG
   dsgw_logstringarray( "env", env );
#endif

    get_request();

    exit( 0 );
}


static void
get_request()
{
    LDAP                *ld;
    char                *dn, *p, *tmplname, *parent;
    unsigned long       options;


    options = DSGW_DISPLAY_OPT_EDITABLE;

    if (( dn = getenv( "PATH_INFO" )) == NULL ) {
        dsgw_error( DSGW_ERR_MISSINGINPUT, NULL, DSGW_ERROPT_EXIT, 0 );
    }

    if ( *dn == '/' ) {
        ++dn;   /* skip past leading slash */
    }

    if (*dn == 'p')
      {
        tmplname = "rename-person";
      }
    else if(*dn == 'o')
      {
        tmplname = "rename-ou";
      }
    else if(*dn == 'g')
      {
        tmplname = "rename-group";
      }

    ++dn;


#ifdef DSGW_DEBUG
    dsgw_log( "get_request: dn: \"%s\", tmplname: \"%s\" "
              "dnattr: \"%s\", dndesc: \"%s\"\n", dn,
            ( tmplname == NULL ) ? "(null)" : tmplname,
            ( dsgw_dnattr == NULL ) ? "(null)" : dsgw_dnattr,
            ( dsgw_dndesc == NULL ) ? "(null)" : dsgw_dndesc );
#endif

    (void)dsgw_init_ldap( &ld, NULL, 0 );

    if (( options & DSGW_DISPLAY_OPT_ADDING ) == 0 ) {
        /*
         * editing an existing entry -- if no DN is provided and we are running
         * under the admin server, try to get DN from admin. server
         */
        if ( *dn == '\0' ) {
            (void)dsgw_get_adm_identity( ld, NULL, &dn, NULL,
                    DSGW_ERROPT_EXIT );
        }

        dsgw_read_entry( ld, dn, NULL, tmplname, options );

    } else {
        dsgwtmplinfo        *tip;
        char                *matched;

        /*
         * new entry -- check to make sure it doesn't exist
         */
        if ( dsgw_ldap_entry_exists( ld, dn, &matched, DSGW_ERROPT_EXIT )) {
            char        **rdns;

            dsgw_html_begin( XP_GetClientStr(DBT_entryAlreadyExists_), 1 );
            fputs( XP_GetClientStr(DBT_anEntryNamed_), stdout );
            rdns = ldap_explode_dn( dn, 1 );
            dsgw_html_href(
                    dsgw_build_urlprefix( dsgw_getvp( DSGW_CGINUM_DOSEARCH )),
                    dn, ( rdns == NULL || rdns[ 0 ] == NULL ) ? dn : rdns[ 0 ],
                    NULL, XP_GetClientStr(DBT_onmouseoverWindowStatusClickHere_) );
            if ( rdns != NULL ) {
                ldap_value_free( rdns );
            }
            fputs( XP_GetClientStr(DBT_alreadyExistsPPleaseChooseAnothe_), stdout );

            fputs( "<FORM>\n<CENTER><TABLE border=2 width=\"100%\"><TR>\n",
                    stdout );
            fputs( "<TD WIDTH=\"50%\" ALIGN=\"center\">\n", stdout );
            fprintf( stdout, "<INPUT TYPE=\"button\" VALUE=\"%s\" "
                "onClick=\"parent.close()\">", XP_GetClientStr(DBT_closeWindow_1) );
            fputs( "<TD WIDTH=\"50%\" ALIGN=\"center\">\n", stdout );
            dsgw_emit_helpbutton( "ENTRYEXISTS" );
            fputs( "\n</TABLE></CENTER></FORM>\n", stdout );
            dsgw_html_end();
        } else if ( !dsgw_is_dnparent( matched, dn ) &&
                !dsgw_dn_cmp( dn, gc->gc_ldapsearchbase )) {
            /*
             * The parent entry does not exist, and the dn being added is not
             * the same as the suffix for which the gateway is configured.
             */
            dsgw_html_begin( XP_GetClientStr(DBT_parentEntryDoesNotExist_), 1 );
            printf( XP_GetClientStr(DBT_youCannotAddAnEntryByTheNamePBSB_),
                    dn );
            parent = dsgw_dn_parent( dn );
            if ( parent == NULL || strlen( parent ) == 0 ) {
                fputs( XP_GetClientStr(DBT_itsParentN_), stdout );
            } else {
                printf( XP_GetClientStr(DBT_anEntryNamedPBSBN_), parent );
                free( parent );
            }
            fputs( "<FORM>\n<CENTER><TABLE border=2 width=\"100%\"><TR>\n",
                    stdout );
            fputs( "<TD WIDTH=\"50%\" ALIGN=\"center\">\n", stdout );
            fprintf( stdout, "<INPUT TYPE=\"button\" VALUE=\"%s\" "
                "onClick=\"parent.close()\">", XP_GetClientStr(DBT_closeWindow_2) );
            fputs( "<TD WIDTH=\"50%\" ALIGN=\"center\">\n", stdout );
            dsgw_emit_helpbutton( "ADD_NOPARENT" );
            fputs( "\n</TABLE></CENTER></FORM>\n", stdout );
            dsgw_html_end();
        } else {
            /*
             * The parent exists, or the user is adding the entry whose DN
             * is the same as the suffix for which the gateway is configured.
             * Display the "add entry" form.
             */

            if ( tmplname == NULL ) {
                dsgw_error( DSGW_ERR_MISSINGINPUT, XP_GetClientStr(DBT_missingTemplate_),
                        DSGW_ERROPT_EXIT, 0 );
            }

            tip = dsgw_display_init( DSGW_TMPLTYPE_DISPLAY, tmplname, options );

            dsgw_display_entry( tip, ld, NULL, dn );
            dsgw_display_done( tip );
        }
    }

    ldap_unbind( ld );
}