/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#include "dsgw.h"

static void initialize_html_strings();
static void get_request();
static void do_searchtype_popup( struct ldap_searchobj *sop );

static char *html1, *html2, *html3;



main( argc, argv, env )
    int		argc;
    char	*argv[];
#ifdef DSGW_DEBUG
    char	*env[];
#endif
{
    int		reqmethod;

    initialize_html_strings();
    reqmethod = dsgw_init( argc, argv, DSGW_METHOD_GET );
    dsgw_send_header();

#ifdef DSGW_DEBUG
   dsgw_logstringarray( "env", env ); 
{
    char	buf[ 1024 ];
    getcwd( buf, sizeof(buf));
    dsgw_log( "cwd: \"%s\"\n", buf );
}
#endif

    if ( reqmethod == DSGW_METHOD_GET ) {
	get_request();
    }

    exit( 0 );
}


static void
get_request()
{
    FILE			*html;
    char			line[ BIG_LINE ];
    int				argc;
    char			**argv;
    struct ldap_searchobj	*sop;
    char			*ufn;

    dsgw_init_searchprefs( &sop );
    printf( html1, dsgw_getvp( DSGW_CGINUM_DOSEARCH ));
    do_searchtype_popup( sop );
    ufn = ldap_dn2ufn( gc->gc_ldapsearchbase );
    printf( html2, ufn );
    dsgw_emit_helpbutton_qe( "SMARTSEARCH" );
    printf( html3, gc->gc_ldapsearchbase, gc->gc_ldapserver,
	    gc->gc_ldapport );
    free( ufn );
}



static void
do_searchtype_popup(
struct ldap_searchobj *sop
)
{
    int i;
    struct ldap_searchobj *so;

    printf( "'<SELECT NAME=\"type\">\\n' +\n" );
    for ( so = ldap_first_searchobj( sop ); so != NULL;
	    so = ldap_next_searchobj( sop, so )) {
	/* Skip any marked "internal-only" */
	if ( LDAP_IS_SEARCHOBJ_OPTION_SET( so, LDAP_SEARCHOBJ_OPT_INTERNAL )) {
	    continue;
	}
	printf( "'<OPTION>%s\\n' +\n", so->so_objtypeprompt );
    }
    printf( "'</SELECT>\\n' +\n" );
}



static void
initialize_html_strings()
{
    html1 =
"<HTML>\n"
"<HEAD>\n"
"<TITLE>\n"
"Directory Server Gateway: Smart Search\n"
"</TITLE>\n"
"<SCRIPT LANGUAGE=\"JavaScript\"\n>\n"
"<!-- Hide from non-JavaScript-capable browsers\n"
"var emptyFrame = '<HTML><BODY></BODY></HTML>';\n"
"\n"
"\n"
"function reppath(path, newstr)\n"
"{\n"
"    var tmpstr;\n"
"    var ind = path.lastIndexOf( \"/\" );\n"
"    if ( ind < 0) {\n"
"        ind = 0;\n"
"    }\n"
"    tmpstr = path.substring(0, ind) + \"/\" + newstr;\n"
"    return tmpstr;\n"
"}\n"
"\n"
"function setHiddenFields(sform)\n"
"{\n"
/* 
 * It would have been nice to detect when the user pressed return without
 * typing anything into the searchstring area, but on Navigator 2.x, the
 * form variable's value seems to get set *after* the onSubmit handler
 * executes, which is unfortunate.
 */
"    if (sform.searchstring.value == \"\") {\n"
"         alert(\"You did not supply a search string\");\n"
"         return false;\n"
"    }\n"
"    sform.action = window.location.protocol + '//' + window.location.host + '%s';\n"
"    return true;\n"
"}\n"
"\n"
"sshtml = '<BODY onLoad=\"document.searchForm.searchstring.focus();\">\\n' +\n"
"'<FORM NAME=\"searchForm\" \\n' +\n"
"'METHOD=POST \\n' +\n"
"'TARGET=\"outputFrame\" onSubmit=\"return parent.setHiddenFields(this)\">\\n' +\n"
"'<TABLE BORDER=0 CELLSPACING=0 CELLPADDING=0 WIDTH=100%%>\\n' +\n"
"'<TR>' +\n"
"'<TH ALIGN=RIGHT>' +\n"
"'Find:' +\n"
"'</TH>\\n' +\n"
"'<TD>\\n' +\n";

    html2 =
"'<TH>within:</TH>\\n<TD>%s</TD>\\n' +\n"
"'</TR>\\n' +\n"
"'<TR>\\n' +\n"
"'<TH ALIGN=RIGHT>Search For: </TH>\\n' + \n"
"'<TD COLSPAN=3><INPUT NAME=\"searchstring\" SIZE=30\">\\n' +\n"
"'<INPUT TYPE=\"SUBMIT\" VALUE=\"Search\" NAME=\"submitButton\">&nbsp"
"&nbsp\\n' +\n"
"'";


    html3 = 
"\\n' +\n"
"'</NOBR></TD>\\n' +\n"
"'</TR>\\n' +\n"
"'</TABLE>' +\n"
"'<INPUT TYPE=HIDDEN NAME=\"mode\" VALUE=\"smart\">\\n' +\n"
"'<INPUT TYPE=HIDDEN NAME=\"type\" VALUE=\"People\">\\n' +\n"
"'<INPUT TYPE=HIDDEN NAME=\"base\" VALUE=\"%s\">\\n' +\n"
"'<INPUT TYPE=HIDDEN NAME=\"ldapserver\" VALUE=\"%s\">\\n' +\n"
"'<INPUT TYPE=HIDDEN NAME=\"ldapport\" VALUE=\"%d\">\\n' +\n"
"'</FORM>' +\n"
"'</BODY>';\n"
"\n"
"\n"
"// End hiding -->\n"
"</SCRIPT>\n"
"</HEAD>\n"
"<FRAMESET ROWS=110,110,* BORDER=2>\n"
"    <FRAME SRC=\"/dshtml/searchtitle.html\" NAME=\"searchTitleFrame\" SCROLLING=\"NO\">\n"
"    <FRAME SRC=\"javascript:parent.sshtml\" NAME=\"searchStringFrame\"\n"
"	NORESIZE SCROLLING=\"NO\">\n"
"    <FRAME SRC=\"javascript:parent.emptyFrame\" NAME=\"outputFrame\">\n"
"</FRAMESET>\n"
"</HTML>\n";

}
