/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/* 
 * tutor.c - Take a qs, and spit out the appropriate tutorial
 *
 * All blame to Mike McCool
 */

#include <stdio.h>
#include <stdlib.h>
#include "dsgw.h"

#define BASE_MAN_DIRECTORY "manual/"
#define BASE_INFO_DIRECTORY "info/"
#define HELP_INDEX_HTML "manual/index.html"
#define MAN_INDEX "../html/manual/index.map"
#define MANUAL_HPATH "dshtml/manual/"

/* Copied from ns/netsite/lib/base/util.c */
static int
util_uri_is_evil(char *t)
{
    register int x;
 
    for(x = 0; t[x]; ++x) {
        if(t[x] == '/') {
            if(t[x+1] == '/')
                return 1;
            if(t[x+1] == '.') {
                switch(t[x+2]) {
                case '.':
                    if((!t[x+3]) || (t[x+3] == '/'))
                        return 1;
                case '/':
                case '\0':
                    return 1;
                }
            }
        }
#ifdef XP_WIN32
        /* On NT, the directory "abc...." is the same as "abc"
         * The only cheap way to catch this globally is to disallow
         * names with the trailing "."s.  Hopefully this is not over
         * restrictive
         */
        if ((t[x] == '.') && ( (t[x+1] == '/') || (t[x+1] == '\0') )) {
            return 1;
        }
#endif
    }
    return 0;
}


FILE *
_open_html_file( char *filename )
{
    FILE *f;

    filename = dsgw_file2htmlpath( "", filename );

    if (!(f = fopen(filename, "r"))) {
	dsgw_error( DSGW_ERR_OPENHTMLFILE, filename, DSGW_ERROPT_EXIT, 0 );
    }

    free( filename );

    return f;
}



/* Had to copy and paste so wouldn't set referer. */
void _my_return_html_file(char *filename, char *base)  {
    char line[BIG_LINE];
    FILE *html = _open_html_file(filename);

    if(base)  {
        char *tmp;
        char *surl=getenv("SERVER_URL");
        char *sn=dsgw_ch_strdup(getenv("SCRIPT_NAME"));
        tmp=strchr(&(sn[1]), '/');
        *tmp='\0';
        fprintf(stdout, "<BASE href=\"%s%s/%s\">\n", surl, sn, base);
    }
    while( fgets(line, BIG_LINE, html))  {
	fputs( line, stdout);
    }
}


int main(int argc, char *argv[])
{
    char *qs = getenv("QUERY_STRING");
    char *html=NULL;
    char *base=NULL;

    dsgw_init( argc, argv, DSGW_METHOD_GET );

    if(qs == NULL)  {
        fprintf(stdout, "Content-type: text/html\n\n");
        _my_return_html_file(BASE_MAN_DIRECTORY HELP_INDEX_HTML, NULL);
        exit(0);
    }

    html = (char *) dsgw_ch_malloc(strlen(qs)+10+10);
    sprintf(html, "%s.html", qs);
    if(util_uri_is_evil(html))  {
        fprintf(stdout, "Content-type: text/html\n\n");
        fputs( "<CENTER><H2>Error</H2></CENTER>\n"
	       "<P>\n"
	       "URL contains dangerous characters.  Cannot display\n"
	       "help text.", stdout );
	exit( 0 );
    }

    if(qs[0]=='!')  {
        qs++;
        if(!strncmp(qs, BASE_INFO_DIRECTORY, strlen(BASE_INFO_DIRECTORY)))  {
            sprintf(html, "%s.html", qs);
        } else if(!strncmp(qs, BASE_MAN_DIRECTORY, strlen(BASE_MAN_DIRECTORY))) {
            if(!strstr(qs, ".html"))  {
                sprintf(html, "%s.htm", qs);
            }  else  {
                sprintf(html, "%s", qs);
            }
            base=qs;
        } 
        else  {
            char line[BIG_LINE];
            FILE *map=NULL;

            html[0]='\0';

            map=fopen(MAN_INDEX, "r");
            if(!map) 
                goto ohwell;
            while(fgets(line, BIG_LINE, map))  {
                if(line[0]==';')  
                    continue;
                else if(isspace(line[0]))
                    continue;
                else  {
                    /* parse out the line */
                    register char *head=NULL, *tail=NULL;
                    int found;

                    head=&(line[0]);
                    tail=head;
                    found=0;
                    while(*tail)  {
                        if(isspace(*tail) || *tail=='=')  {
                            *tail='\0';
                           found=1;
                            /* get rid of extra stuff at the end */
                            tail++;
                            while(*tail++)  {
                                if((!isspace(*tail)) && (*tail!='='))
                                    break;
                            }
                            break;
                        }
                        tail++;
                    }
                    if(!found) continue;

                    /* script name is in head */
                    if(strncasecmp(head, qs, strlen(qs)))  {
                        continue;
                    }
                    /* match found.  get the actual file name */
                    head=tail;
/* No longer whitespace at end of line.  Now is whitespace in link. */
/* Step on CRs and LFs. */
                    while(*tail)  {
                        if((*tail=='\r') || (*tail=='\n') || (*tail==';'))  {
                            *tail='\0';
                            break;
                        }
                        tail++;
                    }
#if 0
                    while(*(--tail))  {
                        if(isspace(*tail)) *tail='\0';
                        else break;
                    }
#endif
                    /* assumedly, head should now have the proper HTML file
                     * from the manual inside. redirect the client 'cause
                     * there's no other way to get them to jump to the 
                     * right place. */ 
                    fprintf(stdout, "Location: %s/%s%s\n\n", 
                                    getenv("SERVER_URL"), MANUAL_HPATH, head);
                    fclose(map);
                    exit(0);
                }
            }
            fclose(map);

ohwell:
            if(!html[0])
                sprintf(html, "%s%s.html", BASE_MAN_DIRECTORY, qs);
        }
        fprintf(stdout, "Content-type: text/html\n\n");
        _my_return_html_file(html, base);
    }  else  {
        fprintf(stdout, "Content-type: text/html\n\n");
        fprintf(stdout, "<TITLE>Info</TITLE>\n");
        fprintf(stdout, "\n"); 
        fprintf(stdout, "<frameset rows=57,*>\n");
        fprintf(stdout, "<frame src=\"tutor?!info/infonav\" scrolling=no "
                        "marginwidth=0 marginheight=0 "
                        "name=\"infobuttons\">\n");
        fprintf(stdout, "<frame src=\"tutor?!%s\" "
                        "name=\"infotopic\">\n", qs);
        fprintf(stdout, "</frameset>\n");
    }
    return 1;
}

