/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#include "dsgw.h"

char *get_auth_cookie( char *cookie );
void generate_message( int type );

#define	CKEXP_SUCCESS	1
#define	CKEXP_FAILURE	2

main( int argc, char **argv )
{
    int		reqmethod;
    char	*expck;
    char	*cookie;
    char	*authck;
    int		rc;

    reqmethod = dsgw_init( argc, argv,  DSGW_METHOD_GET );

    authck = dsgw_get_auth_cookie();
    if ( authck == NULL ) {
	/* No cookie.  Generate an informational message. */
	generate_message( CKEXP_SUCCESS );
	free( authck );
	exit( 0 );
    }
    
    /* Remove the cookie from the cookie database */
    rc = dsgw_delcookie( authck );

    /* Generate a cookie header with the cookie set to [unauthenticated] */
    expck = dsgw_ch_malloc( strlen( DSGW_CKHDR ) + strlen( DSGW_AUTHCKNAME ) +
	    strlen( DSGW_UNAUTHSTR ) + 2 );
    sprintf( expck, "%s%s=%s; path=/", DSGW_CKHDR, DSGW_AUTHCKNAME, DSGW_UNAUTHSTR );
    dsgw_add_header( expck );
    generate_message( CKEXP_SUCCESS );
    free( authck );
    free( expck );
    exit( 0 );
}



/*
 * It's quite likely that there will be more than one cookie in the
 * Cookie: header.  See if we've got an authentication cookie, and if
 * so, parse it out and return a pointer to it.  If no auth cookie
 * is present, return NULL.
 */
char *
get_auth_cookie( char *cookie )
{
    char *p, *e;

    if ( cookie == NULL ) {
	return NULL;
    }

    if (( p = strstr( cookie, DSGW_AUTHCKNAME )) == NULL ) {
	return NULL;
    }
    
    if (( e = strchr( p, ';' )) != NULL ) {
	*e = '\0';
    }

    return p;
}
	


void
generate_message( int type )
{
    dsgw_send_header();
    fputs( "<HTML>\n"
	"<HEAD>\n"
	"<TITLE>", stdout );
    if ( type == CKEXP_SUCCESS ) {
	fputs( "Success", stdout );
    } else if ( type == CKEXP_FAILURE ) {
	fputs( "Error", stdout );
    }
    printf( "</TITLE>\n</HEAD>\n" );
    printf( "<BODY>\n" );

    fputs( "<CENTER>\n"
	"<FONT SIZE=+2>\n"
	"Success"
	"</FONT>\n"
	"</CENTER>\n"
	"<P>\n"
	"Your authentication credentials have been "
	"destroyed.  You are no longer authenticated to the \n"
	"directory.\n", 
	stdout );

    if ( type != CKEXP_SUCCESS ) {
	/*
	 * Something went wrong, so generate some JavaScript to
	 * discard the cookie.
	 */
	fputs( "<SCRIPT LANGUAGE=\"JavaScript\">\n", stdout );
	printf( "document.cookie = '%s=%s; path=/';\n", DSGW_AUTHCKNAME,
		DSGW_UNAUTHSTR );
	fputs( "</SCRIPT>\n", stdout );
    }
    fputs( "<FORM>\n"
	"<TABLE BORDER=2 WIDTH=100%%>\n"
	"<TR>\n"
	"<TD ALIGN=CENTER WIDTH=50%%>\n"
	"<INPUT TYPE=BUTTON VALUE=\"Go Back\" "
	"onClick=\"window.location.href='/ds/auth';\">\n"
	"<TD ALIGN=CENTER WIDTH=50%%>\n", stdout );
    dsgw_emit_helpbutton( "UNAUTH" );
    fputs( "</TABLE>\n"
	"</BODY></HTML>\n", stdout );
}
