# BEGIN COPYRIGHT BLOCK
# Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
# Copyright (C) 2005 Red Hat, Inc.
# All rights reserved.
# END COPYRIGHT BLOCK
#
# nsldap.mak: Gnu make file for ldap server and related tools
# 

INSTROOT= $(OBJDIR)/ldap

ifeq ($(ARCH), WINNT)
LDIR = $(OBJDIR)/ldap/lib
else
LDIR = $(LDAPSRC)/libraries
endif

ifeq ($(BUILD_MODULE), HTTP_ADMIN)
SECGLUEOBJ=$(MCOM_ROOT)/netsite/built/$(NS_BUILD_FLAVOR)/admin-lib/secglue.o
SDKLDIR = $(LDAP_LIBPATH)
SDKHDIR = $(LDAP_INCLUDE)
else
SECGLUEOBJ=$(MCOM_ROOT)/netsite/built/$(NS_BUILD_FLAVOR)/httpd-lib/secglue.o
# XXXggood need to pick up the /share/builds versions of the shared libs
# because ones we build here don't appear to be compatible with existing
# shared libs, which are used by admin server.
#SDKROOT = /share/builds/components/ldapsdk/19961107-bad/$(NC_BUILD_FLAVOR)
SDKROOT = $(LDAPSRC)
SDKLDIR = $(SDKROOT)/libraries
SDKHDIR = $(SDKROOT)/include
endif

ALIBDIR = $(OBJDIR)/ldap/admin/lib
ADMININCLUDEDIR = $(MCOM_ROOT)/netsite/include

ifeq ($(ARCH), HPUX)
ADMSONAME=libnsadmin.sl
ADMSOLINK=-L$(OBJDIR) -lnsadmin
else
ifeq ($(ARCH), AIX)
ADMSONAME=libnsadmin.so
ADMSOLINK=-L../.. -L../../../../lib -L$(OBJDIR) -lnsadmin
else
ADMSONAME=ns-admin.so
ADMSOLINK=../../ns-admin.so
endif
endif

ifeq ($(ARCH), WINNT)
LIBEXT=lib
else
LIBEXT=a
endif

HTMLDEFS=-DPRODUCT_NAME=$(PRODUCT) -D$(ARCH) -DARCH=$(PRETTY_ARCH)

LDAP_LIBDIR = $(LDIR)
LIBDIR = $(LDIR)

ifeq ($(ARCH), WINNT)
LDAPALIBS = $(ALIBDIR)/libds_admin.lib
# MLM 
# LDAPLIBS = $(LDAP_LIBDIR)/libldap.$(LIBEXT) $(LDAP_LIBDIR)/liblber.$(LIBEXT) \
# 	$(LDAP_LIBDIR)/libutil.$(LIBEXT) $(LDAP_LIBDIR)/liblthread.$(LIBEXT) \
# 	$(LDAP_LIBDIR)/libldif.$(LIBEXT) $(LDAP_LIBDIR)/libldbm.$(LIBEXT) \
# 	$(LDAP_LIBDIR)/libavl.$(LIBEXT) $(LDAP_LIBDIR)/libdb.$(LIBEXT) \
# 	$(LDAP_LIBDIR)/backldbm.$(LIBEXT) $(LDAP_LIBDIR)/libslapd.$(LIBEXT) 
LDAPLIBS = $(LIBLDAP) $(addsuffix .$(LIB_SUFFIX), $(addprefix $(COMMON_OBJDIR)/lib/lib, $(LIBLDAPU)))
else
LDAPALIBS = $(ALIBDIR)/libds_admin.a
LDAPLIBS  = $(LIBDIR)/libldif.a $(LIBDIR)/libldap.a $(LIBDIR)/liblber.a
endif

LDB_HDIR = $(LDAPSRC)/libraries/berkeley_db/PORT/include
VERSIONFILE = $(LDAPSRC)/build/version

LDAP_REFERRALS=-DLDAP_REFERRALS
SLAPD_BACKENDS=-DLDAP_LDBM
LDBMBACKEND=-DLDBM_USE_DBBTREE
SLAPD_PASSWD_HASH=-DSLAPD_PASSWD_SHA1
LDAP_DEBUG=-DLDAP_DEBUG
NEEDPROTOS=-DNEEDPROTOS
WINSOCK=-DWINSOCK
USE_LOCKF=-DUSE_LOCKF

# uncomment this line to use soundex for approximate matches in slapd.
# the default is to use the metaphone algorithm.
#PHONETIC=-DSOUNDEX

# DEFS are included in CFLAGS
DEFS	= $(PLATFORMCFLAGS) $(LDAP_DEBUG) $(KERBEROS) $(AFSKERBEROS) \
		$(UOFM) $(UOFA) $(NO_USERINTERFACE) $(CLDAP) $(NO_CACHE) \
		$(LDAP_REFERRALS) $(LDAP_DNS) $(STR_TRANSLATION) \
		$(LIBLDAP_CHARSETS) $(LIBLDAP_DEF_CHARSET) \
		$(SLAPD_BACKENDS) $(LDBMBACKEND) $(LDBMINCLUDE) $(PHONETIC) \
		$(SLAPD_PASSWD_HASH)

ifeq ($(ARCH), WINNT)
DEFS += $(NEEDPROTOS)
endif

ifeq ($(ARCH), WINNT)
OFFLAG=/Fo
else 
OFFLAG=-o 
endif

ifneq ($(ARCH), WINNT)
EXTRACFLAGS=-g
endif

EXTRALDFLAGS=-g

#   DEFS += $(USE_LOCKF)

# ACFLAGS are added to CFLAGS but not passed to mkdep, lint, etc
ACFLAGS		= $(EXTRACFLAGS) $(UNPROTOCFLAGS)

# ALDFLAGS are always placed near the beginning of all linker (cc -o) commands
ALDFLAGS	= $(EXTRALDFLAGS) $(PLATFORMLDFLAGS)

# ALIBS are always placed at the end of all linker (cc -o) commands
ALIBS		= $(PLATFORMLIBS)

HDIR = $(LDAPSRC)/include

ifeq ($(ARCH), WINNT)
ifeq ($(DEBUG), full)
DSLDDEBUG=/debug
else
ifeq ($(DEBUG), purify)
DSLDDEBUG=/debug
endif
endif
PDBOPT=NONE
endif

#############################################################################
## If you are NOT using SSL, you can skip this section.                    ##
#############################################################################
#
# Otherwise, to enable SSL authentication, uncomment the SSL line.
# Also uncomment and change the various SSL* lines to point to where the
# SSL libraries and include files are available at your site.
#
ifeq ($(SECURITY), export)
SECURITY_EXTN=export
endif
ifeq ($(SECURITY), domestic)
SECURITY_EXTN=us
endif

SSL = -DNET_SSL -DUSE_NSPR_MT
#EXTRASSLLIBS = $(NS_DIST)/lib/xp/$(NC_BUILD_FLAVOR)/allxpstr.o \
#	$(NS_DIST)/lib/xp/$(NC_BUILD_FLAVOR)/xp_str.o \
#	$(NS_DIST)/lib/xp/$(NC_BUILD_FLAVOR)/xp_file.o \
#	$(NS_DIST)/lib/xp/$(NC_BUILD_FLAVOR)/xp_reg.o \
#	$(NS_DIST)/nspr/src/$(NC_BUILD_FLAVOR)/longlong.o \
#	$(NS_DIST)/nspr/src/$(NC_BUILD_FLAVOR)/prprf.o
#EXTRASSLLIBS = -lxp -lnspr
EXTRASSLLIBS = $(ADMSOLINK) $(LIBARES) 
SSLINCLUDE = -I$(MCOM_ROOT)/include \
	-I$(MCOM_ROOT)/dist/$(NC_BUILD_FLAVOR)/include \
	-I$(MCOM_ROOT)/dist/$(NC_BUILD_FLAVOR)/include/nspr20/pr

ifeq ($(ARCH), WINNT)
SSLLIBS = $(EXTRASSLLIBS)
ifdef FORTEZZA
SSLLIBS += $(MCOM_ROOT)/dist/$(NC_BUILD_FLAVOR)/lib/tssp32.dll
endif
else
SSLLIBFLAG = -L$(MCOM_ROOT)/dist/$(NC_BUILD_FLAVOR)/lib
ifeq ($(ARCH), IRIX)
SSLLIBS = $(SSLLIBFLAG) $(EXTRASSLLIBS) -lsec-$(SECURITY_EXTN) $(EXTRASSLLIBS)
else
SSLLIBS = $(SSLLIBFLAG) $(EXTRASSLLIBS) -lsec-$(SECURITY_EXTN)
endif
ifdef FORTEZZA
SSLLIBS += -lci
endif
endif

STATIC_SECDEPS= $(addsuffix .$(LIB_SUFFIX), \
                           $(addprefix $(COMMON_OBJDIR)/lib/lib, \
				$(LIBADMIN) $(BASE) $(FRAME) \
				$(LIBACCESS) $(CRYPT))) \
               $(LIBSEC) $(LIBNSPR)

# If you are certain that an executable will not be using libsec, include
# the following early in the link command.  secglue.o includes "do nothing"
# shims for most libsec functions.  We do this to reduce our size.
ifeq ($(ARCH), WINNT)
# MLM
# SECGLUE= ns-admin.lib
SECGLUE = $(addsuffix .$(LIB_SUFFIX), $(OBJDIR)/$(ADMINDLL_NAME)) $(OSDEPLIBS) $(LIBNT)

NOSSLLIBS = $(LDIR)/libldap.lib \
		$(SSLLIBS) $(ALIBS) \
		$(LDIR)/liblthread.lib
else
SECGLUE= $(SECGLUEOBJS) $(SECGLUEOBJ) $(ADMSOLINK) $(LIBARES) 
ifeq ($(ARCH), AIX)
NOSSLLIBS = -lldap30$(LDAP_DLL_PRESUF) -llcache30$(LDAP_DLL_PRESUF) $(SECGLUE) $(ALIBS)
else
NOSSLLIBS = -lldap30 -llcache30 $(SECGLUE) $(ALIBS)
endif
endif

ifeq ($(BUILD_DLL), yes)
STATIC_SECDEPS= $(addsuffix .$(LIB_SUFFIX), \
                           $(addprefix $(COMMON_OBJDIR)/lib/lib, \
				$(LIBADMIN) $(FRAME) $(LIBACCESS) $(CRYPT))) \
               $(LIBSEC) $(LIBNSPR)

DYNAMIC_DEPLIBS=$(LDAPLIBS) 
DYNAMIC_DEPLINK=$(DYNAMIC_DEPLIBS)
else
STATIC_DEPLIBS=$(addsuffix .$(LIB_SUFFIX), \
               $(addprefix $(OBJDIR)/lib/lib, \
                        $(LIBADMIN) $(FRAME) $(LIBACCESS) $(CRYPT))) \
               $(LIBDBM) $(LIBXP) $(LIBNSPR)

STATIC_SECDEPS=$(addsuffix .$(LIB_SUFFIX), \
               $(addprefix $(OBJDIR)/lib/lib, \
			$(LIBADMIN) $(FRAME) $(LIBACCESS) $(CRYPT))) \
               $(LIBSEC) $(LIBNSPR)

DYNAMIC_DEPLIBS=$(LDAPLIBS)
DYNAMIC_DEPLINK=$(LDAPLIBS)
endif

ifeq ($(ARCH), WINNT)
LIBDBM_LIB = $(MCOM_ROOT)/dist/$(NSOBJDIR_NAME)/lib/libdbm.lib
XP_OBJ = $(MCOM_ROOT)/lib/xp/$(NSOBJDIR_NAME)
XP_OBJS = $(XP_OBJ)/allxpstr.o $(XP_OBJ)/xp_str.o $(XP_OBJ)/xp_file.o \
		$(XP_OBJ)/xp_reg.o 
endif

ifndef DEPLIBS
DEPLIBS = $(DYNAMIC_DEPLIBS)
DEPLINK = $(DYNAMIC_DEPLINK)
ifeq ($(ARCH), WINNT)
SECDEPS = $(DEPLIBS) $(LIBDBM_LIB) $(SECGLUE) $(XP_OBJS)
else
SECDEPS = $(STATIC_SECDEPS)
endif
SECLINK = $(SECDEPS)
endif

HTMLDEFS=-DPRODUCT_NAME=$(PRODUCT) -D$(ARCH) -DARCH=$(PRETTY_ARCH)

# default definitions for utilities

ifneq ($(ARCH), WINNT)
SHELL	= /bin/sh
endif

RANLIB	= ranlib
AR	= ar
RM	= rm -f
MV	= mv -f
CP	= cp
CHMOD	= chmod
CAT	= cat

ifneq ($(ARCH), WINNT)
LN	= ln -s
HARDLN	= ln
endif

TAIL	= tail
SED	= sed
LINT	= lint
5LINT	= lint
MKDIR	= mkdir

ifneq ($(ARCH), WINNT)
INSTALL	= install
INSTALLFLAGS = -c
BASENAME= basename
DIRNAME	= dirname
else
INSTALL = cp
endif

MKDEP	= $(LDAPSRC)/build/mkdep -s -f Make-template
PWD	= pwd
DATE	= date
HOSTNAME= hostname


ifeq ($(ARCH), WINNT)
NS_BUILD_FLAVOR=$(ARCH)-$(SECURITY)-$(DEBUG)-$(B_FORTEZZA)-$(DIR)
NC_BUILD_FLAVOR=$(NSCONFIG)$(NSOBJDIR_TAG).OBJ
endif # WINNT

ifeq ($(ARCH), SOLARIS)
#
# LDAP SunOS5 
#

#
# add any platform-specific overrides below here
#
PLATFORM=sunos5

# ranlib not needed under SunOS5
RANLIB	= ""

# be explicit about which CC to use
CC=cc

# give full path to hostname since it may not be in user's path
HOSTNAME=/usr/ucb/hostname

# don't count on /usr/ucb/install being present or first in path
INSTALL=$(LDAPSRC)/build/install.sh

NS_BUILD_FLAVOR=$(ARCH)-$(SECURITY)-$(DEBUG)-$(B_FORTEZZA)-$(DIR)
NC_BUILD_FLAVOR=$(NSCONFIG)$(NSOBJDIR_TAG).OBJ
PLAT_ADMCFLAGS=	-DSVR4 -DSOLARIS
PLAT_ADMLIBS= 
PLATFORMCFLAGS=	-D$(PLATFORM) -D_REENTRANT -DSVR4
PLATFORMLIBS= -lresolv -lsocket -lnsl -lgen -ldl -lthread -lposix4
THREADS= -DTHREAD_SUNOS5_LWP
THREADSLIB=-lthread
endif # SOLARIS


ifeq ($(ARCH), SOLARISx86)
#
# LDAP for SOLARISx86
 
#
# add any platform-specific overrides below here
#
PLATFORM=sunos5
 
# ranlib not needed under SunOS5
RANLIB  = ""
 
# be explicit about which CC to use
CC=cc
 
# give full path to hostname since it may not be in user's path
HOSTNAME=/usr/ucb/hostname
 
# don't count on /usr/ucb/install being present or first in path
INSTALL=$(LDAPSRC)/build/install.sh
 
NS_BUILD_FLAVOR=$(ARCH)-$(SECURITY)-$(DEBUG)-$(B_FORTEZZA)-$(DIR)
NC_BUILD_FLAVOR=$(NSCONFIG)$(NSOBJDIR_TAG).OBJ
#NS_BUILD_FLAVOR=SOLARIS-export-slapd
#NC_BUILD_FLAVOR=SunOS5.5_DBG.OBJ
PLAT_ADMCFLAGS= -DSVR4 -DSOLARIS
PLAT_ADMLIBS=
PLATFORMCFLAGS= -D$(PLATFORM) -D_REENTRANT -DSVR4
PLATFORMLIBS= -lresolv -lsocket -lnsl -lgen -ldl -lthread -lposix4
THREADS= -DTHREAD_SUNOS5_LWP
THREADSLIB=-lthread
endif # SOLARISx86



ifeq ($(ARCH), SUNOS4)
#
# LDAP SunOS 
#

#
# add any platform-specific overrides below here
#
5LINT	= /usr/5bin/lint

PLATFORMCFLAGS= -Dsunos4
THREADS= -DTHREAD_SUNOS4_LWP
THREADSLIB=-llwp

#
# the SunOS 4 cc compiler doesn't understand function prototypes, so we
# need the unproto preprocessor
#
NEEDUNPROTO=yes
UNPROTOCFLAGS=-Qpath $(LDAPSRC)/build/unproto
endif # SUNOS4

ifeq ($(ARCH), IRIX)
#
# LDAP IRIX 
#

#
# add any platform-specific overrides below here
#

NS_BUILD_FLAVOR=$(ARCH)-$(SECURITY)-$(DEBUG)-$(B_FORTEZZA)$(BUILD_PTHREADS)-$(DIR)
NC_BUILD_FLAVOR=IRIX6.2$(NSOBJDIR_TAG).OBJ
#NS_BUILD_FLAVOR=IRIX-export-slapd
#NC_BUILD_FLAVOR=IRIX5.3_OPT.OBJ
PLAT_ADMCFLAGS=	-DSVR4 -DIRIX
PLAT_ADMLIBS= 

PLATFORM=irix
# ranlib not needed under IRIX
RANLIB	= ""

# be explicit about which CC to use
CC=cc

# give full path to hostname since it may not be in user's path
HOSTNAME=/usr/bsd/hostname

# don't count on a BSD install being present or first in path
INSTALL=$(LDAPSRC)/build/install.sh

PLATFORMCFLAGS=-DUSE_WAITPID -D_SGI_MP_SOURCE -D$(PLATFORM)
PLATFORMLIBS=
THREADS= -DTHREAD_SGI_SPROC
THREADSLIB=

endif # IRIX

ifeq ($(ARCH), OSF1)

#
# LDAP OSF1 
#

#
# add any platform-specific overrides below here
#
NS_BUILD_FLAVOR=$(ARCH)-$(SECURITY)-$(DEBUG)-$(B_FORTEZZA)-$(DIR)
#NC_BUILD_FLAVOR=OSF1V3.2$(NSOBJDIR_TAG).OBJ
NC_BUILD_FLAVOR=$(NSCONFIG)$(NSOBJDIR_TAG).OBJ
#NS_BUILD_FLAVOR=OSF1-export-slapd
#NC_BUILD_FLAVOR=OSF1V3.2_OPT.OBJ


PLATFORM=OSF1

# Even though prototypes are supported by the compiler, OSF's CC doesn't
# seem to define __STDC__ so we explicitly defined NEEDPROTOS here.
PLATFORMCFLAGS= -D$(PLATFORM) -DNEEDPROTOS
PLATFORMLIBS=
THREADS= -DTHREAD_DCE_PTHREADS
THREADSLIB= -threads

# the BSD-like install under OSF/1 is called installbsd
INSTALL=installbsd

endif # OSF1

ifeq ($(ARCH), AIX)

#
# LDAP AIX 
#
NS_BUILD_FLAVOR=$(ARCH)-$(SECURITY)-$(DEBUG)-$(B_FORTEZZA)-$(DIR)
NC_BUILD_FLAVOR=$(NSCONFIG)$(NSOBJDIR_TAG).OBJ
PLAT_ADMCFLAGS= -DAIX 
PLAT_ADMLIBS=

#
# add any platform-specific overrides below here
#

PLATFORM=aix

# ranlib for aix 
RANLIB=ranlib

CC=xlC_r

# install with BSD semantics
INSTALL=$(LDAPSRC)/build/install.sh

PLATFORMCFLAGS= -qarch=com -DAIX -DAIXV3 -DAIXV4 -DSYSV -D_PR_NTHREAD -D_PR_USECPU

EXTRA_LIBS=-ldl -lm

PLATFORMLIBS= $(EXTRA_LIBS)
THREADS= -DTHREAD_AIX_PTHREADS

endif # AIX

ifeq ($(ARCH), HPUX)
#
# LDAP HP-UX 
#

#
# add any platform-specific overrides below here
#
NS_BUILD_FLAVOR=$(ARCH)-$(SECURITY)-$(DEBUG)-$(B_FORTEZZA)-$(DIR)
NC_BUILD_FLAVOR=$(NSCONFIG)$(NSOBJDIR_TAG).OBJ

CC=cc
PLATFORM=hpux

# ranlib not needed under HP-UX
RANLIB  = ""

# install under HP-UX is not like on BSD systems, so we use our own script
INSTALL=$(LDAPSRC)/build/install.sh

# we need to link a separate library to get ndbm routines under HP/UX
LDBMLIB=-lndbm

# we need to link in the V3 library to get sigset()
PLATFORMLIBS= -lV3

PLATFORMCFLAGS= -Dhpux -D$(PLATFORM) -D_REENTRANT -Ae +DAportable +DS1.1 +ESlit
# PVO
#THREADS=-DTHREAD_HP_DCE_PTHREADS
#THREADSLIB=-lcma
THREADSLIB=-lpthread

endif # HPUX

ifeq ($(ARCH), UNIXWARE)
#
# LDAP SVR4 standard cc Make-platform file
# Uses the std SVR4 stuff whenever possible.  
# Some references to the BSD compatibility required.
#

#
# add any platform-specific overrides below here
#
NS_BUILD_FLAVOR=$(ARCH)-$(SECURITY)-$(DEBUG)-$(B_FORTEZZA)-$(DIR)
NC_BUILD_FLAVOR=$(NSCONFIG)$(NSOBJDIR_TAG).OBJ

# compiler to use, e.g. CC=cc or CC=gcc
ifndef CC
CC = cc
endif

# give full path to hostname since it may not be in user's path
HOSTNAME=/usr/ucb/hostname

# don't count on /usr/ucb/install being present or first in path
INSTALL=$(LDAPSRC)/build/install.sh

PLAT_ADMCFLAGS=	 -DUNIXWARE -DSVR4 -DSYSV

# flags added to every compile
PLATFORMCFLAGS=	-DUNIXWARE -DSYSV -DSVR4

# flags added to every link
PLATFORMLDFLAGS = 

# extra libraries needed (added to the end of all link commands)
PLATFORMLIBS =  -lsocket -lnsl -lresolv -lgen

# Flags required to cause linker to create a shared library
DYNAFLAGS= -G

# Extra linker options needed then creating shared libraries
DYNALIBS= -ldl

# Filename extension for shared libraries
DYNAEXT=so

# ndbm library, needed if not in libc (e.g. LDBMLIB=-lndbm)
LDBMLIB = -L/usr/ucblib -ldbm

# BSD-like install command; if necessary, you can use a script
INSTALL = /usr/ucb/install

# command to convert libraries for efficient random access;
RANLIB = ""

# other commands  - see the file build/Make-append for a list
endif

ifeq ($(ARCH), NCR)
#
# LDAP SVR4 standard cc Make-platform file
# Uses the std SVR4 stuff whenever possible.  
# Some references to the BSD compatibility required.
#

#
# add any platform-specific overrides below here
#
NS_BUILD_FLAVOR=$(ARCH)-$(SECURITY)-$(DEBUG)-$(B_FORTEZZA)-$(DIR)
NC_BUILD_FLAVOR=$(NSCONFIG)$(NSOBJDIR_TAG).OBJ

# compiler to use, e.g. CC=cc or CC=gcc
ifndef CC
CC = cc
endif

# give full path to hostname since it may not be in user's path
HOSTNAME=/usr/ucb/hostname

# don't count on /usr/ucb/install being present or first in path
INSTALL=$(LDAPSRC)/build/install.sh

PLAT_ADMCFLAGS=	 -DNCR -DSVR4 -DSYSV

# flags added to every compile
PLATFORMCFLAGS=	-DNCR -DSYSV -DSVR4

# flags added to every link
PLATFORMLDFLAGS = 

# extra libraries needed (added to the end of all link commands)
PLATFORMLIBS =  -lsocket -lnsl -lresolv -lgen

# Flags required to cause linker to create a shared library
DYNAFLAGS= -G

# Extra linker options needed then creating shared libraries
DYNALIBS= -ldl

# Filename extension for shared libraries
DYNAEXT=so

# ndbm library, needed if not in libc (e.g. LDBMLIB=-lndbm)
LDBMLIB = -L/usr/ucblib -ldbm

# BSD-like install command; if necessary, you can use a script
INSTALL = /usr/ucb/install

# command to convert libraries for efficient random access;
RANLIB = ""

# other commands  - see the file build/Make-append for a list
endif

ifeq ($(ARCH), SCO)
#
# add any platform-specific overrides below here
#
NS_BUILD_FLAVOR=$(ARCH)-$(SECURITY)-$(DEBUG)-$(B_FORTEZZA)-$(DIR)
NC_BUILD_FLAVOR=$(NSCONFIG)$(NSOBJDIR_TAG).OBJ
 
# compiler to use, e.g. CC=cc or CC=gcc
ifndef CC
CC = cc -b elf -KPIC -DPRFSTREAMS_BROKEN
endif
 
# give full path to hostname since it may not be in user's path
HOSTNAME=/usr/bin/hostname
 
# don't count on /usr/ucb/install being present or first in path
INSTALL=$(LDAPSRC)/build/install.sh
 
PLAT_ADMCFLAGS=-DSCO -DSYSV
 
# flags added to every compile
PLATFORMCFLAGS=-DSYSV
 
# flags added to every link
PLATFORMLDFLAGS =
 
# extra libraries needed (added to the end of all link commands)
PLATFORMLIBS =  -lsocket -lnsl
 
# Flags required to cause linker to create a shared library
DYNAFLAGS= -G
 
# Extra linker options needed then creating shared libraries
DYNALIBS= -ldl
 
# Filename extension for shared libraries
DYNAEXT=so
 
# ndbm library, needed if not in libc (e.g. LDBMLIB=-lndbm)
LDBMLIB = -lndbm
 
# BSD-like install command; if necessary, you can use a script
INSTALL = $(LDAPSRC)/build/install.sh
 
# command to convert libraries for efficient random access;
RANLIB = ""
 
# other commands  - see the file build/Make-append for a list
endif
