/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * dll.c: Abstracts loading of dynamic libraries
 * 
 * Rob McCool
 */


#include "netsite.h"
#include "systems.h"
#include "dll.h"

#ifdef DLL_CAPABLE

#if !defined(DLL_DLOPEN)
#include "file.h"
#endif

#if defined(USE_NSPR_DLL)
#include <nspr/prlink.h>

#elif defined(DLL_DLOPEN)
#include <dlfcn.h>

#elif defined(DLL_HPSHL)
#include <dl.h>
#endif

#ifdef DLL_WIN32
#include "file.h"
#include <sys/stat.h>
#endif

#ifdef HPUX
#include <dl.h>
#endif

/*
 * NSPR is broken for shared libs.
 * It doesn't allow dynamic managment.
 * Once loaded, they are never unloaded.
 */
NSAPI_PUBLIC DLHANDLE dll_open(const char *fn)
{
#if defined(USE_NSPR_DLL)
    if(!(PR_LoadLibrary(fn)))
        return NULL;
    return (void *) 1;

#elif defined(DLL_DLOPEN)
#ifdef	AIX
    return (void *) dlopen((char *)fn, DLL_DLOPEN_FLAGS);
#else
    return (void *) dlopen(fn, DLL_DLOPEN_FLAGS);
#endif

#elif defined(DLL_HPSHL)
    return (void *) shl_load(fn, BIND_IMMEDIATE | BIND_VERBOSE, NULL);

#elif defined(DLL_WIN32)
    struct stat finfo;
    char d2[MAX_PATH];

    file_unix2local((char *)fn, d2);
    /* LoadLibrary returns a bad pointer instead of NULL on error */
    if(stat(d2, &finfo) == -1)
        return NULL;
    return (void *) LoadLibrary(d2);
#endif
}



NSAPI_PUBLIC void *dll_findsym(DLHANDLE dlp, const char *name)
{
#if defined(USE_NSPR_DLL)
    return PR_FindSymbol(name);

#elif defined(DLL_DLOPEN)
#ifdef	AIX
    return (void *) dlsym(dlp, (char *)name);
#else
    return (void *) dlsym(dlp, name);
#endif

#elif defined(DLL_HPSHL)
    void *ret = NULL;
    int rv;

    rv = shl_findsym((shl_t *)&dlp, name, TYPE_PROCEDURE, &ret);
    return ( rv == -1 ? NULL : ret);
#elif defined(DLL_WIN32)
    return (void *) GetProcAddress((HINSTANCE) dlp, name);
#endif
}

NSAPI_PUBLIC char *dll_error(DLHANDLE)
{
#if defined(USE_NSPR_DLL)
    return PR_DLLError();

#elif defined(DLL_DLOPEN)
    return dlerror();

#else
    return system_errmsg();
#endif
}

NSAPI_PUBLIC void dll_close(DLHANDLE dlp)
{
#if defined(USE_NSPR_DLL)
    /* nothing */

#elif defined(DLL_DLOPEN)
    dlclose(dlp);

#elif defined (DLL_HPSHL)
    shl_unload((shl_t) dlp);

#elif defined(DLL_WIN32)
    FreeLibrary((HINSTANCE) dlp);
#endif
}

#endif /* DLL_CAPABLE */
