/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * clockthread.c - Platform independent clockthread
 *
 */
#if defined(AIX)
#if defined(FD_SETSIZE)
#undef FD_SETSIZE
#endif
#endif

#ifndef WIN32
#include <unistd.h>
#include <sys/time.h>
#define SLEEP(x) PR_Sleep (PR_SecondsToInterval (x))
#else
#include "windows.h"
#include "time.h"
static DWORD init_secs;
static DWORD init_msecs;
#define SLEEP(x) Sleep(x*1000)
#endif

#include "base/fasttime.h"
#include "nspr.h"
#ifdef SOLARIS
#include "private/pprthred.h"
#endif

static PRBool      _ft_initialized = PR_FALSE;
static PRUint32    _ft_myclock = 0;
static PRThread   *_ft_clockthread;

static void
clockthread_main(void *unused)
{
    PRLock *l;
    PRCondVar *cv;
    PRIntervalTime t = PR_SecondsToInterval(1);
    PRUint32 count = 0;

    l = PR_NewLock();
    if (!l)
        return;
    cv = PR_NewCondVar(l);
    if (!l)
        return;

    PR_Lock(l);
    _ft_myclock = ft_md_time();
    while(1) {
        PR_WaitCondVar(cv, t);
        if (count++ > 5) {
            /* Resync the clock every 5 seconds */
            _ft_myclock = ft_md_time();
            count = 0;
        } else
            _ft_myclock++;
    }

    /* Not reached */
    PR_Unlock(l);
}

PR_IMPLEMENT(PRBool) ft_init()
{
    PRBool rv = PR_FALSE;
    int wait = 60;  /* seconds */
    char *strwait;

    if ( (strwait = getenv("_NSPR_FASTTIME_WAIT")) != 0) {
        wait = atoi(strwait);
    }

#ifdef SOLARIS
    /* Unbound threads aren't guaranteed to wakeup every second... */
    _ft_clockthread = PR_CreateThreadGCAble(PR_SYSTEM_THREAD,
#else
    _ft_clockthread = PR_CreateThread(PR_SYSTEM_THREAD,
#endif
                                      clockthread_main,
                                      NULL,
                                      PR_PRIORITY_NORMAL,
                                      PR_GLOBAL_THREAD,
                                      PR_UNJOINABLE_THREAD,
                                      0);
    if (!_ft_clockthread)
        return PR_FALSE;

    /* Wait for a few seconds for the clockthread to start */
    while(wait--) {
        if (_ft_myclock != 0)
            break;
        SLEEP(1);
    }
    if (_ft_myclock == 0) {
        /* failed to initialize! */
        return PR_FALSE;
    }
    
    rv = ft_md_init();
    if (rv == PR_TRUE)
        _ft_initialized = PR_TRUE;
    return rv;
}

PR_IMPLEMENT(PRUint32) ft_time()
{
    if (!_ft_initialized)
        return ft_md_time();
    return _ft_myclock;
}

PR_IMPLEMENT(PRBool) ft_md_init()
{
#ifdef WIN32
    init_secs = time(NULL);
    init_msecs = timeGetTime();
#endif 

    return PR_TRUE;
}
 
PR_IMPLEMENT(PRUint32) ft_md_time() {
#ifdef WIN32
  if (!_ft_initialized) {
    return time(NULL);
    } 
  else {
    DWORD now = timeGetTime();
    return (init_secs) + ((now-init_msecs)/1000);
  }
#else
#ifdef SOLARIS
  struct timeval tv;
  int rv;
 
  /* On solaris, gettimeofday() is NOT a system call */
  rv = gettimeofday(&tv);
  if (rv == -1) {
    /* Argh; a fatal error; punt and just call time() */
    return time(NULL);
  } 
 
  return tv.tv_sec;
#else
  return time(NULL);
#endif
#endif
}
