/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * netio.h: low-level IO routines
 * 
 * These are intended to be internal only; do not expose.
 * 
 */

#ifndef _LIB_BASE_NETIO_H_
#define _LIB_BASE_NETIO_H_

#include "netsite.h"
#include "base/net.h"
#ifdef NET_SSL
/*
#include "base/minissl.h"
*/
#endif /* NET_SSL */

/* removed for ns security HCL_1_5 integration
typedef void (*_net_accept_hook_fn_t)(SSLAcceptFunc);
*/
typedef void (*_net_accept_hook_fn_t)(void);
typedef int (*_net_accept_fn_t)(SYS_NETFD, struct sockaddr *, int *);
typedef int (*_net_real_accept_fn_t)(SYS_NETFD, struct sockaddr *, int *);
typedef int (*_net_bind_fn_t)(SYS_NETFD, const struct sockaddr *, int);
typedef int (*_net_close_fn_t)(SYS_NETFD);
typedef int (*_net_connect_fn_t)(SYS_NETFD, const void *, int);
typedef int (*_net_data_pending_hack_fn_t)(SYS_NETFD);
typedef int (*_net_force_handshake_fn_t)(SYS_NETFD);
typedef int (*_net_security_status_fn_t)(SYS_NETFD, int *, char **, int *, int *, char **, char **);
typedef int (*_net_getsockopt_fn_t)(SYS_NETFD, int, int, void *, int *);
typedef int (*_net_getpeername_fn_t)(SYS_NETFD, struct sockaddr *, int *);
typedef int (*_net_ioctl_fn_t)(SYS_NETFD, int, void *);
typedef int (*_net_import_fn_t)(SYS_NETFD);
typedef int (*_net_listen_fn_t)(SYS_NETFD, int);
typedef int (*_net_setsockopt_fn_t)(SYS_NETFD, int, int, const void *, int);
typedef int (*_net_select_fn_t)(int, fd_set *, fd_set *, fd_set *, struct timeval *);
typedef int (*_net_socket_fn_t)(int, int, int);
typedef int (*_net_read_fn_t)(SYS_NETFD, char *buf, int);
typedef int (*_net_write_fn_t)(SYS_NETFD, char *buf, int);
typedef int (*_net_writev_fn_t)(SYS_NETFD, struct iovec *, int);

#ifdef THREAD_NSPR_USER
extern int nspr_accept(SYS_NETFD, struct sockaddr *, int *);
#endif /* THREAD_NSPR_USER */
#ifdef NET_SSL
/* removed for ns security HCL_1_5 integration
extern void SSL_AcceptHook(SSLAcceptFunc);
*/
#endif /* NET_SSL */
extern int _net_simulated_writev(SYS_NETFD, struct iovec *, int);

typedef struct net_io_t {
	_net_accept_fn_t 			real_accept;
	_net_bind_fn_t				bind;
	_net_close_fn_t				close;
	_net_connect_fn_t			connect;
	_net_getsockopt_fn_t		getsockopt;
	_net_getpeername_fn_t		getpeername;
	_net_ioctl_fn_t				ioctl;
	_net_import_fn_t			import;
	_net_import_fn_t			import_nonblocking;
	_net_listen_fn_t			listen;
	_net_setsockopt_fn_t		setsockopt;
	_net_select_fn_t			select;
	_net_socket_fn_t			socket;
	_net_read_fn_t				read;
	_net_write_fn_t				write;
	_net_writev_fn_t			writev;
		/* SSL specific functions */
	_net_accept_hook_fn_t		accept_hook;
	_net_real_accept_fn_t		accept;
	_net_data_pending_hack_fn_t	data_pending_hack;
	_net_force_handshake_fn_t	force_handshake;
	_net_security_status_fn_t	security_status;
} net_io_t;

#ifdef __cplusplus
extern "C" net_io_t net_io_functions;
#else
extern net_io_t net_io_functions;
#endif

#endif /* _LIB_BASE_NETIO_H_ */
