/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/* 
 * pathnames.c
 *
 *
 * Intended to be a module for getting/setting/extracting path pieces.
 * Should help avoid some of the "/" nightmare we had with NT; and maybe also
 * make foreign ports much easier.
 *
 *
 */


#include <stdio.h>
#include "netsite.h"
#include "base/pathnames.h"
#include "util.h"
#include "nsassert.h"

char *pn_extract_filename(char *full_name)
{
#ifdef XP_WIN32
	char *ptr;
	char *last_delimiter = NULL;
	char *filename;
	int filename_len;

	NS_ASSERT(full_name);

	for (ptr = full_name; ptr && *ptr; ptr++)
		if (*ptr == '/')
			last_delimiter = ptr;

	if (last_delimiter)
		filename_len = (int)(ptr - last_delimiter);
	else
		filename_len = (int)(ptr - full_name)+1;

	if ( !(filename = (char *)MALLOC(filename_len+1-1)) ) 
		return NULL;

    if (last_delimiter)
		memmove(filename, last_delimiter+1, filename_len);
	else
		memmove(filename, full_name, filename_len);
	filename[filename_len-1] = '\0';

	return filename;
#endif
	return NULL;
}

char *pn_extract_directory(char *full_name)
{
#ifdef XP_WIN32
	char *pathname;
	int pathname_len;
	char *ptr;
	char *last_delimiter = NULL;

	for(ptr=full_name; ptr && *ptr; ptr++)
		if (*ptr == '/')
			last_delimiter = ptr;

	if (last_delimiter)
		pathname_len = (int)(last_delimiter - full_name);
	else
		return NULL;

	if ( !(pathname = (char *)MALLOC(pathname_len+1)) ) 
		return NULL;

	memmove(pathname, full_name, pathname_len);
	pathname[pathname_len] = '\0';

	return pathname;
#endif

	return NULL;
}

char *pn_concatenate(char *directory, int dirlen, char *filename, int filelen)
{
#ifdef XP_WIN32
	char *fullname;
	int fullname_len;

	NS_ASSERT(directory);
	NS_ASSERT(filename);

	if (dirlen < 0)
		dirlen = strlen(directory);
	if (filelen < 0)
		filelen = strlen(filename);
	
	fullname_len = dirlen + 1 + filelen;

	if ( !(fullname = (char *)MALLOC(fullname_len+1)) ) 
		return NULL;

	memmove(fullname, directory, dirlen);
	fullname[dirlen] = '/';
	memmove(&(fullname[dirlen+1]), filename, filelen);
	fullname[dirlen+filelen+1] = '\0';

	return fullname;
#endif

	return NULL;
}

int 
pn_directory_equal(char *d1, char *d2)
{
#ifdef XP_WIN32
	return strcasecmp(d1, d2);
#endif

	return -1;
}

int 
pn_pathname_equal(char *p1, char *p2)
{
#ifdef XP_WIN32
	return strcasecmp(p1, p2);
#endif

	return -1;
}
