/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
#include "secpwd_pvt.h"

/* This file is copy from old libsec to do password checking */

/*
 * NOTE:  The contents of this file are NOT used by the client.
 * (They are part of the security library as a whole, but they are
 * NOT USED BY THE CLIENT.)  Do not change things on behalf of the
 * client (like localizing strings), or add things that are only
 * for the client (put them elsewhere).  Clearly we need a better
 * way to organize things, but for now this will have to do.
 */

#ifdef XP_UNIX

#include <termios.h>
#include <stdio.h>

static void echoOff(int fd)
{
    if (isatty(fd)) {
	struct termios tio;
	tcgetattr(fd, &tio);
	tio.c_lflag &= ~ECHO;
	tio.c_cc[VKILL] = '\0';
	tcsetattr(fd, TCSAFLUSH, &tio);
    }
}

static void echoOn(int fd)
{
    if (isatty(fd)) {
	struct termios tio;
	tcgetattr(fd, &tio);
	tio.c_lflag |= ECHO;
	tio.c_cc[VKILL] = '\0';
	tcsetattr(fd, TCSAFLUSH, &tio);
    }
}



unsigned char *SEC_GetPassword(FILE *input, FILE *output, char *prompt,
			       PRBool (*ok)(unsigned char *))
{
    unsigned char phrase[200];
    int infd = fileno(input);
    int isTTY = isatty(infd);
    char *fdstr = getenv("WD_PIPE_FD");
    char *fdstrout = getenv("WD_PIPE_OUT");
    FILE *wdin = NULL;
    FILE *wdout = NULL;
    int wdpipe;
    int wdpipeout;
    int plen;

    /* Got a watchdog process keeping passwords for us? */
    if (fdstr && fdstrout) {

        phrase[0] = 0;

        /* Get the watchdog pipe fd from the environment variable */
        wdpipe = atoi(fdstr);

        /* Dup the fd so that we can open and close a stream */
        wdpipe = dup(wdpipe);
        if (wdpipe >= 0) {

            /* Open an input stream on the watchdog pipe */
            wdin = fdopen(wdpipe, "r");
        }

        /* Same for output */
        wdpipeout = atoi(fdstrout);
        if (wdpipeout >= 0) {
            wdout = fdopen(wdpipeout, "w");
        }

        if (wdin && wdout) {

            /* Use the prompt string to request the password */
            fprintf(wdout, "GETPWD \"%s\"\n",
                    prompt ? prompt : "Password: ");
            fflush(wdout);

            /* Read the password from the watchdog */
            phrase[0] = 0;
            fgets((char *)phrase, sizeof(phrase), wdin);

            plen = PL_strlen((char*)phrase);
            if (plen > 0) {
                phrase[plen-1] = 0;
            }

            /* 
	     * Don't need to check for a weird password input here...
	     * the password has already been set !!
	     */

        }
        else {
            fprintf(stderr, "Failed to open wdin (0x%x), wdout (0x%x)\n",
                    wdin, wdout);
        }

        if (wdout) {
            fclose(wdout);
            wdpipeout = -1;
        }

        if (wdin) {
            fclose(wdin);
            wdpipe = -1;
        }

        if (wdpipe >= 0) {

            /* Close the dup of the watchdog pipe fd */
            close(wdpipe);
        }

        if (wdpipeout >= 0) {
            close(wdpipeout);
        }

        return (unsigned char *)PL_strdup((char *)phrase);
    }

    for (;;) {
      /* Prompt for password */
      if (isTTY) {
	fprintf(output, "%s", prompt);
	echoOff(infd);
      }
      fgets((char*) phrase, sizeof(phrase), input);
      if (isTTY) {
	fprintf(output, "\n");
	echoOn(infd);
      }
 
      /* stomp on newline */
      phrase[PL_strlen((char*)phrase)-1] = 0;
 
      /* 
       * Don't need to check for a weird password input here...
       * the password has already been set !!
       */

      return (unsigned char*) PL_strdup((char*)phrase);
    }
}

#endif	/* XP_UNIX */


PRBool SEC_CheckPassword(unsigned char *cp)
{
    int len;
    unsigned char *end;

    len = PL_strlen((char*)cp);
    if (len < 8) {
	return PR_FALSE;
    }
    end = cp + len;
    while (cp < end) {
	unsigned char ch = *cp++;
	if (!((ch >= 'A') && (ch <= 'Z')) &&
	    !((ch >= 'a') && (ch <= 'z'))) {
	    /* pass phrase has at least one non alphabetic in it */
	    return PR_TRUE;
	}
    }
    return PR_FALSE;
}

PRBool SEC_BlindCheckPassword(unsigned char *cp)
{
    if (cp != NULL) {
	return PR_TRUE;
    }
    return PR_FALSE;
}

char*
getPassword_cb(PK11SlotInfo* slot, void *wincx)
{
  char *pw;
  char *prompt;

  /* For i18n, later ....
  prompt = PR_smprintf(XP_GetString(XP_SEC_ENTER_PWD),
		       PK11_GetTokenName(slot));
  */

  prompt = PR_smprintf("Please enter password for (%s) ", PK11_GetTokenName(slot));

  pw = (char*)SEC_GetPassword(stdin, stdout, prompt, SEC_CheckPassword);

  PR_Free(prompt);
  return pw;
}
