/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * sslinit.c: Initialize SSL for a server
 * 
 * Rob McCool
 */


#ifdef NET_SSL

#include "systems.h"

/* removed due to ne-security integration 
#include "sec.h"
*/
#include "sechash.h"
#include "secerr.h"
#include "key.h"
#include "cert.h"
#include "secmod.h"
#include "pk11func.h"
#include "ssl.h"

#include "daemon.h"
#include "libadminutil/psetc.h" /* pset commands */


/* extern int SEC_ERROR_BAD_PASSWORD;  */

/* extern char *XP_GetBuiltinString(int16 i);  */

/* XXX probably belong in a .h file (already in libadmin.h) */
#define SERVER_KEY_NAME		"Server-Key"
#define SERVER_CERT_NAME	"Server-Cert"

#if defined(XP_WIN32) || defined(FORTEZZA)
#include "ereport.h"
#endif

#include <stdarg.h>
#include "util.h"

#ifdef XP_WIN32
#include "ereport.h"
#include <windows.h>
#include <nt/resource.h>
#if !defined(MCC_ADMSERV)
#include <nt/regparms.h>    /* to get PRODUCT_NAME  */
/* service_name was moved here from various nt*.c modules */
/* making this external here causes undefined external errors in cgi */
/* programs that include this lib. */
#endif
static char *service_name = NULL;  /* moved from ntmain.c -ahakim WatchDog */

/*
 * Referenced by lib/libsec, implemented for Unix in lib/xp/xp_file.c.
 * This is a dummy implementation for XP_WIN32.
 */
    /* Hopefully this will have the desired effect */
/* Removed for ns security  integration (no libsec is used)
PUBLIC char *
WH_FileName(const char *name, XP_FileType type)
{
    return NULL;
}
*/
#endif /* XP_WIN32 */

#define SERVSSL_ERROR_LEN 1024


#ifdef FORTEZZA
static void servssl_warn(char *fmt, ...)
{
    va_list args;
    char errstr[SERVSSL_ERROR_LEN];

    va_start(args, fmt);
    util_vsnprintf(errstr, sizeof(errstr), fmt, args);
#ifdef XP_UNIX
    fprintf(stderr, "%s\n", errstr);
#else  /* XP_WIN32 */
    ereport(LOG_WARN, "SSL_Warning: %s\n", errstr);
#endif
}

static int do_Alert(void *arg,PRBool onOpen, char *string, 
		    int value1, void *value2) {
    ereport(LOG_WARN,string,value1,value2);
    return 0;
}

#else
#define servssl_warn servssl_error
#endif


#ifndef MCC_ADMSERV

static char *errstrs[] = {
	"I/O error" ,				/* 0 */
	"Security library error",		/* 1 */
	"Bad data",				/* 2 */
	"Security library error",		/* 3 */
	"Security library error",		/* 4 */
	"Security library error",		/* 5 */
	"Security library error",		/* 6 */
	"Security library error",		/* 7 */
	"Security library error",		/* 8 */
	"Bad DER-encoded message",		/* 9 */
	"Invalid signature",			/* 10 */
	"Expired certificate",			/* 11 */
	"Revoked certificate",			/* 12 */
	"Unknown issuer",			/* 13 */
	"Invalid public key",			/* 14 */
	"Bad password",				/* 15 */
	"Unknown Error (SEC+16)",		/* 16 */
	"Security library error",		/* 17 */
	"Database error",			/* 18 */
	"Out of memory error",			/* 19 */
	"Untrusted issuer",			/* 20 */
	"Untrusted certificate",		/* 21 */
	"Duplicate certificate",		/* 22 */
	"Duplicate certificate name",		/* 23 */
	"Couldn't add certificate",		/* 24 */
	"Couldn't refile private key",		/* 25 */
	"No Private key found",			/* 26 */
	"Certificate is valid",			/* 27 */
	"Invalid Certificate",			/* 28 */
	"No Response",				/* 29 */
	"Issuer Cert Expired",			/* 30 */
	"Expired CRL",				/* 31 */
	"Invalid CRL signature",		/* 32 */
	"Invalid CRL",				/* 33 */
	"Loading CRL is Older then DB CRL",	/* 34 */
	"No more random data available",	/* 35 */
	"Unknown Error (SEC+36)",		/* 36 */
	"Unknown Error (SEC+37)",		/* 37 */
	"Unknown Error (SEC+38)",		/* 38 */
	"Unknown Error (SEC+39)",		/* 39 */
	"Unknown Error (SEC+40)",		/* 40 */
	"Unknown Error (SEC+41)",		/* 41 */
	"Unknown Error (SEC+42)",		/* 42 */
	"Unknown Error (SEC+43)",		/* 43 */
	"Unknown Error (SEC+44)",		/* 44 */
	"Unknown Error (SEC+45)",		/* 45 */
	"Unknown Error (SEC+46)",		/* 46 */
	"Unknown Error (SEC+47)",		/* 47 */
	"Unknown Error (SEC+48)",		/* 48 */
	"Unknown Error (SEC+49)",		/* 49 */
#ifdef FORTEZZA
	"Bad Fortezza card",			/* 50 */
	"No Fortezza cards found",		/* 51 */
	"No Fortezza card selected",		/* 52 */
	"Fortezza invalid",			/* 53 */
	"No Fortezza personality found",	/* 54 */
	"Fortezza invalid",			/* 55 */
	"Invalid Pin",				/* 56 */
	"Fortezza Couldn't initialize personalities", /* 57 */
	"No CKL (Fortezza)",			/* 58 */
	"Expired CKL (Fortezza)",		/* 59 */
	"Invalid CKL Signature (Fortezza)",	/* 60 */
	"Revolked Key Material",		/* 61 */
	"Invalid CKL",				/* 62 */
	"No matching Cert for CKL",		/* 63 */
	"Loading KRL is Older then DB KRL",	/* 64 */
#endif
};

/*
 * SSL errors are a different range from normal security errors.
 */
static char *sslstrs[] = {
	"Export Server Only",			/* 0 */
	"US Server Only",			/* 1 */
	"No cipher overlap",			/* 2 */
	"No certificate",			/* 3 */
	"Bad certificate",			/* 4 */
	"Unknown Error (SSL+5)",		/* 5 */
	"Bad client Data",			/* 6 */
	"Bad server Data",			/* 7 */
	"Unsupported Certificate Encountered",	/* 8 */
	"Unsupported version",			/* 9 */
	"Unknown Error (SSL+10)",		/* 10 */
	"Certificate does not match key",	/* 11 */
	"Bad Certificate Domain",		/* 12 */
	"Unknown Error (SSL+13)",		/* 13 */
	"SSL Version 2 not Enabled",		/* 14 */
	"Received Bad Mac Read Alert",		/* 15 */
	"Received Bad Mac Data Alert",		/* 16 */
	"Received Bad Cert Alert",		/* 17 */
	"Received Revoked Cert Alert",		/* 18 */
	"Received Expired Cert Alert",		/* 19 */
#ifdef FORTEZZA
	"Fortezza PQG Domain mismatch",		/* 20 */
#endif

};

static int error_str_size = sizeof(errstrs)/sizeof(errstrs[0]);
extern int SEC_ERROR_IO;

static int ssl_str_size = sizeof(sslstrs)/sizeof(sslstrs[0]);
extern int SSL_ERROR_EXPORT_ONLY_SERVER;

static char *fake_error_str(int error) {
    if ((error >= SEC_ERROR_IO)  &&
    			(error < SEC_ERROR_IO+error_str_size)) {
    	return errstrs[error - SEC_ERROR_IO];
    }
    if ((error >= SSL_ERROR_EXPORT_ONLY_SERVER)  &&
    			(error < SSL_ERROR_EXPORT_ONLY_SERVER+ssl_str_size)) {
    	return sslstrs[error - SSL_ERROR_EXPORT_ONLY_SERVER];
    }

    return ("Unknown");
}

/*
 * Callback to log problems with Client Certificate checking.
 * Make sure the above errorlist matches allxpstr.h numbers.
 */
static SECStatus serverCertError(void *arg,int fd) {
	char *name = NULL;
	char *issuer = NULL;
	int error,status;

	error = PR_GetError();
	SSL_SecurityStatus(fd,&status,NULL,NULL,NULL,&issuer,&name);


	ereport(LOG_WARN,"Client CertError: %s\n Cert:[%s]\n CA:[%s]",
				fake_error_str(error),name,issuer);
	free(name);
	free(issuer);
	return SECFailure;
}

#endif /* !MCC_ADMSERV */


#if 0
/* Removed for ns security integration ....., do know how all this password
   prompting business  .... */
static PRBool local_true(unsigned char *cp) { return PR_TRUE; }

/*
 * Return key file password in response to security library upcall.
 * Prompt for the password on stdin if necessary.
 *
 * Note: SEC_GetPassword is conditionally compiled in libsec, and is 
 * UNIX-only. NT defines it below.
 */
static SECItem *
GetPwd(void *arg, SECKEYKeyDBHandle *keydb)
{
    unsigned char **ppwd = (unsigned char **)arg;
    unsigned char *pw = *ppwd;

    if (!pw) {
#ifndef FORTEZZA
      /* Since SEC_CheckPassword has been removed in new ns security, and
	 no equvalence function is available, so just change
	 SEC_CheckPassword to local_true
	 pw = (char *)SEC_GetPassword(stdin, stdout, "Key File Password: ",
	 SEC_CheckPassword);
	 */
      pw = SEC_GetPassword(stdin, stdout, "Key File Password: ",
			   SEC_CheckPassword);
#else
      pw = SEC_GetPassword(stdin, stdout, "Key File Password: ",
			   local_true);
#endif
      if (!pw) return 0;
      *ppwd = pw;

	/* update the private key database if necessary */
      if ( keydb->version < PRIVATE_KEY_DB_FILE_VERSION ) {
	SECKEY_UpdateKeyDBPass1(keydb);
	SECKEY_UpdateKeyDBPass2(keydb, pw);
      }
    }

    return (SECItem *)SECKEY_HashPassword((char*)pw, keydb->global_salt);
}
/*
 * Another level of password abstraction [jules] 1/18/95
 *  This is for the Secure News Server which has 2 processes that need to share
 *  the password.  By default GetPwd() is called so no one else will be effected,
 *  but this function allows SNS to replace the call.
 */

/* the global which is defaulted to the function above*/
SECKEYGetPasswordKey do_GetPwd = GetPwd;

/* the call to replace it */
void
GetPwdHook (SECKEYGetPasswordKey func) {
    do_GetPwd = func;
}

#endif  /* 0 */

#ifdef FORTEZZA
static void *
GetPin(void *arg)
{
    char **ppwd = (char **)arg;
    char *pw;

    pw = *ppwd;

    if (!pw) {
    	pw = SEC_GetPassword(stdin, stdout, "Enter Fortezza Pin:",
			     local_true); /* accept anything as a pin */
   	if (!pw) return 0;
	*ppwd = pw;
    }

    return pw;
}
#endif


#ifdef FORTEZZA
FortezzaPinFunc do_GetPin = (FortezzaPinFunc)GetPin;

void
GetPinHook (FortezzaPinFunc func) {
    do_GetPin = func;
}
#endif


/* Takes one cert file name, produces two, does SEC_OpenCertDB() on them */
static SECStatus
OpenCertDBFiles(CERTCertDBHandle *handle, char * cert_fn)
{
    char cert_db_fn[1024];

    if (strstr(cert_fn, ".db")) {
       strcpy(cert_db_fn, cert_fn);
    } else {
       sprintf(cert_db_fn, "%s.db", cert_fn);
    }

    return cmgOpenCertDB(cert_db_fn, &handle);
}

#if 0
/* Removed for ns security integration  */
/* Hope that the new ns security does not cache
   pointer ........ will know it SOOOOON  */

/*
 * I made this global (though static) because 
 * SEC_SetDefaultCertDB wants a pointer to it
 * that it caches. What a crock!
 */
static CERTCertDBHandle certdb;

NSAPI_PUBLIC
int servssl_certinit(char *certfn, void **pcert, int read_server_cert)
{
    CERTCertificate *cert = 0;
    int rv;

    /* Open the server cert database */
    rv = OpenCertDBFiles(&certdb, certfn);
    if (rv) {
        servssl_warn("Open of server certificate file failed (%d)",
                      PR_GetError());
	return(rv);
    }

    /* $$$
     * The data base being set as default here is the data base
     * that this server of certificate authorities that the
     * server is willing to accept client certificates from.
     * What I'm doing here is saying that list is equivalent
     * to the certificate(s) the server uses for its server
     * authentication. That's wrong
     * $$$$
     */
    CERT_SetDefaultCertDB(&certdb);

    if (read_server_cert) {
	/* Read the server cert from the database */
	cert = CERT_FindCertByNickname(&certdb, SERVER_CERT_NAME);
	if (!cert) {
	    servssl_warn("Could not read server certificate (%d)", PR_GetError());
	    return rv;
	}

	rv = CERT_CertTimesValid(cert);
	if (rv) {
	    servssl_warn("Server certificate validity period has expired.");
	    CERT_DestroyCertificate(cert);
	    return rv;
	}

	if (pcert) *pcert = (void *)cert;
    }
    return rv;
}

#endif


/* Random number generation */
/* WARNING: These calls are fine for Server V3.X, but if we want to be
 * FIPS complient in V4.X, these functions should cal PK11 calls...
 */

NSAPI_PUBLIC void *nsapi_random_create(void)
{
   /* There is no notion of context with PK11_Random functions */
}

NSAPI_PUBLIC
void nsapi_random_update(void *rctx, unsigned char *inbuf, int length)
{
    PK11_RandomUpdate(inbuf, length);
}

NSAPI_PUBLIC
void nsapi_random_generate(void *rctx, unsigned char *outbuf, int length)
{
    PK11_GenerateRandom(outbuf, length);
}

NSAPI_PUBLIC void nsapi_random_destroy(void *rctx)
{
   /* There is no notion of context with PK11_Random functions */
}

/* MD5 hash computation */

NSAPI_PUBLIC void *nsapi_md5hash_create(void)
{
    return (void *)PK11_CreateDigestContext(SEC_OID_MD5);
}

NSAPI_PUBLIC void *nsapi_md5hash_copy(void *hctx)
{
    return (void *)PK11_CloneContext((PK11Context *)hctx);
}

NSAPI_PUBLIC void nsapi_md5hash_begin(void *hctx)
{
    PK11_DigestBegin((PK11Context *)hctx);
}

NSAPI_PUBLIC
void nsapi_md5hash_update(void *hctx, unsigned char *inbuf, int length)
{
    PK11_DigestOp((PK11Context *)hctx, inbuf, (unsigned)length);
}

NSAPI_PUBLIC void nsapi_md5hash_end(void *hctx, unsigned char *outbuf)
{
    unsigned int outlen;

    PK11_DigestFinal((PK11Context *)hctx, outbuf, &outlen, 16);
}

NSAPI_PUBLIC void nsapi_md5hash_destroy(void *hctx)
{
    PK11_DestroyContext((PK11Context *)hctx, PR_TRUE);
}

NSAPI_PUBLIC void
nsapi_md5hash_data(unsigned char *outbuf, unsigned char *inbuf, int length)
{
    PK11_HashBuf(SEC_OID_MD5, outbuf, inbuf, (PRUint32)length);
}

#ifdef XP_WIN32
static char password[512];

//--------------------------------------------------------------------------//
// borrowed from \msvc20\mfc\src\dlgcore.cpp                                //
//--------------------------------------------------------------------------//
void static CenterDialog(HWND hwndParent, HWND hwndDialog)
{
   RECT DialogRect;
   RECT ParentRect;
	POINT Point;
   int nWidth;
   int nHeight;
 
	// Determine if the main window exists. This can be useful when
   // the application creates the dialog box before it creates the
   // main window. If it does exist, retrieve its size to center
   // the dialog box with respect to the main window.
   if(hwndParent != NULL)
	{
      GetClientRect(hwndParent, &ParentRect);
	}
   else
   {
	   // if main window does not exist, center with respect to desktop
   	hwndParent = GetDesktopWindow();
      GetWindowRect(hwndParent, &ParentRect);
   }
 
   // get the size of the dialog box
   GetWindowRect(hwndDialog, &DialogRect);
 
   // calculate height and width for MoveWindow()
   nWidth = DialogRect.right - DialogRect.left;
   nHeight = DialogRect.bottom - DialogRect.top;
 
   // find center point and convert to screen coordinates
   Point.x = (ParentRect.right - ParentRect.left) / 2;
   Point.y = (ParentRect.bottom - ParentRect.top) / 2;

	ClientToScreen(hwndParent, &Point);
 
   // calculate new X, Y starting point
   Point.x -= nWidth / 2;
   Point.y -= nHeight / 2;
 
   MoveWindow(hwndDialog, Point.x, Point.y, nWidth, nHeight, FALSE);
}




#if !defined(MCC_ADMSERV)
//--------------------------------------------------------------------------//
// get password from watchdog process global window                         //
//--------------------------------------------------------------------------//
BOOL SEC_GetPasswordFromWatchDog(char *szLocalPassword)
{
	// following defines ideally should be common files shared WatchDog and Server
	#define GWL_PROCESS_HANDLE  (sizeof(LONG) * 0)
	#define GWL_PASSWORD_ADDR   (sizeof(LONG) * 1)
	#define GWL_PASSWORD_LENGTH (sizeof(LONG) * 2)

	BOOL bReturn = FALSE;
	HWND hwndRemote;
	int cbRemotePassword = 0;
	char *szRemotePassword = NULL;
	HANDLE hRemoteProcess;
	DWORD dwNumberOfBytesRead;

	// find WatchDog application window
	if(hwndRemote = FindWindow("httpd", service_name))
	{
		if(hRemoteProcess = (void *)GetWindowLong(hwndRemote, GWL_PROCESS_HANDLE))
		{
			cbRemotePassword = GetWindowLong(hwndRemote, GWL_PASSWORD_LENGTH);
			if(cbRemotePassword > 512)
				cbRemotePassword = 512;
			if(szRemotePassword = (char *)GetWindowLong(hwndRemote, GWL_PASSWORD_ADDR))
			{
				if(ReadProcessMemory(hRemoteProcess, szRemotePassword, szLocalPassword, cbRemotePassword, &dwNumberOfBytesRead))
					bReturn = TRUE;
			}
		}
	}
	return(bReturn);
}
#endif /* MCC_ADMSERV */

#endif   /* XP_WIN32 */
#endif /* NET_SSL */


#if 0


DO NOT COMPILE THE DONGLE STUFF IT IS DANGEROUS AND A THOUSAND CURSES BE UPON
YOUR HEAD IF YOU DO --Rob


UNIX SECTION:

/* ------------------------------ dongle ---------------------------------- */


static unsigned char data[] = {
    0x30, 0x81, 0x41,					/* junk */

    0x00, 0xa2, 0xe1, 0x69, 0x7b, 0x34, 0x13, 0x66,	/* modulus */
    0xbf, 0x2c, 0xb3, 0xee, 0x62, 0x21, 0x25, 0x6f,
    0x94, 0xbe, 0xd4, 0x06, 0xd9, 0xbf, 0xcc, 0x39,
    0x34, 0x3f, 0x20, 0xc7, 0xeb, 0x6a, 0x35, 0x71,
    0xac, 0x64, 0x8e, 0xbb, 0xf9, 0x76, 0x07, 0x1a,
    0x01, 0x6f, 0xe6, 0xbf, 0x46, 0xab, 0x8b, 0xaa,
    0x3f, 0x4c, 0x00, 0xd1, 0x73, 0x40, 0x84, 0x21,
    0xb2, 0xaf, 0xa8, 0x82, 0x55, 0xbc, 0x27, 0xb9,
    0xe9
};

/* Copy modulus from data[] above */
SECStatus Copy(SECItem *dest, int offset)
{
    unsigned char *cp = data + offset;
    int len = sizeof(data) - offset;

    dest->data = (unsigned char*) DS_Alloc(len);
    if (!dest->data) return SECFailure;
    dest->len = len;
    XP_MEMCPY(dest->data, cp, len);
    return SECSuccess;
}

SECStatus Set(SECItem *dest, int value)
{
    char cv = value;
    dest->data = (unsigned char*) DS_Alloc(1);
    if (!dest->data) return SECFailure;
    dest->len = 1;
    XP_MEMCPY(dest->data, &cv, 1);
    return SECSuccess;
}


#include <sys/utsname.h>


/* This should be larger. But the security lib won't let us
   perform RSA encryption on anything larger than modulusLen - 11 */
#define UNAME_BUFSIZE 50

/* Butthead, this SUCKS! */
SECStatus  GetSysInfo(char *namebuf, int bufsize)
{
    struct utsname uname_s;
    int l, sz;

    if(uname(&uname_s) == -1)
        return SECFailure;

    sz = bufsize;
    l  = util_snprintf(namebuf, sz, "%s ", uname_s.sysname);
    l += util_snprintf(&namebuf[l], sz - l, "%s ", uname_s.nodename);
    l += util_snprintf(&namebuf[l], sz - l, "%s ", uname_s.release);
    l += util_snprintf(&namebuf[l], sz - l, "%s ", uname_s.version);
    l += util_snprintf(&namebuf[l], sz - l, "%s", uname_s.machine);

    return SECSuccess;
}

/*
** Generate *static* key that will encrypt the password. Make it depend
** on the machine it's running on...
*/
SECStatus GenerateDongleFile(unsigned char *password, char *file)
{
    char namebuf[UNAME_BUFSIZE];
    unsigned part1, part2;
    int nb;
    SECStatus rv;
    SECItem eb;
    SECItem data;
    char ob[4000];
    char digest[32];
    RC4Context *rc4;
    RSAContext *rsa;
    SECPublicKey *k;
    MD5Context *md5;
    FILE *fp;

    /* Get system information */
    ZERO(namebuf, sizeof(namebuf));
    rv = GetSysInfo(namebuf, sizeof(namebuf));
    if(rv)
        return SECFailure;

    /* Get public key */
    k = (SECPublicKey*) DS_Zalloc(sizeof(SECPublicKey));
    if (!k) return SECFailure;
    rv = Copy(&k->modulus, 3);
    if (rv) return rv;
    rv = Set(&k->publicExponent, 3);
    if (rv) return rv;

    /* Format an encryption block */
    data.data = (unsigned char*) namebuf;
    data.len = sizeof(namebuf);
    rv = RSA_FormatBlock(&eb, SEC_PublicModulusLen(k), 0x01, &data);
    if (rv) return rv;

    /* Encrypt the encryption block */
    rsa = RSA_CreateContext(0, k);
    if (!rsa) return SECFailure;
    rv = RSA_Encrypt(rsa, ob, &part1, sizeof(ob), eb.data, eb.len);
    if (rv) return rv;
    rv = RSA_End(rsa, ob + part1, &part2, sizeof(ob) - part1);
    if (rv) return rv;

    /* Hash it down to a key */
    md5 = MD5_NewContext();
    if (!md5) return SECFailure;
    MD5_Begin(md5);
    MD5_Update(md5, ob, part1 + part2);
    rv = MD5_End(md5, digest, &part1, sizeof(digest));
    if (rv) return rv;

    /* Now make an RC4 key to encrypt the password with */
    rc4 = RC4_CreateContext(digest, part1);
    if (!rc4) return SECFailure;
    rv = RC4_Encrypt(rc4, ob, &part1, sizeof(ob), password,
		     STRLEN(password));
    if (rv) return rv;

    /* Write out encrypted password. */
    fp = fopen(file, "w");
    if (!fp) return SECFailure;
    nb = fwrite(ob, 1, part1, fp);
    if (nb != (int)part1) return SECFailure;
    fflush(fp);
    fclose(fp);
    return SECSuccess;
}

unsigned char *ProcessDongleFile(FILE *fp)
{
    char pb[4000];
    char namebuf[UNAME_BUFSIZE];
    unsigned part1, part2;
    int nb;
    SECStatus rv;
    SECItem eb;
    SECItem data;
    char ob[4000];
    char digest[32];
    RC4Context *rc4;
    RSAContext *rsa;
    SECPublicKey *k;
    MD5Context *md5;
    unsigned char *cp;

    nb = fread(pb, 1, sizeof(pb), fp);
    if (nb <= 0) return 0;

    /* Get system information */
    ZERO(namebuf, sizeof(namebuf));
    rv = GetSysInfo(namebuf, sizeof(namebuf));
    if(rv)
        return 0;

    /* Get public key */
    k = (SECPublicKey*) DS_Zalloc(sizeof(SECPublicKey));
    if (!k) return 0;
    rv = Copy(&k->modulus, 3);
    if (rv) return 0;
    rv = Set(&k->publicExponent, 3);
    if (rv) return 0;

    /* Format an encryption block */
    data.data = (unsigned char*) namebuf;
    data.len = sizeof(namebuf);
    rv = RSA_FormatBlock(&eb, SEC_PublicModulusLen(k), 0x01, &data);
    if (rv) return 0;

    /* Encrypt the encryption block */
    rsa = RSA_CreateContext(0, k);
    if (!rsa) return 0;
    rv = RSA_Encrypt(rsa, ob, &part1, sizeof(ob), eb.data, eb.len);
    if (rv) return 0;
    rv = RSA_End(rsa, ob + part1, &part2, sizeof(ob) - part1);
    if (rv) return 0;

    /* Hash it down to a key */
    md5 = MD5_NewContext();
    if (!md5) return 0;
    MD5_Begin(md5);
    MD5_Update(md5, ob, part1 + part2);
    rv = MD5_End(md5, digest, &part1, sizeof(digest));
    if (rv) return 0;

    /* Now make an RC4 key to decrypt the password with */
    rc4 = RC4_CreateContext(digest, part1);
    if (!rc4) return 0;
    rv = RC4_Decrypt(rc4, ob, &part1, sizeof(ob), pb, nb);
    if (rv) return 0;

    /* Return the password */
    cp = (unsigned char*) DS_Alloc(part1+1);
    if (!cp) return 0;
    XP_MEMCPY(cp, ob, part1);
    cp[part1] = 0;
    return cp;
}

/*
** Setup/use a dongle file. The dongle file contains an encrypted version
** of the key file password. The only real security here is security by
** obfuscation, which isn't much at all.
*/
void dongle(char *keyfile_dongle)
{
    FILE *fp;
    SECStatus rv;

    fp = fopen(keyfile_dongle, "r");
    if (!fp) {
	/* Generate dongle file */
	for (;;) {
	    password = SEC_GetPassword(stdin, stderr, "Key File Password: ",
				       SEC_CheckPassword);
	    if (password) break;
	}

	rv = GenerateDongleFile(password, keyfile_dongle);
	if (rv) {
	    fprintf(stderr, "Can't generate dongle for this machine\n");
	    exit(-1);
	}
	return;
    }

    /* Read existing dongle file and get the password */
    password = (char*) ProcessDongleFile(fp);
    fclose(fp);
}


NT SECTION:

/* ------------------------------ dongle ---------------------------------- */

static char *password;
static SECPrivateKey *SecurityKey;

static unsigned char data[] = {
    0x30, 0x81, 0x41,					/* junk */

    0x00, 0xa2, 0xe1, 0x69, 0x7b, 0x34, 0x13, 0x66,	/* modulus */
    0xbf, 0x2c, 0xb3, 0xee, 0x62, 0x21, 0x25, 0x6f,
    0x94, 0xbe, 0xd4, 0x06, 0xd9, 0xbf, 0xcc, 0x39,
    0x34, 0x3f, 0x20, 0xc7, 0xeb, 0x6a, 0x35, 0x71,
    0xac, 0x64, 0x8e, 0xbb, 0xf9, 0x76, 0x07, 0x1a,
    0x01, 0x6f, 0xe6, 0xbf, 0x46, 0xab, 0x8b, 0xaa,
    0x3f, 0x4c, 0x00, 0xd1, 0x73, 0x40, 0x84, 0x21,
    0xb2, 0xaf, 0xa8, 0x82, 0x55, 0xbc, 0x27, 0xb9,
    0xe9
};

SECStatus Copy(SECItem *dest, int offset)
{
    unsigned char *cp = data + offset;

    dest->data = (unsigned char*) DS_Alloc(65);
    if (!dest->data) 
    	return SECFailure;
    dest->len = 65;
    XP_MEMCPY(dest->data, cp, 65);
    return SECSuccess;
}

SECStatus Set(SECItem *dest, int value)
{
    char cv = value;

    dest->data = (unsigned char*) DS_Alloc(1);
    if (!dest->data) 
    	return SECFailure;
    dest->len = 1;
    XP_MEMCPY(dest->data, &cv, 1);
    return SECSuccess;
}


/* This should be larger. But the security lib won't let us
   perform RSA encryption on anything larger than modulusLen - 11
   Thanks for dropping this on my lap Kipp. */

#define UNAME_BUFSIZE 50

/* Butthead, this SUCKS! */
SECStatus GetSysInfo(char *namebuf)
{
    LPTSTR lpszSystemInfo;
	DWORD cchBuff = 25;

	SYSTEM_INFO siSysInfo;
	TCHAR tchBuffer[25];

    int l, sz;
	
	ZeroMemory(tchBuffer, 25);
	lpszSystemInfo = tchBuffer;
	if (!GetComputerName(lpszSystemInfo, &cchBuff)) {
		return SECFailure;
	}
    GetSystemInfo(&siSysInfo);

    sz = sizeof(namebuf);
    l  = util_snprintf(namebuf, sz, "%d ", siSysInfo.dwOemId);
    l += util_snprintf(&namebuf[l], sz - l, "%s ", lpszSystemInfo);
    l += util_snprintf(&namebuf[l], sz - l, "%d ", GetVersion());
    l += util_snprintf(&namebuf[l], sz - l, "%d", siSysInfo.dwProcessorType);

    return SECSuccess;
}

/*
** Generate *static* key that will encrypt the password. Make it depend
** on the machine it's running on...
*/
SECStatus GenerateDongleFile(unsigned char *password, char *file)
{
	char namebuf[UNAME_BUFSIZE];
    unsigned part1, part2;
    int nb;
    SECStatus rv;
    SECItem eb;
    SECItem data;
    char ob[4000];
    char digest[32];
    RC4Context *rc4;
    RSAContext *rsa;
    SECPublicKey *k;
    MD5Context *md5;
    FILE *fp;

    /* Get system information */
    ZERO(namebuf, sizeof(namebuf));
    rv = GetSysInfo(namebuf);
    if(rv)
        return SECFailure;

    /* Get public key */
    k = (SECPublicKey*) DS_Zalloc(sizeof(SECPublicKey));
    if (!k) return SECFailure;
    rv = Copy(&k->modulus, 3);
    if (rv) return rv;
    rv = Set(&k->publicExponent, 3);
    if (rv) return rv;

    /* Format an encryption block */
    data.data = (unsigned char*) namebuf;
    data.len = sizeof(namebuf);
    rv = RSA_FormatBlock(&eb, SEC_PublicModulusLen(k), 0x01, &data);
    if (rv) return rv;

    /* Encrypt the encryption block */
    rsa = RSA_CreateContext(0, k);
    if (!rsa) return SECFailure;
    rv = RSA_Encrypt(rsa, ob, &part1, sizeof(ob), eb.data, eb.len);
    if (rv) return rv;
    rv = RSA_End(rsa, ob + part1, &part2, sizeof(ob) - part1);
    if (rv) return rv;

    /* Hash it down to a key */
    md5 = MD5_NewContext();
    if (!md5) return SECFailure;
    rv = MD5_Begin(md5);
    if (rv) return rv;
    rv = MD5_Update(md5, ob, part1 + part2);
    if (rv) return rv;
    rv = MD5_End(md5, digest, &part1, sizeof(digest));
    if (rv) return rv;

    /* Now make an RC4 key to encrypt the password with */
    rc4 = RC4_CreateContext(digest, part1);
    if (!rc4) return SECFailure;
    rv = RC4_Encrypt(rc4, ob, &part1, sizeof(ob), password,
		     STRLEN(password));
    if (rv) return rv;

    /* Write out encrypted password. */
    fp = fopen(file, "w+b");
    if (!fp) return SECFailure;
    nb = fwrite(ob, 1, part1, fp);
    if (nb != (int)part1) return SECFailure;
    fflush(fp);
    fclose(fp);
    return SECSuccess;
}

unsigned char *ProcessDongleFile(FILE *fp)
{
    char pb[4000];
    char namebuf[UNAME_BUFSIZE];
    unsigned part1, part2;
    int nb;
    SECStatus rv;
    SECItem eb;
    SECItem data;
    char ob[4000];
    char digest[32];
    RC4Context *rc4;
    RSAContext *rsa;
    SECPublicKey *k;
    MD5Context *md5;
    unsigned char *cp;

    nb = fread(pb, 1, sizeof(pb), fp);
    if (nb <= 0) return 0;

    /* Get system information */
    ZERO(namebuf, sizeof(namebuf));
    rv = GetSysInfo(namebuf);
    if(rv)
        return 0;

    /* Get public key */
    k = (SECPublicKey*) DS_Zalloc(sizeof(SECPublicKey));
    if (!k) return 0;
    rv = Copy(&k->modulus, 3);
    if (rv) return 0;
    rv = Set(&k->publicExponent, 3);
    if (rv) return 0;

    /* Format an encryption block */
    data.data = (unsigned char*) namebuf;
    data.len = sizeof(namebuf);
    rv = RSA_FormatBlock(&eb, SEC_PublicModulusLen(k), 0x01, &data);
    if (rv) return 0;

    /* Encrypt the encryption block */
    rsa = RSA_CreateContext(0, k);
    if (!rsa) return 0;
    rv = RSA_Encrypt(rsa, ob, &part1, sizeof(ob), eb.data, eb.len);
    if (rv) return 0;
    rv = RSA_End(rsa, ob + part1, &part2, sizeof(ob) - part1);
    if (rv) return 0;

    /* Hash it down to a key */
    md5 = MD5_NewContext();
    if (!md5) return 0;
    rv = MD5_Begin(md5);
    if (rv) return 0;
    rv = MD5_Update(md5, ob, part1 + part2);
    if (rv) return 0;
    rv = MD5_End(md5, digest, &part1, sizeof(digest));
    if (rv) return 0;

    /* Now make an RC4 key to decrypt the password with */
    rc4 = RC4_CreateContext(digest, part1);
    if (!rc4) return 0;
    rv = RC4_Decrypt(rc4, ob, &part1, sizeof(ob), pb, nb);
    if (rv) return 0;

    /* Return the password */
    cp = (unsigned char*) DS_Alloc(part1+1);
    if (!cp) return 0;
    XP_MEMCPY(cp, ob, part1);
    cp[part1] = 0;
    return cp;
}



/*
** Setup/use a dongle file. The dongle file contains an encrypted version
** of the key file password. The only real security here is security by
** obfuscation, which isn't much at all.
*/
void dongle(char *keyfile_dongle)
{
    FILE *fp;
    SECStatus rv;

    fp = fopen(keyfile_dongle, "r+b");
    if (!fp) {
		/* Generate dongle file */
		for (;;) {
		    password = SEC_GetPassword( stdin, stderr,
		    	"Key File Password: ",SEC_CheckPassword);
		    if (password) break;
		}

		rv = GenerateDongleFile(password, keyfile_dongle);
		if (rv) {
		    servssl_error(LOG_FAILURE, "Can't generate dongle for this machine\n");
		    exit(-1);
		}
		return;
    }

    /* Read existing dongle file and get the password */
    password = (char*) ProcessDongleFile(fp);
    fclose(fp);
}

#endif /* 0 */
