/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/* 
 * cgi-util.c: CGI utilities. See cgiutil.h for documentation
 *
 * Rob McCool
 */

#include <stdarg.h>
#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include "cgiutils/cgi-util.h"
#include "base/shexp.h"

#define NUM_ENTRIES 64
#ifndef PATH_MAX
#define PATH_MAX 256
#endif


void die(char *fmt, ...)
{
    va_list args;

    printf("<TITLE>Error</TITLE><H1>Error</H1>%c", 10);

    va_start(args, fmt);
    vprintf(fmt, args); 
    va_end(args);

    exit(1);
}


int _dumbsort(const void *s1, const void *s2)
{
    return strcmp(*((char **)s1), *((char **)s2));
}

void send_fd(FILE *f, FILE *fd)
{
    char c;

    while (1) {
        c = fgetc(f);
        if(feof(f))
            return;
        fputc(c,fd);
    }
}

int ind(char *s, char c) {
    register int x;

    for(x=0;s[x];x++)
        if(s[x] == c) return x;

    return -1;
}

void escape_shell_cmd(char *cmd) {
    register int x,y,l;

    l=strlen(cmd);
    for(x=0;cmd[x];x++) {
        if(ind("&;`'\"|*?~<>^()[]{}$\\",cmd[x]) != -1){
            for(y=l+1;y>x;y--)
                cmd[y] = cmd[y-1];
            l++; /* length has been increased */
            cmd[x] = '\\';
            x++; /* skip the character */
        }
    }
}

/* Compresses spaces and replaces commas with an or list */
char * compress_and_replace(char *source)  {
    char *answer;
    int hascomma, x, y;
    hascomma = (strchr(source, ',')) ? 1 : 0;

    answer = malloc(strlen(source));

    for(x=0, y=0; source[x]; x++)  {
	if(source[x] == ',')
	     answer[y++] = '|';
	else if(source[x] != ' ')
	     answer[y++] = source[x];
	answer[y] = 0;
    }
    if(!hascomma)
	return(answer);
    else {
	char *newanswer = malloc(strlen(answer)+2);
	sprintf(newanswer, "(%s)", answer);
	return(newanswer);
    }
}

void negate_wildcard(char **string)
{
    if(*string == NULL)
        return;

    if(((*string)[0] == '*') && ((*string)[1] == '~'))  {
        (*string) += 2;
    }  else  {
        char *t = malloc(strlen(*string) + 3);
        sprintf(t, "*~%s", (*string));
        *string = t;
    }
}

void validate_wildcard(char *wildcard)  
{
    char line[1024];

    if(!wildcard) 
        return;
    if(shexp_valid(wildcard) == VALID_SXP)
        return;

    sprintf(line, "The wildcard pattern <b>%s</b> is not a valid wildcard "
                  "pattern.  <p>\n "
                  "Also, do not mix comma separated lists and wildcard "
                  "patterns - you can use one or the other, but not both at "
                  "once.\n", wildcard);
                  
    die(line);
}

char *fix_hn_exp(char *exp)  {
    char *answer;

    if(!exp)
        return NULL;

    if((strchr(exp, ' ')) && (!strchr(exp, ',')))
        die("You should use commas to separate hosts, not spaces.");

    if((exp[0] == '*') && (exp[1] == '~'))  {
        negate_wildcard(&exp);
        answer = compress_and_replace(exp);
    }  else  {
        answer = compress_and_replace(exp);
        negate_wildcard(&answer);
    }
    validate_wildcard(answer);
 
    return answer;
}

/* ------------------------ Post handler functions ------------------------ */


static int cl;


void post_init(FILE *f)
{
    register int x;
    register char *t;

    if(strcmp(getenv("REQUEST_METHOD"),"POST")) {
        printf("Content-type: text/html\n\n");
        printf("<TITLE>Sorry</TITLE><H1>Sorry</H1>\n");
        printf("This script should be referenced with a method of POST.\n");
        printf("You should access it only through the proper form.\n");
        printf("You should not reload this page either.\n");
        exit(1);
    }
    if(!(t = getenv("CONTENT_TYPE"))) {
        printf("Content-type: text/html\n\n");

        printf("No content-type given. Your browser is Wrong.\n");
        exit(1);
    }
    if(strcmp(t,"application/x-www-form-urlencoded")) {
        printf("Content-type: text/html\n\n");

        printf("This script can only be used to decode form results. \n");
        exit(1);
    }
    if(!(t = getenv("CONTENT_LENGTH"))) {
        printf("Content-type: text/html\n\n");

        printf("No content-length given. Your browser is Wrong.\n");
        exit(1);
    }

    for(x=0; t[x]; x++) {
        if(!isdigit(t[x])) {
            printf("Content-type: text/html\n\n");

            printf("Your browser gave a letter in content-length.\n");
            printf("Your browser is very Wrong.\n");
            exit(1);
        }
    }
    cl = atoi(t);

    if(cl > MAX_CONTENT_LEN) {
        printf("Content-type: text/html\n\n");

        printf("You're going to spew %d bytes of data and I'm not going to listen.\n", cl);
        exit(1);
    }
}

char unescape(FILE *f) 
{
    char digit;
    char what[2];
    int i;

    if( (i = fgetc(f)) == -1) {
        /* Grumble grumble. */
        return '%';
    }
    what[0] = (char) i;
    if( (i = fgetc(f)) == -1) {
        /* Grumble grumble. */
        return '%';
    }
    what[1] = (char) i;

    digit = (what[0] >= 'A' ? ((what[0] & 0xdf) - 'A')+10 : (what[0] - '0'));
    digit *= 16;
    digit += (what[1] >= 'A' ? ((what[1] & 0xdf) - 'A')+10 : (what[1] - '0'));
    return(digit);
}


int post_next(char *name, char *value, FILE *f)
{
    register int i, x;
    int norv;
    char c;

    norv = 0;
    x = 0;
    while(cl) {
        if( (i = fgetc(f)) == -1)
            break;

        if(norv) {
            if((char) i == '&') {
                value[x] = '\0';
                --cl;
                return 0;
            }
        }
        else {
            if((char) i == '=') {
                name[x] = '\0';
                x = 0;
                norv = 1;
                --cl;
                continue;
            }
        }

        if((char) i == '%') {
            c = unescape(f);
            cl -= 2;
        }
        else if((char) i == '+')
            c = ' ';
        else
            c = (char) i;
        if(norv)
            value[x++] = c;
        else
            name[x++] = c;
        --cl;
    }
    if(norv)
        value[x] = '\0';
    else
        name[0] = '\0';
    return 1;
}

/* Lists all files in a directory. */
char **list_directory(char *path, int dashA)  
{
    char **ar;
    DIR *ds;
    struct dirent *d;
    int n, p;

    n = NUM_ENTRIES;
    p = 0;

    ar = (char **) malloc(n * sizeof(char *));

    if(!(ds = opendir(path))) {
        return NULL;
    }

    while( (d = readdir(ds)) ) {
        if ( ( d->d_name[0] != '.' ) ||
	     ( dashA && d->d_name[1] &&
	       ( d->d_name[1] != '.' || d->d_name[2] ) ) ) {
            if(p == (n-1)) {
                n += NUM_ENTRIES;
                ar = (char **) realloc(ar, n*sizeof(char *));
            }
            /* 2: Leave space to add a trailing slash later */
            ar[p] = (char *) malloc(strlen(d->d_name) + 2);
            strcpy(ar[p++], d->d_name);
        }
    }
    closedir(ds);

    qsort((void *)ar, p, sizeof(char *), _dumbsort);
    ar[p] = NULL;

    return ar;
}

