/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * Utilities for dealing with httpd config files etc.
 * 
 * Rob McCool
 */


/* From magnus */
#include "frame/objset.h"
#include "frame/req.h"
#include "base/file.h"
#include "base/buffer.h"
#include "frame/conf.h"
#include "frame/func.h"

#define TECH_MAXLINES 1024

#include "cgiutils/ns-util.h"
#include "cgiutils/cgi-util.h"

#include <stdio.h>


struct FuncStruct func_standard[] = {
    {NULL,NULL,NULL}
};

extern const char *directive_num2name(int num);

void dump_database(httpd_objset *os)
{
    register int x, y, z;
    pb_param *pp;
    dtable *dt;
    const char *u;
    char *t;
    FILE *f;

    if(!(f = fopen(OBJ_DATABASE, "w")))
        die("Can't open object database for writing.");

    fputs("# This file was automatically generated by the server.\n", f);
    fputs("# Edit at your own risk.\n\n", f);

    for(x = 0; x < os->pos; x++) {
        t = pblock_pblock2str(os->obj[x]->name, NULL);
        fprintf(f, "<Object %s>\n", t);
        free(t);
        for(y = 0; y < os->obj[x]->nd; y++) {
            dt = &os->obj[x]->dt[y];
            u = directive_num2name(y);
            for(z = 0; z < dt->ni; z++) {
                if(dt->inst[z].client) {
                    pp = pblock_remove("refcount", dt->inst[z].client);
                    t = pblock_pblock2str(dt->inst[z].client, NULL);
                    if(pp) pblock_pinsert(pp, dt->inst[z].client);
                    fprintf(f, "<Client %s>\n", t);
                    free(t);
                }
                t = pblock_pblock2str(dt->inst[z].param, NULL);
                fprintf(f, "%s %s\n", u, t);
                free(t);
                if(dt->inst[z].client)
                    fputs("</Client>\n", f);
            }
        }
        fputs("</Object>\n\n", f);
    }
    fclose(f);
}


httpd_objset *read_config()
{
    char errstr[MAGNUS_ERROR_LEN];
    httpd_objset *os;
    filebuf_t *buf;
    SYS_FILE fd;

    if((fd = system_fopenRO(OBJ_DATABASE)) == SYS_ERROR_FD)
        die("Can't open object database.\n");
    if(!(buf = filebuf_open(fd, FILE_BUFFERSIZE)))
        die("Can't open buffer from object file.\n");
    if(!(os = objset_scan_buffer(buf, errstr, NULL)))
        die("objset_scan returned NULL. Object file corrupt...\n");

    filebuf_close(buf);
    return os;
}


void insert_ntrans(pblock *p, pblock *c, httpd_object *obj)
{
    register dtable *d;
    register int x, y;
    register char *t;
    int len;
    directive i;
    extern int directive_name2num(char *dir);

    d = &obj->dt[directive_name2num("NameTrans")];
    len = strlen(pblock_findval("from", p));

    for(x = 0; x < d->ni; x++) {
        if((!(t = pblock_findval("from", d->inst[x].param))) ||
           (strcmp(pblock_findval("fn", d->inst[x].param), "pfx2dir")))
            break;
        else
            if(strlen(t) < len)
                break;
    }
    object_add_directive(directive_name2num("NameTrans"), p, c, obj);
    i = d->inst[d->ni - 1];

    for(y = d->ni - 1; y > x; y--)
        d->inst[y] = d->inst[y-1];

    d->inst[x] = i;
}

extern int directive_name2num(char *dir);

void insert_alias(pblock *p, pblock *c, httpd_object *obj)
{
    register dtable *d;
    register int y;
    directive i;

    d = &obj->dt[directive_name2num("NameTrans")];
    object_add_directive(directive_name2num("NameTrans"), p, c, obj);
    i = d->inst[d->ni - 1];

    for(y = d->ni - 1; y; y--)
        d->inst[y] = d->inst[y-1];

    d->inst[0] = i;
}



char **scan_tech(char *fn)
{
    FILE *f;
    char **ret, buf[CONF_MAXLEN];
    register int x, l;

    if(!(f = fopen(fn, "r")))
        return NULL;

    ret = (char **) malloc(TECH_MAXLINES*sizeof(char *));
    x = 0;
    /* leave space at end for routine to insert a line */
    for(x = 0; x < (TECH_MAXLINES - 1); x++) {
        if(!(fgets(buf, CONF_MAXLEN, f)))
            break;
        /* F*** stdio. */
        l = strlen(buf) - 1;
        while(l && isspace(buf[l])) --l;
        if(l) buf[l+1] = '\0';
        ret[x] = strdup(buf);
    }
    fclose(f);
    ret[x] = NULL;
    return ret;
}


int write_tech(char *fn, char **lines)
{
    FILE *f;
    int x;

    if(!(f = fopen(fn, "w")))
        return -1;

    for(x = 0; lines[x]; x++)
        fprintf(f, "%s\n", lines[x]);

    fclose(f);
    return 0;
}


httpd_object *findliteralppath(char *qs, httpd_objset *os)
{
    register int x;
    register char *t;

    for(x = 0; x < os->pos; x++) {
        if( (t = pblock_findval("ppath", os->obj[x]->name)) )
            if(!strcmp(t, qs))
                return os->obj[x];
    }
    return NULL;
}

httpd_object *qs2obj(char *qs, httpd_objset *os)
{
    httpd_object *obj;
    pblock *pb;

    if(qs[0] == 't') {
        if(!(obj = objset_findbyname(&qs[1], NULL, os))) {
            pb = pblock_create(PARAMETER_HASH_SIZE);
            pblock_nvinsert("name", &qs[1], pb);
            return objset_new_object(pb, os);
        }
        else return obj;
    }
    else if(qs[0] == 'p') {
        if( (obj = findliteralppath(&qs[1], os)) )
            return obj;
        pb = pblock_create(PARAMETER_HASH_SIZE);
        pblock_nvinsert("ppath", &qs[1], pb);
        return objset_new_object(pb, os);
    }
    else die("Invalid query string, something went wrong.");
    return 0;
}
