/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#include	"sys/types.h"
#ifndef UNIXWARE /* conflicts with stdlib.h */
#include	"malloc.h" 
#endif
#include	"netsite.h"
#include	<base/util.h>
#include	<base/file.h>
#include	<base/session.h>
#include	<base/plist.h>
#include	<base/ereport.h>
#include 	<libaccess/acl.h>
#include	<libaccess/aclproto.h>
#include 	<libaccess/nserror.h>
#include	<libaccess/nsauth.h>
#include 	<libaccess/las.h>
#include	<libaccess/aclglobal.h>
#include 	<libaccess/dbtlibaccess.h>
#include	<frame/req.h>
#include	<frame/acl.h>
#include	<frame/aclinit.h>
#include	<frame/log.h>

/* For ForceAuth and CacheFlush */
#include	<frame/http.h>
#include	<frame/conf.h>

#ifdef MCC_ADMSERV
NSPR_BEGIN_EXTERN_C
static struct program_groups	none = { NULL, NULL, NULL };
NSAPI_PUBLIC struct program_groups	*ADM_programGroups = &none;
NSPR_END_EXTERN_C
#endif

/*	Kludge to pass the clauth and session stuff into the ACL code until the
 *  Property List code is ready.  Only meant to be called from the Web Server,
 */
int
ACL_StubStuffSession(ACLEvalHandle_t *acleval, Session *sn, Request *rq, const
char *user)
{
    PList_t resource=NULL;
    int	rv;

    /*  If there's already a Subject PList, skip  */
    if (!sn->subject) {
        /*  Allocate new PList  */
        sn->subject = PListCreate(sn->pool, ACL_ATTR_INDEX_MAX, NULL, NULL);
        if (sn->subject == NULL) {
            ereport(LOG_SECURITY, 
	      "Unable to allocate Subject property list.\n");
            return -1;
        }
        /*  Set the Session pointer  */
        rv = PListInitProp(sn->subject, ACL_ATTR_SESSION_INDEX, ACL_ATTR_SESSION, sn, 0);
        if (rv < 0) {
            ereport(LOG_SECURITY, 
	      "Unable to set session ptr in Subject property list - error=%d\n",
	      rv);
            return -1;
        }
    }

    /*  Might as well set the user ID if we know it  */
    if (user) {
        PListInitProp(sn->subject, ATTR_USER_INDEX, ATTR_USER, pool_strdup(sn->pool, user), 0);
    } else if (((ClAuth_t *)sn->clauth) &&
               ((ClAuth_t *)sn->clauth)->cla_uoptr &&
               ((ClAuth_t *)sn->clauth)->cla_uoptr->uo_name) {
        PListInitProp(sn->subject, ATTR_USER_INDEX, ATTR_USER, pool_strdup(sn->pool, (const
	char *)((ClAuth_t *)sn->clauth)->cla_uoptr->uo_name), 0);
    }

    /*  Attach the property list to the Evaluation context  */
    ACL_EvalSetSubject(acleval, sn->subject);

    /*  Cannot be a resource plist yet, that's object specific  */
    resource = PListCreate(sn->pool, ACL_ATTR_INDEX_MAX, NULL, NULL);
    if (resource == NULL) {
	ereport(LOG_SECURITY, "Unable to allocate Resource property list.\n");
	return -1;
    }

    rv = PListInitProp(resource, ACL_ATTR_REQUEST_INDEX, ACL_ATTR_REQUEST, rq, 0);
    if (rv < 0) {
	ereport(LOG_SECURITY, 
	  "Unable to set request ptr in Resource property list - error=%d\n", 
	  rv);
	PListDestroy(resource);
	return -1;
    }

#ifdef MCC_ADMSERV
/*	FIGURE OUT WHICH PRODUCT THIS IS AND SELECT THE PROGRAM GROUP LIST */
    {
      pb_param	*prodParam;
      char	*product;
      int	i;
      int	foundGroups = 0;

      prodParam = pblock_find( "product", rq->vars );
      product = STRDUP( prodParam->value );
      *(strchr( product, '-' )) = '\0';	/* trim off non server-type */
      for ( i = 0 ; ADM_programGroups[i].type ; ++i ) {
	if ( strcmp( product, ADM_programGroups[i].type ) == 0 ) {
	  foundGroups = 1;
	  break;
	}
      }
      FREE( product );
      if ( foundGroups ) {
	rv = PListInitProp(resource, ACL_ATTR_PROGRAMS_INDEX, ACL_ATTR_PROGRAMS,
			   ADM_programGroups + i, 0);
	if (rv < 0) {
	  ereport(LOG_SECURITY, 
		  "Unable to get program group in Resource property list - error=%d\n", 
		  rv);
	  PListDestroy(resource);
	  return -1;
	}
      }
    }
#endif

   /*  Attach the property lists to the Evaluation context  */
    ACL_EvalSetResource(acleval, resource);

    return 0;
}

NSAPI_PUBLIC int
ACL_SetupEval(ACLListHandle_t *acllist, Session *sn, Request *rq, char **rights, char **map_generic, const char *user)
{
    int rv;
    NSErr_t     errp;
    ACLEvalHandle_t *acleval;
    char *bong;
    char *bong_type;
    char *acl_tag;
    int  expr_num;
    char *user_str;
    pb_param *path;

    memset((void *)&errp, 0, sizeof(NSErr_t));

    /* Create an ACL evaluation context */
    acleval = ACL_EvalNew(NULL);
    rv = ACL_EvalSetACL(acleval, acllist);
    rv = ACL_StubStuffSession(acleval, sn, rq, user);

    /* Check access */
    rv = ACL_EvalTestRights(&errp, acleval, rights, map_generic, 
                            &bong_type, &bong, &acl_tag, &expr_num);

    if ( rv != ACL_RES_ALLOW && 
	 rq->status_num != PROTOCOL_UNAUTHORIZED && !user ) {
        path = pblock_find("path", rq->vars);
        /* log the denial */
        log_error(LOG_SECURITY, "acl-state", sn, rq,
                  "access of %s denied by ACL %s directive %d",
                  path->value, acl_tag, expr_num);
    }

    if ( rv == ACL_RES_DENY && bong && 
         rq->status_num != PROTOCOL_UNAUTHORIZED &&  !user) {
    
        if ( strcmp(bong_type, "url") == 0 ) {
            protocol_status(sn, rq, PROTOCOL_REDIRECT, NULL);
            pblock_nvinsert("url", bong, rq->vars);
        } else if ( strcmp(bong_type, "uri") == 0 ) {
            char *newloc = http_uri2url_dynamic(bong, "", sn, rq);	
            protocol_status(sn, rq, PROTOCOL_REDIRECT, NULL);
            pblock_nvinsert("url", newloc, rq->vars);
            FREE(newloc);
        } 
    
#if defined(MCC_ADMSERV) || defined(MCC_PROXY)

/* 
 * This is not very secure, since it doesn't go through ACL
 * processing.  ADMSERV and PROXY need it though.  Don't
 * compile it into the web server.
 */

        else if ( strcmp("file", bong_type) == 0 ) {
            struct stat finfo;
#ifdef MCC_PROXY
            pb_param *escp = pblock_find("escaped", rq->vars);
#endif
    

            if ( stat(bong, &finfo) ) {
                log_error(LOG_MISCONFIG, "acl-state", sn, rq,
                    "access denied response file %s is unreadable",
                    bong);
            } else {
                pb_param *type = pblock_find("content-type", rq->srvhdrs);
/*
 * Clear the ACL state for the request so it doesn't get
 * applied to the bong file.
 */
                rq->aclstate = 0;
                rq->acldirno = 0;
                rq->aclname = 0;
                rq->aclpb = 0;
    
                FREE(path->value);
                path->value = STRDUP(bong);
    
#ifdef MCC_ADMSERV
                /* Set the content type to text/html */
                if ( type )  {
                    FREE(type->value);
                    type->value = STRDUP("text/html");
                }  else  {
                    pblock_nvinsert("content-type", "text/html", 
		                rq->srvhdrs);
                }
#endif

#ifdef MCC_PROXY
                if( escp ) {
                    FREE(escp->value);
                    escp->value = STRDUP(bong);
                }
#endif
                rv = ACL_RES_ALLOW;
            }
        }
#endif

        else { 
            log_error(LOG_MISCONFIG, "check-acl", sn, rq,
		          "access denied response type %s is undefined",
		           bong_type);
            protocol_status(sn, rq, PROTOCOL_SERVER_ERROR, NULL);
        } 

    }
    
    PListDestroy(ACL_EvalGetResource(acleval));
    user_str = (char *)PListDeleteProp(sn->subject, ATTR_USER_INDEX, ATTR_USER);
    if (user)
	pool_free(sn->pool, (void *)user);

    ACL_EvalDestroy(NULL, acleval);
    nserrDispose(&errp);
 
    return (rv);
}

/*  ACL_AgentEval
 *  INPUT
 *      uri
 *      sn
 *      rq
 *      user
 *      rights
 *  OUTPUT
 *      retcode ACL_RES_ALLOW or else failure/denial.
 */
NSAPI_PUBLIC int
ACL_AgentInvokeEval(char *uri, char *path, Session *sn, Request *rq, char *user, char **rights, char **map_generic)
{

    /*  Find the applicable ACL  */
    rq->acllist = NULL;
    if (!ACL_CacheCheck(uri, &rq->acllist)) {
        ACL_GetPathAcls(uri, &rq->acllist, "uri=");
        ACL_GetPathAcls(path, &rq->acllist, "path=");
        ACL_CacheEnter(uri, &rq->acllist);
    }
 
    if (rq->acllist != ACL_LIST_NO_ACLS) {     /* Any ACLs to begin with? */
        char *      rights[2];      /* access right name */
 
        /* Look up the needed access right by name */
 
	rq->request_is_cacheable = 0;
	return(ACL_SetupEval(rq->acllist, sn, rq, rights, map_generic, user));
 
    }
 
    return ACL_RES_ALLOW;
}
 
/*  ACL_AgentEval
 *  INPUT
 *      file
 *      user
 *      rights
 *  OUTPUT
 *      retcode ACL_RES_ALLOW or else failure/denial.
 */
NSAPI_PUBLIC int
ACL_AgentRuntimeEval(char *file, char *path, char *user, char **rights, char **map_generic)
{
    ACLListHandle_t     *acllist;
    Session *sn;
    int rv;

    sn = (Session *)CALLOC(sizeof(Session));
 
    /*  Find the applicable ACL  */
    if (!ACL_CacheCheck(file, &acllist)) {
        ACL_GetPathAcls(file, &acllist, "uri=");
        ACL_GetPathAcls(path, &acllist, "path=");
        ACL_CacheEnter(file, &acllist);
    }
 
    if (acllist != ACL_LIST_NO_ACLS) {     /* Any ACLs to begin with? */
        char *      rights[2];      /* access right name */

	sn->pool = pool_create();
 
	rv = ACL_SetupEval(acllist, sn, NULL, rights, map_generic, user);

	pool_destroy(sn->pool);
        FREE(sn);
        return rv;
 
    }
 
    FREE(sn);
    return ACL_RES_ALLOW;
}
 

NSAPI_PUBLIC int
ACL_SearchRuntimeEval(char *file, char *path, char *user, char **rights, char **map_generic, Session* sn, Request* rq)
{
    ACLListHandle_t     *acllist;
    int rv;

    /*  Find the applicable ACL  */
    if (!ACL_CacheCheck(file, &acllist)) {
        ACL_GetPathAcls(file, &acllist, "uri=");
        ACL_GetPathAcls(path, &acllist, "path=");
        ACL_CacheEnter(file, &acllist);
    }
 
    if (acllist != ACL_LIST_NO_ACLS) {     /* Any ACLs to begin with? */
	rv = ACL_SetupEval(acllist, sn, rq, rights, map_generic, user);

        if ( rv != ACL_RES_ALLOW ) {
          protocol_start_response(sn, rq);
        }

        return rv;
 
    }
 
    return ACL_RES_ALLOW;
}
 

static ACLHandle_t *
acl_create_forceauth_acl()
{
ACLHandle_t *acl;
ACLExprHandle_t *expr = NULL;
int error = 0;

    acl = ACL_AclNew("forceauth");
    if ( acl == NULL ) {
	ereport(LOG_SECURITY, 
	       "unable to create default forceauth ACL.\n");
        return(NULL);
    }

    expr = ACL_ExprNew(ACL_EXPR_TYPE_ALLOW);
    if ( expr == NULL ) {
	ereport(LOG_SECURITY, 
	       "unable to create forceauth expression.\n");
        error++;
    } else if ( ACL_ExprAppend(NULL, acl, expr) < 0 ) {
        /* 
         * this should never happen 
         */
	ereport(LOG_SECURITY, 
	       "unable to append forceauth expression to ACL.\n");
        ACL_ExprDestroy(expr);
        error++;
    } else if ( ACL_ExprTerm(NULL, expr, "user", CMP_OP_EQ, "all") < 0 ) {
	ereport(LOG_SECURITY, 
	       "unable to create forceauth expression.\n");
        error++;
    } else if ( ACL_ExprAddArg(NULL, expr, "http_get") < 0 ) {
	ereport(LOG_SECURITY, 
	       "unable to create forceauth expression.\n");
        error++;
    } else if ( ACL_ExprAddArg(NULL, expr, "http_post") < 0 ) {
	ereport(LOG_SECURITY, 
	       "unable to add right to forceauth expression.\n");
        error++;
    } else if ( ACL_ExprAddArg(NULL, expr, "http_index") < 0 ) {
	ereport(LOG_SECURITY, 
	       "unable to add right to forceauth expression.\n");
        error++;
    }

    if ( error && acl ) {
        ACL_AclDestroy(acl);
        acl = NULL;
    }

    return(acl);
    
}

/*  ACL_ForceAuth
 *  INPUT
 *  OUTPUT
 *	rv	REQ_PROCEED if the acl succeeds, else REQ_ABORTED.
 */
NSAPI_PUBLIC int
ACL_ForceAuth(Session *sn, Request *rq)
{
    NSErr_t errp;
    int rv;
    char *rights[4];
    ACLHandle_t *acl;
    static ACLListHandle_t *ACLForceAuthList = NULL;

    /*  Locate the special ACL List  */
    if ( ACLForceAuthList == NULL ) {
	ACLForceAuthList = ACL_ListNew();
	if ( ACLForceAuthList  == NULL ) {
	    ereport(LOG_SECURITY, 
                    "unable to allocate forceauth ACL List.\n");
            return REQ_ABORTED;
	}

	acl = ACL_ListFind((ACLListHandle_t *)acl_root_30, "forceauth",
	ACL_CASE_INSENSITIVE);
	if  ( acl == NULL ) {
            acl = acl_create_forceauth_acl();
            if ( acl == NULL ) {
	        ereport(LOG_SECURITY, 
                        "unable to create default forceauth ACL List.\n");
                ACL_ListDestroy(ACLForceAuthList);
                ACLForceAuthList = NULL;
                return REQ_ABORTED;
            }
	}
	ACL_ListAppend(&errp, ACLForceAuthList, acl, 0);
    }

    /*  Evaluate it  */
    rights[0] = "http_get";
    rights[1] = "http_post";
    rights[2] = "http_index";
    rights[3] = NULL;

    rv = ACL_SetupEval(ACLForceAuthList, sn, rq, rights, NULL, NULL);

    /*  Process the result  */
    if ( rv != ACL_RES_ALLOW ) {
        if (!pblock_findval("status", rq->srvhdrs)) {
            /* Set the status only if it is not already set */
            protocol_status(sn, rq, PROTOCOL_NOT_FOUND, NULL);
        }
        return REQ_ABORTED;
    }

    return REQ_PROCEED;
}


int
LASIpGetter(NSErr_t *errp, PList_t subject, PList_t resource, PList_t
           auth_info, PList_t global_auth)
{
    Session *sn=NULL;
    int rv;
    IPAddr_t ip;

    rv = PListGetValue(subject, ACL_ATTR_SESSION_INDEX, (void **)&sn, NULL);
    if (rv < 0) {
        ereport(LOG_SECURITY, "LASIpGetter unable to get session address %d\n", 
		rv);
        return LAS_EVAL_FAIL;
    }

    ip    = ((ClAuth_t *)sn->clauth)->cla_ipaddr;

    rv = PListInitProp(subject, ACL_ATTR_IP_INDEX, ACL_ATTR_IP, (void *)ip, NULL);
    if (rv < 0) {
        ereport(LOG_SECURITY, "LASIpGetter unable to save ip address %d\n", rv);
        return LAS_EVAL_FAIL;
    }

    return 0;
}


int 
LASDnsGetter(NSErr_t *errp, PList_t subject, PList_t resource, PList_t
           auth_info, PList_t global_auth)
{
    Session *sn=NULL;
    char *my_dns=NULL;
    int rv;

    rv = PListGetValue(subject, ACL_ATTR_SESSION_INDEX, (void **)&sn, NULL);
    if (rv < 0) {
        ereport(LOG_SECURITY, "LASDnsGetter unable to get session address %d\n", 
		rv);
        return LAS_EVAL_FAIL;
    }

    my_dns = ((ClAuth_t *)sn->clauth)->cla_dns;
    if (!my_dns) {
        my_dns = session_dns_lookup(sn, 0);
        if (!my_dns || !*my_dns) {
            my_dns = "unknown";
        }
    }

    rv = PListInitProp(subject, ACL_ATTR_DNS_INDEX, ACL_ATTR_DNS, my_dns, NULL);
    if (rv < 0) {
        ereport(LOG_SECURITY, "LASDnsGetter unable to save DNS address %d\n", rv);
        return LAS_EVAL_FAIL;
    }

    return 0;
}
static int 
ACL_CacheFlushFrame(void)
{
    filebuf_t *buf;
    SYS_FILE fd;
    int ln = 0;
    char line[CONF_MAXLEN];

#ifdef	FLUSHDATABASES
    ACL_Init2();
    ACL_InitFrame();
    ACL_InitSafs();
#endif

    acl_root_30 = NULL;

    if ((fd = system_fopenRO(ACLGlobal->conffilename)) == SYS_ERROR_FD) {
	ereport(LOG_SECURITY, XP_GetAdminStr(DBT_aclcacheflushCannotOpenMagnus), ACLGlobal->conffilename);
	return -1;
    }
    buf = filebuf_open(fd, FILE_BUFFERSIZE);
    if (!buf) {
	ereport(LOG_SECURITY, XP_GetAdminStr(DBT_aclcacheflushCannotOpenMagnus), ACLGlobal->conffilename);
	return -1;
    }

    while(1) {
        switch((util_getline(buf, 0, CONF_MAXLEN, line))) {
          case -1:
            /* error case - util_getline returns the error string */
            filebuf_close(buf);
            break;
          case 1:
            /* EOF case - done */
            filebuf_close(buf);
            return 0;
          default:
            /* Normal case */
	    {
	        char *v=NULL, *d = line;
	     
	        if((*d) == '#')
	            break;
	     
	        while((*d) && (isspace(*d))) ++d;
	        if(!(*d))
	    	    break;
	     
	        for(v = d; (*v) && (*v != ' '); ++v);
	        if(!(*v))
	    	    goto noval;
	     
	        *v++ = '\0';
	        while((*v) && isspace(*v)) ++v;
	     
	        if(!(*v)) {
noval:
		    ereport(LOG_SECURITY, XP_GetAdminStr(DBT_aclcacheflushCannotOpenMagnus), ACLGlobal->conffilename);
	    	    return -1;
	        }

		    if(!strcasecmp(d, "ACLFile")) {
			NSErr_t err = NSERRINIT;
			ACLListHandle_t *tmp_acl_root;
		 
			/* Read the specified file and create ACL data structures */
			tmp_acl_root = ACL_ParseFile(&err, v);
			if (tmp_acl_root == NULL) {
			    /* generate error message into l */
		            ereport(LOG_SECURITY, XP_GetAdminStr(DBT_aclcacheflushCannotParseFile), ACLGlobal->conffilename);
			    return(-1);
			}
		 
			if (acl_root_30 == NULL) {
			    acl_root_30 = tmp_acl_root;
			} else {
			    if (ACL_ListConcat(&err, (ACLListHandle_t *) acl_root_30,
						tmp_acl_root, 0) < 0) {
		                ereport(LOG_SECURITY, XP_GetAdminStr(DBT_aclcacheflushCannotConcatList), ACLGlobal->conffilename);
				return(-1);
			    }
			    ACL_ListDestroy(tmp_acl_root);
			}
		 
			/* Just in case ... */
			nserrDispose(&err);
		    }

            break;
	    }
        }
    }


#ifdef notdef
    ACL_InitPostMagnus();
    ACL_InitFramePostMagnus();
    ACL_InitSafsPostMagnus();
#endif

    return 0;
    
}

NSAPI_PUBLIC void
ACL_InitFrame(void)
{
    MethodInfo_t *info;

    if (ACL_FindMethod(NULL, METHOD_DEFAULT, &info) == LAS_EVAL_TRUE) {
	ACL_RegisterAttrGetter(NULL, info, ATTR_IP, LASIpGetter);
	ACL_RegisterAttrGetter(NULL, info, ATTR_DNS, LASDnsGetter);
    }

    ACL_CacheFlushRegister(ACL_CacheFlushFrame);

    return;
}

NSAPI_PUBLIC void
ACL_InitFramePostMagnus(void)
{
    return;
}

