/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#include	"sys/types.h"
#include	<limits.h>
#ifndef UNIXWARE /* conflicts with stdlib.h */
#include	"malloc.h" 
#endif
#include	"netsite.h"
#ifdef	XP_WIN32
#include	"winsock.h"
#else
#include	<arpa/inet.h>
#endif
#include	<base/util.h>
#include	<base/file.h>
#include	<base/session.h>
#include	<base/plist.h>
#include	<base/ereport.h>
#include 	<libaccess/acl.h>
#include	<libaccess/aclproto.h>
#include 	<libaccess/nserror.h>
#include	<libaccess/nsauth.h>
#include 	<libaccess/las.h>
#include	<libaccess/aclglobal.h>
#include 	<libaccess/dbtlibaccess.h>
#include	<libaccess/aclerror.h>
#include	<httpdaemon/multihome.h>
#include	<frame/req.h>
#include	<frame/acl.h>
#include	<frame/aclinit.h>
#include	<frame/log.h>
#include	<frame/httpact.h>
#include	<frame/http.h>
#include	<frame/conf.h>
#include	<frame/conf_api.h>
#include	<frame/accel_file_cache.h>

#ifdef MCC_ADMSERV
NSPR_BEGIN_EXTERN_C
static struct program_groups	none = { NULL, NULL, NULL };
NSAPI_PUBLIC struct program_groups	*ADM_programGroups = &none;
NSPR_END_EXTERN_C
#endif

ACLListHandle_t *ACLForceAuthList = NULL;
static ACLListHandle_t *new_acl_root;


/*	Kludge to pass the clauth and session stuff into the ACL code until the
 *  Property List code is ready.  Only meant to be called from the Web Server,
 */
int
ACL_StubStuffSession(ACLEvalHandle_t *acleval, Session *sn, Request *rq, const
char *user)
{
    PList_t resource=NULL;
    int	rv;
    NSErr_t err = NSERRINIT;
    NSErr_t *errp = &err;

    /*  If there's already a Subject PList, skip  */
    if (!sn->subject) {
        /*  Allocate new PList  */
        sn->subject = PListCreate(sn->pool, ACL_ATTR_INDEX_MAX, NULL, NULL);
        if (sn->subject == NULL) {
            ereport(LOG_SECURITY, 
	      "Unable to allocate Subject property list.\n");
            return -1;
        }
        /*  Set the Session pointer  */
        rv = PListInitProp(sn->subject, ACL_ATTR_SESSION_INDEX, ACL_ATTR_SESSION, sn, 0);
        if (rv < 0) {
            ereport(LOG_SECURITY, 
	      "Unable to set session ptr in Subject property list - error=%d\n",
	      rv);
            return -1;
        }
    } else {
    	/*  Need to clear the owner off an existing subject list */
			PListDeleteProp(sn->subject, ACL_ATTR_IS_OWNER_INDEX, ACL_ATTR_IS_OWNER);
		}

    /*  Might as well set the user ID if we know it  */
    if (user) {
        PListInitProp(sn->subject, ACL_ATTR_USER_INDEX, ACL_ATTR_USER, pool_strdup(sn->pool, user), 0);
		PListInitProp(sn->subject, ACL_ATTR_CACHED_USER_INDEX, ACL_ATTR_CACHED_USER, pool_strdup(sn->pool, user), 0);
    } else if (((ClAuth_t *)sn->clauth) &&
               ((ClAuth_t *)sn->clauth)->cla_uoptr &&
               ((ClAuth_t *)sn->clauth)->cla_uoptr->uo_name) {
        PListInitProp(sn->subject, ACL_ATTR_USER_INDEX, ACL_ATTR_USER, pool_strdup(sn->pool, (const
	char *)((ClAuth_t *)sn->clauth)->cla_uoptr->uo_name), 0);
    }

    /*  Attach the property list to the Evaluation context  */
    ACL_EvalSetSubject(errp, acleval, sn->subject);

    /*  Cannot be a resource plist yet, that's object specific  */
    resource = PListCreate(sn->pool, ACL_ATTR_INDEX_MAX, NULL, NULL);
    if (resource == NULL) {
	ereport(LOG_SECURITY, "Unable to allocate Resource property list.\n");
	return -1;
    }

    rv = PListInitProp(resource, ACL_ATTR_REQUEST_INDEX, ACL_ATTR_REQUEST, rq, 0);
    if (rv < 0) {
	ereport(LOG_SECURITY, 
	  "Unable to set request ptr in Resource property list - error=%d\n", 
	  rv);
	PListDestroy(resource);
	return -1;
    }

#ifdef MCC_ADMSERV
/*	FIGURE OUT WHICH PRODUCT THIS IS AND SELECT THE PROGRAM GROUP LIST */
    {
      pb_param	*prodParam;
      char	*product;
      int	i;
      int	foundGroups = 0;

      prodParam = pblock_find( "product", rq->vars );
      product = STRDUP( prodParam->value );
      *(strchr( product, '-' )) = '\0';	/* trim off non server-type */
      for ( i = 0 ; ADM_programGroups[i].type ; ++i ) {
	if ( strcmp( product, ADM_programGroups[i].type ) == 0 ) {
	  foundGroups = 1;
	  break;
	}
      }
      FREE( product );
      if ( foundGroups ) {
	rv = PListInitProp(resource, ACL_ATTR_PROGRAMS_INDEX, ACL_ATTR_PROGRAMS,
			   ADM_programGroups + i, 0);
	if (rv < 0) {
	  ereport(LOG_SECURITY, 
		  "Unable to get program group in Resource property list - error=%d\n", 
		  rv);
	  PListDestroy(resource);
	  return -1;
	}
      }
    }
#endif

    /*  Attach the property lists to the Evaluation context  */
    ACL_EvalSetResource(errp, acleval, resource);

    return 0;
}

/* rq can be null, in which case skip all the response processing. */
NSAPI_PUBLIC int
ACL_SetupEval(ACLListHandle_t *acllist, Session *sn, Request *rq, char **rights, char **map_generic, const char *user)
{
    int rv;
    NSErr_t     err = NSERRINIT;
    NSErr_t	*errp = &err;
    ACLEvalHandle_t *acleval;
    char *bong;
    char *bong_type;
    char *acl_tag;
    int  expr_num;
    char *user_str = 0;
    char *cached_user = 0;
    pb_param *path;

    /* Create an ACL evaluation context */
    acleval = ACL_EvalNew(errp, NULL);
    rv = ACL_EvalSetACL(errp, acleval, acllist);
    rv = ACL_StubStuffSession(acleval, sn, rq, user);

    /* Check access */
    rv = ACL_EvalTestRights(errp, acleval, rights, map_generic, 
                            &bong_type, &bong, &acl_tag, &expr_num);


    if ( rv == ACL_RES_DENY && rq ) {
	char buf[256];
	char *prompt = 0;
	PList_t resource = ACL_EvalGetResource(errp, acleval);
	int auth_req = PListGetValue(resource, ACL_ATTR_WWW_AUTH_PROMPT_INDEX,
				      (void **)&prompt, NULL);
	if (auth_req >= 0) {
	    if (!prompt) prompt = "unknown prompt";
	    util_snprintf(buf, sizeof(buf), "Basic realm=\"%s\"", prompt);
	    pblock_nvinsert("WWW-authenticate", buf, rq->srvhdrs);
	    protocol_status(sn, rq, PROTOCOL_UNAUTHORIZED, NULL);
	}
    }

    if ( rv != ACL_RES_ALLOW &&  rq  &&
	 rq->status_num != PROTOCOL_UNAUTHORIZED && !user ) {
        path = pblock_find("path", rq->vars);
        /* log the denial */
	ACL_EreportError(errp);
        log_error(LOG_SECURITY, "acl-state", sn, rq,
                  "access of %s denied by ACL %s directive %d",
                  path->value, acl_tag, expr_num);
    }

    if ( rv == ACL_RES_DENY && bong && rq &&
         rq->status_num != PROTOCOL_UNAUTHORIZED &&  !user) {
    
        if ( strcmp(bong_type, "url") == 0 ) {
            protocol_status(sn, rq, PROTOCOL_REDIRECT, NULL);
            pblock_nvinsert("url", bong, rq->vars);
        } else if ( strcmp(bong_type, "uri") == 0 ) {
            char *newloc = http_uri2url_dynamic(bong, "", sn, rq);
            protocol_status(sn, rq, PROTOCOL_REDIRECT, NULL);
            pblock_nvinsert("url", newloc, rq->vars);
            FREE(newloc);
        } 
    
#if defined(MCC_ADMSERV) || defined(MCC_PROXY)

/* 
 * This is not very secure, since it doesn't go through ACL
 * processing.  ADMSERV and PROXY need it though.  Don't
 * compile it into the web server.
 */

        else if ( strcmp("file", bong_type) == 0 ) {
            struct stat finfo;
#ifdef MCC_PROXY
            pb_param *escp = pblock_find("escaped", rq->vars);
#endif
    

            if ( stat(bong, &finfo) ) {
                log_error(LOG_MISCONFIG, "acl-state", sn, rq,
                    "access denied response file %s is unreadable",
                    bong);
            } else {
                pb_param *type = pblock_find("content-type", rq->srvhdrs);
/*
 * Clear the ACL state for the request so it doesn't get
 * applied to the bong file.
 */
                rq->aclstate = 0;
                rq->acldirno = 0;
                rq->aclname = 0;
                rq->aclpb = 0;
    
                FREE(path->value);
                path->value = STRDUP(bong);
    
#ifdef MCC_ADMSERV
                /* Set the content type to text/html */
                if ( type )  {
                    FREE(type->value);
                    type->value = STRDUP("text/html");
                }  else  {
                    pblock_nvinsert("content-type", "text/html", 
		                rq->srvhdrs);
                }
#endif

#ifdef MCC_PROXY
                if( escp ) {
                    FREE(escp->value);
                    escp->value = STRDUP(bong);
                }
#endif
                rv = ACL_RES_ALLOW;
            }
        }
#endif

        else { 
            ACL_EreportError(errp);
            log_error(LOG_MISCONFIG, "check-acl", sn, rq,
		          "access denied response type %s is undefined",
		           bong_type);
            protocol_status(sn, rq, PROTOCOL_SERVER_ERROR, NULL);
        } 

    }
    
    PListDestroy(ACL_EvalGetResource(errp, acleval));
    user_str = (char *)PListDeleteProp(sn->subject, ACL_ATTR_USER_INDEX, ACL_ATTR_USER);
    if (user_str)
		pool_free(sn->pool, (void *)user_str);
    cached_user = (char *)PListDeleteProp(sn->subject, ACL_ATTR_CACHED_USER_INDEX,
 											ACL_ATTR_CACHED_USER);
    if (cached_user)
		pool_free(sn->pool, (void *)cached_user);

    ACL_EvalDestroy(errp, NULL, acleval);
    nserrDispose(errp);
 
    return (rv);
}

/*  ACL_AgentEval
 *  INPUT
 *      uri
 *      sn
 *      rq
 *      user
 *      rights
 *  OUTPUT
 *      retcode ACL_RES_ALLOW or else failure/denial.
 */
NSAPI_PUBLIC int
ACL_AgentInvokeEval(char *uri, char *path, Session *sn, Request *rq, char *user, char **rights, char **map_generic)
{

    /*  Find the applicable ACL  */
    rq->acllist = NULL;
    if (ACL_UriToAclList(uri, sn, &rq->acllist))
	return -1;
 
    if (rq->acllist != ACL_LIST_NO_ACLS) {     /* Any ACLs to begin with? */
        char *      rights[2];      /* access right name */
 
        /* Look up the needed access right by name */
 
	rq->request_is_cacheable = 0;
	return(ACL_SetupEval(rq->acllist, sn, rq, rights, map_generic, user));
 
    }
 
    return ACL_RES_ALLOW;
}
 
/*  ACL_AgentEval
 *  INPUT
 *      file
 *      user
 *      rights
 *  OUTPUT
 *      retcode ACL_RES_ALLOW or else failure/denial.
 */
NSAPI_PUBLIC int
ACL_AgentRuntimeEval(char *uri, char *path, char *user, char **rights, char **map_generic, Session* origSession)
{
    ACLListHandle_t     *acllist;
    Session *sn;
    int rv;

    if (!origSession && !ACLUriCacheOn) /* virtual servers and no sn*/
	return ACL_RES_DENY;
    sn = (Session *)CALLOC(sizeof(Session));
    if (sn) {
       sn->inbuf = origSession->inbuf;
       memcpy((void *)&sn->local_addr, (const void *)&origSession->local_addr, sizeof(sn->local_addr));
       if (ACL_UriToAclList(uri, sn, &acllist)) {
          FREE(sn);
          return -1;
       }
 
       if (acllist != ACL_LIST_NO_ACLS) {     /* Any ACLs to begin with? */

	  sn->pool = pool_create();
 
	  rv = ACL_SetupEval(acllist, sn, NULL, rights, map_generic, user);

	  pool_destroy(sn->pool);
          FREE(sn);
          return rv;
 
       }
    }
 
    FREE(sn);
    return ACL_RES_ALLOW;
}

NSAPI_PUBLIC int
ACL_SearchRuntimeEval(char *uri, char *path, char *user, char **rights, char **map_generic, Session* sn, Request* rq)
{
    ACLListHandle_t     *acllist;
    int rv;

/*
   if (!ACL_CacheCheck(uri, &acllist)) {
        ACL_GetPathAcls(uri, &acllist, "uri=", (ACLListHandle_t *)acl_root_30);
        ACL_GetPathAcls(path, &acllist, "path=", (ACLListHandle_t *)acl_root_30);
        ACL_CacheEnter(uri, &acllist);
    }
*/

    if (ACL_UriToAclList(uri, sn, &acllist))
        return -1;

    if (acllist != ACL_LIST_NO_ACLS) {     /* Any ACLs to begin with? */
	rv = ACL_SetupEval(acllist, sn, rq, rights, map_generic, user);

//       if ( rv != ACL_RES_ALLOW ) {
//          protocol_start_response(sn, rq);
//        }

        return rv;
 
    }
 
    return ACL_RES_ALLOW;
}
 
static ACLListHandle_t *
acl_create_forceauth_acl()
{
    ACLListHandle_t *acl_list;
    int error = 0;
    NSErr_t err = NSERRINIT;
    NSErr_t *errp = &err;
#define FORCEAUTHACL "version 3.0; acl forceauth; \
authenticate (user, group) { \
prompt = \"Enterprise Server\"; \
}; \
allow (http_get, http_post, http_index) (user=all);"
    
    acl_list = ACL_ParseString(errp, FORCEAUTHACL);
    if ( acl_list == NULL ) {
	ereport(LOG_SECURITY, 
	       "unable to create default forceauth ACL.\n");
    }
    return(acl_list);
}


/*  ACL_ForceAuth
 *  INPUT
 *  OUTPUT
 *	rv	REQ_PROCEED if the acl succeeds, else REQ_ABORTED.
 */
NSAPI_PUBLIC int
ACL_ForceAuth(Session *sn, Request *rq)
{
    int rv;
    char *rights[4];

    /*  Locate the special ACL List  */
    if ( ACLForceAuthList == NULL ) {
        ereport(LOG_SECURITY,
                "unable to find forceauth ACL List.\n");
        return REQ_ABORTED;
    }
    /*  Increment the ref_count for the list.  SetupEval decrements it  */
    ACL_ListHashEnter(&ACLForceAuthList);

    /*  Evaluate it  */
    rights[0] = "http_get";
    rights[1] = "http_post";
    rights[2] = "http_index";
    rights[3] = NULL;

    rv = ACL_SetupEval(ACLForceAuthList, sn, rq, rights, NULL, NULL);

    /*  Process the result  */
    if ( rv != ACL_RES_ALLOW ) {
        if (!pblock_findval("status", rq->srvhdrs)) {
            /* Set the status only if it is not already set */
            protocol_status(sn, rq, PROTOCOL_NOT_FOUND, NULL);
        }
        return REQ_ABORTED;
    }

    return REQ_PROCEED;
}


int
LASIpGetter(NSErr_t *errp, PList_t subject, PList_t resource, PList_t
           auth_info, PList_t global_auth, void *arg)
{
    Session *sn=NULL;
    int rv;
    IPAddr_t ip;

    rv = PListGetValue(subject, ACL_ATTR_SESSION_INDEX, (void **)&sn, NULL);
    if (rv < 0) {
        ereport(LOG_SECURITY, "LASIpGetter unable to get session address %d\n", 
		rv);
        return LAS_EVAL_FAIL;
    }

    ip    = ((ClAuth_t *)sn->clauth)->cla_ipaddr;

    rv = PListInitProp(subject, ACL_ATTR_IP_INDEX, ACL_ATTR_IP, (void *)ip, NULL);
    if (rv < 0) {
        ereport(LOG_SECURITY, "LASIpGetter unable to save ip address %d\n", rv);
        return LAS_EVAL_FAIL;
    }

    return LAS_EVAL_TRUE;
}


int 
LASDnsGetter(NSErr_t *errp, PList_t subject, PList_t resource, PList_t
           auth_info, PList_t global_auth, void *arg)
{
    Session *sn=NULL;
    char *my_dns=NULL;
    int rv;
    struct in_addr in;

    rv = PListGetValue(subject, ACL_ATTR_SESSION_INDEX, (void **)&sn, NULL);
    if (rv < 0) {	/* via accel_file_cache */
        rv = PListGetValue(subject, ACL_ATTR_IP_INDEX, (void **)&in.s_addr, NULL);
        if (
            (!(my_dns = net_ip2host(inet_ntoa(in), 0)))  ||
            (!strcmp(my_dns, "-none"))
           ) {
            my_dns = "unknown";
        }
	else {
	    /* Copy my_dns into PList's pool & free the original */
	    char *tmp = my_dns;
	    my_dns = pool_strdup(PListGetPool(subject), tmp);
	    FREE(tmp);
	}
    } else {		/* unaccelerated path */
        my_dns = ((ClAuth_t *)sn->clauth)->cla_dns;
        if (!my_dns) {
            my_dns = session_dns_lookup(sn, 0);
            if (!my_dns || !*my_dns) {
                my_dns = "unknown";
            }
        }
    }

    rv = PListInitProp(subject, ACL_ATTR_DNS_INDEX, ACL_ATTR_DNS, my_dns, NULL);
    if (rv < 0) {
        ereport(LOG_SECURITY, "LASDnsGetter unable to save DNS address %d\n", rv);
        return LAS_EVAL_FAIL;
    }

    return LAS_EVAL_TRUE;
}

char ACLConffilename[REQ_MAX_LINE];

NSAPI_PUBLIC int
ACL_Conffilename(const char *conffilename)
{
    strncpy(ACLConffilename, conffilename, REQ_MAX_LINE-1);
    ACLConffilename[REQ_MAX_LINE-1] = '\0';
    return 0;
}

static char ACLAclfilename[REQ_MAX_LINE]; /* name of the first ACL file in magnus.conf */
static int initialized = 0;

NSAPI_PUBLIC int
ACL_SetAclfilename(const char *aclfilename)
{
    if (!initialized) {
        initialized = 1;
        strncpy(ACLAclfilename, aclfilename, REQ_MAX_LINE-1);
        ACLAclfilename[REQ_MAX_LINE-1] = '\0';
    }
    return 0;
}

NSAPI_PUBLIC const char *
ACL_GetAclfilename(void)
{
    return (const char *)ACLAclfilename;
}



static int 
ACL_CacheFlushFrame(void)
{
    filebuf_t *buf;
    SYS_FILE fd;
    int ln = 0;
    char line[CONF_MAXLEN];

    new_acl_root = NULL;

    if ((fd = system_fopenRO(ACLConffilename)) == SYS_ERROR_FD) {
	ereport(LOG_SECURITY, XP_GetAdminStr(DBT_aclcacheflushCannotOpenMagnus), ACLConffilename);
	return -1;
    }
    buf = filebuf_open(fd, FILE_BUFFERSIZE);
    if (!buf) {
	ereport(LOG_SECURITY, XP_GetAdminStr(DBT_aclcacheflushCannotOpenMagnus), ACLConffilename);
	return -1;
    }

    while(1) {
        switch((util_getline(buf, 0, CONF_MAXLEN, line))) {
          case -1:
            /* error case - util_getline returns the error string */
            filebuf_close(buf);
            break;
          case 1:
            /* EOF case - done */
            filebuf_close(buf);
            goto done;
          default:
            /* Normal case */
	    {
	        char *v=NULL, *d = line;
	     
	        if((*d) == '#')
	            break;
	     
	        while((*d) && (isspace(*d))) ++d;
	        if(!(*d))
	    	    break;
	     
	        for(v = d; (*v) && (*v != ' '); ++v);
	        if(!(*v))
	    	    goto noval;
	     
	        *v++ = '\0';
	        while((*v) && isspace(*v)) ++v;
	     
	        if(!(*v)) {
noval:
		    ereport(LOG_SECURITY, XP_GetAdminStr(DBT_aclcacheflushCannotOpenMagnus), ACLConffilename);
	    	    return -1;
	        }

		    if(!strcasecmp(d, "ACLFile")) {
			NSErr_t err = NSERRINIT;
			ACLListHandle_t *tmp_acl_root;
		 
			/* Read the specified file and create ACL data structures */
			tmp_acl_root = ACL_ParseFile(&err, v);
			if (tmp_acl_root == NULL) {
			    /* generate error message into l */
		            ereport(LOG_SECURITY, XP_GetAdminStr(DBT_aclcacheflushCannotParseFile), ACLConffilename);
			    exit(-1);
			}
		 
			if (new_acl_root == NULL) {
			    new_acl_root = tmp_acl_root;
			} else {
			    if (ACL_ListConcat(&err, (ACLListHandle_t *) new_acl_root,
						tmp_acl_root, 0) < 0) {
		                ereport(LOG_SECURITY, XP_GetAdminStr(DBT_aclcacheflushCannotConcatList), ACLConffilename);
				exit(-1);
			    }
			    ACL_ListDestroy(&err, tmp_acl_root);
			}
		 
			/* Just in case ... */
			nserrDispose(&err);
		    }

            break;
	    }
        }
    }

done:
    ACLForceAuthList = NULL;
    ACL_InitFramePostMagnus();
    if (ACL_ListPostParseForAuth(NULL, (ACLListHandle_t *)new_acl_root)) {
	ereport(LOG_MISCONFIG,
                "Error in processing authentication method or DB name");
        exit(-1);
    }

    acl_root_30 = new_acl_root;
    accel_file_cache_acl_flush(NULL);

    return 0;
    
}

NSAPI_PUBLIC int
ACL_InitFrame(void)
{
    NSErr_t *errp = 0;

    ACL_AttrGetterRegister(errp, ACL_ATTR_IP, LASIpGetter, ACL_METHOD_ANY,
			   ACL_DBTYPE_ANY, ACL_AT_END, NULL);
    ACL_AttrGetterRegister(errp, ACL_ATTR_DNS, LASDnsGetter, ACL_METHOD_ANY,
			   ACL_DBTYPE_ANY, ACL_AT_END, NULL);

    ACL_CacheFlushRegister(ACL_CacheFlushFrame);

    /* Register the ACL free memory call at system restart */
#if 0
    daemon_atrestart(ACL_Restart, NULL);
#endif

    return 0;
}

/* Non-zero if there are virtual servers ON */
int ACLUriCacheOn = 0;

NSAPI_PUBLIC int
ACL_InitFramePostMagnus(void)
{
    char *rv;
    ACLHandle_t *acl;

    ACLGlobal->masterlist = (ACLListHandle_t *)new_acl_root;
    
    rv = conf_findGlobal("VirtualServers");
    if (rv  &&  !strcmp(rv, "on")) {
	ACLUriCacheOn = 0;
    } else {
        ACLUriCacheOn = 1;
    }

    /*  We must be single threaded here, locate the special ACL List  */
    if ( ACLForceAuthList == NULL ) {
        acl = ACL_ListFind(NULL, (ACLListHandle_t *)new_acl_root, "forceauth",
                           ACL_CASE_INSENSITIVE);
        if ( acl ) {
            ACLForceAuthList = ACL_ListNew(NULL);
            if ( ACLForceAuthList  == NULL ) {
                ereport(LOG_SECURITY,
                        "unable to allocate forceauth ACL List.\n");
            }
            ACL_ListAppend(NULL, ACLForceAuthList, acl, 0);
        } else {
            ACLForceAuthList = acl_create_forceauth_acl();
            if ( ACLForceAuthList == NULL ) {
                ereport(LOG_SECURITY,
                        "unable to create default forceauth ACL List.\n");
            }
        }
        ACL_ListHashEnter(&ACLForceAuthList);
    }
    return 0;
}

void
ACL_EreportError(NSErr_t *errp)
{
    char errmsg[1024];

    errmsg[0] = '\0';
    aclErrorFmt(errp, errmsg, 1023, 4);
    errmsg[1023] = '\0';
    if (*errmsg)
	ereport(LOG_SECURITY, errmsg);
    nserrDispose(errp);
}

