/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * cmstat.cpp: Content manager status
 * 
 * Daryoush Paknad
 * 
 */

int cmFlag;


#include "base/util.h"            /* unescape */
#include "base/plist.h"
#include "frame/cmstat.h"
#include "frame/log.h"
#include "frame/protocol.h"
#include "frame/conf.h"           /* default object */

#include "frame/object.h"

#include <base/fasttime.h>

#include <signal.h>        /* signal, alarm */
#include <ctype.h>         /* isupper, tolower */




CMVtbl cmVtbl;




#define CM_BUFF_LEN    2048

/* Sets "Link" header, if content manager is on */
NSAPI_PUBLIC void CM_set_link_header(Request *rq)
{
  char link[CM_BUFF_LEN];
  char *uri;

  if ( !CM_get_status() )  /* if cm is off. don't do anything */
	return;
  
  uri = pblock_findval("uri", rq->reqpb);

  /* e.g. Link: <http://www.glyphica.com/index.html?services>; rel="services" */
  if (security_active) {
	if (server_portnum == 443)
	  util_snprintf(link, CM_BUFF_LEN, "<https://%s%s?%s>; rel=\"%s\"", 
					server_hostname, uri, CM_SERVICES_NS,CM_SERVICES_NS  );
	else
	  util_snprintf(link, CM_BUFF_LEN, "<https://%s:%d%s?%s>; rel=\"%s\"", 
					server_hostname, server_portnum, uri, CM_SERVICES_NS,CM_SERVICES_NS );
  } else {
	if (server_portnum == 80)
	  util_snprintf( link, CM_BUFF_LEN, "<http://%s%s?%s>; rel=\"%s\"", 
					 server_hostname, uri, CM_SERVICES_NS,CM_SERVICES_NS );
	else
	  util_snprintf( link, CM_BUFF_LEN, "<http://%s:%d%s?%s>; rel=\"%s\"", 
					 server_hostname,  server_portnum, uri, CM_SERVICES_NS,CM_SERVICES_NS );
  }
  pblock_nvinsert("Link", link, rq->srvhdrs);  
}



/* returns "Link" header, if content manager is on. Null otherwise */
NSAPI_PUBLIC int CM_get_link_header(char *uri, char *link, int size )
{
  if ( !CM_get_status() )  /* if cm is off. don't do anything */
	return 0;
  
  /* e.g. Link: <http://www.glyphica.com/index.html?services>; rel ="services" */
  if (security_active) {
	if (server_portnum == 443)
	  util_snprintf(link, size, "Link: <https://%s%s?%s>; rel=\"%s\"", 
					server_hostname, uri, CM_SERVICES_NS,CM_SERVICES_NS  );
	else
	  util_snprintf(link, size, "Link: <https://%s:%d%s?%s>; rel=\"%s\"", 
					server_hostname, 
			  server_portnum, uri, CM_SERVICES_NS,CM_SERVICES_NS  );
  } else {
	if (server_portnum == 80)
	  util_snprintf( link, size, "Link: <http://%s%s?%s>; rel=\"%s\"", 
					 server_hostname, uri, CM_SERVICES_NS,CM_SERVICES_NS  );
	else
	  util_snprintf( link, size, "Link: <http://%s:%d%s?%s>; rel=\"%s\"", 
					 server_hostname, 
			   server_portnum, uri, CM_SERVICES_NS,CM_SERVICES_NS  );
  }
  return 1;
}


NSAPI_PUBLIC void CM_enable_cm(void *CMTrigger) 
{
  cmFlag = 1;
  cmVtbl.CMTrigger = (CMTriggerProc) CMTrigger;
}

NSAPI_PUBLIC void CM_disable_cm(void) 
{
  cmFlag = 0;
}


NSAPI_PUBLIC int CM_get_status(void) 
{
  return cmFlag;
}



NSAPI_PUBLIC int CM_is_cm_query( char *query ) 
{

  if ( !strcmp( query, CM_SERVICES_NS  ) ||
	   !strcmp( query, CM_PROPERTY_STR ) ||
	   !strcmp( query, CM_WEBPUB_NS )    ||
	   !strcmp( query, CM_SYS_PROP_NS )  || 
	   !strcmp( query, CM_TOC_NS )       || 
	   !strcmp( query, CM_VER_INFO_NS )  ||
	   !strcmp( query, CM_LINK_INFO_NS ) ||
	   !strcmp( query, CM_CUSTOM_FEILD_NS) ||
	   !strcmp( query, CM_VER_DIFF_NS )    || 
	   !strcmp( query, CM_START_VERSION_NS) ||
	   !strcmp( query, CM_STOP_VERSION_NS ) ||
	   !strcmp( query, CM_VER_LINKS_NS )    ||
	   !strcmp( query, CM_UNCHECKOUT_NS )   ||
	   !strcmp( query, CM_ADMIN_NS )        ||
	   !strcmp( query, CM_ADMIN_SYS_NS )    ||
	   !strcmp( query, CM_USR_PROP_NS )     ||
	   !strcmp( query, CM_HTML_REND_NS )    ||
	   !strcmp( query, CM_ADMIN_DUMP )      )
	return 1;

  return 0;
}
