/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * conf.c: Deals with the server configuration files.
 * 
 * Object configuration files are dealt with in objset.c and object.c.
 * 
 * Rob McCool
 */


#define CONF_C
#include "netsite.h"

#ifdef NET_SSL
extern "C" {
#include "plarena.h"
#include "secmodt.h"
#include "ssl.h"         /* SSL_EnableCipher */
#include "sslproto.h"    /* Cipher #defines */
}
#endif


#include "frame/conf.h"

#include "base/file.h"
#include "base/net.h"     /* net_enabledns */
#include "base/buffer.h"
#include "base/util.h"    /* getline, sprintf */
#include "base/pblock.h"  /* for init directive */
#include "frame/func.h"   /* func_exec */
#include "frame/protocol.h" /* protocol_set_keepalive_timeout */
#include <frame/conf_api.h>
#include <frame/aclinit.h>
#include <frame/log.h>

#include "libaccess/nserror.h"		/* error stack */
#include "libaccess/aclbuild.h"		/* ACL construction */
#include "libaccess/aclstruct.h"	/* ACContext_t */
#include "libaccess/aclerror.h"		/* aclErrorFmt() */
#include <libaccess/acl.h>
#include <libaccess/aclproto.h>
#include <libaccess/las.h>
#include <libaccess/usrcache.h>

#include "i18n.h"  /* SetClient,Admin,Default languages */
#include "libadmsslutil/admsslutil.h"

#include <ctype.h>       /* isspace */
#include <string.h>      /* strdup, strcmp */
#include <sys/types.h>

#ifdef XP_UNIX
#include <unistd.h>
#else /* XP_WIN32 */
#include <base/systems.h>
#endif /* XP_UNIX */

static conf_global_vars_s conf_globals;

int port;

/* This is a hack to work around NT's build break caused by C and C++ 
   generate symbol differeently */
#ifdef XP_WIN32
extern "C" {
#endif
char *addr;
#ifdef XP_WIN32
}
#endif

char *pidfn;
char *chr;

#ifdef NET_SSL
/* MLM - needs to be static? */
char *secure_cipherlist;

char *secure_keyfn;
char *secure_certfn;
char *dbstr;
int secure_auth;

#ifdef FORTEZZA
char *fortezza_personality = NULL;
uint32 fortezza_card_mask = 0xffffffff;
char *krlname = NULL;

static uint32 xtoi(char *in) {
    unsigned char *str= (unsigned char *)in;
    uint32 value = 0;

    if (str == NULL) return 0;

    /* skip passed spaces and 0x's */
    while (*str && (isspace(*str) || (*str == '0'))) str++;
    if ((*str == 'x') || (*str == 'X')) str++;

    while (*str) {
	if ((*str <= '9') && (*str >= '0')) {
	    value = value * 0x10 + (*str) - '0';
	} else if ((*str <= 'f') && (*str >= 'a')) {
	    value = value * 0x10 + (*str) - 'a' + 0xa;
	} else if ((*str <= 'F') && (*str >= 'A')) {
	    value = value * 0x10 + (*str) - 'A' + 0xa;
	} else break;
	str++;
    }
    return value;
}
#endif

extern int   HTTPVersion;
extern char  *sHTTPVersion;

/* This is a hack, file is in ns security util/(mac,unix,win)_rand.c */
// <Not in NSS3> extern "C" void RNG_FileForRNG(char *filename);
#endif

/* -------------------------- _conf_getvariable --------------------------- */

NSAPI_PUBLIC conf_global_vars_s *conf_getglobals(void) 
{
    return &conf_globals;
}

/* For NT, this function allows the EXE to pass the server root to the DLL */
NSAPI_PUBLIC void conf_setroot(char *root)
{
   server_root = STRDUP(root);
}



/* -------------------------- _conf_setdefaults --------------------------- */


void _conf_setdefaults(void) {
    int i = 0;
    master_error_log = NULL;
    root_object = NULL;
    std_os = NULL;
    acl_root = NULL;
    acl_root_30 = NULL;
    server_hostname = util_hostname();
    server_root = NULL;

    port = DEFAULT_PORT;
    conf_globals.Vport = DEFAULT_PORT;
    addr = NULL;
    conf_globals.Vaddr = NULL;
#ifdef XP_UNIX
    user_pw = NULL;
#endif /* XP_UNIX */
    chr = NULL;
    conf_globals.Vchr = NULL;

    pidfn = NULL;
    conf_globals.Vpidfn = NULL;

    pool_maxthreads = -1;

    secure_keyfn = NULL;
    conf_globals.Vsecure_keyfn = NULL;
    secure_certfn = NULL;
    conf_globals.Vsecure_certfn = NULL;
    security_active = 1;
    ssl3_active = 1;
    ssl2_active = 1;
    security_session_timeout = 0;
    ssl3_session_timeout = 0;
    secure_auth = 0;
    conf_globals.Vsecure_auth = 0;
    secure_cipherlist=NULL;

    conf_globals.single_accept = SINGLE_ACCEPT_NONE;
    conf_globals.num_keep_alives = 0;
    conf_globals.log_verbose = 0;
    conf_globals.mmap_flags = -1;
    conf_globals.mmap_prots = -1;
	/* Accept Languauge on/off */
    conf_globals.accept_language = 0;
    conf_globals.wait_for_cgi = 0;
	/* XXX cgi watchdog disabled by default, as there is race condition
	       in cgiwatch.cpp (between check and remove clients). 
    	   conf_globals.cgiwatch_timeout= PR_SecondsToInterval(10);
	 */
    conf_globals.cgiwatch_timeout= 0;

    /* agent configuration */
    AgentFilePath = 0;
    conf_globals.agentsACLFile = 0;

	/* Allowed Methods */
	AllowedMethods = 0;

    conf_initGlobal();
}

/* ---------------------------- conf_terminate ---------------------------- */


#define FRNN(p) if(p) FREE(p);

NSAPI_PUBLIC void conf_terminate(void)
{
    NSErr_t *errp = 0;
    FRNN(addr);
    FRNN(master_error_log);
    FRNN(root_object);
    FRNN(server_hostname);

    if (acl_root) {
	accDestroy((ACContext_t *) acl_root, 0);
    }

    FRNN(chr);
    FRNN(pidfn);
#ifdef XP_UNIX
    if(user_pw) {
        FREE(user_pw->pw_name); /* this does the right thing for getpwnam_r */
        FREE(user_pw);
    }
#endif /* XP_UNIX */
#ifdef NET_SSL
    FRNN(secure_cipherlist);
#endif
}


/* ---------------------------- _conf_process ----------------------------- */


/*
 * Parses a directive and value and takes action depending on the value.
 * Returns 0 upon success, and -1 upon error (with an error string in l).
 */

int _conf_process(char *l, int ln) 
{
    NSErr_t *errp = 0;
    char *v, *d = l;

    if((*d) == '#')
        return 0;

    while((*d) && (isspace(*d))) ++d;
    if(!(*d))
        return 0;

    for(v = d; (*v) && (*v != ' '); ++v);
    if(!(*v))
        goto noval;

    *v++ = '\0';
    while((*v) && isspace(*v)) ++v;

    if(!(*v)) {
      noval:
        util_sprintf(l,"line %d: directive with no value",ln);
        return -1;
    }

    if(!strcasecmp(d,"LoadObjects")) {
        SYS_FILE fd;
        filebuf_t *buf;
        char *v2;

#ifdef NET_SSL
        /* Continue seeding the random number generator */
// <Not in NSS3>        RNG_FileForRNG(v);
#endif
        if((fd = system_fopenRO(v)) == SYS_ERROR_FD) {
            v2 = STRDUP(v);
            util_sprintf(l,"line %d: LoadObjects failed to open %s (%s)",ln, 
                         v2, system_errmsg());
            FREE(v2);
            return -1;
        }
        buf = filebuf_open(fd, FILE_BUFFERSIZE);
        v2 = STRDUP(v);
        if(!(std_os = objset_scan_buffer(buf, l, std_os))) {
            strcat(l, " while processing load file ");
            strcat(l, v2);
            FREE(v2);
            filebuf_close(buf);
            return -1;
        }
        FREE(v2);
        filebuf_close(buf);
    }

    else if(!strcasecmp(d, "RootObject")) {
        if(root_object)
            FREE(root_object);
        root_object = STRDUP(v);
    }

    else if(!strcasecmp(d, "ServerRoot")) {
        if(server_root)
            FREE(server_root);
        server_root = STRDUP(v);
    }

    else if(!strcasecmp(d, "OLDACLFile")) {
	ACContext_t * acc;
	int rv;
	NSErr_t err = NSERRINIT;

	/* Get current ACL context, if any */
	acc = (ACContext_t *)acl_root;

	/* Read the specified file and create ACL data structures */
	rv = accReadFile(&err, v, &acc);
	if (rv < 0) {
	    /* generate error message into l */
	    aclErrorFmt(&err, l, CONF_MAXLEN, 4);
	    nserrDispose(&err);
	    return -1;
	}

	/* Just in case ... */
	nserrDispose(&err);

	/* Set the root of ACL data structures */
	acl_root = (void *)acc;
    }

    else if(!strcasecmp(d, "ACLFile")) {
	NSErr_t err = NSERRINIT;
	ACLListHandle_t *tmp_acl_root;

	/* Save the first ACL File's name - used by WebPub ACL UI */
	ACL_SetAclfilename(v);

	/* Read the specified file and create ACL data structures */
	tmp_acl_root = ACL_ParseFile(&err, v);
	if (tmp_acl_root == NULL) {
	    /* generate error message into l */
	    aclErrorFmt(&err, l, CONF_MAXLEN, 6);
	    nserrDispose(&err);
	    return(-1);
	}

        if (acl_root_30 == NULL) {
            acl_root_30 = tmp_acl_root;
        } else {
            if (ACL_ListConcat(&err, (ACLListHandle_t *) acl_root_30, 
                                tmp_acl_root, 0) < 0) {
                util_sprintf(l,"line %d: LoadObjects failed (%s)",ln, 
                            system_errmsg());
                return(-1);
            }
            ACL_ListDestroy(errp, tmp_acl_root);
        }

	/* Just in case ... */
	nserrDispose(&err);
    }

    else if(!strcasecmp(d,"ACLCacheLifetime")) {
        int timeout = atoi(v);
        acl_usr_cache_set_timeout(timeout);
    }

    else if(!strcasecmp(d, "ErrorLog")) {
        if(master_error_log)
            FREE(master_error_log);
        master_error_log = STRDUP(v);
    }

    else if(!strcasecmp(d, "ServerName")) {
        if(server_hostname)
            FREE(server_hostname);
        server_hostname = STRDUP(v);

    } else if(!strcasecmp(d, "MtaHost")) {

	SetMtaHost(v);
    } if(!strcasecmp(d, "NntpHost")) {
        SetNntpHost(v);

    } else if(!strcasecmp(d, "DNS")) {
        net_enabledns = (strcmp(v, "off") ? 1 : 0);
    } else if(!strcasecmp(d, "AsyncDNS")) {
        net_enableAsyncDNS = (strcmp(v, "on") ? 0 : 1);
    }
    else if(!strcasecmp(d, "ListenQ")) {
        net_listenqsize = atoi(v);
    }
    else if(!strcasecmp(d, "CGIWaitPid")) {
        conf_globals.wait_for_cgi=(strcmp(v, "on") ? 0 : 1);
    }
    else if(!strcasecmp(d, "CGIKillTimeout")) {
        conf_globals.cgiwatch_timeout=PR_SecondsToInterval(atoi(v));
    }
    else if(!strcasecmp(d, "KeepAliveTimeout")) {
        protocol_set_keepalive_timeout(atoi(v));
        /* We also need to add to generic globals - daemonsession.cpp uses it */
        if (conf_setGlobal(d, v) == PR_FALSE) {
            util_sprintf(l,"line %d: unknown directive %s",ln,d);
            FREE(d);
            return -1;
        }
    }
    else if(!strcasecmp(d, "HTTPVersion")) {  
        int maj,min;
        if ( sscanf(v,"%d.%d",&maj,&min) == 2) {
             HTTPVersion = 100*maj+min;
             strncpy(sHTTPVersion,v,3);
        } else {
             util_sprintf(l, 
                "line %d: HTTPVersion format should be <maj.min>.", ln);
             return -1;
        }
    }

    /* XXXrobm this should go away before 2.0 ships */
    else if(!strcasecmp(d, "Init")) {
        pblock *pb = pblock_create(PARAMETER_HASH_SIZE);

        if(pblock_str2pblock(v, pb) != -1) {
            pblock_nvinsert("server-version", MAGNUS_VERSION_STRING, pb);
            if(func_exec(pb, NULL, NULL) == REQ_ABORTED) {
                v = pblock_findval("error", pb);
                util_sprintf(l, "line %d: %s", ln, 
                             v ? v : "unknown error");
                return -1;
            }
        }
        pblock_free(pb);
    }

    else if(!strcasecmp(d,"Port")) {
        port = atoi(v);
        conf_globals.Vport = port;
    }
    else if(!strcasecmp(d, "Address")) {
        if(addr)
            FREE(addr);
        addr = STRDUP(v);
        conf_globals.Vaddr = addr;
    }

#if defined(THREAD_ANY)
    else if(!strcasecmp(d, "MaxThreads"))
        pool_maxthreads = atoi(v);
    else if (!strcasecmp(d, "Personal"))
        /* Nothing to do here */;
#endif
    else if (!strcasecmp(d, "MapFlags")) {
            conf_globals.mmap_flags = atoi(v);
    }
    else if (!strcasecmp(d, "MapProts")) {
            conf_globals.mmap_prots = atoi(v);
    }
    else if (!strcasecmp(d, "LogVerbose")) {
        if (!strcasecmp(v, "off"))
            conf_globals.log_verbose = 0;
        else
            conf_globals.log_verbose = 1;
    }
    else if (!strcasecmp(d, "SingleAccept")) {
        if (!strcasecmp(v, "on"))
            conf_globals.single_accept = SINGLE_ACCEPT_ON;
        else if (!strcasecmp(v, "partial"))
            conf_globals.single_accept = SINGLE_ACCEPT_PARTIAL;
        else
            conf_globals.single_accept = SINGLE_ACCEPT_OFF;
    }
    else if (!strcasecmp(d, "KeepAliveThreads"))
        conf_globals.num_keep_alives = atoi(v);
#ifdef XP_UNIX
    else if(!strcasecmp(d, "User")) {
        if(!geteuid()) {
	    char *pwline = (char *)MALLOC(4 * DEF_PWBUF * sizeof(char));
            user_pw = (struct passwd *) MALLOC(sizeof(struct passwd));
            setpwent();
	    /* getpwnam_r fills in the pw struct with pointers into pw line */
            if(!(util_getpwnam(v, user_pw, pwline, 4 * DEF_PWBUF))) {
                d = STRDUP(v);
                util_sprintf(l, "line %d: cannot find user %s in passwd file",
                             ln, d);
                FREE(d);
                FREE(user_pw);
                FREE(pwline);
                return -1;
            }
        }
    }
#else /* XP_UNIX */
    else if(!strcasecmp(d, "User")) {
    }
#endif /* XP_WIN32 */

    else if(!strcasecmp(d, "Chroot")) {
        if(chr)
            FREE(chr);
        chr = STRDUP(v);
        conf_globals.Vchr = chr;
    }

    else if(!strcasecmp(d, "PidLog")) {
        if(pidfn)
            FREE(pidfn);
        pidfn = STRDUP(v);
        conf_globals.Vpidfn = pidfn;
    }

#ifdef NET_SSL
    else if(!strcasecmp(d, "KeyFile")) {
        if(secure_keyfn)
            FREE(secure_keyfn);
        secure_keyfn = STRDUP(v);
        conf_globals.Vsecure_keyfn = STRDUP(v);
    }
    else if(!strcasecmp(d, "CertFile")) {
        if(secure_certfn)
            FREE(secure_certfn);
        secure_certfn = STRDUP(v);
	/* removed by yjh
        if ((dbstr = strstr(secure_certfn, ".db")) != NULL)
          *(secure_certfn+(strlen(secure_certfn)-strlen(dbstr))) = '\0';
	  */
        conf_globals.Vsecure_certfn = STRDUP(v);
    }
    else if(!strcasecmp(d, "Ciphers")) {
        if(secure_cipherlist)  {
            secure_cipherlist=(char *) 
                              REALLOC(secure_cipherlist, 
                                      strlen(secure_cipherlist)+
                                      strlen(v)+4);
            strcat(secure_cipherlist, ",");
            strcat(secure_cipherlist, v);
        }  else  {
            secure_cipherlist=STRDUP(v);
        }
    }
    else if(!strcasecmp(d, "SSL3Ciphers")) {
        if(secure_cipherlist)  {
            secure_cipherlist=(char *) 
                              REALLOC(secure_cipherlist, 
                                      strlen(secure_cipherlist)+
                                      strlen(v)+4);
            strcat(secure_cipherlist, ",");
            strcat(secure_cipherlist, v);
        }  else  {
            secure_cipherlist=STRDUP(v);
        }
    }
    else if(!strcasecmp(d, "Security")) {
        if(!strcasecmp(v, "on")) security_active = 1;
        else if(!strcasecmp(v, "off")) security_active = 0;
        else {
            util_sprintf(l, "line %d: Security must be on or off.", ln);
            return -1;
        }
    }
    else if(!strcasecmp(d, "SSL3")) {
        if(!strcasecmp(v, "on")) ssl3_active = 1;
        else if(!strcasecmp(v, "off")) ssl3_active = 0;
        else {
            util_sprintf(l, "line %d: SSL3 must be on or off.", ln);
            return -1;
        }
    }
    else if(!strcasecmp(d, "SSL2")) {
        if(!strcasecmp(v, "on")) ssl2_active = 1;
        else if(!strcasecmp(v, "off")) ssl2_active = 0;
        else {
            util_sprintf(l, "line %d: SSL2 must be on or off.", ln);
            return -1;
        }
    }
#ifndef MCC_PROXY
    else if (!strcasecmp(d, "SSLClientAuth") && server_enterprise)
#else
    else if (!strcasecmp(d, "SSLClientAuth"))
#endif
    {
	if (!strcasecmp(v, "on")) secure_auth = 1;
        else if(!strcasecmp(v, "off")) secure_auth = 0;
        else {
            util_sprintf(l, "line %d: SSLClientAuth must be on or off.", ln);
            return -1;
        }
        conf_globals.Vsecure_auth = secure_auth;
    }
    else if(!strcasecmp(d, "SSLSessionTimeout")) {
        security_session_timeout = atoi(v);        
    }
    else if(!strcasecmp(d, "SSL3SessionTimeout")) {
        ssl3_session_timeout = atoi(v);        
    }
#ifdef FORTEZZA
    else if (!strcasecmp(d, "FortezzaCardMask")) {
	fortezza_card_mask = xtoi(v);
	conf_globals.fortezza_card_mask = fortezza_card_mask;
    }
    else if (!strcasecmp(d, "FortezzaPersonality")) {
	fortezza_personality = STRDUP(v);
	conf_globals.fortezza_personality = STRDUP(v);
    }
    else if (!strcasecmp(d, "FortezzaCKL")) {
	krlname = STRDUP(v);
	conf_globals.krlname = STRDUP(v);
    }
#endif
#endif

    /* Agent configuration file */
    else if (!strcasecmp(d, "AgentsFilePath")) {
        if (AgentFilePath != 0)
                FREE(AgentFilePath);
        AgentFilePath  = STRDUP(v);
    }
    else if (!strcasecmp(d, "agentsACLFile")) {
        if (conf_globals.agentsACLFile != 0)
                FREE(conf_globals.agentsACLFile);
        conf_globals.agentsACLFile = STRDUP(v);
    }

    /* Accept Language */
    else if (!strcasecmp(d, "AcceptLanguage")) {
        if (!strcasecmp(v, "on"))
            conf_globals.accept_language = 1;
        else
            conf_globals.accept_language = 0;
    }
	/* UI language for administrator */
    else if(!strcasecmp(d, "AdminLanguage")) {
        SetLanguage(ADMIN_LANGUAGE, v);
    }
	/* UI language for client user */
    else if(!strcasecmp(d, "ClientLanguage")) {
        SetLanguage(CLIENT_LANGUAGE, v);
    }
	/* Default UI language if translated UI is not available */
    else if(!strcasecmp(d, "DefaultLanguage")) {
        SetLanguage(DEFAULT_LANGUAGE, v);
    }
#ifdef XP_UNIX
    else if(!strcasecmp(d, "ServerID")) {
        conf_globals.Vserver_id = STRDUP(v);
        log_ereport(LOG_VERBOSE, "Set SERVER_ID to %s", server_id);
    }
#endif

    else {
        /* Just add to generic globals */
        if (conf_setGlobal(d, v) == PR_FALSE) {
            util_sprintf(l,"line %d: unknown directive %s",ln,d);
            FREE(d);
            return -1;
        }
    }

    return 0;    
}


/* ------------------------------ conf_init ------------------------------- */

/*
 * IRIX threads are quite expensive.  In order to ship on every box they
 * want us to be very lightweight.  In order to do that we need to shrink
 * our NSPR sizes to be very small.  The size we feed to some IRIX 
 * arenas (INITUSERS) is related to the number of threads we allow.  
 * Normally, we allow NSPR to be expandible and able to handle many threads.
 * When this "personal" mode is on, we purposefully configure NSPR
 * to NOT be expandible. 
 *
 * Unfortunately we need this info very early- before we call PR_Init(),
 * so this routine does a quick parse so that it can get the right info
 * to NSPR.
 *
 * cfn is the name of the config file.  We expact that the file is either
 * an absolute path or that the server has already chdir()'d to the 
 * correct directory.
 *
 * Returns 0 on success, non-zero on failure.
 */
NSAPI_PUBLIC char *conf_init_lightweight(char *cfn)
{
    filebuf_t *buf;
    char line[CONF_MAXLEN];
    int ln = 0, t;
    SYS_FILE fd;
    int max_procs = -1;
    int max_threads = -1;
    int personal = 0;
#ifdef OBSOLETE_IRIX_NSPR
    extern void PR_SetSize(int, int);
#endif /* IRIX */

#ifdef OBSOLETE_IRIX_NSPR
    PR_SetSize(513, 0);		/* default */
#endif /* IRIX */

    if((fd = system_fopenRO(cfn)) == SYS_ERROR_FD) {
        util_snprintf(line, CONF_MAXLEN, "error opening %s: %s", 
            cfn, system_errmsg());
        return PERM_STRDUP(line);
    }

    buf = filebuf_open(fd, FILE_BUFFERSIZE);
    if (!buf)
        return "Out of memory";

    while(1) {
        ++ln;
        switch((t = util_getline(buf, ln, CONF_MAXLEN, line))) {
          case -1:
            /* error case - util_getline returns the error string */
            filebuf_close(buf);
            return STRDUP(line);
            break;
          case 1:
            /* EOF case - done */
            filebuf_close(buf);
            if (personal && max_procs>0 && max_threads>0) {
                int users = max_procs*(max_threads*2+1);
#ifdef OBSOLETE_IRIX_NSPR
                /* 1024 is the IRIX max */
                PR_SetSize(users<1024?users:1024, 0);
#endif /* IRIX */
            } else if (!personal && max_procs>0 && max_threads>0) {
                int users = max_procs*max_threads+256;
#ifdef OBSOLETE_IRIX_NSPR
                PR_SetSize(users<1024?users:1024, 0);
#endif /* IRIX */
            }
            return NULL;
            break;
          default:
            /* Normal case */
            {
              char *v = NULL, *d = line;	/* d = directive, v = value */

              if((*d) == '#')		/* Check for comments */
                  break;

              while((*d) && (isspace(*d))) ++d;
              if(!(*d))
                  break;			/* Blank line (nothing but whitespace) */

              for (v = d; (*v) && (*v != ' '); ++v);	/* skip directive */

              *v++ = '\0';
              while((*v) && isspace(*v)) ++v;			/* skip whitespace */

              if (!strcasecmp(d, "MaxProcs")) {
                  max_procs = atoi(v);
              } else if (!strcasecmp(d, "MaxThreads")) {
                  max_threads = atoi(v);
              } else if (!strcasecmp(d, "Personal")) {
                  if (!v || !strcasecmp(v, "On"))
                      personal = 1;
              }
            }
            break;
        }
    }

    /* Never reached */
    return NULL;
}

NSAPI_PUBLIC char *conf_init(char *cfn) {
    filebuf_t *buf;
    SYS_FILE fd;
    int ln = 0, t;
    char line[CONF_MAXLEN], *err;

#ifdef NET_SSL
    /* Continue seeding the random number generator */
// <Not in NSS3>    RNG_FileForRNG(cfn);
#endif
    if((fd = system_fopenRO(cfn)) == SYS_ERROR_FD) {
        util_sprintf(line,"error opening %s: %s", cfn, system_errmsg());
        return STRDUP(line);
    }

    _conf_setdefaults();

    buf = filebuf_open(fd, FILE_BUFFERSIZE);
    if (!buf)
        return "Out of memory";

    while(1) {
        ++ln;
        switch((t = util_getline(buf, ln, CONF_MAXLEN, line))) {
          case -1:
            filebuf_close(buf);
            return STRDUP(line);

          case 1:
            t = _conf_process(line, ln);
            filebuf_close(buf);
            if(t) return STRDUP(line);

            if(!(root_object))
                return STRDUP("no root object specified");
            if(!(std_os))
                return STRDUP("no object sets loaded");
            if(!(master_error_log))
                return STRDUP("no error log specified");
            if(!(server_hostname)) 
                return STRDUP("no hostname available (set manually)");
#ifndef FORTEZZA
            /* DON'T DO THIS HERE!!!! WE CHECK IN SERVSSL()... NOT ALL
             * Configurations need a key File. Hardware Solutions for instance!
             * Anyway This breaks Fortezza so we turn it off!! */
            if(security_active && ((!secure_keyfn) || (!secure_certfn)))
                return STRDUP("with security on, you must provide both a key file and a certificate file");
#endif
            /* The initial value for the keepalive timeout is -1.  If security
             * is active, don't enable keepalives unless the user manually
             * requested that they be enabled.  The reason is because 
             * navigator 2.0 breaks with keepalives enabled in ssl mode.
             * 
             * At this point, if the keepalive has not been specified (-1),
             * then set it to the default for non-ssl or 0 for ssl.
             */
            if(http_get_keepalive_timeout() == -1) {
                if (!security_active)
                    protocol_set_keepalive_timeout(HTTP_DEFAULT_KEEPALIVE_TIMEOUT);
                else
                    protocol_set_keepalive_timeout(0);
            }
            if( (err = _conf_setciphers(secure_cipherlist)) )
                return err;
            /* With version 2.0, Init functions go in obj.conf */
            if(!std_os->initfns)
                return NULL;
            return NULL;

          default:
            if((t = _conf_process(line, ln))) {
                filebuf_close(buf);
                return STRDUP(line);
            }
        }
    }
}

/*********************************************************************
  NSAPI_PUBLIC char *conf_run_init_functions(void) 
  The late init functions are identified by having LateInit=yes
  These are called after the fork, in the child, in case of unix.
  In case of NT, they do not have any special significance and 
  should be called immediately following the init_functions
  *****************************************************************/

#define LATE_INIT_IS_DEFAULT
NSAPI_PUBLIC char *conf_run_init_functions(void) 
{
    char line[CONF_MAXLEN];
    char *fn;
    int x;

    if(!std_os->initfns)
        return NULL;

    for(x = 0; std_os->initfns[x]; ++x) {
        char *v;
	char *fn = pblock_findval("fn", std_os->initfns[x]);
	char *lateInit = pblock_findval("LateInit", std_os->initfns[x]);
	int  isEarlyInitFunction = 1;
#ifdef LATE_INIT_IS_DEFAULT
	if (!lateInit || (lateInit && !strcasecmp(lateInit, "yes")))
	   isEarlyInitFunction = 0;
#else
	if (lateInit && !strcasecmp(lateInit, "yes"))
	   isEarlyInitFunction = 0;
#endif

        /* special case the cache-init function to run as early init */
        if (!isEarlyInitFunction && !strcasecmp("cache-init", fn)) 
            isEarlyInitFunction = 1;

	if (isEarlyInitFunction) {
	  pblock_nvinsert("server-version", MAGNUS_VERSION_STRING, 
                        std_os->initfns[x]);
	  if(func_exec(std_os->initfns[x], NULL, NULL) == REQ_ABORTED) {
            v = pblock_findval("error", std_os->initfns[x]);
            fn = pblock_findval("fn", std_os->initfns[x]);
            util_sprintf(line, "Error running init function %s: %s", fn, 
                         v ? v : "unknown error");
            return STRDUP(line);
	  }
	  param_free(pblock_remove("server-version",std_os->initfns[x]));
	}
    }

    return NULL;
}


NSAPI_PUBLIC char *conf_run_late_init_functions(void)
{
    char line[CONF_MAXLEN];
    char *fn;
    int x;
 
    if(!std_os->initfns)
        return NULL;

    for(x = 0; std_os->initfns[x]; ++x) {
        char *lateInit = pblock_findval("LateInit", std_os->initfns[x]);
    int isLateInitFunction = 0;
#ifdef LATE_INIT_IS_DEFAULT
	if (!lateInit || (lateInit && !strcasecmp(lateInit, "yes")))
	   isLateInitFunction = 1;
#else
	if (lateInit && !strcasecmp(lateInit, "yes"))
	   isLateInitFunction = 1;
#endif
	if (isLateInitFunction) {
	     char *v;
	     pblock_nvinsert("server-version", MAGNUS_VERSION_STRING,
                        std_os->initfns[x]);
	     if(func_exec(std_os->initfns[x], NULL, NULL) == REQ_ABORTED) {
	        v = pblock_findval("error", std_os->initfns[x]);
		fn = pblock_findval("fn", std_os->initfns[x]);
		util_sprintf(line, "Error running init function(late) %s: %s", fn,
                         v ? v : "unknown error");
		return STRDUP(line);
	     }
	     param_free(pblock_remove("server-version",std_os->initfns[x]));
	}
    }
 
    return NULL;
}



/* --------------------------- conf_vars2daemon --------------------------- */


NSAPI_PUBLIC void conf_vars2daemon(daemon_s *d)
{
    d->port = port;
    d->ipstr = addr;
#ifdef XP_UNIX
    d->pw = user_pw;
    d->chr = chr;
    d->pidfn = pidfn;
#endif /* XP_UNIX */

#if defined(THREAD_ANY)
    d->maxthreads = pool_maxthreads;
#endif
#ifdef NET_SSL
    d->secure_keyfn = secure_keyfn;
    d->secure_certfn = secure_certfn;
    d->secure_auth = secure_auth;
    d->secure_session_timeout = security_session_timeout;
    d->ssl3_secure_session_timeout = ssl3_session_timeout;
    d->security = security_active;
    d->ssl3 = ssl3_active;
    d->ssl2 = ssl2_active;
#ifdef FORTEZZA
/* whether this is being run by admin server or web server, both have
   a conf_globals structure with valid FORTEZZA variables */
    d->fortezza_card_mask = conf_globals.fortezza_card_mask;
    d->fortezza_personality = conf_globals.fortezza_personality;
    d->krlname = conf_globals.krlname;
#endif
#endif
    d->single_accept = conf_globals.single_accept;
    d->num_keep_alives = conf_globals.num_keep_alives;
    d->accept_language = conf_globals.accept_language;
}


char *
conf_getServerString()
{
    return MAGNUS_VERSION_STRING;
}
