/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 *
 * Access primitives to server configuration paramerters.
 * Created by: Antoni Lapinski, Oct 1996
 *
 */

#include <frame/conf.h>
#include <frame/conf_api.h>
#include <base/pblock.h>
#include <frame/log.h>

static PRBool conf_api_initialized = PR_FALSE;

NSAPI_PUBLIC PRBool conf_initGlobal()
{
    conf_global_vars_s *globals = conf_getglobals();

    globals->genericGlobals = pblock_create(16);

    if (!globals->genericGlobals)
        return PR_FALSE;

    conf_api_initialized = PR_TRUE;

    return PR_TRUE;
}

NSAPI_PUBLIC char *conf_findGlobal(char *name)
{
    if (!conf_api_initialized) {
        log_ereport(LOG_VERBOSE, "conf_findGlobal(%s) called before conf_api initialized");
        return NULL;
    }
    return pblock_findval(name, conf_getglobals()->genericGlobals);
}

NSAPI_PUBLIC PRBool conf_setGlobal(char *name, char *value)
{
    pb_param *oldParam = pblock_find(name, conf_getglobals()->genericGlobals);
    pb_param *newParam;
    pblock *globals = conf_getglobals()->genericGlobals;

    if (oldParam) {
        pblock_remove(name, globals);
    }

    newParam = pblock_nvinsert(name, value, globals);

    return newParam!=NULL?PR_TRUE:PR_FALSE;
}

NSAPI_PUBLIC PRBool conf_deleteGlobal(char *name)
{
    pb_param *oldParam;
    pblock *globals = conf_getglobals()->genericGlobals;

    oldParam = pblock_remove(name, globals);

    return oldParam==NULL?PR_FALSE:PR_TRUE;
}

/* Hostname of the Mail Transport Agent :
 * typically "localhost" for the Unix boxes,
 * or the hostname of a POP server otherwise.
 * Needed by the Agents subsystem.
 * Current implementation relies on access to the global config. structure.
 */

NSAPI_PUBLIC const char *GetMtaHost(){
    return (conf_getglobals()->mtahost ? conf_getglobals()->mtahost : "");
}

NSAPI_PUBLIC const char *SetMtaHost(const char *HostName){

    if(conf_getglobals() -> mtahost)
	FREE(conf_getglobals() -> mtahost);

    conf_getglobals() -> mtahost = STRDUP(HostName);

    return conf_getglobals() -> mtahost;
}

NSAPI_PUBLIC const char *GetNntpHost(){
    return (conf_getglobals()->nntphost ? conf_getglobals()->nntphost : "");
}

NSAPI_PUBLIC const char *SetNntpHost(const char *HostName){

    if(conf_getglobals() -> nntphost)
        FREE(conf_getglobals() -> nntphost);

    conf_getglobals() -> nntphost = STRDUP(HostName);

    return conf_getglobals() -> nntphost;
}

