/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * error.c: Tries to find an action it should take on errors. Default
 * is to send a short HTML page.
 * 
 * Rob McCool
 */


#include "netsite.h"

#include "base/pblock.h"

#include "base/session.h"
#include "frame/req.h"
#include "base/util.h"
#include "frame/http.h"
#include "frame/protocol.h"

#include "frame/servact.h"  /* servact_finderror */

#include <stdio.h>   /* sscanf */

#include "base/language.h"

#include "frame/dbtframe.h"

/* The most data we'll swallow in order to send an error back */
/* XXXrobm a hard coded value is not cool. */
/* XXXJB - removed hard coded value because of associated problems with */
/* very large uploads at GE.  Need to find a better way to solve this problem. */
#if 0
#define DATA_SWALLOW_MAX 5242880
#endif


/* --------------------------- error_check_link --------------------------- */

/*
 * Tells the user to check the link they came from for problems upon not
 * found.
 */

#define CHECK_MSG XP_GetStringFromDatabase(LIBRARY_NAME, lang_acceptlanguage(rq->headers), DBT_CheckMsg_)
#define REFERER_MSG XP_GetStringFromDatabase(LIBRARY_NAME, lang_acceptlanguage(rq->headers), DBT_RefererMsg_)

#define CHECK_LEN strlen(CHECK_MSG)


/* -4: remove \%s\ */
#define REFERER_LEN strlen(REFERER_MSG)

int error_check_link(pblock *param, Session *sn, Request *rq)
{
    char *referer;
    int cl, ret;
    char *buf;
    char digit[16];

    /* We're already handling an error. */
    if(request_header("referer", &referer, sn, rq) == REQ_ABORTED)
        referer = NULL;

    pblock_nvinsert("content-type", "text/html", rq->srvhdrs);
    cl = CHECK_LEN;
    if(referer)
        cl += REFERER_LEN + strlen(referer);
    util_itoa(cl, digit);
    pblock_nvinsert("content-length", digit, rq->srvhdrs);
    switch(protocol_start_response(sn, rq)) {
      case REQ_NOACTION:
      case REQ_EXIT:
        return REQ_PROCEED;
    }

    if(referer) {
        buf = (char *) MALLOC(cl + 1);
        strcpy(buf, CHECK_MSG);
        util_sprintf(&buf[CHECK_LEN], REFERER_MSG, referer);
        ret = net_write(sn->csd, buf, cl);
        FREE(buf);
        if(ret == IO_ERROR)
            return REQ_EXIT;
    }
    else 
        if(net_write(sn->csd, CHECK_MSG, CHECK_LEN) == IO_ERROR)
            return REQ_EXIT;

    return REQ_PROCEED;
}


/* ----------------------------- error_report ----------------------------- */

/*
 * Error messages sent back to the client;
 * This looks really ugly -- boy do I hate 80 character terminals.
 */

#ifdef MCC_PROXY

#define PROTOCOL_BAD_REQUEST_MSG \
        XP_GetStringFromDatabase(LIBRARY_NAME,lang_acceptlanguage(rq->headers), DBT_ProtocolBadRequestMsg_)

#define PROTOCOL_UNAUTHORIZED_MSG \
        XP_GetStringFromDatabase(LIBRARY_NAME,lang_acceptlanguage(rq->headers), DBT_ProtocolUnauthorizedMsg_)

#define PROTOCOL_PROXY_UNAUTHORIZED_MSG \
        XP_GetStringFromDatabase(LIBRARY_NAME,lang_acceptlanguage(rq->headers), DBT_ProtocolProxyUnauthorizedMsg_)

#define PROTOCOL_FORBIDDEN_MSG \
        XP_GetStringFromDatabase(LIBRARY_NAME,lang_acceptlanguage(rq->headers), DBT_ProtocolForbiddenMsg_)

#define PROTOCOL_SERVER_ERROR_MSG \
        XP_GetStringFromDatabase(LIBRARY_NAME,lang_acceptlanguage(rq->headers), DBT_ProtocolServerErrorMsg_)

#define PROTOCOL_NOT_IMPLEMENTED_MSG \
        XP_GetStringFromDatabase(LIBRARY_NAME,lang_acceptlanguage(rq->headers), DBT_ProtocolNotImplementedMsg_)

#define PROTOCOL_DEFAULT_ERROR_MSG \
        XP_GetStringFromDatabase(LIBRARY_NAME,lang_acceptlanguage(rq->headers), DBT_ProtocolDefaultErrorMsg_)

#else

#define PROTOCOL_BAD_REQUEST_MSG \
        XP_GetStringFromDatabase(LIBRARY_NAME,lang_acceptlanguage(rq->headers), DBT_ProtocolBadRequestMsg_2)

#define PROTOCOL_UNAUTHORIZED_MSG \
        XP_GetStringFromDatabase(LIBRARY_NAME,lang_acceptlanguage(rq->headers), DBT_ProtocolUnauthorizedMsg_2)

#define PROTOCOL_FORBIDDEN_MSG \
        XP_GetStringFromDatabase(LIBRARY_NAME,lang_acceptlanguage(rq->headers), DBT_ProtocolForbiddenMsg_2)

#define PROTOCOL_SERVER_ERROR_MSG \
        XP_GetStringFromDatabase(LIBRARY_NAME,lang_acceptlanguage(rq->headers), DBT_ProtocolServerErrorMsg_2)

#define PROTOCOL_NOT_IMPLEMENTED_MSG \
        XP_GetStringFromDatabase(LIBRARY_NAME,lang_acceptlanguage(rq->headers), DBT_ProtocolNotImplementedMsg_2)

#define PROTOCOL_DEFAULT_ERROR_MSG \
        XP_GetStringFromDatabase(LIBRARY_NAME,lang_acceptlanguage(rq->headers), DBT_ProtocolDefaultErrorMsg_2)

#endif


/* Special cased to preserve WWW-authenticate and Status */

NSAPI_PUBLIC int error_report(Session *sn, Request *rq)
{
    char *s;
    char *msg;
    pb_param *pp, *auth, *cclen;
    int code, cl;
#define MAX_ERROR_LINE	(REQ_MAX_LINE+512)
    char buf[MAX_ERROR_LINE];
    char digit[16];
#ifdef MCC_PROXY
    pb_param *pauth;
#endif
    char *url, *escape;
    extern char *sHTTPVersion;

    if(!rq) {
        cl = util_snprintf(buf,
			   MAX_ERROR_LINE,
			   "HTTP/%s 400 Bad Request\r\nServer: %s\r\n\r\n%s",
			   sHTTPVersion,
			   MAGNUS_VERSION_STRING, 
			   XP_GetStringFromDatabase(LIBRARY_NAME, 
						    "en",
						    DBT_Http10400BadRequestCCserverSCCCC_));

        net_write(sn->csd, buf, cl);
	/*        return REQ_PROCEED;  */
        return REQ_EXIT;
    }

    pp = pblock_remove("status", rq->srvhdrs);
    auth = pblock_remove("WWW-authenticate", rq->srvhdrs);
#ifdef MCC_HTTPD
    /* XXXrobm HTTP/1.1 two-phase put will time out here */
    cclen = pblock_remove("content-length", rq->headers);
    if(cclen && !pblock_find("data-removed", rq->vars)) {
        cl = atoi(cclen->value);
        if((cl > 0) /** && (cl < DATA_SWALLOW_MAX) **/) {
	  int temp = (sn->inbuf)->rdtimeout;
	  (sn->inbuf)->rdtimeout = 15; /* quick timeout in this case (only) */
	  (void) netbuf_buf2sd(sn->inbuf, SYS_NET_ERRORFD, cl);
	  (sn->inbuf)->rdtimeout = temp;
	}
        param_free(cclen);
    }
#endif

#ifdef MCC_PROXY
    pauth = pblock_remove("Proxy-authenticate", rq->srvhdrs);
#endif

    /* Clear out any garbage that may be lingering around */
    pblock_free(rq->srvhdrs);
    rq->srvhdrs = pblock_create(REQ_HASHSIZE);

    if(pp) {
        pblock_pinsert(pp, rq->srvhdrs);
        s = &pp->value[4];
        pp->value[3] = '\0';
        code = atoi(pp->value);
        pp->value[3] = ' ';
    }
    else {
        pblock_nvinsert("status", "500 Server Error", rq->srvhdrs);
        s = "Server Error";
        code = 500;
    }
    if(auth)
        pblock_pinsert(auth, rq->srvhdrs);

#ifdef MCC_PROXY
    if(pauth)
        pblock_pinsert(pauth, rq->srvhdrs);
#endif

    if(servact_finderror(sn, rq) == REQ_PROCEED)
        return REQ_PROCEED;


    /* Failed to find custom function. Do what we can. */

    /* 
     * These should be re-visited to provide more information. Also, the
     * strlen() call at the end on a constant string is dumb.
     */

    switch(code) {
      case PROTOCOL_REDIRECT:
        url = pblock_findval("url", rq->vars);
        escape = pblock_findval("escape", rq->vars);

        if (url) {
            /* Check to see if the directive specifies that the URL to be
             * redirected should not be escaped. By default, URLs that are
             * redirected are always escaped */
            if (escape == NULL || strcmp(escape, "no")) {
                msg = util_url_escape(NULL, url);
            } else
                msg = STRDUP(url);
        } else 
            msg = STRDUP("URL UNKNOWN");

        pblock_nvinsert("location", msg, rq->srvhdrs);
        cl = util_snprintf(buf, MAX_ERROR_LINE,
                           XP_GetStringFromDatabase(LIBRARY_NAME, 
                                lang_acceptlanguage(rq->headers),DBT_HtmlHeadTitleSTitleHeadBodyH1SH1_),
                           s, s, msg);
        FREE(msg);
        goto finish;

      case PROTOCOL_NOT_MODIFIED:
        return protocol_start_response(sn, rq);

      case PROTOCOL_BAD_REQUEST:
        msg = PROTOCOL_BAD_REQUEST_MSG;
        break;

      case PROTOCOL_UNAUTHORIZED:
        msg = PROTOCOL_UNAUTHORIZED_MSG;
        break;

#ifdef MCC_PROXY
      case PROTOCOL_PROXY_UNAUTHORIZED:
        msg = PROTOCOL_PROXY_UNAUTHORIZED_MSG;
        break;
#endif

      case PROTOCOL_FORBIDDEN:
        msg = PROTOCOL_FORBIDDEN_MSG;
        break;

      case PROTOCOL_NOT_FOUND:
        return error_check_link(NULL, sn, rq);

      case PROTOCOL_SERVER_ERROR:
        msg = PROTOCOL_SERVER_ERROR_MSG;
        break;

      case PROTOCOL_NOT_IMPLEMENTED:
        msg = PROTOCOL_NOT_IMPLEMENTED_MSG;
        break;

      default:
        msg = PROTOCOL_DEFAULT_ERROR_MSG;
    }

    cl = util_snprintf(buf, 
		       MAX_ERROR_LINE,
		       XP_GetStringFromDatabase(LIBRARY_NAME, 
						lang_acceptlanguage(rq->headers), 
						DBT_HtmlHeadTitleSTitleHeadNBodyH1SH_),
                    
		       s, 
		       s, 
		       msg);

  finish:
    util_itoa(cl, digit);
    pblock_nvinsert("content-length", digit, rq->srvhdrs);
    pblock_nvinsert("content-type", "text/html", rq->srvhdrs);

    switch(protocol_start_response(sn, rq)) {
      case REQ_NOACTION:
        return REQ_NOACTION;
      case REQ_EXIT:
        return REQ_EXIT;
    }

    if(net_write(sn->csd, buf, cl) == IO_ERROR)
        return REQ_EXIT;
    return REQ_PROCEED;
}
