/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * log.c:  Records transactions, reports errors to administrators, etc.
 * 
 * Rob McCool
 */


#include "frame/log.h"

#include "netsite.h"
#include "base/util.h"       /* util_sprintf */
#include "frame/conf.h"
#include <stdarg.h>
#include <stdio.h>           /* vsprintf */


NSAPI_PUBLIC int log_error_v(int degree, char *func,
                             Session *sn, Request *rq, char *fmt, va_list args)
{
    char *t, *u, err2[MAX_ERROR_LEN];
    char errstr[MAX_ERROR_LEN], *rhost, *method, *uri;
    int pos;

    pos = 0;
    if(sn) {
        if(!(rhost = session_dns(sn))) 
            rhost = pblock_findval("ip", sn->client);
        pos += util_snprintf(errstr, MAX_ERROR_LEN, "for host %s", rhost);
    }
    if(rq) {
        method = pblock_findval("method", rq->reqpb);
        if((method) && (strlen(method) > ERROR_CUTOFF))
            method[ERROR_CUTOFF] = '\0';
        uri = pblock_findval("uri", rq->reqpb);
        if((uri) && (strlen(uri) > ERROR_CUTOFF))
            uri[ERROR_CUTOFF] = '\0';
        if(method && uri) {
            pos += util_snprintf(&errstr[pos], sizeof(errstr) - pos,
                                 "%strying to %s %s", (sn ? " " : ""), 
                                 method, uri);
        }
    }
    if(sn || rq)
        pos += util_snprintf(&errstr[pos], sizeof(errstr) - pos, ", ");
    if(func)
        pos += util_snprintf(&errstr[pos], sizeof(errstr) - pos, 
                             "%s reports: ", func);

    pos += util_vsnprintf(&errstr[pos], sizeof(errstr) - pos, fmt, args);

    errstr[pos] = '\0';

    for(pos = 0, t = errstr, u = err2; 
	pos < (MAX_ERROR_LEN-2) && *t; 
	++t, ++u, pos++) {
        if (*t == '%') {
            /* double '%' to avoid printf interpretation */
            *u++ = '%'; pos++;
            *u = '%';
        }
        else {
            *u = *t;
        }
    }
    *u = '\0';
    return ereport(degree, err2);
}

NSAPI_PUBLIC int log_error(int degree, char *func,
                           Session *sn, Request *rq, char *fmt, ...)
{
    va_list args;
    int rv;
    va_start(args, fmt);
    rv = log_error_v(degree, func, sn, rq, fmt, args);
    va_end(args);
    return rv;
}

/* Had to create this mapping to ereport in order to get
 * data from conf_getglobals into ereport- if I tried to 
 * do this in ereport.c, I got link errors for getglobals
 * to some of the supplemental programs.
 */
NSAPI_PUBLIC int log_ereport_v(int degree, char *fmt, va_list args)
{
    int rv;

    if (degree == LOG_VERBOSE && !conf_getglobals()->log_verbose)
        return IO_OKAY;

    rv = ereport_v(degree, fmt, args);
    return rv;      
}

NSAPI_PUBLIC int log_ereport(int degree, char *fmt, ...)
{
    int rv;
    va_list args;

    va_start(args, fmt);
    rv = log_ereport_v(degree, fmt, args);
    va_end(args);

    return rv;
}
