/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
#ifndef LIB_FRAME_NSAPI_H
#define LIB_FRAME_NSAPI_H

typedef void filebuf_t;
typedef void *PASSWD;
#ifndef XP_WIN32
typedef void VOID;
#endif /* !XP_WIN32 */
typedef void *SYS_FILE30;
typedef void *SYS_NETFD30;

/* From netsite.h */
NSAPI_PUBLIC char *INTsystem_version();
NSAPI_PUBLIC void *INTsystem_malloc(int size);
NSAPI_PUBLIC void *INTsystem_calloc(int size);
NSAPI_PUBLIC void *INTsystem_realloc(void *ptr, int size);
NSAPI_PUBLIC void INTsystem_free(void *ptr);
NSAPI_PUBLIC char *INTsystem_strdup(const char *ptr);
#ifndef NS_MALLOC_DEBUG
NSAPI_PUBLIC void *INTsystem_malloc_perm(int size);
NSAPI_PUBLIC void *INTsystem_calloc_perm(int size);
NSAPI_PUBLIC void *INTsystem_realloc_perm(void *ptr, int size);
NSAPI_PUBLIC void INTsystem_free_perm(void *ptr);
NSAPI_PUBLIC char *INTsystem_strdup_perm(const char *ptr);
#endif /* !NS_MALLOC_DEBUG */
NSAPI_PUBLIC int INTgetThreadMallocKey(void);
NSAPI_PUBLIC void INTmagnus_atrestart(void (*fn)(void *), void *data);
NSAPI_PUBLIC void INTsystem_version_set(char *str);

/* From base/buffer.h */
NSAPI_PUBLIC filebuf_t *INTfilebuf_open(SYS_FILE30 fd, int sz);
NSAPI_PUBLIC netbuf *INTnetbuf_open(SYS_NETFD30 sd, int sz);
NSAPI_PUBLIC
filebuf_t *INTfilebuf_create(SYS_FILE30 fd, caddr_t mmap_ptr, int mmap_len, 
                             int bufsz);
NSAPI_PUBLIC void INTfilebuf_close_buffer(filebuf_t *buf, int clean_mmap);
#ifdef FILE_MMAP
#include <sys/stat.h>
NSAPI_PUBLIC 
filebuf_t *INTfilebuf_open_nostat(SYS_FILE30 fd, int sz, struct stat *finfo);
#else  /* !MMAP */
#define INTfilebuf_open_nostat(fd,sz,finfo) INTfilebuf_open(fd,sz)
#endif
#ifdef XP_WIN32
NSAPI_PUBLIC 
filebuf_t *INTpipebuf_open(SYS_FILE30 fd, int sz, struct stat *finfo);
#endif /* XP_WIN32 */
#ifndef FILE_MMAP
NSAPI_PUBLIC int INTfilebuf_next(filebuf_t *buf, int advance);
#endif /* !FILE_MMAP */
NSAPI_PUBLIC int INTnetbuf_next(netbuf *buf, int advance);
#ifdef XP_WIN32 
NSAPI_PUBLIC int INTpipebuf_next(filebuf_t *buf, int advance);
#endif /* XP_WIN32 */
NSAPI_PUBLIC void INTfilebuf_close(filebuf_t *buf);
NSAPI_PUBLIC void INTnetbuf_close(netbuf *buf);
#ifdef XP_WIN32
NSAPI_PUBLIC void	INTpipebuf_close(filebuf_t *buf);
#endif /* XP_WIN32 */
NSAPI_PUBLIC int INTfilebuf_grab(filebuf_t *buf, int sz);
NSAPI_PUBLIC int INTnetbuf_grab(netbuf *buf, int sz);
#ifdef XP_WIN32
NSAPI_PUBLIC int INTpipebuf_grab(filebuf_t *buf, int sz);
#endif /* XP_WIN32 */
NSAPI_PUBLIC int INTnetbuf_buf2sd(netbuf *buf, SYS_NETFD30 sd, int len);
NSAPI_PUBLIC int INTfilebuf_buf2sd(filebuf_t *buf, SYS_NETFD30 sd);
#ifdef XP_WIN32
NSAPI_PUBLIC int INTpipebuf_buf2sd(filebuf_t *buf, SYS_NETFD30 sd, int len);
NSAPI_PUBLIC int INTpipebuf_netbuf2sd(netbuf *buf, SYS_FILE30 sd, int len);
NSAPI_PUBLIC int INTpipebuf_netbuf2pipe(netbuf *buf, SYS_NETFD30 sd, int len);
#endif /* XP_WIN32 */

/* From base/cinfo.h */
NSAPI_PUBLIC void INTcinfo_init(void);
NSAPI_PUBLIC void INTcinfo_terminate(void);
NSAPI_PUBLIC char *INTcinfo_merge(char *fn);
NSAPI_PUBLIC cinfo *INTcinfo_find(char *uri);
NSAPI_PUBLIC cinfo *INTcinfo_lookup(char *type);
#include <stdio.h>
NSAPI_PUBLIC void INTcinfo_dump_database(FILE *dump);

/* From base/crit.h */
NSAPI_PUBLIC CRITICAL INTcrit_init(void);
NSAPI_PUBLIC void INTcrit_enter(CRITICAL id);
NSAPI_PUBLIC void INTcrit_exit(CRITICAL id);
NSAPI_PUBLIC void INTcrit_terminate(CRITICAL id);
NSAPI_PUBLIC CONDVAR INTcondvar_init(CRITICAL id);
NSAPI_PUBLIC void INTcondvar_wait(CONDVAR cv);
NSAPI_PUBLIC void INTcondvar_notify(CONDVAR cv);
NSAPI_PUBLIC void INTcondvar_terminate(CONDVAR cv);
NSAPI_PUBLIC COUNTING_SEMAPHORE INTcs_init(int initial_count);
NSAPI_PUBLIC void INTcs_terminate(COUNTING_SEMAPHORE csp);
NSAPI_PUBLIC int INTcs_wait(COUNTING_SEMAPHORE csp);
NSAPI_PUBLIC int INTcs_trywait(COUNTING_SEMAPHORE csp);
NSAPI_PUBLIC int INTcs_release(COUNTING_SEMAPHORE csp);

/* From base/daemon.h */
NSAPI_PUBLIC void INTdaemon_atrestart(void (*fn)(void *), void *data);
#ifdef XP_UNIX
NSAPI_PUBLIC pid_t INTchild_fork(void);
#endif /* XP_UNIX */

/* From base/ereport.h */
NSAPI_PUBLIC int INTereport(int degree, char *fmt, ...);
NSAPI_PUBLIC int INTereport_v(int degree, char *fmt, va_list args);
NSAPI_PUBLIC char *INTereport_init(char *err_fn, char *email, PASSWD pw, char *buildid);
NSAPI_PUBLIC void INTereport_terminate(void);
NSAPI_PUBLIC SYS_FILE30 INTereport_getfd(void);

/* From base/file.h */
NSAPI_PUBLIC SYS_FILE30 INTsystem_fopenRO(char *path);
NSAPI_PUBLIC SYS_FILE30 INTsystem_fopenWA(char *path);
NSAPI_PUBLIC SYS_FILE30 INTsystem_fopenRW(char *path);
NSAPI_PUBLIC SYS_FILE30 INTsystem_fopenWT(char *path);
NSAPI_PUBLIC int INTsystem_fread(SYS_FILE30 fd, char *buf, int sz);
NSAPI_PUBLIC int INTsystem_fwrite(SYS_FILE30 fd,char *buf,int sz);
NSAPI_PUBLIC int INTsystem_fwrite_atomic(SYS_FILE30 fd, char *buf, int sz);
NSAPI_PUBLIC int INTsystem_lseek(SYS_FILE30 fd, int off, int wh);
NSAPI_PUBLIC int INTsystem_fclose(SYS_FILE30 fd);
NSAPI_PUBLIC int INTsystem_stat(char *name, struct stat *finfo);
NSAPI_PUBLIC int INTsystem_rename(char *oldpath, char *newpath);
NSAPI_PUBLIC int INTsystem_unlink(char *path);
NSAPI_PUBLIC int INTsystem_tlock(SYS_FILE30 fd);
NSAPI_PUBLIC int INTsystem_flock(SYS_FILE30 fd);
NSAPI_PUBLIC int INTsystem_ulock(SYS_FILE30 fd);
#ifdef XP_UNIX
#ifndef OS_DIRENT_H
#include <dirent.h>
#define OS_DIRENT_H
#endif /* !OS_DIRENT_H */
#define INTdir_open opendir
#define INTdir_read readdir
#define INTdir_close closedir
#define INTdir_create(path) mkdir(path, 0755)
#define INTdir_remove rmdir
#define INTsystem_chdir chdir
#elif defined XP_WIN32
NSAPI_PUBLIC SYS_DIR INTdir_open(char *path);
NSAPI_PUBLIC SYS_DIRENT *INTdir_read(SYS_DIR ds);
NSAPI_PUBLIC void INTdir_close(SYS_DIR ds);
#define INTdir_create _mkdir
#define INTdir_remove _rmdir
#define INTsystem_chdir SetCurrentDirectory
#endif /* XP_WIN32 */
NSAPI_PUBLIC int INTdir_create_all(char *dir);
#ifdef HAVE_TIME_R
#ifdef OSF1 /* grumble */
#define INTsystem_localtime(curtime, ret) (localtime_r(curtime, ret), ret)
#define INTsystem_gmtime(curtime, ret) (gmtime_r(curtime, ret), ret)
#else
#define INTsystem_localtime(curtime, ret) localtime_r(curtime, ret)
#define INTsystem_gmtime(curtime, ret) gmtime_r(curtime, ret)
#endif
#else
#define INTsystem_localtime(curtime, ret) localtime(curtime)
#define INTsystem_gmtime(curtime, ret) gmtime(curtime)
#endif
#ifdef XP_WIN32
NSAPI_PUBLIC char *INTsystem_winsockerr(void);
NSAPI_PUBLIC char *INTsystem_winerr(void);
NSAPI_PUBLIC int INTsystem_pread(SYS_FILE30 fd, char *buf, int sz);
NSAPI_PUBLIC int INTsystem_pwrite(SYS_FILE30 fd, char *buf, int sz);
NSAPI_PUBLIC void INTfile_unix2local(char *path, char *p2);
#endif /* XP_WIN32 */
#ifdef XP_UNIX
#define INTfile_unix2local(path,p2) strcpy(p2,path)
#endif /* XP_UNIX */
NSAPI_PUBLIC int INTsystem_nocoredumps(void);
NSAPI_PUBLIC int INTfile_setinherit(SYS_FILE30 fd, int value);
NSAPI_PUBLIC int INTfile_notfound(VOID);
NSAPI_PUBLIC char *INTsystem_errmsg(void);
NSAPI_PUBLIC int INTsystem_errmsg_fn(char **buff, size_t maxlen);

/* From base/net.h */
NSAPI_PUBLIC SYS_NETFD30 INTnet_socket(int domain, int type, int protocol);
NSAPI_PUBLIC int INTnet_listen(SYS_NETFD30 s, int backlog);
NSAPI_PUBLIC SYS_NETFD30 INTnet_create_listener(char *ipaddr, int port);
NSAPI_PUBLIC
int INTnet_connect(SYS_NETFD30 s, const void *sockaddr, int namelen);
NSAPI_PUBLIC
int INTnet_getpeername(SYS_NETFD30 s, struct sockaddr *name, int *namelen);
NSAPI_PUBLIC int INTnet_close(SYS_NETFD30 s);
NSAPI_PUBLIC
int INTnet_bind(SYS_NETFD30 s, const struct sockaddr *name, int namelen);
NSAPI_PUBLIC
SYS_NETFD30 INTnet_accept(SYS_NETFD30 s, struct sockaddr *addr, int *addrlen);
NSAPI_PUBLIC int INTnet_read(SYS_NETFD30 sd, char *buf, int sz, int timeout);
NSAPI_PUBLIC int INTnet_write(SYS_NETFD30 sd, char *buf, int sz);
NSAPI_PUBLIC int INTnet_writev(SYS_NETFD30 sd, struct iovec *iov, int iovlen);
NSAPI_PUBLIC int INTnet_isalive(SYS_NETFD30 sd);
NSAPI_PUBLIC char *INTnet_ip2host(char *ip, int verify);
NSAPI_PUBLIC int INTnet_getsockopt(SYS_NETFD30 s, int level, int optname,
                                  void *optval, int *optlen);
NSAPI_PUBLIC int INTnet_setsockopt(SYS_NETFD30 s, int level, int optname,
                                  const void *optval, int optlen);
NSAPI_PUBLIC int INTnet_select(int nfds, fd_set *r, fd_set *w, fd_set *e, 
                              struct timeval *timeout);
NSAPI_PUBLIC int INTnet_ioctl(SYS_NETFD30 s, int tag, void *result);

/* From base/pblock.h */
NSAPI_PUBLIC pb_param *INTparam_create(char *name, char *value);
NSAPI_PUBLIC int INTparam_free(pb_param *pp);
NSAPI_PUBLIC pblock *INTpblock_create(int n);
NSAPI_PUBLIC void INTpblock_free(pblock *pb);
#define INTpblock_find(name, pb) (INTpblock_fr(name,pb,0))
NSAPI_PUBLIC char *INTpblock_findval(const char *name, pblock *pb);
#define INTpblock_remove(name, pb) (INTpblock_fr(name,pb,1))
NSAPI_PUBLIC pb_param *INTpblock_nvinsert(const char *name, const char *value, pblock *pb);
NSAPI_PUBLIC pb_param *INTpblock_nninsert(const char *name, int value, pblock *pb);
NSAPI_PUBLIC void INTpblock_pinsert(pb_param *pp, pblock *pb);
NSAPI_PUBLIC int INTpblock_str2pblock(const char *str, pblock *pb);
NSAPI_PUBLIC char *INTpblock_pblock2str(pblock *pb, char *str);
NSAPI_PUBLIC void INTpblock_copy(pblock *src, pblock *dst);
NSAPI_PUBLIC pblock *INTpblock_dup(pblock *src);
NSAPI_PUBLIC char **INTpblock_pb2env(pblock *pb, char **env);
NSAPI_PUBLIC pb_param *INTpblock_fr(const char *name, pblock *pb, int remove);
NSAPI_PUBLIC char * INTpblock_replace(const char *name,char * new_value,pblock *pb);

/* From base/pool.h */
NSAPI_PUBLIC pool_handle_t *INTpool_create(void);
NSAPI_PUBLIC void INTpool_destroy(pool_handle_t *pool_handle);
NSAPI_PUBLIC int INTpool_enabled(void);
NSAPI_PUBLIC void *INTpool_malloc(pool_handle_t *pool_handle, size_t size );
NSAPI_PUBLIC void INTpool_free(pool_handle_t *pool_handle, void *ptr );
NSAPI_PUBLIC 
void *INTpool_calloc(pool_handle_t *pool_handle, size_t nelem, size_t elsize);
NSAPI_PUBLIC 
void *INTpool_realloc(pool_handle_t *pool_handle, void *ptr, size_t size );
NSAPI_PUBLIC char *INTpool_strdup(pool_handle_t *pool_handle, const char *orig_str );

/* From base/regexp.h */
#ifdef USE_REGEX
NSAPI_PUBLIC int INTregexp_valid(char *exp);
NSAPI_PUBLIC int INTregexp_match(char *str, char *exp);
NSAPI_PUBLIC int INTregexp_cmp(char *str, char *exp);
NSAPI_PUBLIC int INTregexp_casecmp(char *str, char *exp);
#endif /* USE_REGEX */

/* From base/sem.h */
NSAPI_PUBLIC SEMAPHORE INTsem_init(char *name, int number);
NSAPI_PUBLIC void INTsem_terminate(SEMAPHORE id);
NSAPI_PUBLIC int INTsem_grab(SEMAPHORE id);
NSAPI_PUBLIC int INTsem_tgrab(SEMAPHORE id);
NSAPI_PUBLIC int INTsem_release(SEMAPHORE id);

/* From base/session.h */
NSAPI_PUBLIC Session *INTsession_alloc(SYS_NETFD30 csd, struct sockaddr_in *sac); /* internal */
NSAPI_PUBLIC Session *INTsession_fill(Session *sn); /* internal */
NSAPI_PUBLIC Session *INTsession_create(SYS_NETFD30 csd, struct sockaddr_in *sac);
NSAPI_PUBLIC void INTsession_free(Session *sn);
#define INTsession_dns(sn) INTsession_dns_lookup(sn, 0)
#define INTsession_maxdns(sn) INTsession_dns_lookup(sn, 1)
NSAPI_PUBLIC char *INTsession_dns_lookup(Session *sn, int verify);

/* From base/shexp.h */
NSAPI_PUBLIC int INTshexp_valid(char *exp);
NSAPI_PUBLIC int INTshexp_match(char *str, char *exp);
NSAPI_PUBLIC int INTshexp_cmp(char *str, char *exp);
NSAPI_PUBLIC int INTshexp_casecmp(char *str, char *exp);

/* From base/shmem.h */
#if defined (SHMEM_UNIX_MMAP) || defined (SHMEM_WIN32_MMAP)
NSAPI_PUBLIC shmem_s *INTshmem_alloc(char *name, int size, int expose);
NSAPI_PUBLIC void INTshmem_free(shmem_s *region);
#endif  /* SHMEM_UNIX_MMAP || SHMEM_WIN32_MMAP */

/* From base/systems.h */

/* From base/systhr.h */
NSAPI_PUBLIC
SYS_THREAD INTsysthread_start(int prio, int stksz, void (*fn)(void *), void *arg);
NSAPI_PUBLIC SYS_THREAD INTsysthread_current(void);
NSAPI_PUBLIC void INTsysthread_yield(void);
NSAPI_PUBLIC SYS_THREAD INTsysthread_attach(void);
NSAPI_PUBLIC void INTsysthread_detach(SYS_THREAD thr);
NSAPI_PUBLIC void INTsysthread_terminate(SYS_THREAD thr);
NSAPI_PUBLIC void INTsysthread_sleep(int milliseconds);
NSAPI_PUBLIC void INTsysthread_init(char *name);
NSAPI_PUBLIC void INTsysthread_timerset(int usec);
NSAPI_PUBLIC int INTsysthread_newkey(void);
NSAPI_PUBLIC void *INTsysthread_getdata(int key);
NSAPI_PUBLIC void INTsysthread_setdata(int key, void *data);
NSAPI_PUBLIC 
void INTsysthread_set_default_stacksize(unsigned long size);

/* From base/util.h */
NSAPI_PUBLIC
int INTutil_getline(filebuffer *buf, int lineno, int maxlen, char *l);
NSAPI_PUBLIC char **INTutil_env_create(char **env, int n, int *pos);
NSAPI_PUBLIC char *INTutil_env_str(char *name, char *value);
NSAPI_PUBLIC void INTutil_env_replace(char **env, char *name, char *value);
NSAPI_PUBLIC void INTutil_env_free(char **env);
NSAPI_PUBLIC char **INTutil_env_copy(char **src, char **dst);
NSAPI_PUBLIC char *INTutil_env_find(char **env, char *name);
NSAPI_PUBLIC char *INTutil_hostname(void);
NSAPI_PUBLIC int INTutil_chdir2path(char *path);
NSAPI_PUBLIC int INTutil_is_mozilla(char *ua, char *major, char *minor);
NSAPI_PUBLIC int INTutil_is_url(char *url);
NSAPI_PUBLIC int INTutil_later_than(struct tm *lms, char *ims);
NSAPI_PUBLIC int INTutil_time_equal(struct tm *lms, char *ims);
NSAPI_PUBLIC int INTutil_str_time_equal(char *t1, char *t2);
NSAPI_PUBLIC int INTutil_uri_is_evil(char *t);
NSAPI_PUBLIC void INTutil_uri_parse(char *uri);
NSAPI_PUBLIC void INTutil_uri_unescape(char *s);
NSAPI_PUBLIC char *INTutil_uri_escape(char *d, char *s);
NSAPI_PUBLIC char *INTutil_url_escape(char *d, char *s);
NSAPI_PUBLIC char *INTutil_sh_escape(char *s);
NSAPI_PUBLIC int INTutil_mime_separator(char *sep);
NSAPI_PUBLIC int INTutil_itoa(int i, char *a);
#ifndef OS_STDARG_H
#include <stdarg.h>
#define OS_STDARG_H
#endif /* !OS_STDARG_H */
NSAPI_PUBLIC int INTutil_vsprintf(char *s, register const char *fmt, va_list args);
NSAPI_PUBLIC int INTutil_sprintf(char *s, const char *fmt, ...);
NSAPI_PUBLIC int INTutil_vsnprintf(char *s, int n, register const char *fmt, 
                                va_list args);
NSAPI_PUBLIC int INTutil_snprintf(char *s, int n, const char *fmt, ...);
NSAPI_PUBLIC int INTutil_strftime(char *s, const char *format, const struct tm *t);
#ifdef NEED_STRCASECMP
NSAPI_PUBLIC int INTutil_strcasecmp(CASECMPARG_T char *one, CASECMPARG_T char *two);
NSAPI_PUBLIC int strncasecmp(CASECMPARG_T char *one, CASECMPARG_T char *two, int n);
#endif /* NEED_STRCASECMP */
#ifdef XP_UNIX
#ifndef OS_SYS_STAT_H
#include <sys/stat.h>
#define OS_SYS_STAT_H
#endif /* !OS_SYS_STAT_H */
#ifndef OS_SYS_TYPES_H
#include <sys/types.h>
#define OS_SYS_TYPES_H
#endif /* !OS_SYS_TYPES_H */
NSAPI_PUBLIC int INTutil_can_exec(struct stat *finfo, uid_t uid, gid_t gid);
#endif /* XP_UNIX */
#ifdef XP_WIN32
NSAPI_PUBLIC VOID INTutil_delete_directory(char *FileName, BOOL delete_directory);
#endif /* XP_WIN32 */

/* From frame/conf.h */
NSAPI_PUBLIC char *INTconf_init(char *cfn);
NSAPI_PUBLIC char *INTconf_run_init_functions(void);
NSAPI_PUBLIC void INTconf_terminate(void);
NSAPI_PUBLIC conf_global_vars_s *INTconf_getglobals(void);
NSAPI_PUBLIC void INTconf_vars2daemon(daemon_s *d);
#ifdef XP_WIN32
NSAPI_PUBLIC BOOLEAN INTGetDomainAddresses(PCHAR ErrMsg);
#endif /* XP_WIN32 */

/* From frame/func.h */
#ifndef XP_WIN32
NSAPI_PUBLIC void INTfunc_init(struct FuncStruct *func_standard);
#endif /* !XP_WIN32 */
NSAPI_PUBLIC FuncPtr INTfunc_find(char *name);
NSAPI_PUBLIC int INTfunc_exec(pblock *pb, Session *sn, Request *rq);
NSAPI_PUBLIC struct FuncStruct *INTfunc_insert(char *name, FuncPtr fn);
NSAPI_PUBLIC int INTobject_execute(directive *inst, Session *sn, void *rq);

/* From frame/http.h */
#define INTprotocol_find_request INThttp_find_request
NSAPI_PUBLIC Request *INThttp_find_request(netbuf *buf, Session *sn, int *reply);
#define INTprotocol_parse_request INThttp_parse_request
NSAPI_PUBLIC int INThttp_parse_request(char *t, Request *rq, Session *sn);
#define INTprotocol_scan_headers INThttp_scan_headers
NSAPI_PUBLIC int INThttp_scan_headers(Session *sn, netbuf *buf, char *t, 
                                   pblock *headers);
#define INTprotocol_start_response INThttp_start_response
NSAPI_PUBLIC int INThttp_start_response(Session *sn, Request *rq);
NSAPI_PUBLIC char **INThttp_hdrs2env(pblock *pb);
#define INTprotocol_status INThttp_status
NSAPI_PUBLIC void INThttp_status(Session *sn, Request *rq, int n, char *r);
#define INTprotocol_set_finfo INThttp_set_finfo
NSAPI_PUBLIC int INThttp_set_finfo(Session *sn, Request *rq, struct stat *finfo);
NSAPI_PUBLIC char *INThttp_dump822(pblock *pb, char *t, int *pos, int tsz);
#define INTprotocol_finish_request INThttp_finish_request
NSAPI_PUBLIC void INThttp_finish_request(Session *sn, Request *rq);
#define INTprotocol_handle_session INThttp_handle_session
NSAPI_PUBLIC void INThttp_handle_session(Session *sn);
#define INTprotocol_uri2url INThttp_uri2url
#define INTprotocol_uri2url_dynamic INThttp_uri2url_dynamic
NSAPI_PUBLIC char *INThttp_uri2url(const char *prefix, const char *suffix);
NSAPI_PUBLIC char *INThttp_uri2url_dynamic(const char *prefix, const char *suffix,
                                        Session *sn, Request *rq);
#define INTprotocol_set_keepalive_timeout INThttp_set_keepalive_timeout
NSAPI_PUBLIC void INThttp_set_keepalive_timeout(int secs);

/* From frame/log.h */
NSAPI_PUBLIC int INTlog_error_v(int degree, char *func, Session *sn,
                               Request *rq, char *fmt, va_list args);
NSAPI_PUBLIC int INTlog_error(int degree, char *func, Session *sn, Request *rq, 
                             char *fmt, ...);
NSAPI_PUBLIC int INTlog_ereport_v(int degree, char *fmt, va_list args);
NSAPI_PUBLIC int INTlog_ereport(int degree, char *fmt, ...);

/* From frame/object.h */
NSAPI_PUBLIC httpd_object *INTobject_create(int nd, pblock *name);
NSAPI_PUBLIC void INTobject_free(httpd_object *obj);
NSAPI_PUBLIC void INTobject_add_directive(int dc, pblock *p, pblock *c, 
                                         httpd_object *obj);

/* From frame/objset.h */
NSAPI_PUBLIC httpd_objset *INTobjset_scan_buffer(filebuffer *buf, char *errstr, 
                                              httpd_objset *os);
NSAPI_PUBLIC httpd_objset *INTobjset_create(void);
NSAPI_PUBLIC void INTobjset_free(httpd_objset *os);
NSAPI_PUBLIC void INTobjset_free_setonly(httpd_objset *os);
NSAPI_PUBLIC httpd_object *INTobjset_new_object(pblock *name, httpd_objset *os);
NSAPI_PUBLIC void INTobjset_add_object(httpd_object *obj, httpd_objset *os);
NSAPI_PUBLIC void INTobjset_add_init(pblock *initfn, httpd_objset *os);
NSAPI_PUBLIC httpd_object *INTobjset_findbyname(char *name, httpd_objset *ign, 
                                             httpd_objset *os);
NSAPI_PUBLIC httpd_object *INTobjset_findbyppath(char *ppath, httpd_objset *ign, 
                                              httpd_objset *os);

/* From frame/protocol.h */

/* From frame/req.h */
NSAPI_PUBLIC Request *INTrequest_create(void);
NSAPI_PUBLIC void INTrequest_free(Request *req);
NSAPI_PUBLIC Request *INTrequest_restart_internal(char *uri, Request *rq);
NSAPI_PUBLIC int INTrequest_header(char *name, char **value, 
                                Session *sn, Request *rq);
#define INTrequest_loadheaders(sn,rq) (int)(REQ_PROCEED)
NSAPI_PUBLIC struct stat *INTrequest_stat_path(char *path, Request *rq);

/* From frame/httpact.h */
NSAPI_PUBLIC int INTservact_uri2path(Session *sn, Request *rq);
NSAPI_PUBLIC int INTservact_pathchecks(Session *sn, Request *rq);
NSAPI_PUBLIC int INTservact_fileinfo(Session *sn, Request *rq);
NSAPI_PUBLIC int INTservact_service(Session *sn, Request *rq);
NSAPI_PUBLIC int INTservact_handle_processed(Session *sn, Request *rq);
NSAPI_PUBLIC char *INTservact_translate_uri(char *uri, Session *sn);
NSAPI_PUBLIC int INTservact_finderror(Session *sn, Request *rq);

#endif /* !LIB_FRAME_NSAPI_H */
