/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * ntobjset.c: gets parameters from registry to make an object set
 * 
 * See objset.h for details.
 * 
 * Rob McCool
 */


#include "frame/objset.h"
#include <frame/conf.h>

#include "netsite.h"     /* safe malloc, free, strdup */

#include "base/shexp.h"   /* shexp_cmp */
#include "base/util.h"    /* util_itoa, util_sprintf */
#include <base/pblock.h>
#include <base/eventlog.h>
#include <base/ereport.h>

#include <string.h>          /* strlen, strncmp, strcpy */
#include <ctype.h>           /* isalpha, toupper */
#include <nt/regparms.h>
#include <nt/messages.h>

#include "frame/dbtframe.h"

/* -------------------------- objset_scan_buffer -------------------------- */

#define syntax_error(oops) \
  { objset_free(nos);  util_sprintf(errstr, "%d: %s", ln, oops); return NULL; }



httpd_objset *objset_scan_registry(char *errstr, httpd_objset *os, char *product_key)
{
    httpd_objset *nos = (os ? os : objset_create());
    int clrefs;
    pblock *client;

	CHAR ObjectsKey[512];
	HKEY hObjectsKey, hObjectKey;

	CHAR Object[20];
	DWORD ObjectIndex;
	CHAR ErrorMsg[256];

	DWORD ObjectLength, dwDisposition;
	LONG Result, ScanResult;
	PCHAR key;

	client = NULL;
    clrefs = 0;


	// Figure out the product key from the service name

	if (product_key) {
		sprintf(ObjectsKey, "%s\\%s\\%s\\%s", COMPANY_KEY, product_key, 
			VERSION_KEY, OBJECTS_KEY);
	} else {
		key = strchr(service_name, ' ');
		key++;
		sprintf(ObjectsKey, "%s\\%s\\%s\\%s", COMPANY_KEY, key, 
			VERSION_KEY, OBJECTS_KEY);
	}	  
	// Open the top level registry key

	Result = RegCreateKeyEx(HKEY_LOCAL_MACHINE, ObjectsKey, 0, NULL, 0,
		KEY_ALL_ACCESS, NULL, &hObjectsKey, &dwDisposition);
	if (Result != ERROR_SUCCESS) {
		sprintf(ErrorMsg,":Objects Key failed %d",Result);
		LogErrorEvent(service_name, EVENTLOG_ERROR_TYPE, 0, 
			MSG_BAD_REGISTRY_KEY_OPEN, ErrorMsg, NULL);
		return NULL;
	}

	// There will be three sub loops in the following procedure	for the object,directive and 
	// function.First scan all the objects. This procedure executes the first loop (ie) it
	// scans for objects

	ObjectIndex = 0;
    while(1) {
		ObjectLength = 20;   
    	ScanResult = RegEnumKey(hObjectsKey, ObjectIndex++, Object,
							ObjectLength);
		if (ScanResult != ERROR_SUCCESS) {
			if (ScanResult == ERROR_NO_MORE_ITEMS) {
				RegCloseKey(hObjectsKey);
				return nos;
			} else {
				sprintf(ErrorMsg,"ObjectsKey failed: %d",ScanResult);
				LogErrorEvent(service_name, EVENTLOG_ERROR_TYPE, 0, 
					MSG_BAD_REGISTRY_KEY_ENUM, ErrorMsg, NULL);
				return NULL;
			}
		}
		// LogErrorEvent("ProcessObject",Object);
		ScanResult = RegCreateKeyEx( hObjectsKey, Object, 0, NULL, 0,
			KEY_ALL_ACCESS, NULL, &hObjectKey, &dwDisposition);
		if (ScanResult != ERROR_SUCCESS) {
			sprintf(ErrorMsg, "ObjectKey %s failed: %d", Object, Result);
			LogErrorEvent(service_name, EVENTLOG_ERROR_TYPE, 0, 
				MSG_BAD_REGISTRY_KEY_OPEN, ErrorMsg, NULL);
			return NULL;
		} else {
			// LogErrorEvent("RegCreateKey succeeded",Object);
		}
		ProcessObject(nos, hObjectKey, Object, ObjectIndex);
		RegCloseKey(hObjectKey);

	} // while(1) loop of scan of objects
}


VOID 
ProcessObject(
	httpd_objset *nos,
	HKEY hObjectKey,
	CHAR *Object,
	DWORD ObjectIndex
	)
{
	DWORD ValueStringLength;
	CHAR  DirectiveString[20], *ParameterNameString, *ParameterValueString;
	DWORD DirectiveIndex, ScanResult,DirectiveLength, ObjectValueIndex, NameStringLength;
	DWORD CurrentNameLength, CurrentValueLength;
	CHAR ErrorMsg[512];
	httpd_object *cobj;
	pblock *param;
	int refs;
	BOOLEAN Result;

	ParameterNameString = (PCHAR)MALLOC(256);
	ParameterValueString = (PCHAR)MALLOC(256);
	CurrentNameLength = 256; CurrentValueLength = 256;

	param = pblock_create(PARAMETER_HASH_SIZE);
	cobj = objset_new_object(param, nos);
	ObjectValueIndex = 0;
	while (1) {
		NameStringLength = CurrentNameLength; ValueStringLength = CurrentValueLength;
ENUM_OBJECT:
		ScanResult = RegEnumValue (
							hObjectKey, ObjectValueIndex++, ParameterNameString,
							&NameStringLength, 0, NULL, ParameterValueString,
							&ValueStringLength);
		if (ScanResult == ERROR_NO_MORE_ITEMS) {
			break;
		} else if (ScanResult != ERROR_SUCCESS) {
			if ( ScanResult != ERROR_MORE_DATA) {
				sprintf(ErrorMsg,"%s Key failed: %d",
							Object, ScanResult);
				LogErrorEvent(service_name, EVENTLOG_ERROR_TYPE, 0,
					MSG_BAD_REGISTRY_VALUE_ENUM, 
					ErrorMsg,NULL);
				FREE(ParameterNameString); FREE(ParameterValueString);
				return;
			} else if ( NameStringLength > CurrentNameLength) {
				CurrentNameLength = NameStringLength;
				ParameterNameString = (PCHAR)REALLOC(ParameterNameString, 
					CurrentNameLength);
				ObjectValueIndex--;
				goto ENUM_OBJECT;
			} else if (ValueStringLength > CurrentValueLength) {
				CurrentValueLength = ValueStringLength;
 				ParameterValueString = (PCHAR)REALLOC(ParameterValueString, 
					CurrentValueLength);
				ObjectValueIndex--;
				goto ENUM_OBJECT;
			}
		} else {
			if (strcmp(ParameterNameString, OBJECT_NAME_KEY) && 
				strcmp(ParameterNameString, OBJECT_PATH_KEY)) {
				// error in this object key. Move onto the next object key
				// print a message and return
				sprintf(ErrorMsg,"%s has been given for Object Key: %d",
							ParameterNameString, ObjectIndex);
				LogErrorEvent(service_name, EVENTLOG_ERROR_TYPE, 0, 
					MSG_BAD_OBJECT_VALUE, ErrorMsg,NULL);
				FREE(ParameterNameString); FREE(ParameterValueString);
				return;
			} else {

				//enter the parameters
				// LogErrorEvent(ParameterNameString,ParameterValueString);
				if (ntpblock_str2pblock(ObjectValueIndex,ParameterNameString, ParameterValueString,
						NameStringLength, ValueStringLength, param) == -1) { 
					sprintf(ErrorMsg, "%s for Object Value Name %s",
						ParameterValueString, ParameterNameString);
					LogErrorEvent(service_name, EVENTLOG_ERROR_TYPE, 0, 
						MSG_BAD_PBLOCK, ErrorMsg,NULL);
					FREE(ParameterNameString); FREE(ParameterValueString);
					return;
				}

			}
		}
	} // while(1) more values to scan in the Client sub-key
	
	FREE(ParameterNameString); FREE(ParameterValueString);						 
	DirectiveIndex = 0;
	while (1) {
		// Open the directives under the object ...
		DirectiveLength = 20;
		ScanResult = RegEnumKey(hObjectKey, DirectiveIndex++, DirectiveString,
				DirectiveLength);

		if (ScanResult != ERROR_SUCCESS) {
			if (ScanResult == ERROR_NO_MORE_ITEMS) {
					return;
			} else if (ScanResult != ERROR_MORE_DATA){
				sprintf(ErrorMsg,"Directive Key %d under Object Key %d failed : %d",
							DirectiveIndex, ObjectIndex, ScanResult);
				LogErrorEvent(service_name, EVENTLOG_ERROR_TYPE, 0, 
					MSG_BAD_REGISTRY_KEY_ENUM,ErrorMsg,0);
				return;
			}
		} else {
			//LogErrorEvent("Directive",DirectiveString);
			Result = ProcessDirective(cobj, param, NULL, hObjectKey, DirectiveString, ObjectIndex, 
					DirectiveIndex, &refs, TRUE);
		}
	}	//while(1)  scan of directives loop
}

BOOLEAN
ProcessClient(
	httpd_object *cobj,
	HKEY hObjectKey,
	PCHAR Client,
	DWORD ObjectIndex
	)
{
	DWORD NameStringLength, ValueStringLength, dwDisposition;
	DWORD CurrentNameLength, CurrentValueLength;
	CHAR  DirectiveString[256], *ParameterNameString, *ParameterValueString;
	DWORD DirectiveIndex, ScanResult, DirectiveLength, ClientValueIndex;
	HKEY hClientKey;
	CHAR ErrorMsg[512];
	pblock *client;
	int clrefs;
	LONG Result;

	// Open the directive...
	Result = RegCreateKeyEx( hObjectKey, Client, 0, NULL, 0,
		KEY_ALL_ACCESS, NULL, &hClientKey, &dwDisposition);
	if (Result != ERROR_SUCCESS) {
		sprintf(ErrorMsg, "Client Key %s under Object Key %d failed %d",
				Client, ObjectIndex, Result);
		LogErrorEvent(service_name, EVENTLOG_ERROR_TYPE, 0, 
			MSG_BAD_REGISTRY_KEY_OPEN,ErrorMsg, 0);
		return FALSE;
	}

	ClientValueIndex = 0;
	// Now get the Client values..
	client = pblock_create(PARAMETER_HASH_SIZE);
	
	ParameterNameString = (PCHAR)MALLOC(256);
	ParameterValueString = (PCHAR)MALLOC(256);
	CurrentNameLength = CurrentValueLength = 256;

	while (1) {
		NameStringLength = CurrentNameLength ; ValueStringLength = CurrentNameLength;
ENUM_CLIENT:
		ScanResult = RegEnumValue(
							hClientKey, ClientValueIndex++, ParameterNameString,
							&NameStringLength, 0, NULL, ParameterValueString,
							&ValueStringLength);

		if (ScanResult == ERROR_NO_MORE_ITEMS) {
			break;
		} else if (ScanResult != ERROR_SUCCESS) {
			if (ScanResult != ERROR_MORE_DATA) {
				sprintf(ErrorMsg,"Client Key %s under Object Key %d failed %d",
							Client, ObjectIndex, ScanResult);
				LogErrorEvent(service_name, EVENTLOG_ERROR_TYPE, 
					0, MSG_BAD_REGISTRY_VALUE_ENUM, 
					ErrorMsg, NULL);
				return FALSE;
			} else if ( NameStringLength > CurrentNameLength) {
				CurrentNameLength = NameStringLength;
				ParameterNameString = (PCHAR)REALLOC(ParameterNameString, 
					CurrentNameLength);
				ClientValueIndex--;
				goto ENUM_CLIENT;
			} else if (ValueStringLength > CurrentValueLength) {
				CurrentValueLength = ValueStringLength;
 				ParameterValueString = (PCHAR)REALLOC(ParameterValueString, 
					CurrentValueLength);
				ClientValueIndex--;
				goto ENUM_CLIENT;
			}
		} else {
			//if (strcmp(ParameterNameString, DNS_KEY) && strcmp(ParameterNameString, IP_KEY)) {
				// error in this object key. Move onto the next object key
				// print a message and return
				//sprintf(ErrorMsg,"Value %s given for client key %s under object key %d",
					//ParameterNameString, Client, ObjectIndex);
				//LogErrorEvent(service_name, EVENTLOG_ERROR_TYPE, 0, 
					//MSG_BAD_CLIENT_VALUE,ErrorMsg,NULL);
				//continue;
			//} else {

				//enter the paramters
				//LogErrorEvent(ParameterNameString, ParameterValueString);
				if (ntpblock_str2pblock(ClientValueIndex, ParameterNameString, 
					ParameterValueString, 
					NameStringLength, ValueStringLength, client) == -1) {
					sprintf(ErrorMsg,"%s for Value %s for ClientKey %s under ObjectKey %d",
						ParameterValueString, ParameterNameString, Client, ObjectIndex);
					LogErrorEvent(service_name, EVENTLOG_ERROR_TYPE, 0, 
						MSG_BAD_PBLOCK, ErrorMsg, NULL);
				}
			//}
		}
	} // while(1) more values to scan in the Client sub-key
	
	FREE(ParameterNameString); FREE(ParameterValueString); 
	DirectiveIndex = 0;
	while (1) {
		// Open the directives under the client ...
		DirectiveLength = 256;
		ScanResult = RegEnumKey(hClientKey, DirectiveIndex++, DirectiveString,
							DirectiveLength);

		if (ScanResult != ERROR_SUCCESS) {
			if (ScanResult == ERROR_NO_MORE_ITEMS) {
						return TRUE;
			} else {
					sprintf(ErrorMsg,"Client Key %s (Directive %d) under ObjectKey %d failed : %d",
							Client,DirectiveIndex, ObjectIndex, ScanResult);
					LogErrorEvent(service_name, EVENTLOG_ERROR_TYPE, 0, 
						MSG_BAD_REGISTRY_KEY_ENUM, ErrorMsg,NULL);
					return FALSE;
			}
		} else {
				Result = ProcessDirective(cobj, NULL, client, hClientKey, DirectiveString, 
					ObjectIndex, DirectiveIndex, &clrefs, TRUE);
		}
	}	//while(1)  scan of directives loop
	if(clrefs) {
    	char t[20];
   		util_itoa(clrefs, t);
   		pblock_nvinsert("refcount", t, client);
   		client = NULL;
   	} else {
    	pblock_free(client);
	}
	return TRUE;
}

BOOLEAN ProcessDirective (
	httpd_object *cobj,
	pblock *param,
	pblock *client,
	HKEY hObjectKey,
	CHAR *Directive,
	DWORD ObjectIndex,
	DWORD DirectiveIndex,
	DWORD *clrefs,
	BOOLEAN IsObject
	)
{
	DWORD Result;
	// Open the client key...
	DWORD FunctionLength, dwDisposition;
	DWORD ValueStringLength, NameStringLength, CurrentNameLength, CurrentValueLength;
	CHAR  FunctionName[20], *ParameterNameString, *ParameterValueString;
	DWORD FunctionIndex, ScanResult;
	HKEY hDirectiveKey;
	CHAR ErrorMsg[512];
	int dnum;
	pblock *pb;

	ParameterNameString = (PCHAR)MALLOC(256);
	ParameterValueString = (PCHAR)MALLOC(256);
	CurrentNameLength = CurrentValueLength = NameStringLength = ValueStringLength = 256;

	// Open the directive...
	Result = RegCreateKeyEx( hObjectKey, Directive, 0, NULL, 0,
		KEY_ALL_ACCESS, NULL, &hDirectiveKey, &dwDisposition);
	if (Result != ERROR_SUCCESS) {
		sprintf(ErrorMsg, "Directive Key %d under Object Key %d failed :%d",
				DirectiveIndex, ObjectIndex, Result);
		LogErrorEvent(service_name, EVENTLOG_ERROR_TYPE, 0, 
			MSG_BAD_REGISTRY_KEY_OPEN, ErrorMsg, NULL);
		FREE(ParameterNameString); FREE(ParameterValueString);
		return FALSE;
	}

ENUM_DIRECTIVE_VALUE:
	ScanResult = RegEnumValue (
					hDirectiveKey, 0, ParameterNameString,
					&NameStringLength, 0, NULL, ParameterValueString,
					&ValueStringLength);

	if (ScanResult != ERROR_SUCCESS) {
		if (ScanResult != ERROR_MORE_DATA) {
			sprintf(ErrorMsg,"Directive Key %d under Object Key %d failed :%d",
							DirectiveIndex, ObjectIndex, ScanResult);
			LogErrorEvent(service_name, EVENTLOG_ERROR_TYPE, 0, 
				MSG_BAD_REGISTRY_KEY_ENUM, ErrorMsg, NULL);
			// Skip over this...
			return FALSE;
		} else if ( NameStringLength > CurrentNameLength) {
			CurrentNameLength = NameStringLength;
			ParameterNameString = (PCHAR)REALLOC(ParameterNameString, 
					CurrentNameLength);
			goto ENUM_DIRECTIVE_VALUE;
		} else if (ValueStringLength > CurrentValueLength) {
			CurrentValueLength = ValueStringLength;
 			ParameterValueString = (PCHAR)REALLOC(ParameterValueString, 
					CurrentValueLength);
			goto ENUM_DIRECTIVE_VALUE;
		}
	}
	if (!(strcmp(ParameterNameString, DNS_KEY)) || (!strcmp(ParameterNameString, IP_KEY)) ||
		!(strncmp(Directive, "Client", 6))) {
		// What we thought was a directive turns out to be a client sub-key with directives of
		// its own. Call ProcessClient to process this one.
		sprintf(ErrorMsg,"Processing Client...");
		//LogErrorEvent(ErrorMsg, NULL);
		RegCloseKey(hDirectiveKey);
 		return(ProcessClient(cobj, hObjectKey, Directive, ObjectIndex));
	
	}  
	/* Find directive number */
    if((dnum = directive_name2num(ParameterValueString)) == -1) {
		sprintf(ErrorMsg,"%s given for Directive %s under Object %d",
			ParameterValueString, Directive, ObjectIndex );
		LogErrorEvent(service_name, EVENTLOG_ERROR_TYPE, 0, 
			MSG_BAD_DIRECTIVE, ErrorMsg, NULL);
		FREE(ParameterNameString); FREE(ParameterValueString);
		return FALSE;
	}

	FunctionIndex = 0;
	// Now scan for each function under the directive...
	while (1) {
		FunctionLength = 20;
		// Open the sub-keys under the directive ...
		ScanResult = RegEnumKey ( hDirectiveKey, FunctionIndex++, FunctionName, 
			FunctionLength);

		if (ScanResult != ERROR_SUCCESS) {
			if (ScanResult == ERROR_NO_MORE_ITEMS) {
				break;
			} else {
				sprintf(ErrorMsg," DirectiveKey %s under Object %d failed :%d",
							Directive, ObjectIndex, ScanResult);
				LogErrorEvent(service_name, EVENTLOG_ERROR_TYPE, 0, 
					MSG_BAD_REGISTRY_KEY_ENUM, ErrorMsg, NULL);
				FREE(ParameterNameString); FREE(ParameterValueString);
				return FALSE;
			}
		} else {
			// LogErrorEvent("Function",FunctionName);
			pb = pblock_create(PARAMETER_HASH_SIZE);
			if (!ProcessFunction(&pb, hDirectiveKey, FunctionName,
					ObjectIndex, DirectiveIndex, FunctionIndex ))
					goto error;
    		object_add_directive(dnum, pb, client, cobj);
		}
	}

	if(!IsObject)
        ++(*clrefs);
	FREE(ParameterNameString); FREE(ParameterValueString);
	return TRUE;

error:
	return FALSE;

}

BOOLEAN
ProcessFunction(
	pblock **param,
	HKEY hDirectiveKey,
	CHAR *Function,
	DWORD ObjectIndex,
	DWORD DirectiveIndex,
	DWORD FunctionIndex
	)
{
	DWORD ValueStringLength, NameStringLength, dwDisposition;
	CHAR  *ParameterNameString, *ParameterValueString;
	DWORD ScanResult, Result,FunctionValueIndex, CurrentNameLength, CurrentValueLength;
	HKEY hFunctionKey;
	CHAR ErrorMsg[512];

	// Open the function...

	Result = RegCreateKeyEx( hDirectiveKey, Function, 0, NULL, 0,
		KEY_ALL_ACCESS, NULL, &hFunctionKey, &dwDisposition);
	if (Result != ERROR_SUCCESS) {
		sprintf(ErrorMsg, "Function Key %s under Directive %d Object %d failed :%d",
				Function, DirectiveIndex, ObjectIndex, Result);
		LogErrorEvent(service_name, EVENTLOG_ERROR_TYPE, 0, 
			MSG_BAD_REGISTRY_KEY_OPEN, ErrorMsg, NULL);
		return FALSE;
	}
	ParameterNameString = (PCHAR)MALLOC(256);
	ParameterValueString = (PCHAR)MALLOC(256);
	CurrentNameLength = CurrentValueLength = 256;

	FunctionValueIndex = 0;
	while (1) {
		// scan for all the parameters...
		NameStringLength = CurrentNameLength; ValueStringLength = CurrentValueLength;
ENUM_FUNCTION:
		ScanResult = RegEnumValue (
			hFunctionKey, FunctionValueIndex++, ParameterNameString,
			&NameStringLength, 0, NULL, ParameterValueString,
			&ValueStringLength);

		if (ScanResult == ERROR_NO_MORE_ITEMS) {
				RegCloseKey(hFunctionKey);
				break;
		} else if (ScanResult != ERROR_SUCCESS) {
			if (ScanResult != ERROR_MORE_DATA) {
				sprintf(ErrorMsg,"Function Key %s under Directive Key %d, "
					"Object Key %d failed : %d",
					Function, DirectiveIndex, ObjectIndex, ScanResult);
				LogErrorEvent(service_name, EVENTLOG_ERROR_TYPE, 0, 
					MSG_BAD_REGISTRY_KEY_ENUM, ErrorMsg, NULL);
				FREE(ParameterNameString); FREE(ParameterValueString);
				return FALSE;
			} else if ( NameStringLength > CurrentNameLength) {
				CurrentNameLength = NameStringLength;
				ParameterNameString = (PCHAR)REALLOC(ParameterNameString, 
					CurrentNameLength);
				FunctionValueIndex--;
				goto ENUM_FUNCTION;
			} else if (ValueStringLength > CurrentValueLength) {
				CurrentValueLength = ValueStringLength;
 				ParameterValueString = (PCHAR)REALLOC(ParameterValueString, 
					CurrentValueLength);
				FunctionValueIndex--;
				goto ENUM_FUNCTION;
			}
		}
		//LogErrorEvent(ParameterNameString, ParameterValueString);
		// Null terminate the values...
		ParameterNameString[NameStringLength] = '\0';
		ParameterValueString[ValueStringLength] = '\0';
        if(ntpblock_str2pblock(FunctionValueIndex, ParameterNameString, ParameterValueString,
        		NameStringLength, ValueStringLength, (*param)) < 1) {
			sprintf(ErrorMsg,"%s of Value %s of FunctionKey %s under"
					"Directive Key %d,Object Key %d failed",
					ParameterValueString, ParameterNameString, Function,
					DirectiveIndex, ObjectIndex);
			LogErrorEvent(service_name, EVENTLOG_ERROR_TYPE, 0, 
				MSG_BAD_PBLOCK, ErrorMsg, NULL);
			FREE(ParameterNameString); FREE(ParameterValueString);
			return FALSE;
		}
	}
	FREE(ParameterNameString); FREE(ParameterValueString);
	return TRUE;
}

int _ntverify_parameter(char *str) {
    register int x,np;

    for(x=0,np=0; str[x]; ++x) {
        switch(str[x]) {
          case '\\':
            if(!str[++x])
                return -1;
            break;
          case '=':
            if((!x) || isspace(str[x-1]))
                return -1;
            break;
          case '\"':
            for(++x;str[x] && (str[x] != '\"');++x) 
                if(str[x] == '\\')
                    ++x;
            if(!str[x])
                return -1;
            else if(!str[++x])
                return np;
            else if(!isspace(str[x]))
                return -1;
            break;
          default:
            if((!isspace(str[x])) && ((!x) || (isspace(str[x-1]))))
                ++np;
            break;
        }
    }
    return np;
}

int ntpblock_str2pblock(int ParameterIndex, char *name, char *value, 
	int namelength, int valuelength, pblock *pb) 
{
    pb_param *pp;
    int np, p, q;
	CHAR String[1024];
	CHAR ErrMsg[512];

	sprintf(String,"%s=%s",name,value);
    if((np = _ntverify_parameter(String)) < 1) {
		sprintf(ErrMsg, "Verify Parameter %s Value %s  failed",name, value);
		LogErrorEvent(service_name, EVENTLOG_ERROR_TYPE, 0, 
			MSG_BAD_PARAMETER, ErrMsg, NULL);
        return -1;
	}
    pp = param_create(0, 0);
    if(value) {
		pp->name = (char *)MALLOC(namelength + 1);
		strcpy(pp->name, name);
		pp->name[namelength] = '\0';
    }
    else {
        pp->name = (char *) MALLOC(10*sizeof(char));
        util_itoa(ParameterIndex, pp->name);
		pp->name[strlen(pp->name)] = '\0';
    }

    pp->value = (char *) MALLOC(valuelength + 1);
	for (p = 0,q = 0; p != valuelength; ++p) {
		if ( value[p] == '\\')
			pp->value[q++] = '/';
		else
			pp->value[q++] = value[p];
	}
	pp->value[q] = '\0';
    pblock_pinsert(pp,pb);
    return np;
}

int
ntpblock_pblock2str(struct pb_entry *p, PCHAR *ParameterName, PCHAR *ParameterValue) 
{
    register char *s ;
    register char *r ;

    int ln = strlen(p->param->name);
    int lv = strlen(p->param->value);
	
	/* Check for " or \ because we'll have to escape them */\
#ifdef NOTDEF
    for(t = p->param->value; *t; ++t)
    	if((*t == '\"') || (*t == '\\'))
                    ++lv;
#endif /* NOTDEF */

 	/* +1 for a null */
    s = (char *) MALLOC((ln + 1)*sizeof(char));
    strcpy(s,p->param->name);
	s[ln] = '\0';
			
	r = (PCHAR)MALLOC((lv + 1)*sizeof(CHAR));
	strcpy(r, p->param->value);
	r[lv] = '\0';
#ifdef NOTDEF
    for(t = r, u = p->param->value; *u; ++t, ++u) {
    	if((*u == '\\') || (*u == '\"'))
        	*t++ = '\\';
        *t = *u;
   	}
    *(t) = '\0';
#endif /* NOTDEF */
	*ParameterName = s;
	*ParameterValue = r;
	return TRUE;
}


void
dump_objset(httpd_objset *os) {
	int x, y, z, a;
	httpd_object *object;
	struct pb_entry *pb;

	ereport(LOG_INFORM,XP_GetAdminStr(DBT_osHasDObjects_), os->pos);
	for (x = 0; x < os->pos; x++) {
		object = os->obj[x];
		for(z = 0; z < object->name->hsize; z++) { 
			if (!object->name->ht[z]) {
				ereport(LOG_INFORM, XP_GetAdminStr(DBT_objDHasNoHashTableAtD_), x+1, z+1);
				continue;
			}
			if (!object->name->ht[z]->param) {
				ereport(LOG_INFORM, XP_GetAdminStr(DBT_objDHasNoParam_),x+1);
				continue;
			}
			ereport(LOG_INFORM, XP_GetAdminStr(DBT_objDNameSValueS_),x+1, 
				object->name->ht[z]->param->name,
				object->name->ht[z]->param->value);
			ereport(LOG_INFORM,XP_GetAdminStr(DBT_DirectivesD_),
				object->nd);
		}
		for(y =0; y < object->nd; y++) {
			ereport(LOG_INFORM,XP_GetAdminStr(DBT_DirectiveD_),y);
			if(object->dt[y].ni)
			for (a = 0; a < object->dt[y].ni;a++) {
				ereport(LOG_INFORM,XP_GetAdminStr(DBT_InstanceD_), a);
				for (z = 0; z < object->dt[y].inst[a].param->hsize; z++) {
					pb = object->dt[y].inst[a].param->ht[z];
					while(pb ) {
						ereport(LOG_INFORM,XP_GetAdminStr(DBT_ParamNameSValueS_),
							pb->param->name,
							pb->param->value);
						pb = pb->next;	
					}
				}
			}
		} 
	}

}

BOOLEAN ClearConfigurationParameters(HKEY hParentKey, PCHAR CompleteKeyName, PCHAR KeyName)
{
	HKEY hChildKey;
	CHAR ChildKeyName[512], CompleteChildKeyName[512] ;
	DWORD KeyIndex;

	DWORD KeyLength, dwDisposition;
	LONG Result, ScanResult;
		  
	// Open the top level registry key

	Result = RegCreateKeyEx(hParentKey, KeyName, 0, NULL, 0,
		KEY_ALL_ACCESS, NULL, &hChildKey, &dwDisposition);
	if (Result != ERROR_SUCCESS) {
		ereport(LOG_FAILURE, XP_GetAdminStr(DBT_clearconfigurationparametersRegc_), CompleteKeyName); 
		return FALSE;
	}

	KeyIndex = 0;
    while(1) {
		KeyLength = 512;   
    	ScanResult = RegEnumKey(hChildKey, KeyIndex, ChildKeyName, KeyLength);
		if (ScanResult != ERROR_SUCCESS) {
			if (ScanResult == ERROR_NO_MORE_ITEMS) {
				RegCloseKey(hChildKey);
				RegDeleteKey(hParentKey, KeyName);
				return TRUE;
			} else {
				ereport ( LOG_FAILURE ,XP_GetAdminStr(DBT_clearconfigurationparametersRege_),
							KeyName);
				return FALSE;
			}
		}
		sprintf(CompleteChildKeyName,"%s\\%s",CompleteKeyName,ChildKeyName);
		if (!ClearConfigurationParameters(hChildKey, CompleteChildKeyName,
				ChildKeyName)) {
				return FALSE;
		}
	} // while(1) loop of scan of objects
}


