/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * object.c: Handles individual HTTP objects, or documents
 * 
 * See object.h for documentation
 * 
 * Rob McCool
 * 
 */

#include "netsite.h"
#include "frame/object.h"
#include "frame/func.h" /* object_execute */

#include "frame/log.h"

#include "base/session.h"
#include "frame/req.h"
#include "base/util.h"         /* util_itoa */

#include "base/shexp.h"        /* shexp_cmp */

#ifdef NET_SSL
#include "frame/conf.h"        /* security_active */
#endif



/* ---------------------------- object_create ----------------------------- */



dtable *_dtable_create(int n) 
{
    dtable *ndt = (dtable *) MALLOC(n*sizeof(dtable));
    register int x;

    for(x=0;x<n;x++)
        ndt[x].ni = 0;

    return ndt;
}


NSAPI_PUBLIC httpd_object *object_create(int nd, pblock *name) 
{
    httpd_object *no = (httpd_object *) MALLOC(sizeof(httpd_object));

    no->name = name;
    no->nd = nd;
    no->dt = _dtable_create(nd);
    return no;
}


/* ----------------------------- object_free ------------------------------ */


NSAPI_PUBLIC void object_free(httpd_object *obj) 
{
    register int x,y;
    pblock *client;

    pblock_free(obj->name);

    for(x=0; x < obj->nd; x++) {
        if(obj->dt[x].ni) {
            for(y=0; y < obj->dt[x].ni; y++) {
                pblock_free(obj->dt[x].inst[y].param);
                if((client = obj->dt[x].inst[y].client)) {
                    int refs;
                    char *tmp;

                    tmp = pblock_findval("refcount", client);
                    refs = atoi(tmp) - 1;
                    if(!refs)
                        pblock_free(obj->dt[x].inst[y].client);
                    else
                        util_itoa(refs, tmp);
                }
            }
            FREE(obj->dt[x].inst);
        }
    }
    FREE(obj->dt);
    FREE(obj);
}


/* ------------------------- object_add_directive ------------------------- */


NSAPI_PUBLIC 
void object_add_directive(int dc, pblock *p, pblock *c, httpd_object *obj)
{
    dtable *dt;

    dt = &obj->dt[dc];
    dt->inst = (directive *) 
                    (!dt->ni ? MALLOC(sizeof(directive))
                             : REALLOC(dt->inst,(dt->ni+1)*sizeof(directive)));
    dt->inst[dt->ni].param = p;
    dt->inst[dt->ni++].client = c;
}



/* ---------------------------- object_execute ---------------------------- */


/*
 * MOVED TO FUNC.C TO AVOID INTERDEPENDENCIES IN ADMIN PROGRAMS --Rob
 */
