/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
** daemon.cpp: mainline daemon 
**/

extern "C" {
/* New ns security does not export this function */
// << NOT in NSS3 >> extern void SEC_Init(void);
#include "ssl.h"
#include "pk11func.h"
#include "secmod.h"
}

#include "nspr.h"
#include "prtypes.h"
#include "prthread.h"
#include "prlock.h"
#include "prcvar.h"
/* nspr21 integration, gone from nspr21 19971028
#include "private/primpl.h"
*/
#include "daemon.h"
#include "daemonprocessor.h"
#include "httpdaemon/dstats.h"
#include "base/ereport.h"
#include "base/util.h"
#include "base/file.h"
#include "base/eventlog.h"
#include "base/daemon.h"
#include "base/pool.h"
#include "frame/conf.h"
#include "frame/conf_api.h"
#include "frame/log.h"
#include "safs/ntlogbuf.h"
#include "nt/messages.h"
#include "nt/regparms.h"
/* New ns security does not have sec.h  
#include "sec.h"
*/

static void (*chcb)(Session *);

DaemonProcessor *globalDaemonProcessor;

#include "daemonsession.h"

PRBool
GetKeepAliveInfo(keepAliveInfo *info)
{
    KeepAliveList *KAList = DaemonSession::GetKeepAliveList();

    if (KAList)
        return KAList->GetKeepAliveInfo((keepAliveInfo*)info);

    return PR_FALSE;
}
extern "C" void daemon_run(void  *_pDaemon)
{
    daemon_s *pDaemon = (daemon_s *)_pDaemon;
    PRUintn key;
    PRInt32 result;
    char *dstats_disabled;
    char *rq_throttle_str;
    int rq_throttle = 256;
    char *str_dstats_max_threads;
    int dstats_max_threads = 0;
   

    chcb = pDaemon->child_callback;
    
    rq_throttle_str = conf_findGlobal("RqThrottle");
    if (rq_throttle_str)
        rq_throttle = atoi(rq_throttle_str);
    
    str_dstats_max_threads = conf_findGlobal("DaemonStatsMaxThreads");
    if (str_dstats_max_threads)
        dstats_max_threads = atoi(str_dstats_max_threads);
    
    dstats_disabled = conf_findGlobal("DaemonStats");
    if (!dstats_disabled || !strieq(dstats_disabled, "off")) {
        if (dstats_max_threads) 
            pDaemon->maxthreads = dstats_max_threads;
        else
            pDaemon->maxthreads = 2*rq_throttle;   /* XXXMB - just a guess*/
        result = dstat_init(pDaemon->port, pDaemon->ipstr, pDaemon->maxthreads,
                            pDaemon->pw);
        NS_ASSERT(result == 0);
    }

    dstat_get_thread_slot();

    net_init(pDaemon->security);

    // << NOT in NSS3 >> SEC_Init();

    PR_NewThreadPrivateIndex(&key, NULL);

    setThreadMallocKey(key);
    
    system_errmsg_init();

    {
        PRBool fSuccess;

        fSuccess = DaemonProcessor::Initialize(pDaemon);

        if (fSuccess) {
            globalDaemonProcessor = DaemonProcessor::NewDaemonProcessor();

            globalDaemonProcessor->Run();
        } else {

            /* Failed to come up. Terminate the server now. */
            exit(-1);
        }
    }

#ifndef IRIX
//
// Since the monitoring/restart main process shares a lock in the ds struct
// it's not a good idea for the children to delete locks.  These are created
// by the main and should remain in existence until the main process
// dies.
//
// This is IRIX specific because on IRIX, deleting cvars or locks removes them
// from the arena which is still shared with the parent process.  So we have
// to leave them around so that they still exist when we get respawned.
//
    dstat_destroy();
#endif

    NotifyExitVariable();

}

NSAPI_PUBLIC void daemon_terminate ()
{
    globalDaemonProcessor->Terminate();

    // Wait for official signal that server is dead 
    WaitForExitVariable();

    // For historical reasons, we allow NSAPI plugins to have restart
    // callback routines, but not shutdown callbacks.  Since the restart 
    // handling was changed to do a stop/start, we'll be nice guys and call
    // the restart handlers here, since they primarily just shut things down
    // nicely.
    daemon_dorestart();
}

extern "C" void HandleSynchronousRequestSession (
    char *pzAddressServer,
    struct sockaddr_in *pServerAddr,
    struct sockaddr *saClient, 
    PRFileDesc *hSocket, 
    void *  pData, 
    size_t  sData,
    PRBool *  fKeepAliveRequested
) {

    // Set socket non inheritable
    file_setinherit(hSocket, 0);

    *fKeepAliveRequested =  PR_FALSE; // tentatively

    // fill out session data
    Session *pSn = session_create((SYS_NETFD) hSocket, (struct sockaddr_in *) saClient);

    if (pSn) {
        // initialize netbuf for this connection 
        // XXXMB - this is messy; should just pass a sockaddr_in to 
        //         session_create for the local address as well as client addr
        memset(pSn->inbuf->address, 0, 64);   // XXXMB- why 64?
        if (pzAddressServer) 
            strcpy(pSn->inbuf->address, pzAddressServer);
        pSn->local_addr.sin_family = AF_INET;
        pSn->local_addr.sin_port = pServerAddr->sin_port;
        pSn->local_addr.sin_addr.s_addr = pServerAddr->sin_addr.s_addr;

        // place read-data into inbuf for reinterpretation in backend
        if (pData && sData) {
            if (pSn->inbuf->pos >= pSn->inbuf->cursize) {
                if ((PRUint32) pSn->inbuf->maxsize < sData) {
                    NS_ASSERT (0); // mis-coordinated buffer sizes
                    sData = pSn->inbuf->maxsize; // truncate and let server field error (XXXJB)
                }
                if(!pSn->inbuf->inbuf)
                    pSn->inbuf->inbuf = (unsigned char *) MALLOC(pSn->inbuf->maxsize); // XXXJB
                memcpy (pSn->inbuf->inbuf, pData, sData); 
                pSn->inbuf->pos = 0;
                pSn->inbuf->cursize = sData;
            }
            else
                NS_ASSERT (0);  // data already in buffer!?
        }

        // forward request to server mainline
        (chcb) (pSn);

        if (pSn->csd_open)
            *fKeepAliveRequested = PR_TRUE;

        // free resources, etc.
        session_free (pSn);
    }
}


/* ----  this is a bunch of junk from ntdaemon; clean it up XXXMB ---- */

#ifdef __cplusplus
extern "C" {
#endif

static PRBool     daemon_dead = PR_FALSE;
static PRLock    *daemon_exit_cvar_lock;
static PRCondVar *daemon_exit_cvar;

NSAPI_PUBLIC void
InitializeExitVariable()
{
    daemon_exit_cvar_lock = PR_NewLock();
    daemon_exit_cvar = PR_NewCondVar(daemon_exit_cvar_lock);
}

#define DEFAULT_TERMINATE_TIMEOUT	3
NSAPI_PUBLIC void
WaitForExitVariable()
{
    char *waitTime_s = conf_findGlobal("TerminateTimeout");
    PRInt32 waitTime = DEFAULT_TERMINATE_TIMEOUT;

    if (waitTime_s) 
        waitTime = atoi(waitTime_s);
    waitTime = PR_SecondsToInterval(waitTime);

    PR_Lock(daemon_exit_cvar_lock);

    if (daemon_dead == PR_FALSE) 
        PR_WaitCondVar(daemon_exit_cvar, waitTime);

    if ( daemon_dead == PR_FALSE )
        log_ereport(LOG_VERBOSE, "Daemon not dead after %d seconds", 
                    PR_IntervalToSeconds(waitTime));

    PR_Unlock(daemon_exit_cvar_lock);
}

NSAPI_PUBLIC void
NotifyExitVariable()
{
    PR_Lock(daemon_exit_cvar_lock);
    daemon_dead = PR_TRUE;
    PR_NotifyCondVar(daemon_exit_cvar);
    PR_Unlock(daemon_exit_cvar_lock);
}

#include "httpdaemon/internalstats.h"

PRBool
GetListenSocketInfo(PRUint32 *numSockets, listenSocketInfo **infoArray)
{
    if (globalDaemonProcessor)
       return globalDaemonProcessor->GetListenSocketInfo(numSockets, infoArray);

    return PR_FALSE;
}


#ifdef __cplusplus
}
#endif
