/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#ifdef HPUX
#include <pthread.h>
#include <sys/signal.h>
#endif
#include "nspr.h"
#include "prtypes.h"
#include "prthread.h"
#include "prlock.h"
#include "prcvar.h"


/* removed for nspr21 for all the platform since it is gone (19971028 nspr
 binary release
#include "private/primpl.h"
*/

#include "daemonprocessorux.h"
#include "daemonsessionux.h"
#include "httpdaemon/multihome.h"
#include "httpdaemon/dbthttpdaemon.h"
#include "frame/conf.h"
#include "frame/log.h"
#include "base/util.h"

#include "base/fasttime.h"
#include "safs/ntlogbuf.h"

#ifdef	AIX
#include <time.h>
#endif
#include <unistd.h>
#include <sys/time.h>
#include <sys/resource.h>

extern DaemonProcessor *globalDaemonProcessor;

/* --- Globals for signal hander routines --------------------------- */
PRBool                  _global_main_terminating;
struct sigaction        _global_sa;
void                    (*_global_rcb)(int);
char                   *_global_pidfn;


/* --- DaemonProcessorUX globals ------------------------------------ */
PRBool  DaemonProcessorUX::detach = PR_TRUE;
PRInt32 DaemonProcessorUX::num_processors;


/* --- Forwards for internal UX routines ---------------------------- */
void _daemon_signal_exit(int);

#ifdef DAEMON_NEEDS_SEMAPHORE
extern PRInt32 single_accept;
#endif

/* ---------------- SIGNAL HANDLING STUFF ------------------------- */
#ifdef HPUX

typedef struct {
    int signo;
    void (*sigfunc)(int);
} SigArg_t;

extern "C" pid_t HP_SysFork(void);
void signal_Phandler_thread(void*);

#define RSIGNALP { \
	PR_CreateThread(PR_USER_THREAD, \
                      signal_Phandler_thread, (void*)NULL, \
                      PR_PRIORITY_NORMAL, \
                      PR_LOCAL_THREAD, \
                      PR_UNJOINABLE_THREAD, \
                      0); \
	}
#endif /* HPUX */

#define RSIGNAL(s, fn) { _global_sa.sa_handler = SA_HANDLER_T(fn); sigaction(s, &_global_sa, NULL); }

#ifdef HPUX
#define SIGCONTEXT , int code, struct sigcontext *scp
#else
#define SIGCONTEXT
#endif

static void server_hard_exit(int code)
{
    exit(code);
}

PRBool DaemonProcessorUX::Initialize(daemon_s *pDaemon)
{

    detach = pDaemon->detach?PR_TRUE:PR_FALSE;
    _global_rcb = pDaemon->rcback;

    /* --- Determine setting for single_accept ---*/
#ifdef DAEMON_NEEDS_SEMAPHORE
#if defined(SOLARIS)
    single_accept = DaemonProcessorUX::_daemon_solaris_single_accept();
#endif
    if (pDaemon->single_accept != SINGLE_ACCEPT_NONE)
        single_accept = pDaemon->single_accept;

    switch(single_accept) {
        case SINGLE_ACCEPT_ON:
            log_ereport(LOG_VERBOSE, XP_GetAdminStr(DBT_usingSingleThreadedAccepts_));
            break;
        case SINGLE_ACCEPT_OFF:
            log_ereport(LOG_VERBOSE, XP_GetAdminStr(DBT_usingMultiThreadedAccepts_));
            break;
        case SINGLE_ACCEPT_PARTIAL:
            log_ereport(LOG_VERBOSE, XP_GetAdminStr(DBT_usingPartialSingleThreadedAccept_));

    }
#endif

    _daemon_setup(pDaemon);

    /* --- Print out version ID to console --- */
    AdminFprintf(stderr, "%s B%s\n", MAGNUS_VERSION_STRING, BUILD_NUM);
    if (strcmp(BUILD_NUM, pDaemon->version)) {
        AdminFprintf(stderr, XP_GetAdminStr(DBT_warningNetscapeExecutableAndLibr_));
    }

    return PR_TRUE;
}

PRBool DaemonProcessorUX::LateInitialize(daemon_s *pDaemon)
{
    char *wdstr = getenv("WD_PIPE_FD");

    if (!wdstr) {
	DaemonProcessorUX::_daemon_set_uid(geteuid(), pDaemon->pw);
    }
    else {
        /* Signal initialization complete to watchdog process */
        kill(getppid(), SIGUSR1);
    }
    return PR_TRUE;
}

DaemonProcessorUX::DaemonProcessorUX() : DaemonProcessor()
{
    char *wdstr = getenv("WD_PIPE_FD");

    if (!wdstr && (detach == PR_TRUE))
        _daemon_detach();

    _global_pidfn = PERM_STRDUP(conf_getglobals()->Vpidfn);
    if (!wdstr) {
        _daemon_log_pid(_global_pidfn);
    }

    _global_main_terminating = PR_FALSE;

#ifndef HPUX
    _global_sa.sa_flags = SA_RESTART;
#endif

#ifdef HPUX
	sigemptyset(&_global_sa.sa_mask);
#endif

#ifdef HPUX
    signal(SIGPIPE, SIG_IGN);
    signal(SIGHUP, SIG_IGN);
    signal(SIGCHLD, SIG_IGN);
    RSIGNALP
#else
    RSIGNAL(SIGHUP, SIG_IGN);
    RSIGNAL(SIGTERM,  _daemon_signal_exit);
    RSIGNAL(SIGPIPE, SIG_IGN);
    /* Don't wait() unless the user insists - mikep */
    if (conf_getglobals()->wait_for_cgi)
            RSIGNAL(SIGCHLD, SIG_DFL)
    else {
            /* This prevents zombies without waits */
#ifdef LINUX
      _global_sa.sa_flags |= SA_NOCLDSTOP;
#else
      _global_sa.sa_flags |= SA_NOCLDWAIT | SA_NOCLDSTOP;
#endif
      RSIGNAL(SIGCHLD, SIG_IGN)
    }
#endif

#ifdef PUMPKIN_HOUR
#ifndef MCC_ADMSERV
    if(time(NULL) > (PUMPKIN_HOUR + 6868)) {
        ereport(LOG_FAILURE, XP_GetAdminStr(DBT_thisBetaSoftwareHasExpiredN_));
        server_hard_exit(-1);
    }
#endif
#endif

#ifdef MCC_DEBUG
    /*
     * In many debuggers, there's no easy way to get to the child
     * process before it dies.  This allows you to hang the child
     * process immediately after creation and attach via a debugger
     * - mikep
     */
    if (getenv("HANG_HTTPD_FOR_DEBUG") != NULL) 
	while(1);
#endif

    /* Initialize the clock thread */
    if (ft_init() == PR_FALSE)
	ereport(LOG_FAILURE, "Clock Thread Init failed.\n");

	/* no longer using libares3 PR_AR_ -ahakim 10/16/2001
    //if (net_enableAsyncDNS && net_enabledns && PR_AR_Init() == NULL) {
    //    ereport(LOG_FAILURE, "failed to enable asynchronous DNS");
    } */

    /* Initialize the logbuf thread */
    if (logbufthread_init() == PR_FALSE)
        ereport(LOG_CATASTROPHE, "Logbuf Thread Init failed.\n");
	         
    /* Disable passing the listen sockets across future 
     * fork()s. 
     */
    SetListenSocketsNonInheritable();
			 
    /* return out of this routine to continue processing */
    return;

}

DaemonProcessorUX::~DaemonProcessorUX()
{
}

DaemonSession *DaemonProcessorUX::NewAcceptSession(PRInt32 _index)
{
    DaemonSessionUX *pNewAcceptSession;
    PRBool fSuccess;

    pNewAcceptSession = new DaemonSessionUX(_index, this);
 
    fSuccess = pNewAcceptSession->StartThread();
    NS_ASSERT(fSuccess);

    return pNewAcceptSession;
}


/* FUNCTION: _daemon_signal_exit
** DESCRIPTION:
**    Signal handler for killing the server
*/
void _daemon_signal_exit(int sig)
{
	_global_main_terminating = PR_TRUE;
}

/* FUNCTION: _daemon_exit
** DESCRIPTION:
**    Handler for exiting the server
*/
void _daemon_exit()
{
    char *wdstr = getenv("WD_PIPE_FD");

    ereport(LOG_INFORM, "Received SHUTDOWN command!\n");

#ifdef MCC_ADMSERV
    /* Kill cron */
    (*_global_rcb)(SIGTERM);
#endif

    globalDaemonProcessor->Terminate();

    // Flush any outstanding log buffers
    logbuf_cleanup_all();

    // For historical reasons, we allow NSAPI plugins to have restart
    // callback routines, but not shutdown callbacks.  Since the restart 
    // handling was changed to do a stop/start, we'll be nice guys and call
    // the restart handlers here, since they primarily just shut things down
    // nicely.
    daemon_dorestart();


    // Cleanup the accelerator cache; otherwise tmp dirs may be left around
    accel_file_cache_destroy(NULL);

    dstat_destroy();

    // Remove pidfile 
    if (!wdstr && _global_pidfn)
        unlink(_global_pidfn);

    exit(0);
}

/* FUNCTION: _daemon_detach
** DESCRIPTION:
**     Detaches from user-started process, which exits. 
*/
void DaemonProcessorUX::_daemon_detach() 
{
#ifdef USE_DDE_DEBUG
  switch(0) {
#elif defined(HPUX)
  switch(HP_SysFork()) { 
#else
  switch(fork()) {
#endif
  case -1:
    AdminFprintf(stderr,XP_GetAdminStr(DBT_daemonUnableToForkNewProcessSN_),
		 system_errmsg());
    server_hard_exit(1);
  case 0:
    if(setsid() == -1) {
      AdminFprintf(stderr,XP_GetAdminStr(DBT_daemonSetsidFailedSN_), 
		   system_errmsg());
      server_hard_exit(1);
    }
    /* Make sure the daemon does not attach to terminal */
    fclose(stdin);
    fclose(stdout);
    fclose(stderr);

    break;
  default:
    exit(0);
  }
}

/* FUNCTION: _daemon_log_pid
** DESCRIPTION:
**     Write our PID out to a log file 
*/
void
DaemonProcessorUX::_daemon_log_pid(char *fn) 
{
  char buffer[64];
  int length;

  if (fn) {
    SYS_FILE f = system_fopenRW(fn);

    if (f == SYS_ERROR_FD) {
      AdminFprintf(stderr, 
		   XP_GetAdminStr(DBT_daemonCanTLogPidToSSN_), 
		   fn, 
		   system_errmsg());
      server_hard_exit(1);
    }
    length = util_sprintf(buffer, "%d\n", (int)getpid());
    system_fwrite(f, buffer, length);
    system_fclose(f);
  }
}


/* FUNCTION: _daemon_set_uid
** DESCRIPTION:
**     Set our uid
*/
void DaemonProcessorUX::_daemon_set_uid(uid_t uid, struct passwd *pwd)
{
    if(!uid) {
        if(setgid((gid_t)pwd->pw_gid) == -1)
            AdminFprintf(stderr, XP_GetAdminStr(DBT_warningCouldNotSetGroupIdToDSN_),
                    (int)pwd->pw_gid, system_errmsg());
#ifndef AIX	/* QQQQ find out what this should be */
        else
            initgroups(pwd->pw_name, pwd->pw_gid);
#endif

        if(pwd->pw_uid) {
            if(setuid((uid_t)pwd->pw_uid) == -1)
                AdminFprintf(stderr,XP_GetAdminStr(DBT_warningCouldNotSetUserIdToDSN_),
                        (int)pwd->pw_uid, system_errmsg());
        }
        else
            AdminFputs(XP_GetAdminStr(DBT_warningDaemonIsRunningAsSuperUse_), stderr);
    }
}

PRBool DaemonProcessorUX::_daemon_setup(daemon_s *d)
{
    uid_t uid = geteuid();

    if(uid || (!d->pw)) {
        setpwent();
        if(!(d->pw = getpwuid(uid))) {
            AdminFprintf(stderr, XP_GetAdminStr(DBT_couldNotDetermineCurrentUserName_));
            server_hard_exit(1);
        }
    }
    if(d->chr && (chroot(d->chr) == -1)) {
        AdminFprintf(stderr, XP_GetAdminStr(DBT_errorChrootToSFailedSN_), d->chr, 
            system_errmsg());
        server_hard_exit(1);
    }

    return PR_TRUE;
}

/* ------------------ keepalive related routines -------------------- */

/* ------------------ solaris specific routines --------------------- */

#ifdef SOLARIS
#include <unistd.h>
#include <sys/sockio.h>
/* FUNCTION: _daemon_solaris_single_accept()
** DESCRIPTION:
**     Detect if this version of solaris can only handle one
**     thread or process entering accpet() simultaneously.
** RETURNS:
**     PR_TRUE if this version can only handle 1
**     PR_FALSE if this version can handle multithreaded accepts.
** RESTRICTIONS:
**     If there are problems determining what this version of solaris
**     supports; it returns PR_TRUE.  The only known reason for such 
**     failures are EMFILE errors to the socket call.
**
**     Implemented based on the assumption that only kernel implementations
**     of sockets can handle process groups as socket signal destinations.
**     This presumably will return 0 on all SVr4 based systems that have
**     the libsocket/sockmod socket implementation.
**/
PRBool
DaemonProcessorUX::_daemon_solaris_single_accept()
{
        int s;
        int pgrp, npgrp;

        s = socket(AF_INET, SOCK_STREAM, 0);
        if (s == -1) 
                return (PR_TRUE);
        pgrp = -getpgrp();

        if (ioctl(s, SIOCSPGRP, &pgrp) == -1) {
                close(s);
                return (PR_TRUE);
        }
        if (ioctl(s, SIOCGPGRP, &npgrp) == -1) {
                close(s);
                return (PR_TRUE);
        }
        close(s);
        if (npgrp == pgrp)
                return (PR_FALSE);
        else
                return (PR_TRUE);
}
#endif

/*---------------------------HPUX hack for fork()------------------------*/
#ifdef HPUX
extern "C" syscall(int);
/*
 * Needed because the normal fork() does not copy all threads created from
 * parent.
 */
extern "C"
pid_t	HP_SysFork()
{
	pid_t parentPid = getpid();
	pid_t retPid = syscall(SYS_FORK);	

    if (retPid == -1)
        return (-1);

    return ( (parentPid != retPid) ? retPid : 0);
}
/*--------------------------HPUX signal handling routine --------------------*/

/* Signal Catchers */
static int  P_Catcher[]= { SIGTERM, 0};
		
void signal_Phandler_thread ( void *targ)
{
   int *p_sig;
   int 		sig, signum;
   int			status;

   sigemptyset(&_global_sa.sa_mask);

   p_sig = P_Catcher;
   while (sig = *p_sig++) {
        if ( sigaddset(&_global_sa.sa_mask, sig) == -1){
	    ereport(LOG_FAILURE, 
	    "signal_handler_thread: Failed to add signal %d. (%s)",
                           sig, system_errmsg());
		    exit(1);
       }
   }

   if ( sigprocmask(SIG_BLOCK,&_global_sa.sa_mask, NULL) == -1) {
      ereport(LOG_FAILURE, 
	"signal_handler_thread: Failed to block signals. (%s)",
                           system_errmsg());
      exit(1);
   }

   while(1) {	/* Simply wait for the signal */
#if defined (HPUX)
      sig    = sigwait(&_global_sa.sa_mask, &signum);
#else
      signum = sigwait(&_global_sa.sa_mask);
      sig    = signum;
#endif

      switch(signum) {
	  case SIGTERM: 
             _daemon_signal_exit(signum);
	     break;
      }

#if defined (HPUX)
      if (sig != 0)
#else
      if (sig == -1)
#endif
      {
 	 ereport(LOG_FAILURE, 
		"signal_Phandler_thread: Failed to wait on signals. (%s)", system_errmsg());
	 exit(1);
			
      }

   } /* end of while */
}

#endif
