/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
** DaemonRequest.h
**
*/

#ifndef _DAEMONREQUEST_H_
#define _DAEMONREQUEST_H_

#include "libdaemon.h"
#include "prcvar.h"

class DaemonRequest;
class DaemonProcessor;
class DaemonSession;

class DaemonRequest  {

 public:
                        DaemonRequest(DaemonProcessor*, DaemonSession* session);
                        ~DaemonRequest();

    static PRBool       Initialize();
    void                Timeout();
    /* Return TRUE if the socket is open on return from Respond or FALSE
     * if the socket has already been closed.
     */
    virtual PRBool      Respond (void *pHeader, PRUint32 sHeader) = 0;

    // Initialize the request for reuse
    virtual void        InitializeRequest() = 0;

    PRFileDesc *           GetSocket();

 protected:
    PRBool                FFree ();

 protected:

    // each DaemonRequest is dedicated to a processor
    DaemonProcessor        *pProcessor;

    // active client/server connection for this request
    PRFileDesc *               socket;
    DaemonSession          *pSession;

    // next request on a free list
    DaemonRequest          *pNext;

 private:

    static PRBool           fInitialized;	

    // some magic # tags to verify requests on free list, etc
    PRInt32                 iMagic;
    static PRInt32          iMagicAlloc;
    static PRInt32          iMagicFree;

};

inline PRBool DaemonRequest::FFree() 
{
    return (iMagic == iMagicFree)?PR_TRUE:PR_FALSE;
}

inline PRFileDesc * DaemonRequest::GetSocket ()
{
    return socket;
}

#endif // _DaemonRequest_h_
