/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
#include "daemonsession.h"
#include "daemonsessionnt.h"

const PRUint32 sInetAddrPadded = sizeof(struct sockaddr_in) + 16;

extern PRIntervalTime acceptTimeoutInterval;

void 
DaemonSessionNT::Initialize()
{
}

DaemonSessionNT::DaemonSessionNT(
                   PRInt32 _index, 
                   DaemonProcessor *_pDaemon) :
    DaemonSession(_index, _pDaemon)
{
}

DaemonSessionNT::~DaemonSessionNT()
{
}


PRBool DaemonSessionNT::HandleCachedResponse(
      PRFileDesc *sock, accel_file_cache_entry_t *pCacheEntry,
      PRBool fKeepAliveRequested, PRInt32 *bytesSent)
{
    accel_cache_entry_t *pAccelCacheEntry = pCacheEntry->accelcache;
    file_cache_entry_t *pFileCacheEntry = pCacheEntry->filecache;
    char *   pResponseHeader; 
    PRUint32 sResponseHeader;
    PRFileDesc *hFile;
    PRFileDesc *diskFile = NULL;
    PRTransmitFileFlags mTfOptions;
    PRInt32 rv;

    pResponseHeader = fKeepAliveRequested?
       pAccelCacheEntry->headers_keepalive:pAccelCacheEntry->headers;
    sResponseHeader = fKeepAliveRequested?
       pAccelCacheEntry->header_keepalive_len:pAccelCacheEntry->header_len;

    if (!fKeepAliveRequested)
      /* mTfOptions = PR_TRANSMITFILE_CLOSE_SOCKET|PR_TRANSMITFILE_REUSE_SOCKET; */
      mTfOptions = PR_TRANSMITFILE_CLOSE_SOCKET;

    if (!pFileCacheEntry) {
        /* The cache doesn't always have the file open.  If not, open it now.
         * Originally we didn't want the cache to ever have to do an open,
         * since it could potentially block.  But for specweb, we need to cache
         * a huge fileset- and it is realistic to assume that we can't keep
         * all files open.  Granted, this is more of a problem for UX than NT,
         * but the cache now has unopened, cached files, and the accelerator
         * needs to support it.  In tests so far, the thread-concurrency from
         * CreateIoCompletionPort seems to save us from being hurt by blocking
         * open()s.
         */
        diskFile = PR_Open(
            pAccelCacheEntry->path,
            PR_RDONLY,
            0);
        if (diskFile == NULL) {
            NS_ASSERT(0);
            PR_Close(sock);
            return PR_FALSE;
        }
        hFile = diskFile;
    } else 
        hFile = pFileCacheEntry->fd;

    rv = PR_TransmitFile (
                sock,
                hFile,
                pResponseHeader,
                sResponseHeader,
                mTfOptions,
                PR_INTERVAL_NO_TIMEOUT);

    if (diskFile)
        PR_Close(diskFile);

    /* If keepalive was not requested, we passed the close flag to 
     * transmitfile, so the socket is already closed.  Otherwise, keepalive
     * was requested, so we aren't going to close the socket.  Either way-
     * don't manually close the socket, unless there is an error.
     */
    if (rv < 0) {
        *bytesSent = -1;
        PR_Close(sock);
        return PR_FALSE;
    }

    *bytesSent = rv;
    return fKeepAliveRequested;
}
