/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
//
// HttpHeader.h
//

#ifndef _HTTPHEADER_H
#define _HTTPHEADER_H

const size_t sHttpHeaderMax = 512;

class HttpHeader {

 public:
    PRBool ScanHeader (void *pHeader, size_t sHeader);

    char               *GetHeader ()          const;
    int                 GetHeaderLen ()       const;

    char               *GetRequest()          const;
    const char         *GetUri ()             const;
    const char         *GetProtocol ()        const;
    const char         *GetReferer ()         const;
    const char         *GetUserAgent ()       const;
    const char         *GetHost ()            const;
    const char         *GetIfModifiedSince () const;
    const char	       *GetAuthenticate ()    const;
	
    PRBool              FKeepAlive ()	      const;
    PRBool              FRangeRequested ()    const;
    PRBool              FPipeLined ()         const;
    int                 GetVersionNumber ()   const;

 private:
    PRBool              ScanHeadline ();
    PRBool              ScanAttributes ();
    PRBool              SetAttribute (const char *zTag, const char *zValue);

 private:   
    char                vzHeader [sHttpHeaderMax+1];
    char               *pzHeader;
    int                 sHeader;
    char               *pzRequest;     // Copy of request line
    PRUint32            iRequestLen;   // Length of request line
    const char         *pzUri;         // Ptr into vzHeader for the URI
    const char         *pzProtocol;    // Ptr into vzHeader for the protocol
    const char         *pzReferer;     // Ptr into vzHeader for referer line
    const char         *pzUserAgent;   // Ptr into vzHeader for user-agent line
    const char         *pzAuthenticate;// Ptr into vzHeader for www-auth
    const char         *pzIfModifiedSince;// Ptr into vzHeader for if-modified-since
    const char         *pzHost;        // Ptr into vzHeader for host name
    PRBool		        fKeepAliveRequested;
    PRBool              fRangeRequested;
    PRBool              fPipeLined;    // Request is packet is pipelined
    int                 sVersionNumber;// ProtocolVersion Number
};

inline char *HttpHeader::GetHeader() const 
{ 
    return pzHeader;
}

inline int HttpHeader::GetHeaderLen() const 
{ 
    return sHeader;
}

inline char *HttpHeader::GetRequest() const 
{ 
    // Note: GetRequest is currently only needed for logging.
    // pzRequest points to the buffer which we original read data into.
    // So if GetRequest is called and we then try to pass the data to the
    // back-end, the original buffer will have been modified.

    if (iRequestLen) 
        pzRequest[iRequestLen] = '\0';
    return pzRequest; 
}
inline const char *HttpHeader::GetUri () const 
{
    return pzUri;
}
inline const char *HttpHeader::GetProtocol () const
{
    return pzProtocol;
}
inline const char *HttpHeader::GetReferer () const
{
    return pzReferer;
}
inline const char *HttpHeader::GetUserAgent () const
{
    return pzUserAgent;
}
inline const char *HttpHeader::GetHost () const
{
    return pzHost;
}
inline const char *HttpHeader::GetIfModifiedSince () const
{
    return pzIfModifiedSince;
}
inline const char *HttpHeader::GetAuthenticate () const
{
    return pzAuthenticate;
}
inline PRBool HttpHeader::FPipeLined () const
{
    return fPipeLined;
}
inline PRBool HttpHeader::FKeepAlive () const 
{
    if ( fPipeLined )
        return PR_FALSE;  // Currently do not honor pipelined requests
    else
        return fKeepAliveRequested;
}
inline PRBool HttpHeader::FRangeRequested () const
{
    return fRangeRequested;
}
inline int HttpHeader::GetVersionNumber () const
{
    return sVersionNumber;
}

#endif /* _HTTPHEADER_H */
