/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
** HttpRequest.h
**
**
** This module is to DaemonRequest as other frame components are to their 
** base counterparts.  We contain a number of functions specialized to a 
** particular service protocol (probably useful for proxy or httpd).
** 
** There still would be a lot of duplication between this module and a peer 
** that provided different state machines for another server.  A couple of 
** things might be useful; make this class based on a template framework; 
** also, DaemonProcessor should take a DaemonRequest factory function as an 
** argument (ie, to point to HttpRequest as the factory function, or some 
** alternate high-level factory - DaemonProcessor itself is pretty generic).
*/

#ifndef _HTTPREQUEST_H_
#define _HTTPREQUEST_H_

#include "daemonrequest.h"
#include "httpheader.h"

class DaemonSession;

struct accel_file_cache_entry_t;

class HttpRequest : public DaemonRequest {

 public:
                     HttpRequest (DaemonProcessor *, DaemonSession *);
                     ~HttpRequest ();

    static PRBool    Initialize ();
    virtual void     InitializeRequest();
    virtual PRBool   Respond (void *pHeader, PRUint32 sHeader);
    PRBool           UnacceleratedRespond ();

    void             Cleanup(const char *user, const char *authorization);

 private:
    PRBool           RespondNotModified ();
    void             Log (const char *user, const char *authorization);

 private:
    DaemonSession              *pSession;
    HttpHeader                  rqHdr;

    accel_file_cache_entry_t   *pCacheEntry;
    PRBool                      fAcceleratedResponse;
    PRInt32                     bytesSent;
    PRBool                      fCleanup;

    const char                 *pzStatus;  // "200 OK" or "304 Use local"
    PRUint32                    iStatus;   // 200, 304, etc
    static PRBool		fTransmitFile;
};

#endif // _HttpRequest_h_
