/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*	lasprogram.c
 *	Copyright (c) 1996, 1997 Netscape Communications Corporation
 *	This file contains the admin server's program LAS code.
 *
 * 	All blame to Mike McCool (mlm@netscape.com), based on code from ACL guys
 */
#include <stdio.h>
#include <string.h>
#ifdef AIX
#include <strings.h>
#endif

#include <netsite.h>
#include <base/util.h>
#include <base/session.h>
#include <base/shexp.h>
#include <base/plist.h>
#include <libaccess/nserror.h>
#include <libaccess/acl.h>
#include <libaccess/las.h>
#include <libaccess/dbtlibaccess.h>
#include <libaccess/aclerror.h>
#include "aclutil.h"

#ifdef MAX
#undef MAX
#endif
#define MAX( x, y ) ((x)>(y)?(x):(y))


static void
matchName( int *match, char *program, char *name, program_groups *groups,
	   char **allowBase, char **allowed, int *allowedSize );

static void
matchList( int *match, char *program, char *list, program_groups *groups,
	   char **allowBase, char **allowed, int *allowedSize )
{
  char	*working = STRDUP( list );
  char	*src = working;
  char	*name = working;

  while ( *src ) {
    if ( *src == ',' ) {
      *src++ = '\0';
      matchName( match, program, name, groups,
		 allowBase, allowed, allowedSize );
      /* skip whitespace */
      while ( *src == ' ' || *src == '\t' ) {
	++src;
      }
      name = src;	/* next name */
    } else {
      ++src;
    }
  }
  /* finish up */
  matchName( match, program, name, groups,
	     allowBase, allowed, allowedSize );
  FREE( working );
}

/***********************************************************************
** FUNCTION:	matchName
** DESCRIPTION:
**   see if the current program matches the current name in ACL, and
**   expand any program groups into the list of programs this user can
**   execute.
** INPUTS:	match -		has it matched already?
**		program -	the program the dude wants to execute
**		name -		the current part of the attr_pattern
**		groups -	the program groups structure
** OUTPUTS:	match -		did it match now?  Matched if already matched
**		allowBase -	the whole allow string
**		allowed -	a shell expression of allowed programs
**		allowedSize -	How big is the string?
** RETURN:	None
** SIDE EFFECTS:
**	allowed string gets bigger, maybe
** RESTRICTIONS:
**	None
** MEMORY:	May realloc allowBase
***********************************************************************/
/* weirdly enough, we are doing match the strcmp style, and 0 means matched */

static void
matchName( int *match, char *program, char *name, program_groups *groups,
	   char **allowBase, char **allowed, int *allowedSize )
{
  int	t;

  if ( groups ) {	/* see if it's a program group */
    int	i;

    /* Look for a program group that matches the name */
    for ( i = 0 ; groups->groups[i] && strcmp( name, groups->groups[i] ) != 0 ;
	  ++i )
      ;
    if ( groups->groups[i] ) {	/* found a group to expand */
      matchList( match, program, groups->programs[i], groups,
		 allowBase, allowed, allowedSize );
    } else {	/* not a group */
      if ( *match != 0 ) {	/* not matched, yet */
	*match = WILDPAT_CMP( program, name );
      }
      if ( ! *allowBase ) {	/* Don't bother expanding the allow list */
	return;
      }
      if ( ( t = *allowed - *allowBase + strlen( name ) ) >
	   *allowedSize - 3 ) {		/* leave space for )\0 */
	*allowedSize *= 2;
	*allowBase = (char *) REALLOC( *allowBase, *allowedSize );
      }
      if ( *allowBase + 1 != *allowed ) {
	*(*allowed)++ = '|';
      }
      strcpy( *allowed, name );
      *allowed += strlen( name );
    }
  } else {	/* no groups */
    if ( *match != 0 ) {	/* not matched, yet */
      *match = WILDPAT_CMP( program, name );
    }
    if ( ! *allowBase ) {	/* Don't bother expanding the allow list */
      return;
    }
    if ( ( t = *allowed - *allowBase ) >
	 *allowedSize - 2 + strlen( name ) ) {
      if ( *allowBase == *allowed ) {
	*(*allowed)++ = '|';
      }
      *allowedSize *= 2;
      *allowBase = (char *) REALLOC( *allowBase, *allowedSize );
    }
    strcpy( *allowed, name );
    *allowed += strlen( name );
  }
}

/*
 *  LASProgramEval
 *    INPUT
 *	attr_name	The string "program" - in lower case.
 *	comparator	CMP_OP_EQ or CMP_OP_NE only
 *	attr_pattern	A comma separated list of shell expressions and
 *			program groups of programs
 *	*cachable	This is set to ACL_NOT_CACHABLE, because we need
 *			to add stuff to the pblocks.
 *      subject		Subject property list
 *      resource	Resource property list
 *	auth_info	Authentication info, if any
 *    RETURNS
 *	retcode	        The usual LAS return codes.
 */
int LASProgramEval(NSErr_t *errp, char *attr_name, CmpOp_t comparator, 
                   char *attr_pattern, ACLCachable_t *cachable,
                   void **LAS_cookie, PList_t subject, PList_t resource,
                   PList_t auth_info, PList_t global_auth)
{
    char *uri;
    char *program;
    char *programs = attr_pattern;
    char *workingPrograms;
    char *allowedPrograms = NULL;
    int  allowedProgramSize;
    pb_param	*allowed;
    int	 retcode;
    int	 matched = 1;
    int  rv;
    Request *rq=0;
    struct program_groups *program_groups;

    *cachable=ACL_NOT_CACHABLE;
    *LAS_cookie = (void *)0;

    if (strcmp(attr_name, "program") != 0) {
	nserrGenerate(errp, ACLERRINVAL, ACLERR5300, ACL_Program, 2, XP_GetAdminStr(DBT_lasProgramEvalReceivedRequestFor_), attr_name);
	return LAS_EVAL_INVALID;
    }

    if ((comparator != CMP_OP_EQ) && (comparator != CMP_OP_NE)) {
	nserrGenerate(errp, ACLERRINVAL, ACLERR5310, ACL_Program, 2, XP_GetAdminStr(DBT_lasprogramevalIllegalComparatorD_), comparator_string(comparator));
	return LAS_EVAL_INVALID;
    }

#ifndef	UTEST
    /* Get the program from the URI and store it in program. */
    rv = PListGetValue(resource, ACL_ATTR_REQUEST_INDEX, (void **)&rq, NULL);
    if (rv < 0) {
	char rv_str[16];
	sprintf(rv_str, "%d", rv);
	nserrGenerate(errp, ACLERRINVAL, ACLERR5320, ACL_Program, 2, XP_GetAdminStr(DBT_lasProgramUnableToGetRequest_), rv_str);
        return LAS_EVAL_FAIL;
    }
 
    uri = STRDUP(pblock_findval("uri", rq->reqpb));
    program = strrchr(uri, '/');
    *program++ = '\0';
#else
    program = (char *)subject;
#endif

    if (!strcasecmp(attr_pattern, "all")) {	/* simplify */
       matched = 0;
       programs = "*";
    }
    /* Let's expand the program group list into programs */
#ifndef	UTEST
    rv = PListGetValue(resource, ACL_ATTR_PROGRAMS_INDEX, (void **)&program_groups, NULL);
#else
    rv = 0;
    program_groups = (struct program_groups *)resource;
#endif

    if (rv < 0) {
      program_groups = NULL;
    }
    allowed = pblock_find( "ADM_Programs", rq->vars );
    allowedProgramSize = allowed ? MAX( strlen( allowed->value ) + 2, 1024u ) :
      1024u;
    allowedPrograms = (char *) MALLOC( allowedProgramSize );
    if ( allowed ) {
      strcpy( allowedPrograms, allowed->value );
    } else {
      allowedPrograms[0] = '\0';
    }
    workingPrograms = allowedPrograms + strlen( allowedPrograms );

    /* Expand programs into allowedPrograms, using the program_groups.
       If we happen to match along the way, so much the better */
    matchList( &matched, program, programs, program_groups,
	       &allowedPrograms, &workingPrograms, &allowedProgramSize );

    if(matched == -1)  {
	nserrGenerate(errp, ACLERRINVAL, ACLERR5330, ACL_Program, 2, XP_GetAdminStr(DBT_lasProgramReceivedInvalidProgramExpression), attr_pattern);
        return LAS_EVAL_INVALID;
    }

    if (comparator == CMP_OP_EQ) {
	retcode = (matched ? LAS_EVAL_FALSE : LAS_EVAL_TRUE);
    }
    else {
	retcode = (matched ? LAS_EVAL_TRUE : LAS_EVAL_FALSE);
    }

#ifndef	UTEST
    FREE(uri);

    if(retcode == LAS_EVAL_TRUE)  {
      if ( allowed ) {
	FREE( allowed->value );
	allowed->value = allowedPrograms;
      } else {
	pblock_nvinsert("ADM_Programs", allowedPrograms, rq->vars);
      }
    } /* do this if we know there are no more ACEs QQQQ  else  {
	nserrGenerate(errp, ACLERRINVAL, ACLERR5340, ACL_Program, 3, XP_GetAdminStr(DBT_lasProgramRejectingRequestForProgram_), program, attr_pattern);
    } */
#endif

    return retcode;
}


/*	LASProgramFlush
 *	Deallocates any memory previously allocated by the LAS
 */
void
LASProgramFlush(void **las_cookie)
{
    /* do nothing */
    return;
}
