/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * admconf.c - Read the admin configuration file to determine document
 *             track, index settings, manipulate current resource, and 
 *             other fun jazz
 *
 * All blame to Mike McCool
 */

#include "libadminutil/admutil.h"
#include "libadmin/libadmin.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef MCC_PROXY
#define TOTAL_INDEX_VARS 14U
#else
#define TOTAL_INDEX_VARS 20U
#endif

#define CONFIG1_DEF "name=default"
#define CONFIG3_DEF "default"
#define CONFIG4_DEF "0:0:0"
#define CONFIG5_DEF "0:0:0:0:0"
#define CONFIG5_SIZE 4  /* 4 colons, of course */

/*
 * NS_MAIL - force_default_referer - because madm.c does setuid(mta) and
 * therefore doesn't write to ADM CONF; also, sec-* write to it, so we
 * don't want madm.c to find sec-*'s stuff
 */
int force_default_referer = 0;

NSAPI_PUBLIC char **get_mult_adm_config(int whichone)
{
    FILE *f;
    char **config, scratch[1024];
    char *fn = NULL;
    
    config = new_strlist(6);

    if(whichone != -1)  {
        char *cfdir = CONFDIR(whichone);
        char *t2, *t3;

        t2=strchr(cfdir, '/');
        *t2='\0';
        *t2='/';

        t3=ADM_CONF;
        
        fn=(char *) MALLOC(strlen(cfdir) + strlen(ADM_CONF) + 12);
        sprintf(fn, "%s%s", cfdir, ADM_CONF);
    }
/* NS_MAIL - force_default_referer */
    if(force_default_referer || (whichone == -1) || !(f = fopen(fn, "r")))  {
        /* config read failed.  Generate a default. */
        scratch[0] = '\0';
        while(strlen(scratch) < TOTAL_INDEX_VARS)  {
            strcat(scratch, "1");
        }

        /* BONEHEAD XXX GET RID OF ME MLM */
        scratch[0]='0';
        /* */

        config[0] = STRDUP(scratch);
        config[1] = STRDUP(CONFIG1_DEF);
#ifdef NS_MAIL
/* NS_MAIL needs to use HTTP_REFERER since there may be a query string */
/* Maybe all products should use this. */
	if (getenv("HTTP_REFERER")) strcpy(scratch, getenv("HTTP_REFERER")); else
/* next sprintf is the 'else' part */
#endif
        sprintf(scratch, "%s%s", getenv("SERVER_URL"), getenv("SCRIPT_NAME"));
        config[2] = STRDUP(scratch);
        config[3] = STRDUP(CONFIG3_DEF);
        config[4] = STRDUP(CONFIG4_DEF);
        config[5] = STRDUP(CONFIG5_DEF);
#if !defined(MCC_ADMSERV) && !defined(NS_MAIL)
        if(whichone != -1)  {
            /* hack for help */
            char *sname=get_srvname(0);
            char *t=strchr(sname, '-');  /* skip product */
	    if (t) {
		t++;
		if(strcmp(t, "help"))  {
		    /* update the mod times if they're going to be zero and they 
		     * might be making manual changes */
		    write_adm_config(config);
		    set_all_org_mtimes();
		}
	    }
	}
#endif
    }
    else  {
        scratch[0]='\0';
        /* fgets leaves the \n on.*/
        fgets(scratch, 1024, f);
        scratch[strlen(scratch)-1] = '\0';
        config[0] = STRDUP(scratch);
        if(strlen(config[0]) < TOTAL_INDEX_VARS)  {
            while(strlen(scratch) < TOTAL_INDEX_VARS)  {
                strcat(scratch, "1");
            }
            config[0] = STRDUP(scratch);
        }
        /* BONEHEAD XXX GET RID OF ME MLM */
        config[0][0]='0';
        /* */


        if(!fgets(scratch, 1024, f))
            sprintf(scratch, "%s", CONFIG1_DEF);
        else
            scratch[strlen(scratch)-1] = '\0';
        config[1] = STRDUP(scratch);

        if(!fgets(scratch, 1024, f))
            scratch[0] = '\0';
        else 
            scratch[strlen(scratch)-1] = '\0';
        config[2] = STRDUP(scratch);

        if(!fgets(scratch, 1024, f))
            sprintf(scratch, "%s", CONFIG3_DEF);
        else
            scratch[strlen(scratch)-1] = '\0';
        config[3] = STRDUP(scratch);

        if(!fgets(scratch, 1024, f))
            sprintf(scratch, "%s", CONFIG4_DEF);
        else
            scratch[strlen(scratch)-1] = '\0';
        config[4] = STRDUP(scratch);

        if(!fgets(scratch, 1024, f))
            sprintf(scratch, "%s", CONFIG5_DEF);
        else
            scratch[strlen(scratch)-1] = '\0';
        {int n=0, x=0; for(x=0; scratch[x]; x++) if(scratch[x]==':') n++;
         if(n<CONFIG5_SIZE)  {
             while(n++ < CONFIG5_SIZE) strcat(scratch, ":0");
         }
        } 
         
        config[5] = STRDUP(scratch);

        fclose(f);
    }

    return config;
}

NSAPI_PUBLIC char **get_adm_config(void)
{
    return get_mult_adm_config(0);
}

NSAPI_PUBLIC int write_mult_adm_config(int whichone, char **config)
{
    FILE *f;
    char admconf[512];

    sprintf(admconf, "%s%s", CONFDIR(whichone), ADM_CONF);
    if(!(f = fopen_l(admconf, "w")))
        return(1);
    else  {
        fprintf(f, "%s\n", config[0] ? config[0] : "");
        fprintf(f, "%s\n", config[1] ? config[1] : CONFIG1_DEF);
        fprintf(f, "%s\n", config[2] ? config[2] : "");
        fprintf(f, "%s\n", config[3] ? config[3] : CONFIG3_DEF);
        fprintf(f, "%s\n", config[4] ? config[4] : CONFIG4_DEF);
        fprintf(f, "%s\n", config[5] ? config[5] : CONFIG5_DEF);
        fclose_l(f);
    }
    return(0);
}

NSAPI_PUBLIC int write_adm_config(char **config)  {
    return write_mult_adm_config(0, config);
}

NSAPI_PUBLIC char *get_current_resource(char **config)
{
  char	*res = cookieValue( "resource", NULL );

  return res ? strdup( res ) : strdup( "default" );
}

NSAPI_PUBLIC char *get_current_typestr(char **config)
{
  char	*type = cookieValue( "resourceType", NULL );

  return type ? strdup( type ) : strdup( NAME );
}

NSAPI_PUBLIC int get_current_restype(char **config)
{
    char *tmp = get_current_typestr(&tmp);

    if(!strcmp(tmp, FILE_OR_DIR))
      return PB_PATH;
    if(!strcmp(tmp, WILDCARD))
      return PB_PATH;
    if(!strcmp(tmp, TEMPLATE))
      return PB_NAME;
    return PB_NAME;
}

NSAPI_PUBLIC void set_current_resource(char **config, char *nrestype, char *nres)
{
  char	*server = get_srvname( 0 );

  printf( "<script language="MOCHA_NAME">\n"
	  "document.cookie=\"resourceType=%s; path=/%s/\";\n"
	  "document.cookie=\"resource=%s; path=/%s/\";\n"
	  "</script>\n", nrestype, server, jsEscape( nres ), server );

  /* set the in-memory versions of the cookies */
  cookieValue( "resourceType", nrestype );
  cookieValue( "resource", nres );
  FREE( server );
}

