/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * admserv.c: Routines for dealing with ns-admin.conf
 *
 * Based on the magconf.c by the stupid looking one
 * Based on the magconf.c by the Grand, Supreme Being
 * 
 * Now also contains code to choose a server or several servers
 * 
 * Rob McCool
 */


#include "libadminutil/admutil.h"
#include "libadmin/libadmin.h"


#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

static char **nsadmconf = NULL;

void read_nsadm_conf(void)
{
   if(nsadmconf == NULL) {
       char nsadmpath[512];

       sprintf(nsadmpath, "%s%c%s", ADMCONFDIR, FILE_PATHSEP, NSADMIN_CONF);
       nsadmconf = scan_tech(nsadmpath);
   }
   if(nsadmconf == NULL)
       report_error(FILE_ERROR, NULL, get_msg(ADMSERV_ERR_NOREAD_NSADM));
}

void write_nsadm_conf(void)
{
    register int x = 0;
    FILE *f;
    char nsadmpath[512];

    sprintf(nsadmpath, "%s%c%s", ADMCONFDIR, FILE_PATHSEP, NSADMIN_CONF);
    f = fopen_l(nsadmpath, "w");

    if(f == NULL)
       report_error(FILE_ERROR, NULL, get_msg(ADMSERV_ERR_NOWRITE_NSADM));

    for(x=0; nsadmconf[x]; x++)
        if(strcmp(nsadmconf[x], "\n"))
            fprintf(f, "%s\n", nsadmconf[x]);
        else
            fprintf(f, "%s", nsadmconf[x]);

    fclose_l(f);
}

void insert_nsadm_var(char *var)
{
    int nsadmlen;

    for(nsadmlen=0; nsadmconf[nsadmlen]; nsadmlen++);

    nsadmconf = grow_strlist(nsadmconf, nsadmlen+2);

    nsadmconf[nsadmlen] = STRDUP(var);
    nsadmconf[nsadmlen+1] = NULL;

    write_nsadm_conf();
}

void delete_nsadm_var(int offset)
{
    int nsadmlen;
    register int x;
    
    for(nsadmlen=0; nsadmconf[nsadmlen]; nsadmlen++);

    FREE(nsadmconf[offset]);

    for(x = offset+1; x <= nsadmlen; x++)  {
        nsadmconf[x-1] = nsadmconf[x];
    }
    
    nsadmconf[nsadmlen] = NULL;
}

NSAPI_PUBLIC char *get_nsadm_var(char *var)
{
   register int x; 
   char *value = NULL;
   int len = strlen(var);
   
   read_nsadm_conf();

   if(var[len-1] == ' ')
       var[--len] = '\0';

   for(x=0;nsadmconf[x];x++)  {
       if (!strncasecmp(nsadmconf[x], var, len))  {
           value = STRDUP(nsadmconf[x]+len+1); 
           break;
       }
   }
   return value;
}

NSAPI_PUBLIC void set_nsadm_var(char *name, char *value)
{
    register int x=0; 
    int found = 0, l = strlen(name);
    char line[BIG_LINE];

    read_nsadm_conf();

    if(name[l-1] == ' ')
        name[--l] = '\0';

    for(x=0; nsadmconf[x]; x++)  {
        if(!strncmp(nsadmconf[x], name, l))  {
            found = 1;
            break;
        }
    }
    if(found)  {
        if(value)  {
            sprintf(line, "%s %s", name, value);
            nsadmconf[x] = STRDUP(line);
            log_change(TO_ADMIN, get_msg(ADMSERV_LOG_SET_VAR), name, value);
        }  else  {
            delete_nsadm_var(x);
            log_change(TO_ADMIN, get_msg(ADMSERV_LOG_RM_VAR), name);
        }
    }  else  {
        if(value)  {
            sprintf(line, "%s %s", name, value);
            insert_nsadm_var(line);
            log_change(TO_ADMIN, get_msg(ADMSERV_LOG_ADD_VAR), name, value);
        }
    }

    write_nsadm_conf();
}


#ifdef USE_ADMSERV

#define BUFFER_SIZE 10

/*
 * Spawned off as idependent routine for used by both admserv and cluster
 * - 8/2/96 Adrian
 */
NSAPI_PUBLIC char **
scan_server_instance(char *dirname, char **namelist)
{
    SYS_DIR ds;
    SYS_DIRENT *d;
    char **ans = new_strlist(BUFFER_SIZE);
    int totalsize=BUFFER_SIZE, curentry=0;

    if(!(ds = dir_open(dirname))) {
        report_error(FILE_ERROR, dirname, get_msg(ADMSERV_ERR_ADMDIR_NOREAD));
    }
 
    if (namelist == NULL)
       return NULL;

    while( (d = dir_read(ds)) )  {
        char *entry;
        register int z;

        entry = d->d_name;

        for(z=0; namelist[z]; z++)  {
	    char *srv_prefix;
	    srv_prefix = (char *) MALLOC(strlen(namelist[z])+2);
	    sprintf(srv_prefix, "%s-", namelist[z]);
            if( (!strncmp(entry, srv_prefix, strlen(srv_prefix))) )  {
                if( !(curentry < totalsize) )  {
                    totalsize += BUFFER_SIZE;
                    ans=grow_strlist(ans, totalsize);
                }
                ans[curentry++] = STRDUP(entry);
                ans[curentry] = NULL;
            }
	    (void) FREE(srv_prefix);
        }
    }

    dir_close(ds);

    if(curentry)
        return ans;
    else
        return NULL;
}

/* Generates a list of all the servers on your machine. */
NSAPI_PUBLIC char **list_installed_servers(char **namelist)
{
    char *nsroot = getenv("NETSITE_ROOT");
    char *admdir = (char *)MALLOC(strlen(nsroot) + 10);

    sprintf(admdir, "%s", nsroot); 

    return(scan_server_instance(admdir, namelist));
}

/*
 * Spawned off as idependent routine for used by both admserv and cluster
 * - 8/2/96 Adrian
 */
NSAPI_PUBLIC int
read_AbbrDescType_file(char *path, char ***namelist, char ***desclist)
{
    FILE *sf;
    char line[BIG_LINE];
    char *sname, *sdesc;
    int listsize=0;

    *namelist = NULL;
    *desclist = NULL;

    sf=fopen(path, "r");
 
    if (!sf) return(-1);

    while(fgets(line, BIG_LINE, sf))  {
	if ((line[0] == '#') ||
	    (line[0] == '\n')) { /* get rid of comment or blank lines*/
	   continue;
	}

        sname=line;
        sdesc = strchr(line, ':');
	if(!sdesc) continue;   /* invalid line */
        *sdesc++ = '\0';
        listsize++;
        if(!*namelist)  {
            *namelist = (char **) MALLOC((listsize+2)*sizeof(char *));
            *desclist = (char **) MALLOC((listsize+2)*sizeof(char *));
        }  else  {
            *namelist=(char **) REALLOC(*namelist, (listsize+2)*sizeof(char *));
            *desclist=(char **) REALLOC(*desclist, (listsize+2)*sizeof(char *));
        }
        (*namelist)[listsize-1] = STRDUP(sname);
        (*desclist)[listsize-1] = STRDUP(sdesc);
        (*namelist)[listsize] = NULL;
        (*desclist)[listsize] = NULL;
    }
    fclose(sf);

    return(0);
}

/* Makes a list of the server _types_ installed on your machine. */
void read_server_lst(char ***namelist, char ***desclist)
{
    char path[PATH_MAX];

    sprintf(path, "%s%cadmin-serv%cconfig%cservers.lst", getenv("NETSITE_ROOT"),
                  FILE_PATHSEP, FILE_PATHSEP, FILE_PATHSEP);

    if (read_AbbrDescType_file(path, namelist, desclist) < 0 || *namelist == NULL) {
      /* used to complain, but there might not be anything installed, yet */
      *namelist = (char **) MALLOC( sizeof( char ** ) );
      *desclist = (char **) MALLOC( sizeof( char ** ) );
      **namelist = NULL;
      **desclist = NULL;
    }
}

#endif


