/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * commit.c: Handle the New Way of first making changes, then committing them.
 * 
 * Also deals with config file backup systems.
 *
 * All blame to Mike McCool
 */

#include "libadminutil/admutil.h"
#include "libadmin/libadmin.h"
#include "base/util.h"
#include <libadmin/backup.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef XP_UNIX
#include <utime.h>
#else /* XP_NT */
#include <sys/utime.h>
#endif
#ifndef MAX
#define MAX(x,y) ((x)>(y)?(x):(y))
#endif

#ifdef USE_ADMSERV

/* Size of the network buffer */
#define NBUF_SIZE 1024

/* The POST request string to commit the changes */
#define POST_REQ "POST %s HTTP/1.0\nAuthorization: basic %s\nContent-type: text/plain\nContent-length: %d\nNS-Interactive: No\n\n"

/* The GET request string to undo the changes */
#define GET_REQ "GET %s HTTP/1.0\nAuthorization: basic %s\nContent-type: text/plain\n\n"

/* The POST request string to upload a userdb. */
#define DB_REQ "POST /%s-database-transfer/bin/dbtrans?upload HTTP/1.0\nAuthorization: basic %s\nNS-Interactive: No\nNS-User-Database: %s\nContent-length: %d\nContent-type: text/plain\n\n"

static char *getBackupConf();

/* return code	- -1: error, check errno
		- 0: No action needed
		- 1: working file(s) need backing up
		- 2: server file(s) were manually edited
		- 3: both 1 and 2 */

NSAPI_PUBLIC int
needs_commit( char **whoCares )	/* old interface, can figure it out from env */
{
  int		result = 0;	/* don't need backing up, yet */
  char		*confFile = getBackupConf();
  ADM_bkTree	*tree = ADM_bk_initTree( getenv( "NETSITE_ROOT" ),
					 confFile, 1, 1 );
  int		i;
  char		buf[BUFSIZ];
  struct stat	statBuf;

  if ( ! tree ) {
    return -1;
  }
  for ( i = 0 ; i < tree->numFiles ; ++i ) {
    /* check working copy to see if it's been backed up */
    stat( tree->files[i].fullPath, &statBuf );
    if ( statBuf.st_mtime != tree->files[i].timestamp ) {
      result |= 1;
    }
    /* check server copy, to see if it's the same as the working copy */
    if ( tree->files[i].extraInfo ) {	/* Has a server copy */
      if ( tree->files[i].extraInfo[0] == '/' ) {
	strcpy( buf, tree->files[i].extraInfo );
      } else {
	strcpy( buf, tree->root );
	strcat( buf, "/" );
	strcat( buf, tree->files[i].extraInfo );
      }
      buf[strlen( buf ) - 1] = '\0';	/* kill \n */
      stat( buf, &statBuf );
      if ( statBuf.st_mtime > tree->files[i].timestamp ) {
	result |= 2;
      }
    }
  }
  FREE( confFile );
  ADM_bk_done( tree );
  return result;
}

/* Set a flag for whether we need to commit or we don't need to. */
NSAPI_PUBLIC void set_commit(int whichsrv, int needscommit)
{
    if(!needscommit)  {
    /* If we no longer need a commit, then we no longer need the log of *
     * changes. */
        char *cdir = CONFDIR(whichsrv);
        char *log = (char *) MALLOC(strlen(cdir) + strlen("/commit")+2);

        sprintf(log, "%scommit", cdir);
        unlink(log);
	FREE( log );
    }
}
 
/* Gets the modification time of when the file was last transmitted. */
NSAPI_PUBLIC time_t get_org_mtime(int whichsrv, int whichfile)
{
    char **config = get_mult_adm_config(whichsrv);
    char *c0, *c1, *c2, *c3, *c4;
    long ans = 0;

    c0 = STRDUP(config[5]);
    c1 = strchr(c0, ':');
    *c1++ = '\0';
    c2 = strchr(c1, ':');
    *c2++ = '\0';
    c3 = strchr(c2, ':');
    *c3++ = '\0';
    c4 = strchr(c3, ':');
    *c4++ = '\0';

    if(whichfile == BK_MAGNUS)  {
        ans = atol(c0);
    }  else
    if(whichfile == BK_OBJ)  {
        ans = atol(c1);
    }  else
    if(whichfile == BK_MIMETYPES)  {
        ans = atol(c2);
    }  else
    if(whichfile == BK_ACLFILE)  {
        ans = atol(c3);
    }  else
    if(whichfile == BK_BU)  {
        ans = atol(c4);
    }
    return (time_t) ans;
}

NSAPI_PUBLIC char *get_mtime_str(int whichsrv)
{
    char **config = get_mult_adm_config(whichsrv);
    return STRDUP(config[5]);
}

NSAPI_PUBLIC void set_mtime_str(int whichsrv, char *str)
{
    char **config = get_mult_adm_config(whichsrv);
    config[5] = STRDUP(str);
    write_mult_adm_config(whichsrv, config);
}

/* Set the modification time of last transmission or whatever. */
NSAPI_PUBLIC void set_org_mtime(int whichsrv, int whichfile, time_t mtime)
{
    char **config = get_mult_adm_config(whichsrv);
    char *c0, *c1, *c2, *c3, *c4;
    char buf[256];
 
    c0 = STRDUP(config[5]);
    c1 = strchr(c0, ':');
    *c1++ = '\0';
    c2 = strchr(c1, ':');
    *c2++ = '\0';
    c3 = strchr(c2, ':');
    *c3++ = '\0';
    c4 = strchr(c3, ':');
    *c4++ = '\0';

    sprintf(buf, "%lu", mtime);
 
    if(whichfile == BK_MAGNUS)  {
        c0 = STRDUP(buf);
    }  else
    if(whichfile == BK_OBJ)  {
        c1 = STRDUP(buf);
    }  else
    if(whichfile == BK_MIMETYPES)  {
        c2 = STRDUP(buf);
    }  else
    if(whichfile == BK_ACLFILE)  {
        c3 = STRDUP(buf);
    }  else
    if(whichfile == BK_BU)  {
        c4 = STRDUP(buf);
    }
    sprintf(buf, "%s:%s:%s:%s:%s", c0, c1, c2, c3, c4);
    config[5] = STRDUP(buf);
    write_mult_adm_config(whichsrv, config);
    return;
}

/* Set the modification times for *all* of the files needing this check,
 * assuming admin.conf got lost or hasn't been created yet. */
/* When it doubt, set to zero. */
NSAPI_PUBLIC void set_all_org_mtimes(void)
{
    char *srcdir=CONFDIR(0);
    char *dstdir=COMMDEST(0);
    char *srcfn=NULL;
    char *dstfn=NULL;
    time_t srcmod=0;
    time_t dstmod=0;

    /* In this instance, we know that the time in the config file is 0.
     * So we set it to the only thing we can set it to: the value of the
     * mod time in the destination.  Hopefully, they've come to the index
     * page at least once so that we get the correct mod time and don't
     * overwrite manual changes. */
    /* Files: BK_MAGNUS, BK_OBJ, BK_MIMETYPES, BK_ACLFILE, BK_BU
     */
    /* BK_MAGNUS */
    srcfn=(char *)MALLOC(strlen(srcdir)+strlen(MAGNUS_CONF)+4);
    sprintf(srcfn, "%s%s", srcdir, MAGNUS_CONF);
    dstfn=(char *)MALLOC(strlen(dstdir)+strlen(MAGNUS_CONF)+4);
    sprintf(dstfn, "%s%s", dstdir, MAGNUS_CONF);
    srcmod=get_mtime(srcfn);
    dstmod=get_mtime(dstfn);
    if(srcmod && dstmod)  {
        set_org_mtime(0, BK_MAGNUS, dstmod);
    }
    FREE(srcfn);
    FREE(dstfn);

    /* BK_OBJ */
    srcfn=(char *)MALLOC(strlen(srcdir)+strlen(OBJ_DATABASE)+4);
    sprintf(srcfn, "%s%s", srcdir, OBJ_DATABASE);
    dstfn=(char *)MALLOC(strlen(dstdir)+strlen(OBJ_DATABASE)+4);
    sprintf(dstfn, "%s%s", dstdir, OBJ_DATABASE);
    srcmod=get_mtime(srcfn);
    dstmod=get_mtime(dstfn);
    if(srcmod && dstmod)  {
        set_org_mtime(0, BK_OBJ, dstmod);
    }
    FREE(srcfn);
    FREE(dstfn);

    /* BK_MIMETYPES */
    srcfn=(char *)MALLOC(strlen(srcdir)+strlen(MIME_TYPES)+4);
    sprintf(srcfn, "%s%s", srcdir, MIME_TYPES);
    dstfn=(char *)MALLOC(strlen(dstdir)+strlen(MIME_TYPES)+4);
    sprintf(dstfn, "%s%s", dstdir, MIME_TYPES);
    srcmod=get_mtime(srcfn);
    dstmod=get_mtime(dstfn);
    if(srcmod && dstmod)  {
        set_org_mtime(0, BK_MIMETYPES, dstmod);
    }
    FREE(srcfn);
    FREE(dstfn);

    /* BK_ACLFILE */
    srcfn=get_workacl_file();
    dstfn=get_acl_file();
    srcmod=get_mtime(srcfn);
    dstmod=get_mtime(dstfn);
    if(srcmod && dstmod)  {
        set_org_mtime(0, BK_ACLFILE, dstmod);
    }

#ifdef MCC_PROXY 
    /* BK_BU */
    {
        pblock *pb=get_mag_init("init-batch-update");
        if(pb)  {
            char *bu_fn=pblock_findval("conf-file", pb);
            if(bu_fn)  {
                srcfn=(char *)MALLOC(strlen(srcdir)+strlen(bu_fn)+4);
                sprintf(srcfn, "%s%s", srcdir, bu_fn);
                dstfn=(char *)MALLOC(strlen(dstdir)+strlen(bu_fn)+4);
                sprintf(dstfn, "%s%s", dstdir, bu_fn);
                srcmod=get_mtime(srcfn);
                dstmod=get_mtime(dstfn);
                if(srcmod && dstmod)  {
                    set_org_mtime(0, BK_BU, dstmod);
                }
            }
        }
    }
#endif
    return;
}

/* List the DB's that need to be transmitted over the network. */
char **list_uncommitted_dbs(void)
{
    char **ans = NULL, **dirlist = NULL;
    int listsize=0, curpos = 0;
    char *ext = NULL;
    int x = 0;

    dirlist = list_directory(get_userdb_dir(),0);
    if(!dirlist) return NULL;
    for(x=0; dirlist[x]; x++)  {
        ext = strrchr(dirlist[x], '.');
        if(!ext) continue;
        ext++;
        if(!strcmp(ext, DB_INC))  {
            if((curpos+1) > listsize)  {
                listsize+=5;
                if(!ans) ans = new_strlist(listsize);
                   else  ans = grow_strlist(ans, listsize);
            }
            ans[curpos++] = dirlist[x];
            ans[curpos] = NULL;
        }
    }
    return ans;
}

void remove_uncommitted_dbs(void)
{
    int d = 0;
    char **dblist = list_uncommitted_dbs();
    char *source = NULL;
    char *dbdir = get_userdb_dir();

    if(!dblist) return;
    source = (char *) MALLOC(strlen(dbdir) + 1024);

    for(d=0; dblist[d]; d++)  {
        if(strlen(dblist[d]) > 1024u) continue;
        sprintf(source, "%s%c%s", dbdir, FILE_PATHSEP, dblist[d]);
        fprintf(stdout, "<i>Removing db increment file</i> %s<br>\n", source);
        unlink(source); 
    }
}

NSAPI_PUBLIC int
do_commit( char **whoCares )
{
  char			*confFile = getBackupConf();
  ADM_bkTree		*tree = ADM_bk_initTree( getenv( "NETSITE_ROOT" ),
						 confFile, 0, 1 );
  char			buf[BUFSIZ];
  struct stat		statBuf;
  struct utimbuf	timeBuf;
  int			i;

  
  if ( ! tree ) {
    return -1;
  }
  ADM_bk_doBackups( tree, NULL );	/* backup working copies */

  /* go looking for server copies that are different from working copies */
  for ( i = 0 ; i < tree->numFiles ; ++i ) {
    if ( tree->files[i].extraInfo ) {	/* Has a server copy */
      if ( tree->files[i].extraInfo[0] == '/' ) {
	strcpy( buf, tree->files[i].extraInfo );
      } else {
	strcpy( buf, tree->root );
	strcat( buf, "/" );
	strcat( buf, tree->files[i].extraInfo );
      }
      buf[strlen( buf ) - 1] = '\0';	/* kill \n */
      stat( buf, &statBuf );
      if ( statBuf.st_mtime != tree->files[i].timestamp ) {
	/* need to commit to server copy */
	cp_file( tree->files[i].fullPath, buf, 0644 );
	timeBuf.actime = tree->files[i].timestamp;
	timeBuf.modtime = tree->files[i].timestamp;
	utime( buf, &timeBuf );
      }
    }
  }
  ADM_bk_done( tree );
  FREE( confFile );
  set_commit( 0, 0 );	/* unlink the commit log */
  log_change(TO_ADMIN, "Committed changes from %s to %s", 
                             CONFDIR( 0 ), COMMDEST( 0 ) ); 
  return 0;
}

/* Back out changes. */
NSAPI_PUBLIC int do_undo(char **authlist)
{
  char			*confFile = getBackupConf();
  ADM_bkTree		*tree = ADM_bk_initTree( getenv( "NETSITE_ROOT" ),
						 confFile, 1, 1 );
  int			i;
  char			buf[BUFSIZ];
  struct stat		statBuf;
  struct utimbuf	timeBuf;
  int			doBackup = 0;
  int			checkWorking;

  if ( ! tree ) {
    return -1;
  }
  for ( i = 0 ; i < tree->numFiles ; ++i ) {
    /* Table of possibilities.  E means editted, U means untouched
       Server	Working	Action
       U	U	Nothing
       U	E	Revert Working to last backup
       E	U	Manual edit, copy Server to Working, then backup
       E	E	Manual edit overrides, do EU line */
    checkWorking = 1;
    /* check for server change, since it's most important */
    if ( tree->files[i].extraInfo ) {
      if ( tree->files[i].extraInfo[0] == '/' ) {
	strcpy( buf, tree->files[i].extraInfo );
      } else {
	strcpy( buf, tree->root );
	strcat( buf, "/" );
	strcat( buf, tree->files[i].extraInfo );
      }
      buf[strlen( buf ) - 1] = '\0';	/* kill \n */
      stat( buf, &statBuf );
      if ( statBuf.st_mtime != tree->files[i].timestamp ) {	/* editted */
	cp_file( buf, tree->files[i].fullPath, 0644 );
	doBackup = 1;
	checkWorking = 0;
      }
    }
    if ( checkWorking ) {
      stat( tree->files[i].fullPath, &statBuf );
      if ( statBuf.st_mtime != tree->files[i].timestamp ) {
	sprintf( buf, "%s/%s.%d", tree->backupDir, tree->files[i].name,
		 tree->files[i].version );
	cp_file( buf, tree->files[i].fullPath, 0644 );
	/* set the time, as though this file was never touched */
	timeBuf.actime = tree->files[i].timestamp;
	timeBuf.modtime = tree->files[i].timestamp;
	utime( tree->files[i].fullPath, &timeBuf );
      }
    }
  }
  ADM_bk_done( tree );
  if ( doBackup ) {
    do_commit( NULL );
  }
  set_commit( 0, 0 );	/* unlink log */
  log_change( TO_ADMIN, "Undo changes from %s to %s", 
	      COMMDEST( 0 ), CONFDIR( 0 ) );
  return 0;
}

char *get_changes(char *curline)
{
    int x = 0;
    char *ans;

    if(!curline) return NULL;
    ans = STRDUP(curline);

    while(ans[x])  {
        if(!isspace(ans[x]))  {
            break;
        }
        x++;
    }
    if(!ans[x]) return NULL;

    while(ans[x])  {
        if(isspace(ans[x]))  {
            break;
        }
        x++;
    }
    if(!ans[x]) return NULL;

    x++;
    return(&ans[x]);
}

char *get_servname(char *curline)
{
    int x = 0, y;
    char *ans;

    if(!curline) return NULL;
    ans = STRDUP(curline);

    while(ans[x])  {
        if(ans[x] == '@')  {
            break;
        }
        x++;
    }
    if(!ans[x]) return NULL;

    while(ans[x])  {
        if(ans[x] == ':')  {
            break;
        }
        x++;
    }
    if(!ans[x]) return NULL;

    x+=2;

    y=x;
    while(ans[y])  {
        if(ans[y] == ' ')  {
            break;
        }
        y++;
    }
    if(!ans[y]) return NULL;
    ans[y] = '\0';

    return (&(ans[x]));
}

NSAPI_PUBLIC void output_uncommitted(void)
{
    char *log = NULL;
    FILE *f;
    char line[BIG_LINE];
    char *curserver = NULL, *t;
    int found = 0, howmany = make_conflist();
    register int y;
   
    fprintf(stdout, "<tr></tr><tr>\n");

    for(y=0; y<howmany; y++)  {
        char *cdir = CONFDIR(y);
        log = (char *)MALLOC(strlen(cdir)+strlen("/commit")+2);
        sprintf(log, "%s%ccommit", cdir, FILE_PATHSEP);

        if(!(f = fopen(log, "r")))  {
            continue;
        }
        found=1;

        while(fgets(line, BIG_LINE, f)) {
            if(isspace(line[0]))  {
                if(!curserver)  {
                    curserver = STRDUP("NO_SUCH_SERVER");
                    fprintf(stdout, "<td colspan=2>\n");
                }
                t = get_changes(line);
                if(t) fprintf(stdout, "%s<br>", t);
            }
            else if((t = get_servname(line)))  {
                if((!curserver) || (strcmp(t, curserver)))  {
                                    /* Heh.  Tables suck. */
                    if(curserver)
                        fprintf(stdout, "</td></tr><tr></tr><tr>");
    
                    fprintf(stdout, "<td colspan=2>On the server <b>%s</b>:"
                                    "</td>\n</tr><tr><td></td>"
                                    "<td nowrap>", t);
                    curserver = STRDUP(t);
                }  else  {
                    fprintf(stdout, "</td></tr><tr><td width=8></td>"
                                    "<td nowrap>");
                }
            }
        }
        fprintf(stdout, "</pre></td></tr>\n");
        fclose(f);
    }
   
    return;
}

/* Have we been here before? */
static int *has_backed = NULL;

NSAPI_PUBLIC int get_bknum(int which, int index)
{
    char **config = get_mult_adm_config(index);
    char *t1, *t2;
 
    t1 = &(config[4][2]);
    t2 = strchr(t1, ':');
    if(t2) *t2++ = '\0';

    if(*t1 && t2)  {
        return (!which) ? atoi(t1) : atoi(t2);
    } else
        return 0;
}
 
NSAPI_PUBLIC void set_bknum(int num, int which, int index)
{
    char **config = get_mult_adm_config(index);
    char line[BIG_LINE];
    char *tmp1, *tmp2;
    int t1, t2;

    sprintf(line, "%s", config[4]);
    tmp1 = &(line[2]);
    tmp2 = strchr(tmp1, ':');
    *tmp2++ = '\0';

    t1=atoi(tmp1); 
    t2=atoi(tmp2);

    if(!which) t1=num;
    else t2=num;

    sprintf(line, "%c:%d:%d", config[4][0], t1, t2);
    config[4] = STRDUP(line);

    write_mult_adm_config(index, config);
}
 
/* Keep the same parameters, but ignore them, and backup what has changed */
NSAPI_PUBLIC void conf_backup(char *whichfile, int index, int whichsrv)
{
  char	*confFile = COMMDEST(0);	/* we'll work on this */
  int	i;

  /* trim 'config' from confFile */
  for ( i = strlen( confFile ) ; i && confFile[i] != '/' ; --i )
    ;
  confFile[i + 1] = '\0';
  confFile = (char *)REALLOC( confFile, strlen( confFile ) + 21 );
  strcat( confFile, "conf_bk/backups.conf" );
  ADM_bk_makeBackups( confFile, getenv( "NETSITE_ROOT" ), NULL );
  FREE( confFile );
}

/* Don't forget to free this */

static char *
getBackupConf()
{
  char	*confFile = COMMDEST(0);	/* we'll work on this */
  int	i;

  /* trim 'config' from confFile */
  for ( i = strlen( confFile ) - 2 ; i && confFile[i] != '/' ; --i )
    ;
  confFile[i + 1] = '\0';
  confFile = (char *)REALLOC( confFile, strlen( confFile ) + 21 );
  strcat( confFile, "conf_bk/backups.conf" );

  return confFile;
}

#endif
