/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/* 
 * distadm.c: Functions for distributed admin
 *            
 * All blame to Mike McCool
 */

#include "private/pprio.h"
#include "base/util.h"
#include "safs/cgi.h"
#include "libadmin/libadmin.h"

/* Yeah, I know, globals suck. */
#define ADM_NOT_INITIALIZED "NS-not-initialized"

static char *user = ADM_NOT_INITIALIZED;
static char *pass = ADM_NOT_INITIALIZED;
static char *auth = ADM_NOT_INITIALIZED;
static char *userDN = ADM_NOT_INITIALIZED;
#ifdef FORTEZZA
static char *pin = ADM_NOT_INITIALIZED;
#endif /* FORTEZZA */

/* The function to use when reading the stuff from the pipe */
#ifdef XP_WIN32
#define SYSTEM_PREAD system_pread
#else /* XP_UNIX */
#define SYSTEM_PREAD system_fread
#endif

NSAPI_PUBLIC int ADM_InitializePermissions(int *errcode)
{
    int _ai=ADM_Init();
    char *t = getenv("PASSWORD_PIPE");
    PRInt32 osfd;
    SYS_FILE fd;
    char *buf;
    int bufsize;
    int numread = 0;
    int totalread = 0;
    char *head, *tail;
    

    /* No error in this case, because it's expected to happen sometimes */
    if(!t)   {
        user = NULL;
        pass = NULL;
        auth = NULL;
        return 0;
    }

    osfd = atol(t);
    fd = PR_ImportFile(osfd);

    buf = (char *) MALLOC(BIG_LINE);
    bufsize = BIG_LINE;

    while(1)  {
#ifdef	XP_WIN32
        numread = SYSTEM_PREAD(fd, buf, bufsize);
#else	/* UNIX */
        numread = PR_Read(fd, buf, bufsize);
#endif	/* XP_WIN32 */
        totalread += numread;
        if(numread < 0)  {
            PR_Close(fd);
            /* MLM XXX - ERROR CODE */
            report_error(SYSTEM_ERROR, "Could not read from pipe",
                        "Could not read authentication information from pipe.");
            return -1;
        }  else  
        if(numread == 0)  {
            PR_Close(fd);
            break;
        }
        if(buf[numread - 1] == '\0')  {
            PR_Close(fd);
            break;
        }
    }
    buf[totalread] = '\0';

    /* Parse the buffer */
    head = buf;
    tail = strchr(head, '\n');
    *tail++ = '\0';
    if(!strncmp(head, ADM_USER_STRING, strlen(ADM_USER_STRING)))  {
        user = STRDUP(&(head[strlen(ADM_USER_STRING)]));
        if(!strcmp(user, ADM_NO_VALUE_STRING))  {
            user = NULL;
        }
    }
    head = tail;
    tail = strchr(head, '\n');
    *tail++ = '\0';
    if(!strncmp(head, ADM_PASS_STRING, strlen(ADM_PASS_STRING)))  {
        pass = STRDUP(&(head[strlen(ADM_PASS_STRING)]));
        if(!strcmp(pass, ADM_NO_VALUE_STRING))  {
            pass = NULL;
        }
    }
    head = tail;
    tail = strchr(head, '\n');
    *tail++ = '\0';
    if(!strncmp(head, ADM_AUTH_STRING, strlen(ADM_AUTH_STRING)))  {
        auth = STRDUP(&(head[strlen(ADM_AUTH_STRING)]));
        if(!strcmp(auth, ADM_NO_VALUE_STRING))  {
            auth = NULL;
        }
    }

#ifdef FORTEZZA
    head = tail;
    tail = strchr(head, '\n');
    *tail++ = '\0';
    if(!strncmp(head, ADM_PIN_STRING, strlen(ADM_PIN_STRING)))  {
        pin = STRDUP(&(head[strlen(ADM_PIN_STRING)]));
        if(!strcmp(pin, ADM_NO_VALUE_STRING))  {
            pin = NULL;
        }
    }
#endif /* FORTEZZA */

    head = tail;
    tail = strchr(head, '\n');
    *tail++ = '\0';
    if(!strncmp(head, ADM_USERDN_STRING, strlen(ADM_USERDN_STRING)))  {
        userDN = STRDUP(&(head[strlen(ADM_USERDN_STRING)]));
        if(!strcmp(userDN, ADM_NO_VALUE_STRING))  {
            userDN = NULL;
        }
    }

    return 0;
}

NSAPI_PUBLIC int ADM_GetCurrentUsername(int *errcode, char **username)
{
    int err;

    if((user) && (!strcmp(user, ADM_NOT_INITIALIZED)))  {
        if(ADM_InitializePermissions(&err))  {
            *errcode = err;
            return -1;
        }
    }
    *username = user;
    return 0;
}

NSAPI_PUBLIC int ADM_GetCurrentPassword(int *errcode, char **password)
{
    int err;

    if((pass) && (!strcmp(pass, ADM_NOT_INITIALIZED)))  {
        if(ADM_InitializePermissions(&err))  {
            *errcode = err;
            return -1;
        }
    }
    *password = pass;
    return 0;
}

NSAPI_PUBLIC int ADM_GetAuthorizationString(int *errcode, char **authstr)
{
    int err;

    if((auth) && (!strcmp(auth, ADM_NOT_INITIALIZED)))  {
        if(ADM_InitializePermissions(&err))  {
            *errcode = err;
            return -1;
        }
    }
    *authstr = auth;
    return 0;
}

NSAPI_PUBLIC int ADM_GetUserDNString(int *errcode, char **userdn)
{
    int err;

    if((userDN) && (!strcmp(userDN, ADM_NOT_INITIALIZED)))  {
        if(ADM_InitializePermissions(&err))  {
            *errcode = err;
            return -1;
        }
    }
    *userdn = userDN;
    return 0;
}

#ifdef FORTEZZA
NSAPI_PUBLIC int ADM_GetPinString(int *errcode, char **pinstr)
{
    int err;

    if((pin) && (!strcmp(pin, ADM_NOT_INITIALIZED)))  {
        if(ADM_InitializePermissions(&err))  {
            *errcode = err;
            return -1;
        }
    }
    *pinstr = pin;
    return 0;
}
#endif /* FORTEZZA */
