/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * form_get.c - Functions to help with handling GET requests
 *
 * All blame to Mike McCool
 */

#include "libadmin/libadmin.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <i18n.h>

#define HTML_DIR "../html/"

NSAPI_PUBLIC FILE *open_error_file(char *filename)
{
    FILE *f = NULL;
    char line[BIG_LINE];

    sprintf(line, "%s%s", HTML_DIR, filename);

    if(!(f = fopen(line, "r")))  {
        fprintf(stdout, "\n<h1 align=center>Catastrophic error</h1>\n");
        fprintf(stdout, "Cannot open any administrative HTML files (errno "
                        "is %d).\n", errno);
        exit(0);
    }
    return f;
}

NSAPI_PUBLIC FILE *open_html_file(char *filename)
{
    FILE *f = NULL;
    char line[BIG_LINE];

    sprintf(line, "%s%s", HTML_DIR, filename);
    if(!(f = fopen(line, "r")))  {
        report_error(FILE_ERROR, line, "Could not open the HTML file.  "
                     "Perhaps the permissions have changed or someone "
                     "has moved the file.");
    }
    return f;
}

/*********************************************************************
  Open an html file from the specified language subdirectory, if 
  available, else from the default language subdirectory.
 *********************************************************************/

NSAPI_PUBLIC
FILE*
open_html_file_lang(char* filename,char* language)
{
    FILE *f = NULL;
    char filePattern[BIG_LINE];
    char line[BIG_LINE];
  
    sprintf(filePattern, "%s%s%s", HTML_DIR, "$$LANGDIR/", filename);
  
    GetFileForLanguage(filePattern,language,line);
    if(!(f = fopen(line, "r")))  {
        report_error(FILE_ERROR, line, "Could not open the HTML file.  "
                     "Perhaps the permissions have changed or someone "
                     "has moved the file.");
    }
    return f;
}

/*******************************************************************************/

void _do_return_html_file(char *filename, int doref)  {
#ifndef MCC_ADMSERV
    char **config = get_adm_config();
#else
    char **config = get_mult_adm_config(-1);
#endif
    char line[BIG_LINE];
    FILE *html = open_html_file(filename);

    while(next_html_line(html, line))  {
    /* If we don't recognize it, just spit it out anyway. */
        if(parse_line(line, config))
            fputs(line, stdout);
    }
    if(doref)
        set_referer(config);
}

NSAPI_PUBLIC void return_html_file(char *filename)  {
    _do_return_html_file(filename, 1);
}

NSAPI_PUBLIC void return_html_noref(char *filename)  {
    _do_return_html_file(filename, 0);
}

NSAPI_PUBLIC int next_html_line(FILE *f, char *line)
{
    if(!(fgets(line, BIG_LINE, f)))  {
#ifdef XP_UNIX
        fclose(f);
        return 0;
#else /* XP_WIN32 */
		if (feof(f)) {
        	fclose(f);
        	return 0;
		}
#endif /* XP_WIN32 */
    }
    else
        return 1;
}

NSAPI_PUBLIC void output_input(char *type, char *name, char *value, char *other)
{
    char * cp;
    char * dp;
    int qcnt = 0;

    /* Do a backslash quote on any double quotes in the value string */
    if (value) {

	/* Count the double-quotes */
	cp = value;
	while ((cp = strpbrk(cp, "\"")) != 0) {
	    ++cp;
	    ++qcnt;
	}

	if (qcnt > 0) {
	    /* Allocate buffer for result string */
	    dp = (char *)MALLOC(strlen(value) + (qcnt * 5) + 1);
	    cp = value;
	    value = dp;
	    while (*cp) {
		if (*cp == '"') {
		    strcpy(dp, "&quot;");
		    dp += 6;
		    cp += 1;
		}
		else {
		    *dp++ = *cp++;
		}
	    }
	    *dp = 0;
	}
    }
	
    fprintf(stdout, "<INPUT type=\"%s\" name=\"%s\" value=\"%s\" %s>%s",
                     type, name, (value) ? value : "", (other) ? other : "",
                     ((strcmp(type, "radio")) && (strcmp(type, "checkbox"))) ? 
                       "\n" : "");

    if (value && (qcnt > 0)) {
	FREE(value);
    }
}


#ifdef MCC_PROXY
/* This crap doesn't belong here, but I couldn't think of any better place,
 * either.  Basically, this crap *shouldn't* be *anywhere*, we should be
 * able to do without it.  But, we can't.  Some linkers disagree.
 */

#include "base/session.h"
#include "frame/req.h"
#include "frame/func.h"

#include <pwd.h>

struct passwd *userpw = NULL;
#if !defined(IRIX) && !defined(SOLARIS)
char *chr = NULL;
#endif
int ProxyTraceFlag = 0;
int sockd_service()       { return REQ_ABORTED; }
int record_proxy_access() { return REQ_ABORTED; }
int sockd_read_request()  { return REQ_ABORTED; }
int http_parse_request(char *t, Request *rq, Session *sn)
{
    return REQ_ABORTED;
}

struct FuncStruct func_standard[] = {
  {NULL, NULL, NULL}
};

pid_t bu_pid = (pid_t)-1;
bu_terminate() {}
bu_start() {}


/*
 * Turn seconds into a human readable string.
 * This function intentionally only shows at most *two* different
 * units.  This is to keep it short.  The values elsewhere are selected
 * so that this doesn't pose a problem.
 *
 */
char *human_time_str(long t)
{
    char buf[50];
    int s = t % 60;
    int m = (t / 60) % 60;
    int h = (t / 3600) % 24;
    int d = t / (3600 * 24) % 7;
    int w = t / (3600 * 24 * 7);

    if (t < 48 * 3600) {
	h += 24 * d;
	d = 0;
    }

    if (w==1) {
	d += 7;
	w = 0;
    }

    if (t < 60) {
	sprintf(buf, "%d sec", s);
    }
    else if (t < 3600) {
	if (s)
	    sprintf(buf, "%d min %d sec", m, s);
	else
	    sprintf(buf, "%d min", m);
    }
    else if (t < 48 * 3600) {
	if (m)
	    sprintf(buf, "%d hour%s %d min", h, (h>1 ? "s" : ""), m);
	else
	    sprintf(buf, "%d hour%s", h, (h>1 ? "s" : ""));
    }
    else if (t < 14 * 24 * 3600) {
	if (h)
	    sprintf(buf, "%d day%s %d hour%s", d, (d>1 ? "s" : ""), h, (h>1 ? "s" : ""));
	else
	    sprintf(buf, "%d day%s", d, (d>1 ? "s" : ""));
    }
    else {
	if (d)
	    sprintf(buf, "%d week%s %d day%s", w, (w>1 ? "s" : ""), d, (d>1 ? "s" : ""));
	else
	    sprintf(buf, "%d week%s", w, (w>1 ? "s" : ""));
    }
    return STRDUP(buf);
}


long cache_size_tbl[] = {
       10L,    15L,    20L,    25L,    50L,    75L,
      100L,   150L,   200L,   300L,   400L,   500L,
      600L,   700L,   800L,   900L,  1000L,  1200L,
     1400L,  1600L,  1800L,  2000L,  2500L,  3000L,
     3500L,  4000L,  4500L,  5000L,  5500L,  6000L,
     7000L,  8000L,  9000L, 10000L, 11000L, 12000L,
    14000L, 16000L, 20000L, 24000L, 28000L, 32000L,
        0L
};

long inst_cache_size_tbl[] = {
       10L,    25L,    50L,    75L,
      100L,   200L,   300L,   400L,   500L,
      600L,   700L,   800L,   900L,  1000L,  1200L,
     1400L,  1600L,  1800L,  2000L,
        0L
};

long cache_capacity_tbl[] = {
      100L,	/*   1 dir  ==	100 MB	*/
      250L, 	/*   2 dirs ==	250 MB	*/
      500L,	/*   4 dirs ==	500 MB	*/
     1000L,	/*   8 dirs ==	  1 GB	*/
     2000L,	/*  16 dirs ==	  2 GB	*/
     4000L,	/*  32 dirs ==	  4 GB	*/
     8000L,	/*  64 dirs ==	  8 GB	*/
    16000L,	/* 128 dirs ==	 16 GB	*/
    32000L,	/* 256 dirs ==	 32 GB	*/
        0L
};

long inst_cache_capacity_tbl[] = {
      100L,	/*   1 dir  ==	100 MB	*/
      250L, 	/*   2 dirs ==	250 MB	*/
      500L,	/*   4 dirs ==	500 MB	*/
     1000L,	/*   8 dirs ==	  1 GB	*/
     2000L,	/*  16 dirs ==	  2 GB	*/
        0L
};

float lm_factor_tbl[] = {
    0.02, 0.04, 0.06, 0.08,
    0.1,  0.2,  0.3,  0.4,  0.5,
    0.6,  0.7,  0.8,  0.9,  1.0,
    -1.0
};

long time_interval_tbl[] = {
    /* 5 min	*/         300L,
    /* 15 min	*/         900L,
    /* 30 min	*/        1800L,
    /* 45 min	*/        2700L,
    /* 60 min	*/        3600L,
    /* 2 hours	*/    2 * 3600L,
    /* 4 hours	*/    4 * 3600L,
    /* 6 hours	*/    6 * 3600L,
    /* 8 hours	*/    8 * 3600L,
    /* 12 hours	*/   12 * 3600L,
    /* 24 hours	*/   24 * 3600L,
    /* 2 days	*/   48 * 3600L,
    /* 3 days	*/   72 * 3600L,
    /* 4 days	*/   96 * 3600L,
    /* 5 days	*/  120 * 3600L,
    /* 6 days	*/  144 * 3600L,
    /* 7 days	*/  168 * 3600L,
    /* 10 days	*/  240 * 3600L,
    /* 14 days	*/  336 * 3600L,
			     0L
};

long timeout_tbl[] = {
    /*  5 sec */	   5L,
    /* 10 sec */	  10L,
    /* 15 sec */	  15L,
    /* 20 sec */	  20L,
    /* 25 sec */	  25L,
    /* 30 sec */	  30L,
    /* 40 sec */	  40L,
    /* 50 sec */	  50L,
    /* 60 sec */	  60L,
    /* 90 sec */	  90L,
    /*  2 min */	 120L,
    /*  3 min */	 180L,
    /*  4 min */	 240L,
    /*  5 min */	 300L,
    /*  6 min */	 360L,
    /*  8 min */	 480L,
    /* 10 min */	 600L,
    /* 12 min */	 720L,
    /* 15 min */	 900L,
    /* 20 min */	1200L,
    /* 25 min */	1500L,
    /* 30 min */	1800L,
    /* 45 min */	2700L,
    /* 60 min */	3600L,
			   0L
};

int percent_tbl[] = {
    0, 5, 10, 15, 20, 25, 35, 50, 65, 75, 80, 85, 90, 95, 100,
    0
};


char *mb_str(long mb)
{
    char buf[50];

    if (mb < 1000) {
	sprintf(buf, "%ld MB", mb);
    }
    else {
	int n = mb / 1000;
	int d = (mb % 1000) / 100;

	if (n < 6 && d) {
	    sprintf(buf, "%d.%d GB", n, d);
	}
	else {
	    sprintf(buf, "%d GB", n);
	}
    }
    return STRDUP(buf);
}

void output_interval_select(char *name, char *other, long selected, long *tbl)
{
    int i, found = 0;

    printf("<SELECT NAME=%s %s>\n", name, other ? other : "");

    for(i=0; tbl[i]; i++) {
	if (!found && tbl[i] > selected) {
	    found = 1;
	    printf("<OPTION VALUE=\"%ld\" SELECTED>%s\n", selected, human_time_str(selected));
	}
	printf("<OPTION VALUE=\"%ld\"%s>%s\n",
	       tbl[i], tbl[i]==selected ? " SELECTED" : "", human_time_str(tbl[i]));
	if (!found && tbl[i]==selected) {
	    found = 1;
	}
    }
    if (!found)
	printf("<OPTION VALUE=\"%ld\" SELECTED>%s\n", selected, human_time_str(selected));
    printf("</SELECT>");
}


void output_mb_select(char *name, char *other, long selected, long *tbl)
{
    int i, found = 0;

    printf("<SELECT NAME=%s %s>\n", name, other ? other : "");

    for(i=0; tbl[i]; i++) {
	if (!found && tbl[i] > selected) {
	    found = 1;
	    printf("<OPTION VALUE=\"%ld\" SELECTED>%s\n", selected, mb_str(selected));
	}
	printf("<OPTION VALUE=\"%ld\"%s>%s\n",
	       tbl[i], tbl[i]==selected ? " SELECTED" : "", mb_str(tbl[i]));
	if (!found && tbl[i]==selected) {
	    found = 1;
	}
    }
    if (!found)
	printf("<OPTION VALUE=\"%ld\" SELECTED>%s\n", selected, mb_str(selected));
    printf("</SELECT>");
}



char *lm_str(float f)
{
    char buf[50];

    if (f < 0.1)
	sprintf(buf, "%.2f", f);
    else
	sprintf(buf, "%.1f", f);

    return STRDUP(buf);
}


#define LM_ACCURACY 0.00001

void output_lm_select(char *name, char *other, float selected, float *tbl)
{
    int i, found = 0;

    printf("<SELECT NAME=%s %s>\n", name, other ? other : "");

    for(i=0; tbl[i] > 0; i++) {
	if (!found && tbl[i] - LM_ACCURACY > selected) {
	    found = 1;
	    printf("<OPTION VALUE=\"%.3f\">%s\n", selected, lm_str(selected));
	}
	if (!found &&
	    tbl[i] - LM_ACCURACY <= selected &&
	    tbl[i] + LM_ACCURACY >= selected) {
	    found = 1;
	    printf("<OPTION VALUE=\"%.3f\" SELECTED>%s\n", tbl[i], lm_str(tbl[i]));
	}
	else {
	    printf("<OPTION VALUE=\"%.3f\">%s\n", tbl[i], lm_str(tbl[i]));
	}
    }
    if (!found)
	printf("<OPTION VALUE=\"%.3f\">%s\n", selected, lm_str(selected));
    printf("</SELECT>");
}


void output_percentage_select(char *name, char *other, int selected, int *tbl)
{
    int i, found = 0;

    printf("<SELECT NAME=%s %s>\n", name, other ? other : "");

    for(i=0; !i || tbl[i]; i++) {
	if (!found && tbl[i] > selected) {
	    found = 1;
	    printf("<OPTION VALUE=\"%d\" SELECTED>%d %%\n", selected, selected);
	}
	printf("<OPTION VALUE=\"%d\"%s>%d %%\n",
	       tbl[i], tbl[i]==selected ? " SELECTED" : "", tbl[i]);
	if (!found && tbl[i]==selected) {
	    found = 1;
	}
    }
    if (!found)
	printf("<OPTION VALUE=\"%d\" SELECTED>%d %%\n", selected, selected);
    printf("</SELECT>");
}


#endif	/* MCC_PROXY */

