/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * form_post.c - Functions to help with handling POSTs
 *
 * All blame to Mike McCool
 */

#include "libadmin/libadmin.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "base/shexp.h" /* wildcard routines */

#define SUCCESS_HTML "success.html"

static char **input;

NSAPI_PUBLIC char **get_input_ptr(void)
{
    return input;
}

NSAPI_PUBLIC void form_unescape(char *str) 
{
    register int x = 0, y = 0;
    int l = strlen(str);
    char digit;

    while(x < l)  {
        if((str[x] == '%') && (x < (l - 2)))  {
            ++x;
            digit = (str[x] >= 'A' ? 
                         ((str[x] & 0xdf) - 'A')+10 : (str[x] - '0'));
            digit *= 16;

            ++x;
            digit += (str[x] >= 'A' ? 
                         ((str[x] & 0xdf) - 'A')+10 : (str[x] - '0'));

            str[y] = digit;
        } 
        else if(str[x] == '+')  {
            str[y] = ' ';
        } else {
            str[y] = str[x];
        }
        x++;
        y++;
    }
    str[y] = '\0';
}

NSAPI_PUBLIC void post_begin(FILE *in) 
{
    char *vars = NULL, *tmp = NULL;
    int cl;

    if(!(tmp = getenv("CONTENT_LENGTH")))
        report_error(INCORRECT_USAGE, "Browser Error", 
                     "Your browser sent no content length with a POST command."
                     "  Please be sure to use a fully compliant browser.");
        
    cl = atoi(tmp);

    vars = (char *)MALLOC(cl+1);

    if( !(fread(vars, 1, cl, in)) )
        report_error(SYSTEM_ERROR, NULL, 
                     "The POST variables could not be read from stdin.");

    vars[cl] = '\0';

    input = string_to_vec(vars);
}

NSAPI_PUBLIC void get_begin(char *qs)
{
    input = string_to_vec(qs);
}

NSAPI_PUBLIC char **string_to_vec(char *in)
{
    char **ans;
    int vars = 0;
    register int x = 0;
    char *tmp;

    in = STRDUP(in);

    while(in[x])
        if(in[x++]=='=')
            vars++;
    
    ans = new_strlist(vars+1);
  
    x=0;
    tmp = strtok(in, "&");
    ans[x]=STRDUP(tmp);
    form_unescape(ans[x++]);

    while((tmp = strtok(NULL, "&")))  {
        ans[x] = STRDUP(tmp);
        form_unescape(ans[x++]);
    }

    FREE(in);

    return(ans);
}

NSAPI_PUBLIC char *get_cgi_var(char *varname, char *elem_id, char *bongmsg)
{
    register int x = 0;
    int len = strlen(varname);
    char *ans = NULL;
   
    while(input[x])  {
    /*  We want to get rid of the =, so len, len+1 */
        if((!strncmp(input[x], varname, len)) && (*(input[x]+len) == '='))  {
            ans = STRDUP(input[x] + len + 1);
            if(!strcmp(ans, ""))
                ans = NULL;
            break;
        }  else
            x++;
    }
    if(ans == NULL)  {
        if ((bongmsg) && strlen(bongmsg))
            report_error(ELEM_MISSING, elem_id, bongmsg);
        else
            return NULL;
    }
    else
        return(ans);
    /* shut up gcc */
    return NULL;
}

NSAPI_PUBLIC long get_cgi_long(char *varname)
{
    char *ans = get_cgi_var(varname, NULL, NULL);
 
    return ans ? atol(ans) : -1L;
}

NSAPI_PUBLIC int get_cgi_int(char *varname)
{
    return (int)get_cgi_long(varname);
}

NSAPI_PUBLIC int get_cgi_bool(char *varname)
{
    char *ans = get_cgi_var(varname, NULL, NULL);
 
    return (ans && *ans && (*ans - '0')) ? 1 : 0;
}

NSAPI_PUBLIC char **get_cgi_multiple(char *varname, char *elem_id, char *bongmsg)
{
    register int n, x;
    int len = strlen(varname);
    char **ans = NULL;

    for(n=0; input[n]; n++);
    ans = new_strlist(n + 1);

    for(x=n=0; input[x]; x++)  {
        if ((!strncmp(input[x], varname, len)) &&
	    (*(input[x]+len) == '=') &&
	    (*(input[x]+len+1))) {
            ans[n] = STRDUP(input[x] + len + 1);
	    n++;
        }
    }
    ans[n] = NULL;

    if (!n)  {
	FREE(ans);
        if(bongmsg)
            report_error(ELEM_MISSING, elem_id, bongmsg);
        else
            return NULL;
    }
    else {
        return ans;
    }
    /* shut up gcc */
    return NULL;
}


NSAPI_PUBLIC void return_success(char *description)  {
    char **config=get_adm_config();
    WSACleanup();

    fprintf(stdout, "\n<SCRIPT language=%s>", MOCHA_NAME);
#ifdef MCC_HTTPD /* template->style usability */
    /* If we're working with styles, jump directly back to the edit style
     * page. */
    if(!strcmp(get_current_typestr(config), TEMPLATE) ||
       ((get_current_restype(config)==PB_NAME) && 
        (strcmp(get_current_resource(config), "default"))))  {
        fprintf(stdout, "top.%s.%s.location='%s';", 
                BOTTOM_NAME, CONTENT_NAME, "index/tmpled");
        fprintf(stdout, "</SCRIPT>\n");
    }  else
#endif
#ifndef NS_MAIL
#ifndef MCC_ADMSERV
    if(needs_commit(get_adm_config())>0)  {
        fprintf(stdout, "top.%s.%s.location='%s';", 
                BOTTOM_NAME, CONTENT_NAME, "commit");
        fprintf(stdout, "</SCRIPT>\n");
    }  else  {
#endif
#endif
        fprintf(stdout, "alert('Success!\\n%s');", 
                        alert_word_wrap(description, WORD_WRAP_WIDTH, "\\n"));
        fprintf(stdout, "</SCRIPT>\n");
        js_open_referer();
#ifndef NS_MAIL
#ifndef MCC_ADMSERV
    }
#endif
#else
    char line[BIG_LINE];
    char **config = get_adm_config();
    FILE *html=open_html_file(SUCCESS_HTML);

    WSACleanup();

    /* Be sure headers are terminated. */
    fputs("\n", stdout);

    while(next_html_line(html, line))  {
        if(parse_line(line, config))  {
            if(directive_is(line, "SUCCESS_DESCRIPTION"))  {
                fprintf(stdout, "<dl><dt><FONT size=+1>Success!</FONT>\n");
                fprintf(stdout, "<dd>%s\n</dl>", description);
            }
        }
    }
#endif
    exit(0);
}

NSAPI_PUBLIC void rm_trail_slash(char *target)
{
    int l;
    
    if(!target) return;

    l = strlen(target) - 1;

    if((target[l] == '/') && (l != 0))
        target[l] = '\0';
}

/* Compresses spaces only. */
NSAPI_PUBLIC void compress_spaces(char *source)  {
    register int x, y;

    if(!source) return;

    for(x=0, y=0; source[x]; x++)  {
        if(source[x] != ' ')
             source[y++] = source[x];
    }
    source[y] = '\0';
}

/* Compresses spaces and replaces commas with an or list */
NSAPI_PUBLIC char * compress_and_replace(char *source)  {
    char *answer;
    int hascomma, x, y;

    if(!source) return source;

    hascomma = (strchr(source, ',')) ? 1 : 0;

    /* + 2 = 1 for trailing \0 and one put in by Aruna for NT */
    answer = (char *)MALLOC(strlen(source) + 2);

    for(x=0, y=0; source[x]; x++)  {
        if(source[x] == ',')
             answer[y++] = '|';
        else if(source[x] != ' ')
             answer[y++] = source[x];
        answer[y] = '\0';
    }
    if(!hascomma)
        return(answer);
    else {
        char *newanswer = (char *)MALLOC(strlen(answer)+2);
        sprintf(newanswer, "(%s)", answer);
        return(newanswer);
    }
}

NSAPI_PUBLIC void negate_wildcard(char **string)
{
    if(*string == NULL)
        return;

    if(((*string)[0] == '*') && ((*string)[1] == '~'))  {
        (*string) += 2;
    }  else  {
        char *t = (char *)MALLOC(strlen(*string) + 3);
        sprintf(t, "*~%s", (*string));
        *string = t;
    }
}

NSAPI_PUBLIC void validate_wildcard(char *wildcard)  
{
    char line[BIG_LINE];

    if(!wildcard) 
        return;
    if(shexp_valid(wildcard) != INVALID_SXP)
        return;

    sprintf(line, "The wildcard pattern <b>%s</b> is not a valid wildcard "
                  "pattern.  <p>\n "
                  "Also, do not mix comma separated lists and wildcard "
                  "patterns - you can use one or the other, but not both at "
                  "once.\n", wildcard);
                  
    report_error(INCORRECT_USAGE, "Bad wildcard pattern", line);
}

#ifdef USE_REGEX
NSAPI_PUBLIC void validate_regex(char *exp)
{
    char line[BIG_LINE];

    if(!exp) 
        return;
    if(regexp_valid(exp) != INVALID_REGEXP)
        return;

    sprintf(line, "The pattern %s is not a valid regular expression.\n", exp);

    report_error(INCORRECT_USAGE, "Bad regular expression", line);
}
#endif

NSAPI_PUBLIC char *fix_hn_exp(char *exp)  {
    char *answer;

    if(!exp)
        return NULL;

    if((strchr(exp, ' ')) && (!strchr(exp, ',')))
        report_error(INCORRECT_USAGE, "Bad wildcard pattern", 
                     "You should use commas to separate hosts, not spaces.");

    if((exp[0] == '*') && (exp[1] == '~'))  {
        negate_wildcard(&exp);
        answer = compress_and_replace(exp);
    }  else  {
        answer = compress_and_replace(exp);
        negate_wildcard(&answer);
    }
    validate_wildcard(answer);
 
    return answer;
}

