/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * hinstall.c: Routines shared among httpd installation programs
 * 
 * Rob McCool
 */


#include "libadmin/libadmin.h"
#include "libadmin/hinstall.h"

#include "base/systems.h"      /* ZERO */
#include "base/file.h"         /* FILE_PATHSEP */
#include "base/util.h"         /* sprintf */

#include <string.h>


#ifdef XP_WIN32
#define EACCES WSAEACCES
#define EADDRINUSE WSAEADDRINUSE
#endif /* XP_WIN32 */

/* ------------------------------ servconf_* ------------------------------ */


struct servconf *servconf_create(void)
{
    struct servconf *sc = (struct servconf *) MALLOC(sizeof(struct servconf));

    ZERO(sc, sizeof(struct servconf));
    return(sc);
}



void servconf_write(struct servconf *sc, char *filename)
{
    char fn[1024], err[sizeof(fn) + 50];
    FILE *f;

    sprintf(fn, "%s%c%s", CONFIG_DIRECTORY, FILE_PATHSEP, 
            filename ? filename : DEFAULT_SERVCONF);
    if(!(f = fopen_l(fn, "w"))) {
        sprintf(err, "Can't write to file %s", fn);
        report_error(FILE_ERROR, system_errmsg(), err);
    }
    fprintf(f, "ServerName %s\n", sc->servname);
    fprintf(f, "Port %s\n", sc->servport);
    if(sc->servaddr)
        fprintf(f, "BindAddr %s\n", sc->servaddr);
    fprintf(f, "ServerRoot %s\n", sc->servroot);
    fprintf(f, "ServerID %s\n", sc->servid);
#ifdef XP_UNIX
    fprintf(f, "User %s\n", sc->servuser);
#else /* XP_WIN32 */
	if(sc->servuser)
    	fprintf(f, "User %s\n", sc->servuser);
	if(sc->password)
    	fprintf(f, "Password %s\n", sc->password);
#endif /* XP_WIN32 */
    fprintf(f, "MinProcs %s\n", sc->minprocs);
    fprintf(f, "MaxProcs %s\n", sc->maxprocs);
    fprintf(f, "Syslog %s\n", sc->slog);
    fprintf(f, "DNS %s\n", sc->dns);
    fprintf(f, "CLF %s\n", sc->clf);

#ifdef MCC_PROXY
    fprintf(f, "ExtLogFmt %s\n", sc->ext);
#endif

    fclose_l(f);
}

struct servconf *servconf_scan(char *filename)
{
    struct servconf *sc = servconf_create();
    char fn[1024], err[sizeof(fn) + 50], **lines, *t;
    int x;
    sprintf(fn, "%s%c%s", CONFIG_DIRECTORY, FILE_PATHSEP, 
            filename ? filename : DEFAULT_SERVCONF);

    if(!(lines = scan_tech(fn))) {
        sprintf(err, "Can't open file %s", fn);
        report_error(FILE_ERROR, system_errmsg(), err);
    }
    sc->lines = lines;
    for(x = 0; lines[x]; x++) {
        if( (t = strchr(lines[x], ' ')) ) {
            *t++ = '\0';
            if(!strcasecmp(lines[x], "servername"))
                sc->servname = t;
            if(!strcasecmp(lines[x], "serverid"))
                sc->servid = t;
            else if(!strcasecmp(lines[x], "port"))
                sc->servport = t;
            else if(!strcasecmp(lines[x], "bindaddr"))
                sc->servaddr = t;
            else if(!strcasecmp(lines[x], "serverroot"))
                sc->servroot = t;
            else if(!strcasecmp(lines[x], "user"))
                sc->servuser = t;
#ifdef XP_WIN32
            else if(!strcasecmp(lines[x], "password"))
                sc->password = t;
#endif /* XP_WIN32 */
            else if(!strcasecmp(lines[x], "minprocs"))
                sc->minprocs = t;
            else if(!strcasecmp(lines[x], "maxprocs"))
                sc->maxprocs = t;
            else if(!strcasecmp(lines[x], "dns"))
                sc->dns = t;
            else if(!strcasecmp(lines[x], "syslog"))
                sc->slog = t;
            else if(!strcasecmp(lines[x], "clf"))
                sc->clf = t;
#ifdef MCC_PROXY
	    else if(!strcasecmp(lines[x], "extlogfmt"))
		sc->ext = t;
#endif
        }
    }
#if 0   /* No longer used, naming is explicit */
    if(sc->servaddr) {
        sc->servid = (char *) MALLOC(strlen(sc->servport) + 1 + 
                                     strlen(sc->servaddr) + 1 + 1);
        sprintf(sc->servid, "%s.%s", sc->servport, sc->servaddr);
    }
    else
        sc->servid = sc->servport;
#endif
    return sc;
}


/* ------------------------------ docsconf_* ------------------------------ */

#ifndef MCC_PROXY

struct docsconf *docsconf_create(void)
{
    struct docsconf *dc = (struct docsconf *) MALLOC(sizeof(struct docsconf));

    ZERO(dc, sizeof(struct docsconf));
    return(dc);
}


void docsconf_write(struct docsconf *dc, char *filename)
{
    char fn[1024], err[sizeof(fn) + 50];
    FILE *f;

    sprintf(fn, "%s%c%s", CONFIG_DIRECTORY, FILE_PATHSEP, 
            filename ? filename : DEFAULT_DOCSCONF);
    if(!(f = fopen_l(fn, "w"))) {
        sprintf(err, "Can't write to file %s", fn);
        report_error(FILE_ERROR, system_errmsg(), err);
    }
    fprintf(f, "DocumentRoot %s\n", dc->docroot);
    fprintf(f, "IdxFiles %s\n", dc->idxfiles);
    fprintf(f, "FancyIndex %s\n", dc->fancyidx);
    fprintf(f, "HomePage %s\n", dc->homepage ? dc->homepage : "");

    fclose_l(f);
}


struct docsconf *docsconf_scan(char *filename)
{
    struct docsconf *dc = docsconf_create();
    char fn[1024], err[sizeof(fn) + 50], **lines, *t;
    int x;

    sprintf(fn, "%s%c%s", CONFIG_DIRECTORY, FILE_PATHSEP, 
            filename ? filename : DEFAULT_DOCSCONF);

    if(!(lines = scan_tech(fn))) {
        sprintf(err, "Can't open file %s", fn);
        report_error(FILE_ERROR, system_errmsg(), err);
    }
    dc->lines = lines;
    for(x = 0; lines[x]; x++) {
        if( (t = strchr(lines[x], ' ')) ) {
            *t++ = '\0';
            if(!strcasecmp(lines[x], "documentroot"))
                dc->docroot = t;
            else if(!strcasecmp(lines[x], "idxfiles"))
                dc->idxfiles = t;
            else if(!strcasecmp(lines[x], "fancyindex"))
                dc->fancyidx = t;
            else if(!strcasecmp(lines[x], "homepage"))
                dc->homepage = t;
        }
    }
    return dc;
}


#else	/* MCC_PROXY */


struct docsconf *docsconf_create(void)
{
    struct docsconf *pc = (struct docsconf *) MALLOC(sizeof(struct docsconf));

    ZERO(pc, sizeof(struct docsconf));
    return(pc);
}


void docsconf_write(struct docsconf *pc, char *filename)
{
    char fn[1024], err[sizeof(fn) + 50];
    FILE *f;

    sprintf(fn, "%s%c%s", CONFIG_DIRECTORY, FILE_PATHSEP, 
            filename ? filename : DEFAULT_PROXYCONF);
    if(!(f = fopen_l(fn, "w"))) {
        sprintf(err, "Can't write to file %s", fn);
        report_error(FILE_ERROR, system_errmsg(), err);
    }

#define NONEMPTY(str) ((str) ? (str) : "")

    fprintf(f, "ProxyHTTP %s\n",	  BOOL2STR(pc->proxy_http));
    fprintf(f, "ProxyFTP %s\n",		  BOOL2STR(pc->proxy_ftp));
    fprintf(f, "ProxyGopher %s\n",	  BOOL2STR(pc->proxy_gopher));
    fprintf(f, "Proxy-HTTPS %s\n",	  BOOL2STR(pc->proxy_https));
    fprintf(f, "TunnelSSL-HTTPS %s\n",	  BOOL2STR(pc->tunnel_ssl_https));
    fprintf(f, "TunnelSSL-SNEWS %s\n",	  BOOL2STR(pc->tunnel_ssl_snews));
    fprintf(f, "Caching %s\n",		  BOOL2STR(pc->caching));
    fprintf(f, "CacheRoot %s\n",	  NONEMPTY(pc->cache_root));
    fprintf(f, "CacheSize %ld\n",	  pc->cache_size);
    fprintf(f, "CacheCapacity %ld\n",	  pc->cache_capacity);
    fprintf(f, "CacheHTTP %s\n",	  BOOL2STR(pc->cache_http));
    fprintf(f, "CacheFTP %s\n",		  BOOL2STR(pc->cache_ftp));
    fprintf(f, "CacheGopher %s\n",	  BOOL2STR(pc->cache_gopher));
    fprintf(f, "IMSAlways %s\n",	  BOOL2STR(pc->cache_ims_always));	/* vs expired */
    fprintf(f, "MaxUncheck-HTTP %ld\n",	  pc->cache_max_uncheck_http);
    fprintf(f, "MaxUncheck-FTP %ld\n",	  pc->cache_max_uncheck_ftp);
    fprintf(f, "MaxUncheck-Gopher %ld\n", pc->cache_max_uncheck_gopher);
    fprintf(f, "UseLM %s\n",		  BOOL2STR(pc->cache_use_lm));		/* vs explicit */
    fprintf(f, "LMFactor %f\n",		  pc->cache_lm_factor);

    fclose_l(f);
}


struct docsconf *docsconf_scan(char *filename)
{
    struct docsconf *pc = docsconf_create();
    char fn[1024], err[sizeof(fn) + 50], **lines, *t;
    int x;

    sprintf(fn, "%s%c%s", CONFIG_DIRECTORY, FILE_PATHSEP, 
            filename ? filename : DEFAULT_PROXYCONF);

    if(!(lines = scan_tech(fn))) {
        sprintf(err, "Can't open file %s", fn);
        report_error(FILE_ERROR, system_errmsg(), err);
    }
    pc->lines = lines;
    for(x = 0; lines[x]; x++) {
        if( (t = strchr(lines[x], ' ')) ) {
            *t++ = '\0';
            if(!strcasecmp(lines[x], "proxyhttp"))
                pc->proxy_http = STR2BOOL(t);
            else if(!strcasecmp(lines[x], "proxyftp"))
                pc->proxy_ftp = STR2BOOL(t);
            else if(!strcasecmp(lines[x], "proxygopher"))
                pc->proxy_gopher = STR2BOOL(t);
            else if(!strcasecmp(lines[x], "proxy-https"))
                pc->proxy_https = STR2BOOL(t);
            else if(!strcasecmp(lines[x], "tunnelssl-https"))
                pc->tunnel_ssl_https = STR2BOOL(t);
            else if(!strcasecmp(lines[x], "tunnelssl-snews"))
                pc->tunnel_ssl_snews = STR2BOOL(t);
            else if(!strcasecmp(lines[x], "caching"))
                pc->caching = STR2BOOL(t);
            else if(!strcasecmp(lines[x], "cacheroot"))
                pc->cache_root = t;
            else if(!strcasecmp(lines[x], "cachesize"))
                pc->cache_size = atol(t);
            else if(!strcasecmp(lines[x], "cachecapacity")) {
		int ndirs = 1;
		int dim = 0;
		long x = pc->cache_capacity = atol(t);

		while (x > 150L && ndirs < 256) {
		    dim++;
		    ndirs *= 2;
		    x /= 2;
		}
		pc->cache_ndirs = ndirs;
		pc->cache_dim = dim;
	    }
            else if(!strcasecmp(lines[x], "cachehttp"))
                pc->cache_http = STR2BOOL(t);
            else if(!strcasecmp(lines[x], "cacheftp"))
                pc->cache_ftp = STR2BOOL(t);
            else if(!strcasecmp(lines[x], "cachegopher"))
                pc->cache_gopher = STR2BOOL(t);
            else if(!strcasecmp(lines[x], "imsalways"))
                pc->cache_ims_always = STR2BOOL(t);
            else if(!strcasecmp(lines[x], "maxuncheck-http"))
                pc->cache_max_uncheck_http = atol(t);
            else if(!strcasecmp(lines[x], "maxuncheck-ftp"))
                pc->cache_max_uncheck_ftp = atol(t);
            else if(!strcasecmp(lines[x], "maxuncheck-gopher"))
                pc->cache_max_uncheck_gopher = atol(t);
            else if(!strcasecmp(lines[x], "uselm"))
                pc->cache_use_lm = STR2BOOL(t);
            else if(!strcasecmp(lines[x], "lmfactor"))
                pc->cache_lm_factor = atof(t);
        }
    }
    return pc;
}

#endif /* MCC_PROXY */

