/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * keyconf.c - Read in the keycert.lst file, get info from it, and modify it
 *
 * 12/04/96  dlau   Initial created.
 */

#include "libadminutil/admutil.h"
#include "libadmin/libadmin.h"
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

static char ***certconfarray = NULL;
static char **certconf = NULL;

/*************************************************************************************/

NSAPI_PUBLIC char *get_alias_dir(void) 
{
    char *cftmp = getenv("NETSITE_ROOT");
    char *cfdir;
    char *dir;

    dir = (char *)MALLOC(PATH_MAX);

    sprintf(dir, "%s%calias", cftmp, FILE_PATHSEP);
    
    CONVERT_TO_HTTP_FORMAT(dir);
    return dir;
}

/* Makes a list of the key pair files and certificate files installed on your machine. */
void read_alias_files(char ***aliasfiles)
{
    char path[PATH_MAX];
    char **afile;
    char **nfile;
    int  x, i=0;

    nfile = (char **) MALLOC(sizeof(char *));

    sprintf(path, "%s%calias", getenv("NETSITE_ROOT"), FILE_PATHSEP);
    afile = list_directory(path,0);

    for (x=0; afile[x]; x++) {
      if (strcmp(afile[x], "cert.log")) {
        nfile = (char **) REALLOC(nfile, (x+1)*sizeof(char *));
        nfile[i] = afile[x]; 
        i++;
      }
    }
    *aliasfiles = nfile;
}

/* Makes a list of the aliases installed on your machine. */
void read_aliases(char ***aliaslist)
{
    char path[PATH_MAX];
    char *remstr;
    char *keystr = "-key.db";
    char **alist;
    char **nlist;
    int  len, x, i=0;

    nlist = (char **) MALLOC(sizeof(char *));

    sprintf(path, "%s%calias", getenv("NETSITE_ROOT"), FILE_PATHSEP);
    alist = list_directory(path,0);

    for (x=0; alist[x]; x++) {
      if ((remstr = (strstr(alist[x], keystr))) != NULL) {
        *(alist[x] + (strlen(alist[x]) - strlen(remstr))) = '\0';
        nlist = (char **) REALLOC(nlist, (x+1)*sizeof(char *));
        nlist[i] = alist[x]; 
        i++;
      }
    }
    nlist[i] = NULL;
    *aliaslist = nlist;
}

void get_key_cert_files(char *alias, char **keyfile, char **certfile)
{
    char * kfile = NULL;
    char * cfile = NULL;

    kfile = (char *) MALLOC(strlen(alias)+PATH_MAX);
    cfile = (char *) MALLOC(strlen(alias)+PATH_MAX);

    sprintf(kfile, "%s%calias%c%s-key.db", getenv("NETSITE_ROOT"), FILE_PATHSEP, FILE_PATHSEP, alias);
    sprintf(cfile, "%s%calias%c%s-cert.db", getenv("NETSITE_ROOT"), FILE_PATHSEP, FILE_PATHSEP, alias);
 
    *keyfile = kfile;
    *certfile = cfile;
} 

void display_aliases(char *keyfile, char **aliaslist)
{
    int  x; 
    int  cnt=0;
    int  remcnt=0;
    char *seps = "/";
    char *token; 
    char *curalias = NULL;
    char *keyword = NULL;

    if (keyfile != NULL) {
      token = strtok(keyfile, seps);
      while(token != NULL) {
        curalias = token;
        token = strtok(NULL, seps);
      }
      if ((keyword = strstr(curalias, "-key.db")) != NULL)
        *(curalias+(strlen(curalias)-strlen(keyword))) = '\0';
      if ((keyword = strstr(curalias, "-cert.db")) != NULL)
        *(curalias+(strlen(curalias)-strlen(keyword))) = '\0';
    }
     
    if(aliaslist) { 
      fprintf(stdout, "<select name=alias>\n");
      if(aliaslist) {
        for(x=0; aliaslist[x]; x++) {
          if (curalias != NULL) {
            if (!strcmp(curalias, aliaslist[x])) {
                fprintf(stdout, "<option value=%s>%s\n",
                        aliaslist[x], aliaslist[x]);
                curalias = NULL;
            }
            else {
              remcnt++;
            }
          }
          else {
            fprintf(stdout, "<option value=%s>%s\n",
                    aliaslist[x], aliaslist[x]);
          }
          cnt++;
        }
        for(x=0; x<remcnt; x++) {
          fprintf(stdout, "<option value=%s>%s\n",
                  aliaslist[x], aliaslist[x]);
        }
        if (cnt == 0)
          fprintf(stdout, "<option value=No alias>No alias\n");
      }
      fprintf(stdout, "</select>");
    }
    else
      output_input("text", "alias", "No alias", "size=53");
}

void read_cert_conf(void)
{
   char *root;
   char certpath[512];
   FILE *f;

   root = getenv("NETSITE_ROOT"); 

#ifdef USE_ADMSERV
   if(certconfarray == NULL)  {
       int howmany = 5;
       register int x=0;

       certconfarray = (char ***) MALLOC(howmany*sizeof(char **));
       for(x=0; x < howmany; x++)  {
           sprintf(certpath, "%s%calias%c%s", root, FILE_PATHSEP, FILE_PATHSEP, CERT_LOG);

           if(!file_exists(certpath))  {
               certconfarray[x] = NULL;
           }  else  {
               certconfarray[x] = scan_tech(certpath);
           }
#ifndef MCC_ADMSERV
           if(!certconfarray[x])
               report_error(FILE_ERROR, certpath,
                       "Could not read certificate log file.");
#endif
       }
       /* Set to default for subsequent stuffs */
       certconf = certconfarray[0];
   }
#else
   if(certconf == NULL) {

       sprintf(certpath, "%s%calias%c%s", root, FILE_PATHSEP, FILE_PATHSEP, CERT_LOG);
       certconf = scan_tech(certpath);
   }
   if(certconf == NULL)
       report_error(FILE_ERROR, certpath,
                    "Could not read certificate log file.");
#endif
}

void write_cert_conf(void)
{
    register int x = 0;
    FILE *f;
    char certpath[512];
    char *root;

#ifdef USE_ADMSERV
    register int y, howmany;
 
    root = getenv("NETSITE_ROOT"); 
    howmany=make_conflist();

    for(y=0; y<howmany; y++)  {
        sprintf(certpath, "%s%calias%c%s", root, FILE_PATHSEP, FILE_PATHSEP, CERT_LOG);
        certconf = certconfarray[y];

        f = fopen_l(certpath, "w");
 
        if(f == NULL)
           report_error(FILE_ERROR, certpath,
                    "Could not write to certificate log file.");
 
        for(x=0; certconf[x]; x++)
            if(strcmp(certconf[x], "\n"))
                fprintf(f, "%s\n", certconf[x]);
            else
                fprintf(f, "%s", certconf[x]);
 
        fclose_l(f);
 
        set_commit(y, 1);
    }
#else
    f = fopen_l(magpath, "w");

    if(f == NULL)
       report_error(FILE_ERROR, certpath,
                    "Could not write to certificate log file.");

    for(x=0; certconf[x]; x++)
        if(strcmp(certconf[x], "\n"))
            fprintf(f, "%s\n", certconf[x]);
        else
            fprintf(f, "%s", certconf[x]);

    fclose_l(f);
#endif
}

void insert_cert_var(char *var)
{
    int certlen;
    register int x, y;

    for(certlen=0; certconf[certlen]; certlen++);
    x = certlen-1;
    y = certlen+1;

/* Algorithm: Look for last line before the Init directives that is not a 
 *            comment. */
    while(x >= 0 &&
          ((!strncmp(certconf[x], "Init ", 5)) || (certconf[x][0] == '#'))) {
        x--;
    }

/* Grow the strlist.  Don't forget: keylen+1 is a NULL. */
    certconf = grow_strlist(certconf, certlen+3);

/* And insert right after it. */
    for(; y > x; y--)
        certconf[y+1] = certconf[y];

    certconf[y+1] = STRDUP(var);
}

void delete_cert_var(int offset)
{
    int certlen;
    register int x;
    
    for(certlen=0; certconf[certlen]; certlen++);

    FREE(certconf[offset]);

    for(x = offset+1; x <= certlen; x++)  {
        certconf[x-1] = certconf[x];
    }
    
    certconf[certlen] = NULL;
}

NSAPI_PUBLIC char *get_cert_var(char *var)
{
   return get_num_cert_var(0, var);
}

NSAPI_PUBLIC char *get_num_cert_var(int whichsrv, char *var)
{
   register int x;
   char *value = NULL;
   char *work = STRDUP(var);
   int len = strlen(work);
  
   read_cert_conf();
 
   certconf = certconfarray[whichsrv];
 
#ifdef MCC_ADMSERV
   if(!certconf) return NULL;
#endif

   if(work[len-1] == ' ')
       work[--len] = '\0';
 
   for(x=0;certconf[x];x++)  {
       if ((!strncasecmp(certconf[x], work, len)) && (certconf[x][len]==' '))  {
           value = STRDUP(certconf[x]+len+1);
           break;
       }
   }
   certconf = certconfarray[0];
   return value;
}

NSAPI_PUBLIC void set_cert_var(char *name, char *value)
{
   char certpath[512];

    register int x=0, y=0, howmany = 0;
    int found = 0, l = strlen(name);
    char line[BIG_LINE];

    read_cert_conf();
    howmany = make_conflist();

    if(name[l-1] == ' ')
        name[--l] = '\0';

    for(y=0; y < howmany; y++)  {
        certconf = certconfarray[y];
        found = 0;

        for(x=0; certconf[x]; x++)  {
            if(!strncmp(certconf[x], name, l))  {
                found = 1;
                break;
            }
        }
        if(found)  {
            if(value)  {
                if(y==0)  {
                    /* If this one hasn't changed, don't change the rest. */
                    if(!strcmp(&(certconf[x][l+1]), value))  {
                        return;
                    }
                }
                sprintf(line, "%s %s", name, value);
                certconf[x] = STRDUP(line);
            }  else  {
                delete_cert_var(x);
            }
        }  else  {
            if(value)  {
                sprintf(line, "%s %s", name, value);
                insert_cert_var(line);
            }
        }
        /* Ah ah ah - insert_key_var can grow the strlist.  Must be sure 
         * that keyconfarray[y] has the right pointer. */
        certconfarray[y] = certconf;
    }
    write_cert_conf();
}

