/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * magconf.c - Read in the magnus.conf file, get info from it, and modify it
 *
 * All blame to Mike McCool
 */

#include "libadminutil/admutil.h"
#include "libadmin/libadmin.h"
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

static char ***magconfarray = NULL;
static char **magconf = NULL;


void read_mag_conf(void)
{
   char magpath[512];
#ifdef USE_ADMSERV
   if(magconfarray == NULL)  {
       int howmany = make_conflist();
       register int x=0;

       magconfarray = (char ***) MALLOC(howmany*sizeof(char **));
       for(x=0; x < howmany; x++)  {
           sprintf(magpath, "%s%s", CONFDIR(x), MAGNUS_CONF);
           magconfarray[x] = scan_tech(magpath);
#ifndef MCC_ADMSERV
           if(!magconfarray[x])
               report_error(FILE_ERROR, magpath,
                       "Could not read configuration file magnus.conf.");
#endif
       }
       /* Set to default for subsequent stuffs */
       magconf = magconfarray[0];
   }
#else
   if(magconf == NULL) {

       sprintf(magpath, "%s%s", CONFDIR(0), MAGNUS_CONF);
       magconf = scan_tech(magpath);
   }
   if(magconf == NULL)
       report_error(FILE_ERROR, magpath,
                    "Could not read configuration file magnus.conf.");
#endif
}

void write_mag_conf(void)
{
    register int x = 0;
    FILE *f;
    char magpath[512];

#ifdef USE_ADMSERV
    register int y, howmany;
 
    howmany=make_conflist();

    for(y=0; y<howmany; y++)  {
#if 0 /* MLM - backup ONLY upon confirm */
    /* Back up obj.conf at the same time to keep the numbers consistent. MLM */
        sprintf(magpath, "%s%s", CONFDIR(y), OBJ_DATABASE);
        conf_backup(magpath, BK_OBJ, y);

        sprintf(magpath, "%s%s", CONFDIR(y), MAGNUS_CONF);
        conf_backup(magpath, BK_MAGNUS, y);
#else
        sprintf(magpath, "%s%s", CONFDIR(y), MAGNUS_CONF);
#endif
        magconf = magconfarray[y];

        f = fopen_l(magpath, "w");
 
        if(f == NULL)
           report_error(FILE_ERROR, magpath,
                    "Could not write configuration file magnus.conf.");
 
        for(x=0; magconf[x]; x++)
            if(strcmp(magconf[x], "\n"))
                fprintf(f, "%s\n", magconf[x]);
            else
                fprintf(f, "%s", magconf[x]);
 
        fclose_l(f);
 
        set_commit(y, 1);
    }
#else
    f = fopen_l(magpath, "w");

    if(f == NULL)
       report_error(FILE_ERROR, magpath,
                    "Could not write configuration file magnus.conf.");

    for(x=0; magconf[x]; x++)
        if(strcmp(magconf[x], "\n"))
            fprintf(f, "%s\n", magconf[x]);
        else
            fprintf(f, "%s", magconf[x]);

    fclose_l(f);
#endif
}

void insert_mag_var(char *var)
{
    int maglen;
    register int x, y;

    for(maglen=0; magconf[maglen]; maglen++);
    x = maglen-1;
    y = maglen+1;

/* Algorithm: Look for last line before the Init directives that is not a 
 *            comment. */
    while(x >= 0 &&
          ((!strncmp(magconf[x], "Init ", 5)) || (magconf[x][0] == '#'))) {
        x--;
    }

/* Grow the strlist.  Don't forget: maglen+1 is a NULL. */
    magconf = grow_strlist(magconf, maglen+3);

/* And insert right after it. */
    for(; y > x; y--)
        magconf[y+1] = magconf[y];

    magconf[y+1] = STRDUP(var);
}

void delete_mag_var(int offset)
{
    int maglen;
    register int x;
    
    for(maglen=0; magconf[maglen]; maglen++);

    FREE(magconf[offset]);

    for(x = offset+1; x <= maglen; x++)  {
        magconf[x-1] = magconf[x];
    }
    
    magconf[maglen] = NULL;
}

NSAPI_PUBLIC char *get_mag_var(char *var)
{
   return get_num_mag_var(0, var);
}

NSAPI_PUBLIC char *get_num_mag_var(int whichsrv, char *var)
{
   register int x;
   char *value = NULL;
   char *work = STRDUP(var);
   int len = strlen(work);
  
   read_mag_conf();
 
   magconf = magconfarray[whichsrv];
 
#ifdef MCC_ADMSERV
   if(!magconf) return NULL;
#endif

   if(work[len-1] == ' ')
       work[--len] = '\0';
 
   for(x=0;magconf[x];x++)  {
       if ((!strncasecmp(magconf[x], work, len)) && (magconf[x][len]==' '))  {
           value = STRDUP(magconf[x]+len+1);
           break;
       }
   }
   magconf = magconfarray[0];
   return value;
}

NSAPI_PUBLIC void set_mag_var(char *name, char *value)
{
    register int x=0, y=0, howmany = 0;
    int found = 0, l = strlen(name);
    char line[BIG_LINE];

    read_mag_conf();
    howmany = make_conflist();

    if(name[l-1] == ' ')
        name[--l] = '\0';

    for(y=0; y < howmany; y++)  {
        magconf = magconfarray[y];
        found = 0;

        for(x=0; magconf[x]; x++)  {
            if((!strncasecmp(magconf[x], name, l)) && (magconf[x][l]==' ')) {
                found = 1;
                break;
            }
        }
        if(found)  {
            if(value)  {
                if(y==0)  {
                    /* If this one hasn't changed, don't change the rest. */
                    if(!strcmp(&(magconf[x][l+1]), value))  {
                        return;
                    }
                }
                sprintf(line, "%s %s", name, value);
                magconf[x] = STRDUP(line);
            }  else  {
                delete_mag_var(x);
            }
        }  else  {
            if(value)  {
                sprintf(line, "%s %s", name, value);
                insert_mag_var(line);
            }
        }
        /* Ah ah ah - insert_mag_var can grow the strlist.  Must be sure 
         * that magconfarray[y] has the right pointer. */
        magconfarray[y] = magconf;
    }
    write_mag_conf();

    if(value)  {
        if(found)
            log_change(TO_MAGNUS, "'%s' set to '%s'", name, value);
        else
            log_change(TO_MAGNUS, "'%s' added, value '%s'", 
                                   name, value);
    } else {
        if(found)  {
            log_change(TO_MAGNUS, "'%s' removed", name);
        }
    }
}
