/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * multconf.c - Deal with problems introduced by changing multiple config 
 *              files
 *
 * All blame to Mike McCool
 */

#include "libadminutil/admutil.h"
#include "libadmin/libadmin.h"
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

static char **conflist = NULL;
static int conftotal = 0;

/* This is grotesque.  To let the admin server do some faking, we allow 
 * fake_conflist to "fool" the programs into thinking they're configuring
 * a particular server (when they're not.)  DON'T REUSE THIS!  It is only
 * provided so I can do a very particular task.  */
NSAPI_PUBLIC int fake_conflist(char *fakename);

NSAPI_PUBLIC int make_conflist(void)
{
    char *snames = NULL, *tmp = NULL;

    tmp = getenv("SERVER_NAMES");
    if(tmp) snames=STRDUP(tmp);
 
    /* already been here */
    if(conflist) return conftotal;
    /* XXXrobm this is needed when no servers are installed but fopen_l
       is being used */
    if(!snames)  snames = "admin-serv";
    return fake_conflist(snames);
} 

NSAPI_PUBLIC int fake_conflist(char *fakename)
{
    char *snames = STRDUP(fakename);

    if(snames[0] == '(')  {
        /* we know there's at least one... */
        register int numservs = 1, x = 0;
        register char *tmp;
  
        for(x=0; snames[x]; x++)  {
            if(snames[x] == '|') numservs++;
        }
        conflist = new_strlist(numservs+2);
        x=0;
        tmp=strtok(++snames, "|)");
        while(tmp)  {
            conflist[x++] = STRDUP(tmp);
            conflist[x] = NULL;
            tmp=strtok(NULL, "|)");
        }
        conftotal = numservs;
    }  else  {
        conflist = new_strlist(3);
        conflist[0] = STRDUP(snames);
        conflist[1] = NULL;
        conftotal=1;
    }
    return conftotal;
}

NSAPI_PUBLIC char *get_conf_dir(int whichone) 
{
    char *cftmp = getenv("CONFIG_DIR");
    char *cfdir;
    
    CONVERT_TO_HTTP_FORMAT(cftmp);
    if(!strstr(cftmp, "%s")) return cftmp;

    if(!conflist) make_conflist();

    cfdir = (char *)MALLOC(strlen(cftmp) + strlen(conflist[whichone]) + 2);
    sprintf(cfdir, cftmp, conflist[whichone]);
    return cfdir;
}

NSAPI_PUBLIC int is_remote_server(int whichone)
{
    return 0;
}

NSAPI_PUBLIC int is_admserv(void)
{
    make_conflist();
    if(!strncmp(conflist[0], "admin-serv", 10))  {
        return 1;
    }
    return 0;
}

NSAPI_PUBLIC int remote_server_inlist(void)
{
    return 0;
}

NSAPI_PUBLIC char *get_commit_dest(int whichone)
{
    char *ans = NULL;

    {
        char *sname = get_srvname(whichone);
        char *sroot = getenv("NETSITE_ROOT");

        ans = (char *) MALLOC(strlen(sname) + strlen(sroot) + 16);
        sprintf(ans, "%s%c%s%cconfig%c", sroot, FILE_PATHSEP,
                     sname, FILE_PATHSEP, FILE_PATHSEP);
    }
    return ans;
}

NSAPI_PUBLIC char *get_srvname(int whichsrv)
{
    if(!conflist) make_conflist();

    if(whichsrv != -1)
        return STRDUP(conflist[whichsrv]);
    else  {
        char *ans = STRDUP(getenv("SERVER_NAMES"));
        register int x;
   
        for(x=0; ans[x]; x++) 
            if(ans[x] == '|')
                ans[x] = ',';
        return ans;
    }
}
