/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * nsnews.c: Handles nsnews.conf for news-related stuff
 * 
 * Rob McCool
 */

#ifdef MCC_NEWS
/*
 * There is a LOT of copy-and-paste code in here, and this file got rather
 * large. Some of this should be moved into the individual scripts, and
 * some of it should be generalized, but time constraints prevent me
 * from doing that.
 */

#include "libadminutil/admutil.h"
#include "libadmin/hinstall.h"
#include "libadmin/libadmin.h"
#include "base/file.h"
#include "base/util.h"  /* util_sprintf */
#include "inn/configdata.h" /* BOOL */
#include "inn/libinn.h"

/* For kill below */
#include <sys/types.h>
#include <signal.h>
#include <stdlib.h>

/* -------------------------- nsnews.conf stuff --------------------------- */



#define DEFAULT_NEWSCONF "nsnews.conf"


char **scan_nsnews(char *fn)
{
    char err[1024], **lines;

    if(!(lines = scan_tech(fn))) {
        sprintf(err, "Can't open file %s", fn);
        report_error(FILE_ERROR, system_errmsg(), err);
    }
    return lines;
}


void nsnews_file2path_admin(char *filename, char *path)
{
    if(filename)
        strcpy(path, filename);
    else
/* XXX for now get around cluster stuff by ignoring it [jules] */

/* This needs to be fixed to point to news-server/config directory because
   the structure of the 2.0 tree is different....
   - Nirmal
   CONFDIR (0) = ns-root/admserv/news-poy-nirmal/
*/
   util_sprintf(path, "%s%s", get_commit_dest(0), DEFAULT_NEWSCONF); 
}

char **scan_nsnews_admin(char *filename)
{
    char fn[1024];

    nsnews_file2path_admin(filename, fn);
    return scan_nsnews(fn);
}


void nsnews_file2path_install(char *filename, char *path)
{
    if(filename)
        strcpy(path, filename);
    else
        util_sprintf(path, "%s%c%s", CONFIG_DIRECTORY, FILE_PATHSEP,
                     DEFAULT_NEWSCONF);
}

char **scan_nsnews_install(char *filename)
{
    char fn[1024];

    nsnews_file2path_admin(filename, fn);
    return scan_nsnews(fn);
}


void write_nsnews(char *fn, char **lines)
{
    char err[1024];

    if(write_tech(fn, lines) == -1) {
        sprintf(err, "Can't open file %s", fn);
        report_error(FILE_ERROR, system_errmsg(), err);
    }
}


void write_nsnews_admin(char *filename, char **lines)
{
    char fn[1024];

    nsnews_file2path_admin(filename, fn);
    write_nsnews(fn, lines);
}

void write_nsnews_install(char *filename, char **lines)
{
    char fn[1024];

    nsnews_file2path_install(filename, fn);
    write_nsnews(fn, lines);
}


int find_nsnews_line(char *var, char **lines)
{
    register int x;
    int l = strlen(var);

    for(x = 0; lines[x]; ++x) {
        if((!strncmp(lines[x], var, l)) && (isspace(lines[x][l])))
            return x;
    }
    return -1;
}

char *find_nsnews_var(char *var, char **lines)
{
    int vnum = find_nsnews_line(var, lines);
    char *ret;

    if(vnum == -1)
        return NULL;

    /* Skip leading whitespace, then return */
    for(ret = &lines[vnum][strlen(var)]; (*ret) && isspace(*ret); ++ret);
    return ret;
}


#include "inn/prefs.h"         /* NEWSCONF_MAXLEN */

void set_nsnews_var(char *name, char *val, char **lines)
{
    int vnum = find_nsnews_line(name, lines);
    char buf[NEWSCONF_MAXLEN], **ret;

    if(vnum == -1) {
        /* Scan to end, and add a new string */
        for(vnum = 0; lines[vnum]; vnum++);
        lines[vnum+1] = NULL;
    }
    else {
        lines[vnum] = NULL;
        ret = lines;
    }
    util_sprintf(buf, "%s %s", name, val);
    lines[vnum] = strdup(buf);
}


void remove_nsnews_var(char *name, char **lines)
{
    int x, start = find_nsnews_line(name, lines);

    if(start != -1) {
        for(x = start; lines[x]; ++x)
            lines[x] = lines[x+1];
    }
}


void replace_nsnews_prefix(char *opfx, char *npfx, char **lines)
{
    register int x, ol, nl;
    char *d, *v, *nstr, buf[NEWSCONF_MAXLEN];

    ol = strlen(opfx);
    nl = strlen(npfx);
    for(x = 0; lines[x]; ++x) {
        strcpy(buf, lines[x]);
        if(config_find_directive(buf, &d, &v)) {
            if(!strncmp(v, opfx, ol)) {
                nstr = (char *)MALLOC(strlen(d) + 1 + nl + strlen(v) - ol + 1);
                util_sprintf(nstr, "%s %s%s", d, npfx, &v[ol]);
                lines[x] = nstr;
            }
        }
    }
}


/* ---------------------------- nsnews_running ---------------------------- */


/* UNIX-specific: Makes sure the given process id is really running */
/* Nirmal: added NT process checking */

int nsnews_running(char **nscnf)
{
    char *pidlog;
    FILE *f;
#ifdef WIN32
    HANDLE hproc;
    ULONG  pid;
#else    
    pid_t pid;
    int x;
#endif
    
    if(!nscnf)
        nscnf = scan_nsnews_admin(NULL);
    pidlog = find_nsnews_var("_PATH_SERVERPID", nscnf);

    if (! (f = fopen(pidlog, "r")))
        return 0;
#ifdef WIN32
    if (fscanf(f, "%ul", &pid) != 1)
        return 0;
/*    I am not able to get access to open this, because innd runs as
      a service now. So, assume the process is alive, if the pid exists.
hproc = OpenProcess(PROCESS_ALL_ACCESS | STANDARD_RIGHTS_REQUIRED,
			    FALSE, pid);

    hproc = OpenProcess(SYNCHRONIZE, FALSE, pid);
    if (hproc == NULL) {
	    pid = GetLastError();
// 	    fprintf(stdout, "OpenProcess: failed. <pre>\n\n\n</pre>" ); 
	    return 0;
    }
*/    
    else
	return 1;
#else
    if (fscanf(f, "%d", &x) != 1)
        return 0;
    pid = (pid_t) x;
    return (kill(pid, 0) == -1 ? 0 : 1);
#endif
    
}



/* ------------------------------- ctlinnd -------------------------------- */


void run_newscmd(char *bin, char *args)
{
    char cmd[1024], ctlpath[1024], junk[1024];
    char **nscnf, *newsbin;
    struct runcmd_s rm;
#ifdef WIN32
    HANDLE hSaveStdin, hSaveStdout, hSaveStderr, hReadPipe, hWritePipe;
    SECURITY_ATTRIBUTES saAttr;
    char   Buffer[1024];	// address of buffer that receives data
    DWORD  NumberOfBytesRead;	// address of number of bytes read
#endif    
    
    nsnews_file2path_admin(NULL, ctlpath);
    nscnf = scan_nsnews(ctlpath);
    if(!(newsbin = find_nsnews_var("_PATH_NEWSBIN", nscnf)))
        report_error(FILE_ERROR, NULL, "Error getting _PATH_NEWSBIN from nsnews.conf");


/* Nirmal: NT env. handling is different */    
#ifdef WIN32
    SetEnvironmentVariable("NS_NEWSCONF" , ctlpath);
    /* Nirmal: This is to circumvent bug 
    util_sprintf(cmd, "\"%s\\\\%s.exe\" %s", bin, newsbin, args);
    */
    util_sprintf(cmd, "%s/%s.exe %s", newsbin, bin, args);
    
    /* Now before createProcess, setup a pipe to get stdout of the child. */

    /* Set the bInheritHandle flag so pipe handles are inherited. */
    saAttr.nLength = sizeof(SECURITY_ATTRIBUTES);
    saAttr.bInheritHandle = TRUE;
    saAttr.lpSecurityDescriptor = NULL;

    /*
     * The steps for redirecting child's STDOUT:
     *     1.  Save current STDOUT, to be restored later.
     *     2.  Create anonymous pipe to be STDOUT for child.
     *     3.  Set STDOUT of parent to be write handle of pipe, so
     *         it is inherited by child.
     */

    /* Save the handle to the current STDOUT. */
    hSaveStdout = GetStdHandle(STD_OUTPUT_HANDLE);
    hSaveStderr = GetStdHandle(STD_ERROR_HANDLE);

    /* Create a pipe for the child's STDOUT. */
    if (! CreatePipe(&hReadPipe, &hWritePipe, &saAttr, 0))
        report_error(SYSTEM_ERROR, NULL, "run_newscmd: Could not createPipe");

    /* Set write handle to the pipe to be STDOUT. */
    if (! SetStdHandle(STD_OUTPUT_HANDLE, hWritePipe))
        report_error(SYSTEM_ERROR, NULL, "run_newscmd: Could not SetStdHandle");
    if (! SetStdHandle(STD_ERROR_HANDLE, hWritePipe))
        report_error(SYSTEM_ERROR, NULL, "run_newscmd: Could not SetStdHandle");

    /*
     * Make the child's STDIN to be null : Also make read end of pipe
     * non-inheritable by child.
     * Save the handle to the current STDIN. */
    SetHandleInformation (hReadPipe, HANDLE_FLAG_INHERIT, 0) ;
    hSaveStdin = GetStdHandle(STD_INPUT_HANDLE);
    if (! SetStdHandle(STD_INPUT_HANDLE, NULL))
        report_error(SYSTEM_ERROR, NULL, "run_newscmd: Could not SetStdHandle");

    if(run_cmd(cmd, NULL, &rm) == -1) {
        util_sprintf(junk, "Error running %s: ", bin, rm.msg, rm.arg);
	/* After process creation, restore the saved STDIN and STDOUT. */
	if (! SetStdHandle(STD_INPUT_HANDLE, hSaveStdin))
           report_error(SYSTEM_ERROR, NULL,
			"run_newsbin: Couldn't Redirect i/p");
	if (! SetStdHandle(STD_OUTPUT_HANDLE, hSaveStdout))
	   report_error(SYSTEM_ERROR, NULL,
			"run_newsbin: Couldn't Redirect o/p");
	if (! SetStdHandle(STD_ERROR_HANDLE, hSaveStderr))
	   report_error(SYSTEM_ERROR, NULL,
			"run_newsbin: Couldn't Redirect err");

	report_error(SYSTEM_ERROR, rm.title, junk);
    }
    
    /* After process creation, restore the saved STDIN and STDOUT. */
    if (! SetStdHandle(STD_INPUT_HANDLE, hSaveStdin))
        report_error(SYSTEM_ERROR, NULL,
		     "run_newsbin: Could not Redirect i/p");
    if (! SetStdHandle(STD_OUTPUT_HANDLE, hSaveStdout))
        report_error(SYSTEM_ERROR, NULL,
		     "run_newsbin: Could not Redirect o/p");
    if (! SetStdHandle(STD_ERROR_HANDLE, hSaveStderr))
	report_error(SYSTEM_ERROR, NULL,
		     "run_newsbin: Couldn't Redirect err");

    /* Read from pipe the standard output for child process.
       When the child terminates, the pipe will be broken.
    */
    CloseHandle( hWritePipe );
    while( TRUE ) {
	 NumberOfBytesRead = 0 ;
	 ZeroMemory(&Buffer, sizeof(Buffer));

	 if ( ! ReadFile(
	        hReadPipe,	// handle of file to read
	        &Buffer,	// address of buffer that receives data
	        sizeof(Buffer),	// number of bytes to read
	        &NumberOfBytesRead,	// address of number of bytes read
	        NULL		// address of structure for data
	    )) {
		if( GetLastError() == ERROR_BROKEN_PIPE ) {
			CloseHandle( hReadPipe );
			return;
		}
		else
			report_error(SYSTEM_ERROR, NULL,
				     "run_newsbin: ReadFile");
	 }
	 else if( NumberOfBytesRead  && (!strnicmp(Buffer, "Ok", 2)) ) {
		 CloseHandle( hReadPipe );
		 return;
	 }
	 else   //  error is indicated by this result.
		report_error(SYSTEM_ERROR, NULL, Buffer);
    }	 
		
#else
    util_sprintf(cmd, "NS_NEWSCONF=%s; export NS_NEWSCONF; %s/%s %s",
                 ctlpath, newsbin, bin, args);
    if(run_cmd(cmd, NULL, &rm) == -1) {
        util_sprintf(junk, rm.msg, rm.arg);
        report_error(SYSTEM_ERROR, rm.title, junk);
    }
#endif /* WIN32 */
    
}

void run_ctlinnd(char *cmd)
{
    run_newscmd("ctlinnd", cmd);
}


char **nsnews_status(void)
{
    char cmd[1024], ctlpath[1024], outfn[1024], *newsbin;
    char **nscnf, **ret;

#ifdef WIN32
    HANDLE hproc;
    char *tmpdir;
#endif

    nsnews_file2path_admin(NULL, ctlpath);
    nscnf = scan_nsnews(ctlpath);
    if(!(newsbin = find_nsnews_var("_PATH_NEWSBIN", nscnf)))
        report_error(FILE_ERROR, NULL, "Can't open nsnews.conf");

#ifdef WIN32
        /* Nirmal: Setup a proper comm. when time permits....
    */	    
    hproc = OpenProcess(STANDARD_RIGHTS_REQUIRED, FALSE, GetCurrentProcessId());
    if (hproc == NULL) {
	    fprintf(stdout, "Content-type: text/html\n\n");
	    fflush(stdout);
	    report_error(SYSTEM_ERROR, NULL, "nsnews_status: Couldn't get my pid");
	    return NULL;
    }
    else {
	    SetEnvironmentVariable("NS_NEWSCONF" , ctlpath);
	    tmpdir = find_nsnews_var("_PATH_TMP", nscnf) ;
	    util_sprintf(outfn, "%s/newsmode.%d", tmpdir, hproc);
	    util_sprintf(cmd, "%s/ctlinnd.exe mode > %s  2>&1",
			 newsbin, outfn);
    }
#else
    util_sprintf(outfn, "/tmp/newsmode.%d", getpid());
    util_sprintf(cmd, "NS_NEWSCONF=%s; export NS_NEWSCONF; %s/ctlinnd mode"
                 " > %s 2>&1", ctlpath, newsbin, outfn);
#endif
    system(cmd);
    if(!(ret = scan_tech(outfn)))
        report_error(SYSTEM_ERROR, NULL, "Can't execute ctlinnd");
    
#ifdef XP_WIN32
    // Do nothing.   This section is added for my ease of readability
    // - Nirmal
#else    
    unlink(outfn);
#endif
    
    return ret;
}


/* --------------------------- moderator stuff ---------------------------- */


char **scan_moderators(char *fn)
{
    char **nscnf = scan_nsnews_admin(NULL);
    char *path_moderators = find_nsnews_var("_PATH_MODERATORS", nscnf);
    char **lines;

    if(!(lines = scan_tech(fn ? fn : path_moderators)))
        report_error(FILE_ERROR, system_errmsg(), 
                     "Can't open moderators file");
    return lines;
}


void write_moderators(char *fn, char **lines)
{
    char **nscnf = scan_nsnews_admin(NULL);
    char *path_moderators = find_nsnews_var("_PATH_MODERATORS", nscnf);

    if(write_tech(fn ? fn : path_moderators, lines) == -1)
        report_error(FILE_ERROR, system_errmsg(), 
                     "Can't write to moderators file");
}


void set_moderator(char *group, char *email)
{
    register int x, y;
    char **lines, *t, *newline;

    lines = scan_moderators(NULL);

    newline = (char *) MALLOC(strlen(group) + 1 + strlen(email) + 1);
    util_sprintf(newline, "%s:%s", group, email);

    /* The moderators file uses shell patterns. Try to guess where
       this entry would go by the length of the patterns. */
    for(x = 0; lines[x]; ++x) {
        if((lines[x][0] != '#') && (t = strchr(lines[x], ':'))) {
            *t = '\0';
            if(!strcmp(group, lines[x])) {
                lines[x] = newline;
                break;
            }
            *t = ':';
            if((t - lines[x]) <= strlen(group)) {
                /* Insert here */
                for(y = x; lines[y]; ++y);
                while(y != (x-1)) {
                    lines[y+1] = lines[y];
                    --y;
                }
                lines[x+1] = lines[x];
                lines[x] = newline;
                break;
            }
        }
    }
    if(!lines[x]) {
        lines[x] = newline;
        lines[x+1] = NULL;
    }
    write_moderators(NULL, lines);
}


char *find_moderator(char *group, char **lines)
{
    register int x, ret;
    char *t;

    if(!lines)
        lines = scan_moderators(NULL);

    for(x = 0; lines[x]; ++x) {
        if((lines[x][0] != '#') && (t = strchr(lines[x], ':'))) {
            *t = '\0';
            ret = wildmat(group, lines[x]);
            *t = ':';
            if(ret)
                return t+1;
        }
    }
    return NULL;
}


/* ----------------------------- active file ------------------------------ */


char **scan_active(char **nscnf)
{
    char err[1024], **lines;
    char *actpath;

    if(!nscnf)
        nscnf = scan_nsnews_admin(NULL);
    actpath = find_nsnews_var("_PATH_ACTIVE", nscnf);

    if(!(lines = scan_tech(actpath))) {
        sprintf(err, "Can't open file %s", actpath);
        report_error(FILE_ERROR, system_errmsg(), err);
    }
    return lines;
}

int find_active_group(char *grp, char **active)
{
    int x, l = strlen(grp);

    if(!active)
        active = scan_active(NULL);

    for(x = 0; active[x]; ++x) {
        if((!strncmp(grp, active[x], l)) && isspace(active[x][l]))
            return x;
    }
    return -1;
}

/* SIDE EFFECT: Destroys line! */
int active_groupmatch(char *grppat, char *line)
{
    char *t;

    if(!(t = strchr(line, ' ')))
        return 0;
    *t = '\0';
    if((!strcmp(line, "control")) || (!strcmp(line, "junk")) || 
       (!wildmat(line, grppat)))
        return 0;
    return 1;
}


char *active_flags(char *line)
{
    char *ret = strrchr(line, ' ');

    return ret+1;
}

char **active_collapsedlist(char **active, char *filter)
{
    char **list, *match;
    register char *p, *q, save;
    register int i, j, mlen, elen, lsiz;
    extern int _admin_dumbsort(const void *s1, const void *s2);

    for (i = 0; active[i]; i++)
	;

    list = (char **)MALLOC((i+1) * sizeof(char *));
    list[0] = NULL;
    
    match = strdup(filter);
    p = match;
    while (*p && *p != '*')
	p++;
    *p = '\0';
    if (p == match) {
	free(match);
        match == NULL;
	mlen = 0;
    } else {
	mlen = strlen(match);
    }


    for (i = 0, lsiz = 0; active[i]; i++) {
	if (mlen) {
	    if (strncmp(active[i], match, mlen))
	        continue;
	}
	p = &(active[i][mlen]);
	while(*p && *p != '.' && !isspace(*p)) {
	    p++;
	}
	save = *p;
	*p = '\0';
	elen = strlen(active[i]);
	for (j = 0; list[j]; j++) {
	    if (strncmp(list[j], active[i], elen))
		continue;

	    /* Matched a list entry. */
	    if (strlen(list[j]) == elen && save == '.')
		continue;

	    if (list[j][strlen(list[j])-1] == '*')
		break;

	    q = (char *)MALLOC(strlen(list[j])+2);
	    strcpy(q, active[i]);
	    strcat(q, ".*");
	    free(list[j]);
	    list[j] = strdup(q);
	    break;
        }
	if (!list[j]) {
	    *p = save;
	    while(*p && !isspace(*p))
		p++;
	    save = *p;
	    *p = '\0';
	    list[j++] = strdup(active[i]);
	    list[j] = NULL;
	    lsiz = j;
	}
	*p = save;
    }
    free(match);
    qsort((void *)list, lsiz, sizeof(char *), _admin_dumbsort);
    list[lsiz] = NULL;

    return list;
}

/* -------------------------- expiration control -------------------------- */

char **scan_expirectl(char *fn)
{
    char err[1024], **lines;

    char **nscnf = scan_nsnews_admin(NULL);
    char *exppath = find_nsnews_var("_PATH_EXPIRECTL", nscnf);

    if(!(lines = scan_tech(fn ? fn : exppath))) {
        sprintf(err, "Can't open file %s", exppath);
        report_error(FILE_ERROR, system_errmsg(), err);
    }
    return lines;
}

void write_expirectl(char *fn, char **lines)
{
    char **nscnf = scan_nsnews_admin(NULL);
    char *exppath = find_nsnews_var("_PATH_EXPIRECTL", nscnf);

    if(write_tech(fn ? fn : exppath, lines) == -1)
        report_error(FILE_ERROR, system_errmsg(), 
                     "Can't write to expire control file");
}


int find_expire_line(char *find, char **lines)
{
    register int x, len = strlen(find);


    if(!lines)
        lines = scan_expirectl(NULL);
    for(x = 0; lines[x]; ++x) {
        if(lines[x][0] != '#') {
            if(!strncmp(lines[x], find, len))
                return x;
        }
    }
    return -1;
}

char *find_expire_string(char *find, char **lines)
{
    int lnum, len = strlen(find);

    if(!lines)
        lines = scan_expirectl(NULL);
    if( (lnum = find_expire_line(find, lines)) == -1)
        return NULL;
    return &lines[lnum][len];
}


void set_expire_remember(char *days, char **lines)
{
    int lnum = find_expire_line(EXPREM_STRING, lines);
    char *newstr;

    newstr = (char *) MALLOC(EXPREM_LEN + strlen(days) + 1);
    util_sprintf(newstr, "%s%s", EXPREM_STRING, days);

    if(lnum == -1) {
        for(lnum = 0; lines[lnum]; ++lnum);
        lines[lnum] = newstr;
        lines[lnum+1] = NULL;
    }
    else
        lines[lnum] = newstr;
}

void set_expire_default(char *def, char *keep, char *purge, char **lines)
{
    int lnum = find_expire_line(EXPDEF_STRING, lines);
    char *newstr;

    newstr = (char *) MALLOC(EXPDEF_LEN + strlen(def) + 1 + strlen(keep) + 1 +
                             strlen(purge) + 1);
    util_sprintf(newstr, "%s%s:%s:%s", EXPDEF_STRING, keep, def, purge);

    if(lnum == -1) {
        /* What to do... */
        report_error(INCORRECT_USAGE, "Expiration control file corrupt",
                     "Your expiration control file has somehow lost or "
                     "corrupted its default entry. Your server can't expire "
                     "articles properly in this condition. Please edit it "
                     "and add a default line.");
    }
    lines[lnum] = newstr;
}


int expire_entry(char *line, expire_s *ret)
{
    char *t, *u, *newstr;

    if((line[0] == '#') || (!strncmp(line, EXPREM_STRING, EXPREM_LEN)))
        return 0;

    newstr = strdup(line);

    /* Patterns first */
    t = newstr;
    if(!(u = strchr(t, ':')))
        goto bad;
    *u++ = '\0';
    ret->patterns = t;

    /* Modifier flag next */
    t = u;
    if(!(u = strchr(t, ':')))
        goto bad;
    *u++ = '\0';
    ret->flag = *t;

    /* Keep time */
    t = u;
    if(!(u = strchr(t, ':')))
        goto bad;
    *u++ = '\0';
    ret->keep = t;

    /* Default time */
    t = u;
    if(!(u = strchr(t, ':')))
        goto bad;
    *u++ = '\0';
    ret->def = t;

    /* Purge time */
    ret->purge = u;

    return 1;

  bad:
    FREE(newstr);
    return 0;
}


expire_s *expire_entry_default(char **lines)
{
    expire_s *ex;
    int lnum;

    if(!lines)
        lines = scan_expirectl(NULL);

    ex = (expire_s *) MALLOC(sizeof(expire_s));

    lnum = find_expire_line(EXPDEF_STRING, lines);

    if((lnum == -1) || (!expire_entry(lines[lnum], ex))) {
        /* What to do... */
        report_error(INCORRECT_USAGE, "Expiration control file corrupt",
                     "Your expiration control file has somehow lost or "
                     "corrupted its default entry. Your server can't expire "
                     "articles properly in this condition. Please edit it "
                     "and add a default line.");
    }
    return ex;
}


expire_s *find_expire_entry(char *find, char **lines)
{
    int lnum;
    expire_s *ret;

    if(!lines)
        lines = scan_expirectl(NULL);
    if( (lnum = find_expire_line(find, lines)) == -1)
        return NULL;

    ret = (expire_s *) MALLOC(sizeof(expire_s));
    if(!(expire_entry(lines[lnum], ret)))
        return NULL;
    return ret;
}


char *expire_struct2str(expire_s *ex)
{
    char *ret;

    ret = (char *) MALLOC(strlen(ex->patterns) + 1 + 
                          1 + 1 + 
                          strlen(ex->keep) + 1 + 
                          strlen(ex->def) + 1 + 
                          strlen(ex->purge) + 1);
    util_sprintf(ret, "%s:%c:%s:%s:%s", ex->patterns, ex->flag, ex->keep,
                 ex->def, ex->purge);
    return ret;
}


void new_expire_entry(expire_s *ex, char **lines)
{
    char *newline = expire_struct2str(ex), *t;
    int x, y;

    /* The expire.ctl file, unlike moderators, goes from GENERAL to 
       SPECIFIC. */
    for(x = 0; lines[x]; ++x) {
        if((lines[x][0] != '#') && (t = strchr(lines[x], ':'))) {
            *t = '\0';
            if(!strcmp(ex->patterns, lines[x])) {
                lines[x] = newline;
                break;
            }
            *t = ':';
            if((t - lines[x]) >= strlen(ex->patterns)) {
                /* Insert here */
                for(y = x; lines[y]; ++y);
                while(y != (x-1)) {
                    lines[y+1] = lines[y];
                    --y;
                }
                lines[x+1] = lines[x];
                lines[x] = newline;
                break;
            }
        }
    }
    if(!lines[x]) {
        lines[x] = newline;
        lines[x+1] = NULL;
    }
}

void change_expire_entry(char *find, expire_s *ex, char **lines)
{
    int lnum = find_expire_line(find, lines);

    if(lnum == -1) {
        report_error(INCORRECT_USAGE, "Entry missing",
                     "The entry you chose to edit no longer exists "
                     "in the expiration control file. Is someone "
                     "else making changes?");
    }
    lines[lnum] = expire_struct2str(ex);
}


void remove_expire_entry(char *find, char **lines)
{
    int x, lnum = find_expire_line(find, lines);

    if(lnum == -1) {
        report_error(INCORRECT_USAGE, "Entry missing",
                     "The entry you chose to edit no longer exists "
                     "in the expiration control file. Is someone "
                     "else making changes?");
    }
    for(x = lnum; lines[x]; ++x)
        lines[x] = lines[x+1];
}


/* ---------------------------- access control ---------------------------- */

void compress_whitespace(char *);

static void
tidy_nsaccess(char **lines)
{
    register int x;

    for (x = 0; lines[x]; x++) {
	if (lines[x][0] == '#')
	    continue;
	compress_whitespace(lines[x]);
    }
}

char **scan_nsaccess(char *fn)
{
    char err[1024], **lines;

    char **nscnf = scan_nsnews_admin(NULL);
    char *acpath = find_nsnews_var("_PATH_NNRPACCESS", nscnf);

    if(!(lines = scan_tech(fn ? fn : acpath))) {
        sprintf(err, "Can't open file %s", acpath);
        report_error(FILE_ERROR, system_errmsg(), err);
    }
    tidy_nsaccess(lines);
    return lines;
}


void write_nsaccess(char *fn, char **lines)
{
    char **nscnf = scan_nsnews_admin(NULL);
    char *acpath = find_nsnews_var("_PATH_NNRPACCESS", nscnf);

    tidy_nsaccess(lines);
    if(write_tech(fn ? fn : acpath, lines) == -1)
        report_error(FILE_ERROR, system_errmsg(), 
                     "Can't write to access control file");
}


int find_nsaccess_line(char *find, char **lines)
{
    register int x, len = strlen(find);

    if(!lines)
        lines = scan_nsaccess(NULL);
    for(x = 0; lines[x]; ++x) {
        if(lines[x][0] != '#') {
            if((!strncmp(lines[x], find, len)) && (lines[x][len] == '\0'))
                return x;
        }
    }
    return -1;
}


/*
 * Default line is the first non-blank/non-comment line.
 */
int nsaccess_default_index(char **lines)
{
    register int x, y;
    char c;

    if(!lines)
        lines = scan_nsaccess(NULL);
    for(x = 0; lines[x]; ++x) {
        if(lines[x][0] == '#')
	    continue;
	y = 0;
	while((c = lines[x][y]) != '\0' && isspace(c))
	    y++;
	if (c != '\0')
	    break;
    }
    return x;
}


int nsaccess_entry(char *line, permission_s *ret)
{
    char *t, *u, *newstr;

    if(line[0] == '#')
        return 0;

    newstr = strdup(line);

    /* Patterns first */
    t = newstr;
    if(!(u = strchr(t, ':')))
        goto bad;
    *u++ = '\0';
    ret->grp = t;

    /* Host patterns next */
    t = u;
    if(!(u = strchr(t, ':')))
        goto bad;
    *u++ = '\0';
    ret->hostpats = t;

    /* Flags next */
    t = u;
    if(!(u = strchr(t, ':')))
        goto bad;
    *u++ = '\0';
    ret->flags = t;

    /* Valid users next */
    ret->userpat = u;

    return 1;

  bad:
    FREE(newstr);
    return 0;
}


permission_s *find_nsaccess_entry(char *find, char **lines)
{
    int lnum;
    permission_s *ret;

    if(!lines)
        lines = scan_nsaccess(NULL);
    if( (lnum = find_nsaccess_line(find, lines)) == -1)
        return NULL;

    ret = (permission_s *) MALLOC(sizeof(permission_s));
    if(!(nsaccess_entry(lines[lnum], ret)))
        return NULL;
    return ret;
}


char *nsaccess_struct2str(permission_s *ps)
{
    char *ret;

    ret = (char *) MALLOC(strlen(ps->grp) + 1 + 
                          strlen(ps->hostpats) + 1 + 
                          strlen(ps->flags) + 1 + 
                          strlen(ps->userpat) + 1);
    util_sprintf(ret, "%s:%s:%s:%s", ps->grp, ps->hostpats, ps->flags,
                 ps->userpat);
    return ret;
}


void new_nsaccess_entry(permission_s *ps, char **lines)
{
    char *newline = nsaccess_struct2str(ps), *t;
    int x, y;

    x = nsaccess_default_index(lines);
    if (x < 0)
	x = 0; /* XXXrs ugh! */
    x++; /* don't touch the first (default) entry */

    /* Try and keep things semi-tidy by sorting the entries */

    for(; lines[x]; ++x) {
        if((lines[x][0] != '#') && (t = strchr(lines[x], ':'))) {
            *t = '\0';
            if(strcmp(ps->grp, lines[x]) < 0) {
                *t = ':';
                /* Insert here */
                for(y = x; lines[y]; ++y);
                while(y != (x-1)) {
                    lines[y+1] = lines[y];
                    --y;
                }
                lines[x+1] = lines[x];
                lines[x] = newline;
                break;
            }
            *t = ':';
        }
    }
    if(!lines[x]) {
        lines[x] = newline;
        lines[x+1] = NULL;
    }
}


void change_nsaccess_entry(char *find, permission_s *ps, char **lines)
{
    int lnum = find_nsaccess_line(find, lines);

    if(lnum == -1) {
        report_error(INCORRECT_USAGE, "Entry missing",
                     "The entry you chose to edit no longer exists "
                     "in the access control file. Is someone "
                     "else making changes?");
    }
    lines[lnum] = nsaccess_struct2str(ps);
}


void remove_nsaccess_entry(char *find, char **lines)
{
    int x, lnum = find_nsaccess_line(find, lines);

    if(lnum == -1) {
        report_error(INCORRECT_USAGE, "Entry missing",
                     "The entry you chose to edit no longer exists "
                     "in the access control file. Is someone "
                     "else making changes?");
    }
    for(x = lnum; lines[x]; ++x)
        lines[x] = lines[x+1];
}

/*
 * The default entry is defined as the first in the file and must
 * begin with "*:"
 */
permission_s *find_nsaccess_default(char **lines)
{
    permission_s *nps = (permission_s *) MALLOC(sizeof(permission_s));
    int i;

    if(!lines)
        lines = scan_nsaccess(NULL);
    i = nsaccess_default_index(lines);
    if(lines[i] == NULL || lines[i][0] != '*' || lines[i][1] != ':' || 
       (!nsaccess_entry(lines[i], nps))) {
        report_error(INCORRECT_USAGE, "Access control file corrupt",
                     "Your access control file has somehow lost or "
                     "corrupted its default entry. Your server can't  "
                     "operate properly in this condition. Please edit it "
                     "and add a default line.");
    }
    return nps;
}


void set_nsaccess_default(char *hosts, char *flags, char *defuser, 
                          char **lines)
{
    int i;

    i = nsaccess_default_index(lines);
    if(lines[i] == NULL || lines[i][0] != '*' || lines[i][1] != ':') {
        report_error(INCORRECT_USAGE, "Access control file corrupt",
                     "Your access control file has somehow lost or "
                     "corrupted its default entry. Your server can't  "
                     "operate properly in this condition. Please edit it "
                     "and add a default line.");
    }
    lines[i] = (char *) MALLOC(1 + 1 + strlen(hosts) + 1 + 
                                  strlen(flags) + 1 + strlen(defuser) + 1);
    util_sprintf(lines[i], "*:%s:%s:%s", hosts, flags, defuser);
}


/* Newsfeeds stuff --------------------------------------------- */
/* ------------------------------------------------------------- */

static char **feedfile;
 
void feed_read_file()
{
    char **nscnf = scan_nsnews_admin(NULL);
    char *feedpath = find_nsnews_var("_PATH_NEWSFEEDS", nscnf);
 
    if(!feedpath)
        report_error(FILE_ERROR, NULL,
                     "Could not find a path to your newsfeeds file.");
    feedfile=scan_tech(feedpath);
    if(!feedfile)
        report_error(FILE_ERROR, NULL, "Could not read your newsfeeds file.");
}
 
void feed_write_file()
{
    char **nscnf = scan_nsnews_admin(NULL);
    char *feedpath = find_nsnews_var("_PATH_NEWSFEEDS", nscnf);
 
    if(!feedpath)
        report_error(FILE_ERROR, NULL,
                     "Could not find a path to your newsfeeds file.");
    write_tech(feedpath, feedfile);
}

char *feed_get_ind_var(int *x)
{
    register int y = 0;
    char *answer = NULL, *data = NULL;
    register int anssize = 0;
 
    anssize = 2;
    answer = (char *) MALLOC(2);
    answer[0] = '\0';
    y = strlen(feedfile[*x]) - 1;  if(y<0) y=0;
    data=strdup(feedfile[*x]);
 
    while(data[y]=='\\')  {
        data[y] = '\0';
        answer = (char *) REALLOC(answer, strlen(data) + anssize);
        strcat(answer, data);
        anssize+= strlen(data);
 
        if((feedfile[(*x)+1]) && (feedfile[(*x)+1][0] != '#'))  {
            (*x)++;
            data=strdup(feedfile[*x]);
            while(isspace(*data)) data++;
            y = strlen(data)-1;  if(y<0) y=0;
        }  else
            break;
    }
    answer = (char *) REALLOC(answer, strlen(data) + anssize);
    strcat(answer, data);
 
    return answer;
}
 
char *feed_get_host_var(char *host)
{
    int x = 0;
 
    while(feedfile[x])  {
        if(!strncmp(feedfile[x], host, strlen(host)))
            return feed_get_ind_var(&x);
        else
            x++;
    }
    return NULL;
}
 
char *feed_get_newsgroups(char *feedline)
{
    char *tmp = strdup(feedline);
    char *answer = strchr(tmp, ':');
 
    if(answer) answer++; else return NULL;
    tmp = strchr(answer, ':');
    *tmp = '\0';
    return answer;
}

char *feed_get_param(char *feedline)
{
    char *tmp = strdup(feedline);
    char *answer = strchr(tmp, ':');
 
    if(answer) answer++; else return NULL;

    tmp = strchr(answer, ':');
    if(tmp) tmp++; else return NULL;

    answer = strchr(tmp, ':');
    if(answer) answer++; else return NULL;

    return answer;
}
 
void feed_split_newsgroups(char *ngroups, char **allow, char **deny)
{
    register int x = 0, y = 0, z = 0;
 
    (*allow) = (char *) MALLOC(strlen(ngroups)+1);
    (*deny) = (char *) MALLOC(strlen(ngroups)+1);
 
    (*allow)[0] = '\0';
    (*deny)[0] = '\0';
 
    while(ngroups[x])  {
        if(ngroups[x] != '!')  {
            while((ngroups[x]) && (ngroups[x] != ','))
                (*allow)[y++] = ngroups[x++];
            (*allow)[y++] = ',';
            if(ngroups[x] == ',')
                x++;
        }  else  {
            ++x;
            while((ngroups[x]) && (ngroups[x] != ','))
                (*deny)[z++] = ngroups[x++];
            (*deny)[z++] = ',';
            if(ngroups[x] == ',')
                x++;
        }
    }
 
    if(y) (*allow)[y-1] = '\0';
    if(z) (*deny)[z-1] = '\0';
}
 
void compress_whitespace(char *source)
{
    register int x,y;
 
    if(!source) return;
 
    for(x=0, y=0; source[x]; x++)  {
        if(!isspace(source[x]))
             source[y++] = source[x];
    }
    source[y] = '\0';
}

char *feed_merge_newsgroups(char *allow_in, char *deny_in)
{
    char *answer;
    char **grouplist;
    int x, y, z;
    char *tmp;
    char *allow, *deny;
 
    allow = (allow_in? strdup(allow_in) : NULL);
    deny = (deny_in? strdup(deny_in) : NULL);
 
    x = (allow? strlen(allow) : 0) + (deny? strlen(deny) : 0) + 2;
 
    answer = (char *) MALLOC(x);
    grouplist = new_strlist(x);
 
    x=0; y=0;
    if(allow)  {
        tmp = strtok(allow, ",");
        grouplist[x++] = strdup(tmp);
 
        while((tmp = strtok(NULL, ",")))  {
            for(y=0; (grouplist[y] &&
                      (strlen(grouplist[y]) < strlen(tmp))); y++);
            if(x == y)
                grouplist[x++] = strdup(tmp);
            else  {
                for(z=x; z>=y; z--)
                    grouplist[z]=grouplist[z-1];
                grouplist[y] = strdup(tmp);
                x++;
            }
        }
    }
 
    if(deny)  {
        tmp = strtok(deny, ",");
        for(y=0; (grouplist[y] &&
                  (strlen(grouplist[y]) < strlen(tmp))); y++);
        if(x == y)
            grouplist[x++] = strdup(tmp);
        else  {
            for(z=x; z>=y; z--)
                grouplist[z]=grouplist[z-1];
            grouplist[y] = strdup(tmp);
            x++;
        }
 
        while((tmp = strtok(NULL, ",")))  {
            for(y=0; (grouplist[y] &&
                      (strlen(grouplist[y]) < strlen(tmp))); y++);
            if(x == y)
                grouplist[x++] = strdup(tmp);
            else  {
                for(z=x; z>=y; z--)
                    grouplist[z]=grouplist[z-1];
                grouplist[y] = strdup(tmp);
                x++;
            }
        }
    }
 
    answer[0] = '\0';
    for(x=0; grouplist[x]; x++)  {
        strcat(answer, grouplist[x]);
        strcat(answer, ",");
    }
    answer[strlen(answer) - 1] = '\0';
 
    return answer;
}
 
void feed_set_groups(char *host, char *groups)
{
    char *current = feed_get_host_var(host);
    char *target;
    register int x=0, y=0;
    int found;
 
    if(current)  {
        target = (char *) MALLOC((groups ? strlen(groups) : 0)
			         +strlen(current)+2);
        while((current[x]) && (current[x] != ':'))
            target[y++] = current[x++];
        target[y++] = ':';
        target[y] = '\0';
        x++;
 
	if (groups) {
            strcat(target,groups);
            y+=strlen(groups);
	}
 
        while((current[x]) && (current[x] != ':'))
            x++;
 
        while(current[x])
            target[y++] = current[x++];
        target[y]='\0';
    }  else  {
        /* 3 = the three colons.  6 = "Tf,Wnm" */
        target = (char *) MALLOC((groups ? strlen(groups) : 0)
				 +strlen(host)+3+6+2);
        sprintf(target, "%s:%s:%s:", host, (groups ? groups : ""),
                                     strcmp(host, "ME") ? "Tf,Wnm" : "");
    }
 
    /* Ick.  Find where it was before (if it was.)  If it was somewhere  *
     * before, put it there.  If not, add it to the end.  The icky part? *
     * If it was there before, but it was broken onto multiple lines,    *
     * put it all on one line.  Hrmph. */
    x=0; y=0; found=0;
    while(feedfile[x])  {
        int idlen;
 
        idlen = strlen(host);  
 
        if(!strncmp(feedfile[x], host, idlen) &&
           (feedfile[x][idlen] == ':' ||
            feedfile[x][idlen] == '\\'))  {
 
            found = 1;
            y = strlen(feedfile[x]) - 1;  if(y<0) y=0;
            if(feedfile[x][y] == '\\')  {
                int p = x + 1, q;
 
                while(feedfile[p])  {
                    q = strlen(feedfile[p]) - 1;  if(q<0) q=0;
                    feedfile[p][0] = '\0';
 
                    if(feedfile[p][q] == '\\')
                        ++p;
                    else
                        break;
                }
            }
            feedfile[x] = target;
            break;
        }  else
            x++;
    }
    if(!found)  {
        int feedlen = 0;
 
        for(; feedfile[feedlen]; feedlen++);
        feedfile = grow_strlist(feedfile, feedlen+2);
 
        x=0; y=feedlen+1;
 
    /* Just tack it on the end if it's anything but ME.  Put ME as the first
     * line after the initial comments. */
        if(!strcmp(host, "ME"))  {
            while(feedfile[x][0] == '#')
                x++;
            for(; y>=x; y--)
                feedfile[y+1] = feedfile[y];
            feedfile[x] = target;
        }  else  {
            feedfile[y-1] = target;
            feedfile[y] = NULL;
        }
    }
}
 
void feed_set_entry(char *id, char *ngroups, char *feedtype, char *params)
{
    char *target;
    register int x=0, y=0;
    int found;

    x += (id)       ? strlen(id)       : 0;
    x += (ngroups)  ? strlen(ngroups)  : 0;
    x += (feedtype) ? strlen(feedtype) : 0;
    x += (params)   ? strlen(params)   : 0;
 
    target = (char *) MALLOC(x+2);
    if(!target) report_error(MEMORY_ERROR, "Out of memory", 
                             "Could not allocate memory.  Please try again.");
 
    sprintf(target, "%s:%s:%s:%s", (id) ? id : "", 
                                   (ngroups) ? ngroups : "",
                                   (feedtype) ? feedtype : "", 
                                   (params) ? params: "");

    /* Ick.  Find where it was before (if it was.)  If it was somewhere  *
     * before, put it there.  If not, add it to the end.  The icky part? *
     * If it was there before, but it was broken onto multiple lines,    *
     * put it all on one line.  Hrmph. */
    x=0; y=0; found=0;
    while(feedfile[x])  {
        int idlen;

        idlen = strlen(id);

        if(!strncmp(feedfile[x], id, idlen) &&
           (feedfile[x][idlen] == ':' ||
            feedfile[x][idlen] == '\\'))  {

            found = 1;
            y = strlen(feedfile[x]) - 1;  if(y<0) y=0;
            if(feedfile[x][y] == '\\')  {
                int p = x + 1, q;
 
                while(feedfile[p])  {
                    q = strlen(feedfile[p]) - 1;  if(q<0) q=0;
                    feedfile[p][0] = '\0';
 
                    if(feedfile[p][q] == '\\')
                        ++p;
                    else
                        break;
                }
            }
            feedfile[x] = target;
            break;
        }  else
            x++;
    }
    if(!found)  {
        int feedlen = 0;
 
        for(; feedfile[feedlen]; feedlen++);
        feedfile = grow_strlist(feedfile, feedlen+2);
 
        x=0; y=feedlen+1;
 
    /* Just tack it on the end if it's anything but ME.  Put ME as the first
     * line after the initial comments. */
        if(!strcmp(id, "ME"))  {
            while(feedfile[x][0] == '#')
                x++;
            for(; y>=x; y--)
                feedfile[y+1] = feedfile[y];
            feedfile[x] = target;
        }  else  {
            feedfile[y-1] = target;
            feedfile[y] = NULL;
        }
    }
}

void feed_delete_host(char *host)
{
    register int x=0, y=0, found=0;
 
    while(feedfile[x])  {
        int idlen;
 
        idlen = strlen(host);
 
        if(!strncmp(feedfile[x], host, idlen) &&
           (feedfile[x][idlen] == ':' ||
            feedfile[x][idlen] == '\\'))  {
 
            found = 1;
            y = strlen(feedfile[x]) - 1;  if(y<0) y=0;
            if(feedfile[x][y] == '\\')  {
                int p = x + 1, q;
 
                while(feedfile[p])  {
                    q = strlen(feedfile[p]) - 1;  if(q<0) q=0;
                    feedfile[p][0] = '\0';
 
                    if(feedfile[p][q] == '\\')
                        ++p;
                    else
                        break;
                }
            }
            for(y=0; feedfile[y]; y++);
            for(x++; x<=y; x++)
                feedfile[x-1] = feedfile[x];
            feedfile[y]=NULL;
            break;
        }  else x++;
    }
    if(!found) report_error(INCORRECT_USAGE, "Could not find that host",
                            "Could not find that host in the feed file.");
}

void
feed_dump_vars(char *feedtype, char *dest)
{
    int x = 0, gotone = 0, isdef = 0;
    char *item, *tmp;
    char *hostname, *newsgroups, *types, *params, *allow, *deny;
    char ftype, *p;
 
    while(feedfile[x])  {
        if((feedfile[x][0] != '#') &&
           (feedfile[x][0] != '\0') &&
           (feedfile[x][0] != '\n'))  {
 
            item = feed_get_ind_var(&x);
            tmp = strchr(item, ':');
            if(!tmp) goto next;
            *tmp++ = '\0';
            hostname = item;
            newsgroups=tmp;
            tmp=strchr(newsgroups, ':');
            if(!tmp) goto next;
            *tmp++ = '\0';
            types=tmp;
            tmp=strchr(types, ':');
            *tmp++ = '\0';
            params=tmp;
 
	    if(!strcmp(feedtype, "ME")) {
		isdef = 1;
		if (strcmp(hostname, "ME"))
		    goto next;
	    } else {
                if(!strcmp(hostname, "ME"))
                    goto next;
                if(!strstr(types, feedtype))
                    goto next;
	    }
 
            if(!gotone)  {
                fprintf(stdout, "<pre><hr>\n");
                gotone = 1;
            }
 
	    if (isdef)
                fprintf(stdout, "By <b>default</b>:\n");
	    else
                fprintf(stdout, "For host <b>%s</b>:\n", hostname);


	    /* Parse newsgroups */
	    p = parsesublist(newsgroups, &ftype);
	    switch(ftype) {
		case NSSUB_ALL_GROUPS:
		    fprintf(stdout, "  Feed <b>all</b> groups\n");
		    break;
        	case NSSUB_ALL_EXCEPT:
		    fprintf(stdout, "  Feed all groups"
				    " except: <b>%s</b>\n", p);
		    break;
        	case NSSUB_JUST_THESE:
		    fprintf(stdout, "  Feed only groups: <b>%s</b>\n", p);
		    break;
        	case NSSUB_NO_GROUPS:
		    fprintf(stdout, "  Feed <b>no</b> groups\n");
		    break;
		case NSSUB_INN_STYLE:
		    fprintf(stdout, "  Feed groups: <b>%s</b>\n", p);
		    break;
		case NSSUB_USE_DEFAULTS:
	            if (isdef)
		        fprintf(stdout,
			        "  Feed <b>no</b> groups\n");
		    else
		        fprintf(stdout,
			        "  Feed <b>default</b> groups only\n");
		    break;
		case NSSUB_UNKNOWN_TYPE:
		    fprintf(stdout, "  <b>UNPARSABLE FEED LIST</b>\n");
		    break;
	    }

            if(*params != '\0')  {
                fprintf(stdout, (!strstr(feedtype, "Tp")) ? 
                            "  With these parameters:            <b>%s</b>\n" :
                            "  Run this program:                 <b>%s</b>\n",
                            params);
            }
	    if (isdef) {
                fprintf(stdout, "    <b><a href=%s?hn=ME>Edit</a>  ", dest);
		fprintf(stdout, "</pre>\n");
		return;
	    }
	    else
                fprintf(stdout, "    <b><a href=%s?hn=%s>Edit</a>  "
                                "<a href=%s?remove=yes&hn=%s>Remove</a></b>\n",
                                dest, hostname, dest, hostname);
            fprintf(stdout, "\n<hr>\n");
        }
next:
        x++;
    }
    if(gotone) fprintf(stdout, "</pre>\n");
}
 
void old_feed_dump_vars(char *feedtype, char *dest)
{
    int x = 0, gotone = 0;
    char *item, *tmp;
    char *hostname, *newsgroups, *types, *params, *allow, *deny;
 
    while(feedfile[x])  {
        if((feedfile[x][0] != '#') &&
           (feedfile[x][0] != '\0') &&
           (feedfile[x][0] != '\n'))  {
 
            item = feed_get_ind_var(&x);
            tmp = strchr(item, ':');
            if(!tmp) goto onext;
            *tmp++ = '\0';
            hostname = item;
            newsgroups=tmp;
            tmp=strchr(newsgroups, ':');
            if(!tmp) goto onext;
            *tmp++ = '\0';
            types=tmp;
            tmp=strchr(types, ':');
            *tmp++ = '\0';
            params=tmp;
 
            if(!strcmp(hostname, "ME"))
                goto onext;
 
            if(!strstr(types, feedtype))
                goto onext;
 
            if(!gotone)  {
                fprintf(stdout, "<pre><hr><b><center>"
                                "Your current settings:"
                                "</center></b><hr>\n");
                gotone = 1;
            }
 
            fprintf(stdout, "For host <b>%s</b>:\n", hostname);
            feed_split_newsgroups(newsgroups, &allow, &deny);
 
            fprintf(stdout, "  Exchange these newsgroups:        <b>%s</b>\n",
                            allow ? allow : "");
            fprintf(stdout, "  Do not exchange these newsgroups: <b>%s</b>\n",
                            deny ? deny : "");
            if(*params != '\0')  {
                fprintf(stdout, (!strstr(feedtype, "Tp")) ? 
                            "  With these parameters:            <b>%s</b>\n" :
                            "  Run this program:                 <b>%s</b>\n",
                            params);
            }
            fprintf(stdout, "    <b><a href=%s?hn=%s>Edit</a>  "
                            "<a href=%s?remove=yes&hn=%s>Remove</a></b>\n",
                            dest, hostname, dest, hostname);
            fprintf(stdout, "<hr>\n");
        }
onext:
        x++;
    }
    if(gotone) fprintf(stdout, "</pre>\n");
}
 
/* Add a host to hosts.nntp.  If it's already there, don't add it. */
void nnhost_add(char *hn) 
{
    char **nscnf = scan_nsnews_admin(NULL);
    char *hf = find_nsnews_var("_PATH_INNDHOSTS", nscnf);
    char *target = (char *) MALLOC(strlen(hn)+1);
    char **nhosts;
    register int x;

    sprintf(target, "%s:", hn);

    if(!hf)
        report_error(FILE_ERROR, NULL,
                     "Could not find a path to your hosts.nntp file.");
    nhosts=scan_tech(hf);
    if(!nhosts)
        report_error(FILE_ERROR, NULL, "Could not read your hosts.nntp file.");

    for(x=0; nhosts[x]; x++)  {
        if(!strcmp(nhosts[x], target))
            return;
    }
    nhosts=grow_strlist(nhosts,x+2);
    nhosts[x] = target;
    nhosts[x+1] = NULL;

    write_tech(hf, nhosts);

    return;
}
 

/* Delete a host from hosts.nntp.  If it's not there, don't delete it.  
 * (This should, perhaps, generate an error.)  */
void nnhost_delete(char *hn)
{
    char **nscnf = scan_nsnews_admin(NULL);
    char *hf = find_nsnews_var("_PATH_INNDHOSTS", nscnf);
    char *target = (char *) MALLOC(strlen(hn)+1);
    char **nhosts;
    register int x;
 
    sprintf(target, "%s:", hn);
 
    if(!hf)
        report_error(FILE_ERROR, NULL,
                     "Could not find a path to your hosts.nntp file.");
    nhosts=scan_tech(hf);
    if(!nhosts)
        report_error(FILE_ERROR, NULL, "Could not read your hosts.nntp file.");
 
    for(x=0; nhosts[x]; x++)  {
        if(!strcmp(nhosts[x], target))
            goto gotit;
    }
/* not found */
    return;

gotit:
    for(; nhosts[x]; x++)  {
        nhosts[x] = nhosts[x+1];
    }

    write_tech(hf, nhosts);

    return;
}

/* Add a host to nntpsend.ctl.  If it's already there, don't add it. */
void nnctl_add(char *hn) 
{
    char **nscnf = scan_nsnews_admin(NULL);
    char *cf = find_nsnews_var("_PATH_NEWSLIB", nscnf);
    char *fn = (char *) MALLOC(strlen(cf)+strlen("nntpsend.ctl")+2); 

    char *target = (char *) MALLOC((strlen(hn)*2)+4);
    char **nnctl;
    register int x;

    sprintf(target, "%s:%s::", hn, hn);
    sprintf(fn, "%s%cnntpsend.ctl", cf, FILE_PATHSEP); 

    if(!fn)
        report_error(FILE_ERROR, NULL,
                     "Could not find a path to your nntpsend.ctl file.");
    nnctl=scan_tech(fn);
    if(!nnctl)
        report_error(FILE_ERROR,NULL, "Could not read your nntpsend.ctl file.");

    for(x=0; nnctl[x]; x++)  {
        if(!strcmp(nnctl[x], target))
            return;
    }
    nnctl=grow_strlist(nnctl,x+2);
    nnctl[x] = target;
    nnctl[x+1] = NULL;

    write_tech(fn, nnctl);

    return;
}
 

/* Delete a host from nntpsend.ctl.  If it's not there, don't delete it.  
 * (This should, perhaps, generate an error.)  */
void nnctl_delete(char *hn)
{
    char **nscnf = scan_nsnews_admin(NULL);
    char *cf = find_nsnews_var("_PATH_NEWSLIB", nscnf);
    char *fn = (char *) MALLOC(strlen(cf)+strlen("nntpsend.ctl")+2); 
    char *target = (char *) MALLOC((strlen(hn)*2)+4);
    char **nnctl;
    register int x;

    sprintf(target, "%s:%s::", hn, hn);
    sprintf(fn, "%s%cnntpsend.ctl", cf, FILE_PATHSEP);
 
    if(!fn)
        report_error(FILE_ERROR, NULL,
                     "Could not find a path to your nntpsend.ctl file.");
    nnctl=scan_tech(fn);
    if(!nnctl)
        report_error(FILE_ERROR,NULL, "Could not read your nntpsend.ctl file.");
 
    for(x=0; nnctl[x]; x++)  {
        if(!strcmp(nnctl[x], target))
            goto gotit;
    }
/* not found */
    return;

gotit:
    for(; nnctl[x]; x++)  {
        nnctl[x] = nnctl[x+1];
    }

    write_tech(fn, nnctl);

    return;
}

/* News authdb utilities ---------------------------------------- */
/* -------------------------------------------------------------- */

/*
 * XXXrs - "DBM_PATH" is only being used here for historical
 *         non-reasons.
 */

char *
get_active_news_authdb(char **nscnf)
{
    return (find_nsnews_var("DBM_PATH", nscnf));
}

void
set_active_news_authdb(char *name, char **nscnf)
{
    set_nsnews_var("DBM_PATH", name, nscnf);
}

char *
get_active_news_authdb_name(char **nscnf)
{
    char *p;
    char *name = get_active_news_authdb(nscnf);

    if ((p = strrchr(name, FILE_PATHSEP)) != NULL)
	name = ++p;
    else
	p = name;

    if (!p || *p == '\0')
	return NULL;
    return strdup(p);
}

void
output_active_news_authdb(char **nscnf)
{
    char *p;
    char *name = get_active_news_authdb_name(nscnf);

    if (name == NULL) 
        fprintf(stdout, "<b>NO DATABASE SELECTED</b>\n");
    else {
        fprintf(stdout, "<b>Active database: %s</b>\n", name);
	/* free(p); */
    }
}


/* Groups stuff (cut 'n paste from authdb.c) -------------------- */
/* -------------------------------------------------------------- */

/*
 * passfilter - function returns non-zero if the regular expression
 *              passed in matches the string passed in.
 */
static int
passfilter(char *str, char *regexp)
{
    if (!str)
        return 0; /* NULL string never matches */
    else if (!regexp)
        return 0; /* XXXrs - NULL regexp matches nothing */
    else
        return(!shexp_casecmp(str, regexp));
}

int
newsgroups_match(char *ng, char *regexp)
{
    return(passfilter(ng, regexp));
}

/*
 * Create a link to another newsgroups CGI:
 *     val - value text on the link
 *     targ - name of the CGI to start
 *     arg - argument to pass to the CGI
 */
static void
output_newsgroups_cgi_link(char *val, char *cgi, char *group)
{
    char line[BIG_LINE];

    sprintf(line, "%s?group=%s", cgi, group);
    printf("<a href=index?options+newsgroups+%s target='options'>%s</a>",
           util_uri_escape(NULL, line), val);
}

void
output_newsgroups_list(char **active, char *filter)
{
    char **list = active_collapsedlist(active, filter);
    int gotone = 0;
    register int x;

    for (x = 0; list[x]; x++) {
	if (!gotone) {
            printf("<table border=1><caption align=left>\n");
            printf("<b>Newsgroups:</b>");
            printf("</caption>\n");
	}
	gotone++;
        printf("<tr><td>");
        printf("<b>%s</b></td><td>", list[x]);
        output_newsgroups_cgi_link("Edit", "ngrped", list[x]);
        printf("</td><td>\n");
        output_newsgroups_cgi_link("Remove", "ngrprem", list[x]);
        printf("</td>\n");
	if (list[x][strlen(list[x])-1] == '*') {
            printf("</td><td>\n");
            output_newsgroups_cgi_link("Expand", "ngrplst", list[x]);
	}
    }
    if (gotone)
        printf("</table>\n");
    else
        printf("<b>There are no matching newsgroups configured.</b>");
}

void
output_full_newsgroups_list(char **active, char *filter)
{
    int gotone;
    register int x;
    char *p, *g;


    gotone = 0;
    for (x = 0; active[x]; x++) {
	p = g = active[x];
	while (*p && !isspace(*p))
	    p++;
	*p = '\0';

	if (*g && passfilter(g, filter)) {
	    if (!gotone) {
                printf("<table border=1><caption align=left>\n");
        	printf("<b>Newsgroups:</b>");
        	printf("</caption>\n");
	    }
	    gotone++;
            printf("<tr><td>");
            printf("<b>%s</b></td><td>", g);
            output_newsgroups_cgi_link("Edit", "ngrped", g);
            printf("</td><td>\n");
            output_newsgroups_cgi_link("Remove", "ngrprem", g);
            printf("</td>\n");
	}
    }
    if (gotone)
        printf("</table>\n");
    else
        printf("<b>There are no matching newsgroups configured.</b>");
}

/* Control article handling configuration  ---------------------- */
/* -------------------------------------------------------------- */

static char **ctlartfile;
 
void ctlart_read_file()
{
    char **nscnf = scan_nsnews_admin(NULL);
    char *ctlartpath = find_nsnews_var("_PATH_CONTROLCTL", nscnf);
 
    if(!ctlartpath)
        report_error(FILE_ERROR, NULL,
                     "Could not find a path to your control.ctl file.");
    ctlartfile=scan_tech(ctlartpath);
    if(!ctlartfile)
        report_error(FILE_ERROR, NULL, "Could not read your control.ctl file.");
}
 
void ctlart_write_file()
{
    char **nscnf = scan_nsnews_admin(NULL);
    char *ctlartpath = find_nsnews_var("_PATH_CONTROLCTL", nscnf);
 
    if(!ctlartpath)
        report_error(FILE_ERROR, NULL,
                     "Could not find a path to your control.ctl file.");
    write_tech(ctlartpath, ctlartfile);
}

static void ctlart_add_line(char *line)
{
    register int i;

    for (i = 0; ctlartfile[i]; i++)
	;
    ctlartfile = grow_strlist(ctlartfile,i+2);
    ctlartfile[i] = line;
    ctlartfile[i+1] = NULL;
}

/* ctlname and act are required.  snd and grp may be NULL */
int ctlart_get_line(char *ctlname, char **act, char *snd, char *grp, int len)
{
    char *n, *s, *g, *a;
    int i;

    for (i = 0; ctlartfile[i]; i++) {
	if (!strncmp(ctlartfile[i], ctlname, strlen(ctlname)))
	    break;
    }
    if ((n = ctlartfile[i]) == NULL) {
	if (snd)
	    *snd = '\0';
	if (grp)
	    *grp = '\0';
	*act = NULL;
	return 0; /* not found */
    }

    if ((s = strchr(n, ':')) == NULL)
	return -1;
    *s++ = '\0';
    if (!(*s) || (g = strchr(s, ':')) == NULL)
	return -1;
    *g++ = '\0';
    if (!(*g) || (a = strchr(g, ':')) == NULL)
	return -1;
    *a++ = '\0';
    if (!(*a))
	return -1;
    if (len && (strlen(s)+1 > len || strlen(g)+1 > len))
	return -1;
    if (snd != NULL)
        strcpy(snd, s);
    if (grp != NULL)
        strcpy(grp, g);
    *act = a; 
    return 1; /* found */
}

void ctlart_set_line(char *ctlname, char *snd, char *grp, char *act)
{
    char *p, *r;
    char line[BIG_LINE];
    int i;

    sprintf(line, "%s:%s:%s:%s", ctlname, snd ? snd : "*",
					  grp ? grp : "*", act);
    r = strdup(line);
    for (i = 0; ctlartfile[i]; i++) {
	if (!strncmp(ctlartfile[i], ctlname, strlen(ctlname)))
	    break;
    }
    if ((p = ctlartfile[i]) == NULL) {
	ctlart_add_line(r);
	return;
    }
    free(p);
    ctlartfile[i] = r;
}
#endif /* MCC_NEWS */
