/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * pcontrol.c: Start up HTTP server.  Shut down HTTP server.  Restart HTTP
 *             server.  Figure out if it's a remote or local server, do 
 *             the right thing.
 *
 * All blame to Mike McCool
 */

#include "libadmin/libadmin.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <base/ereport.h>

#define NBUF_SIZE 1024
/* #define DEBUG_TRACE 1 */
#undef DEBUG_TRACE

#ifdef XP_UNIX
#include <signal.h>
#include <ctype.h>
#include <sys/types.h>
#else
#include <windows.h>
#include <nt/regparms.h>

#if defined(MCC_ADMSERV)
#define PRODUCT_KEY "This is a garbage key that should not be here"
#endif

SC_HANDLE schService;
SC_HANDLE schSCManager;

int StartServer(int whichsrv); 
int StopandRestartServer(int whichsrv);
int StopServer(int whichsrv);
 
int StopNetsiteProgram(int whichsrv);
int StartNetsiteProgram(int whichsrv);

int StopNetsiteService(int whichsrv);
int StartNetsiteService(int whichsrv);

NSAPI_PUBLIC int StopAdmin();

int StopAdminProgram();
int StartAdminProgram();

int StopAdminService();
int StartAdminService();
#endif

/* Check to see if a given server is running. */
NSAPI_PUBLIC int is_server_running(int whichsrv)
{
#ifdef XP_UNIX
    char *pidlog;
    pid_t pid;
    int z;
    char *srvname=get_srvname(whichsrv);

    if(is_remote_server(whichsrv))
        return -1;

    pidlog = get_num_mag_var(whichsrv, "PidLog");  
    if(!pidlog) return -1;
    if(file_exists(pidlog))  {
        FILE *p = fopen(pidlog, "r");
        if(!p) goto false_alarm;
        if((fscanf(p, "%d\n", &z)) == -1)
            goto false_alarm;
        pid = (pid_t) z;
        if(kill(pid, 0) == -1) goto false_alarm;
 
        /* success! it must be running. */
        return 1;
false_alarm:
        delete_file(pidlog);
        return 0;
    }  else  {
        /* assume it's not running. */
        return 0;
    }
#elif defined(XP_WIN32)
    char *srvname = get_srvname(whichsrv);
    HANDLE hServerDoneEvent = NULL;
    char *szTemp = NULL;
    char szServiceName[MAX_PATH];
 
    /* For the mail-server, return -1 so there won't be any on/off button */
    if (strncmp(srvname, "mail-", 5) == 0)
       return -1;

    strcpy(szServiceName, srvname);

    /* watchdog.c creates a global event of this same name */
    if((hServerDoneEvent = OpenEvent(EVENT_ALL_ACCESS, TRUE, szServiceName)) != NULL) {
       CloseHandle(hServerDoneEvent);
       return 1;
    }
    if(GetLastError() == ERROR_ACCESS_DENIED)  /* it exists */
       return 1;
    /* assume it's not running. */
    return 0;
#endif
    return -1;
}

void _stat_error(char *errstr, int x, int print_errno)
{
    if(getenv("HTTP_NS_INTERACTIVE"))  {
        fprintf(stdout, "error: %s", errstr);
        if(print_errno)
            fprintf(stdout, " (%d: %s)", errno, system_errmsg());
        fprintf(stdout, "\n");
        fflush(stdout);
    }  else  {
        fprintf(stdout, "<FONT color=%s>[%s]:</FONT> %s",
                        HTML_ERRCOLOR, get_srvname(x), errstr);
        if(print_errno)
            fprintf(stdout, " (%d: %s)", errno, system_errmsg());
        fprintf(stdout, "<br>\n");
        fflush(stdout);
    }
}

int _stat_log(char *success, char *statline, int x, int *wasSuccess)
{
    int ans = 0;

    *wasSuccess = 0;
    if(getenv("HTTP_NS_INTERACTIVE"))  {
        fprintf(stdout, "%s\n", statline);
    }  else  {
#ifdef MCC_PROXY
        char *str = NULL;

        if( (!success) || (!strncmp(statline, success, strlen(success))) ||
            (str = strstr(statline, "info:")) ||
            (str = strstr(statline, "warning:")) ) {
#else
        if( (!success) || (!strncmp(statline, success, strlen(success))) ) {
#endif
	    *wasSuccess = 1;
            ans = 0;
#ifdef MCC_PROXY
            fprintf(stdout, "[%s]: %s<br>\n", get_srvname(x), str ? str : statline);
#else
            fprintf(stdout, "[%s]: %s<br>\n", get_srvname(x), statline);
#endif
            fflush(stdout);
            return ans;
        }  else  {
            ans = 1;
#ifdef MCC_PROXY
            if (!(str = strstr(statline, "failure:")) &&
                !(str = strstr(statline, "catastrophe:")) &&
                !(str = strstr(statline, "security:")) &&
                !(str = strstr(statline, "config:")))
                str = statline;
            fprintf(stdout, "<FONT color=%s>[%s]:</FONT> %s<br>\n",
                    HTML_ERRCOLOR, get_srvname(x), str);
#else
            fprintf(stdout, "<FONT color=%s>[%s]:</FONT> %s<br>\n",
                            HTML_ERRCOLOR, get_srvname(x), statline);
#endif
            fflush(stdout);
            return ans;
        }
    }
    return ans;
}


/* Attempts to start up the HTTP server.  Returns 0 on success, 1 on fail. */
/* Places startup errors in /tmp/startup.[whichsrv].[pid] */
NSAPI_PUBLIC int startup_http(int whichsrv, char *auth, char *passwd)
{
    char line[BIG_LINE];
    char *tmp = get_admcf_dir(whichsrv);
    int	seenSuccess;

#ifdef XP_UNIX
    FILE	*start;

    {
        char *pidlog = get_num_mag_var(whichsrv, "PidLog");
        char statfile[PATH_MAX];
        FILE *sf = NULL;
        int haderror = 0;

        if(!pidlog) {
            _stat_error("no pidlog file found.", whichsrv, 0);
            return 1;
        }
        if(file_exists(pidlog))  {
            pid_t pid;
            int z;
            FILE *p = fopen(pidlog, "r");
            if(!p)
                goto bad_pidlog;
            if ((fscanf(p, "%d\n", &z)) == -1) 
                goto bad_pidlog;
            pid = (pid_t) z;

            if(kill(pid, 0) == -1)
                goto bad_pidlog;

            /* Bong - server is running happily. */
            _stat_error("server is already running.", whichsrv, 0);
            return 1;

bad_pidlog:
            delete_file(pidlog);
        }

        sprintf(line, "%s", tmp);
        line[strlen(line) - 2] = '\0';
        tmp = strrchr(line, FILE_PATHSEP);
        tmp++;
        sprintf(statfile, "/tmp/startup.%d.%d", whichsrv, (int) getpid());
        sprintf(tmp, "start > %s 2>&1", statfile);
    
	start = popen( line, "w" );
	if ( passwd ) {
	  fprintf( start, "%s\n", passwd );
	}
	pclose( start );

        if( !(sf = fopen(statfile, "r")) )  {
            _stat_error("could not read status file.", whichsrv, 1);
            return 1;
        }
	seenSuccess = 0;
        while(fgets(line, BIG_LINE, sf))  {
	    int success;
	    int error;

	    error = _stat_log("startup: ", line, whichsrv, &success);
	    seenSuccess |= success;
	    if ( seenSuccess && error
#if defined( NS_ENTERPRISE ) || defined( NS_PERSONAL )
		 && strstr( line, "rror" )
#endif
		 ) {
	      haderror += error;
	    }
        }
        fclose(sf);
        unlink(statfile);

        if(!haderror && seenSuccess)  {
            log_change(TO_ADMIN, "Started up HTTP server %s", 
                       get_srvname(whichsrv));
            return(0);
        }  else  {
            return(1);
        }
    }
#else /* XP_WIN32 */
    {
        if (StartServer(whichsrv)) {
            return(1);
        } else {
	    int	dummy;

            _stat_log("startup: ", "startup: server started", whichsrv, &dummy);
           log_change(TO_ADMIN, "Started up HTTP server %s", 
                       get_srvname(whichsrv));
           return(0);
        }
    }
#endif /* XP_WIN32 */
    return 0;
}

int _restart_http(int whichsrv, char *passwd)
{
#ifdef XP_UNIX
    char *pidlog = get_num_mag_var(whichsrv, "PidLog");
    pid_t pid;
    char *chroot = get_num_mag_var(whichsrv, "Chroot");

    if(chroot)  {
        _stat_error("cannot restart chroot'ed server.", whichsrv, 0);
        return 1;
    }

    if(!pidlog) {
        _stat_error("no pidlog file found.", whichsrv, 0);
        return 1;
    }

    if(file_exists(pidlog))  {
        int z;
        FILE *p = fopen(pidlog, "r");
        if(!p)
            goto bad_pidlog;
        if ((fscanf(p, "%d\n", &z)) == -1)
            goto bad_pidlog;
        pid = (pid_t) z;

        if(kill(pid, 0) == -1)
            goto bad_pidlog;
 
        /* Bong - server is running happily. */
        goto good_pidlog;

bad_pidlog:
        delete_file(pidlog);
    }
    return startup_http(whichsrv, NULL, passwd);

good_pidlog:
    if( (kill(pid, SIGHUP)) == -1)  {
        _stat_error("could not send restart signal.", whichsrv, 1);
        return 1;
    }  else  {
        /* wait to see if it dies */
        if(kill(pid, 0) != -1)  {
	    int dummy;
            _stat_log("startup: ", "startup: server restarted", whichsrv, &dummy);
        }  else  {
            char *errlog=get_num_mag_var(whichsrv, "ErrorLog");
            char line[BIG_LINE];
            FILE *cmd;
            
            sprintf(line, "%s", errlog);
            escape_for_shell(line);
            errlog=STRDUP(line);

            sprintf(line, "tail -1 %s", errlog);
            cmd=popen(line, "r");
            if(!cmd)
                _stat_error("restart failed (could not read error log)", 
                            whichsrv, 1);
            while(fgets(line, BIG_LINE, cmd))  {
                _stat_error(line, whichsrv, 0);
            }
            pclose(cmd);
            return 1;
        }
        return 0;
    }
#else /* XP_WIN32 */
    if (StopandRestartServer(whichsrv)) {
        return(1);
    } else {
        int	dummy;

        _stat_log("startup: ", "startup: server restarted", whichsrv, &dummy);
       return(0);
    }
#endif /* XP_WIN32 */
}

int _shutdown_http(int whichsrv)
{
#ifdef XP_UNIX
    char *pidlog = get_num_mag_var(whichsrv, "PidLog");
    pid_t pid;
 
    if(!pidlog) {
        _stat_error("no pidlog file found.", whichsrv, 0);
        return 1;
    }
 
    if(file_exists(pidlog))  {
        int z;
        FILE *p = fopen(pidlog, "r");
        if(!p)
            goto bad_pidlog;
        if ((fscanf(p, "%d\n", &z)) == -1)
            goto bad_pidlog;
        pid = (pid_t) z;
 
        if(kill(pid, 0) == -1)
            goto bad_pidlog;
 
        /* Bong - server is running happily. */
        goto good_pidlog;
 
bad_pidlog:
        delete_file(pidlog);
        _stat_error("server is not running.", whichsrv, 0);
        return 1;
    }  else  {
        _stat_error("server is not running.", whichsrv, 0);
        return 1;
    }
 
good_pidlog:
    if( (kill(pid, SIGTERM)) == -1)  {
        _stat_error("could not shut down server.", whichsrv, 1);
        return 1;
    }  else  {
       if (kill(pid, 0) == -1) {
	   int	dummy;

           _stat_log("shutdown: ", "shutdown: server shut down", whichsrv, &dummy);
           return 0;
       } else {
           int count = 0;
           while (kill(pid, 0) != -1) {
              PR_Sleep(PR_SecondsToInterval(2));
              if (count++ > 30) {
                 _stat_error("server failed to shutdown.", whichsrv, 1);
                 return 1;
              }
           }
	   {
	     int	dummy;

	     _stat_log("shutdown: ", "shutdown: server shut down", whichsrv, &dummy);
	   }
           return 0;
       }
    }
#else /* XP_WIN32 */
    if (StopServer(whichsrv)) {
        return(1);
    } else {
        int	dummy;

        _stat_log("startup: ", "shutdown: server shut down", whichsrv, &dummy);
       return(0);
    }
#endif /* XP_WIN32 */
    return 0; /* shut the compiler warnings up */
}

NSAPI_PUBLIC int shutdown_http(int whichsrv, char *auth)
{
    {
        log_change(TO_ADMIN, "Server %s shutdown",
                             get_srvname(whichsrv));
        return _shutdown_http(whichsrv);
    }
}

NSAPI_PUBLIC int restart_http(int whichsrv, char *auth, char *passwd) 
{
    {
	/*
	 * It used to call _restart_http to send SIGHUP and it does not
	 * work for ES and this will actually better in the case of clean
	 * startup.
	 */
        log_change(TO_ADMIN, "Restarting server %s", get_srvname(whichsrv));
	(void) _shutdown_http(whichsrv);
        return startup_http(whichsrv, auth, passwd);
    }
}

#ifdef XP_WIN32

void WaitForServertoStop(char *szDoneEventName)
{
    HANDLE hServDoneEvent;
    int nRetry = 0;

    do
    {
        hServDoneEvent = OpenEvent(EVENT_ALL_ACCESS, FALSE, szDoneEventName);
        if(hServDoneEvent)
        {
            CloseHandle(hServDoneEvent);
            Sleep(1000);
        }
    } while(hServDoneEvent && nRetry < 240); /* bug 39159 */
    return;
}


NSAPI_PUBLIC BOOLEAN
IsService()
{
	LPSTR szCommandLine;

	// in 2.0, the IsService=Yes|No registry key determined if
	// the server was an app or server.  Now it is determined
	// by the -D (debug) command line option.  TODO: automate 
	// this further, -ahakim 11/20/96
	if(szCommandLine = GetCommandLine())
	{
		strlwr(szCommandLine);
		if(strstr(szCommandLine, "-d"))
			return FALSE;
	}
	return TRUE;
}

NSAPI_PUBLIC BOOLEAN
IsAdminService()
{
	LPSTR szCommandLine;

	// in 2.0, the IsService=Yes|No registry key determined if
	// the admin server was an app or server.  Now it is determined
	// by the -D (debug) command line option.  -ahakim 11/20/96
	if(szCommandLine = GetCommandLine())
	{
		strlwr(szCommandLine);
		if(strstr(szCommandLine, "-d"))
			return FALSE;
	}
	return TRUE;
}

int
StartServer(int whichsrv)
{
    CHAR ErrorString[512];
    BOOLEAN Service;

    /* First figure out if the server is a service or an exe */
    Service = IsService();

     if(Service) {
        if (!(schSCManager = OpenSCManager(
                NULL,                   // machine (NULL == local)
                NULL,                   // database (NULL == default)
                SC_MANAGER_ALL_ACCESS   // access required
                ))) {
            sprintf(ErrorString, 
                "Error: Could not open the ServiceControlManager:%d "
                "Please restart the server %s from the Services Program Item "
                "in the Control Panel", get_srvname(whichsrv), GetLastError());
            _stat_error (ErrorString, whichsrv, 0);
            return 1;  
        }
        return(StartNetsiteService(whichsrv));
    } else {
        return(StartNetsiteProgram(whichsrv));
    }
}
 
int
StopandRestartServer(int whichsrv)
{
    CHAR ErrorString[512];
    BOOLEAN Service;


    /* First figure out if the server is a service or an exe */
    Service = IsService();
     if(Service) {
        if (!(schSCManager = OpenSCManager(
                NULL,                   // machine (NULL == local)
                NULL,                   // database (NULL == default)
                SC_MANAGER_ALL_ACCESS   // access required
                ))) {
            sprintf(ErrorString, 
                "Error: Could not restart server."
                "Please restart the server %s from the Services Program Item "
                "in the Control Panel", get_srvname(whichsrv));
            _stat_error (ErrorString, whichsrv, 0);
            return 1;  
        }
        if (StopNetsiteService(whichsrv))
            return 1;

        return(StartNetsiteService(whichsrv));
    } else {
        if (StopNetsiteProgram(whichsrv))
            return 1;
        return(StartNetsiteProgram(whichsrv));
    }
 
}
 
int
StopServer(int whichsrv)
{
    CHAR ErrorString[512];
    BOOLEAN Service;

    /* First figure out if the server is a service or an exe */
    Service = IsService();

    if(Service) {
        if (!(schSCManager = OpenSCManager(
                NULL,                   // machine (NULL == local)
                NULL,                   // database (NULL == default)
                SC_MANAGER_ALL_ACCESS   // access required
                ))) {
            sprintf(ErrorString, 
                "Error: Could not open the ServiceControlManager:%d "
                "Please restart the server %s from the Services Program Item "
                "in the Control Panel", get_srvname(whichsrv), GetLastError());
            _stat_error (ErrorString, whichsrv, 0);
            return 1;  
        }
        return(StopNetsiteService(whichsrv));
    } else {
        return(StopNetsiteProgram(whichsrv));
    }
}

int
StartNetsiteProgram(int whichsrv)
{
    char line[BIG_LINE], cmd[BIG_LINE];
    char *tmp = get_admcf_dir(whichsrv);

	CHAR ErrorString[512];
	STARTUPINFO siStartInfo;
	PROCESS_INFORMATION piProcInfo;
    FILE *CmdFile;
    int arglength;

    ZeroMemory(line, BIG_LINE);

    sprintf(line, "%s", tmp);
    line[strlen(line) - 2] = '\0';
    tmp = strrchr(line, FILE_PATHSEP);
    tmp++;
    sprintf(tmp, "startsrv.bat");
    

    CmdFile = fopen(line, "r");
    if (!CmdFile) {
        sprintf(ErrorString, "Error:Tried to start Fedora server %s "
            ": Could not open the startup script %s :Error %d. Please "
            "run startsrv.bat from the server's root directory.",
            get_srvname(whichsrv), line, errno);
            _stat_error (ErrorString, whichsrv, 0);
            return 1;
    }

    ZeroMemory(cmd, BIG_LINE);
    if (!fread(cmd, 1, BIG_LINE, CmdFile)) {
        sprintf(ErrorString, "Error:Tried to start Fedora server %s "
            ": Could not read the startup script %s :Error %d. Please "
            "run startsrv.bat from the server's root directory.",
            get_srvname(whichsrv), line, errno);
            _stat_error (ErrorString, whichsrv, 0);
            return 1;
    }
    
	ZeroMemory(&siStartInfo, sizeof(STARTUPINFO));
	siStartInfo.cb = sizeof(STARTUPINFO);
	siStartInfo.lpReserved = siStartInfo.lpReserved2 = NULL;
	siStartInfo.cbReserved2 = 0;
	siStartInfo.lpDesktop = NULL;

	if (!CreateProcess(NULL, cmd, NULL, NULL, FALSE,
		0, NULL, NULL, &siStartInfo, &piProcInfo)) {
        sprintf(ErrorString, "Error:Tried to start Fedora server %s "
            ": Could not start up the startup script %s :Error %d. Please "
            "run startsrv.bat from the server's root directory.",
            get_srvname(whichsrv), line, GetLastError());
            _stat_error (ErrorString, whichsrv, 0);
            return 1;
	} 

	CloseHandle(piProcInfo.hProcess);
	CloseHandle(piProcInfo.hThread);
    return 0;
}

int
StopNetsiteProgram(int whichsrv)
{
    HANDLE hEvent;
    CHAR ErrorString[512];
    char *servid = get_srvname(whichsrv);

    hEvent = CreateEvent(NULL, TRUE, FALSE, servid);  
    if(!SetEvent(hEvent)) {
        sprintf(ErrorString, "Tried to stop existing Fedora server %s"
            ": Could not signal it to stop :Error %d",
            servid, GetLastError());
            _stat_error (ErrorString, whichsrv, 0);
            return 1;
    }
    return 0;  
}

int
StopNetsiteService(int whichsrv)
{
    BOOL ret;
    SERVICE_STATUS ServiceStatus;
    DWORD Error;
    CHAR ErrorString[512];
    char *serverName = get_srvname(whichsrv);
    char *szTemp = NULL;
    char szServiceName[MAX_PATH];
 
    strcpy(szServiceName, serverName);

    schService = OpenService(schSCManager, szServiceName, SERVICE_ALL_ACCESS);
 
    if (schService == NULL) {
        sprintf(ErrorString, "Tried to stop Fedora service"
            " %s: Error %d. Please"
            " stop the server from the Services Item in the Control Panel",
            szServiceName, GetLastError());
            _stat_error (ErrorString, whichsrv, 0);
            return 1;
    }
 
    ret = ControlService(schService, SERVICE_CONTROL_STOP, &ServiceStatus);
 
    if (ret) {
        CloseServiceHandle(schService);
        /* We make sure that the service is stopped */
        WaitForServertoStop(szServiceName);
        return 0;
    } else if (( Error = GetLastError()) != ERROR_SERVICE_NOT_ACTIVE) {
        sprintf(ErrorString, "Tried to stop Fedora service"
            " %s: Error %d."
            " Please stop the server from the Services Item in the"
            " Control Panel", szServiceName, Error);
            _stat_error (ErrorString, whichsrv, 0);
            return 1;
    }
    return 0;
}
 
 
int
StartNetsiteService(int whichsrv)
{
    CHAR ErrorString[512];
    int retries = 0;
    SERVICE_STATUS lpssServiceStatus;
    BOOL ret;
    char *serverName = get_srvname(whichsrv);
    char *szTemp = NULL;
    char szServiceName[MAX_PATH];
 
    strcpy(szServiceName, serverName);

    schService = OpenService(
                    schSCManager,               // SCManager database
                    szServiceName,                // name of service
                    SERVICE_ALL_ACCESS);
 
    if (schService == NULL) {
        CloseServiceHandle(schService);
        sprintf(ErrorString, "Tried to start"
            " the Fedora service %s: Error %d. Please"
            " start the server from the Services Item in the Control Panel",
            szServiceName, GetLastError());
            _stat_error (ErrorString, whichsrv, 0);
            return 1;
    }
 
    if (!StartService(schService, 0, NULL)) {
        CloseServiceHandle(schService);
        sprintf(ErrorString, "StartNetsiteService:Could not start "
            "the Netsite service %s: Error %d. Please restart the server "
            "from the Services Item in the Control Panel",
            szServiceName, GetLastError());
        _stat_error (ErrorString, whichsrv, 0);
        return 1;
    }
    CloseServiceHandle(schService);
    return 0;
}


int
StopAdminProgram()
{
    HANDLE hEvent;
    CHAR ErrorString[512];

    hEvent = CreateEvent(NULL, TRUE, FALSE, ADM_ID_SERVICE "-serv");
    if(!SetEvent(hEvent)) {
        sprintf(ErrorString, "Tried to stop Administration server "
            ": Could not signal it to stop :Error %d", GetLastError());
            /* _stat_error (ErrorString, 0, 0); */
            return 1;
    }
    return 0;  
}

int
StopAdminService()
{
    BOOL ret;
    SERVICE_STATUS ServiceStatus;
    DWORD Error;
    CHAR ErrorString[512];
 
    schService = OpenService(schSCManager, ADM_ID_SERVICE "-serv", SERVICE_ALL_ACCESS);
 
    if (schService == NULL) {
        sprintf(ErrorString, "Tried to stop Fedora service"
            ": Could not open service: Error %d. Please "
            "stop the server from the Services Item in the Control Panel",
            GetLastError());
            /* _stat_error (ErrorString, whichsrv, 0); */
            return 1;
    }
 
    ret = ControlService(schService, SERVICE_CONTROL_STOP, &ServiceStatus);
 
    if (ret) {
        CloseServiceHandle(schService);
        return 0;
    } else if (( Error = GetLastError()) != ERROR_SERVICE_NOT_ACTIVE) {
        sprintf(ErrorString, "Tried to stop Fedora service"
            "StopService:could not send Control to service Error %d."
            "Please stop the server from the Services Item in the "
            "Control Panel", Error);
            /* _stat_error (ErrorString, whichsrv, 0); */
            return 1;
    }
    return 0;
}
 
NSAPI_PUBLIC int
StopAdmin()
{
    CHAR ErrorString[512];
    BOOLEAN Service;

    /* First figure out if the admin server is a service or an exe */
    Service = IsAdminService();

    if(Service) {
        if (!(schSCManager = OpenSCManager(
                NULL,                   // machine (NULL == local)
                NULL,                   // database (NULL == default)
                SC_MANAGER_ALL_ACCESS   // access required
                ))) {
            sprintf(ErrorString, 
                "Error: Could not open the ServiceControlManager:%d "
                "Please restart the admin server from the Services Program Item "
                "in the Control Panel", GetLastError());
            /* _stat_error (ErrorString, whichsrv, 0); */
            return 1;  
        }
        return(StopAdminService());
    } else {
        return(StopAdminProgram());
    }
}

#endif /* XP_WIN32 */


/* SNMP startup, restart and shutdown functions */

NSAPI_PUBLIC int startup_snmp(void)
{
    return 0;
}

NSAPI_PUBLIC int restart_snmp(void)
{
    return 0;
}

NSAPI_PUBLIC int shutdown_snmp(void)
{
    return 0;
}

