/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/* 
 * userdb.c:  Files to help with user database handling
 *            
 * All blame to Mike McCool
 */

#ifdef XP_UNIX
#include <dirent.h>
#include "mcom_ndbm.h"
#endif /* WIN32? */

#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <stdio.h>
#include "libadminutil/admutil.h"
#include "libadmin/libadmin.h"

#define BUF_SIZE 10

NSAPI_PUBLIC void list_udbs(char *fullpath, char *partialpath);

static char **list;
static int listsize;
static int curentry;

NSAPI_PUBLIC char **list_user_dbs(char *fullpath)
{
    list =  new_strlist(BUF_SIZE);
    listsize = BUF_SIZE;
    curentry = 0;
    list_udbs(fullpath, "");

    return(list);
}

NSAPI_PUBLIC void output_db_selector(char *path, char *element, char *current)  {
    char **pathlist = list_user_dbs(path);
    int currentnum = -1, found = 0, hascurrent = 0;
    register int x;
    char *noext, *tmp;

    hascurrent = (strcmp(current, "default"));
    for(x=0; pathlist[x]; x++)  {
        if( (hascurrent) && 
            (!strncmp(current, pathlist[x], strlen(pathlist[x]))) )
            currentnum = x;
        if(strrchr(pathlist[x], '.'))
            found = 1;
    }

    if(found)  {
        fprintf(stdout, "<SELECT name=\"%s\">\n", element);

        if(currentnum != -1)  {
            noext = STRDUP(pathlist[currentnum]);
            tmp = strrchr(noext, '.');
            if(tmp)  {
                *tmp++ = '\0';
                if((!strcmp(tmp, DBM_EXT_1)) || (!strcmp(tmp, NCSA_EXT)))  {
                    fprintf(stdout, "<OPTION value=\"%s\">%s\n", 
                                    pathlist[currentnum], noext);
                }
            }
            FREE(noext);
        }  else  {
            if(hascurrent)
                fprintf(stdout, "<OPTION value=\"%s\">%s\n", current, current);
            else
                fprintf(stdout, "<OPTION value=\"default\">NONE\n");
        }
        for(x=0; pathlist[x]; x++)  {
            if(x==currentnum) 
                continue;

            noext = STRDUP(pathlist[x]);
            tmp = strrchr(noext, '.');
            if(tmp)  {
                *tmp++ = '\0';
                if((!strcmp(tmp, DBM_EXT_1)) || (!strcmp(tmp, NCSA_EXT)))
                    fprintf(stdout, "<OPTION value=\"%s\">%s\n", 
                                    pathlist[x], noext);
            }
            FREE(noext);
        }
        fprintf(stdout, "</SELECT>");
    }  else  {
        fprintf(stdout, "<b>No databases found.</b>");
    }
}

NSAPI_PUBLIC void set_current_db(char *current)  
{
    char **config = get_adm_config();
    config[3] = STRDUP(current);
    write_adm_config(config);
}

void list_udbs(char *fullpath, char *partialpath)
{
    int stat_good;
    struct stat finfo;
    char **dirlisting;
    char *path = (char *)MALLOC(strlen(fullpath)+strlen(partialpath)+2);
 
    sprintf(path, "%s%c%s", fullpath, FILE_PATHSEP, partialpath);
    if( !(dirlisting = list_directory(path,0)))
        return;       
    else  {
        register int x;
        char *entry, *newppath;
 
        for(x=0; dirlisting[x]; x++)  {
            entry = (char *)MALLOC(strlen(path)+strlen(dirlisting[x])+2);
            sprintf(entry, "%s%s", path, dirlisting[x]);

#ifdef XP_UNIX
            stat_good = (lstat(entry, &finfo) == -1 ? 0 : 1);
#else /* WIN32 */
            stat_good = (stat(entry, &finfo) == -1 ? 0 : 1);
#endif /* XP_UNIX */ 
 
            if(!stat_good)
                continue;
            newppath = (char *)MALLOC(strlen(partialpath)+strlen(dirlisting[x])+3);
 
            if(S_ISDIR(finfo.st_mode))  {
                sprintf(newppath, "%s%s%c", partialpath, dirlisting[x],
                                  FILE_PATHSEP);
                list_udbs(fullpath, newppath);
            }  else  {
                sprintf(newppath, "%s%s", partialpath, dirlisting[x]);
                curentry++;

                if(!(curentry < listsize))  {
                    listsize += BUF_SIZE;
                    list = grow_strlist(list, listsize);
                }
                list[curentry-1] = STRDUP(newppath);
                list[curentry] = NULL;
            }
 
            FREE(entry);
            FREE(newppath);
        }
    }
} 

NSAPI_PUBLIC int detect_db_type(char *db_name)  {
    char *ext = strrchr(db_name, '.');
    char *period = ext;

    if(!ext) 
        return 0;

    *ext++ = '\0';

    if(!strcmp(ext, DBM_EXT_1))
        return IS_A_DBM;
    if(!strcmp(ext, NCSA_EXT))  {
        *period = '.';
        return IS_A_NCSA;
    }

/* duh, I dunno */
    return 0;
}

NSAPI_PUBLIC char *find_user_ncsa(char *db, char *user)  
{
    char line[BIG_LINE];
    char *answer = NULL;
    FILE *f;

    if(!(f = fopen(db, "r")))  {
        char line[BIG_LINE];
        sprintf(line, "Could not open the database <b>%s</b>.", db);
        report_error(FILE_ERROR, NULL, line);
    }
    while(fgets(line, BIG_LINE, f))  {
        if((!strncmp(user, line, strlen(user))) && (line[strlen(user)]==':')) {
            answer = STRDUP(line + (strlen(user) + 1));
            if(answer[strlen(answer)-1] == '\n')
                answer[strlen(answer)-1] = '\0';
        }
    }
    fclose(f);
    return answer;
}

NSAPI_PUBLIC char **list_users_ncsa(char *db)
{
    char line[BIG_LINE];
    char *user = NULL;
    FILE *f;
    int curuser = 0, listsize = BUF_SIZE;
    char **answer = new_strlist(listsize);
    answer[0] = NULL;

    if(!(f = fopen(db, "r")))  {
        char line[BIG_LINE];
        sprintf(line, "Could not open the database <b>%s</b>.", db);
        report_error(FILE_ERROR, NULL, line);
    }
    while(fgets(line, BIG_LINE, f))  {
        user = strtok(line, ":");
        if(user)  {
            curuser++;
            if(!(curuser < listsize))  {
                listsize += BUF_SIZE;
                answer = grow_strlist(answer, listsize);
            }
            answer[curuser-1] = STRDUP(user);
            answer[curuser] = NULL;
        }
    }
    fclose(f);
    return(answer);
}

NSAPI_PUBLIC void add_user_ncsa(char *db, char *user, char *pw, int enc)
{
    FILE *f;

    if(find_user_ncsa(db, user))  {
        char line[BIG_LINE];
        sprintf(line, "The user %s already exists in that database.", user);
        report_error(INCORRECT_USAGE, "User exists", line);
    }

    if(!(f = fopen_l(db, "a")))  {
        report_error(FILE_ERROR, db, 
                     "Could not open database file for writing.");
    }
    fprintf(f, "%s:%s\n", user, enc ? admin40_pw_enc(pw) : pw);
    fclose_l(f);

    if(remote_server_inlist())  {
        char *dbinc = (char *) MALLOC(strlen(db)+strlen(DB_INC)+4);
        char *tmp1 = STRDUP(db);
        char *tmp2 = strrchr(tmp1, '.');
 
        *tmp2 = '\0';
        sprintf(dbinc, "%s.%s", tmp1, DB_INC);

        if(!(f = fopen_l(dbinc, "a")))  {
            report_error(FILE_ERROR, dbinc, 
                         "Could not open database file for writing.");
        }
        fprintf(f, "%s:%s\n", user, enc ? admin40_pw_enc(pw) : pw);
        fclose_l(f);
        set_commit(0, 1);
    }

    log_change(TO_USERDB, "User %s added to NCSA-style database %s", 
               user, db);
}

NSAPI_PUBLIC int modify_user_ncsa(char *db, char *user, char *pw)
{
    char line[BIG_LINE], *escdb, tmp[128];
    FILE *f; 
    FILE *t; 
    int ans=0;
#ifdef XP_WIN32
	char tmpdir[512];
	DWORD result;
#endif /* XP_WIN32 */
    
#ifdef XP_UNIX    
    sprintf(tmp, "/tmp/dbtmp.%d", (int)getpid());
#else /* WIN32 */
	result = GetEnvironmentVariable("temp",tmpdir, 512);
	if (result > 512 || result == 0) {
		strcpy(tmpdir, "\\temp");
		CreateDirectory("\\temp", NULL);
	}	
    sprintf(tmp, "%s\\dbtmp.%d", tmpdir, GetCurrentProcessId());
#endif /* XP_WIN32 */

    if(!(f = fopen(db, "r")))  {
        report_error(FILE_ERROR, NULL, 
                     "Could not open database file for reading.");
    }
    if(!(t = fopen_l(tmp, "w")))  {
        report_error(FILE_ERROR, NULL, 
                     "Could not open tmp file for writing.");
    }
    while(fgets(line, BIG_LINE, f))  {
        if((!strncmp(user, line, strlen(user))) && (line[strlen(user)]==':')) {
            ans = 1;
            fprintf(t, "%s:%s\n", user, admin40_pw_enc(pw));
        }
        else
            fprintf(t, "%s", line);
    }
    fclose(f);
    fclose_l(t);

    escdb = (char *)MALLOC(2*strlen(db)+4);
    sprintf(escdb, "%s", db);
#ifdef XP_UNIX
    escape_for_shell(escdb);
    sprintf(line, "mv %s %s", tmp, escdb);
    system(line);
#else /* XP_WIN32 */
    CopyFile(tmp, escdb, FALSE);
#endif /* XP_WIN32 */

    if(ans)  {
        log_change(TO_USERDB, "User %s modified in NCSA-style database %s", 
                   user, db);

        if(remote_server_inlist())  {
            char *dbinc = (char *) MALLOC(strlen(db)+strlen(DB_INC)+4);
            char *tmp1 = STRDUP(db);
            char *tmp2 = strrchr(tmp1, '.');
        
            *tmp2 = '\0';
            sprintf(dbinc, "%s.%s", tmp1, DB_INC);

            if(!(f = fopen_l(dbinc, "a")))  {
                report_error(FILE_ERROR, dbinc,
                             "Could not open database file for writing.");
            }
            fprintf(f, "%s:%s\n", user, admin40_pw_enc(pw));
            fclose_l(f);
            set_commit(0, 1);
        }
    }
 

    return(ans);
}

NSAPI_PUBLIC int verify_adm_ncsa(char *db, char *pw)  
{
    char *adm = find_user_ncsa(db, "admin");

    if(adm)  {
        if(!pw)
            report_error(ELEM_MISSING, "Adminitrative password missing", 
                         "This database has an administrative password.  "
                         "You must give that password to affect the database.");

        if(pw_cmp(pw, adm))
            report_error(INCORRECT_USAGE, "Administrative password incorrect", 
                         "The administrative password you gave does "
                         "not match.  Please enter it again.");
        return 1;
    }  else
        return 0;
}

NSAPI_PUBLIC int remove_user_ncsa(char *db, char *user)  {
    char line[BIG_LINE], *escdb, tmp[128];
    FILE *f; 
    FILE *t; 
#ifdef XP_WIN32
	DWORD result;
	char tmpdir[512];
#endif /* XP_WIN32 */
    int ans=0;
    
#ifdef XP_UNIX    
    sprintf(tmp, "/tmp/dbtmp.%d", (int)getpid());
#else /* WIN32 */
	result = GetEnvironmentVariable("temp",tmpdir, 512);
	if (result > 512 || result == 0) {
		strcpy(tmpdir, "\\temp");
		CreateDirectory("\\temp", NULL);
	}	
    sprintf(tmp, "%s\\dbtmp.%d", tmpdir, GetCurrentProcessId());
#endif /* XP_UNIX */

    if(!(f = fopen(db, "r")))  {
        report_error(FILE_ERROR, NULL, 
                     "Could not open database file for reading.");
    }
    if(!(t = fopen_l(tmp, "w")))  {
        report_error(FILE_ERROR, NULL, 
                     "Could not open tmp file for writing.");
    }
    while(fgets(line, BIG_LINE, f))  {
        if((!strncmp(user, line, strlen(user))) && (line[strlen(user)]==':'))
            ans = 1;
        else
            fprintf(t, "%s", line);
    }
    fclose(f);
    fclose_l(t);

    escdb = (char *)MALLOC(2*strlen(db)+4);
    sprintf(escdb, "%s", db);
#ifdef XP_UNIX
    escape_for_shell(escdb);
    sprintf(line, "mv %s %s", tmp, escdb);
    system(line);
#else /* XP_UNIX */
	CopyFile(tmp, escdb, FALSE);
#endif /* XP_UNIX */

    if(ans)  {
        log_change(TO_USERDB, "User %s removed from NCSA-style database %s", 
                   user, db);
        if(remote_server_inlist())  {
            char *dbinc = (char *) MALLOC(strlen(db)+strlen(DB_INC)+4);
            char *tmp1 = STRDUP(db);
            char *tmp2 = strrchr(tmp1, '.');
 
            *tmp2 = '\0';
            sprintf(dbinc, "%s.%s", tmp1, DB_INC);
 
            if(!(f = fopen_l(dbinc, "a")))  {
                report_error(FILE_ERROR, dbinc,
                             "Could not open database file for writing.");
            }
            fprintf(f, "%s:%s\n", user, REMOVE_FROM_DB);
            fclose_l(f);
            set_commit(0, 1);
        }
    }
    return ans; 
}

#ifdef XP_UNIX /* WIN32 has no dbms */
char *find_user_dbm(char *db, char *user)
{
    DBM *d = dbm_open(db, O_RDWR, 0644);
    datum ud, pd;
    
    if(!d)  {
        char line[BIG_LINE];
        sprintf(line, "Could not open the database <b>%s</b>.", db);
        report_error(FILE_ERROR, NULL, line);
    }

    ud.dptr = user;
    ud.dsize = strlen(user) + 1;

    pd = dbm_fetch(d, ud);
    dbm_close(d);

    if(!pd.dptr)
        return NULL;
    else
        return STRDUP(pd.dptr);
}

void add_user_dbm(char *db, char *user, char *pw, int enc)
{
    DBM *d;
    datum nd, vd;

    if(find_user_dbm(db, user))
        report_error(INCORRECT_USAGE, "User exists", 
                             "That user already exists in that database.");

    if(!(d = dbm_open(db, O_RDWR, 0644)))  {
        report_error(FILE_ERROR, NULL, 
            "Could not open the database to add a user.");
    }
    nd.dptr = STRDUP(user);
    nd.dsize = strlen(user) + 1;
    vd.dptr = enc ? admin40_pw_enc(pw) : pw;
    vd.dsize = strlen(vd.dptr) + 1;
    if(dbm_store(d, nd, vd, DBM_INSERT))  {
        char line[BIG_LINE];
        dbm_close(d);
        sprintf(line, "Could not add the user %s to the database.", user);
        report_error(FILE_ERROR, NULL, line);
    }
    dbm_close(d);

    if(remote_server_inlist())  {
        char *dbinc = (char *) MALLOC(strlen(db)+strlen(DB_INC)+4);
        FILE *f = NULL;
 
        sprintf(dbinc, "%s.%s", db, DB_INC);
 
        if(!(f = fopen_l(dbinc, "a")))  {
            report_error(FILE_ERROR, dbinc,
                         "Could not open database file for writing.");
        }
        fprintf(f, "%s:%s\n", user, enc ? admin40_pw_enc(pw) : pw);
        fclose_l(f);
        set_commit(0, 1);
    }

    log_change(TO_USERDB, "User %s added to DBM %s", user, db);
}

char **list_users_dbm(char *db)  
{
    DBM *d;
    datum key;
    int curuser = 0, listsize = BUF_SIZE;
    char **answer = new_strlist(listsize);
    answer[0] = NULL;

    if(!(d = dbm_open(db, O_RDWR, 0644)))  {
        report_error(FILE_ERROR, NULL,
            "Could not open the database to list users.");
    }
    for(key = dbm_firstkey(d); key.dptr != NULL; key = dbm_nextkey(d))  {
        curuser++;
        if(!(curuser < listsize))  {
            listsize += BUF_SIZE;
            answer = grow_strlist(answer, listsize);
        }
        answer[curuser-1] = STRDUP(key.dptr);
        answer[curuser] = NULL;
    }
    return answer;
}

int modify_user_dbm(char *db, char *user, char *pw)
{
    DBM *d;
    datum nd, vd;
    int ans=1;

    if(!find_user_dbm(db, user))
        report_error(INCORRECT_USAGE, "User does not exist",
                             "That user does not exist in that database.");

    if(!(d = dbm_open(db, O_RDWR, 0644)))  {
        report_error(FILE_ERROR, NULL,
            "Could not open the database to modify a user.");
    }
    nd.dptr = STRDUP(user);
    nd.dsize = strlen(user) + 1;
    vd.dptr = admin40_pw_enc(pw);
    vd.dsize = strlen(vd.dptr) + 1;
    if(dbm_store(d, nd, vd, DBM_REPLACE))  {
        ans = 0;
    }
    dbm_close(d);
    if(ans)  {
        log_change(TO_USERDB, "User %s modified in DBM %s", user, db);
        if(remote_server_inlist())  {
            char *dbinc = (char *) MALLOC(strlen(db)+strlen(DB_INC)+4);
            FILE *f = NULL;
 
            sprintf(dbinc, "%s.%s", db, DB_INC);
 
            if(!(f = fopen_l(dbinc, "a")))  {
                report_error(FILE_ERROR, dbinc,
                             "Could not open database file for writing.");
            }
            fprintf(f, "%s:%s\n", user, admin40_pw_enc(pw));
            fclose_l(f);
            set_commit(0, 1);
        }
    }
 
    return ans;
}

int verify_adm_dbm(char *db, char *pw)  {
    char *adm = find_user_dbm(db, "admin");

    if(adm)  {
        if(!pw)
            report_error(ELEM_MISSING, "Adminitrative password missing", 
                         "This database has an administrative password.  "
                         "You must give that password to affect the database.");

        if(pw_cmp(pw, adm))
            report_error(INCORRECT_USAGE, "Administrative password incorrect", 
                         "The administrative password you gave does not match. "
                         "Please enter it again.");
        return 1;
    }  else
        return 0;
}

int remove_user_dbm(char *db, char *user)  {
    DBM *d = dbm_open(db, O_RDWR, 0644);
    datum ud;
    int ans = 1;

    if(!d)
        report_error(FILE_ERROR, NULL, 
            "Could not open the database.");

    ud.dptr = user;
    ud.dsize = strlen(user) + 1;
    if((dbm_delete(d, ud)) < 0)  {
        ans = 0;
    }
    dbm_close(d);

    if(ans)  {
        log_change(TO_USERDB, "User %s removed from DBM %s", user, db);
        if(remote_server_inlist())  {
            char *dbinc = (char *) MALLOC(strlen(db)+strlen(DB_INC)+4);
            FILE *f = NULL;
 
            sprintf(dbinc, "%s.%s", db, DB_INC);
 
            if(!(f = fopen_l(dbinc, "a")))  {
                report_error(FILE_ERROR, dbinc,
                             "Could not open database file for writing.");
            }
            fprintf(f, "%s:%s\n", user, REMOVE_FROM_DB);
            fclose_l(f);
            set_commit(0, 1);
        }
    }
    return ans;
}
#endif /* WIN32? */
