/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * Aruna Victor
 */

#include <nt/nsapi.h>

__declspec(dllexport) int InitSafTable(SafFunction **Table)
{
	SafTable = Table;
	return TRUE;
}

/* Functions from buffer.c */
	
filebuf *filebuf_open(SYS_FILE fd, int sz)
{
	return ((filebuf *)SafTable[FILEBUF_OPEN](fd, sz));
	
}

netbuf *netbuf_open(SYS_NETFD sd, int sz)
{
	return((netbuf *)SafTable[NETBUF_OPEN](sd, sz));
}

filebuf *filebuf_open_nostat(SYS_FILE fd, int sz, struct stat *finfo)
{
	return((filebuf *)SafTable[FILEBUF_OPEN_NOSTAT](fd, sz, finfo));
}

filebuf *pipebuf_open(SYS_FILE fd, int sz, struct stat *finfo)
{
	return(SafTable[PIPEBUF_OPEN](fd, sz, finfo));
}

void	pipebuf_close(filebuf *buf)
{
	(SafTable[PIPEBUF_CLOSE](buf));
}

int filebuf_next(filebuf *buf, int advance)
{
	return((int)SafTable[FILEBUF_OPEN_NOSTAT](buf, advance));
}

int netbuf_next(netbuf *buf, int advance)
{
	return((int)SafTable[NETBUF_NEXT](buf, advance));
}

int pipebuf_next(filebuf *buf, int advance)
{
	return((int)SafTable[PIPEBUF_NEXT](buf, advance));
}

void filebuf_close(filebuf *buf)
{
	(SafTable[FILEBUF_CLOSE](buf));
}

void netbuf_close(netbuf *buf)
{
	(SafTable[NETBUF_CLOSE](buf));
}

int filebuf_grab(filebuf *buf, int sz)
{
	return((int)SafTable[FILEBUF_GRAB](buf, sz));
}

int netbuf_grab(netbuf *buf, int sz)
{
	return((int)SafTable[NETBUF_GRAB](buf, sz));
}

int pipebuf_grab(filebuf *buf, int sz)
{
	return((int)SafTable[PIPEBUF_GRAB](buf, sz));
}

int netbuf_buf2sd(netbuf *buf, SYS_NETFD sd, int len)
{
	return((int)SafTable[NETBUF_BUF2SD](buf, sd, len));
}

int filebuf_buf2sd(filebuf *buf, SYS_NETFD sd)
{
	return((int)SafTable[FILEBUF_BUF2SD](buf, sd));
}


int pipebuf_buf2sd(filebuf *buf, SYS_NETFD sd, int len)
{
	return((int)SafTable[PIPEBUF_BUF2SD](buf, sd, len));
}

int pipebuf_netbuf2sd(netbuf *buf, SYS_FILE sd, int len)
{
	return((int)SafTable[PIPEBUF_NETBUF2SD](buf, sd, len));
}


/* Functions from ntdaemon.h */

#ifdef USE_NSPR
void _ntdaemon_run(void *d)
{
	SafTable[NTDAEMON_RUN](d);
}

#else /* !USE_NSPR */
unsigned  int __stdcall _ntdaemon_run(void *d)
{
	return((unsigned int)(SafTable[NTDAEMON_RUN](d)));
}
#endif /* !USE_NSPR */

void child_status(int code)
{
	(SafTable[CHILD_STATUS](code));
}

/* Functions from file.h */

int system_stat(char *name, struct stat *finfo)
{
	return((int)SafTable[SYSTEM_STAT](name, finfo));
}

int system_fread(SYS_FILE fd, char *buf, int sz)
{
	return((int)SafTable[SYSTEM_FREAD](fd, buf, sz));
}

int system_pread(SYS_FILE fd, char *buf, int sz)
{
	return((int)SafTable[SYSTEM_PREAD](fd, buf, sz));
}


SYS_FILE system_fopenRO(char *path)
{
	return((SYS_FILE)SafTable[SYSTEM_FOPENRO](path));
}

SYS_FILE system_fopenWA(char *path)
{
	return((SYS_FILE)SafTable[SYSTEM_FOPENWA](path));
}

SYS_FILE system_fopenRW(char *path)
{
	return((SYS_FILE)SafTable[SYSTEM_FOPENRW](path));
}

void system_fclose(SYS_FILE fd)
{
	(SafTable[SYSTEM_FCLOSE])(fd);
}

int system_nocoredumps(void)
{
	return((int)SafTable[SYSTEM_NOCOREDUMPS]());
}

int system_fwrite(SYS_FILE fd,char *buf,int sz)
{
	return((int)SafTable[SYSTEM_FWRITE](fd, buf, sz));
}

int system_fwrite_atomic(SYS_FILE fd, char *buf, int sz)
{
	return((int)SafTable[SYSTEM_FWRITE_ATOMIC](fd, buf, sz));
}

char *system_winerr(void)
{
	return((char *)SafTable[SYSTEM_WINERR]());
}

char *system_winsockerr(void)
{
	return((char *)SafTable[SYSTEM_WINSOCKERR]());
}

int file_notfound(void)
{
	return((int)SafTable[FILE_NOTFOUND]());
}


int system_initlock(SYS_FILE fd)
{
	return((int)SafTable[SYSTEM_INITLOCK](fd));
}


void file_unix2local(char *path, char *p2)
{
	(SafTable[FILE_UNIX2LOCAL](path, p2));
}

SYS_DIR dir_open(char *path)
{
	return(SafTable[DIR_OPEN](path));
}

SYS_DIRENT *dir_read(SYS_DIR ds)
{
	return(SafTable[DIR_READ](ds));
}

void dir_close(SYS_DIR ds)
{
	(SafTable[DIR_CLOSE](ds));
}


/* Functions from cinfo.h */
void cinfo_init(void)
{
	(SafTable[CINFO_INIT]());
}

void cinfo_terminate(void)
{
	(SafTable[CINFO_TERMINATE]());
}

char *cinfo_merge(char *fn)
{
	return(SafTable[CINFO_MERGE](fn));
}

cinfo *cinfo_find(char *uri)
{
	return(SafTable[CINFO_FIND](uri));
}

cinfo *cinfo_lookup(char *type)
{
	return(SafTable[CINFO_LOOKUP](type));
}

void cinfo_dump_database(FILE *dump)
{
	(SafTable[CINFO_DUMP_DATABASE](dump));
}


/* Functions from ereport.h */
int ereport(int degree, char *fmt, ...)
{
	char errstr[8192];
    va_list args;
    va_start(args, fmt);
    util_vsnprintf(errstr, -1, fmt, args);

	return((int)SafTable[EREPORT](degree, errstr));
}

char *ereport_init(char *err_fn, char *email, struct passwd *pw)
{
	return(SafTable[EREPORT_INIT](err_fn, email, pw));
}

void ereport_terminate(void)
{
	(SafTable[EREPORT_TERMINATE]());
}

SYS_FILE ereport_getfd(void)
{
	return(SafTable[EREPORT_GETFD]());
}


/* Functions from minissl.h */
int SSL_Close(int socket)
{
	return((int)SafTable[SSL_CLOSE](socket));
}

int SSL_Socket(int af, int type, int protocol)
{
	return((int)SafTable[SSL_SOCKET](af, type, protocol));
}

int SSL_GetSockOpt(int s, int level, int optname, 
	void *optval, int *optlen)
{
	return((int)SafTable[SSL_GET_SOCKOPT](s, level, optname, optval, optlen));
}

int SSL_SetSockOpt(int s, int level, int optname, 
	const void *optval, int optlen)
{
	return((int)SafTable[SSL_SET_SOCKOPT](s, level, optname, optval, optlen));
}

int SSL_Bind(int s, const void *addr, int addrlen)
{
	return((int)SafTable[SSL_BIND](s, addr, addrlen));
}

int SSL_Listen(int s, int backlog)
{
	return((int)SafTable[SSL_LISTEN](s, backlog));
}

int SSL_Accept(int s, void *sockaddr, int *namelen)
{
	return((int)SafTable[SSL_ACCEPT](s, sockaddr, namelen));
}

int SSL_Read(int s, void *buf, int len)
{
	return((int)SafTable[SSL_READ](s, buf, len));
}

int SSL_Write(int s, const void *buf, int len)
{
	return((int)SafTable[SSL_WRITE](s, buf, len));
}

int SSL_GetPeerName(int s, void *addr, int *addrlenp)
{
	return((int)SafTable[SSL_GETPEERNAME](s, addr, addrlenp));
}


/* Functions from net.h */
int net_bind(SYS_NETFD s, const struct sockaddr *name, int namelen)
{
	return((int)SafTable[NET_BIND](s, name, namelen));
}

int net_read(SYS_NETFD sd, char *buf, int sz, int timeout)
{
	return((int)SafTable[NET_READ](sd, buf, sz, timeout));
}

int net_write(SYS_NETFD sd, char *buf, int sz)
{
	return((int)SafTable[NET_WRITE](sd, buf, sz));
}



char *net_find_fqdn(struct hostent *p)
{
	return(SafTable[NET_FIND_FQDN](p));
}

char *net_ip2host(char *ip, int verify)
{
	return(SafTable[NET_IP2HOST](ip, verify));
}

int net_sendmail(char *to, char *subject, char *msg)
{
	return((int)SafTable[NET_SENDMAIL](to, subject, msg));
}


/* Functions from pblock.h */
pb_param *param_create(char *name, char *value)
{
	return(SafTable[PARAM_CREATE](name, value));
}

int param_free(pb_param *pp)
{
	return((int)SafTable[PARAM_FREE](pp));
}

pblock *pblock_create(int n)
{
	return(SafTable[PBLOCK_CREATE](n));
}

void pblock_free(pblock *pb)
{
	(SafTable[PBLOCK_FREE](pb));
}

char *pblock_findval(char *name, pblock *pb)
{
	return(SafTable[PBLOCK_FINDVAL](name, pb));
}

pb_param *pblock_nvinsert(char *name, char *value, pblock *pb)
{
	return(SafTable[PBLOCK_NVINSERT](name, value, pb));
}

pb_param *pblock_nninsert(char *name, int value, pblock *pb)
{
	return(SafTable[PBLOCK_NNINSERT](name, value, pb));
}

void pblock_pinsert(pb_param *pp, pblock *pb)
{
	(SafTable[PBLOCK_PINSERT](pp, pb));
}

int pblock_str2pblock(char *str, pblock *pb)
{
	return((int)SafTable[PBLOCK_STR2PBLOCK](str, pb));
}

char *pblock_pblock2str(pblock *pb, char *str)
{
	return(SafTable[PBLOCK_PBLOCK2STR](pb, str));
}

void pblock_copy(pblock *src, pblock *dst)
{
	(SafTable[PBLOCK_COPY](src, dst));
}

char **pblock_pb2env(pblock *pb, char **env)
{
	return(SafTable[PBLOCK_PB2ENV](pb, env));
}

pb_param *_pblock_fr(char *name, pblock *pb, int remove)
{
	return(SafTable[PBLOCK_FR](name, pb, remove));
}


/* Functions from sem.h */
SEMAPHORE sem_init(char *name, int number)
{
	return(SafTable[SEM_INIT](name, number));
}

void sem_terminate(SEMAPHORE id)
{
	(SafTable[SEM_TERMINATE](id));
}

int sem_grab(SEMAPHORE id)
{
	return((int)SafTable[SEM_GRAB](id));
}

int sem_tgrab(SEMAPHORE id)
{
	return((int)SafTable[SEM_TGRAB](id));
}

int sem_release(SEMAPHORE id)
{
	return((int)SafTable[SEM_RELEASE](id));
}


/* Functions from session.h */
Session *session_create(SYS_NETFD csd, struct sockaddr_in *sac)
{
	return(SafTable[SESSION_CREATE](csd, sac));
}

void session_free(Session *sn)
{
	(SafTable[SESSION_FREE](sn));
}

char *session_dns_lookup(Session *sn, int verify)
{
	return(SafTable[SESSION_DNS_LOOKUP](sn, verify));
}


/* Functions from shexp.h */
int shexp_valid(char *exp)
{
	return((int)SafTable[SHEXP_VALID](exp));
}

int shexp_match(char *str, char *exp)
{
	return((int)SafTable[SHEXP_MATCH](str, exp));
}

int shexp_cmp(char *str, char *exp)
{
	return((int)SafTable[SHEXP_CMP](str, exp));
}

int shexp_casecmp(char *str, char *exp)
{
	return((int)SafTable[SHEXP_CASECMP](str, exp));
}


/* Functions from systems.h */
int util_strcasecmp(CASECMPARG_T char *s1, CASECMPARG_T char *s2)
{
	return((int)SafTable[UTIL_STRCASECMP](s1, s2));
}

int util_strncasecmp(CASECMPARG_T char *s1, CASECMPARG_T char *s2, int n)
{
	return((int)SafTable[UTIL_STRNCASECMP](s1, s2, n));
}


/* Functions from util.h */
int util_getline(filebuf *buf, int lineno, int maxlen, char *l)
{
	return((int)SafTable[UTIL_GETLINE](buf, lineno, maxlen, l));
}

char **util_env_create(char **env, int n, int *pos)
{
	return((char **)SafTable[UTIL_ENV_CREATE](env, n, pos));
}

char *util_env_str(char *name, char *value)
{
	return(SafTable[UTIL_ENV_STR](name, value));
}

#if 0
/* Removed from ntcgi.c --- obsolete; MB 1-26-96 */
char *ntutil_env_str(char *name, char *value, char *env, 
	int *envPosition, int *remainingLength)
{
	return(SafTable[NTUTIL_ENV_STR](name, value, env, envPosition, remainingLength));
}
#endif

void util_env_replace(char **env, char *name, char *value)
{
	(SafTable[UTIL_ENV_REPLACE](env, name, value));
}

void util_env_free(char **env)
{
	(SafTable[UTIL_ENV_FREE](env));
}

char *util_env_find(char **env, char *name)
{
	return(SafTable[UTIL_ENV_FIND](env, name));
}

char *util_hostname(void)
{
	return(SafTable[UTIL_HOSTNAME]());
}

int util_chdir2path(char *path)
{
	return((int)SafTable[UTIL_CHDIR2PATH](path));
}

int util_is_mozilla(char *ua, char *major, char *minor)
{
	return((int)SafTable[UTIL_IS_MOZILLA](ua, major, minor));
}

int util_is_url(char *url)
{
	return((int)SafTable[UTIL_IS_URL](url));
}

int util_later_than(struct tm *lms, char *ims)
{
	return((int)SafTable[UTIL_LATER_THAN](lms, ims));
}

int util_uri_is_evil(char *t)
{
	return((int)SafTable[UTIL_URI_IS_EVIL](t));
}

void util_uri_parse(char *uri)
{
	(SafTable[UTIL_URI_PARSE](uri));
}

void util_uri_unescape(char *s)
{
	(SafTable[UTIL_URI_UNESCAPE](s));
}

char *util_uri_escape(char *d, char *s)
{
	return(SafTable[UTIL_URI_ESCAPE](d, s));
}

char *util_url_escape(char *d, char *s)
{
	return(SafTable[UTIL_URL_ESCAPE](d, s));
}

char *util_sh_escape(char *s)
{
	return((char *)SafTable[UTIL_SH_ESCAPE](s));
}

int util_itoa(int i, char *a)
{
	return((int)SafTable[UTIL_ITOA](i, a));
}

int util_vsprintf(char *s, register char *fmt, va_list args)
{
	return((int)SafTable[UTIL_VSPRINTF](s, fmt, args));
}

int util_sprintf(char *s, char *fmt, ...)
{
	va_list args;

	va_start(args, fmt);
	return((int)SafTable[UTIL_VSNPRINTF](s, -1, fmt, args));
}

int util_vsnprintf(char *s, int n, register char *fmt, va_list args)
{
	return((int)SafTable[UTIL_VSNPRINTF](s, n, fmt, args));
}

int util_snprintf(char *s, int n, char *fmt, ...)
{
	va_list args;

	va_start(args, fmt);
	return((int)SafTable[UTIL_VSNPRINTF](s, -1, fmt, args));
}


/* Functions from eventlog.h */
VOID InitializeAdminLogging(PCHAR ServiceName, PCHAR MessageFile)
{
	(SafTable[INITIALIZE_ADMIN_LOGGING](ServiceName, MessageFile));
}

VOID InitializeHttpdLogging(PCHAR ServiceName, PCHAR MessageFile)
{
	(SafTable[INITIALIZE_HTTPD_LOGGING](ServiceName, MessageFile));
}

VOID InitializeHttpsLogging(PCHAR ServiceName, PCHAR MessageFile)
{
	(SafTable[INITIALIZE_HTTPS_LOGGING](ServiceName, MessageFile));
}


VOID TerminateAdminLogging()
{
	(SafTable[TERMINATE_ADMIN_LOGGING]());
}

VOID TerminateHttpdLogging()
{
	(SafTable[TERMINATE_HTTPD_LOGGING]());
}

VOID TerminateHttpsLogging()
{
	(SafTable[TERMINATE_HTTPS_LOGGING]());
}


VOID LogErrorEvent(PCHAR ServiceName, WORD fwEventType, WORD fwCategory, DWORD IDEvent,
	LPTSTR chMsg, LPTSTR lpszMsg)
{
	(SafTable[LOG_ERROR_EVENT](ServiceName, fwEventType,
		fwCategory, IDEvent, chMsg, lpszMsg));
}


/* Functions from conf.h */
char *conf_init(char *cfn)
{
	return(SafTable[CONF_INIT](cfn));
}

void conf_terminate(void)
{
	(SafTable[CONF_TERMINATE]());
}

#ifdef NOTDEF
char *conf_getvalue(char *name)
{
	return(SafTable[CONF_GETVALUE](name));
}
#endif /* NOTDEF */

conf_global_vars_s *conf_getglobals(void)
{
	return(SafTable[CONF_GETGLOBALS]());
}

void conf_vars2daemon(daemon_s *d)
{
	(SafTable[CONF_VARS2DAEMON](d));
}

/* Functions from systhr.h */
SYS_THREAD systhread_start(int prio, int stksz, void (*fn)(void *), void *arg)
{
	return(SafTable[SYSTHREAD_START](prio, stksz, fn, arg));
}

SYS_THREAD systhread_attach()
{
	return(SafTable[SYSTHREAD_ATTACH]());
}

void systhread_sleep(int milliseconds)
{
	(SafTable[SYSTHREAD_SLEEP](milliseconds));
}

void systhread_terminate(SYS_THREAD thr)
{
	(SafTable[SYSTHREAD_TERMINATE](thr));
}

void systhread_init(char *name)
{
	(SafTable[SYSTHREAD_INIT](name));
}

int systhread_newkey(void)
{
	return((int)SafTable[SYSTHREAD_NEWKEY]());
}

void *systhread_getdata(int key)
{
	return(SafTable[SYSTHREAD_GETDATA](key));
}

void systhread_setdata(int key, void *data)
{
	(SafTable[SYSTHREAD_SETDATA](key, data));
}

/* Functions from shmem.h */

shmem_s *shmem_alloc(char *name, int size, int expose)
{
	return(SafTable[SHMEM_ALLOC](name, size, expose));
}

void shmem_free(shmem_s *region)
{
	(SafTable[SHMEM_FREE](region));
}

/* Functions from func.h */

void func_init(void)
{
	(SafTable[FUNC_INIT]());
}

FuncPtr func_find(char *name)
{
	return((FuncPtr)SafTable[FUNC_FIND](name));
}

int func_exec(pblock *pb, Session *sn, Request *rq)
{
	return((int)SafTable[FUNC_EXEC](pb, sn, rq));
}

struct FuncStruct *func_insert(char *name, FuncPtr fn)
{
	return((struct FuncStruct *)SafTable[FUNC_INSERT](name, fn));
}


/* Functions from http.h */
int http_parse_request(char *t, Request *rq, Session *sn)
{
	return((int)SafTable[HTTP_PARSE_REQUEST](t, rq, sn));
}

int http_scan_headers(Session *sn, netbuf *buf, char *t, pblock *headers)
{
	return((int)SafTable[HTTP_SCAN_HEADERS](sn, buf, t, headers));
}

int http_start_response(Session *sn, Request *rq)
{
	return((int)SafTable[HTTP_START_RESPONSE](sn, rq));
}

char **http_hdrs2env(pblock *pb)
{
	return((char **)SafTable[HTTP_HDRS2_ENV](pb));
}

void http_status(Session *sn, Request *rq, int n, char *r)
{
	(SafTable[HTTP_STATUS](sn, rq, n, r));
}

int http_set_finfo(Session *sn, Request *rq, struct stat *finfo)
{
	return((int)SafTable[HTTP_SET_FINFO](sn, rq, finfo));
}

char *http_dump822(pblock *pb, char *t, int *pos, int tsz)
{
	return((char *)SafTable[HTTP_DUMP822](pb, t, pos, tsz));
}

void http_finish_request(Session *sn, Request *rq)
{
	(SafTable[HTTP_FINISH_REQUEST](sn, rq));
}

void http_handle_session(Session *sn)
{
	(SafTable[HTTP_HANDLE_SESSION](sn));
}

char *http_uri2url(char *prefix, char *suffix)
{
	return((char *)SafTable[HTTP_URI2URL](prefix, suffix));
}


/* Functions from log.h */
int log_error(int degree, char *func, Session *sn, Request *rq, 
              char *fmt, ...)
{
	char errstr[8192];
    va_list args;
    va_start(args, fmt);
    util_vsnprintf(errstr, -1, fmt, args);

	return((int)SafTable[LOG_ERROR](degree, func, sn, rq, errstr));
}


/* Functions from object.h */
int directive_name2num(char *dir)
{
	return((int)SafTable[DIRECTIVE_NAME2NUM](dir));
}

const char *directive_num2name(int num)
{
	return((const char *)SafTable[DIRECTIVE_NUM2NAME](num));
}

httpd_object *object_create(int nd, pblock *name)
{
	return((httpd_object *)SafTable[OBJECT_CREATE](nd, name));
}

void object_free(httpd_object *obj)
{
	(SafTable[OBJECT_FREE](obj));
}

void object_add_directive(int dc, pblock *p, pblock *c, httpd_object *obj)
{
	(SafTable[OBJECT_ADD_DIRECTIVE](dc, p, c, obj));
}


/* Functions from objset.h */
httpd_objset *objset_scan_buffer(filebuf *buf, char *errstr, httpd_objset *os)
{
	return(SafTable[OBJSET_SCAN_BUFFER](buf, errstr, os));
}

httpd_objset *objset_create(void)
{
	return(SafTable[OBJSET_CREATE]());
}

void objset_free(httpd_objset *os)
{
	(SafTable[OBJSET_FREE](os));
}

void objset_free_setonly(httpd_objset *os)
{
	(SafTable[OBJSET_FREE_SETONLY](os));
}

httpd_object *objset_new_object(pblock *name, httpd_objset *os)
{
	return(SafTable[OBJSET_NEW_OBJECT](name, os));
}

void objset_add_object(httpd_object *obj, httpd_objset *os)
{
	(SafTable[OBJSET_ADD_OBJECT](obj, os));
}

httpd_object *objset_findbyname(char *name, httpd_objset *ign, 
                                httpd_objset *os)
{
	return(SafTable[OBJSET_FINDBYNAME](name, ign, os));
}

httpd_object *objset_findbyppath(char *ppath, httpd_objset *ign, 
                                 httpd_objset *os)
{
	return(SafTable[OBJSET_FINDBYPPATH](ppath, ign, os));
}


/* Functions from req.h */
Request *request_create(void)
{
	return(SafTable[REQUEST_CREATE]());
}

void request_free(Request *req)
{
	(SafTable[REQUEST_FREE](req));
}

Request *request_restart_internal(char *uri, Request *rq)
{
	return(SafTable[REQUEST_RESTART_INTERNAL](uri, rq));
}

char *request_translate_uri(char *uri, Session *sn)
{
	return(SafTable[REQUEST_TRANSLATE_URI](uri, sn));
}

int request_header(char *name, char **value, Session *sn, Request *rq)
{
	return((int)SafTable[REQUEST_HEADER](name, value, sn, rq));
}

struct stat *request_stat_path(char *path, Request *rq)
{
	return(SafTable[REQUEST_STAT_PATH](path, rq));
}

int request_uri2path(Session *sn, Request *rq)
{
	return((int)SafTable[REQUEST_URI2PATH](sn, rq));
}

int request_pathchecks(Session *sn, Request *rq)
{
	return((int)SafTable[REQUEST_PATHCHECKS](sn, rq));
}

int request_fileinfo(Session *sn, Request *rq)
{
	return((int)SafTable[REQUEST_FILEINFO](sn, rq));
}

int request_handle_processed(Session *sn, Request *rq)
{
	return((int)SafTable[REQUEST_HANDLE_PROCESSED](sn, rq));
}

int request_service(Session *sn, Request *rq)
{
	return((int)SafTable[REQUEST_SERVICE](sn, rq));
}

void request_handle(Session *sn)
{
	(SafTable[REQUEST_HANDLE](sn));
}


/* magnus.h */
void magnus_atrestart(void (*fn)(void *), void *data)
{
	(SafTable[MAGNUS_ATRESTART](fn, data));
}

/* Added in 2.0 server */
SYS_FILE system_fopenWT(char *name)
{
    return (SafTable[SYSTEM_FOPENWT](name));
}

void *system_malloc(int size)
{
    return (SafTable[SYSTEM_MALLOC](size));
}
void system_free(void *ptr)
{
    (SafTable[SYSTEM_FREE](ptr));
}
void *system_realloc(void *ptr, int size)
{
    return (SafTable[SYSTEM_REALLOC](ptr, size));
}
char *system_strdup(char *ptr)
{
    return (SafTable[SYSTEM_STRDUP](ptr));
}
CRITICAL crit_init(void)
{
    return (SafTable[CRIT_INIT]());
}
void crit_enter(CRITICAL crit)
{
    (SafTable[CRIT_ENTER](crit));
}
void crit_exit(CRITICAL crit)
{
    (SafTable[CRIT_EXIT](crit));
}
void crit_terminate(CRITICAL crit)
{
    (SafTable[CRIT_TERMINATE](crit));
}
SYS_THREAD systhread_current(void)
{
    return (SafTable[SYSTHREAD_CURRENT]());
}
SYS_NETFD net_accept(SYS_NETFD s, struct sockaddr *addr, int *addrlen)
{
    return (SafTable[NET_ACCEPT](s, addr, addrlen));
}
int net_close(SYS_NETFD s)
{
    return (SafTable[NET_CLOSE](s));
}
int net_connect(int s, const void *sockaddr, int namelen)
{
    return (SafTable[NET_CONNECT](s, sockaddr, namelen));
}
int net_ioctl(int s, int tag, void *result)
{
    return (SafTable[NET_IOCTL](s, tag, result));
}
int net_listen(SYS_NETFD s, int backlog)
{
    return (SafTable[NET_LISTEN](s, backlog));
}
int net_setsockopt(SYS_NETFD s, int level, int optname, const void *optval,
                   int optlen)
{
    return (SafTable[NET_SETSOCKOPT](s, level, optname, optval, optlen));
}
SYS_NETFD net_socket(int domain, int type, int protocol)
{
    return (SafTable[NET_SOCKET](domain, type, protocol));
}


/* Functions from upload.c */
int upload_file(pblock *pb, Session *sn, Request *rq)
{
    return ((int)SafTable[UPLOAD_FILE](pb, sn, rq));
}

#ifdef NOTDEF
/* NSPR Functions */

/* prhash.c */

PRHashTable *PR_NewHashTable(uint32 n, PRHashFunction keyHash,
				    PRHashComparator keyCompare,
				    PRHashComparator valueCompare,
				    PRHashAllocOps *allocOps, void *allocPool)
{
	return(SafTable[PR_NEWHASHTABLE](n, keyHash, keyCompare, valueCompare,
			allocOps, allocPool));
}

void PR_HashTableDestroy(PRHashTable *ht)
{
	(SafTable[PR_HASHTABLEDESTROY](ht));
}

PRHashEntry **PR_HashTableRawLookup(PRHashTable *ht,
					   PRHashNumber keyHash, void *key)
{
	return(SafTable[PR_HASHTABLERAWLOOKUP](ht, keyHash, key));
}

PRHashEntry *PR_HashTableRawAdd(PRHashTable *ht, PRHashEntry **hep,
				       PRHashNumber keyHash,
				       void *key, void *value)
{
	return(SafTable[PR_HASHTABLERAWADD](ht, hep, keyHash, key, value));
}

void PR_HashTableRawRemove(PRHashTable *ht, PRHashEntry **hep,
				  PRHashEntry *he)
{
	(SafTable[PR_HASHTABLERAWREMOVE](ht, hep, he));
}

PRHashEntry *PR_HashTableAdd(PRHashTable *ht, void *key, void *value)
{
	return(SafTable[PR_HASHTABLEADD](ht, key, value));
}

PRBool PR_HashTableRemove(PRHashTable *ht, void *key)
{
	return((PRBool)SafTable[PR_HASHTABLEREMOVE](ht, key));
}

int PR_HashTableEnumerateEntries(PRHashTable *ht, PRHashEnumerator f,
					void *arg)
{
	return((int)SafTable[PR_HASHTABLEENUMERATEENTRIES](ht, f, arg));
}

void *PR_HashTableLookup(PRHashTable *ht, void *key)
{
	return(SafTable[PR_HASHTABLELOOKUP](ht, key));
}

int PR_HashTableDump(PRHashTable *ht, PRHashEnumerator dump, FILE *fp)
{
	return((int)SafTable[PR_HASHTABLEDUMP](ht, dump, fp));
}


/* Functions from prfile.h */

PRFileHandle PR_OpenFile(const char *fileName, int flags, int mode)
{
	return((PRFileHandle)SafTable[PR_OPENFILE](fileName, flags, mode));
}

void PR_NonBlockIO(int desc, int on)
{
	(SafTable[PR_NONBLOCKIO](desc, on));
}

PRFileHandle PR_MapFileHandle(PROSFD osfd, void *data, PRIOMethods *methods)
{
	return((PRFileHandle)SafTable[PR_MAPFILEHANDLE](osfd, data, methods));
}

int32 PR_READ(PRFileHandle hdl, void *buf, int32 size)
{
	return((int32)SafTable[PRREAD](hdl, buf, size));
}

int32 PR_WRITE(PRFileHandle hdl, const void *buf, int32 size)
{
	return((int32)SafTable[PRWRITE](hdl, buf, size));
}

int32 PR_CLOSE(PRFileHandle hdl)
{
	return((int32)SafTable[PRCLOSE](hdl));
}

int PR_Select(int nfd, fd_set *r, fd_set *w, fd_set *e, struct timeval *t)
{
	return((int)SafTable[PR_SELECT](nfd, r, w, e, t));
}

void _PR_InitIO(void)
{
	(SafTable[_PR_INITIO]());
}

void _OS_InitIO(void)
{
	(SafTable[_OS_INITIO]());
}

void _PR_WaitForFD(PRFileDesc *fd, int rd)
{
	(SafTable[_PR_WAITFORFD](fd, rd));
}

void PR_SetError(int oserror)
{
	(SafTable[PR_SETERROR](oserror));
}

/* Functions from prglobal.h */

void PR_Init(char *name, int priority, int maxCpus, int flags)
{
	(SafTable[PR_INIT](name, priority, maxCpus, flags));
}

void PR_InitPageStuff(void)
{
	(SafTable[PR_INITPAGESTUFF]());
}

/* Functions from prmon.h */

PRMonitor *PR_NewMonitor(int entryCount)
{
	return(SafTable[PR_NEWMONITOR](entryCount));
}

PRMonitor *PR_NewNamedMonitor(int entryCount, char *name)
{
	return(SafTable[PR_NEWNAMEDMONITOR](entryCount, name));
}

void PR_InitMonitor(PRMonitor *mon, int entryCount, char *name)
{
	(SafTable[PR_INITMONITOR](mon, entryCount, name));
}

void PR_DestroyMonitor(PRMonitor *mon)
{
	(SafTable[PR_DESTROYMONITOR](mon));
}

void PR_EnterMonitor(PRMonitor *mon)
{
	(SafTable[PR_ENTERMONITOR](mon));
}

int PR_InMonitor(PRMonitor *mon)
{
	return((int)SafTable[PR_INMONITOR](mon));
}

int PR_ExitMonitor(PRMonitor *mon)
{
	return((int)SafTable[PR_EXITMONITOR](mon));
}

int PR_Wait(PRMonitor *mon, int64 sleep)
{
	return((int)SafTable[PR_WAIT](mon, sleep));
}

int PR_Notify(PRMonitor *mon)
{
	return((int)SafTable[PR_NOTIFY](mon));
}

int PR_NotifyAll(PRMonitor *mon)
{
	return((int)SafTable[PR_NOTIFYALL](mon));
}

int PR_CEnterMonitor(void *address)
{
	return((int)SafTable[PR_CENTERMONITOR](address));
}

int PR_CExitMonitor(void *address)
{
	return((int)SafTable[PR_CEXITMONITOR](address));
}

int PR_CWait(void *address, int64 howlong)
{
	return((int)SafTable[PR_CWAIT](address, howlong));
}

int PR_CNotify(void *address)
{
	return((int)SafTable[PR_CNOTIFY](address));
}

int PR_CNotifyAll(void *address)
{
	return((int)SafTable[PR_CNOTIFYALL](address));
}

int PR_MakeMonitorSelectable(PRMonitor* mon)
{
	return((int)SafTable[PR_MAKEMONITORSELECTABLE](mon));
}

void PR_SelectNotify(PRMonitor* mon)
{
	(SafTable[PR_SELECTNOTIFY](mon));
}

void PR_ClearSelectNotify(PRMonitor* mon)
{
	(SafTable[PR_CLEARSELECTNOTIFY](mon));
}

/* Functions from prthread.h */

PRThread *PR_CurrentThread(void)
{
	return(SafTable[PR_CURRENTTHREAD]());
}

PRThread *PR_CreateThread(char *name, int priority, size_t stackSize)
{
	return(SafTable[PR_CREATETHREAD](name, priority, stackSize));
}

PRThread *PR_AttachThread(char *name, int priority, PRThreadStack *stk)
{
	return(SafTable[PR_ATTACHTHREAD](name, priority, stk));
}

void PR_DestroyThread(PRThread *thread)
{
	(SafTable[PR_DESTROYTHREAD](thread));
}

void PR_Yield()
{
	(SafTable[PR_SLEEP]());
}

void PR_Sleep(int64 sleep)
{
	(SafTable[PR_SLEEP](sleep));
}

void PR_Exit(void)
{
	(SafTable[PR_EXIT]());
}

int PR_Start(PRThread *thread, void (*e)(void*,void*), void *o, void *a)
{
	return((int)SafTable[PR_START](thread, e, o, a));
}

void PR_SetPendingException(PRThread *thread)
{
	(SafTable[PR_SETPENDINGEXCEPTION](thread));
}

void PR_ClearPendingException(PRThread *thread)
{
	(SafTable[PR_CLEARPENDINGEXCEPTION](thread));
}

int PR_Resume(PRThread *thread)
{
	return((int)SafTable[PR_RESUME](thread));
}

int PR_GetThreadPriority(PRThread *thread)
{
	return((int)SafTable[PR_GETTHREADPRIORITY](thread));
}

void PR_SetThreadPriority(PRThread *thread, int newPri)
{
	(SafTable[PR_SETTHREADPRIORITY](thread, newPri));
}

int PR_ActiveCount(void)
{
	return((int)SafTable[PR_ACTIVECOUNT]());
}

void PR_SingleThread(void)
{
	(SafTable[PR_SINGLETHREAD]());
}

void PR_MultiThread(void)
{
	(SafTable[PR_MULTITHREAD]());
}

int PR_EnumerateThreads(PREnumerator func, void *arg)
{
	return((int)SafTable[PR_ENUMERATETHREADS](func, arg));
}

int PR_SetThreadPrivate(PRThread *t, int32 id, void GCPTR *priv)
{
	return((int)SafTable[PR_MAKEMONITORSELECTABLE](t, id, priv));
}

void GCPTR *PR_GetThreadPrivate(PRThread *t, int32 id)
{
	return(SafTable[PR_SETTHREADPRIVATE](t, id));
}

int32 PR_NewThreadPrivateID(void)
{
	return((int32)SafTable[PR_NEWTHREADPRIVATEID]());
}

char *PR_GetThreadName(PRThread *t)
{
	return(SafTable[PR_GETTHREADNAME](t));
}

int32 PR_GetCurrentThreadID(void)
{
	return((int32)SafTable[PR_GETCURRENTTHREADID]());
}

/* Functions from prtime.h */

int64 PR_Now(void)
{
	return((int64)SafTable[PR_NOW]());
}

int64 PR_NowMS(void)
{
	return((int64)SafTable[PR_NOWMS]());
}

int64 PR_NowS(void)
{
	return((int64)SafTable[PR_NOWS]());
}

int64 PR_ToGMT(int64 time)
{
	return((int64)SafTable[PR_TOGMT](time));
}

void PR_ExplodeTime(PRTime *to, int64 time)
{
	(SafTable[PR_EXPLODETIME](to, time));
}

int64 PR_ComputeTime(PRTime *tm)
{
	return((int64)SafTable[PR_COMPUTETIME](tm));
}

void PR_FormatTime(char *buf, int buflen, char *fmt, PRTime *tm)
{
	(SafTable[PR_FORMATTIME](buf, buflen, fmt, tm));
}

/* Functions from prlog.h */

void PR_Log_flush(void)
{
	(SafTable[PR_LOG_FLUSH]());
}

void PR_Abort(void)
{
	(SafTable[PR_ABORT]());
}

PRBool PR_Log_test(PR_LogModuleID moduleID, PR_LogModuleLevel level)
{
	return((PRBool)SafTable[PR_LOGTEST](moduleID, level));
}

void PR_Log_print(PR_LogModuleID moduleID, PR_LogModuleLevel level, char *fmt, ...)
{
	va_list args;

	va_start(args, fmt);
	(SafTable[PR_LOG_PRINT](moduleID, level, fmt, args));
}

void PR_Log_setLevel(PR_LogModuleID moduleID, PR_LogModuleLevel level)
{
	(SafTable[PR_LOG_SETLEVEL](moduleID, level));
}

/* Functions from prdump.h */

void PR_DumpOneThread(PRThread *t)
{
	(SafTable[PR_DUMPONETHREAD](t));
}

void PR_DumpThreads(void)
{
	(SafTable[PR_DUMPTHREADS]());
}

void PR_DumpMonitors(void)
{
	(SafTable[PR_DUMPMONITORS]());
}

void PR_DumpStuff(void)
{
	(SafTable[PR_DUMPSTUFF]());
}

void PR_Monitor_print(FILE* file, PRMonitor* mon)
{
	(SafTable[PR_MONITOR_PRINT](file, mon));
}

void PR_Thread_print(FILE* file, PRThread* t)
{
	(SafTable[PR_THREAD_PRINT](file, t));
}

void PR_ThreadFun(char* funName, PRThread* thread, PRMonitor* mon)
{
	(SafTable[PR_THREADFUN](funName, thread, mon));
}

/* Functions in prprf.h */

size_t PR_snprintf(char *out, size_t outlen, const char *fmt, ...)
{
	va_list args;

	va_start(args, fmt);
	return((size_t)SafTable[PR_SNPRINTF](out, outlen, args));
}

char *PR_smprintf(const char *fmt, ...)
{
	va_list args;

	va_start(args, fmt);
	return(SafTable[PR_SMPRINTF](args));
}

size_t PR_sxprintf(PR_sxprintf_callback stuff, void *arg, const char *fmt, ...)
{
	va_list args;

	va_start(args, fmt);
	return((size_t)SafTable[PR_SXPRINTF](stuff,arg, args));
}

char *PR_sprintf_append(char *last, const char *fmt, ...)
{
	va_list args;

	va_start(args, fmt);
	return(SafTable[PR_SPRINTF_APPEND](last, args));
}

size_t PR_vsnprintf(char *out, size_t outlen, const char *fmt,
			   va_list ap)
{
	return((size_t)SafTable[PR_VSNPRINTF](out, outlen, fmt, ap));
}

char *PR_vsmprintf(const char *fmt, va_list ap)
{
	return(SafTable[PR_VSMPRINTF](fmt, ap));
}

size_t PR_vsxprintf(PR_vsxprintf_callback stuff,
			   void *arg, const char *fmt, va_list ap)
{
	return((size_t)SafTable[PR_VSXPRINTF](stuff, arg, fmt, ap));
}

char *PR_vsprintf_append(char *last, const char *fmt, va_list ap)
{
	return(SafTable[PR_VSPRINTF_APPEND](last, fmt, ap));
}

/* Functions from prgc.h */

void PR_InitGC(prword_t flags, prword_t initialHeapSize)
{
	(SafTable[PR_INITGC](flags, initialHeapSize));
}

void PR_SetBeginGCHook(GCBeginGCHook *hook, void *arg)
{
	(SafTable[PR_SETBEGINGCHOOK](hook, arg));
}

void PR_GetBeginGCHook(GCBeginGCHook **hook, void **arg)
{
	(SafTable[PR_GETBEGINGCHOOK](hook, arg));
}

void PR_SetEndGCHook(GCBeginGCHook *hook, void *arg)
{
	(SafTable[PR_SETENDGCHOOK](hook, arg));
}

void PR_GetEndGCHook(GCEndGCHook **hook, void **arg)
{
	(SafTable[PR_GETENDGCHOOK](hook, arg));
}

void PR_SetBeginFinalizeHook(GCBeginFinalizeHook *hook, void *arg)
{
	(SafTable[PR_SETBEGINFINALIZEHOOK](hook, arg));
}

void PR_GetBeginFinalizeHook(GCBeginFinalizeHook **hook, void **arg)
{
	(SafTable[PR_GETBEGINFINALIZEHOOK](hook, arg));
}

void PR_SetEndFinalizeHook(GCBeginFinalizeHook *hook, void *arg)
{
	(SafTable[PR_SETENDFINALIZEHOOK](hook, arg));
}

void PR_GetEndFinalizeHook(GCEndFinalizeHook **hook, void **arg)
{
	(SafTable[PR_GETBEGINFINALIZEHOOK](hook, arg));
}

int PR_RegisterType(GCType *type)
{
	return((int)SafTable[PR_REGISTERTYPE](type));
}

void PR_RegisterRootFinder(GCRootFinder func, char *name, void *arg)
{
	(SafTable[PR_REGISTERROOTFINDER](func, name, arg));
}

prword_t GCPTR *PR_AllocMemory(prword_t bytes, int typeIndex, int flags)
{
	return(SafTable[PR_ALLOCMEMORY](bytes, typeIndex, flags));
}

void PR_GC(void)
{
	(SafTable[PRGC]());
}

void PR_ForceFinalize(void)
{
	(SafTable[PR_FORCEFINALIZE]());
}

void PR_DumpGCHeap(FILE *out)
{
	(SafTable[PR_DUMPGCHEAP](out));
}

void PR_DumpIndent(FILE *out, int indent)
{
	(SafTable[PR_DUMPINDENT](out, indent));
}

void PR_DumpMonitorCache(void)
{
	(SafTable[PR_DUMPMONITORCACHE]());
}

/* Functions from prtypes.h */

int32 PR_CeilingLog2(uint32 n)
{
	return((int32)SafTable[PR_ALLOCMEMORY](n));
}

/* Functions from mdint.h */

prword_t *_MD_HomeGCRegisters(PRThread *t, int isCurrent, int *np)
{
	return(SafTable[_MD_HOMEGCREGISTERS](t, isCurrent, np));
}

void *_MD_GrowGCHeap(size_t *sizep)
{
	return(SafTable[_MD_INITOS](sizep));
}

int _MD_NewStack(PRThread *thread, size_t stackSize)
{
	return((int)SafTable[_MD_NEWSTACK](thread, stackSize));
}

void _MD_FreeStack(PRThread *thread)
{
	(SafTable[_MD_FREESTACK](thread));
}

void _MD_StartClock(void)
{
	(SafTable[_MD_STARTCLOCK]());
}

/* Functions from prfile.c */
PRFileHandle _PR_InitFileHandle(PROSFD osfd, PRIOMethods *m)
{
	return(SafTable[_PR_INITFILEHANDLE](osfd, m));
}

void _PR_DestroyFileHandle(PRFileHandle hdl)
{
	(SafTable[_PR_DESTROYFILEHANDLE](out));
}

/* Functions from prfileio.c */

int32 _PR_CloseFile(PRFileDesc *fd)
{
	return((int32)SafTable[_PR_CLOSEFILE](n));
}

int32 _PR_ReadFile(PRFileDesc *fd, void *buf, int32 amount)
{
	return((int32)SafTable[_PR_READFILE](n));
}

int32 _PR_WriteFile(PRFileDesc *fd, const void *buf, int32 amount)
{
	return((int32)SafTable[_PR_WRITEFILE](n));
}

/* Functions from prgcapi.c */

void _PR_InitGC(int flags)
{
	(SafTable[_PR_INITGC](out));
}

/* Functions from hwmon.c */

void _PR_CondNotify(PRMonitor *mon, int all)
{
	(SafTable[_PR_CONDNOTIFY](out));
}

void _PR_CondWait(PRMonitor *mon, int64 sleep)
{
	(SafTable[_PR_CONDWAIT](out));
}

/* Functions from prmsgc.c */

void _PR_CreateFinalizer(void)
{
	(SafTable[_PR_CREATEFINALIZER](out));
}

void _PR_ScanFinalQueue(void *notused)
{
	(SafTable[_PR_SCANFINALQUEUE](out));
}

void _PR_DumpSegmentFreelist(GCSeg *sp)
{
	(SafTable[_PR_DUMPSEGMENTFREELIST](out));
}

void _PR_Log_enable(void)
{
	(SafTable[_PR_LOG_ENABLE](out));
}

void _PR_Assert(const char *ex, const char *file, int line)
{
	(SafTable[_PR_ASSERT](out));
}

/* Functions from prmcache.c */

void _PR_InitMonitorCache(void)
{
	(SafTable[_PR_INITMONITORCACHE](out));
}

/* Functions from hwsched.c */

void _PR_Pause(void)
{
	(SafTable[_PR_PAUSE](out));
}

int _PR_IntsOff(void)
{
	return((int)SafTable[_PR_INTSOFF](n));
}

int _PR_IntsOn(int status, int wantYield)
{
	return((int)SafTable[_PR_INTSON](n));
}
#endif /* NOTDEF */
