/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
//
// Name:    nscgiargs.cpp
// Contact: mjb<barbarin@netscape.com>
// -------------------------------------------------------------------------------

#include "nscgiargs.h"

// inline member functions

inline s16 
NScgiargs::DirCompare(void* a, void* b) {
   Str srckey = ((DirNode*)a)->attr;
   return( strncmp( ((DirNode*)b)->attr, srckey, strlen(srckey) ) );
   }

inline void
NScgiargs::DirPrint(void* a, ostream& o) {
   DirNode* pNode = (DirNode*)a;
   o << "  attr = " << pNode->attr << ", cb=" << (void*)pNode->cb << endl;
   }

inline void
NScgiargs::DirDelete(void* a, ostream&) {
   DirNode* pNode = (DirNode*)a;
   delete []pNode->attr;
   delete pNode;
   }


// constructors/destuctors

NScgiargs::NScgiargs(NScgiargsLoad load, NSavpair* opts)
:  _argstring(0), _avpairs(opts), _avpairsAllocated(FALSE), _cb(0),   
   _contentLength(0), _cookie(0), _pathInfo(0),
   _pDir(new NSbtree(DirCompare, DirPrint)), _querystring(0), _serverHost(0), 
   _serverURL(0), _userAgent(0), _remoteUser(0) {

   if (_avpairs == 0) {
      _avpairs = new NSavpair;
      _avpairsAllocated = (_avpairs != 0);
      } 
   if (_avpairs == 0 || _avpairs->Bad())
      Errnum(_avpairs->Errnum());
   else if (_pDir == 0)
      Errnum(outOfMem);
   else if (load == IMMEDIATE_LOAD)
      Load();
   }


NScgiargs::NScgiargs(NSavpair* opts)
:  _argstring(0), _avpairs(opts), _avpairsAllocated(FALSE), _contentLength(0), 
   _cookie(0), _pDir(new NSbtree(DirCompare, DirPrint)),
   _querystring(0), _serverURL(0), _userAgent(0), _remoteUser(0) {

   if (_avpairs == 0) {
      _avpairs = new NSavpair;
      _avpairsAllocated = (_avpairs != 0);
      } 
   if (_avpairs == 0 || _avpairs->Bad())
      Errnum(_avpairs->Errnum());
   else if (_pDir == 0)
      Errnum(outOfMem);
   else 
      Load();
   }


NScgiargs::~NScgiargs() {
   delete []_argstring;
   if (_pDir) {
      _pDir->Traverse(DirDelete, cout);  // using cout, but it's ignored.
      delete _pDir;
      _pDir = 0;
      }
   if (_avpairsAllocated)
      delete _avpairs;
   }


NScgiargs&
NScgiargs::Install(NScgiargsCB cb)  {
   _cb = cb;
   return(*this);
   }

NScgiargs&
NScgiargs::Install(CStr attr, NScgiargsCB cb)  {
/*   DirNode key = {(Str)attr, 0};   commented out since HPUX CC doesn't accept general initializer in initializer list*/

   DirNode key;
   key.attr = (Str)attr;
   key.cb   = 0;  
   DirNode* pDirNode = (DirNode*)(_pDir->Find(&key));
   if (pDirNode != 0) {
      delete []pDirNode->attr;
      if ((pDirNode->attr = new char[strlen(attr)+1]) == 0)
         Errnum(outOfMem);
      else {
         strcpy(pDirNode->attr, attr);
         pDirNode->cb = cb;      
         }
      }
   else if ((pDirNode = new DirNode) == 0) 
      Errnum(outOfMem);
   else if ((pDirNode->attr = new char[strlen(attr)+1]) == 0)
      Errnum(outOfMem);
   else {
      strcpy(pDirNode->attr, attr);
      pDirNode->cb = cb;      
      if (_pDir->Insert(pDirNode).Bad()) 
         Errnum(_pDir->Errnum());
      }

   return(*this);
   }


NScgiargs&
NScgiargs::Load() {
   _serverURL = getenv("SERVER_URL");
   _remoteUser = getenv("REMOTE_USER");
   _cookie = getenv("HTTP_COOKIE");
   _userAgent = getenv("HTTP_USER_AGENT");
   _serverHost = getenv("REMOTE_HOST");
   _pathInfo = getenv("SCRIPT_NAME");

   if (_cookie != 0) 
      CGIparse(_cookie, _cookiepairs, ';');
   // note that we uppercase the user agent (to make it easier to do
   // string compares later on....
   if (_userAgent != 0)
      for (unsigned int i=0; i < strlen(_userAgent); i++)
         _userAgent[i] = toupper(_userAgent[i]);
   _querystring = getenv("QUERY_STRING");
   Str contentLengthString = getenv("CONTENT_LENGTH");
   if (contentLengthString != 0) {
      _contentLength = atoi(contentLengthString)+1;
      if ((_argstring = new char[_contentLength+1]) == 0)
         Errnum(outOfMem);
      else {
         memset(_argstring, '\0', _contentLength+1);
         cin.getline(_argstring, _contentLength);
         _argstring[_contentLength-1] = '&';
         Parse(_argstring, *_avpairs);
         }
      }
   return(*this);
   }


NScgiargs&
NScgiargs::Next(CStr attribute, Str& value, boolean clean) {
   Clear();
   Str tmpKey;
   if (_avpairs->Next(tmpKey, value, _cookieForNSavpair).Bad())
      Errnum(_avpairs->Errnum());
   else if (strcmp(attribute, tmpKey) != 0)
      value = 0;
   else if (clean == TRUE)
      UnescapeArg(value);
   return(*this);
   }


// protected member functions

void
NScgiargs::Dump(ostream& o) const {
   o << "  content length = " << _contentLength << endl
     << "  cookie=" << _cookie << endl
     << "  query string=" << _querystring << endl
     << "  arg string=" << _argstring << endl
     << "  serverURL=" << _serverURL << endl
     << "  remoteUser=" << _remoteUser << endl
     << "  userAgent=" << _userAgent << endl
     << *_avpairs << endl;
   _pDir->Traverse(o);
   }


CStr
NScgiargs::Name() const {
   return("NScgiargs");
   }


// private member functions

boolean
NScgiargs::Parse(Str args, NSavpair& opts, char endOfValue) {
   size_t   argsLen = strlen(args);
   Str      tmpbuf = new char[argsLen+2];
   Str      attr = tmpbuf;

   if (attr == 0)
      return(FALSE);
   
   strcpy(attr, args);
   if (attr[argsLen-1] != endOfValue) {
      attr[argsLen] = endOfValue;
      attr[argsLen+1] = '\0';
      }

   char     delim = '=';
   Str      val = 0;
   Str      ptr = attr;
   boolean  status = TRUE;

   while ((ptr = strchr(ptr, delim)) != 0) {
      *ptr = '\0';
      if (delim == '=') {
         delim = endOfValue;
         val = ++ptr;
         }
      else {
         delim = '=';
         if (strcspn(val, "+") != 0) {
            if (_cb)
               (_cb)(attr, val, opts);
            else {
   	       DirNode key;
   	       key.attr = (Str)attr;
   	       key.cb   = 0;  
               DirNode* pDirNode = (DirNode*)(_pDir->Find(&key));
               if (pDirNode != 0) 
                  (pDirNode->cb)(attr, val, opts);
               else if (opts.Put(attr, val, TRUE).Bad()) {
                  status = FALSE;
                  break;
                  }
               }
            }
         attr = ++ptr;
         }
      }
   delete []tmpbuf;
   return(status);
   }


