/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
//
// Name:    nscgimsg.cpp
// Contact: mjb<barbarin@netscape.com>
// -------------------------------------------------------------------------------

#include "nscgimsg.h"

// public member functions

NScgimsg::NScgimsg(ostream& o) 
:  _cookie(0), _errLogged(0), _hdrSent(FALSE), _out(o), _title(0), 
   _titleSent(FALSE), _header(0), _headerSent(FALSE), _footer(0), 
   _footerSent(FALSE) {
   }


NScgimsg::~NScgimsg() {
   // Make sure the footer is flushed out
   SendFooter();

   delete []_cookie;
   delete []_title;
   delete []_header;
   delete []_footer;
   }


NScgimsg&
NScgimsg::Cookie(CStr cookie) {
   u16 cookiesz = 0;
   if (_cookie != 0)
      cookiesz = strlen(_cookie);
   if (strlen(cookie) > cookiesz) {
      delete []_cookie;
      if ((_cookie = new char[strlen(cookie)+1]) == 0)
         Errnum(outOfMem);
      }

   if (Good())
      strcpy(_cookie, cookie);
   return(*this);
   }


NScgimsg&
NScgimsg::Open(CStr filename) {
   _errormsg.Clear();
   if (_errormsg.Open(filename).Bad())
      Errnum(_errormsg.Errnum());
   return(*this);
   }


NScgimsg&  
NScgimsg::SendHdr() {
   if (!_hdrSent) {
      _out << "Content-type: text/html" << endl;
      if (_cookie)
         _out << "Set-Cookie:" << _cookie << endl; 
      _out << endl; 
      _hdrSent = TRUE;
      }
   return(*this);
   }


NScgimsg&
NScgimsg::Title(CStr title) {
   u16 titlesz = 0;
   if (_title != 0)
      titlesz = strlen(_title);
   if (strlen(title) > titlesz) {
      delete []_title;
      if ((_title = new char[strlen(title)+1]) == 0)
         Errnum(outOfMem);
      }

   if (Good())
      strcpy(_title, title);
   return(*this);
   }


// protected member functions

void
NScgimsg::Dump(ostream& o) const {
   o << " header sent = " << (_hdrSent ? "TRUE" : "FALSE") << endl
     << "cookie=" << _cookie << endl
     << "title=" << _title << endl
     << "header=" << _header << endl
     << "footer=" << _footer << endl;
   }


CStr
NScgimsg::Name() const {
   return("NScgimsg");
   }

// private member functions

NScgimsg&  
NScgimsg::SendTitle() {
   if (_title) {
      if (_titleSent == FALSE) 
          _out << "<TITLE>" << _title << "</TITLE>" << endl;
      _titleSent = TRUE;
      }
   return(*this);
   }

NScgimsg&  
NScgimsg::SendHeader() {
   if (_header) {
      if (_headerSent == FALSE) 
         _out << _header;
      _headerSent = TRUE;
      }
   return(*this);
   }


NScgimsg&  
NScgimsg::SendFooter() {
   if (_footer) {
      if (_footerSent == FALSE) 
         _out << _footer;
      _footerSent = TRUE;
      }
   return(*this);
   }


