/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
//
// Name:    nscnout.cpp
// Contact: mjb<barbarin@netscape.com>
// -------------------------------------------------------------------------------

#include "nscnout.h"

#ifdef PRFSTREAMS_BROKEN
#include <private/pprio.h>
#endif

// public member functions

ClientStream::ClientStream(CStr fnx, Session* sn, Request* rq)
:  _contentHeader(0), _errLogged(FALSE), _errMsgFile(0), _funcname(fnx), 
   _headerSent(FALSE), _logMode(LOG_WARN), _putLogged(FALSE), _rq(rq), _sn(sn) {
   
   // send content/type header prior to writing anything to the client

   if (rq) {
      SetContentHeader("text/html");
      // These two lines send the headers to the client, and setup everything
      // everything so successive new_write()s will work properly

      if (sn) {
         SendHeader();
         // bind the client socket descriptor to an ostream so that i can
         // use some of the old "vengance" code.
#ifdef PRFSTREAMS_BROKEN
 	 _o.attach(_sn->csd -> osfd);
#else
         _o.attach(_sn->csd);
#endif
         }
      }
   }


ClientStream::ClientStream(CStr fnx, Session* sn, Request* rq, int)
:  _contentHeader(0), _errLogged(FALSE), _errMsgFile(0), _funcname(fnx), 
   _headerSent(FALSE), _logMode(LOG_WARN), _putLogged(FALSE), _rq(rq), _sn(sn) {
   
   // send content/type header prior to writing anything to the client

   if (rq) {
      SetContentHeader("text/html");
      // These two lines send the headers to the client, and setup everything
      // everything so successive new_write()s will work properly

      if (sn) {
         // bind the client socket descriptor to an ostream so that i can
         // use some of the old "vengance" code.

#ifdef PRFSTREAMS_BROKEN
 	 _o.attach(_sn->csd -> osfd);
#else
         _o.attach(_sn->csd);
#endif
         }
      }
   }


// Member:  ~ClientStream
// Description:
//    Frees any allocated resources, closes files, shutdown connections,
//    whatever.... basically, last one out turns off the lights.

ClientStream::~ClientStream() {
   delete []_contentHeader;  _contentHeader = 0;
   }


ClientStream&  
ClientStream::Put(CStr msgKey, NSavpair* opts, ostrstream* outp) {
   _putLogged = TRUE;
   Str outMsg = _errMsgFile ? (Str)_errMsgFile->Get(msgKey) : 0;
   if (outp == 0) {
      if (!_headerSent)
         SendHeader();
      ostrstream o;  o.rdbuf()->setbuf(0,512);
      if (!outMsg)
         o << (Str)msgKey;
      else if (opts) {
         NSirmle cvt;
         cvt.Cvt(outMsg, strlen(outMsg), *opts, o);
         }
      else 
         o << outMsg;

      o << ends;
      int   i = o.pcount();
      char* p = o.str();
      net_write(_sn->csd, p, i);
      delete []p;  p=0;
      }
   else {
      if (!outMsg)
         (*outp) << (Str)msgKey;
      else if (opts) {
         NSirmle cvt;
         cvt.Cvt(outMsg, strlen(outMsg), *opts, *outp);
         }
      else 
         (*outp) << outMsg;
      }

   return(*this);
   }


ClientStream& 
ClientStream::SendHeader() {
   if (!_headerSent) {
      protocol_status(_sn, _rq, PROTOCOL_OK, NULL);
      protocol_start_response(_sn, _rq);
      _headerSent = TRUE;
      }
   return(*this);
   }


ClientStream&  
ClientStream::SetContentHeader(CStr header) {
   delete []_contentHeader;  _contentHeader = 0;
   _contentHeader = new char[strlen(header)+1];
   strcpy(_contentHeader, header);
   param_free(pblock_remove("content-type", _rq->srvhdrs));
   pblock_nvinsert("content-type", _contentHeader, _rq->srvhdrs);

   return(*this);
   }


// protected member functions

// Member:  Dump
// Description:
//    Implementation for the pure virtual Dump function in NSbase.  Dumps the
//    current state of the object.

void
ClientStream::Dump(ostream& o) const {
   o << "error logged = " << _errLogged << ", mode=" << _logMode << endl;
   }


// Member:  Name
// Description:
//    Implementation for the pure virtual Name function in NSbase.  Returns the
//    name of the object.

CStr
ClientStream::Name() const {
   return("ClientStream");
   }
