/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
//
// Name:    nsirmle.cpp
// Contact: mjb<barbarin@netscape.com>
// -------------------------------------------------------------------------------
#include "nsirmle.h"

char NSirmle::VALIDCHARS[] 
   = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_-";

inline s16 
NSirmle::DirCompare(PGen a, PGen b) {
   return( strcmp( ((DirNode*)a)->tag, ((DirNode*)b)->tag) );
   }

inline void
NSirmle::DirPrint(PGen a, ostream& o) {
   DirNode* pNode = (DirNode*)a;
   o << "  key = " << pNode->tag << ", cb=" << (PGen)pNode->cb << endl;
   }

inline void
NSirmle::DirDelete(PGen a, ostream&) {
   DirNode* pNode = (DirNode*)a;
   delete []pNode->tag;  pNode->tag=0;
   delete pNode;  pNode=0;
   }

// constructors/destuctors

NSirmle::NSirmle(CStr marker) 
:  _marker(new char[sizeof(marker)+1]), _markersz(strlen(marker)),
   _pDir(new NSbtree(DirCompare, DirPrint)) {
   if (_marker == 0 || _pDir == 0)
      Errnum(outOfMem);
   else
      strcpy(_marker, marker);
   }


NSirmle::~NSirmle() {
   delete []_marker;  _marker= 0;
   if (_pDir) {
      _pDir->Traverse(DirDelete, cout);  // using cout, but it's ignored.
      delete _pDir;  _pDir = 0;
      }
   }


// protected member functions

void
NSirmle::Dump(ostream& o) const {
   o << "  marker = " << _marker << endl;
   _pDir->Traverse(o);
   }


CStr    
NSirmle::Name() const {
   return("NSirmle");
   }

NSirmle&
NSirmle::Cvt(Str patbuf, int patlen, NSavpair& opts, ostream& o) {
   Str buf = new char[ patlen + 1 ];
   if (buf == 0) {
      Errnum(outOfMem);
      return *this;
      }
   memcpy( buf, patbuf, patlen );
   buf[ patlen ] = '\0';
   Str currbuf = buf;
   int currlen = patlen;
   int markerloc;
   Str markerptr;
   Str tag = 0;

   while ((markerptr = strstr(currbuf, _marker)) != 0) {
      markerloc = markerptr - currbuf; 
      // Write out stuff before tag marker
      o.write(currbuf, markerloc);

      // Get tag
      int tagloc = markerloc+_markersz;
      int taglen = strspn(currbuf+tagloc, VALIDCHARS);
      int nextloc = tagloc+taglen;
      
      if (taglen != 0) {
         Str databuf = 0;
         
         // tag found, fetch it
         if ((tag = new char[taglen+1]) == 0) {
            Errnum(outOfMem);
            delete [] buf;  buf = 0;
            return(*this);
            }
         memcpy(tag, &currbuf[tagloc], taglen);
         tag[taglen] = '\0';
         // Try to find tag in doc source Btree
         DirNode key;
         key.tag = tag; key.cb = 0; key.parm = 0;
         DirNode* pDirNode = (DirNode*)(_pDir->Find(&key));

         // Look for '[' conditional immediately after tag name
         if (currbuf[nextloc] == '[' ) {
            nextloc++;   // skip over [
            Str nextlocptr = &currbuf[nextloc];
            Str brptr = strchr(nextlocptr, ']');
            int brindex = brptr ? brptr - nextlocptr : -1;
            
            if ((pDirNode != 0) || 
               (opts.Get(tag, databuf), databuf && *databuf != 0)) {
               // Tag exists. Remove right bracket and keep going
               if (brindex >= 0) 
                  nextlocptr[brindex] = ' ';
               }
            else { 
               // Tag not found, skip over past closing ]
               if (brindex >= 0) nextloc += brindex +1;
               else nextloc = currlen;
               }
            }
         else if (pDirNode != 0) {
            // Look up in doc source Btree
            Str savedCurrbuf = currbuf;
            currbuf+=tagloc;
            if (pDirNode->cb != 0)
               (pDirNode->cb)(o, currbuf, pDirNode->parm);
            else 
               currbuf += strlen(tag);
            nextloc = currbuf-savedCurrbuf;
            currbuf = savedCurrbuf;
            }
         else if (opts.Get(tag, databuf), databuf != 0) {
            // Look up in options
            o << databuf;
            }
         delete []tag; tag = 0;
         }
      // Get rid of stuff we've already handled
      currbuf += nextloc;
      currlen -= nextloc;
      }
    
   // Now flush out the rest of the buffer
   if (Good()) 
      o << currbuf; 
   

   delete [] buf; buf=0;
   delete [] tag; tag=0;
   return(*this);
   }

NSirmle&
NSirmle::Install(Str tag, NSmleCB cb, PGen parm)  {
   DirNode key;
   key.tag = tag; key.cb = 0; key.parm = 0;
   DirNode* pDirNode = (DirNode*)(_pDir->Find(&key));
   if (pDirNode != 0) {
      delete []pDirNode->tag;  pDirNode->tag=0;
      if ((pDirNode->tag = new char[strlen(tag)+1]) == 0)
         Errnum(outOfMem);
      else {
         strcpy(pDirNode->tag, tag);
         pDirNode->cb = cb;      
         pDirNode->parm = parm;
         }
      }
   else if ((pDirNode = new DirNode) == 0) 
      Errnum(outOfMem);
   else if ((pDirNode->tag = new char[strlen(tag)+1]) == 0)
      Errnum(outOfMem);
   else {
      strcpy(pDirNode->tag, tag);
      pDirNode->cb = cb;      
      pDirNode->parm = parm;
      if (_pDir->Insert(pDirNode).Bad()) 
         Errnum(_pDir->Errnum());
      }

   return(*this);
   }


