/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
//
// Name:    nsirsafutil.cpp
// Contact: mjb<barbarin@netscape.com>
// -------------------------------------------------------------------------------

#include "nsirsafutil.h"


CStr _validArgs[] = {
   "NS-attributes",
   "NS-collection",  
   "NS-default-html-title",
   "NS-display-query", 
   "NS-doc-offset", 
   "NS-doc-number",
   "NS-doc-path",
   "NS-doc-score",
   "NS-docs-found", 
   "NS-docs-matched", 
   "NS-docs-searched",
   "NS-file-key",
   "NS-get-next",
   "NS-get-prev",
   "NS-html-descriptions-pat",
   "NS-max-records", 
   "NS-ms",
   "NS-ms-tocend", 
   "NS-ms-tocstart", 
   "NS-query",
   "NS-query-pat", 
   "NS-rdm-output",
   "NS-record-pat",
   "NS-rel-doc-name",
   "NS-search-offset", 
   "NS-search-page",
   "NS-search-type",
   "NS-sort-by",
   "NS-tocend-pat", 
   "NS-tocrec-pat", 
   "NS-tocstart-pat", 
   "\0"};

Str _validArgSynonyms[] = {
   "csid;NS-collection",
   "view-attributes;NS-attributes",
   "view-order;NS-sort-by",
   "type;NS-search-page",
   "view-hits;NS-max-records",
   "scope;NS-query",
   "\0"};

/* Maximum string length of the above */
#define MAX_ARG_SYN 80


NSbool 
BuildValidArgsList(NSavpair& validationList) {
   size_t ix = 0;
   while (*_validArgs[ix] != '\0') {
      validationList.Put(_validArgs[ix], "N/A");
      if (validationList.Bad())
         break;
      ix++;
      }

   return(validationList.Good());
   }


NSbool 
BuildValidSynonymList(NSavpair& validationList) {
   size_t ix = 0;
   while (*_validArgSynonyms[ix] != '\0') {
      // Always make a copy of _validArgSynonyms before
      // doing the store. Many loaders put constant strings in read-only
      // text segments
      char firstpart[MAX_ARG_SYN];
      strcpy(firstpart, _validArgSynonyms[ix]);
      Str semicolon = strchr(firstpart, ';');
      *semicolon = '\0';
      validationList.Put(firstpart,semicolon+1);
      if (validationList.Bad())
         break;
      ix++;
      }

   return(validationList.Good());
   }


void
DumpPblocks(ostream& o, pblock* pb, Session* sn, Request* rq) {
   Str pblockbuf = 0;
   Str marker = "------------------- ";
   if (pb)
      pblockbuf = pblock_pblock2str(pb, pblockbuf);
   if (pblockbuf != 0)
      o << "------------------- pblockbuf" << endl << pblockbuf << endl;

   if (rq != 0) {
      pblockbuf = pblock_pblock2str(rq->vars, pblockbuf);
      if (pblockbuf != 0)
         o << "-------------------- rq->vars" << endl << pblockbuf << endl;

      pblock_pblock2str(rq->reqpb, pblockbuf);
      if (pblockbuf != 0)
         o << marker << "rq->reqpb" << endl << pblockbuf << endl;

      pblock_pblock2str(rq->headers, pblockbuf);
      if (pblockbuf != 0)
         o << marker << "rq->headers" << endl << pblockbuf << endl;

      pblock_pblock2str(rq->srvhdrs, pblockbuf);
      if (pblockbuf != 0)
         o << marker << "rq->srvhdrs" << endl << pblockbuf << endl;
      }
   
   if (sn != 0) {
      pblock_pblock2str(sn->client, pblockbuf);
      if (pblockbuf != 0)
         o << marker << "sn->client" << endl << pblockbuf << endl;
      }
   }


NSbool 
OpenPatternFile(ClientStream* msg, CStr root, CStr fname, NSmapfile& patfile) {
   if (fname == 0 || root == 0)
      return(FALSE);

   NSbool  success = TRUE;
   if (util_uri_is_evil((Str)fname))
      util_uri_parse((Str)fname);

   // now build the full path from the root and fname, inserting a '/'
   // if needed.  Note: we don't normalize for filepath names, since for
   // unix, these should be '/', and for XP_WIN32, the fopen will work
   // with mixed '/' and '\'

   char fullpath[_POSIX_PATH_MAX];
   char ch = *(root + strlen(root)-1);
   if (ch != '/' && ch!= '\\' && *fname != '/' && *fname != '\\')
      util_snprintf(fullpath, _POSIX_PATH_MAX, "%s/%s", root, fname);
   else
      util_snprintf(fullpath, _POSIX_PATH_MAX, "%s%s", root, fname);

   if (patfile.Open(fullpath).Bad()) {
      // create a error message to log to the server log.
      ostrstream  tmpStream;    
      tmpStream.rdbuf()->setbuf(0,1024);
      tmpStream << "root = " << root << ", file = " << fname << ends;
      Str tmp = tmpStream.str();
      msg->Log("NOPATTERN", tmp);
      delete []tmp;
      }

   return(success);
   }


