/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
//
// Name:    nsirutil.cpp
// Contact: mjb<barbarin@netscape.com>
// -------------------------------------------------------------------------------

#include "nsirutil.h"


void 
ChangeChar(Str src, char from, char to) {
   while (src) {
      src = strchr(src, from);
      if (src)
         *src++ = to;
      }
   }

int
CompareStrToLower(Str anyString, Str lowerCaseString, int len) {
   Str ptr1 = anyString;
   Str ptr2 = lowerCaseString;
   int currLen = 0;
   while (*ptr1 && *ptr2) {
      if (tolower(*ptr1++) != *ptr2++)
         return(FALSE);   
      if (len != 0 && ++currLen == len)
         return(TRUE);
      }
   return(*ptr1 == *ptr2);
   }


void
HTMLescape(Str input, ostream& outp, int appendEnds) {
   Str p =    input;
   while (*p != '\0')  {
      switch (*p) {
         case '&' :
            outp << "&amp;";
            break;
         case '"' :
            outp << "&quot;";
            break;
         case '<' :
            outp << "&lt;";
            break;
         case '>' :
            outp << "&gt;";
            break;
         default :
            outp << *p;
         }
      p++;
      }
   if (appendEnds)
      outp << ends;
   }


void
NormalizePathSlashes(Str fname) {
   if (fname) 
      for (Str s = fname; *s; s++)
         if (*s == '\\')
            *s = '/';
   }


NSbool 
StrArgCopy(Str& target, CStr source) {
   if (source) {
      delete [] target;  target=0;
      target = new char[strlen(source)+1];
      if (target == 0)
         return(FALSE);
      else
         strcpy(target, source);
      }
   
   return(TRUE);
   }


Str
URIescape(Str dest, Str src) {
   char *d;
 
   if (!dest)
      dest = new char [(strlen(src)*3) + 1];
   d = dest;
   while (*src) {
      if (strchr("% ?#:+&*\"<>\r\n", *src)) {
         sprintf(d, "%%%2x", *src);
         ++src;
         d += 3;
         }
      else
         *d++ = *src++;
      }
   *d = '\0';
   return dest;
   }

void 
StreamURIescape(Str arg, ostrstream& o) {
   char* curr = arg;
   char* prev = arg;
   while ((curr = strpbrk(curr, "&=\"!#$%^()_+=|'~?<>[]{};:' ")) != 0) {
      if (curr != prev) 
         o.write(prev, curr-prev);
      o << "%" << (0x0F & (*curr >> 4))["0123456789ABCDEF"]
           << (*curr & 0x0F)["0123456789ABCDEF"];
      prev = ++curr;
      }
   o <<  prev << ends;
   }


void 
UnescapeArg(Str val) {
   Str p = val;
   Str c = val;
   while (*p != '\0') {
      if (*p == '+')
         *c++ = ' ';
      else if (*p == '%') {
         char tmp = (*++p >= 'A') ? 10+((*p & 0xDF)-'A') : (*p - '0');
         tmp <<= 4;
         tmp += (*++p >= 'A') ? 10+((*p & 0xDF)-'A') : (*p - '0');
         // do a little check, in case there's an encoded '+' sign.
         *c++ = (tmp == '+') ? ' ' : tmp;
         }
      else
         *c++ = *p;
        
      p++;
      }

   *c = '\0';
   }

int
NSutil_itoa(int i, char *a)
{
   register int x, y, p;
   register char c;
   int negative;
 
   negative = 0;
   if (i < 0) {
      *a++ = '-';
      negative = 1;
      i = -i;
      }
   p = 0;
   while (i > 9) {
      a[p++] = (i%10) + '0';
      i /= 10;
      }
   a[p++] = i + '0';
 
   if (p > 1) {
      for (x = 0, y = p - 1; x < y; ++x, --y) {
         c = a[x];
         a[x] = a[y];
         a[y] = c;
         }
      }
   a[p] = '\0';
   return p + negative;
}

