/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
//
// Name:    nssafarg.cpp
// Contact: mjb<barbarin@netscape.com>
// -------------------------------------------------------------------------------

#include "nssafarg.h"
#include "libir/dbtlibir.h"

// constructors/destuctors

NSsafargs::NSsafargs
(NSavpair*& opts, Session* sn, Request* rq,  NSsafargsCB cb, PGen cbParm)
:  _argstring(0), _avpairs(opts), _avpairsAllocated(FALSE), _cb(cb),
   _contentLength(0), _cbParm(cbParm), _rq(rq), _sn(sn) {

   if (_avpairs == 0) {
      _avpairs = new NSavpair;
      _avpairsAllocated = (_avpairs != 0);
      opts = _avpairs;
      } 
   if (_avpairs == 0 || _avpairs->Bad())
      Errnum(_avpairs->Errnum());
   else 
      Load();
   }

NSsafargs::NSsafargs
(NSavpair*& opts, Str buf, int len, NSsafargsCB cb, PGen cbParm)
:  _argstring(0), _avpairs(opts), _avpairsAllocated(FALSE), _cb(cb),
   _contentLength(0), _cbParm(cbParm), _rq(0), _sn(0) {

   if (_avpairs == 0) {
      _avpairs = new NSavpair;
      _avpairsAllocated = (_avpairs != 0);
      opts = _avpairs;
      } 
   if (_avpairs == 0 || _avpairs->Bad())
      Errnum(_avpairs->Errnum());
   else 
      Load(buf,len);
   }

NSsafargs::~NSsafargs() {
   delete []_argstring;  _argstring=0;
   if (_avpairsAllocated) {
      delete _avpairs; _avpairs=0;
      }
   }


Str
NSsafargs::Get(CStr attribute) {
   Clear();
   Str value = 0;
   if (_avpairs != 0)
      _avpairs->Get(attribute, value);
   return(value);
   }


Str
NSsafargs::Get(CStr attribute, PGen& cookie) {
   Str attr = (char *) attribute;
   Clear();
   Str value = 0;
   if (_avpairs != 0)
      _avpairs->First(attr, value, cookie);
   return(value);
   }


NSbool 
NSsafargs::IsMozilla() const {
   Str  userAgent = _rq ? pblock_findval("user-agent", _rq->headers) : 0;
   return(userAgent ? strncasecmp(userAgent, "MOZILLA", 7) == 0 : 0);
   }


NSbool 
NSsafargs::Next(CStr attribute, Str& value, PGen& cookie) {
   Clear();
   Str tmpKey;
   _avpairs->Next(tmpKey, value, cookie);
   if (tmpKey && strcmp(attribute, tmpKey) != 0)
      value = 0;
   return(Good());
   }


// protected member functions

void
NSsafargs::Dump(ostream& o) const {
   o << "   method = " << Method() << endl
     << "   path = " << Path() << endl
     << "   nametrans base = " << NameTransBase() << endl
     << "   path info = " << PathInfo() << endl
     << "   content length = " << Length() << endl
     << "   query string = " << QueryString() << endl
     << "   arg string = " << Args() << endl
     << "   serverURL = " << ServerURL() << endl
     << "   remote addr = " << RemoteAddr() << endl
     << "   remote user = " << RemoteUser() << endl
     << "   user agent = " << UserAgent() << endl
     << "   host = " << Host() << endl
     << "   uri = " << URI() << endl
     << *_avpairs << endl;
   }


CStr
NSsafargs::Name() const {
   return("NSsafargs");
   }


// private member functions

int
NSsafargs::Load() {
   Str contentLengthString = pblock_findval("content-length", _rq->headers);
   if (contentLengthString)
      _contentLength = atoi(contentLengthString);
   if (_contentLength != 0) {
      // add one character in case we're missing a trailing '&' sentinel
      // then add another for the null terminator
      if ((_argstring = new char[_contentLength+4]) == 0)
         Errnum(outOfMem);
      else {
         int i = 0;
         int ichar = !IO_EOF;
         int clen = _contentLength;
         while (clen && ichar != IO_EOF) {
            ichar = netbuf_getc(_sn->inbuf);
            if (ichar == IO_ERROR) {
               protocol_status(_sn, _rq, PROTOCOL_SERVER_ERROR,
                  "An I/O error occurred before all form data could be read.");
               log_error(LOG_FAILURE, "NSSAFARGS", _sn, _rq, 
                  "An I/O error occurred before all form data could be read.");
//                  XP_GetAdminStr(DBT_anIOErrorOccurredBeforeAllFormDa_));
               return(REQ_ABORTED);
               }
            _argstring[i++] = ichar;
            clen--;
            }
         _argstring[i] = 0;
         if (Parse(_argstring, i, *_avpairs) == FALSE)
            Errnum(invalidData);
         }
      }
   return(Good());
   }

int
NSsafargs::Load(Str buf, int len) {
   _contentLength = len;
   if (_contentLength != 0) {
      // add one character in case we're missing a trailing '&' sentinel
      // then add another for the null terminator
      if ((_argstring = new char[_contentLength+4]) == 0)
         Errnum(outOfMem);
      else {
         int i = 0;
         int ichar = !IO_EOF;
         int clen = _contentLength;
         for (int j = 0; clen && ichar != IO_EOF; ++j) {
            ichar = buf[j];
            if (ichar == IO_ERROR) {
               return(REQ_ABORTED);
               }
            _argstring[i++] = ichar;
            clen--;
            }
         _argstring[i] = 0;
         if (Parse(_argstring, i, *_avpairs) == FALSE)
            Errnum(invalidData);
         }
      }
   return(Good());
   }


NSbool 
NSsafargs::Parse(Str args, int argsLen, NSavpair& opts) {
   if (args[argsLen-1] != '&') {
      args[argsLen] = '&';
      args[argsLen+1] = '\0';
      }
   
   Str newArgs = new char[strlen(args)+4];
   strcpy(newArgs, args);

   Str      attr = newArgs;
   char     delim = '=';
   Str      val = 0;
   Str      ptr = newArgs;
   NSbool   status = TRUE;

   while ((ptr = strchr(ptr, delim)) != 0) {
      *ptr = '\0';
      if (delim == '=') {
         delim = '&';
         val = ++ptr;
         }
      else {
         delim = '=';
         if (_cb) {
            if ((_cb)(attr, val, opts, _cbParm) == FALSE) {
               status = FALSE;
               break;
               }
            }
         else if (val != 0 && *val != 0)
            if (opts.Put(attr, val, TRUE).Bad()) {
               status = FALSE;
               break;
               }
         *(val-1)= '=';
         if (attr != newArgs)
           *(attr-1) = '&';
         attr = ++ptr;
         }
      }

   delete []newArgs;  newArgs=0;
   return(status);
   }


