/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
//
// Name:    nssrch.cpp
// Contact: mjb<barbarin@netscape.com>
// -------------------------------------------------------------------------------
#include "nssrch.h"
#include "nssrchstructs.h"
#include "vdk.h"

inline void 
VDK_CALLBACK NSsessionMsgCB(VdkCBArg sessionData, VdkMessageCBArgRec* msgArg) {
   NSsearchSessionData* pSess = (NSsearchSessionData*)sessionData;
   NSsearchCB cb = (NSsearchCB)(pSess->callbackFunction);
   if (cb)
      (*cb)(int(msgArg->error), Str(msgArg->text), pSess->callbackParm);
   }


NSbool 
InitResults(NSsearchResultsData* pRes, NSsearchArgsData* pArg, VdkSession& sess) {
   pRes->sessHdl = sess;
   if (pRes->srchHdl)
      VdkSearchFree(pRes->srchHdl);
   pRes->srchHdl = 0;
   pRes->errorMsg[0] = '\0';
   pRes->errorNum = 0;   

   delete pRes->tableFieldsMap;  pRes->tableFieldsMap=0;
   delete []pRes->tableFields;  pRes->tableFields=0;
   pRes->tableFieldsMap = new NSavpair;

   pRes->tableFieldsCount = pArg->tableFieldsCount;
   if (pRes->tableFieldsCount != 0) {
      StrArgCopy(pRes->tableFields, pArg->tableFields);
      delete []pRes->table;  pRes->table = 0;
      if ((pRes->table = new Str[pRes->tableFieldsCount]) == 0) 
         return(FALSE);

      int     fieldNum = 0;
      NSbool  newField = TRUE;
      Str     field = pRes->tableFields;

      for ( ; *field != '\0'; field++) {
         if (newField) {
            pRes->table[fieldNum++] = field;
            newField = FALSE;
            }
         else if (*field == ';') {
            *field = '\0';
            newField = TRUE;
            }
         }

      for (int cnt=0; cnt < fieldNum; cnt++) {
         char ix[16];
         sprintf(ix, "%d", cnt);
         pRes->tableFieldsMap->Put(pRes->table[cnt], ix);
         }
      }

   return(TRUE);
   }


NSbool 
LoadResults(NSsearchArgsData* pArg, NSsearchResultsData* pRes) {
   pRes->searchBase = pArg->searchBase;
   pRes->docsFound = 0;
   pRes->docsSearched = 0;
   pRes->docsReturned = 0;

   VdkSearchGetOut srchInfo = 0;
   VdkSearchGetArgRec getInfo;
   VdkStructInit(&getInfo);
   pRes->errorNum = VdkSearchGetInfo(pRes->srchHdl, &getInfo, &srchInfo);
   if (pRes->errorNum != VdkSuccess) 
      return(TRUE);
   else {
      pRes->docsFound = srchInfo->retrieved;
      pRes->docsSearched = srchInfo->processed;
      pRes->docsReturned = srchInfo->merged;
      if (pRes->searchBase < pRes->docsReturned)
         pRes->docsReturned -= pRes->searchBase;
      if (pRes->docsReturned > pArg->maxrecords)
         pRes->docsReturned = pArg->maxrecords;
      }
   
   NSbool success = TRUE;
   if (pRes->docsFound == 0)
      success = TRUE;
   else {
      int fldCnt = NSSEARCH_DEFAULT_ATTR_CNT + pRes->tableFieldsCount;
      VdkCString*  columnArray = new VdkCString[fldCnt];
   
      VdkDocReadBatchArgRec batch;
      VdkStructInit(&batch);

      batch.search      = pRes->srchHdl;
      batch.searchBase  = pArg->searchBase;
      batch.searchRange = (VdkInt2)(pRes->docsReturned);
      batch.columnCount = fldCnt;
      batch.columnArray = columnArray;
      batch.maxColumnSize = pArg->maxAttributeSize;
   
      columnArray[0] = (VdkCString)"";  // should get SCORE (for any language)
      columnArray[1] = (VdkCString)"DOC_FN";
      columnArray[2] = (VdkCString)"VdkVgwKey";
      for (int ix=NSSEARCH_DEFAULT_ATTR_CNT; ix < fldCnt; ix++) 
         columnArray[ix]=(VdkCString)(pRes->table[ix-NSSEARCH_DEFAULT_ATTR_CNT]);
   
      if (pRes->docReadOut) {
         VdkDocReadBatchFree(pRes->docReadOut);
         pRes->docReadOut = 0;
         }
      pRes->errorNum = VdkDocReadBatch(pRes->sessHdl, &batch, &pRes->docReadOut);

      if (pRes->errorNum != VdkSuccess)
         success = FALSE;
      else 
         pRes->docsReturned = pRes->docReadOut->numRows;
   
      delete [] columnArray;  columnArray = 0;
      }
   if (srchInfo)
      VdkSearchGetInfoFree(srchInfo);
   srchInfo = 0;
   return(success);
   }


void
SetQueryType(NSsearchArgs& args, VdkSearchNewArgRec& searchNew) {
   switch (args.QueryType()) {
      case NSsearchArgs::QP_FREETEXT:
         searchNew.queryQParser = VdkQParser_FreeText;
         break;
      case NSsearchArgs::QP_OLDFREETEXT:
         searchNew.queryQParser = VdkQParser_OldFreeText;
         break;
      default:
         searchNew.queryQParser = VdkQParser_Simple;
      }
   }


// public member functions

NSsearch::NSsearch(CStr initParms, NSsearchCB cb, void* cbParm, CStr langParms, 
   void* parm) : _sessionData((void*)new NSsearchSessionData) {

   if (!_sessionData)
      Errnum(outOfMem);
   else {
      NSsearchSessionData* pSess = (NSsearchSessionData*)_sessionData;
      pSess->initParms = 0;
      pSess->langParms = 0;
      pSess->errorNum = 0;
      pSess->errorMsg[0] = '\0';
      pSess->collectionCount = 0;
      pSess->callbackFunction = cb;
      pSess->callbackParm = cbParm;

      // parse out libpath, msg level, and (optional) topicpath from initParms;
      StrArgCopy(pSess->initParms, initParms);
      Str libpath = pSess->initParms;
      Str breakOutMsgLevel = strchr(pSess->initParms, ';');
      Str breakTopicPath = 0;
      Str outMsgLevelStr = 0;
      Str topicPath = 0;
      if (breakOutMsgLevel != 0) {
         outMsgLevelStr = breakOutMsgLevel+1;
         *breakOutMsgLevel = 0;
         if (outMsgLevelStr != 0) {
            breakTopicPath = strchr(outMsgLevelStr, ';');
            if (breakTopicPath != 0) {
               topicPath = breakTopicPath+1;
               *breakTopicPath = 0;
               }
            }
         }

      // set default error message level for verity callbacks
      int cbMessageVal = outMsgLevelStr ? atoi(outMsgLevelStr) : 15;
      
      // parse out language and charset (both optional) from languageparms
      Str newParms = 0;
      Str language = 0;
      Str charset = 0;
      Str breaklang = 0;
      if (langParms != 0) {   
         StrArgCopy(pSess->langParms, langParms);
         language = pSess->langParms;       
         if ((breaklang = strchr(pSess->langParms, ';')) != 0) {
            charset = breaklang+1;
            *breaklang = 0;
            }
         }

      VdkSessionNewArgRec   vdkSession;
      VdkStructInit(&vdkSession);

      vdkSession.appName        = (VdkCString)VdkAppName;
      vdkSession.appSignature   = (VdkCString)VdkAppSig;
      vdkSession.argc           = 1;
      vdkSession.argv           = (Str*)&libpath;
      vdkSession.vdkHome        = (VdkCString)libpath;
      if (cbMessageVal != 0) {
         vdkSession.outLevel       = (VdkMessageType)cbMessageVal;
         vdkSession.msgFnc         = (VdkMessageCBFnc)NSsessionMsgCB;
         vdkSession.msgArg         = (VdkCBArg)pSess;
         }
      vdkSession.charMapDriver  = (VdkCString)"8859";
      vdkSession.internalLocale = (VdkCString)"english";
      vdkSession.threadControl  = (VdkThread_InitOS 
					| VdkThread_MTHot 
					| VdkThread_ReentrantAPI);
// turned off this flag with VDK2.2, since it caused problems with the
// log_error (NSPR/NSAPI) in the callback for logging error messages.
// basically, the server created a thread to run the plugin.  the plugin
// calls verity, which does it's own threading.  when an error occurred in 
// a verity thread, it invoded the callback running in the NSAPI service
// thread, which called log_error, which caused an assertion failure
// since NSPR state was not there...
// 					| VdkThread_SelfService);

      if (topicPath != 0)
         vdkSession.topicSet = (VdkCString)topicPath;
      if (language != 0)
         vdkSession.internalLocale = (VdkCString)language;
      if (charset != 0)
         vdkSession.charMapDriver = (VdkCString)charset;

      if (parm) {
        NSavpair* pOpts = (NSavpair*)parm;
         Str maxMemory = pOpts->Get("NS-search-max-memory");
         Str maxFiles = pOpts->Get("NS-search-max-files");
         Str dateInputFormat = pOpts->Get("NS-date-input-format");
         Str dateOutputFormat = pOpts->Get("NS-date-output-format");
         if (maxMemory && *maxMemory)
            vdkSession.maxMemory = atoi(maxMemory);
         if (maxFiles && *maxFiles)
            vdkSession.maxFiles = atoi(maxFiles);
         if (dateInputFormat && *dateInputFormat)
            vdkSession.dateInputFormat = (VdkCString)dateInputFormat;
         if (dateOutputFormat && *dateOutputFormat)
            vdkSession.dateOutputFormat = (VdkCString)dateOutputFormat;
         }


      pSess->errorNum = VdkSessionNew(&pSess->sessHdl, &vdkSession);
      if (pSess->errorNum != VdkSuccess) 
         Errnum(sessionCreateFailed);
      if (pSess->langParms != 0 && breaklang != 0)
         *breaklang = ';';
      if (breakOutMsgLevel)
         *breakOutMsgLevel = ';';
      if (breakTopicPath)
         *breakTopicPath = ';';
      }
   }


NSsearch::~NSsearch() {
   if (_sessionData) {
      NSsearchSessionData* pSess = (NSsearchSessionData*)_sessionData;
      delete pSess->langParms;	pSess->langParms = 0;
      delete pSess->initParms;	pSess->initParms = 0;
      if (pSess->sessHdl)
         VdkSessionFree(pSess->sessHdl);
      pSess->sessHdl = 0;
      delete (NSsearchSessionData*)_sessionData;  _sessionData = 0;
      }
   }


NSsearch&  
NSsearch::GetDoc(NSsearchResults& results, NSsearchArgs& args, ostream& outp, 
   NSsearchGetDocCB cb, int offset) {

   NSsearchSessionData* pSess = (NSsearchSessionData*)_sessionData;
   if (!pSess) {
      Errnum(ctrGetErr);
      return(*this);
      }
   
   Clear();
   NSsearchResultsData* pResults = (NSsearchResultsData*)(results._resultsData);

   NSbool 	endOfDoc = FALSE;
   NSbool	hlMode = FALSE;
   CStr		hlBegin = args.HLbegin();
   CStr		hlEnd = args.HLend();

   VdkDocStream  stream = 0;
   VdkDocStreamNewArgRec  dsArg;
   VdkStructInit(&dsArg);

   dsArg.type    = VdkDocStreamType_TaggedAsciiHL;
   dsArg.search  = pResults->srchHdl;
   dsArg.nth	 = offset;
   dsArg.maxSize = 16384;

   if (hlBegin && *hlBegin == 0)
      hlBegin = 0;

   if (hlEnd && *hlEnd == 0)
      hlEnd = 0;

   pResults->errorNum = VdkDocStreamNew(pSess->sessHdl, &stream, &dsArg);
   if (pResults->errorNum != VdkSuccess) 
      Errnum(docStreamCreateFailed);
   else while (Good() && !endOfDoc) {
      VdkDocStreamReadArgRec dsArg;
      VdkStructInit(&dsArg);

      VdkDocStreamReadOut dsRead = 0;

      pResults->errorNum = VdkDocStreamRead(stream, &dsArg, &dsRead);
      if (pResults->errorNum != VdkSuccess) 
         Errnum(docStreamReadFailed);
      else for (int i = 0; i < dsRead->tokenPtrCount; i++){
         VdkToken   token = dsRead->tokenPtrArray[i];
         VdkCString p = 0;
         switch (token->type) {
            case VdkTokenType_Eof:
	       endOfDoc = TRUE; 
               break;
            case VdkTokenType_Highlight:
	       hlMode = TRUE;
               p = token->data.hl.buffer;
	       break;
            case VdkTokenType_Buffer:
	       p = token->data.buf.buffer;
               break;
            }
         if (!p) 
            continue;
         if (hlMode && hlBegin && hlEnd)
	    outp << hlBegin;
         if (cb)
            (cb)((CStr)p, outp, 0);
         else
            outp << (CStr)p;
         if (hlMode && hlEnd)
	   outp << hlEnd;
         hlMode = FALSE;
         }

      if (dsRead)
         VdkDocStreamReadFree(dsRead);
      dsRead = 0;
      }
   if (stream) 
      VdkDocStreamFree(stream);
   stream = 0;

   return(*this);
   }


NSsearch& 
NSsearch::MapAttributes(void* collHdl, NSavpair& attrMap) {
   VdkCollectionSetArgRec collInfo;
   VdkStructInit(&collInfo);
   collInfo.serviceLevel = VdkServiceType_DataPrep;

   int arySize = attrMap.Count();
   VdkFieldAlias* aryAttrAlias= new VdkFieldAlias[sizeof(VdkFieldAlias*)*arySize];
   PGen cookie = 0;
   Str  realName = 0;
   Str  alias = 0;
   attrMap.First(realName, alias, cookie);
   for (int i=0; i < arySize; i++) {
      VdkFieldAliasRec* ptr= new VdkFieldAliasRec;
      VdkStructInit(ptr);
      ptr->name = (VdkCString)alias;
      ptr->definition= (VdkCString)realName;
      ptr->preferred = VdkFlag_On;
      aryAttrAlias[i] = ptr;
      attrMap.Next(realName, alias, cookie);
      }
   collInfo.aliasArray= aryAttrAlias;
   collInfo.aliasCount= arySize;
   VdkCollectionSetInfo((VdkCollection)collHdl,&collInfo);

   for (i=0; i < arySize; i++)   {
      delete collInfo.aliasArray[i];  collInfo.aliasArray[i] = 0;
      }
   delete []collInfo.aliasArray;  collInfo.aliasArray = 0;
   collInfo.aliasArray=0;
   collInfo.aliasCount=0;
   return(*this);
   }


NSsearch&  
NSsearch::OpenCollection(CStr idxPath, void*& collHdl, NSsearchOpenStruct* state){
   NSsearchSessionData* pSess = (NSsearchSessionData*)_sessionData;
   if (!pSess) {
      Errnum(ctrGetErr);
      return(*this);
      }

   // check to see if this collection already exists.

   if (pSess->collectionOffsets.Get(idxPath) != 0) {
      Errnum(ctrGetErr);
      return(*this);
      }

   VdkCollectionOpenArgRec collOpen;
   VdkStructInit(&collOpen);
   collOpen.path = (VdkCString)idxPath;
   collOpen.serviceLevel = VdkServiceType_Search;
   VdkCollection& pColl= pSess->collHdlArray[pSess->collectionCount];
   pSess->errorNum = VdkCollectionOpen(pSess->sessHdl, &pColl, &collOpen);
   collHdl = pColl;
   if (pSess->errorNum != VdkSuccess)
      Errnum(searchError);
   else {
      char collectionOffset[32];
      memset(collectionOffset, sizeof(collectionOffset), 0);
      sprintf(collectionOffset, "%d", pSess->collectionCount);
      pSess->collectionOffsets.Put(idxPath, collectionOffset);
      pSess->collectionCount++;
      if (state) {
         VdkCollectionGetArgRec infoIn;
         VdkCollectionGetOut infoOut = 0;
         VdkStructInit(&infoIn);
         state->status = COLLECTION_STATUS_UNKNOWN;
         state->collectionSize = 0;
         if (VdkCollectionGetInfo(pColl, &infoIn, &infoOut) == 0) {
            switch (infoOut->state) {
               case VdkCollectionState_Up: 
                  state->status = COLLECTION_OPEN; 
                  break;
               case VdkCollectionState_Down: 
                  state->status = COLLECTION_CLOSED; 
                  break;
               case VdkCollectionState_NeedsRepair: 
                  state->status = COLLECTION_BROKEN; 
                  break;
               default:
                  state->status = COLLECTION_STATUS_UNKNOWN; 
               }
            state->collectionSize = (infoOut->maxId - infoOut->minId) + 1;
            }
         if (infoOut)
            VdkCollectionGetInfoFree(infoOut);
         infoOut = 0;
         }
      }
   return(*this);
   }


NSsearch&  
NSsearch::Search(NSsearchResults& results, NSsearchArgs& args) {
   NSsearchSessionData* pSess = (NSsearchSessionData*)_sessionData;
   if (!pSess) {
      Errnum(ctrGetErr);
      return(*this);
      }
   
   Clear();
   NSsearchResultsData* pRes = (NSsearchResultsData*)(results._resultsData);
   NSsearchArgsData* pArgs = (NSsearchArgsData*)(args._argsData);
   if (InitResults(pRes, pArgs, pSess->sessHdl) == FALSE)
      return(*this);

   int collCount = args.CollectionCount();
   if (collCount == 0){
      Errnum(searchError);
      return(*this);
      }

   VdkCollection* collHdls = new VdkCollection[collCount];
   if (collHdls == 0) {
      Errnum(outOfMem);
      return(*this);
      }

   int	collIx = 0;
   CStr	idxPath = args.CollectionPath(collIx);
   if (idxPath == 0)
      Errnum(searchError);
   else while (idxPath != 0) {
      Str collHdlOffset = pSess->collectionOffsets.Get(idxPath);
      if (collHdlOffset == 0) {
         Errnum(searchError);
         break;
         }
      collHdls[collIx] = pSess->collHdlArray[atoi(collHdlOffset)];
      idxPath = args.CollectionPath(++collIx);
      }

   if (Good())  {
      VdkSearchNewArgRec searchNew;
      VdkStructInit(&searchNew);

      searchNew.queryQuestion = (VdkCString)args.Query();
      searchNew.maxDocs = args.LargestSet();
      searchNew.enabledCount = collCount;
      searchNew.enabledArray = collHdls;
      if (pArgs->sortby)
         searchNew.sortSpec = (VdkCString)(pArgs->sortby);
      SetQueryType(args, searchNew);

      pRes->errorNum = VdkSearchNew(pSess->sessHdl, &pRes->srchHdl, &searchNew);
      if (pRes->errorNum != VdkSuccess)
         Errnum(searchError);
      }

   if (Good()) 
      if (LoadResults(pArgs, pRes) == FALSE)
         Errnum(searchError);

   delete []collHdls;  collHdls = 0;
   
   return(*this);
   }


NSsearch&  
NSsearch::GetMore(NSsearchResults& results, NSsearchArgs& args, int searchBase) {
   NSsearchSessionData* pSess = (NSsearchSessionData*)_sessionData;
   if (!pSess) {
      Errnum(ctrGetErr);
      return(*this);
      }
   
   Clear();
   NSsearchResultsData* pRes = (NSsearchResultsData*)(results._resultsData);
   NSsearchArgsData* pArgs = (NSsearchArgsData*)(args._argsData);
   pArgs->searchBase = searchBase;
   if (LoadResults(pArgs, pRes) == FALSE)
      Errnum(searchError);
   return(*this);
   }


NSsearch&  
NSsearch::SearchDoc(NSsearchResults& results, NSsearchArgs& args, CStr docPath) {
   NSsearchSessionData* pSess = (NSsearchSessionData*)_sessionData;
   if (!pSess) {
      Errnum(ctrGetErr);
      return(*this);
      }
   
   Clear();
   NSsearchResultsData* pRes = (NSsearchResultsData*)(results._resultsData);
   NSsearchArgsData* pArgs = (NSsearchArgsData*)(args._argsData);
   if (InitResults(pRes, pArgs, pSess->sessHdl) == FALSE)
      return(*this);

   CStr idxPath = args.CollectionPath(0);
   if (idxPath == 0)
      Errnum(searchError);
   else  {
      Str collHdlOffset = pSess->collectionOffsets.Get(idxPath);
      if (collHdlOffset == 0)
         Errnum(searchError);
      else {
         NormalizePathSlashes((char *)docPath);
         // note that DOC_FN will always contain the file path to the HTML version
         // of a document.  VdkVgwKey is used to store the source document.
         Str sq = (Str)docPath;

         VdkCollection* collHdl = &(pSess->collHdlArray[atoi(collHdlOffset)]);
         VdkSearchNewArgRec searchNew;
         VdkStructInit(&searchNew);
         SetQueryType(args, searchNew);
         searchNew.queryQuestion = (VdkCString)pArgs->query;
         searchNew.sourceQuestion = (VdkCString)sq;
         searchNew.sourceQParser = VdkQParser_Simple;
         searchNew.maxDocs = 1;
         searchNew.enabledCount = 1;
         searchNew.enabledArray = collHdl;

         int rc = VdkSearchNew(pSess->sessHdl, &pRes->srchHdl, &searchNew);
         if (rc != VdkSuccess) 
            Errnum(searchError);
         }
      }

   if (Good()) 
      if (LoadResults(pArgs, pRes) == FALSE)
         Errnum(searchError);

   return(*this);
   }


// protected member functions

void
NSsearch::Dump(ostream& o) const {
   o << endl;
   }

CStr
NSsearch::Name() const {
   return("NSsearch");
   }
