/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
// CONFIDENTIAL AND PROPRIETARY SOURCE CODE OF NETSCAPE COMMUNICATIONS CORPORATION
// Copyright (c) 1997 Netscape Communications Corporation.  All Rights Reserved.
// Use of this Source Code is subject to the terms of the applicable license 
// agreement from Netscape Communications Corporation. The copyright notice(s) in 
// this Source Code does not indicate actual or intended publication of this 
// Source Code.
//
// Name:    nssrchres.cpp
// Contact: mjb<barbarin@netscape.com>
// -------------------------------------------------------------------------------

#include "nssrchres.h"
#include "nssrchstructs.h"
#include "nsirutil.h"
#include "vdk.h"

NSsearchResults::NSsearchResults() 
: _resultsData((void*)new NSsearchResultsData) {
   if (_resultsData == 0)
      Errnum(outOfMem);
   else {  // set all the default for the results;
      NSsearchResultsData* pResults = (NSsearchResultsData*)_resultsData;

      pResults->collectionPath = 0;
      pResults->docsFound = 0;
      pResults->docsSearched = 0;
      pResults->docsReturned = 0;
      pResults->errorMsg[0] = '\0';
      pResults->errorNum = 0;
      pResults->score = 0;
      pResults->sessHdl = 0;
      pResults->srchHdl = 0;
      pResults->success = TRUE;
      pResults->table = 0;
      pResults->tableFields = 0;
      pResults->tableFieldsCount = 0;
      pResults->tableFieldsMap = 0;
      pResults->docReadOut = 0;
      pResults->searchBase = 0;     
      }
   } 


NSsearchResults::~NSsearchResults() {
   if (_resultsData) {
      NSsearchResultsData* pResults = (NSsearchResultsData*)_resultsData;

      delete pResults->tableFieldsMap;	
      pResults->tableFieldsMap=0;

      delete []pResults->table;		
      pResults->table=0;

      delete []pResults->tableFields;	
      pResults->tableFields=0;

      delete []pResults->collectionPath;
      pResults->collectionPath=0;

      if (pResults->srchHdl)
         VdkSearchFree(pResults->srchHdl);
      pResults->srchHdl = 0;
      if (pResults->docReadOut)
         VdkDocReadBatchFree(pResults->docReadOut);
      pResults->docReadOut = 0;
      delete (NSsearchResultsData*)_resultsData;  _resultsData = 0;
      }
   }
   

Str  
NSsearchResults::AltKey(int offset) const {
   NSsearchResultsData* pResults = (NSsearchResultsData*)_resultsData;
   if (!pResults)
      return(0);

   VdkCString*  values = pResults->docReadOut->values;
   VdkInt2      numCols = pResults->docReadOut->numColumns;
   // retrieving VdkVgwKey
   return((Str)values[offset*numCols + 2]);
   }


Str 
NSsearchResults::CollectionPath(int offset) {
   NSsearchResultsData* pResults = (NSsearchResultsData*)_resultsData;
   if (!pResults)
      return(0);

   pResults->errorMsg[0] = '\0';	// initialize error message to 0
   delete []pResults->collectionPath;	// delete the collection path
   pResults->collectionPath = 0;	// set collection path to 0

   VdkDocId  docID;
   offset += pResults->searchBase;
   pResults->errorNum = VdkSearchNthToDocId(pResults->srchHdl, offset, &docID, 0);
   if (pResults->errorNum != VdkSuccess) 
      Errnum(searchError);
   else {
      VdkCollectionGetArgRec  getInfo;
      VdkStructInit(&getInfo);

      VdkDocMapArgRec  docMap;
      VdkStructInit(&docMap);

      VdkDocMapOut  mapOut = 0;

      docMap.id = docID;
      pResults->errorNum = VdkDocMap(pResults->sessHdl, &docMap, &mapOut);
      if (pResults->errorNum != VdkSuccess) 
         Errnum(searchError);
      else {
         VdkCollectionGetOut  info = 0;
         int err = VdkCollectionGetInfo(mapOut->collection, &getInfo, &info);
         if (err != VdkSuccess) {
            pResults->errorNum = err;
            Errnum(searchError);
            }
         else if (StrArgCopy(pResults->collectionPath, (Str)info->path) == FALSE)
            Errnum(outOfMem);
         else
            NormalizePathSlashes(pResults->collectionPath);

         if (info)
            VdkCollectionGetInfoFree(info);
         info = 0;
         }

      if (mapOut)
         VdkDocMapFree(mapOut);
      mapOut = 0;
      }
   return(pResults->collectionPath);
   }


int  
NSsearchResults::DocsFound() const {
   if (!_resultsData)
      return(0);
   return(((NSsearchResultsData*)_resultsData)->docsFound);
   }


Str  
NSsearchResults::DocPath(int offset) const {
   NSsearchResultsData* pResults = (NSsearchResultsData*)_resultsData;
   if (!pResults)
      return(0);

   VdkCString*  values = pResults->docReadOut->values;
   VdkInt2      numCols = pResults->docReadOut->numColumns;
   return((Str)values[offset*numCols + 1]);
   }


int  
NSsearchResults::DocsReturned() const {
   if (!_resultsData)
      return(0);
   return(((NSsearchResultsData*)_resultsData)->docsReturned);
   }


int  
NSsearchResults::DocScore(int offset) const {
   NSsearchResultsData* pResults = (NSsearchResultsData*)_resultsData;
   if (!pResults)
      return(0);

   VdkCString*  values = pResults->docReadOut->values;
   VdkInt2      numCols = pResults->docReadOut->numColumns;

   char cleanScore[8];
   Str  ptr = cleanScore;
   Str  score = (Str)values[offset*numCols];
   while (*score) {
      if (*score >= '0' && *score <= '9')
        *ptr++ = *score;
      score++;
      }
   *ptr = 0;

   pResults->score = atoi(cleanScore);
   return(pResults->score);
   }


int  
NSsearchResults::DocsSearched() const {
   if (!_resultsData)
      return(0);

   return(((NSsearchResultsData*)_resultsData)->docsSearched);
   }


Str
NSsearchResults::ErrorMsg() const {
   if (!_resultsData)
      return(0);

   return(((NSsearchResultsData*)_resultsData)->errorMsg);
   }


int
NSsearchResults::ErrorNum() const {
   if (!_resultsData)
      return(0);

   return(((NSsearchResultsData*)_resultsData)->errorNum);
   }


Str  
NSsearchResults::Field(int offset, CStr fieldname) const {
   NSsearchResultsData* pResults = (NSsearchResultsData*)_resultsData;
   if (!pResults || offset > pResults->docsReturned)
      return(0);

   Str data = 0;
   if (pResults->tableFieldsMap != 0) {
      Str ixStr = pResults->tableFieldsMap->Get(fieldname);
      if (ixStr) {
         int ix = atoi(ixStr);
         VdkCString*  values = pResults->docReadOut->values;
         VdkInt2      numCols = pResults->docReadOut->numColumns;
         data = (Str)values[offset*numCols + NSSEARCH_DEFAULT_ATTR_CNT + ix];
         }
      }
   return(data);
   }


Str  
NSsearchResults::FieldName(int offset) const {
   NSsearchResultsData* pResults = (NSsearchResultsData*)_resultsData;
   if (!pResults)
      return(0);

   return(offset < pResults->tableFieldsCount ? pResults->table[offset] : 0);
   }


// protected member functions

void
NSsearchResults::Dump(ostream&) const {
   }


CStr
NSsearchResults::Name() const {
   return("NSsearchResults");
   }
