/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * A set of utility functions
 *
 * Daryoush Paknad
 *
 */
#include <string.h>
#include "mdtypes.h"
#include "mdutil.h"

#include "memory.h"


#define PRIME1                 37
#define PRIME2                 1048583


NSPR_BEGIN_EXTERN_C

NSAPI_PUBLIC long MakeUID( char *name )
{
	int len, i;
    unsigned long   uid;

    uid = 0;
    while ( *name ){
#ifdef XP_WIN32
				char normalized = tolower(*name++);
#else
				char normalized = *name++;
#endif
        uid = uid*PRIME1 ^ (normalized - ' ');
		}

#if 0
    while ( *hostName )
        uid = uid*PRIME1 ^ (*hostName++ - ' ');
#endif
    uid %= PRIME2;

	return uid;
}


NSAPI_PUBLIC int CMCopyString( char **dest, char *src )
{
	int  len;
	
	if ( src == (char *) NULL )
	    return 0;
	
	len = CS_strlen( src );
	
	*dest = (char *) PERM_MALLOC( len + 1 );
	if ( !*dest )
	    return 0;

	CS_strcpy( *dest, src );
	return len;
}




NSAPI_PUBLIC CMStrObj *NewCMStrObj( int size )
{
	CMStrObj  *tPtr;
        
	tPtr = (CMStrObj *) PERM_MALLOC( sizeof( CMStrObj ));
	if ( tPtr == (CMStrObj *) NULL )
	    return (CMStrObj *) NULL;

	CS_Memset( tPtr, 0, sizeof( CMStrObj ));
	tPtr->buffer = (char *) PERM_MALLOC(  size * sizeof( char )  );
	if ( tPtr->buffer == (void *) NULL ) {
		PERM_FREE( tPtr );
		return (CMStrObj *) NULL;
	}
	CS_Memset( tPtr->buffer, 0, ( size * sizeof( char )  ));
	tPtr->size = size;
	tPtr->used = 0;
	return tPtr;
}




static int CMStrObjCheckSize( CMStrObj  *obj, int len )
{
	if ( obj->size > (obj->used+len+1) )
	    return true;
        

	obj->buffer = (char  *) PERM_REALLOC( obj->buffer,
									   ( obj->size + obj->size )* sizeof( char ));

	CS_Memset( (char*)((long ) obj->buffer + obj->size), 0, obj->size );

	obj->size += obj->size;

	if ( obj->buffer == (char *) NULL )
	    return false;

	return true;
}


NSAPI_PUBLIC char * CMStrObjAdd( CMStrObj *obj, char *str ) 
{

	int  len;
	
	if ( !obj || !str )
	    return (char *) NULL;

	len = CS_strlen( str );
	
	if ( CMStrObjCheckSize( obj, len ) == false )
	    return (char *) NULL;

        
	CS_strcat( obj->buffer, str );
	obj->used += len;
	
	return obj->buffer;
}




NSAPI_PUBLIC char * CMStrObjAddBytes( CMStrObj *obj, char *str, int len ) 
{
  if ( !obj || !str )
	return (char *) NULL;

  if ( CMStrObjCheckSize( obj, len ) == false )
	return (char *) NULL;

  memcpy( (char *) (obj->buffer+obj->used), str, len );
  obj->used += len;
	
  return obj->buffer;
}


NSAPI_PUBLIC char * CMStrObjGetString( CMStrObj *obj ) 
{
	if ( !obj || !obj->buffer )
	    return (char *) NULL;
	
	return obj->buffer;
}


NSAPI_PUBLIC int CMStrObjGetSize( CMStrObj *obj ) 
{
	if ( !obj || !obj->buffer )
	    return 0;
	
	return obj->used;
}



NSAPI_PUBLIC void CMStrObjFree( CMStrObj *obj ) 
{
	if ( !obj )
	    return;
	
	if ( obj->buffer )
	    PERM_FREE( obj->buffer );
	
	PERM_FREE( obj );
}


NSAPI_PUBLIC void CMStrObjRelease( CMStrObj *obj ) 
{
	if ( !obj || !obj->buffer )
	    return;
	
	CS_Memset( obj->buffer, 0, obj->size );
	obj->used = 0;
}

NSPR_END_EXTERN_C
