/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * A dynamic object array/list implementation
 *
 * Daryoush Paknad
 *
 */
#include "mdtypes.h"
#include "nsarray.h"

NSPR_BEGIN_EXTERN_C


static int NSObjArrayCheckSize(NSObjArray *objArray);


NSAPI_PUBLIC NSObjArray *NSObjArrayNew( unsigned long initSize, int objSize )
{
        NSObjArray  *tPtr;
        
        tPtr = (NSObjArray *) PERM_MALLOC( sizeof( NSObjArray ));
        if ( tPtr == (NSObjArray *) NULL )
            return (NSObjArray *) NULL;

        CS_Memset( tPtr, 0, sizeof( NSObjArray ));
        tPtr->objects = (void *) PERM_MALLOC( (initSize * objSize) );
        if ( tPtr->objects == (void *) NULL ) {
                PERM_FREE( tPtr );
            return (NSObjArray *) NULL;
        }
        CS_Memset( tPtr->objects, 0, ( initSize * objSize ));
        tPtr->size = initSize;
        tPtr->numObj = 0;
        tPtr->objSize = objSize;
        
        return tPtr;
}



static int NSObjArrayCheckSize( NSObjArray  *objArray )
{
        long  size;
        
        if ( objArray->size > objArray->numObj )
            return true;
        
        size =  (long) objArray->size  * (long)objArray->objSize;
        objArray->objects = (void *) PERM_REALLOC( objArray->objects,
                                   ( objArray->size + objArray->size )* (long)objArray->objSize );

#if 0  /* ifndef WIN32 */
        CS_Memset( ((NSHugePtr) objArray->objects + size), 0, size );
#else
        CS_Memset( (void*)((long ) objArray->objects + size), 0, size );
#endif  
        objArray->size += objArray->size;

        if ( objArray->objects == (void *) NULL )
            return false;

        return true;
}


NSAPI_PUBLIC void * NSObjArrayNewObj( NSObjArray  *objArray ) 
{
        long index;
        
        if ( NSObjArrayCheckSize( objArray ) == false )
            return (void *) NULL;
        

        index = (long) ((long) objArray->numObj) * (long) objArray->objSize;
        objArray->numObj++;

#if 0  /* ifndef WIN32 */
        return (void *) ((NSHugePtr) objArray->objects + index);
#else
        return (void *) ((long) objArray->objects + index);
#endif
}

        
NSAPI_PUBLIC void *NSObjArrayGetObj( NSObjArray *objArray, int elemNum ) 
{
#if 0  /* ifndef WIN32 */
        return (void *) ((NSHugePtr)objArray->objects + (elemNum*(long)objArray->objSize));
#else
        return (void *) ((long)objArray->objects + (elemNum*(long)objArray->objSize));
#endif
}

        
NSAPI_PUBLIC void *NSObjArrayGetObjNum( NSObjArray *objArray, int elemNum ) 
{
#if 0  /* ifndef WIN32 */
        return (void *) ((NSHugePtr)objArray->objects + (elemNum*(long)objArray->objSize));
#else
        return (void *) ((long)objArray->objects + (elemNum*(long)objArray->objSize));
#endif
}


NSAPI_PUBLIC int NSObjArrayRemoveObj( NSObjArray *objArray, int elemNum ) 
{
        void *tmpPtr;
        long  size;

        size =  (long) objArray->size  * (long)objArray->objSize - 1;

        tmpPtr = (void *) PERM_MALLOC( size );
        if ( tmpPtr == (void *) NULL ) 
            return 0;

        CS_Memset( tmpPtr, 0, size );
        memcpy( tmpPtr, (void *) objArray->objects, (elemNum*(long)objArray->objSize) );

#if 0  /* ifndef WIN32 */
        memcpy((void *)((NSHugePtr) tmpPtr + (elemNum*(long)objArray->objSize)), 
(void *) ((NSHugePtr)objArray->objects + ((elemNum+1)*(long)objArray->objSize)), 
(size_t) ((objArray->numObj - (elemNum + 1)) * (long)objArray->objSize) );
#else
        memcpy( (void *) ((long) tmpPtr + (elemNum*(long)objArray->objSize) ), 
   (void *) ((long)objArray->objects + ((elemNum+1)*(long)objArray->objSize)), 
   (size_t) ((objArray->numObj - (elemNum + 1)) * (long)objArray->objSize) );

#endif
   PERM_FREE( objArray->objects );
   objArray->objects = tmpPtr;
   objArray->numObj --;
   objArray->size --;

   return 1;
}

        
NSAPI_PUBLIC void *NSObjArrayGetLastObj( NSObjArray  *objArray ) 
{
#if 0  /* ifndef WIN32 */
        return (void *) ((NSHugePtr)objArray->objects + 
                       ( (objArray->numObj - 1 ) * (long) objArray->objSize));
#else
        return (void *) ((long)objArray->objects + 
                      ( (objArray->numObj - 1 ) * (long) objArray->objSize));
#endif
}

NSAPI_PUBLIC int NSObjArrayGetNumObj( NSObjArray  *objArray )
{
        return objArray->numObj;
}

NSAPI_PUBLIC void NSObjArrayReset( NSObjArray  *objArray )
{
        if ( objArray == (NSObjArray *) NULL )
            return;
        
#if defined(WIN)
        CS_Memset( ((NSHugePtr) objArray->objects ), 0, 
				  ( (objArray->numObj) * (long) objArray->objSize) );
#else
        CS_Memset( (void*)((long ) objArray->objects ), 0, 
				  ( (objArray->numObj) * (long) objArray->objSize) );
#endif  
        objArray->numObj = 0;
}



NSAPI_PUBLIC void  NSObjArrayFree( NSObjArray  *objArray ) 
{
        if ( objArray == (NSObjArray *) NULL )
            return;

        if ( objArray->objects != (void *) NULL )
            PERM_FREE( objArray->objects );

        PERM_FREE( objArray ); 
}

NSPR_END_EXTERN_C
