/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include <netsite.h>
#include <libmsgdisp/timer_list.h>

TIMERLIST TimerListInit()
{
	TIMERLIST me = (TIMERLIST) PERM_MALLOC(sizeof(_timerList));
	if (me)
		me->_head     = NULL;
	return me; 
}

void TimerListDelete(TIMERLIST me)
{
	if (me) {
   		TriggerInfoStruct* curr = me->_head;
		TriggerInfoStruct* ptr  = NULL;
   		while (curr) {
			ptr = curr;
			curr = curr->next;
			PERM_FREE(ptr);
		}
		PERM_FREE(me);
	}
}	




TriggerInfoStruct* TimerListFirst(TIMERLIST me)
{
	if (me) 
		return me->_head;
	else
		return NULL;
}

TriggerInfoStruct* TimerListGet(TIMERLIST me)
{
	TriggerInfoStruct* result = NULL;
	if (me) {
		result = me->_head;
		if (me->_head)
			me->_head = me->_head->next;
	}
	return result;
}
		
	
void TimerListInsertInOrder(TIMERLIST me, TriggerInfoStruct *newNode)
{
	// Empty list
	if (me->_head == NULL) {
		me->_head = newNode;
	}
	else if (me->_head->timeToTrigger > newNode->timeToTrigger) {
		// Insert at beginning
		newNode->next = me->_head;
		me->_head = newNode;
	}
	else {
		TriggerInfoStruct* prev = NULL;
		TriggerInfoStruct* curr = me->_head;
		while (curr != NULL) {
			if (curr->timeToTrigger <= newNode->timeToTrigger) {
				prev = curr;
				curr = curr->next;
			}
			else
				break;
		}
		newNode->next = prev->next;
		prev->next = newNode;
	}
}


int  TimerListDeleteElementGivenTriggerTime(TIMERLIST me, time_t secs)
{
	int result = NOT_FOUND;
	if (me->_head == NULL)
		result = NOT_FOUND;
	else {
		TriggerInfoStruct* curr = me->_head;
		TriggerInfoStruct* prev = NULL;
		while ((curr != NULL) && (curr->timeToTrigger != secs)) {
			prev = curr;
			curr = curr->next;
		}
		if (curr != NULL) {
			if (curr == me->_head)
				me->_head = me->_head->next;
			else
				prev->next = curr->next;
			PERM_FREE(curr);
			result = TIMERLIST_SUCCESS;
		}
		else
			result = NOT_FOUND;
	}
	return result;
}

void  TimerListPrintList(TIMERLIST me)
{
#ifdef TIMER_DEBUG
	TriggerInfoStruct* curr = me->_head;
	if (me->_head == NULL)
		printf("Empty List \n");
	else {
		int i = 0;
		while (curr) {
			i++;
			printf("Node %d triggers at %ld \n", i,
				curr->timeToTrigger);
			curr = curr->next;
		}
 	}
#endif /* TIMER_DEBUG */
}

int  TimerListIsEmpty(TIMERLIST me)
{
	return (me->_head == NULL ? 1 : 0);
}

size_t TimerListEntries(TIMERLIST me)
{
	TriggerInfoStruct* curr = me->_head;
	int i = 0;
	while (curr) {
		i++;
		curr = curr->next;
	}
	return i;
}
	
	
		
