/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
#ifndef WIN32

#if HAS_TIMER  

#include <iostream.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
 
 
#include <rw/rwtime.h>
 
#include <base/systhr.h>
#include <base/crit.h>
#include <base/net.h>
 
#include <libmsgdisp/msgchnel.h>

#include "TimerServiceProvider.h"
#include "message.h"

struct _pipeCommunicationStruct {
  unsigned long _timeToTrigger;
  void*  _clntData;
  long (*_fptr)(short, long, void*, void*);
  char _whatToDo;
};


void PrintList();

static RWTIsvSlist<TriggerInfoStruct> _list;


int        TimerServiceProvider::_minDelta = 300;
int        TimerServiceProvider::_fd[2] = {-1, -1};
SYS_THREAD TimerServiceProvider::_myTimerThread = -1;
CRITICAL   TimerServiceProvider::_timerCriticalSecVar = NULL;
timeval* TimerServiceProvider::_myTimeOutStruct = NULL;



TimerServiceProvider::TimerServiceProvider(int minDelta)
{
   if (minDelta > 180)
      _minDelta = minDelta;
   _valid = 1; 
   if (pipe(_fd) < 0)
      _valid = 0;
   _myTimeOutStruct = new timeval;
   _myTimerThread = systhread_start(20, 0, serviceTimerThread, this);
   _timerCriticalSecVar = crit_init();
   if (_myTimerThread == SYS_THREAD_ERROR) 
	_valid = 0;
}


TimerServiceProvider::~TimerServiceProvider()
{
   delete _myTimeOutStruct;
   systhread_terminate(_myTimerThread);
   crit_terminate(_timerCriticalSecVar);
}




void TimerServiceProvider::serviceTimerThread(void* clntData)
{
   TimerServiceProvider* myself = (TimerServiceProvider*)clntData;
   fd_set readSet;
   fd_set writeSet;
   fd_set exectSet;
   FD_ZERO(&readSet);
   FD_ZERO(&writeSet);
   FD_ZERO(&exectSet);
   FD_SET(_fd[0], &readSet);
   int maxfd = _fd[0] + 1;
  
   
   while (1) {
      timeval* ptr;
      TriggerInfoStruct*  firstElement = NULL;
      if ((firstElement = (TriggerInfoStruct*)_list.first())) {
         _myTimeOutStruct->tv_sec = firstElement ->timeToTrigger - 
				    RWTime::now().seconds();
         _myTimeOutStruct->tv_usec = 0;
	 ptr = _myTimeOutStruct;
         //cout << " TimerServiceProvider::serviceTimerThread" << _myTimeOutStruct->tv_sec << endl;
       }
       else {
	 ptr = NULL;
       }
      
      FD_ZERO(&readSet);
      FD_SET(_fd[0], &readSet);
      int res = net_select(maxfd, &readSet, &writeSet, &exectSet, ptr);
      if (FD_ISSET(_fd[0], &readSet)) {
        _pipeCommunicationStruct pipeStruct;
      	int n = read(_fd[0],  (void*)&pipeStruct, sizeof(pipeStruct));
        if (pipeStruct._whatToDo == 'A')
	   myself->InsertInOrder(pipeStruct._timeToTrigger, pipeStruct._fptr, 
			      	pipeStruct._clntData);
        else if (pipeStruct._whatToDo == 'D')
	   myself->DeleteFromList(pipeStruct._timeToTrigger);
     }
     else if (res == 0) 
	MyTimeOutHandler();
   }
}

	

RWBoolean 
TimerServiceProvider::myTesterFunc(const TriggerInfoStruct* currListElement,
				  void* newElement)
{
    TriggerInfoStruct* p = (TriggerInfoStruct*) newElement;
    if (currListElement->timeToTrigger > p->timeToTrigger)
	return TRUE;
    else
        return FALSE;
}
 
int 
TimerServiceProvider::AddTimeObject(void* timerServiceObj, 
			unsigned long timeToTrigger, 
			long (*fnPtr)(short msdID, long objID, 
				      void *supplierData, void *consumerData ),
			 void* clntData)
{
    TimerServiceProvider* myself = (TimerServiceProvider*) timerServiceObj;
    // Check for validity of timeToTrigger
    // It should be > 0 and should be atleast "delta" away from the
    // current time
    if  ((timeToTrigger - RWTime::now().seconds() <= myself->_minDelta) ||
	 (timeToTrigger <= 0)) {
	cout << "Min Delta failed" << endl;
      return 0;
    }
    // Need to serialize accesses to this function
    crit_enter(_timerCriticalSecVar);
    _pipeCommunicationStruct pipeStruct;
    pipeStruct._timeToTrigger = timeToTrigger;
    pipeStruct._fptr          = fnPtr;
    pipeStruct._clntData      = clntData;
    pipeStruct._whatToDo      = 'A';
    char buff[256];
    int result = 1;
    if (write(_fd[1], &pipeStruct, sizeof(pipeStruct)) <= 0)
        result = 0;
    crit_exit(_timerCriticalSecVar);
    return 1;
}

int
TimerServiceProvider::DeleteTimeObject(void* timerServiceObj,
					unsigned long timeToTrigger)
{
    TimerServiceProvider* myself = (TimerServiceProvider*) timerServiceObj;
    // Check for validity of timeToTrigger
    // It should be > 0 and should be atleast "delta" away from the
    // current time
    if  ((timeToTrigger - RWTime::now().seconds() <= myself->_minDelta) ||
	 (timeToTrigger <= 0))
      return 0;

    // Need to serialize accesses to this function
    crit_enter(_timerCriticalSecVar);
    _pipeCommunicationStruct pipeStruct;
    pipeStruct._timeToTrigger = timeToTrigger;
    pipeStruct._fptr          = NULL;
    pipeStruct._clntData      = NULL;
    pipeStruct._whatToDo      = 'D';
    int result = 1;
    if (write(_fd[1], &pipeStruct, sizeof(pipeStruct)) <= 0)
        result = 0;
    crit_exit(_timerCriticalSecVar);
    return 1;
}

   

int
TimerServiceProvider::InsertInOrder(unsigned long timeToTrigger, 
				    long (*fnPtr) (short msdID, long objID, 
						void *supplierData,
						void *consumerData ),
		         	    void* clntData)
{
   TriggerInfoStruct* newElement = new TriggerInfoStruct;
   newElement->timeToTrigger = timeToTrigger;
   newElement->fnToCall      = fnPtr;
   newElement->clntData      = clntData;
   
      
   if (_list.entries() == 0) {
       // No entries in the list
	_list.insert(newElement);
   }
   else {
      // Non-empty list
      size_t posToInsert = _list.index(myTesterFunc, (void*) newElement);
      if (posToInsert == RW_NPOS) 
         _list.insert(newElement);
      else 
          _list.insertAt(posToInsert, newElement);
   }
   return 1;
}


int 
TimerServiceProvider::DeleteFromList(unsigned long timeToTrigger)
{
   TriggerInfoStruct* delElement = NULL;
   int result = 1;
   delElement = _list.remove(deleteTestFn, &timeToTrigger);
   if (delElement) {
	delete delElement;
   }
   else
	result = 0;
   return result;
}
  
   


RWBoolean 
TimerServiceProvider::deleteTestFn(const TriggerInfoStruct* lstElement, 
				   void* ptr)
{
   unsigned long* t = (unsigned long*)ptr;
   if (*t == lstElement->timeToTrigger)
      return TRUE;
   else
      return FALSE;
}


void TimerServiceProvider::MyTimeOutHandler()
{
    TriggerInfoStruct*  ptr = _list.removeFirst();
    if (ptr) {
	cout << "Timed Out !!!" << ptr->timeToTrigger << endl;
        if (ptr->fnToCall != NULL)
         ptr->fnToCall(NS_TIMER_MSG, ptr->timeToTrigger, NULL, ptr->clntData);
        delete ptr;
        ptr = NULL;
    }
}


void PrintList()
{
  for (int l = 0; l < _list.entries(); l++) {
	TriggerInfoStruct* p = (TriggerInfoStruct*)_list.at(l);
	cout << p->timeToTrigger << endl;
  }
}


#endif  /* HAS_TIMER */

#endif /* WIN32 */
