/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
#include <stdio.h>
#ifndef XP_WIN32
#include <unistd.h>
#endif

#ifdef XP_WIN32
#include <time.h>
#else
#include <sys/time.h>
#endif
#include <sys/types.h>
 
 
#ifdef XP_WIN32
#ifdef max
#undef max
#undef min
#endif
#endif 

#include <netsite.h> 
#include <base/systhr.h>
#include <base/crit.h>
#include <base/net.h>

#include <libmsgdisp/msgchnel.h>
#include <libmsgdisp/timer_srvc.h>
#include <libmsgdisp/timer_list.h>
#include <libmsgdisp/message.h>

#ifdef IRIX
#include <bstring.h>   /* fd_zero uses bzero */
#endif
#ifdef AIX
#include <strings.h>   /* fd_zero uses bzero */
#endif
static _timerServiceProvider _gTimerStruct;
static void serviceTimerThread(void* clntData);
static int InsertInOrder(TIMERSERVICEPROVIDER me, time_t timeToTrigger, 
		      long (*fnPtr)(short msdID, long objID,
                      void *supplierData, void *consumerData ),
		      void* clntData);
void Dump(TIMERSERVICEPROVIDER me, char* str);

// A correction factor to ensure that we dont miss out on the margins for a "delta" failure
static const int TIMER_CORRECTION = 5; 



NSAPI_PUBLIC TIMERSERVICEPROVIDER TimerServiceProviderInit(time_t minDelta)
{
   	if (minDelta > 0)
      		_gTimerStruct._minDelta = minDelta;
	else
			_gTimerStruct._minDelta = 60;
	_gTimerStruct._valid = 1; 
	_gTimerStruct._gResult = 0;
	_gTimerStruct._terminate = 0;
	_gTimerStruct._timeToTrigger = 0;
	_gTimerStruct._fptr = 0;
   	_gTimerStruct._list 	     = TimerListInit();
	_gTimerStruct._crit1	     = crit_init();
	_gTimerStruct._critPC        = crit_init();
	_gTimerStruct._dataAvailable = condvar_init(_gTimerStruct._critPC);
	_gTimerStruct._dataConsumed  = condvar_init(_gTimerStruct._critPC);
	_gTimerStruct._threadKilled  = condvar_init(_gTimerStruct._critPC);

	crit_enter(_gTimerStruct._critPC);
   	_gTimerStruct._myTimerThread = systhread_start(16, 1024*64, 
					serviceTimerThread, &_gTimerStruct);
   	if (_gTimerStruct._myTimerThread == SYS_THREAD_ERROR) 
		_gTimerStruct._valid = 0;
	else {
		condvar_wait(_gTimerStruct._dataConsumed);
	}
	crit_exit(_gTimerStruct._critPC);

	Dump(&_gTimerStruct,"After Init");
   	return &_gTimerStruct;
}

 


NSAPI_PUBLIC void TimerServiceProviderDelete(TIMERSERVICEPROVIDER me)
{
   if (_gTimerStruct._myTimerThread) {
	crit_enter(_gTimerStruct._critPC);
	_gTimerStruct._terminate = 1;
	condvar_notify(_gTimerStruct._dataAvailable);
	condvar_wait(_gTimerStruct._threadKilled);
	crit_exit(_gTimerStruct._critPC);
   }
   if (_gTimerStruct._crit1)
      crit_terminate(_gTimerStruct._crit1);
   if (_gTimerStruct._dataAvailable)
     condvar_terminate(_gTimerStruct._dataAvailable);
   if (_gTimerStruct._dataConsumed)
     condvar_terminate(_gTimerStruct._dataConsumed);
   if (_gTimerStruct._threadKilled)
     condvar_terminate(_gTimerStruct._threadKilled);
   if (_gTimerStruct._critPC)
      crit_terminate(_gTimerStruct._critPC);
}


NSAPI_PUBLIC int TimerServiceProviderIsValid(TIMERSERVICEPROVIDER me)
{ 
	return me->_valid;
}

void serviceTimerThread(void* clntData)
{

  crit_enter(_gTimerStruct._critPC);
  condvar_notify(_gTimerStruct._dataConsumed);

  while (1) {
    Dump(&_gTimerStruct, "while in service function"); 
    PRIntervalTime waitTime = 0;
    int secs;
    TriggerInfoStruct*  firstElement = NULL;
    
    if ((firstElement = TimerListFirst(_gTimerStruct._list)))
      secs = firstElement->timeToTrigger - time(0);
    else secs = 0;


    condvar_timed_wait(_gTimerStruct._dataAvailable, secs);
    
    if (_gTimerStruct._terminate) {
	// request to terminate thread
	condvar_notify(_gTimerStruct._threadKilled);
	crit_exit(_gTimerStruct._critPC);
	return;
    }
    if ((_gTimerStruct._timeToTrigger > 0) && (_gTimerStruct._fptr)) {
    		Dump(&_gTimerStruct, "before insert in consumer");
       		_gTimerStruct._gResult = InsertInOrder(&_gTimerStruct, 
				_gTimerStruct._timeToTrigger,
				_gTimerStruct._fptr,
			      	_gTimerStruct._clntData);
		_gTimerStruct._timeToTrigger = 0;
		_gTimerStruct._fptr = 0;
		_gTimerStruct._clntData = 0;
		condvar_notify(_gTimerStruct._dataConsumed);
	      	Dump(&_gTimerStruct,"after insert in consumer");
    }
    TimerServiceProviderTimeOutHandler(&_gTimerStruct);
  } //while (1)
}



	
 
NSAPI_PUBLIC int 
TimerServiceProviderAddTimeObject(TIMERSERVICEPROVIDER me,
			time_t timeToTrigger, 
			long (*fnPtr)(short msdID, long objID, 
				     void *supplierData, void *consumerData ),
			 void* clntData)
{
    int result;

    // Need to serialize accesses to this function
    crit_enter(_gTimerStruct._crit1);
    crit_enter(_gTimerStruct._critPC);
#ifdef TIMER_DEBUG
    fprintf(stderr, "Entered critical section in producer \n");
#endif
    

    // Check for validity of timeToTrigger
    // It should be > 0 and should be atleast "delta" away from the
    // current time
    if  (timeToTrigger <= 0) {
        crit_exit(_gTimerStruct._critPC);
	crit_exit(_gTimerStruct._crit1);
     	 return TIMER_INVALID_TIME;
    }

    if  ((timeToTrigger - time(0) + TIMER_CORRECTION) <= me->_minDelta) {
#ifdef TIMER_DEBUG
    	fprintf(stderr, "Min Delta failed \n");
#endif
	crit_exit(_gTimerStruct._critPC);
	crit_exit(_gTimerStruct._crit1);
	return TIMER_DELTA_FAILED;
    } 

    _gTimerStruct._timeToTrigger = timeToTrigger;
    _gTimerStruct._fptr          = fnPtr;
    _gTimerStruct._clntData      = clntData;

    condvar_notify(_gTimerStruct._dataAvailable);
    condvar_wait(_gTimerStruct._dataConsumed);
    result = _gTimerStruct._gResult;
    crit_exit(_gTimerStruct._critPC);
    crit_exit(_gTimerStruct._crit1);
    Dump(&_gTimerStruct, "after exiting critical section in consumer");
    return result;
}

int
InsertInOrder(TIMERSERVICEPROVIDER me, time_t timeToTrigger, 
	   long (*fnPtr) (short msdID, long objID, 
	   void *supplierData,
	   void *consumerData ),
	   void* clntData)
{
 	//TimerListPrintList(me->_list);
	TriggerInfoStruct* newElement = (TriggerInfoStruct*) 
			PERM_MALLOC(sizeof(TriggerInfoStruct));
	if (newElement) {
#ifdef TIMER_DEBUG
		fprintf(stderr, "Before TimerListInsertInOrder \n");
#endif
		newElement->timeToTrigger = timeToTrigger;
		newElement->fnToCall      = fnPtr;
		newElement->clntData      = clntData;
		newElement->next      	  = NULL;

		TimerListInsertInOrder(me->_list, newElement);
#ifdef TIMER_DEBUG
		fprintf(stderr, "After TimerListInsertInOrder \n");
#endif
		return TIMER_SUCCESS;
	}
	else 
		return TIMER_FAILURE;
}


NSAPI_PUBLIC void TimerServiceProviderTimeOutHandler(TIMERSERVICEPROVIDER me)
{
    int process = 1;
    int count  = 0;
    while (process) {
	    TriggerInfoStruct*  ptr = TimerListFirst(me->_list);
    	    if (ptr) {
		if (ptr->timeToTrigger <= time(0)+ me->_minDelta) {
			// First node in list has trigger time less than
			// current time + delta. Trigger this node
	    		ptr = TimerListGet(me->_list); // Removes and gets list head
        		if (ptr->fnToCall != NULL)
         		ptr->fnToCall(NS_TIMER_MSG, ptr->timeToTrigger, 
					NULL, ptr->clntData);
        		PERM_FREE(ptr);
        		ptr = NULL;
			count++;
#ifdef TIMER_DEBUG
			fprintf(stderr, "Number of nodes triggered = %d \n", count);
#endif
		}
		else  // first node in list is atleast delta away ; return to select
			process = 0;
    	    }
	    else 
		//would i ever come here ????
 		process = 0;
    }
}

void Dump(TIMERSERVICEPROVIDER me, char* str)
{
#ifdef TIMER_DEBUG
	fprintf(stderr, "&gTimerStruct = %ld %s \n", &_gTimerStruct, str);
	fprintf(stderr, "_gTimerStruct._gResult = %d %ld \n", 
		_gTimerStruct._gResult, &(_gTimerStruct._gResult));
	fprintf(stderr, "_gTimerStruct._gResultCopy = %d %ld \n", 
		_gTimerStruct._gResultCopy,
		&(_gTimerStruct._gResultCopy));
	fprintf(stderr, "_gTimerStruct._list = %ld \n", _gTimerStruct._list);
	fprintf(stderr, "_gTimerStruct._list->_head = %ld \n", _gTimerStruct._list->_head);
	TimerListPrintList(_gTimerStruct._list);
#endif
}



