/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

/***********************************************************************
** Includes
***********************************************************************/
#include "libnscore/nscore.h"
#include "libnscore/nsbtree.h"


NSbtree::~NSbtree() {
   _traverse = DeleteNode;
   Traverse2(_root, cout);
   }

//********************************************************************
// MEMBER:	Insert
// DESCRIPTION:
//	Inserts a generic pointer into the tree.
//********************************************************************
NSbtree& NSbtree::Insert(PGen data) {
   if (_root == 0) {
      if ((_root = new NSbnode(data)) == 0)
         Errnum(outOfMem);
      return(*this);
      }

   NSbnode* temp = _root;
   NSbnode* old = 0;

   while (temp != 0) {
      old = temp;
      if ((_comp)(temp->data, data) == 0)    // duplicates not allowed
         return(*this);

      if ((_comp)(temp->data, data) > 0)
         temp = temp->left;
      else
         temp = temp->right;
      }

   if ((_comp)(old->data, data) > 0) {
      if ((old->left = new NSbnode(data)) == 0)
         Errnum(outOfMem);
     }
   else {
      if ((old->right = new NSbnode(data)) == 0)
         Errnum(outOfMem);
      }

   return(*this);
   }


//********************************************************************
// MEMBER:	Dump
// DESCRIPTION:
//	Debugging aid to dump the contents of the current object.
//********************************************************************
void NSbtree::Dump(ostream& o) const {
   Traverse(_root, o);
   }


//********************************************************************
// MEMBER:	Name
// DESCRIPTION:
//	Returns the class name for the instance.
//********************************************************************
CStr NSbtree::Name() const {
   return("NSbtree");
   }


//********************************************************************
// MEMBER:	Find
// DESCRIPTION:
//	Find a node in the btree matching the given pointer.
//********************************************************************
PGen NSbtree::Find(NSbnode* r, PGen data) {
   if (r == 0)
      return(0);
   s16 compareReturn = (_comp)(r->data, data);
   if (compareReturn == 0) 
     return(r->data);
   else if (compareReturn > 0)
      return(Find(r->left, data));
   else
      return(Find(r->right, data));
   }


//********************************************************************
// MEMBER:	Traverse
// DESCRIPTION:
//	Traverse the btree.
//********************************************************************
void NSbtree::Traverse(NSbnode* r, ostream& o) const{
   if (r != 0) {
      Traverse(r->left, o);
      (_traverse)(r->data, o);
      Traverse(r->right, o);
      }
   }


//********************************************************************
// MEMBER:	Traverse2
// DESCRIPTION:
//      This is duplicate code to fix a memory deallocation bug.
//      Previously Traverse() was being used to destroy a NSbtree
//      which led to problems since Traverse() operates on the 'data'
//      not the NSbnode.  Also for freeing memory it is desirable to
//      do 'post' traversal processing rather than 'in' traversal
//      processing since we don't want to access a node which has been
//      freed in order to retrieve the 'right' node.
//
//      Alternatively the 'right' node can be saved prior to removal
//      of the current node.
//********************************************************************
void NSbtree::Traverse2(NSbnode* r, ostream& o) const{
   if (r != 0) {
      Traverse2(r->left, o);
      Traverse2(r->right, o);
      (_traverse)(r, o);
      }
   }

