/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

/***********************************************************************
** Includes
***********************************************************************/
#include <assert.h>

#include "libnscore/nsfile.h"
#include "libnscore/nsfname.h"


//********************************************************************
// MEMBER:	NSfile
// DESCRIPTION:
//	Constructors
//********************************************************************
NSfile::NSfile()
:  _bytecnt(0), _filebuf(0), _isopen(FALSE), _pFile(0) {
   }

NSfile::NSfile(CStr name, NSfilemode filemode)
:  _bytecnt(0), _filebuf(new char[BUFSZ]), _isopen(FALSE), _pFile(0) {
   if (_filebuf == 0)
      Errnum(outOfMem);
   else
      Open(name, filemode);
   }

NSfile::NSfile(CStr name, CStr* paths, NSfilemode filemode)
:  _bytecnt(0), _filebuf(new char[BUFSZ]), _isopen(FALSE), _pFile(0) {
   if (_filebuf == 0)
      Errnum(outOfMem);
   else
      Open(name, paths, filemode);
   }


//********************************************************************
// MEMBER:	~NSfile
// DESCRIPTION:
//	Destructor
//********************************************************************
NSfile::~NSfile() {
   Close();
   delete [] _filebuf;
   }


//********************************************************************
// MEMBER:	Close
// DESCRIPTION:
//	@+@member description
//********************************************************************
NSfile& NSfile::Close() {
   if (_isopen) {
      if (EOF == fclose(_pFile))
         Errnum(fileCloseErr);
      _isopen = FALSE;
      _pFile = 0;
      }
   return *this;
   }


//********************************************************************
// MEMBER:	Open
// DESCRIPTION:
//	@+@member description
//********************************************************************
NSfile& NSfile::Open(CStr name, NSfilemode filemode) {
   Clear();
   Close();

   char* mode;
   switch (filemode)
   {
     case READONLY:
       mode = "rb";
       break;

     case UPDATE:
       mode = "rb+";
       break;

     case READWRITE:
       mode = "wb+";
       break;

     case APPEND:
       mode = "ab+";
       break;

     default:       // previously there was no default case so the code
       mode = "";   // would likely have core dumped in a weird place
       assert(0);   // here we get rid of warning and force a dump
       break;
   }

   if ((_pFile = fopen(name, mode)) == 0)
      Errnum(fileOpenErr);
   else {
      setvbuf(_pFile, _filebuf, _IOFBF, BUFSZ);
      _isopen = TRUE;
      SeekToBegin();  // should be open at the start, anyhow...
      }

   return(*this);
   }

NSfile& NSfile::Open(CStr fn, CStr *paths, NSfilemode fm) {
   Close();

   while (*paths) {
      NSfname full(fn, *paths);
      Clear();
      if (Open(full.fullname(), fm).Good()) 
         break;
      paths += 1;
      }

   return(*this);
   }


// protected member functions

//********************************************************************
// MEMBER:	Dump
// DESCRIPTION:
//	Debugging aid to dump the contents of the current object.
//********************************************************************
void NSfile::Dump(ostream& o) const {
   o << "  IsOpen = " << (_isopen ? "TRUE" : "FALSE")
     << ", bytes last transferred = " << _bytecnt << endl;
   }


//********************************************************************
// MEMBER:	Name
// DESCRIPTION:
//	Returns the class name for the instance.
//********************************************************************
CStr NSfile::Name() const {
   return("NSfile");
   }
