/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

/***********************************************************************
** Includes
***********************************************************************/
#include "libnscore/nsfname.h"


//********************************************************************
// MEMBER:	NSfname
// DESCRIPTION:
//	Constructor
//********************************************************************
NSfname::NSfname(CStr fname, CStr dir)
     : _fullname(0)
{
    Init(fname, dir);
}

NSfname::~NSfname() { 
   delete[] _fullname;
   }

NSfname::NSfname(CStr fname, CStr* paths)
     : _fullname(0)
{
    // This is UNIX specific...
    if ((*fname == '/') || (*fname == '.'))
    {
	_fullname = new char[strlen(fname)+1];
	if (!_fullname)
	    Errnum(outOfMem);
	else
	    strcpy(_fullname, fname);
    }
    else
    {
	s32 flen = strlen(fname) + 2;
	s32 tlen;

	for ( ; *paths ; paths++)
	{
	    tlen = flen + strlen(*paths);

	    delete[] _fullname;
	    _fullname = new char[tlen];
	    if (!_fullname)
	    {
		Errnum(outOfMem);
		return;
	    }
	    else
	    {
		sprintf(_fullname, "%s/%s", *paths, fname);
		if (exists()) return;
	    }
	}
	delete[] _fullname;
	_fullname = 0;
	Errnum(notFound);
    }
}


//********************************************************************
// MEMBER:	Init
// DESCRIPTION:
//	@+@member description
//********************************************************************
NSfname& NSfname::Init(CStr fname, CStr dir)
{
    Clear();
    if (!fname) {
	delete[] _fullname;
	_fullname = 0;
	return(*this);
    }
#ifdef _WINDOWS
    char *sep = "\\";
#else
    char *sep = "/";
#endif
    if (!dir) {
	dir = sep = "";		// don't write any dir info
    }

    s32 flen = strlen(fname) + strlen(sep) + strlen(dir) + 1;

    _fullname = new char[flen];
    if (!_fullname)
	Errnum(outOfMem);
    else
	sprintf(_fullname, "%s%s%s", dir, sep, fname);
    return(*this);
}


//********************************************************************
// MEMBER:	access
// DESCRIPTION:
//	@+@member description
//********************************************************************
NSbool  NSfname::access(AccessType how)
{
    int mode;

    // This is POSIX specific, and we do not support ORed flags
    switch (how)
    {
      case READ:
        #ifdef _WINDOWS
        mode = 4;
        #else
	mode = R_OK;
        #endif
	break;

      case WRITE:
        #ifdef _WINDOWS
        mode = 2;
        #else
	mode = W_OK;
        #endif
	break;

      case EXECUTE:
        #ifdef _WINDOWS
        mode = 1;
        #else
	mode = X_OK;
        #endif
	break;

      case EXISTS:
        #ifdef _WINDOWS
        mode = 0;
        #else
	mode = F_OK;
        #endif
	break;
    }

    return (::access(_fullname, mode) == 0) ? TRUE : FALSE;
}


void 
NSfname::Dump(ostream&) const {
   }

CStr 
NSfname::Name() const { 
   return "NSfname"; 
   }
