/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

/***********************************************************************
** Includes
***********************************************************************/
#if defined(AIX) || defined(SCO)
#include <sys/types.h>
#endif
#include "libnscore/nsmfile.h"


//********************************************************************
// MEMBER:	NSmapfile
// DESCRIPTION:
//	Constructors
//********************************************************************
NSmapfile::NSmapfile() 
:  _buffer(0), _ismapped(FALSE), _isopen(FALSE), _mode(READONLY), _size(0), 
   _fd(0), _mapmode(0), _sharemode(0) 
#ifdef NT
   , _fdMapObj(0), _mapObjectExists(FALSE)
#endif
{
   }

NSmapfile::NSmapfile(CStr name, NSmapfileMode mode) 
:  _buffer(0), _ismapped(FALSE), _isopen(FALSE), _mode(mode), _size(0), _fd(0) 
#ifdef NT
   , _fdMapObj(0), _mapObjectExists(FALSE)
#endif
{
   Open(name, mode);
   }


//********************************************************************
// MEMBER:	~NSmapfile
// DESCRIPTION:
//	Destructor
//********************************************************************
NSmapfile::~NSmapfile() {
   Close();
   }


//********************************************************************
// MEMBER:	Close
// DESCRIPTION:
//	@+@member description
//********************************************************************
NSmapfile& NSmapfile::Close() {
#ifndef NT
   if (_isopen) {
      close(_fd);
      _isopen = FALSE;
      }
   if (_ismapped) {
      munmap(_buffer, _size);
      _ismapped = FALSE;
      }
#else
   if (_ismapped) {
      UnmapViewOfFile(_buffer);
      _ismapped = FALSE;
      }
   if (_mapObjectExists) {
      CloseHandle(_fdMapObj);
      _mapObjectExists = FALSE;
      }
   if (_isopen) {
      CloseHandle(_fd);
      _isopen = FALSE;
      }
#endif
   return(*this);
   }


//********************************************************************
// MEMBER:	Open
// DESCRIPTION:
//	@+@member description
//********************************************************************
NSmapfile& NSmapfile::Open(CStr name, NSmapfileMode mode) {
   Clear();    
   if (_isopen) 
      Close();

#ifndef NT
   // NSstatbuf statbuf;
   struct stat statbuf;
   int filemode = 0;

   _mode = mode;
   switch (_mode) {
      case READONLY:
         filemode = O_RDONLY;
         _mapmode = PROT_READ | PROT_WRITE;
         _sharemode = MAP_PRIVATE;
         break;
      case READWRITE:
         filemode = O_RDWR | O_CREAT;
         _mapmode = PROT_READ | PROT_WRITE;
         _sharemode = MAP_SHARED;
         break;
      }

   if ((_fd = open(name, filemode)) < 0) 
      Errnum(fileOpenErr);
   else if (fstat(_fd, &statbuf) < 0)
      Errnum(fileOpenErr);
   else {
      _isopen = TRUE;
      if ((_size = (int) statbuf.st_size) == 0)
         Errnum(fileOpenErr);
      // else if ((_buffer = mmap(0, _size, _mapmode, _sharemode, _fd, 0)) == MAP_FAILED) 
      else if ((_buffer = (Str)mmap(0, _size, _mapmode, _sharemode, _fd, 0)) == (void *) -1) 
         Errnum(fileOpenErr);
      else 
         _ismapped = TRUE;
      }
#else
   DWORD filemode = 0;
   DWORD createFlags;

   _mode = mode;
   switch (_mode) {
      case READONLY:
         filemode = GENERIC_READ;
         createFlags = OPEN_EXISTING;
         _mapmode = PAGE_WRITECOPY;
         _sharemode = FILE_MAP_COPY;
         break;
      case READWRITE:
         filemode = GENERIC_READ | GENERIC_WRITE;
         createFlags = OPEN_ALWAYS;
         _mapmode = PAGE_READWRITE;
         _sharemode = FILE_MAP_WRITE;
         break;
      }
   if ((_fd = CreateFile(name, filemode, FILE_SHARE_READ, 0, createFlags,
      FILE_ATTRIBUTE_NORMAL, 0)) == INVALID_HANDLE_VALUE) { 
      Errnum(fileOpenErr);
      DWORD lastError = GetLastError();
      }
   else if ((_size = GetFileSize(_fd, 0)) < 0)
      Errnum(fileOpenErr);
   else {
      _isopen = TRUE;
      if (_size == 0)
         Errnum(fileOpenErr);
      else if ((_fdMapObj =
         CreateFileMapping(_fd, 0, _mapmode, 0, DWORD(_size), 0)) == 0)
         Errnum(fileOpenErr);
      else {
         _mapObjectExists = TRUE;
         if ((_buffer = (Str)MapViewOfFile(_fdMapObj, _sharemode, 0, 0,
            DWORD(_size)) ) == 0)
            Errnum(fileOpenErr);
         else 
            _ismapped = TRUE;
         }
      }
#endif
   return(*this);
   }


//********************************************************************
// MEMBER:	Reset
// DESCRIPTION:
//	@+@member description
//********************************************************************
NSmapfile& NSmapfile::Reset() {
   if (_ismapped) {
#ifndef NT
      munmap(_buffer, _size);
      _buffer = (Str)mmap(0, _size, _mapmode, _sharemode, _fd, 0);
#else
      UnmapViewOfFile(_buffer);
      if ((_fdMapObj =
         CreateFileMapping(_fd, 0, _mapmode, 0, DWORD(_size), 0)) == 0)
         Errnum(fileOpenErr);
      else if ((_buffer =
         (Str)MapViewOfFile(_fdMapObj, _sharemode, 0, 0, DWORD(_size)) ) == 0)
         Errnum(fileOpenErr);      
#endif
      }
   return(*this);
   }


//********************************************************************
// MEMBER:	strchr
// DESCRIPTION:
//	@+@member description
//********************************************************************
Str NSmapfile::strchr(CStr src, char target) {
   Str lastByte = LastByte();
   while (src < lastByte && *src != target)
      src++;
   return((*src == target) ? (Str)src : 0);
   }


//********************************************************************
// MEMBER:	strncmp
// DESCRIPTION:
//	@+@member description
//********************************************************************
int NSmapfile::strncmp(CStr src, CStr target, size_t targetlen) {
   Str    lastByte = LastByte()+1;
   size_t ncomp = nsMIN(lastByte-src, targetlen);
   int    retval;

   retval = ::strncmp(src, target, ncomp);
   if (retval != 0) 
      return(retval); // one of the two is not equal
   else if (ncomp < targetlen) {
      // equal if premature null in target, otherwise a < b
      return(target[ncomp] ? -1 : 0);
      }
   else 
      return(0);		// match
   }


//********************************************************************
// MEMBER:	strstr
// DESCRIPTION:
//	@+@member description
//********************************************************************
Str NSmapfile::strstr(CStr src, CStr target) {
   Str    lastByte = LastByte()+1;
   size_t targetLen = strlen(target);

   while ((src = NSmapfile::strchr(src, *target)) != 0) {
      if ((lastByte - src) < targetLen) 
         return(0);
      else if (strncmp(src, target, targetLen) == 0)
         break;
      else 
         src++;         
      }
   return((Str)src);
   }


// protected member functions

//********************************************************************
// MEMBER:	Dump
// DESCRIPTION:
//	Debugging aid to dump the contents of the current object.
//********************************************************************
void NSmapfile::Dump(ostream& o) const {
   o << "IsOpen=" << _isopen << ", IsMapped=" << _ismapped 
      << ", Mode=" << (int)_mode 
      << ", size=" << _size << ", fd=" << _fd << endl;
   }


//********************************************************************
// MEMBER:	Name
// DESCRIPTION:
//	Returns the class name for the instance.
//********************************************************************
CStr NSmapfile::Name() const {
   return("NSmapfile");
   }


