/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

/***********************************************************************
** Includes
***********************************************************************/
#include "libnscore/nscore.h"
#include "libnscore/nsmisc.h"


/***********************************************************************
** FUNCTION:	CGIescape
** DESCRIPTION:
**   Convert special characters in a string into valid CGI escape chars
***********************************************************************/
void CGIescape(Str arg, ostream& o) {
   char* curr = arg;
   char* prev = arg;
   while ((curr = strpbrk(curr, "&=\"!#$%^()_+=|'~?<>[]{};:' ")) != 0) {
      if (curr != prev) 
         o.write(prev, curr-prev);
      o << "%" << (0x0F & (*curr >> 4))["0123456789ABCDEF"]
           << (*curr & 0x0F)["0123456789ABCDEF"];
      prev = ++curr;
      }
   o <<  prev;
   }


/***********************************************************************
** FUNCTION:	CGIparse
** DESCRIPTION:
**   Parse a delimited CGI string (e.g. '=' and '&') into the AV pair
**   object
***********************************************************************/
NSbool  CGIparse(Str args, NSavpair& opts, char endOfValue, CGIparseCB cb) {
   size_t   argsLen = strlen(args);
   Str      tmpbuf = new char[argsLen+2];
   Str      attr = tmpbuf;

   if (attr == 0)
      return(FALSE);
   
   strcpy(attr, args);
   if (attr[argsLen-1] != endOfValue) {
      attr[argsLen] = endOfValue;
      attr[argsLen+1] = '\0';
      }

   char     delim = '=';
   Str      val = 0;
   Str      ptr = attr;
   NSbool   status = TRUE;

   while ((ptr = strchr(ptr, delim)) != 0) {
      *ptr = '\0';
      if (delim == '=') {
         delim = endOfValue;
         val = ++ptr;
         }
      else {
         delim = '=';
         if (cb) 
            status = (cb)(attr, val, opts);
         else
            status = opts.Put(attr, val, NSavpair::DUPES_OK).Good();
         if (status == FALSE)
            break;
         while (*(attr = ++ptr) == endOfValue)
            ;
         }
      }

   delete []tmpbuf;
   return(status);
   }


/***********************************************************************
** FUNCTION:	CGIunescape
** DESCRIPTION:
**   Convert escaped characters in a CGI string back into the valid
**   chars
***********************************************************************/
void CGIunescape(Str val) {
   Str p = val;
   Str c = val;
   while (*p != '\0') {
      if (*p == '+')
         *c++ = ' ';
      else if (*p == '%') {
         char tmp = (*++p >= 'A') ? 10+((*p & 0xDF)-'A') : (*p - '0');
         tmp <<= 4;
         tmp += (*++p >= 'A') ? 10+((*p & 0xDF)-'A') : (*p - '0');
         *c++ = tmp;
         }
      else
         *c++ = *p;
        
      p++;
      }

   *c = '\0';
   }


/***********************************************************************
** FUNCTION:	Hexdump
** DESCRIPTION:
**   Do a hexadecimal dump of a buffer
***********************************************************************/
void Hexdump(ostream& o, PGen buf, s32 len) {
   Str data = (Str)buf;
   for (u32 i=0; i<len; i++) {
      if ((i%32) == 0) o << endl << " ";  // newline every 32 bytes
      if ((i%8) == 0) o << " | ";         // spacer every 8 bytes
      o << (0x0F & (data[i] >> 4))["0123456789abcdef"];
      o << (data[i] & 0x0F)["0123456789abcdef"];
      }
   }


/***********************************************************************
** FUNCTION:	RTrim
** DESCRIPTION:
**   Remove trailing blanks from a string. This procedure overwrites the
**   string, so that it should not be called on a string you want to
**   keep whole.
***********************************************************************/
Str RTrim(Str str) {
   if (str) {
      Str p = str + strlen(str);
      if ((*--p == ' ') || (*p == '\t')) {
         while ((*--p == ' ') || (*p == '\t'))
            ;
         *++p = '\0';
         }
      }
   return(str);
   }


/***********************************************************************
** FUNCTION:	StrSame
** DESCRIPTION:
**   Compare two strings, case-insensitive
***********************************************************************/
NSbool  StrSame(Str str1, Str str2) {
   Str ptr1 = str1;
   Str ptr2 = str2;
   while (*ptr1 && *ptr2) {
      if (toupper(*ptr1++) != toupper(*ptr2++))
         return(FALSE);   
      }
   return(*ptr1 == *ptr2);
   }


/***********************************************************************
** FUNCTION:	Translate
** DESCRIPTION:
**   Translate one character to another in a string
***********************************************************************/
void Translate(Str src, char from, char to) {
   while (src) {
      src = strchr(src, from);
      if (src)
         *src++ = to;
      }
   }


