/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 *	Standard Header Files
 */
#include "ame/syspub.h"
#include "ame/odepub.h"
#include "ame/snmperrs.h"
#include "ode/classdef.h"
#include "ode/mgmtpub.h"
#include "ame/smitypes.h"
#include "ame/snmpvbl.h"

/*
 *	Run-Time Library constant data structures
 */
extern READONLY struct index_entry	IX_dotZero[];


/************************************************************************
 *
 *	httpEntityIndex - access methods for SNMP MIB variable
 *
 *	Description:
 *
 *	 the index of this row in the table 
 *
 ************************************************************************/

/*
 *	httpEntityIndex is known as 1.3.6.1.4.1.1450.1.1.1.1.1
 */
static READONLY ubyte	AIA_httpEntityIndex[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 0x01, 
	0x01, 0x01, 
};

static READONLY struct object_id	AI_httpEntityIndex =
{	sizeof(AIA_httpEntityIndex),
	AIA_httpEntityIndex,
};

/*ARGSUSED*/
static int
#ifdef USE_PROTOTYPES
GET_httpEntityIndex(void *ctxt, void **indices, void *attr_ref)
#else
GET_httpEntityIndex(ctxt, indices, attr_ref)
void			*ctxt;
void			**indices;
void			*attr_ref;
#endif
{
	INTEGER	*attr = (INTEGER *) attr_ref;

	*attr = ((struct httpEntity *)ctxt)->index;
	return(SNMP_ERR_NO_ERROR);
}

extern READONLY struct index_entry	IX_httpEntityEntry[];

/*
 *	The following structure binds the access methods for
 *	this attribute to its object identifier, type, range,
 *	and indexing informtaion.
 */
static READONLY struct attribute	A_httpEntityIndex =
{
	&AI_httpEntityIndex,
	GET_httpEntityIndex,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **))) NULL,
	(Void *(*)PEER_PROTO_ARG((Void *, Void**, int))) NULL,
	IX_httpEntityEntry,
	SNMP_NUMBER,		/* INTEGER */
	0x80000000,
	0x7fffffff,
};


/************************************************************************
 *
 *	httpEntityDescr - access methods for SNMP MIB variable
 *
 *	Description:
 *
 *	A general textual description of the entity.
 *					It includes information of system OS 
 *
 ************************************************************************/

/*
 *	httpEntityDescr is known as 1.3.6.1.4.1.1450.1.1.1.1.2
 */
static READONLY ubyte	AIA_httpEntityDescr[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 0x01, 
	0x01, 0x02, 
};

static READONLY struct object_id	AI_httpEntityDescr =
{	sizeof(AIA_httpEntityDescr),
	AIA_httpEntityDescr,
};

/*ARGSUSED*/
static int
#ifdef USE_PROTOTYPES
GET_httpEntityDescr(void *ctxt, void **indices, void *attr_ref)
#else
GET_httpEntityDescr(ctxt, indices, attr_ref)
void			*ctxt;
void			**indices;
void			*attr_ref;
#endif
{
	OCTETSTRING	*attr = (OCTETSTRING *) attr_ref;

	*attr = ((struct httpEntity *)ctxt)->descr;
	return(SNMP_ERR_NO_ERROR);
}


/*
 *	The following structure binds the access methods for
 *	this attribute to its object identifier, type, range,
 *	and indexing informtaion.
 */
static READONLY struct attribute	A_httpEntityDescr =
{
	&AI_httpEntityDescr,
	GET_httpEntityDescr,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **))) NULL,
	(Void *(*)PEER_PROTO_ARG((Void *, Void**, int))) NULL,
	IX_httpEntityEntry,
	SNMP_STRING,		/* OCTETSTRING */
#define MIN_httpEntityDescr	0
	MIN_httpEntityDescr,
#define MAX_httpEntityDescr	255
	MAX_httpEntityDescr,
};


/************************************************************************
 *
 *	httpEntityId - access methods for SNMP MIB variable
 *
 *	Description:
 *
 *	Enterprise subtree for Vendors. For example
 *					Netscape has 1.3.6.1.4.1.1450. we assign our
 *					http server 1.3.6.1.4.1.1450.1 
 *
 ************************************************************************/

/*
 *	httpEntityId is known as 1.3.6.1.4.1.1450.1.1.1.1.3
 */
static READONLY ubyte	AIA_httpEntityId[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 0x01, 
	0x01, 0x03, 
};

static READONLY struct object_id	AI_httpEntityId =
{	sizeof(AIA_httpEntityId),
	AIA_httpEntityId,
};

/*ARGSUSED*/
static int
#ifdef USE_PROTOTYPES
GET_httpEntityId(void *ctxt, void **indices, void *attr_ref)
#else
GET_httpEntityId(ctxt, indices, attr_ref)
void			*ctxt;
void			**indices;
void			*attr_ref;
#endif
{
	OBJECTIDENTIFIER	*attr = (OBJECTIDENTIFIER *) attr_ref;

	*attr = ((struct httpEntity *)ctxt)->svrobjid;
	return(SNMP_ERR_NO_ERROR);
}


/*
 *	The following structure binds the access methods for
 *	this attribute to its object identifier, type, range,
 *	and indexing informtaion.
 */
static READONLY struct attribute	A_httpEntityId =
{
	&AI_httpEntityId,
	GET_httpEntityId,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **))) NULL,
	(Void *(*)PEER_PROTO_ARG((Void *, Void**, int))) NULL,
	IX_httpEntityEntry,
	SNMP_OBJECT,		/* OBJECTIDENTIFIER */
	0x0,
	0x80,
};


/************************************************************************
 *
 *	httpEntityProtocol - access methods for SNMP MIB variable
 *
 *	Description:
 *
 *	http protocol number
 *
 ************************************************************************/

/*
 *	httpEntityProtocol is known as 1.3.6.1.4.1.1450.1.1.1.1.4
 */
static READONLY ubyte	AIA_httpEntityProtocol[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 0x01, 
	0x01, 0x04, 
};

static READONLY struct object_id	AI_httpEntityProtocol =
{	sizeof(AIA_httpEntityProtocol),
	AIA_httpEntityProtocol,
};

/*ARGSUSED*/
static int
#ifdef USE_PROTOTYPES
GET_httpEntityProtocol(void *ctxt, void **indices, void *attr_ref)
#else
GET_httpEntityProtocol(ctxt, indices, attr_ref)
void			*ctxt;
void			**indices;
void			*attr_ref;
#endif
{
	OCTETSTRING	*attr = (OCTETSTRING *) attr_ref;

	*attr = ((struct httpEntity *)ctxt)->protocol;
	return(SNMP_ERR_NO_ERROR);
}


/*
 *	The following structure binds the access methods for
 *	this attribute to its object identifier, type, range,
 *	and indexing informtaion.
 */
static READONLY struct attribute	A_httpEntityProtocol =
{
	&AI_httpEntityProtocol,
	GET_httpEntityProtocol,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **))) NULL,
	(Void *(*)PEER_PROTO_ARG((Void *, Void**, int))) NULL,
	IX_httpEntityEntry,
	SNMP_STRING,		/* OCTETSTRING */
	0x0,
	0x100,
};


/************************************************************************
 *
 *	httpEntityVersion - access methods for SNMP MIB variable
 *
 *	Description:
 *
 *	http entity implementation version
 *
 ************************************************************************/

/*
 *	httpEntityVersion is known as 1.3.6.1.4.1.1450.1.1.1.1.5
 */
static READONLY ubyte	AIA_httpEntityVersion[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 0x01, 
	0x01, 0x05, 
};

static READONLY struct object_id	AI_httpEntityVersion =
{	sizeof(AIA_httpEntityVersion),
	AIA_httpEntityVersion,
};

/*ARGSUSED*/
static int
#ifdef USE_PROTOTYPES
GET_httpEntityVersion(void *ctxt, void **indices, void *attr_ref)
#else
GET_httpEntityVersion(ctxt, indices, attr_ref)
void			*ctxt;
void			**indices;
void			*attr_ref;
#endif
{
	OCTETSTRING	*attr = (OCTETSTRING *) attr_ref;

	*attr = ((struct httpEntity *)ctxt)->vers;
	return(SNMP_ERR_NO_ERROR);
}


/*
 *	The following structure binds the access methods for
 *	this attribute to its object identifier, type, range,
 *	and indexing informtaion.
 */
static READONLY struct attribute	A_httpEntityVersion =
{
	&AI_httpEntityVersion,
	GET_httpEntityVersion,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **))) NULL,
	(Void *(*)PEER_PROTO_ARG((Void *, Void**, int))) NULL,
	IX_httpEntityEntry,
	SNMP_STRING,		/* OCTETSTRING */
	0x0,
	0x100,
};


/************************************************************************
 *
 *	httpEntityOrganization - access methods for SNMP MIB variable
 *
 *	Description:
 *
 *	Organization responsible for this entity
 *
 ************************************************************************/

/*
 *	httpEntityOrganization is known as 1.3.6.1.4.1.1450.1.1.1.1.6
 */
static READONLY ubyte	AIA_httpEntityOrganization[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 0x01, 
	0x01, 0x06, 
};

static READONLY struct object_id	AI_httpEntityOrganization =
{	sizeof(AIA_httpEntityOrganization),
	AIA_httpEntityOrganization,
};

/*ARGSUSED*/
static int
#ifdef USE_PROTOTYPES
GET_httpEntityOrganization(void *ctxt, void **indices, void *attr_ref)
#else
GET_httpEntityOrganization(ctxt, indices, attr_ref)
void			*ctxt;
void			**indices;
void			*attr_ref;
#endif
{
	OCTETSTRING	*attr = (OCTETSTRING *) attr_ref;

	*attr = ((struct httpEntity *)ctxt)->org;
	return(SNMP_ERR_NO_ERROR);
}


/*
 *	The following structure binds the access methods for
 *	this attribute to its object identifier, type, range,
 *	and indexing informtaion.
 */
static READONLY struct attribute	A_httpEntityOrganization =
{
	&AI_httpEntityOrganization,
	GET_httpEntityOrganization,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **))) NULL,
	(Void *(*)PEER_PROTO_ARG((Void *, Void**, int))) NULL,
	IX_httpEntityEntry,
	SNMP_STRING,		/* OCTETSTRING */
#define MIN_httpEntityOrganization	0
	MIN_httpEntityOrganization,
#define MAX_httpEntityOrganization	255
	MAX_httpEntityOrganization,
};


/************************************************************************
 *
 *	httpEntityLocation - access methods for SNMP MIB variable
 *
 *	Description:
 *
 *	Phsical location of this entity. For example,
 *					 it can be a fullpath name to a server
 *
 ************************************************************************/

/*
 *	httpEntityLocation is known as 1.3.6.1.4.1.1450.1.1.1.1.7
 */
static READONLY ubyte	AIA_httpEntityLocation[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 0x01, 
	0x01, 0x07, 
};

static READONLY struct object_id	AI_httpEntityLocation =
{	sizeof(AIA_httpEntityLocation),
	AIA_httpEntityLocation,
};

/*ARGSUSED*/
static int
#ifdef USE_PROTOTYPES
GET_httpEntityLocation(void *ctxt, void **indices, void *attr_ref)
#else
GET_httpEntityLocation(ctxt, indices, attr_ref)
void			*ctxt;
void			**indices;
void			*attr_ref;
#endif
{
	OCTETSTRING	*attr = (OCTETSTRING *) attr_ref;

	*attr = ((struct httpEntity *)ctxt)->loc;
	return(SNMP_ERR_NO_ERROR);
}


/*
 *	The following structure binds the access methods for
 *	this attribute to its object identifier, type, range,
 *	and indexing informtaion.
 */
static READONLY struct attribute	A_httpEntityLocation =
{
	&AI_httpEntityLocation,
	GET_httpEntityLocation,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **))) NULL,
	(Void *(*)PEER_PROTO_ARG((Void *, Void**, int))) NULL,
	IX_httpEntityEntry,
	SNMP_STRING,		/* OCTETSTRING */
#define MIN_httpEntityLocation	0
	MIN_httpEntityLocation,
#define MAX_httpEntityLocation	255
	MAX_httpEntityLocation,
};


/************************************************************************
 *
 *	httpEntityContact - access methods for SNMP MIB variable
 *
 *	Description:
 *
 *	contact persons responsible for the entity.
 *					together with information on how to contact
 *
 ************************************************************************/

/*
 *	httpEntityContact is known as 1.3.6.1.4.1.1450.1.1.1.1.8
 */
static READONLY ubyte	AIA_httpEntityContact[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 0x01, 
	0x01, 0x08, 
};

static READONLY struct object_id	AI_httpEntityContact =
{	sizeof(AIA_httpEntityContact),
	AIA_httpEntityContact,
};

/*ARGSUSED*/
static int
#ifdef USE_PROTOTYPES
GET_httpEntityContact(void *ctxt, void **indices, void *attr_ref)
#else
GET_httpEntityContact(ctxt, indices, attr_ref)
void			*ctxt;
void			**indices;
void			*attr_ref;
#endif
{
	OCTETSTRING	*attr = (OCTETSTRING *) attr_ref;

	*attr = ((struct httpEntity *)ctxt)->contact;
	return(SNMP_ERR_NO_ERROR);
}


/*
 *	The following structure binds the access methods for
 *	this attribute to its object identifier, type, range,
 *	and indexing informtaion.
 */
static READONLY struct attribute	A_httpEntityContact =
{
	&AI_httpEntityContact,
	GET_httpEntityContact,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **))) NULL,
	(Void *(*)PEER_PROTO_ARG((Void *, Void**, int))) NULL,
	IX_httpEntityEntry,
	SNMP_STRING,		/* OCTETSTRING */
#define MIN_httpEntityContact	0
	MIN_httpEntityContact,
#define MAX_httpEntityContact	255
	MAX_httpEntityContact,
};


/************************************************************************
 *
 *	httpEntityAddress - access methods for SNMP MIB variable
 *
 *	Description:
 *
 *	It is IP address of a machine  on which 
 *					the entity is running
 *
 ************************************************************************/

/*
 *	httpEntityAddress is known as 1.3.6.1.4.1.1450.1.1.1.1.9
 */
static READONLY ubyte	AIA_httpEntityAddress[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 0x01, 
	0x01, 0x09, 
};

static READONLY struct object_id	AI_httpEntityAddress =
{	sizeof(AIA_httpEntityAddress),
	AIA_httpEntityAddress,
};

/*ARGSUSED*/
static int
#ifdef USE_PROTOTYPES
GET_httpEntityAddress(void *ctxt, void **indices, void *attr_ref)
#else
GET_httpEntityAddress(ctxt, indices, attr_ref)
void			*ctxt;
void			**indices;
void			*attr_ref;
#endif
{
	IpAddress	*attr = (IpAddress *) attr_ref;

	*attr = ((struct httpEntity *)ctxt)->addr;
	return(SNMP_ERR_NO_ERROR);
}


/*
 *	The following structure binds the access methods for
 *	this attribute to its object identifier, type, range,
 *	and indexing informtaion.
 */
static READONLY struct attribute	A_httpEntityAddress =
{
	&AI_httpEntityAddress,
	GET_httpEntityAddress,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **))) NULL,
	(Void *(*)PEER_PROTO_ARG((Void *, Void**, int))) NULL,
	IX_httpEntityEntry,
	SNMP_ADDR,		/* IpAddress */
	0x0,
	0xffffffff,
};


/************************************************************************
 *
 *	httpEntityPort - access methods for SNMP MIB variable
 *
 *	Description:
 *
 *	port number which the entity is listening on
 *
 ************************************************************************/

/*
 *	httpEntityPort is known as 1.3.6.1.4.1.1450.1.1.1.1.10
 */
static READONLY ubyte	AIA_httpEntityPort[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 0x01, 
	0x01, 0x0a, 
};

static READONLY struct object_id	AI_httpEntityPort =
{	sizeof(AIA_httpEntityPort),
	AIA_httpEntityPort,
};

/*ARGSUSED*/
static int
#ifdef USE_PROTOTYPES
GET_httpEntityPort(void *ctxt, void **indices, void *attr_ref)
#else
GET_httpEntityPort(ctxt, indices, attr_ref)
void			*ctxt;
void			**indices;
void			*attr_ref;
#endif
{
	OCTETSTRING	*attr = (OCTETSTRING *) attr_ref;

	*attr = ((struct httpEntity *)ctxt)->port;
	return(SNMP_ERR_NO_ERROR);
}


/*
 *	The following structure binds the access methods for
 *	this attribute to its object identifier, type, range,
 *	and indexing informtaion.
 */
static READONLY struct attribute	A_httpEntityPort =
{
	&AI_httpEntityPort,
	GET_httpEntityPort,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **))) NULL,
	(Void *(*)PEER_PROTO_ARG((Void *, Void**, int))) NULL,
	IX_httpEntityEntry,
	SNMP_STRING,		/* OCTETSTRING */
	0x0,
	0x100,
};


/************************************************************************
 *
 *	httpEntityName - access methods for SNMP MIB variable
 *
 *	Description:
 *
 *	The HTTP entity name, such as HTTP server
 *					offa.mcom.com
 *
 ************************************************************************/

/*
 *	httpEntityName is known as 1.3.6.1.4.1.1450.1.1.1.1.11
 */
static READONLY ubyte	AIA_httpEntityName[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 0x01, 
	0x01, 0x0b, 
};

static READONLY struct object_id	AI_httpEntityName =
{	sizeof(AIA_httpEntityName),
	AIA_httpEntityName,
};

/*ARGSUSED*/
static int
#ifdef USE_PROTOTYPES
GET_httpEntityName(void *ctxt, void **indices, void *attr_ref)
#else
GET_httpEntityName(ctxt, indices, attr_ref)
void			*ctxt;
void			**indices;
void			*attr_ref;
#endif
{
	OCTETSTRING	*attr = (OCTETSTRING *) attr_ref;

	*attr = ((struct httpEntity *)ctxt)->name;
	return(SNMP_ERR_NO_ERROR);
}


/*
 *	The following structure binds the access methods for
 *	this attribute to its object identifier, type, range,
 *	and indexing informtaion.
 */
static READONLY struct attribute	A_httpEntityName =
{
	&AI_httpEntityName,
	GET_httpEntityName,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **))) NULL,
	(Void *(*)PEER_PROTO_ARG((Void *, Void**, int))) NULL,
	IX_httpEntityEntry,
	SNMP_STRING,		/* OCTETSTRING */
	0x0,
	0x100,
};


/************************************************************************
 *
 *	httpEntityType - access methods for SNMP MIB variable
 *
 *	Description:
 *
 *	the type of entity. For example, it can be 
 *					commerce server.
 *					0 - commerce
 *					1 - communication 
 *
 ************************************************************************/

/*
 *	httpEntityType is known as 1.3.6.1.4.1.1450.1.1.1.1.12
 */
static READONLY ubyte	AIA_httpEntityType[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 0x01, 
	0x01, 0x0c, 
};

static READONLY struct object_id	AI_httpEntityType =
{	sizeof(AIA_httpEntityType),
	AIA_httpEntityType,
};

/*ARGSUSED*/
static int
#ifdef USE_PROTOTYPES
GET_httpEntityType(void *ctxt, void **indices, void *attr_ref)
#else
GET_httpEntityType(ctxt, indices, attr_ref)
void			*ctxt;
void			**indices;
void			*attr_ref;
#endif
{
	OCTETSTRING	*attr = (OCTETSTRING *) attr_ref;

	*attr = ((struct httpEntity *)ctxt)->svrtype;
	return(SNMP_ERR_NO_ERROR);
}


/*
 *	The following structure binds the access methods for
 *	this attribute to its object identifier, type, range,
 *	and indexing informtaion.
 */
static READONLY struct attribute	A_httpEntityType =
{
	&AI_httpEntityType,
	GET_httpEntityType,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **))) NULL,
	(Void *(*)PEER_PROTO_ARG((Void *, Void**, int))) NULL,
	IX_httpEntityEntry,
	SNMP_STRING,		/* OCTETSTRING */
	0x0,
	0x100,
};


/************************************************************************
 *
 *	httpEntityMethods - access methods for SNMP MIB variable
 *
 *	Description:
 *
 *	which methods are supported by the entity.
 *					Currently, for example, GET, POST
 *
 ************************************************************************/

/*
 *	httpEntityMethods is known as 1.3.6.1.4.1.1450.1.1.1.1.13
 */
static READONLY ubyte	AIA_httpEntityMethods[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 0x01, 
	0x01, 0x0d, 
};

static READONLY struct object_id	AI_httpEntityMethods =
{	sizeof(AIA_httpEntityMethods),
	AIA_httpEntityMethods,
};

/*ARGSUSED*/
static int
#ifdef USE_PROTOTYPES
GET_httpEntityMethods(void *ctxt, void **indices, void *attr_ref)
#else
GET_httpEntityMethods(ctxt, indices, attr_ref)
void			*ctxt;
void			**indices;
void			*attr_ref;
#endif
{
	OCTETSTRING	*attr = (OCTETSTRING *) attr_ref;

	*attr = ((struct httpEntity *)ctxt)->methods;
	return(SNMP_ERR_NO_ERROR);
}


/*
 *	The following structure binds the access methods for
 *	this attribute to its object identifier, type, range,
 *	and indexing informtaion.
 */
static READONLY struct attribute	A_httpEntityMethods =
{
	&AI_httpEntityMethods,
	GET_httpEntityMethods,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **))) NULL,
	(Void *(*)PEER_PROTO_ARG((Void *, Void**, int))) NULL,
	IX_httpEntityEntry,
	SNMP_STRING,		/* OCTETSTRING */
	0x0,
	0x100,
};


/************************************************************************
 *
 *	httpEntityMaxProcess - access methods for SNMP MIB variable
 *
 *	Description:
 *
 *	 maximum number of processes a http entity
 *					can have active 
 *
 ************************************************************************/

/*
 *	httpEntityMaxProcess is known as 1.3.6.1.4.1.1450.1.1.1.1.14
 */
static READONLY ubyte	AIA_httpEntityMaxProcess[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 0x01, 
	0x01, 0x0e, 
};

static READONLY struct object_id	AI_httpEntityMaxProcess =
{	sizeof(AIA_httpEntityMaxProcess),
	AIA_httpEntityMaxProcess,
};

/*ARGSUSED*/
static int
#ifdef USE_PROTOTYPES
GET_httpEntityMaxProcess(void *ctxt, void **indices, void *attr_ref)
#else
GET_httpEntityMaxProcess(ctxt, indices, attr_ref)
void			*ctxt;
void			**indices;
void			*attr_ref;
#endif
{
	INTEGER	*attr = (INTEGER *) attr_ref;

	*attr = ((struct httpEntity *)ctxt)->maxprocess;
	return(SNMP_ERR_NO_ERROR);
}


/*
 *	The following structure binds the access methods for
 *	this attribute to its object identifier, type, range,
 *	and indexing informtaion.
 */
static READONLY struct attribute	A_httpEntityMaxProcess =
{
	&AI_httpEntityMaxProcess,
	GET_httpEntityMaxProcess,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **))) NULL,
	(Void *(*)PEER_PROTO_ARG((Void *, Void**, int))) NULL,
	IX_httpEntityEntry,
	SNMP_NUMBER,		/* INTEGER */
	0x80000000,
	0x7fffffff,
};


/************************************************************************
 *
 *	httpEntityMinProcess - access methods for SNMP MIB variable
 *
 *	Description:
 *
 *	 minimum number of processes a http entity
 *					can have active 
 *
 ************************************************************************/

/*
 *	httpEntityMinProcess is known as 1.3.6.1.4.1.1450.1.1.1.1.15
 */
static READONLY ubyte	AIA_httpEntityMinProcess[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 0x01, 
	0x01, 0x0f, 
};

static READONLY struct object_id	AI_httpEntityMinProcess =
{	sizeof(AIA_httpEntityMinProcess),
	AIA_httpEntityMinProcess,
};

/*ARGSUSED*/
static int
#ifdef USE_PROTOTYPES
GET_httpEntityMinProcess(void *ctxt, void **indices, void *attr_ref)
#else
GET_httpEntityMinProcess(ctxt, indices, attr_ref)
void			*ctxt;
void			**indices;
void			*attr_ref;
#endif
{
	INTEGER	*attr = (INTEGER *) attr_ref;

	*attr = ((struct httpEntity *)ctxt)->minprocess;
	return(SNMP_ERR_NO_ERROR);
}


/*
 *	The following structure binds the access methods for
 *	this attribute to its object identifier, type, range,
 *	and indexing informtaion.
 */
static READONLY struct attribute	A_httpEntityMinProcess =
{
	&AI_httpEntityMinProcess,
	GET_httpEntityMinProcess,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **))) NULL,
	(Void *(*)PEER_PROTO_ARG((Void *, Void**, int))) NULL,
	IX_httpEntityEntry,
	SNMP_NUMBER,		/* INTEGER */
	0x80000000,
	0x7fffffff,
};


/************************************************************************
 *
 *	httpEntityMaxThread - access methods for SNMP MIB variable
 *
 *	Description:
 *
 *	 maximum number of processes a http entity
 *					can have active 
 *
 ************************************************************************/

/*
 *	httpEntityMaxThread is known as 1.3.6.1.4.1.1450.1.1.1.1.16
 */
static READONLY ubyte	AIA_httpEntityMaxThread[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 0x01, 
	0x01, 0x10, 
};

static READONLY struct object_id	AI_httpEntityMaxThread =
{	sizeof(AIA_httpEntityMaxThread),
	AIA_httpEntityMaxThread,
};

/*ARGSUSED*/
static int
#ifdef USE_PROTOTYPES
GET_httpEntityMaxThread(void *ctxt, void **indices, void *attr_ref)
#else
GET_httpEntityMaxThread(ctxt, indices, attr_ref)
void			*ctxt;
void			**indices;
void			*attr_ref;
#endif
{
	INTEGER	*attr = (INTEGER *) attr_ref;

	*attr = ((struct httpEntity *)ctxt)->maxthread;
	return(SNMP_ERR_NO_ERROR);
}


/*
 *	The following structure binds the access methods for
 *	this attribute to its object identifier, type, range,
 *	and indexing informtaion.
 */
static READONLY struct attribute	A_httpEntityMaxThread =
{
	&AI_httpEntityMaxThread,
	GET_httpEntityMaxThread,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **))) NULL,
	(Void *(*)PEER_PROTO_ARG((Void *, Void**, int))) NULL,
	IX_httpEntityEntry,
	SNMP_NUMBER,		/* INTEGER */
	0x80000000,
	0x7fffffff,
};


/************************************************************************
 *
 *	httpEntityMinThread - access methods for SNMP MIB variable
 *
 *	Description:
 *
 *	 minimum number of processes a http entity
 *					can have active 
 *
 ************************************************************************/

/*
 *	httpEntityMinThread is known as 1.3.6.1.4.1.1450.1.1.1.1.17
 */
static READONLY ubyte	AIA_httpEntityMinThread[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 0x01, 
	0x01, 0x11, 
};

static READONLY struct object_id	AI_httpEntityMinThread =
{	sizeof(AIA_httpEntityMinThread),
	AIA_httpEntityMinThread,
};

/*ARGSUSED*/
static int
#ifdef USE_PROTOTYPES
GET_httpEntityMinThread(void *ctxt, void **indices, void *attr_ref)
#else
GET_httpEntityMinThread(ctxt, indices, attr_ref)
void			*ctxt;
void			**indices;
void			*attr_ref;
#endif
{
	INTEGER	*attr = (INTEGER *) attr_ref;

	*attr = ((struct httpEntity *)ctxt)->minthread;
	return(SNMP_ERR_NO_ERROR);
}


/*
 *	The following structure binds the access methods for
 *	this attribute to its object identifier, type, range,
 *	and indexing informtaion.
 */
static READONLY struct attribute	A_httpEntityMinThread =
{
	&AI_httpEntityMinThread,
	GET_httpEntityMinThread,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **))) NULL,
	(Void *(*)PEER_PROTO_ARG((Void *, Void**, int))) NULL,
	IX_httpEntityEntry,
	SNMP_NUMBER,		/* INTEGER */
	0x80000000,
	0x7fffffff,
};

READONLY struct attribute *AS_httpEntityEntry[] =
{
	&A_httpEntityIndex,
	&A_httpEntityDescr,
	&A_httpEntityId,
	&A_httpEntityProtocol,
	&A_httpEntityVersion,
	&A_httpEntityOrganization,
	&A_httpEntityLocation,
	&A_httpEntityContact,
	&A_httpEntityAddress,
	&A_httpEntityPort,
	&A_httpEntityName,
	&A_httpEntityType,
	&A_httpEntityMethods,
	&A_httpEntityMaxProcess,
	&A_httpEntityMinProcess,
	&A_httpEntityMaxThread,
	&A_httpEntityMinThread,
	(struct attribute READONLY *) NULL
};

/*static*/ READONLY struct index_entry	IX_httpEntityEntry[] =
{
	{ 0 + 0,  &A_httpEntityIndex	},
	{ 0,  (struct attribute READONLY *) NULL}
};
static READONLY ubyte	GIA_httpEntityEntry[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 0x01, 
	0x01, 
};

static READONLY struct object_id	GI_httpEntityEntry =
{
	11,
	GIA_httpEntityEntry
};

#ifdef __cplusplus
extern READONLY struct class_definition	SMI_GROUP_httpEntityEntry;
#endif
READONLY struct class_definition	SMI_GROUP_httpEntityEntry =
{
	SNMP_CLASS,
	&GI_httpEntityEntry,
	AS_httpEntityEntry,
	(struct group_attribute READONLY **) NULL,
	(struct action READONLY **) NULL,
	(struct notification READONLY **) NULL,
	(int (*) PEER_PROTO_ARG((void))) NULL,
	(struct class_definition READONLY **) NULL,
	IX_httpEntityEntry,
	(struct contained_obj READONLY **) NULL,
	(int (*)PEER_PROTO_ARG((void))) NULL,
	(void (*)PEER_PROTO_ARG((void))) NULL,
};

static READONLY struct contained_obj	CG_httpEntityEntry = 
{
	&SMI_GROUP_httpEntityEntry,
	NULL
};
static READONLY struct contained_obj *CGS_httpEntityTable[] =
{
	&CG_httpEntityEntry,
	(struct contained_obj READONLY *) NULL
};
#define AS_httpEntityTable (struct attribute READONLY **) NULL
static READONLY ubyte	GIA_httpEntityTable[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 0x01, 
};

static READONLY struct object_id	GI_httpEntityTable =
{
	10,
	GIA_httpEntityTable
};

#ifdef __cplusplus
extern READONLY struct class_definition	SMI_GROUP_httpEntityTable;
#endif
READONLY struct class_definition	SMI_GROUP_httpEntityTable =
{
	SNMP_CLASS,
	&GI_httpEntityTable,
	AS_httpEntityTable,
	(struct group_attribute READONLY **) NULL,
	(struct action READONLY **) NULL,
	(struct notification READONLY **) NULL,
	(int (*) PEER_PROTO_ARG((void))) NULL,
	(struct class_definition READONLY **) NULL,
	(struct index_entry READONLY *) NULL,
	CGS_httpEntityTable,
	(int (*)PEER_PROTO_ARG((void))) NULL,
	(void (*)PEER_PROTO_ARG((void))) NULL,
};

static READONLY struct contained_obj	CG_httpEntityTable = 
{
	&SMI_GROUP_httpEntityTable,
	NULL
};

/************************************************************************
 *
 *	httpStatisticsIndex - access methods for SNMP MIB variable
 *
 *	Description:
 *
 *	 the index of this row in the table 
 *
 ************************************************************************/

/*
 *	httpStatisticsIndex is known as 1.3.6.1.4.1.1450.1.1.2.1.1
 */
static READONLY ubyte	AIA_httpStatisticsIndex[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 0x02, 
	0x01, 0x01, 
};

static READONLY struct object_id	AI_httpStatisticsIndex =
{	sizeof(AIA_httpStatisticsIndex),
	AIA_httpStatisticsIndex,
};

/*ARGSUSED*/
static int
#ifdef USE_PROTOTYPES
GET_httpStatisticsIndex(void *ctxt, void **indices, void *attr_ref)
#else
GET_httpStatisticsIndex(ctxt, indices, attr_ref)
void			*ctxt;
void			**indices;
void			*attr_ref;
#endif
{
	INTEGER	*attr = (INTEGER *) attr_ref;

	*attr = ((struct httpStatistics *)ctxt)->index;
	return(SNMP_ERR_NO_ERROR);
}

extern READONLY struct index_entry	IX_httpStatisticsEntry[];

/*
 *	The following structure binds the access methods for
 *	this attribute to its object identifier, type, range,
 *	and indexing informtaion.
 */
static READONLY struct attribute	A_httpStatisticsIndex =
{
	&AI_httpStatisticsIndex,
	GET_httpStatisticsIndex,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **))) NULL,
	(Void *(*)PEER_PROTO_ARG((Void *, Void**, int))) NULL,
	IX_httpStatisticsEntry,
	SNMP_NUMBER,		/* INTEGER */
	0x80000000,
	0x7fffffff,
};


/************************************************************************
 *
 *	httpStatisticsStatus - access methods for SNMP MIB variable
 *
 *	Description:
 *
 *	 status of http entity.
 *						0 -- up
 *						1 -- down
 *					
 *
 ************************************************************************/

/*
 *	httpStatisticsStatus is known as 1.3.6.1.4.1.1450.1.1.2.1.2
 */
static READONLY ubyte	AIA_httpStatisticsStatus[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 0x02, 
	0x01, 0x02, 
};

static READONLY struct object_id	AI_httpStatisticsStatus =
{	sizeof(AIA_httpStatisticsStatus),
	AIA_httpStatisticsStatus,
};

/*ARGSUSED*/
static int
#ifdef USE_PROTOTYPES
GET_httpStatisticsStatus(void *ctxt, void **indices, void *attr_ref)
#else
GET_httpStatisticsStatus(ctxt, indices, attr_ref)
void			*ctxt;
void			**indices;
void			*attr_ref;
#endif
{
	INTEGER	*attr = (INTEGER *) attr_ref;

	*attr = ((struct httpStatistics *)ctxt)->status;
	return(SNMP_ERR_NO_ERROR);
}


/*
 *	The following structure binds the access methods for
 *	this attribute to its object identifier, type, range,
 *	and indexing informtaion.
 */
static READONLY struct attribute	A_httpStatisticsStatus =
{
	&AI_httpStatisticsStatus,
	GET_httpStatisticsStatus,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **))) NULL,
	(Void *(*)PEER_PROTO_ARG((Void *, Void**, int))) NULL,
	IX_httpStatisticsEntry,
	SNMP_NUMBER,		/* INTEGER */
	0x80000000,
	0x7fffffff,
};


/************************************************************************
 *
 *	httpStatisticsUptime - access methods for SNMP MIB variable
 *
 *	Description:
 *
 *	up time since the entity is started
 *
 ************************************************************************/

/*
 *	httpStatisticsUptime is known as 1.3.6.1.4.1.1450.1.1.2.1.3
 */
static READONLY ubyte	AIA_httpStatisticsUptime[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 0x02, 
	0x01, 0x03, 
};

static READONLY struct object_id	AI_httpStatisticsUptime =
{	sizeof(AIA_httpStatisticsUptime),
	AIA_httpStatisticsUptime,
};

/*ARGSUSED*/
static int
#ifdef USE_PROTOTYPES
GET_httpStatisticsUptime(void *ctxt, void **indices, void *attr_ref)
#else
GET_httpStatisticsUptime(ctxt, indices, attr_ref)
void			*ctxt;
void			**indices;
void			*attr_ref;
#endif
{
	TimeTicks	*attr = (TimeTicks *) attr_ref;

	*attr = ((struct httpStatistics *)ctxt)->uptime;
	return(SNMP_ERR_NO_ERROR);
}


/*
 *	The following structure binds the access methods for
 *	this attribute to its object identifier, type, range,
 *	and indexing informtaion.
 */
static READONLY struct attribute	A_httpStatisticsUptime =
{
	&AI_httpStatisticsUptime,
	GET_httpStatisticsUptime,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **))) NULL,
	(Void *(*)PEER_PROTO_ARG((Void *, Void**, int))) NULL,
	IX_httpStatisticsEntry,
	SNMP_TICKS,		/* TimeTicks */
	0x0,
	0xffffffff,
};


/************************************************************************
 *
 *	httpStatisticsNumProcessIdle - access methods for SNMP MIB variable
 *
 *	Description:
 *
 *	the number of idle  process
 *
 ************************************************************************/

/*
 *	httpStatisticsNumProcessIdle is known as 1.3.6.1.4.1.1450.1.1.2.1.4
 */
static READONLY ubyte	AIA_httpStatisticsNumProcessIdle[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 0x02, 
	0x01, 0x04, 
};

static READONLY struct object_id	AI_httpStatisticsNumProcessIdle =
{	sizeof(AIA_httpStatisticsNumProcessIdle),
	AIA_httpStatisticsNumProcessIdle,
};

/*ARGSUSED*/
static int
#ifdef USE_PROTOTYPES
GET_httpStatisticsNumProcessIdle(void *ctxt, void **indices, void *attr_ref)
#else
GET_httpStatisticsNumProcessIdle(ctxt, indices, attr_ref)
void			*ctxt;
void			**indices;
void			*attr_ref;
#endif
{
	INTEGER	*attr = (INTEGER *) attr_ref;

	*attr = ((struct httpStatistics *)ctxt)->numpidle;
	return(SNMP_ERR_NO_ERROR);
}


/*
 *	The following structure binds the access methods for
 *	this attribute to its object identifier, type, range,
 *	and indexing informtaion.
 */
static READONLY struct attribute	A_httpStatisticsNumProcessIdle =
{
	&AI_httpStatisticsNumProcessIdle,
	GET_httpStatisticsNumProcessIdle,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **))) NULL,
	(Void *(*)PEER_PROTO_ARG((Void *, Void**, int))) NULL,
	IX_httpStatisticsEntry,
	SNMP_NUMBER,		/* INTEGER */
	0x80000000,
	0x7fffffff,
};


/************************************************************************
 *
 *	httpStatisticsNumProcessProc - access methods for SNMP MIB variable
 *
 *	Description:
 *
 *	the number of  process processing requests
 *
 ************************************************************************/

/*
 *	httpStatisticsNumProcessProc is known as 1.3.6.1.4.1.1450.1.1.2.1.5
 */
static READONLY ubyte	AIA_httpStatisticsNumProcessProc[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 0x02, 
	0x01, 0x05, 
};

static READONLY struct object_id	AI_httpStatisticsNumProcessProc =
{	sizeof(AIA_httpStatisticsNumProcessProc),
	AIA_httpStatisticsNumProcessProc,
};

/*ARGSUSED*/
static int
#ifdef USE_PROTOTYPES
GET_httpStatisticsNumProcessProc(void *ctxt, void **indices, void *attr_ref)
#else
GET_httpStatisticsNumProcessProc(ctxt, indices, attr_ref)
void			*ctxt;
void			**indices;
void			*attr_ref;
#endif
{
	INTEGER	*attr = (INTEGER *) attr_ref;

	*attr = ((struct httpStatistics *)ctxt)->numpproc;
	return(SNMP_ERR_NO_ERROR);
}


/*
 *	The following structure binds the access methods for
 *	this attribute to its object identifier, type, range,
 *	and indexing informtaion.
 */
static READONLY struct attribute	A_httpStatisticsNumProcessProc =
{
	&AI_httpStatisticsNumProcessProc,
	GET_httpStatisticsNumProcessProc,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **))) NULL,
	(Void *(*)PEER_PROTO_ARG((Void *, Void**, int))) NULL,
	IX_httpStatisticsEntry,
	SNMP_NUMBER,		/* INTEGER */
	0x80000000,
	0x7fffffff,
};


/************************************************************************
 *
 *	httpStatisticsNumProcessDns - access methods for SNMP MIB variable
 *
 *	Description:
 *
 *	the number of process doing DNS
 *
 ************************************************************************/

/*
 *	httpStatisticsNumProcessDns is known as 1.3.6.1.4.1.1450.1.1.2.1.6
 */
static READONLY ubyte	AIA_httpStatisticsNumProcessDns[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 0x02, 
	0x01, 0x06, 
};

static READONLY struct object_id	AI_httpStatisticsNumProcessDns =
{	sizeof(AIA_httpStatisticsNumProcessDns),
	AIA_httpStatisticsNumProcessDns,
};

/*ARGSUSED*/
static int
#ifdef USE_PROTOTYPES
GET_httpStatisticsNumProcessDns(void *ctxt, void **indices, void *attr_ref)
#else
GET_httpStatisticsNumProcessDns(ctxt, indices, attr_ref)
void			*ctxt;
void			**indices;
void			*attr_ref;
#endif
{
	INTEGER	*attr = (INTEGER *) attr_ref;

	*attr = ((struct httpStatistics *)ctxt)->numpdns;
	return(SNMP_ERR_NO_ERROR);
}


/*
 *	The following structure binds the access methods for
 *	this attribute to its object identifier, type, range,
 *	and indexing informtaion.
 */
static READONLY struct attribute	A_httpStatisticsNumProcessDns =
{
	&AI_httpStatisticsNumProcessDns,
	GET_httpStatisticsNumProcessDns,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **))) NULL,
	(Void *(*)PEER_PROTO_ARG((Void *, Void**, int))) NULL,
	IX_httpStatisticsEntry,
	SNMP_NUMBER,		/* INTEGER */
	0x80000000,
	0x7fffffff,
};


/************************************************************************
 *
 *	httpStatisticsRequests - access methods for SNMP MIB variable
 *
 *	Description:
 *
 *	 the total requests received/generated 
 *
 ************************************************************************/

/*
 *	httpStatisticsRequests is known as 1.3.6.1.4.1.1450.1.1.2.1.7
 */
static READONLY ubyte	AIA_httpStatisticsRequests[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 0x02, 
	0x01, 0x07, 
};

static READONLY struct object_id	AI_httpStatisticsRequests =
{	sizeof(AIA_httpStatisticsRequests),
	AIA_httpStatisticsRequests,
};

/*ARGSUSED*/
static int
#ifdef USE_PROTOTYPES
GET_httpStatisticsRequests(void *ctxt, void **indices, void *attr_ref)
#else
GET_httpStatisticsRequests(ctxt, indices, attr_ref)
void			*ctxt;
void			**indices;
void			*attr_ref;
#endif
{
	Counter	*attr = (Counter *) attr_ref;

	*attr = ((struct httpStatistics *)ctxt)->requests;
	return(SNMP_ERR_NO_ERROR);
}


/*
 *	The following structure binds the access methods for
 *	this attribute to its object identifier, type, range,
 *	and indexing informtaion.
 */
static READONLY struct attribute	A_httpStatisticsRequests =
{
	&AI_httpStatisticsRequests,
	GET_httpStatisticsRequests,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **))) NULL,
	(Void *(*)PEER_PROTO_ARG((Void *, Void**, int))) NULL,
	IX_httpStatisticsEntry,
	SNMP_CNTER,		/* Counter */
	0x0,
	0xffffffff,
};


/************************************************************************
 *
 *	httpStatisticsRequestError - access methods for SNMP MIB variable
 *
 *	Description:
 *
 *	 the total request errors detected 
 *
 ************************************************************************/

/*
 *	httpStatisticsRequestError is known as 1.3.6.1.4.1.1450.1.1.2.1.8
 */
static READONLY ubyte	AIA_httpStatisticsRequestError[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 0x02, 
	0x01, 0x08, 
};

static READONLY struct object_id	AI_httpStatisticsRequestError =
{	sizeof(AIA_httpStatisticsRequestError),
	AIA_httpStatisticsRequestError,
};

/*ARGSUSED*/
static int
#ifdef USE_PROTOTYPES
GET_httpStatisticsRequestError(void *ctxt, void **indices, void *attr_ref)
#else
GET_httpStatisticsRequestError(ctxt, indices, attr_ref)
void			*ctxt;
void			**indices;
void			*attr_ref;
#endif
{
	Counter	*attr = (Counter *) attr_ref;

	*attr = ((struct httpStatistics *)ctxt)->requesterr;
	return(SNMP_ERR_NO_ERROR);
}


/*
 *	The following structure binds the access methods for
 *	this attribute to its object identifier, type, range,
 *	and indexing informtaion.
 */
static READONLY struct attribute	A_httpStatisticsRequestError =
{
	&AI_httpStatisticsRequestError,
	GET_httpStatisticsRequestError,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **))) NULL,
	(Void *(*)PEER_PROTO_ARG((Void *, Void**, int))) NULL,
	IX_httpStatisticsEntry,
	SNMP_CNTER,		/* Counter */
	0x0,
	0xffffffff,
};


/************************************************************************
 *
 *	httpStatisticsProcessNum - access methods for SNMP MIB variable
 *
 *	Description:
 *
 *	 the number of running process
 *
 ************************************************************************/

/*
 *	httpStatisticsProcessNum is known as 1.3.6.1.4.1.1450.1.1.2.1.13
 */
static READONLY ubyte	AIA_httpStatisticsProcessNum[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 0x02, 
	0x01, 0x0d, 
};

static READONLY struct object_id	AI_httpStatisticsProcessNum =
{	sizeof(AIA_httpStatisticsProcessNum),
	AIA_httpStatisticsProcessNum,
};

/*ARGSUSED*/
static int
#ifdef USE_PROTOTYPES
GET_httpStatisticsProcessNum(void *ctxt, void **indices, void *attr_ref)
#else
GET_httpStatisticsProcessNum(ctxt, indices, attr_ref)
void			*ctxt;
void			**indices;
void			*attr_ref;
#endif
{
	INTEGER	*attr = (INTEGER *) attr_ref;

	*attr = ((struct httpStatistics *)ctxt)->numprocess;
	return(SNMP_ERR_NO_ERROR);
}


/*
 *	The following structure binds the access methods for
 *	this attribute to its object identifier, type, range,
 *	and indexing informtaion.
 */
static READONLY struct attribute	A_httpStatisticsProcessNum =
{
	&AI_httpStatisticsProcessNum,
	GET_httpStatisticsProcessNum,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **))) NULL,
	(Void *(*)PEER_PROTO_ARG((Void *, Void**, int))) NULL,
	IX_httpStatisticsEntry,
	SNMP_NUMBER,		/* INTEGER */
	0x80000000,
	0x7fffffff,
};


/************************************************************************
 *
 *	httpStatisticsThreadNum - access methods for SNMP MIB variable
 *
 *	Description:
 *
 *	 the number of running process
 *
 ************************************************************************/

/*
 *	httpStatisticsThreadNum is known as 1.3.6.1.4.1.1450.1.1.2.1.14
 */
static READONLY ubyte	AIA_httpStatisticsThreadNum[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 0x02, 
	0x01, 0x0e, 
};

static READONLY struct object_id	AI_httpStatisticsThreadNum =
{	sizeof(AIA_httpStatisticsThreadNum),
	AIA_httpStatisticsThreadNum,
};

/*ARGSUSED*/
static int
#ifdef USE_PROTOTYPES
GET_httpStatisticsThreadNum(void *ctxt, void **indices, void *attr_ref)
#else
GET_httpStatisticsThreadNum(ctxt, indices, attr_ref)
void			*ctxt;
void			**indices;
void			*attr_ref;
#endif
{
	INTEGER	*attr = (INTEGER *) attr_ref;

	*attr = ((struct httpStatistics *)ctxt)->numthread;
	return(SNMP_ERR_NO_ERROR);
}


/*
 *	The following structure binds the access methods for
 *	this attribute to its object identifier, type, range,
 *	and indexing informtaion.
 */
static READONLY struct attribute	A_httpStatisticsThreadNum =
{
	&AI_httpStatisticsThreadNum,
	GET_httpStatisticsThreadNum,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **, Void *))) NULL,
	(int (*)PEER_PROTO_ARG((Void *, Void **))) NULL,
	(Void *(*)PEER_PROTO_ARG((Void *, Void**, int))) NULL,
	IX_httpStatisticsEntry,
	SNMP_NUMBER,		/* INTEGER */
	0x80000000,
	0x7fffffff,
};

READONLY struct attribute *AS_httpStatisticsEntry[] =
{
	&A_httpStatisticsIndex,
	&A_httpStatisticsStatus,
	&A_httpStatisticsUptime,
	&A_httpStatisticsNumProcessIdle,
	&A_httpStatisticsNumProcessProc,
	&A_httpStatisticsNumProcessDns,
	&A_httpStatisticsRequests,
	&A_httpStatisticsRequestError,
	&A_httpStatisticsProcessNum,
	&A_httpStatisticsThreadNum,
	(struct attribute READONLY *) NULL
};

/*static*/ READONLY struct index_entry	IX_httpStatisticsEntry[] =
{
	{ 0 + 0,  &A_httpStatisticsIndex	},
	{ 0,  (struct attribute READONLY *) NULL}
};
static READONLY ubyte	GIA_httpStatisticsEntry[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 0x02, 
	0x01, 
};

static READONLY struct object_id	GI_httpStatisticsEntry =
{
	11,
	GIA_httpStatisticsEntry
};

#ifdef __cplusplus
extern READONLY struct class_definition	SMI_GROUP_httpStatisticsEntry;
#endif
READONLY struct class_definition	SMI_GROUP_httpStatisticsEntry =
{
	SNMP_CLASS,
	&GI_httpStatisticsEntry,
	AS_httpStatisticsEntry,
	(struct group_attribute READONLY **) NULL,
	(struct action READONLY **) NULL,
	(struct notification READONLY **) NULL,
	(int (*) PEER_PROTO_ARG((void))) NULL,
	(struct class_definition READONLY **) NULL,
	IX_httpStatisticsEntry,
	(struct contained_obj READONLY **) NULL,
	(int (*)PEER_PROTO_ARG((void))) NULL,
	(void (*)PEER_PROTO_ARG((void))) NULL,
};

static READONLY struct contained_obj	CG_httpStatisticsEntry = 
{
	&SMI_GROUP_httpStatisticsEntry,
	NULL
};
static READONLY struct contained_obj *CGS_httpStatisticsTable[] =
{
	&CG_httpStatisticsEntry,
	(struct contained_obj READONLY *) NULL
};
#define AS_httpStatisticsTable (struct attribute READONLY **) NULL
static READONLY ubyte	GIA_httpStatisticsTable[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 0x02, 
};

static READONLY struct object_id	GI_httpStatisticsTable =
{
	10,
	GIA_httpStatisticsTable
};

#ifdef __cplusplus
extern READONLY struct class_definition	SMI_GROUP_httpStatisticsTable;
#endif
READONLY struct class_definition	SMI_GROUP_httpStatisticsTable =
{
	SNMP_CLASS,
	&GI_httpStatisticsTable,
	AS_httpStatisticsTable,
	(struct group_attribute READONLY **) NULL,
	(struct action READONLY **) NULL,
	(struct notification READONLY **) NULL,
	(int (*) PEER_PROTO_ARG((void))) NULL,
	(struct class_definition READONLY **) NULL,
	(struct index_entry READONLY *) NULL,
	CGS_httpStatisticsTable,
	(int (*)PEER_PROTO_ARG((void))) NULL,
	(void (*)PEER_PROTO_ARG((void))) NULL,
};

static READONLY struct contained_obj	CG_httpStatisticsTable = 
{
	&SMI_GROUP_httpStatisticsTable,
	NULL
};
static READONLY struct contained_obj *CGS_httpEntityInfo[] =
{
	&CG_httpEntityTable,
	&CG_httpStatisticsTable,
	(struct contained_obj READONLY *) NULL
};
#define AS_httpEntityInfo (struct attribute READONLY **) NULL
static READONLY ubyte	GIA_httpEntityInfo[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 0x01, 
};

static READONLY struct object_id	GI_httpEntityInfo =
{
	9,
	GIA_httpEntityInfo
};

#ifdef __cplusplus
extern READONLY struct class_definition	SMI_GROUP_httpEntityInfo;
#endif
READONLY struct class_definition	SMI_GROUP_httpEntityInfo =
{
	SNMP_CLASS,
	&GI_httpEntityInfo,
	AS_httpEntityInfo,
	(struct group_attribute READONLY **) NULL,
	(struct action READONLY **) NULL,
	(struct notification READONLY **) NULL,
	(int (*) PEER_PROTO_ARG((void))) NULL,
	(struct class_definition READONLY **) NULL,
	(struct index_entry READONLY *) NULL,
	CGS_httpEntityInfo,
	(int (*)PEER_PROTO_ARG((void))) NULL,
	(void (*)PEER_PROTO_ARG((void))) NULL,
};


/************************************************************************
 *
 *	httpEntityStatChg - data structures for SNMP TRAP
 *
 *	Description:
 *
 *	 A httpEntityStatChg trap signifies that
 *					this http entity has status change
 *
 ************************************************************************/

/*
 *	httpEntityStatChg is known as 1.3.6.1.4.1.1450.1
 */
static READONLY ubyte	TIA_httpEntityStatChg[] =
{
	0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01, 
};

static READONLY struct object_id	TI_httpEntityStatChg =
{
	sizeof(TIA_httpEntityStatChg),
	TIA_httpEntityStatChg
};

/*
 *	Variables carried by this trap type
 */
static READONLY struct attribute	*TAS_httpEntityStatChg[] =
{
	&A_httpEntityIndex,
	(struct attribute READONLY *) NULL
};
/*
 *	This structure relates the trap enterprise and
 *	related fields to its variable list
 */
#ifdef __cplusplus
extern READONLY struct notification	TRAP_httpEntityStatChg;
#endif
READONLY struct notification	TRAP_httpEntityStatChg =
{
	&TI_httpEntityStatChg,
	(unsigned char (*)PEER_PROTO_ARG((void))) NULL,
	TAS_httpEntityStatChg,
	6,					/* generic trap		*/
	100001,					/* specific trap	*/
};

/************************************************************************
 *
 *	send_httpEntityStatChg_trap - generate SNMP httpEntityStatChg TRAP
 *
 *	Calling Sequence:
 *		status = send_httpEntityStatChg_trap(mgmt_env, );
 *
 *	Inputs:
 *
 *	Outputs:
 *	Description:
 *
 *	 A httpEntityStatChg trap signifies that
 *					this http entity has status change
 *
 ************************************************************************/

int
#ifdef USE_PROTOTYPES
send_httpEntityStatChg_trap(Void	*handle,
		INTEGER		INDEX_1_httpEntityIndex,
		INTEGER		httpEntityIndex)
#else
send_httpEntityStatChg_trap(handle,
	INDEX_1_httpEntityIndex,
	httpEntityIndex)

Void	*handle;
INTEGER		INDEX_1_httpEntityIndex;
INTEGER		httpEntityIndex;
#endif
{
	Void	*parms[3];

	/*
	 *	build parameter vector for library
	 */
	parms[2] = NULL;
	parms[0] = (Void *) &INDEX_1_httpEntityIndex;
	parms[1] = (Void *) &httpEntityIndex;

	return(mgmt_lowlevel_trap(handle, &TRAP_httpEntityStatChg, parms));
}
