/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#ifdef XP_WIN32

#include <windows.h>
#include <winsock.h>

#else /* ! XP_WIN32 */

#include <stdio.h>
#include <signal.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <arpa/nameser.h>
#include <resolv.h>

#endif /* ! XP_WIN32 */

#include "prtypes.h"
#include "libsnmp/conf.h"	/* SNMP subagent config */
#include "libsnmp/httppoll.h"	/* HTTP polling state machine */
#include "libadmin/dstats.h"	/* daemon statistics */
#include "base/ereport.h"
#include "base/net.h"

#include "frame/conf.h"         /* admin server conf */
#include "sec.h"

#include "ame/machtype.h"
#include "ame/syspub.h"
#include "ame/errno.h"
#include "ame/smitypes.h"
#include "ode/mgmtpub.h"

#include "libsnmp/ns_snmp.h"
#include "libsnmp/ns_snmpif.h"
#include "libsnmp/httpagt.h"
#include "netscape.h"	/* SMI compiler output */


#if 0
#define MYTRACE(x) fprintf x
#endif


#ifndef XP_WIN32

#ifdef BSD_SIGNALS
#define RSIGNAL(s, fn) signal(s, fn)
#else
static struct sigaction sa;

#define RSIGNAL(s, fn) { sa.sa_handler = fn; sigaction(s, &sa, NULL); }
#endif

#endif


typedef struct _SubagtContext {
    SubagtConfig *	cfg;
    struct snmp_contxt *sx;
    struct mib_contxt *	mib;
    PollContext *	px;
    DStatsHandle *	ds;
    int                 just_started;
} SubagtContext;


typedef enum _FDOwnerType {
    FD_OWNER_UNKNOWN = 0,
    FD_OWNER_SNMP    = 1,
    FD_OWNER_HTTP    = 2
} FDOwnerType;

typedef struct _FDOwnerMapping {
    FDOwnerType type;
    union {
        struct snmp_contxt *sx;
        PollContext        *px;
    } owner;
} FDOwnerMapping;


SubagtContext **cxs = NULL;
int cxs_cnt = 0;


SubagtContext *subagt_context_create(SubagtConfig *cfg);
SubagtContext *start_manage(SubagtConfig *cfg);
void subagt_context_destroy(SubagtContext *cx);
#ifdef XP_WIN32
void	http_subagt_pause(void);
#endif


/*
 *	Our "identity" is 1.3.6.1.4.1.1450.1 (for http subtree)
 */
static ubyte  me[] = {0x2b, 0x06, 0x01, 0x04, 0x01, 0x8b, 0x2a, 0x01};

static ObjId_t	my_id = {sizeof(me),me,};


static int restarted = 0;	/* flag set when SIGHUP received */
static int shut_down = 0;	/* flag set when SIGTERM received */


/**************** MULTIPLE POLL SUPPORT *************************/

#define BUFFER_SIZE 10

/* Generates a list of all the servers on your machine. */
char **list_installed_servers(char *nsroot, char **namelist, int *cnt)
{
    SYS_DIR ds;
    SYS_DIRENT *d;
    char **names = new_strlist(BUFFER_SIZE);
    char **roots = new_strlist(BUFFER_SIZE);
    int totalsize=BUFFER_SIZE, curentry=0;

    if (cnt)
	*cnt = 0;

    if (!nsroot || !namelist)
	return NULL;

    if(!(ds = dir_open(nsroot)))
	return NULL;

    while( (d = dir_read(ds)) )  {
        char *entry;
        register int z;

        entry = d->d_name;

        for(z=0; namelist[z]; z++)  {
            if( !strncmp(entry, namelist[z], strlen(namelist[z])) &&
                entry[strlen(namelist[z])] == '-')  {
                if( !(curentry < totalsize) )  {
                    totalsize += BUFFER_SIZE;
		    names = grow_strlist(names, totalsize);
		    roots = grow_strlist(roots, totalsize);
                }
		names[curentry] = STRDUP(entry);
		roots[curentry] = (char *)MALLOC(strlen(nsroot) +
						 strlen(entry) + 2);
		sprintf(roots[curentry], "%s%c%s",
			nsroot, FILE_PATHSEP, entry);

		curentry++;
		names[curentry] = NULL;
                roots[curentry] = NULL;
            }
        }
    }

    dir_close(ds);

    if (cnt)
	*cnt = curentry;

    return curentry ? roots : NULL;
}


static void grab_stats(SubagtContext *cx)
{
    struct snmp_contxt *sx = cx->sx;
    struct httpStatistics *row = cx->mib->statistics;
    SubagtConfig *cfg = cx->cfg;
    PollContext *px = cx->px;
    DStatsHandle *ds = cx->ds;
    char *err_title = NULL;
    char *err_msg = NULL;

    struct httpEntity *row_entity = cx->mib->entity;

    MYTRACE((stderr, "-- grabbing stats, row=%p, px=%p, *ds=%p\n",
	     row, px, ds));

    if (!row)
	return;

    row->port = cfg->snmp_port;

    /*
    row->addr = cfg->snmp_addr ? inet_addr(cfg->snmp_addr) : 0;
    */
    row->addr = row_entity->addr;

    if (ds && dstats_poll_for_restart(ds) && !row->status) {
	restarted = 1;	/* cause subagent to restart entirely */
    }

    if (px) {

	/* server status (up/down) */
	if (px->status == SERVER_STATUS_UP) {
	    if (row->status != 0) {
		/* server just came up */
		row->status = 0;

		/* reopen dstats */
		if (ds)
		    dstats_close(ds);
		ds = cx->ds = dstats_open(cfg->snmp_port, cfg->snmp_addr, &err_title, &err_msg);
		if (!cx->just_started) {
		    restarted = 1;	/* later cause an entire subagt restart */
		}
	    }
	}
	else {
	    if (row->status != 1) {
		/* server just went down */
		row->status = 1;

		restarted = 0;

		/* send trap */
		send_httpEntityStatChg_trap(sx->snmp_env, row_entity->port, 
		       row_entity->addr, row_entity->port, row_entity->port, 
		       row_entity->addr, row_entity->addr);
		/*
		subagt_trap(sx, &TRAP_httpEntityStatChg);
		*/
	    }
	}

	MYTRACE((stderr, "-- set server status to %d (%s)\n",
		 row->status,
		 row->status ? "down" : "up"));

	/* server uptime */
	row->uptime =
	  ((px->status == SERVER_STATUS_UP)
	   ? 100 * (time(NULL) - px->status_time_stamps[SERVER_STATUS_UP])
	   : 0);
    }

    if (ds) {
#ifdef HPUX
       dstats_close(ds);
       ds = cx->ds = dstats_open(cfg->snmp_port, cfg->snmp_addr, &err_title,
&err_msg);
#endif
#ifdef IRIX 
       dstats_close(ds);
       ds = cx->ds = dstats_open(cfg->snmp_port, cfg->snmp_addr, &err_title,
&err_msg);
#endif
      row->uptime = ((row->status)
	? 0 : 100 * (time(NULL) - ds->hdr->start_time));
      
	dstats_snapshot(ds);

	/* current # of threads */
	row->numthread = ((row->status) ? 0 : ds->snew.active);

	/* # of idle processes */
	row->numpidle =((row->status) ? 0 : ds->snew.idle);

	/* # of busy processes */
	row->numpproc =((row->status)
	        ? 0 : ds->snew.process + ds->snew.read + ds->snew.write);

	/* # of processes doing DNS */
	row->numpdns = ((row->status) ? 0 : ds->snew.resolv);

	/* # of requests serviced since startup */
	row->requests = ds->totals.tot_reqs;

	/* # of errors since startup */
	row->requesterr = ds->totals.tot_errs;
    } else {

	/* current # of threads */
	row->numthread = 0;

	/* # of idle processes */
	row->numpidle = 0;

	/* # of busy processes */
	row->numpproc = 0;

	/* # of processes doing DNS */
	row->numpdns = 0;
    }

    if (cx->just_started > 0)
	cx->just_started--;
}


void start_multiple_polls(char *nsroot, char **installed)
{
    int i, entries = 0, need_certdb = security_active ? 0 : 1;
    char *err = NULL;
    char **roots = list_installed_servers(nsroot, installed, &entries);
    SubagtConfig *cfg = NULL;
    static SECCertDBHandle certdb;  /* libsec keeps a pointer to this */

    cxs_cnt = 0;
    cxs = (SubagtContext **)MALLOC((entries + 1) * sizeof(SubagtContext *));
    ZERO(cxs, (entries + 1) * sizeof(SubagtContext *));

    for(i=0; i<entries; i++) {
	if (!(cfg = subagt_conf_load(roots[i], &err))) {
#ifdef OLD_CODE
	    ereport(LOG_FAILURE, "SNMP: %s for %s", err ? err : "unknown error", roots[i]);
#else
	    if (err)
		ereport(LOG_FAILURE, "SNMP: %s for %s", err, roots[i]);
	    /* otherwise silently ignore -- config file didn't exist */
#endif
	}
	else if (!cfg->snmp_on) {
	    subagt_conf_destroy(cfg);
	}
	else {
            if(cfg->snmp_secure && need_certdb) {
                need_certdb = 0;
                if(SEC_OpenCertDB(&certdb, "pollcert.db", PR_TRUE))
                    goto monitor_failed;  /* XXXrobm shoot me */
                SEC_SetDefaultCertDB(&certdb);
            }
            if (!(cxs[cxs_cnt] = start_manage(cfg))) {
              monitor_failed:
                ereport(LOG_FAILURE, "SNMP monitoring failed for %s",roots[i]);
                subagt_conf_destroy(cfg);
            }
            else {
                ereport(LOG_INFORM, "SNMP monitoring started for %s",roots[i]);
                cxs_cnt++;
            }
        }
    }
}


SubagtContext *subagt_context_create(SubagtConfig *cfg)
{
    SubagtContext *cx = (SubagtContext *)MALLOC(sizeof(SubagtContext));

    ZERO(cx, sizeof(SubagtContext));
    cx->cfg = cfg;
    cx->just_started = 30;

    return cx;
}


static void set_entity_table(struct httpEntity *row, SubagtConfig *cfg)
{
    char *p = NULL;
    char host[205];

    gethostname(host, 205);

    p = cfg->snmp_description ? cfg->snmp_description : "N/A";
    row->descr.val = p;
    row->descr.len = strlen(p);

    row->svrobjid.val = me;
    row->svrobjid.len = sizeof(me);

    p = "HTTP/1.1";
    row->protocol.val = p;
    row->protocol.len = strlen(p);

    p = MAGNUS_VERSION_STRING;
    row->vers.val = p;
    row->vers.len = strlen(p);

    p = cfg->snmp_organization ? cfg->snmp_organization : "N/A";
    row->org.val = p;
    row->org.len = strlen(p);

    p = cfg->snmp_location ? cfg->snmp_location : "N/A";
    row->loc.val = p;
    row->loc.len = strlen(p);

    p = cfg->snmp_contact ? cfg->snmp_contact : "N/A";
    row->contact.val = p;
    row->contact.len = strlen(p);

    /* XXXari this probably needs fixin' 
    row->addr = cfg->snmp_addr ? inet_addr(cfg->snmp_addr) : 0; */

    row->addr = get_ip_by_name(host);

    row->port = cfg->snmp_port;

    p = cfg->snmp_host ? cfg->snmp_host : "N/A";
    row->name.val = p;
    row->name.len = strlen(p);

    p = "Generic Fedora Server";
    row->svrtype.val = p;
    row->svrtype.len = strlen(p);

    p = "GET/POST";
    row->methods.val = p;
    row->methods.len = strlen(p);

/*
    row->maxprocess = cfg->snmp_max_procs;
    row->minprocess = (cfg->snmp_min_procs > 0
		       ? cfg->snmp_min_procs
		       : cfg->snmp_max_procs);

    row->maxthread = cfg->snmp_max_threads;
    row->minthread = (cfg->snmp_min_threads > 0
		      ? cfg->snmp_min_threads
		      : cfg->snmp_max_threads);
*/
}


SubagtContext *start_manage(SubagtConfig *cfg)
{
    SubagtContext *cx = subagt_context_create(cfg);
    char *err_title = NULL;
    char *err = NULL;

    /* open daemon statistics file */
    if (!(cx->ds = dstats_open(cfg->snmp_port, cfg->snmp_addr, &err_title, &err))) {
	ereport(LOG_FAILURE, "SNMP: %s",
		(err_title ? err_title :
		 (err      ? err       : "unknown error opening daemonstats")));
    }

    /* set up SNMP */
    if (!(cx->sx = subagt_conn_init(&my_id))) {
	ereport(LOG_FAILURE, "SNMP: can't connect to master agent");
	return NULL;
    }

    if (!(cx->mib = (struct mib_contxt *)MALLOC(sizeof(struct mib_contxt)))) {
	ereport(LOG_CATASTROPHE, "SNMP: out of memory, can't allocate MIB");
	return NULL;
    }
    ZERO(cx->mib, sizeof(struct mib_contxt));

    if (!(cx->mib->entity =
	  (struct httpEntity *)MALLOC(sizeof(struct httpEntity)))) {
	ereport(LOG_CATASTROPHE, "SNMP: out of memery, can't allocate httpEntity");
	return NULL;
    }
    ZERO(cx->mib->entity, sizeof(struct httpEntity));

    if (!(cx->mib->statistics =
	  (struct httpStatistics *)MALLOC(sizeof(struct httpStatistics)))) {
	ereport(LOG_CATASTROPHE, "SNMP: out of memory, can't allocate httpStatistics");
	return NULL;
    }
    ZERO(cx->mib->statistics, sizeof(struct httpStatistics));

    /* initialize static info */
    set_entity_table(cx->mib->entity, cx->cfg);

    /* update SNMP info */
    grab_stats(cx);

    /* connect to SNMP master agent */
    if (!cfg->snmp_master_host) {
	if (cfg->snmp_host) {
	    cfg->snmp_master_host = STRDUP(cfg->snmp_host);
	}
	else {
	    ereport(LOG_MISCONFIG, "SNMP: no master host specified\n");
	    return NULL;
	}
    }

    if (subagt_conn_magt(cx->sx, cfg->snmp_master_host) == ERR_CONN_FAIL) {
	ereport(LOG_FAILURE, "SNMP: can't connect to SNMP master agent");
	return NULL;
    }

    /* register MIB rows */
    subagt_register_mibrow(cx->sx, &SMI_GROUP_httpEntityEntry,
			   (struct httpEntity *)cx->mib->entity);
    subagt_register_mibrow(cx->sx, &SMI_GROUP_httpStatisticsEntry,
			   (struct httpStatistics *)cx->mib->statistics);

    return cx;
}


void subagt_context_destroy(SubagtContext *cx)
{
    if (cx) {
	if (cx->ds) {
	    dstats_close(cx->ds);
	    cx->ds = NULL;
	}
	if (cx->px) {
	    http_setup_free(cx->px);
	    cx->px = NULL;
	}
	if (cx->sx) {
	    subagt_conn_term(cx->sx);
	    cx->sx = NULL;
	}
	if (cx->mib) {
	    if (cx->mib->entity)
		FREE(cx->mib->entity);
	    if (cx->mib->statistics)
		FREE(cx->mib->statistics);
	    FREE(cx->mib);
	    cx->mib = NULL;
	}
	if (cx->cfg) {
	    subagt_conf_destroy(cx->cfg);
	    cx->cfg = NULL;
	}
	FREE(cx);
    }
}


int handle_subagt_events(struct snmp_contxt *sx)
{
    struct timeval tv;
    int rv;

    tv.tv_sec = 0;
    tv.tv_usec = 10000;

    MYTRACE((stderr, "SNMP events\n"));

    do {
	rv = subagt_poll(sx, &tv);
    } while (rv != NO_REQ_PROC);

    MYTRACE((stderr, "SNMP events done\n"));

    return rv;
}


void mainloop(void)
{
    fd_set subagt_fds;
    fd_set rd_fds;
    fd_set wr_fds;
    fd_set ex_fds;
    int    n_fds = 0;
    struct timeval tv;
    time_t next;
    int rv, f, x;


    MYTRACE((stderr, "DEBUG: starting mainloop\n"));


    /*
     * SNMP subagent mainloop
     *
     * Break out if subagent gets restarted or shut down.
     *
     */
    while (!restarted && !shut_down) {

	/* set up HTTP polling */
	for (x=0; x<cxs_cnt; x++) {
	    if (cxs[x] && !cxs[x]->px) {
		/*
		 * XXXari this does busy retries -- should be avoided
		 *
		 */
		if (!(cxs[x]->px = http_setup_poll(cxs[x]->cfg->snmp_host,
						   cxs[x]->cfg->snmp_port,
						   cxs[x]->cfg->snmp_secure,
						   cxs[x]->cfg->snmp_poll_url))) {
		    ereport(LOG_FAILURE, "can't start HTTP polling for %s://%s:%d%s",
			    cxs[x]->cfg->snmp_secure ? "https" : "http",
			    cxs[x]->cfg->snmp_host,
			    cxs[x]->cfg->snmp_port,
			    (cxs[x]->cfg->snmp_poll_url ?
			     cxs[x]->cfg->snmp_poll_url : "/index.html"));
		}
	    }
	}

	/*
	 * Start polling the server right away.
	 *
	 * Next poll after NS_SNMP_HTTP_POLL_INTERVAL seconds.
	 *
	 * If the previous poll hasn't finished by that time,
	 * it will be interrupted and marked as timed out.
	 *
	 */
#if 0  /* XXXrobm what exactly is cfg??? */
	next = time(NULL) + (cfg->snmp_poll_interval > 0 ?
			     cfg->snmp_poll_interval :
			     NS_SNMP_HTTP_POLL_INTERVAL);
#else
        next = time(NULL) + NS_SNMP_HTTP_POLL_INTERVAL;
#endif

	for (x=0; x<cxs_cnt; x++) {
	    if (cxs[x] && cxs[x]->px) {
		MYTRACE((stderr, "state before new start = %d\n", cxs[x]->px->state));
		http_start_poll(cxs[x]->px);
		MYTRACE((stderr, "state after new start = %d\n", cxs[x]->px->state));
	    }
	}

	/*
	 * Loop here while doing connect poll and/or SNMP.
	 *
	 * Break out of this loop only to start the next poll,
	 * or if the agent gets restarted or shut down.
	 *
	 */
	while (!restarted && !shut_down) {

	    for (x=0; x<cxs_cnt; x++) {
		if (cxs[x]) {
		    /* update SNMP info */
		    grab_stats(cxs[x]);

		    /* handle first any pending SNMP events asap */
		    handle_subagt_events(cxs[x]->sx);
		}
	    }

	    /* prepare for select() */
	    FD_ZERO(&rd_fds);
	    FD_ZERO(&wr_fds);
	    FD_ZERO(&ex_fds);
	    FD_ZERO(&subagt_fds);

	    /* HTTP poll descriptors */
	    memcpy(&rd_fds, &poll_rd_fds, sizeof(fd_set));
	    memcpy(&wr_fds, &poll_wr_fds, sizeof(fd_set));
	    memcpy(&ex_fds, &poll_wr_fds, sizeof(fd_set));

#if 0
	    for (f=0; f<poll_n_fds; f++) {
		if (FD_ISSET(f, &rd_fds))
		  fprintf(stderr, "| | | READ  : %d\n", f);
		if (FD_ISSET(f, &wr_fds))
		  fprintf(stderr, "| | | WRITE : %d\n", f);
	    }

	    fprintf(stderr, "DEBUG: subagt_fd_count() => %d\n", subagt_fd_count());
	    fprintf(stderr, "DEBUG: maxfiles()        => %d\n", maxfiles());
#endif

	    /* SNMP descriptors */
	    for (x=0; x<cxs_cnt; x++) {
		if (cxs[x]) {
		    int biggest = 0;

		    subagt_select(cxs[x]->sx, &subagt_fds);
		    subagt_select(cxs[x]->sx, &rd_fds);

		    if (n_fds <= subagt_fd_count())
			n_fds = subagt_fd_count();
		    if (n_fds > FD_SETSIZE)
			n_fds = FD_SETSIZE;

#if 0
		    fprintf(stderr, "| | | SUBAGT(%d):", x);
#endif
		    for (f=0; f<n_fds; f++) {
			if (FD_ISSET(f, &subagt_fds)) {
			    biggest = f;
#if 0
			    fprintf(stderr, " %d", f);
#endif
#if 0
			    FD_SET(f, &rd_fds);
#endif
			}
		    }
		    n_fds = (poll_n_fds > biggest) ? poll_n_fds : biggest + 1;
#if 0
		    fprintf(stderr, "\n");
		    fprintf(stderr, "Read fd_set size %d (poll %d, snmp %d)\n",
			    n_fds, poll_n_fds, biggest);
#endif
		}
	    }

	    /* max # of descriptors to select() over */
	    if (n_fds < poll_n_fds)
		n_fds = poll_n_fds;

	    /* timeout */
	    tv.tv_sec = next - time(NULL);
	    tv.tv_usec = 0;

	    MYTRACE((stderr, "timeout is %ld sec\n", tv.tv_sec));

	    if (tv.tv_sec <= 0) {
		MYTRACE((stderr, "timeout\n"));
		break;
	    }
#ifdef IRIX
	    if ((rv = _select(n_fds, &rd_fds, &wr_fds, &ex_fds, &tv)) == 0) {
#else
	    if ((rv = net_select(n_fds, &rd_fds, &wr_fds, &ex_fds, &tv)) == 0) {
#endif
		MYTRACE((stderr, "HTTP poll timed out\n"));
		break;
	    }
	    if (rv == -1) {
		MYTRACE((stderr, "select() returned -1, errno %d\n", errno));
		abort();
	    }

	    MYTRACE((stderr, "select returned %d\n", rv));

	    for (x=0; x<cxs_cnt; x++) {
		if (cxs[x]) {
		    /* update SNMP info again */
		    grab_stats(cxs[x]);

		    /* Give priority to SNMP events */
		    handle_subagt_events(cxs[x]->sx);
		}
	    }

	    MYTRACE((stderr, "scanning fd's\n"));

	    /* Then do everything else */
	    for (f=0; f<n_fds; f++) {
		if (FD_ISSET(f, &rd_fds) ||
		    FD_ISSET(f, &wr_fds) ||
		    FD_ISSET(f, &ex_fds)) {
		    for (x=0; x<cxs_cnt; x++) {
			if (cxs[x] && cxs[x]->px && cxs[x]->px->sd == f) {
			    http_handle_events(cxs[x]->px);
			}
		    }
		}
	    }
	    MYTRACE((stderr, "fd scan done\n"));
	}
	MYTRACE((stderr, "Next HTTP poll ----------------------------------\n"));
    }

    /* shutdown everything */

    MYTRACE((stderr, "DEBUG: shutting down\n"));

    for (x=0; x<cxs_cnt; x++) {
	if (cxs[x]) {
	    subagt_context_destroy(cxs[x]);
	}
    }
}


#ifndef XP_WIN32
void http_subagt_pause(void);
void http_subagt_notify(void);


void httpagt_restart(int sig)
{
    restarted = 1;

    http_subagt_notify();
    RSIGNAL(SIGHUP, httpagt_restart);
}


void httpagt_shutdown(int sig)
{
    shut_down = 1;
    http_subagt_notify();
}


void httpagt_coredump(int sig)
{
    ereport(LOG_CATASTROPHE,
	    "HTTP SNMP subagent caught %s (core dumped)",
	    sig == SIGBUS  ? "SIGBUS" :
	    sig == SIGSEGV ? "SIGSEGV" : "unhandled signal");
    abort();
}
#endif


int http_subagt_run(char *nsroot, char **installed)
{
    int once = 0;

    if (nsroot && installed) {
	do {
	    start_multiple_polls(nsroot, installed);
	    restarted = 0;
	    if (cxs_cnt > 0) {
		if (!(once++))
		    ereport(LOG_INFORM, "SNMP agent active");
		mainloop();
	    }
	    else {
#if 0
		ereport(LOG_INFORM, "SNMP thread idle");
#endif
                http_subagt_pause();
	    }
	} while (!shut_down);
    }

    return 0;
}


#if defined(THREAD_ANY) && defined(DAEMON_STATS)

#include "base/systhr.h"
#include "base/crit.h"

typedef struct _snmp_thread_data {
    char *nsroot;
    char **installed;
    SYS_THREAD thr;
} snmp_thread_data;

static CRITICAL http_subagt_pausecrit;
static CONDVAR http_subagt_pausecond;


void http_subagt_notify(void)
{
    crit_enter(http_subagt_pausecrit);
    condvar_notify(http_subagt_pausecond);
    crit_exit(http_subagt_pausecrit);
}

void http_subagt_pause(void)
{
    crit_enter(http_subagt_pausecrit);
    condvar_wait(http_subagt_pausecond);
    crit_exit(http_subagt_pausecrit);
}


void snmp_thread_main(void *data)
{
    snmp_thread_data *d = (snmp_thread_data *)data;

    http_subagt_pausecrit = crit_init();
    http_subagt_pausecond = condvar_init(http_subagt_pausecrit);

    http_subagt_run(d->nsroot, d->installed);
}


int http_subagt_run_in_thread(char *nsroot, char **installed)
{
    snmp_thread_data *d = (snmp_thread_data *)MALLOC(sizeof(snmp_thread_data));

    d->nsroot = nsroot;
    d->installed = installed;
    d->thr = systhread_start(SYSTHREAD_DEFAULT_PRIORITY, 262144,
			     snmp_thread_main, d);
    if (d->thr == SYS_THREAD_ERROR) {
	ereport(LOG_CATASTROPHE,
		"could not create new thread for SNMP subagent: %d (%s)",
		errno, system_errmsg());
	return -1;
    }

    return 0;
}

#else

void http_subagt_notify(void)
{
}

void http_subagt_pause(void)
{
    pause();
}

#endif /* THREAD_ANY && DAEMON_STATS */

