/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * George Dong - 01/15/96
 *
 */

#include <stdio.h>
#include <signal.h>
#include <string.h>

#ifndef XP_WIN32
#include <unistd.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <arpa/nameser.h>
#include <resolv.h>
#endif

#include "netsite.h"
#include "prtypes.h"		/* int32 etc */
#include "prlong.h"

#include "ame/smitypes.h"
#include "ame/portnums.h"

#ifdef AIX
#include <sys/select.h>		/* define fd_set */
#endif

#include "libsnmp/ns_snmp.h"
#include "libsnmp/ns_snmpif.h"

#ifdef PEER_SNMP
#include "ame/portpub.h"
#include "ame/portnums.h"
#endif


int subagt_fd_count(void)
{
    return maxfiles();
}


struct snmp_contxt *
subagt_conn_init(ObjId_t * myid)
{
  struct snmp_contxt      *contxt;
  struct sockaddr_in      *agent_alias;;

 
  if ((contxt = (struct snmp_contxt *)MALLOC(sizeof(struct snmp_contxt))) 
      == NULL)
    return NULL;

  if ((contxt->agent =(struct sockaddr *)MALLOC(sizeof(struct sockaddr))) 
       == NULL)
    return NULL;

  contxt->myid = myid;
  agent_alias = (struct sockaddr_in *)contxt->agent;

  contxt->passwd = NULL;
  contxt->mib_list = NULL;

  contxt->agent->sa_family = AF_INET;
  agent_alias->sin_port = SMUX_LISTEN_PORT;
 
  return contxt;
}


int subagt_conn_magt(struct snmp_contxt * contxt, char * host)
{
  struct sockaddr_in      *agent_alias = (struct sockaddr_in *)contxt->agent;
 
  agent_alias->sin_addr.s_addr = get_ip_by_name(host);

  contxt->snmp_env = mgmt_init_env(contxt->agent, 
				    contxt->myid,
				    SUBAGENT_DESC,
				    (Octets_t *) contxt->passwd,
				    (void_function) NULL);
  if(contxt->snmp_env == NULL)
    return ERR_CONN_FAIL;

  return NO_ERR;
}


int subagt_register_mib(struct snmp_contxt * contxt, void * class, void * mib)
{
 struct mib_list  * lpmib = contxt->mib_list;

  while (lpmib != NULL)
      lpmib = lpmib->next;

  if ((lpmib = (struct mib_list *)MALLOC(sizeof(struct mib_list))) == NULL)
      return ERR_MEM_FAIL;
      
  lpmib->handle = mgmt_new_instance(contxt->snmp_env,
			       (struct class_definition *)class, 
			       mib);

  if (lpmib->handle == NULL)
    return ERR_MIB_INIT;

  lpmib->next = NULL;

  return NO_ERR;
}


int subagt_register_mibrow(struct snmp_contxt * contxt,
                           READONLY void * class, void * mib)
{

  struct mib_list  * lpmib = contxt->mib_list;

  while (lpmib != NULL)
      lpmib = lpmib->next;

  if ((lpmib = (struct mib_list *)MALLOC(sizeof(struct mib_list))) == NULL)
      return ERR_MEM_FAIL;
      
  lpmib->handle = mgmt_new_row(contxt->snmp_env,
			       (READONLY struct class_definition *)class, 
			       mib);

  if (lpmib->handle == NULL)
    return ERR_MIB_INIT;

  lpmib->next = NULL;

  return NO_ERR;
}


void subagt_conn_term(struct snmp_contxt * contxt)
{
  struct mib_list  * lptmp;
  struct mib_list  * lpmib = contxt->mib_list;

  while (lpmib != NULL){
    lptmp = lpmib->next;
    mgmt_del_instance(contxt->snmp_env, lpmib->handle);
    FREE(lpmib);
    lpmib = lptmp;
  }

  if (contxt->passwd != NULL)
    FREE(contxt->passwd);
  if (contxt->agent != NULL)
    FREE(contxt->agent);

  if (contxt->snmp_env != NULL){
    mgmt_term_env(contxt->snmp_env);
  }

  FREE(contxt);
}

void subagt_select(struct snmp_contxt * contxt, fd_set * mask)
{
  mgmt_select_mask(contxt->snmp_env, mask);
}

int subagt_poll(struct snmp_contxt * contxt, struct timeval * wait)
{
  int status;

  if ((status=mgmt_poll(contxt->snmp_env, wait)) < 0)
    return ERR_AGT_FAIL;
  else {
    if (status == 0)
      return NO_REQ_PROC;
  }

  return NO_ERR;
}


int subagt_trap(struct snmp_contxt * contxt, void * trap)
{
  if( mgmt_trap(contxt->snmp_env, (struct notification *)trap) != 0)
    return ERR_TRAP_FAIL;

  return NO_ERR;
}


