/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * dl.c: Handles loading application functions from dynamic libraries
 * 
 * Rob McCool
 */


#include "netsite.h"
#include "safs/dl.h"
#include "base/systems.h"
#include "frame/func.h"
#include "base/dll.h"
#include "base/util.h"
#ifdef XP_WIN32
#include <nt/nsapi.h>
// extern SafFunction **SafTable;
typedef void _cdecl InitFnType( SafFunction **);
InitFnType* InitFn;
#endif /* XP_WIN32 */


#ifdef DLL_CAPABLE


/* ------------------------------- dl_init -------------------------------- */


void *_fsfree(struct FuncStruct *fs)
{ 
    FREE(fs->name);
    FREE(fs);
    return 0;
}

int dl_init(pblock *pb, Session *, Request *)
{
    char *libname = pblock_findval("shlib", pb); 
    char *funcNames = pblock_findval("funcs", pb);
    char *cNativeThreadFlag = pblock_findval("NativeThread", pb);
    int funcflags = FUNC_USE_NATIVE_THREAD;
    char c, err[MAGNUS_ERROR_LEN];
    register char *t, *u, *function, *w;
    DLHANDLE dlp;
    FuncPtr tfn;

    if((!libname) || (!funcNames)) {
        pblock_nvinsert("error","missing parameter (need shlib and funcs)",pb);
        return REQ_ABORTED;
    }
    if(!(dlp = dll_open(libname))) {
	char* dllerr = dll_error(dlp);
        util_snprintf(err, MAGNUS_ERROR_LEN, "dlopen of %s failed (%s)", libname, dllerr);
        pblock_nvinsert("error", err, pb);
        return REQ_ABORTED;
    }
    /* We're going to mess this string up a little... */
    funcNames = STRDUP(funcNames);
    t = funcNames;

    if (cNativeThreadFlag && !strcasecmp(cNativeThreadFlag, "no")) 
        funcflags &= ~(FUNC_USE_NATIVE_THREAD);

    while(1) {
        for(u = t; *u && *u != ','; ++u);
        c = *u;
        *u = '\0';

        function = STRDUP(t);
        //
        // Convert dash to underscore
        for(w = function; *w; ++w)
            if(*w == '-')
                *w = '_';

        tfn = (FuncPtr) dll_findsym(dlp, function);
        FREE(function);

        if(!tfn) {
            util_sprintf(err, "dlsym for %s failed (%s)", t, dll_error(dlp));
            pblock_nvinsert("error", err, pb);
            FREE(funcNames);
            return REQ_ABORTED;
        }
#if 0
        magnus_atrestart((void (*)(void *))_fsfree, 
                         (void *)func_insert2(STRDUP(t), tfn, funcflags));
#else
        func_insert2(STRDUP(t), tfn, funcflags);
#endif
        t = u;
        if(!(*t++ = c))
            break;
    }
#if defined (XP_WIN32) && ! defined(BUILD_DLL)
    InitFn = (InitFnType*)dll_findsym(dlp, "InitSafTable");
    if(!InitFn) {
        util_sprintf(err, "dlsym for InitSafTable failed (%s)", t, 
                     dll_error(dlp));
        FREE(funcNames);
        return REQ_ABORTED;
    }
    // Call the function...
        (*InitFn)(SafTable);
#endif /* XP_WIN32 */
#if 0
    magnus_atrestart((void (*)(void *))dll_close, (void*)dlp);
#endif
    return REQ_PROCEED;
}
#endif
