/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/
/*
 * logbuf.h: General functions to handle buffered log writes. Requires a 
 * threaded server.
 * 
 * Rob McCool
 */


#ifndef _LOGBUF_H
#define _LOGBUF_H

#ifdef THREAD_ANY

#include "base/crit.h"
#include "base/pblock.h"
#include "base/file.h"

#define LOGBUF_T struct logbuf_s

struct logdata_s {
    char *data;
    int pos;
    LOGBUF_T *log;
    struct logdata_s *next;
};

struct logdata_thread_s {
    CRITICAL crit;
    struct logdata_s *logdata;
    struct logdata_thread_s *next;
};


struct logbuf_s {
    SYS_FILE fd;
    int data_key;

    CRITICAL thread_crit;
    struct logdata_thread_s *thread_list;

    CRITICAL free_crit;
    struct logdata_s *free_list;
    CRITICAL flush_crit;
    int flush_len;
    struct logdata_s *flush_list, *flush_list_end;

    struct logbuf_s *next;
};



#define LOGBUF_DIRECTIVE(str) \
   ((!strcmp(str, "buffer-size")) || (!strcmp(str, "buffer-flush")))

/* Default sync rate is 60 seconds. */
#define DEFAULT_LOGBUF_FLUSH (60 * 1000)


int logbuf_init(pblock *args);
LOGBUF_T *logbuf_new(pblock *pb, SYS_FILE fd);
void logbuf_write(LOGBUF_T *lb, SYS_FILE fd, char *buf, int len);
void logbuf_close(LOGBUF_T *lb);


#endif  /* THREAD_ANY */
#endif  /* LOGBUF_H */
